#========================================================================================
# package      DE_EPAGES::Address::API::Address
# state        public
# description  This module includes a function to update VATID
#========================================================================================
package DE_EPAGES::Address::API::Address;
use base qw( Exporter );
use strict;

use DE_EPAGES::Address::API::Constants qw ( $REGULAREXPRESSIONS_FOR_ZIPCODE );
use DE_EPAGES::Core::API::Html qw ( RemoveSuspiciousChars );
use DE_EPAGES::Core::API::PerlTools qw(ElementOf Compare);
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestString );
use DE_EPAGES::Database::API::DataEncryption qw ( Encrypt );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountry );

use constant MAX_BANKNAME_LENGTH           =>  64;
use constant MAX_ENCRYPTED_BANKNAME_LENGTH => 255;

use constant ADDRESS_FIELDS => (qw(
    Salutation
    Title
    FirstName
    MiddleName
    LastName
    DisplayName
    AddressExtension
    Street
    Street2
    CodePorte
    City
    Zipcode
    State
    Company
    Department
    JobTitle
    Phone
    PhoneBusiness
    PhonePrivate
    PhoneCell
    Fax
    URL
    BankAccountHolder
    BankAccountNo
    BankCode
    BankName
    VATID
    FiscalCode
));

our @EXPORT_OK = qw (
    CheckZipcode
    IsValidBankName
    IsUnambiguousBasketAddress
    SecureAddressFields
    CompareZipCodes
);

#========================================================================================
# function     CheckZipcode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $check = CheckZipcode( $Zipcode, $CountryID );
#----------------------------------------------------------------------------------------
# description  Returns true if zipcode is valid for the country and the country is one
#               of the know countries incuded in
#               $DE_EPAGES::Address::API::Constants::REGULAREXPRESSIONS_FOR_ZIPCODE.
#               Returns true if $ZipCode or $CountryID is undefined or an empty string.
#               Returns true if $CountryID is not included in
#               $DE_EPAGES::Address::API::Constants::REGULAREXPRESSIONS_FOR_ZIPCODE.
#----------------------------------------------------------------------------------------
# input        $Zipcode | zipcode | string
# input        $CountryID | country id | int
# return       $check | valid | boolean
#========================================================================================
sub CheckZipcode {
    my ($Zipcode, $CountryID) = @_;
    return 1 if !defined $Zipcode || $Zipcode !~ /\S/;
    TestString('Zipcode', $Zipcode);
    return 1 if !defined $CountryID || $CountryID !~ /\S/;
    TestInteger('CountryID', $CountryID);

    my $CountryCode = GetInfoCountry($CountryID)->{'Code2'};
    my $RegEx = $REGULAREXPRESSIONS_FOR_ZIPCODE->{$CountryCode};
    return 1 unless defined $RegEx;
    return 1 if $Zipcode =~ /^($RegEx)$/;
    return 0;
}

#========================================================================================
# function     CompareZipCodes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $result = CompareZipCodes($ZipCode1,$ZipCode2);
#----------------------------------------------------------------------------------------
# description  Returns result of compare funtion see also: DE_EPAGES::Core::API::PerlTools::Compare
#               The comparison type is determined from the input values.
#----------------------------------------------------------------------------------------
# input        $ZipCode1 | value 1 | *
# input        $ZipCode2 | value 2 | *
# return       $result | 0 if equal / -1 if smaller / 1 if greater | int
#========================================================================================
sub CompareZipCodes {
    my ($ZipCode1,$ZipCode2) = @_;

    return 0 unless defined $ZipCode1 or defined $ZipCode2;
    return 1 if defined $ZipCode1 and not defined $ZipCode2;
    return -1 if not defined $ZipCode1 and defined $ZipCode2;

    # find the type for comparison
    my $Type = 'Integer';
    $Type = 'String' if $ZipCode1 =~ /\D/;
    $Type = 'String' if $ZipCode2 =~ /\D/;

    # remove spaces and convert to lc for correct string comparison
    my $hCodes = {};
    $hCodes->{'ZipCode1'} = $ZipCode1;
    $hCodes->{'ZipCode2'} = $ZipCode2;
    foreach my $Name (qw(ZipCode1 ZipCode2)) {
        next unless $hCodes->{$Name};
        $hCodes->{$Name} = lc($hCodes->{$Name});
        $hCodes->{$Name} =~ s/\s+//g;
    }

    return Compare($Type,$hCodes->{'ZipCode1'},$hCodes->{'ZipCode2'})
}

#========================================================================================
# function     IsValidBankName
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IsValid = IsValidBankName( $BankName );
# example      if ( !IsValidBankName( $FormValues->{'BankName'} ) ) {
#                   $Servlet->form->addFormError({
#                       Reason => 'BankNameTooLong',
#                       Form   => 'SavePaymentDirectDebit',
#                       Name   => 'BankName',
#                   });
#                   $Servlet->form->executeFormError;
#               }
#----------------------------------------------------------------------------------------
# description  Determines if the bank name is valid. A bank name is valid if its
#               encrypted hash is not greater than 255 characters. In order to avoid
#               unnecessary encryptions, the bank name is first checked for non-ascii
#               characters or having a length greater than 64 characters --- which is the
#               maximum length defined for a bank name (cf. EPG-23503). Empty bank names
#               are considered valid.
#----------------------------------------------------------------------------------------
# input        $BankName | a bank name | string
# return       $IsValidBankName | true if the bank name is valid | boolean
#========================================================================================
sub IsValidBankName {
    my ($BankName) = @_;

    return 0 unless defined $BankName;
    if ( length $BankName > MAX_BANKNAME_LENGTH || $BankName =~ /[^[:ascii:]]/ ) {
        my $EncryptedBankName = Encrypt( $BankName );
        if ( length $EncryptedBankName > MAX_ENCRYPTED_BANKNAME_LENGTH ) {
            return 0;
        }
    }

    return 1;
}

#========================================================================================
# function       IsUnambiguousBasketAddress
# state          public
#----------------------------------------------------------------------------------------
# syntax         my $Unambiguous = IsUnambiguousBasketAddress($Basket);
#----------------------------------------------------------------------------------------
# description    Determines if the shipping address in the Basket is either unset or equal
#                 to the billing address.
#                 Returns false if no billing address has been set.
#----------------------------------------------------------------------------------------
# input          $Basket | Basket | Object
# return         $Unambiguous | true if billing address equals shipping address | boolean
#========================================================================================
sub IsUnambiguousBasketAddress {
    my ($Basket) = @_;

    my $BillingAddress = $Basket->get('BillingAddress');
    return 0 unless defined($BillingAddress);

    if ($Basket->get('User')->instanceOf('AnonymousUser')) {
        return 0 if defined($Basket->get('ShippingAddress'));
    }
    else {
        my $ShippingAddress = $Basket->get('ShippingAddress');
        return 0 if(defined($ShippingAddress) && !$BillingAddress->CompareAddress($ShippingAddress));
    }

    return 1;
}

#========================================================================================
# function       SecureAddressFields
# state          public
#----------------------------------------------------------------------------------------
# syntax         my $hValues = SecureAddressFields($hValues);
#----------------------------------------------------------------------------------------
# description    Secures the address fields by removing suspicious characters from
#                 String fields.
#----------------------------------------------------------------------------------------
# input          $hValues | Address values | ref.hash
# return         $hValues | updated Address values | ref.hash
#========================================================================================
sub SecureAddressFields {
    my ($hValues) = @_;

    foreach my $FieldKey (keys %$hValues) {
        if (defined $hValues->{$FieldKey} and ElementOf($FieldKey, ADDRESS_FIELDS)) {
            $hValues->{$FieldKey} = RemoveSuspiciousChars($hValues->{$FieldKey});
        }
    }

    return $hValues;
}

1;
