#========================================================================================
# package      DE_EPAGES::Amazon::API::Object::AmazonOffer
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for AmazonOffer.
#========================================================================================
package DE_EPAGES::Amazon::API::Object::AmazonOffer;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;

use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Core::API::PerlTools qw (fcmp ElementOf);
use DE_EPAGES::Core::API::TestParameter qw ( TestFloat TestString );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_ONLINESTATUS );

sub ErrorDefinition {
    return {
            'MissingUPCEAN' => 'UPC or EAN code missing. You cannot list a product without UPC or EAN code.',
            'MissinASIN'    => 'ASIN code missing. You cannot list a product without an assigned ASIN.',
    };
}



#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $AmazonOffer->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'AmazonOffer' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $AmazonOffer->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $AmazonOffer = DE_EPAGES::Amazon::API::Object::AmazonOffer->insert($hInfo);
# example      my $Class = LoadClassByAlias('AmazonOffer');
#               my $AmazonOffer = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertAmazonOffer' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $AmazonOffer | new object | object
# hook         OBJ_InsertAmazonOffer | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AmazonOfferID - object identifier    - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertAmazonOffer', {'Object' => $self, 'AmazonOfferID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Product CurrencyID AmazonCatalog AmazonProductTypeID PortalConfig AmazonPrice LastStockLevel LastStockUpdate OnlineStatus LastBestPriceUpdate BestPrice IsPriceChanged ParentOffer ASIN ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $AmazonOffer->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteAmazonOffer' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteAmazonOffer | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AmazonOfferID - object identifier    - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteAmazonOffer', {'Object' => $self, 'AmazonOfferID' => $self->id });
    LoadClassByAlias('AmazonOffer')->deleteObjects('ByParentOfferID', [$self->id]);
    LoadClassByAlias('AmazonFeedMessage')->deleteObjects('ByAmazonOfferID', [$self->id]);
    LoadClassByAlias('AmazonSubmitFeedQueue')->deleteObjects('ByAmazonOfferID', [$self->id]);
    $self->set({ 'ParentOffer' => undef });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $AmazonOffer->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateAmazonOffer' and 'OBJ_AfterUpdateAmazonOffer'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateAmazonOffer | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AmazonOfferID - object identifier    - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateAmazonOffer | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AmazonOfferID - object identifier    - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    # find child offers and set there as well.
    foreach my $Child (@{$self->children}){
        $Child->set($hValues, $LanguageID);
    }

    TriggerHook('OBJ_BeforeUpdateAmazonOffer', {'Object'=>$self, 'Values'=> $hValues, 'AmazonOfferID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateAmazonOffer',  {'Object'=>$self, 'Values'=> $hValues, 'AmazonOfferID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     findChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AmazonOffer->findChildren
#----------------------------------------------------------------------------------------
# description  This function returns a list of child offers if they exist.
#----------------------------------------------------------------------------------------
# return       $aAmazonOrders | list reference on AmazonOffers | ref.array.object
#========================================================================================
sub findChildren {
    my $self = shift;
    return [
        map  { $_->{'ObjectID'} }
        # sort by Position and if Position is equal, sort by Alias
        sort { $a->{'Position'} <=> $b->{'Position'} || $a->{'Alias'} cmp $b->{'Alias'} }
        @{__PACKAGE__->dal->get('ByParentOfferID', [$self->id])}];
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Amazon::API::Object::AmazonOffer->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'OnlineStatus' => 0,
        'IsPriceChanged' => 0,
    };
}

#========================================================================================
# function     checkNextState
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Amazon::API::Object::AmazonOffer->checkNextState( $event, $hInfo );
# example      $Offer->checkNextState("start", {}); # trigger event "assign" to transit to next state.
#----------------------------------------------------------------------------------------
# description  Sets AmazonOffer to a new state depending on the existing state and the
#               triggered event
#----------------------------------------------------------------------------------------
# input        $event | event key | string
#               valid events are:
#               <li>start                - start offer (Transition to OFFER_PROCESSING)
#               <li>processResultFailure - failed upload (Transition to OFFER_FAILED)
#               <li>processResultSuccess - successful upload (.. to OFFER_ONLINE)
#               <li>disable              - disable offer (.. to OFFER_HIDDEN)
#               <li>enable               - enable offer (.. to OFFER_ONLINE)
# return       $NewStatus | new status value, undef if there is no change. | integer
#========================================================================================
sub checkNextState {
    my $self = shift;
    my ($event) = @_;
    my $Status = $self->get('OnlineStatus');

    LogDebug('event:', $event);
    LogDebug('OnlineStatus:', $Status);

    my $NewStatus;
    # force valid state changes
    if (ElementOf($Status,
            ($AMAZON_ONLINESTATUS{OFFER_OFFLINE},
             $AMAZON_ONLINESTATUS{OFFER_FAILED},
             $AMAZON_ONLINESTATUS{OFFER_HIDDEN}))) {
        if ($event eq 'start' and $self->get('ASIN')) {
            $self->set({'OnlineStatus' =>
                            ($NewStatus = $AMAZON_ONLINESTATUS{'OFFER_PROCESSING'})});
        }
    }
    elsif ($Status eq $AMAZON_ONLINESTATUS{'OFFER_PROCESSING'}) {
        if ($event eq 'processResultSuccess') {
            $self->set({'OnlineStatus' =>
                            ($NewStatus = $AMAZON_ONLINESTATUS{'OFFER_ONLINE'})});
        } elsif ($event eq 'processResultFailure') {
            $self->set({'OnlineStatus' =>
                            ($NewStatus = $AMAZON_ONLINESTATUS{'OFFER_FAILED'})});
        } elsif ($event eq 'disable') {
            $self->set({'OnlineStatus' =>
                            ($NewStatus = $AMAZON_ONLINESTATUS{'OFFER_HIDDEN'})});
        }
    }
    elsif ($Status eq $AMAZON_ONLINESTATUS{'OFFER_ONLINE'}) {
        if ($event eq 'disable') {
            $self->set({'OnlineStatus' =>
                            ($NewStatus = $AMAZON_ONLINESTATUS{'OFFER_HIDDEN'})});
        } elsif ($event eq 'addPending') {
            $self->set({'OnlineStatus' =>
                            ($NewStatus = $AMAZON_ONLINESTATUS{'OFFER_PROCESSING'})});
        }
    }
    elsif ($Status eq $AMAZON_ONLINESTATUS{'OFFER_HIDDEN'}) {
        if ($event eq 'enable') {
            $self->set({'OnlineStatus' =>
                            ($NewStatus = $AMAZON_ONLINESTATUS{'OFFER_ONLINE'})});
        }
    }

    LogDebug('NewStatus:', $NewStatus);
    return $NewStatus;
}

#========================================================================================
# function     updateValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->updateValues( $hVormValues );
#----------------------------------------------------------------------------------------
# description  Updates AmazonOffer with new form values and communicates changes to
#               Amazon platform.
#----------------------------------------------------------------------------------------
# input        $hFormValues | input form values | hash.ref
#               possible keys are:
#               <li>AmazonPrice</li>
#               <li>AmazonCondition</li>
#               <li>AmazonConditionNote</li>
#               <li>AmazonDeliveryLatency</li>
#               <li>AmazonPrice</li>
# return       null
#========================================================================================
sub updateValues {
    my $self = shift;
    my ($hFormValues) = @_;

    my $hValuesOffer = {};
    my $hValuesProduct = {};

    my $OnlineStatus = $self->get('OnlineStatus');

    my $OldAmazonPrice = $self->get('AmazonPrice') // 0.00;
    my $NewAmazonPrice = $hFormValues->{'AmazonPrice'} // 0.00;
    my $OldAmazonConditionNote = $self->get('AmazonConditionNote') // '';
    my $NewAmazonConditionNote = $hFormValues->{'AmazonConditionNote'} // '';

    if (fcmp($OldAmazonPrice, $NewAmazonPrice) != 0) {
        $hValuesOffer->{'AmazonPrice'} = $self->_roundPrice($hFormValues->{'AmazonPrice'}, $self->get('CurrencyID'));
    }

    my $Product = $self->get('Product');

    if (defined ($hFormValues->{'AmazonDeliveryLatency'}) and
       ($Product->getWithParent('AmazonDeliveryLatency') != $hFormValues->{'AmazonDeliveryLatency'})) {
        $hValuesProduct->{'AmazonDeliveryLatency'} = $hFormValues->{'AmazonDeliveryLatency'};
    }

    if(ElementOf($OnlineStatus,($AMAZON_ONLINESTATUS{'OFFER_OFFLINE'},$AMAZON_ONLINESTATUS{'OFFER_FAILED'}))) {
        if (defined $hFormValues->{'AmazonCondition'} and
           ($Product->getWithParent('AmazonCondition') != $hFormValues->{'AmazonCondition'})) {
            $hValuesProduct->{'AmazonCondition'} = $hFormValues->{'AmazonCondition'};
        }
        if ($OldAmazonConditionNote ne $NewAmazonConditionNote) {
            $hValuesOffer->{'AmazonConditionNote'} = $hFormValues->{'AmazonConditionNote'};
        }
    }

    $self->set($hValuesOffer) if (scalar keys %$hValuesOffer);
    $Product->set($hValuesProduct) if (scalar keys %$hValuesProduct);

    my $oC = $self->get('PortalConfig');

    if($OnlineStatus == $AMAZON_ONLINESTATUS{'OFFER_FAILED'}) {
        # set inventory to zero so that condition can be changed.
        $oC->uploadStock([$self],0) if (scalar keys %$hValuesProduct);
        $self->startOffer();
    }
    elsif ( ElementOf($OnlineStatus, ($AMAZON_ONLINESTATUS{'OFFER_ONLINE'}, $AMAZON_ONLINESTATUS{'OFFER_PROCESSING'})) ) {
        # update stock and price if customer saved settings.
        $oC->uploadStock([$self]) if (scalar keys %$hValuesProduct);
        $oC->uploadPrices([$self]) if (scalar keys %$hValuesOffer);
    };
}

#========================================================================================
# function     startOffer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->startOffer;
#----------------------------------------------------------------------------------------
# description  Uploads an AmazonOffer
#========================================================================================
sub startOffer {
    my $self = shift;

    if (not defined $self->get('ASIN')) {
        Error('MissingASIN',{});
    }

    my $oC = $self->get('PortalConfig');

    # for offline and failed offers we resend the offer feed.
    $self->checkNextState('start');
    $oC->uploadOffers([$self]);
    $oC->uploadStock([$self]);
    $oC->uploadPrices([$self]);
}

#========================================================================================
# function     _roundPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object->_roundPrice($Price,$CurrencyID);
#----------------------------------------------------------------------------------------
# description  Correcly formats price before saving it to the database.
#----------------------------------------------------------------------------------------
# input        $Price | unformatted price | float
# input        $CurrencyID | CurrencyID   | currency
#----------------------------------------------------------------------------------------
# return       $RoundedPrice | rounded price | float
#========================================================================================
sub _roundPrice {
  my $self = shift;
  my ($Price, $CurrencyID) = @_;

  TestFloat('Price',$Price);
  TestString('CurrencyID',$CurrencyID);

  # round price
  my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new();
  my $MinorUnit = GetInfoCurrency($CurrencyID)->{'MinorUnit'};
  return $Formatter->round($Price, $MinorUnit);
}

1;
