#========================================================================================
# package      DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
#----------------------------------------------------------------------------------------
# description  object interface for Payment.
#========================================================================================
package DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use strict;

use DE_EPAGES::AmazonPayments::API::Constants qw (
    AMAZON_REGIONS
    IPN_PATH
);
use DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient;
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountryByCode2 );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::ShopMailType::API::ShopMailType qw ( GetShopMailType );
use DE_EPAGES::Shop::API::ShopUriBuilder;
use DE_EPAGES::AmazonPayments::API::AmazonWidgetUtil;

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethod->featureName;
#----------------------------------------------------------------------------------------
# description   Returns the feature name 'AmazonPayments'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'AmazonPayments'; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethodAmazonPayments = DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethodAmazonPayments');
#               my $PaymentMethodAmazonPayments = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  make sure you can add PaymentMethodAmazonPayments only once to a shop
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PaymentMethodAmazonPayments | new object | object
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    # test if we can use PaymentMethodAmazonPayments as paymentmethod
    my $Shop = $hInfo->{Shop} || $hInfo->{Parent}->getSite();

    # prevent creation of two payment method instances in the same shop
    my $aPaymentMethods = $Shop->get('PaymentMethods');
    foreach my $PaymentMethod (@$aPaymentMethods) {
        if ($PaymentMethod->instanceOf('PaymentMethodAmazonPayments')) {
            Error('NoSecondInstanceAllowed', {'Shop' => $Shop});
        }
    }

    # activate email event
    my $MailType  = GetShopMailType($Shop, 'AmazonPaymentsPaymentDeclined');
    $MailType->set({ 'IsActive' => 1 }) if $MailType;

    return $class->SUPER::insert($hInfo);
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method is visible in the storefront.
#               Overrides function <function DE_EPAGES::Payment::API::Object::PaymentMethod::usableAtStorefront>
#               to evaluate if payment method is configured
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    return unless $self->getSite()->featureMaxValue('AmazonPayments');
    return unless $self->getSite()->get('OrderProcess'); # classic checkout is not supported
    return unless $self->get('CredentialsChecked');
    return if ($self->get('IsDefault') == 1);

    return $self->get('MerchantID') && $self->SUPER::usableAtStorefront($hVars);
}

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               Valid currencies are: EUR, GBP, USD
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>LocaleID      - locale id - string
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    my $RequestedRegion = $self->regionForSF($hVars->{LocaleID}, $hVars->{CurrencyID});
    my $MerchantRegion  = $self->get('AmazonRegion');
    return 0 unless (defined($RequestedRegion) && defined($MerchantRegion));
    return 0 unless ($RequestedRegion eq $MerchantRegion);
    return 0 unless ($self->usableAtStorefront($hVars));
    return $self->SUPER::canAddToBasket($hVars);
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::AmazonPayments::API::Object::LineItemPaymentAmazonPayments->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsInTestMode' => 0
    };
}

#========================================================================================
# function     regions
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRegions = $PaymentMethod->regions;
#----------------------------------------------------------------------------------------
# description  Returns all regions that are supported by Amazon Payments.
#               Use <function activeRegions> to get the regions that are actually usable
#               in the current database.
#----------------------------------------------------------------------------------------
# return       $hRegions | a hash of regions, with region identifiers as keys
#               <ul>
#                 <li>Label | text to display for this region on the region selection element
#                             in the MBO | string</li>
#                 <li>IsActive | true if there are developer keys for this region | boolean</li>
#               </ul> | ref.hash.hash
#========================================================================================
sub regions {
    my $self = shift;

    # determine what regions to display in the MBO
    # filters out regions that do not have all required keys set in the PBO
    my $hAllRegions = AMAZON_REGIONS;
    return {
        map {
            $_ => {
                'Label' => GetInfoCountryByCode2($hAllRegions->{$_}->{'COUNTRY_CODE'})->{'Native'},
            }
        } keys %$hAllRegions
    };
}

#========================================================================================
# function     regionForSF
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $RegionID = $PaymentMethod->regionForSF($LocaleID, $CurrencyID)
#----------------------------------------------------------------------------------------
# description  Returns the region id that should be used with a given locale and currency.
#----------------------------------------------------------------------------------------
# input        $LocaleID | the locale to check | string
# input        $CurrencyID | the currency to check | string
# return       $RegionID | the id of the Amazon Payment Region that matches the locale and currency.
#                           undef if there is no matching region activated | string
#========================================================================================
sub regionForSF {
    my $self = shift;
    my ($LocaleID, $CurrencyID) = @_;

    return unless defined $LocaleID;
    return unless defined $CurrencyID;

    # currently we're only using the language for identification instead of the locale
    my ($LanguageID) = $LocaleID =~ /^(\w{2})/;
    foreach my $Key (keys %{AMAZON_REGIONS()}) {
        my $Region = AMAZON_REGIONS->{$Key};
        return $Key if ElementOf($CurrencyID, $Region->{CURRENCIES})
            and ElementOf($LanguageID, $Region->{LANGUAGES});
    }

    return;
}

#========================================================================================
# function     client
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Client = $PaymentMethod->client($hArgs);
#----------------------------------------------------------------------------------------
# description  Returns an initialized AmazonRestClient
#----------------------------------------------------------------------------------------
# input        $hArgs | additional arguments for the client
#               <ul>
#                 <li>refid | Amazon order reference id (optional) | string</li>
#               </ul> | ref.hash
# return       $Client | AmazonRestClient | DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient
#========================================================================================
sub client {
    my $self = shift;
    my ($hArgs) = @_;

    return DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient->create($self, $hArgs);
}


#========================================================================================
# function     callbackUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Url = $PaymentMethod->callbackUrl;
#----------------------------------------------------------------------------------------
# description  Returns the callbackUrl for instant payment notifications.
#               Only returns a value, if the shop/site has an ssl certificate.
#               Having a vaild (!) SSL certificate is an IPN requirement.
#----------------------------------------------------------------------------------------
# return       $Url | IPN callback url | DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient
#========================================================================================
sub callbackUrl {
    my $self = shift;
    my $ShopUriBuilder = $self->_createShopUriBuilder();
    my $URL = $ShopUriBuilder->buildUri(IPN_PATH);
    return $URL;
}

sub _createShopUriBuilder {
    my $self = shift;
    return DE_EPAGES::Shop::API::ShopUriBuilder->new($self->getSite());
}

#========================================================================================
# function     getAlternativeCheckoutButtonInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAlternativeCheckoutButtonInfo( $Basket );
#----------------------------------------------------------------------------------------
# description  return hash with information about the oneclick button for this paymentmethod.
#               returns undef if preconditions for displaying the button are not met.
#----------------------------------------------------------------------------------------
# input        $Basket | basket object | object
# return       info hash | refhash
#========================================================================================
sub getAlternativeCheckoutButtonInfo {
    my $self = shift;
    my ($Basket) = @_;

    my $hParams = {
        'LocaleID' => $Basket->container()->get('LocaleID'),
        'CurrencyID' => $Basket->container()->get('CurrencyID'),
    };
    if($self->canAddToBasket($hParams) and
       $Basket->get('IsMinOrderAmountReached')) {
        return $self->_getInfoHash($Basket);
    }

    return;
}

sub _getInfoHash {
    my $self = shift;
    my ($Basket) = @_;

    my $DataMerchantId = $self->get('MerchantID');

    my $UrlUtil = $self->_createAmazonWidgetUtil();
    my $Region = $UrlUtil->getActiveAmazonRegion($self, $Basket);
    my $IconUrl = $UrlUtil->getAmazonButtonUrl($self, $Region);
    my $WidgetUrl = $UrlUtil->getAmazonWidgetUrl($self, $Region);

    my $hInfo = {
        'name' => 'amazonpayments',
        'image' => $IconUrl,
        'dataMerchantId' => $DataMerchantId,
        'widgetUrl' => $WidgetUrl
    };

    return $hInfo;
}

sub _createAmazonWidgetUtil {
    my $self = shift;
    return DE_EPAGES::AmazonPayments::API::AmazonWidgetUtil->new();
}

1;
