#==============================================================================
# package      DE_EPAGES::AmazonPayments::Hooks::OrderProcess
# state        private
#==============================================================================
package DE_EPAGES::AmazonPayments::Hooks::OrderProcess;

use strict;
use DE_EPAGES::AmazonPayments::API::Constants qw ( AUTHORIZATION_OPEN AUTHORIZATION_DECLINED );
use DE_EPAGES::AmazonPayments::API::Payment qw (
    ConvertAmazonAddress
    StoreAmazonShippingAddress
);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;

#========================================================================================
# function     OnRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# description  register payment step for checkout process
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    _AmazonLineItem($Servlet->object) or return;
    # register Amazon Payments checkout step
    $BasketOrderProcess->addStep('AmazonAddress', '2000');
    $BasketOrderProcess->addStep('AmazonPayment', '4000');

    return;
}

#========================================================================================
# function     AfterRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# description  set order step; remove payment step if necessary
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object;
    return unless $BasketOrder->instanceOf('Basket');
    my $Container = $BasketOrder->container;
    return unless defined $Container;

    my $pli = $Container->get('Payment');
    return unless defined $pli;

    if ($pli->instanceOf('LineItemPaymentAmazonPayments')) {
        # remove vanilla checkout steps
        $BasketOrderProcess->deleteStep('Address');
        $BasketOrderProcess->deleteStep('Payment');
        $BasketOrderProcess->setOrderStep('Confirmation');
    }
     else {
         $BasketOrderProcess->deleteStep('AmazonAddress');
         $BasketOrderProcess->deleteStep('AmazonPayment');
    }

    return;
}

sub AfterSaveMultiCheckoutConfirmation {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    my $Basket = $Servlet->object;
    my $PaymentLineItem = _AmazonLineItem($Basket) or return;

    # check for AcceptTac is done later, so avoid authorizing the money here.
    # see DE_EPAGES::Order::UI::OrderProcess
    return unless ($Basket->get('IsTacOK') and $Basket->get('IsCustomerInformationOK'));

    eval {
        $PaymentLineItem->authorize();
        # TODO: Check soft-declines / hard declines
        # possibilities: InvalidPaymentMethod -> redirect back to payment selection,
        # otherwise, user will have to select a different payment method.

        # Possible Amazon Transaction Status: AUTHORIZATION_PENDING, AUTHORIZATION_OPEN,
        #                                             AUTHORIZATION_DECLINED, AUTHORIZATION_CLOSED
        my $TransactionStatus = $PaymentLineItem->get('TransStatus');
        Error('AuthorizeDeclined') if ($TransactionStatus eq AUTHORIZATION_DECLINED);
        Error('AuthorizeFailed') if($TransactionStatus ne AUTHORIZATION_OPEN);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code() eq 'ConfirmOrderFailedRecoverable') {
            _CheckAllConstraints($PaymentLineItem, $Servlet);
        } elsif ($Error->code() eq 'AuthorizeDeclined') {
            _handleDeclines($PaymentLineItem, $Servlet);
        } else {
            _handleErrors($Servlet);
        }
    }
    return;
}

#========================================================================================
# function     OnSaveMultiCheckoutAmazonAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSaveMultiCheckoutAmazonAddress($hParams);
#----------------------------------------------------------------------------------------
# description  on saving addressbook - check amazon order reference for constraints and
#               if present put some error messages:
#                 ShippingAddressNotSet -> stay in Address step
#                 AmountNotSet -> ignored, check comes later
#                 PaymentPlanNotSet -> ignored, check comes later
#                 PaymentMethodNotAllowed -> ignored, check comes later
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveMultiCheckoutAmazonAddress {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object();

    my $PaymentLineItem = _AmazonLineItem($Basket) or return;

    my $OrderDetails = $PaymentLineItem->orderDetails;
    _validateShippingAddress($PaymentLineItem, $Servlet); # Do not wrap with eval, because the form error will be catched, then changing the behavior
    eval {

        my $hAddress = ConvertAmazonAddress($PaymentLineItem, $OrderDetails->address);
        _validateCountry($Basket, $PaymentLineItem, $hAddress);

        StoreAmazonShippingAddress($Basket, $hAddress); #Update basket with information from amazon(shipping address/ billing address/ tax)
        $Basket->set({'IsAddressOK' => 1});
    };
    if (ExistsError()) {
        _handleErrors($Servlet);
    }

    return;
}

#========================================================================================
# function     OnViewMultiCheckoutAmazonPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnViewMultiCheckoutAmazonPayment($hParams);
#----------------------------------------------------------------------------------------
# description  on viewing wallet details - check if we come redirected by Strong Customer
#               Authentication (SCA) MFA challenge, since ViewMultiCheckoutAmazonPayment
#               is provided as failureUrl to Amazon
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnViewMultiCheckoutAmazonPayment {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    _CheckAuthenticationStatus($Servlet);

    return;
}
#========================================================================================
# function     OnSaveMultiCheckoutAmazonPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSaveMultiCheckoutAmazonPayment($hParams);
#----------------------------------------------------------------------------------------
# description  on saving wallet details - check amazon order reference for constraints and
#               if present put some error messages:
#                 ShippingAddressNotSet -> back to Address step
#                 AmountNotSet -> setOrderDetails not called yet
#                 PaymentPlanNotSet -> back to Payment step
#                 PaymentMethodNotAllowed -> back to Payment step
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveMultiCheckoutAmazonPayment {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    my $PaymentLineItem = _AmazonLineItem($Servlet->object) or return;
    _CheckAllConstraints($PaymentLineItem, $Servlet);

    return;
}

#========================================================================================
# function     AfterSaveMultiCheckoutShipping
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterSaveMultiCheckoutShipping($hParams);
#----------------------------------------------------------------------------------------
# description  after saving shipping method info - transmit order details to amazon
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterSaveMultiCheckoutShipping {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object;
    my $PaymentLineItem = _AmazonLineItem($Basket) or return;

    eval {
        my $OrderReferenceInfo = $PaymentLineItem->orderDetails();
        if ($OrderReferenceInfo->isInDraftStatus()) {
            $PaymentLineItem->updateOrderDetails;
        }
    };
    if ( ExistsError() ) {
        _handleErrors($Servlet);
    }
    return;
}

#---------------------------------------------------------------------------
#  private getter
#---------------------------------------------------------------------------
sub _AmazonLineItem {
    my ($Basket) = @_;
    my $Container = $Basket->container or return;
    my $PaymentLineItem = $Container->get('Payment') or return;
    return unless $PaymentLineItem->instanceOf('LineItemPaymentAmazonPayments');
    return $PaymentLineItem;
}

sub _CheckAllConstraints {
    my ($PaymentLineItem, $Servlet) = @_;
    my $aConstraints = $PaymentLineItem->orderDetails->constraints;
    if (scalar @$aConstraints) {
        if(ElementOf('ShippingAddressNotSet', $aConstraints)) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonAddress');
            $Servlet->form->executeFormError({
                'Name'   => 'ShippingAddressNotSet',
                'Reason' => 'ShippingAddressNotSet',
            });
        }
        if(ElementOf('PaymentPlanNotSet', $aConstraints)) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonPayment');
            $Servlet->form->executeFormError({
                'Name'   => 'PaymentPlanNotSet',
                'Reason' => 'PaymentPlanNotSet',
            });
        }
        if(ElementOf('PaymentMethodNotAllowed', $aConstraints)) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonPayment');
            $Servlet->form->executeFormError({
                'Name'   => 'PaymentMethodNotAllowed',
                'Reason' => 'PaymentMethodNotAllowed',
            });
        }
        if(ElementOf('AmountNotSet', $aConstraints)) {
            $PaymentLineItem->updateOrderDetails;
        }
    }
}

sub _validateShippingAddress() {
    my ($PaymentLineItem, $Servlet) = @_;
    my $OrderDetails = $PaymentLineItem->orderDetails();

    if(ElementOf('ShippingAddressNotSet', @{$OrderDetails->constraints()})) {
      $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonAddress');
            $Servlet->form->executeFormError({
                'Name'   => 'ShippingAddressNotSet',
                'Reason' => 'ShippingAddressNotSet',
            });
    }
    return;
}

sub _validateCountry {
    my ($Basket, $PaymentLineItem, $hAddress) = @_;

    # check for supported CountryID in shop
    Error('InvalidCountry') unless grep { $_->{'CountryID'} == $hAddress->{'CountryID'} } @{$Basket->getSite->get('Countries')};

    # check for supported CountryID in region.
    my $Region = $PaymentLineItem->get('PaymentMethod')->get('Region');
    if (defined $Region) {
        Error('InvalidCountry') unless grep { $_->{'CountryID'} == $hAddress->{'CountryID'} } @{$Region->get('Countries')};
    }
    return;
}

sub _handleDeclines {
    my ($PaymentLineItem, $Servlet) = @_;

    # see the logic of below options at https://developer.amazon.com/docs/amazon-pay-onetime/handle-decline.html
    my $OrderReferenceInfo = $PaymentLineItem->orderDetails();
    if ($OrderReferenceInfo->isInSuspendedStatus()) {
        $Servlet->vars('ViewAction','ViewMultiCheckoutAmazonPayment');
        $Servlet->form()->executeFormError({ 'Name' => 'AuthorizationDeclined', 'Reason' => 'InvalidPaymentMethod' });
    } elsif ($OrderReferenceInfo->isInOpenStatus()) {
        $PaymentLineItem->cancelOrder();
        $Servlet->vars('AuthenticationStatus', 'Failure');
        $Servlet->vars('ViewAction','ViewMultiCheckoutAmazonPayment');
        $Servlet->form()->executeFormError({ 'Name' => 'AuthorizationDeclined', 'Reason' => 'TransactionTimedOutOrProcessingFailure' });
    } elsif ($OrderReferenceInfo->isInClosedStatus()) {
        $Servlet->vars('AuthenticationStatus', 'Failure');
        $Servlet->vars( 'ViewAction','ViewMultiCheckoutAmazonPayment');
        $Servlet->form()->executeFormError({ 'Name' => 'AuthorizationDeclined', 'Reason' => 'AmazonRejected' });
    } else {
        $Servlet->vars('AuthenticationStatus', 'Failure');
        $Servlet->vars( 'ViewAction','ViewMultiCheckoutAmazonPayment');
        $Servlet->form()->executeFormError({ 'Name' => 'AuthorizationDeclined', 'Reason' => 'UnexpectedStatus' });
    }
}

sub _handleErrors {
    my ($Servlet) = @_;
    my $Error = GetError();
    LogError($Error);

    if($Error->code() eq 'InvalidCountry') {
        $Servlet->vars( 'ViewAction','ViewMultiCheckoutAmazonAddress');
        $Servlet->form()->executeFormError({ 'Name'   => 'UnsupportedShippingAddress', 'Reason' => 'UnsupportedShippingAddress' });

    } else {
        Error('INVALID_FORM', {
            'ViewVars'=> { 'FormError' => 0, 'ViewObjectID' => $Servlet->object()->id(), 'AmazonPaymentsError' => 1 },
            'ViewAction' => 'ViewMultiCheckoutBasket'
        });
    }
    return;
}

sub _CheckAuthenticationStatus {
    my ($Servlet) = @_;

    my $AuthenticationStatus = $Servlet->params('AuthenticationStatus');

    return unless (
            isValid($AuthenticationStatus)
            && ($AuthenticationStatus eq 'Failure' || $AuthenticationStatus eq 'Abandoned')
    );

    $Servlet->vars('AuthenticationStatus', $AuthenticationStatus);

    return;
}

1;
