#========================================================================================
# state        private
#========================================================================================
package DE_EPAGES::AppStore::t::ScriptTagAppClient;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::AppStore::API::Object::ScriptTagAppClient;
use DE_EPAGES::EPagesJ::API::Object::UriTemplate;
use DE_EPAGES::Test::Mock::EpagesObject;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use JSON qw ( from_json to_json );
use Test::Exception;
use Test::MockObject;
use Test::More;

use strict;

## no critic qw(ProhibitLongLines ProhibitMagicNumbers ProhibitImplicitNewlines)

use constant SCOPES => ['scripttags_read','scripttags_write'];
use constant SCRIPT_URL => 'https://www.test.de';
use constant REQUEST_HEADERS => {
    'User-Agent'    => 'epages6 via EpagesUIScriptTags app',
    'Authorization' => 'Bearer abc123'
};

sub test {

    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
        Alias => 'test',
        GUID  => 'shopGUID',
        UnityScriptTagsAppGUID => undef,
    });

    my $Client = DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->create( {
            BaseURL => 'http://localhost:8088/rs-test/',
            Shop    => $Shop,
        }
    );
    $Client->{'HttpClient'} = _mockHTTPClient( {
            'post'   => \&_assertCreate,
            'get'    => \&_assertGet,
            'delete' => \&_assertDelete,
        }
    );

    # check urls
    ## no critic (ProhibitPrivateSubroutineUse)
    like( $Client->_scriptTagsUrl(),   qr#shops/test/script-tags#, 'script tags url OK' );
    like( $Client->_myScriptTagsUrl(), qr#shops/test/script-tags\?myScriptTagsOnly=true#, 'my script tags url OK' );
    like( $Client->singleScriptTagUrl(333), qr#shops/test/script-tags/333#, 'single script tag url OK' );

    ok ( !$Client->existsApp(), 'app not yet created');
    throws_ok(sub { $Client->getApp() }, qr/App 'EpagesUIScriptTags' is not installed/, 'get app without App throws error');

    # create Developer app
    my $privateApp = $Client->createApp();
    is( $privateApp->{'name'},                'EpagesUIScriptTags', 'App name is set to: ' . $privateApp->{'name'} );
    is( $privateApp->{'shopGUID'},            'shopGUID',           'App shopGUID is set to: ' . $privateApp->{'shopGUID'} );
    is( $Shop->get('UnityScriptTagsAppGUID'), '333', 'Shop attribute UnityScriptTagsAppGUID is set to: ' . $Shop->get('UnityScriptTagsAppGUID') );
    ok ( $Client->existsApp(), 'app exists');

    # get app
    is( $Client->getApp()->{'name'}, 'EpagesUIScriptTags', 'App name is set to: ' . $privateApp->{'name'} );;

    # add script tag
    is( $Client->{RequestHeaders}, undef, 'RequestHeaders not yet stored' );
    $Client->addScriptTag(SCRIPT_URL);
    is_deeply( $Client->{RequestHeaders}, REQUEST_HEADERS, 'RequestHeaders stored' );
    ok( $Client->isScriptTagPresent(SCRIPT_URL), 'script-tag with URL "' . SCRIPT_URL . '" exists');
    is( $Client->getScriptTag(SCRIPT_URL)->{scriptTagId}, 12345, 'id of script-tag with URL "' . SCRIPT_URL . '" ok');
    $Client->deleteScriptTag(SCRIPT_URL);

    # delete app
    $Client->deleteApp(333);
    is( $Shop->get('UnityScriptTagsAppGUID'), undef, 'Shop attribute UnityScriptTagsAppGUID unset' );
    is( $Client->{RequestHeaders}, undef, 'RequestHeaders deleted' );
}

sub _mockHTTPClient {
    my ($hMockMethods) = @_;
    my $mockRemote = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

sub _assertGet {
    shift;

    my ( $URL ) = @_;

    if ($URL->abs =~ m:/private-apps:) {
        return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef,
            '{ "privateAppId": 333, "name": "EpagesUIScriptTags","callbackUrl": null, "shopGUID": "shopGUID", "token": "abc123" }'
        );
    } elsif ($URL->abs =~ m:/script-tags:) {
        return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef,
            '{
                "items": [
                    {
                      "scriptTagId": 12345,
                      "scriptUrl": "' . SCRIPT_URL . '"
                    }
                ]
            }'
        );
    }
}

sub _assertDelete {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    if ($URL->abs =~ m:/private-apps:) {
        is( $URL->abs, 'http://localhost:8088/rs-test/appstore/shopGUID/private-apps/333',
            'path for delete app' );
    } elsif ($URL->abs =~ m:/script-tags:) {
        is( $URL->abs, 'http://localhost:8088/rs-test/shops/test/script-tags/12345',
            'path for delete script-tag' );
    }
    return HTTP::Response->new( HTTP_NO_CONTENT, status_message(HTTP_NO_CONTENT),
                                undef, undef );
}

sub _assertCreate {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    if ($URL->abs =~ m:/private-apps:) {
        is( $URL->abs, 'http://localhost:8088/rs-test/appstore/shopGUID/private-apps',
            'post path for create script tag app' );
        my $ExpectedResult = to_json({ 'shopGUID' => 'shopGUID', 'callbackUrl' => undef,
                                       'name' => 'EpagesUIScriptTags', 'scopes' => SCOPES,
                                     }, {pretty => 1});
        is( join('', sort map { s/,//g; $_ }
                 split(/\n/, to_json(from_json($Content), {pretty => 1}))),
            join('', sort map { s/,//g; $_ } split(/\n/, $ExpectedResult)),
            'post contents (private-apps)' );
        return HTTP::Response->new( HTTP_CREATED, status_message(HTTP_CREATED), undef,
            <<'EOL'
{ "privateAppId": 333,
    "name": "EpagesUIScriptTags",
    "callbackUrl": "callback",
    "shopGUID": "shopGUID",
    "credentials" : {
           "type": "oauth1",
           "consumerKey": "demo_consumer",
           "consumerSecret": "demo_consumer_key",
           "accessTokenKey": "demo_token",
           "accessTokenSecret": "demo_token_key"
    }
}
EOL
        );
    } elsif ($URL->abs =~ m:/script-tags:) {
        is( $URL->abs, 'http://localhost:8088/rs-test/shops/test/script-tags',
            'post path for add script tag' );
        my $ExpectedResult = to_json({ 'scriptUrl' => 'https://www.test.de' });
        is( join('', sort map { s/,//g; $_ } split(/\n/, $Content)),
            join('', sort map { s/,//g; $_ } split(/\n/, $ExpectedResult)),
            'post contents (script-tags)' );
        return HTTP::Response->new( HTTP_CREATED, status_message(HTTP_CREATED));
    }
}

__PACKAGE__->runScript;
done_testing();
