#========================================================================================
# package      DE_EPAGES::AvaTax::API::AvaTax
# base         Exporter
# state        public
# description  public API for AvaTax
#========================================================================================
package DE_EPAGES::AvaTax::API::AvaTax;
use base Exporter;

use strict;
use Data::Dumper;

use DE_EPAGES::AvaTax::API::Constants qw (
    AVATAX_RESULTCODE_Error
    AVATAX_RESULTCODE_Exception
    AVATAX_RESULTCODE_Success
    AVATAX_TEXTCASE_Default
);
use DE_EPAGES::AvaTax::API::DTO::Address;
use DE_EPAGES::AvaTax::API::DTO::BaseAddress;
use DE_EPAGES::AvaTax::API::DTO::GetTaxRequest;
use DE_EPAGES::AvaTax::API::DTO::Line;
use DE_EPAGES::AvaTax::API::DTO::ValidateRequest;
use DE_EPAGES::AvaTax::API::Mail qw ( SendServiceErrorMail );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw ( GetLog LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf GetMD5Digest IsA isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByGUID LoadRootObject );
use DE_EPAGES::Object::API::Table::Country qw ( ExistsCountry GetInfoCountry );
use DE_EPAGES::AvaTax::API::GetTaxRequestBuilder;
use DE_EPAGES::AvaTax::API::RemoteConnector;
use DE_EPAGES::AvaTax::API::OrderExtractor;

sub ErrorDefinition {
    return {
        'AvaTaxRequestFailure'   => 'AvaTax Request Failure. ResultCode: #ResultCode. Summary: #Summary. Details: #Details',
        'AvaTaxCalculationError' => 'AvaTax Tax Calculation failed. #ResultCode. Summary: #Summary. Details: #Details',
        'ItemSizeMismatch'       => 'AvaTax count of lineitems (#ReturnedLineItems) does not match local count of lineitems (#ExpectedLineItems)',
        'InvalidObject'          => 'Invalid Object GUID in AvaTax request. #ObjectGUID',
        'NoSuchItem'             => 'No item with number #No found in AvaTax result.',
        'NoTaxMethod'            => 'No TaxMethod defined. ShopAlias:#ShopAlias,Store:#Store',
    };
}

our @EXPORT_OK = qw (
    UpdateTax
    Validate
    UseAvaTax
    NotifyShopOwner
    IsAuthorized
    Ping
    AllowTaxChange
);

our $ALLOW_LINEITEMTAX_CHANGE = 0;

#========================================================================================
# function     AllowTaxChange
# state        public
#----------------------------------------------------------------------------------------
# syntax       AllowTaxChange( );
#----------------------------------------------------------------------------------------
# description  Returns true if lineitemavatax items may be modified in attributes
#               TaxRate and TaxClass
#----------------------------------------------------------------------------------------
# return       $AllowChange | true, if attributes may be changed | boolean
#========================================================================================
sub AllowTaxChange {
    return $ALLOW_LINEITEMTAX_CHANGE;
}



#========================================================================================
# function     IsAuthorized
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsAuthorized( $Shop, $Service, $Username, $Password, $IsInTestMode );
#----------------------------------------------------------------------------------------
# description  Tests Authorization for AvaTax Webservice
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $Service | which AvaTax webservice | string
# input        $Username | Webservice username | string
# input        $Password | Webservice password | string
# input        $IsInTestMode | connect to Webservice in test mode | Boolean
# return       $Result | webservice result | object
#========================================================================================
sub IsAuthorized {
    my ($Shop, $Service, $Username, $Password, $IsInTestMode) = @_;

    my $RemoteConnector = DE_EPAGES::AvaTax::API::RemoteConnector->new($Shop);

    my $Remote;
    if ($Service eq 'AddressService') {
        $Remote = $RemoteConnector->connectToAddressServiceWithConfiguration($Username, $Password, $IsInTestMode);
    } else {
        $Remote = $RemoteConnector->connectToTaxServiceWithConfiguration($Username, $Password, $IsInTestMode);
    }

    return $Remote->isAuthorized();
}



#========================================================================================
# function     Ping
# state        public
#----------------------------------------------------------------------------------------
# syntax       Ping( $Shop, $Service, $Username, $Password, $IsInTestMode );
#----------------------------------------------------------------------------------------
# description  Tests Ping for AvaTax Webservice
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $Service | which AvaTax webservice | string
# input        $Username | Webservice username | string
# input        $Password | Webservice password | string
# input        $IsInTestMode | connect to Webservice in test mode | Boolean
# return       $Result | webservice result | object
#========================================================================================
sub Ping {
    my ($Shop, $Service, $Username, $Password, $IsInTestMode) = @_;

    my $RemoteConnector = DE_EPAGES::AvaTax::API::RemoteConnector->new($Shop);

    my $Remote;
    if ($Service eq 'AddressService') {
        $Remote = $RemoteConnector->connectToAddressServiceWithConfiguration($Username, $Password, $IsInTestMode);
    } else {
        $Remote = $RemoteConnector->connectToTaxServiceWithConfiguration($Username, $Password, $IsInTestMode);
    }

    return $Remote->ping();
}



#========================================================================================
# function     UseAvaTax
# state        public
#----------------------------------------------------------------------------------------
# syntax       UseAvaTax( $Shop, $hAddressInfo );
#----------------------------------------------------------------------------------------
# description  Check if we will use AvaTax for this order.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $hAddressInfo | Address hash. used keys <ul>
#                 <li>CountryID - ISO CountryID </li>
#               </ul> | ref.hash
# return       $UseAvaTax | use code. Only use AvaTax if this is 1.
#                1 if AvaTax should be used.
#               -1 if LineItemClass does not match
#               -2 If TaxMethod does not match
#               -3 if Address Country does not match
#                | integer
#========================================================================================
sub UseAvaTax {
    my ($Shop, $hAddressInfo) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    # used AvaTax taxmatrix?
    my $LineItemClass = $Shop->get('TaxMatrix')->get('LineItemClass');
    return -1 unless (
        defined($LineItemClass) && $LineItemClass->alias eq 'LineItemAvaTax'
    );

    my $TaxMethod = $Shop->get('TaxMethod');
    return -2 ## no critic qw(ProhibitMagicNumbers)
        unless defined($TaxMethod) &&
        IsA($TaxMethod, 'DE_EPAGES::AvaTax::API::Object::TaxMethodAvaTax');

    return 1;
}



#========================================================================================
# function     UpdateTax
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateTax( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  calls GetTax for a lineitemcontainer and updates lineitemtax entries
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | lineitem container | object
#========================================================================================
sub UpdateTax {
    my ($LineItemContainer) = @_;
    TestObject('LineItemContainer', $LineItemContainer, 'DE_EPAGES::Order::API::Object::LineItemContainer');
    my $Shop            = $LineItemContainer->getSite;

    my $CustomerOrder = $LineItemContainer->parent();
    my $OrderExtractor = DE_EPAGES::AvaTax::API::OrderExtractor->new();
    my $WasDataExtractedSuccessful = $OrderExtractor->extractDataFrom($CustomerOrder);
    return unless $WasDataExtractedSuccessful;

    my $RequestBuilder = DE_EPAGES::AvaTax::API::GetTaxRequestBuilder->new();
    my $Request = $RequestBuilder->buildRequest($OrderExtractor);

    my $AvataxTaxExemption = $OrderExtractor->getAvataxTaxExemption();
    if(isValid($AvataxTaxExemption)) {
        my $CustomerUsageType = $AvataxTaxExemption->alias();
        $Request->CustomerUsageType($CustomerUsageType);
    }

    # check if we have lineitems in the request
    return unless (scalar @{$Request->LineList()});

    my $CurrentHash     = $LineItemContainer->get('AvaTax_Hash');
    my $RequestHash     = _CreateRequestHash($Request);
    GetLog()->debug('UpdateTax // Old hash ' . ($CurrentHash//'') . ' new hash ' . ($RequestHash //''));
    return if (defined($CurrentHash) && $CurrentHash eq $RequestHash);

    my $RemoteConnector = DE_EPAGES::AvaTax::API::RemoteConnector->new($Shop);
    my $Remote = $RemoteConnector->connectToTaxService();
    my $Result = $Remote->getTax($Request);
    my $hAvataxGuidMap = $OrderExtractor->getAvataxGuidMap();

    if ($Result->ResultCode() eq AVATAX_RESULTCODE_Success) {
        $LineItemContainer->set({'AvaTax_Hash' => $RequestHash});
        local $DE_EPAGES::AvaTax::API::AvaTax::ALLOW_LINEITEMTAX_CHANGE = 1;
        _UpdateLineItemTaxes($LineItemContainer, $Request, $Result, $hAvataxGuidMap);
        return;
    }
    if ($Result->ResultCode() eq AVATAX_RESULTCODE_Exception) {
        if (scalar @{$Result->Messages()}) {
            my $Message = $Result->popMessage;
            # my $MessageAttributes = $Message->Attributes;
            Error('AvaTaxRequestFailure',{
                'ResultCode' => $Result->ResultCode,
                'Messages'   => $Result->Messages(),
                'Summary'    => $Message->Summary,
                'Details'    => $Message->Details,
            });
        } else {
            Error('AvaTaxRequestFailure',{
                'ResultCode' => $Result->ResultCode,
            });
        }
    } elsif ($Result->ResultCode() eq AVATAX_RESULTCODE_Error) {
        if (!scalar @{$Result->Messages()}) {
            Error('AvaTaxRequestFailure',{
                'ResultCode' => $Result->ResultCode,
            });
        }
    }

}



#----------------------------------------------------------------------------------------
# private helpers
#----------------------------------------------------------------------------------------
##no critic (RequireInterpolationOfMetachars)
#========================================================================================
# function     _UpdateLineItemTaxes
# state        private
#----------------------------------------------------------------------------------------
# syntax       _UpdateLineItemTaxes( $LineItemContainer, $Request, $Result, $AvataxGuidMap);
#----------------------------------------------------------------------------------------
# description  Updates LineItemTax entries for LineItemContainer
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | lineitemcontainer | object
# input        $Request | GetTax request | object
# input        $Result | GetTax result | object
# input        $AvataxGuidMap | AvataxGuidMap | DE_EPAGES::AvaTax::API::AvataxGuidMap
# $AvataxGuidMap $AvataxGuidMap | Map to get the GUID | object
#========================================================================================
sub _UpdateLineItemTaxes {
    my ($LineItemContainer, $Request, $Result, $hAvataxGuidMap) = @_;

    my @RequestItems = @{$Request->LineList};
    LogDebug( q{\@RequestItems:}, \@RequestItems);
    my @ResultItems  = @{$Result->TaxLineList};
    LogDebug( q{\@ResultItems:}, \@ResultItems);

    Error('ItemSizeMismatch',{
        'ExpectedLineItems' => scalar @RequestItems,
        'ReturnedLineItems' => scalar @ResultItems,
    }) unless (scalar @RequestItems eq scalar @ResultItems);

    # reset taxes
    foreach my $LineItemTax (@{$LineItemContainer->get('Taxes')}){
        $LineItemTax->set({
            'TaxAmount' => 0.00,
            'TaxRate'   => 0.00,
        });
    }

    my $Formatter = $LineItemContainer->formatter;

    foreach my $RequestItem (@RequestItems){
        # find matching result
        my $ResultItem = _findResultItem($RequestItem->No(),\@ResultItems);
        LogDebug( q{$ResultItem->Tax}, $ResultItem->Tax);
        my $GUID = $hAvataxGuidMap->{$RequestItem->No()};
        my $LineItem   = LoadObjectByGUID($GUID);
        Error('InvalidObject',{
            'ObjectGUID' => $RequestItem->ItemCode
        }) unless (IsA($LineItem, 'DE_EPAGES::Order::API::Object::LineItem') && $LineItem->parent eq $LineItemContainer);
        my $TaxClass = $LineItem->get('TaxClass');
        next unless (defined $TaxClass);
        my $LineItemClass = $TaxClass->get('TaxMatrix')->get('LineItemClass');
        # FIXME: This could potentially create a new LineItemTax.
        my $LineItemTax = $LineItemContainer->setLineItem($LineItemClass, {
            'TaxClass' => $TaxClass,
        });
        my $NewTaxAmount = $Formatter->round($LineItemTax->get('TaxAmount')//0)
                         + $Formatter->round($ResultItem->Tax);
        $NewTaxAmount    = $Formatter->round($NewTaxAmount);

        my $NewBasePrice = $Formatter->round($LineItemTax->get('BasePrice')//0)
                         + $Formatter->round($ResultItem->Taxable);
        $NewBasePrice    = $Formatter->round($NewBasePrice);

        LogDebug( q{$NewTaxAmount:}, $NewTaxAmount);
        $LineItemTax->set({
            'TaxAmount'     => $NewTaxAmount,
            'LineItemPrice' => $NewTaxAmount,
            'BasePrice'     => $NewBasePrice,
            'TaxRate'       => $ResultItem->Rate(),
        });
        $LineItem->set({
            'TaxRate'   => $ResultItem->Rate(),
        });
    }
}



#========================================================================================
# function     Validate
# state        public
#----------------------------------------------------------------------------------------
# syntax       Validate( $Address );
#----------------------------------------------------------------------------------------
# description  Validates an address with AvaTax Address service
#               The actual input values are taken from the Form element, if provided.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# input        $Address | Address object | object
# input        $Form | form | object
# return       $Result | Validation Result | object
#========================================================================================
sub Validate {
    my ($Shop, $Address, $Form) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    my $Request = _CreateValidateRequest($Form);
    my ($CurrentHash, $RequestHash);
    if (defined $Address) {
        $CurrentHash = $Address->get('AvaTax_Hash');
        $RequestHash = _CreateRequestHash($Request);
    }

    # TODO: enable caching of address validation and do not repatedly validate
    # addresses, even though they might be invalid (request from Avalara.)
    # this is currently not possible, as we do not have the $Address object
    # yet.
    GetLog()->debug('Validate // Old hash ' . ($CurrentHash//'') . ' new hash ' . ($RequestHash //''));
    if (isValid($CurrentHash) && $CurrentHash eq $RequestHash) {
        my $MockResult = DE_EPAGES::AvaTax::API::DTO::ValidateResult->new();
        $MockResult->ResultCode(AVATAX_RESULTCODE_Success);
        return $MockResult;
    }

    my $RemoteConnector = DE_EPAGES::AvaTax::API::RemoteConnector->new($Shop);
    my $Remote = $RemoteConnector->connectToAddressService();

    my $Result = $Remote->validate($Request);

    $Address->set({'AvaTax_Hash' => $RequestHash}) if (defined $Address);

    if ($Result->ResultCode() eq AVATAX_RESULTCODE_Success) {
        # update hash. return validation result
        return $Result;
    }
    if ($Result->ResultCode() eq AVATAX_RESULTCODE_Error) {
        # don't update hash, but return validation result
        return $Result;
    }
    if ($Result->ResultCode() eq AVATAX_RESULTCODE_Exception) {
        if (scalar @{$Result->Messages()}) {
            my $Message = $Result->popMessage;
            # my $MessageAttributes = $Message->Attributes;
            Error('AvaTaxRequestFailure',{
                'ResultCode' => $Result->ResultCode,
                'Messages'   => $Result->Messages(),
                'Summary'    => $Message->Summary,
                'Details'    => $Message->Details,
                'Result'     => $Result,
            });
        }
    }
    Error('AvaTaxRequestFailure',{
        'ResultCode' => $Result->ResultCode,
        'Result'     => $Result,
    });
}



#========================================================================================
# function     NotifyShopOwner
# state        public
#----------------------------------------------------------------------------------------
# syntax       NotifyShopOwner( $ErrorCode );
#----------------------------------------------------------------------------------------
# description  Notfies shop owner of a problem
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# input        $ErrorCode | Error code | string
#========================================================================================
sub NotifyShopOwner {
    my ($Shop, $ErrorCode) = @_;
    my $System = LoadRootObject();
    my $MailType = $System->folder('MailTypes')->child('ErrorAvaTaxService');
    GetLog()->debug('error code: ' . $ErrorCode);
    SendServiceErrorMail($Shop, $MailType, $ErrorCode);
}



#========================================================================================
# function     _CreateValidateRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _CreateValidateRequest( $Form );
#----------------------------------------------------------------------------------------
# description  creates a ValidateRequest object from form input
#----------------------------------------------------------------------------------------
# input        $Form | form input | object
# return       $Request | request object | object
#========================================================================================
sub _CreateValidateRequest {
    my ($Form) = @_;
    # my $hShippingAttrs = _GetShippingAddress($Address);
    my $Request = DE_EPAGES::AvaTax::API::DTO::ValidateRequest->new;
    $Request->TextCase(AVATAX_TEXTCASE_Default);
    # to transmit both addreses, use ('Shipping','') - but currently
    # we don't get any error when at least one address validates
    my @LoopShipping;
    # check for Address.* fields like used in our booking system
    if ($Form->value('Address.CountryID')) {
        @LoopShipping = ('Address.');
    } else {
        @LoopShipping = $Form->value('EnableShippingAddress') ? ('Shipping'):('');
    }
    foreach my $Prefix (@LoopShipping) {
        my $Dest = DE_EPAGES::AvaTax::API::DTO::Address->new;
        $Dest->AddressCode($Prefix.'Address');
        $Dest->Line1($Form->value($Prefix.'Company')//$Form->value($Prefix.'FullName'));
        $Dest->Line2($Form->value($Prefix.'Street'));
        $Dest->Line3($Form->value($Prefix.'Street2'));
        $Dest->City($Form->value($Prefix.'City'));
        # disallow the one always-validating zipcode.
        if ($Form->value($Prefix.'Zipcode') eq '12345') {
            Error('MissingShippingZipCode', { } );
        }
        $Dest->PostalCode($Form->value($Prefix.'Zipcode'));
        $Dest->Region($Form->value($Prefix.'State'));
        $Dest->Country(ExistsCountry($Form->value($Prefix.'CountryID')) ?
        GetInfoCountry($Form->value($Prefix.'CountryID'))->{'Code3'} : '');
        $Request->pushAddress($Dest);
    }

    return $Request;
}



#========================================================================================
# function     _CreateValidateRequestFromAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _CreateValidateRequestFromAddress( $Address );
#----------------------------------------------------------------------------------------
# description  creates a ValidateRequest object. Will be used for validating an address
#               coming from the database.
#----------------------------------------------------------------------------------------
# input        $Address | address | object
# return       $Request | request object | object
#========================================================================================
sub _CreateValidateRequestFromAddress { ## no critic (ProhibitUnusedPrivateSubroutines)
    my ($Address) = @_;
    my $hShippingAttrs = _GetShippingAddress($Address);
    my $Request = DE_EPAGES::AvaTax::API::DTO::ValidateRequest->new;
    $Request->TextCase(AVATAX_TEXTCASE_Default);
    my $Dest = DE_EPAGES::AvaTax::API::DTO::Address->new;
    $Dest->Line1($hShippingAttrs->{'Company'}//$hShippingAttrs->{'FullName'});
    $Dest->Line2($hShippingAttrs->{'Street'});
    $Dest->Line3($hShippingAttrs->{'Street2'});
    $Dest->City($hShippingAttrs->{'City'});
    $Dest->PostalCode($hShippingAttrs->{'Zipcode'});
    $Dest->Country($hShippingAttrs->{'Country'}->{'Code3'});

    $Request->pushAddress($Dest);
    return $Request;
}



sub _GetShippingAddress {
    my ($ShippingAddress) = @_;
    Error('MissingShippingAddress', { } )  unless defined $ShippingAddress;
    my $hInfo = $ShippingAddress->get([qw( Company FullName Street Street2 Zipcode City State Country )]);

    Error('MissingShippingStreet', { } )   unless $hInfo->{'Street'};
    Error('MissingShippingZipCode', { } )  if (!$hInfo->{'Zipcode'} || $hInfo->{'Zipcode'} eq '12345');
    Error('MissingShippingCity', { } )     unless $hInfo->{'City'};
    Error('MissingShippingCountry', { } )  unless $hInfo->{'Country'};
    # US or CA
    Error('ShippingCountryMismatch')
        unless(ElementOf($hInfo->{'Country'}->{'Code2'}, qw(US CA)));
    return $hInfo;
}



sub _findResultItem {
    my ($No, $aResultItems) = @_;
    foreach my $Item (@{$aResultItems}){
        return $Item if ($Item->No() eq $No);
    }
    Error('NoSuchItem',{
        'No' => $No,
    });
}



sub _CreateRequestHash {
    my ($Request) = @_;
    my $Dump = join('', sort map { s/,//g; } split(/\n/, Dumper($Request)));
    return GetMD5Digest(\$Dump);
}

1;
