#========================================================================================
# package      DE_EPAGES::AvaTax::Hooks::Address
# base         DE_EPAGES::base
# state        public
#----------------------------------------------------------------------------------------
# description  Gets called for Address validation
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package DE_EPAGES::AvaTax::Hooks::Address;

use strict;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::AvaTax::API::AvaTax qw ( UseAvaTax Validate NotifyShopOwner );
use DE_EPAGES::Core::API::PerlTools qw ( IsA ElementOf );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Core::API::Log qw ( GetLog LogDebug );
use DE_EPAGES::AvaTax::API::Constants qw ( $AVATAX_COUNTRIES_SUPPORTING_ADDRESS_VALIDATION );

sub ErrorDefinition {
    return {
            'TaxMethodNotUsableAtStorefront' => 'TaxMethod #TaxMethod not usable at storefront. Shop:#Shop, Store:#Store',
    };
}

#========================================================================================
# function     OnAddressValidation
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAddressValidation( $hParams );
#----------------------------------------------------------------------------------------
# description  Does address validation using the avatax webservice
#----------------------------------------------------------------------------------------
# input        $hParams | Input hash. Used keys:
# <ul>
#               <li>Servlet - Servlet</li>
# </ul> | hash.ref
#========================================================================================
sub OnAddressValidation {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};
    TestObject('Servlet',$Servlet,'DE_EPAGES::Presentation::Servlet::Permission');
    GetLog()->debug('OnAddressValidation');

    my $Address;
    my $Object = $Servlet->object;
    if (IsA($Object, 'DE_EPAGES::Address::API::Object::Address')) {
        $Address = $Object;
    } elsif (IsA($Object, 'DE_EPAGES::Order::API::Object::Basket') or
             IsA($Object, 'DE_EPAGES::Order::API::Object::CustomerOrder')
    ) {
        $Address = $Object->get('ShippingAddress') // $Object->get('BillingAddress');
    }

    unless ( defined $Address ){
        GetLog()->debug('undefined address for object of class '.$Object->class->alias);
    }

    my $Result;
    my $Form = $Servlet->form;
    my $hForm = $Servlet->form->values();

    LogDebug( q{$hForm:}, $hForm); ## no critic (RequireInterpolationOfMetachars)
    my $Shop = $Object->getSite;

    my ($CountryID, $Prefix);
    if ($Form->value('EnableShippingAddress')) {
        $CountryID = $Form->value('ShippingCountryID');
        $Prefix = 'Shipping';
    } elsif( $Form->value('Address.CountryID')) {
        # check for Address.* fields like used in our booking system
        $CountryID = $Form->value('Address.CountryID');
        $Prefix = 'Address.';
    } else {
        $CountryID = $Form->value('CountryID');
        $Prefix = '';
    }
    my $UseAvaTax = UseAvaTax($Shop, {'CountryID' => $CountryID});
    GetLog()->debug('UseAvaTax : '.  $UseAvaTax);
    return if ($UseAvaTax <= 0 );

    eval {
        my $TaxMethod = $Shop->get('TaxMethod');
        Error('TaxMethodNotUsableAtStorefront',{
            'TaxMethod' => $TaxMethod->alias,
            'Shop'      => $Shop->alias,
            'Store'     => GetCurrentDBHandle->cacheKey,
        }) unless ($TaxMethod->usableAtStorefront());

        if(defined($CountryID) and ElementOf($CountryID, $AVATAX_COUNTRIES_SUPPORTING_ADDRESS_VALIDATION)) {
            $Result = Validate($Shop, $Address, $Form);
        } else {
            $Result = 1;
        }
    };
    # error -> validation exception
    if (ExistsError()) {
        _HandleValidationException(GetError(), $Shop, $Form, $Prefix);
    } elsif (! $Result) {
        $Form->executeFormError({
            'Name'   => 'ServiceFailure',
            'Reason' => 'ServiceFailure'
        });
    }

    return;
}

#========================================================================================
# function     _HandleValidationException
# state        private
#----------------------------------------------------------------------------------------
# syntax       _HandleValidationException( $Error, $Shop, $Form );
#----------------------------------------------------------------------------------------
# description  handle exceptions, e.g. service excpetion, wrong login data etc.
#----------------------------------------------------------------------------------------
# input        $Error | error to handle | error object
# input        $Shop | shop | object
# input        $Form | form of current servlet | form
# input        $Prefix | prefix for form input fields | string
#========================================================================================
sub _HandleValidationException {
    my ($Error, $Shop, $Form, $Prefix) = @_;
    my %ErrorFieldMapping = (
        'MissingShippingStreet'   => 'Street',
        'MissingShippingZipCode'  => 'Zipcode',
        'MissingShippingCity'     => 'City',
        'MissingShippingCountry'  => 'Country',
    );

    my $ErrorCode = $Error->code;
    if ($ErrorCode eq 'AvaTaxRequestFailure') {
        NotifyShopOwner($Shop, $ErrorCode);
        Error($Error);
    }
    elsif ($ErrorCode eq 'TaxMethodNotUsableAtStorefront') {
        NotifyShopOwner($Shop, $ErrorCode);
        Error($Error);
    }
    elsif (defined $ErrorFieldMapping{$ErrorCode}) {
        $Form->executeFormError(
            { 'Name'   => "$Prefix$ErrorFieldMapping{$ErrorCode}",
              'Reason' => 'MissingInput', }
        );
    }
    elsif (ElementOf($ErrorCode,
               qw(MissingShopAddress MissingShopStreet MissingShopZipCode
                  MissingShopCity MissingShopCountry ShopCountryMismatch))) {
        NotifyShopOwner($Shop, $ErrorCode);
    }
    else {
        $Form->executeFormError(
            { 'Name'   => $ErrorCode,
              'Reason' => 'ServiceFailure', }
        );
    }
}

1;
