#========================================================================================
# package      DE_EPAGES::CSVExportImport::API::CSV::ObjectExportDriver;
# base         DE_EPAGES::Presentation::API::CSV::MonitoredExportDriver
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to export objects via csv
# example      use DE_EPAGES::CSVExportImport::API::CSV::ObjectExportDriver;
#               my @Attributes;
#               push @Attributes, {
#                   Alias => 'Alias',
#                   Type => 'String',
#                   HeaderString => 'Alias',
#               };
#               push @Attributes, {
#                   Alias => 'Name',
#                   Type => 'String',
#                   HeaderString => 'Name (en)',
#                   LanguageID => GetPKeyLanguageByCode('en'),
#               };
#               push @Attributes, {
#                   Alias => 'ListPrice',
#                   HeaderString => 'ListPrice (EUR)',
#                   Type => 'Money'
#               };
#               my $Exporter = DE_EPAGES::CSVExportImport::API::CSV::ObjectExportDriver->new;
#               my $Shop = LoadObjectByPath( '/Shops/DemoShop' );
#               $Exporter->exportCSV(
#                   'Products.csv',
#                   { LocaleID => 'de_DE' },
#                   \@Attributes,
#                   $Shop,
#                   $Shop->get('Products')
#               );
#========================================================================================
package DE_EPAGES::CSVExportImport::API::CSV::ObjectExportDriver;
use base qw ( DE_EPAGES::Presentation::API::CSV::MonitoredExportDriver Exporter );

use strict;
use DE_EPAGES::CSVExportImport::API::Constants qw( @FLOAT_TYPES @DATETIME_TYPES);
use DE_EPAGES::Core::API::PerlTools qw( ElementOf GetDoubleHashFromLoop );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Object::API::Factory qw(DisableObjectCache ExistsClassByAlias LoadRootObject);
use DE_EPAGES::Object::API::Table::Language qw(GetInfoLanguagesByCode2);
use DE_EPAGES::WebInterface::API::ApplicationServer qw ( GetAppServer );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NO_ITEMS'          => 'CSV file: no items to export',
    };
}
#ERRORDEFINITION

#========================================================================================
# function     exportCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exporter->exportCSV($FileName, $hFormat, $aColumns, $ViewObject, $aObjects );
#----------------------------------------------------------------------------------------
# description  exports object attributes to a CVS file.
#----------------------------------------------------------------------------------------
# input        $FileName |  export file | string
# input        $hFormat | export file format
#                   <ul>
#                       <li>LocaleID - locale id, e.g. 'de_DE' - string
#                   </ul>
#                   The following parameters are optional. Their default values are
#                   determined by the given locale id:
#                   <ul>
#                       <li>eol - row separator, e.g. "\n" - string</li>
#                       <li>sep_char - quote character, e.g. ',' - string</li>
#                       <li>quote_char - quote character, e.g. '"' - string</li>
#                       <li>escape_char - escape character, e.g. '"' - string</li>
#                       <li>DecimalSep - decimal point, e.g. '.' - string</li>
#                       <li>ThousandSep - thousands separator, e.g. ',' - string</li>
#                       <li>encoding - character encoding, e.g. 'iso-8859-1' - string</li>
#                   </ul> | ref.hash
# input        $ahColumns | list of column attributes (used for header and values)
#                   <ul>
#                       <li>Alias - attribute alias - string</li>
#                       <li>Type - attribute type (see <function DE_EPAGES::CSVExportImport::API::Tools::FormatValue>) - string</li>
#                       <li>HeaderString - column name (for the header line) - string</li>
#                       <li>LanguageID - language id, required for localizable attributes - integer</li>
#                   </ul>  | ref.array.hash
# input        $ViewObject | object to view monitor | object
# input        $aObjects | list of objects | ref.array.object
#========================================================================================
sub exportCSV {
    my $self = shift;
    my ($FileName, $hFileFormat, $aColumns, $ViewObject, $aObjects) = @_;

    $self->{'Object'} = $ViewObject;
    if (exists $hFileFormat->{'LocaleID'}) {
        my $Site = defined $ViewObject ? $ViewObject->getSite() : LoadRootObject();
        my $Locale = $Site->siteLocale($hFileFormat->{'LocaleID'});
        my $hInfo = $Locale->get([qw (DecimalSep DefaultCharSet ListColumnSeparator ListQuotCharacter ListRowSeparator ThousandSep) ]);
        $hFileFormat->{'eol'}        = $hInfo->{'ListRowSeparator'}     unless exists $hFileFormat->{'eol'};
        $hFileFormat->{'encoding'}   = lc($hInfo->{'DefaultCharSet'})   unless exists $hFileFormat->{'encoding'};
        $hFileFormat->{'quote_char'} = $hInfo->{'ListQuotCharacter'}    unless exists $hFileFormat->{'quote_char'};
        $hFileFormat->{'escape_char'}= $hInfo->{'ListQuotCharacter'}    unless exists $hFileFormat->{'escape_char'};
        $hFileFormat->{'sep_char'}   = $hInfo->{'ListColumnSeparator'}  unless exists $hFileFormat->{'sep_char'};
        $hFileFormat->{'DecimalSep'} = $hInfo->{'DecimalSep'}           unless exists $hFileFormat->{'DecimalSep'};
        $hFileFormat->{'ThousandSep'}= $hInfo->{'ThousandSep'}          unless exists $hFileFormat->{'ThousandSep'};
        $self->{ 'Locale'} =   $Locale;
        delete $hFileFormat->{'LocaleID'};
    }

    my $ahLanguages;
    if (defined $hFileFormat->{'LanguageCode'}) {
        $ahLanguages = GetInfoLanguagesByCode2($hFileFormat->{'LanguageCode'});
        delete $hFileFormat->{'LanguageCode'};
    } else {
        $ahLanguages = $self->object->get('Languages'); #get all site languages
    }
    $self->{ 'Languages'} = GetDoubleHashFromLoop($ahLanguages, 'LanguageID');

    $self->SUPER::exportCSV($FileName, $hFileFormat,  $aColumns, $ViewObject, $aObjects);
    return;
}

#========================================================================================
# function     drive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->drive();
#----------------------------------------------------------------------------------------
# description  starts the monitor 'CSVExportMonitor'
#               and starts the export of data via $self->SUPER::drive()
#========================================================================================
sub drive {
    my $self = shift;

    # we use the UnityCSVExportMonitor for unity shops
    my $PageTypeName = (ExistsClassByAlias('Shop')
                        and $self->object->instanceOf('Shop')
                        and $self->object->getSite()->get('IsUnityRegistered')) ? 'UnityCSVExportMonitor' : 'CSVExportMonitor';

    $self->startMonitor($PageTypeName);
    DisableObjectCache( sub {
        $self->SUPER::drive();
    });
    return;
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Items = $self->items();
#----------------------------------------------------------------------------------------
# description  Returns the objects to export, throws a Warning 'NO_ITEMS' if no items,
#               set Monitor 'Total' if defined $self->monitor
#----------------------------------------------------------------------------------------
# return       $Items | parameter of exportCSV | ref.array.array.string or DE_EPAGES::Core::API::Iterator
#========================================================================================
sub items {
    my $self = shift;

    my $Items = $self->SUPER::items;
    my $ItemCount = ref $Items eq 'ARRAY' ? scalar @$Items : $Items->count;
    Warning('NO_ITEMS') unless $ItemCount ;

    my $Monitor = $self->monitor;
    $Monitor->set({'Total'=> $ItemCount}) if defined $Monitor;
    return $Items;
}

#========================================================================================
# function     writeHeader
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->writeHeader();
#----------------------------------------------------------------------------------------
# description  Writes the CSV header line using the 'HeaderString' key of the column
#               definition (see <function exportCSV>).
#========================================================================================
sub writeHeader {
    my $self = shift;

    my @ATTRIBUTES = map {$_->{'HeaderString'}} @{$self->columns()};
    #write headerline
    $self->writeLine(\@ATTRIBUTES);
    return;
}

#========================================================================================
# function     exportLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->exportLine();
#----------------------------------------------------------------------------------------
# description  Writes one line to the CSV file.
#               Gets the values of the current export item from the function
#               <function getValues>.
#               Increments the monitor counter if defined $self->monitor.
#========================================================================================
sub exportLine {
    my $self = shift;
    my $Monitor = $self->monitor;

    if (defined $Monitor) {
        $self->monitorIncrement;
        $self->monitorContent;
    }
    my $Values = $self->getValues;
    $self->writeLine($Values );

    return;
}

#========================================================================================
# function     getValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aValues = $self->getValues;
#----------------------------------------------------------------------------------------
# description  returns the values of the attributes from function columns(),
#               uses function getObjectAttributeValue,
#               formats the value via function DE_EPAGES::CSVExportImport::API::Tools::FormatValue()
#----------------------------------------------------------------------------------------
# return       $aValues | values of attributes ($self->columns) for object | ref.array.scalar
#========================================================================================
sub getValues {
    my $self = shift;

    my $Object= $self->topID('Item');

    return [
        map { $self->formatValue( $self->getObjectAttributeValue($Object,$_), $_->{'Type'} ) }
        @{ $self->columns() }
    ];
}

#========================================================================================
# function     formatValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FormattedValue = $self->formatValue( $Value, $Type );
#----------------------------------------------------------------------------------------
# description  Formats $Value according to $Type.
#----------------------------------------------------------------------------------------
# input        $Value | reference to value of the attribute | scalar
# input        $Type  | type of the attribute, could be 'Boolean', 'Integer', 'Float',
#               'Money', 'Price', 'Date', 'Time', 'DateTime' | string
# return       $FormattedValue | formatted value | string
#========================================================================================
sub formatValue {
    my $self = shift;
    my ( $Value, $Type ) = @_;

    if (not defined $Value or $Value !~ /\S/) {
        return '';
    }
    elsif ( ElementOf($Type, @FLOAT_TYPES) ) {
        return $self->_numberFormat->format_number( $Value );
    }
    elsif ( ElementOf($Type, @DATETIME_TYPES) ) {
        return $self->locale->formatDateTime( $Value, lc $Type );
    }
    else {
        return $Value;
    }
}

#========================================================================================
# function     getObjectAttributeValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Value = getObjectAttributeValue($Object, $hAttribute);
#----------------------------------------------------------------------------------------
# description  Returns the raw value of a single attribute.
#               Use <function formatValue> to get the formatted value.
#----------------------------------------------------------------------------------------
# input        $Object    | object | object
# input        $hAttribute | properties of object attribute to get:
#               <ul>
#                   <li>Alias     - attribute alias, e.g. Alias, Color -string</li>
#                   <li>LanguageID - only set if Type =~ /Localized/ - integer</li>
#                   <li> ..
#               </ul> | ref.hash
#----------------------------------------------------------------------------------------
# return       $Value | value of attribute for object | scalar
#========================================================================================
sub getObjectAttributeValue  {
    my $self = shift;
    my ($Object, $hAttribute ) = @_;

    my $Alias = $hAttribute->{'Alias'};
    return $Object->get($Alias, $hAttribute->{'LanguageID'});
}

#========================================================================================
# function     startMonitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->startMonitor($PageTypeName);
#----------------------------------------------------------------------------------------
# description  starts the progress monitor. Creates an object of
#               <package DE_EPAGES::Presentation::API::Object::ProgressMonitor>
#               if run inside the application server, otherwise the package
#               <package DE_EPAGES::Core::API::Object::ProgressMonitor>
#               is used.
#----------------------------------------------------------------------------------------
# input        $PageTypeName | name of pagetype (only used inside application server)
#                           | string
#========================================================================================
sub startMonitor {
    my $self = shift;
    my ($PageTypeName) = @_;

    return if exists $self->{'Monitor'};
    my $AppServer = GetAppServer();
    if (defined $AppServer) {
        my $Monitor = DE_EPAGES::Presentation::API::Object::ProgressMonitor->new(
            'PageTypeName' => $PageTypeName,
            'Object'       => $self->object,
        );
        $self->{'Monitor'} = $Monitor;
    } else {
        my $Monitor = DE_EPAGES::Core::API::Object::ProgressMonitor->new(
            'ContentTemplate' => "export objects (%d / %d)\n",
        );
        $self->{'Monitor'} = $Monitor;
    }
    return;
}

#========================================================================================
# function     locale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Locale = $self->locale;
#----------------------------------------------------------------------------------------
# description  return the current Locale
#----------------------------------------------------------------------------------------
# return       $Locale  | Locale | hash
#========================================================================================
sub locale { return shift->{'Locale'}; }

sub _numberFormat {
    my $self = shift;

    my $NumberFormat = $self->{'NumberFormat'};
    unless ( defined $NumberFormat ) {
        my $hFormat = $self->locale->get([qw( DecimalSep ThousandSep )]);
        $NumberFormat = DE_EPAGES::Core::API::Object::NumberFormat->new(
            thousands_sep => $hFormat->{ThousandSep},
            decimal_point => $hFormat->{DecimalSep},
        );
        $self->{'NumberFormat'} = $NumberFormat;
    }

    return $NumberFormat;
}

#========================================================================================
# function     languages
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahLanguages = $self->languages;
#----------------------------------------------------------------------------------------
# description  return the languages for attributes
#----------------------------------------------------------------------------------------
# return       $ahLanguages  | Languages | array.hash
#========================================================================================
sub languages { return shift->{'Languages'}; }

1;
