#========================================================================================
# package      DE_EPAGES::CSVExportImport::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements functions needed for csv export / import.
#========================================================================================
package DE_EPAGES::CSVExportImport::API::Tools;
use base qw (  Exporter );
use strict;

our @EXPORT_OK = qw (
                     ParseValue
                     ParseFloat
                     ParseDateTime
                     ParseString
                     FormatValue
                     FormatFloat
                     FormatDateTime
                     );

use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError);
use DE_EPAGES::Core::API::PerlTools qw( GetLocalTimeZone ElementOf );
use DE_EPAGES::Core::API::TestParameter qw( TestInteger TestFloat TestString TestDateTime TestObject TestUrl );
use DE_EPAGES::CSVExportImport::API::Constants qw( MAX_FLOAT MIN_FLOAT MAX_MONEY MIN_MONEY @DATETIME_TYPES @FLOAT_TYPES @CSV_WARNING_CODES );
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );

use vars qw( @DATETIME_TYPES @FLOAT_TYPES );

#========================================================================================
# function     ParseFloat
# state        public
#----------------------------------------------------------------------------------------
# syntax       ParseFloat( $Name, $rValue, $hFormat )
#----------------------------------------------------------------------------------------
# description  Normalizes float values.
#----------------------------------------------------------------------------------------
# input        $Name  | alias of the attribute | string
# input        $rValue | reference to value of the attribute | ref
# input        $hFormat | format parameters, keys are
#               <ul>
#                   <li>DecimalSep    -  decimal separator - string
#                   <li>ThousandSep    -  thousands separator - string
#               </ul> | ref.hash.*
#========================================================================================
sub ParseFloat ($$$) {
    my ( $Name, $Value, $Format ) = @_;
    TestString( 'Name', $Name );
    TestString( 'Value', $$Value );
    die "Format->{$_} should be defined." foreach grep { not defined $Format->{$_} } qw(DecimalSep ThousandSep);

    my $TS = "\Q$Format->{'ThousandSep'}\E";
    my $DP = "\Q$Format->{'DecimalSep'}\E";;
    $$Value =~ s/$TS//g;
    $$Value =~ s/$DP/./g;

    return;
}

#========================================================================================
# function     ParseDateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       ParseDateTime( $Name, $Type, $rValue, $hFormat )
#----------------------------------------------------------------------------------------
# description  Normalizes DateTime values.
#----------------------------------------------------------------------------------------
# input        $Name  | alias of the attribute | string
# input        $Type  | type of the attribute, could be 'Date', 'Time', 'DateTime' | string
# input        $rValue | reference to value of the attribute | ref
# input        $hFormat | format parameters, keys are
#               <ul>
#                   <li>LocaleID    -  locale id - string
#               </ul> | ref.hash.*
#========================================================================================
sub ParseDateTime ($$$$) {
    my ( $Name, $Type, $Value, $Format ) = @_;
    TestString( 'Name', $Name );
    TestString( 'Type', $Type );
    TestString( 'Value', $$Value );
    TestString( 'Format->{LocaleID}', $Format->{'LocaleID'} );

    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new(
        locale => $Format->{'LocaleID'},
    );

    $Type =~ s/^reg_//i;
    $Type = lc($Type);
    my @Pattern = ('%Y-%m-%dT%T', qw (full long medium short) );
    if (($Type eq 'time') or ($Type eq 'time_tz') ) {
        unshift @Pattern, '%{hour_12}:%M %P', '%{hour_12} %P', '%{hour_12}:%M %p', '%{hour_12} %p';
        push @Pattern, '%H';
    }
    my $LastError = undef;
    foreach my $pattern (@Pattern) {
        my $DateTimeValue = undef;
        eval {
            $DateTimeValue = $Formatter->parse_datetime($$Value, $Type, $pattern);
            $DateTimeValue->set( year => 2000, month => 1, day => 1) if ($Type eq 'time') or ($Type eq 'time_tz');
            GetCurrentDBHandle()->testDateTimeRange($DateTimeValue);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $DateTimeValue = undef;
            my $ErrorCode = $Error->code;
            Error($Error) unless
                ElementOf($ErrorCode, qw(FORMAT_NOT_DATETIME  DATETIME_TOO_LARGE DATETIME_TOO_SMALL));
            $LastError = $Error;
        }

        if (defined $DateTimeValue) {
            my $TimeZone = LoadRootObject()->get('TimeZone');
            $DateTimeValue->set_time_zone( 'floating');
            # Don't change date values, for example Birthdate or Time.

            $DateTimeValue->set_time_zone( $TimeZone) if ($Type eq 'datetime') or ($Type eq 'date_tz') or ($Type eq 'time_tz');
            $DateTimeValue->set_time_zone( GetLocalTimeZone());
            $$Value = $DateTimeValue->clone;
            return;
        }
   }
    Error($LastError);

    return;
}

#========================================================================================
# function     FormatFloat
# state        public
#----------------------------------------------------------------------------------------
# syntax       FormatFloat( $rValue, $hFormat )
#----------------------------------------------------------------------------------------
# description  Obsolete. Use <function DE_EPAGES::CSVExportImport::API::CSV::ObjectExportDriver::formatValue> instead.
#----------------------------------------------------------------------------------------
# input        $rValue | reference to value of the attribute | ref
# input        $hFormat | format parameters, keys are
#               <ul>
#                   <li>DecimalSep    -  decimal separator - string
#                   <li>ThousandSep    -  thousands separator - string
#               </ul> | ref.hash.*
#========================================================================================
sub FormatFloat ($$) {
    my ( $Value, $Format ) = @_;
    TestFloat( 'Value', $$Value );
    die "Format->{$_} should be defined." foreach grep { not defined $Format->{$_} } qw(DecimalSep ThousandSep);

    my $NF = DE_EPAGES::Core::API::Object::NumberFormat->new(
        thousands_sep => $Format->{ThousandSep},
        decimal_point => $Format->{DecimalSep},
    );
    $$Value = $NF->format_number($$Value);
    return;
}


#========================================================================================
# function     FormatDateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       FormatDateTime( $Type, $rValue, $hFormat );
#----------------------------------------------------------------------------------------
# description  Obsolete. Use <function DE_EPAGES::CSVExportImport::API::CSV::ObjectExportDriver::formatValue> instead.
#----------------------------------------------------------------------------------------
# input        $Type  | type of the attribute, could be 'Date', 'Time', 'DateTime' | string
# input        $rValue | reference to value of the attribute | ref
# input        $hFormat | format parameters, keys are
#               <ul>
#                   <li>LocaleID    -  locale id - string
#               </ul> | ref.hash.*
#========================================================================================
sub FormatDateTime ($$$) {
    my ( $Type, $Value, $Format ) = @_;
    TestString( 'Type', $Type );
    TestObject( 'Value', $$Value );
    TestString('Format->{LocaleID}', $Format->{'LocaleID'});

    my $SiteLocale = LoadRootObject()->siteLocale( $Format->{'LocaleID'} );
    $$Value = $SiteLocale->formatDateTime($$Value, lc($Type));
    return;
}

#========================================================================================
# function     ParseString
# state        public
#----------------------------------------------------------------------------------------
# syntax       ParseString( $Name, $rValue );
# syntax       ParseString( $Name, $rValue, $hLimits );
#----------------------------------------------------------------------------------------
# description  Tests if the value is a string.
#               Throws an error if the format is not ok.
#----------------------------------------------------------------------------------------
# input        $Name  | alias of the attribute | string
# input        $rValue | reference to value of the attribute | ref
# input        $hLimits | (optional) upper and lower limits for some data types, at the moment
#               <ul>
#                   <li>MaxLength   -  maximum string length - int
#               </ul> | ref.hash.*
# output       $rValue | set to the parsed value if the format is ok,
#                         set to undef if the original value contains whitespace only | ref
#========================================================================================
sub ParseString {
    my ( $Name, $rValue, $hLimits ) = @_;
    TestString( 'Name', $Name );
    $hLimits //= {};

    $$rValue =~ s/\s+$//;
    $$rValue =~ s/^\s+//;
    if( $$rValue eq '' ) {
        $$rValue = undef;
        return;
    };

    TestString( $Name, $$rValue, undef, $hLimits->{'MaxLength'} );
    return;
}

#========================================================================================
# function     ParseValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       ParseValue( $Name, $Type, $rValue );
# syntax       ParseValue( $Name, $Type, $rValue, $hFormat );
# syntax       ParseValue( $Name, $Type, $rValue, $hFormat, $hLimits );
#----------------------------------------------------------------------------------------
# description  Tests variable and converts from external to internal representation.
#               Changes the value of $$rValue.
#               Throws an error if the format is not ok.
#----------------------------------------------------------------------------------------
# input        $Name  | alias of the attribute | string
# input        $Type  | type of the attribute, could be 'Boolean','Integer', 'Float',
#               'Money', 'Price', 'Date', 'Time', 'DateTime' | string
# input        $rValue | reference to value of the attribute | ref
# input        $hFormat | (optional) format parameters, keys are
#               <ul>
#                   <li>LocaleID    -  locale id - string
#                   <li>DecimalSep    -  decimal separator - string
#                   <li>ThousandSep    -  thousands separator - string
#               </ul> | ref.hash.*
# input        $hLimits | (optional) upper and lower limits for some data types, at the moment
#               <ul>
#                   <li>MaxFloat    -  max. valid float value - float
#                   <li>MinFloat    -  min. valid float value - float
#               </ul> | ref.hash.*
# output       $rValue | set to the parsed value if the format is ok,
#                         set to undef if the original value contains whitespace only | ref
#========================================================================================
sub ParseValue ($$$;$$) {
    my ( $Name, $Type, $rValue, $hFormat, $hLimits ) = @_;
    TestString( 'Name', $Name );
    TestString( 'Type', $Type );

    $$rValue =~ s/\s+$//;
    $$rValue =~ s/^\s+//;
    if( $$rValue eq '' ) {
        $$rValue = undef;
        return;
    };

    if ($Type eq 'Boolean') {
        TestInteger($Name, $$rValue, 0, 1);
    }
    elsif($Type eq 'Integer') {
        TestInteger($Name, $$rValue);
    }
    elsif($Type eq 'String') {
        TestString($Name, $$rValue);
    }
    elsif(ElementOf($Type, @FLOAT_TYPES)) {
        ParseFloat($Name, $rValue, $hFormat);

        my ( $FloatMin, $FloatMax );
        # get constant limits
        if ($Type eq 'Price' or $Type eq 'Money') {
            ( $FloatMin, $FloatMax ) = ( MIN_MONEY, MAX_MONEY );
        }
        elsif($Type eq 'Float') {
            ( $FloatMin, $FloatMax ) = ( MIN_FLOAT, MAX_FLOAT );
        }
        # override constant limits if custom ones are set
        $FloatMin = $hLimits->{MinFloat} if ( defined $hLimits->{MinFloat} );
        $FloatMax = $hLimits->{MaxFloat} if ( defined $hLimits->{MaxFloat} );

        TestFloat($Name, $$rValue, $FloatMin, $FloatMax);
    }
    elsif(ElementOf($Type, @DATETIME_TYPES)) {
        TestDateTime($Name, $$rValue); # yes, first!
        ParseDateTime($Name, $Type, $rValue, $hFormat);
    }
    elsif($Type eq 'File') {
        if ($$rValue =~ m!//|:! ){
            TestUrl($Name, $$rValue);
        }
    }
    return;
}

#========================================================================================
# function     FormatValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       FormatValue( $rValue, $Type, $hFormat );
#----------------------------------------------------------------------------------------
# description  Obsolete. Use <function DE_EPAGES::CSVExportImport::API::CSV::ObjectExportDriver::formatValue> instead.
#----------------------------------------------------------------------------------------
# input        $rValue | reference to value of the attribute | ref
# input        $Type  | type of the attribute, could be 'Boolean', 'Integer', 'Float',
#               'Money', 'Price', 'Date', 'Time', 'DateTime' | string
# input        $hFormat | format parameters, keys are
#               <ul>
#                   <li>LocaleID    -  locale id - string
#                   <li>DecimalSep    -  decimal separator - string
#                   <li>ThousandSep    -  thousands separator - string
#               </ul> | ref.hash.*
#========================================================================================
sub FormatValue ($$$) {
    my ( $Value, $Type, $Format ) = @_;
    TestString( 'Type', $Type );

    if (not defined $$Value or $$Value !~ /\S/) {
        $$Value = '';
    }
    elsif ( ElementOf($Type, @FLOAT_TYPES) ) {
        FormatFloat($Value, $Format);
    }
    elsif ( ElementOf($Type, @DATETIME_TYPES) ) {
        $Format->{'Type'} = lc($Type);
        FormatDateTime($Type, $Value, $Format);
    }
    return;
}

1;
