#========================================================================================
# package      DE_EPAGES::Cartridge::API::CartridgeInstaller
# state        public
# base         DE_EPAGES::Trigger::API::TriggerInstaller
#----------------------------------------------------------------------------------------
# description  Cartridge installer for the Cartridge cartridge.
#               It registers cartridge at store database.
#========================================================================================
package DE_EPAGES::Cartridge::API::CartridgeInstaller;
use base DE_EPAGES::Trigger::API::TriggerInstaller;

use strict;
use DE_EPAGES::Cartridge::API::Constants qw(
    STATE_INSTALLATION
    STATE_INSTALLED
    STATE_DEINSTALLATION
    STATE_PATCH
    STATE_PATCHDBSTRUCTURE
    STATE_START_INSTALL
    STATE_INSTALL_DBSTRUCTURE
);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw (
    InsertCartridge
    DeleteCartridge
    UpdateCartridge
    ExistsCartridgeByPackage
    GetPKeyCartridgeByPackage
    GetInfoCartridge
    GetInfoCartridgeByPackage
    GetInfoCartridgesByPackage
);
use DE_EPAGES::Cartridge::API::Table::CartridgeDependency qw (
    GetInfoCartridgeDependency
    GetInfoCartridgeDependenciesByCartridgeID
    InsertCartridgeDependency
    DeleteCartridgeDependency
    ExistsCartridgeDependency
    UpdateCartridgeDependency
);
use DE_EPAGES::Core::API::EpagesConfig qw (EpagesConfig);
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf);
use DE_EPAGES::Core::API::TestParameter qw(TestString);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::XML::API::XML::DependencyImportHandler qw (GetDependencies);

#========================================================================================
# function     id
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CartridgeID = $Installer->id;
#----------------------------------------------------------------------------------------
# description  Returns the cartridge id in the database or undef if the cartridge is
#               not installed in the current database.
#----------------------------------------------------------------------------------------
# return       $CartridgeID | cartridge id | integer
#========================================================================================
sub id {
    my $self = shift;

    return undef unless GetCurrentDBHandle()->existsTable('cartridge');
    return GetPKeyCartridgeByPackage($self->cartridgePackage);
}

#========================================================================================
# function     startInstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->startInstall;
#----------------------------------------------------------------------------------------
# description  Inserts entry at cartridge table with state STATE_INSTALLATION.
#========================================================================================
sub startInstall {
    my $self = shift;
    $self->SUPER::startInstall;
    if ($self->cartridgePackage ne 'DE_EPAGES::Cartridge') {

        unless( ExistsCartridgeByPackage($self->cartridgePackage) ) {
            InsertCartridge({
                'Package'     => $self->cartridgePackage,
                'Version'     => $self->version,
                'InstallState'=> STATE_START_INSTALL,
                'DBStructureVersion' => $self->version,
            });
        }
        $self->insertRequiredCartridges();
    }
    return;
}

#========================================================================================
# function     finishInstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->finishInstall;
#----------------------------------------------------------------------------------------
# description  Updates entry at cartridge table with state STATE_INSTALLED.
#========================================================================================
sub finishInstall {
    my $self = shift;
    if ($self->cartridgePackage ne 'DE_EPAGES::Cartridge') {
        UpdateCartridge($self->id, { 'InstallState'=> STATE_INSTALLED });
    } else {
        InsertCartridge({
            'Package'            => $self->cartridgePackage,
            'Version'            => $self->version,
            'InstallState'       => STATE_INSTALLED,
            'DBStructureVersion' => $self->version,
        });
    }
    $self->SUPER::finishInstall;
    return;
}

#========================================================================================
# function     installDBStructure
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->installDBStructure;
#----------------------------------------------------------------------------------------
# description  Creates database objects, such as tables, views and stored procedures.
#               At the beginning sets the cartridge installation state to
#               STATE_INSTALL_DBSTRUCTURE and at the ent to STATE_INSTALLATION.
#========================================================================================
sub installDBStructure {
    my $self = shift;

    my $CartridgeID = $self->id;
    if( defined $CartridgeID ) {
        my $InstallState = GetInfoCartridge($CartridgeID)->{'InstallState'};
        if( $InstallState == STATE_START_INSTALL || $InstallState == STATE_INSTALL_DBSTRUCTURE ) {
            UpdateCartridge($CartridgeID, { 'InstallState'=> STATE_INSTALL_DBSTRUCTURE });
            $self->SUPER::installDBStructure;
            UpdateCartridge($CartridgeID, { 'InstallState'=> STATE_INSTALLATION });
        }
    }
    else {
        # only for DE_EPAGES::Cartridge
        $self->SUPER::installDBStructure;
    }
}

#========================================================================================
# function     startPatch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->startPatch;
#----------------------------------------------------------------------------------------
# description  Updates entry at cartridge table with state STATE_PATCH.
#========================================================================================
sub startPatch {
    my $self = shift;

    # cartridge not installed/registered
    my $CartridgeID = $self->id;
    $self->SUPER::startPatch;
    UpdateCartridge($CartridgeID, { 'InstallState'=> STATE_PATCH });
    return;
}

#========================================================================================
# function     finishPatch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->finishPatch;
#----------------------------------------------------------------------------------------
# description  Updates entry at cartridge table with state STATE_INSTALLED.
#========================================================================================
sub finishPatch {
    my $self = shift;

    # cartridge not installed/registered
    my $CartridgeID = $self->id;
    UpdateCartridge($CartridgeID, {'InstallState'=> STATE_INSTALLED });
    $self->SUPER::finishPatch;
    return;
}

#========================================================================================
# function     setVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->setVersion($NewVersion);
#----------------------------------------------------------------------------------------
# description  Updates entry at cartridge table with Version $NewVersion.
#----------------------------------------------------------------------------------------
# input        $NewVersion | version | string
#========================================================================================
sub setVersion {
    my $self = shift;
    my ($NewVersion) = @_;

    # cartridge not installed/registered
    my $CartridgeID = $self->id;
    UpdateCartridge($CartridgeID, {'Version' => $NewVersion});
    $self->SUPER::setVersion($NewVersion);
    return;
}

#========================================================================================
# function     startPatchDBStructure
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->startPatchDBStructure;
#----------------------------------------------------------------------------------------
# description  Sets the cartridge InstallState to STATE_PATCHDBSTRUCTURE.
#========================================================================================
sub startPatchDBStructure {
    my $self = shift;

    $self->SUPER::startPatchDBStructure;
    UpdateCartridge($self->id, { 'InstallState'=> STATE_PATCHDBSTRUCTURE });
    return;
}

#========================================================================================
# function     getDBStructureVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DBStructureVersion = $Installer->getDBStructureVersion;
#----------------------------------------------------------------------------------------
# description  Returns the current database structure version.
#========================================================================================
sub getDBStructureVersion {
    my $self = shift;

    return GetInfoCartridgeByPackage($self->cartridgePackage)->{'DBStructureVersion'};
}

#========================================================================================
# function     setDBStructureVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->setDBStructureVersion( $Version );
#----------------------------------------------------------------------------------------
# description  Sets the current database structure version.
#========================================================================================
sub setDBStructureVersion {
    my $self = shift;
    my ($Version) = @_;
    TestString('Version', $Version);

    UpdateCartridge($self->id, { 'DBStructureVersion'=> $Version });
    return;
}

#========================================================================================
# function     finishPatchDBStructure
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->finishPatchDBStructure;
#========================================================================================
sub finishPatchDBStructure {
    my $self = shift;

    GetLog->info(sprintf("db structure cartridge %s is patched to version %s\n", $self->cartridgePackage, $self->version));
    return;
}

#========================================================================================
# function     startUninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->startUninstall;
#----------------------------------------------------------------------------------------
# description  Sets the install state of the cartridge to STATE_DEINSTALLATION.
#========================================================================================
sub startUninstall {
    my $self = shift;
    $self->SUPER::startUninstall;
    my $CartridgeID = $self->id;
    return 1 unless defined $CartridgeID;
    eval { UpdateCartridge($CartridgeID, { 'InstallState'=> STATE_DEINSTALLATION }); }; warn $@ if $@; # eval because hook cant exists
    return;
}

#========================================================================================
# function     finishUninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->finishUninstall;
#----------------------------------------------------------------------------------------
# description  Removes the cartridge entry from the database (table 'cartridge').
#========================================================================================
sub finishUninstall {
    my $self = shift;
    if ($self->cartridgePackage ne 'DE_EPAGES::Cartridge') {
        my $CartridgeID = $self->id;
        DeleteCartridge($CartridgeID) if defined $CartridgeID;
    }
    $self->SUPER::finishUninstall;
    return;
}

#========================================================================================
# function     installedVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $InstalledVersion = $Installer->installedVersion();
#----------------------------------------------------------------------------------------
# description  gets the installed Version for the cartridge.
#========================================================================================
sub installedVersion {
    my $self = shift;
    return undef unless $self->isInstalled;
    return GetInfoCartridgeByPackage($self->cartridgePackage)->{'Version'};
}

#========================================================================================
# function     testPatch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testPatch;
#----------------------------------------------------------------------------------------
# description  Test if cartridge package is complaint to the existing enviroment.
#               Calls <function DE_EPAGES::Core::API::BaseInstaller::addReason> for each
#               reason why the cartridge cannot be updated, for example if one or more
#               required cartridges are not installed with a supported version.
#========================================================================================
sub testPatch {
    my $self = shift;

    # cartridge not installed
    return unless $self->isInstalled;

    my $DBVersion = GetInfoCartridgeByPackage($self->cartridgePackage)->{'Version'};

    # version is up-to-date
    return if $DBVersion eq $self->version;

    # test if this cartridge can patch the current db version
    $self->testDBVersionPatch($DBVersion);

    # test if all required cartridges are installed with the correct version
    $self->testInstallRequiredCartridges;

    # test dependent cartridges if the new version is isCompliant
    $self->testPatchDependentCartridges;
    return;
}

#========================================================================================
# function     insertRequiredCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->insertRequiredCartridges;
#----------------------------------------------------------------------------------------
# description  Inserts required cartridges into the cartridgedependency table.
#========================================================================================
sub insertRequiredCartridges ($) {
    my $self = shift;

    foreach my $Cartridge (@{$self->directRequiredCartridges}) {

        # skip core cartridges that don't have a cartridge id, e.g. DE_EPAGES::Trigger
        next unless defined $Cartridge->id;

        my $Comment = $self->requiredCartridgeComment( $Cartridge->cartridgePackage );
        InsertCartridgeDependency({
            'CartridgeID'          => $Cartridge->id,
            'DependendCartridgeID' => $self->id,
            'Comment'              => $Comment,
        }) unless ExistsCartridgeDependency( $Cartridge->id,  $self->id );
    }
    return;
}

#========================================================================================
# function     directDependentCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->directDependentCartridges;
#----------------------------------------------------------------------------------------
# description  Returns a list of cartridges that have declared dependencies on this
#               cartridge. The list is generated from the database table
#               'cartridgedependency'.
#----------------------------------------------------------------------------------------
# return       $aDependentCartridges | list of cartridge objects | ref.array.object
#========================================================================================
sub directDependentCartridges ($) {
    my $self = shift;

    # maybe cartridge table is already deleted?
    return [] unless $self->id;

    # get dependent cartridges
    my @Cartridges;
    my $DepCartridges = GetInfoCartridgeDependenciesByCartridgeID( $self->id );
    foreach my $DepCartridge (@$DepCartridges) {
        # get package
        my $CartridgeInfo = GetInfoCartridge($DepCartridge->{'DependendCartridgeID'});
        push @Cartridges, $self->loadInstaller($CartridgeInfo->{'Package'});
    }
    return \@Cartridges;
}

#========================================================================================
# function     testDependentCartridges
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testDependentCartridges;
#----------------------------------------------------------------------------------------
# description  Tests if new version cartridge package is isCompliant to dependent
#               cartridges.
#               Calls <function DE_EPAGES::Core::API::BaseInstaller::addReason> for each
#               cartridge that does not exist with a supported version.
#========================================================================================
sub testPatchDependentCartridges {
    my $self = shift;
    foreach my $Cartridge (@{$self->directDependentCartridges}) {
        $Cartridge->testCartridgeCompliant($self);
        $self->addReason($_) foreach @{$Cartridge->getReasons};
    }
    return;
}

#========================================================================================
# function     testDBVersionPatch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->testDBVersionPatch($DBVersion);
#----------------------------------------------------------------------------------------
# description  Tests if cartridge can patch database version.
#               Calls <function DE_EPAGES::Core::API::BaseInstaller::addReason> if the
#               currently installed version cannot be updated.
#----------------------------------------------------------------------------------------
# input        $DBVersion | version | string
#========================================================================================
sub testDBVersionPatch {
    my $self = shift;
    my ($DBVersion) = @_;

    $self->addReason({
        'Reason'    => 'CANT_PATCH_VERSION',
        'Package'   => $self->cartridgePackage,
        'Version'   => $self->version,
        'DBVersion' => $DBVersion,
    }) unless ElementOf($DBVersion, $self->patches);
    return;
}

#========================================================================================
# function     isInstalled
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsInstalled= $Installer->isInstalled;
#----------------------------------------------------------------------------------------
# description  Returns true if the cartridge is installed in the current database.
#----------------------------------------------------------------------------------------
# return       $IsInstalled | true if the cartridge is installed | boolean
#========================================================================================
sub isInstalled {
    my $self = shift;
    return 0 unless $self->loadInstaller('DE_EPAGES::Cartridge')->isInstalled;
    my $aCartridges = GetInfoCartridgesByPackage($self->cartridgePackage);
    my $IsInstalled = 0;
    if( scalar @$aCartridges ) {
        my $InstallState = $aCartridges->[0]{'InstallState'};
        $IsInstalled = 1 if ElementOf( $InstallState, STATE_INSTALLED,
            STATE_DEINSTALLATION, STATE_PATCH, STATE_PATCHDBSTRUCTURE);
    }
    GetLog->debug($self->cartridgePackage . ' is ' . ($IsInstalled ? '' : 'not '), 'installed');
    return $IsInstalled;
}

#========================================================================================
# function     isCompliant
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsCompliant = $Installer->isCompliant($RequiredVersion, $SupportedVersion);
#----------------------------------------------------------------------------------------
# description  Is the supported version complaint to the new version of patch.
#----------------------------------------------------------------------------------------
# input        $RequiredVersion | required version | string
# input        $SupportedVersion | supported version (optional default version in database) | string
# return       $IsCompliant | true if the required cartridge is compatible | boolean
#========================================================================================
sub isCompliant ($$) {
    my $self = shift;
    my ($RequiredVersion, $SupportedVersion) = @_;
    # get version from database, else (install) current version
    my $Infos = GetInfoCartridgesByPackage($self->cartridgePackage);
    $SupportedVersion = scalar @$Infos ? $Infos->{'Version'} : undef;
    return $self->SUPER::isCompliant($RequiredVersion, $SupportedVersion);
}

#========================================================================================
# function     addDependency
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->addDependency($CartridgePackage, $Comment);
# example      $Installer->addDependency('DE_EPAGES::Design', 'adds navigation elements');
#----------------------------------------------------------------------------------------
# description  Adds a cartridge to dependencies of executed cartridge and
#               install required cartridge if this is not installed.
#               Updates the comment if the cartridge dependency is alreads registered.
#----------------------------------------------------------------------------------------
# input        $CartridgePackage | required cartridge | string
# input        $Comment | reason for the dependency (optional) | string
#========================================================================================
sub addDependency {
    my $self = shift;
    my ($CartridgePackage, $Comment) = @_;
    TestString('CartridgePackage', $CartridgePackage);

    $Comment = undef if (defined $Comment and $Comment eq '');

    my $RequiredCartridge = $self->loadInstaller($CartridgePackage);
    $RequiredCartridge->install unless $RequiredCartridge->isInstalled;

    my $CartridgeID = $self->id;
    my $RequiredCartridgeID = $RequiredCartridge->id;
    if (defined $RequiredCartridgeID) { # could be a non database cartridge
        if( ExistsCartridgeDependency($RequiredCartridgeID, $CartridgeID) ) {
            UpdateCartridgeDependency( $RequiredCartridge->id, $self->id, {
                'Comment' => $Comment,
            });
        }
        else {
            InsertCartridgeDependency({
                'CartridgeID'          => $RequiredCartridgeID,
                'DependendCartridgeID' => $CartridgeID,
                'Comment'              => $Comment,
            });
        }
    }
    return;
}

#========================================================================================
# function     removeDependency
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->removeDependency($CartridgePackage);
#----------------------------------------------------------------------------------------
# description  removes a cartridge to dependencies of executed cartridge.
#----------------------------------------------------------------------------------------
# input        $CartridgePackage | not longer required cartridge | string
#========================================================================================
sub removeDependency {
    my $self = shift;
    my ($CartridgePackage) = @_;

    my $RequiredCartridge = eval { return $self->loadInstaller($CartridgePackage); };
    if (ExistsError()) {
        return if GetError->code eq 'CARTRIDGE_NOT_EXISTS';
        Error(GetError());
    }

    my $CartridgeID = $self->id;
    my $RequiredCartridgeID = $RequiredCartridge->id;

    # ignore cartridges that don't have an id, such as Dictionary, TLE etc.
    if( defined $RequiredCartridgeID
        && ExistsCartridgeDependency($RequiredCartridgeID, $CartridgeID) )
    {
        DeleteCartridgeDependency({
            'CartridgeID'          => $RequiredCartridgeID,
            'DependendCartridgeID' => $CartridgeID,
        });
    }
    return;
}

#========================================================================================
# function     onPatchImportDependencies
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchImportDependencies($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchImportDependencies(
#                         'Database/XML/Dependencies.xml',
#                         {'CVS' => 'Changed',
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by method onPatchImportDependencies
#               Imports the cartridge dependencies from file $File.
#----------------------------------------------------------------------------------------
# input        $File | XML file without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could be New or Changed - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchImportDependencies {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    my $FileName = join '/', $ENV{'EPAGES_CARTRIDGES'}, $self->cartridgeDirectory, $File;
    my $hDependencies = GetDependencies($FileName);
    foreach my $Package (keys %$hDependencies) {
        my $hDependency = $hDependencies->{$Package};
        $self->addDependency($Package, $hDependency->{'Comment'});
    }
    return;
}

#========================================================================================
# function     onPatchDeleteDependencies
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteDependencies($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteDependencies(
#                         'Database/XML/Dependencies.xml',
#                         {'CVS' => 'Changed',
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by method onPatchDeleteDependencies
#               Imports the cartridge dependencies from file $File.
#----------------------------------------------------------------------------------------
# input        $File | XML file without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could be New or Changed - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeleteDependencies {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    my $FileName = join '/', $ENV{'EPAGES_CARTRIDGES'}, $self->cartridgeDirectory, $File;
    my $hDependencies = GetDependencies($FileName);
    foreach my $Package (keys %$hDependencies) {
        $self->removeDependency($Package);
    }
    return;
}

#========================================================================================
# function     dependencyComment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->dependencyComment($hPackage);
#----------------------------------------------------------------------------------------
# example      my $Comment = $Installer->dependencyComment('DE_EPAGES::ExternalPayment');
#----------------------------------------------------------------------------------------
# description  Returns the comment for a cartridge dependency.
#----------------------------------------------------------------------------------------
# input        $Package | package of dependend cartridge | string
# input        $hParams | parameters, keys are
#               <ul>
#                   <li>DependsOnActualCartridge - show#s if given package depends on cartridge or cartridge depends on package - bool
#               </ul> | ref.hash.*
# return       $Comment | dependency comment | string
#========================================================================================
sub dependencyComment{
    my $self = shift;
    my ($Package, $hParams) = @_;

    my $ThisCartridgeID = $self->id;
    my $CartridgeID = GetPKeyCartridgeByPackage($Package);

    my $hInfoCartridgeDependency;

    if ($hParams->{'DependOnActualCartridge'}){
        $hInfoCartridgeDependency = GetInfoCartridgeDependency($ThisCartridgeID, $CartridgeID);
    }
    else{
        $hInfoCartridgeDependency = GetInfoCartridgeDependency($CartridgeID,$ThisCartridgeID);
    }

    return $hInfoCartridgeDependency->{'Comment'};
}

#========================================================================================
# function     testLicenseKey
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->testLicenseKey($Package, $LicenseKey);
#----------------------------------------------------------------------------------------
# example      $Installer->testLicenseKey(
#                         $self->cartridgePackage,
#                         'GwEQT3AnmLsLSCbQlzCq'
#                         );
#----------------------------------------------------------------------------------------
# description  Called by Cartridge.pm before install to check
#               whether an exclusive Cartridge may be installed.
#----------------------------------------------------------------------------------------
# input        $Package    | Name of the CartridgePackage to be installed | string
# input        $LicenseKey | The Licensekey of that exclusive Cartridge   | string
#========================================================================================
sub testLicenseKey{
    my $self = shift;
    my ($Package,$LicenseKey) = @_;

    my $Key = EpagesConfig->get($Package, 'Key', '');
    $Key =~ s/^\s*|\s*$//g;
    die "Not allowed to install cartridge $Package" unless $Key eq $LicenseKey;
    return;
}

1;
