#========================================================================================
# package      DE_EPAGES::Catalog::API::CategoryService
# state        public
#----------------------------------------------------------------------------------------
# description  Service to read, update and create categories.
#========================================================================================
package DE_EPAGES::Catalog::API::CategoryService;

use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadClassByAlias Transaction);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::Catalog::API::Catalog qw (
    AssignProductToCategory
    RemoveProductFromCategory
    RenumberCategoryProducts
    RenumberProductCategories
);
use DE_EPAGES::Catalog::API::Resource::Category;
use DE_EPAGES::WebService::API::Util qw (ReadJSONBoolean);

use strict;

no warnings 'redefine'; ##no critic qw(ProhibitNoWarnings)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Catalog::API::CategoryService
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}



sub deleteCategory {
    my $self = shift;
    my ($Category ) = @_;
    my $Shop = $Category->getSite();

    # check if category is not a root category
    if ($Category->id == $Shop->get('Categories')->id){
        Error('BadRequest', {'root category can not be deleted! categoryId'=>$Category->guid});
    }

    my $ParentCategory = $Category->parent;
    $Category->delete;
    if($Category->exists()){
        Error('Error', {'Error occured while deleting category! categoryId'=>$Category->guid});
    }
    $ParentCategory->renumberChildren();
    return;
}



sub createCategoriesProductsMap {
    my $self = shift;
    my ($Shop ,$aCategoriesGUID, $aProductsGUID) = @_;

    # generate array of objects products and categories from guid.
    my($aProducts, $aCategories);
    foreach my $CatGUID (@$aCategoriesGUID){
        my $Category = $self->_getObjectFromGuid($Shop, $CatGUID, 'Category');
        push @$aCategories, $Category;
    }
    foreach my $ProdGUID (@$aProductsGUID){
        my $Product = $self->_getObjectFromGuid($Shop, $ProdGUID, 'Product');
        push @$aProducts, $Product;
    }


    # For every product.
    foreach my $Product (@$aProducts){
        # For every category.
        RunBatchProcess('UpdateProducts', sub{
          foreach my $Category (@$aCategories) {
            # add the product to the category
            Transaction( sub {
                # Assign products and categories in the end of the list (max sort-order)
                AssignProductToCategory($Product, $Category, '9999');
                RenumberCategoryProducts($Category);
            });
          }
        });
      RenumberProductCategories($Product);
    }
    $self->_updatePageCache($aCategories, $aProducts);
    return undef;
}



sub removeCategoriesProductsMap {
    my $self = shift;
    my ($Shop ,$aCategoriesGUID, $aProductsGUID) = @_;

    # generate array of objects products and categories from guid.
    my($aProducts, $aCategories);
    foreach my $CatGUID (@$aCategoriesGUID){
        my $Category = $self->_getObjectFromGuid($Shop, $CatGUID, 'Category');
        push @$aCategories, $Category;
    }
    foreach my $ProdGUID (@$aProductsGUID){
        my $Product = $self->_getObjectFromGuid($Shop, $ProdGUID, 'Product');
        push @$aProducts, $Product;
    }

    # remove products and categories mapping
    foreach my $Product (@$aProducts){
        RunBatchProcess('UpdateProducts', sub{
            Transaction( sub {
                foreach my $Category (@$aCategories) {
                    RemoveProductFromCategory($Product, $Category);
                    RenumberCategoryProducts($Category);
                }
            });
        });
        RenumberProductCategories($Product);
    }
    _updatePageCache($aCategories, $aProducts);
    return undef;
}



sub createCategory {
    my $self = shift;
    my ($Category, $LocaleID, $hRequestBody) = @_;

    if($Category->instanceOf('CategorySpecialOffer')){
        Error('BadRequest', {'New category can not be created under leaf category(CategorySpecialOffer)! categoryId'=>$Category->guid});
    }

    my $Locale = $self->_loadLocale($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my $IsVisible = defined $hRequestBody->{'visible'} ? ReadJSONBoolean($hRequestBody->{'visible'}) : 0;
    my $Alias = $Category->uniqueChildAlias($hRequestBody->{'alias'});
    my $Shop = $Category->getSite();
    my $Class = $self->_LoadClassByAlias('Category');

    my $hInfo = {
        'Alias' => $Alias,
        'Site' => $Shop,
        'Class' => $Class,
        'Parent' => $Category,
        'IsVisible' => $IsVisible,
        'VisibleInNavigation' => 1,
    };

    my $NewCategory = $Category->class()->packageObject()->insert($hInfo);

    $NewCategory->set({
        'Name'              => $hRequestBody->{'name'},
        'Title'             => $hRequestBody->{'pageTitle'},
        'NavigationCaption' => $hRequestBody->{'navigationCaption'},
        'Description'       => $hRequestBody->{'description'}
    }, $LanguageID);

    $Category->renumberChildren();

    return $NewCategory;
}

sub updateCategory {
    my $self = shift;
    my ($Category, $LocaleID, $hRequestBody) = @_;

    my $Shop = $Category->getSite();
    my $RootCategory= $Shop->child('Categories');

    if($Category->guid() eq $RootCategory->guid() && $hRequestBody->{'alias'} ne $RootCategory->alias()) {
        Error('BadRequest', {'Root category alias \''.$RootCategory->alias().'\' cannot be changed! guid' => $Category->guid()});
    }

    my $Locale = $self->_loadLocale($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');
    my $hRequestValues =  {
        'Alias'                 => $hRequestBody->{'alias'},
        'Name'                  => $hRequestBody->{'name'},
        'Title'                 => $hRequestBody->{'pageTitle'},
        'NavigationCaption'     => $hRequestBody->{'navigationCaption'},
        'Description'           => $hRequestBody->{'description'},
     };
    if(defined $hRequestBody->{'visible'}){
        $hRequestValues->{'IsVisible'}  = ReadJSONBoolean($hRequestBody->{'visible'});
    }

    if(defined $hRequestBody->{'visibleInNavigation'}){
        $hRequestValues->{'VisibleInNavigation'}  = ReadJSONBoolean($hRequestBody->{'visibleInNavigation'});
    }

    $Category->set($hRequestValues, $LanguageID);

    return;
}



sub _LoadClassByAlias{
    my $self = shift;
    my ($ClassName) = @_;
    return LoadClassByAlias($ClassName);
}



sub getCategoryData {
    my $self = shift;
    my ($Category, $LocaleID, $Context, $IncludeInvisible) = @_;

    my $Locale = $self->_loadLocale($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my @Attributes = qw(
        GUID
        ObjectID
        Alias
        Name
        DisplayTitle
        NavigationCaption
        DisplayDescription
        WebPath
        WebUrl
        ImageMedium
        ImageSmall
        Parent
        IsVisibleContentView
        ReadPermissionFor
        IsVisible
        VisibleInNavigation
    );

    if(defined $IncludeInvisible && lc($IncludeInvisible) eq 'true'){
        push @Attributes, 'SubCategories'
    } else {
        push @Attributes, 'VisibleSubCategories'
    }

    my $hCategory = $Category->get(\@Attributes, $LanguageID);

    my $IsVisible = $hCategory->{'IsVisible'} ? JSON::XS::true : JSON::XS::false;
    my $VisibleInNavigation = $hCategory->{'VisibleInNavigation'} ? JSON::XS::true : JSON::XS::false;

    # check read permission, and only allow, if Everyone is allowed.
    if (defined($hCategory->{'ReadPermissionFor'}) && $hCategory->{'ReadPermissionFor'} != $self->_LoadRootObject()->child('Groups')->child('Everyone')->id()) {
        Error('Forbidden');
    }

    my $ImageBaseURL = $Context->imageBaseUrl() . $hCategory->{'WebPath'};
    my $APIBaseURL   = $Context->apiBaseUrl();
    my $UnityFacetedSearchShowFacetsOnCategory = $Context->unityFacetedSearchShowFacetsOnCategory();
    my $FacetedSearchStartDepth = $Context->facetedSearchStartDepth();

    my $SpecialOffer = $self->_getSpecialOffer($Category);

    my $hResult = {
        'categoryId'        => $hCategory->{'GUID'},
        'objectId'          => $hCategory->{'ObjectID'},
        'alias'             => $hCategory->{'Alias'},
        'name'              => $hCategory->{'Name'},
        'visible'           => $IsVisible,
        'pageTitle'         => $hCategory->{'DisplayTitle'},
        'specialOffer'      => $SpecialOffer,
        'navigationCaption' => $hCategory->{'NavigationCaption'},
        'description'       => $hCategory->{'DisplayDescription'},
        'sfUrl'             => $hCategory->{'WebUrl'},
        'images'            => $self->_getImages( $hCategory, $ImageBaseURL ),
        'parent'            => $self->_getParentLink( $hCategory, $APIBaseURL, $LanguageID ),
        'subCategories'     => $self->_getSubCategoryLinks( $hCategory, $APIBaseURL, $LanguageID, $IncludeInvisible ),
        'visibleInNavigation' => $VisibleInNavigation,
        'facetedSearchShowFacetsOnCategory' => $UnityFacetedSearchShowFacetsOnCategory ? JSON::XS::true : JSON::XS::false,
        'facetedSearchCategoryStartDepth' => $FacetedSearchStartDepth,
    };

    return $hResult;
}



sub _getSpecialOffer {
    my $self = shift;
    my ($Category) = @_;
    my $CategoryClassAlias = $Category->class()->get('Alias');
    if($CategoryClassAlias eq 'CategorySpecialOffer'){
        return 1;
    }
    return 0;
}



sub getCategoriesForProduct {
    my $self = shift;
    my ($ProductID, $LocaleID, $Context, $Shop) = @_;

    my $Product = $self->_getObjectFromGuid($Shop, $ProductID, 'Product');

    my $aCategories = $Product->get('Categories');
    my @aOutCategories;
    foreach my $Category (@$aCategories){
            my $hCategory = $self->getCategoryData($Category, $LocaleID, $Context);
            push @aOutCategories, $hCategory;
    }
    return \@aOutCategories;
}



sub _getParentLink {
    my $self = shift;
    my ($hCategory, $APIBaseURL, $LanguageID) = @_;

    if ( $hCategory->{'Parent'}->instanceOf('Category') ) {
        my $hLink = $self->_getLinkToCategory( $hCategory->{'Parent'}, $APIBaseURL, $LanguageID );
        return $hLink;
    }
    return undef;
}



sub _getSubCategoryLinks {
    my $self = shift;
    my ($hCategory, $APIBaseURL, $LanguageID, $IncludeInvisible) = @_;

    my @SubCategories;
    if(defined $IncludeInvisible && lc($IncludeInvisible) eq 'true'){
        @SubCategories = @{ $hCategory->{'SubCategories'} };
    } else {
        @SubCategories = @{ $hCategory->{'VisibleSubCategories'} };
    }

    my @CheckedCategegories = grep { $self->_isCategory($_) } @SubCategories;
    return [ map { $self->_getLinkToCategory( $_, $APIBaseURL, $LanguageID ) } @CheckedCategegories ];
}



sub _isCategory {
    my $self = shift;
    my ($Category) = @_;

    return $Category->class()->isSubClassOf('Category');
}



sub _getLinkToCategory {
    my $self = shift;
    my ($Category, $APIBaseURL, $LanguageID) = @_;

    return {
        'rel' => $Category->alias(),
        'title' => $Category->get( 'NavigationString', $LanguageID ),
        'href' => DE_EPAGES::Catalog::API::Resource::Category->href( $APIBaseURL, $Category->guid() )
    };
}



sub _getImages {
    my $self = shift;
    my ($hCategory, $BaseURL) = @_;

    return [
        map { $self->_getImage( $_, $hCategory->{$_}, $BaseURL ) }
        grep { defined $hCategory->{$_} } qw ( ImageMedium ImageSmall )
    ];
}



sub _getImage {
    my $self = shift;
    my ($Classifier, $Value, $BaseUrl) = @_;

    # Remove 'Image'-prefix (ImageHotdeal -> Hotdeal)
    $Classifier =~ s/^Image//;

    return {
        'classifier' => $Classifier,
        'url'        => $BaseUrl . '/' . $Value,
    };
}



sub _LoadRootObject {
    return LoadRootObject();
}



sub _loadLocale {
    my $self = shift;
    my ($LocaleID) = @_;
    return DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
}



sub _getObjectFromGuid {
    my $self = shift;
    my($Shop, $GUID, $Type) = @_;
    my $Object = eval {$Shop->loadObjectByGuid($GUID, $Type)};
        if (ExistsError()) {
            my ($Error) = GetError();
            Error('BadRequest', {
                $Type.' not found! '.lcfirst $Type.'Id' => $GUID,
            }) if ElementOf($Error->code, qw(ClassMismatch SiteMismatch
                                             NoResults_ObjectByGUID));
            Error($Error);
        }
    return $Object;
}



sub _updatePageCache {
    my $self = shift;
    my($aCategories, $aProducts) = @_;

    foreach my $Category (@$aCategories) { $Category->updatePageCache; }

    foreach my $Product (@$aProducts){ $Product->updatePageCache; }
}

1;
