#========================================================================================
# package      DE_EPAGES::Catalog::API::Object::PagerCategoryProducts
# base         DE_EPAGES::Presentation::API::Object::Pager
# state        public
#----------------------------------------------------------------------------------------
# description Page visible product from Category. Support basic sorting by Attribute.
#              Requiere option CurrencyID for sorting by list price.
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package DE_EPAGES::Catalog::API::Object::PagerCategoryProducts;
use parent DE_EPAGES::Presentation::API::Object::Pager;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

sub items {
    my $self = shift;
    my ($Page) = @_;

    my $aSorted = $self->sortObjects($self->get('Items'));
    my @Items = @{$aSorted}[$self->firstItemOfPage($Page) .. $self->lastItemOfPage($Page)];

    return \@Items;
}

sub countItems {
    my $self = shift;

    return scalar @{$self->get('Items')};
}

#========================================================================================
# function     sortObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSortedObjects = $Pager->sortObjects($aObjects);
#----------------------------------------------------------------------------------------
# description  Sorts the objects according to the 'OrderBy' and 'OrderDesc' attributes.
#               The parameter CurrencyID is required for sorting by ListPrice.
#               For sorting by attributes with value selection, add ".Value", for example
#               OrderBy => 'Size.Value'.
#               See also: <function DE_EPAGES::Product::API::Object::PagerLoopProducts::sortObjects>.
#----------------------------------------------------------------------------------------
# input        $aObjects | list of products | ref.array.object
# return       $aSortedObjects | sorted list of products  | ref.array.object
#========================================================================================
sub sortObjects {
    my $self = shift;
    my ($aObjects) = @_;

    return [] unless scalar @$aObjects;

    my @OrderBy = defined $self->orderBy ? split /\./, $self->orderBy : ();

    # use the unsorted/presorted list if OrderBy is empty
    return $aObjects unless scalar @OrderBy;

    # get values to sort
    my $OrderBy = shift @OrderBy;
    my @Values;
    my $Type;
    my $FirstObject = $aObjects->[0];
    if( $OrderBy eq 'ListPrice' ) {
        my $TaxModel = $FirstObject->getSite()->get('TaxModel');
        my $CurrencyID = $self->get('CurrencyID');
        Error( 'NoCurrencyID' ) unless defined $CurrencyID;
        @Values = map { $_->getListPrice($CurrencyID, $TaxModel) } @$aObjects;
        $Type = 'Money';
    }
    else {
        @Values = map {
            my $Value = $_->getWithParent($OrderBy);
            $Value = $Value->get($_) foreach @OrderBy; # e.g. Size.Value
            $Value;
        } @$aObjects;
        $Type = $FirstObject->class->attribute( $OrderBy )->get('Type');
    }

    # start with undefined values
    my @SortedKeys = grep { not defined $Values[$_] } (0..scalar(@Values)-1);

    # sort different on different types
    if ( ElementOf($Type, qw(Integer Float Boolean Money)) ) {
        push @SortedKeys, sort {$Values[$a] <=> $Values[$b]} grep { defined $Values[$_] } (0..scalar(@Values)-1);
    } else {
        push @SortedKeys, sort {uc $Values[$a] cmp uc $Values[$b]} grep { defined $Values[$_] } (0..scalar(@Values)-1);
    }
    my @Objects = @$aObjects[@SortedKeys];

    # sort descending ?
    @Objects = reverse @Objects if $self->orderDesc;

    return \@Objects;
}

1;
