#========================================================================================
# package      DE_EPAGES::Catalog::API::Resource::Category
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for a category
#========================================================================================
package DE_EPAGES::Catalog::API::Resource::Category;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use DE_EPAGES::Catalog::API::CategoryService;
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use JSON;

use strict;

no warnings 'redefine'; ##no critic qw(ProhibitNoWarnings)

sub service {
    return LoadObjectByPath('/RestServices/Category');
}



sub class {
    return 'Category';
}



sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/json'],
            ['get', 'application/vnd.epages.v1+json'],
        ],
        POST => [
            ['post', 'application/json'],
            ['post', 'application/vnd.epages.v1+json'],
        ],
        PUT => [
            ['put', 'application/json'],
            ['put', 'application/vnd.epages.v1+json'],
        ],
        DELETE => [
            ['delete', 'application/json'],
            ['delete', 'application/vnd.epages.v1+json'],
        ]
    }->{$RequestMethod};
}



sub match {
    my $self = shift;
    my ($Request) = @_;

    return 1 if ( $Request->uri->path =~ m|^/api/categories/?$| );
    return unless $Request->uri->path =~ m|^/api/categories/([^/]+)/?$|;
    return $1;
}



sub requestContext {
    my $self = shift;
    my ( $Site, $GUID, $Request ) = @_;

    if ( $GUID eq '1' ) {
        $GUID = $Site->child('Categories')->guid;
    }
    return $self->SUPER::requestContext( $Site, $GUID, $Request );
}



sub params {
    my $self = shift;
    my ( $ActionName, $Object, $Request ) = @_;

    if ($ActionName eq 'get') {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        my $ProductID = $hQuery->{'productId'}->[0];
        my $IncludeInvisible = $hQuery->{'includeInvisible'}->[0];
        return [ $Object, $LocaleID, $ProductID, $IncludeInvisible ];
    } elsif ($ActionName eq 'post') {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        return [ $Object, $LocaleID ];
    } elsif ($ActionName eq 'put') {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        return [ $Object, $LocaleID ];
    } elsif ($ActionName eq 'delete') {
        return [ $Object ];
    }
    else {
        return [$Object];
    }
}



sub href {
    my $self = shift;
    my ( $APIBaseURL, $GUID ) = @_;
    return "$APIBaseURL/api/categories/$GUID";
}



sub get {
    my $self = shift;
    my ( $Category, $LocaleID, $ProductID, $IncludeInvisible ) = @_;

    my $Shop = $Category->getSite();
    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $CategoryService = $self->_createCategoryService();

    if(defined $ProductID){
        return $CategoryService->getCategoriesForProduct($ProductID, $LocaleID, $Context, $Shop);
    }
    return $CategoryService->getCategoryData($Category, $LocaleID, $Context, $IncludeInvisible);
}



sub post {
    my $self = shift;
    my ( $Category, $LocaleID, $hRequestBody ) = @_;

    my $Shop = $Category->getSite();
    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $CategoryService = $self->_createCategoryService();

    my $NewCategory;
    $self->_RunBatchProcess('CreateCategories', sub {
        $NewCategory = $CategoryService->createCategory($Category, $LocaleID, $hRequestBody);
    });

    return $CategoryService->getCategoryData($NewCategory, $LocaleID, $Context, 'true');
}



sub put{
    my $self = shift;
    my ( $Category, $LocaleID, $hRequestBody ) = @_;

    my $Shop = $Category->getSite();
    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $CategoryService = $self->_createCategoryService();

    $self->_RunBatchProcess('UpdateCategories', sub {
        $CategoryService->updateCategory($Category, $LocaleID, $hRequestBody);
    });

    return $CategoryService->getCategoryData($Category, $LocaleID, $Context, 'true');
}



sub delete {
    my $self = shift;
    my ( $Category ) = @_;

    my $CategoryService = $self->_createCategoryService();

    $self->_RunBatchProcess('DeleteCategories', sub {
        $CategoryService->deleteCategory($Category);
    });

    return;
}



sub _createCategoryService {
    my $self = shift;
    return DE_EPAGES::Catalog::API::CategoryService->new();
}



sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}



sub _RunBatchProcess {
    my $self = shift;
    my ($BatchTaskName, $cSub) = @_;
    RunBatchProcess($BatchTaskName, $cSub);
    return;
}

1;
