#========================================================================================
# package      DE_EPAGES::Catalog::API::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  Contains basic functions for ShopPagerSettings
#========================================================================================
package DE_EPAGES::Catalog::API::Shop;
use parent qw ( Exporter);

use strict;

use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString TestInteger );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Catalog::API::Constants qw ( PAGER_SETTING_SF_PREFIX );
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::TemplateType qw ( GetTemplateTypeAlias );

our @EXPORT_OK = qw (
    GetPageSizeForCategory
    GetPageSizeForObject
    GetPageSize
    SetPageSize
);

#========================================================================================
# function     SetPageSize
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetPageSize( $Shop, $Alias, $PageSize );
#----------------------------------------------------------------------------------------
# description  Store PageSize for specific layout for the current shop.
#----------------------------------------------------------------------------------------
# input        $Shop | current shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $Alias | valid alias for layout | string
# input        $PageSize | page size for specific layout | integer
#========================================================================================
sub SetPageSize {
    my ( $Shop, $Alias, $PageSize ) = @_;
    TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );
    TestString( 'Alias', $Alias );
    TestInteger( 'PageSize', $PageSize );

    my $SysFolder = LoadRootObject->child('PagerSettings');
    my $SysSetting = $SysFolder->child($Alias);
    $SysSetting->validatePageSize($PageSize);
    my $hPageSizes = $Shop->get('PageSizes');
    $hPageSizes->{$Alias} = $PageSize;
    $Shop->set({'PageSizes' => $hPageSizes});
    # show correct PageSize in Pager templates
    $Shop->clearPageCache();
}

#========================================================================================
# function     GetPageSize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageSize = GetPageSize( $Shop, $Alias );
#----------------------------------------------------------------------------------------
# description  Return PageSize for requested layout
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $Alias | valid alias for layout | string
# return       $PageSize | number of products per page  | integer
#========================================================================================
sub GetPageSize {
    my ( $Shop, $Alias ) = @_;
    TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );
    TestString( 'Alias', $Alias );

    my $hPageSizes = $Shop->get('PageSizes');
    my $PageSize = $hPageSizes->{$Alias} //
        LoadRootObject->child('PagerSettings')->child($Alias)->get('DefaultPageSize');

    GetLog->debug( "Get page size for $Alias = $PageSize");
    return $PageSize;
}

#========================================================================================
# function     GetPageSizeForCategory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageSize = GetPageSizeForCategory( $Category );
#----------------------------------------------------------------------------------------
# description  Return PagerSetting for this $Category. Is 'CoverFlow_Color1' or
#               'CoverFlow_Color2' as category layout selected, then $PageSize is not
#               defined.
#----------------------------------------------------------------------------------------
# input        $Category | category object | DE_EPAGES::Content::API::Object::Category
# return       $PageSize | number of products per page  | integer or undef
#========================================================================================
sub GetPageSizeForCategory($) {
    my ($Category) = @_;
    TestObject( 'Category', $Category, 'DE_EPAGES::Content::API::Object::Category' );

    my $PageType = LoadPageTypeByAlias('SF-Category');
    my $TemplateTypeAlias = GetTemplateTypeAlias( $Category, $PageType, 'Content-Products' );
    return undef if ElementOf($TemplateTypeAlias, qw(CoverFlow_Color1 CoverFlow_Color2));

    return GetPageSize( $Category->getSite(), PAGER_SETTING_SF_PREFIX . $TemplateTypeAlias );
}

#========================================================================================
# function     GetPageSizeForObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageSize = GetPageSizeForObject( $Object, $PageTypeAlias, $TemplateTypeFolderAlias );
#----------------------------------------------------------------------------------------
# example      $PageSize = GetPageSizeForObject( $Category, 'SF-Category', 'Content-Products' );
#----------------------------------------------------------------------------------------
# description  Return default PageSize for given $Object and $PageTypeAlias. Don't use this function directly
#               instead use wrapper function like <function GetPageSizeForCategory>.
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $PageTypeAlias | PageType name | string
# input        $TemplateTypeFolderAlias | folder name | string
# return       $PageSize | number of product per page | integer
#========================================================================================
sub GetPageSizeForObject ($$$) {
    my ( $Object, $PageTypeAlias, $TemplateTypeFolderAlias ) = @_;
    TestObject('Object', $Object);
    TestString('PageTypeAlias', $PageTypeAlias);
    TestString('TemplateTypeFolderAlias', $TemplateTypeFolderAlias);

    my $PageType          = LoadPageTypeByAlias($PageTypeAlias);
    my $TemplateTypeAlias = GetTemplateTypeAlias( $Object, $PageType, $TemplateTypeFolderAlias );
    return GetPageSize( $Object->getSite(), PAGER_SETTING_SF_PREFIX . $TemplateTypeAlias );
}

1;
