#========================================================================================
# package      DE_EPAGES::Catalog::Hooks::ProductFolder
# state        public
# description  assign categories from the original product to the cloned product,
#               load category for new product
#========================================================================================
package DE_EPAGES::Catalog::Hooks::Product;

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Catalog::API::Catalog qw (AssignProductToCategory
                                         RemoveProductFromCategory
                                         RenumberCategoryProducts
                                         RenumberProductCategories);
use DE_EPAGES::Object::API::Factory qw ( Transaction );
use DE_EPAGES::Presentation::UI::Object;
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

#========================================================================================
# function     OnSaveProduct
#----------------------------------------------------------------------------------------
# syntax       OnSaveProduct($hParams);
#----------------------------------------------------------------------------------------
# description  Assign categories to a newly created product
#               (Hook UI_OnSaveProduct).
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - product - object</li>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveProduct {
    my ($hParams) = @_;
    my $Product = $hParams->{'Object'};
    my $Servlet = $hParams->{'Servlet'};

    my $Shop = $Product->getSite();
    my $ProductsFolder = $Shop->folder('Products');

    #-- See Templates/UnityMBO/UnityMBO-ProductNew.Visibility-AssignCategory.html
    my $ParameterAlias = $Shop->id . '_CategoryName_' . $Product->id;

    my @FormCategoryIDs;
    #-- Existing products will have this parameter using the product id
    if ( defined $Servlet->form->value($ParameterAlias) ) {
      @FormCategoryIDs = split(/,/, $Servlet->form->value($ParameterAlias));
    } else {
      #-- New product screen, presumably, uses the products folder as the id
      $ParameterAlias = $Shop->id . '_CategoryName_' . $ProductsFolder->id;
      @FormCategoryIDs = split(/,/, $Servlet->form->value($ParameterAlias));
    }

    my @ProductCategoryIDs;
    my $aProductCategories = $Product->get('Categories');
    for my $ProductCategory (@$aProductCategories) {
        $ProductCategory->id;
        push(@ProductCategoryIDs, $ProductCategory->id);
    }

    RunBatchProcess('UpdateProducts', sub {
        foreach my $CategoryID (grep { /\S/ } @ProductCategoryIDs) {

            #Category of product is not present in the input form --> remove it
            unless (ElementOf($CategoryID, @FormCategoryIDs)) {
                my $Category = $Shop->loadObject($CategoryID, 'Category');

                # remove the product from the category
                Transaction( sub {
                    RemoveProductFromCategory($Product, $Category);
                    RenumberCategoryProducts($Category);
                    RenumberProductCategories($Product);
                });
            }
        }

        foreach my $CategoryID (grep { /\S/ } @FormCategoryIDs) {
            my $Category = $Shop->loadObject($CategoryID, 'Category');

            # add the product to the category
            Transaction( sub {
                my $ReturnValue = AssignProductToCategory($Product, $Category);
                # only renumber category products in case a new product has been assigned
                RenumberCategoryProducts($Category) if $ReturnValue == 1;
            });
        }
    });

    my $ahListItems = DE_EPAGES::Presentation::UI::Object->listItems($Servlet, $Product, 'CategoryProductMap');

    # save the new positions
    foreach my $hMap (@$ahListItems) {
        $hMap->{'Object'}->set({'PositionCategory', $hMap->{'Position'}});
    }

    RenumberProductCategories($Product);

    return;
}

1;
