#========================================================================================
#   Required StoreDB
#========================================================================================

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath );

use DE_EPAGES::Catalog::API::Shop qw ( GetPageSize SetPageSize GetPageSizeForCategory);

use Test::More tests            => 15;

use constant PAGE_SIZES_ARRAY   => [ 10, 20, 30, 40, 50 ];
use constant PAGE_SIZE          => 50;
use constant INVALID_PAGE_SIZE  => 35;
use constant SHOP_PAGE_SIZES    => {
   'SF_ProductsSingleColumn' => 50
};
my $ROOTPATH = 'Test';
my $SHOP = 'TestShop';
my $SHOPPATH = join '/', $ROOTPATH, $SHOP;
my $SINGLE_COL = 'SF_ProductsSingleColumn';

## no critic (Tics::ProhibitLongLines)
#========================================================================================
#   MAIN
#========================================================================================
RunScript(
    Sub => sub {
        RunOnStore(
            'Store' => GetTestSite(),
            'Sub'   => sub {
                SKIP: {
                    skip('Not a StoreDB - folder /System/PagerSettings is missing')
                        unless LoadRootObject->existsChild('PagerSettings');
                     ok(Init(), "import $SHOPPATH");

                     TestSytemPagerSetting();
                     TestShopPagerSizes();
                     ok(Cleanup(), "clean $SHOPPATH");
                }
            }
        );
    }
);

sub TestSytemPagerSetting {
    my $Sys                 = LoadRootObject();
    my $FolderPagerSettings = $Sys->child('PagerSettings');

    my $SingleColumn        = $FolderPagerSettings->child($SINGLE_COL);
    my $aPageSizes = $SingleColumn->get('PageSizes');
    is_deeply( $aPageSizes, PAGE_SIZES_ARRAY, 'get handler \'PageSizes\'' );

    foreach ( @{ $FolderPagerSettings->children } ) {
        $aPageSizes = $_->get('PageSizes');
        ok( ElementOf($_->get('DefaultPageSize'), $aPageSizes),
            sprintf( 'Setting \'%s\' - DefaultPageSize is valid value', $_->alias ) )
        or Test::More::diag($_->get('DefaultPageSize') . ' is not element of ' . join (', ', @$aPageSizes));
    }
}

sub TestShopPagerSizes {
    my $Sys = LoadRootObject;
    my $SysSetting = $Sys->child('PagerSettings')->child($SINGLE_COL);
    my $Shop = GetShop();

    my $hPageSizes = $Shop->get('PageSizes');
    is_deeply($hPageSizes, undef, 'new shop has no PageSizes stored' );

    my $PageSize = GetPageSize($Shop, $SINGLE_COL);
    is($PageSize, $SysSetting->get('DefaultPageSize'), 'new shop show system default as PageSize');

    SetPageSize($Shop, $SINGLE_COL, PAGE_SIZE);
    $hPageSizes = $Shop->get('PageSizes');
    is_deeply( $hPageSizes, SHOP_PAGE_SIZES, 'Shop.PageSizes was created');

    $PageSize = GetPageSize($Shop, $SINGLE_COL);
    is($PageSize, PAGE_SIZE, 'get new PageSize from Shop (no longer sys defaul)');

    eval {SetPageSize($Shop, $SINGLE_COL, INVALID_PAGE_SIZE);};
    ok(ExistsError() && GetError->code eq 'InvalidPageSize', 'PageSize validation works');

    $Shop = LoadRootObject->child('Shops')->child('DemoShop');
    my $Jackets = $Shop->child('Categories')->child('Jackets');
    $PageSize = undef;
    $PageSize = GetPageSizeForCategory($Jackets);
    ok($PageSize =~ m/\d+/, 'PageSize for Jackets');
}

#========================================================================================
#   Helper Functions
#========================================================================================
sub GetShop {
    return LoadObjectByPath($SHOPPATH);
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    my $Root = LoadRootObject();
    $Root->child($ROOTPATH)->delete if $Root->existsChild('Test');

    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { 'Alias' => $ROOTPATH } )->insertChild('Shop', {'Alias' => $SHOP});
    return 1;
}
