#========================================================================================
# package      DE_EPAGES::CodeGenerator::API::ModelTLEHandler
# state        private
#========================================================================================

package DE_EPAGES::CodeGenerator::API::ModelTLEHandler;

use strict;

use DE_EPAGES::CodeGenerator::API::XMIProfileImportHandler qw (importXML);
use DE_EPAGES::Core::API::Error qw (Error);

## no critic qw(ProhibitMatchVars ProhibitCaptureWithoutTest)

my %TYPE_POSITION = ('Input'=>0,'Regional'=>1,'Html'=>2,'Perl'=>3, 'MySQL'=>4, 'PgPP'=>5, 'SQLite'=>6 );
my %TYPEMAPPING = (
    'bit'       => ['boolean',  'integer',   '_INT',   'Boolean',  'bool',  'boolean',  'integer'],
    'char'      => ['text',     'string',    '_HTML',  'String',   'char',      'char',     'text'],
    'datetime'  => ['text',     'reg_date',  '_DATE',  'DateTime', 'datetime',  'timestamp','text'],
    'float'     => ['text',     'reg_float', '_FLOAT', 'Float',    'double',    'float',    'real'],
    'int'       => ['text',     'integer',   '_INT',   'Integer',  'integer',   'integer',  'integer'],
    'integer'   => ['text',     'integer',   '_INT',   'Integer',  'integer',   'integer',  'integer'],
    'money'     => ['text',     'reg_money', '_MONEY', 'Money',    'decimal',   'decimal',  'numeric'],
    'nchar'     => ['text',     'string',    '_HTML',  'String',   'varchar',   'char',     'text'],
    'numeric'   => ['text',     'reg_float', '_FLOAT', 'Float',    'decimal',   'decimal',  'numeric'],
    'nvarchar'  => ['text',     'string',    '_HTML',  'String',   'varchar',   'varchar',  'text'],
    'smallint'  => ['boolean',  'integer',   '_INT',   'Integer',  'smallint',  'smallint', 'integer'],
    'text'      => ['textarea', 'textarea',  '_HTML',  'String',   'longtext',  'text',     'text'],
    'tinyint'   => ['integer',  'integer',   '_INT',   'Integer',  'tinyint',   'smallint', 'integer'],
    'varchar'   => ['text',     'string',    '_HTML',  'String',   'varchar',   'varchar',  'text'],
);

my %MODEL_DEFAULTS = (
#   'Mand'                  => 0,
#    'IsPartOfPKey'          => 0,
#    'AllowEmpty'            => 0,
#    'IsFirstNonPKeyItem'    => 0,
#    'HasText'               => 0,
#    'Desc'                  => 0,
#    'SelfReference'         => 0,
#    'AutoFill'              => '',
    'DateFormat'            => 'Date',
#    'DeleteReference'       => 1,
);

my @OTHER_ATTRIBUTES = qw ( Type_Length Type_StringLength XMIDEFAULTS );
sub new { bless {}, shift; }

sub register {
    my $self = shift;
    my ($Processor) = @_;
    foreach my $Type (keys %TYPE_POSITION) {
        $Processor->registerHandler('VariableHandler', $self, 'Type'.$Type);
    }
    foreach my $TLEName (keys %MODEL_DEFAULTS, @OTHER_ATTRIBUTES) {
        $Processor->registerHandler('VariableHandler', $self, $TLEName);
    }
    $Processor->registerHandler('FunctionHandler', $self, 'FileViaPackage');
    $Processor->registerHandler('FunctionHandler', $self, 'GetXMIID');
    $Processor->registerHandler('FunctionHandler', $self, 'XMIIDforName');
    $Processor->registerHandler('FunctionHandler', $self, 'StereoTypes');
    $Processor->registerHandler('FunctionHandler', $self, 'ElementAt');
    $Processor->registerHandler('FunctionHandler', $self, 'Cardinality');
    $Processor->registerHandler('FunctionHandler', $self, 'CartridgeForClassName');
    return $self;
}

sub tle {
    my $self = shift;
    my ($Processor, $TLEName) = @_;
    if ($TLEName eq 'Type_Length') {
       my $Type = $Processor->tle('Type');
        return '(19,4)' if $Type eq 'money';
        $Type =~ s/^[^\(]+//;
        return $Type;
    }
    elsif ($TLEName eq 'XMIDEFAULTS') {
        return q(isSpecification = 'false' isRoot = 'false' isLeaf = 'false' isAbstract = 'false');
    }
    elsif ($TLEName eq 'Type_StringLength') {
        my $Type = $Processor->tle('Type');
        $Type =~ s/^[^\(]+//;
        my ($Length) = $Type =~ /\((\d+)/;
        return $Length;
    }
    elsif ($TLEName =~ /^Type/) {
        my $Type = $';
        my $ID = $Processor->tle('Type');
        return undef unless defined $ID;
        $ID =~ s/\(.*$//g; #remove length
        return undef unless defined $TYPEMAPPING{$ID}[$TYPE_POSITION{$Type}];
        return $TYPEMAPPING{$ID}[$TYPE_POSITION{$Type}];
    }

    return undef;
}

sub existsTLE {
    my $self = shift;
    my ($Processor, $TLEName) = @_;
    foreach my $Type (keys %TYPE_POSITION) {
        return 1 if $TLEName eq 'Type'.$Type;
    }
    foreach my $Name (keys %MODEL_DEFAULTS, @OTHER_ATTRIBUTES) {
        return 1 if $TLEName eq $Name;
    }
}

sub FileViaPackage {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my $Package = $raParams->[0];
    $Package =~ s/::/\//g;
    return $Package;
}

sub _loadXMIProfile {
    my $self = shift;
    my ($Processor) = @_;
    my $Context = $Processor->getContext();
    my $XMIProfile = $Context->{'XMIProfile'};
    unless ( $XMIProfile ) {
        my $Importer = DE_EPAGES::CodeGenerator::API::XMIProfileImportHandler->new;
        $XMIProfile = $Importer->importXML("$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/CodeGenerator/Models/EPAGES.xmi", 1);
        $Context->{'XMIProfile'} = $XMIProfile;
    }
    return $XMIProfile;
}

sub XMIIDforName {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($Name) = @$raParams;

    my $XMIProfile = $self->_loadXMIProfile( $Processor );
    return $XMIProfile->{'DataTypes'}->{$Name} if $XMIProfile->{'DataTypes'}->{$Name};
    return $XMIProfile->{'StereoTypes'}->{$Name} if $XMIProfile->{'StereoTypes'}->{$Name};
    return $XMIProfile->{'TagDefinitions'}->{$Name} if $XMIProfile->{'TagDefinitions'}->{$Name};

    return $XMIProfile->{'DataTypes'}->{'char'} if $Name =~ /^char/;
    return $XMIProfile->{'DataTypes'}->{'nchar'} if $Name =~ /^nchar/;
    return $XMIProfile->{'DataTypes'}->{'varchar'} if $Name =~ /^varchar/;
    return $XMIProfile->{'DataTypes'}->{'nvarchar'} if $Name =~ /^nvarchar/;
    return $XMIProfile->{'DataTypes'}->{'int'} if $Name =~ /int/;
    return $XMIProfile->{'DataTypes'}->{'numeric'} if $Name =~ /(decimal|numeric)/;

    Error('DATATYPE_MISSING', { Datatype=>$Name });
}

sub GetXMIID {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($Name) = @$raParams;
    my $Context = $Processor->getContext();

    my $XMIIDs = $Context->{'XMIIDs'};
    unless($XMIIDs) {
        $XMIIDs = {};
        $Context->{'XMIIDs'} = $XMIIDs;
    }
    return $XMIIDs->{$Name} if $XMIIDs->{$Name};

    # ID has to be unique inside of a single document
    # according to the xmi spec, argo will work with
    # IDs generated this way
    my $ID = $Context->{'XMIID'} // 1;
    my $XMIID = $ID++;
    $Context->{'XMIID'} = $ID;
    $XMIIDs->{$Name} = $XMIID;
    return $XMIID;
}

sub StereoTypes {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($Object) = @$raParams;
    my @Result;
    push @Result, {'StereoType' => 'AutoIncr'} if $Object->{'AutoIncremental'} eq 'true';
    push @Result, {'StereoType' => 'AFCreation'} if $Object->{'AutoFill'} eq 'CreationDate';
    push @Result, {'StereoType' => 'AFChange'} if $Object->{'AutoFill'} eq 'ChangeDate';
    push @Result, {'StereoType' => 'NotNull'} if $Object->{'IsNotNull'} eq 'true' && (!$Object->{'IsPartOfPK'} || $Object->{'IsPartOfPK'} eq 'false');
    push @Result, {'StereoType' => 'PK'} if $Object->{'IsPartOfPK'} eq 'true';
    push @Result, {'StereoType' => 'RefAttribute'} if $Object->{'ObjectClass'} || $Object->{'ObjectName'};
    return \@Result;
}

sub Cardinality {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($Type) = @$raParams;

    return [4,4] if $Type =~ /^(int|integer)$/;
    return [2,2] if $Type =~ /^smallint$/;
    return [1,1] if $Type =~ /^tinyint$/;

    return [$1,$1] if $Type =~ /\(([^,]*?)\)$/;
    return [$1,$2] if $Type =~ /\(([^,]*?),([^,]*?)\)$/;
    return [1,1];
}

sub ElementAt {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($Array, $Index) = @$raParams;

    return $Array->[$Index];
}

sub CartridgeForClassName {
    my $self = shift;
    my ($Processor, $raParams) = @_;
    my ($Model, $ClassName) = @$raParams;
    foreach my $Table ( @{ $Model->{'Tables'} } ) {
        return $Table->{'Cartridge'} if $Table->{'Name'} eq $ClassName;
    }

    foreach my $RefModel ( @{ $Model->{'RefModels'} } ) {
        foreach my $RefTable ( @{ $RefModel->{'RefTables'} } ) {
            return $RefTable->{'Cartridge'} if $RefTable->{'Name'} eq $ClassName;
        }
    }
    Error('NO_CARTDRIDGE', { Model=>$Model, ClassName=>$ClassName});
}

1;
