#========================================================================================
# package      DE_EPAGES::Content::API::CSV::ContentImportHandler;
# base         DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to import content objects
#               See <package DE_EPAGES::Content::API::CSV::ContentExportImport> for
#               usage examples.
#========================================================================================
package DE_EPAGES::Content::API::CSV::ContentImportHandler;
use base qw ( DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler );

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError);
use DE_EPAGES::Core::API::Image qw (ScaleImage);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf trim);
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Content::API::Constants qw ( CATEGORY_IMAGES_SIZES );
use DE_EPAGES::Presentation::API::Constants qw (
    IS_LENGTH_CHECK_ACTIVE
    EPAGES6_LOCALIZED_FIELDS_LENGTH
    UNITY_LOCALIZED_FIELDS_LENGTH
);
use DE_EPAGES::Content::API::ShortUrl qw ( GetShortUrlByClass TestShortUrl );
use DE_EPAGES::CSVExportImport::API::Tools qw ( ParseValue ParseString );
use DE_EPAGES::Object::API::Factory qw( LoadObject LoadClassByAlias ExistsClassByAlias );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Permission::API::Permission qw ( ActionByName );
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByViewAction);
use List::Util qw ( max );

#ERRORDEFINITION
sub ErrorDefinition {
    my $hSuperErrorDefinition = __PACKAGE__->SUPER::ErrorDefinition();
    # override base classes warning
    $hSuperErrorDefinition->{'REFERENCED_IMAGE_MISSING'} = 'line #Line, column #AttributeName: referenced image not found';
    return { %$hSuperErrorDefinition,
            'CONTENT_EMPTY_ALIAS'           => 'line #Line, column Alias: alias has empty value',
            'CONTENT_EMPTY_CLASSALIAS'      => 'line #Line, column Class: class has empty value',
            'NO_SUCH_CONTENTCLASS'  => 'line #Line, column #ID: class is not in shop (#ClassAlias)',
            'NO_PARENT'             => 'line #Line, column Parent: Category does not exist (#Parent)',
            'WRONG_CONTENTCLASS'    => 'line #Line, column #ID: wrong class: #New (must be #Old)',
            'SHORTURL_ALREADY_EXISTS' => 'line #Line, column #Column: shorturl already exists.',
            'INVALID_CHARACTER'     => 'line #Line, column #Column: invalid character in shorturl.',
            'SHORTURL_NOT_ALLOWED'  => 'line #Line, column #Column: setting shorturl for this contentobject is not allowed.',
            'WRONG_TEMPLATE'        => 'line #Line, column #Attribute: child \'#Value\' does not exist.',
            'NAME_NOT_VALID'        => 'line #Line, column #Attribute: name \'#Value\' must not include special characters only.',
        }
}
#ERRORDEFINITION

#========================================================================================
# function     analyzeAttributeLine
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $hAttributeValues = $self->analyzeAttributeLine();
#----------------------------------------------------------------------------------------
# description  Analyzes the current line in the CSV file ($self->topID('Item')), parses
#               localized number formats.
#               Returns a hash containing the current object, new attribute values and
#               processing instructions.
#----------------------------------------------------------------------------------------
# example      hAttributeValues='HASH
#                   'DirectAttributes'  =>'HASH'
#                       'Alias' =>'NewShopServices'
#                       'Class' =>'DE_EPAGES::Object::API::Object::Class=HASH'
#                       'Date'  => undef
#                       'Image' =>'NewsImage.gif'
#                       'IsVisible' =>'1'
#                       'Parent'    =>'DE_EPAGES::Content::API::Object::ContentObject=HASH'
#                       'Position'  =>'10'
#                   'DoNotImport'   =>'0'
#                   'LocalizedAttributes'   =>'HASH(0xc8d968c)'
#                       '1' =>'HASH(0xc8dae24)'
#                           'AttachmentTitle'   =>'Unsere Angebote zum Ausdrucken'
#                           'Name'  =>'Frischer Wind im Onlineshop'
#                       '2' =>'HASH(0xc65dc0c)'
#                           'AttachmentTitle'   =>'Print our Specials'
#                           'Name'  =>'Fresh wind in the online shop'
#                   'Layout'    => { $TemplateTypeFolderAlias => $TemplateTypeAlias }
#                   'Object'    =>'DE_EPAGES::Content::API::Object::ContentObject=HASH'
#----------------------------------------------------------------------------------------
# return       $hAttributeValues | properties of a object of CSV file with following keys
#               <li>DirectAttributes  | direct attributes of the object with following keys
#                   <ul>
#                       <li>Alias | alias of the object | string
#                       <li>Class | class object of the object | object
#                       <li>Parent | parent object of the object | object
#                       <li> more direct attributes which should set via $Object->set method,
#                            or can used by $Class->insert($hAttributeValues->{'DirectAttributes'});
#                   </ul> | ref.hash
#               <li>DoNotImport |  0 or 1, set to 1 if there are errors to set the parsed values | boolean
#               <li>Object | object if it is found or undef if it doesn't exist | object
#               <li>LocalizedAttributes |localized attributes hash with language id as key | ref.hash
#               | ref.hash
#========================================================================================
sub analyzeAttributeLine {
    my $self = shift;

    my $DoNotImport      = 0;

    my $Item              = $self->topID('Item');
    my $ImportAttributes  = $self->topID('ImportAttributes');
    my $Shop              = $self->object;

    my $Alias = $Item->[$ImportAttributes->{'AliasColumn'}];
    $Alias =~ s/\s+$//; $Alias =~ s/^\s+//;
    if ($Alias eq '') {
        Warning( 'CONTENT_EMPTY_ALIAS', { Line => $self->lineNumber , ID => $ImportAttributes->{'AliasColumn'}+1} );
        $DoNotImport = 1;
        return;
    }

    my $ContentClass = undef;
    my $ClassAlias = $Item->[$ImportAttributes->{'ClassColumn'}];
    $ClassAlias =~ s/\s+$//; $ClassAlias =~ s/^\s+//;

    # test if we have unity and import only StartPage and Categories
    if($Shop->get('IsUnityRegistered') and not ElementOf($ClassAlias, qw(StartPage Category)) ){
        $DoNotImport = 1;
        return;
    }

    if ($ClassAlias eq '') {
        Warning( 'CONTENT_EMPTY_CLASSALIAS', { Line => $self->lineNumber , ID => $ImportAttributes->{'ClassColumn'}+1} );
        $DoNotImport = 1;
        return;
    } else {
        # get content object class
        $ClassAlias = 'Category' if $ClassAlias eq 'FreeContent'; # for compatibility
        if(ExistsClassByAlias($ClassAlias)){
            $ContentClass = LoadClassByAlias($ClassAlias);
            if(not $ContentClass->isSubClassOf('ContentObject')){
                Warning( 'NO_SUCH_CONTENTCLASS', { Line => $self->lineNumber, ID => $ImportAttributes->{'ClassColumn'} + 1, ClassAlias => $ClassAlias } );
                $DoNotImport = 1;
                return;
            }
        }else{
            Warning( 'NO_SUCH_CONTENTCLASS', { Line => $self->lineNumber, ID => $ImportAttributes->{'ClassColumn'} + 1, ClassAlias => $ClassAlias } );
            $DoNotImport = 1;
            return;
        }
    }

    my $BaseCategory = $Shop->get('Categories');

    # get parent
    my $Parent;
    my $ParentPath = $Item->[$ImportAttributes->{'ParentColumn'}];
    $ParentPath =~ s/\s+$//; $ParentPath =~ s/^\s+//;
    $ParentPath= undef if $ParentPath eq '';
    if ( $Item->[$ImportAttributes->{'ClassColumn'}] eq 'StartPage' ) {
        $Parent = $BaseCategory->parent;
    } elsif (defined $ParentPath) {
        my $ParentID = $BaseCategory->findPath($ParentPath);
        if ( defined $ParentID ) {
            $Parent = LoadObject($ParentID);
        } else {
            Warning('NO_PARENT', { Line => $self->lineNumber, Alias => $Alias,
                                          Parent => $ParentPath } );
            $DoNotImport = 1;
            $Parent = $BaseCategory;
        }
    } else {
        $Parent = $BaseCategory;
    }

    # exists content object ?
    my $ContentObject = $Parent->existsChild($Alias) ? $Parent->child($Alias) : undef;

    if (defined $ContentObject ) {
        my $ExistingContentClass = $ContentObject->class;
        # test that the existing content object has the same class like the import line
        if ( $ExistingContentClass->id != $ContentClass->id ) {
                Warning( 'WRONG_CONTENTCLASS', { Line => $self->lineNumber,
                                                 ID   => $ImportAttributes->{'ClassColumn'}+1,
                                                 Old  => $ExistingContentClass->alias ,
                                                 New  => defined $ClassAlias ? $ClassAlias : ''} );
                $DoNotImport = 1;
                $ContentClass = $ExistingContentClass;
        }
    }

    my %AttributeValues;
    my $hLocalizedFieldsLength = $Shop->get('IsUnityRegistered')
        ? UNITY_LOCALIZED_FIELDS_LENGTH()
        : EPAGES6_LOCALIZED_FIELDS_LENGTH();
    my $hFormat = $self->locale->get([qw(LocaleID DecimalSep ThousandSep)]);

    for (my $i=0; $i<scalar(@$Item) ;$i++) {
        my $hAttribute =  $ImportAttributes->{$i};
        my $Type = $hAttribute->{'Type'};
        #ignore all attributes which are not base, direct or my class
        next unless ($Type eq 'Template' or
                     $hAttribute->{'ClassAlias'} eq 'ContentObject' or
                     $hAttribute->{'ClassAlias'} eq $ContentClass->alias );
        next if ($hAttribute->{'Alias'} eq 'Class' or $hAttribute->{'Alias'} eq 'Parent');

        my ($Value) = trim($Item->[$i]);
        $Value = undef if $Value eq '';

        if ( (!defined $Value) && ($Type ne 'Template') && ($hAttribute->{'Attribute'}->get('IsMandatory')) ) {
            Warning( 'NO_MAND_VALUE', {
                Line => $self->lineNumber,
                Attribute => $hAttribute->{'Alias'}
            });
            $DoNotImport = 1;
            next;
        }

        next unless defined $Value;

        if( $Type =~ /^(?:Localized)String$/ ) {
            my $CurrentAttribute = $hAttribute->{'Attribute'}->get('Alias');
            my $MaxLength = (IS_LENGTH_CHECK_ACTIVE && exists $hLocalizedFieldsLength->{$CurrentAttribute}) ?
                $hLocalizedFieldsLength->{$CurrentAttribute} : $hAttribute->{'Attribute'}->get('Length');
            eval { ParseString($hAttribute->{'Alias'}, \$Value, { MaxLength => $MaxLength }) };
            if ( ExistsError() ) {
                my $Error = GetError();
                Error( $Error) unless $Error->code eq 'STRING_TOO_LONG';
                Warning( 'STRING_TOO_LONG', {
                    Attribute => $hAttribute->{'Alias'},
                    ID => $i+1,
                    Line => $self->lineNumber,
                    Error => $Error->shortMessage,
                    Value => $Value,
                    MaxLength => $MaxLength,
                } );
                $DoNotImport = 1;
                next;
            }
        }
        else {
            eval { ParseValue($hAttribute->{'Alias'}, $Type, \$Value, $hFormat) };
            if ( ExistsError() ) {
                Warning( 'VOID_FIELD', {
                    Attribute => $hAttribute->{'Alias'},
                    ID => $i+1,
                    Line => $self->lineNumber,
                    Error => GetError()->shortMessage,
                } );
                $DoNotImport = 1;
                next;
            }
        }
        if ($hAttribute->{'Alias'} eq 'URI') {
            my $URI = $Value ;
            eval {
                TestShortUrl($Shop, $ContentObject, $URI, $hAttribute->{'LanguageID'});
            };
            if (ExistsError()) {
                my $Error = GetError();
                Warning('SHORTURL_ALREADY_EXISTS',
                    { Line => $self->lineNumber, Column => 'URI/'.GetCodeByLanguageID($hAttribute->{'LanguageID'}) })
                    if $Error->code eq 'SHORTURL_ALREADY_EXISTS';
                Warning('SHORTURL_INVALID_CHARACTER',
                    { Line => $self->lineNumber, Column => 'URI/'.GetCodeByLanguageID($hAttribute->{'LanguageID'}) })
                    if $Error->code eq 'INVALID_CHARACTER';
                Warning('SHORTURL_NOT_ALLOWED',
                    { Line => $self->lineNumber, Column => 'URI/'.GetCodeByLanguageID($hAttribute->{'LanguageID'}) })
                    if $Error->code eq 'ShortURL_NOT_ALLOWED';
                $DoNotImport = 1;
                return;
            }
        }

        # do not allow names consisting of special characters only (UNITY-3683)
        if ($hAttribute->{'Alias'} eq 'Name' && defined $Value && $Value !~ /\w/) {
            Warning( 'NAME_NOT_VALID', {
                'Attribute' => $hAttribute->{'Alias'},
                'ID'=> $i + 1,
                'Line' => $self->lineNumber,
                'Value' => $Value
            });
            $DoNotImport = 1;
            next;
        }

        if ($Type eq 'Template') {
            $AttributeValues{'Layout'}{$hAttribute->{'Alias'}} = $Value;
        }
        elsif ($Type =~ /^Localized/ ) {
            $AttributeValues{'LocalizedAttributes'}{$hAttribute->{'LanguageID'}}{$hAttribute->{'Alias'}} = $Value;
        }
        else {
            $AttributeValues{'DirectAttributes'}{$hAttribute->{'Alias'}} = $Value;
        }
    }

    $AttributeValues{'DirectAttributes'}{'Class'} = $ContentClass;
    $AttributeValues{'DirectAttributes'}{'Parent'} = $Parent;

    $AttributeValues{'DoNotImport'}  = $DoNotImport;
    $AttributeValues{'Object'}       = $ContentObject;
    return \%AttributeValues;
}

#========================================================================================
# function     importSpecialData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importSpecialData($Object, $hAttributeValues);
#----------------------------------------------------------------------------------------
# description  possibility to import special object data, called by importObject
#               saves template design
#----------------------------------------------------------------------------------------
# input        $Object | object which is imported | object
# input        $hAttributeValues    | attribute values | ref.hash
#========================================================================================
sub importSpecialData {
    my $self = shift;
    my ( $Object, $hAttributeValues ) = @_;

    return unless exists $hAttributeValues->{'Layout'};

    foreach my $Key (keys %{$hAttributeValues->{'Layout'}}) {
        my ($ContentClassAlias, $TemplateTypeAlias) = split /\//, $Key;
        next unless $ContentClassAlias eq $Object->class->alias;

        my $TemplateTypeName = $hAttributeValues->{'Layout'}{$Key};
        my $PageType = LoadPageTypeByViewAction( ActionByName( $Object->class, 'View' ) );
        my $TemplateFolder = $PageType->templateTypeFolder($TemplateTypeAlias);

        if (defined $TemplateTypeName) {
            unless ($TemplateFolder->existsChild($TemplateTypeName)) {
            Warning( 'WRONG_TEMPLATE', {
                    Line => $self->lineNumber,
                    Attribute => $Key,
                    Value => $TemplateTypeName,
                });
                return;
            }
            $PageType->setTemplateType( $Object, $TemplateFolder->alias, $TemplateTypeName);
        } else {
            $PageType->setTemplateType( $Object, $TemplateFolder->alias);
        }
    }
}

#========================================================================================
# function     testMissingFields
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->testMissingFields($hUsedAttributes);
#----------------------------------------------------------------------------------------
# description  test for mandatory columns Class,Parent called by analyzeHeaderLine
#               throws error MISSING_FIELD if they don't exist
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
#========================================================================================
sub testMissingFields {
    my $self = shift;
    my ( $hUsedAttributes ) = @_;
    $self->SUPER::testMissingFields($hUsedAttributes);
    Error( 'MISSING_FIELD' => { 'Value' => 'Class', 'FileName' => $self->{'FileName'}}) unless exists $hUsedAttributes->{'ClassColumn'};
    Error( 'MISSING_FIELD' => { 'Value' => 'Parent', 'FileName' => $self->{'FileName'}}) unless exists $hUsedAttributes->{'ParentColumn'};
    return;
}

#========================================================================================
# function     baseAttributeClassAlias
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->baseAttributeClassAlias();
#----------------------------------------------------------------------------------------
# description  return Base AttributeClass Alias, here : 'ContentObject'
#----------------------------------------------------------------------------------------
# return        $BaseAttributeClassAlias | attribute class alias of normal attributes | string
#========================================================================================
sub baseAttributeClassAlias {
    my $self = shift;
    return 'ContentObject';
}

#========================================================================================
# function     headerColumn
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $hHeaderInfo = $self->headerColumn($HeaderString, $Index);
#----------------------------------------------------------------------------------------
# description  Replaces attribute information for compatibility with import files from
#               previous ePages versions.
#========================================================================================
my %COMPAT_MAP = (
    'Image' => 'ImageMedium',
    'Article/Abstract' => 'Description',
    'Article/Text' => 'Text',
    'Category/Keywords' => 'Keywords',
    'Category/LongText' => 'Text',
    'Category/Text' => 'Description',
    'FreeContent/Text' => 'Text',
);

sub headerColumn {
    my $self = shift;
    my ($HeaderString, $Index) = @_;

    # replace obsolete attribute aliases
    $HeaderString =~ s/\[\Q$_\E\b/\[$COMPAT_MAP{$_}/ foreach keys %COMPAT_MAP;

    return $self->SUPER::headerColumn( $HeaderString, $Index );
}

#========================================================================================
# function     modifyObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->modifyObject($Category, $hOldAttributes, $hAttributeValues);
#----------------------------------------------------------------------------------------
# description  Modifies the object after its creation/modification by the import.
#               Called by importObject.
#----------------------------------------------------------------------------------------
# input        $Object | object which is imported | object
# input        $hOldAttributes | attributes of the object before the import.
#               May be undef if the product is created new and not modified.
#               | ref.hash or undefined
# input        $hDirectAttributes | current Category's direct attribute values | ref.hash
#========================================================================================
sub modifyObject {
    my $self = shift;
    my ($ContentObject, $hOldAttributes, $hDirectAttributes) = @_;

    if($ContentObject) {
        # for all sizes from constants (ImageSmall ImageMedium Image1 Image2)
        foreach my $AttributeName (keys %{CATEGORY_IMAGES_SIZES()}) {
            if($hDirectAttributes->{$AttributeName}) {
                my $hScaleDimension = {
                    'Width'  => CATEGORY_IMAGES_SIZES->{$AttributeName}->[0],
                    'Height' => CATEGORY_IMAGES_SIZES->{$AttributeName}->[1],
                };
                $self->processImportedImageAttribute(
                    $ContentObject,
                    $hOldAttributes,
                    $hDirectAttributes,
                    $AttributeName,
                    1, #Scale
                    $hScaleDimension
                );
            }
        }

        # set short url if empty
        my $Parent = $ContentObject->parent;
        my $Shop = $self->object;
        my $hLanguages = $self->languages;
        if ($Parent->id != $Shop->id) {
            my $Class = $ContentObject->class;
            foreach my $LanguageID (keys %$hLanguages) {
                my $URI = $ContentObject->get('URI', $LanguageID);
                if ((not defined $URI) || $URI eq '') {
                    $URI = GetShortUrlByClass($Parent, $Class, $ContentObject->get('NameOrAlias', $LanguageID), $LanguageID);
                    $ContentObject->set({'URI' => $URI}, $LanguageID) if defined $URI and $URI ne '';
                }
            }

        }
    }

    $self->SUPER::modifyObject($ContentObject, $hOldAttributes, $hDirectAttributes);
    return;
}

#========================================================================================
# function     processMediaGalleryImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hNewAttributes = $self->processMediaGalleryImage(
#                   $hDirectAttributes,
#                   $SourceFileName,
#                   $TargetFileName
#               );
# example      $hDirectAttributes = $self->processMediaGalleryImage(
#                   $hDirectAttributes,
#                   $MGNewImageLargeAbs,
#                   $Category->get('PublicPath') . '/' . $NewImageLargeFileName.$NewImageLargeExtension
#               );
#----------------------------------------------------------------------------------------
# description  Copies the category image from the given source and generates its
#               scaled variations. Updates the given attribute hash with the image names.
#----------------------------------------------------------------------------------------
# input        $Object | Category object which was imported | object
# input        $SourceFileName     | full file path and name of the source image | string
# input        $TargetFileName     | full file path and name for the target image | string
# input        $hDirectAttributes  | updated direct attribute values | ref.hash
# input        $ScaleImages | boolean value if automatic generation of other image sizes
#               should be applied.
# input        $hScaleDimension | Width and Height for the scaled image | ref.hash
#========================================================================================
sub processMediaGalleryImage {
    my $self = shift;
    my ($Object, $hDirectAttributes, $SourceFileName, $TargetFileName, $ScaleImages, $hScaleDimension) = @_;

    # base class handler does copyfile
    $self->SUPER::processMediaGalleryImage($Object, $hDirectAttributes, $SourceFileName, $TargetFileName);

    if($ScaleImages) {
        # Process scale image
        eval {
            ScaleImage(
                $Object->get('PublicPath') . '/' . $TargetFileName,
                $Object->get('PublicPath') . '/' . $TargetFileName,
                max($hScaleDimension->{'Width'},$hScaleDimension->{'Height'})
            );
        };
        if (ExistsError()) {
            # TODO: THROW RESIZE ERROR
            #my $Error = GetError();
        }
    }
}

1;
