#========================================================================================
# package      DE_EPAGES::Content::API::XML::StyleImportHandler
# state        private
#----------------------------------------------------------------------------------------
# description  This module implements a handler to import styles.
#
# example      use DE_EPAGES::Object::API::XML::StyleConverterImportHandler;
#               use XML::SAX::ParserFactory;
#
#               my $Handler = DE_EPAGES::Object::API::XML::StyleConverterImportHandler->new;
#               my $Parser = XML::SAX::ParserFactory->parser( Handler => $Handler );
#               $Parser->parse_file( 'export.xml' );
#========================================================================================
package DE_EPAGES::Content::API::XML::StyleImportHandler;
use base qw ( DE_EPAGES::XML::API::BaseImportHandler Exporter );
use strict;

our @EXPORT_OK = qw (
    ImportStyle
);

use JSON;
use XML::DOM;
use XML::SAX::ParserFactory;
use DE_EPAGES::Core::API::Archive qw( ExtractFilesFromZipArchive );
use DE_EPAGES::Core::API::Error qw( VarsOnError Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw( DeleteDirectory CopyDirectory ExistsDirectory  GetFileNames);
use DE_EPAGES::Core::API::Log qw( LogDebug LogError );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadRootObject LoadObjectByPath LoadClassByAlias Transaction );
use DE_EPAGES::Presentation::API::PageType qw(LoadPageTypeByAlias);
use DE_EPAGES::WebInterface::API::Config qw ( WebInterfaceConfig );
use DE_EPAGES::XML::API::DateTimeFormatXML;

my $TIME_PARSER = DE_EPAGES::XML::API::DateTimeFormatXML->new;

sub warningCodes {
    return [qw(
        WRONG_TEMPLATE
        UNKNOWN_OBJECT
        UNKNOWN_ATTRIBUTE
        FeatureLimitExceeded
        MISSING_FEATURE
        WRONG_TEMPLATETYPE
        MissingAttribute
        READ_ZIP_ERROR
        FileNotFound
        READ_FILE_ERROR
        UNKNOWN_STYLETEMPLATE
        UNKNOWN_IMAGESET
        NativeDie
    )];
}

#========================================================================================
# function     ImportStyle
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImportStyle($FileName, $Shop, $LanguageID, $ImportStartPage, $StyleDirectory, $StyleTemplate);
#----------------------------------------------------------------------------------------
# description  Imports Style, Object, NavBarElements, Startpage design.
#----------------------------------------------------------------------------------------
# input        $FileName | zip file name | string
# input        $Shop | shop object | object
# input        $LanguageID | language identifier | integer
# input        $ImportStartPage | true if data for startpage should be imported | boolean
# input        $StyleDirectory | path to export.xml. not used for import *.style | string
# input        $StyleTemplate | style template (optional) | object
# return       ($aWarnings, $Style) | <ul>
#                   <li>aWarnings | array of warnings | ref.array.*
#                   <li>Style | imported style | object
#               </ul> | ref.array.*
#========================================================================================
sub ImportStyle {
    my ($FileName, $Shop, $LanguageID, $ImportStartPage, $StyleDirectory, $StyleTemplate) = @_;
    my $Handler = __PACKAGE__->new;
    $Handler->{'Folder'} = $StyleDirectory ? $StyleDirectory : $Shop->get('PrivatePath') . '/ImportStyle/';
    $Handler->{'Shop'} = $Shop;
    $Handler->{'LanguageID'} = $LanguageID;
    $Handler->{'ImportStartPage'} = $ImportStartPage;
    $Handler->{'OverwriteStyle'} = $StyleDirectory ? 1 : 0;
    $Handler->{'StyleTemplate'} = $StyleTemplate;

    my $XMLFileName = $Handler->{'Folder'} . 'export.xml';
    my $Style;
    my $aWarnings = BlockWarnings( sub {
        eval {
            # Extract zip.
            $Handler->extractFilesFromZip($FileName) unless $StyleDirectory;

            if (!$StyleDirectory) {
                my $Version = $Handler->version($XMLFileName);
                my $EpagesVersion = LoadRootObject()->get('EpagesVersion');
                if ($Version ne $EpagesVersion) {
                    LogDebug('Not same ePages version.', {Shop => $Shop->alias, 'FileVersion' => $Version, 'SystemVersion' => $EpagesVersion});
                }
            }

            # we made a transaction here and delete the style if the import fails to prevent an inconsistent style
            Transaction(
                sub {
                    # try to import style
                    $Style = $Handler->importXML($XMLFileName);
                },
                sub {
                    # delete the wrong style because we must see and not activate this style!
                    $Style->delete if defined $Style;
                }
            );

            # Copy files.
            $Handler->copyFiles($Style);

            DeleteDirectory($Handler->{'Folder'}, { 'recursive' => 1 }) unless $StyleDirectory;
        };

        if (ExistsError()) {
            my $Error = GetError();
            if( ElementOf($Error->code,  $Handler->warningCodes) ) {
                # log only native dies but show a UI message.
                LogError($Error) if $Error->code eq 'NativeDie';
                Warning($Error);
            } else {
                Error($Error);
            }
        }
    }, $Handler->warningCodes );

    return ($aWarnings, $Style);
}

#========================================================================================
# function     extractFilesFromZip
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->extractFilesFromZip( $FileName );
#----------------------------------------------------------------------------------------
# description  Extracts all files from zip and deletes zip archive.
#----------------------------------------------------------------------------------------
# input        $FileName | XML file name | string
#========================================================================================
sub extractFilesFromZip {
    my $self = shift;
    my ($FileName) = @_;

    my $MaxFileSize = WebInterfaceConfig()->get('GLOBAL', 'MAXCONTENTLENGTH');
    my $hParams= {'AllowOnlyImagesAndPDF' => 0,  # we extract other file types for a style
                  'DeleteAfterExtraction' => 1,
                  'FixPathErrors'         => 0,  # ignore useess path entries
                  'WarningOnFileError'    => 1,
                  'MaxContentLength'      => $MaxFileSize
    };

    ExtractFilesFromZipArchive($FileName, $self->{'Folder'}, $hParams);
}

#========================================================================================
# function     version
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->version( $FileName );
#----------------------------------------------------------------------------------------
# description  Gets the epages version from export.xml.
#----------------------------------------------------------------------------------------
# input        $FileName | XML file name | string
# return       $Version | ePages version from the export.xml file | | string
#========================================================================================
sub version {
    my $self = shift;
    my ($FileName) = @_;
    TestString( 'FileName', $FileName );
    Error( 'FileNotFound', { FileName => $FileName } ) unless -f $FileName;

    my $Parser = XML::DOM::Parser->new;
    my $Version;
    eval {
        my $Document = $Parser->parsefile($FileName);
        my $Node = $Document->getElementsByTagName('epages')->item(0);
        $Version = $Node->getAttributeNode('Version')->getValue();
        $Document->dispose;
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error('READ_FILE_ERROR', { FileName => $FileName, Code => $Error->code, Message => $Error->message });
    }
    return $Version;
}

#========================================================================================
# function     importXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->importXML( $FileName );
#----------------------------------------------------------------------------------------
# description  Imports classes and attributes from an XML file.
#----------------------------------------------------------------------------------------
# input        $FileName | XML file name | string
# return       $Style | new style object | object
#========================================================================================
sub importXML {
    my $self = shift;
    my ($FileName) = @_;
    TestString( 'FileName', $FileName );
    Error( 'FileNotFound', { FileName => $FileName } ) unless -f $FileName;

    my $Parser = XML::SAX::ParserFactory->parser( Handler => $self );
    eval { $Parser->parse_uri( $FileName ); };
    VarsOnError({'FileName'=>$FileName});

    return $self->{'Style'};
}

#========================================================================================
# Style
#========================================================================================
sub start_Style {
    my $self = shift;
    my ($hElement) = @_;
    my $Shop = $self->{'Shop'};
    my $Class = LoadClassByAlias('Style');

    my $hAttributes = $self->getAttributes( $hElement );
    LogDebug('Attributes Style', $hAttributes);

    my ($hParsedAttributes, $hLocalizedStringAttributes);

    my $aWarnings = BlockWarnings( sub {
        ($hParsedAttributes, $hLocalizedStringAttributes) = $self->_checkAttributes($Class, $hAttributes);
    }, [qw(UNKNOWN_OBJECT)] );

    foreach my $Warning (@$aWarnings) {
        my $hWarningVars = $Warning->vars;
        if ($hWarningVars->{'Attribute'} eq 'StyleTemplate') {
            Warning('UNKNOWN_STYLETEMPLATE', {%$hWarningVars});
        } else {
            Warning($Warning);
        }
    }

    $hParsedAttributes->{'StyleTemplate'} = $self->{'StyleTemplate'} if $self->{'StyleTemplate'};

    # The imported style references a styletemplate that does not exists in the system.
    # Hence use the style's styleextension.css from the imported data instead.
    my $Folder = $self->{'Folder'};
    if(ExistsDirectory($Folder . 'Style')) {
        my @Files = @{GetFileNames($Folder . 'Style', { 'relative' => 1 }) };
        $hParsedAttributes->{'IsImportedWithoutTemplate'} = 1
            if ElementOf('StyleExtension.css', @Files);
    }

    unless (ExistsDirectory(LoadRootObject()->get('PublicStoreTypePath') . '/' . $hParsedAttributes->{'LayoutImageSet'})) {
        Warning('UNKNOWN_IMAGESET', {
            Attribute => 'LayoutImageSet',
            Value => $hParsedAttributes->{'LayoutImageSet'},
        });
        $hParsedAttributes->{'LayoutImageSet'} = 'SF/ImageSet/Transparent';
    }

    my $Styles = $Shop->folder('Styles');
    $self->{'OverwriteStyle'} = 0 if $self->{'OverwriteStyle'} && !$Styles->existsChild($hParsedAttributes->{'Alias'});

    my $Style;
    if ($self->{'OverwriteStyle'}) {
        $Style = $Styles->child($hParsedAttributes->{'Alias'});
        $Style->set($hParsedAttributes);
    } else {
        $hParsedAttributes->{'Alias'} = $self->_checkAlias($Styles, $hParsedAttributes->{'Alias'});
        $Style = $Styles->insertChild($Class, $hParsedAttributes);
    }
    $Style->set($hLocalizedStringAttributes, $self->{'LanguageID'});

    $self->{'Style'} = $Style; # return value of importXML()
    $self->pushID( 'Style', $Style);
    $self->pushID( 'Object', $Style);
    return;
}

sub end_Style {
    my $self = shift;

    $self->popID( 'Object' );
    $self->popID( 'Style' );
}

#========================================================================================
# NavBarElement
#========================================================================================
sub start_NavBarElement {
    my $self = shift;
    my ($hElement) = @_;
    my $hAttributes = $self->getAttributes( $hElement );
    my $Style = $self->topID('Style');
    LogDebug('Attributes NavBarElement', $hAttributes);

    unless (ExistsObjectByPath($hAttributes->{'NavElement'})) {
        Warning('UNKNOWN_OBJECT', {
            Attribute => 'NavElement',
            Value => $hAttributes->{'NavElement'},
        });

        # make sure the 'Object' stack is balanced, but don't push undef.
        $self->pushID( 'Object', 'NO_OBJECT');

        return;
    }

    my $Class = LoadObjectByPath($hAttributes->{'NavElement'});
    my ($hParsedAttributes, $hLocalizedStringAttributes) = $self->_checkAttributes($Class, $hAttributes);
    if (defined $hParsedAttributes) { # at least Alias must be set
        my $Folder = $Style->folder('NavBarElements');
        $hParsedAttributes->{'Alias'} = $self->_checkAlias($Folder, $hParsedAttributes->{'Alias'});
        my $NavBarElement = $Folder->insertChild($Class, $hParsedAttributes);
        $NavBarElement->set($hLocalizedStringAttributes, $self->{'LanguageID'});

        $self->pushID( 'Object', $NavBarElement);
    }
    else {
        # make sure the 'Object' stack is balanced, but don't push undef.
        $self->pushID( 'Object', 'NO_OBJECT');
    }

    return;
}

sub end_NavBarElement {
    my $self = shift;

    $self->popID( 'Object' );
}

#========================================================================================
# StartPage
#========================================================================================
sub start_StartPage {
    my $self = shift;
    my ($hElement) = @_;
    my $Shop = $self->{'Shop'};

    unless ($self->{'ImportStartPage'}) {
        my $StartPage = $Shop->get('Categories');
        $self->pushID( 'Object', $StartPage);
        return;
    }

    my $Class = LoadClassByAlias('StartPage');

    my $hAttributes = $self->getAttributes( $hElement );
    LogDebug('Attributes StartPage', $hAttributes);

    my ($hParsedAttributes, $hLocalizedStringAttributes) = $self->_checkAttributes($Class, $hAttributes);
    my $StartPage = $Shop->get('Categories');
    $StartPage->set($hParsedAttributes);
    $StartPage->set($hLocalizedStringAttributes, $self->{'LanguageID'});

    $self->pushID( 'Object', $StartPage);

    return;
}

sub end_StartPage {
    my $self = shift;

    $self->popID( 'Object' );
}

#========================================================================================
# Template
#========================================================================================
sub start_Template {
    my $self = shift;
    my ($hElement) = @_;
    return unless $self->{'ImportStartPage'};
    my $Shop = $self->{'Shop'};

    my $hAttributes = $self->getAttributes( $hElement );
    my $StartPage = $Shop->get('Categories');

    my $PageType = LoadPageTypeByAlias('SF-StartPage');
    my $TemplateName = $hAttributes->{'Alias'};

    unless ($PageType->child('TemplateTypes')->existsChild($TemplateName)) {
        Warning('WRONG_TEMPLATETYPE', {
            Attribute => 'Alias',
            Value => $TemplateName,
        });
        return;
    }

    my $TemplateFolder = $PageType->templateTypeFolder($TemplateName);

    my $TemplateTypeName = $hAttributes->{'ActiveTemplateType'};

    unless ($TemplateFolder->existsChild($TemplateTypeName)) {
        Warning( 'WRONG_TEMPLATE', {
            Attribute => $TemplateName,
            Value => $TemplateTypeName,
        });
        return;
    }
    $PageType->setTemplateType( $StartPage, $TemplateFolder->alias, $TemplateTypeName );

    return;
}

#========================================================================================
# AttributeValue
#========================================================================================
sub start_AttributeValue {
    my $self = shift;
    my ($hElement) = @_;
    my $Object = $self->topID('Object');

    # test if we have really an object on the stack
    return if $Object eq 'NO_OBJECT';

    return if $Object->instanceOf('StartPage') && !$self->{'ImportStartPage'};

    my $hAttributes = $self->getAttributes( $hElement );
    LogDebug('Attributes Object ' . $Object->alias, $hAttributes);

    # Get the class from the Class attribute.
    Error('MissingAttribute', {
        'Attribute' => 'Name',
        'Element' => 'AttributeValue',
        'Object' => $Object->get('NameOrAlias', $self->{'LanguageID'}),
    }) unless exists $hAttributes->{'Name'};

    if( exists $hAttributes->{'Value'} ) {
        my ($hParsedAttributes, $hLocalizedStringAttributes) = $self->_checkAttributes($Object->class, {$hAttributes->{'Name'} => $hAttributes->{'Value'}},1);
        $Object->set($hParsedAttributes) if defined $hParsedAttributes;
        my $LanguageID = exists $hAttributes->{'Language'} ? $hAttributes->{'Language'} : $self->{'LanguageID'};
        $Object->set($hLocalizedStringAttributes, $LanguageID) if defined $hLocalizedStringAttributes;
    }

    return;
}

#========================================================================================
# ignore these tags
#========================================================================================
sub start_Object { return; }

#========================================================================================
# function     _checkAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       ($hValues, $hLocalizedValues) = $self->_checkAttributes($Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Converts paths to objects, #MediaGallery to path of mediagallery,
#               #ShoPath to path of shop.
#----------------------------------------------------------------------------------------
# input        $Class | class object | object
# input        $hAttributes | attribute values | ref.hash
# input        $DisableFeatureCheck | disable feature check e.g. we write attribute values | boolean
# return       $hValues | non-localized attribute values (name => value) | ref.hash
# return       $hLocalizedValues | localized attribute values (name => value) | ref.hash
#========================================================================================
sub _checkAttributes {
    my $self = shift;
    my ($Class, $hAttributes, $DisableFeatureCheck) = @_;
    my $Shop = $self->{'Shop'};

    my $Feature = $Class->class->existsAttribute('Feature') ? $Class->get('Feature') : undef;
    $Feature    = undef if $DisableFeatureCheck;
    return unless $self->_checkFeature($Shop, $Feature, $Class);

    my %Attributes;
    my %LocalizedStringAttributes;

    foreach my $Key (keys %$hAttributes) {
        next if $Key eq 'optional';
        unless ($Class->existsAttribute($Key)) {
            Warning('UNKNOWN_ATTRIBUTE', {
                'Attribute' => $Key,
                'Class' => $Class->get('NameOrAlias', $self->{'LanguageID'}),
            });
            next;
        }

        my $Value = $hAttributes->{$Key};
        next unless defined $Value;
        my $Attribute = $Class->attribute($Key);
        $Value = _parseAttribute($Value, $Shop, $Attribute, $self->{'LanguageID'});
        my $Type = $Attribute->get('Type');
        if ($Type eq 'LocalizedString') {
            $LocalizedStringAttributes{$Key} = $Value;
        } elsif ($Type eq 'JSON') {
            $Attributes{$Key} = from_json($Value);
        } else {
            $Attributes{$Key} = $Value;
            $Attributes{$Key} = 'Layout1' if ($Key eq 'LayoutTemplate' && $Value eq '');
        }
    }
    LogDebug('Attributes for ' . $Class->alias, \%Attributes);
    LogDebug('LocalizedString Attributes for ' . $Class->alias, \%LocalizedStringAttributes);

    return (\%Attributes, \%LocalizedStringAttributes);
}

sub _checkFeature {
    my $self = shift;
    my ($Shop, $Feature, $Class) = @_;
    return 1 unless defined $Feature;
    my $MaxValue = $Shop->featureMaxValue($Feature);
    my $LanguageID = $self->{'LanguageID'};
    if ($MaxValue == 0) {
        Warning('MISSING_FEATURE', {
            'Class' => $Class->get('NameOrAlias', $LanguageID),
            'Feature' => $Feature->get('NameOrAlias', $LanguageID),
            'MaxValue' => $MaxValue
        });
        return 0;
    } elsif($Feature->alias eq 'UserDefinedNavBarElements' ) {
        my $Style = $self->{'Style'};
        if ($Style->get('UserDefinedNavBarElementCount') >= $MaxValue) {
            Warning('FeatureLimitExceeded', {
                'Class' => $Class->get('NameOrAlias', $LanguageID),
                'Feature' => $Feature->get('NameOrAlias', $LanguageID),
                'MaxValue' => $MaxValue
            });
            return 0;
        }
    } elsif(($MaxValue == 1 && $Shop->featureCurrentValue($Feature) > $MaxValue) || ($MaxValue != 1 && $Shop->featureCurrentValue($Feature) >= $MaxValue)) {
        Warning('FeatureLimitExceeded', {
            'Class' => $Class->get('NameOrAlias', $LanguageID),
            'Feature' => $Feature->get('NameOrAlias', $LanguageID),
            'MaxValue' => $MaxValue
        });
        return 0;
    }
    return 1;
}

sub _parseAttribute {
    my ($Value, $Shop, $Attribute, $LanguageID) = @_;
    # Set correct path to MediaGallery and Shop.
    $Value =~ s/\#MediaGallery/$Shop->get('MediaGallery')->get('WebPath')/eg;
    $Value =~ s/\#ShopPath/$Shop->get('Path')/eg;

    if ($Attribute->get('IsObject') && !$Attribute->get('IsArray')) {
        if (ExistsObjectByPath($Value)) {
            $Value = LoadObjectByPath($Value);
        } else {
            Warning( 'UNKNOWN_OBJECT', {
                'Value' => $Value,
                'Attribute' => $Attribute->get('NameOrAlias', $LanguageID),
            });
            $Value = undef;
        }
    } elsif( ElementOf($Attribute->get('Type'), 'DateTime', 'Date', 'Time', 'Duration') ) {
        $Value = $TIME_PARSER->parse_datetime($Value);
    }

    return $Value;
}

sub _checkAlias {
    my $self = shift;
    my ($Object, $Alias) = @_;
    return $Alias unless $Object->existsChild($Alias);
    my $cnt = 1;
    # Test if Alias has already an ext like [1] and replace ext if exists.
    if ($Alias =~ /.*\[\d{1,}\]$/gx){
        while ($Object->existsChild($Alias)){
            $cnt++;
            $Alias =~ s/(\[)\d{1,}(\])$/$1$cnt$2/gx;
        }
    }else{
        while ($Object->existsChild($Alias."[$cnt]")){
            $cnt++;
        }
        $Alias .= "[$cnt]";
    }
    return $Alias;
}

sub copyFiles {
    my $self = shift;
    my ($Style) = @_;
    my $Folder = $self->{'Folder'};
    my $PublicFolder = $self->{'Shop'}->get('PublicPath');

    CopyDirectory($Folder . 'Categories', $PublicFolder . '/Categories', undef, { recursive => 1 } ) if ExistsDirectory($Folder . 'Categories');
    CopyDirectory($Folder . 'MediaGallery', $PublicFolder . '/MediaGallery', undef, { recursive => 1 } ) if ExistsDirectory($Folder . 'MediaGallery');
    CopyDirectory($Folder . 'Style', $Style->get('PublicPath'), undef, { recursive => 1 } ) if ExistsDirectory($Folder . 'Style');
}

1;
