#========================================================================================
# package      DE_EPAGES::Content::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to show, save and delete objects.
#========================================================================================
package DE_EPAGES::Content::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use strict;
use DE_EPAGES::Content::API::Content qw ( ContentChildClasses );
use DE_EPAGES::Content::API::ShortUrl qw ( GetShortUrlByClass );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw (TestInteger);
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Alias );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Presentation::API::Template qw ( Translate );
use DE_EPAGES::Search::API::Object::PagerSearch qw (ReplaceWildcards);
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::Search::API::Table::SearchSort qw ( ExistsSearchSortBySearchDefinitionID );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use POSIX;

#========================================================================================
# function     JSONContentClasses
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONContentClasses( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns the avaiable childclasses for a specified class and shop
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONContentClasses {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $ClassID = $Servlet->form->value('ClassID');
    TestInteger('ClassId',$ClassID);
    my @Stack = ($Shop->loadObject($ClassID, 'Class', {'AllowSystemSite' => 1}));

    my %Classes;
    while (my $Class = pop @Stack) {
        my $hInfo = $Class->get(['Alias', 'NameOrAlias','ObjectID','Position', 'Description']);

        $Classes{$Class->id} = $hInfo;
        my $Classes = ContentChildClasses($Shop, $Class);
        push @Stack, $_ foreach grep { not exists $Classes{$_->id}} @$Classes;
        $hInfo->{'ContentChildClasses'} = [ map { $_->id } @$Classes ];
        my $Package = $Class->packageObject();
        $hInfo->{'EffectedVisibilityAttributes'} = $Package->effectedVisibilityAttributes;

        my $Feature = $Package->featureName();
        if($Feature){
            $hInfo->{'FeatureMaxValue'} = $Shop->featureMaxValue($Feature);
            $hInfo->{'FeatureCurrentValue'} = $Shop->featureCurrentValue($Feature);
        }

    }
    $self->writeContentOutput([values %Classes]);
    return;
}

#========================================================================================
# function     JSONGenerateShortURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONGenerateShortURL( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns a suggested URI for a given name of a new content object.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONGenerateShortURL {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $Parent = $Servlet->object()->getSite()->loadObject($Form->value('ParentID'), 'Object');
    my $Class = LoadClassByAlias($Form->value('ClassName'));
    my $LanguageID = $Form->value('LanguageID');

    my @TemporaryReserved;
    @TemporaryReserved = map {{'Uri' => $_}} split (/,/, $Form->value('TemporaryReserved')) if defined $Form->value('TemporaryReserved');

    $self->writeContentOutput({ 'ShortURL' => GetShortUrlByClass($Parent, $Class, $Form->value('Name'), $LanguageID, \@TemporaryReserved)});
    return;
}

#========================================================================================
# function     JSONValidateShortURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONValidateShortURL( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns true if shorturl is valid
#               Returns false, a suggested URI and the reason if shorturl is not valid.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONValidateShortURL {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $Shop = $Servlet->object()->getSite();
    my $Parent = $Shop->loadObject($Form->value('ParentID'), 'Object');

    my $LanguageID = $Form->value('LanguageID');
    my $ShortUrl = $Form->value('ShortUrl');
    my @TemporaryReserved = defined $Form->value('TemporaryReserved')
        ? map {{'Uri' => $_}} split (/,/, $Form->value('TemporaryReserved'))
        : ();

    my %hInfo;
    if( $#TemporaryReserved > 0 and ElementOf(lc($ShortUrl), map {lc($_->{'Uri'})} @TemporaryReserved) ){
        $hInfo{'Valid'} = 0;
        $hInfo{'Reason'} = 'SHORTURL_ALREADY_EXISTS';
        my $Class = LoadClassByAlias($Form->value('ClassName'));
        $hInfo{'ShortURL'} = GetShortUrlByClass($Parent, $Class, $ShortUrl, $LanguageID, \@TemporaryReserved);
    } else {
        $hInfo{'Valid'} = 1;
    }

    $self->writeContentOutput(\%hInfo);
    return;
}

sub JSONGetUniqueAlias {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->writeContentOutput({
        'Alias' => $Servlet->object->uniqueChildAlias(Unicode2Alias($Servlet->params('Name')))
    });
}

sub JSONSearchCategoriesToInsert {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hParam = $Servlet->form->form($Shop, 'JSONSearchCategoriesToInsert');

    my $hHookParams = {
        'BlackListedClassIDs' => [],
        'Shop' => $Shop,
    };
    TriggerHook('UI_SearchCategoriesToInsertBlackListedClassIDs', $hHookParams);

    my $StartPage = $Shop->get('Categories');
    my $hClassIDs;
    foreach my $SubCat (@{$StartPage->get('SubCategories')}) {
        $hClassIDs->{$SubCat->class->id} = undef;
    }
    $hClassIDs->{$StartPage->class->id} = undef;
    my @ClassIDs = keys %$hClassIDs;
    my $SearchName = 'CategorySearch';
    my $ContentClass = LoadClassByAlias('ContentObject');
    my $CategoryClass = LoadClassByAlias('Category');
    my $SearchDefinition = GetSearchDefinition($ContentClass, $SearchName);

    my %Options = (
        'Servlet' => $Servlet,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'ClassIDs'   => \@ClassIDs,
            'LanguageID' => $Servlet->languageID,
            'IsVisible'  => 1,
            'Search' => $SearchName,
            'AttributeIDName' => $CategoryClass->attribute('Name')->id,
        },
        'UriParameters'  => {
            'Search' => $SearchName,
        },
    );

    # Save entry number of a page.
    my $EntriesPerPage = $hParam->{pageEntries} // 10;  ## no critic qw(ProhibitMagicNumbers)
    # Calculate how many entries (page size * pages) should be loaded.
    my $Pages = $hParam->{pages} // 1;
    $Options{'PageSize'} = $EntriesPerPage * $Pages;

    # Set sorting.
    my $OrderBy = $hParam->{orderBy};
    $Options{'OrderBy'} = 'NameOrAlias';
    if(defined $OrderBy && ExistsSearchSortBySearchDefinitionID($SearchDefinition->id, $OrderBy)) {
        $Options{'OrderBy'} = $OrderBy;
    }
    my $OrderDesc = $hParam->{'orderDesc'};
    $Options{'OrderDesc'} = $OrderDesc ? 1 : 0;

    if ( defined $hParam->{WildSearchString} ) {
        # EPG-9002: limit the search string
        Readonly::Scalar my $MaxSearchStringLength => 80;
        my $WCSearchString = substr($hParam->{WildSearchString}, 0, $MaxSearchStringLength);

        # replace * with %
        $Options{'Parameters'}{'SearchString'} = ReplaceWildcards( $WCSearchString );
    }

    my $ParentObjectID = $hParam->{ParentObjectID};
    my $ParentObject;

    if (defined $ParentObjectID) {
        $ParentObject = $Shop->loadObject($ParentObjectID, 'Category');
    }
    else {
        $ParentObject = $StartPage;
    }

    if ($ParentObject->id == $Shop->id) {
        $Options{'Parameters'}{'ClassIDs'} = [$StartPage->class->id];
    }
    else {
        if(not defined $Options{'Parameters'}{'SearchString'}) {
            $Options{'Parameters'}{'Parent.ID'} = $ParentObject->id;
        }
    }
    my $aParentPath = $ParentObject->get('PathFromSite');

    my @PathElements;
    foreach my $PathElement (@{$aParentPath}) {
        push @PathElements, {
            'ObjectID' => int($PathElement->id),
            'NameOrAlias' => $PathElement->get('NameOrAlias')
        };
    }

    $Options{'Parameters'}{'BlackListedClassIDs'} = $hHookParams->{'BlackListedClassIDs'} if scalar @{$hHookParams->{'BlackListedClassIDs'}} > 0;
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    my $Page = $Search->page;

    my $hPager = $Page->{'Pager'};
    my $hJSONResults = {};
    my $Counter = 1;

    # put Homepage only on top level
    my $IsHighlighted;

    if ((not defined $Options{'Parameters'}{'SearchString'}) and (scalar @{$aParentPath} == 1) and
         not ElementOf($StartPage->class->id, $Options{'Parameters'}{'BlackListedClassIDs'}) ) {
        push @{$hJSONResults->{1}}, {
            'ObjectID' => int($StartPage->id),
            'NameOrAlias' => $StartPage->get('NameOrAlias'),
            'isHighlighted' => 1,
        };

        $Counter++;
        $hPager->{'CountOfItems'}++;
        $IsHighlighted = 1;
    }

    my $LastPage = ceil($hPager->{'CountOfItems'} / $EntriesPerPage);

    if($hPager->{'CountOfItems'} > 0) {
        my $CurrentPage = $hParam->{currentPage} // 1;

        if ($CurrentPage > $LastPage || $CurrentPage < 0) {
            $CurrentPage = 1;
        }
        my $PagerSearchPage = ceil($CurrentPage / $Pages);

        my $aFoundObjects = $Search->items($PagerSearchPage);

        my $Pager = ($PagerSearchPage - 1) * $Pages + 1;

        my @ToSort;
        my $hSortPaths;
        foreach my $FoundObject ( @$aFoundObjects ) {
            my $hObjectAttributes = $FoundObject->get([ 'ObjectID', 'NameOrAlias', 'CategoryPages', 'PathFromSite' ]);

            my @Paths;
            my $PathLength = scalar @{$hObjectAttributes->{'PathFromSite'}};
            if ($PathLength > 1 && defined $hParam->{WildSearchString}) {
                foreach my $i (0 .. $PathLength - 1) {
                    push @Paths, {
                        'ObjectID' => int($hObjectAttributes->{'PathFromSite'}->[$i]->id),
                        'NameOrAlias' => $hObjectAttributes->{'PathFromSite'}->[$i]->get('NameOrAlias'),
                    };
                    $hSortPaths->{$hObjectAttributes->{'ObjectID'}} .= $hObjectAttributes->{'PathFromSite'}->[$i]->get('NameOrAlias');
                }
            } else {
                $hSortPaths->{$hObjectAttributes->{'ObjectID'}} .= $hObjectAttributes->{'NameOrAlias'};
            }

            my $hRowData = {
                'ObjectID' => int($hObjectAttributes->{'ObjectID'}),
                'NameOrAlias' => $hObjectAttributes->{'NameOrAlias'},
                'Parents' => \@Paths,
                'SortString' => $hSortPaths->{$hObjectAttributes->{'ObjectID'}},
            };

            if (defined $hObjectAttributes->{'CategoryPages'} and scalar @{$hObjectAttributes->{'CategoryPages'}}) {
                my @CategoryPages = grep {$_->instanceOf('Category')} @{$hObjectAttributes->{'CategoryPages'}};
                $hRowData->{'hasSubTable'} = scalar @CategoryPages ? 1 : 0;
            }
            push @ToSort, $hRowData;
        }

        # sort row data by SortString
        #$Options{'OrderDesc'}
        my @Sorted = sort { lc($a->{'SortString'}) cmp lc($b->{'SortString'}) } @ToSort;
        @Sorted = reverse @Sorted if $Options{'OrderDesc'};

        foreach my $Element (@Sorted) {
            push @{$hJSONResults->{$Pager}}, $Element;

            if ($Counter % $EntriesPerPage == 0) {
                $Pager++;
            }
            $Counter++;
        }

    }

    my $hResponse = {
        'totalNumber' => $hPager->{'CountOfItems'},
        'lastPage' => $LastPage,
        'tableContent' => $hJSONResults,
        'tableHead' => _BuildTableHead(),
        'orderBy' => $Options{'OrderBy'},
        'orderDesc' => $Options{'OrderDesc'},
        'path' => \@PathElements,
        'isHighlighted' => $IsHighlighted,
    };

    # write result data on the current servlet.
    $self->writeContentOutput($hResponse);
}

sub _BuildTableHead() {
     return [
            {
                'id' => 'ObjectID',
                'name' => '',
                'isSelectable' => 1
            },
            {
                'id' => 'NameOrAlias',
                'name' => Translate('Name', undef, 'DE_EPAGES::Dictionary'),
                'isLinkable' => 1,
                'isSortable' => 1
            }
        ]
}
1;
