#========================================================================================
# package      DE_EPAGES::Core::API::PPI::Transform::CriticTransform
# state        public
#----------------------------------------------------------------------------------------
# description  PPI Transformation class for refactoring perlcritic violations out of
#               our perl code.
#========================================================================================
package DE_EPAGES::Core::API::PPI::Transform::CriticTransform;

use base qw (PPI::Transform Exporter);

use strict;
use PPI::Find;

use DE_EPAGES::Core::API::PPI::Tools qw (
    ReverseByLocation
    LocationIsMatch
);


#========================================================================================
# function     violations
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Transform->violations( @violations );
#----------------------------------------------------------------------------------------
# description  Set perlcritic violations as refactoring input
#----------------------------------------------------------------------------------------
# input        @violations | perlcritic violations | array.object
#========================================================================================
sub violations {
    my $self = shift;
    my ($aViolations, $Location) = @_;
    return $self->{'violations'}//[] unless ($aViolations && scalar @{$aViolations});
    if (defined $Location && $Location =~ /(\d+)\s*,(\d+)\s*,(\d+)/x) {
        $aViolations = [grep { $self->applies_to_location($_, $1, $2, $3) } @{$aViolations}];
    }
    $aViolations = [grep { $self->applies_to_violation($_) } @{$aViolations}];
    my @sortedviolations = sort ReverseByLocation @{$aViolations};
    $self->{'violations'} = \@sortedviolations;
}

#========================================================================================
# function     name
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Transform->name();
#----------------------------------------------------------------------------------------
# description  Returns refactoring rule name. Has to be implemented by child classes
#----------------------------------------------------------------------------------------
# return       $Name | rule name | string
#========================================================================================
sub name {
    'NOT_IMPLEMENTED';
}

#========================================================================================
# function     description
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Transform->description();
#----------------------------------------------------------------------------------------
# description  Returns refactoring rule description. Must be implemented by child classes
#----------------------------------------------------------------------------------------
# return       $Description | rule description | string
#========================================================================================
sub description {
    'NOT_IMPLEMENTED';
}

#========================================================================================
# function     applies_to
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Transform->applies_to();
#----------------------------------------------------------------------------------------
# description  Returns a string, an array or a regular expression for perlcritic
#               policies this rule can be applied to. Must be implemented in child classes.
#----------------------------------------------------------------------------------------
# return       $AppliesTo | A pattern of perlcritic policies this rule applies to | string or regex
#========================================================================================
sub applies_to {
    return qr/^$/;
}

#========================================================================================
# function     applies_to_violation
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Transform->applies_to_violation( $violation );
#----------------------------------------------------------------------------------------
# description  Checks by regular expression or substring match if refactoring rule can be
#               applied to this perlcritic violation. Uses perlcritic policy shortname and
#               applies_to() as input.
#               This is used to filter out the wanted perlcritic violations.
#----------------------------------------------------------------------------------------
# return       $DoesApply | True if rule can be applied | boolean
#========================================================================================
sub applies_to_violation {
    my $self = shift;
    my ($violation) = @_;
    my $applies_to = $self->applies_to;
    my $short_name = $self->policy_short_name($violation);

    if (! ref $applies_to) {
        return $short_name =~ m/$applies_to/;
    }

    return $short_name =~ $applies_to;
}

#========================================================================================
# function     applies_to_location
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Transform->applies_to_location( $violation, $Line, $PhysicalColumn, $VisualColumn);
#----------------------------------------------------------------------------------------
# description  Checks by regular expression or substring match if refactoring rule can be
#               applied to this perlcritic violation. Uses perlcritic policy shortname and
#               applies_to() as input.
#               This is used to filter out the wanted perlcritic violations.
#----------------------------------------------------------------------------------------
# return       $DoesApply | True if rule can be applied | boolean
#========================================================================================
sub applies_to_location {
    my $self = shift;
    my ($violation, $Line, $PhysicalColumn, $VisualColumn) = @_;
    my $Loc = $violation->location();
    return ($Loc->[0] eq $Line &&
            $Loc->[1] eq $PhysicalColumn &&
            $Loc->[2] eq $VisualColumn);
}

#========================================================================================
# function     policy_short_name
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Transform->policy_short_name( $violation );
#----------------------------------------------------------------------------------------
# description  returns the perlcritic policy short name (e.g. ProhibitBooleanGrep)
#----------------------------------------------------------------------------------------
# return       $PolicyName | policy short name | string
#========================================================================================
sub policy_short_name {
    my $self = shift;
    my ($violation) = @_;
    my $Policy = $violation->policy;
    my $PolicyName = $Policy->policy_short_name;
    $PolicyName = pop @{[split(/::/ , $PolicyName)]} if ($PolicyName =~ /::/);
    return $PolicyName;
}

#========================================================================================
# function     find_matching_element
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Transform->find_matching_element( $violation, $doc );
#----------------------------------------------------------------------------------------
# description  Returns the matching element in the PPI parse tree that the perlcritic
#               violation reported.
#----------------------------------------------------------------------------------------
# return       $PPIElement | Matching PPI element or undef | object
#========================================================================================
sub find_matching_element {
    my $self = shift;
    my ($violation, $doc) = @_;
    my $Find = PPI::Find->new( sub {
        my $ElemLoc = $violation->location();
        my $DocLoc  = $_[0]->location();
        return 1 if (LocationIsMatch($ElemLoc, $DocLoc) && $_[0]->class() eq $violation->element_class());
        return 0;
    });

    $Find->start($doc);
    return $Find->match();
}

1;
