#========================================================================================
# package      DE_EPAGES::Core::API::Program::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  Provides auxiliary functions for DE_EPAGES::Core::API::Program methods.
#========================================================================================
package DE_EPAGES::Core::API::Program::Tools;
use base qw( Exporter );

use strict;
use File::Basename qw(basename);
use DE_EPAGES::Core::API::Error qw( CallerError );
use DE_EPAGES::Core::API::File qw( WindowsPath RunOnDirectory GetFileContent WriteFile );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf UsePackage );
use DE_EPAGES::Core::API::SystemCommand qw(SystemCommand);

BEGIN {
    UsePackage( 'Win32::TieRegistry' ) if $^O eq 'MSWin32';
}

our @EXPORT_OK = qw(
    FindProgramFile
    FindExeInPath
    ProgramFileDir
    Has64BitOS
    GetEnvFromRegistry
    AddJavaOpt
);

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'OSnotSupported' => 'Can\'t run #Function on #OS',
        'TooManyWildcards'=> 'Path glob has more than one wildcard \'*\': #PathGlob',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     FindProgramFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ExePath = FindProgramFile($PathGlob);
#----------------------------------------------------------------------------------------
# example      $ExePath = FindProgramFile('Java/jdk1.7*/bin/java.exe');
#----------------------------------------------------------------------------------------
# description  Returns the absolute path (latest version) of $PathGlob in $ENV{PROGRAMFILES}
#               For 64-bit windows returns the 64-bit exe if exists, else the 32-bit exe.
#               $PathGlob must not have more than one '*'.
#----------------------------------------------------------------------------------------
# input        $PathGlob | executable with glob pattern, relative to PROGRAMFILES | string
# return       $ExePath  | absolute windows path (latest version) of the executable | string
#========================================================================================
sub FindProgramFile {
    my ($PathGlob) = @_;

    CallerError('OSnotSupported', { Function => 'FindProgramFile', OS => $^O }) if $^O ne 'MSWin32';
    CallerError('TooManyWildcards', { PathGlob => $PathGlob }) if $PathGlob =~ /\*.*\*/;

    # split $PathGlob in pre-glob, glob and post-glob
    my ($Dir, $Glob, $File) = ('', '', '');
    if ($PathGlob =~ /\*/) {
        foreach (split /[\\\/]/, $PathGlob) {
            if (/\*/) { $Glob = $_; }
            elsif ($Glob) { $File .= "/$_"; }
            else { $Dir .= "/$_"; }
        }
    }

    foreach my $Place (ProgramFileDir(64), ProgramFileDir(32)) { ## no critic qw(ProhibitMagicNumbers)
        next unless $Place;

        my $DirPlace = "$Place$Dir";
        next unless -d $DirPlace;

        if (not $Glob) {
            return WindowsPath("$DirPlace/$PathGlob") if -e "$DirPlace/$PathGlob";
        } else {
            my $ExePath = RunOnDirectory( $DirPlace, sub {
                foreach my $VersionDir (sort { $b cmp $a } glob($Glob)) {
                    return "$DirPlace/$VersionDir$File" if -e "$DirPlace/$VersionDir$File";
                }
                                          });
            return WindowsPath($ExePath) if $ExePath;
        }
    }
    return;
}

#========================================================================================
# function     ProgramFileDir
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Directory = ProgramFileDir($Bits);
#----------------------------------------------------------------------------------------
# example      $Directory = ProgramFileDir();
# example      $Directory = ProgramFileDir(32);
#----------------------------------------------------------------------------------------
# description  Returns the program files directory derived from $ENV{PROGRAMFILES}
#               If $Bits is set, returns the 32 or 64 bit variant depending on $Bits.
#               If $Bits is unset, returns the 64 bit variant (if exist), else 32 bit.
#               Returns undef if the 64-bit directory is requested on a 32-bit system.
#----------------------------------------------------------------------------------------
# input        $Bits | bit size (undef|32|64) | integer
# return       $Directory | the program files directory | string
#========================================================================================
sub ProgramFileDir {
    my ($Bits) = @_;

    CallerError('OSnotSupported', { Function => 'ProgramFileDir', OS => $^O }) if $^O ne 'MSWin32';

    ## no critic qw(ProhibitMagicNumbers)
    return if $Bits == 64 and not Has64BitOS();

    my @ProgramFilesDirs;
    push @ProgramFilesDirs, $ENV{PROGRAMW6432} if $Bits == 64 or not $Bits;
    push @ProgramFilesDirs, $ENV{PROGRAMFILES} if $Bits == 32 or not $Bits;
    ##use critic

    foreach (@ProgramFilesDirs) {
        next unless /\S/;
        next unless -d $_;
        return $_;
    }
    return;
}

#========================================================================================
# function     FindExeInPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ExePath = FindExeInPath($File);
#----------------------------------------------------------------------------------------
# example      $ExePath = FindExeInPath('dot.exe');
#----------------------------------------------------------------------------------------
# description  Returns the absolute path (first occurence) of $File in $ENV{PATH}.
#               Returns undef if the file is not found.
#----------------------------------------------------------------------------------------
# input        $File | file name | string
# return       $Status | absolute path of the executable | string
#========================================================================================
sub FindExeInPath {
    my ($File) = @_;

    my $BaseName = basename($File);

    if ($^O eq 'MSWin32') {
        foreach (split(/;/, $ENV{PATH})) {
            return WindowsPath("$_/$BaseName") if -f "$_/$BaseName";
            return WindowsPath("$_/$BaseName.exe") if -f "$_/$BaseName.exe";
        }
    } else {
        foreach (split(/:/, $ENV{PATH})) {
            return "$_/$BaseName" if -f "$_/$BaseName";
        }
    }
    return;
}

#========================================================================================
# function     Has64BitOS
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Is64 = Has64BitOS();
#----------------------------------------------------------------------------------------
# example      $ENV{PROCESSOR_ARCHITECTURE} = 'AMD64' if Has64BitOS();
#----------------------------------------------------------------------------------------
# description  Returns 1 if operating system is 64 bit, else 0.
#----------------------------------------------------------------------------------------
# return       $Is64 | 1 if 64 bit, else 0 | integer
#========================================================================================
sub Has64BitOS {

    my $Arch;
    if ($^O eq 'MSWin32') {
        $Arch = GetEnvFromRegistry('PROCESSOR_ARCHITECTURE');
    } else {
        $Arch = SystemCommand('uname -m', { 'Capture' => 1 });
    }
    return $Arch =~ /64/ ? 1 : 0;
}


#========================================================================================
# function     GetEnvFromRegistry
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = GetEnvFromRegistry($Env);
#----------------------------------------------------------------------------------------
# example      $Arch = GetEnvFromRegistry('PROCESSOR_ARCHITECTURE');
#----------------------------------------------------------------------------------------
# description  Returns the value of the environment variable $Env from registry
#----------------------------------------------------------------------------------------
# input        $Env  | environment variable name | string
# return       $Value | environment variable value | integer
#========================================================================================
sub GetEnvFromRegistry {
    my ($Env) = @_;

    return if $^O ne 'MSWin32';

    my $Key = 'HKEY_LOCAL_MACHINE/System/CurrentControlSet/Control/Session Manager/Environment/';
    my $RegistryEntry = Win32::TieRegistry->new($Key, { Delimiter => '/' });
    return $RegistryEntry->GetValue($Env);
}

#========================================================================================
# function     AddJavaOpt
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddJavaOpt($Option, $Servers);
#----------------------------------------------------------------------------------------
# example      AddJavaOpt('-Xmx256M');
# example      AddJavaOpt('-Xmx256M', [qw(epagesj)]);
#----------------------------------------------------------------------------------------
# description  adds option $Option to environment of @$Servers.
#----------------------------------------------------------------------------------------
# input        $Option | java option | string
# input        $Servers | list of java servers
#                          allowed values: epagesj, solr
#                          default/unset: both epagesj and solr
#                        | array ref
#========================================================================================
sub AddJavaOpt ($;$) {
    my ($Option, $Servers) = @_;

    return if $^O eq 'MSWin32';

    $Servers = [qw(epagesj solr)] unless defined $Servers;
    my @Variables;
    push @Variables, 'EPAGESJ_SERVER_OPTS' if ElementOf('epagesj', $Servers);
    push @Variables, 'EPAGESJ_SOLR_OPTS' if ElementOf('solr', $Servers);

    my $DefaultFile = '/etc/default/';
    $DefaultFile .= $ENV{EPAGES_PROJECT} ? $ENV{EPAGES_PROJECT} : 'epages6';

    return unless -f $DefaultFile;

    my $sContent = GetFileContent($DefaultFile);
    my ($ContentHasChanged, $NewlineAppended);

    foreach my $EnvVar (@Variables) {
        if ($$sContent !~ /\n\s*$EnvVar=.*-Xmx/) {
            $ContentHasChanged = 1;
            if ($$sContent =~ /\n\s*$EnvVar=/) {
                my $Opts = SystemCommand( ". $DefaultFile ; echo \$$EnvVar", { 'Capture' => 1 });
                chomp($Opts);
                $Opts .= ' ' if $Opts;
                $$sContent =~ s,(\n\s*$EnvVar)=.*,$1="$Opts-Xmx256M",;
            } else {
                $$sContent .= "\n" unless $NewlineAppended;
                $NewlineAppended = 1;
                $$sContent .= "$EnvVar=-Xmx256M\n";
            }
        }
        if ($$sContent !~ /\n\s*export .*$EnvVar/) {
            $ContentHasChanged = 1;
            $$sContent .= "\n" unless $NewlineAppended;
            $NewlineAppended = 1;
            $$sContent .= "export $EnvVar\n";
        }
    }
    WriteFile($DefaultFile, $sContent) if $ContentHasChanged;
    return;
}

1;
