#========================================================================================
# package      DE_EPAGES::Core::API::Transformation
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides image manipulation functionality. See PerlMagick
#               documentation for possible functions:
#               http://www.imagemagick.org/script/perl-magick.php
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::Core::API::Transformation qw( Process );
#========================================================================================
package DE_EPAGES::Core::API::Transformation;
use base Exporter;

use strict;
our @EXPORT_OK = qw(
    Transform
);

# don't import any functions
use Image::Magick qw ( );
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf unique UsePackage);
use DE_EPAGES::Core::API::Constants;

#ERRORDEFINITION
sub ErrorDefinition () { return {
    'INVALID_TRANSFORMATION_TYPE' => 'Unknown transformation type: #Type',
}}
#ERRORDEFINITION END

#========================================================================================
# function     Transform
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResult = Transform($hTransformation, $hVars, $aChangedObjects);
#----------------------------------------------------------------------------------------
# description  Using the list of transformation types given with the transformation
#               hash (which is read from a transform.xml file), the responsible perl
#               module for each transformation is loaded. The transformation process is
#               then called using the Prepare(), Transform() and Finalize() methods
#               supplied by the loaded perl module.
#----------------------------------------------------------------------------------------
# input        $hTransformation | hash of transformation | hash.mixed
# input        $hVars | hash of vars like paths etc. | hash.mixed
# input        $aChangedObjects | list of objects, that have changed and trigger the
#               needed transformations (optional) | array
# return       $hResult | ? | ?
#========================================================================================
sub Transform {
    my ($hTransformation, $hVars, $aChangedObjects) = @_;
    $hVars = {} unless defined $hVars;

    my $hObjects = $hTransformation->{'Objects'};
    if (defined $aChangedObjects) {
        push @$aChangedObjects, grep { $hObjects->{$_}{'permanent'} } keys %$hObjects;
        @$aChangedObjects = unique @$aChangedObjects;
    } else {
        $aChangedObjects = [ keys %$hObjects ];
    }

    # prepare objects
    foreach my $Alias (keys %$hObjects) {
        my $hObject = $hObjects->{$Alias};

        my $Package = _getPackageForTransformType($hObject->{'Type'});
        UsePackage($Package);

        $hObject = $Package->Prepare($hObject, $hVars);
    }

    # these are needed to check if a transformation is mandatory
    my @Sources = (defined $aChangedObjects) ? @$aChangedObjects : keys %{$hTransformation->{'Objects'}};
    my $ahTransforms = $hTransformation->{'Transformations'};
    my $aUsedObjects = [];
    _GetUsedObjects($aUsedObjects, $_, $ahTransforms) foreach @Sources;

    my @Targets = @$aChangedObjects;
    my $Package;
    foreach my $hTransform (@$ahTransforms) {
        my $HasDepends = 0;
        foreach my $object (@$aUsedObjects) {
            if (ElementOf($object, $hTransform->{'Depends'})) {
                $HasDepends = 1;
                last;
            }
        }
        if (ElementOf($hTransform->{'Source'}, $aUsedObjects) || $HasDepends) {
            $Package = _getPackageForTransformType($hTransform->{'Type'});
            # check needed parameters
            Error('MISSING_SOURCE', { Type => $hTransform->{'Type'} }) unless (defined $hTransform->{'Source'});
            Error('MISSING_SOURCE_OBJECT', {
                Type => $hTransform->{'Type'},
                Object => $hTransform->{'Source'},
            }) unless (defined $hObjects->{$hTransform->{'Source'}});
            Error('WRONG_SOURCE_TYPE', {
                Type => $hTransform->{'Type'},
                Object => $hTransform->{'Source'},
                Type =>
            }) unless ($hObjects->{$hTransform->{'Source'}}->{'Type'} eq $hTransform->{'Type'});
            Error('MISSING_TARGET', { Type => $hTransform->{'Type'} }) unless (defined $hTransform->{'Target'});
            Error('MISSING_TARGET_OBJECT', {
                Type => $hTransform->{'Type'},
                Object => $hTransform->{'Target'},
            }) unless (defined $hObjects->{$hTransform->{'Target'}});
            Error('WRONG_TARGET_TYPE', {
                Type => $hTransform->{'Type'},
                Object => $hTransform->{'Target'},
                Type =>
            }) unless ($hObjects->{$hTransform->{'Target'}}->{'Type'} eq $hTransform->{'Type'});

            $Package->Transform($hTransform, $hObjects, $hVars);
            push @Targets, $hTransform->{'Target'};
        }
    }

    # perform finalize function on all objects and generate result
    my $hResult = {};
    my $hOptimizationByPackage;
    my $hForOptimization;
    foreach my $Alias (@Targets) {
        my $hObject = $hObjects->{$Alias};

        $Package = _getPackageForTransformType($hObject->{'Type'});
        UsePackage( $Package );

        my $hObjectValue = $Package->Finalize($hObject, $hVars);
        $hForOptimization = $hOptimizationByPackage->{$Package};
        $hForOptimization->{$hObjectValue} = $hObject;

        $hOptimizationByPackage->{$Package} = $hForOptimization;
        my $hModifiedObjectValue = $Package->ModifyObjectValue($hObjectValue, $hVars);
        $hResult->{$Alias} = $hModifiedObjectValue unless $hObject->{'IsTemporary'};
        $hResult->{$_->{'Alias'}} = $hModifiedObjectValue foreach (@{$hObject->{'Targets'}});
    }

    foreach my $Package (keys %$hOptimizationByPackage){
        my $hFinal = $hOptimizationByPackage->{$Package};
        $Package->Optimize($hFinal, $hVars->{'BaseDirectory'});
    }

    return $hResult;
}

#========================================================================================
# function     _GetUsedObjects
# state        private
#----------------------------------------------------------------------------------------
# syntax       _GetUsedObjects($aUsedObjects, $ChangedObject, $ahTransforms, $Direction);
# example      my @Sources;
#               if(defined $aChangedObjects) {
#                   @Sources = @$aChangedObjects;
#               } else {
#                   @Sources = keys %{$hTransformation->{'Objects'}};
#               }
#               my $ahTransforms = $hTransformation->{'Transformations'};
#               my $aUsedObjects = []; # target array
#               _GetUsedObjects($aUsedObjects, $_, $hObjects, $ahTransforms) foreach @Sources;
#----------------------------------------------------------------------------------------
# description  Includes the required perl modules for the changed object's
#               transformations. The method requires the list of functions to be applied
#               within the transforms and retrieves the used objects for all attributes
#               as well as for the source and target of the transform operation.
#               The final array of used objects is stored in $aUsedObjects, which
#               modifies the given array ref for this parameter from the caller function.
#----------------------------------------------------------------------------------------
# input        $aUsedObjects | list of objects to be affected by the transformation.
#               output parameter: the array is changed from this method | array.ref
# input        $ChangedObject | object, that has changed and triggers the
#               needed transformations | object
# input        $ahTransforms | list of transformations, each a hash containing the
#               transformation type, the source and the target for the transformation as
#               well as a list of the transformation functions | ref.arry.hash
# input        $Direction | Direction of the transform operation between source and
#               target parameters. Optional, default is 1. | integer
#========================================================================================
sub _GetUsedObjects {
    my ($aUsedObjects, $ChangedObject, $ahTransforms, $Direction) = @_;
    $Direction = 1 unless defined $Direction;

    return if (ElementOf($ChangedObject, $aUsedObjects));
    push @$aUsedObjects, $ChangedObject;
    foreach my $hTransform (@$ahTransforms) {
        my $Package = _getPackageForTransformType($hTransform->{'Type'});
        UsePackage($Package);

        my $ReplacableAttributes = $Package->ReplacableAttributes;
        if ($Direction > 0 and $ChangedObject eq $hTransform->{'Source'}) {
            # check if transformation source is a changed object
            _GetUsedObjects($aUsedObjects, $hTransform->{'Target'}, $ahTransforms, $Direction);
        } elsif ($Direction <= 0 and $ChangedObject eq $hTransform->{'Target'}) {
            # check if transformation source is a changed object
            _GetUsedObjects($aUsedObjects, $hTransform->{'Source'}, $ahTransforms, $Direction);
        }
        # check if a transformation's function needs a changed object
        foreach my $hFunction (@{$hTransform->{'Functions'}}) {
            next unless defined($ReplacableAttributes->{$hFunction->{'Function'}});
            my @Attributes = keys %{$ReplacableAttributes->{$hFunction->{'Function'}}};
            foreach my $Attribute (@Attributes) {
                if (defined($hFunction->{$Attribute}) &&
                    ($hTransform->{'Source'} eq $ChangedObject || $hFunction->{$Attribute} eq $ChangedObject)) {
                    _GetUsedObjects($aUsedObjects, $hFunction->{$Attribute}, $ahTransforms, 0);
                    push @$aUsedObjects, $hFunction->{$Attribute};
                    _GetUsedObjects($aUsedObjects, $hTransform->{'Target'}, $ahTransforms);
                    _GetUsedObjects($aUsedObjects, $hTransform->{'Source'}, $ahTransforms, 0);
                }
            }
        }
    }
    return;
}

#========================================================================================
# function     _getPackageForTransformType
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package = _getPackageForTransformType($hObject->{'Type'});
#----------------------------------------------------------------------------------------
# description  Returns the package name which corresponds to the given transform type.
#----------------------------------------------------------------------------------------
# input        $Type | transformation type | string
# return       $Package | transformation package name | string
#========================================================================================
sub _getPackageForTransformType {
    my ($Type) = @_;

    my @Types = TRANSFORMATION_TYPES();
    Error('INVALID_TRANSFORMATION_TYPE' , {'Type'=> $Type}) unless ElementOf($Type, @Types);
    return __PACKAGE__ . '::' . $Type;

}

1;
