use strict;
use Config;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw ( SystemCommand );
use DE_EPAGES::Database::API::Config qw ( StoreTypesWithCartridge );
use File::Slurp qw ( read_dir );
use Getopt::Long;
use Time::HiRes qw ( time );

my $Verbose;

sub Main {
    my $Help;
    my $Version;
    my $SonarServer;
    my $Company;

    GetOptions(
        'help' => \$Help,
        'version=s' => \$Version,
        'sonarserver=s' => \$SonarServer,
        'company=s' => \$Company,
        'v' => \$Verbose,
    );

    Usage() if ($Help);
    Usage() if (not $Version);
    Usage() if (not $SonarServer);
    Usage() if (not $Company);

    my $StartTime = time;
    my $VerboseBoolean = $Verbose?'true':'false';
    my $aSonarScannerArgs = [
        "sonar.projectVersion=$Version",
        "sonar.host.url=$SonarServer",
        "sonar.verbose=$VerboseBoolean",
        'sonar.sourceEncoding=ISO-8859-1',
    ];
    my $SonarScannerArgs = join(' ',map( {"--define $_"} @{$aSonarScannerArgs}));

    my $BaseDir = $ENV{'EPAGES_CARTRIDGES'}.'/'.$Company;
    my $aCartridgeNames = read_dir($BaseDir);

    foreach my $CartridgeName (@{$aCartridgeNames}){
        print '*' x 80 . "\n** $CartridgeName\n".'*' x 80 . "\n"; ## no critic (ProhibitMagicNumbers)
        my $Store = _GetStore($Company, $CartridgeName);
        my $CartridgeDir = "$BaseDir/$CartridgeName";
        if(-d $CartridgeDir){
            chdir($CartridgeDir);
            next unless -e 'Makefile.PL';
            SystemCommand( [$^X, 'Makefile.PL'], { 'Warning' => 1 }); #$^X current perl interpreter
            SystemCommand( ['make', 'sonar_reports', "STORE=$Store" ], { 'Warning' => 1 });
            SystemCommand( ['make', 'sonar_scan', "SONAR_SCANNER_ARGS=$SonarScannerArgs" ], { 'Warning' => 1 });
        } else {
            print"'$CartridgeDir' doesn't exist!\n";
        }
    }
    my $NeededTime = time - $StartTime;
    my $Minutes = $NeededTime / 60; ## no critic (ProhibitMagicNumbers)
    my $Seconds = $NeededTime % 60; ## no critic (ProhibitMagicNumbers)
    printf "\n\nSonar scanner needed %.2d:%.2d  min:sec\n", $Minutes, $Seconds;
}

sub _GetStore {
    my ($Company, $CartridgeName) = @_;
    my $StoreTypes = StoreTypesWithCartridge($Company.'::'.$CartridgeName);
    my $StoreType = $StoreTypes->[0];
    if($StoreType){
        return $StoreType;
    }else{
        print "Fallback to 'Store' as Storetype \n";
        return 'Store';
    }
}

sub Usage {
    print <<END_USAGE;
Description:
    Iterates through all company cartridges, creates sonar reports and forwards them to the sonar server.

flags:
    -help        show the command line options
    -version     epages version number
    -sonarserver url of sonar server
    -company     name of company folder
    -v           verbose

Example:
    perl $0 -version 7.2.0 -sonarserver http://localhost:9000 -company DE_EPAGES -v
END_USAGE
    exit 2;
}
RunScript(Sub => sub {
    exit Main();
});
