use strict;
use threads;
use Test::More tests => 7;
use IO::Socket::INET6;
use DE_EPAGES::Core::API::Error qw( HandleError );
use DE_EPAGES::Core::API::Object::UDPMultiClient;
use DE_EPAGES::Core::API::PerlTools qw( GetIPAddress ElementOf );

## no critic qw( ProhibitMagicNumbers );
my $Host = 'localhost';
my $Port = 5151;
my $NumServers = 3;
my $Debug = 0;
my $Mode = $ARGV[0];
if ( not defined $Mode ) {
    # test mode

    my $pid = fork();
    die "fork() failed: $!" unless defined $pid;
    if ($pid) {
        eval {
            sleep 1; # wait for the server tor start
            Client(); # run test cases
            sleep 1; # wait for the server to shutdown
            my $wpid = wait;
            is( $wpid, $pid, 'server has finished' ) or die "Server process not finished: $!";
        };
        HandleError( 'OnError' => sub {
            Test::More::diag "killing server $pid";
            kill('TERM', $pid);
        });
    }
    else {
        Server();
    }
}
elsif( $Mode eq 'server' ) {
    Server();
}
elsif( $Mode eq 'client' ) {
    Client();
}

sub Server {
    my @Threads;
    my $IP = GetIPAddress( $Host );
    my $Family = $IP =~ /:/ ? AF_INET6 : AF_INET;

    for my $ServerIndex (0 .. $NumServers - 1) {
        push(@Threads, async {
            my $Socket = IO::Socket::INET6->new(
                LocalHost => $Host,
                LocalPort => $Port + $ServerIndex,
                Type      => SOCK_DGRAM,
                Proto     => 'udp',
                Domain    => $Family,
            ) or die "socket: $@";

            print "Awaiting UDP messages on port $Port\n" if $Debug;

            my $Shutdown;
            my $Request;
            my $MAXLEN = 1024;
            while ($Socket->recv($Request, $MAXLEN)) {
                my $ClientHost = $Socket->peerhost;
                print "Client $ClientHost said '$Request'\n" if $Debug;
                $Socket->send("$Request done");

                if( $Request eq 'Shutdown' ) {
                    $Shutdown = 1;
                    last;
                }
            }

            if( $Shutdown ) {
                print "shutting down\n" if $Debug;
            }
            else {
                die "recv: $!";
            }
        });
    }
    $_->join() foreach (@Threads);
}

sub Client {
    my @Hosts;
    push(@Hosts, [$Host, $Port + $_]) for (0 .. $NumServers - 1);
    push(@Hosts, ['192.168.231.123', 52573]); # random unreachable/offline host

    my $UDPClient = DE_EPAGES::Core::API::Object::UDPMultiClient->new(\@Hosts);
    is( $UDPClient->sendReceive('Hello1'), 'Hello1 done', 'sendReceive');
    $UDPClient->send('Hello2');
    is( $UDPClient->receive(1.0, 1000), 'Hello2 done', 'receive 1');
    $UDPClient->send('Hello3');
    is( $UDPClient->receive(1.0, 1000), 'Hello3 done', 'receive 2' );

    # order independency test. requires autoflush to be disabled
    $UDPClient->useAutoflush(0);
    $UDPClient->send('Hello5');
    sleep(0.1);
    $UDPClient->send('Hello4');

    my @Responses;
    my $Response;
    do {
        $Response = $UDPClient->receive(1.0, 1000);
        push(@Responses, $Response) if(defined($Response));
    } while(defined($Response));

    ok( ElementOf('Hello4 done', @Responses), 'receive 3' );
    ok( ElementOf('Hello5 done', @Responses), 'receive 4' );

    $UDPClient->useAutoflush(1);
    $UDPClient->send('Shutdown');
    is( $UDPClient->receive(1.0, 1000), 'Shutdown done', 'receive Shutdown' );
}
