#========================================================================================
# package      DE_EPAGES::CustomForm::API::Object::CustomForm
# state        public
# base         DE_EPAGES::Content::API::Object::ContentObject
# description  object interface for CustomForm.
#========================================================================================
package DE_EPAGES::CustomForm::API::Object::CustomForm;
use base DE_EPAGES::Content::API::Object::ContentObject;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadClassByAlias );
use DE_EPAGES::Object::API::Language;
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguage);
use DE_EPAGES::Object::API::Template qw (ProcessFileTemplate);

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $CustomForm->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name ('CustomForm').
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name | String
#========================================================================================
sub featureName { return 'CustomForm'; }

#========================================================================================
# function     insert
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CustomForm = DE_EPAGES::CustomForm::API::Object::CustomForm->insert($hInfo);
# example      my $Class = LoadClassByAlias('CustomForm');
#               my $CustomForm = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $CustomForm | new object | object
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);
    $self->setDefaultTranslations();

    return $self;
}

#========================================================================================
# function     setDefaultTranslations
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CustomForm->setDefaultTranslations();
#----------------------------------------------------------------------------------------
# description  set MailSubject and ConfirmMessage in all languages
#========================================================================================
sub setDefaultTranslations {
    my $self = shift;

    # set translated attributes
    my $MailSubjectContent    = $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/CustomForm/Templates/MBO/MailSubjectContent.txt';
    my $ConfirmMessageContent = $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/CustomForm/Templates/MBO/ConfirmMessageContent.txt';

    foreach my $LanguageID (@{$self->getSite()->languages}) {
        local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;
        my $Code2 = GetInfoLanguage($LanguageID)->{'Code2'};
        my $MailSubject    = ProcessFileTemplate($MailSubjectContent, $Code2, {});
        my $ConfirmMessage = ProcessFileTemplate($ConfirmMessageContent, $Code2, {});

        $self->set({
            'MailSubject'    => $MailSubject,
            'ConfirmMessage' => $ConfirmMessage
        }, $LanguageID);
    }
}

#========================================================================================
# function     createEmailFormField
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CustomForm->createEmailFormField();
#----------------------------------------------------------------------------------------
# description  Creates a new email form field if it isn't existing yet
#========================================================================================
sub createEmailFormField {
    my $self = shift;

    return if ($self->existsChild('Fields') and $self->child('Fields')->existsChild('EMail'));

    my $Folder = $self->folder('Fields');
    my $FieldClass = LoadClassByAlias('EMailFormField');
    my $NewFieldObject = $Folder->insertChild($FieldClass, { 'Alias' => 'EMail' });

    $NewFieldObject->set({
        IsAlterable => 0,
        Position => 9999,
        IsMandatory => 1
    });
    $Folder->renumberChildren;

    return;
}

#========================================================================================
# function     saveFields
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CustomForm->saveFields($ahListPositions, $aSetMandatory, $ahListDeletings);
#----------------------------------------------------------------------------------------
# description  Saves attributes of all fields
#----------------------------------------------------------------------------------------
# input       $ahListPositions | PositionObjectID and it's (new) Position | ref.array.hash
# input       $aSetMandatory | field id's which have to be set to mandatory | ref.array
# input       $ahListDeletings | field id's which have to be deleted  | ref.array.hash
#========================================================================================
sub saveFields {
    my $self = shift;
    my ($ahListPositions, $aSetMandatory, $ahListDeletings) = @_;

    my $Folder = $self->folder('Fields');
    my $aExistingFields = $self->get('Fields');

    # save positions
    foreach my $hListPosition (@$ahListPositions) {
        my $Field = LoadObject($hListPosition->{'PositionObjectID'});
        $Field->set({'Position' => $hListPosition->{'Position'}});
    }

    # save IsMandatory
    foreach my $Field (grep { $_->class->alias ne 'EMailFormField' } @$aExistingFields) {
        $Field->set({ 'IsMandatory' => ElementOf($Field->id, $aSetMandatory) });
    }

    # delete marked fields
    foreach my $hDeleting (@$ahListDeletings) {
        if (defined $hDeleting->{'DeleteObjectID'}) {
            my $Field = LoadObject($hDeleting->{'DeleteObjectID'});
            $Field->delete if ($Field->exists and $Field->class->alias =~ /FormField$/);
        }
    }

    $Folder->renumberChildren;

    return;
}

1;
