#========================================================================================
# package      DE_EPAGES::Customer::API::Attributes::Shop
# state        private
# description  Attribute interface for Shop.
#========================================================================================
package DE_EPAGES::Customer::API::Attributes::Shop;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Core::API::CSV;
use DE_EPAGES::Customer::API::Constants qw ( @UNITY_REGISTRATION_SET $UNITY_REGISTRATION_SET_ADDITIONAL );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf unique );

my $SEP_CHAR = ',';

sub getAttributes {
    shift;
    my ($Object, $aNames) = @_;
    my %Info;
    foreach my $Name (@$aNames) {
        if( $Name eq 'Customers' ) {
            $Info{$Name} = LoadClassByAlias('Customer')->getObjects('ByShopID', [ $Object->id ]);
        }
        elsif( $Name eq 'CustomerGroups' ) {
            $Info{$Name} = LoadClassByAlias('CustomerGroup')->getObjects('ByShopID', [ $Object->id ]);
        }
        elsif( $Name eq 'CustomerClassID' ) {
            $Info{$Name} = _CustomerClass( $Object )->id;
        }
        elsif( $Name eq 'RegistrationSet') {
            $Info{$Name} = _RegistrationSet( $Object );
        }
        elsif( $Name eq 'UnityRegistrationSet') {
            my $FullRegistrationSet = _RegistrationSet( $Object );
            my %FilteredRegistrationSet;
            my @FinalRegistrationSet = @UNITY_REGISTRATION_SET;

            # add additional fields to registration set for some countries
            my $Address = $Object->get('Address');
            my $ShopCountry = defined($Address) ? $Address->get('Country') : undef;

            if(defined $ShopCountry and exists $UNITY_REGISTRATION_SET_ADDITIONAL->{$ShopCountry->{'Code2'}} ){
                push @FinalRegistrationSet, $_ foreach @{$UNITY_REGISTRATION_SET_ADDITIONAL->{$ShopCountry->{'Code2'}}};
            }

            foreach my $RegistrationField (@$FullRegistrationSet) {
                my $Alias = $RegistrationField->{'Attribute'}->alias;
                next unless ElementOf($Alias, @FinalRegistrationSet);
                $FilteredRegistrationSet{$Alias} = $RegistrationField;
            }
            # sort registration set
            my @UnityRegistrationSet = map { $FilteredRegistrationSet{$_} } @FinalRegistrationSet;
            $Info{$Name} = \@UnityRegistrationSet;
        }
        elsif ($Name eq 'RegistrationField') {
            my %h;
            my $RegistrationSet = _RegistrationSet($Object);
            foreach my $Set (@$RegistrationSet){
                $h{$Set->{'Attribute'}->alias} =  $Set;
            }
            $Info{$Name} = \%h;
        }
    }
    return \%Info;
}

sub setAttributes {
    shift;
    my ($Object, $hUpdate) = @_;
    foreach my $Name (keys %$hUpdate) {
        if ($Name eq 'RegistrationSet' || $Name eq 'UnityRegistrationSet') {
             _SetRegistrationSet($Object,$hUpdate->{$Name});
        }
    }
    return;
}

sub deleteObject {
    shift;
    my ($Object) = @_;
    $_->delete foreach @{$Object->get('Customers')};
    $_->delete foreach @{$Object->get('CustomerGroups')};
}

sub _CustomerClass {
    my ($Object) = @_;

    $Object->insertChild( 'Class', {
        Alias => 'CustomerClass',
        BaseClass => LoadClassByAlias( 'Customer' )
    } ) unless $Object->existsChild( 'CustomerClass' );

    return $Object->child( 'CustomerClass' );
}

sub _RegistrationSet{
    my ($Object) = @_;

    my $IsVisible     = $Object->get('RegistrationSetVisibleFields');
    my $IsMandatory   = $Object->get('RegistrationSetMandatoryFields');
    my $IsBAVisible   = $Object->get('BillingAddressSetVisibleFields');
    my $IsBAMandatory = $Object->get('BillingAddressSetMandatoryFields');
    my $IsSAVisible   = $Object->get('ShippingAddressSetVisibleFields');
    my $IsSAMandatory = $Object->get('ShippingAddressSetMandatoryFields');

    $IsVisible     = '' unless defined $IsVisible     ;
    $IsMandatory   = '' unless defined $IsMandatory   ;
    $IsBAVisible   = '' unless defined $IsBAVisible   ;
    $IsBAMandatory = '' unless defined $IsBAMandatory ;
    $IsSAVisible   = '' unless defined $IsSAVisible   ;
    $IsSAMandatory = '' unless defined $IsSAMandatory ;

    my $Converter = DE_EPAGES::Core::API::CSV->new( { 'sep_char' => $SEP_CHAR } );
    my $aVisible = $Converter->parse( $IsVisible );
    my $aMandatory = $Converter->parse( $IsMandatory );
    my $aBAVisible = $Converter->parse( $IsBAVisible );
    my $aBAMandatory = $Converter->parse( $IsBAMandatory );
    my $aSAVisible = $Converter->parse( $IsSAVisible );
    my $aSAMandatory = $Converter->parse( $IsSAMandatory );

    my $ClassCustomerAttributes = $Object->get('CustomerClass')->attributes;
    my $ClassAddressAttributes  = LoadClassByAlias('Address')->attributes;
    my @RegistrationSet;
    foreach my $Attribute (@$ClassCustomerAttributes, @$ClassAddressAttributes) {
        next if not $Attribute->get('IsExportable') or $Attribute->get('IsReadOnly');
        my $Alias = $Attribute->alias;
        next if ElementOf($Alias, qw(GUID Alias Inherit Position CurrencyID));
        my %Info;
        $Info{'Attribute'}   = $Attribute;
        $Info{'IsVisible'}   = (ElementOf($Alias, $aVisible)) ? 1 : 0;
        $Info{'IsMandatory'} = (ElementOf($Alias, $aMandatory)) ? 1 : 0;
        $Info{'IsBillingAddressVisible'}   = (ElementOf($Alias, $aBAVisible)) ? 1 : 0;
        $Info{'IsBillingAddressMandatory'} = (ElementOf($Alias, $aBAMandatory)) ? 1 : 0;
        $Info{'IsShippingAddressVisible'}   = (ElementOf($Alias, $aSAVisible)) ? 1 : 0;
        $Info{'IsShippingAddressMandatory'} = (ElementOf($Alias, $aSAMandatory)) ? 1 : 0;
        $Info{'IsShippingAndBillingAddressVisible'}   = (ElementOf($Alias, (@$aSAVisible, @$aBAVisible))) ? 1 : 0;
        $Info{'IsShippingAndBillingAddressMandatory'} = (ElementOf($Alias, (@$aSAMandatory, @$aBAMandatory))) ? 1 : 0;
        $Info{'IsRegistrationAndShippingAndBillingAddressVisible'}   = (ElementOf($Alias, (@$aVisible, @$aSAVisible, @$aBAVisible))) ? 1 : 0;
        $Info{'IsRegistrationAndShippingAndBillingAddressMandatory'} = (ElementOf($Alias, (@$aMandatory, @$aSAMandatory, @$aBAMandatory))) ? 1 : 0;
        push @RegistrationSet, \%Info;
    }

    @RegistrationSet = sort { $a->{Attribute}->get('Position') <=> $b->{'Attribute'}->get('Position') } @RegistrationSet;

    return \@RegistrationSet;
}

sub _SetRegistrationSet {
    my ($Object, $RegistrationSet) = @_;

    my (@AreVisible, @AreMandatory, @AreBAVisible, @AreBAMandatory, @AreSAVisible, @AreSAMandatory);
    foreach my $Info (@$RegistrationSet) {
        push @AreVisible,     $Info->{'Attribute'}->alias if $Info->{'IsVisible'};
        push @AreMandatory,   $Info->{'Attribute'}->alias if $Info->{'IsMandatory'};
        push @AreBAVisible,   $Info->{'Attribute'}->alias if $Info->{'IsBillingAddressVisible'};
        push @AreBAMandatory, $Info->{'Attribute'}->alias if $Info->{'IsBillingAddressMandatory'};
        push @AreSAVisible,   $Info->{'Attribute'}->alias if $Info->{'IsShippingAddressVisible'};
        push @AreSAMandatory, $Info->{'Attribute'}->alias if $Info->{'IsShippingAddressMandatory'};
    }

    #The country selection is necessary for triggering the state/county selectbox (AD-2336)
    push @AreVisible,     'CountryID' if ( ElementOf('State', @AreVisible     ) and not ElementOf('CountryID', @AreVisible) );
    push @AreMandatory,   'CountryID' if ( ElementOf('State', @AreMandatory   ) and not ElementOf('CountryID', @AreMandatory) );
    push @AreBAVisible,   'CountryID' if ( ElementOf('State', @AreBAVisible   ) and not ElementOf('CountryID', @AreBAVisible) );
    push @AreBAMandatory, 'CountryID' if ( ElementOf('State', @AreBAMandatory ) and not ElementOf('CountryID', @AreBAMandatory) );
    push @AreSAVisible,   'CountryID' if ( ElementOf('State', @AreSAVisible   ) and not ElementOf('CountryID', @AreSAVisible) );
    push @AreSAMandatory, 'CountryID' if ( ElementOf('State', @AreSAMandatory ) and not ElementOf('CountryID', @AreSAMandatory) );

    my $Converter = DE_EPAGES::Core::API::CSV->new( { 'sep_char' => $SEP_CHAR } );
    my $IsVisible = $Converter->combine([ unique @AreVisible, @AreMandatory ]);
    my $IsMandatory = $Converter->combine( \@AreMandatory );
    my $IsBAVisible = $Converter->combine([ unique @AreBAVisible, @AreBAMandatory ]);
    my $IsBAMandatory = $Converter->combine( \@AreBAMandatory );
    my $IsSAVisible = $Converter->combine([ unique @AreSAVisible, @AreSAMandatory ]);
    my $IsSAMandatory = $Converter->combine( \@AreSAMandatory );

    $Object->set({
        'RegistrationSetVisibleFields'          => $IsVisible,
        'RegistrationSetMandatoryFields'        => $IsMandatory,
        'BillingAddressSetVisibleFields'        => $IsBAVisible,
        'BillingAddressSetMandatoryFields'      => $IsBAMandatory,
        'ShippingAddressSetVisibleFields'       => $IsSAVisible,
        'ShippingAddressSetMandatoryFields'     => $IsSAMandatory,
    });
}

1;
