#========================================================================================
# package      DE_EPAGES::Customer::Hooks::Shop
# state        public
# description  Changes tax class on declared products of a shop if the shops' tax
#               matrix is changed. When a new shop is created the folders 'ProductTypes'
#               and 'Products' are imported.
#========================================================================================
package DE_EPAGES::Customer::Hooks::Shop;

use strict;
use DE_EPAGES::Customer::API::Cartridge;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Presentation::API::Template qw ( Translate );
use DE_EPAGES::ShopMailType::API::ShopMailType qw ( GetShopMailType );

#========================================================================================
# function     OnInsertShop
#----------------------------------------------------------------------------------------
# syntax       OnInsertShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Imports the folders 'ProductTypes' and 'Products' into a new shop when
#               it is created.
#               (Hook OBJ_InsertShop)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    my $Cartridge = DE_EPAGES::Customer::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShop.xml', $Shop->pathString);
}

#========================================================================================
# function     OnBeforeUpdateShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Changes tax classes of customers if the shops' tax matrix was changed.
#               (Hook OBJ_BeforeUpdateShop)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - shop - object
#                   <li>Values
#                       <ul>
#                           <li>TaxMatrix - tax matrix - object
#                       </ul>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateShop {
    my ($hParams) = @_;
    return unless exists $hParams->{'Values'}{'TaxMatrix'};

    my $Shop = $hParams->{'Object'};
    return if $Shop->get('TaxMethod');

    my $OldTaxMatrix = $Shop->get('TaxMatrix');
    my $NewTaxMatrix = $hParams->{'Values'}->{'TaxMatrix'};
    return if !defined $OldTaxMatrix || $NewTaxMatrix->id == $OldTaxMatrix->id;

    my %TaxAreas;
    $TaxAreas{$_->get('Alias')} = $_ foreach @{$NewTaxMatrix->get('TaxAreas')};
    my $Customers = $Shop->get('Customers');
    foreach my $Customer (@$Customers) {
        my $TaxArea = $Customer->get('TaxArea');
        next unless defined $TaxArea;
        my $NewTaxArea = $TaxAreas{$TaxArea->get('Alias')} // $NewTaxMatrix->get('DefaultTaxArea');
        $Customer->set({'TaxArea' => $NewTaxArea});
    }
}
#========================================================================================
# function     OnJSONAdvancedLinks
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnJSONAdvancedLinks( $hParams );
#----------------------------------------------------------------------------------------
# description  adds a link to to advanced links section in the link picker inside the
#               MBO
#               (Hook OBJ_JSONAdvancedLinks)
#----------------------------------------------------------------------------------------
# input        $hParams |
#               <ul>
#                   <li>Object - shop - object</li>
#                   <li>Values - attribute values - ref.hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnJSONAdvancedLinks {
    my ($hParams) = @_;
    my $SitePath = $hParams->{'Object'}->pathString;
    push @{$hParams->{'Links'}},{
        Alias=>Translate('Registration',undef,'DE_EPAGES::Customer'),
        Link=>"?ViewAction=ViewRegistration&ObjectPath=$SitePath"
    },
    {
        Alias=>Translate('Login',undef,'DE_EPAGES::Customer'),
        Link=>"?ViewAction=ViewRegistrationRedirect&HideNotice=1&ObjectPath=$SitePath"
    };
}

#========================================================================================
# function     OnFinishShopWizard
#----------------------------------------------------------------------------------------
# syntax       OnFinishShopWizard( $hParams );
# state        public
#----------------------------------------------------------------------------------------
# description  Sets the BCC email addresses for mail type CustomerWelcome
#               on completion of the shop wizard (Hook UI_FinishShopWizard).
#----------------------------------------------------------------------------------------
# input        $hParams | form values which are supplied by Form ShopWizard of class Shop.
#               <ul>
#                   <li>Shop - the current Shop - object
#                   <li>Data - form values from Shop.ShopWizard - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnFinishShopWizard {
    my ($hValues) = @_;

    my $Shop = $hValues->{'Shop'};
    my $hForm = $hValues->{'Data'};

    my $ShopMailType = GetShopMailType( $Shop, 'CustomerWelcome' );
    my $BCC = $ShopMailType->get('BCC');
    $ShopMailType->set({ 'BCC' =>  $hForm->{'EMail'} }) unless defined $BCC;
    return;
}

#========================================================================================
# function     OnSaveNumberFormats
#----------------------------------------------------------------------------------------
# syntax       OnSaveNumberFormats( $hParams );
# state        public
#----------------------------------------------------------------------------------------
# description  Evaluates the form 'SaveNumberFormats_Customer' and returns its data.
#               Form errors are not thrown directly but passed to the caller.
#----------------------------------------------------------------------------------------
# input        $hParams | form values which are supplied by Form SaveNumberFormats_Customer of class Shop.
#               <ul>
#                   <li>Shop - the current Shop - DE_EPAGES::Shop::API::Object::Shop
#                   <li>Form - form object to evaluate - DE_EPAGES::Presentation::API::Form
#                   <li>Data - form values to return to caller - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveNumberFormats {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Shop'};
    my $Form = $hParams->{'Form'};
    my $hValues = $hParams->{'Values'};

    my $hFormValues = $Form->form($Shop, 'SaveNumberFormats_Customer', 1);
    my $NextCustomerNo = $hFormValues->{'NextCustomerNo'};
    return unless defined $NextCustomerNo;

    my $hOldNumberFormats = $Shop->get('NumberFormats');
    my %Values;
    foreach my $key (grep { /^Customer/ } keys %$hFormValues){
        my ($fl,$sl) = split(/_/,$key);
        $Values{$fl}{$sl} = $hFormValues->{$key} if $hFormValues->{$key};
    }
    $Shop->set({ 'NumberFormats' => \%Values });

    my $Number = $Shop->formatNumber('Customer', $NextCustomerNo);
    my $Class = $Shop->get('CustomerClass');
    if (scalar @{$Class->findObjects('BySiteIDClassIDAlias', [$Shop->id, $Class->id, $Number])}) {
        $Form->addFormError({
            'Form'   => $hFormValues->{'FormName'},
            'Reason' => 'OBJECT_EXISTS',
            'Name'   => 'NextCustomerNo',
        });
        $Shop->set({ 'NumberFormats' => $hOldNumberFormats });
    }

    %$hValues = ( %$hValues, %$hFormValues );
    return;
}

1;

