#========================================================================================
# package      DE_EPAGES::DHL::API::XML::DHLProductHandler
# base         DE_EPAGES::XML::API::BaseImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Import handler for DHL product service definition files.
#----------------------------------------------------------------------------------------
# example      my $fh = FileHandle->new($FileName, "r");
#               ParseProductFile($fh, $NoOutput, 'IMPORT');
#========================================================================================
package DE_EPAGES::DHL::API::XML::DHLProductHandler;
use base qw(DE_EPAGES::XML::API::BaseImportHandler Exporter);

use strict;

use XML::SAX::ParserFactory;
use DateTime::Format::ISO8601;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::DHL::API::Table::DHLProductList qw( InsertDHLProductList );
use DE_EPAGES::DHL::API::Table::DHLProdListAttribute qw(
    InsertDHLProdListAttribute
    UpdateDHLProdListAttribute
    ExistsDHLProdListAttributeByDHLProductListID
    GetPKeyDHLProdListAttributesByDHLProductListID
);
use DE_EPAGES::DHL::API::Table::DHLProdListDeliveryDest qw(
    InsertDHLProdListDeliveryDest
    UpdateDHLProdListDeliveryDest
    ExistsDHLProdListDeliveryDestByDHLProductListID
    GetPKeyDHLProdListDeliveryDestByDHLProductListID
);

use DE_EPAGES::Core::API::Log qw( LogDebug GetLog );

our @EXPORT_OK = qw( ParseProductFile ParseProductData );

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless {}, $class;
    $self->{'NoOutput'} = $args{'NoOutput'};
    $self->{'Action'}   = $args{'Action'};
    return $self;
}

#========================================================================================
# function     ParseProductFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       ParseProductFile( $FileHandler, $NoOutput, $Action );
#----------------------------------------------------------------------------------------
# description  Parse DHL product service description file
#----------------------------------------------------------------------------------------
# input        $FileHandler | XML parser | object
# input        $NoOutput | suppress output | boolean
# input        $Action | use string 'IMPORT' for importing. Nothing else implemented. | string
#========================================================================================
sub ParseProductFile {
    my ($FileHandler, $NoOutput, $Action) = @_;
    my $Handler = __PACKAGE__->new(
                                   'NoOutput' => $NoOutput,
                                   'Action'   => $Action,
                                   );
    my $Parser = XML::SAX::ParserFactory->parser( Handler => $Handler );

    $Parser->parse_file($FileHandler);
    return;
}

#========================================================================================
# function     ParseProductData
# state        public
#----------------------------------------------------------------------------------------
# syntax       ParseProductData( $hData, $NoOutput, $Action );
#----------------------------------------------------------------------------------------
# description  Parse DHL product service description. Takes an already parsed hash
#               instead of XML data
#----------------------------------------------------------------------------------------
# input        $hData | Data parsed from XML file | ref.hash
# input        $NoOutput | suppress output | boolean
# input        $Action | use string 'IMPORT' for importing. Nothing else implemented. | string
#========================================================================================
sub ParseProductData {
    my ($hData, $NoOutput, $Action) = @_;

    my $Handler = __PACKAGE__->new(
                                   'NoOutput' => $NoOutput,
                                   'Action'   => $Action,
                                   );

    my $hProducts = $hData->{'products'};
    foreach my $ProductID (keys %$hProducts) {
        my $hProduct = $hProducts->{$ProductID};
        $Handler->_ImportDHLProductList($hProduct);
    }

    return;
}

sub start_dpwn {}
sub start_Stand {}

sub start_produkte {
}

sub end_produkte {
}

sub start_Produkt {
    my $self = shift;
#    my ($el) = @_;
    $self->pushID('Structure', {});
}

sub end_Produkt {
    my $self = shift;
    my $hProduct = $self->popID('Structure');
    #push(@{$self->topID('Structure')}, $hProduct);

    if ($self->{'Action'} eq 'DELETE') {
        _DeleteDHLProductList($self, $hProduct);
    }
    else {
        _ImportDHLProductList($self, $hProduct);
    }
}

sub start_anbieter {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_anbieter {
    my $self = shift;
    $self->topID('Structure')->{'anbieter'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_id {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_id {
    my $self = shift;
    $self->topID('Structure')->{'id'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_anzeigename {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_anzeigename {
    my $self = shift;
    $self->topID('Structure')->{'anzeigename'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_anzeigetext {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_anzeigetext {
    my $self = shift;
    $self->topID('Structure')->{'anzeigetext'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_bildUrl {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_bildUrl {
    my $self = shift;
    $self->topID('Structure')->{'bildUrl'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_gueltigVon {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_gueltigVon {
    my $self = shift;
    $self->topID('Structure')->{'gueltigVon'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_gueltigBis {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_gueltigBis {
    my $self = shift;
    $self->topID('Structure')->{'gueltigBis'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_url {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_url {
    my $self = shift;
    $self->topID('Structure')->{'url'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_agb {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_agb {
    my $self = shift;
    $self->topID('Structure')->{'agb'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_druckhinweis {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_druckhinweis {
    my $self = shift;
    $self->topID('Structure')->{'druckhinweis'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_lieferziele {
    my $self = shift;
    # my ($el) = @_;
    $self->pushID('Structure', []);
}

sub end_lieferziele {
    my $self = shift;
    my $ahLieferziele = $self->popID('Structure');
    $self->topID('Structure')->{'Lieferziele'} = $ahLieferziele;
}

sub start_Lieferziel {
    my $self = shift;
    # my ($el) = @_;
    $self->pushID('Structure', {});
}

sub end_Lieferziel {
    my $self = shift;
    my $hLieferziel = $self->popID('Structure');
    push(@{$self->topID('Structure')}, $hLieferziel);
}

sub start_ausnahmen { # at the moment ignored
}

sub start_laendercode {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_laendercode {
    my $self = shift;
    $self->topID('Structure')->{'laendercode'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_preis {
    my $self = shift;
    # my ($el) = @_;
    $self->pushID('Structure', {});
}

sub end_preis {
    my $self = shift;
    my $hPrice = $self->popID('Structure');
    $self->topID('Structure')->{'Price'} = $hPrice;
}

sub start_gesamtpreisBetragEur {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_gesamtpreisBetragEur {
    my $self = shift;
    $self->topID('Structure')->{'gesamtpreisBetragEur'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_gesamtpreisAnzeigename {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_gesamtpreisAnzeigename {
    my $self = shift;
    $self->topID('Structure')->{'gesamtpreisAnzeigename'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_zollerklaerung {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_zollerklaerung {
    my $self = shift;
    $self->topID('Structure')->{'zollerklaerung'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_CN22Limit {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_CN22Limit {
    my $self = shift;
    $self->topID('Structure')->{'CN22Limit'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_vorausverfuegung {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_vorausverfuegung {
    my $self = shift;
    $self->topID('Structure')->{'vorausverfuegung'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_merkmale {
    my $self = shift;
    # my ($el) = @_;
    $self->pushID('Structure', []);
}

sub end_merkmale {
    my $self = shift;
    my $ahMerkmale = $self->popID('Structure');
    $self->topID('Structure')->{'Merkmale'} = $ahMerkmale;
}

sub start_Merkmal {
    my $self = shift;
    # my ($el) = @_;
    $self->pushID('Structure', {});
}

sub end_Merkmal {
    my $self = shift;
    my $hMerkmal = $self->popID('Structure');
    push(@{$self->topID('Structure')}, $hMerkmal);
}

sub start_name {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_name {
    my $self = shift;
    $self->topID('Structure')->{'name'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_maximalbetragEur {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_maximalbetragEur {
    my $self = shift;
    $self->topID('Structure')->{'maximalbetragEur'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_paket {
    my $self = shift;
    # my ($el) = @_;
    $self->pushID('Structure', {});
}

sub end_paket {
    my $self = shift;
    my $hPaket = $self->popID('Structure');
    $self->topID('Structure')->{'Paket'} = $hPaket;
}

sub start_gewichtKgMax {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_gewichtKgMax {
    my $self = shift;
    $self->topID('Structure')->{'gewichtKgMax'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_gewichtKgMin {
    my $self = shift;
    my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_gewichtKgMin {
    my $self = shift;
    $self->topID('Structure')->{'gewichtKgMin'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_laengeCmMax {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_laengeCmMax {
    my $self = shift;
    $self->topID('Structure')->{'laengeCmMax'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_laengeCmMin {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_laengeCmMin {
    my $self = shift;
    $self->topID('Structure')->{'laengeCmMin'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_breiteCmMax {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_breiteCmMax {
    my $self = shift;
    $self->topID('Structure')->{'breiteCmMax'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_breiteCmMin {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_breiteCmMin {
    my $self = shift;
    $self->topID('Structure')->{'breiteCmMin'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_hoeheCmMin {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_hoeheCmMin {
    my $self = shift;
    $self->topID('Structure')->{'hoeheCmMin'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_hoeheCmMax {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_hoeheCmMax {
    my $self = shift;
    $self->topID('Structure')->{'hoeheCmMax'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_laengeBreiteHoeheCmMin {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_laengeBreiteHoeheCmMin {
    my $self = shift;
    $self->topID('Structure')->{'laengeBreiteHoeheCmMin'} = $self->getCharacters;
    $self->collectCharacters(0);
}

sub start_laengeBreiteHoeheCmMax {
    my $self = shift;
    # my ($el) = @_;
    $self->collectCharacters(1);
}

sub end_laengeBreiteHoeheCmMax {
    my $self = shift;
    $self->topID('Structure')->{'laengeBreiteHoeheCmMax'} = $self->getCharacters;
    $self->collectCharacters(0);
}


sub _ConvertStringToDateTimeUTC {
    my ($UnconvertedString) = @_;

    my $DateTime;
    eval {
        my $formatter = DateTime::Format::ISO8601->new();
        $DateTime = $formatter->parse_datetime($UnconvertedString);
    };
    if (ExistsError()) {
        my $Error = GetError();
        GetLog()->warn('Could not convert String to dateTime: '.$Error->message);
    }
    LogDebug('DHL _ConvertStringToDateTimeUTC DateTimeObject: ',$DateTime->ymd.' '.
        $DateTime->hms.' '.$DateTime->time_zone->name);
    return $DateTime;
}


sub _ImportDHLProductList {
    my $self = shift;
    my ($hProduct) = @_;

    #IMPORT THE DHL PRODUCT INTO THE DATABASE
    my $DHLProductListID = _ImportDHLProduct($self, $hProduct);

    #IMPORT THE PRODUCT DELIVERY DESTINATIONS INTO THE DATABASE
    _ImportDHLProductDeliveryDestinations($self, $hProduct, $DHLProductListID);

    #IMPORT THE PRODUCT ATTRIBUTES INTO THE DATABASE
    _ImportDHLProductAttributes($self, $hProduct, $DHLProductListID);

    return;
}


sub _ImportDHLProduct {
    my $self = shift;
    my ($hProduct) = @_;

    #IMPORT THE DHL PRODUCT INTO THE DATABASE

    #Parse the DateString and get a DateTimeObject
    my $ValidFrom = $hProduct->{'gueltigVon'};
    if ($ValidFrom eq '') {
            $ValidFrom = undef;
    }
    else {
        $ValidFrom = _ConvertStringToDateTimeUTC($ValidFrom);
    }
    my $ValidTo = $hProduct->{'gueltigBis'};
    if ($ValidTo eq '') {
        $ValidTo = undef;
    }
    else {
        $ValidTo = _ConvertStringToDateTimeUTC($ValidTo);
    }

    # Replace the floatingpoint komma with a dot
    # Get the value of WeightKgMin and WeightKgMax
    my $WeightKgMin = $hProduct->{'Paket'}->{'gewichtKgMin'};
    if ($WeightKgMin eq '') {
        $WeightKgMin = undef;
    }
    else {
        $WeightKgMin =~ s/,/./;
    }
    my $WeightKgMax = $hProduct->{'Paket'}->{'gewichtKgMax'};
    if ($WeightKgMax eq '') {
        $WeightKgMax = undef;
    }
    else {
        $WeightKgMax =~ s/,/./;
    }

    # Get the value of LengthCmMin and LengthCmMax
    my $LengthCmMin = $hProduct->{'Paket'}->{'laengeCmMin'};
    if ($LengthCmMin eq '') {
        $LengthCmMin = undef;
    }
    else {
        $LengthCmMin =~ s/,/./;
    }
    my $LengthCmMax = $hProduct->{'Paket'}->{'laengeCmMax'};
    if ($LengthCmMax eq '') {
        $LengthCmMax = undef;
    }
    else {
        $LengthCmMax =~ s/,/./;
    }

    # Get the value of WidthCmMin and WidthCmMax
    my $WidthCmMin  = $hProduct->{'Paket'}->{'breiteCmMin'};
    if ($WidthCmMin eq '') {
        $WidthCmMin = undef;
    }
    else {
        $WidthCmMin =~ s/,/./;
    }
    my $WidthCmMax  = $hProduct->{'Paket'}->{'breiteCmMax'};
    if ($WidthCmMax eq '') {
        $WidthCmMax = undef;
    }
    else {
        $WidthCmMax =~ s/,/./;
    }

    # Get the value of HeightCmMin and HeightCmMax
    my $HeightCmMin = $hProduct->{'Paket'}->{'hoeheCmMin'};
    if ($HeightCmMin eq '') {
        $HeightCmMin = undef;
    }
    else {
        $HeightCmMin =~ s/,/./;
    }
    my $HeightCmMax = $hProduct->{'Paket'}->{'hoeheCmMax'};
    if ($HeightCmMax eq '') {
        $HeightCmMax = undef;
    }
    else {
        $HeightCmMax =~ s/,/./;
    }

    # Get the value of LengthWidthHeightCmMin and LengthWidthHeightCmMax
    my $LengthWidthHeightCmMin = $hProduct->{'Paket'}->{'laengeBreiteHoeheCmMin'};
    if ($LengthWidthHeightCmMin eq '') {
        $LengthWidthHeightCmMin = undef;
    }
    else {
        $LengthWidthHeightCmMin =~ s/,/./;
    }
    my $LengthWidthHeightCmMax = $hProduct->{'Paket'}->{'laengeBreiteHoeheCmMax'};
    if ($LengthWidthHeightCmMax eq '') {
        $LengthWidthHeightCmMax = undef;
    }
    else {
        $LengthWidthHeightCmMax =~ s/,/./;
    }

    my %hParameter = (
        ProductID              => $hProduct->{'id'},
        Supplier               => $hProduct->{'anbieter'},
        DisplayName            => $hProduct->{'anzeigename'},
        DisplayText            => $hProduct->{'anzeigetext'},
        PictureURL             => $hProduct->{'bildUrl'},
        ValidFrom              => $ValidFrom,
        ValidTo                => $ValidTo,
        WeightKgMin            => $WeightKgMin,
        WeightKgMax            => $WeightKgMax,
        LengthCmMin            => $LengthCmMin,
        LengthCmMax            => $LengthCmMax,
        WidthCmMin             => $WidthCmMin,
        WidthCmMax             => $WidthCmMax,
        HeightCmMin            => $HeightCmMin,
        HeightCmMax            => $HeightCmMax,
        LengthWidthHeightCmMin => $LengthWidthHeightCmMin,
        LengthWidthHeightCmMax => $LengthWidthHeightCmMax,
        URL                    => $hProduct->{'url'},
        TOS                    => $hProduct->{'agb'},
        PrintAdvice            => $hProduct->{'druckhinweis'},
    );
    my $rhParameter = \%hParameter;

    my $DHLProductListID;

    print "\nInsert DHL product '$hProduct->{'id'}' ... " unless $self->{'NoOutput'};
    $DHLProductListID = InsertDHLProductList($rhParameter);
    print "done.\n" unless $self->{'NoOutput'};

    return $DHLProductListID;
}


sub _ImportDHLProductDeliveryDestinations {
    my $self = shift;
    my ($hProduct, $DHLProductListID) = @_;

    #IMPORT THE PRODUCT DELIVERY DESTINATIONS INTO THE DATABASE

    my $ahDeliveryDestinations = $hProduct->{'Lieferziele'};

    foreach my $hDeliveryDestination (@{$ahDeliveryDestinations}) {
        my $Exceptions = ''; # at the moment ignored

        my $OverallPriceAmountEur = $hDeliveryDestination->{'Price'}{'gesamtpreisBetragEur'};
        if ($OverallPriceAmountEur eq '') {
            $OverallPriceAmountEur = undef;
        }
        else {
            $OverallPriceAmountEur =~ s/,/./;
        }

        my $CN22Limit = $hDeliveryDestination->{'CN22Limit'};
        if ($CN22Limit eq '') {
            $CN22Limit = undef;
        }
        else {
            $CN22Limit =~ s/,/./;
        }
        my $Endorsement = $hDeliveryDestination->{'vorausverfuegung'};
        if ($Endorsement eq 'false') {
            $Endorsement = 0;
        }
        else {
            $Endorsement = 1;
        }

        my %hDeliveryDestinationParameter = (
            CountryCode             => $hDeliveryDestination->{'laendercode'},
            CurrencyID              => 'EUR',
            DHLProductListID        => $DHLProductListID,
            Exceptions              => $Exceptions,
            OverallPriceAmountEur   => $OverallPriceAmountEur,
            OverallPriceDisplayName => $hDeliveryDestination->{'Price'}{'gesamtpreisAnzeigename'},
            Declaration             => $hDeliveryDestination->{'zollerklaerung'},
            CN22Limit               => $CN22Limit,
            Endorsement             => $Endorsement,
        );

        my $rhDeliveryDestinationParameter = \%hDeliveryDestinationParameter;

        if ( ExistsDHLProdListDeliveryDestByDHLProductListID($DHLProductListID,$hDeliveryDestination->{'laendercode'}) ) {
            print "Update DHL delivery destination '$hDeliveryDestination->{'laendercode'}' " .
                "for DHL product '$DHLProductListID: $hProduct->{'id'}' ... " unless $self->{'NoOutput'};
            my $DHLProdListDeliveryDestID = GetPKeyDHLProdListDeliveryDestByDHLProductListID($DHLProductListID,
                $hDeliveryDestination->{'laendercode'});
            UpdateDHLProdListDeliveryDest($DHLProdListDeliveryDestID, $rhDeliveryDestinationParameter);
            print "done.\n" unless $self->{'NoOutput'};
        }
        else {
            print "Insert DHL delivery destination '$hDeliveryDestination->{'laendercode'}' for DHL product " .
                "'$DHLProductListID: $hProduct->{'id'}' ... " unless $self->{'NoOutput'};
            InsertDHLProdListDeliveryDest($rhDeliveryDestinationParameter);
            print "done.\n" unless $self->{'NoOutput'};
        }
    }

    return;
}


sub _ImportDHLProductAttributes {
    my $self = shift;
    my ($hProduct, $DHLProductListID) = @_;

    #IMPORT THE PRODUCT ATTRIBUTES INTO THE DATABASE
    my $ahAttributes = $hProduct->{'Merkmale'};

    foreach my $hAttribute (@{$ahAttributes}) {
        #Save the product atrributes
        my $MaxAmountEUR = $hAttribute->{'maximalbetragEur'};
        if ($MaxAmountEUR eq '') {
            $MaxAmountEUR = undef;
        }
        else {
            $MaxAmountEUR =~ s/,/./;
        }
        my %hAttributeParameter = (
            AttributeID       => $hAttribute->{'id'},
            Name              => $hAttribute->{'name'},
            DHLProductListID  => $DHLProductListID,
            MaxAmountEUR      => $MaxAmountEUR,
            CurrencyID        => 'EUR',
        );

        my $rhAttributeParameter = \%hAttributeParameter;

        if ( ExistsDHLProdListAttributeByDHLProductListID($DHLProductListID,$hAttribute->{'id'}) ) {
            print "Update DHL attribute '$hAttribute->{'name'}' for DHL product " .
                "'$DHLProductListID: $hProduct->{'id'}' ... " unless $self->{'NoOutput'};
            my $aDHLProdListAttributeID = GetPKeyDHLProdListAttributesByDHLProductListID($DHLProductListID,
                $hAttribute->{'id'});
            UpdateDHLProdListAttribute($aDHLProdListAttributeID->[0], $rhAttributeParameter);
            print "done.\n" unless $self->{'NoOutput'};
        }
        else {
            print "Insert DHL attribute '$hAttribute->{'name'}' for DHL product " .
                "'$DHLProductListID: $hProduct->{'id'}' ... " unless $self->{'NoOutput'};
            InsertDHLProdListAttribute($rhAttributeParameter);
            print "done.\n" unless $self->{'NoOutput'};
        }
    }

    return;
}


sub _DeleteDHLProductList {
    my $self = shift;
    print 'To clean the Database please run the script DeleteDHLProductList.pl' unless $self->{'NoOutput'};
}


1;

