#========================================================================================
# package      DE_EPAGES::Object::API::XML::DHLProductInfoHandler
# state        private
#----------------------------------------------------------------------------------------
# description  Imports DHL product list XMLs
# example      use DE_EPAGES::DHL::API::XML::DHLProductInfoHandler;
#========================================================================================
package DE_EPAGES::DHL::API::XML::DHLProductInfoHandler;
use base qw(Exporter);

use strict;

use XML::Simple;
use XML::LibXML;
use XML::Twig;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::DHL::API::Constants qw (
    BUYONLINE_ATTRIBUTE_ID
    BULKGOODS_ATTRIBUTE_ID
    PROOF_ATTRIBUTE_ID
    PICKUPSERVICE_ATTRIBUTE_ID
    GOGREEN_ATTRIBUTE_ID
    ROLEPACKAGE_ATTRIBUTE_ID
    INSURANCE_ATTRIBUTE_ID
    TRACKING_ATTRIBUTE_ID
);

our @EXPORT_OK = qw(ParseProductInfoXML);

#========================================================================================
# function     ParseProductInfoXML
#----------------------------------------------------------------------------------------
# syntax       $hProductInfo = $Package->ParseProductInfoXML(1, $XML);
#----------------------------------------------------------------------------------------
# description  Parses product infos as returned by the DHL ProductInfo web service. Returns
#               data that can be fed to DHLProductHandler::ParseProductData
#----------------------------------------------------------------------------------------
# input        $NoOuput      | if 1, no output will be printed | boolean
# input        $XML          | XML data fetched from the DHL web service | string
# return       $hProductInfo | parsed data to be used with DHLProductHandler::ParseProductData | ref.hash
#========================================================================================
sub ParseProductInfoXML {
    my ($NoOuput, $XML) = @_;

    $XML = _removeNamespacesFromXMLString($XML);
    my $hInfo = XML::Simple->new()->XMLin($XML,
        # make XML element attribute 'name' a key attribute for the resulting hash
        KeyAttr => '+name',
        # force these elements to always be parsed into an array, regardless of their count
        ForceArray => [qw(ProductGroup CountryProducts BaseProduct Product Prices Service)]
    );

    my $aProductGroups = $hInfo->{'Body'}
        ->{'ProductInfoResponse'}
        ->{'MarketplaceProducts'}
        ->{'ProductGroup'};

    my $hRefs = {};
    my $hOffers;
    my $hCollections;

    my $ParserTable = {
        'COLLECTION' => sub {
            my ($hGroup) = @_;
            $hCollections = _ParseCollectionProductGroup($hGroup, $hRefs);
        },

        'STANDARD'   => sub {
            my ($hGroup) = @_;
            $hOffers = _ParseStandardProductGroup($hGroup, $hRefs);
        },
    };
    my $ParseOrder = [qw(COLLECTION STANDARD)];

    foreach my $GroupName (@$ParseOrder) {
        foreach my $hProductGroup (@$aProductGroups) {
            next unless($hProductGroup->{'name'} eq $GroupName);
            $ParserTable->{$GroupName}->($hProductGroup);
        }
    }

    # the source xml contains references to internal nodes, but the docs do not define
    # whether forward references are allowed or not. we'll resolve all references in
    # a separate pass, just to be sure
    _ResolveProductReferences($hOffers, $hRefs);
    _ResolveCollectionsReferences($hCollections, $hRefs);

    return _GenerateImportStructure($hOffers, $hCollections);
}

sub _ParseCollectionProductGroup {
    my ($hCollectionGroup, $hRefs) = @_;
    my $hCollections;
    my $aCollections = $hCollectionGroup->{'Collection'};
    foreach my $hCollection (@$aCollections) {
        $hCollections->{$hCollection->{'name'}} = _ParseCollection($hCollection, $hRefs);
    }

    return $hCollections;
}

sub _ParseCollection {
    my ($hCollection, $hRefs) = @_;
    my $hResult;

    return $hCollection->{'idref'} if(isValid($hCollection->{'idref'}));

    $hResult->{'description'} = _ParseCollectionDesc($hCollection->{'CollectionDesc'}, $hRefs)
        if(defined($hCollection->{'CollectionDesc'}));
    $hResult->{'prices'} = _ParsePrices($hCollection->{'Prices'}, $hRefs)
        if(defined($hCollection->{'Prices'}));

    return $hResult;
}

sub _ParseCollectionDesc {
    my ($hDesc) = @_;
    my $hResult = {
        'name' => $hDesc->{'name'},
    };

    return $hResult;
}

sub _ParseStandardProductGroup {
    my ($hProductGroup, $hRefs) = @_;
    my $hOffers;

    my $GroupDisplayText = $hProductGroup->{'displayText'};
    my $aCountryProducts = $hProductGroup->{'CountryProducts'};

    foreach my $hCountryProduct (@$aCountryProducts) {
        my $CountryName = $hCountryProduct->{'name'};
        my $RefID = $hCountryProduct->{'idref'};
        if(isValid($RefID)) {
            $hOffers->{$CountryName} = $RefID;
        } else {
            my $NodeID = $hCountryProduct->{'id'};
            my $hProduct = _ParseCountryProduct($hCountryProduct, $hRefs);
            if(defined($hProduct)) {
                $hProduct->{'displaytext'} = $GroupDisplayText;
                Error('MultipleCountryDefinition_ParseProductInfoXML', $CountryName)
                    if(defined($hOffers->{$CountryName}));

                $hOffers->{$CountryName} = $hProduct;
                _StoreRef($NodeID, $hProduct, $hRefs) if(defined($NodeID));
            }
        }
    }

    return $hOffers;
}

sub _ResolveCollectionsReferences {
    my ($hCollections, $hRefs) = @_;

    foreach my $Collection (keys %$hCollections) {
        my $CollectionRef = $hCollections->{$Collection};
        # if $CollectionRef is not a reference, it's a node id that needs to be resolved
        $hCollections->{$Collection} = _ResolveRef($CollectionRef, $hRefs) unless(ref($CollectionRef));
    }

    foreach my $Collection (keys %$hCollections) {
        my $hCollection = $hCollections->{$Collection};
        my $aCollectionPrices = $hCollection->{'prices'};
        _ResolveArrayRefs($aCollectionPrices, $hRefs);
    }

    return;
}

# resolve idref references in products
sub _ResolveProductReferences {
    my ($hProducts, $hRefs) = @_;

    # $hRefs is a hash of id/value pairs, containing all objects that can be referenced.
    # a reference is a string that contains the id of another element.
    # loop over all referenceable elements; if it's a scalar, it needs to be resolved.
    foreach my $Country (keys %$hProducts) {
        my $CountryRef = $hProducts->{$Country};
        $hProducts->{$Country} = _ResolveRef($CountryRef, $hRefs) unless(ref($CountryRef));

        # resolve the baseproducts assigned to a delivery destination
        my $aCountryProducts = $hProducts->{$Country}->{'products'};
        _ResolveArrayRefs($aCountryProducts, $hRefs);

        # resolve the subproducts of all baseproducts
        foreach my $hCountryOffer (@$aCountryProducts) {
            my $hSubProducts = $hCountryOffer->{'subproducts'};
            foreach my $ProductID (keys %$hSubProducts) {
                my $SubProductRef = $hSubProducts->{$ProductID};
                unless(ref($SubProductRef)) {
                    $SubProductRef = _ResolveRef($SubProductRef, $hRefs);
                    $hSubProducts->{$ProductID} = $SubProductRef;
                }

                my $DescriptionRef = $SubProductRef->{'description'};
                $SubProductRef->{'description'} = _ResolveRef($DescriptionRef, $hRefs) unless(ref($DescriptionRef));
            }
        }

        # resolve child hashes of now resolved products
        foreach my $hCountryOffer (@$aCountryProducts) {
            my $aOfferPrices = $hCountryOffer->{'prices'};
            _ResolveArrayRefs($aOfferPrices, $hRefs);

            my $hSubProducts = $hCountryOffer->{'subproducts'};
            foreach my $ProductID (keys %$hSubProducts) {
                my $ProductRef = $hSubProducts->{$ProductID};
                $ProductRef = $hSubProducts->{$ProductID} = _ResolveRef($ProductRef, $hRefs) unless(ref($ProductRef));

                my $aProductServices = $ProductRef->{'services'};
                _ResolveArrayRefs($aProductServices, $hRefs);
                if(defined($aProductServices) && scalar(@$aProductServices)) {
                    for(0 .. scalar(@$aProductServices) - 1) {
                        my $aServicePrices = $aProductServices->[$_]->{'prices'};
                        _ResolveArrayRefs($aServicePrices, $hRefs) if isValid($aServicePrices);
                    }
                }

                my $aProductPrices = $ProductRef->{'prices'};
                _ResolveArrayRefs($aProductPrices, $hRefs) if isValid($aProductPrices);
            }
            my $DescriptionRef = $hCountryOffer->{'description'};
            $hCountryOffer->{'description'} = $hRefs->{$DescriptionRef} unless(ref($DescriptionRef));
        }
    }

    return;
}

# resolve an array of references
sub _ResolveArrayRefs {
    my ($aArray, $hRefs) = @_;

    return unless defined $aArray;

    foreach my $Ref (@$aArray) {
        $Ref = _ResolveRef($Ref, $hRefs) unless ref $Ref;
    }

    return;
}

# resolve a single reference
sub _ResolveRef {
    my ($ID, $hRefs) = @_;
    my $Result = $hRefs->{$ID};
    Error('UndefinedReference_ResolveRef', $ID) unless(defined($Result));
    return $Result;
}

# store a reference
sub _StoreRef {
    my ($ID, $Data, $hRefs) = @_;
    Error('MultipleIDDefinitions_StoreRef', $ID) if(defined($hRefs->{$ID}));
    Error('UndefinedData_StoreRef', $ID) unless(defined($Data));

    $hRefs->{$ID} = $Data;
}

sub _ParseCountryProduct {
    my ($hCountryProduct, $hRefs) = @_;

    my $aBaseProducts = _ParseBaseProducts($hCountryProduct->{'BaseProduct'}, $hRefs);

    my $hResult = {
        'name'     => $hCountryProduct->{'name'},
        'products' => $aBaseProducts,
    };

    return $hResult;
}

sub _ParseBaseProducts {
    my ($aBaseProducts, $hRefs) = @_;

    my $aResult;
    push(@$aResult, _ParseBaseProduct($_, $hRefs)) foreach (@$aBaseProducts);

    return $aResult;
}

sub _ParseBaseProduct {
    my ($hBaseProduct, $hRefs) = @_;
    my $hResult;

    return $hBaseProduct->{'idref'} if(defined($hBaseProduct->{'idref'}));

    $hResult->{'name'} = $hBaseProduct->{'name'};
    $hResult->{'description'} = _ParseProductDesc($hBaseProduct->{'BaseProductDesc'}, $hRefs);
    my $aBasePrices = _ParsePrices($hBaseProduct->{'Prices'}, $hRefs);
    $hResult->{'prices'} = $aBasePrices if(defined($aBasePrices));

    my $aSourceProducts = $hBaseProduct->{'Product'};
    my $aProducts = _ParseProducts($aSourceProducts, $hBaseProduct, $hRefs);
    $hResult->{'subproducts'}->{$_->{'name'}} = $_ foreach(@$aProducts);

    _StoreRef($hBaseProduct->{'id'}, $hResult, $hRefs) if(defined($hBaseProduct->{'id'}));

    return $hResult;
}

sub _ParseProducts {
    my ($aProducts, $hBaseProduct, $hRefs) = @_;

    my $aResult;
    foreach my $hProductInfo (@$aProducts) {
        my $hResult = _ParseProduct($hProductInfo, $hBaseProduct, $hRefs);
        push(@$aResult, $hResult) if(defined($hResult));
    }

    return $aResult;
}

sub _ParseProduct {
    my ($hProduct, $hBaseProduct, $hRefs) = @_;

    my $hParsers = {
        'ProductDesc' => {
            'name' => 'description',
            'sub'  => \&_ParseProductDesc,
        },

        'Service' => {
            'name'  => 'services',
            'sub'   => \&_ParseServices,
        },

        'Prices' => {
            'name'  => 'prices',
            'sub'   => \&_ParsePrices,
        }
    };

    return $hProduct->{'idref'} if(defined($hProduct->{'idref'}));

    my $hResult = {
        'name'       => $hProduct->{'name'},
        'collection' => $hProduct->{'collectionServices'},
    };

    foreach my $NodeType (keys %$hProduct) {
        my $hParser = $hParsers->{$NodeType};
        next unless(defined($hParser));

        my $hNode = $hParser->{'sub'}->($hProduct->{$NodeType}, $hRefs);
        next unless(defined($hNode));

        $hResult->{$hParser->{'name'}} = $hNode;
    }

    # assume BuyOnline is always possible, and available for free
    # in the old product list there were flags for that on each product
    my $aServices = $hResult->{'services'} // [];
    push(@$aServices, { 'name' => 'BuyOnline', 'id' => BUYONLINE_ATTRIBUTE_ID });

    $hResult->{'services'} = $aServices;

    _StoreRef($hResult->{'id'}, $hResult, $hRefs) if(defined($hResult->{'id'}));

    return $hResult;
}

sub _ParseProductDesc {
    my ($hProductDesc, $hRefs) = @_;

    return $hProductDesc->{'idref'} if(defined($hProductDesc->{'idref'}));

    my $hResult;
    foreach my $Field (qw(minWidth maxWidth minLength maxLength minDepth maxDepth minWeight maxWeight name
        displayName displayDescription attributes)) {
        $hResult->{$Field} = $hProductDesc->{$Field} if(defined($hProductDesc->{$Field}));
    }

    my $NodeID = $hProductDesc->{'id'};
    _StoreRef($NodeID, $hResult, $hRefs) if(defined($NodeID));

    return $hResult;
}

sub _ParseServices {
    my ($aServices, $hRefs) = @_;

    my $aResult;
    foreach my $hService (@$aServices) {
        my $Service = _ParseService($hService, $hRefs);
        push(@$aResult, $Service) if(defined($Service));
    }

    return $aResult;
}

sub _ParseService {
    my ($hService, $hRefs) = @_;

    # use regexp matching here, because the service names may include variations like SPERR10, SPERR60 etc.
    my $ServiceAttributeMap = {
        '^SPERR'    => { 'id' => BULKGOODS_ATTRIBUTE_ID,   'name' => 'Sperrgut' },
        '^VERSI'    => { 'id' => INSURANCE_ATTRIBUTE_ID,   'name' => 'Haftung'    },
        '^GRUENEU'  => { 'id' => GOGREEN_ATTRIBUTE_ID,     'name' => 'GOGREEN'  },
        '^ROLLE'    => { 'id' => ROLEPACKAGE_ATTRIBUTE_ID, 'name' => 'ROLLE'    },
        '^RECOMAND' => { 'id' => PROOF_ATTRIBUTE_ID,       'name' => 'Ablieferquittung' }
    };

    my $hResult;
    my $ServiceName = $hService->{'name'};
    foreach my $AttrName (keys %$ServiceAttributeMap) {
        if($ServiceName =~ /$AttrName/) {
            my $ServiceAttr = $ServiceAttributeMap->{$AttrName};
            $hResult->{'id'} = $ServiceAttr->{'id'};
            $hResult->{'name'} = $ServiceAttr->{'name'};
            if(defined($hService->{'Prices'})) {
                my $aServicePrices = _ParsePrices($hService->{'Prices'}, $hRefs) ;
                $hResult->{'prices'} = $aServicePrices if(defined($aServicePrices));
            }
            last;
        }
    }

    Error('UnknownService_ParseService', $ServiceName) unless(defined($hResult));
    return $hResult;
}

sub _ParsePrices {
    my ($aPrices, $hRefs) = @_;

    # currently only a single price entry is supported
    Error('MultiplePriceDefinitions', 'Unsupported number of price entries found. Expected 1, found ' .
        scalar(@$aPrices)) if(scalar(@$aPrices) != 1);

    my $aResult = [];
    foreach my $hPriceEntry (@$aPrices) {
        my $hPrice = _ParsePrice($hPriceEntry->{'Price'}, $hRefs);
        push(@$aResult, $hPrice) if(defined($hPrice));
    }

    return $aResult;
}

sub _ParsePrice {
    my ($hPrice, $hRefs) = @_;

    return $hPrice->{'idref'} if(defined($hPrice->{'idref'}));

    my $hResult;
    my $hFields = {
        'name'        => 'name',
        'country'     => 'laendercode',
    };

    foreach my $Field (keys %$hFields) {
        $hResult->{$hFields->{$Field}} = $hPrice->{$Field} if(defined($hPrice->{$Field}));
    }

    $hResult->{'Price'} = { 'gesamtpreisBetragEur' => $hPrice->{'bruttoPrice'} };
    $hResult->{'laendercode'} = 'ALL' unless(defined($hResult->{'laendercode'}));

    if(defined($hResult)) {
        my $NodeID = $hPrice->{'id'};
        _StoreRef($NodeID, $hResult, $hRefs) if(defined($NodeID));
    }

    return $hResult;
}

sub _ClonePrice {
    my ($hPrice) = @_;
    my $hResult = {
        'laendercode' => $hPrice->{'laendercode'},
        'Price'       => { 'gesamtpreisBetragEur' => $hPrice->{'Price'}->{'gesamtpreisBetragEur'} },
        'name'        => $hPrice->{'name'},
    };

    return $hResult;
}

sub _CloneServices {
    my ($aServices) = @_;
    my $aResult;

    foreach my $hService (@$aServices) {
        my $hClone = {
            'name'   => $hService->{'name'},
            'id'     => $hService->{'id'},
        };

        my $aPrices = $hService->{'prices'};
        if(defined($aPrices)) {
            my $aDestPrices;
            push(@$aDestPrices, _ClonePrice($_)) foreach (@$aPrices);
            $hClone->{'prices'} = $aPrices;
        }
        push(@$aResult, $hClone);
    }

    return $aResult;
}

# clones a product and adds a collection service
sub _CloneProductForCollection {
    my ($hProduct, $hCollections) = @_;
    my $hResult;

    # currently ignoring all collection services but 'TAS' because the UI does not
    # yet provide a collection service selection
    if($hProduct->{'collection'} =~ /TAS/) {
        my $Collection = $hCollections->{'TAS'};
        return unless(defined($Collection));

        my $aCollectionPrices = $Collection->{'prices'};
        my $aProdServices = _CloneServices($hProduct->{'services'});
        push(@$aProdServices, {
            'id'     => PICKUPSERVICE_ATTRIBUTE_ID,
            'name'   => 'Abholung',
            'prices' => [_ClonePrice($aCollectionPrices->[0])],
        });

        $hResult->{'services'} = $aProdServices;
        $hResult->{$_} = $hProduct->{$_} foreach (qw(name collection description));
        $hResult->{'name'} .= '.TAS';
    }

    return $hResult;
}

sub _GenerateImportStructure {
    my ($hProducts, $hCollections) = @_;
    my $hDestProducts = {};
    my $hDestinations = {};

    foreach my $DestinationID (keys %$hProducts) {
        my $aDestinations = [];
        my $aBaseProductList = $hProducts->{$DestinationID}->{'products'};
        foreach my $hBaseProduct (@$aBaseProductList) {
            #my $hBaseDesc = $hBaseProduct->{'description'};
            my $aBasePrices = $hBaseProduct->{'prices'};

            my $hProductList = $hBaseProduct->{'subproducts'};
            foreach my $ProductID (keys %$hProductList) {
                my $hProduct = $hProductList->{$ProductID};
                my $hDestProduct = _ImportProduct($hProduct, $aBasePrices, $hDestProducts, $aDestinations);
                next unless(defined($hDestProduct));
                $hDestProducts->{$hDestProduct->{'id'}} = $hDestProduct;
                # make distinct products from those that have 'TAS' listed as a collection service
                if($hProduct->{'collection'} =~ /TAS/) {
                    my $hClonedProduct = _CloneProductForCollection($hProduct, $hCollections);
                    $hClonedProduct = _ImportProduct($hClonedProduct, $aBasePrices, $hDestProducts, $aDestinations);
                    $hDestProducts->{$hClonedProduct->{'id'}} = $hClonedProduct if(defined($hClonedProduct));
                }

                # resolve tracking attribute
                my $aProductServices = $hProduct->{'services'};
                my $Attr = $hProduct->{'description'}{'attributes'} // $hBaseProduct->{'description'}{'attributes'};
                push(@$aProductServices, { 'name' => 'Tracking', 'id'  => TRACKING_ATTRIBUTE_ID })
                    if(defined($Attr) && $Attr =~ /TrackingType=NOLB/);

            }
        }

        $hDestinations->{$DestinationID} = $aDestinations;
    }

    my $hResult = {
        'destinations' => $hDestinations,
        'products' => $hDestProducts,
    };

    return $hResult;
}

sub _ImportProduct {
    my ($hProduct, $aBasePrices, $hDestProducts, $aDestinations) = @_;
    use constant KG => 1000;

    # clone base product prices, store in product
    my $aPrices = [];
    push(@$aPrices, _ClonePrice($_)) foreach(@$aBasePrices);

    my $hDesc = $hProduct->{'description'};
    my $ProductName = $hProduct->{'name'};
    push(@$aDestinations, $ProductName) unless ElementOf($ProductName, $aDestinations);
    return if(defined($hDestProducts->{$ProductName}));

    my $hDestPaket = {
        'gewichtKgMin' => $hDesc->{'minWeight'} / KG,
        'gewichtKgMax' => $hDesc->{'maxWeight'} / KG,
        'laengeCmMin'  => $hDesc->{'minLength'},
        'laengeCmMax'  => $hDesc->{'maxLength'},
        'breiteCmMin'  => $hDesc->{'minWidth'},
        'breiteCmMax'  => $hDesc->{'maxWidth'},
        'hoeheCmMin'   => $hDesc->{'minDepth'},
        'hoeheCmMax'   => $hDesc->{'maxDepth'},
    };

    my $hDestProduct = {
        'id'          => $ProductName,
        'anzeigename' => $hDesc->{'displayName'},
        'anzeigetext' => $hDesc->{'displayText'},
        'Paket'       => $hDestPaket
    };

    if(defined($hProduct->{'services'})) {
        _AddServicePricesToProduct($hProduct, $aPrices);
        $hDestProduct->{'Merkmale'} = $hProduct->{'services'};
    }

    $hDestProduct->{'Lieferziele'} = $aPrices;
    return $hDestProduct;
}

# add prices of services to a product
sub _AddServicePricesToProduct {
    my ($hProduct, $aPrices) = @_;
    my $aServices = $hProduct->{'services'};
    foreach my $hService (@$aServices) {
        my $aServicePrices = $hService->{'prices'};
        next unless(defined($aServicePrices));
        foreach my $hServicePrice (@$aServicePrices) {
            next unless(defined($hServicePrice->{'Price'}) &&
                defined($hServicePrice->{'Price'}->{'gesamtpreisBetragEur'}));
            my $ServicePrice = $hServicePrice->{'Price'}->{'gesamtpreisBetragEur'};
            foreach (@$aPrices) {
                next unless(defined($_->{'Price'}) && defined($_->{'Price'}->{'gesamtpreisBetragEur'}));
                $_->{'Price'}->{'gesamtpreisBetragEur'} += $ServicePrice;
            }
        }
    }

    return;
}

sub _removeNamespacesFromXMLString {
    my ($XMLString) = @_;

    return XML::Twig->new( start_tag_handlers => {
                            _all_ => sub {
                                    my $Tag= $_->tag;
                                    $Tag=~ s{^[^:]*:}{};
                                    $_->set_tag( $Tag );
                                }
                            },
                            keep_spaces => 1)
         ->parse( $XMLString )
         ->toString();
}

1;
