#========================================================================================
# package      DE_EPAGES::DHL::DAL::SQL::DHLProducts
# state        private
# description  standard SQL database interface for DHL products
#========================================================================================
package DE_EPAGES::DHL::DAL::SQL::DHLProducts;

use strict;
use DE_EPAGES::Core::API::PerlTools qw (ElementOf GetLoopFromDoubleArray);

#========================================================================================
# function     GetRecentCategories
#----------------------------------------------------------------------------------------
# syntax       $rahRecentCategories = $Package->GetRecentCategories($dbi,$Length,$Width,$Height,$LengthWidthHeight,$Weight,$CountryCode,$AttributeID);
#----------------------------------------------------------------------------------------
# description  return all recently used categories order by usuage
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# input        $Length            | Length of the delivery | float
# input        $Width             | Width of the delivery | float
# input        $Height            | Height of the delivery | float
# input        $LengthWidthHeight | Lenght+width+height of the delivery | float
# input        $Weight            | Weight of the delivery | float
# input        $CountryCode       | Countrycode of the delivery destination | ISO_3
# input        $AttributeID       | Attribute ID of the DHLProduct | integer
# return       $rahDHLProducts    | with following keys:
#               <ul>
#                   <li>DHLProductID  - DHL Product id  - String
#               </ul>| reference to array of hashes
#========================================================================================
sub GetDHLProducts {
    my $self = shift;
    my ($dbi, $Length, $Width, $Height, $LengthWidthHeight, $Weight, $CountryCode, $AttributeID) = @_;

    my $dbResult = $dbi->execute('select p.productid, p.dhlproductlistid, p.validfrom, p.validto, d.countrycode, d.overallpriceamounteur, p.displayname, p.displaytext '.
                                 'from dhlproductlist p '.
                                       'inner join dhlprodlistdeliverydest d on p.dhlproductlistid = d.dhlproductlistid '.
                                       'inner join dhlprodlistattribute a on p.dhlproductlistid = a.dhlproductlistid '.
                                 'where p.weightkgmin <= ? AND p.weightkgmax >= ? AND '.
                                       'p.lengthcmmin <= ? AND p.lengthcmmax >= ? AND '.
                                       'p.widthcmmin <= ? AND (p.widthcmmax >= ? OR p.widthcmmax is null) AND '.
                                       'p.heightcmmin <= ? AND (p.heightcmmax >= ? OR p.heightcmmax is null) AND '.
                                       '(p.lengthwidthheightcmmin <= ? OR p.lengthwidthheightcmmin is null) AND '.
                                       '(p.lengthwidthheightcmmax >= ? OR p.lengthwidthheightcmmax is null) AND '.
                                       'a.attributeid = ?'.
                                 'order by d.overallpriceamounteur asc',
                                 [$Weight, $Weight, $Length, $Length, $Width,  $Width,  $Height, $Height, $LengthWidthHeight, $LengthWidthHeight, $AttributeID],
                                 ['String', 'Integer', 'DateTime', 'DateTime', 'String', 'Float', 'String', 'String']);

    my $EUCountries = [qw(BEL BGR DNK DEU EST FIN FRA GRC IRL ITA LVA LTU LUX MLT NLD AUT POL PRT ROU SWE SVK SVN ESP CZE HUN GBR CYP)];
    my $aResult;
    foreach my $aRow (@$dbResult) {
        my $CtyCode = $aRow->[4];
        if($CtyCode eq 'ALL' || $CtyCode eq $CountryCode) {
            push(@$aResult, $aRow);
        }
        elsif($CtyCode eq 'EU') {
            push(@$aResult, $aRow) if ElementOf($CountryCode, $EUCountries);
        }
    }

    return GetLoopFromDoubleArray($aResult, [ qw(DHLProductID DHLProductListID ValidFrom ValidTo CountryCode OverallPriceAmountEUR DisplayName DisplayText) ]);
}


#========================================================================================
# function     GetMinShippingLength
#----------------------------------------------------------------------------------------
# syntax       $MinShippingLength = $Package->GetMinShippingLength();
#----------------------------------------------------------------------------------------
# description  Returns the minimal length for the delivery
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# return       $MinShippingLength  -  float
#========================================================================================
sub GetMinShippingLength {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute('select min(lengthcmmin) from dhlproductlist');

    return $dbResult->[0][0];
}


#========================================================================================
# function     GetMaxShippingLength
#----------------------------------------------------------------------------------------
# syntax       $MaxShippingLength = $Package->GetMaxShippingLength();
#----------------------------------------------------------------------------------------
# description  Returns the maximal length for the delivery
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# return       $MaxShippingLength  -  float
#========================================================================================
sub GetMaxShippingLength {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute('select max(lengthcmmax) from dhlproductlist');

    return $dbResult->[0][0];
}


#========================================================================================
# function     GetMinShippingWidth
#----------------------------------------------------------------------------------------
# syntax       $MinShippingWidth = $Package->GetMinShippingWidth();
#----------------------------------------------------------------------------------------
# description  Returns the minimal width for the delivery
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# return       $MinShippingWidth   -  float
#========================================================================================
sub GetMinShippingWidth {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute('select min(widthcmmin) from dhlproductlist');

    return $dbResult->[0][0];
}


#========================================================================================
# function     GetMaxShippingWidth
#----------------------------------------------------------------------------------------
# syntax       $MaxShippingWidth = $Package->GetMaxShippingWidth();
#----------------------------------------------------------------------------------------
# description  Returns the maximal width for the delivery
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# return       $MaxShippingWidth   -  float
#========================================================================================
sub GetMaxShippingWidth {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute('select max(widthcmmax) from dhlproductlist');

    return $dbResult->[0][0];
}


#========================================================================================
# function     GetMinShippingHeight
#----------------------------------------------------------------------------------------
# syntax       $MinShippingHeight = $Package->GetMinShippingHeight();
#----------------------------------------------------------------------------------------
# description  Returns the minimal height for the delivery
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# return       $MinShippingHeight  -  float
#========================================================================================
sub GetMinShippingHeight {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute('select min(heightcmmin) from dhlproductlist');

    return $dbResult->[0][0];
}


#========================================================================================
# function     GetMaxShippingHeight
#----------------------------------------------------------------------------------------
# syntax       $MaxShippingHeight = $Package->GetMaxShippingHeight();
#----------------------------------------------------------------------------------------
# description  Returns the maximal height for the delivery
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# return       $MaxShippingHeight  -  float
#========================================================================================
sub GetMaxShippingHeight {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute('select max(heightcmmax) from dhlproductlist');

    return $dbResult->[0][0];
}


#========================================================================================
# function     GetMinShippingWeight
#----------------------------------------------------------------------------------------
# syntax       $MinShippingWeight = $Package->GetMinShippingWeight();
#----------------------------------------------------------------------------------------
# description  Returns the minimal weight for the delivery
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# return       $MinShippingWeight  -  float
#========================================================================================
sub GetMinShippingWeight {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute('select min(weightkgmin) from dhlproductlist');

    return $dbResult->[0][0];
}


#========================================================================================
# function     GetMaxShippingWeight
#----------------------------------------------------------------------------------------
# syntax       $MaxShippingWeight = $Package->GetMaxShippingWeight();
#----------------------------------------------------------------------------------------
# description  Returns the maximal weight for the delivery
#----------------------------------------------------------------------------------------
# input        $dbh               | database handle | object
# return       $MaxShippingWeight  -  float
#========================================================================================
sub GetMaxShippingWeight {
    my $self = shift;
    my ($dbi) = @_;

    my $dbResult = $dbi->execute('select max(weightkgmax) from dhlproductlist');

    return $dbResult->[0][0];
}


1;
