#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationResponse
# state        public
# description  Merchant credentilas validator. Parse response from getManifest service
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationResponse;

use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

use DE_EPAGES::DHLIntraship::API::Constants qw (
        %DHLVERSENDEN_STATUS_CODES
        $GET_MANIFEST_OPERATION_IS_VALID
        $GET_MANIFEST_OPERATION_IS_UNVALID
        $GET_MANIFEST_OPERATION_IS_UNKNOWN
);

use strict;

#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;
    return $self;
}

#========================================================================================
# function     parseResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       parseResponse( $SOAPResponse );
#----------------------------------------------------------------------------------------
# description  From the XML response of the webservice retrives the most important
#               information, and stores it in an easy to access hash.
#----------------------------------------------------------------------------------------
# input        $SOAPResponse   | SOAP response, must contain the XML the parse | Object
# return       $ReturnVariable | Hash with the relevant values of the response | refhash
#========================================================================================
sub parseResponse {
    my $self = shift;
    my ($SOAPResponse) = @_;

    if ( $SOAPResponse->{'Fault'} && $SOAPResponse->{'Fault'}->{'detail'} &&
         $SOAPResponse->{'Fault'}->{'detail'}->{'errorDetails'}) {
        $self->_Warning( 'DHL_RESPONSE_FAULT', {
            'FaultCode' => $SOAPResponse->{'Fault'}->{'detail'}->{'errorDetails'}->{'code'},
            'FaulMessage' => $SOAPResponse->{'Fault'}->{'detail'}->{'errorDetails'}->{'description'},
            'FaultReason' => $SOAPResponse->{'Fault'}->{'detail'}->{'errorDetails'}->{'description'},
        });
    }

    my $SOAPResponseInfo = $SOAPResponse->{'part1'};
    return $self->_parseManifest( $SOAPResponseInfo );
}

#========================================================================================
# function     _parseManifest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hParsedManifest = $self->_parseManifest( $SOAPResponseInfo );
#----------------------------------------------------------------------------------------
# description  It converts SOAP response body hash into another hash with key data only.
#----------------------------------------------------------------------------------------
# input        $SOAPResponseInfo | SOAP response hash | ref.hash
# return       $hParsedManifest | key response data hash | ref.hash
#========================================================================================
sub _parseManifest {
    my $self = shift;
    my ($SOAPResponseInfo) = @_;

    my $StatusCode;
    my $StatusText;
    my $hResponse = { 'login' => $GET_MANIFEST_OPERATION_IS_UNKNOWN };

    if ( defined($SOAPResponseInfo->{'Status'}) ) {
        $StatusCode = $SOAPResponseInfo->{'Status'}->{'statusCode'};
        $StatusText = $SOAPResponseInfo->{'Status'}->{'statusText'};
    } else {
        return $hResponse;
    }

    $hResponse = {
        'statusCode' => $StatusCode,
        'statusText' => $StatusText
    };

    if (ElementOf($StatusCode, $DHLVERSENDEN_STATUS_CODES{'valid_login'})) {
        $hResponse->{'login'} = $GET_MANIFEST_OPERATION_IS_VALID;
    } elsif (ElementOf($StatusCode, $DHLVERSENDEN_STATUS_CODES{'unvalid_login'})) {
        $hResponse->{'login'} = $GET_MANIFEST_OPERATION_IS_UNVALID;
    }

    return $hResponse;
}

#========================================================================================
# Wrappers
#========================================================================================
sub _Warning {
    my $self = shift;
    my ($WarningTitle, $hWarning) = @_;

    Warning($WarningTitle, $hWarning);

    return;
}

1;
