#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::AddressValidation
# state        public
#----------------------------------------------------------------------------------------
# description  class for unit test realted to AddressValidation.pm
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::AddressValidation;
use base DE_EPAGES::Test::API::UnitTest::Test;

use strict;

use DE_EPAGES::DHLIntraship::API::AddressValidation;
use DE_EPAGES::DHLIntraship::API::Constants qw (
    MANDATORY_WARNING_KEYWORD
    MANDATORY_WARNING_STRING
    TOO_LONG_WARNING_STRING
    TOO_SHORT_WARNING_KEYWORD
    TOO_SHORT_WARNING_STRING
);
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Test::MockObject;
use Test::Exception;

use constant TEST_PHONE_TOO_LONG => '+40-00-123-456.6758-0344-22233356666';
use constant TEST_PACKSTATION_TOO_SHORT => '22';
use constant TEST_VALID_HOUSE_NUMBER => 209;
use constant TEST_INVALID_HOUSE_NUMBER => 123456789012;

#sub Main {
sub test {
    my $self = shift;
    $self->test_Address_IsNotValid();
    $self->test_Address_HasWarningStates_positive();
    $self->test_Address_HasWarningStates_negative_wrong_parameters();
    $self->test_Address_HasWarningStates_negative_because_extra_parameters();
    $self->test_Address_Processable();
    $self->test_Address_Not_Processable_House_Number_Too_Long();
    $self->test_Address_Not_Processable_FullName_Not_Set();
    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub test_Address_IsNotValid {
    my $self = shift;
    my $subTest = (caller(0))[3];
    $subTest =~ s,.*:,,;

    my $ShopAddress = undef;
    ## no critic ( ProhibitComplexRegexes )
    throws_ok( sub {
                        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress );
                   },
                   qr/PARAMETER_MISSING: Missing parameter AddressValidation needs valid address object in call to DE_EPAGES::DHLIntraship::API::AddressValidation::new/ms,
                   "$subTest - ShopAddress is undefined"
    );
    $ShopAddress = $self->_createMock_WrongObject();
    throws_ok( sub {
                        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress );
                   },
                   qr/WRONG_CLASS: AddressValidation needs valid address object is not an instance of DE_EPAGES::Address::API::Object::Address/ms,
                   "$subTest - ShopAddress is blessed object but not AddressObject"
    );
    ## use critic ( ProhibitComplexRegexes )
    $ShopAddress = $self->_createMock_AddressObject();
    isa_ok (
        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress ),
        'DE_EPAGES::DHLIntraship::API::AddressValidation',
        "$subTest - ShopAddress"
    );
    return;
}

sub test_Address_HasWarningStates_positive {
    my $self = shift;
    my $subTest = (caller(0))[3];
    $subTest =~ s,.*:,,;

    my $BillingAddress = $self->_createMock_AddressObject();

    my $ExtraAttributeDefinition = {};
    my $AddressValidation1 = DE_EPAGES::DHLIntraship::API::AddressValidation->new($BillingAddress, $ExtraAttributeDefinition);

    ok(!$AddressValidation1->hasWarningStates(), "$subTest - hasWarningStates: not (w/o extra attributes)");
    is (scalar @{$AddressValidation1->getStatesAsWarning()},0,"$subTest - getStatesAsWarning: empty");

    $ExtraAttributeDefinition = {
            'ExtraAttribute' => {'Mandatory'=> 0,'MinValue'=> 1, 'MaxValue'=>50},
            'ExtraAttributeMandatory' => {'Mandatory'=> 1,'MinValue'=> 1, 'MaxValue'=>50},
    };
    my $AddressValidation2 = DE_EPAGES::DHLIntraship::API::AddressValidation->new($BillingAddress, $ExtraAttributeDefinition);
    ok(!$AddressValidation2->hasWarningStates(), "$subTest - hasWarningStates: not (with extra attributes)");

    my $Found = $AddressValidation2->{'FirstName'}; my $Expected = 'Jon';
    is($Found, $Expected, "$subTest - $Found vs $Expected");
    $Found = $AddressValidation2->{'Zipcode'}; $Expected = '12345';
    is($Found, $Expected, "$subTest - $Found vs $Expected");
    $Found = $AddressValidation2->{'ExtraAttribute'}; $Expected = 'ExtraAttributeValue';
    is($Found, $Expected, "$subTest - $Found vs $Expected");
    $Found = $AddressValidation2->{'ExtraAttributeMandatory'}; $Expected = 'ExtraAttributeValueMandatory';
    is($Found, $Expected, "$subTest - $Found vs $Expected");
}

sub test_Address_HasWarningStates_negative_wrong_parameters {
    my $self = shift;
    my $subTest = (caller(0))[3];
    $subTest =~ s,.*:,,;

    my $BillingAddress = $self->_createMock_AddressObject(
            {
                'FullName' => '',
                'Phone'   => TEST_PHONE_TOO_LONG,
                'DHLPackstationId' => TEST_PACKSTATION_TOO_SHORT,
            }
    );
    my $ExtraAttributeDefinition = {};
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress, $ExtraAttributeDefinition );

    ok($AddressValidation->hasWarningStates(), "$subTest - hasWarningStates: standard fields do not fit spec");

    my $aWarningStates = [sort {$a->message() cmp $b->message()} @{$AddressValidation->getStatesAsWarning()}];
    is (scalar @$aWarningStates, 3, "$subTest - getStatesAsWarning: non-empty array");

    my $Found = $aWarningStates->[0]->code(); my $Expected = 'AddressValidationWarning';
    is($Found, $Expected, "$subTest - $Found vs $Expected");

    my $hWarning = $aWarningStates->[0]->vars();
    $Found = $hWarning->{'attribute'}; $Expected = 'DHLPackstationId';
    is($Found, $Expected, "$subTest - $Found vs $Expected");
    $Found = $hWarning->{'reason'}; $Expected = TOO_SHORT_WARNING_KEYWORD;
    is($Found, $Expected, "$subTest - $Found vs $Expected");

    my $aWarningVars = [sort { $a->{attribute} cmp $b->{attribute} } @{$AddressValidation->{'WarningStates'}}];
    is_deeply( $aWarningVars,
               [
                { 'attribute' => 'DHLPackstationId', 'reason' => TOO_SHORT_WARNING_STRING},
                { 'attribute' => 'FullName', 'reason' => MANDATORY_WARNING_STRING},
                { 'attribute' => 'Phone', 'reason' => TOO_LONG_WARNING_STRING},
               ],
               "$subTest - AddressValidation->{'WarningStates'}: non-empty array"
    );
}

sub test_Address_HasWarningStates_negative_because_extra_parameters {
    my $self = shift;
    my $subTest = (caller(0))[3];
    $subTest =~ s,.*:,,;

    my $BillingAddress = $self->_createMock_AddressObject();
    my $ExtraAttributeDefinition = {
            'StreetOnly' => { 'Mandatory'=> 1,'MinValue'=> 20, 'MaxValue'=> 50 },
            'Company'    => { 'Mandatory'=> 1,'MinValue'=> 1, 'MaxValue'=> 50 }
    };
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress, $ExtraAttributeDefinition );

    ok($AddressValidation->hasWarningStates(), "$subTest - hasWarningStates: extra fields do not fit spec");
    my $aWarningStates = [sort {$a->message() cmp $b->message()} @{$AddressValidation->getStatesAsWarning()}];

    my $Found = $aWarningStates->[0]->code(); my $Expected = 'AddressValidationWarning';
    is($Found, $Expected, "$subTest - $Found vs $Expected");

    is_deeply($aWarningStates->[0]->vars(),
            { 'attribute' => 'Company', 'reason' => MANDATORY_WARNING_KEYWORD},
            "$subTest - getStatesAsWarning->[0]->vars: MANDATORY_WARNING_KEYWORD"
    );
    is_deeply($aWarningStates->[1]->vars(),
            { 'attribute' => 'StreetOnly', 'reason' => TOO_SHORT_WARNING_KEYWORD},
            "$subTest - getStatesAsWarning->[1]->vars: TOO_SHORT_WARNING_KEYWORD"
    );
}

sub test_Address_Processable {
    my $self = shift;
    my $subTest = (caller(0))[3];
    $subTest =~ s,.*:,,;

    my $BillingAddress = $self->_createMock_AddressObject();
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress );
    my $aNotProcessableWarningStates = $AddressValidation->getNotProcessableWarningStates();

    is(scalar(@{$aNotProcessableWarningStates}), 0, "$subTest - getNotProcessableWarningStates: no warnings");
}

sub test_Address_Not_Processable_House_Number_Too_Long {
    my $self = shift;
    my $subTest = (caller(0))[3];
    $subTest =~ s,.*:,,;

    my $BillingAddress = $self->_createMock_AddressObject({'HouseNumber' => TEST_INVALID_HOUSE_NUMBER});
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress );
    my $aNotProcessableWarningStates = [sort @{$AddressValidation->getNotProcessableWarningStates()}];

    my $hFirstStatus = (sort {$a->message() cmp $b->message()} @$aNotProcessableWarningStates)[0];
    is_deeply($hFirstStatus,
            { 'attribute' => 'HouseNumber', 'reason' => TOO_LONG_WARNING_STRING },
            "$subTest - getNotProcessableWarningStates[0]: TOO_LONG_WARNING_STRING"
    );
}

sub test_Address_Not_Processable_FullName_Not_Set {
    my $self = shift;
    my $subTest = (caller(0))[3];
    $subTest =~ s,.*:,,;

    my $BillingAddress = $self->_createMock_AddressObject({'FullName' => ''});
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress );
    my $aNotProcessableWarningStates = [sort @{$AddressValidation->getNotProcessableWarningStates()}];

    my $hFirstStatus = (sort {$a->message() cmp $b->message()} @$aNotProcessableWarningStates)[0];
    is_deeply($hFirstStatus,
            { 'attribute' => 'FullName', 'reason' => MANDATORY_WARNING_STRING },
            "$subTest - getNotProcessableWarningStates[0]: MANDATORY_WARNING_STRING"
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createMock_AddressObject {
    my $self = shift;
    my ($hAtributeParams) = @_;

    $hAtributeParams //= {};
    my $hAtributes = {
        'AddressExtension'        => $hAtributeParams->{'AddressExtension'} // 'TestAddressExtension',
        'Alias'                   => $hAtributeParams->{'Alias'} // 'BillingAddress ',
        'City'                    => $hAtributeParams->{'City'} // 'Hamburg',
        'Company'                 => $hAtributeParams->{'Company'} // '',
        'Country'                 => $hAtributeParams->{'Country'} // {'Code2'=> 'DE'},
        'DHLPackstationId'        => $hAtributeParams->{'DHLPackstationId'} // '123',
        'DHLPostNumber'           => $hAtributeParams->{'DHLPostNumber'} // '0123456789',
        'EMail'                   => $hAtributeParams->{'EMail'} // 'test@epages.com',
        'ExtraAttribute'          => $hAtributeParams->{'ExtraAttribute'} // 'ExtraAttributeValue',
        'ExtraAttributeMandatory' => $hAtributeParams->{'ExtraAttributeMandatory'} // 'ExtraAttributeValueMandatory',
        'FirstName'               => $hAtributeParams->{'FirstName'} // 'Jon',
        'FullName'                => $hAtributeParams->{'FullName'} // 'FullNameText',
        'HouseNumber'             => $hAtributeParams->{'HouseNumber'} // TEST_VALID_HOUSE_NUMBER,
        'IsPickupAddress'         => $hAtributeParams->{'IsPickupAddress'} // '1',
        'LastName'                => $hAtributeParams->{'LastName'} // 'Doe',
        'Phone'                   => $hAtributeParams->{'Phone'} // '',
        'Street2'                 => $hAtributeParams->{'Street2'} // 'c/o 123',
        'StreetOnly'              => $hAtributeParams->{'StreetOnly'} // 'Pilatuspool',
        'Zipcode'                 => $hAtributeParams->{'Zipcode'} // '12345',
    };
    my $MockifyAddressObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Address::API::Object::Address',$hAtributes);

    return $MockifyAddressObject->getMockObject();
}

sub _createMock_WrongObject {
    my $self = shift;
    my ($hAtributes) = @_;

    $hAtributes //= {};
    my $MockifyObjectObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object',$hAtributes);

    return $MockifyObjectObject->getMockObject();
}

__PACKAGE__->runScript();
