#========================================================================================
# package      DE_EPAGES::Database::API::DBI
# state        public
# description  central database interface, is based on standard perl module DBI.
#               Use <function DE_EPAGES::Database::API::Connection::GetCurrentDBHandle> to
#               get the currrent connected DBI handle.
#========================================================================================
package DE_EPAGES::Database::API::DBI;

use strict;

use DBI;
use DateTime;
use Encode qw (decode);
use Scalar::Util qw (blessed);
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Log qw (GetLog LogDebug LogWarnings);
use DE_EPAGES::Core::API::Error qw (Error GetError ExistsError);
use DE_EPAGES::Core::API::PerlTools qw( ElementOf GetCurrentDateTime GetLocalTimeZone IsA );
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Core::API::Statistics qw( AddStatistic );
use DE_EPAGES::Core::API::String qw( DecodeStructure );
use DE_EPAGES::Core::API::TestParameter qw( TestArray TestString );

sub ErrorDefinition {
    return {
        'DBIPrepare'    => 'prepare failed : #message at #statement',
        'DBIExecute'    => 'execute failed : #message at #statement',
        'DBIDo'         => 'do failed : #message at #statement',
        'DBIConnect'    => 'login failed : at #source with user #user (#message)',
        'NO_DATETIME_OBJECT' => 'no datetime object #Value',
        'DATETIME_TOO_LARGE' => 'datetime is too large: #Value',
        'DATETIME_TOO_SMALL' => 'datetime is too small: #Value',
        'TABLE_CYCLE' => 'cant delete table cycle (#Tables).',
        'UNKNOWN_TYPE' => 'cant resolve type #Type (for value #Value).',
        'STRING_CONTAINS_0x0' => 'string can not include char 0x0.',
    }
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi = DE_EPAGES::Database::API::DBI->new( $DataSource, $User, $Password, $CacheKey );
# example      DE_EPAGES::Database::API::DBI->new( 'dbi:mysql:storedb', 'root', 'geheim', 'DAL' );
#----------------------------------------------------------------------------------------
# description  Creates a database connection using the supplied connection parameters.
#----------------------------------------------------------------------------------------
# input        $DataSource | DBI connection string without user name/password | string
# input        $User | Database user or login name | string
# input        $Password | Database user password | string
# input        $CacheKey | key to identify the connection | string
# return       $dbi | databse handle | DE_EPAGES::Database::API::DBI
#========================================================================================
sub new ($$$$$) {
    my $class = shift;
    my ($source, $user, $password, $cachekey) = @_;

    my %options;
    $options{'connectInfo'} = [$source, $user, $password, { AutoCommit => 1}];
    LogDebug('Connect', $options{'connectInfo'});

    $options{'dbh'} = DBI->connect(@{$options{'connectInfo'}}) or Error ('DBIConnect', {
        'message'   => DBI->errstr,
        'code'      => DBI->err,
        'user'      => $user,
        'source'    => $source,
    });
    $options{'cachekey'} = $cachekey;
    $options{'source'}   = $source;
    $options{'transactionLogOn'} = undef;
    $options{'transactionLog'} = [];
    $options{'datetime_parser'} = undef;
    $options{'driver_name'} = $options{'dbh'}->{'Driver'}->{'Name'};
    $options{'cache_sth'} = 1; # cache prepared statements
    $options{'maxTextLength'} = 2097152; ## no critic qw(ProhibitMagicNumbers) default 2 MB
    return bless \%options, $class;
}

#========================================================================================
# function     reconnect
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $dbh = $dbi->reconnect();
#----------------------------------------------------------------------------------------
# description  Rebuilds a database connection, usefull if connect was closed by server.
#----------------------------------------------------------------------------------------
# return       $dbh | dbi handle | string
#========================================================================================
sub reconnect () {
    my $self = shift;
    $self->{'dbh'} = DBI->connect(@{$self->{'connectInfo'}}) or Error ('DBIConnect', {
        'message'   => DBI->errstr,
        'code'      => DBI->err,
        'user'      => $self->{'connectInfo'}[1],
        'source'    => $self->{'connectInfo'}[0],
    });
    return $self->{'dbh'};
}

#========================================================================================
# function     source
# state        public
#----------------------------------------------------------------------------------------
# syntax       $source = $dbi->source
#----------------------------------------------------------------------------------------
# description  Returns connection string to current dbi.
#----------------------------------------------------------------------------------------
# return       $source |  DBI connection string without user name/password | string
#========================================================================================
sub source {
    return shift->{'source'};
}

#========================================================================================
# function     disconnect
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->disconnect;
#----------------------------------------------------------------------------------------
# description  Disconnects the underlying database handle.
#========================================================================================
sub disconnect ($) {
    my $self = shift;
    if( defined $self->{'dbh'} ) {
        $self->{'dbh'}->disconnect;
        delete $self->{'dbh'};
    }
    return;
}

#========================================================================================
# function     dbh
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbh = $dbi->dbh
# example      my $sth = $dbi->dbh->prepare( 'SELECT * FROM dual' );
#----------------------------------------------------------------------------------------
# description  Returns database handle of the current connection.
#----------------------------------------------------------------------------------------
# return       $dbh | DBI database handle | object
#========================================================================================
sub dbh { shift->{'dbh'} }

#========================================================================================
# function     driverName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DriverName = $dbi->driverName
#----------------------------------------------------------------------------------------
# description  Returns the name of the DBD driver of the underlying database handle.
#               See also $dbh->{'Driver'}->{'Name'} in the DBI specification.
#               Returns 'Sybase' for DBD::Sybase and 'mysql' for DBD::mysql.
#----------------------------------------------------------------------------------------
# return       $DriverName | DBD driver name | string
#========================================================================================
sub driverName { shift->{'driver_name'} }

#========================================================================================
# function     cacheKey
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CacheKey = $dbi->cacheKey
#----------------------------------------------------------------------------------------
# description  Returns the indentifier of the connection used to identify the right cache.
#               different connections to one database can use the same key.
#----------------------------------------------------------------------------------------
# return       $CacheKey | key to identify the connection | string
#========================================================================================
sub cacheKey { shift->{'cachekey'} }

#========================================================================================
# function     storeName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $storeName = $dbi->storeName
#----------------------------------------------------------------------------------------
# description  Returns the storename of the connection.
#----------------------------------------------------------------------------------------
# return       $storeName |  store name | string
#========================================================================================
sub storeName { shift->{'cachekey'} }

#========================================================================================
# function     databaseName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $databaseName = $dbi->databaseName
#----------------------------------------------------------------------------------------
# description  Returns the name of the currently used database.
#               Returns undef if the DBMS does not have the concept of a database.
#----------------------------------------------------------------------------------------
# return       $databaseName | database name | string
#========================================================================================
sub databaseName {
    return undef;
}

#========================================================================================
# function     datetimeParser
# state        public
#----------------------------------------------------------------------------------------
# syntax       $datetimeParser = $dbi->datetimeParser
#----------------------------------------------------------------------------------------
# description  Returns the parser for datetime objects.
#----------------------------------------------------------------------------------------
# return       $datetimeParser |  datetime parser | object
#========================================================================================
sub datetimeParser { shift->{'datetime_parser'} }

#========================================================================================
# function     createUserAndStoreDBName
# state        private
#----------------------------------------------------------------------------------------
# syntax       ($DBName, $UserName, $Password ) = $dbi->createUserAndStoreDBName( $StoreName )
#----------------------------------------------------------------------------------------
# description  Generates a new username, password and a database name for the store.
#               The epages standard format for the db name is used, the users name
#               if like the database name prefixed with 'usr_', the passwort is 'epages'.
#----------------------------------------------------------------------------------------
# input        $StoreName | Name of the store | String
# return       $DBName |  Database name | String
# return       $UserName |  Database name | String
# return       $Password |  Password | String
#========================================================================================
sub createUserAndStoreDBName {
    my $self = shift;
    my ($StoreName) = @_;

    my $DBName = lc($StoreName) . 'db';
    my $DBLogin = lc($StoreName);

    return ($DBName, $DBLogin, 'epages');
}

#========================================================================================
# function     execute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aRows = $dbi->execute( $SQL );
# syntax       $aRows = $dbi->execute( $SQL, $aParams );
# syntax       $aRows = $dbi->execute( $SQL, $aParams, $aDttps, $aColumnNames );
# example      $aRows = $dbi->execute('SELECT * FROM shops');
# example      $aRows = $dbi->execute('SELECT * FROM shops WHERE name = ?', [ $ShopName ] );
#               print "Shop id: " . $aRow->[0]->[0];
# example      $aRows = $dbi->execute('SELECT siteid, creationdate FROM object WHERE objectid = ?', [ $ObjectID ], [ 'Integer', 'DateTime']);
#               print "Shop id: " . $aRow->[0]->[0];
#----------------------------------------------------------------------------------------
# description  Executes an SQL statement and fetches all results.
#----------------------------------------------------------------------------------------
# input        $SQL | SQL statement | string
# input        $aParams | (optional) list of parameters to replace '?' placeholders in
#                           the SQL string | ref.array.string
# input        $aDttps | (optional) list data type position of return values
#                           (see <function unquoteResultSet> | ref.array.string
# input        $aColumnNames | (optional) list of field names in the result set
#                           (see <function unquoteResultSet> | ref.array.string
# return       $aRows | list of result rows
#                           if $aColumnNames is specified, each row is a hash ref,
#                           otherwise each row is an array ref | array ref
#========================================================================================
sub execute {
    my $self = shift;
    my ($SQL, $aParams, $aDttps, $aColumnNames) = @_;

    my $sth = $self->executeStatement($SQL, $aParams, $aDttps, $aColumnNames);
    return [] unless $SQL =~ /^\s*(select|show|check|optimize|analyze)\b/i;

    my $aRows = $self->unquoteResultSet($sth->fetchall_arrayref, $aDttps, $aColumnNames);
    LogDebug('Result', $aRows, __PACKAGE__ . '::execute::Result');

    return $aRows;
}

#========================================================================================
# function     iterator
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $dbi->iterator( $SQL );
# syntax       $Iterator = $dbi->iterator( $SQL, $aParams );
# syntax       $Iterator = $dbi->iterator( $SQL, $aParams, $aDttps, $aColumnNames );
# example      $Iterator = $dbi->iterator( 'SELECT languageid, code2 FROM language WHERE isused = ?',
#                                           [ 1 ], ['Integer', 'String'], ['LanguageID', 'Code2'] );
#               while( $hLanguage = <$Iterator> ) {
#                   print "ID: $hLanguage->{LanguageID}, Code: $hLanguage->{Code2}\n";
#               }
#----------------------------------------------------------------------------------------
# description  Executes an SQL statement and returns an iterator to fetch the results.
#----------------------------------------------------------------------------------------
# input        $SQL | SQL statement | string
# input        $aParams | (optional) list of parameters to replace '?' placeholders in
#                           the SQL string | ref.array.string
# input        $aDttps | (optional) list data type position of return values
#                           (see <function unquoteResultSet> | ref.array.string
# input        $aColumnNames | (optional) list of field names in the result set
#                           (see <function unquoteResultSet> | ref.array.string
# return       $Iterator | resultset iterator | DE_EPAGES::Core::API::Iterator
#========================================================================================
sub iterator {
    my $self = shift;
    my ($SQL, $aParams, $aDttps, $aColumnNames) = @_;

    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     executeStatement
# state        private
#----------------------------------------------------------------------------------------
# syntax       $sth = $dbi->executeStatement( $SQL, $aParams );
#----------------------------------------------------------------------------------------
# description  Executes an SQL statement and returns the statement handle.
#----------------------------------------------------------------------------------------
# input        $SQL | SQL statement | string
# input        $aParams | (optional) list of parameters to replace '?' placeholders in
#                           the SQL string | ref.array.string
# return       $sth | DBI statement handle | object
#========================================================================================
sub executeStatement {
    my $self = shift;
    my ($SQL, $aParams) = @_;

    $aParams = [] unless defined $aParams;
    $_ = $self->quoteDateTime($_) foreach grep { ref $_ && IsA( $_, 'DateTime') } @$aParams;

    LogDebug('Statement', $SQL, __PACKAGE__ . '::execute::Statement');
    LogDebug('Parameter', $aParams, __PACKAGE__ . '::execute::Parameter');

    my $time1 = time;

    my $dbh = $self->dbh;
    my $sth = $self->{'cache_sth'} ? $dbh->prepare_cached($SQL) : $dbh->prepare($SQL);
    Error ('DBIPrepare', {
        'message'   => $dbh->errstr,
        'statement' => $SQL,
        'state'     => $dbh->state,
        'code'      => $dbh->err,
        'source'    => $self->source,
    }) unless $sth;

    my $Error = undef;
    my $DeadLock = 0;
    my $TryCounter = 0;
    do {
        $DeadLock = 0;
        $TryCounter++;
        eval {
            $sth->execute(@$aParams) or Error ('DBIExecute', {
                'message'   => $sth->errstr,
                'statement' => $SQL,
                'parameter' => join (',', map { defined $_ ? $_ : '(undef)' } @$aParams),
                'state'     => $sth->state,
                'code'      => $sth->err,
                'source'    => $self->source,
            });
        };
        if (ExistsError()) {
            $Error = GetError();
            my $Message = $Error->message;
            Error($Error) if (defined $Message && $Message !~ /Deadlock/);

            $Error->addVars({ 'TryCounter' => $TryCounter });
            LogWarnings([$Error]);

            $DeadLock = 1;
            sleep(1);
        }
    } while ($DeadLock && $TryCounter < 5);
    if ($DeadLock) {
        eval { $sth->finish; }; ##no critic qw(RequireExistsError)
        Error($Error);
    }

    my $timediff = time - $time1;
    AddStatistic($self->driverName . 'Execute', $timediff);
    AddStatistic('DBI:'.$SQL, $timediff);

    return $sth;
}

#========================================================================================
# function     do
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->do( $SQL, $aParams );
# syntax       $dbi->do( $SQL );
# example      $dbi->do('DELETE FROM shops');
# example      $dbi->do('DELETE FROM shops WHERE name = ?', [ $ShopName ] );
#----------------------------------------------------------------------------------------
# description  Executes an SQL statement without result set, such as INSERT, UPDATE or
#               DELETE.
#----------------------------------------------------------------------------------------
# input        $SQL | SQL statement | string
# input        $aParams | list of parameters to replace '?' placeholders in the SQL
#                           string | array ref
#========================================================================================
sub do {
    my $self = shift;
    my ($SQL, $aParams) = @_;

    $self->executeStatement( $SQL, $aParams );
    return;
}

#========================================================================================
# function     transaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->transaction( $cCode, $cRollback );
# example      $dbi->transaction( sub {
#                   $dbi->do( "DELETE FROM objects WHERE classid = 25" );
#                   $dbi->do( "DELETE FROM classes WHERE classid = 25" );
#               } );
#----------------------------------------------------------------------------------------
# description  Executes a code block in a transaction.
#----------------------------------------------------------------------------------------
# input        $cCode | code block | code reference
# input        $cRollback | roll back code block (optional) | code reference
#========================================================================================
sub transaction {
    my $self = shift;
    my ($cCode, $cRollback) = @_;

    # Temporary disable auto-commit.
    my $dbh = $self->dbh;
    no warnings 'once'; ## no critic (ProhibitNoWarnings)
    if ($dbh->{'AutoCommit'} && !defined($DB::sub)) { ## no critic (ProhibitPackageVars)
        $dbh->begin_work or die $dbh->errstr;
        eval {
            # Execute the code block.
            $cCode->();
            $dbh->commit or die $dbh->errstr;
        };
        if( ExistsError() ) {
            my $Error = GetError();
            $dbh->rollback or die $dbh->errstr;
            $cRollback->() if defined $cRollback;
            Error( $Error );
        }
    } else {
        $cCode->();
    }
    use warnings;
    return;
}

#========================================================================================
# function     inTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsInTransaction = $dbi->inTransaction;
#----------------------------------------------------------------------------------------
# description  Returns true if database connection is in transaction mode.
#----------------------------------------------------------------------------------------
# return       $IsInTransaction | true if connection is in transaction mode | boolean
#========================================================================================
sub inTransaction {
    my $self = shift;

    my $dbh = $self->dbh;
    return $dbh->{'AutoCommit'} ? 0 : 1;
}

#========================================================================================
# function     lockTables
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ret = $dbi->lockTables( $ahTables, $cCode );
# example      $dbi->lockTables( [
#                   { 'table' => 'test', 'mode' => 'WRITE'},
#                   { 'table' => 'test2', 'mode' => 'WRITE'},
#               ], sub {
#                   $dbi->do('DELETE FROM test2 WHERE id = ?', $id);
#                   $dbi->do('DELETE FROM test WHERE id = ?', $id);
#               });
#----------------------------------------------------------------------------------------
# description  Locks the specified tables during execution of a code block.
#               The actual implementation is database-specific.
#               The default implementation only executeds $cCode, but does not lock any
#               tables.
#----------------------------------------------------------------------------------------
# input        $ahTables | tables and locking modes. Hash keys are:
#                   <ul>
#                   <li>table - table name - string</li>
#                   <li>mode - locking mode (READ|WRITE) - string</li>
#                   </ul> | ref.array.hash
# input        $cCode | code block | code ref
# return       $ret | result of $cCode->() | any
#========================================================================================
sub lockTables {
    my $self = shift;
    my ($ahTables, $cCode) = @_;

    return $cCode->();
}

#========================================================================================
# function     existsTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsTable( $Table );
# example      if( $dbi->existsTable( 'sysobjects' ) ) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the database table exists.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $Exists | true if the table exists | boolean
#========================================================================================
sub existsTable {
    my $self = shift;
    my ($table) = @_;
    my $dbh = $self->dbh;
    my $sth = $dbh->table_info(undef, undef, $table) or die $dbh->errstr;
    my $aResult = $sth->fetchall_arrayref;
    $sth->finish;
    return scalar @$aResult ? 1 : 0;
}


#========================================================================================
# function     existsView
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsView( $View );
# example      if( $dbi->existsView( 'sysobjects' ) ) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the database view exists.
#----------------------------------------------------------------------------------------
# input        $View | view name | string
# return       $Exists | true if the view exists | boolean
#========================================================================================
sub existsView {
    my $self = shift;
    my ($view) = @_;
    my $dbh = $self->dbh;
    my $sth = $dbh->table_info(undef, undef, $view, 'VIEW') or die $dbh->errstr;
    my $aResult = $sth->fetchall_arrayref;
    $sth->finish;
    return scalar @$aResult ? 1 : 0;
}

#========================================================================================
# function     renameTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->renameTable( $OldName, $NewName );
# example      $dbi->renameTable( 'land', 'country' );
#----------------------------------------------------------------------------------------
# description  Renames a table.
#----------------------------------------------------------------------------------------
# input        $OldName | old table name | string
# input        $OldName | new table name | string
#========================================================================================
sub renameTable {
    my $self = shift;
    my ($OldName, $NewName) = @_;
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     renameTableColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->renameTableColumn( $Table, $OldColumnName, $NewColumnName );
# example      $dbi->renameTableColumn( 'person', 'prename', 'firstname' );
#----------------------------------------------------------------------------------------
# description  Renames a table column.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $OldColumnName | old column name | string
# input        $NewColumnName | new column name | string
#========================================================================================
sub renameTableColumn {
    my $self = shift;
    my ($Table, $OldColumnName, $NewColumnName) = @_;
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     existsTableColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsTableColumn( $Table, $Column );
# example      if( $dbi->existsTableColumn( 'sysobjects', 'name' ) ) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the database table column exists.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $Column | column name | string
# return       $Exists | true if the column exists | boolean
#========================================================================================
sub existsTableColumn {
    my $self = shift;
    my ($table, $column) = @_;
    my $dbh = $self->dbh;
    my $sth = $dbh->column_info(undef, undef, $table, $column) or die $dbh->errstr;
    my $aResult = $sth->fetchall_arrayref;
    $sth->finish;
    return scalar @$aResult ? 1 : 0;
}

#========================================================================================
# function     existsProcedure
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsProcedure( $Procedure );
# example      if( $dbi->existsProcedure( 'sp_InsertAttribute' ) ) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the stored procedure exists.
#----------------------------------------------------------------------------------------
# input        $Procedure | stored procedure name | string
# return       $Exists | true if the stored procedure exists | boolean
#========================================================================================
sub existsProcedure {
    my $self = shift;
    my ($Procedure) = @_;

    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     existsForeignKey
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsForeignKey( $Name );
# example      if( $dbi->existsForeignKey( 'sp_InsertAttribute' ) ) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the foreign key exists.
#----------------------------------------------------------------------------------------
# input        $ForeignKey | foreign key name | string
# return       $Exists | true if the foreign key exists | boolean
#========================================================================================
sub existsForeignKey {
    my $self = shift;
    my ($Name) = @_;

    die 'Not implemented for standard SQL';
}

#=======================================================================================
# function     buildQueryString
#----------------------------------------------------------------------------------------
# syntax       $DBSearchString = $dbi->buildQueryString($SearchString);
#----------------------------------------------------------------------------------------
# description  Substitute any intermediate spaces in search string with %.
#               e.g. " John &?    Doe " -> "John%Doe".
#               String is prepared for sql like statements.
#----------------------------------------------------------------------------------------
# input        $SearchString | string received by form | string
# return       $DBSearchString | possible database string prepared for like statements | string
#========================================================================================
sub buildQueryString {
    my $self = shift;
    local $_ = shift;
    s/["'*&?%+<>]/ /gx;  # " # removed specials from replacestring
    s/^\s*//;              # spaces befor and after
    s/\s*$//;
    s/\s+/ /g;             # one or more spaces to one space
    s/ /%/g;
    $_ = '%' if $_ eq '';
    return $_;
}

#========================================================================================
# function     quoteString
#----------------------------------------------------------------------------------------
# syntax       $result = $dbi->quoteString($instring);
#----------------------------------------------------------------------------------------
# example      $qstring = $dbi->quoteString("Don't worry.");
#               $dbi->do( "DELETE FROM shops WHERE description = $qstring" );
#----------------------------------------------------------------------------------------
# description  Formats a string for use as argument in an SQL statement.
#               <li> Replaces single quotes with a pair of single quotes, a' la str2database()
#               <li> Adds leading and trailing single quotes
#               <li> If the parameter is undefinded, the return value is the sting 'NULL'
#               without qoutes.<br>
#               This function does not copy the parameters locally, because it is
#               optimized for speed.
#----------------------------------------------------------------------------------------
# input        $instring  | input string (iso 1 charset),
#                            <li>example: "Don't worry."
#                          | string
# return       $result | quoted string, ready for database
#                            <li>example: "'Don''t worry.'" | string
#========================================================================================
sub quoteString {
    my $self = shift;
    local $_ = shift;
    return 'NULL' unless defined $_;
    Error('STRING_CONTAINS_0x0', {'String'=> $_}) if s/\0//g;
    s/\'/\'\'/g;
    s/\r\n/\n/g;
    s/\t//g;
    return "'$_'";
}

#========================================================================================
# function     quoteInteger
#----------------------------------------------------------------------------------------
# syntax       $StrInteger = $dbi->quoteInteger($Integer);
#----------------------------------------------------------------------------------------
# example      $StrInteger = $dbi->quoteInteger(1223);
#----------------------------------------------------------------------------------------
# description  Formats an integer number for use as argument in an SQL statement.
#               <li> If the parameter is undefinded, the return value is the sting 'NULL'
#               without qoutes.<br>
#               This function does not copy the parameters locally, because it is
#               optimized for speed.
#----------------------------------------------------------------------------------------
# return       $StrInteger | string ready for database
#                            <li>example: NULL | string
#========================================================================================
sub quoteInteger { ## no critic qw( RequireArgUnpacking )
    my $self = shift;
    return (defined $_[0]) ? $_[0] : 'NULL';
}

#========================================================================================
# function     quoteBoolean
#----------------------------------------------------------------------------------------
# syntax       $StrBool = $dbi->quoteBoolean($Bool);
#----------------------------------------------------------------------------------------
# example      $StrBool = $dbi->quoteBoolean(1);
#----------------------------------------------------------------------------------------
# description  Formats an boolean number for use as argument in an SQL statement.
#               <li> If the parameter is undefinded, the return value is the sting 'NULL'
#               without qoutes.<br>
#               This function does not copy the parameters locally, because it is
#               optimized for speed.
#----------------------------------------------------------------------------------------
# return       $StrBool | string ready for database
#                            <li>example: NULL | string
#========================================================================================
sub quoteBoolean { ## no critic qw( RequireArgUnpacking )
    my $self = shift;
    return 'NULL' unless defined $_[0];
    return $_[0] ? '1' : '0';
}

#========================================================================================
# function     quoteFloat
#----------------------------------------------------------------------------------------
# syntax       $StrFloat = $dbi->quoteFloat($Float);
#----------------------------------------------------------------------------------------
# example      $StrFloat = $dbi->quoteFloat(1223.4534);
#----------------------------------------------------------------------------------------
# description  Formats a number for use as argument in an SQL statement.
#               <li> If the parameter is undefinded, the return value is the sting 'NULL'
#               without qoutes.<br>
#               This function does not copy the parameters locally, because it is
#               optimized for speed.
#----------------------------------------------------------------------------------------
# return       $StrFloat | string ready for database
#                            <li>example: NULL | string
#========================================================================================
sub quoteFloat { ## no critic qw( RequireArgUnpacking )
    my $self = shift;
    return (defined $_[0]) ? $_[0] : 'NULL';
}

#========================================================================================
# function     quoteDateTime
#----------------------------------------------------------------------------------------
# syntax       $StrDateTime = $dbi->quoteDateTime( $DateTime );
#----------------------------------------------------------------------------------------
# example      $StrDateTime = $dbi->quoteDateTime( DateTime->now );
#----------------------------------------------------------------------------------------
# description  Converts a DateTime object to a string representation suitable for the
#               as argument in an SQL statement. Returns undef if the
#               <li> If the parameter is undefinded, the return value is the sting 'NULL'
#               without qoutes.<br>
#----------------------------------------------------------------------------------------
# input        $DateTime | DateTime object or undef | datetime
# return       $StrDateTime | string ready for database | string
#========================================================================================
sub quoteDateTime {
    my $self = shift;
    my ($DateTime) = @_;

    return undef unless defined $DateTime;
    Error('NO_DATETIME_OBJECT', {'Value'=>$DateTime}) if blessed($DateTime) ne 'DateTime';

    if( $DateTime->time_zone_long_name ne 'floating' ) {
        $DateTime = $DateTime->clone;
        $DateTime->set_time_zone(GetLocalTimeZone());
    }
    return $self->datetimeParser->format_datetime( $DateTime );
}

#========================================================================================
# function     quoteParameter
#----------------------------------------------------------------------------------------
# syntax       $Parameter = $dbi->quoteParameter($Type, $Value);
#----------------------------------------------------------------------------------------
# example      $Parameter = $dbi->quoteParameter('String', "don't");
#----------------------------------------------------------------------------------------
# description  Tests and quotes the parameters according to the data type spefification.
#----------------------------------------------------------------------------------------
# input        $Type | data type specification for parameters | ref.hash
# input        $Value | value | *
# return       $Parameter | parameter for database | string
#========================================================================================
sub quoteParameter {
    my $self = shift;
    my ($Type, $Value) = @_;

    return 'null' unless defined $Value;
    return $self->quoteString  ($Value) if $Type eq 'String';
    return $self->quoteInteger ($Value) if $Type eq 'Integer';
    return $self->quoteBoolean ($Value) if $Type eq 'Boolean';
    return $self->quoteFloat   ($Value) if ElementOf($Type, qw( Float Money ));
    return $self->quoteString  ($self->quoteDateTime($Value)) if $Type =~ /^DateTime|Time|Date$/;
    Error('UNKNOWN_TYPE',{ 'Type'=>$Type, 'Value'=>$Value });
}

#========================================================================================
# function     getParameterString
#----------------------------------------------------------------------------------------
# syntax       $ProcParameter = $dbi->getParameterString($hTypes, $hValues);
#----------------------------------------------------------------------------------------
# example      $ProcParameter = $dbi->getParameterString({'Name' => 'String;NotNull',
#                                       'Descr' => 'String',
#                                       'AnyNumber' => 'Integer'
#                                   }, {'Name'=>'TestName','AnyNumber'=> 1});
#               $ProcParameter = "@Name='TestName', @AnyNumber=1";
#----------------------------------------------------------------------------------------
# description  Create the parameter string for a procedure call, for example
#               "@Name='MyName', @Number=42". Tests and quotes the parameters according
#               to the data type spefification given in $hTypes.
#----------------------------------------------------------------------------------------
# input        $hTypes | data type specification for parameters | ref.hash
# input        $hValues | parameter name => value hash | ref.hash
# return       $ProcParameter | stored procedure parameter string | string
#========================================================================================
sub getParameterString {
    my $self = shift;
    my ($hTypes, $hValues) = @_;

    return join (', ',
        map { "\@$_=". $self->quoteParameter(($hTypes->{$_} =~ /(.*?);/ ? $1 :$hTypes->{$_}), $hValues->{$_}) }
        grep {defined $hValues->{$_}} keys %$hTypes
    );
}

#========================================================================================
# function     unquoteDateTime
#----------------------------------------------------------------------------------------
# syntax       $DateTime = $dbi->unquoteDateTime($StrDateTime);
#----------------------------------------------------------------------------------------
# example      $DateTime = $dbi->unquoteDateTime('20031127/09:56:57');
#----------------------------------------------------------------------------------------
# description  Parses a date/time string from a database result set. Returns undef
#               if the database returned a NULL value.
#----------------------------------------------------------------------------------------
# input        $StrDateTime | input string or undef | string
# return       $DateTime | DateTime object | datetime
#========================================================================================
sub unquoteDateTime {
    my $self = shift;
    my ($DateTime) = @_;
    return undef unless defined $DateTime;
    return $self->datetimeParser->parse_datetime( $DateTime );
}

#========================================================================================
# function     testDateTimeRange
#----------------------------------------------------------------------------------------
# syntax       $DateTime = $dbi->testDateTimeRange($StrDateTime);
#----------------------------------------------------------------------------------------
# example      $DateTime = $dbi->testDateTimeRange('20031127/09:56:57');
#----------------------------------------------------------------------------------------
# description  Test a date/time string for a passes range. Returns Error
#               if the date/time string is to large or to small.
#----------------------------------------------------------------------------------------
# input        $DateTime | input string | string
#========================================================================================
sub testDateTimeRange {
    my $self = shift;
    my ($DateTime) = @_;
    Error('DATETIME_TOO_LARGE',{ Value => "$DateTime" }) if $DateTime->year > 9999; ## no critic qw(ProhibitMagicNumbers)
    Error('DATETIME_TOO_SMALL',{ Value => "$DateTime" }) if $DateTime->year < 1753; ## no critic qw(ProhibitMagicNumbers)
    return;
}

#========================================================================================
# function     unquoteResultSet
#----------------------------------------------------------------------------------------
# syntax       $aaResult = $dbi->unquoteResultSet($aaResult, $aDttps);
# syntax       $ahResult = $dbi->unquoteResultSet($aaResult, $aDttps, $aColumnNames);
#----------------------------------------------------------------------------------------
# example      $result = $self->unquoteResultSet($sth->fetchall_arrayref, ['Integer', 'DateTime', 'String']);
#----------------------------------------------------------------------------------------
# description  Formats a date/time string to datetime object, and converts string to
#               utf-8 perl strings. If the $aColumnNames are defined the rows will be
#               converted to hashes.
#----------------------------------------------------------------------------------------
# input        $aaResult | resultset of sql statement | ref.array.array.string
# input        $aDttps | data type definition (String|DateTime|...)| ref.array.string
# input        $aColumnNames | names of columns | ref.array.string
# return       $aaResult | resultset to process in perl | ref.array.array.*
# return       $ahResult | resultset with columnNames | ref.array.hash.*
#========================================================================================
sub unquoteResultSet {
    my $self = shift;
    my ($ResultSet, $aDttps, $aColumnNames) = @_;
    my $MakeHash = (defined $aColumnNames and scalar @$aColumnNames > 0) ? 1 : 0;
    if (defined $aDttps) {
        foreach my $Column ( 0 .. scalar @$aDttps - 1 ) {
            my $Type = $aDttps->[$Column];
            next unless defined $Type;

            my $Converter;
            if( $Type eq 'String') {
                $Converter = sub {
                    unless (utf8::decode($_[0])) {
                        warn "(utf8::decode failed for '$_[0]' is_utf8:".utf8::is_utf8($_[0]).' valid:'.utf8::valid($_[0]);
                        $_[0] = decode('utf-8', $_[0]);
                    }

                    # convert line endings to Unix style to ensure that outgoing line
                    # endings are independent of how the data was inserted and
                    # independent of the merchant's operating system
                    $_[0] =~ s/\x{0D}\x{0A}/\x{0A}/gs;
                };
            }
            elsif( $Type eq 'DateTime' ) {
                # speed hack -- avoid getter datetimeParser()
                $Converter = sub { $_[0] = $self->{'datetime_parser'}->parse_datetime($_[0]) };
            }
            elsif( ElementOf($Type, qw( Float Money )) ) {
                # EPG-14440: Force Perl to convert the value into a number; this removes trailing zeros,
                # which MySQL inserts by default for decimal values (we don't want that!).
                $Converter = sub { $_[0] = $_[0] + 0; }
            }

            next unless defined $Converter;

            foreach my $aValues (@$ResultSet) {
                $Converter->( $aValues->[$Column] ) if defined $aValues->[$Column];
            }
        }

        if ($MakeHash) {
            return [
                map {
                    my %Info;
                    @Info{@$aColumnNames} = @$_;
                    \%Info;
                }
                @$ResultSet
            ];
        }
        else {
            return $ResultSet;
        }
    }
    return DecodeStructure( 'utf-8', $ResultSet );
}

#========================================================================================
# function     currentDateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DateTime = $dbi->currentDateTime
# example      print $dbi->currentDateTime->ymd;
#----------------------------------------------------------------------------------------
# description  Returns the current date and time from the database server. Use this
#               function to generate comparable timestamp values that are independent
#               of the randomly selected application server.
#----------------------------------------------------------------------------------------
# return       $DateTime | current date and time | DateTime object
#========================================================================================
sub currentDateTime {
    my $self = shift;

    # Default implementation returns the Application server time as local time with time
    # zone set to 'floating' for compatibility with other database drivers.
    # This should be overridden by the database driver to return the actual database time.
    my $Now = GetCurrentDateTime();
    $Now->set_time_zone('floating');
    return $Now;
}

#========================================================================================
# function     setPriority
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->setPriority( $Priority )
# example      $dbi->setPriority( 'LOW' )
#----------------------------------------------------------------------------------------
# description  Sets the priority of the current database process.
#----------------------------------------------------------------------------------------
# input        $Priority | priority (LOW|NORMAL|HIGH) | string
#========================================================================================
sub setPriority {
    my $self = shift;
    my ($Priority) = @_;

    # Default implementation does nothing.
    return;
}

#========================================================================================
# function     processID
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ProcessID = $dbi->processID;
#----------------------------------------------------------------------------------------
# description  Returns the ID of the current database process.
#               Returns 0 if the DBMS does not support this feature.
#----------------------------------------------------------------------------------------
# return       $ProcessID | process id | integer
#========================================================================================
sub processID {
    my $self = shift;

    # Default implementation.
    return 0;
}

#========================================================================================
# function     killConnections
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->killConnections( $DBName );
#----------------------------------------------------------------------------------------
# description  Terminates all connections to the given database.
#----------------------------------------------------------------------------------------
# input        $DBName | database name | string
#========================================================================================
sub killConnections {
    my $self = shift;
    my ($DBName) = @_;

    # Default implementation.
    return;
}

#========================================================================================
# function     addTableColumn
#----------------------------------------------------------------------------------------
# syntax       $dbi->addTableColumn($table, $columndefinition);
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $columndefinition = 'columnname varchar(30) NULL';
#----------------------------------------------------------------------------------------
# description  Adds a new column to an existing table $table.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $columndefinition | sql definiton of the column | string
#========================================================================================
sub addTableColumn {
    my $self = shift;
    my ($table, $columndefinition) = @_;
    $self->do("ALTER TABLE `$table` ADD $columndefinition", []);
    return;
}

#========================================================================================
# function     addTableColumnFirst
#----------------------------------------------------------------------------------------
# syntax       $dbi->addTableColumnFirst($table, $columndefinition);
#----------------------------------------------------------------------------------------
# example      $dbi->addTableColumnFirst('productprice', 'currencyid CHAR(3) NULL');
#----------------------------------------------------------------------------------------
# description  Adds a new column to an existing table $table, before all existing columns.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $columndefinition | sql definiton of the column | string
#========================================================================================
sub addTableColumnFirst {
    my $self = shift;
    my ($table, $columndefinition) = @_;
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     addTableColumnAfterColumn
#----------------------------------------------------------------------------------------
# syntax       $dbi->addTableColumnAfterColumn($table, $column, $columndefinition);
#----------------------------------------------------------------------------------------
# example      $dbi->addTableColumnAfterColumn('product', 'weighunitid', 'manufacturerid INT NULL');
#----------------------------------------------------------------------------------------
# description  Adds a new column to an existing table $table, after a specified existing column.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
# input        $columndefinition | sql definiton of the column | string
#========================================================================================
sub addTableColumnAfterColumn {
    my $self = shift;
    my ($table, $column, $columndefinition) = @_;
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     modifyTableColumn
#----------------------------------------------------------------------------------------
# syntax       $dbi->modifyTableColumn($table, $columndefinition);
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $columndefinition contains sql code to create a column
#----------------------------------------------------------------------------------------
# description  Modifies an existing column of an existing table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $columndefinition | sql definiton of the column | string
#               column_name datatype [null | not null]
#========================================================================================
sub modifyTableColumn {
    my $self = shift;
    my ($table, $columndefinition) = @_;

    # we must use disableForeignKeyChecks for MySQL 5.6 or higher, see EPG-29751
    $self->disableForeignKeyChecks(
        sub { $self->do("alter table `$table` MODIFY $columndefinition", []); }
    );
    return;
}

#========================================================================================
# function     modifyTableColumnReplaceDefault
#----------------------------------------------------------------------------------------
# syntax       $dbi->modifyTableColumnReplaceDefault($table, $column ,$defaultvalue);
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $column = 'columnname';
#               $defaultvalue = 0 use functions quote* to quote the value before
#----------------------------------------------------------------------------------------
# description  Modifies a default of an existing column $column of an existing table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
# input        $defaultvalue | default value of the column | string
#========================================================================================
sub modifyTableColumnReplaceDefault {
    my $self = shift;
    my ($table, $column ,$defaultvalue) = @_;
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     modifyTableColumnDropDefault
#----------------------------------------------------------------------------------------
# syntax       $dbi->modifyTableColumnDropDefault($table, $column );
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $column = 'columnname';
#----------------------------------------------------------------------------------------
# description  Deletes a Default of an existing column of an existing table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
#========================================================================================
sub modifyTableColumnDropDefault {
    my $self = shift;
    my ($table, $column) = @_;
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     dropTableColumn
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropTableColumn($table, $column);
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $column = 'columnname';
#----------------------------------------------------------------------------------------
# description  Drops an existing column $column from a table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
#========================================================================================
sub dropTableColumn {
    my $self = shift;
    my ($table, $column) = @_;

    $self->do("alter table `$table` DROP $column", []);
    return;
}

#========================================================================================
# function     dropTableColumnIfExists
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropTableColumnIfExists($table, $column);
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $column = 'columnname';
#----------------------------------------------------------------------------------------
# description  Drops an existing column $column from a table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
#========================================================================================
sub dropTableColumnIfExists {
    my $self = shift;
    my ($table, $column) = @_;

    $self->do("alter table `$table` DROP $column", []) if $self->existsTableColumn($table, $column);
    return;
}

#========================================================================================
# function     createIndex
#----------------------------------------------------------------------------------------
# syntax       $dbi->createIndex($Table, $IndexName, $Options, $aColumns, $hColumnOptions, $ExtraOptions);
#----------------------------------------------------------------------------------------
# example      $dbi->createIndex('selectedvariation', 'i_selectedvariation_product', undef,
#                   [ 'productid', 'predefattributeid' ],
#                   { 'productid' => DESC, 'predefattributeid' => 'DESC' }
#               );
#----------------------------------------------------------------------------------------
# description  Creates an index on some table columns.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $IndexName | (optional) index name | string
# input        $Options | (optional) options, for example 'UNIQUE' | string
# input        $aColumns | list of column names | array
# input        $hColumnOptions | (optional) hash column name => options | ref.hash.string
# input        $ExtraOptions | (optional) special options after the CREATE INDEX statement | string
#========================================================================================
sub createIndex {
    my $self = shift;
    my ($Table, $IndexName, $Options, $aColumns, $hColumnOptions, $ExtraOptions) = @_;
    TestString('Table', $Table);
    $IndexName = '' unless defined $IndexName;
    $Options = '' unless defined $Options;
    TestArray('aColumns', $aColumns);
    $hColumnOptions = {} unless defined $hColumnOptions;
    $ExtraOptions = '' unless defined $ExtraOptions;

    my $sql = "CREATE $Options INDEX $IndexName ON `$Table` (";
    $sql .= join ', ', map {
        defined $hColumnOptions->{$_} ? "$_ $hColumnOptions->{$_}" : $_
    } @$aColumns;
    $sql .= ") $ExtraOptions";
    $self->do($sql);
    return;
}

#========================================================================================
# function     createDatabase
#----------------------------------------------------------------------------------------
# syntax       $dbi->createDatabase($DBName);
# example      $dbi->createDatabase('storedb');
#----------------------------------------------------------------------------------------
# description  Creates a new database.
#----------------------------------------------------------------------------------------
# input        $DBName | database name | string
#========================================================================================
sub createDatabase {
    my $self = shift;
    my ($DBName) = @_;

    $self->do( sprintf 'CREATE DATABASE %s', $DBName );
    return;
}

#========================================================================================
# function     existsDatabase
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsDatabase($DBName);
#----------------------------------------------------------------------------------------
# description  Returns true if the database exists.
#----------------------------------------------------------------------------------------
# input        $DBName | logical database name | string
# return       $Exists | database exists | boolean
#========================================================================================
sub existsDatabase {
    my ($DBName) = @_;

    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     dropDatabase
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropDatabase($DBName);
# example      $dbi->dropDatabase('storedb');
#----------------------------------------------------------------------------------------
# description  Deletes a database.
#----------------------------------------------------------------------------------------
# input        $DBName | database name | string
#========================================================================================
sub dropDatabase {
    my $self = shift;
    my ($DBName) = @_;

    $self->do( sprintf 'DROP DATABASE %s', $DBName );
}

#========================================================================================
# function     createEpagesDatabase
#----------------------------------------------------------------------------------------
# syntax       $dbi->createEpagesDatabase($DBName, $DBLogin, $hOptions);
# example      $dbi->createEpagesDatabase('sitedb', 'usr_sitedb');
# example      $dbi->createEpagesDatabase('storedb');
#----------------------------------------------------------------------------------------
# description  Creates a database with epages default sizes and names.
#----------------------------------------------------------------------------------------
# input        $DBName | database name | string
# input        $DBLogin | database login (optional) | string
# input        $hOptions | DBMS specific options | ref.hash
#========================================================================================
sub createEpagesDatabase {
    my $self = shift;
    my ($DBName, $DBLogin, $hOptions) = @_;

    $self->createDatabase( $DBName );
}

#========================================================================================
# function     createTempTable
#----------------------------------------------------------------------------------------
# syntax       $dbi->createTempTable($Table);
#----------------------------------------------------------------------------------------
#               $dbi->createTempTable('temptable(objectid int not null primary key)'));
#----------------------------------------------------------------------------------------
# description  Creates a temporary table with the given name and parameters.
#----------------------------------------------------------------------------------------
# input        $Table | table name and parameters | string
#========================================================================================
sub createTempTable {
    my $self = shift;
    my ($Table) = @_;
    TestString('Table', $Table);

    my $sql = "CREATE TEMPORARY TABLE $Table";
    $self->do($sql);
    return;
}

#========================================================================================
# function     tempTableName
#----------------------------------------------------------------------------------------
# syntax       $dbi->tempTableName($TableName);
#----------------------------------------------------------------------------------------
#               $dbi->tempTableName('temptable');
#----------------------------------------------------------------------------------------
# description  Returns the name of the temporary table in the correct database style.
#----------------------------------------------------------------------------------------
# input        $TableName | table name | string
# return       $TableName | correct table name | string
#========================================================================================
sub tempTableName {
    my $self = shift;
    my ($TableName) = @_;
    TestString('TableName', $TableName);

    return $TableName ;
}

#========================================================================================
# function     dropTempTableIfExists
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropTempTableIfExists($Table);
#----------------------------------------------------------------------------------------
# description  Removes the temporary table with the given name if it exists.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
#========================================================================================
sub dropTempTableIfExists {
    my $self = shift;
    my ($Table) = @_;
    TestString('Table', $Table);

    eval { ## no critic qw( RequireExistsError )
        local $SIG{__WARN__} = sub {};
        $self->dropTable($Table);
    };
    return;
}

#========================================================================================
# function     dropIndex
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropIndex($table, $indexname);
#----------------------------------------------------------------------------------------
# example      $dbi->dropIndex('product', 'i_product_super');
#----------------------------------------------------------------------------------------
# description  Drops index $indexname from table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $indexname | index name | string
#========================================================================================
sub dropIndex {
    my $self = shift;
    my ($table, $indexname) = @_;

    $self->do("drop index `$table`.`$indexname`", []);
    return;
}

#========================================================================================
# function     dropIndexIfExists
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropIndexIfExists($table, $indexname);
#----------------------------------------------------------------------------------------
# example      $dbi->dropIndexIfExists('product', 'i_product_super');
#----------------------------------------------------------------------------------------
# description  Drops index $indexname from table $tablename if this index exists
#               already in the table. Do nothing if the table doesnt exist.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $indexname | index name | string
#========================================================================================
sub dropIndexIfExists {
    my $self = shift;
    my ($table, $indexname) = @_;

    return unless $self->existsTable($table);
    $self->dropIndex( $table, $indexname ) if $self->existsIndex( $table, $indexname );
    return;
}

#========================================================================================
# function     existsIndex
#----------------------------------------------------------------------------------------
# syntax       $exists = $dbi->existsIndex($Table, $IndexName);
#----------------------------------------------------------------------------------------
# example      if( $dbi->existsIndex('product', 'i_product_super') ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if the index exists.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $IndexName | index name | string
#========================================================================================
sub existsIndex {
    my $self = shift;
    my ($Table, $IndexName) = @_;

    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     recreateIndex
#----------------------------------------------------------------------------------------
# syntax       $dbi->recreateIndex($Table, $IndexName, $Options, $aColumns, $hColumnOptions, $ExtraOptions);
#----------------------------------------------------------------------------------------
# example      $dbi->createIndex('selectedvariation', 'i_selectedvariation_product', undef,
#                   [ 'productid', 'predefattributeid' ],
#                   { 'productid' => DESC, 'predefattributeid' => 'DESC' }
#               );
#----------------------------------------------------------------------------------------
# description  Creates an index on some table columns, drop it before if it still exists.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $IndexName | (optional) index name | string
# input        $Options | (optional) options, for example 'UNIQUE' | string
# input        $aColumns | list of column names | array
# input        $hColumnOptions | (optional) hash column name => options | ref.hash.string
# input        $ExtraOptions | (optional) special options after the CREATE INDEX statement | string
#========================================================================================
sub recreateIndex {
    my $self = shift;
    my ($Table, $IndexName, $Options, $aColumns, $hColumnOptions, $ExtraOptions) = @_;

    $self->dropIndex($Table, $IndexName) if $self->existsIndex($Table, $IndexName);
    $self->createIndex($Table, $IndexName, $Options, $aColumns, $hColumnOptions, $ExtraOptions);
    return;
}

#========================================================================================
# function     addForeignKey
#----------------------------------------------------------------------------------------
# syntax       $dbi->addForeignKey($Table, $Constraint, $aKeys, $RefTable, $aRefKeys);
#----------------------------------------------------------------------------------------
# example      $dbi->addForeignKey('product', 'fk_product_object', ['productid'],
#                   'object' => ['objectid']);
#----------------------------------------------------------------------------------------
# description  Adds a foreign key constaint.
#----------------------------------------------------------------------------------------
# input        $Table | slave table name | string
# input        $Constraint | constraint name (optional) | string
# input        $aKeys | list of columns in the slave table | ref.array.string
# input        $RefTable | master table name | string
# input        $aRefKeys | list of columns in the master table | ref.array.string
#========================================================================================
sub addForeignKey {
    my $self = shift;
    my ($Table, $Constraint, $aKeys, $RefTable, $aRefKeys) = @_;
    TestString('Table', $Table);
    TestString('Constraint', $Constraint) if defined $Constraint;
    TestArray('aKeys', $aKeys);
    TestString('RefTable', $RefTable);
    TestArray('aRefKeys', $aRefKeys);

    my $sql = "ALTER TABLE `$Table` ADD";
    $sql .= " CONSTRAINT $Constraint" if defined $Constraint;
    $sql .= ' FOREIGN KEY (' . join(', ', @$aKeys) . ')';
    $sql .= " REFERENCES `$RefTable` (" . join(', ', @$aRefKeys) . ')';
    $self->do($sql);
    return;
}

#========================================================================================
# function     primaryKeyColumns
#----------------------------------------------------------------------------------------
# syntax       @Columns = $dbi->primaryKeyColumns($Table);
#----------------------------------------------------------------------------------------
# description  Returns a list of columns of the primary of a table.
#               Returns an empty list if the table does not have a primary key.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       @Columns | column names | array.string
#========================================================================================
sub primaryKeyColumns {
    my $self = shift;
    my ($Table) = @_;
    TestString('Table', $Table);

    return $self->dbh->primary_key( undef, undef, $Table );
}

#========================================================================================
# function     tableColumns
#----------------------------------------------------------------------------------------
# syntax       @Columns = $dbi->tableColumns($Table);
#----------------------------------------------------------------------------------------
# description  Returns a list of all columns of a table.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       @Columns | column names | array.string
#========================================================================================
sub tableColumns {
    my $self = shift;
    my ($Table) = @_;
    TestString('Table', $Table);

    my $dbh = $self->dbh;
    my $sth = $dbh->column_info( $self->databaseName, undef, $Table, undef );
    die $dbh->errstr unless $sth;

    return map { $_->{'COLUMN_NAME'} }
        sort { $a->{ORDINAL_POSITION} <=> $b->{ORDINAL_POSITION} }
        @{ $sth->fetchall_arrayref({}) };
}

#========================================================================================
# function     addPrimaryKey
#----------------------------------------------------------------------------------------
# syntax       $dbi->addPrimaryKey($Table, $Constraint, $aColumnNames);
#----------------------------------------------------------------------------------------
# example      $dbi->addPrimaryKey('product', 'pk_product', ['productid']);
#----------------------------------------------------------------------------------------
# description  Adds constaint (primary key) $constraint to a table $tablename.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $Constraint | constraint name (optional) | string
# input        $aColumnNames | column names | ref.array.string
#========================================================================================
sub addPrimaryKey {
    my $self = shift;
    my ($Table, $Constraint, $aColumnNames) = @_;
    TestString('Table', $Table);
    TestString('Constraint', $Constraint) if defined $Constraint;
    TestArray('aColumnNames', $aColumnNames);

    my $sql = sprintf 'ALTER TABLE `%s` ADD CONSTRAINT %s PRIMARY KEY (%s)',
        $Table,
        (defined $Constraint ? $Constraint : 'pk_' . $Table),
        join(', ', @$aColumnNames);
    $self->do($sql);
    return;
}

#========================================================================================
# function     dropForeignKey
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropForeignKey($table, $constraint);
#----------------------------------------------------------------------------------------
# example      $dbi->dropForeignKey('product', 'fk_product_object');
#----------------------------------------------------------------------------------------
# description  Drops constaint (foreign key) $constraint from a table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $constraint | constraint name | string
#========================================================================================
sub dropForeignKey {
    my $self = shift;
    my ($table, $constraint) = @_;

    $self->do("ALTER TABLE `$table` DROP CONSTRAINT $constraint", []);
    return;
}

#========================================================================================
# function     dropForeignKeyIfExists
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropForeignKeyIfExists($table, $constraint);
#----------------------------------------------------------------------------------------
# example      $dbi->dropForeignKeyIfExists('product', 'fk_product_object');
#----------------------------------------------------------------------------------------
# description  Drops constaint (foreign key) $constraint from a table $tablename
#               if this foreign key exists already in the table.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $constraint | constraint name | string
#========================================================================================
sub dropForeignKeyIfExists {
    my $self = shift;
    my ( $table, $constraint ) = @_;

    $self->dropForeignKey($table, $constraint) if  $self->existsForeignKey($constraint);
    return;
}

#========================================================================================
# function     dropPrimaryKey
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropPrimaryKey($table, $constraint);
#----------------------------------------------------------------------------------------
# example      $dbi->dropPrimaryKey('product', 'pk_product');
#----------------------------------------------------------------------------------------
# description  Drops constaint (Primary key) $constraint from a table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $constraint | constraint name | string
#========================================================================================
sub dropPrimaryKey {
    my $self = shift;
    my ($table, $constraint) = @_;

    $self->do("alter table `$table` drop constraint $constraint", []);
    return;
}

#========================================================================================
# function     dropTable
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropTable($Table);
#----------------------------------------------------------------------------------------
# example      $dbi->dropTable('product');
#----------------------------------------------------------------------------------------
# description  Removes a table from the database (deleting all data, of cause).
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
#========================================================================================
sub dropTable {
    my $self = shift;
    my ($Table) = @_;
    TestString('Table', $Table);

    $self->do("DROP TABLE `$Table`");
    return;
}

#========================================================================================
# function     dropTableIfExists
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropTableIfExists($Table);
#----------------------------------------------------------------------------------------
# description  Drops an existing table from the database.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
#========================================================================================
sub dropTableIfExists {
    my $self = shift;
    my ($Table) = @_;

    $self->dropTable($Table) if $self->existsTable($Table);
    return;
}

#========================================================================================
# function     dropView
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropView($View);
#----------------------------------------------------------------------------------------
# example      $dbi->dropView('v_lookup');
#----------------------------------------------------------------------------------------
# description  Removes a view from the database.
#----------------------------------------------------------------------------------------
# input        $Table | view name | string
#========================================================================================
sub dropView {
    my $self = shift;
    my ($View) = @_;
    TestString('View', $View);

    $self->do("DROP VIEW $View");
    return;
}

#========================================================================================
# function     dropProcedure
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropProcedure($Procedure);
#----------------------------------------------------------------------------------------
# example      $dbi->dropProcedure('sp_InsertAttribute');
#----------------------------------------------------------------------------------------
# description  Removes a stored procedure from the database
#----------------------------------------------------------------------------------------
# input        $Procedure | procedure name | string
#========================================================================================
sub dropProcedure {
    my $self = shift;
    my ($Procedure) = @_;
    TestString('Procedure', $Procedure);

    $self->do("DROP PROCEDURE $Procedure");
    return;
}

#========================================================================================
# function     dropProcedureIfExists
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropProcedureIfExists($Procedure);
#----------------------------------------------------------------------------------------
# example      $dbi->dropProcedureIfExists('sp_InsertAttribute');
#----------------------------------------------------------------------------------------
# description  Removes a stored procedure from the database if exists
#----------------------------------------------------------------------------------------
# input        $Procedure | procedure name | string
#========================================================================================
sub dropProcedureIfExists { ## no critic qw(RequireCommentForExportedFunctions)
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     dropFunction
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropFunction($Function);
#----------------------------------------------------------------------------------------
# description  Removes a function from the database.
#----------------------------------------------------------------------------------------
# input        $Function | Function name | string
#========================================================================================
sub dropFunction {
    my $self = shift;
    my ($Function) = @_;
    TestString('Function', $Function);

    $self->do("DROP FUNCTION $Function");
    return;
}

#========================================================================================
# function     sortDeleteAllTables
#----------------------------------------------------------------------------------------
# syntax       $aTables = $dbi->sortDeleteAllTables($aaReferences);
#----------------------------------------------------------------------------------------
# example      $aTables = $dbi->sortDeleteAllTables([['product','object'],['object','object'],['object_nextid', undef]]);
# example      $aTables = $dbi->sortDeleteAllTables( $dbi->getAllTableReferences );
#----------------------------------------------------------------------------------------
# description  Sort tables by references. Deleting tables in this order is possible.
#----------------------------------------------------------------------------------------
# input        $aaReferences | list of table references each list element is a pair of
#                       tables [ table => master_table ]. If master_table is undef, then
#                       the table does not depend on other tables | ref.array.array.string
# return       $aTables | table names | ref.array.string
#========================================================================================
sub sortDeleteAllTables {
    shift;
    my ($aaReferences) = @_;

    # Get all tables.
    my @SortTables;
    my %AllTables;
    foreach my $aRef (@$aaReferences) {
        $AllTables{$aRef->[0]} = 1;
        $AllTables{$aRef->[1]} = 1 if defined $aRef->[1];
    }

    # Sort out slaves without master.
    my $Removed = 1;
    while ($Removed) {
        $Removed = 0;
        my %MasterTables = ();
        $MasterTables{$_->[1]} = 1 foreach
            grep {$_->[0] ne $_->[1] and exists $AllTables{$_->[0]}}
            @$aaReferences;

        foreach my $Slave (grep {!$MasterTables{$_}} keys %AllTables) {
            push @SortTables, $Slave;
            delete $AllTables{$Slave};
            $Removed = 1;
        }
    }

    # If there are tables always in the list, there must be a cycle.
    Error('TABLE_CYCLE', { 'Tables' => join (',', keys %AllTables)}) if scalar keys %AllTables;
    return \@SortTables;
}

#========================================================================================
# function     getAllTableReferences
#----------------------------------------------------------------------------------------
# syntax       $aaReferences = $dbi->getAllTableReferences();
#----------------------------------------------------------------------------------------
# description  Determines all table references
#----------------------------------------------------------------------------------------
# return       $aaReferences | list of table references
#                       each list element is a pair of tables [ table => master_table ].
#                       If master_table is undef, then the table does not depend on other
#                       tables | ref.array.array.string
#========================================================================================
sub getAllTableReferences {
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     dropAllTables
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropAllTables();
#----------------------------------------------------------------------------------------
# description  Removes all user tables from the database (deleting all data, of cause).
#========================================================================================
sub dropAllTables {
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     modifyTableLockingScheme
#----------------------------------------------------------------------------------------
# syntax       $dbi->modifyTableLockingScheme($table, $lockingscheme );
#----------------------------------------------------------------------------------------
# example      $dbi->modifyTableLockingScheme('tablename', 'datarows');
#----------------------------------------------------------------------------------------
# description  Changes the locking scheme of a table.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $lockingscheme | locking scheme (datarows,datapages or allpages) | string
#========================================================================================
sub modifyTableLockingScheme {
    my $self = shift;
    my ($table, $lockingscheme) = @_;
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     getTableLockingScheme
#----------------------------------------------------------------------------------------
# syntax       $lockingscheme = $dbi->getTableLockingScheme($table );
#----------------------------------------------------------------------------------------
# example      $lockingscheme = $dbi->getTableLockingScheme('tablename' );
#----------------------------------------------------------------------------------------
# description  Gets the locking scheme of a table.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# return       $lockingscheme | locking scheme (datarows,datapages or allpages) | string
#========================================================================================
sub getTableLockingScheme {
    my $self = shift;
    my ($table) = @_;
    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     ping
#----------------------------------------------------------------------------------------
# syntax       $isAlive = $dbi->ping;
#----------------------------------------------------------------------------------------
# description  test if dbi connection is alive.
#----------------------------------------------------------------------------------------
# return       $isAlive | is connection alive | boolean
#========================================================================================
sub ping {
    my $self = shift;
    return $self->dbh->ping;
}

#========================================================================================
# function     escapeChar
#----------------------------------------------------------------------------------------
# syntax       $escapeChar = $dbi->escapeChar;
#----------------------------------------------------------------------------------------
# description  Returns the escape character '' to escape like parameter.
#----------------------------------------------------------------------------------------
# return       $escapeChar | empty string | string
#========================================================================================
sub escapeChar { return ''; }

#========================================================================================
# function     doEscapeChars
#----------------------------------------------------------------------------------------
# syntax       $doEscapeChars = $dbi->doEscapeChars;
#----------------------------------------------------------------------------------------
# description  Returns which character should be escaped.
#----------------------------------------------------------------------------------------
# return       $doEscapeChars | '%', '_', '[' and ']' | string
#========================================================================================
sub doEscapeChars { return '%_'; }

#========================================================================================
# function     escapeFromExtension
#----------------------------------------------------------------------------------------
# syntax       $EscapeFromExtension = $dbi->escapeFromExtension;
#----------------------------------------------------------------------------------------
# description  Returns string should be added after each like. <function escapeChar> is
#               used to get the escape char.
#----------------------------------------------------------------------------------------
# return       $EscapeFromExtension | empty string for standard | string
#========================================================================================
sub escapeFromExtension { return ''; }

#========================================================================================
# function     getSearchCollation
#----------------------------------------------------------------------------------------
# syntax       $SearchCollation = $dbi->getSearchCollation;
#----------------------------------------------------------------------------------------
# description  Returns additional SQL syntax for specification of collations for searches.
#               This is ususally used for mysql to return
#               'COLLATE utf8_general_ci'
#----------------------------------------------------------------------------------------
# return       $SearchCollation | empty string for standard | string
#========================================================================================
sub getSearchCollation { return ''; }

#========================================================================================
# function     replaceLikeWildcards
#----------------------------------------------------------------------------------------
# syntax       $likestring = $dbi->replaceLikeWildcards($string);
#----------------------------------------------------------------------------------------
# description  Replace chars used at where clause operator like (%_).
#               <function escapeChar> and <function doEscapeChars> are used.
#----------------------------------------------------------------------------------------
# input        $string | string incl. chars (eg test[1]) | string
# return       $likestring | string with replaced chars (eg test~[1~]) | string
#========================================================================================
sub replaceLikeWildcards {
    my $self = shift;
    local $_ = shift;
    my $esc = $self->escapeChar;
    my $find = $self->doEscapeChars;
    s/([\Q$find\E])/$esc$1/gs;
    return $_;
}

#========================================================================================
# function     enableTransactionLog
#----------------------------------------------------------------------------------------
# syntax       $dbi->enableTransactionLog($enabled);
#----------------------------------------------------------------------------------------
# description  Activates or deactivates transaction logging.
#----------------------------------------------------------------------------------------
# input        $enabled | true to enable transaction log | boolean
#========================================================================================
sub enableTransactionLog {
    my $self = shift;
    my ($enable ) = @_;

    $self->{'transactionLogOn'} = $enable;
    $self->resetTransactions;
}

#========================================================================================
# function     isTransactionLogEnabled
#----------------------------------------------------------------------------------------
# syntax       $enabled = $dbi->isTransactionLogEnabled;
#----------------------------------------------------------------------------------------
# description  Returns whether transaction logging is currently enabled.
#----------------------------------------------------------------------------------------
# return       $enabled | true if transaction log is enabled | boolean
#========================================================================================
sub isTransactionLogEnabled {
    return shift->{'transactionLogOn'};
}

#========================================================================================
# function     logTransaction
#----------------------------------------------------------------------------------------
# syntax       $dbi->logTransaction($statement, $aParameter, $aTypes);
#----------------------------------------------------------------------------------------
# description  Logs the transaction of this dbi and collect this at an internal
#               structure. Use <function getTransactions> and <function resetTransactions>
#               for your own purpose.
#----------------------------------------------------------------------------------------
# input        $statement | statement incl. ? as placeholder for parameter | string
# input        $aParameter | parameters of statement| ref.array.*
# input        $aTypes | types of parameter (String|Integer|DateTime|Float|Boolean) | ref.array.string
#========================================================================================
sub logTransaction {
    my $self = shift;
    my ($statement, $aParameter, $aTypes) = @_;
    return unless $self->isTransactionLogEnabled;
    if(defined $aParameter) {
        my @Parameters = ();
        if (defined $aTypes and ref $aTypes eq 'ARRAY' and scalar @$aParameter == scalar @$aTypes) {
            for(my $i=0;$i<scalar @$aParameter;$i++) {
                my $Type = $aTypes->[$i];
                if    ( $Type =~ /^String/   ) { push @Parameters, $self->quoteString ($aParameter->[$i]); }
                elsif ( $Type =~ /^DateTime/ ) { push @Parameters, $self->quoteString ($self->quoteDateTime($aParameter->[$i])); }
                elsif ( $Type =~ /^Integer/  ) { push @Parameters, $self->quoteInteger($aParameter->[$i]); }
                elsif ( $Type =~ /^Boolean/  ) { push @Parameters, $self->quoteBoolean($aParameter->[$i]); }
                elsif ( ElementOf($Type, qw( Float Money )) ) { push @Parameters, $self->quoteFloat($aParameter->[$i]); }
                else {
                    Warning('UNKNOWN_TYPE',{ 'Key'=>$Type, 'Value'=>$aParameter->[$i]});
                    return;
                }
            }
        } else {
            warn 'types are not defined at statement '.$statement;
            for(my $i=0;$i<scalar @$aParameter;$i++) {
                if (not defined $aParameter->[$i]) {
                    push @Parameters, 'NULL';
                } elsif ($aParameter->[$i] =~ /^[-+]?(?:\d*(?:\.\d*)|\d+)(?:[eE][-+]?\d+)?$/x) {
                    push @Parameters, $self->quoteInteger($aParameter->[$i]);
                } else {
                    push @Parameters, $self->quoteString($aParameter->[$i]);
                }
            }
        }
        $statement =~ s/\?/shift @Parameters/eg;
    }
    GetLog()->debug($statement);
    push @{$self->{'transactionLog'}}, $statement;
}

#========================================================================================
# function     getTransactions
#----------------------------------------------------------------------------------------
# syntax       $aStatements = $dbi->getTransactions;
#----------------------------------------------------------------------------------------
# description  Returns the logged transactions, use (<function logTransactions> to push
#               statements to the collection.
#----------------------------------------------------------------------------------------
# return       $aStatements | statements ? placeholder are replaced | ref.array.string
#========================================================================================
sub getTransactions {
    my $self = shift;
    return [@{$self->{'transactionLog'}}];
}

#========================================================================================
# function     resetTransactions
#----------------------------------------------------------------------------------------
# syntax       $dbi->resetTransactions;
#----------------------------------------------------------------------------------------
# description  Clears the statement collection.
#========================================================================================
sub resetTransactions {
    my $self = shift;
    $self->{'transactionLog'} = [];
    return;
}

#========================================================================================
# function     atIsolationLevel
#----------------------------------------------------------------------------------------
# syntax       $dbi->atIsolationLevel($IsolationLevel, $Sub);
#----------------------------------------------------------------------------------------
# description  executes the function sub at specified transaction isolation level.
#----------------------------------------------------------------------------------------
# input        $IsolationLevel | values (
#                       READ UNCOMMITTED|
#                       READ COMMITTED|
#                       REPEATABLE READ|
#                       SERIALIZABLE
#               ) | string
# input        $Sub | function to execute select statement | ref.code
#========================================================================================
sub atIsolationLevel {
    my $self = shift;
    my ($IsolationLevel, $Sub) = @_;
    $Sub->();
}

#========================================================================================
# function     encodeSpaces
# state        public
#----------------------------------------------------------------------------------------
# syntax       $string = encodeSpaces( $string );
#----------------------------------------------------------------------------------------
# description  Encodes spaces to simulate database-like string encoding.
#               The global version does nothing at all.
#----------------------------------------------------------------------------------------
# input        $string | String to encode | string
# return       $string | Encoded string | string
#========================================================================================
sub encodeSpaces { ## no critic qw(RequireCommentForExportedFunctions RequireArgUnpacking)
    return $_[1];
}

#========================================================================================
# function     maxTextLength
# state        public
#----------------------------------------------------------------------------------------
# syntax       $source = $dbi->maxTextLength
#----------------------------------------------------------------------------------------
# description  Returns maxlength for string/text attributes.
#----------------------------------------------------------------------------------------
# return       $source |  DBI connection string without user name/password | string
#========================================================================================
sub maxTextLength {
    return shift->{'maxTextLength'};
}

#========================================================================================
# function     dropIndexesIfExist
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropIndexesIfExist( $Table, $aIndexes );
#----------------------------------------------------------------------------------------
# description  Drops an index from a table if the given index exists already on the
#               table. Since dropping various indexes in a single SQL statement, an array
#               of indexes is expected as a parameter.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $aIndexes | array of index names to be dropped from $Table | array.string
#========================================================================================
sub dropIndexesIfExist {
    my $self = shift;
    my ($Table, $aIndexes) = @_;

    die 'Not implemented for standard SQL';
}

#========================================================================================
# function     truncateTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->truncateTable( $Table );
#----------------------------------------------------------------------------------------
# description  Delete all table rows without where clause.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
#========================================================================================
sub truncateTable {
    my $self = shift;
    my ($Table) = @_;

    die 'Not implemented for standard SQL';
}

1;
