use strict;
use Config;
use Socket;
use Getopt::Long;
use Sys::Hostname;
use Sys::HostIP;
use Net::IPv6Addr;
use Config::IniFiles;
use File::Basename;
use Date::Format qw(time2str);
use DE_EPAGES::Core::API::File qw( CreateDirectory GetFileContent RunOnDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf UsePackage CheckIpFormat );
use DE_EPAGES::Database::API::Config qw(StoreConnections DatabaseConnectionData DatabaseBackupData);

BEGIN {
    use Config;
    if ( $Config{osname} eq 'MSWin32' ) {
        UsePackage('Win32::Process::Info');
    }
    else {
        UsePackage('Proc::ProcessTable');
    }
}

sub SybaseRuns {
    my $Found = 0;
    my $SqlServerExe = $ENV{SYBASE}.'\\'.$ENV{SYBASE_ASE}.'\bin\sqlsrvr.exe'; ##no critic qw(RequireInterpolationOfMetachars)
    $SqlServerExe = "$ENV{SYBASE}/$ENV{SYBASE_ASE}/bin/dataserver " if $Config{osname} ne 'MSWin32';

    if ( $Config{osname} eq 'MSWin32' ) {
        my $pi =  Win32::Process::Info->new();
        foreach my $Proc (@{$pi->GetProcInfo()}) {
            if (lc($Proc->{ExecutablePath}) eq lc($SqlServerExe))
            { $Found = 1; last; }
        }
    } else {
        my $pi = Proc::ProcessTable->new;
        foreach my $Proc (@{$pi->table}) {
            if (0 == index($Proc->cmndline, $SqlServerExe))
            { $Found = 1; last; }
        }
    }
    return $Found;
}

sub GetIPFromInterfaces {
    my ($Interfaces, $DBServer) = @_;

    my $sContent = GetFileContent($Interfaces) or return '';
    my $Line = $$sContent =~ /(^|\n)$DBServer(\s[^\000]*?)?\n\s+master\s+(.*)/ ? $3 : '';

    # tcp ether greenlinux.intern.epages.de 2512
    # tli tcp /dev/tcp \x000209d1ac1418090000000000000000
    my ($Proto, undef, $IP, $Port) = split(/\s+/, $Line);

    if ($Proto =~ /^tli$/i) {
        my ($Ip1, $Ip2, $Ip3, $Ip4) = $Port =~ /^.{10}(..)(..)(..)(..)/;
        $IP = hex($Ip1). '.'. hex($Ip2). '.'. hex($Ip3). '.'. hex($Ip4);
    }
    return $IP;
}

sub OwnConnection {
    my ($Connection, $IPs) = @_;

    my $hSection = DatabaseConnectionData($Connection);

    my $DBServer = $hSection->{datasource};
    return 0 unless $DBServer;

    if ($DBServer =~ /(^|;)server=(.+?)(;|$)/) {
        $DBServer = $2;
    } else {
        return 0;
    }

    if ($Config{osname} eq 'MSWin32') {
        my $File = "$ENV{SYBASE}/ini/sql.ini";
        my %Ini;
        tie %Ini, 'Config::IniFiles', (-file => $File) or die "$File: $!\n";
        (undef, $DBServer, undef ) = split (/,/, $Ini{$DBServer}{master});
    } else {
        $DBServer = GetIPFromInterfaces("$ENV{SYBASE}/interfaces", $DBServer);
    }
    return 0 unless $DBServer;

    my $Error = CheckIpFormat($DBServer, 4);
    $DBServer = inet_ntoa( scalar(gethostbyname($DBServer)) ) if $Error;
    return 0 unless $DBServer;
    return ElementOf($DBServer, $IPs);
}

sub CheckBackupType {
    my ($Store, $hSection, $Target, $Dump, $Errors) = @_;

    return ' -full' if $Target eq 'full_backup_db';

    my $MomentType = $Target eq 'initial_backup_db' ? 'Initial' : 'Final';
    $MomentType .= 'BackupType';

    if ($hSection->{BackupType} !~ /\S/ and $hSection->{$MomentType} !~ /\S/ ) {
        print "INFO: no \L$Dump\E: $Store: keys $MomentType and BackupType: empty\n";
        return;
    }
    if ($hSection->{$MomentType} =~ /\S/) {
        if ($hSection->{$MomentType} !~ /^-(log|full)$/) {
            $$Errors .= "\nERROR: $Store: $hSection->{$MomentType}: void $MomentType, use -full or -log\n";
            return;
        }
        return " $hSection->{$MomentType}";
    }
    if ($hSection->{BackupType} !~ /^-(log|full)$/) {
        $$Errors .= "\nERROR: $Store: $hSection->{BackupType}: void BackupType, use -full or -log\n";
        return;
    }
    return " $hSection->{BackupType}";
}

my $TargetsString;

sub Main {
    local $| = 1;

    my %Options = ();
    my @GetOpt = qw(help nooutput target=s all own);

    my @TargetList = qw(restore_db initial_backup_db final_backup_db full_backup_db);
    $TargetsString = join ', ', @TargetList;

    usage() unless GetOptions(\%Options, @GetOpt);
    usage() if defined $Options{help};

    usage() unless defined $Options{target};

    my ($Errors, $Dump, $InitialBackupDate);
    my $Target    = basename($Options{target});
    my $TargetDir = dirname($Options{target});

    usage() unless ElementOf($Target, @TargetList);
    if ( -f "$Options{target}" ) {
        print "INFO: $Target skipped, because file exists: $Options{target}\n";
        exit 0;
    }
    $Dump = $Target eq 'restore_db' ? 'Restore' : 'Backup';

    if ($Dump eq 'Restore') {
        $InitialBackupDate = InitialBackupFound($TargetDir, \%Options) or exit 0;
    }

    # check if sybase runs
    my $LocalDatabase = SybaseRuns();

    my $Script = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Database/Scripts/Sybase$Dump.pl";
    $Script = "Cartridges/DE_EPAGES/Database/Scripts/Sybase$Dump.pl"
        if -f "Cartridges/DE_EPAGES/Database/Scripts/Sybase$Dump.pl";

    my $HostName = hostname();
    my $IPs = Sys::HostIP->ips;
    push @$IPs, inet_ntoa( scalar( gethostbyname( $HostName )));

    my $StoreList = StoreConnections();

    HandleDump($_, $Target, $Dump, $InitialBackupDate, $LocalDatabase, $Script, $HostName, $IPs, \%Options, \$Errors) foreach @$StoreList;

    if ($Errors) {
        print STDERR "\nEXITING: Database $Dump Failed:\n";
        print STDERR "======================================\n";
        die $Errors;
    }

    UnlinkTargets($TargetDir, $Dump, \%Options) unless $Target eq 'full_backup_db';
}

sub InitialBackupFound {
    my ($TargetDir, $Options) = @_;

    my $InitialBackupFile = -f "$TargetDir/initial_backup_db" ? "$TargetDir/initial_backup_db" : '';
    if ($Options->{own} and not $InitialBackupFile) {
        $InitialBackupFile = dirname($TargetDir). '/initial_backup_db' if -f dirname($TargetDir). '/initial_backup_db';
    }
    if ( not $InitialBackupFile or not -f $InitialBackupFile ) {
        print "INFO: no DB restore: $TargetDir/initial_backup_db: no such file\n";
        return;
    }
    open my $File, '<', $InitialBackupFile or die "Error reading file '$InitialBackupFile'";
    my $InitialBackupDate = (<$File>)[0];
    close $File or die "Error closing file '$InitialBackupFile'";
    chomp($InitialBackupDate);

    return $InitialBackupDate;
}

sub UnlinkTargets {
    my ($TargetDir, $Dump, $Options) = @_;

    chdir $TargetDir or die "$TargetDir: $!";

    if ($Dump eq 'Backup') {
        unlink('restore_db');
        if ($Options->{all}) {
            unlink ("$_/restore_db") foreach <*>;
        }
    } else {
        unlink('initial_backup_db');
        unlink('final_backup_db');
    }

    my $TimeStamp = time2str('_%Y%m%d%H%M%S', time());

    open my $File, '>>', basename($Options->{target}) or
        die "Cannot write $Options->{target}:$!";
    print $File $TimeStamp;
    close $File or die "Error closing file '$File'";
}

sub HandleDump {
    my ($Store, $Target, $Dump, $InitialBackupDate, $LocalDatabase, $Script, $HostName, $IPs, $Options, $Errors) = @_;

    my $hSection = DatabaseConnectionData($Store);
    my $BackupStore = DatabaseBackupData($Store)->{'Backup'};

    return unless $BackupStore;
    return unless $hSection->{Database};
    my $DBName = $hSection->{Database};
    return if defined $hSection->{active} and not $hSection->{active};
    return unless $hSection->{datasource} =~ /^dbi:Sybase/;

    my $SectionHost = $hSection->{BackupHost};
    # go on if option own is defined but not own connection
    return if $SectionHost =~ /^(ipv6-)?localhost$/ and $Options->{own} and not OwnConnection($BackupStore, $IPs);
    $LocalDatabase = 1 if $Options->{all};

    # go on if host isn't localhost or myself
    return unless $SectionHost eq $HostName or $SectionHost =~ /^(ipv6-)?localhost$/;

    # go on if host is localhost and database isn't local
    if ($SectionHost =~ /^(ipv6-)?localhost$/ and not $LocalDatabase) {
        print "INFO: no DB \L$Dump\E: no Sybase process found on $HostName\n";
        return;
    }

    # check DumpDir

    my $DumpDir = $hSection->{DumpDir};
    $DumpDir =~ s/^_(.+?)_/$ENV{$1}/g;
    $DumpDir =~ s/\$ENV\{(.+?)\}/$ENV{$1}/g;
    if ($DumpDir !~ /\S/) {
        print "INFO: no \L$Dump\E: $Store: key DumpDir: empty\n";
        return;
    }
    CreateDirectory($DumpDir) unless -d $DumpDir;

    # check BackupType

    my $ScriptArgs = "-dumpdir $DumpDir -dbname $DBName -storename ";
    $ScriptArgs .= $Dump eq 'Restore' ? $BackupStore : $Store;
    $ScriptArgs .= ' -nooutput' if $Options->{nooutput};
    $ScriptArgs .= ' '. $hSection->{"${Dump}Args"} if $hSection->{"${Dump}Args"};

    my $BackupType = CheckBackupType($Store, $hSection, $Target, $Dump, $Errors);
    return if not $BackupType and $Dump eq 'Backup';
    $ScriptArgs .= $BackupType if $Dump eq 'Backup' or $BackupType =~ /-log/;

    if ($Dump eq 'Restore' and -d $DumpDir) {
        # move final backup dumps
        RunOnDirectory( $DumpDir, sub {
            foreach my $DumpFile ( glob "${DBName}_*.dmp" ) {
                my $TimeStamp = $DumpFile =~ /.*_([0-9]+)\.dmp$/ ? $1 : '';
                while (length($TimeStamp) < length($InitialBackupDate)) {
                    $TimeStamp .= '0';
                }
                rename $DumpFile, "$DumpFile.final_backup_db" if $TimeStamp > $InitialBackupDate;
            }
                        });
    }

    my $PerlExe = $Config{'osname'} eq 'MSWin32' ? $^X : $ENV{'PERL'};
    my $Command = "\"$PerlExe\" -IInclude $Script $ScriptArgs";
    SystemCommand( $Command, { 'Print' => !$Options->{nooutput} } );
}

sub usage {
    die <<END_USAGE;
Usage:
    perl $0 -target path/to/TARGET

Options:
    -target             TARGET must one of: $TargetsString
    -all                dump/restore even if no running sybase found on localhost
    -own                dump/restore only if own connection

Flags:
    -help               show the command line options
    -nooutput           do not print any output

Example:
    perl $0 -target restore_db
END_USAGE
}

RunScript(
    'Sub' => \&Main
);
