use strict;
use File::Basename  qw( dirname basename );
use Getopt::Long;
use Time::HiRes qw( time );
use DE_EPAGES::Core::API::File qw( fsplit MoveFile DeleteFile CorrectPathSlashes GetFileNames );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw(SystemCommand);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Program::Tools qw(FindExeInPath);

sub usage {
    my ($ErrorNote) = @_;

    print "Error:\n    $ErrorNote\n\n" if $ErrorNote;
    print <<END_USAGE;
Description:
    Compresses MySQL dumps *_YYYYMMDDHHMMSS.dmp to YYYY/MM subdirectories and
    removes files older than -delold days. Folder binlog is zipped inplace.

Usage:
    perl $0 [Options] [Flags]

Options:
    -dumpdir       directory to store backup files (must exist on the server)
    -delold        dumps older than days will be deleted
    -zipper        compression tool, use one of (default: gzip):
                   gzip bzip2 compress lzop xz echo touch
Flags:
    -help          show the command line options
    -nooutput      do not print any output
    -dryrun        perform a trial run with no changes made

Example (use default compression: gzip -f1):
    perl $0 -dumpdir $ENV{EPAGES}/dumps/mysqldumps -delold 24
Example (use compression: bzip -9):
    perl $0 -dumpdir $ENV{EPAGES}/dumps/mysqldumps -delold 24 -zipper "bzip -9"
Example (use no compression):
    perl $0 -dumpdir $ENV{EPAGES}/dumps/mysqldumps -delold 24 -zipper touch
END_USAGE
    exit 2;
}

sub Main {
    local $| = 1;

    my %Options = ();
    my @GetOpt = qw(help output! dumpdir=s delold=i days=i test dryrun zipper=s extension=s);

    usage() unless GetOptions(\%Options, @GetOpt);
    usage() if defined $Options{help};
    usage('Missing parameter -dumpdir') unless defined $Options{dumpdir};
    usage("No such directory: -dumpdir \"$Options{dumpdir}\"") unless -d $Options{dumpdir};

    $Options{output} //= 1;
    $Options{zipper} //= 'gzip -f1';
    $Options{extension} = undef;              # legacy mode
    $Options{delold} //= $Options{days};      # legacy mode
    $Options{dryrun} //= $Options{test};      # legacy mode

    my $Exe = (split /\s+/, $Options{zipper})[0];
    usage("Compression tool not found: $Exe") unless -x $Exe or FindExeInPath($Exe);
    usage("Unsupported compression tool: $Exe") unless defined getSuffix($Exe);

    my $time1 = time();

    my $NonArchivedDumps = getDumpFiles($Options{dumpdir});
    my $ArchivableDumps = removeLastFullDumpFromList($NonArchivedDumps, \%Options);
    ArchiveDumpsToDateDir($ArchivableDumps, \%Options);
    ArchiveBinlogs(\%Options);
    DeleteOldDumps(\%Options);
    DeleteOldBinlogs(\%Options);

    printf ("\nneed %.1f seconds\n", time() - $time1) if $Options{output};
}

#========================================================================================
# function     getSuffix
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Suffix = getSuffix($CommandLine);
#----------------------------------------------------------------------------------------
# description  Returns the default $Suffix for compression tool in $CommandLine
#----------------------------------------------------------------------------------------
# input        $CommandLine | command line starting with compression tool | string
# return       $Suffix      | default suffic for compression tool | string
#========================================================================================
sub getSuffix {
    my ($CommandLine) = @_;

    my $CompressionTool = (split /\s+/, $CommandLine)[0];
    $CompressionTool = (fsplit $CompressionTool)[1];

    my %Suffix = ( 'gzip' => '.gz',
                   'bzip2' => '.bz2',
                   'compress' => '.Z',
                   'lzop' => '.lzo',
                   'xz' => '.xz',
                   'echo' => '',
                   'touch' => '',
        );

    return $Suffix{$CompressionTool};
}

#========================================================================================
# function     getDumpFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $DumpFiles = getDumpFiles($DumpDir, $hFlags);
#----------------------------------------------------------------------------------------
# description  Returns a list of all files ending in .dmp in $DumpDir.
#----------------------------------------------------------------------------------------
# input        $DumpDir | directory with dump files | string
# input        $hFlags  | flags
#                   <ul>
#                       <li>recursive - returns files in subdirectories as well
#                   </ul>
#               | ref.hash
# return       $DumpFiles | list of file names | ref.array
#========================================================================================
sub getDumpFiles {
    my ($DumpDir, $hFlags) = @_;

    return [grep {$_ =~ /\b.*\.(dmp|dmp\..+)$/} @{GetFileNames($DumpDir,$hFlags)}];
}

#========================================================================================
# function     removeLastFullDumpFromList
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ArchivableDumps = removeLastFullDumpFromList($DumpFiles);
#----------------------------------------------------------------------------------------
# description  Returns a list of unzipped full dumps. The list contains only files named
#               *_YYYYMMDDHHMMSS.dmp* and does not contain the newest full dump for each
#               database. If both FILE.dmp and FILE.dmp.* exist, delete FILE.dmp.*
#----------------------------------------------------------------------------------------
# input        $DumpFiles       | list of dump files | ref.array
# input        $Options         | script options | ref.hash
# return       $ArchivableDumps | list of dump files (w/o newest) | ref.array
#========================================================================================
sub removeLastFullDumpFromList {
    my ($DumpFiles, $Options) = @_;

    my @ArchivableDumps;
    my $lastDB = '';
    foreach my $DumpFile ( sort @$DumpFiles ) {
        my $Uncompressed = $DumpFile;
        $Uncompressed =~ s,\.dmp\.[^/]+$,.dmp,;
        if ($DumpFile ne $Uncompressed and ElementOf($Uncompressed, $DumpFiles)) {
            print "DeleteFile($DumpFile)\n" if $Options->{output};
            DeleteFile($DumpFile) unless $Options->{dryrun};
            next;
        }
        my $DB = basename($DumpFile);
        next unless $DB =~ s/(_\d{14}\.dmp).*//;
        pop @ArchivableDumps if ($DB ne $lastDB and scalar @ArchivableDumps);
        push @ArchivableDumps, $DumpFile;
        $lastDB = $DB;
    }
    pop @ArchivableDumps;
    return \@ArchivableDumps;
}

#========================================================================================
# function     ArchiveDumpsToDateDir
# state        private
#----------------------------------------------------------------------------------------
# syntax       ArchiveDumpsToDateDir($ArchivableDumps, $Options);
#----------------------------------------------------------------------------------------
# description  Compresses all dumps in $ArchivableDumps (named *_YYYYMMDDHHMMSS.dmp*)
#               and moves them to directory Archive/YYYY/MM.
#----------------------------------------------------------------------------------------
# input        $ArchivableDumps | list of dump files to archive | ref.array
# input        $Options         | script options | ref.hash
#========================================================================================
sub ArchiveDumpsToDateDir {
    my ($ArchivableDumps, $Options) = @_;

    foreach my $DumpFile ( sort @$ArchivableDumps ) {
        my $cmd = "$Options->{zipper} ". CorrectPathSlashes($DumpFile);
        # don't compress already zipped files
        if ($DumpFile =~ /dmp$/) {
            print "$cmd\n" if $Options->{output};
            SystemCommand($cmd) unless $Options->{dryrun};
            $DumpFile .= getSuffix($Options->{zipper});
        }
        my $Basename = basename($DumpFile);
        next unless $Basename =~ /_(\d{4})(\d{2})/;
        my $TargetDir = dirname($DumpFile). "/Archive/$1/$2";
        my $TargetFile = "$TargetDir/$Basename";

        print "MoveFile($DumpFile, $TargetDir)\n" if $Options->{output};
        MoveFile($DumpFile, $TargetFile, {create_directory => 1}) unless $Options->{dryrun};
    }
}

#========================================================================================
# function     DeleteOldDumps
# state        private
#----------------------------------------------------------------------------------------
# syntax       DeleteOldDumps($Options);
#----------------------------------------------------------------------------------------
# description  Deletes old dump files named *_YYYYMMDDHHMMSS.dmp* in directory Archive/.
#----------------------------------------------------------------------------------------
# input        $Options | script options | ref.hash
#========================================================================================
sub DeleteOldDumps {
    my ($Options) = @_;

    return unless defined $Options->{delold};
    return unless -d "$Options->{dumpdir}/Archive";

    foreach my $DumpFile (sort @{getDumpFiles("$Options->{dumpdir}/Archive", {'recursive' => 1})}) {
        my $Basename = basename($DumpFile);
        next unless $Basename =~ /_(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})/;
        my $dumpdate = DateTime->new(
            year => $1, month  => $2, day => $3,
            hour => $4, minute => $5);
        my $age = DateTime->now->delta_days($dumpdate)->delta_days;
        next if $age < $Options->{delold};

        print "DeleteFile($DumpFile)\n" if $Options->{output};
        DeleteFile($DumpFile) unless $Options->{dryrun};
    }
}

#========================================================================================
# function     ArchiveBinlogs
# state        private
#----------------------------------------------------------------------------------------
# syntax       ArchiveBinlogs($Options);
#----------------------------------------------------------------------------------------
# description  Compresses all binlog files in $Options->{dumpdir}/binlog/
#----------------------------------------------------------------------------------------
# input        $Options | script options | ref.hash
#========================================================================================
sub ArchiveBinlogs {
    my ($Options) = @_;

    foreach my $BinLogFile ( sort @{getBinLogFiles($Options)} ) {
        my $cmd = "$Options->{zipper} ". CorrectPathSlashes($BinLogFile);
        print "$cmd\n" if $Options->{output};
        SystemCommand($cmd) unless $Options->{dryrun};
    }
}

#========================================================================================
# function     getBinLogFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $BinLogFiles = getBinLogFiles($DumpDir, $hFlags);
#----------------------------------------------------------------------------------------
# description  Returns all uncompressed files in $DumpDir/binlog/.
#----------------------------------------------------------------------------------------
# input        $Options | script options | ref.hash
# input        $Zipped | if set, returns all compressed files | boolean
# return       $BinLogFiles | list of file names | ref.array
#========================================================================================
sub getBinLogFiles {
    my ($Options, $Zipped) = @_;

    return [] unless -d "$Options->{dumpdir}/binlog";

    my $Suffix = getSuffix($Options->{zipper});
    my $Pattern = "\.(gz|bz2|Z|lzo|zip|xz)|$Suffix";
    $Pattern = '\b.*('. $Pattern. ')$';
    return [grep {/$Pattern/} @{GetFileNames("$Options->{dumpdir}/binlog",{'recursive' => 1 })}] if $Zipped;
    return [grep {$_ !~ /$Pattern/} @{GetFileNames("$Options->{dumpdir}/binlog",{'recursive' => 1 })}];
}

#========================================================================================
# function     DeleteOldBinlogs
# state        private
#----------------------------------------------------------------------------------------
# syntax       DeleteOldBinlogs($Options);
#----------------------------------------------------------------------------------------
# description  Deletes old binlog files in $Options->{dumpdir}/binlog/
#----------------------------------------------------------------------------------------
# input        $Options | script options | ref.hash
#========================================================================================
sub DeleteOldBinlogs {
    my ($Options) = @_;

    return unless defined $Options->{delold};
    my $DeleteableDumps = getBinLogFiles($Options, 1);

    foreach my $DumpFile ( sort @$DeleteableDumps ) {
        next if -M $DumpFile < $Options->{delold};
        print "DeleteFile($DumpFile)\n" if $Options->{output};
        DeleteFile($DumpFile) unless $Options->{dryrun};
    }
}

RunScript( 'Sub' => \&Main, 'Message' => 'long' );

1;
