#!perl -w
use Test::More;
use strict;
use Config;
use Config::IniFiles;
use Data::Compare qw(Compare);
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::File qw ( AbsolutePath WriteFile DeleteFile CreateDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );
use DE_EPAGES::Core::API::Warning qw( BlockWarnings );
use DE_EPAGES::Database::API::Config qw( GetRootConnections StoreConnections StoreTypes
                                         GetBusinessUnits SetBusinessUnits );
use DE_EPAGES::Database::API::Connection qw(CreateStoreConf);


sub Main {
    my $RootConnections = GetRootConnections();
    my $DataSource = 'dbi:mysql';
    my $Passwd = 'epages';
    my $Backup = scalar @$RootConnections ? $RootConnections->[0] : 'Backup';

    if (-f "$ENV{EPAGES_CONFIG}/Database.d/$Backup.conf") {
        tie my %Ini, 'Config::IniFiles', (-file => "$ENV{EPAGES_CONFIG}/Database.d/$Backup.conf");
        if ($Ini{Connection}{datasource}) {
            if ($Ini{Connection}{datasource} =~ s/^dbi:mysql://) {
                my @Parameters;
                foreach (split(/;/,$Ini{Connection}{datasource})) {
                    # Parameters must not contain the database name
                    next if /^database=/;
                    push @Parameters, $_;
                }
                $DataSource .= ':'. join(';', @Parameters) if scalar @Parameters;
            }
        }
        $Passwd = $Ini{Connection}{passwd} // 'epages';
    }

    my %Configuration = (
    ASPoolDB => <<'END_FILE',
[Connection]
active=1
datasource=dbi:SQLite:dbname=_EPAGES_CONFIG_/ASPool.db
StoreType=ASPoolDB
END_FILE
    Backup => <<"END_FILE",
[Connection]
active=1
datasource=$DataSource
Database=mysql
login=root
passwd=$Passwd
END_FILE
    Backup1 => <<"END_FILE",
[Connection]
active=1
datasource=dbi:mysql:host1
Database=mysql1
login=root
passwd=$Passwd
END_FILE
    Backup2 => <<"END_FILE",
[Connection]
active=1
datasource=$DataSource
Database=mysql
login=root
passwd=epages1
END_FILE
    Backup3 => <<"END_FILE",
[Connection]
active=1
datasource=$DataSource
Database=mysql
login=root
passwd=$Passwd
END_FILE
    Root => <<"END_FILE",
[Connection]
active=1
datasource=$DataSource
Database=mysql
login=root
passwd=$Passwd
END_FILE
    EbayCategories => <<'END_FILE',
[Connection]
active=1
datasource=dbi:SQLite:dbname=_EPAGES_CONFIG_/EbayCategories.db
END_FILE
    Site => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Site
Database=sitedb
login=usr_sitedb
passwd=epages
END_FILE
    Store => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Store
Database=storedb
login=usr_storedb
passwd=epages
END_FILE
    );

    my $PerlScript = AbsolutePath($0);

    if( $Config{'osname'} ne 'MSWin32' ) {
        if ((getpwuid($>))[0] eq 'root') {
            my $Command = "su - ep_appl -c \"$ENV{PERL} $PerlScript\"";
            print $Command, "\n";
            system("$Command");
            return;
        }
    }

    plan tests => 17;

    my $ConfigDir = tempdir();
    my $DBDir ="$ConfigDir/Database.d";

    CreateDirectory($DBDir, { recursive => 1 });

    local $ENV{EPAGES_CONFIG} = $ConfigDir;

    foreach my $Store (keys %Configuration) {
        WriteFile( "$DBDir/$Store.conf", \$Configuration{$Store} );
    }

    my (@Expected, @Found);

    foreach my $Store (sort keys %Configuration) {
        next unless $Configuration{$Store} =~ /StoreType/s;
        push @Expected, $Store;
    }
    @Found = sort @{StoreTypes()};

    # test 1: did we found all stores with StoreType?
    ok(Compare(\@Found, \@Expected), "(@Found) ~~ (@Expected)");

    @Expected = (sort keys %Configuration);
    @Found = sort @{StoreConnections()};

    # test 2: did we found all store connections?
    ok(Compare(\@Found, \@Expected), "(@Found) ~~ (@Expected)");

    chmod oct(111), "$DBDir/Site.conf"; ## no critic qw(ProhibitMagicNumber)
    my $aWarnings = BlockWarnings(sub {@Found = sort @{StoreTypes()}});

    SKIP: {
         skip 'No warnings expected on Windows', 2 if $Config{'osname'} eq 'MSWin32';

        # test 3: did we got the expected warning?
        is($aWarnings->[0]->code, 'FileReadFailed', 'warning code');

        # test 4: with the expected error file?
        is($aWarnings->[0]->vars->{FileName}, 'Site.conf', 'warning file name');
    }

    @Expected = ();
    foreach my $Store (sort keys %Configuration) {
        next if $Store eq 'Site' && $Config{'osname'} ne 'MSWin32';
        next unless $Configuration{$Store} =~ /StoreType/s;
        push @Expected, $Store;
    }

    # test 5: did we found all stores but Site with StoreType?
    ok(Compare(\@Found, \@Expected), "(@Found) ~~ (@Expected)");

    @Expected = ();
    foreach my $Store (sort keys %Configuration) {
        next if $Store eq 'Site' && $Config{'osname'} ne 'MSWin32';
        push @Expected, $Store;
    }
    @Found = sort @{StoreConnections()};

    # test 6: did we found all store connections but Site?
    ok(Compare(\@Found, \@Expected), "(@Found) ~~ (@Expected)");

    chmod oct(640), "$DBDir/Site.conf"; ## no critic qw(ProhibitMagicNumber)

    # test 7: get business units from environment
    $ENV{EPAGES_BUSINESS_UNITS} = 'foobar'; ## no critic qw(RequireLocalizedPunctuationVars)
    my $GetBUFromEnv = GetBusinessUnits({env => 1});
    ok($ENV{EPAGES_BUSINESS_UNITS} eq join(' ', @$GetBUFromEnv),
       "GetBusinessUnits({env => 1}) : (@$GetBUFromEnv) ~~ ($ENV{EPAGES_BUSINESS_UNITS})");

    # test 8: get business units from config files
    my $GetBUFromConf = GetBusinessUnits({conf => 1});
    @Expected = qw(Site Store);
    ok(Compare(\@Expected, $GetBUFromConf),
       "GetBusinessUnits({conf => 1}) : (@Expected) ~~ (@$GetBUFromConf)");

    my $DefaultFile = "/etc/default/$ENV{EPAGES_PROJECT}";
    SKIP: {
         skip "No such file for Windows: $DefaultFile", 2 if $Config{'osname'} eq 'MSWin32';

         SystemCommand("cat $DefaultFile > /tmp/test.~$$~", {Print => 1});

         # test 9: set business units from environment in /etc/default/epages6
         my $Env = SetBusinessUnits($GetBUFromEnv);
         ok($Env eq $ENV{EPAGES_BUSINESS_UNITS},
            "SetBusinessUnits(@$GetBUFromEnv) -> EPAGES_BUSINESS_UNITS='$Env'");

         # test 10: set business units from config files in /etc/default/epages6
         $Env = SetBusinessUnits($GetBUFromConf);
         ok($Env eq $ENV{EPAGES_BUSINESS_UNITS},
            "SetBusinessUnits(@$GetBUFromConf) -> EPAGES_BUSINESS_UNITS='$Env'");

         SystemCommand("cat /tmp/test.~$$~ > $DefaultFile && rm -f /tmp/test.~$$~",
                       {Print => 1});
    }

    my %hConn;
    my %hSettings = (Store=>'PatchSystemInfoDB', login=>'usr_patch', BackupHost=>'none');
    @Expected = qw(Backup=Backup BackupHost=none Database=patchsysteminfodb
                   active=1 datasource=dbi:mysql login=usr_patch passwd=epages);
    $Expected[4] = "datasource=$DataSource";

    my ($ExpectedConn, $FoundConn);
    $ExpectedConn = [qw(Backup Backup2 Backup3 Root)];
    $FoundConn = GetRootConnections();
    ok(Compare($ExpectedConn, $FoundConn), 'GetRootConnections() => '. join(' ', @$FoundConn));

    eval { CreateStoreConf(\%hSettings, {connect => 1}); };
    SKIP: {
      skip GetError()->message, 6 if ExistsError();

      tie %hConn, 'Config::IniFiles', (-file => "$DBDir/$hSettings{Store}.conf") or die $!;

      @Found = map { "$_=$hConn{Connection}{$_}" } sort keys %{$hConn{Connection}};
      ok(Compare(\@Found, \@Expected),
         "CreateStoreConf(PatchSystemInfoDB, {connect => 1}) => by $hConn{Connection}{Backup}");

      DeleteFile("$DBDir/Backup.conf");
      shift @$ExpectedConn;
      $FoundConn = GetRootConnections();
      ok(Compare($ExpectedConn, $FoundConn),
         'GetRootConnections() => '. join(' ', @$FoundConn));

      # do not check backup connection -> Backup2
      $Expected[0] = 'Backup=Backup2';

      DeleteFile("$DBDir/$hSettings{Store}.conf");
      CreateStoreConf(\%hSettings);
      tie %hConn, 'Config::IniFiles', (-file => "$DBDir/$hSettings{Store}.conf") or die $!;

      @Found = map { "$_=$hConn{Connection}{$_}" } sort keys %{$hConn{Connection}};
      ok(Compare(\@Found, \@Expected),
         "CreateStoreConf(PatchSystemInfoDB) => by $hConn{Connection}{Backup}");

      DeleteFile("$DBDir/Backup3.conf");
      $ExpectedConn = [qw(Backup2 Root)];
      $FoundConn = GetRootConnections();
      ok(Compare($ExpectedConn, $FoundConn),
         'GetRootConnections() => '. join(' ', @$FoundConn));

      # check backup connection -> Root
      $Expected[0] = 'Backup=Root';
      DeleteFile("$DBDir/$hSettings{Store}.conf");
      CreateStoreConf(\%hSettings, { connect => 1 });
      tie %hConn, 'Config::IniFiles', (-file => "$DBDir/$hSettings{Store}.conf") or die $!;

      @Found = map { "$_=$hConn{Connection}{$_}" } sort keys %{$hConn{Connection}};
      ok(Compare(\@Found, \@Expected),
         "CreateStoreConf(PatchSystemInfoDB, {connect => 1}) => by $hConn{Connection}{Backup}");

      # check backup connection -> Backup2
      $Expected[0] = 'Backup=Backup2';
      DeleteFile("$DBDir/$hSettings{Store}.conf");
      CreateStoreConf(\%hSettings);
      tie %hConn, 'Config::IniFiles', (-file => "$DBDir/$hSettings{Store}.conf") or die $!;

      @Found = map { "$_=$hConn{Connection}{$_}" } sort keys %{$hConn{Connection}};
      ok(Compare(\@Found, \@Expected),
         "CreateStoreConf(PatchSystemInfoDB) => by $hConn{Connection}{Backup}");
    }
}

RunScript(Sub=>\&Main);
