#==============================================================================
# package      DE_EPAGES::Database::t::LockTables
# state        public
# description  Test class for lock tables functionality.
#==============================================================================
package DE_EPAGES::Database::t::LockTables;
use base DE_EPAGES::Database::API::Test;

use strict;

use threads;

use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use Test::More;
use Data::Dumper;

use constant TABLE_NAME => 'galeratest';
use constant FIELD1     => 'primary_key';
use constant FIELD2     => 'value';

sub test {
    my $self = shift;

    $self->test_lockTables_shortLock();
    $self->test_lockTables_longLock();
    $self->test_lockTables_twoLocks();
    done_testing();

    return;
}

#========================================================================================
# function     test_lockTables_shortLock
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test_lockTables_shortLock()
#----------------------------------------------------------------------------------------
# description  Test case to verify if a short table lock in conjunction with a
#               concurrent update does work correctly and produces an expected result.
#               Two threads are used to simultaneously alter the same talbe.
#========================================================================================
sub test_lockTables_shortLock {
    my $self = shift;

    my $Store = GetTestSite();

    # This first thread performs an update using a table lock. In this thread all entries
    # are updated to ensure that a conflict exists with any other update on the test
    # talbe. After the update is completed, the thread sleeps for 2 seconds. This is done
    # to provoke other threads to be run, while the lock is still in place.
    my $LockTablesThread = $self->_runThreadOnStore(
        $Store,
        sub {
            $self->dbi->lockTables(
                [ { 'table' => TABLE_NAME, 'mode' => 'WRITE' } ],
                sub {
                    $self->dbi->do( 'UPDATE ' . TABLE_NAME . ' SET ' . FIELD2 . ' = ?;', ['modified once'] );
                    sleep(2);
                }
            );
        }
    );

    # In this second thread one entry of the test table is altered. The thread sleeps
    # for one second before starting its work, in order to provoke that the table lock
    # of the first thread is in place, before updating.
    my $UpdateThread = $self->_runThreadOnStore(
        $Store,
        sub {
            sleep(1);
            $self->dbi->do( 'UPDATE ' . TABLE_NAME . ' SET ' . FIELD2 . ' = ? WHERE ' . FIELD1 . ' = ?;',
                [ 'modified twice', 2 ] );
        }
    );

    my $ResultThread1 = $LockTablesThread->join();
    my $ResultThread2 = $UpdateThread->join();

    my $TestName = 'lockTables short lock';

    is( $ResultThread1, 'success', "$TestName result thread 1" );
    is( $ResultThread2, 'success', "$TestName result thread 2" );

    my $hExpectedIdsAndValues = {
        '1' => ['modified once'],
        '2' => [ 'modified once', 'modified twice' ],
        '3' => ['modified once']
    };

    $self->_checkEntriesInTableGaleraTest( $hExpectedIdsAndValues, $TestName );

    return;
}

#========================================================================================
# function     test_lockTables_longLock
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test_lockTables_longLock()
#----------------------------------------------------------------------------------------
# description  Test case to verify if a long table lock in conjunction with a
#               concurrent update does work correctly and produces an expected result.
#               Two threads are used to simultaneously alter the same talbe.
#========================================================================================
sub test_lockTables_longLock {
    my $self = shift;

    my $Store = GetTestSite();

    # This first thread performs an update using a table lock. In this thread all entries
    # are updated to ensure that a conflict exists with any other update on the test
    # talbe. After the update is completed, the thread sleeps for 10 seconds. This is done
    # to provoke other threads to be run, while the lock is still in place.
    my $LockTablesThread = $self->_runThreadOnStore(
        $Store,
        sub {
            $self->dbi->lockTables(
                [ { 'table' => TABLE_NAME, 'mode' => 'WRITE' } ],
                sub {
                    $self->dbi->do( 'UPDATE ' . TABLE_NAME . ' SET ' . FIELD2 . ' = ?;', ['modified three times'] );
                    sleep(10);    ## no critic qw(ProhibitMagicNumbers)
                }
            );
        }
    );

    # In this second thread one entry of the test table is altered. The thread sleeps
    # for one second before starting its work, in order to provoke that the table lock
    # of the first thread is in place, before updating.
    my $UpdateThread = $self->_runThreadOnStore(
        $Store,
        sub {
            sleep(1);
            $self->dbi->do( 'UPDATE ' . TABLE_NAME . ' SET ' . FIELD2 . ' = ? WHERE ' . FIELD1 . ' = ?;',
                [ 'modified four times', 2 ] );
        }
    );

    my $ResultThread1 = $LockTablesThread->join();
    my $ResultThread2 = $UpdateThread->join();

    my $TestName = 'lockTables long lock';

    is( $ResultThread1, 'success', "$TestName result thread 1" );
    is( $ResultThread2, 'success', "$TestName result thread 2" );

    my $hExpectedIdsAndValues = {
        '1' => ['modified three times'],
        '2' => [ 'modified three times', 'modified four times' ],
        '3' => ['modified three times']
    };

    $self->_checkEntriesInTableGaleraTest( $hExpectedIdsAndValues, $TestName );

    return;
}

#========================================================================================
# function     test_lockTables_twoLocks
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test_lockTables_twoLocks()
#----------------------------------------------------------------------------------------
# description  Test case to verify if two concurrenct table locks produce an expected
#               result. Two threads are used to simultaneously alter the same talbe.
#========================================================================================
sub test_lockTables_twoLocks {
    my $self = shift;

    my $Store = GetTestSite();

    # This first thread performs an update using a table lock. In this thread all entries
    # are updated to ensure that a conflict exists, with any other update on the test
    # talbe. After the update is completed, the thread sleeps for 10 seconds. This is done
    # to provoke other threads to be run, while the lock is still in place.
    my $LockTablesThread = $self->_runThreadOnStore(
        $Store,
        sub {
            $self->dbi->lockTables(
                [ { 'table' => TABLE_NAME, 'mode' => 'WRITE' } ],
                sub {
                    $self->dbi->do( 'UPDATE ' . TABLE_NAME . ' SET ' . FIELD2 . ' = ?;', ['modified five times'] );
                    sleep(10);    ## no critic qw(ProhibitMagicNumbers)
                }
            );
        }
    );

    # In this second thread one entry of the test table is altered by also using a table
    # lock. The thread sleeps for one second before starting its work, in order to provoke
    # that the table lock of the first thread is in place, before trying to aquire the
    # second table lock.
    my $UpdateThread = $self->_runThreadOnStore(
        $Store,
        sub {
            $self->dbi->lockTables(
                [ { 'table' => TABLE_NAME, 'mode' => 'WRITE' } ],
                sub {
                    sleep(1);
                    $self->dbi->do( 'UPDATE ' . TABLE_NAME . ' SET ' . FIELD2 . ' = ? WHERE ' . FIELD1 . ' = ?;',
                        [ 'modified six times', 2 ] );
                }
            );
        }
    );

    my $ResultThread1 = $LockTablesThread->join();
    my $ResultThread2 = $UpdateThread->join();

    my $TestName = 'lockTables two locks';

    is( $ResultThread1, 'success', "$TestName result thread 1" );
    is( $ResultThread2, 'success', "$TestName result thread 2" );

    my $hExpectedIdsAndValues = {
        '1' => ['modified five times'],
        '2' => [ 'modified five times', 'modified six times' ],
        '3' => ['modified five times']
    };

    $self->_checkEntriesInTableGaleraTest( $hExpectedIdsAndValues, $TestName );

    return;
}

#========================================================================================
# function     init
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->init()
#----------------------------------------------------------------------------------------
# description  Init method to create a test table with sample entries.
#========================================================================================
sub init {
    my $self = shift;

    $self->dbi->do( 'CREATE TABLE IF NOT EXISTS '
          . TABLE_NAME . '('
          . FIELD1
          . ' INTEGER NOT NULL PRIMARY KEY, '
          . FIELD2
          . ' VARCHAR(255)  NULL);' );

    $self->dbi->do( 'INSERT INTO ' . TABLE_NAME . ' (' . FIELD1 . ', ' . FIELD2 . ') VALUES (?, ?);',
        [ 1, 'original value' ] );
    $self->dbi->do( 'INSERT INTO ' . TABLE_NAME . ' (' . FIELD1 . ', ' . FIELD2 . ') VALUES (?, ?);',
        [ 2, 'original value' ] );
    $self->dbi->do( 'INSERT INTO ' . TABLE_NAME . ' (' . FIELD1 . ', ' . FIELD2 . ') VALUES (?, ?);',
        [ 3, 'original value' ] );

    return;
}

#========================================================================================
# function     cleanup
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->cleanup()
#----------------------------------------------------------------------------------------
# description  Cleanup method to delete test table, if it exists.
#========================================================================================
sub cleanup {
    my $self = shift;

    if ( scalar @{ $self->dbi->execute( 'SHOW TABLES LIKE \'' . TABLE_NAME . '\'' ) } ) {
        $self->dbi->do( 'DROP TABLE ' . TABLE_NAME . ';' );
    }

    return;
}

#========================================================================================
# function     _runThreadOnStore
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_runThreadOnStore('Store', sub {...})
#----------------------------------------------------------------------------------------
# description  Helper method to create and run a thread on a given store.
#----------------------------------------------------------------------------------------
# input        $Store | The store the thread will run on | object
# input        $rSub | The code that will be run in the thread | ref.sub
# return       $ReturnValue | Created thread returns error code or 'success' otherwiese | string
#========================================================================================
sub _runThreadOnStore {
    my $self = shift;
    my ( $Store, $rSub ) = @_;

    return threads->create(
        sub {
            my $ReturnValue;
            RunOnStore(
                Store => $Store,
                Sub   => sub {
                    eval { $rSub->(); };
                    if ( ExistsError() ) {
                        print "\n\n\nError found\n\n\n";
                        my $Error = GetError();
                        LogError($Error);
                        $ReturnValue = $Error->code;
                    }
                    else {
                        $ReturnValue = 'success';
                    }
                }
            );
            return $ReturnValue;
        }
    );
}

#========================================================================================
# function     _checkEntriesInTableGaleraTest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_checkEntriesInTableGaleraTest(
#                   {'id1' => ['expectedValue1', 'expectedValue2'},
#                   'MyTestName'
#               );
#----------------------------------------------------------------------------------------
# description  Helper method to check, if the value with the given id has one of the
#               expected values.
#----------------------------------------------------------------------------------------
# input        $hIdAndValues | Hash with ids and expected values. | ref.hash
# input        $TestName | Test name that will be printed. | string
#========================================================================================
sub _checkEntriesInTableGaleraTest {
    my $self = shift;
    my ( $hIdAndValues, $TestName ) = @_;

    my $SelectWhereField1 = 'SELECT ' . FIELD2 . ' FROM ' . TABLE_NAME . ' WHERE ' . FIELD1 . ' = ?';

    foreach my $Id ( keys %{$hIdAndValues} ) {
        # Retrieve value for given id from the db.
        my $Value = $self->dbi->execute( $SelectWhereField1, [$Id] )->[0][0];
        # Get expected values for given id.
        my @ExpectedValues = @{ $hIdAndValues->{$Id} };
        # Check if value in the db is one of the expected values.
        ok( ElementOf($Value, @ExpectedValues),
            "$TestName entry with id $Id: Value = $Value, expected values = @ExpectedValues" );
    }

    return;
}

__PACKAGE__->runScript();
1;
