#==============================================================================
# package      DE_EPAGES::Diagnostics::API::Diagnostics
# state        public
# description  subs to get cartridge information (before in /UI/Cartridge.pm)
#==============================================================================
package DE_EPAGES::Diagnostics::API::Diagnostics;

use strict;
use base ('Exporter');

our @EXPORT_OK = qw(
  HashCartrigde
  DependenciesLoop
  CartridgesLoop
  CartridgeVars
);

use DE_EPAGES::Cartridge::API::Table::Cartridge qw( ExistsCartridgeByPackage GetInfoCartridgeByPackage );
use DE_EPAGES::Core::API::CartridgeConfig qw( CartridgeConfig );
use DE_EPAGES::Core::API::File qw( fsplit );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf UsePackage );
use DE_EPAGES::Database::API::Config qw( StoreType );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Diagnostics::API::HelpCenter qw( GetModelsInfoFromDocRoot GetApiInfoFromDocRoot );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );

#========================================================================================
# function     _CartridgeInstaller
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CartridgeInstaller = _CartridgeInstaller( $Package );
#----------------------------------------------------------------------------------------
# description  Builds a new cartridge object
#----------------------------------------------------------------------------------------
# input        $Package | package (cartridge) name | string
#========================================================================================
# return       $CartridgeInstaller | cartridge object | object
#========================================================================================
sub _CartridgeInstaller ($) {
    my ($Package) = @_;

    my $CartridgePackage = $Package . '::API::Cartridge';
    UsePackage($CartridgePackage);

    my $StoreName = GetCurrentDBHandle()->cacheKey;
    return $CartridgePackage->new(
        StoreName => $StoreName,
        StoreType => StoreType($StoreName),
    );
}

#========================================================================================
# function     HashCartrigde
#----------------------------------------------------------------------------------------
# syntax       $hCartridge = HashCartrigde( $Package );
#----------------------------------------------------------------------------------------
# description  Builds a hash with the cartridge information
#----------------------------------------------------------------------------------------
# input        $Package | package (cartridge) name | string
#========================================================================================
# return       $hCartridge | cartridge information | ref.hash
#========================================================================================
sub HashCartrigde ($) {
    my ($Package) = @_;

    my $hCartridge = ExistsCartridgeByPackage($Package)
      ? GetInfoCartridgeByPackage($Package)
      : {};

    $hCartridge->{'Installer'} = _CartridgeInstaller($Package);
    $hCartridge->{'Package'}   = $Package;

    return $hCartridge;
}

#========================================================================================
# function     DependenciesLoop
#----------------------------------------------------------------------------------------
# syntax       $aRequiredCartridges = DependenciesLoop( $hCartridge );
#----------------------------------------------------------------------------------------
# description  Gets all dependencies and thei info from a cartridge hash
#========================================================================================
# input        $hCartridge | result of the function Hashcartridge. | ref.hash
#========================================================================================
# return       $aRequiredCartridges | Dependencies | ref.array
#========================================================================================
sub DependenciesLoop {
    my ($hCartridge) = @_;

    my $aRequiredCartridges =
      $hCartridge->{'Installer'}->directRequiredCartridges;

    return [
        sort { $a->{Cartridge}->{Package} cmp $b->{Cartridge}->{Package} }
          map {
            {
                Cartridge => InstallerCartridgeVars($_),

# Exception: Ignore core cartridges that don't have a cartridge id. (e.g. DE_EPAGES::Trigger)
# Reason   : They are not registered at table 'cartridge' and therefore no comment can be found there!
# Caused by: DE_EPAGES::Cartridge::API::CartridgeInstaller::insertRequiredCartridges
                Comment => defined $_->id
                ? $hCartridge->{'Installer'}
                  ->dependencyComment( $_->cartridgePackage,
                    { 'DependOnActualCartridge' => '0' } )
                : undef,
            }
          } @$aRequiredCartridges
    ];
}

#========================================================================================
# function     InstallerCartridgeVars
#----------------------------------------------------------------------------------------
# syntax       $InstallerCartrigeVars = InstallerCartridgeVars( $Installer );
#----------------------------------------------------------------------------------------
# description  Returns InstallerCartridgeVars
#----------------------------------------------------------------------------------------
# input        $Installer | Hash with installer information | ref.hash
#========================================================================================
# return       $InstallerCartrigeVars | information of the installer | ref.hash
#========================================================================================
sub InstallerCartridgeVars {
    my ($Installer) = @_;
    my $DocFile = $Installer->cartridgeDirectory . '/Documents/index.html';
    my $cartridgeName = $Installer->cartridgePackage;
    $cartridgeName =~ s/.*:://;

    my $DocRoot       = LoadRootObject()->get('DocRoot');
    my $aModelFiles   = ModelsLoop( $cartridgeName, $DocRoot );
    my $APIDocu       = sprintf '%s/index.html', $Installer->cartridgeDirectory;
    my $aKnownAPIDocs = GetApiInfoFromDocRoot($DocRoot);
    my $APIDocuURL    = "$DocRoot/API/$APIDocu";

    return {
        ExistsAPIDocu => ElementOf($APIDocu, $aKnownAPIDocs),
        ExistsDocu  => ( -f $ENV{EPAGES_CARTRIDGES} . '/' . $DocFile ) ? 1 : 0,
        ExistsModel => ( scalar @$aModelFiles > 0 )                    ? 1 : 0,
        DocFile     => $DocFile,
        ModelFiles  => $aModelFiles,
        CodeVersion => $Installer->version,
        Version     => $Installer->installedVersion,
        Package     => $Installer->cartridgePackage,
        APIDocu     => $APIDocuURL,
    };
}

#========================================================================================
# function     ModelsLoop
#----------------------------------------------------------------------------------------
# syntax       $aModels = ModelsLoop( $cartridgeName, $DocRoot );
#----------------------------------------------------------------------------------------
# description  Returns models of a cartridge
#========================================================================================
# input        $cartridgeName | cartridge name | string
# input        $DocRoot | path to the root of the doc folder. | string
#========================================================================================
# return       $aModels | models and info | ref.array
#========================================================================================
sub ModelsLoop {
    my ( $cartridgeName, $DocRoot ) = @_;
    my @AllFiles;

    my $hModelsInfo = GetModelsInfoFromDocRoot($DocRoot);
    return [] unless defined $hModelsInfo->{$cartridgeName};

    foreach my $file ( split( /,/, $hModelsInfo->{$cartridgeName} ) ) {
        next unless $file =~ /\S/;

        my ( undef, $FileName, undef ) = fsplit($file);
        my ( $aCartridgeName, $aModelName ) = split( /\_/, $FileName );
        $aModelName = $aCartridgeName unless defined $aModelName;
        push @AllFiles,
          {
            'ModelName' => $aModelName,
            'ModelLink' => "$DocRoot/Models/$file",
          };
    }

    return \@AllFiles;
}

#========================================================================================
# function     ClassesLoop
#----------------------------------------------------------------------------------------
# syntax       $aClasses = ClassesLoop( $Package );
#----------------------------------------------------------------------------------------
# description  Returns classes and information
#========================================================================================
# input        $Package | package name | string
#========================================================================================
# return       $aClasses | classes and info | ref.array
#========================================================================================
sub ClassesLoop {
    my ($Package) = @_;

    # get all classes where the package name matches the cartridge package
    my $aClasses = LoadRootObject()->get('Classes');

    return [
        map { $_->tleHash }
        grep { $_->get('Package') =~ /^\Q$Package\E\:\:/ }
        @$aClasses
    ];
}

#========================================================================================
# function     PatchesLoop
#----------------------------------------------------------------------------------------
# syntax       $aPatches = PatchesLoop( $hCartridge );
#----------------------------------------------------------------------------------------
# description  Gets all patches from a cartridge hash
#========================================================================================
# input        $hCartridge | result of the function Hashcartridge | ref.hash
#========================================================================================
# return       $aPatches | patch versions | ref.array
#========================================================================================
sub PatchesLoop {
    my ($hCartridge) = @_;

    return [
        map { { Version => $_ } }
        @{ $hCartridge->{'Installer'}->patches }
    ]
}

#========================================================================================
# function     CartridgesLoop
#----------------------------------------------------------------------------------------
# syntax       $hCartridges = CartridgesLoop();
#----------------------------------------------------------------------------------------
# description  Returns all Cartridges and their version
#========================================================================================
# return       $hCartridges | key: cartridge name, value: version | ref.hash
#========================================================================================
sub CartridgesLoop {

    my $hCartriges = CartridgeConfig(GetCurrentDBHandle()->cacheKey)->section('Cartridges');

    return [
        map  { CartridgeVars(HashCartrigde($_)) }
        sort { $a cmp $b }
        keys %$hCartriges
    ];
}

#========================================================================================
# function     CartridgeVars
#----------------------------------------------------------------------------------------
# syntax       $hCartridgeVars = CartridgeVars( $hCartridge );
#----------------------------------------------------------------------------------------
# description  Returns all information of a cartridge like dependencies, classes, ...
#----------------------------------------------------------------------------------------
# input        $hCartridge | result of the public function HashCartridge. | ref.hash
#========================================================================================
# return       $hCartridgeVars | cartridge information | ref.hash
#========================================================================================
sub CartridgeVars {
    my ($hCartridge) = @_;

    return {
        %$hCartridge,
        %{InstallerCartridgeVars($hCartridge->{'Installer'})},
        Dependencies => sub { DependenciesLoop($hCartridge)},
        DependendCartridges => sub { DependendCartridgesLoop( $hCartridge ) },
        Classes   => sub { ClassesLoop($hCartridge->{Installer}->cartridgePackage) },
        Patches   => sub { PatchesLoop($hCartridge) },
    }
}

#========================================================================================
# function     DependendCartridgesLoop
#----------------------------------------------------------------------------------------
# syntax       DependendCartridgesLoop( $hCartridge );
#----------------------------------------------------------------------------------------
# description  Returns the dependend cartridges.
#========================================================================================
# input        $hCartridge | result of the public function HashCartridge | ref.hash
#========================================================================================
# return       $aClasses | dependend cartridges and some info | ref.array
#========================================================================================
sub DependendCartridgesLoop {
    my ($hCartridge) = @_;

    my $DependendCartridges = $hCartridge->{'Installer'}->directDependentCartridges;

    return [
        sort { $a->{Cartridge}->{Package} cmp $b->{Cartridge}->{Package} }
        map {
            {
                Cartridge => InstallerCartridgeVars($_),
                Comment => $hCartridge->{'Installer'}->dependencyComment($_->cartridgePackage,{'DependOnActualCartridge' => '1'}),
            }
        }
        @$DependendCartridges
    ]
}

1;
