#==============================================================================
# package      DE_EPAGES::Diagnostics::UI::Class
# state        private
#==============================================================================
package DE_EPAGES::Diagnostics::UI::Class;

use strict;
use JSON qw( to_json );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject Transaction LoadClassByAlias);
use DE_EPAGES::Diagnostics::API::HelpCenter qw( GetApiInfoFromDocRoot );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

#==============================================================================
# Common initialization for all event handlers
#==============================================================================

sub new {
    my $class = shift;
    my $self = bless {}, $class;

    $self->{TemplateDir} = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Diagnostics/Templates";

    return $self;
}

#==============================================================================
# Event handlers
#==============================================================================

sub ViewAllClasses {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/Classes.html" );
}

sub _APIDocu {
    my ($Servlet, $Package) = @_;

    # build link for API documentation
    # example: http://helpcenter.epages.com/Doc/ver_6_11_3/epages/API/DE_EPAGES/Object/API/Object/Object.pm.html
    my $APIDocu = "$Package.pm.html";
    $APIDocu =~ s/::/\//g;

    my $DocRoot = LoadRootObject()->get('DocRoot');
    my $aKnownAPIDocs = GetApiInfoFromDocRoot($DocRoot);
    my $APIDocuURL = "$DocRoot/API/$APIDocu";

    $Servlet->vars( 'ExistsDocu', ElementOf($APIDocu, $aKnownAPIDocs) );
    $Servlet->vars( 'APIDocu', $APIDocuURL );
}

sub ViewClass {
    my $self = shift;
    my ($Servlet) = @_;

    _APIDocu( $Servlet, $Servlet->object->get('Package') );
    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/Class.html" );
}

sub ViewAttribute {
    my $self = shift;
    my ($Servlet) = @_;

    _APIDocu( $Servlet, $Servlet->object->get('Package') );
    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/Attribute.html" );
}

sub ViewObject {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/Object.html" );
}

sub _Hash2String {
    my ($h) = @_;
    return '{ ' . join( ', ', map { "$_=>" . ($h->{$_} // 'undef') } sort keys %$h ) . ' }';
}

sub ViewArrayAttribute {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;

    my $AttributeAlias = $Servlet->params('Attribute');
    my $aValues = $Object->tleHash->{$AttributeAlias};
    my $Attribute = $Object->class->attribute($AttributeAlias);

    if($Attribute->get('IsObject')) {
        $Servlet->vars( 'AttributeTypeClass', LoadClassByAlias($Attribute->get('Type'))->tleHash);
        if (LoadClassByAlias($Attribute->get('Type'))->isSubClassOf('Object')) {
            $Servlet->vars( 'Values', [ map { { Value => $_ } } @$aValues ] );
            $Servlet->vars( 'IsHash', 0);
        } else {
            # expand the next level inside the hash
            $Servlet->vars( 'Values', [ map { { Value => _Hash2String($_) } } @$aValues ] );
            $Servlet->vars( 'IsHash', 1 );
        }
    } elsif( $Attribute->get('Type') eq 'Hash') {
        # expand the next level inside the hash
        $Servlet->vars( 'Values', [ map { { Value => _Hash2String($_) } } @$aValues ] );
        $Servlet->vars( 'IsHash', 1 );
    }
    else {
        $Servlet->vars( 'Values', [ map { { Value => $_ } } @$aValues ] );
        $Servlet->vars( 'IsHash', 0);
    }

    $Servlet->vars( 'Attribute', $Attribute->tleHash );

    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/ArrayAttribute.html" );
}

sub ViewJSONAttribute {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;

    my $AttributeAlias = $Servlet->params('Attribute');
    my $rValues = $Object->tleHash->{$AttributeAlias};
    my $Attribute = $Object->class->attribute($AttributeAlias);

    my $rValue = $rValues ? to_json($rValues, { 'pretty' => 1 }) : undef;

    $Servlet->vars( 'Value', $rValue );
    $Servlet->vars( 'Attribute', $Attribute->tleHash );

    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/JSONAttribute.html" );
}

sub ViewHashAttribute {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;

    my $AttributeAlias = $Servlet->params('Attribute');
    my $hValues = $Object->tleHash->{$AttributeAlias};
    my $Attribute = $Object->class->attribute($AttributeAlias);
    my $aValues = [ map {
        { Name => $_, Value => $hValues->{$_}, }
                    }
                    sort keys %$hValues
        ];

    $Servlet->vars( 'Values', $aValues );
    $Servlet->vars( 'Attribute', $Attribute->tleHash );

    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/HashAttribute.html" );
}

sub ViewLocalizedAttribute {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my $AttributeAlias = $Servlet->params('Attribute');
    my $Attribute = $Object->class->attribute($AttributeAlias);

    my $ahLangauges = LoadRootObject()->get('Languages');
    my $aValues = [ map {
        {
            Language => $_,
            Value    => $Object->get( $AttributeAlias, $_->{LanguageID} ),
        }
                    }
                    @$ahLangauges
        ];

    $Servlet->vars( 'Values', $aValues );
    $Servlet->vars( 'Attribute', $Attribute->tleHash );

    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/LocalizedAttribute.html" );
}

sub ViewAction {
    my $self = shift;
    my ($Servlet) = @_;

    _APIDocu( $Servlet, $Servlet->object->get('Package') );
    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/Action.html" );
}

sub ViewRole {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars( 'TemplateFile', "$self->{TemplateDir}/Role.html" );
}

sub DeleteObject {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my $Parent = $Object->parent;

    Transaction( sub { $Object->delete; } );

    return $Parent;
}

1;
