#========================================================================================
# package      DE_EPAGES::Dictionary::API::Compare
# state        public
#----------------------------------------------------------------------------------------
# description  Functions to compare dictionaries
#========================================================================================
package DE_EPAGES::Dictionary::API::Compare;
use base Exporter;
use strict;

use DE_EPAGES::Core::API::File qw (ExistsFileCaseSensitive);
use DE_EPAGES::Dictionary::API::XML::DictionaryImportHandler qw ( ImportDictionary );
use DE_EPAGES::Dictionary::API::XML::TranslationImportHandler qw ( ImportTranslation );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

our @EXPORT_OK = qw (
    ListMissingKeywords
    ListMissingKeywordsFromTranslationFile
    CompareDictionaryFile
    CompareTranslationFile
);

#========================================================================================
# function     ListMissingKeywords
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aMissingList = ListMissingKeywords ($FilePath1, $FilePath2, $Language1, $Language2);
# example      $aMissingList = ListMissingKeywords ("example.en.xml", "example.de.xml", "en", "de");
#----------------------------------------------------------------------------------------
# description  This function compares two localization files and finds
#               missing keywords in file2 and returns a ref.array with all missing keywords
#----------------------------------------------------------------------------------------
# input        $FilePath1 | file name | string
# input        $FilePath2 | file name | string
# input        $Language1 | language code (en,de ...) | string
# input        $Language2 | language code (en,de ...) | string
# input        $OnlyJS | (optional, dault=false) read ony translation tags that are
#                   marked with the JS="1" attribute | boolean
# return       $aMissingList | each list element has the following structure
#                           <ul>
#                             <li> Error - error message</li>
#                             <li> File - file name where the error occured</li>
#                           </ul> | ref.array.hash.string
#========================================================================================
sub ListMissingKeywords {
    my ($FilePath1, $FilePath2, $Language1, $Language2, $OnlyJS) = @_;

    my $PathError = _CheckFilePaths($FilePath1, $FilePath2);
    return [$PathError] if($PathError);

    my ($LanguageCode1) = split /-/, $Language1;
    my $hDictionary1 = ImportDictionary($FilePath1, undef, $LanguageCode1, $OnlyJS);

    my ($LanguageCode2) = split /-/, $Language2;
    my $hDictionary2 = eval { ImportDictionary($FilePath2, undef, $LanguageCode2, $OnlyJS); };
    return _HandleFileError(GetError(), $FilePath2) if(ExistsError());

    return _CompareDictionaries($hDictionary1->{$LanguageCode1}, $FilePath1,
        $hDictionary2->{$LanguageCode2}, $FilePath2);
}

#========================================================================================
# function     ListMissingKeywordsFromTranslationFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aMissingList = ListMissingKeywordsFromTranslationFile ($Filename1, $Filename2, $Language1, $Language2);
# example      $aMissingList = ListMissingKeywordsFromTranslationFile ("example.en.xml", "example.de.xml", "en", "de");
#----------------------------------------------------------------------------------------
# description  This function compares two translation files and finds
#               missing keywords in file2 and returns a ref.array with all missing keywords
#----------------------------------------------------------------------------------------
# input        $Filename1 | file name | string
# input        $Filename2 | file name | string
# input        $Language1 | language code (en,de ...) | string
# input        $Language2 | language code (en,de ...) | string
# return       $aMissingList | each list element has the following structure
#                           <ul>
#                             <li> Error - error message</li>
#                             <li> File - file name where the error occured</li>
#                           </ul> | ref.array.hash.string
#========================================================================================
sub ListMissingKeywordsFromTranslationFile {
    my ($FilePath1, $FilePath2, $Language1, $Language2) = @_;

    my $PathError = _CheckFilePaths($FilePath1, $FilePath2);
    return [$PathError] if($PathError);

    my ($LanguageCode1) = split /-/, $Language1;
    my $hDictionary1 = ImportTranslation($FilePath1, undef, $Language1 );

    my ($LanguageCode2) = split /-/, $Language2;
    my $hDictionary2 = eval { ImportTranslation($FilePath2, undef, $Language2 ); };
    return _HandleFileError(GetError(), $FilePath2) if(ExistsError());

    return _CompareTranslations($hDictionary1->{$LanguageCode1}, $FilePath1,
        $hDictionary2->{$LanguageCode2}, $FilePath2);
}

# compares translation files using a user defined sub(filename1, filename2, lang1, lang2, onlyjs)
sub _compareFiles {
    my ( $Language1, $Language2, $Filename, $cListFunc) = @_;

    my $Filename1 = $Filename.".$Language1.xml";
    my $Filename2 = $Filename.".$Language2.xml";
    my @Errors;

    my $aMissingList = $cListFunc->($Filename1, $Filename2, $Language1, $Language2);
    foreach my $hListElement ( @$aMissingList ) {
        push(@Errors, "$hListElement->{'File'} $hListElement->{'Error'}");
    }

    # Check for matching JS="1" attributes
    $aMissingList = $cListFunc->($Filename1, $Filename2, $Language1, $Language2, 1);
    foreach my $hListElement ( @$aMissingList ) {
        my $ErrorMessage = "$hListElement->{'File'} $hListElement->{'Error'}";
        # do not report errors twice
        push(@Errors, "$ErrorMessage (difference in JS attribute)") unless ElementOf($ErrorMessage, @Errors);
    }

    return \@Errors;
}

#========================================================================================
# function     CompareDictionaryFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aMissingList = CompareDictionaryFile ($Language1, $Language2, $BaseFilename);
# example      $aMissingList = CompareDictionaryFile ("en", "de", "example");
#----------------------------------------------------------------------------------------
# description  This function compares two localization files, finds
#               missing keywords in file1 & file2 and returns all errors
#----------------------------------------------------------------------------------------
# input        $Language1 | language code (en,de ...) | string
# input        $Language2 | language code (en,de ...) | string
# input        $BaseFilename | filename without language code and .xml extension
# return       $aMissingList | list of errors | ref.array.string
#========================================================================================
sub CompareDictionaryFile {
    my ( $Language1, $Language2, $Filename) = @_;
    return _compareFiles( $Language1, $Language2, $Filename, \&ListMissingKeywords );
}

#========================================================================================
# function     CompareTranslationFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aMissingList = CompareTranslationFile ($Language1, $Language2, $BaseFilename);
# example      $aMissingList = CompareTranslationFile ("en", "de", "example");
#----------------------------------------------------------------------------------------
# description  This function compares two localization files, finds
#               missing keywords in file1 & file2 and returns all errors
#----------------------------------------------------------------------------------------
# input        $Language1 | language code (en,de ...) | string
# input        $Language2 | language code (en,de ...) | string
# input        $BaseFilename | filename without language code and .xml extension
# return       $aMissingList | list of errors | ref.array.string
#========================================================================================
sub CompareTranslationFile {
    my ( $Language1, $Language2, $Filename) = @_;
    return _compareFiles( $Language1, $Language2, $Filename, \&ListMissingKeywordsFromTranslationFile );
}

sub _CheckFilePaths {
    my ($FilePath1, $FilePath2) = @_;

    # file2 must exist
    return {
        'Error' => 'missing file',
        'File'  => $FilePath2,
    } unless (ExistsFileCaseSensitive($FilePath2));

    # file1 must exists; if not, there should also be no file2
    return {
        'Error' => 'extra file',
        'File'  => $FilePath2,
    } unless (ExistsFileCaseSensitive($FilePath1));

    # files must be plain files
    return {
        'Error' => 'extra file',
        'File'  => $FilePath2,
    } unless (-f $FilePath1);

    return {
        'Error' => 'not a plain file',
        'File'  => $FilePath2,
    } unless (-f $FilePath2);

    return;
}

# checks that 2 dictionary files have the same keywords and no keyword has empty content
sub _CompareDictionaries {
    my ($hDictionary1, $FilePath1, $hDictionary2, $FilePath2) = @_;

    my @MissingList;
    foreach my $Keyword ( sort keys %$hDictionary1 ) {
        # dict2 must have the same keywords as dict1
        if (!exists $hDictionary2->{$Keyword} ) {
            push @MissingList, {
                'Error' => "keyword $Keyword missing",
                'File' => $FilePath2,
            };
        }

        # no empty keywords allowed
        if($hDictionary1->{$Keyword} eq '') {
            push @MissingList, {
                'Error' => "keyword $Keyword is empty",
                'File' => $FilePath1,
            };
        }
    }

    foreach my $Keyword ( sort keys %$hDictionary2 ) {
        # dict2 must have no extra keywords
        if (!exists $hDictionary1->{$Keyword} ) {
            push @MissingList, {
                'Error' => "extra keyword $Keyword",
                'File' => $FilePath2,
            };
        }

        # no empty keywords allowed
        if($hDictionary2->{$Keyword} eq '') {
            push @MissingList, {
                'Error' => "keyword $Keyword is empty",
                'File' => $FilePath2,
            };
        }
    }

    return \@MissingList;
}

# checks that 2 translation files have the same object and attributes
sub _CompareTranslations {
    my ($hDictionary1, $FilePath1, $hDictionary2, $FilePath2) = @_;

    my @MissingList;
    foreach my $Object ( sort keys %$hDictionary1 ) {
        if (!exists $hDictionary2->{$Object} ) {
            push @MissingList, {
                'Error' => "object tag $Object missing",
                'File' => $FilePath2,
            };
        }
        else{
            foreach my $Attribute (sort keys %{$hDictionary1->{$Object}} ){
                if (!exists $hDictionary2->{$Object}->{$Attribute} ) {
                    push @MissingList, {
                        'Error' => "attribute tag $Attribute missing in object tag $Object",
                        'File' => $FilePath2,
                    };
                }
            }
        }
    }

    foreach my $Object ( sort keys %$hDictionary2 ) {
        if (!exists $hDictionary1->{$Object} ) {
            push @MissingList, {
                'Error' => "extra object tag $Object",
                'File' => $FilePath2,
            };
        }
        else{
            foreach my $Attribute (sort keys %{$hDictionary2->{$Object}} ){
                if (!exists $hDictionary1->{$Object}->{$Attribute} ) {
                    push @MissingList, {
                        'Error' => "extra attribute tag $Attribute in object tag $Object",
                        'File' => $FilePath2,
                    };
                }
            }
        }
    }

    return \@MissingList;
}

sub _HandleFileError {
    my ($Error, $FilePath) = @_;

    my $Code = $Error->code;
    if ($Code eq 'FileNotFound') {
        return [{
            'Error' => 'missing file',
            'File' => $FilePath,
        }];
    }
    else {
        return [{
            'Error' => $Error->shortMessage,
            'File' => $FilePath,
        }]
    }

    return;
}
