#========================================================================================
# package      DE_EPAGES::DocdataPay::API::SOAP::CommunicationHandler
# state        public
# description  Handles DocdataPay SOAP communication.
#========================================================================================
package DE_EPAGES::DocdataPay::API::SOAP::CommunicationHandler;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::DocdataPay::API::Constants qw ( $VALID_REQUEST_TYPES );
use DE_EPAGES::DocdataPay::API::SOAP::CompiledClient qw ( buildWSDL );
use DE_EPAGES::DocdataPay::API::SOAP::Request::CaptureRequest;
use DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest;
use DE_EPAGES::DocdataPay::API::SOAP::Request::StatusRequest;
use DE_EPAGES::DocdataPay::API::SOAP::Response::CaptureResponse qw ( parseCaptureResponse );
use DE_EPAGES::DocdataPay::API::SOAP::Response::CreateResponse qw ( parseCreateResponse );
use DE_EPAGES::DocdataPay::API::SOAP::Response::StatusResponse;
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new($IsTest);
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $IsTest | True if it should run on test mode | Boolean
# return       $self   | self   | DE_EPAGES::DocdataPay::API::SOAP::CommunicationHandler
#========================================================================================
sub new {
    my $class = shift;
    my ($IsTest) = @_;

    my $self = bless { 'IsTest' => $IsTest, 'BasketOrOrder' => undef, 'LineItemPaymentDocdataPay' => undef }, $class;

    return $self;
}

#========================================================================================
# function     sendRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       sendRequest($RequestType, $BasketOrOrder);
#----------------------------------------------------------------------------------------
# description  Sends a certain request and parses the response.
#----------------------------------------------------------------------------------------
# input        $RequestType |  Type of request | String
# input        $BasketOrOrder |  Customer basket or order | Object
# return       ($hParsedResponse, $hRequestData) | Array containing the parsed response and
#                                          | some request data which needs to be set.
#========================================================================================
sub sendRequest {
    my $self = shift;
    my ( $RequestType, $BasketOrOrder ) = @_;

    unless ( ElementOf($RequestType, $VALID_REQUEST_TYPES) ) {
        $self->_sendError( 'RequestTypeNotSupported', $RequestType );
    }

    $self->{'BasketOrOrder'}             = $BasketOrOrder;
    $self->{'LineItemPaymentDocdataPay'} = $self->{'BasketOrOrder'}->get('LineItemContainer')->get('Payment');
    my $hRequest;

    if ( $RequestType eq 'create' ) {
        my $CreateRequest = DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest->new();
        $hRequest = $self->_createRequestMessage($CreateRequest);
    }
    elsif ( $RequestType eq 'capture' ) {
        my $CaptureRequest = DE_EPAGES::DocdataPay::API::SOAP::Request::CaptureRequest->new();
        $hRequest = $self->_captureRequestMessage($CaptureRequest);
    }
    elsif ( $RequestType eq 'status' ) {
        my $StatusRequest = DE_EPAGES::DocdataPay::API::SOAP::Request::StatusRequest->new();
        $hRequest = $self->_statusRequestMessage($StatusRequest);
    }

    my ($hResponse,$hTrace) = $self->_sendRequest( $RequestType, $hRequest );

    my $hParsedResponse = $self->_parseResponse( $RequestType, $hResponse );

    if ( $RequestType eq 'create' ) {
        my $hRequestData = {
            'merchant_name'           => $hRequest->{'merchant'}->{'name'},
            'client_language'         => $hRequest->{'shopper'}->{'language'}->{'code'},
            'merchant_transaction_id' => $hRequest->{'merchantOrderReference'},
        };

        return ( $hParsedResponse, $hRequestData );
    }
;
    if ( $RequestType eq 'status' ) {
        return ($hParsedResponse, $hTrace)
    }

    return $hParsedResponse;
}

#========================================================================================
# function     _sendRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _sendRequest($RequestType, $hRequest);
#----------------------------------------------------------------------------------------
# description  Sends a certain request and parses the response.
#----------------------------------------------------------------------------------------
# input        $RequestType |  Type of request | String
# input        $hRequest |  Hash containing the request | Hash
# return       $Response | Response | Hash
#========================================================================================
sub _sendRequest {
    my $self = shift;
    my ( $RequestType, $hRequest ) = @_;

    unless ( ElementOf($RequestType, $VALID_REQUEST_TYPES) ) {
        $self->_sendError( 'RequestTypeNotSupported', $RequestType );
    }

    return $self->_buildWSDLAndSubmitRequest( $RequestType, $hRequest );
}

#========================================================================================
# function     _parseResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _parseResponse($RequestType, $Response);
#----------------------------------------------------------------------------------------
# description  Parses a response.
#----------------------------------------------------------------------------------------
# input        $RequestType |  Type of request | String
# input        $hResponse |  Hash containing the response | Hash
# return       $hResponse | Parsed response | Hash
#========================================================================================
sub _parseResponse {
    my $self = shift;
    my ( $RequestType, $hResponse ) = @_;

    unless ( ElementOf($RequestType, $VALID_REQUEST_TYPES) ) {
        $self->_sendError( 'RequestTypeNotSupported', $RequestType );
    }

    if ( $RequestType eq 'create' ) {
        my $CreateResponse = DE_EPAGES::DocdataPay::API::SOAP::Response::CreateResponse->new();
        return $CreateResponse->parseCreateResponse( $hResponse, $self->{'LineItemPaymentDocdataPay'} );
    }
    elsif ( $RequestType eq 'status' ) {
        my $StatusResponse = DE_EPAGES::DocdataPay::API::SOAP::Response::StatusResponse->new();
        return $StatusResponse->parseStatusResponse( $hResponse, $self->{'LineItemPaymentDocdataPay'} );
    }
    elsif ( $RequestType eq 'capture' ) {
        my $CaptureResponse = DE_EPAGES::DocdataPay::API::SOAP::Response::CaptureResponse->new();
        $CaptureResponse->parseCaptureResponse($hResponse);
    }

    return;
}

#========================================================================================
# function     _logCommunication
# state        private
#----------------------------------------------------------------------------------------
# syntax       _logCommunication($RequestType, $Trace);
#----------------------------------------------------------------------------------------
# description  Logs communication.
#----------------------------------------------------------------------------------------
# input        $RequestType |  Type of request | String
# input        $Trace |  Log trace | Hash
#========================================================================================
sub _logCommunication {
    my $self = shift;
    my ( $RequestType, $Trace ) = @_;

    LogPayment(
        'DocdataPay',
        $RequestType,
        $Trace, 1,
        sub {
            my $LogString = shift;
            $$LogString =~ s/'password'\s{1}=>\s{1}'[^']+' /'password' => '---replaced---'/xig;
            $$LogString =~ s/password="[^"]+"/password="---replaced---"/xig;
        }
    );

    return;
}

#========================================================================================
# function     _sendError
# state        private
#----------------------------------------------------------------------------------------
# syntax       _sendError($ErrorType, $RequestType);
#----------------------------------------------------------------------------------------
# description  Logs communication.
#----------------------------------------------------------------------------------------
# input        $ErrorType |  Type of error | String
# input        $RequestType |  Type of request | String
#========================================================================================
sub _sendError {
    my $self = shift;
    my ( $ErrorType, $RequestType ) = @_;

    if ( $ErrorType eq 'RequestTypeNotSupported' ) {
        Error( 'RequestTypeNotSupported', { 'Message' => "Request type $RequestType is not supported." } );
    }

    return;
}

#========================================================================================
# function     _buildWSDLAndSubmitRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildWSDLAndSubmitRequest($RequestType, $hRequest);
#----------------------------------------------------------------------------------------
# description  Build a WSDL CompiledClient and submits the request.
#----------------------------------------------------------------------------------------
# input        $RequestType |  Type of request | String
# input        $hRequest |  Hash containing the request | Hash
# return       $Response | Response | Hash
#========================================================================================
sub _buildWSDLAndSubmitRequest {
    my $self = shift;
    my ( $RequestType, $hRequest ) = @_;

    my $CompiledClient = DE_EPAGES::DocdataPay::API::SOAP::CompiledClient->new( $self->{'IsTest'} );
    my $WebserviceCall = $CompiledClient->buildWSDL($RequestType);
    $self->_logCommunication( $RequestType, $hRequest );
    my ( $hResponse, $hTrace ) = $WebserviceCall->($hRequest);
    $self->_logCommunication( $RequestType . ' Response:', $hResponse );
    $self->_logCommunication( $RequestType . ' Trace:',    $hTrace );

    return ( $hResponse, $hTrace );
}

# Wrappers
#----------------------------------------------------------------------------------------
sub _createRequestMessage {
    my $self = shift;

    my ($CreateRequest) = @_;

    return $CreateRequest->buildCreateRequest( $self->{'BasketOrOrder'} );
}

#----------------------------------------------------------------------------------------
sub _captureRequestMessage {
    my $self = shift;

    my ($CaptureRequest) = @_;

    return $CaptureRequest->buildCaptureRequest( $self->{'LineItemPaymentDocdataPay'} );
}

#----------------------------------------------------------------------------------------
sub _statusRequestMessage {
    my $self = shift;
    my ($StatusRequest) = @_;

    return $StatusRequest->buildStatusRequest( $self->{'LineItemPaymentDocdataPay'} );
}

1;
