#========================================================================================
# package      DE_EPAGES::DocdataPay::UI::Shop
# state        public
# description  UI class
#========================================================================================
package DE_EPAGES::DocdataPay::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;
use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf GetCurrentDateTime isValid );
use DE_EPAGES::DocdataPay::API::Constants qw ( LATEST_API_VERSION
                                               $VALID_PAYABLE_STATUS
                                               $FAILED_PAYABLE_STATUS
                                               @NO_CAPTURE_PAYMENT_METHODS);
use DE_EPAGES::DocdataPay::API::Object::RemoteDocdataPay;
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::DocdataPay::API::SOAP::CommunicationHandler;
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByGUID
  LoadObjectByGUID
);
use DE_EPAGES::Order::UI::Basket;
sub ErrorDefinition {
    return { 'StatusError' => 'Error Getting status',
             'ProcessError' => 'Error processing docdataparameters'
    };
}

#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  return GoogleCheckout as string. Name of PaymentType for search/view
#                               transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName { 'DocdataPay' }

#==============================================================================
# This callback is sent to the success link.
#==============================================================================
sub PaymentBacklinkDocdataPay {
    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);

    my $hFormValues = $Servlet->params();
    $self->_LogPayment( 'DocdataPay', 'Backlink parameters', $hFormValues );

    my $LineItemPaymentGUID = $hFormValues->{'MGUID'};

    # reset cached language id etc.
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    my $Form = $Servlet->form();

    my $ErrorViewAction = 'ViewPaymentDocdataPayResult';

    # check if GUID exists.
    unless ( isValid($LineItemPaymentGUID) && ExistsObjectByGUID($LineItemPaymentGUID) ) {
        $self->_LogPayment( 'DocdataPay', 'no payment line item found with guid', $LineItemPaymentGUID );
        $Form->executeFormError(
            {
                'Name'       => 'MGUID',
                'Reason'     => 'BadCallback_NoPaymentGUID',
                'ViewAction' => $ErrorViewAction,
            }
        );
    }

    my $LineItemPayment   = LoadObjectByGUID($LineItemPaymentGUID);
    my $LineItemContainer = $LineItemPayment->container();
    my $CustomerOrder     = $LineItemContainer->parent();
    my $hLoggerString     = DE_EPAGES::DocdataPay::API::Object::RemoteDocdataPay->getLoggerString($LineItemPayment);
    my $BasketOrOrder     = $LineItemContainer->parent();

    if ( $hFormValues->{'ReturnStatus'} eq 'CANCELED' ) {
        if ( $BasketOrOrder->instanceOf('Basket') ) {
            _deletePaymentMethod($LineItemContainer);
        } elsif( $BasketOrOrder->instanceOf('CustomerOrder') ){
            $BasketOrOrder->set( { 'CancelledOn' => GetCurrentDateTime() } );
        }

        $self->_LogPayment(
            'DocdataPay',
            $hLoggerString . 'PaymentProcess_Canceled',
            {
                'Name'        => 'MGUID',
                'Reason'      => 'PaymentProcess_Canceled',
                'ViewAction'  => $ErrorViewAction,
                'ObjectID'    => $CustomerOrder->id(),
                'ObjectClass' => $CustomerOrder->class()->alias(),
            }
        );
        $Form->executeFormError(
            {
                'Name'        => 'MGUID',
                'Reason'      => 'PaymentProcess_Canceled',
                'ViewAction'  => $ErrorViewAction,
                'ObjectID'    => $CustomerOrder->id(),
                'ObjectClass' => $CustomerOrder->class()->alias(),
            }
        );
    }

    if ( $hFormValues->{'ReturnStatus'} eq 'ERROR' ) {
        $self->_LogPayment(
            'DocdataPay',
            $hLoggerString . 'PaymentProcess_Failed',
            {
                'Name'        => 'MGUID',
                'Reason'      => 'PaymentProcess_Failed',
                'ViewAction'  => $ErrorViewAction,
                'ObjectID'    => $CustomerOrder->id(),
                'ObjectClass' => $CustomerOrder->class()->alias(),
            }
        );
        $Form->executeFormError(
            {
                'Name'        => 'MGUID',
                'Reason'      => 'PaymentProcess_Failed',
                'ViewAction'  => $ErrorViewAction,
                'ObjectID'    => $CustomerOrder->id(),
                'ObjectClass' => $CustomerOrder->class()->alias(),
            }
        );
    }

    if ( $BasketOrOrder->instanceOf('Basket') ) {
        Error(
            'INVALID_FORM',
            {
                'ViewVars' => {
                    'ViewObjectID' => $BasketOrOrder->id(),
                },
                'ViewAction' => 'ViewDocdataPayBlocked',
            }
        );
    }


    if ( defined $CustomerOrder ) {
        $Servlet->vars( 'ViewObjectID', $CustomerOrder->id() );
        $Servlet->registerSession( $CustomerOrder->get('User'), 0 );
        $self->_LogPayment( 'DocdataPay', $hLoggerString . 'order success', $CustomerOrder );
    }

    return;
}

#==============================================================================
sub _deletePaymentMethod {
    my ($LineItemContainer) = @_;
    $LineItemContainer->get('Payment')->delete();
    $LineItemContainer->addLineItemPayment();
    $LineItemContainer->recalculate();
}

#==============================================================================
sub PaymentCallbackDocdataPay {
    my $self = shift;
    my ($Servlet) = @_;

    my $hFormValues = $Servlet->params();

    $self->_LogPayment( 'DocdataPay', 'Callback parameters', $hFormValues );

    my $ModifiedGUID              = $hFormValues->{'DocdataPaymentID'};
    my $LineItemPaymentDocdataPay = $self->_getLineItemPaymentDocdataPay($ModifiedGUID);

    my $BasketOrOrder = $LineItemPaymentDocdataPay->parent()->parent();
    unless ( $BasketOrOrder->instanceOf('Basket') || $BasketOrOrder->instanceOf('CustomerOrder') ) {
        Error(
            'GUIDNotABasketOrOrder',
            {
                'Message' =>
                  'The LineItemDocdataPay associated to the given GUID is not connected to any Basket or CustomerOrder'
            }
        );
    }

    eval {
        $self->_ProcessDocdataPayParameters( $Servlet, $hFormValues, $LineItemPaymentDocdataPay );
    };
    if (ExistsError()) {
        my $Error = GetError();
        Warning('ProcessError',$Error);
    } else {
        $self->_LogPayment( 'DocdataPay', 'DocdataPaymentID ' . $ModifiedGUID . 'has been updated', $hFormValues );
    }

   $self->_writeContentOutput($Servlet);

    return;
}

#==============================================================================
sub _ProcessDocdataPayParameters {
    my $self = shift;
    my ( $Servlet, $hFormValues, $LineItemPayment ) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();

    my $ErrorViewAction = 'ViewPaymentDocdataPayResult';

    $Form->executeFormError(
        {
            'Name'       => 'MGUID',
            'Reason'     => 'BadCallback_ShopIDMisMatch',
            'ViewAction' => $ErrorViewAction
        }
    ) if ( $LineItemPayment->getSite()->id != $Shop->id );

    if ( $LineItemPayment->get('APIVersion') eq LATEST_API_VERSION ) {
        _ProcessApi12Parameters( $Servlet, $LineItemPayment );
        return;
    }
    _ProcessApi04Parameters( $Servlet, $hFormValues, $LineItemPayment );
    return;
}

#==============================================================================
sub _ProcessApi12Parameters {
    my ( $Servlet, $LineItemPayment ) = @_;

    my $OrderStatus;
    my $PaymentMethod;
    my $BasketOrder = $LineItemPayment->parent()->parent();

    eval { ($OrderStatus, $PaymentMethod) = $LineItemPayment->getTransStatus(); };
    if ( ExistsError() ) {
        if($BasketOrder->instanceOf('CustomerOrder')){
             $BasketOrder->set( { 'CancelledOn' => GetCurrentDateTime() } );
        }
        Error('StatusError',{}, GetError() );
    }

    if ($OrderStatus eq 'PAID'){
        if ( $BasketOrder->instanceOf('Basket') ) {
            $BasketOrder = DE_EPAGES::Order::UI::Basket->doBasket2Order( $Servlet, $BasketOrder );
        }

        $LineItemPayment->setOrderStatus('PaidOn');

        if(ElementOf($PaymentMethod, @NO_CAPTURE_PAYMENT_METHODS)){
            return;
        } else{
            $LineItemPayment->capturePayment();
            return;
        }
    }
}

#==============================================================================
sub _ProcessApi04Parameters {
    my ( $Servlet, $hFormValues, $LineItemPayment ) = @_;

    my $LocalStatus = $LineItemPayment->getTransStatus();

    my $BasketOrder = $LineItemPayment->parent()->parent();

    if ( $BasketOrder->instanceOf('Basket') ) {
        $BasketOrder = DE_EPAGES::Order::UI::Basket->doBasket2Order( $Servlet, $BasketOrder );
    }

    if ( ElementOf($LocalStatus, $VALID_PAYABLE_STATUS) ) {
        $LineItemPayment->setOrderStatus('PaidOn');
    } elsif ( ElementOf($LocalStatus, $FAILED_PAYABLE_STATUS) ) {
        $LineItemPayment->setOrderStatus('CancelledOn');
    }

    return;
}

#==============================================================================
sub PaymentUpdateDocdataPay {
    my $self = shift;
    my ($Servlet) = @_;

    my $hFormValues = $Servlet->params();

    $self->_LogPayment( 'DocdataPay', 'Update call parameters', $hFormValues );

    my $OrderID = $hFormValues->{'DocdataPaymentID'};
    my $LineItemPaymentDocdataPay = $self->_getLineItemPaymentDocdataPay($OrderID);

    my $BasketOrder = $LineItemPaymentDocdataPay->parent()->parent();


    if ( $BasketOrder->instanceOf('CustomerOrder') ) {
        $LineItemPaymentDocdataPay->getTransStatus();
    }

    return;
}

#==============================================================================
sub _getLineItemPaymentDocdataPay {
    my $self = shift;
    my ($ModifiedGUID) = @_;

    my $Tools                     = DE_EPAGES::DocdataPay::API::SOAP::Tools->new();
    my $GUID                      = $Tools->addDashesToGUID($ModifiedGUID);
    my $LineItemPaymentDocdataPay = LoadObjectByGUID($GUID);
    TestObject( 'LineItemPaymentDocdataPay', $LineItemPaymentDocdataPay, 'DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay' );

    return $LineItemPaymentDocdataPay;
}

#==============================================================================
sub _LogPayment {
    my $self = shift;
    my ($PaymentTypeAlias, $Name, $DebugStuff) = @_;

    LogPayment( $PaymentTypeAlias, $Name, $DebugStuff );
    return;
}

#==============================================================================
sub _writeContentOutput {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->params( 'ContentType',    'text/plain' );
    $Servlet->params( 'ExitAfterEvent', 1 );
    $Servlet->writeContentOutput('OK');
    return;
}

1;
