use strict;
use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Config qw( StoreType BUStoreTypes StoreTypesWithCartridge );
use DE_EPAGES::Dojo::API::Shrinksafe qw( RunDojoShrinksafe );

sub Main {

    local $| = 1;

    my $time0 = time();
    my ($Help, $NoOutput, $Profile, $Version, $StoreName, $StoreType, $CustomReleaseDir, $HFFiles, $ConfigFile, $ConsultingCartridge, $User, $Group);

    GetOptions(
        'help'           => \$Help,
        'nooutput'       => \$NoOutput,
        'profile=s'      => \$Profile,
        'version=s'      => \$Version,
        'storename=s'    => \$StoreName,
        'storetype=s'    => \$StoreType,
        'releasedir=s'   => \$CustomReleaseDir,
        'hotfix=s'       => \$HFFiles,
        'config=s'       => \$ConfigFile,
        'consulting=s'   => \$ConsultingCartridge,
        'user=s'         => \$User,
        'group=s'        => \$Group,
    );

    # backward compatibility mode
    $StoreType = StoreType($StoreName) if not $StoreType and $StoreName;

    usage() if $Help;
    usage('The store type is not defined.') unless $StoreType;
    usage('The store types version number is not defined.') unless $Version;

    my %Params = (
        'Profile'             => $Profile,
        'Version'             => $Version,
        'StoreType'           => $StoreType,
        'CustomReleaseDir'    => $CustomReleaseDir,
        'HFFiles'             => $HFFiles,
        'ConsultingCartridge' => $ConsultingCartridge,
        'ConfigFile'          => $ConfigFile,
        'User'                => $User,
        'Group'               => $Group,
    );

    my $RequiredCartridge = 'DE_EPAGES::Dojo';
    my $aDojoStoreTypes = StoreTypesWithCartridge($RequiredCartridge);

    foreach ( @{BUStoreTypes()} ) {
        next unless $StoreType eq 'all' or $StoreType eq $_;
        next unless ElementOf($_, $aDojoStoreTypes);
        print "-> StoreType: $_\n" unless $NoOutput;
        $Params{'StoreType'} = $_;
        RunDojoShrinksafe(\%Params);
    }

    printf "need %.1f seconds\n", time()-$time0 unless $NoOutput;
    return;
}

sub usage {
    my ($Msg) = @_;
    print "$Msg\n\n" if $Msg;
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Description:
    Makes release versions of the cartridges javascript files into /shared/webroot/(-storename) or into -releasedir.
    It looks in all database type (-type) cartridges for appropriate files.
    If only the layers of a certain cartridge profile should be build (-cartridge) and (-profile) have to be definded.

Options:
    -storetype     type of store, if set to "all" all store types will be installed (required)
    -version       store types version number and
                   dojo.version will contain this value (required)
    -profile       the used profile file                (optional)
    -releasedir    set the release directory            (optional)
    -hotfix        file with list of changed js files
                   or directory with changed files      (optional)
    -consulting    cartridge with js files to override  (optional)
    -config        configuration file name              (optional)
    -user          username for owner, i.e. 'ep_appl'   (optional)
    -group         username for owner, i.e. 'ep_web'    (optional)

Flags:
    -help          show the command line options (optional)
    -nooutput      do not print any output

Examples:
    perl $0 -storetype all -version 6.10.3
    perl $0 -storetype Store -version 6.10.3
    perl $0 -storetype Store -version 6.10.3 -profile design
    perl $0 -storetype Store -version 6.10.3 -hotfix list.txt -releasedir D:/foo
    perl $0 -storetype Store -version 6.10.3 -consulting DE_EPAGES_TONLINE/TOnline
    perl $0 -storetype Store -version 6.10.3 -releasedir /tmp/testrls -user ep_appl -group ep_web
END_USAGE
exit 1;
}

RunScript(
    'Sub' => \&Main
);
