#========================================================================================
# package      DE_EPAGES::ELogistics::API::Object::RemoteELogisticsBarcode
# state        private
#----------------------------------------------------------------------------------------
# description  This package provides the ELogistics Barcode SOAP Connection
#========================================================================================
package DE_EPAGES::ELogistics::API::RemoteELogisticsBarcode;
use base DE_EPAGES::ELogistics::API::RemoteELogistics;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw (LoadObject);
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::WebService::API::Client;
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::ExternalShipping::API::Table::ShippingLabelFFOption qw( GetPKeyShippingLabelFFOptionsByShippingLabelID );
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw (GetDefaultTimeOut);

use DE_EPAGES::ELogistics::API::SOAP::Barcode::LabelDefinition;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Customer;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::FileInfos;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Recipient;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Attributes;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Item;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Sending;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Provider;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Data;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Envelope;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::GenerateLabel;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Notification;
use DE_EPAGES::ELogistics::API::SOAP::Barcode::Communication;
use DE_EPAGES::ELogistics::API::Constants qw (%ELOGISTICS_AVISIERUNG ZUSTELLAVISIERUNGREPAIRORDER);

sub CallGenerateLabelRequest {
    my $self = shift;
    my ($ShippingFulfillmentMethod,$Transaction,$FileType,$LanguageID) = @_;
    my $Shop = $ShippingFulfillmentMethod->get('Site');

    LogShippingFulfillment('ELogistics','ShippingFulfillmentMethod',$ShippingFulfillmentMethod,'CallGenerateLabelRequest');
    LogShippingFulfillment('ELogistics','Transaction',$Transaction,'CallGenerateLabelRequest');
    LogShippingFulfillment('ELogistics','FileType',$FileType,'CallGenerateLabelRequest');
    LogShippingFulfillment('ELogistics','LanguageID',$LanguageID,'CallGenerateLabelRequest');

    my $isTestShop = $ShippingFulfillmentMethod->get('Site')->get('IsInternalTestShop');
    my $WebserviceURL = $isTestShop ?
                $ShippingFulfillmentMethod->get('ShippingFulfillmentType')->get('WebserviceTestURL') :
                $ShippingFulfillmentMethod->get('ShippingFulfillmentType')->get('WebserviceURL') ;

    my $soap = $self->_getSoapClient(   $WebserviceURL,
                                        $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                        $ShippingFulfillmentMethod->get('WebservicePassword'),
                                        $isTestShop);

    my $Request = $self->_buildGenerateLabelRequest($Transaction,
                                                    $ShippingFulfillmentMethod,
                                                    $FileType,
                                                    $LanguageID);

    my $rSub = sub {
        my $ret;
        my $ErrorCode;
        my $ErrorMsg;
        eval {
           $ret = $soap->call('GenerateLabel', $Request->toSOAP());
        };
        if ( ExistsError() ) {
            my $Error = GetError();
            my $hFaultDetail = $Error->vars()->{'FaultDetail'};
            if ( $hFaultDetail && $hFaultDetail->{'CustomerPickPostNumberByElementsFault'}){
                my $standard = $hFaultDetail->{'CustomerPickPostNumberByElementsFault'}->{'standard'};
                $ErrorCode = $standard->{'ErrorCode'};
                $ErrorMsg = $standard->{'ErrorString'};
            } else {
                $ErrorCode = $Error->code;
                $ErrorMsg = $Error->shortMessage;
            }
        }
        elsif ( ! defined $ret ) {
            $ErrorCode = -1;
            $ErrorMsg = 'ELogistics call has no response';
        }
        if ( defined $ErrorCode || defined $ErrorMsg ) {
            LogShippingFulfillment('ELogistics','webservice call failed for shop: '.$Shop->pathString,{
                'errorcode'    => $ErrorCode,
                'errormessage' => $ErrorMsg,
            },'CallCustomerPickPostNumber');
            # also use Auth required for invalid Frankierlizenz
            if ( $ErrorCode && $ErrorCode eq 'WebServiceFault' && $ErrorMsg =~ /E2010/ ){
                $ErrorMsg = '401 Authorization Required';
            }
            Error('SOAP_REQUEST_FAILED', {'Code' => $ErrorCode, 'Message' => $ErrorMsg});
        }
        return $ret;
    };

    my $ret;
    if ( $self->doLog('ELogistics') ) {
        $ret =
          $soap->logCommunication( 'ELogistics::CallGenerateLabelRequest',
            $rSub );
    }
    else {
        $ret = &$rSub;
    }

    my $aResult = $ret->dataof('//GenerateLabelResponse/Envelope/Data/Provider/Sending/')->value;
    LogShippingFulfillment('ELogistics','aResult',$aResult,'CallGenerateLabelRequest');
    # convert to array of hashes
    my @aConvResult;
    push @aConvResult, $self->makeHash([${$_->value}->value]) foreach( @{[${$aResult}->value]} );
    LogShippingFulfillment('ELogistics','aResult',\@aConvResult,'CallGenerateLabelRequest');
    return \@aConvResult;

}

sub _getSoapClient {
    my $self = shift;
    my ( $WsURL, $User, $Password, $testMode ) = @_;

    my $soap = DE_EPAGES::WebService::API::Client
        ->proxy($WsURL, timeout => GetDefaultTimeOut)
        ->uri('https://www.mypostbusiness.ch/wsbc/barcode/v2_2/types')
        ->userinfo( $User.':'.$Password )
        ->encodingStyle('')
        ->autotype(0)
        ->soapversion('1.1')
        ->readable(1);

    LogShippingFulfillment('ELogistics','soap',$soap,'_getSoapClient');
    return $soap;
}

sub _buildGenerateLabelRequest{
    my $self = shift;
    my ( $Transaction, $ShippingFulfillmentMethod, $FileType, $LanguageID ) = @_;

    my $LabelDefinition = DE_EPAGES::ELogistics::API::SOAP::Barcode::LabelDefinition->new(
                                            'LabelLayout' => 'A6',
                                            'PrintAddresses' => 'RecipientAndCustomer',
                                            'ImageFileType' => $FileType,
                                            'ImageResolution' => 300,
                                            'PrintPreview' => 0);

    # get shop address
    my $ShopAddr = $Transaction->get('Site')->get('Address');
    my $Customer = DE_EPAGES::ELogistics::API::SOAP::Barcode::Customer->new(
                                            'Name' => ($ShopAddr->get('FirstName')//'').' '.($ShopAddr->get('LastName')//''),
                                            'Street' => $ShopAddr->get('Street'),
                                            'Zipcode' => $ShopAddr->get('Zipcode'),
                                            'City' => $ShopAddr->get('City'),
                                            'Country' => defined( $ShopAddr->get('Country') )
                                                            ? $ShopAddr->get('Country')->{'Code2'}
                                                            : undef );

    my $WebserviceLicenceNumber = $ShippingFulfillmentMethod->get('WebserviceLicenceNumber');
    my $FileInfos = DE_EPAGES::ELogistics::API::SOAP::Barcode::FileInfos->new(
                                            'FrankingLicense' => $WebserviceLicenceNumber,
                                            'PpFranking' => 'false',
                                            'Customer' => $Customer);

    my $LanguageCode = GetCodeByLanguageID($LanguageID);
    # default en if not in lang of switzerland
    $LanguageCode = 'en' unless ElementOf($LanguageCode, qw( de en fr it ));

    my $aItems = [];
    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        my $RecipientAddr = $ShippingLabel->get('Address');
        my $Recipient = DE_EPAGES::ELogistics::API::SOAP::Barcode::Recipient->new(
                                                'Name' => ($RecipientAddr->get('FirstName')//'').' '.($RecipientAddr->get('LastName')//''),
                                                'Name2' => $RecipientAddr->get('AddressExtension'),
                                                'Street' => $RecipientAddr->get('Street'),
                                                'Zipcode' => $RecipientAddr->get('Zipcode'),
                                                'City' => $RecipientAddr->get('City'),
                                                'Country' => defined( $RecipientAddr->get('Country') )
                                                                ? $RecipientAddr->get('Country')->{'Code2'}
                                                                : undef );
        # get base product
        my @attr = split(/,/, $ShippingLabel->get('ProductCode'));
        # get additional options
        push @attr , LoadObject($_->{'ShippingFulfillmentOptionID'})->get('ProductCode') foreach @{GetPKeyShippingLabelFFOptionsByShippingLabelID ($ShippingLabel->id)};
        my $Amount;

        # Add for RepairOrders (CST-227)
        my $Order = $ShippingLabel->get('Order');
        push (@attr,ZUSTELLAVISIERUNGREPAIRORDER) if $Order->get('isRepairOrder');

        $Amount = $ShippingLabel->get('Amount') if ElementOf('N', @attr); #amount only for N
        LogShippingFulfillment('ELogistics','Amount',$Amount,'_buildGenerateLabelRequest');
        my $Attributes = DE_EPAGES::ELogistics::API::SOAP::Barcode::Attributes->new(
                                                'PRZL' => \@attr,
                                                'Amount' => $Amount);

        my $aNotifications = [];
        my $FreeText = $ShippingFulfillmentMethod->get('EmailBody',$LanguageID);
        # retrieve LineItemShipping to get the Notification Info
        my $LineItemShipping = $ShippingLabel->get('Order')->get('LineItemContainer')->get('Shipping');

        if ( $LineItemShipping->get('NotifySms') ){
            my $RecipientMobile = $LineItemShipping->get('NotifyPhoneCell');
            # CellPhone number is stored on LineItemShipping only, so attempt to set it in the Customer Address if not yet set
            #$RecipientAddr->get('PhoneCell') = $LineItemShipping->get('NotifyPhoneCell') unless (defined $RecipientAddr->get('PhoneCell'));
            foreach my $Service ( ($ELOGISTICS_AVISIERUNG{'AUFGABEBESTAETIGUNG'},
                                   $ELOGISTICS_AVISIERUNG{'ZUSTELLINFORMATION'},
                                   $ELOGISTICS_AVISIERUNG{'ERINNERUNG_AN_EMPFAENGER'}) ){
                my $Sms = DE_EPAGES::ELogistics::API::SOAP::Barcode::Communication->new( 'Mobile' => $RecipientMobile);
                push( @{$aNotifications}, _createNotification('SMS',$Sms,$Service,$FreeText,$LanguageCode) );
            }
        }
        if ( $LineItemShipping->get('NotifyEmail') ){
            my $RecipientEmail = $RecipientAddr->get('EMail');
            foreach my $Service ( ($ELOGISTICS_AVISIERUNG{'AUFGABEBESTAETIGUNG'},
                                   $ELOGISTICS_AVISIERUNG{'ZUSTELLINFORMATION'},
                                   $ELOGISTICS_AVISIERUNG{'ERINNERUNG_AN_EMPFAENGER'}) ){
                my $Email = DE_EPAGES::ELogistics::API::SOAP::Barcode::Communication->new( 'Email' => $RecipientEmail);
                push( @{$aNotifications}, _createNotification('EMAIL',$Email,$Service,undef,$LanguageCode) );
            }
        }
        # Add for RepairOrders (CST-227)
        if ($Order->get('isRepairOrder')) {
            my $Email = DE_EPAGES::ELogistics::API::SOAP::Barcode::Communication->new('Email' => $RecipientAddr->get('EMail'));
            push(@{$aNotifications}, _createNotification('EMAIL',
                                                          $Email,
                                                          $ELOGISTICS_AVISIERUNG{'AUSTAUSCH_RUECKNAHME'},
                                                          undef,
                                                          $LanguageCode));
        }

        my $Item = DE_EPAGES::ELogistics::API::SOAP::Barcode::Item->new(
                                                'ItemID' => $ShippingLabel->get('GUID'),
                                                'Recipient' => $Recipient,
                                                'Attributes' => $Attributes,
                                                'Notification' => $aNotifications);
       push( @{$aItems}, $Item );
    }
    my $Sending = DE_EPAGES::ELogistics::API::SOAP::Barcode::Sending->new(
                                            'Items' => $aItems );

    my $Provider = DE_EPAGES::ELogistics::API::SOAP::Barcode::Provider->new(
                                            'Sending' => $Sending );

    my $Data = DE_EPAGES::ELogistics::API::SOAP::Barcode::Data->new(
                                            'Provider' => $Provider );

    my $Envelope = DE_EPAGES::ELogistics::API::SOAP::Barcode::Envelope->new(
                                            'LabelDefinition'=>$LabelDefinition,
                                            'FileInfos' => $FileInfos,
                                            'Data' => $Data );

    my $GenerateLabelRequest = DE_EPAGES::ELogistics::API::SOAP::Barcode::GenerateLabel->new(
                                            'Language'=> $LanguageCode,
                                            'Envelope' => $Envelope);

    return $GenerateLabelRequest;
}

sub _createNotification{
    my ($Type,$Communication,$Service,$FreeText,$LanguageCode) = @_;

    return DE_EPAGES::ELogistics::API::SOAP::Barcode::Notification->new(
                'Type'          => $Type,
                'Service'       => $Service,
                'FreeText1'     => $FreeText,
                'Language'      => $LanguageCode,
                'Communication' => $Communication,
            );
}
1;
