#========================================================================================
# package      DE_EPAGES::ELogistics::API::Object::RemoteELogisticsPickPost
# state        private
#----------------------------------------------------------------------------------------
# description  This package provides the ELogistics PickPost SOAP Connection
#========================================================================================
package DE_EPAGES::ELogistics::API::RemoteELogisticsPickPost;
use base DE_EPAGES::ELogistics::API::RemoteELogistics;

use strict;

use Encode qw (encode);
use MIME::Base64 qw(encode_base64);
use DE_EPAGES::WebService::API::Client;
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::Country qw( GetInfoCountry );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw (GetDefaultTimeOut);
use DE_EPAGES::ELogistics::API::SOAP::PickPost::CustomerPickPostNumber;
use DE_EPAGES::ELogistics::API::SOAP::PickPost::CustomerPickPostElements;


sub CallCustomerPickPostNumber {
    my $self = shift;
    my ($ShippingFulfillmentMethod,$hPickPostAddressData,$hFormValues,$CustomerNr,$LanguageID) = @_;
    my $Shop = $ShippingFulfillmentMethod->get('Site');

    my $isTestShop = $Shop->get('IsInternalTestShop');
    my $WebserviceURL = $isTestShop ?
                $ShippingFulfillmentMethod->get('ShippingFulfillmentType')->get('WebservicePickPostTestURL') :
                $ShippingFulfillmentMethod->get('ShippingFulfillmentType')->get('WebservicePickPostURL') ;

    my $soap = $self->_getSoapClient(   $WebserviceURL,
                                        $ShippingFulfillmentMethod->get('WebserviceUsernamePickPost'),
                                        $ShippingFulfillmentMethod->get('WebservicePasswordPickPost'),
                                        $isTestShop );

    my $Request = $self->_buildCustomerPickPostNumberRequest(
                                                    $ShippingFulfillmentMethod->get('WebserviceLicenceNumber'),
                                                    $hPickPostAddressData,
                                                    $hFormValues,
                                                    $CustomerNr,
                                                    $LanguageID);

    my $rSub = sub {
        my $ret;
        my $ErrorCode;
        my $ErrorMsg;
        eval {
            $ret = $soap->call('cus:CustomerPickPostNumberByElementsQuery', $Request->toSOAP());
        };
        if ( ExistsError() ) {
            my $Error = GetError();
            my $hFaultDetail = $Error->vars()->{'FaultDetail'};
            if ( $hFaultDetail && $hFaultDetail->{'CustomerPickPostNumberByElementsFault'}){
                my $standard = $hFaultDetail->{'CustomerPickPostNumberByElementsFault'}->{'standard'};
                $ErrorCode = $standard->{'ErrorCode'};
                $ErrorMsg = $standard->{'ErrorString'};
            } else {
                $ErrorCode = $Error->code;
                $ErrorMsg = $Error->shortMessage;
            }
        }
        elsif ( ! defined $ret ) {
            $ErrorCode = -1;
            $ErrorMsg = 'ELogistics call has no response';
        }
        if ( defined $ErrorCode || defined $ErrorMsg ) {
            LogShippingFulfillment('ELogistics','webservice call failed for shop: '.$Shop->pathString,{
                'errorcode'    => $ErrorCode,
                'errormessage' => $ErrorMsg,
            },'CallCustomerPickPostNumber');
            Error('SOAP_REQUEST_FAILED', {'Code' => $ErrorCode, 'Message' => $ErrorMsg});
        }
        return $ret;
    };
    my $ret;
    if ( $self->doLog('ELogistics') ) {
        $ret =
          $soap->logCommunication( 'ELogistics::CustomerPickPostNumber',
            $rSub );
    }
    else {
        $ret = &$rSub;
    }
    my $aResult = $ret->dataof('//CustomerPickPostNumberByElementsResponse/CustomerPickPostNumberResponse/')->value;
    LogShippingFulfillment('ELogistics','aResult',$aResult,'CallCustomerPickPostNumber');
    # we need an array
    $aResult = [$aResult] if ref($aResult) ne 'ARRAY';
    LogShippingFulfillment('ELogistics','aResult2',$aResult,'CallCustomerPickPostNumber');
    # convert to array of hashes
    my @aConvResult;
    push @aConvResult, $self->makeHash([$_->value]) foreach( map { ${$_} } @{$aResult} );
    LogShippingFulfillment('ELogistics','aConvResult',\@aConvResult,'CallCustomerPickPostNumber');
    return \@aConvResult;

}

sub _getSoapClient {
    my $self = shift;
    my ( $WsURL, $User, $Password, $testMode ) = @_;

    my $soap = DE_EPAGES::WebService::API::Client
        ->proxy($WsURL, timeout => GetDefaultTimeOut )
        ->encodingStyle('')
        ->autotype(0)
        ->soapversion('1.1')
        # the resulting SOAPAction will contain only the URI
        # per default it is #CustomerPickPostNumberByElementsQuery which seems not to work
        # for the new platform
        ->on_action(sub { sprintf '"%s"', shift })
        ->readable(1);

    my $serializer = $soap->serializer();
    $serializer->register_ns( 'http://post.ch/pickpost/customerpickpost', 'cus' );

    # use auth header
    my $Auth = 'Basic '.encode_base64(encode('utf-8',$User.':'.$Password));
    $Auth =~ s/\r|\n//g; #remove linebreak

    LogShippingFulfillment('ELogistics','soap_auth',$Auth,'_getSoapClient');
    $soap->transport->http_request->headers->push_header('Authorization' => $Auth);

    LogShippingFulfillment('ELogistics','soap',$soap,'_getSoapClient');
    return $soap;
}

sub _buildCustomerPickPostNumberRequest{
    my $self = shift;
    my ( $WebserviceLicenceNumber, $hPickPostAddressData, $hFormValues, $CustomerNr, $LanguageID ) = @_;

    LogShippingFulfillment('ELogistics','WebserviceLicenceNumber',$WebserviceLicenceNumber,'_buildCustomerPickPostNumberRequest');
    LogShippingFulfillment('ELogistics','hPickPostAddressData',$hPickPostAddressData,'_buildCustomerPickPostNumberRequest');
    LogShippingFulfillment('ELogistics','hFormValues',$hFormValues,'_buildCustomerPickPostNumberRequest');
    LogShippingFulfillment('ELogistics','CustomerNr',$CustomerNr,'_buildCustomerPickPostNumberRequest');
    LogShippingFulfillment('ELogistics','LanguageID',$LanguageID,'_buildCustomerPickPostNumberRequest');

    my $LanguageCode = GetCodeByLanguageID($LanguageID);
    # default de if not in lang of switzerland
    $LanguageCode = 'de' unless ElementOf($LanguageCode, qw( de fr it ));

    my $CountryID = $hPickPostAddressData->{'CountryID'};
    my $CountryCode = GetInfoCountry($CountryID)->{'Code2'};
    # default CH
    $CountryCode = 'CH' unless ElementOf($CountryCode, qw( CH LI DE FR IT AT ));

    my $NotifyMode;
    if ($hFormValues->{'NotifyEmail'} && $hFormValues->{'NotifySms'} ){
        $NotifyMode = 'SMS_EMAIL'
    } elsif ($hFormValues->{'NotifySms'}){
        $NotifyMode = 'SMS'
    } else{
        $NotifyMode = 'EMAIL'
    }

    my $MobileNumber = $hFormValues->{'NotifyPhoneCell'} // undef;

    my $CustomerPickPostElements = DE_EPAGES::ELogistics::API::SOAP::PickPost::CustomerPickPostElements->new(
                                            'CustomerNumber'    => $CustomerNr,
                                            'LastName'          => $hPickPostAddressData->{'LastName'},
                                            'FirstName'         => $hPickPostAddressData->{'FirstName'},
                                            'StreetAndNumber'   => $hPickPostAddressData->{'Street'},
                                            'StreetPostalCode'  => $hPickPostAddressData->{'Zipcode'},
                                            'CityName'          => $hPickPostAddressData->{'City'},
                                            'CountryCode'       => $CountryCode,
                                            'LanguageCode'      => uc($LanguageCode),
                                            'EmailAddress'      => $hPickPostAddressData->{'EMail'},
                                            'MobileNumber'      => $MobileNumber,
                                            'NotifyMode'        => $NotifyMode,
                                            );

    return DE_EPAGES::ELogistics::API::SOAP::PickPost::CustomerPickPostNumber->new(
                                            'FrankingLicenceNumber'=> $WebserviceLicenceNumber,
                                            'CustomerPickPostElements' => $CustomerPickPostElements->toSOAP()
                                            );
}
1;
