#========================================================================================
# package      DE_EPAGES::ELogistics::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the ELogistics global functionality.
#========================================================================================
package DE_EPAGES::ELogistics::API::Tools;

use base qw( Exporter );

use strict;

use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );

our @EXPORT_OK = qw (   GetLabelImagePath
                        IsValidCombination
                        IsPickPostAllowed
                        IsValidAddressCountry
                        MandatoryPickPostFieldsVisible );

#========================================================================================
# function     GetLabelImagePath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Path = GetLabelImagePath($ShippingLabel,$FileExtension);
# example      my $Path  = GetLabelImagePath($ShippingLabel,'gif');
#----------------------------------------------------------------------------------------
# description  returns the path for the barcode image of a shipping label
#----------------------------------------------------------------------------------------
# input        #ShippingLabel | shipping label object | object
# input        #FileExtension | image file extension | string
# return       #Path | path and image name | string
#========================================================================================
sub GetLabelImagePath {
    my ( $ShippingLabel,$FileExtension ) = @_;

    return sprintf '%s/barcode_%s.%s',
        $ShippingLabel->get('PrivatePath'),
        $ShippingLabel->id,
        $FileExtension // 'gif';
}

#========================================================================================
# function     IsValidCombination
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsValid = IsValidCombination($ShippingFulfillmentProduct,$ShippingFulfillmentOption);
# example      my $IsValid  = IsValidCombination($ShippingFulfillmentProduct,$ShippingFulfillmentOption);
#----------------------------------------------------------------------------------------
# description  checks if the combination of shipping fullfilment product
#               and shipping fullfilment option is valid
#----------------------------------------------------------------------------------------
# input        #ShippingFulfillmentProduct | shipping fullfillment product object | object
# input        #ShippingFulfillmentOption | shipping fullfillment option object | object
# return       #Value | 1 or 0 | boolean
#========================================================================================
sub IsValidCombination {
    my ( $ShippingFulfillmentProduct , $ShippingFulfillmentOption ) = @_;

    return 0 unless $ShippingFulfillmentProduct; # a product has to be selected
    TestObject('ShippingFulfillmentProduct',$ShippingFulfillmentProduct,
            'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentProduct');

    # a product may be selected, an option not
    return 1 unless $ShippingFulfillmentOption; # an option is not mandatory
    TestObject('ShippingFulfillmentOption',$ShippingFulfillmentOption,
            'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentOption');

    return 1 unless ElementOf($ShippingFulfillmentProduct->get('ProductCode'),
                              ('SP,ECO', 'SP,PRI'));
    return $ShippingFulfillmentOption->get('ProductCode') ne 'FRA';
}

#========================================================================================
# function     IsPickPostAllowed
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsAllowed = IsPickPostAllowed($ShippingMethod);
# example      my $IsAllowed  = IsPickPostAllowed($ShippingMethod);
#----------------------------------------------------------------------------------------
# description  checks if the pick post delivery is allowed for this shippingmethod
#----------------------------------------------------------------------------------------
# input        #ShippingMethod | shipping method object | object
# return       #Value | 1 or 0 | boolean
#========================================================================================
sub IsPickPostAllowed {
    my ( $ShippingMethod ) = @_;

    return 0 unless defined $ShippingMethod;
    TestObject('ShippingMethod',$ShippingMethod,'DE_EPAGES::Shipping::API::Object::ShippingMethod');

    my $ShippingFulfillmentMethod = $ShippingMethod->get('SelectedShippingFulfillmentMethod');
    return 0 unless defined $ShippingFulfillmentMethod;

    my $IsAllowed = (
                $ShippingFulfillmentMethod->get('ShippingFulfillmentType')->alias eq 'ELogistics'
            &&  defined($ShippingFulfillmentMethod->get('WebserviceLicenceNumber'))
            &&  defined($ShippingFulfillmentMethod->get('WebserviceUsernamePickPost'))
            &&  (
                    defined($ShippingFulfillmentMethod->get('ShippingFulfillmentType')->get('GoogleApiKeyPickPost'))
                 || defined($ShippingFulfillmentMethod->get('MerchantGoogleApiKeyPickPost') )
                )
            &&  defined $ShippingMethod->get('SelectedShippingFulfillmentProduct')
            &&  $ShippingMethod->get('SelectedShippingFulfillmentProduct')->get('ProductCode') ne 'SEM'
    );

    LogDebug( 'Tools:IsPickPostAllowed.IsAllowed', $IsAllowed );
    return $IsAllowed;
}

#========================================================================================
# function     IsValidAddressCountry
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsAllowed = IsValidAddressCountry($Address);
# example      my $IsAllowed  = IsValidAddressCountry($Address);
#----------------------------------------------------------------------------------------
# description  checks if the shipping address is usable
#----------------------------------------------------------------------------------------
# input        #Address | address object | object
# return       #Value | 1 or 0 | boolean
#========================================================================================
sub IsValidAddressCountry {
    my ( $Address ) = @_;

    my $IsValid = 1;
    $IsValid = 0 unless (   defined($Address)
                         && defined($Address->get('Country') )
                         && $Address->get('Country')->{'Code2'} eq 'CH' );

    LogDebug( sprintf('Tools:IsValidAddressCountry.IsValid(ID:%s)', defined($Address) ? $Address->id : 'undef'), $IsValid );
    return $IsValid;
}

#========================================================================================
# function     MandatoryPickPostFieldsVisible
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsAllowed = MandatoryPickPostFieldsVisible($Shop);
# example      my $IsAllowed  = MandatoryPickPostFieldsVisible($Shop);
#----------------------------------------------------------------------------------------
# description  checks all required fields are visible in the shop
#----------------------------------------------------------------------------------------
# input        #Shop | shop object | object
# return       #Value | 1 or 0 | boolean
#========================================================================================
sub MandatoryPickPostFieldsVisible {
    my ( $Shop ) = @_;

    my $RegistrationFields = $Shop->get('RegistrationField');
    foreach my $Alias ( qw( FirstName LastName AddressExtension Zipcode City CountryID )) {
        my $RegistrationField = $RegistrationFields->{$Alias};
        return 0 if ! $RegistrationField;
        return 0 if ! $RegistrationField->{'IsShippingAddressVisible'};
    }

    return 1;
}
1;
