#========================================================================================
# package      DE_EPAGES::Ebay::API::CSV::EbayAuctionImportHandler;
# base         DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to export ebay auction data to a CVS file.
#               See <package DE_EPAGES::Ebay::API::CSV::EbayAuctionExportImport> for
#               usage examples.
#========================================================================================
package DE_EPAGES::Ebay::API::CSV::EbayAuctionImportHandler;
use base DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler;

use strict;
use DateTime;
use JSON qw( to_json );
use DE_EPAGES::CSVExportImport::API::Tools qw( ParseValue );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError Error );
use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Ebay::API::Constants qw( AUCTION_EDITABLE AUCTION_VERIFIED);
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Product::API::Shop qw( GetProduct );

#ERRORDEFINITION
sub ErrorDefinition {
    my $hSuperErrorDefinition = __PACKAGE__->SUPER::ErrorDefinition();
    return {
        %$hSuperErrorDefinition,
        'VOID_FIELD' => 'line #Line, column #ID: #Error',
    }
}
#ERRORDEFINITION

## no critic qw(ProhibitLongLines)

#========================================================================================
# function     analyzeAttributeLine
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributeValues = $self->analyzeAttributeLine;
#----------------------------------------------------------------------------------------
# description  Analyzes the current line in the CSV file and returns the attribute
#               values that should be imported.
#----------------------------------------------------------------------------------------
# return       $hAttributeValues | attribute name => value hash | ref.hash
#========================================================================================
sub analyzeAttributeLine {
    my $self = shift;

    $self->{'DoNotImport'} = 0;
    my $aItems = $self->topID('Item');
    my $ImportAttributes = $self->topID('ImportAttributes');
    my $Shop = $self->object;
    my $Folder = $Shop->folder('EbayAuctions');

    my $hFormat = $self->locale->get( [qw(LocaleID DecimalSep ThousandSep)] );

    my %EbayAuctionAttributes = (
        Class => LoadClassByAlias('EbayAuction'),
        Parent => $Folder,
    );
    my %LocalizedAttributes;
    my @ItemSpecifics;
    for (my $i=0; $i<scalar(@$aItems) ;$i++) {
        my $hAttribute = $ImportAttributes->{$i};

        my $Value = $aItems->[$i];
        my $Type = $hAttribute->{'Type'};
        my $AttributeAlias = $hAttribute->{'Alias'};

        # strip leading and trailing spaces
        if( defined $Value ) {
            $Value =~ s/\s+$//; $Value =~ s/^\s+//;
            $Value = undef if $Value eq '';
        }


       if ($hAttribute->{'IsObject'}) {
            # prevent empty EbayShopEnvironment attribute (EbayShopEnvironment is mandatory)
            if ($Type eq 'EbayShopEnvironment') {
                my $ShopEnvFolder = $Shop->folder('EbayShopEnvironments');
                $self->error('NO_MAND_VALUE', {
                        Attribute => $AttributeAlias,
                        ID => $i+1,
                        Value => $Value
                }) unless $Value;

                if ($ShopEnvFolder->existsChild($Value)){
                    $EbayAuctionAttributes{$AttributeAlias} = $ShopEnvFolder->child($Value);
                } else {
                    $self->error('INVALID_SHOPEBAYENVIRONMENT', {
                        Attribute => $AttributeAlias,
                        ID => $i+1,
                        Value => $Value
                    });
                }
            }elsif($Type eq 'ProductAttribute' and $AttributeAlias eq 'PicVariationAttribute' ) {
                $EbayAuctionAttributes{$AttributeAlias} = $Value;
            }else {
                $EbayAuctionAttributes{$AttributeAlias} = undef;
            }

        }
        else { # no Object

            # prevent empty EbayCategoryId attribute (EbayCategoryId is mandatory)
            if (not defined $Value and $AttributeAlias eq 'EbayCategoryId') {
                $self->error('NO_MAND_VALUE', {
                        Attribute => $AttributeAlias,
                        ID => $i+1,
                        Value => $Value
                });
                $self->{'DoNotImport'} = 1;
                next;
            }

            # strip leading and trailing spaces
            if( defined $Value ) {

                # eBayCategoryAttribute<number>-columns are joint into the ItemSpecifics-attribute as json
                # Sample: "Untersttzte Druckformate":"A4 (210 x 297 mm)";"A5 (148 x 210 mm)";"IsOther-A180"
                if ($AttributeAlias =~ m/eBayCategoryAttribute/) {

                    my $helper = $Value;
                    $helper =~ s/^(".*":)+(".*?"(;(".*?"))*)*//;
                    if (!$helper eq '') {
                        $self->error('WRONG_ATTRIBUTE_VALUE_FORMAT', {
                                Attribute => $AttributeAlias,
                                ID => $i+1,
                                Value => $Value
                        });
                        $self->{'DoNotImport'} = 1;
                        next;
                    };

                    my @CategoryAttributeArray = split(/:/,$Value);
                    my @InnerCategoryAttributeArray = split(/;/,$CategoryAttributeArray[1]);

                    foreach my $InnerAttributeValue (@InnerCategoryAttributeArray) {
                        my @ValueArray = split(/IsOther-/,_unquote($InnerAttributeValue));
                        push @ItemSpecifics, to_json({
                            'Name' => _unquote($CategoryAttributeArray[0]),
                            'Value' => exists($ValueArray[1]) ? $ValueArray[1] : $ValueArray[0],
                            'IsOther' => exists($ValueArray[1]) ? '1' : ''
                        });
                    }
                    next;
                }

                # note: ParseValue sets $Value to undef if it was ''
                eval { ParseValue( $AttributeAlias, $Type, \$Value, $hFormat ) };
                if ( ExistsError() ) {
                    $self->error('VOID_FIELD', {
                        Attribute => $AttributeAlias,
                        ID => $i+1,
                        Value => $Value
                    });
                    next;
                }
                $EbayAuctionAttributes{$AttributeAlias} = $Value;
            }

            if ($Type =~ /^Localized/) {
                $LocalizedAttributes{$hAttribute->{'LanguageID'}}->{$AttributeAlias} = $Value;
            }
            else {
                # delete all empty values in import file to prevent overriding the default auction values!
                #$EbayAuctionAttributes{$AttributeAlias} = $Value;
                delete $EbayAuctionAttributes{$AttributeAlias} unless defined $Value;

            }
        }
    }


    my $EbayAuction;
    if ($EbayAuctionAttributes{'Alias'} and $Folder->existsChild($EbayAuctionAttributes{'Alias'})) {
        $EbayAuction = $Folder->child($EbayAuctionAttributes{'Alias'});
        my $Status = $EbayAuction->get('Status');
        $EbayAuction = undef if (($Status != AUCTION_EDITABLE) and ($Status != AUCTION_VERIFIED));
    }

    if (scalar(@ItemSpecifics) > 0) {
        $EbayAuctionAttributes{'ItemSpecifics'} = '[' . join(',',@ItemSpecifics) . ']';
    }

    my $ShopEnv = $EbayAuctionAttributes{'EbayShopEnvironment'};
    my $Env = $ShopEnv->get('EbayEnvironment');

    # Get parent -> is sub ebay auction ?
    $self->_getEbayAuctionSubInfo( $aItems, $Shop, $ImportAttributes );

    my $Product = $self->_getProduct( $aItems, $Shop, $ImportAttributes );
    $EbayAuctionAttributes{'Product'} = $Product;

    # try to find EbayUserTemplate
    my $CSVEbayUserTemplate = $self->_getEbayUserTemplate( $aItems, $Shop, $ImportAttributes );
    $EbayAuctionAttributes{'EbayUserTemplate'} = $CSVEbayUserTemplate;

    # find out name and description form csv or from product
    my ($Name,$Descr) = $self->_getNameAndDescription( $aItems, $Shop, $ImportAttributes, $Product,$Env, $CSVEbayUserTemplate );


    if (!$self->{'DoNotImport'}) {
        my $DefaultAuction = $Shop->get('EbayDefaultAuction');

        my $params = {
            'Alias'             => $Folder->nextAlias('EbayAuction%s'),
            'Parent'            => $Folder,
            'Product'           => $Product,
            'Status'            => AUCTION_EDITABLE,
            'ExternalAuctionId' => undef,
            'CurrentBid'        => 0,
            'BidCount'          => 0,
            'StartTime'         => GetCurrentDBHandle()->currentDateTime,
        };

        # clone to get default values (e.g. shipping, payment) or use existing auction, if editable
        if (not defined $EbayAuction) {

            # test if we try to import auction with existing alias
            $self->error('EBAYAUCTION_EXISTS_NOT_EDITABLE', {
                Attribute => 'Alias' ,
                ID => $ImportAttributes->{'AliasColumn'}+1,
                Value => $EbayAuctionAttributes{'Alias'}
            }) if $EbayAuctionAttributes{'Alias'} and $Folder->existsChild($EbayAuctionAttributes{'Alias'});

            $EbayAuction = $DefaultAuction->clone( $params );
        } else {
            # changed auction is always editable
            $EbayAuction->set({'Status' => AUCTION_EDITABLE });
        }



        # change attributes here for overriding defaults on new generated auction

        $EbayAuctionAttributes{'Description'} = $Descr;
        $EbayAuctionAttributes{'Name'} = $Name;

        my $ListingType = $EbayAuctionAttributes{'ListingType'} ? $EbayAuctionAttributes{'ListingType'} : $EbayAuction->get('ListingType');
        # we allow IsReviseInventory only if we have a ListingType "FixedPriceItem" and not "Chinese"
        $EbayAuctionAttributes{'IsReviseInventory'} = 0 if $ListingType eq 'Chinese';

        # If we have a ListingType "Chinese" we only allow Quantity == 1
        $EbayAuctionAttributes{'Quantity'} = 1 if $ListingType eq 'Chinese';

        $EbayAuctionAttributes{'StartPrice'} =  $self->_getStartPrice($aItems, $Shop, $ImportAttributes, $Product,$Env, $ListingType);

        $EbayAuctionAttributes{'BuyItNowPrice'} = $self->_getBuyItNowPrice($aItems, $Shop, $ImportAttributes, $Product,$Env, $ListingType);

        # set ScheduleType to 0 if we not have a ScheduleTime
        $EbayAuctionAttributes{'ScheduleType'} = 0 unless $EbayAuctionAttributes{'ScheduleTime'} ;

        # convert GaleryUrl and PictureUrl to image URLs
        $EbayAuctionAttributes{'GalleryUrl'} = $self->_getGalleryAndPictureUrl('GalleryUrl',\%EbayAuctionAttributes,$Product,$DefaultAuction);

        # test if we have a ParentAuction attribute in csv and set the correct image for the sub auction
        $EbayAuctionAttributes{'ParentAuction'} = $self->{'ParentAuction'} if $self->{'ParentAuction'};

        if($self->{'ParentAuction'} and not $EbayAuctionAttributes{'PictureUrl'} ) {
            $EbayAuctionAttributes{'PictureUrl'} = $self->_getGalleryAndPictureUrl('PicVariationType',\%EbayAuctionAttributes,$Product,$self->{'ParentAuction'});
        }else {
            $EbayAuctionAttributes{'PictureUrl'} = $self->_getGalleryAndPictureUrl('PictureUrl',\%EbayAuctionAttributes,$Product,$DefaultAuction);
        }

        # set PicVariationAttribute for ParentAuction
        my $AttributeAlias = $EbayAuctionAttributes{'PicVariationAttribute'};
        if ($AttributeAlias and $Product->class->existsAttribute($AttributeAlias)) {
            $EbayAuctionAttributes{'PicVariationAttribute'} = $Product->class->attribute($AttributeAlias);
        }else{
            $EbayAuctionAttributes{'PicVariationAttribute'} = undef;
        }


    }

    return {
        DoNotImport => $self->{'DoNotImport'},
        Object => $EbayAuction,
        DirectAttributes => \%EbayAuctionAttributes,
        LocalizedAttributes => \%LocalizedAttributes,
    };
}

sub _unquote {
    my ( $Value ) = @_;
    $Value =~ s/^"//;
    $Value =~ s/"$//;

    return $Value;
}

#========================================================================================
# function     modifyUsedAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->modifyUsedAttributes($hUsedAttributes, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  add keys:
#                 $hUsedAttributes->{$ColumnIndex}{'IsParentAuctionAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'ParentAuction'  ? 1 : 0;
#                 $hUsedAttributes->{'ParentAuctionAliasColumn'} with value=$ColumnIndex  if $hUsedAttributes->{$ColumnIndex}{'IsParentAuctionAliasColumn'};
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
# input        $ColumnIndex | current column | integer
#========================================================================================
sub modifyUsedAttributes {
    my $self = shift;
    my ( $hUsedAttributes, $ColumnIndex) = @_;
    $self->SUPER::modifyUsedAttributes($hUsedAttributes, $ColumnIndex);

    $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'Alias'  ? 1 : 0;
    $hUsedAttributes->{'AliasColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsParentAuctionAliasColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'ParentAuction'  ? 1 : 0;
    $hUsedAttributes->{'ParentAuctionAliasColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsParentAuctionAliasColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsProductColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Product'  ? 1 : 0;
    $hUsedAttributes->{'ProductColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsProductColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsEbayCategoryIdColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'EbayCategoryId'  ? 1 : 0;
    $hUsedAttributes->{'EbayCategoryIdColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsEbayCategoryIdColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsEbayUserTemplateColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'EbayUserTemplate'  ? 1 : 0;
    $hUsedAttributes->{'EbayUserTemplateColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsEbayUserTemplateColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsNameColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Name'  ? 1 : 0;
    $hUsedAttributes->{'NameColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsNameColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsDescriptionColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Description'  ? 1 : 0;
    $hUsedAttributes->{'DescriptionColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsDescriptionColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsEbayShopEnvironmentColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'EbayShopEnvironment'  ? 1 : 0;
    $hUsedAttributes->{'EbayShopEnvironmentColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsEbayShopEnvironmentColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsListingTypeColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'ListingType'  ? 1 : 0;
    $hUsedAttributes->{'ListingTypeColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsListingTypeColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsQuantityColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Quantity'  ? 1 : 0;
    $hUsedAttributes->{'QuantityColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsQuantityColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsStartPriceColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'StartPrice'  ? 1 : 0;
    $hUsedAttributes->{'StartPriceColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsStartPriceColumn'};

    $hUsedAttributes->{$ColumnIndex}{'IsBuyItNowPriceColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'BuyItNowPrice'  ? 1 : 0;
    $hUsedAttributes->{'BuyItNowPriceColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsBuyItNowPriceColumn'};

    return;
}

#========================================================================================
# function     testMissingFields
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->testMissingFields($hUsedAttributes);
#----------------------------------------------------------------------------------------
# description  overrides super function to prevent Alias validation and
#               throws error 'MISSING_FIELD', if field some mandatory field are missing
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
#========================================================================================
sub testMissingFields {
    my $self = shift;
    my ( $hUsedAttributes ) = @_;

    Error( 'MISSING_FIELD' => { 'Value' => 'EbayCategoryId', 'FileName' => $self->{'FileName'}})
    unless exists $hUsedAttributes->{'EbayCategoryIdColumn'};
    Error( 'MISSING_FIELD' => { 'Value' => 'Product', 'FileName' => $self->{'FileName'}})
    unless exists $hUsedAttributes->{'ProductColumn'};
    Error( 'MISSING_FIELD' => { 'Value' => 'EbayShopEnvironment', 'FileName' => $self->{'FileName'}})
    unless exists $hUsedAttributes->{'EbayShopEnvironmentColumn'};

}

sub _getNameAndDescription {
    my $self = shift;
    my ( $aColumns, $Shop, $ImportAttributes,$Product,$Env, $CSVEbayUserTemplate ) = @_;

    my $name = $ImportAttributes->{'NameColumn'} ? $aColumns->[$ImportAttributes->{'NameColumn'}] : undef;
    my $descr = $ImportAttributes->{'DescriptionColumn'} ? $aColumns->[$ImportAttributes->{'DescriptionColumn'}] : undef;

    my $DefaultAuction = $Shop->get('EbayDefaultAuction');

    # get lang of ebay site
    my $langid = GetPKeyLanguageByCode(lc $Env->get('LanguageID'));
    # use default shop lang if ebay site's lang isn't active in the shop
    $langid = undef unless ElementOf($langid, (map {$_->{'LanguageID'}} @{$Shop->get('Languages')}));

    if ($name eq ''){
        $name = $Product->getWithParent('NameOrAlias', $langid) ||
               $Product->getWithParent('NameOrAlias');
    }

    # do we have a template from csv?
    my $TemplateForProcessing;
    $TemplateForProcessing = $CSVEbayUserTemplate ? $CSVEbayUserTemplate : $DefaultAuction->get('EbayUserTemplate') ;

    # get description from defaulttemplate, product or from csvtemplate
    # and process with default auction template or given template in csv

    if ($descr eq ''){
        $descr = $TemplateForProcessing->process( $Product, $langid ) if ($TemplateForProcessing);
        $descr = $Product->getWithParent('Text', $langid) ||
             $Product->getWithParent('Description', $langid) ||
             $Product->getWithParent('Text') ||
             $Product->getWithParent('Description') if $descr eq '';
    }else {
        $descr = $TemplateForProcessing->process( $Product, $langid , {'Description'=> $descr} )
         if ($TemplateForProcessing);
    }

    return ($name,$descr);
}


sub _getStartPrice {
    my $self = shift;
    my ( $aColumns,$Shop,$ImportAttributes,$Product,$Env, $ListingType) = @_;

    my $DefaultAuction = $Shop->get('EbayDefaultAuction');
    my $Price = $ImportAttributes->{'StartPriceColumn'} ? $aColumns->[$ImportAttributes->{'StartPriceColumn'}] : undef;

    # no price from csv
    if(not $Price) {
        # set start price to default auction price if chinese listing
        if ($ListingType eq 'Chinese' ) {
            $Price =  $DefaultAuction->get('StartPrice');
        } else {
            # get price from product
            $Price = $self->_productPrice($Shop,$Product,$Env);
        }
    }

    my $Formatter = $Shop->siteLocale($Env->get('LocaleID'));
    return $Formatter->parse($Price, $Env->get('CurrencyID'));
}

sub _getBuyItNowPrice {
    my $self = shift;
    my ( $aColumns,$Shop,$ImportAttributes,$Product,$Env, $ListingType) = @_;

    my $Price = $ImportAttributes->{'BuyItNowPriceColumn'} ? $aColumns->[$ImportAttributes->{'BuyItNowPriceColumn'}]
     : undef;

    # no price from csv
    # set start price to default auction price (productprice) if chinese listing
    if(not $Price and $ListingType eq 'Chinese') {
        $Price = $self->_productPrice($Shop,$Product,$Env);
    }

    my $Formatter = $Shop->siteLocale($Env->get('LocaleID'));
    return $Formatter->parse($Price, $Env->get('CurrencyID'));
}


sub _productPrice {
    my $self = shift;
    my ( $Shop,$Product,$Env) = @_;

    my $Price;
    eval {
        # get price from product
        $Price = $Product->price({
            'CurrencyID'    => $Env->get('CurrencyID'),
            'TaxModel'      => $Shop->get('TaxModel'),
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'ProductNoPrice') {
            $Price = 0;
        } else {
            Error($Error);
        }
    }

    return $Price;

}

sub _getGalleryAndPictureUrl {
    my $self = shift;
    my ($ImageType, $EbayAuctionAttributes,$Product,$DefaultAuction) = @_;

    my $Image;

    # use URL from default auction
    if (defined $DefaultAuction->get($ImageType) and $DefaultAuction->get($ImageType) =~ /\S/) {
        $Image = $DefaultAuction->get($ImageType);
    }

    # test if we have csv values for GalleryUrl
    if (defined $EbayAuctionAttributes->{$ImageType} and $EbayAuctionAttributes->{$ImageType} =~ /\S/) {
        $Image = $EbayAuctionAttributes->{$ImageType};
    }

    if(defined $Image) {
        if  ($Image eq 'ListView') {
            $Image = $Product->getWithParent('ImageSmall');
        } elsif ($Image eq 'DetailView') {
            $Image = $Product->getWithParent('ImageMedium');
        } elsif ($Image eq 'ZoomView') {
            $Image = $Product->getWithParent('ImageLarge');
        } elsif ($Image eq 'HotDealView') {
            $Image = $Product->getWithParent('ImageHotDeal');
        }
        if ($Image =~ /\/\//) { #contains //hostname
        } else {
            my $ProtocolServerName = $Product->getSite()->get('ProtocolAndServer');
            $Image = $ProtocolServerName .  $Product->get('WebPath') . '/' . $Image;
        }
    }

    return $Image;

}

sub _getEbayAuctionSubInfo {
    my $self = shift;
    my ( $aColumns, $Shop, $ImportAttributes ) = @_;

    $self->{'ParentAuction'} =  undef;
    my $EbayAuctionFolder = $Shop->folder('EbayAuctions');
    return $EbayAuctionFolder unless defined $ImportAttributes->{'ParentAuctionAliasColumn'};

    my $ParentAuctionAlias = $aColumns->[ $ImportAttributes->{'ParentAuctionAliasColumn'} ];
    if ( defined $ParentAuctionAlias ) {
        $ParentAuctionAlias =~ s/\s+$//;
        $ParentAuctionAlias =~ s/^\s+//;
        $ParentAuctionAlias = undef if $ParentAuctionAlias eq '';
    }
    return $EbayAuctionFolder unless defined $ParentAuctionAlias;

    if ( !$EbayAuctionFolder->existsChild($ParentAuctionAlias) ) {
        $self->error('NO_PARENT_AUCTION', {
            ID => $ImportAttributes->{'ParentAuctionAliasColumn'}+1,
            Parent => $ParentAuctionAlias
        });
        return $EbayAuctionFolder;
    }

    $self->{'ParentAuction'} = $EbayAuctionFolder->child($ParentAuctionAlias);
    return $EbayAuctionFolder;
}

sub _getProduct {
    my $self = shift;
    my ( $aColumns, $Shop, $ImportAttributes ) = @_;

    return unless defined $ImportAttributes->{'ProductColumn'};

    my $ProductAlias = $aColumns->[$ImportAttributes->{'ProductColumn'}];
    $ProductAlias =~ s/\s+$//; $ProductAlias =~ s/^\s+//;

    if ($ProductAlias eq '') {
        $self->error('NO_MAND_VALUE', {
            Attribute => 'Product',
            ID => $ImportAttributes->{'ProductColumn'}+1,
            Value => $ProductAlias
        });
        return;
    }
    my $Product = GetProduct($Shop, $ProductAlias);
    if (not defined $Product) {
        $self->error('EBAYAUCTION_INVALID_PRODUCT', {
            ID => $ImportAttributes->{'ProductColumn'}+1
        });
        return;
    }

    if ($Product->get('IsDownloadProduct')) {
        Error('CanNotAuctionDownloadProduct', {'ProductID', $Product->id, 'Product' => $Product});
    }

    return $Product;
}


sub _getEbayUserTemplate {
    my $self = shift;
    my ( $aColumns, $Shop, $ImportAttributes ) = @_;

    return unless defined $ImportAttributes->{'EbayUserTemplateColumn'};

    my $TemplateAlias = $aColumns->[$ImportAttributes->{'EbayUserTemplateColumn'}];
    $TemplateAlias =~ s/\s+$//; $TemplateAlias =~ s/^\s+//;

    return if $TemplateAlias eq '';
    my $TemplateFolder = $Shop->folder('EbayUserTemplates');

    my $Template = $TemplateFolder->existsChild($TemplateAlias) ? $TemplateFolder->child($TemplateAlias) : undef;

    if (not defined $Template) {
        $self->error('EBAYAUCTION_INVALID_TEMPLATE', {
            ID => $ImportAttributes->{'EbayUserTemplateColumn'}+1,
            Value => $TemplateAlias
        });
        return;
    }

    return $Template;
}

#========================================================================================
# function     importCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $importer->importCSV($FileName, $hFormat, $aAttributes, $ViewObject);
#----------------------------------------------------------------------------------------
# description  imports data (2 dim. array) into a file.
#----------------------------------------------------------------------------------------
# input        $FileName |  import file | string
# input        $hFormat |  import file format, monitor| ref.hash
# input        $aAttributes | column attributes (used for header and values) | ref.array of hashes
# input        $ViewObject | object to view monitor | object
# input        $aObjects | list of objects | ref.array.object
#----------------------------------------------------------------------------------------
# return       status of succeeded import
#========================================================================================
sub importCSV {
    my $class = shift;
    my ($FileName, $hFileFormat, $aAttributes, $ViewObject) = @_;

    my @ExtendedAttributes = @$aAttributes;

    for ( my $i=1; $i<=30; $i++ ) { ##no critic qw(ProhibitMagicNumbers)
        push @ExtendedAttributes, {
            'Alias' => 'eBayCategoryAttribute' . $i,
            'Type' => 'String',
            'IsObject' => '0',
            'IsBaseClassAttribute' => 1,
            'AliasString' => 'eBayCategoryAttribute' . $i,
            'Name' => 'eBayCategoryAttribute' . $i,
            'HeaderString' => 'eBayCategoryAttribute' . $i . ' ['. 'eBayCategoryAttribute' . $i . ']'
        };
    }

    return $class->SUPER::importCSV($FileName, $hFileFormat, \@ExtendedAttributes, $ViewObject);
}


sub error {
    my $self = shift;
    my ( $ErrorCode, $hErrorVars ) = @_;

    $hErrorVars->{'Line'} =  $self->lineNumber;
    Error( $ErrorCode, $hErrorVars );

    $self->{'DoNotImport'} = 1;
}
1;
