#========================================================================================
# package      DE_EPAGES::Ebay::Hooks::EbayShopEnvironment
# state        public
# description  hook functions for EbayShopEnvironment
#========================================================================================
package DE_EPAGES::Ebay::Hooks::EbayShopEnvironment;
use base qw( Exporter );

use strict;
use DateTime::Duration;

#========================================================================================
# function     OnBeforeUpdateEbayShopEnvironment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateEbayShopEnvironment( $hParams );
#----------------------------------------------------------------------------------------
# description  OnBeforeUpdateEbayShopEnvironment (OBJ_BeforeUpdateEbayShopEnvironment)
#               is triggered when a EbayShopEnvironment is updated.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object        - EbayShopEnvironment object - object
#                 <li>EbayShopEnvironmentID - object id of EbayShopEnvironment - int
#                 <li>Values        - updated values - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateEbayShopEnvironment {
    my ($hParams) = @_;

    my $EbayShopEnvironment = $hParams->{'Object'};
    return unless defined $EbayShopEnvironment;
    my $Shop = $EbayShopEnvironment->getSite();

    # we do the following actions only for unity shops
    if ($Shop->get('IsUnityRegistered')) {
        my $hValues = $hParams->{'Values'};
        # test if we can delete the notification for EbayAuthExpire 
        _DeleteEbayAuthNotification($Shop, $EbayShopEnvironment) if $hValues->{'EbayTokenExpiryDate'};

        # test if we can delete the notification for EbayNoOAuthToken 
        _DeleteEbayNoOAuthTokenNotification($Shop, $EbayShopEnvironment) if $hValues->{'EbayOAuthToken'};

    }
}

#========================================================================================
# function     OnDeleteEbayShopEnvironment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteEbayShopEnvironment( $hParams );
#----------------------------------------------------------------------------------------
# description  OnDeleteEbayShopEnvironment (OBJ_DeleteEbayShopEnvironment)
#               is triggered when a EbayShopEnvironment is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object        - EbayShopEnvironment object - object
#                 <li>EbayShopEnvironmentID - object id of EbayShopEnvironment - int
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteEbayShopEnvironment {
    my ($hParams) = @_;

    my $EbayShopEnvironment = $hParams->{'Object'};
    return unless defined $EbayShopEnvironment;
    my $Shop = $EbayShopEnvironment->getSite();

    # we do the following actions only for unity shops
    if ($Shop->get('IsUnityRegistered')){
        _DeleteEbayAuthNotification($Shop, $EbayShopEnvironment);
        _DeleteEbayNoOAuthTokenNotification($Shop, $EbayShopEnvironment);
    }
}

#========================================================================================
# function     OnAfterDeleteEbayShopEnvironment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterDeleteEbayShopEnvironment( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterDeleteEbayShopEnvironment (OBJ_AfterDeleteEbayShopEnvironment)
#               is triggered after a EbayShopEnvironment is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object        - EbayShopEnvironment object - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterDeleteEbayShopEnvironment {
    my ($hParams) = @_;

    my $EbayShopEnvironment = $hParams->{'Object'};
    my $Shop = $EbayShopEnvironment->getSite();

    #set a remaining EbayShopEnvironment as default if the former default was deleted
    my $aEbayShopEnvironments = $Shop->get('EbayShopEnvironments');
    return unless defined $aEbayShopEnvironments;
    my $EbayDefaultAuction = $Shop->get('EbayDefaultAuction');

    $EbayDefaultAuction->set({'EbayShopEnvironment' => @$aEbayShopEnvironments[0]});

}

#========================================================================================
# function     _DeleteEbayAuthNotification
# state        private
#----------------------------------------------------------------------------------------
# syntax       _DeleteEbayAuthNotification( $Shop );
#----------------------------------------------------------------------------------------
# description  deletes the ebay auth notification for a given Shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#========================================================================================
sub _DeleteEbayAuthNotification {
    my ($Shop, $EbayShopEnvironment) = @_;

    # delete the notification for EbayAuthExpire 
    my $EbayAuthExpireNotification = $Shop->folder('ShopNotifications')->child('EbayAuthExpire');
    $EbayAuthExpireNotification->deleteObject($EbayShopEnvironment);    

    return;
}

#========================================================================================
# function     _DeleteEbayNoOAuthTokenNotification
# state        private
#----------------------------------------------------------------------------------------
# syntax       _DeleteEbayNoOAuthTokenNotification( $Shop );
#----------------------------------------------------------------------------------------
# description  deletes the ebay OAuth auth notification for a given Shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#========================================================================================
sub _DeleteEbayNoOAuthTokenNotification {
    my ($Shop, $EbayShopEnvironment) = @_;

    # delete Notification for missing OAuth token
    return unless $Shop->folder('ShopNotifications')->existsChild('EbayNoOAuthToken');

    my $EbayNoOAuthTokenNotification = $Shop->folder('ShopNotifications')->child('EbayNoOAuthToken');
    $EbayNoOAuthTokenNotification->deleteObject($EbayShopEnvironment);    

    return;
}



1;