#========================================================================================
# package      DE_EPAGES::Ebay::UI::Ebay
# state        private
# base         DE_EPAGES::Presentation::UI::Object
#========================================================================================
package DE_EPAGES::Ebay::UI::Ebay;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf AddDuration );
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Ebay::API::Object::EbayCategoryTree;
use DE_EPAGES::Ebay::API::Object::RemoteRESTEbay;
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards ReplaceWildcardsWithoutPosOne );
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#ERRORDEFINITION
sub ErrorDefinition { return {} }
#ERRORDEFINITION END

#==============================================================================
# Event handlers
#==============================================================================
sub ViewDefault {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $DefaultAuction = $Shop->get('EbayDefaultAuction');

    # get info of category
    my $ExternalCategoryId = $DefaultAuction->get('EbayCategoryId');
    my $ExternalSecondCategoryId = $DefaultAuction->get('SecondCategoryId');

    my $ShopEnv = $DefaultAuction->get('EbayShopEnvironment');
    if (defined $ShopEnv) {
        my $Env = $DefaultAuction->get('EbayShopEnvironment')->get('EbayEnvironment');
        my $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($Env);
        my $categoryId = $EbayCategoryTree->getPKeyEbayCategoryByExternalCategoryId($ExternalCategoryId);
        my $hCategory = $EbayCategoryTree->getInfoEbayCategory($categoryId);
        if (defined $hCategory) {
            $hCategory->{'Path'} = $EbayCategoryTree->getCategoryPath($hCategory->{'EbayCategoryID'} );
            $Servlet->params('EbayCategory', $hCategory);
        }

        my $secondcategoryId = $EbayCategoryTree->getPKeyEbayCategoryByExternalCategoryId($ExternalSecondCategoryId);
        my $hSecondCategory = $EbayCategoryTree->getInfoEbayCategory($secondcategoryId);
        if (defined $hSecondCategory) {
            $hSecondCategory->{'Path'} = $EbayCategoryTree->getCategoryPath($hSecondCategory->{'EbayCategoryID'} );
            $Servlet->params('SecondEbayCategory', $hSecondCategory);
        }

        eval {
            my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
                'ShopEnvironment' => $ShopEnv,
            );

            # load Ebay condition values for specified category
            if ($ExternalCategoryId > 0) {
                my $ConditionValues = $RemoteEbay->getCategoryFeatures($ExternalCategoryId,[qw(ProductRequiredEnabled ConditionValues ItemSpecificsEnabled)],0,1);
                if (defined $ConditionValues
                    and ref $ConditionValues eq 'HASH'
                    and ref $ConditionValues->{'ConditionValues'} eq 'HASH'
                    and defined($ConditionValues->{'ConditionValues'}->{'Condition'})) {
                        my $ConditionDisplayName;
                        if (ref $ConditionValues->{'ConditionValues'}->{'Condition'} eq 'HASH') {
                            $ConditionDisplayName = $ConditionValues->{'ConditionValues'}->{'Condition'}->{'DisplayName'};
                        } elsif (scalar @{$ConditionValues->{'ConditionValues'}->{'Condition'}}) {
                            foreach my $Condition (@{$ConditionValues->{'ConditionValues'}->{'Condition'}}) {
                                if ($Condition->{'ID'} == $DefaultAuction->get('ConditionID')) {
                                    $ConditionDisplayName = $Condition->{'DisplayName'};
                                }
                            }
                        }
                    $Servlet->vars('ConditionDisplayName', $ConditionDisplayName);
                }
            }

            # load EbayStore categories
            my $hresult         = $RemoteEbay->getEbayDetails('ReturnPolicyDetails');
            my $aReturnsWithin  = $self->getReturnsWithinOptionsCodes($hresult);
            $self->addReturnsWithinOptionsCodes($Servlet,$aReturnsWithin);

            my $aStoreCategories = $RemoteEbay->getStore()->{'CustomCategories'};
            $Servlet->vars('EbayStoreCategories', $aStoreCategories);
        };
        if (ExistsError()) {
            my $Error = GetError();
            if(ElementOf($Error->code, qw( EbayNoToken EbayTokenError ))) {
                $Servlet->vars('EbayWarningNoToken', 1);
            } elsif ($Error->code eq 'EbayRequestNotSuccessful') {
                my $aEbayErrors = $Error->vars->{'EbayErrors'};

                # check if the sole error is "User has no shop"
                if (scalar @$aEbayErrors == 1 and grep {$_->{'ErrorCode'} == 13003} @$aEbayErrors) { ##no critic qw(ProhibitMagicNumbers)
                    GetLog->debug('Shop has no EbayStore.');
                } else {
                    # all other warnings will be shown to the merchant
                    $Servlet->vars('EbayErrors', $aEbayErrors);
                }
            } else {
                Error($Error);
            }
        }
    }

    return;
}

sub ViewEbayAuctions {
    my $self         = shift;
    my ($Servlet)    = @_;

    return $self->SearchEbayAuctions( $Servlet );
}

sub UnityViewEbayAuctions {
    my $self         = shift;
    my ($Servlet)    = @_;

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnityViewEbayAuctions', { 'Servlet' => $Servlet });

    my $Shop = $Servlet->object;
        if (! $Shop->folder('EbayShopEnvironments')->hasChildren()){
            $Servlet->params('ViewAction','UnityMBO-ViewEbayShopEnvironments');
                return $Servlet->doAction('UnityMBO-ViewEbayShopEnvironments');
        }

    return $self->SearchEbayAuctions( $Servlet );
}

sub SearchEbayAuctions {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $Folder = $Shop->folder('EbayAuctions');
    my $SearchForm = $Servlet->form->form($Shop, 'SearchEbayAuctions');
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;

    my $SearchName = $SearchForm->{'Search'} || 'MBO-EbayAuction';
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('EbayAuction'), $SearchName);
    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Folder,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'   => $Shop->id,
            'IsNotSubAuction' => '1',
        },
        'UriParameters'  => {
            'Search' => $SearchForm->{'Search'},
        },
    );

    foreach my $Parameter (keys %$SearchForm) {
        if ( $Parameter eq 'ProductAliasString' ) {

            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        } elsif ($Parameter eq 'Status') {
            if ($SearchForm->{'Status'} == AUCTION_EDITABLE) {
                $Options{'Parameters'}{'EditableOrVerified'} = 1;
            } elsif ($SearchForm->{'Status'} == AUCTION_RUNNING) {
                $Options{'Parameters'}{'Running'} = 1;
            } elsif ($SearchForm->{'Status'} == AUCTION_ENDED or $SearchForm->{'Status'} == AUCTION_COMPLETED) {
                $Options{'Parameters'}{'Ended'} = 1;
            } elsif ($SearchForm->{'Status'} == AUCTION_SCHEDULED) {
                $Options{'Parameters'}{'Scheduled'} = 1;
            }
        } elsif ($Parameter eq 'StartTimeTo') {
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);
        } elsif ($Parameter eq 'EndTimeTo') {
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);
        } else {
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
    }

    delete $SearchForm->{'Search'};

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub getReturnsWithinOptionsCodes {
    my $self = shift;
    my ($hresult) = @_;

    my $aReturnPolicyDetails =  $hresult->{'ReturnPolicyDetails'};
    return unless defined $aReturnPolicyDetails;

    my $hReturnPolicyDetails = $aReturnPolicyDetails->[0];
    return unless defined $hReturnPolicyDetails;

    my $aReturnsWithin = $hReturnPolicyDetails->{'ReturnsWithin'};
    return unless defined $aReturnsWithin;

    return $aReturnsWithin;
}

sub addReturnsWithinOptionsCodes {
    my $self = shift;
    my ($Servlet,$aReturnsWithin) = @_;

    $Servlet->vars('ReturnsWithins', {
            'ReturnsWithin' => $aReturnsWithin
        });

    return $Servlet;
}

1;

