#========================================================================================
# package      DE_EPAGES::Etracker::API::Etracker
# state        public
#----------------------------------------------------------------------------------------
# description  Etracker functions
#========================================================================================
package DE_EPAGES::Etracker::API::Etracker;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    CreateEtrackerAccount
    DeleteEtrackerAccount
);

use Encode qw (encode);
use HTTP::Request::Common;
use LWP::UserAgent;
use URI::Escape qw (uri_escape);
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::TestParameter qw (TestHash TestObject);
use DE_EPAGES::Etracker::API::Table::ShopEtrackerAction qw (UpdateShopEtrackerAction InsertShopEtrackerAction);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Etracker::API::Tools qw (EtrackerLog);
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw (GetDefaultTimeOut);

sub ErrorDefinition {
    return {
        'EtrackerHTTPFailed'    => 'Request to eTracker URL #URL failed with HTTP status #StatusLine',
        'EtrackerRequestFailed' => 'Request to eTracker URL #URL failed with Etracker error: #RetCode',
    }
}

#========================================================================================
# function     CreateEtrackerAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateEtrackerAccount
# example      CreateEtrackerAccount
#----------------------------------------------------------------------------------------
# description  Creates a new etracker account and sets the shop attributes
#               EtrackerClientID, EtrackerCode, and EtrackerSecureKey.
#               Activates all eTracker actions.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $hData | account data <ul>
#                   <li>FirstName - first name - string</li>
#                   <li>LastName - last name - string</li>
#                   <li>Password - password for etracker statistics - string</li>
#                   <li>EMail - e-mail address - string</li>
#                   <li>Street - street address - string</li>
#                   <li>Zipcode - zip code - string</li>
#                   <li>Country - 2-letter country code - string</li>
#                   <li>Company - company name (optional) - string</li>
#                   <li>Description - web site description (optional) - string</li>
#                   <li>LanguageCode - 2-letter language code (optional, default: 'en')
#                                   - string</li>
#               </ul> | ref.hash
#========================================================================================
sub CreateEtrackerAccount {
    my ($Shop, $hData) = @_;
    TestObject( 'Shop', $Shop );
    TestHash( 'hData', $hData, {
        'FirstName' => 'String;NotNull',
        'LastName'  => 'String;NotNull',
        'Password'  => 'String;NotNull',
        'EMail'     => 'String;NotNull',
        'Street'    => 'String;NotNull',
        'Zipcode'   => 'String;NotNull',
        'City'      => 'String;NotNull',
        'Country'   => 'String;NotNull',
        'Company'   => 'String',
        'Description'  => 'String',
        'LanguageCode' => 'String',
    });
    EtrackerLog('ETracker',{
        ShopAlias => $Shop->alias,
        Data => $hData
    },'CreateEtrackerAccount');

    my $System = LoadRootObject();
    my $NewClientURL = $System->get('EtrackerNewClientScriptURL');
    my $AuthorizationCode = $System->get('EtrackerNewClientPassword');
    my $ShopUrl = $Shop->get('WebUrl');
    my ($BaseUrl, $ObjectPath) = split(/\?/, $ShopUrl);
    $ObjectPath = uri_escape('?'.$ObjectPath) if defined $ObjectPath;

    my %EtrackerForm = (
        a => $AuthorizationCode,            # authentication code
        r => 8,                             # Typ etracker Service
        o => 1,                             # Traffic Kontingent
        g => 91,                            # Gruppen-ID: 91 = Allgemein
    );

    # URL der Website
    $EtrackerForm{'u'} = defined $ObjectPath ? $BaseUrl.$ObjectPath : $BaseUrl;

    my %ETRACKER_MAP = (
        'Description' => 'd',
        'Company'     => 'c',
        'FirstName'   => 's',
        'LastName'    => 'n',
        'EMail'       => 'e',
        'Street'      => 'b',
        'Zipcode'     => 'z',
        'City'        => 't',
        'Country'     => 'y',
        'Password'    => 'p',
    );
    foreach my $Key (keys %$hData) {
        next unless exists $ETRACKER_MAP{$Key};
        next unless defined $ETRACKER_MAP{$Key};
        # convert to ISO-1, convert unknown characters to '?'
        $EtrackerForm{$ETRACKER_MAP{$Key}} = encode('iso-8859-1', $hData->{$Key});
    }

    # set the eTracker language if supported by etracker, default: english
    my %ETRACKER_LANGUAGES = (
        'de' => 1,
        'en' => 2,
        'fr' => 3,
        'cs' => 4,
        'es' => 5,
    );
    $EtrackerForm{'l'} = defined $hData->{'LanguageCode'} && exists $ETRACKER_LANGUAGES{$hData->{'LanguageCode'}}
        ? $ETRACKER_LANGUAGES{$hData->{'LanguageCode'}}
        : $ETRACKER_LANGUAGES{'en'};

    my $UserAgent = LWP::UserAgent->new;
    $UserAgent->timeout(GetDefaultTimeOut);
    $UserAgent->agent('ePages/6 ' . $UserAgent->agent());
    my $Request = POST($NewClientURL, \%EtrackerForm);
    EtrackerLog('Request',$Request->as_string,'CreateEtrackerAccount');
    my $Response = $UserAgent->request( $Request );
    EtrackerLog('Response',$Response->as_string,'CreateEtrackerAccount');

    Error('EtrackerHTTPFailed', {
        'URL'        => $NewClientURL,
        'StatusLine' => $Response->status_line,
    }) unless $Response->is_success;

    my $Content = $Response->content;
    my ($RetCode, $ClientID, $SecureCode, $SecureKey, undef) = split /\,/, $Content;
    Error('EtrackerRequestFailed', {
        'URL'     => $NewClientURL,
        'RetCode' => $RetCode,
    }) unless defined $RetCode && $RetCode eq '0';

    $Shop->set({
        'EtrackerClientID'  => $ClientID,
        'EtrackerCode'      => $SecureCode,
        'EtrackerSecureKey' => $SecureKey,
        'EtrackerPassword'  => $hData->{'Password'},
    });

    _ActivateAllEtrackerActions($Shop);
    return;
}

# set all actions to active, set interest level to 1
sub _ActivateAllEtrackerActions {
    my ($Shop) = @_;

    my $ahEtrackerActions = $Shop->get('EtrackerActions');
    foreach my $hEtrackerAction (@$ahEtrackerActions) {
        if( !defined $hEtrackerAction->{'IsActive'} ) {
            InsertShopEtrackerAction( {
                'ShopID'         => $Shop->id,
                'ActionID'       => $hEtrackerAction->{'Action'}->id,
                'IsActive'       => 1,
                'EtrackerILevel' => 1,
            });
        }
        elsif( !$hEtrackerAction->{'IsActive'} ) {
            UpdateShopEtrackerAction( $Shop->id, $hEtrackerAction->{'Action'}->id, {
                'IsActive' => 1
            });
        }
    }
}


#========================================================================================
# function     DeleteEtrackerAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteEtrackerAccount
# example      DeleteEtrackerAccount($Shop);
#----------------------------------------------------------------------------------------
# description  Deletes an etracker account for the Shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#========================================================================================
sub DeleteEtrackerAccount {
   my ($Shop) = @_;
    TestObject( 'Shop', $Shop );
    EtrackerLog('Attempt to delete EtrackerAccount for Shop',{
        ShopAlias => $Shop->alias,
    },'DeleteEtrackerAccount');

    my $EtrackerClientID = $Shop->get('EtrackerClientID');
    my $EtrackerCode = $Shop->get('EtrackerCode');
    my $EtrackerSecureKey = $Shop->get('EtrackerSecureKey');

    return unless defined $EtrackerClientID;
    return unless defined $EtrackerCode;
    return unless defined $EtrackerSecureKey;


    my $System = LoadRootObject();
    my $DeleteClientURL = $System->get('EtrackerDeleteClientScriptURL');
    my $AuthorizationCode = $System->get('EtrackerNewClientPassword');

    my %EtrackerForm = (
        'a' => $AuthorizationCode,            # authentication code
        'p' => $EtrackerSecureKey,            # Sicherheitsschlssel (SECURE_KEY) der Statistik die gelscht werden soll
        'i' => $EtrackerClientID,             # ID der Statistik  der gelscht werden soll
    );

    my $UserAgent = LWP::UserAgent->new;
    $UserAgent->timeout(GetDefaultTimeOut);
    $UserAgent->agent('ePages/6 ' . $UserAgent->agent());
    my $Request = POST($DeleteClientURL, \%EtrackerForm);
    EtrackerLog('Request',$Request->as_string,'DeleteEtrackerAccount');
    my $Response = $UserAgent->request( $Request );
    EtrackerLog('Response',$Response->as_string,'DeleteEtrackerAccount');

    Error( 'EtrackerHTTPFailed', {
        'URL'        => $DeleteClientURL,
        'StatusLine' => $Response->status_line,
    }) unless $Response->is_success;

    my $Content = $Response->content;
    my ($RetCode) = split /\,/, $Content;
    Error( 'EtrackerRequestFailed', {
        'URL'     => $DeleteClientURL,
        'RetCode' => $RetCode,
    }) if not defined $RetCode;

    return if $RetCode eq '0';
    return if $RetCode eq '6'; #Account kann nicht von ET gelscht werden, weil zum Bsp. ein upgrade erfolgte

    my $Message = "Could not delete etracker Account ETClientID:'%s' ETCode:'%s' for Shop:'%s' : ReturnCode '%s'.";
    GetLog('USERFAULT::Etracker::DeleteEtrackerAccount')->warn(sprintf ($Message, $EtrackerClientID, $EtrackerCode, $Shop->id, $RetCode));

    return;

    # ReturnCodes
    # 0  Zugang wurde gelscht

    # Fehlercodes:
    # 1  Keine Authentifizierung
    # 2  Kein Sicherheitsschlssel
    # 3  Sicherheitsschlssel falsch
    # 4  Keine ID
    # 5  ID existiert nicht
    # 6  Account kann nicht von ET gelscht werden, weil zum Bsp. ein upgrade erfolgte
}

1;



