#========================================================================================
# package      DE_EPAGES::ExternalShipping::API::AddressValidation
# state        public
#----------------------------------------------------------------------------------------
# description  Address validation management object for shipping cartridges
#----------------------------------------------------------------------------------------
# example      my $AddressValidation = DE_EPAGES::ExternalShipping::API::AddressValidation->new( $AddressObject )
#               my $isComplete = $AddressValidation->isComplete();
#========================================================================================
package DE_EPAGES::ExternalShipping::API::AddressValidation;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::ExternalShipping::API::Constants qw ( DEFAULT_MIN_LENGHT DEFAULT_MAX_LENGHT );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Address, $ExtraAttributDefinitions );
#----------------------------------------------------------------------------------------
# description  constructor of AddressValidation
#----------------------------------------------------------------------------------------
# input        $Address | ePages AddressObject | object
# input        $AttributDefinitions | Attributes definitions| hash_ref
# return       $self | AddressValidation object | object
#========================================================================================
sub new {
    my $class = shift;
    my ( $Address, $AttributeRequirements) = @_;
    TestObject('AddressValidation needs valid address object', $Address, 'DE_EPAGES::Address::API::Object::Address');
    my $self  = bless {
                'Address' => $Address,
                'AttributeRequirements' => $AttributeRequirements // {},
                'WarningStates' => [],
    } , $class;
    $self->validateAttributes();

    return $self;
}


#========================================================================================
# function     isValidAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $sel->isValidAddress();
#----------------------------------------------------------------------------------------
# description  checks if all mandatory Attributes are ok
#----------------------------------------------------------------------------------------
# return       $isValid | Attributes are fine or not | boolean
#========================================================================================
sub isValidAddress {
    my $self = shift;

    if (scalar @{$self->{'WarningStates'}} == 0) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     validateAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       validateAttributes( );
#----------------------------------------------------------------------------------------
# description  load all Attributes to the class object
#========================================================================================
sub validateAttributes {
    my $self = shift;

    $self->{'WarningStates'} = [];
    foreach my $AddressAttributeName ( keys %{$self->{'AttributeRequirements'}} ) {
        my $AttributeRequirements = $self->{'AttributeRequirements'}->{$AddressAttributeName};
        foreach my $RequirementName ( keys %{$AttributeRequirements} ) {
            my $Requirement = $AttributeRequirements->{$RequirementName};

            if ($RequirementName eq 'Mandatory') {
                $self->_validateAttributeMandatoryCompliance($AddressAttributeName, $Requirement);
            }
            elsif ($RequirementName eq 'Length') {
                $self->_validateAttributeLength($AddressAttributeName, $Requirement);
            }
            elsif ($RequirementName eq 'Enumeration') {
                $self->_validateAttributeEnumerationCompliance($AddressAttributeName, $Requirement);
            }
        }
    }
    return;
}

#========================================================================================
# function     _getAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       $AddressAttribute = _getAttribute( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Gets the Attribute with the Name $AttributeName from Shippinglabel->Shopaddress
#               and write it to the Class object
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
# return       $AddressAttribute | Address Attribute value | String - Object
#========================================================================================
sub _getAttribute {
    my $self = shift;
    my ($AttributeName) = @_;

    return $self->{'Address'}->get($AttributeName);
}

#========================================================================================
# function     _validateAttributeMandatoryCompliance
# state        private
#----------------------------------------------------------------------------------------
# syntax       _validateAttributeMandatoryCompliance( $AttributeName );
#----------------------------------------------------------------------------------------
# description  wrappes the _getAttribute method for mandatory check
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
# input        $isMandatory | boolean
#========================================================================================
sub _validateAttributeMandatoryCompliance {
    my $self = shift;
    my ($AttributeName, $isMandatory) = @_;

    my $AttributeValue = $self->_getAttribute($AttributeName);
    if ( $isMandatory && (!isValid( $AttributeValue)) ) {
        $self->_setWarning($AttributeName);
    }
    return;
}

#========================================================================================
# function     _validateAttributeLength
# state        private
#----------------------------------------------------------------------------------------
# syntax       _validateAttributeLength($AttributeName, $Requirements);
#----------------------------------------------------------------------------------------
# description  validate the Attribute length. If wrong it creates warnings
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
# input        $Requirements | usually a hash with min & max lengths | hash ref
#========================================================================================
sub _validateAttributeLength {
    my $self = shift;
    my ($AttributeName, $Requirements) = @_;

    my $AttributeValue = $self->_getAttribute($AttributeName);
    if ( isValid($AttributeValue) ) {
        #assign some min & max values that will fail if they don't get modified
        my $MinValue = 1;
        my $MaxValue = 0;
        if ($AttributeName eq 'Zipcode') {
            my $AddressCountry = $self->_getAttribute('Country');
            if (isValid($AddressCountry)) {
                my $AddressCountryCode2 = $AddressCountry->{'Code2'};
                my @CountriesCode2 = keys(%{$Requirements});
                if (ElementOf($AddressCountryCode2, @CountriesCode2)) {
                    $MinValue = $Requirements->{$AddressCountryCode2}->{'MinValue'};
                    $MaxValue = $Requirements->{$AddressCountryCode2}->{'MaxValue'};
                } elsif (isValid($Requirements->{'Other'})) {
                    $MinValue = $Requirements->{'Other'}->{'MinValue'};
                    $MaxValue = $Requirements->{'Other'}->{'MaxValue'};
                } else {
                    #since no specification for the current country has been found, we assume it's correct
                    $MinValue = DEFAULT_MIN_LENGHT;
                    $MaxValue = DEFAULT_MAX_LENGHT;
                }
            }
        }
        else {
            $MinValue = $Requirements->{'MinValue'};
            $MaxValue = $Requirements->{'MaxValue'};
        }
        eval {
            TestString($AttributeName, $AttributeValue, $MinValue, $MaxValue);
        };
        if (ExistsError()) {
            my $Error = GetError();
            if ($Error->code eq 'STRING_TOO_SHORT') {
                $self->_setWarning($AttributeName, ' is too Short');
            }
            elsif ($Error->code eq 'STRING_TOO_LONG') {
                $self->_setWarning($AttributeName, ' is too Long');
            }
            else {
                Error($Error);
            }
        }
    }
    return;
}

#========================================================================================
# function     _validateAttributeEnumerationCompliance
# state        private
#----------------------------------------------------------------------------------------
# syntax       _validateAttributeEnumerationCompliance($AttributeName, $aEnumeration);
#----------------------------------------------------------------------------------------
# description  validate that the Attribute value is within the enumerated ones
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
# input        $aEnumeration | array with valid values | array ref
#========================================================================================
sub _validateAttributeEnumerationCompliance {
    my $self = shift;
    my ($AttributeName, $aEnumeration) = @_;

    my $AttributeValue = $self->_getAttribute($AttributeName);
    if ( isValid($AttributeValue) ) {
        if ($AttributeName eq 'Country') {
            my $AddressCountryCode2 = $AttributeValue->{'Code2'};
            my $AddressCountryEngName = $AttributeValue->{'English'};
            $self->_setWarning($AddressCountryEngName, ' is not an allowed country')
                unless ElementOf($AddressCountryCode2, $aEnumeration);
        }
        else {
            $self->_setWarning($AttributeValue, " is not an allowed value for $AttributeName")
                unless ElementOf($AttributeValue, $aEnumeration);
        }
    }
    return;
}

#========================================================================================
# function     _setWarning
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setWarning( $AttributeName );
#----------------------------------------------------------------------------------------
# description  writes a Attribute name to the classvar $self->{'WarningStates'} to show
#               which mandatory attribute is set
#----------------------------------------------------------------------------------------
# input        $AttributeName | Name of Attribute | String
# input        $Reason        | reason for warning | String
#========================================================================================
sub _setWarning {
    my $self = shift;
    my ($AttributeName, $Reason) = @_;

    $Reason = ' is mandatory but not set' if ( not isValid($Reason));
    push( @{$self->{'WarningStates'}}, $AttributeName.$Reason );
    return ;
}

#========================================================================================
# function     getStatesAsWarning
# state        public
#----------------------------------------------------------------------------------------
# syntax       getStatesAsWarning( $Input1 );
#----------------------------------------------------------------------------------------
# description  Get the States and create epages warnings array from this
#----------------------------------------------------------------------------------------
# return       $aWarnigs | epages warnings | array ref
#========================================================================================
sub getStatesAsWarning {
    my $self = shift;
    my $aWarnings =  BlockWarnings (
        sub {
             foreach my $WarningState ( @{$self->{'WarningStates'}} ){
                eval {Warning("$WarningState")};
                if ( ExistsError() ) {
                    my $Error = GetError();
                    LogError($Error) if ( $Error->code =~ /Native/ms );
                }
            }
        },
    );
    return $aWarnings;
}

1;
