#==============================================================================
# package      DE_EPAGES::ExternalShipping::UI::ShippingTransaction
# state        private
#==============================================================================
package DE_EPAGES::ExternalShipping::UI::ShippingTransaction;
use base DE_EPAGES::Presentation::UI::Object;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError HandleError );
use DE_EPAGES::Core::API::File qw ( DeleteFile GetFileContent WriteFile );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::ExternalShipping::API::Object::PagerShippingLabels;
use DE_EPAGES::ExternalShipping::API::Shop qw ( GetCurrentShippingOrder GetShippingOrdersFolder);
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );

#========================================================================================
# function     DeleteFromShippingOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteFromShippingOrder( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to delete a shipping label from shipping order
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub DeleteFromShippingOrder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $hValues = $Form->form($Servlet->object, 'ListedObjects');

    my $Class = LoadClassByAlias('ShippingLabel');

    my @ListObjects =
            map  { $Class->loadObject( $_->{'ListObjectID'} ) }
            grep { $Class->existsObject( $_->{'ListObjectID'} ) }
            @{$hValues->{'ListObjectIDs'}};

    # test permission
    foreach my $Index ( 0 .. scalar(@ListObjects)-1 ) {
        my $Object = $ListObjects[$Index];
        unless ($Servlet->hasPermission($Object, 'Save')) {
            $Form->addFormError( {
                Reason => 'PermissionDenied',
                Index => $Index,
                LoopName => 'ListObjectIDs',
                Form => 'ShippingOrder',
                Name => 'ShippingLabelID'
            } );
            next;
        }
        $Object->deleteFromShippingOrder();
    }
    $Form->executeFormError;

    foreach my $LabelObject (@ListObjects) {
        $LabelObject->deleteFromShippingOrder();
    }
    return;
}


#========================================================================================
# function     View
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->View( $Servlet );
#----------------------------------------------------------------------------------------
# description  ViewAction for shipping order
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $Page | Current page to view | object
#========================================================================================
sub View {
    my $self = shift;
    my ($Servlet) = @_;
    my $Transaction = $Servlet->object;
    my $Pager = DE_EPAGES::ExternalShipping::API::Object::PagerShippingLabels->new({
        'Servlet'  => $Servlet,
        'Object'   => $Transaction,
        'PageSize' => 25,
        'OrderBy'  => 'Alias'
    });
    $Servlet->vars(
        'ShippingOrder',
        GetCurrentShippingOrder( $Transaction->getSite, $Transaction->get('ShippingFulfillmentType') ),
    );
    return $Pager->page;
}


#========================================================================================
# function     ValidateShippingOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ValidateShippingOrder( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to validate shipping order
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub ValidateShippingOrder {
    my $self = shift;
    my ($Servlet) = @_;
    my $Transaction = $Servlet->object;
    my $aWarnings = BlockWarnings( sub {
        eval {
            $Transaction->validateTransaction();
        };
        Warning(GetError()) if (ExistsError());
    });

    if (scalar @{$aWarnings}) {
        # LogDebug('Validation Errors', $aWarnings);
        foreach my $Warning (@$aWarnings) {
            if ($Warning->code eq 'NativeWarn') {
                Warning($Warning);
            }
            else {
                $Servlet->form->mapFormError({
                    'Form'   => 'ExportShippingLabels',
                    'Name'   => 'ExportShippingLabels',
                }, $Warning);
            }
        }
        $Servlet->form->executeFormError;
    }
    return;
}

#========================================================================================
# function     SendShippingOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SendShippingOrder( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to send shipping order of selected Labels. Switches the
#               unselected labels to a new Transaction and preceds with the specialized
#               behaviour.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SendShippingOrder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Transaction = $Servlet->object;
    my $hValues = $Servlet->form->form( $Servlet->object, 'ListedObjects' );
    my $Shop = $Transaction->getSite();

    # create an array with selected (from the available list) shipping label object IDs
    my @aSelectedShippingLabelsIds;
    foreach my $hObjectID  (@{$hValues->{'ListObjectIDs'}}){
        push(@aSelectedShippingLabelsIds, $hObjectID->{'ListObjectID'});
    }

    # keep original transaction (alias) name because it can be processed and renamed
    my $transactionName = $Transaction->alias();

    # create additional shipping transaction object to deal with selected and unselected shipping labels.
    my $Folder = GetShippingOrdersFolder($Shop, $Transaction->get('ShippingFulfillmentType')) ;
    my $auxTransactionName = $transactionName . 'Aux';
    my $AuxTransaction;
    if ( $Folder->existsChild($auxTransactionName) ) {
        $AuxTransaction = $Folder->child($auxTransactionName);
    } else {
        # auxiliary transaction recursivity is set to zero
        $AuxTransaction = $Transaction->clone({'Alias'=> $auxTransactionName}, 0);
    }

    my $Error;
    eval
    {
        # ensure that some shipping labels have been selected (otherwise throw an error)
        Error('Not Shipping Labels selected') unless (@aSelectedShippingLabelsIds);

        #Transfer the unselected labels to the auxiliary transaction
        my $aShippingLabels = $Transaction->get('ShippingLabels');
        foreach my $Label (@{$aShippingLabels}){
            unless ( ElementOf($Label->id, @aSelectedShippingLabelsIds) ){
                $Label->set({'ShippingTransaction' => $AuxTransaction});
            }
        }

        $self->SendShippingOrderHelper($Servlet);

    };

    if ( ExistsError() ) {
        $Error = GetError();
    }

    # send unselected shipping labels back to the pending transaction (probably a new one)
    my $NewTransaction = GetCurrentShippingOrder( $Transaction->getSite, $Transaction->get('ShippingFulfillmentType') );

    foreach my $Label (@{$AuxTransaction->get('ShippingLabels')}){
        $Label->set({'ShippingTransaction' => $NewTransaction});
    }

    # send unsuccessful shipping labels back to the pending transaction
    foreach my $Label (@{$Transaction->get('ShippingLabels')}){
        $Label->set({'ShippingTransaction' => $NewTransaction}) unless ($Label->get('Issued'));
    }

    if ( defined($Error) ) {
        GetLog->warn('Warning: '. $Error->shortMessage);
        $Servlet->form->executeFormError;
    }

    return;
}

#========================================================================================
# function     SendShippingOrderHelper
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SendShippingOrderHelper( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to send shipping order
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SendShippingOrderHelper {
    my $self = shift;
    my ($Servlet) = @_;
    my $Transaction = $Servlet->object;
    $self->ValidateShippingOrder($Servlet);
    return if ($Servlet->form->existsFormError());

    my $PathFileName = $self->getExportFileName($Servlet);
    eval {
        $Transaction->prepareShippingOrder();
        $Transaction->exportTransaction($PathFileName, 1);
        $Transaction->issueShippingOrder();
        $self->setRedirectParams($Servlet);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'NativeDie') {
            Warning($Error);
        } else {
            GetLog->debug('Error: '.$Error->longMessage);
            $Servlet->form->mapFormError({
                'Form'  => 'ExportShippingLabels',
                'Name'  => 'ExportShippingLabels',
            }, $Error);
        }
        DeleteFile($PathFileName) if (isValid($PathFileName));
        $Servlet->form->executeFormError;
    }

    return;
}

sub setRedirectParams {
    my $self = shift;
    my ($Servlet) = @_;
    $Servlet->vars( 'Redirect',    1 );
    $Servlet->vars( 'CSVFileName', '' );
    $Servlet->vars( 'ViewAction',  'MBO-ViewGeneral' );
}

#==============================================================================
sub getExportFileName {
    my $self = shift;
    my ($Servlet) = @_;
    my $Transaction = $Servlet->object;
    my $Method = $Transaction->get('ShippingFulfillmentMethod');
    my $Type   = $Method->get('ShippingFulfillmentType');
    my $ExportFilename = $Type->get('ExportFilename');
    return unless isValid($ExportFilename);
    return $Method->get('PrivatePath') . '/' . $ExportFilename;
}


#==============================================================================
sub ViewExportResult {
    my $self = shift;
    my ($Servlet) = @_;
    my $Transaction = $Servlet->object;
    my $Method = $Transaction->get('ShippingFulfillmentMethod');
    my $Type   = $Method->get('ShippingFulfillmentType');

    my $PathFileName = $self->getExportFileName($Servlet);
    return unless isValid($PathFileName);

    my $rContent = GetFileContent($PathFileName, {'binary'=>1});
    my $Response = $Servlet->response;
    $Response->content($$rContent);
    $Response->header(
        'Content-Type'        => 'application/text',
        'Content-Disposition' => 'attachment; filename=' . $Type->get('ExportFilename'),
        'Content-Description' => 'shippinglabel export file' );

    $Servlet->vars('ExitAfterEvent',1);
    return;
}

sub ReceiveShippingOrderResult {
    my $self = shift;
    my ($Servlet) = @_;

    my $Transaction = $Servlet->object;
    my $Form        = $Servlet->form;
    $Servlet->form->form($Transaction, 'CSVImport');
    my $rContent = \$Form->files->{'ImportFile'}->[0];

    my $PathFileName = $self->getPrivateCSVImportFileName($Servlet);
    WriteFile( $PathFileName , $rContent, { 'binary' => 1, 'create_directory' => 1 } );

    my $aWarnings = BlockWarnings( sub {
        eval {
            $Transaction->importTracking($PathFileName, 1);
        };
        HandleError(
            OnError => sub {
                my ($Error) = @_;
                if (ElementOf($Error->code, qw(FILE_OPEN_FAILED FILE_CLOSE_FAILED
                                               FILE_NOT_TEXT READ_LINE_FAILED
                                               WRONG_FORMAT MISSING_FIELD))) {
                    Warning($Error);
                    return 'FIXED';
                }
            },
            Finally => sub {
                DeleteFile($PathFileName);
            }
        );
    });

    if (scalar @{$aWarnings}) {
        foreach my $Warning (@{$aWarnings}) {
            GetLog()->debug($Warning->longMessage);
            if ($Warning->code eq 'NativeWarn') {
                Warning($Warning);
            }
            else {
                $Servlet->form->mapFormError({
                    'Form'   => 'CSVImport',
                    'Name'   => 'ImportTracking',
                }, $Warning);
            }
        }
        $Servlet->form->executeFormError;
    }
    return;
}

sub getPrivateCSVImportFileName {
    my $self    = shift;
    my ($Servlet) = @_;
    my $User = $Servlet->user;

    return $User->get('PrivatePath') .'/CSVImport.csv';
}

1;
