#========================================================================================
# package      DE_EPAGES::FedEx::API::Object::ShippingFulfillmentMethodFedEx
# base         DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod
# state        public
#----------------------------------------------------------------------------------------
# description  Class for shipping fulfillment connection
#========================================================================================
package DE_EPAGES::FedEx::API::Object::ShippingFulfillmentMethodFedEx;
use base DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( WriteFile );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::FedEx::API::Constants qw (
    %FEDEX_LABELSTATUS
    %SHIPPING_LABEL_STATUS
    @VISIBLE_SHIPPING_ADDRESS_FIELDS
    @MANDATORY_SHIPPING_ADDRESS_FIELDS
);
use DE_EPAGES::FedEx::API::RemoteFedExBarcode;
use DE_EPAGES::FedEx::API::Tools qw ( GetLabelImagePath );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use MIME::Base64 qw ( decode_base64 );

use constant PDF_FILE_EXTENSION => 'PDF';
use constant DEFAUL_LANGUAGE_CODE => 'en';
my @FEDEX_NOTIFICATION_ALLOWED_LONG_LOCATIONS = qw(es_ES es_US fr_CA zh_CN zh_TW zh_HK);
my @FEDEX_NOTIFICATION_ALLOWED_LANGUAGES = qw(ar da de en es fi fr hu it ja ko no nl pl pt ru sv tr zh);

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $ShippingFulfillmentMethod->featureName;
#----------------------------------------------------------------------------------------
# description   Returns the feature name 'FedEx'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'FedEx'; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethodFedEx = DE_EPAGES::FedEx::API::Object::ShippingFulfillmentMethodFedEx->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShippingFulfillmentMethodFedEx');
#               my $ShippingFulfillmentMethodFedEx = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook
#               'OBJ_InsertShippingFulfillmentMethodFedEx' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $ShippingFulfillmentMethodFedEx | new object | object
# hook         OBJ_InsertShippingFulfillmentMethodFedEx | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodFedExID - shipping label id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertShippingFulfillmentMethodFedEx',
            {'Object' => $self, 'ShippingFulfillmentMethodFedExID' => $self->id });

    $self->addVisibleAddressFields(@VISIBLE_SHIPPING_ADDRESS_FIELDS);
    $self->addMandatoryAddressFields(@MANDATORY_SHIPPING_ADDRESS_FIELDS);

    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethodFedEx->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook
#               'OBJ_DeleteShippingFulfillmentMethodFedEx' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShippingFulfillmentMethodFedEx | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodFedExID - shipping label id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;
    TriggerHook('OBJ_DeleteShippingFulfillmentMethodFedEx',
            {'Object' => $self, 'ShippingFulfillmentMethodFedExID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethodFedEx->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShippingFulfillmentMethodFedEx'
#               and 'OBJ_AfterUpdateShippingFulfillmentMethodFedEx'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute values hash| ref.hash
# input        $LanguageID | language id value | integer
# hook         OBJ_BeforeUpdateShippingFulfillmentMethodFedEx | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodFedExID - shipping label id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShippingFulfillmentMethodFedEx | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodFedExID - shipping label id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;
    TriggerHook('OBJ_BeforeUpdateShippingFulfillmentMethodFedEx',
            {
                'Object' => $self,
                'Values' => $hValues,
                'ShippingFulfillmentMethodFedExID' => $self->id,
                'LanguageID' => $LanguageID
            });
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShippingFulfillmentMethodFedEx',
            {
                'Object'=>$self,
                'Values'=> $hValues,
                'ShippingFulfillmentMethodFedExID' => $self->id,
                'LanguageID' => $LanguageID
            });
    return;
}

#========================================================================================
# function     webserviceValidateLabel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod->webserviceValidateLabel( $ShippingLabel );
#----------------------------------------------------------------------------------------
# description  Calls validation webservice for one transaction.
#               Has to be implemented in child class, if
#               ShippingFulfillmentType.IntegrationType=Webservice
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | shipping label | object
#========================================================================================
sub webserviceValidateLabel {
    my $self = shift;
    my ($Transaction) = @_;
    Error('NotImplemented');
}
#========================================================================================
# function     webserviceValidateTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod->webserviceValidateTransaction( $Transaction );
#----------------------------------------------------------------------------------------
# description  Calls validation webservice for one transaction.
#               Has to be implemented in child class, if
#               ShippingFulfillmentType.IntegrationType=Webservice
#----------------------------------------------------------------------------------------
# input        $Transaction | shipping transaction | object
#========================================================================================
sub webserviceValidateTransaction {
    my $self = shift;
    my ($Transaction) = @_;
    Error('NotImplemented');
}

#========================================================================================
# function     generateShippingLabels
# state        public
#----------------------------------------------------------------------------------------
# syntax       generateShippingLabels( $ShippingTransaction, $LanguageID );
#----------------------------------------------------------------------------------------
# description  operates a stack of Shippinglabels and set the transstatus
#----------------------------------------------------------------------------------------
# input        $ShippingTransaction | Shipping transaction object | object_ref
#========================================================================================
sub generateShippingLabels{
    my $self = shift;
    my ($ShippingTransaction) = @_;

    my $Shop = $ShippingTransaction->getSite();
    my $LocaleID = $self->_deduceMerchantLocale($Shop);
    my $FedExNotificationLocaleID = $self->_applyFedExNotificationRulesOnLocale($LocaleID);

    my $aShippingLabels = $ShippingTransaction->get('ShippingLabels');
    my $countFaultyShippinglabels = 0;
    my $Remote = DE_EPAGES::FedEx::API::RemoteFedExBarcode->new();

    foreach my $ShippingLabel (@{$aShippingLabels}){
        my $Result;
        eval{
            $Result = $Remote->ProcessShipmentRequest($self, $ShippingLabel, PDF_FILE_EXTENSION, $FedExNotificationLocaleID);
        };
        if ( ExistsError() ) {
            my $Error = GetError();
            $self->_reportAndUpdateShippingLabelOnError($Error, $ShippingLabel);
            Error($Error) if ( $Error->code eq 'NativDie' );
            LogError($Error);
            $countFaultyShippinglabels++;
        } else {
            my $IsError = $self->_processFedExResult($Result, $ShippingLabel);
            $countFaultyShippinglabels++ if ($IsError);
        }
    }

    my $transStatus = $SHIPPING_LABEL_STATUS{'ALL_OK'};
    if( $countFaultyShippinglabels == scalar @{$aShippingLabels} ) {
        $transStatus = $SHIPPING_LABEL_STATUS{'ALL_KO'};
    } elsif( $countFaultyShippinglabels >= 1 ){
        $transStatus = $SHIPPING_LABEL_STATUS{'SOME_OK'};
    }

    $ShippingTransaction->set({'TransStatus' => $transStatus});

    return;
}

#========================================================================================
# function     _processFedExResult
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsErrorResult = $self->_processFedExResult($Result, $ShippingLabel);
#----------------------------------------------------------------------------------------
# description  it processes the result of FedEx shipment request
#----------------------------------------------------------------------------------------
# input        $Result | FedEx shipment request result | hash ref.
# input        $ShippingLabel | shipping label object | object
# return       $IsErrorResult | True if resulting FedEx shipment returned an error | boolean
#========================================================================================
sub _processFedExResult {
    my $self = shift;
    my ($Result, $ShippingLabel) = @_;

    my $BarcodeError = $Result->{'BarCodeError'};
    if($BarcodeError) {
         #overwrite state if also a warning occurs
         $ShippingLabel->set({ 'LabelStatus' => $FEDEX_LABELSTATUS{'ERROR_NO_BARCODE'},
                               'BarCodeError' => $Result->{'BarCodeError'},
         });
         return 1;
    } else {
        my $Image = decode_base64($Result->{'Image'});
        WriteFile(GetLabelImagePath($ShippingLabel), \$Image, {'binary' => 1, 'create_directory' => 1});
        $ShippingLabel->set({ 'LabelStatus' => $FEDEX_LABELSTATUS{'BAR_CODE_CREATED'},
                              'Issued' => 1,
                              'TrackingID' => $Result->{'TrackingNumber'}
        });
        return 0;
    }
}

#========================================================================================
# function     isUsable
# state        public
#----------------------------------------------------------------------------------------
# syntax       isUsable( );
#----------------------------------------------------------------------------------------
# description  Checks if ShippingFulfillmentMethod is usable
#----------------------------------------------------------------------------------------
# return       $IsUsable | True if ShippingFulfillmentMethod is usable | boolean
#========================================================================================
sub isUsable {
    my $self = shift;
    my $res;

    my $shop = $self->get('Site');
    if ($self->get('IsShippingLabelEnabled'))
    {
        $res = $self->get('WebserviceAccountNr') &&
            $self->get('WebserviceMeterNr') &&
            $self->get('WebserviceUsername') &&
            $self->get('WebservicePassword') &&
            $self->get('Site')->get('Address')->get('Company') &&
            $self->get('Site')->get('Address')->get('Phone') &&
            $shop->get('FedExAreMandatoryShippingFieldsSet') &&
            $self->SUPER::isUsable();
    }
    else
    {
        $res = $self->get('WebserviceAccountNr') &&
            $self->get('WebserviceMeterNr') &&
            defined($self->get('Site')->get('Address')->get('Company')) &&
            defined($self->get('Site')->get('Address')->get('Phone')) &&
            $shop->get('FedExAreMandatoryShippingFieldsSet') &&
            $self->SUPER::isUsable();
    }
    return  $res;
}

#========================================================================================
# function     _reportAndUpdateShippingLabelOnError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_reportAndUpdateShippingLabelOnError($Error, $ShippingLabel);
#----------------------------------------------------------------------------------------
# description  report error and update shipping label information accordingly
#----------------------------------------------------------------------------------------
# input        $Error | Error Object | object
# input        $ShippingLabel | a shippinglabel | object
#========================================================================================
sub _reportAndUpdateShippingLabelOnError {
    my $self = shift;
    my ($Error, $ShippingLabel) = @_;

    $ShippingLabel->set({
        'Issued' => '0',
        'LabelStatus' => $FEDEX_LABELSTATUS{'ERROR_NO_BARCODE'},
    });
    LogShippingFulfillment('FedEx', 'Error at Shipping Label dispatch phase: ',
        {
            'ShippingLabel' => $ShippingLabel->alias,
            'ShippingLabelID' => $ShippingLabel->id,
            'ErrorCode' => $Error->code,
        }
    );
    return;
}

#========================================================================================
# function     _deduceMerchantLocale
# state        private
#----------------------------------------------------------------------------------------
# syntax       $localeID = $self->_deduceMerchantLocale( $Shop );
#----------------------------------------------------------------------------------------
# description  algorithm that calculates merchant location codes (language and country)
#               for the email notification.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $localeID | locale id | string
#========================================================================================
sub _deduceMerchantLocale {
    my $self = shift;
    my ($Shop) = @_;

    my $ShopAddress = $Shop->get('Address');
    my $ShopEmail = $ShopAddress->get('EMail');
    my @MerchantUsers = ();
    my @SuperMerchantUsers = ();

    foreach my $User (@{$self->_getShopUsers($Shop)}) {

        return $User->get('LocaleID') if ($User->get('EMail') eq $ShopEmail);

        push @SuperMerchantUsers, $User if ($User->get('IsSuperMerchant'));
        push @MerchantUsers, $User if ($User->get('IsMerchant'));
    }

    if (scalar(@SuperMerchantUsers) == 1) {
        my $User = pop @SuperMerchantUsers;
        return $User->get('LocaleID');
    }

    if (scalar(@MerchantUsers) == 1) {
        my $User = pop @MerchantUsers;
        return $User->get('LocaleID');
    }

    return $Shop->get('LocaleID');
}

#========================================================================================
# function     _getShopUsers
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aUsers = $self->_getShopUsers( $Shop );
#----------------------------------------------------------------------------------------
# description  get the users of a shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $aUsers | array of shop users | array ref.
#========================================================================================
sub _getShopUsers {
    my $self = shift;
    my ($Shop) = @_;

    return $Shop->folder('Users')->children;
}

#========================================================================================
# function     _applyFedExNotificationRulesOnLocale
# state        private
#----------------------------------------------------------------------------------------
# syntax       $trimmedLocaleID = $self->_applyFedExNotificationRulesOnLocale( $localeID );
#----------------------------------------------------------------------------------------
# description  FedEx Email notification web service is very restrictive and it only allows
#               to add locale codes when required. So that method trims localeID to FedEx
#               requirements.
#----------------------------------------------------------------------------------------
# input        $localeID | locale id | string
# return       $trimmedLocaleID | FedEx tailored locale id | string
#========================================================================================
sub _applyFedExNotificationRulesOnLocale {
    my $self = shift;
    my ($localeID) = @_;

    return $localeID if (ElementOf($localeID, @FEDEX_NOTIFICATION_ALLOWED_LONG_LOCATIONS));

    my $languageCode = (split /_/, $localeID)[0];

    return $languageCode if (ElementOf($languageCode, @FEDEX_NOTIFICATION_ALLOWED_LANGUAGES));

    return DEFAUL_LANGUAGE_CODE;
}

1;
