#========================================================================================
# package      DE_EPAGES::FedEx::unit::t::ShippingFulfillmentMethodFedEx
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for ShippingFulfillmentMethodFedEx class
#========================================================================================
package DE_EPAGES::FedEx::unit::t::ShippingFulfillmentMethodFedEx;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::FedEx::API::Object::ShippingFulfillmentMethodFedEx;
use DE_EPAGES::FedEx::API::Constants qw ( %SHIPPING_LABEL_STATUS );
use DE_EPAGES::Core::API::Error qw ( Error );

use Test::More;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;

use constant DEFAULT_LOCALE_ID => 'de_DE';

use strict;

sub test {
    my $self = shift;

    $self->test_shippingLabelsSuccessGeneration();
    $self->test_shippingLabelsFailureGeneration();
    $self->test_localeDeductionFromShopEmail();
    $self->test_localeDeductionFromUserIsMerchant();
    $self->test_localeDeductionFromUserIsSuperMerchant();
    $self->test_localeDeductionFromNoUserMatching();

    done_testing();
}

## no critic (ProhibitPrivateSubroutineUse)

#----------------------------------------------------------------------------------------
sub test_shippingLabelsSuccessGeneration {
    my $self = shift;

    my $SubTestName = (caller(0))[3];

    local * DE_EPAGES::FedEx::API::RemoteFedExBarcode::ProcessShipmentRequest = \&_processShipmentRequestFakeMethodSuccess; ## no critic (Subroutines::ProtectPrivateSubs)

    my $ShippingFulfillmentMethodFedEx = $self->_generateShippingFulfillmentMethodFedExMock();
    my $ShippingTransactionFedEx = $self->_generateShippingTransactionFedExMock();

    $ShippingFulfillmentMethodFedEx->generateShippingLabels($ShippingTransactionFedEx);

    is(
            $ShippingTransactionFedEx->get('TransStatus'),
            $SHIPPING_LABEL_STATUS{'ALL_OK'},
            "$SubTestName -  all shipping labels successfully generated"
    );
}

#----------------------------------------------------------------------------------------
sub test_shippingLabelsFailureGeneration {
    my $self = shift;

    my $SubTestName = (caller(0))[3];

    local * DE_EPAGES::FedEx::API::RemoteFedExBarcode::ProcessShipmentRequest = \&_processShipmentRequestFakeMethodFailure; ## no critic (Subroutines::ProtectPrivateSubs)

    my $ShippingFulfillmentMethodFedEx = $self->_generateShippingFulfillmentMethodFedExMock();
    my $ShippingTransactionFedEx = $self->_generateShippingTransactionFedExMock();

    $ShippingFulfillmentMethodFedEx->generateShippingLabels($ShippingTransactionFedEx);

    is(
            $ShippingTransactionFedEx->get('TransStatus'),
            $SHIPPING_LABEL_STATUS{'ALL_KO'},
            "$SubTestName - all shipping labels failed to be generated"
    );
}

#----------------------------------------------------------------------------------------
sub test_localeDeductionFromShopEmail {
    my $self = shift;

    my $SubTestName = (caller(0))[3];

    my $ShippingFulfillmentMethodFedEx = $self->_generateShippingFulfillmentMethodFedExMock();
    my $Shop = $self->_generateShopMock('email_match');
    my $localeID = $ShippingFulfillmentMethodFedEx->_deduceMerchantLocale($Shop);
    is(
            $localeID,
            'en_GB',
            "$SubTestName - locale matched from shop email address"
    );

    is(
            $ShippingFulfillmentMethodFedEx->_applyFedExNotificationRulesOnLocale($localeID),
            'en',
            "$SubTestName - only get language code when it is english localeID (en_GB)"
    );
}

#----------------------------------------------------------------------------------------
sub test_localeDeductionFromUserIsMerchant {
    my $self = shift;

    my $SubTestName = (caller(0))[3];

    my $ShippingFulfillmentMethodFedEx = $self->_generateShippingFulfillmentMethodFedExMock();
    my $Shop = $self->_generateShopMock('merchant_match');
    my $localeID = $ShippingFulfillmentMethodFedEx->_deduceMerchantLocale($Shop);
    is(
            $localeID,
            'fr_CA',
            "$SubTestName - locale matched from merchant user flag"
    );

    is(
            $ShippingFulfillmentMethodFedEx->_applyFedExNotificationRulesOnLocale($localeID),
            'fr_CA',
            "$SubTestName - get language and locale codes when it is french from Canada (fr_CA)"
    );
}

#----------------------------------------------------------------------------------------
sub test_localeDeductionFromUserIsSuperMerchant {
    my $self = shift;

    my $SubTestName = (caller(0))[3];

    my $ShippingFulfillmentMethodFedEx = $self->_generateShippingFulfillmentMethodFedExMock();
    my $Shop = $self->_generateShopMock('supermerchant_match');
    my $localeID = $ShippingFulfillmentMethodFedEx->_deduceMerchantLocale($Shop);
    is(
            $localeID,
            'es_ES',
            "$SubTestName - locale matched from super-merchant user flag"
    );

    is(
            $ShippingFulfillmentMethodFedEx->_applyFedExNotificationRulesOnLocale($localeID),
            'es_ES',
            "$SubTestName - get language and locale codes when it is spanish from Spain (es_ES)"
    );
}

#----------------------------------------------------------------------------------------
sub test_localeDeductionFromNoUserMatching {
    my $self = shift;

    my $SubTestName = (caller(0))[3];

    my $ShippingFulfillmentMethodFedEx = $self->_generateShippingFulfillmentMethodFedExMock();
    my $Shop = $self->_generateShopMock('none_match');
    my $localeID = $ShippingFulfillmentMethodFedEx->_deduceMerchantLocale($Shop);

    is(
            $localeID,
            DEFAULT_LOCALE_ID,
            "$SubTestName - locale matched as default locale when not shop email nor user (super)merchant matches"
    );

    is(
            $ShippingFulfillmentMethodFedEx->_applyFedExNotificationRulesOnLocale($localeID),
            'de',
            "$SubTestName - only get language code when it is english localeID (" . DEFAULT_LOCALE_ID . ')'
    );
}

#-----------------------------------------------------------------------------------------
# test doubles generation
#-----------------------------------------------------------------------------------------

sub _generateShippingFulfillmentMethodFedExMock {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::FedEx::API::Object::ShippingFulfillmentMethodFedEx',
            []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_processFedExResult',
            undef,
            ['hashref', {'object' => 'DE_EPAGES::FedEx::API::Object::ShippingLabelFedEx'}]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_reportAndUpdateShippingLabelOnError',
            undef,
            [
                    {'object' => 'DE_EPAGES::Core::API::Error'},
                    {'object' => 'DE_EPAGES::FedEx::API::Object::ShippingLabelFedEx'}
            ]
    );
    $Mockify->addMock(
            '_getShopUsers',
            sub {
                shift;
                my ($Shop) = @_;
                my $UsersContent = $Shop->get('UsersContent');
                if ($UsersContent eq 'email_match') {
                    return [$self->_generateUserMock('j.merino@epages.com', 0, 0, 'en_GB')];
                }
                elsif ($UsersContent eq 'merchant_match') {
                    return [
                            $self->_generateUserMock('no_matching@mail1.com', 0, 0, 'en_GB'),
                            $self->_generateUserMock('no_matching@mail2.com', 1, 0, 'fr_CA')
                    ];
                }
                elsif ($UsersContent eq 'supermerchant_match') {
                    return [
                            $self->_generateUserMock('no_matching@mail1.com', 1, 0, 'fr_CA'),
                            $self->_generateUserMock('no_matching@mail2.com', 1, 1, 'es_ES')
                    ];
                }
                elsif ($UsersContent eq 'none_match') {
                    return [$self->_generateUserMock('no_matching@mail1.com', 0, 0, 'es_ES')];
                }
                else {
                    return [];
                }
            }
    );

    return $Mockify->getMockObject();
}

sub _processShipmentRequestFakeMethodSuccess {
    my $self = shift;

    return {};
}

sub _processShipmentRequestFakeMethodFailure {
    my $self = shift;

    Error('SOAP_REQUEST_FAILED', {'Code' => 'SomeErrorCode', 'Message' => 'Some Error Message'});

    return {};
}

sub _generateShippingTransactionFedExMock {
    my $self = shift;

    my $Attributes = {
        'ShippingLabels' => [
                                $self->_generateShippingLabelFedExMock(),
                                $self->_generateShippingLabelFedExMock(),
                            ],
        'Site' => $self->_generateShopMock('email_match'),
        'TransStatus' => undef,
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::FedEx::API::Object::ShippingTransactionFedEx',
            $Attributes
    );
}

sub _generateShippingLabelFedExMock {
    my $self = shift;

    my $Attributes = {
        'Issued' => 0,
        'LabelStatus' => undef,
        'BarCodeError' => undef,
        'TrackingID' => undef,
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::FedEx::API::Object::ShippingLabelFedEx',
            $Attributes
    );
}

sub _generateUserMock {
    my $self = shift;
    my ($EMail, $isMerchant, $isSuperMerchant, $localeID) = @_;

    my $Attributes = {
        'EMail' => $EMail,
        'IsMerchant' => $isMerchant,
        'IsSuperMerchant' => $isSuperMerchant,
        'LocaleID' => $localeID,
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Permission::API::Object::User',
            $Attributes
    );
}

sub _generateAddressMock {
    my $self = shift;

    my $Attributes = {
        'EMail' => 'j.merino@epages.com',
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            $Attributes
    );
}

sub _generateShopMock {
    my $self = shift;
    my ($userContentBehavior) = @_;

    my $Attributes = {
        'Address' => $self->_generateAddressMock(),
        'LocaleID' => 'de_DE',
        'UsersContent' => $userContentBehavior, # this is not a real attribute but part of hack to return shop users
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop',
            $Attributes
    );
}

__PACKAGE__->runScript();

1;
