#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient
# state        public
# description  Wrapper for GoogleApiClient requests with token refresh if expired
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient;

use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::EPagesJ::API::Object::RestClient;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping GetErrorCode );
use HTTP::Request;
use HTTP::Status qw ( is_success is_error );
use LWP::UserAgent;
use Encode qw ( encode );
use JSON::MaybeXS qw ( decode_json );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    EPAGESJ_BASE_URI
    ERROR_UNAUTHENTICATED
    ERROR_UNAUTHORIZED
);
use Readonly;

use strict;

use constant POST_METHOD => 'POST';
use constant PUT_METHOD => 'PUT';
use constant GET_METHOD => 'GET';
use constant DELETE_METHOD => 'DELETE';

Readonly my $TIMEOUT_IN_SECONDS => 180;

sub new {
    my $class = shift;
    my ($hSettings)  = @_;

    my $self  = bless {
        Shop => $hSettings->{'Shop'},
    }, $class;

    return $self;
}

sub doGet {
    my $self = shift;
    my ($Url, $Headers, $hQueryParams) = @_;

    return $self->_doAction(GET_METHOD, $Url, $Headers, $hQueryParams);
}

sub doPost {
    my $self = shift;
    my ($Url, $Headers, $RequestContent) = @_;

    return $self->_doAction(POST_METHOD, $Url, $Headers, $RequestContent);
}

sub doPut {
    my $self = shift;
    my ($Url, $Headers, $RequestContent) = @_;

    return $self->_doAction(PUT_METHOD, $Url, $Headers, $RequestContent);
}

sub doDelete {
    my $self = shift;
    my ($Url, $Headers,) = @_;

    return $self->_doAction(DELETE_METHOD, $Url, $Headers);
}

sub _doAction {
    my $self = shift;
    my ($ActionMethod, $Url, $Headers, $RequestPayload) = @_;

    $self->loadGoogleSettings();
    my $DecodedResponse;

    eval {
        $DecodedResponse =  $self->_doRequest($ActionMethod, $Url, $Headers, $RequestPayload);
    };
    if (ExistsError()) {
        my $Error = $self->_GetError();
        if($self->_isTokenExpired($Error)) {
            LogGoogleSmartShopping( 'Authentication Error', {'Error' => $Error->message()});
            $self->_doRefreshAccessToken();
            return $self->_doRequest($ActionMethod, $Url, $Headers, $RequestPayload);
        }
        Error($Error);
    }

    return $DecodedResponse;
}

sub _doRequest {
    my $self = shift;
    my ($HttpMethod, $Url, $Headers, $hData)  = @_;

    if ($self->_isRequestToEpagesj($Url)) {
        return $self->_doEpagesjRequest($HttpMethod, $Url, $Headers, $hData);
    }
    return $self->_doExternalRequest($HttpMethod, $Url, $Headers, $hData);
}

sub _doExternalRequest {
    my $self = shift;
    my ($HttpMethod, $Url, $Headers, $hData)  = @_;

    my $Request = HTTP::Request->new($HttpMethod, $Url, $Headers, $hData);
    my $AccessToken = $self->{'GoogleSmartShoppingConfig'}->get('AccessToken');
    $Request->header(Authorization => "Bearer $AccessToken");
    my $UserAgent = LWP::UserAgent->new();

    my $Response = $UserAgent->request($Request);
    my $Status = $Response->code();
    if (is_error($Status)) {
        Error('External API Error', {
            'Status' => $Status,
            'Response' => $Response->decoded_content(),
            'Url' => $Url,
        });
    }
    return decode_json($Response->decoded_content());
}

sub _doEpagesjRequest {
    my $self = shift;
    my ($HttpMethod, $EpjEndpoint, $hRequestHeader, $hData)  = @_;

    $hRequestHeader->{'X-AccessToken'} = $self->{'GoogleSmartShoppingConfig'}->get('AccessToken');

    if (defined $hRequestHeader->{'AdminToken'}) {
        $hRequestHeader->{'X-AccessToken'} = $hRequestHeader->{'AdminToken'};
        # We should not delete the AdminToken afterwards
        # In case an unauthorized response comes, the AccessToken could be used after it is refreshed
    }

    my $RestClient = $self->_createEpagesjRestClient($self->{'Shop'});

    my $Url = $RestClient->{'BaseURL'}.$EpjEndpoint;
    my $RequestUrl = URI::URL->new($Url);

    my $Response;
    if ($HttpMethod eq POST_METHOD) {
        my $JsonRequestPayload = $RestClient->encodeRequest($hData);
        $Response = $RestClient->client()->post($RequestUrl, $hRequestHeader, $JsonRequestPayload);
    } elsif($HttpMethod eq PUT_METHOD) {
        my $JsonRequestPayload = $RestClient->encodeRequest($hData);
        $Response = $RestClient->client()->put($RequestUrl, $hRequestHeader, $JsonRequestPayload);
    } elsif($HttpMethod eq DELETE_METHOD) {
        $RequestUrl->query_form($hData);
        $Response = $RestClient->client()->delete($RequestUrl, $hRequestHeader);
    } elsif($HttpMethod eq GET_METHOD) {
        $RequestUrl->query_form($hData);
        $Response = $RestClient->client()->get($RequestUrl, $hRequestHeader);
    }

    return $self->_decodeFromResponse($RestClient, $Response);
}

sub _isRequestToEpagesj {
    my $self = shift;
    my ($Url)  = @_;

    if (index($Url, EPAGESJ_BASE_URI) != -1) {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     _doRefreshAccessToken
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->doRefreshAccessToken();
#----------------------------------------------------------------------------------------
# description  It refreshes accessToken using a refreshToken.
#               https://developers.google.com/identity/protocols/oauth2/web-server#offline
#----------------------------------------------------------------------------------------
# return       $hResponse | hash with response | hashref
#========================================================================================
sub _doRefreshAccessToken {
    my $self = shift;

    my $AccessToken = $self->GetAccessToken($self->{'GoogleSmartShoppingConfig'}->get('RefreshToken'));

    $self->{'GoogleSmartShoppingConfig'}->set({ 'AccessToken' => $AccessToken });

    return;
}

#========================================================================================
# function     GetAccessToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetAccessToken( $RefreshToken );
#----------------------------------------------------------------------------------------
# description  It gets an accessToken using a refreshToken.
#----------------------------------------------------------------------------------------
# input        $RefreshToken | RefreshToken used to get an AccessToken | string
# return       $AccessToken | returns the AccessToken | string
#========================================================================================
sub GetAccessToken {
    my $self = shift;
    my ($RefreshToken) = @_;

    my $Url = $self->{oauth_token_url};
    my $Header = [ 'Content-Type' => 'application/x-www-form-urlencoded' ];
    my $RequestContent = sprintf(
        'client_id=%s&client_secret=%s&refresh_token=%s&grant_type=%s',
        $self->{'client_id'},
        $self->{'client_secret'},
        $RefreshToken,
        'refresh_token'
    );

    my $Request = HTTP::Request->new('POST', $Url, $Header, encode("UTF-8", $RequestContent));
    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->request($Request);
    my $Status = $Response->code();
    my $ErrorMessagePrefix = $self->{'GoogleSmartShopping_AdminRefreshToken'} eq $RefreshToken ? 'Admin' : '';

    if (is_success($Status)) {
        my $hResponse = decode_json($Response->decoded_content());
        if (defined $hResponse && isValid($hResponse->{'access_token'})) {
            LogGoogleSmartShopping("GetAccessToken - $ErrorMessagePrefix AccessToken refreshed", {});
            return $hResponse->{'access_token'};
        }
    }

    LogGoogleSmartShopping("GetAccessToken - $ErrorMessagePrefix AccessToken NOT refreshed", {
        'Status' => $Status,
        'Response' => $Response->decoded_content(),
    });

    Error("$ErrorMessagePrefix RefreshToken not refreshed", {
        'Status' => $Status,
        'Response' => $Response->decoded_content(),
    });
}

# -----------------------------------------------------------
sub _createEpagesjRestClient {
    my $self = shift;
    my ($Shop) = @_;

    my $System = LoadRootObject();
    my $hArgs = {};

    $hArgs->{BaseURL} = $System->get('OAuthServiceBaseUrl') // $System->get('EPagesJBaseURL');
    $hArgs->{ShopType} = $Shop->get('ShopTypeRef')->alias();
    $hArgs->{Shop} = $Shop;
    $hArgs->{BusinessUnit} = $System->get('ProviderID');
    $hArgs->{HTTPTimeOut}  //= EpagesConfig->get('DE_EPAGES::GoogleSmartShopping', 'HTTPTimeOutGooglePMax', $TIMEOUT_IN_SECONDS);

    return DE_EPAGES::EPagesJ::API::Object::RestClient->create( $System, $hArgs );
}

# -----------------------------------------------------------
sub _isTokenExpired {
    my $self = shift;
    my ($Error) = @_;


    my $ErrorCode = GetErrorCode($Error);

    if ($ErrorCode eq ERROR_UNAUTHENTICATED) {
        return 1;
    }
    if ($ErrorCode eq ERROR_UNAUTHORIZED) {
        return 1;
    }
    return 0;
}

# -----------------------------------------------------------
sub _GetError {
    my $self = shift;

    return GetError();
}
# -----------------------------------------------------------
sub _decodeFromResponse {
    my $self = shift;
    my ($Client, $Response) = @_;

    my $ResponseContent = $Response->decoded_content();
    # Empty content responses fail to decode (i.e. "No Content - 204")
    return $ResponseContent unless (isValid($ResponseContent));

    return $Client->decodeResponse( $ResponseContent );
}

sub loadGoogleSettings {
    my $self = shift;

    if (isValid($self->{'client_id'})) {
        return;
    }

    my $hSystemSettings = LoadRootObject()->get([qw(
        GoogleSmartShopping_AppClientID
        GoogleSmartShopping_AppClientSecret
        GoogleSmartShopping_AuthTokenURL
        GoogleSmartShopping_AppRedirectURL
        GoogleSmartShopping_AdminRefreshToken
        GoogleSmartShopping_AdsDeveloperToken
        GoogleSmartShopping_MerchantCenterMultiClientAccountID
        GoogleSmartShopping_AdsManagerAccountId
        GoogleSmartShopping_GoogleAppURL
    )]);

    $self->{'client_id'} = $hSystemSettings->{'GoogleSmartShopping_AppClientID'};
    $self->{'client_secret'} = $hSystemSettings->{'GoogleSmartShopping_AppClientSecret'};
    $self->{'oauth_token_url'} = $hSystemSettings->{'GoogleSmartShopping_AuthTokenURL'};
    $self->{'oauth_redirect_url'} = $hSystemSettings->{'GoogleSmartShopping_AppRedirectURL'};
    $self->{'GoogleSmartShopping_AdminRefreshToken'} = $hSystemSettings->{'GoogleSmartShopping_AdminRefreshToken'};
    $self->{'GoogleSmartShopping_AdsDeveloperToken'} = $hSystemSettings->{'GoogleSmartShopping_AdsDeveloperToken'};
    $self->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = $hSystemSettings->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'};
    $self->{'GoogleSmartShopping_AdsManagerAccountId'} = $hSystemSettings->{'GoogleSmartShopping_AdsManagerAccountId'};
    $self->{'GoogleSmartShopping_GoogleAppURL'} = $hSystemSettings->{'GoogleSmartShopping_GoogleAppURL'};

    $self->{'GoogleSmartShoppingConfig'} = $self->{'Shop'}->existsChild('GoogleSmartShoppingConfig') ?
        $self->{'Shop'}->get('GoogleSmartShoppingConfig') : undef;

    return;
}

1;