#========================================================================================
# package      DE_EPAGES::Installer::API::CartridgesXMLFilePatcher
# state        public
# description  patch Cartridges.xml file use a delete and a import file.
#========================================================================================
package DE_EPAGES::Installer::API::CartridgesXMLFilePatcher;
use strict;

use base qw (Exporter);

our @EXPORT_OK = qw (
    ReadCartridgesXmlFile
    ReadCartridgesXmlStruct
    SortCartridgesXmlStruct
    GetCartridgeDependencies
    GetCartridgePositions
    SetCartridgePositions
    CartridgeDependencyError
);

use Clone qw(clone);
use Config::IniFiles;
use Data::Dumper qw(Dumper);

use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Object::API::XMLStructureExportDriver;
use DE_EPAGES::XML::API::XML::DependencyImportHandler qw (GetDependencies);
use DE_EPAGES::Core::API::File qw (ExistsFile);
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

# sort order of standard <DatabaseType/> elements in Cartridges.xml
use constant DATABASE_TYPE_POSITION_OFFSET => 1000;
use constant DATABASE_TYPE_POSITION => {
    ASPoolDB => 0,
    System => 1000,
    Common => 2000,
    Site => 3000,
    MinStore => 4000,
    Store => 5000,
    MultiStoreSite => 6000,
    MultiStoreCommon => 7000,
    Master => 8000,
    MultiStore => 9000,
    FlexStore => 10000,
    FlexSite => 11000,
    Monitor => 12000,
};
use constant LAST_POSITION => 100000;

# for each standard <DatabaseType/>, show used <DatabaseType/> (sorted)
use constant DATABASE_TYPE_BASES => {
    ASPoolDB => [qw(ASPoolDB)],
    System => [qw(System)],
    Common => [qw(Common)],
    Site => [qw(Site Common)],
    MinStore => [qw(MinStore Common)],
    Store => [qw(Store MinStore Common)],
    MultiStoreSite => [qw(MultiStoreSite Site Common)],
    MultiStoreCommon => [qw(MultiStoreCommon Store MinStore Common)],
    Master => [qw(Master MultiStoreCommon Store MinStore Common)],
    MultiStore => [qw(MultiStore MultiStoreCommon Store MinStore Common)],
    FlexStore => [qw(FlexStore Store MinStore Common)],
    FlexSite => [qw(FlexSite Site Common)],
    Monitor => [qw(Monitor Common)],
};

#ERRORDEFINITION
sub ErrorDefinition () { return {
    'DEPENDENCY_MISSING'  => 'Cartridges.xml: #Requires missing (required by #Cartridge)',
    'CIRCULAR_DEPENDENCY'  => 'Cartridges in Cartridges.xml have circular dependency: #CircularDependency',
    'DEPENDENCIES_NOT_RESOLVABLE' => 'dependencies can\'t be resolved: #Dependencies',
                         }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $CartridgesXMLFilePatcher = DE_EPAGES::Installer::API::CartridgesXMLFilePatcher->new($CartridgesXMLFile);
#----------------------------------------------------------------------------------------
# example      my $CartridgesXMLFilePatcher = DE_EPAGES::Installer::API::CartridgesXMLFilePatcher->new('Cartridges.xml');
#----------------------------------------------------------------------------------------
# description  creates a new CartridgesXMLFilePatcher object
#----------------------------------------------------------------------------------------
# input        $CartridgesXMLFile | path to Cartridges.xml | string
# return       $CartridgesXMLFilePatcher | CartridgesXMLFilePatcher object | object
#========================================================================================
sub new{
    my $class = shift;
    my ($CartridgesXMLFile) = @_;
    my $self = bless {}, $class;

    $self->{'CartridgesXMLFile'} = $CartridgesXMLFile;

    return $self;
}

#========================================================================================
# function     importElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       importElements($ImportFile, $hFlags);
#----------------------------------------------------------------------------------------
# example      $CartridgesXMLFilePatcher->importElements('imp_Cartridges.xml');
# example      $CartridgesXMLFilePatcher->importElements(undef, {check_dependencies => 1});
#----------------------------------------------------------------------------------------
# description  Adds all elements from import file if not exists or update elements.
#               If no import file is given it just reorders Cartridges.xml.
#----------------------------------------------------------------------------------------
# input        $ImportFile | import cartridges xml file name | string
# input        $hFlags | flags to modify the behaviour of importElements.
#                         The following flag can be used:
#                       <ul>
#                           <li>check_dependencies - dies if dependencies are missing
#                               in Cartridges.xml </li>
#                       </ul>
#                       | hash ref
#========================================================================================
sub importElements{
    my $self = shift;
    my ($ImportFile, $hFlags) = @_;

    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();

    #---------------------------------------------------------------------------
    # read xml structures
    #---------------------------------------------------------------------------

    # read Cartridges.xml
    my $CartridgesXMLStruct = $ImportHandler->getXMLStructure(
        $self->{'CartridgesXMLFile'}, {'XML-Type' => 'epages_cartridges_xml'});

    my ($ImportStruct, $ImportHash);

    #---------------------------------------------------------------------------
    # loop over import file imp_*.xml
    #---------------------------------------------------------------------------
    if (defined $ImportFile) {
        $ImportStruct = $ImportHandler->getXMLStructure(
            $ImportFile, {'XML-Type' => 'epages_cartridges_xml'});
        $ImportHash = ReadCartridgesXmlStruct($ImportStruct);

        foreach my $Element (@{$ImportStruct->getElements()}){

            #------------------------------------------------------------------
            # find <DatabaseType/> elements to update
            #------------------------------------------------------------------
            my $DatabaseType = _getDatabaseType($Element, $ImportStruct);
            next unless $DatabaseType;

            my ($aDatabaseTypes2Update, $CartridgesXMLHash) = _findDatabaseTypesToUpdate(
                $DatabaseType, $CartridgesXMLStruct, $ImportHash);
            next unless scalar @$aDatabaseTypes2Update;

            # if in <DatabaseType Base=$DatabaseTypeBase Name=$DatabaseType>
            # $DatabaseTypeBase is updated then skip update of $DatabaseType
            $aDatabaseTypes2Update = _shrinkDatabaseTypesToUpdate($aDatabaseTypes2Update,
                                                                  $CartridgesXMLStruct);

            my $ElementID = $ImportStruct->getID($Element);
            my $hXMLStructAttributes = $ImportStruct->getXMLStructAttributesHash($Element);
            my $hTagAttributes = $ImportStruct->getTagAttributesHash($Element);

            #------------------------------------------------------------------
            # update <DatabaseType/> element if found
            #------------------------------------------------------------------
            if ($ImportStruct->getClass($Element) eq 'DatabaseType') {
                my $ElementName = $ImportStruct->getIDValue($Element,'Name');
                $CartridgesXMLStruct->addElement(
                    $Element, $ElementID, $hXMLStructAttributes, $hTagAttributes,
                    ) if defined $CartridgesXMLHash->{$ElementName};
            }

            next unless $ImportStruct->getClass($Element) eq 'Cartridge';

            #------------------------------------------------------------------
            # update <Cartridge/> element in each found <DatabaseType/>
            #------------------------------------------------------------------
            my $Package = $ImportStruct->getIDValue($Element,'Package');

            foreach my $DatabaseType2Update (@$aDatabaseTypes2Update){
                my $Cartridge2Update = "/DatabaseType{Name->$DatabaseType2Update}".
                    "/Cartridge{Package->$Package}";
                $CartridgesXMLStruct->addElement(
                    $Cartridge2Update, $ElementID, $hXMLStructAttributes, $hTagAttributes);
            }
        }
    }

    #--------------------------------------------------------------------------
    # sort elements
    #--------------------------------------------------------------------------

    SortCartridgesXmlStruct($CartridgesXMLStruct, $hFlags);

    #---------------------------------------------------------------------------
    # rewrite Cartridges.xml
    #---------------------------------------------------------------------------
    my $ExportDriver = DE_EPAGES::Object::API::XMLStructureExportDriver->new();
    $ExportDriver->exportXML($self->{'CartridgesXMLFile'}, $CartridgesXMLStruct);
}

#========================================================================================
# function     deleteElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       deleteElements($DeleteFile);
#----------------------------------------------------------------------------------------
# example      $CartridgesXMLFilePatcher->deleteElements('del_Cartridges.xml');
#----------------------------------------------------------------------------------------
# description  delete all elements from delete file that have the attribute delete="1"
#----------------------------------------------------------------------------------------
# input        $DeleteFile | delete cartridges xml file name | string
#========================================================================================
sub deleteElements{
    my $self = shift;
    my ($DeleteFile) = @_;

    #use xml structure import handler
    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();

    #---------------------------------------------------------------------------
    # read xml structures
    #---------------------------------------------------------------------------

    # read Cartridges.xml
    my $CartridgesXMLStruct = $ImportHandler->getXMLStructure(
        $self->{'CartridgesXMLFile'}, {'XML-Type' => 'epages_cartridges_xml'});

    # read del_*.xml
    my $DeleteStruct = $ImportHandler->getXMLStructure(
        $DeleteFile, {'XML-Type' => 'epages_cartridges_xml'});
    my $DeleteHash = ReadCartridgesXmlStruct($DeleteStruct);

    #---------------------------------------------------------------------------
    # loop over delete file del_*.xml
    #---------------------------------------------------------------------------
    foreach my $Element (@{$DeleteStruct->getElements()}){

        #----------------------------------------------------------------------
        # find <DatabaseType/> elements to delete
        #----------------------------------------------------------------------
        my $DatabaseType = _getDatabaseType($Element, $DeleteStruct);
        next unless $DatabaseType;

        my ($aDatabaseTypes2Update) = _findDatabaseTypesToUpdate(
            $DatabaseType, $CartridgesXMLStruct, $DeleteHash);
        next unless scalar @$aDatabaseTypes2Update;

        #----------------------------------------------------------------------
        # delete <DatabaseType/> element if found
        #----------------------------------------------------------------------
        if ($DeleteStruct->getClass($Element) eq 'DatabaseType' and
            $DeleteStruct->getTagAttribute($Element, 'delete') eq '1') {
            #delete if exists
            $CartridgesXMLStruct->deleteElement($Element)
                if $CartridgesXMLStruct->existsElement($Element);
        }

        next unless $DeleteStruct->getClass($Element) eq 'Cartridge';
        next unless $DeleteStruct->getTagAttribute($Element, 'delete') eq '1';

        #----------------------------------------------------------------------
        # delete <Cartridge/> element in each found <DatabaseType/>
        #----------------------------------------------------------------------
        my $Package = $DeleteStruct->getIDValue($Element,'Package');

        foreach my $DatabaseType2Update (@$aDatabaseTypes2Update){
            my $Cartridge2Delete = "/DatabaseType{Name->$DatabaseType2Update}".
                "/Cartridge{Package->$Package}";
            #delete if exists
            $CartridgesXMLStruct->deleteElement($Cartridge2Delete)
                if $CartridgesXMLStruct->existsElement($Cartridge2Delete);
        }
    }

    #---------------------------------------------------------------------------
    # rewrite Cartridges.xml
    #---------------------------------------------------------------------------
    my $ExportDriver = DE_EPAGES::Object::API::XMLStructureExportDriver->new();
    $ExportDriver->exportXML($self->{'CartridgesXMLFile'}, $CartridgesXMLStruct);
}

#========================================================================================
# function     ReadCartridgesXmlFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hCartridgesXml = ReadCartridgesXmlFile($CartridgesXmlFile);
#----------------------------------------------------------------------------------------
# description  writes info from Cartridges.xml into $hCartridgesXml
#----------------------------------------------------------------------------------------
# input        $CartridgesXmlFile | path to Cartridges.xml | string
# return       $hCartridgesXml | hash representing Cartridges.xml | ref.hash
#========================================================================================
sub ReadCartridgesXmlFile {
    my ($CartridgesXmlFile) = @_;

    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();

    my $XMLStruct = $ImportHandler->getXMLStructure(
        $CartridgesXmlFile, {'XML-Type' => 'epages_cartridges_xml'});

    my $hCartridgesXml;
    #---------------------------------------------------------------------------
    # loop over <DatabaseType/>
    #---------------------------------------------------------------------------
    foreach my $RootElement (@{$XMLStruct->getRootElements()}){
        my $DatabaseType = $XMLStruct->getIDValue($RootElement,'Name');
        #-----------------------------------------------------------------------
        # loop over <Cartridge/> with parent <DatabaseType/>
        #-----------------------------------------------------------------------
        foreach my $ChildElement (@{$XMLStruct->getChildElements($RootElement)}){
            my $Cartridge = $XMLStruct->getIDValue($ChildElement,'Package');
            my $Position = $XMLStruct->getXMLStructAttribute($ChildElement, 'Position');
            $hCartridgesXml->{$DatabaseType}->{$Cartridge} = $Position;
        }
    }
    return $hCartridgesXml;
}

#========================================================================================
# function     ReadCartridgesXmlStruct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hCartridgesXml = ReadCartridgesXmlStruct($CartridgesXMLStruct);
#----------------------------------------------------------------------------------------
# description  writes info from $CartridgesXMLStruct into $hCartridgesXml
#----------------------------------------------------------------------------------------
# input        $CartridgesXMLStruct | structure representing Cartridges.xml | object
# return       $hCartridgesXml | hash representing Cartridges.xml | ref.hash
#========================================================================================
sub ReadCartridgesXmlStruct{
    my ($XMLStruct) = @_;

    my $hCartridgesXml;
    #---------------------------------------------------------------------------
    # loop over <DatabaseType/>
    #---------------------------------------------------------------------------
    foreach my $RootElement (@{$XMLStruct->getRootElements()}){
        my $DatabaseType = $XMLStruct->getIDValue($RootElement,'Name');
        #-----------------------------------------------------------------------
        # loop over <Cartridge/> with parent <DatabaseType/>
        #-----------------------------------------------------------------------
        foreach my $ChildElement (@{$XMLStruct->getChildElements($RootElement)}){
            my $Cartridge = $XMLStruct->getIDValue($ChildElement,'Package');
            my $Position = $XMLStruct->getXMLStructAttribute($ChildElement, 'Position');
            $hCartridgesXml->{$DatabaseType}->{$Cartridge} = $Position;
        }
    }

    return $hCartridgesXml;
}

#========================================================================================
# function     SortCartridgesXmlStruct
# state        public
#----------------------------------------------------------------------------------------
# syntax       SortCartridgesXmlStruct($CartridgesXMLStruct, $hFlags);
#----------------------------------------------------------------------------------------
# example
#   my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
#   my $CartridgesXml = "$ENV{EPAGES_CARTRIDGES}/Cartridges.xml";
#   my $CartridgesXMLStruct = $ImportHandler->getXMLStructure($CartridgesXml,
#                             {'XML-Type' => 'epages_cartridges_xml'});
#   SortCartridgesXmlStruct($CartridgesXMLStruct);
#   my $ExportDriver = DE_EPAGES::Object::API::XMLStructureExportDriver->new();
#   $ExportDriver->exportXML($CartridgesXml, $CartridgesXMLStruct);
#----------------------------------------------------------------------------------------
# description  sorts <DatabaseType/> and <Cartridge/> elements in $CartridgesXMLStruct
#----------------------------------------------------------------------------------------
# input        $CartridgesXMLStruct | structure representing Cartridges.xml | object
# input        $hFlags | flags to modify the behaviour of SortCartridgesXmlStruct,
#                         The following flag can be used:
#                       <ul>
#                           <li>check_dependencies - dies if dependencies are missing
#                               in Cartridges.xml </li>
#                       </ul>
#                       | hash ref
#========================================================================================
sub SortCartridgesXmlStruct{
    my ($CartridgesXMLStruct, $hFlags) = @_;

    my $hCartridgesXml = ReadCartridgesXmlStruct($CartridgesXMLStruct);
    my ($DependentCartridges, $IndependentCartridges)
        = GetCartridgeDependencies($hCartridgesXml);
    my $hCartridgePosition
        = GetCartridgePositions($DependentCartridges, $IndependentCartridges, $hFlags);
    my ($hSortedCartridgesXml, $hDatabaseTypePosition)
        = SetCartridgePositions($hCartridgesXml, $hCartridgePosition);

    foreach my $Element (@{$CartridgesXMLStruct->getElements()}){
        if ($CartridgesXMLStruct->getClass($Element) eq 'DatabaseType') {
            #-------------------------------------------------------------------
            # set position of <DatabaseType/> element
            #-------------------------------------------------------------------
            my $DatabaseType = $CartridgesXMLStruct->getIDValue($Element,'Name');
            my $Position = $hDatabaseTypePosition->{$DatabaseType};
            $CartridgesXMLStruct->setXMLStructAttribute($Element, 'Position', $Position);
        } elsif ($Element =~ m,^/DatabaseType\{Name->(.+?)\}/Cartridge\{Package->(.+?)\},) {
            #-------------------------------------------------------------------
            # set position of <Cartridge/> element
            #-------------------------------------------------------------------
            my ($DatabaseType, $Cartridge) = ($1, $2);
            my $Position = $hSortedCartridgesXml->{$DatabaseType}{$Cartridge};
            $CartridgesXMLStruct->setXMLStructAttribute($Element, 'Position', $Position);
        }
    }
}

#========================================================================================
# function     GetCartridgeDependencies
# state        public
#----------------------------------------------------------------------------------------
# syntax       my ($DependentCartridges, $IndependentCartridges) = GetCartridgeDependencies($hCartridgesXml);
#----------------------------------------------------------------------------------------
# description  finds dependencies for cartridges in $hCartridgesXml according to
#               $ENV{EPAGES_CARTRIDGES}/<Cartridge>/Database/XML/Dependencies.xml
#----------------------------------------------------------------------------------------
# input        $hCartridgesXml | hash representing Cartridges.xml | ref.hash
# return       $DependentCartridges | hash w/ key cartridge and value hash of dependent cartridges | ref.hash
# return       $IndependentCartridges | arrray of cartridge w/o dependencies | ref.arrray
#========================================================================================
sub GetCartridgeDependencies {
    my ($hCartridgesXml) = @_;
    my (%DependentCartridges, %IndependentCartridges);

    foreach my $DatabaseType (keys %$hCartridgesXml){
        foreach my $Cartridge (keys %{$hCartridgesXml->{$DatabaseType}}){
            # go on if dependencies were found already
            next if exists $DependentCartridges{$Cartridge};
            next if exists $IndependentCartridges{$Cartridge};
            # get path to Dependencies.xml
            my $CartridgePath = "$ENV{EPAGES_CARTRIDGES}/$Cartridge";
            $CartridgePath =~ s,::,/,g;
            my $DependenciesXMLFile = "$CartridgePath/Database/XML/Dependencies.xml";
            # cartridge is independent if no Dependencies.xml exists
            if (not ExistsFile($DependenciesXMLFile)) {
                $IndependentCartridges{$Cartridge} = 1;
                next;
            }
            # otherwise add dependencies
            foreach my $DependsOnCartridge (keys %{GetDependencies($DependenciesXMLFile)}){
                $DependentCartridges{$Cartridge}{$DependsOnCartridge} = 1;
            }
        }
    }

    return (\%DependentCartridges, [keys %IndependentCartridges]);
}

#========================================================================================
# function     GetCartridgePositions
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hCartridgePosition = GetCartridgePositions($DependentCartridges, $IndependentCartridges, $hFlags);
#----------------------------------------------------------------------------------------
# description  finds cartidge positions for sorting
#----------------------------------------------------------------------------------------
# input        $DependentCartridges | hash w/ key cartridge and value hash of dependent cartridges | ref.hash
# input        $IndependentCartridges | arrray of cartridge w/o dependencies | ref.arrray
# input        $hFlags | flags to modify the behaviour of GetCartridgePositions.
#                         The following flag can be used:
#                       <ul>
#                           <li>check_dependencies - dies if dependencies are missing
#                               in Cartridges.xml </li>
#                       </ul>
#                       | hash ref
# return       $hCartridgePosition | hash w/ key cartridge and value position | ref.hash
#========================================================================================
sub GetCartridgePositions {
    my ($hCartridges, $IndependentCartridges, $hFlags) = @_;

    my $DependentCartridges = clone($hCartridges);

    # delete dependencies w/o key in $DependentCartridges
    # if check_dependencies is unused
    if (not $hFlags->{check_dependencies}) {
        foreach my $Cartridge (keys %$DependentCartridges) {
            foreach my $Requires (keys %{$DependentCartridges->{$Cartridge}}) {
                delete $DependentCartridges->{$Cartridge}{$Requires} if
                    not defined $DependentCartridges->{$Requires};
            }
        }
    }

    my $hCartridgePosition = _getCartridgePosition($DependentCartridges, $IndependentCartridges);

    while (scalar keys %$DependentCartridges) {
        # dependencies can't be resolved
        CartridgeDependencyError($DependentCartridges) unless scalar @$IndependentCartridges;
        my @NewIndependentCartridges;
        foreach my $Cartridge (sort { $hCartridgePosition->{$a} <=> $hCartridgePosition->{$b} }
                               keys %$hCartridgePosition) {
            next unless defined $DependentCartridges->{$Cartridge};
            # shorten dependencies by added independent cartridges
            delete $DependentCartridges->{$Cartridge}{$_} foreach @$IndependentCartridges;
            next if scalar keys %{$DependentCartridges->{$Cartridge}};
            # new independent cartridges found
            push @NewIndependentCartridges, $Cartridge;
            delete $DependentCartridges->{$Cartridge};
        }
        $IndependentCartridges = [@NewIndependentCartridges];
    }
    return $hCartridgePosition;
}

sub _getCartridgePosition {
    my ($hCartridges, $hIndependent) = @_;

    my $DependentCartridges = clone($hCartridges);
    my $IndependentCartridges = [@$hIndependent];

    # add cartridges not in Cartridges.xml but in Cartridges.d/*.conf
    my $CartridgesD = $ENV{EPAGES_CONFIG_CARTRIDGES} // "$ENV{EPAGES_CONFIG}/Cartridges.d";
    foreach my $ConfigFile (glob "$CartridgesD/*.conf") {
        tie my %CartridgesConf, 'Config::IniFiles', ( -file => $ConfigFile );
        foreach my $Cartridge (keys %{$CartridgesConf{Cartridges}}) {
            next if exists $DependentCartridges->{$Cartridge};
            next if ElementOf($Cartridge, @$IndependentCartridges);
            # get path to Dependencies.xml
            my $CartridgePath = "$ENV{EPAGES_CARTRIDGES}/$Cartridge";
            $CartridgePath =~ s,::,/,g;
            my $DependenciesXMLFile = "$CartridgePath/Database/XML/Dependencies.xml";
            # cartridge is independent if no Dependencies.xml exists
            if (not ExistsFile($DependenciesXMLFile)) {
                push @$IndependentCartridges, $Cartridge;
                next;
            }
            # otherwise add dependencies
            foreach my $DependsOnCartridge (keys %{GetDependencies($DependenciesXMLFile)}){
                $DependentCartridges->{$Cartridge}{$DependsOnCartridge} = 1;
            }
        }
    }

    my $Position = 1;
    my %hCartridgePosition;

    while (scalar keys %$DependentCartridges) {
        # add independent cartridges to %hCartridgePosition
        $hCartridgePosition{$_} = $Position++ foreach sort @$IndependentCartridges;
        my @NewIndependentCartridges;
        foreach my $Cartridge (sort keys %$DependentCartridges) {
            # shorten dependencies by added independent cartridges
            delete $DependentCartridges->{$Cartridge}{$_} foreach @$IndependentCartridges;
            next if scalar keys %{$DependentCartridges->{$Cartridge}};
            # new independent cartridges found
            push @NewIndependentCartridges, $Cartridge;
            delete $DependentCartridges->{$Cartridge};
        }
        $IndependentCartridges = [@NewIndependentCartridges];
    }
    # finally add not yet considered independent cartridges
    $hCartridgePosition{$_} = $Position++ foreach sort @$IndependentCartridges;

    return \%hCartridgePosition;
}

#========================================================================================
# function     SetCartridgePositions
# state        public
#----------------------------------------------------------------------------------------
# syntax       my ($hReorderedCartridgesXml, $hDatabaseTypePosition) = SetCartridgePositions($hCartridgesXml, $hCartridgePosition);
#----------------------------------------------------------------------------------------
# description  returns reordered cartridges in $hCartridgesXml according to
#               $hCartridgePosition and hash with database type positions
#----------------------------------------------------------------------------------------
# input        $hCartridgesXml | hash representing Cartridges.xml | ref.hash
# input        $hCartridgePosition | hash w/ key cartridge and value position | ref.hash
# return       $hReorderedCartridgesXml | sorted hash representing Cartridges.xml | ref.hash
# return       $hDatabaseTypePosition | hash w/ key db type and value position | ref.hash
#========================================================================================
sub SetCartridgePositions {
    my ($hCartridgesXml, $hCartridgePosition) = @_;

    my %hSortedCartridgesXml;
    my %hDatabaseTypePosition;
    my $Position = LAST_POSITION;

    foreach my $DatabaseType (sort keys %$hCartridgesXml) {
        # append custom <DatabaseType/> elements to the end of Cartridges.xml
        $hDatabaseTypePosition{$DatabaseType} =
            DATABASE_TYPE_POSITION->{$DatabaseType} // $Position;
        $Position += DATABASE_TYPE_POSITION_OFFSET;
        foreach my $Cartridge (sort keys %{$hCartridgesXml->{$DatabaseType}}) {
            # foreach cartidge set position to cartridge position
            # with offset database type position
            $hSortedCartridgesXml{$DatabaseType}{$Cartridge} =
                $hCartridgePosition->{$Cartridge} + $hDatabaseTypePosition{$DatabaseType};
        }
    }
    return (\%hSortedCartridgesXml, \%hDatabaseTypePosition);
}

#========================================================================================
# function     CartridgeDependencyError
# state        public
#----------------------------------------------------------------------------------------
# syntax       CartridgeDependencyError($DependentCartridges);
#----------------------------------------------------------------------------------------
# description  dies because Cartridges.xml/Dependencies.xml is corrupt
#----------------------------------------------------------------------------------------
# input        $DependentCartridges | hash w/ key cartridge and value hash of dependent cartridges | ref.hash
#========================================================================================
sub CartridgeDependencyError {
    my ($DependentCartridges) = @_;

    # dependencies can't be resolved
    foreach my $Cartridge (sort keys %$DependentCartridges) {
        foreach my $Dependency (sort keys %{$DependentCartridges->{$Cartridge}}) {
            next if defined $DependentCartridges->{$Dependency};
            Error('DEPENDENCY_MISSING', {'Cartridge'=>$Cartridge, 'Requires'=>$Dependency});
        }
    }
    my $CircularDependency = _findCircularDependency($DependentCartridges);
    Error('CIRCULAR_DEPENDENCY', {'CircularDependency'=> $CircularDependency})
        if $CircularDependency;
    Error('DEPENDENCIES_NOT_RESOLVABLE', {'Dependencies'=>Dumper($DependentCartridges)});

}

#========================================================================================
# function     _findCircularDependency
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $CircularDependency = _findCircularDependency($DependentCartridges);
#----------------------------------------------------------------------------------------
# description  returns circular dependency if exist
#----------------------------------------------------------------------------------------
# input        $DependentCartridges | hash w/ key cartridge and value hash of dependent cartridges | ref.hash
# return       $CircularDependency | string like 'a->b->c->a' showing circular dependency | string
#========================================================================================
sub _findCircularDependency {
    my ($DependentCartridges) = @_;

    my $hCartridges;
    foreach my $Key (keys %{$DependentCartridges}) {
        $hCartridges->{$Key}{$_} = [] foreach keys %{$DependentCartridges->{$Key}};
    }

    # circular dependencies won't be deeper than number of cartridges to examine
    my $MaxLoops = scalar keys %$hCartridges;
    my $Loop = 0;
    while ($Loop++ < $MaxLoops) {
        foreach my $Key (sort keys %$hCartridges) {
            foreach my $Value (sort keys %{$hCartridges->{$Key}}) {
                # return if cartridge depends on itself
                return "$Key->$Key" if $Key eq $Value;

                foreach my $Dependency (($Value, @{$hCartridges->{$Key}{$Value}})) {
                    # examine dependencies of dependent cartridges (if a->b and b->c then check a->c)
                    foreach my $NewDependency (sort keys %{$hCartridges->{$Dependency}}) {
                        # return if circular dependency found
                        if ($Key eq $NewDependency) {
                            my $DepPath = defined $hCartridges->{$Value}{$Key} ? '' : '...->';
                            return "$Key->$Value->$DepPath$Key";
                        }
                        # otherwise add c to dependencies of a (if a->b and b->c)
                        push @{$hCartridges->{$Key}{$Value}}, $NewDependency
                            unless ElementOf($NewDependency, $hCartridges->{$Key}{$Value});
                    }
                }
            }
        }
    }
}

#========================================================================================
# function     _getRelatedDatabaseType
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $RelatedDatabaseType = _getRelatedDatabaseType($DatabaseType);
#----------------------------------------------------------------------------------------
# example      _getRelatedDatabaseType('StratoFR') == 'Store'
#----------------------------------------------------------------------------------------
# description  finds $RelatedDatabaseType for $DatabaseType:
#               1. if it is a standard database type return this
#               2. if $DatabaseType starts with Monitor return Monitor
#               3. if $DatabaseType starts with MultiStore return MultiStore
#               4. otherwise return 'Store'
#----------------------------------------------------------------------------------------
# input        $DatabaseType | database type | string
# return       $RelatedDatabaseType | releated database type | string
#========================================================================================
sub _getRelatedDatabaseType {
    my ($DatabaseType) = @_;

    return $DatabaseType if exists DATABASE_TYPE_BASES()->{$DatabaseType};

    return 'Monitor' if $DatabaseType =~ /^Monitor/;
    return 'MultiStore' if $DatabaseType =~ /^MultiStore/;
    return 'Store';
}

#========================================================================================
# function     _getDatabaseType
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $DatabaseType = _getDatabaseType($Element, $CartridgesXMLStruct);
#----------------------------------------------------------------------------------------
# example      _getDatabaseType('/DatabaseType{Name->MultiStoreCommon}') == 'MultiStoreCommon'
#----------------------------------------------------------------------------------------
# description  returns database type of $Element in $CartridgesXMLStruct:
#               1. if $Element is <DatabaseType/>, return Name=
#               2. if $Element is <Cartridge/>, return Name= of parentElement
#               3. otherwise return nothing
#----------------------------------------------------------------------------------------
# input        $Element | $CartridgesXMLStruct element | string
# input        $CartridgesXMLStruct | structure representing Cartridges.xml | object
# return       $DatabaseType | found database type | string
#========================================================================================
sub _getDatabaseType {
    my ($Element, $CartridgesXMLStruct) = @_;

    # either element is <DatabaseType/>
    return $CartridgesXMLStruct->getIDValue($Element,'Name')
        if $CartridgesXMLStruct->getClass($Element) eq 'DatabaseType';

    # return nothing if element is not <Cartridge/>
    return undef
        if $CartridgesXMLStruct->getClass($Element) ne 'Cartridge';

    # or element is <Cartridge/> with parent <DatabaseType/>
    my $ParentElement = $CartridgesXMLStruct->parentElement($Element);
    return $CartridgesXMLStruct->getIDValue($ParentElement,'Name');
}

#========================================================================================
# function     _findDatabaseTypesToUpdate
# state        private
#----------------------------------------------------------------------------------------
# syntax       my ($aDatabaseTypes2Update, $CartridgesXMLHash) = _findDatabaseTypesToUpdate($DatabaseType, $CartridgesXMLStruct, $ImportHash);
#----------------------------------------------------------------------------------------
# description  for a $DatabaseType from imp_*.xml/del_*.xml, find database types
#               in Cartridges.xml/imp_*.xml/del_*.xml which are affected by $DatabaseType.
#               1. if custom $DatabaseType -> [$DatabaseType]
#               2. if $DatabaseType in Cartridges.xml + not Store/MultiStore/Monitor -> [$DatabaseType]
#               3. if base($DatabaseType) in Cartridges.xml + not Store/MultiStore/Monitor -> [$DatabaseType]
#               4. existing database type in Cartridges.xml: find bases (Base=) for related database type
#                  and add $DatabaseType to $aDatabaseTypes2Update if one base is $DatabaseType
#               5. new database type in imp_*.xml/del_*.xml: find bases (Base=) for related database type
#                  and add $DatabaseType to $aDatabaseTypes2Update if one base is $DatabaseType
#
#               for all $DatabaseType but Store/MultiStore/Monitor, changes in imp_*.xml/del_*.xml
#               are only applied to the same $DatabaseType in Cartridges.xml
#
#               for $DatabaseType Store/MultiStore/Monitor, changes in imp_*.xml/del_*.xml
#               are applied to all related database types in Cartridges.xml
#----------------------------------------------------------------------------------------
# input        $DatabaseType | database type from imp_*.xml/del_*.xml | string
# input        $CartridgesXMLStruct | structure representing Cartridges.xml | object
# input        $ImportHash | hash representing imp_*.xml/del_*.xml | ref.hash
# return       $aDatabaseTypes2Update | list of database type to update | ref.array.string
# return       $CartridgesXMLHash | hash representation of $CartridgesXMLStruct | ref.hash
#========================================================================================
sub _findDatabaseTypesToUpdate {
    my ($DatabaseType, $CartridgesXMLStruct, $ImportHash) = @_;

    my $CartridgesXMLHash = ReadCartridgesXmlStruct($CartridgesXMLStruct);

    # 1. custom $DatabaseType
    return ([$DatabaseType], $CartridgesXMLHash)
        unless defined DATABASE_TYPE_BASES()->{$DatabaseType};

    if (not ElementOf($DatabaseType, qw(Store MultiStore Monitor))) {
        # 2. standard database type defined in Cartridges.xml
        return ([$DatabaseType], $CartridgesXMLHash) if $CartridgesXMLHash->{$DatabaseType};
        my $DatabaseTypeBase = _getDatabaseTypeBase($DatabaseType, $CartridgesXMLStruct);
        # 3. base database type defined in Cartridges.xml
        if (not $DatabaseTypeBase or $CartridgesXMLHash->{$DatabaseTypeBase}) {
            $CartridgesXMLHash->{$DatabaseType} //= {};
            return ([$DatabaseType], $CartridgesXMLHash);
        }
    }

    my %ElementsToUpdate;
    foreach my $CheckDatabaseType (keys %$CartridgesXMLHash) {
        my $RelatedDatabaseType = _getRelatedDatabaseType($CheckDatabaseType);

        # 4.
        # if $DatabaseType is one of exceptions: Store MultiStore Monitor
        # then update related element
        # example: <DatabaseType Name=Store> updates StoreFR as well
        # if $DatabaseType is unknown in Cartridges.xml but Base= exists
        # then update Base element
        # example: <DatabaseType Name=MinStore> updates Common if MinStore missing

        foreach my $DatabaseTypeBase (@{DATABASE_TYPE_BASES->{$RelatedDatabaseType}}) {
            $ElementsToUpdate{$CheckDatabaseType} = 1 if $DatabaseType eq $DatabaseTypeBase;
        }
    }
    # return if <DatabaseType/> exists in Cartridges.xml
    return ([keys %ElementsToUpdate], $CartridgesXMLHash) if scalar keys %ElementsToUpdate;

    # 5.
    # new <DatabaseType/> found in imp_*.xml/del_*.xml
    foreach my $CheckDatabaseType (keys %$ImportHash) {
        my $RelatedDatabaseType = _getRelatedDatabaseType($CheckDatabaseType);
        foreach my $DatabaseTypeBase (@{DATABASE_TYPE_BASES->{$RelatedDatabaseType}}) {
            next unless $DatabaseType eq $DatabaseTypeBase;
            $ElementsToUpdate{$CheckDatabaseType} = 1;
            $CartridgesXMLHash->{$CheckDatabaseType} //= {};
        }
    }
    return ([keys %ElementsToUpdate], $CartridgesXMLHash);
}

#========================================================================================
# function     _getDatabaseTypeBase
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $DatabaseTypeBase = _getDatabaseTypeBase($DatabaseType, $CartridgesXMLStruct);
#----------------------------------------------------------------------------------------
# description  if <DatabaseType Base=$DatabaseTypeBase Name=$DatabaseType>
#               then returns $DatabaseTypeBase for $DatabaseType
#----------------------------------------------------------------------------------------
# input        $DatabaseType | database type | string
# input        $CartridgesXMLStruct | structure representing Cartridges.xml | object
# return       $DatabaseTypeBase | base database type | string
#========================================================================================
sub _getDatabaseTypeBase {
    my ($DatabaseType, $CartridgesXMLStruct) = @_;
    my $Element = "/DatabaseType{Name->$DatabaseType}";

    return unless $CartridgesXMLStruct->existsElement($Element);
    return $CartridgesXMLStruct->getTagAttribute($Element, 'Base');
}

#========================================================================================
# function     _shrinkDatabaseTypesToUpdate
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $aShrinkedDatabaseTypes = _shrinkDatabaseTypesToUpdate($aDatabaseTypes, $CartridgesXMLStruct)
#----------------------------------------------------------------------------------------
# description  shrink the list $aDatabaseTypes to $aShrinkedDatabaseTypes by deleting elements
#               whose Base=$DatabaseType is included in $aDatabaseTypes as well
#----------------------------------------------------------------------------------------
# input        $aDatabaseTypes | list of database types | ref.array.string
# input        $CartridgesXMLStruct | structure representing Cartridges.xml | object
# return       $aShrinkedDatabaseTypes | shrinked list of database types | ref.array.string
#========================================================================================
sub _shrinkDatabaseTypesToUpdate {
    my ($aDatabaseTypes, $CartridgesXMLStruct) = @_;

    my %hDatabaseTypes;
    map { $hDatabaseTypes{$_} = 1; } @$aDatabaseTypes;

    foreach my $DatabaseType (@$aDatabaseTypes) {
        my $DatabaseTypeBase = _getDatabaseTypeBase($DatabaseType, $CartridgesXMLStruct);
        # do not shrink if Base= missing
        next unless $DatabaseTypeBase;
        # shrink if $DatabaseTypeBase belongs to $aDatabaseTypes;
        delete $hDatabaseTypes{$DatabaseType} if
            ElementOf($DatabaseTypeBase, $aDatabaseTypes);
    }
    return [keys %hDatabaseTypes];
}

1;
