#========================================================================================
# package      DE_EPAGES::Installer::API::JiraHandler
# state        public
#----------------------------------------------------------------------------------------
# description  API for interaction with JiraHandler
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package DE_EPAGES::Installer::API::JiraHandler;
use strict;
use warnings;
use diagnostics;
use REST::Client;
use JSON;
use MIME::Base64;
use LWP::Simple;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $JiraHandler = DE_EPAGES::Installer::API::JiraHandler->new( $Username, $Password, $Host );
#----------------------------------------------------------------------------------------
# description  initializes rest client and header as class variable
#----------------------------------------------------------------------------------------
# input        $Username | jira username for login | string
# input        $Password | jira password for login | string
# input        $Host | jira url (optional)
#                       standard value = https://epages.atlassian.net | string
# input        $hOptions | flags for method | ref.hash
# return       $self | object of jira handler class | ref.object
#========================================================================================
sub new {
  my $class = shift;
  my ( $Username, $Password, $Host, $hOptions ) = @_;
  $hOptions = {} unless defined $hOptions;
  my $self = bless $hOptions, $class;

  #set rest client as class variable
  $self->{'Client'} = REST::Client->new();

  #set host to defaul value unless a host was passed
  $Host = 'https://epages.atlassian.net' unless $Host;

  #set host to rest client
  $self->{'Client'}->setHost($Host);

  #set header
  $self->{'Header'} = { Accept => 'application/json', Authorization => 'Basic ' . encode_base64( $Username . ':' . $Password ) };

  #execute rest get requeset
  $self->{'Client'}->GET( '/rest/api/2/mypermissions', $self->{'Header'} );

  #transform respornse content
  eval { from_json( $self->{'Client'}->responseContent() ); };

  #get errors
  my $Error = $@;

  #test for errors
  die 'Could not load user permissions. Login possibly wrong.' if $Error;

  #return if no error occured
  return $self;
}

#========================================================================================
# function     getParticipants
# state        public
#----------------------------------------------------------------------------------------
# syntax       $JiraHandler->getParticipants( $JiraCase );
#----------------------------------------------------------------------------------------
# description  returns participants for passed issue
#----------------------------------------------------------------------------------------
# input        $JiraCase | EPG number, can be "EPG-123", "EPG 123" or "123" | string
# return       $aResutl | array of participants | ref array
#========================================================================================
sub getParticipants {
  my $self = shift;
  my ($JiraCase) = @_;

  #save client in local variable
  my $Client = $self->{'Client'};

  #save header in local varialbe
  my $Header = $self->{'Header'};

  #test if passed number is valid
  die 'Passed EPG-Number is not valid.' unless ($self->_validJiraCase($JiraCase));

  #execute rest get requeset
  $Client->GET( '/rest/api/2/issue/' . $JiraCase, $Header );

  #get response
  my $hResponse = from_json( $Client->responseContent() );

  #extract participants and return result array
  my $aResult = $hResponse->{'fields'}->{'customfield_10080'};

  #chop off :1 from all strings
  foreach my $Participant (@$aResult) {
    $Participant =~ s/:1//;
  }

  #return result array
  return $aResult;
}

sub _validJiraCase {
  my $self = shift;
  my ($JiraCase) = @_;

  return $JiraCase =~ /^(EPB|EPG)( |-|_)\d+$/;

}

1;
