#========================================================================================
# package      DE_EPAGES::Installer::API::LanguagePack
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides language pack handling.
#----------------------------------------------------------------------------------------
# example      my $Installer = DE_EPAGES::Installer::API::LanguagePack->new(
#                   LanguageCode => 'es',
#               );
#               $Installer->install;
#========================================================================================
package DE_EPAGES::Installer::API::LanguagePack;
use base qw (Exporter);

use strict;
our @EXPORT_OK = qw(
    GetLangPackFile
    GetLangPackURL
    GetUsedLanguages
    DictionaryXmlLocation
    LanguagePackLocation
    DownloadLanguagePack
    UntarLanguagePack
    GetLanguagesByVersion
    GetLanguageVersionList
);

use File::Find;
use File::Basename qw(dirname);
use LWP::UserAgent;
use HTTP::Request;
use HTTP::Status qw(status_message);
use Time::HiRes qw(time);

use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetAllInfoCartridges );
use DE_EPAGES::Core::API::CartridgeConfig qw (CartridgeConfig);
use DE_EPAGES::Core::API::Error qw( Error ExistsError );
use DE_EPAGES::Core::API::File qw(
  CopyDirectory DeleteDirectory DeleteFile ExistsFile WriteFile
  GetFileNames GetSubDirectories SetAccessRights
);
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::Net qw(GetFile);
use DE_EPAGES::Core::API::PerlTools qw( ElementOf unique UsePackage );
use DE_EPAGES::Core::API::SystemCommand qw (UnpackTar);
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Database::API::Config qw( StoreType );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw(LoadRootObject);
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Installer::API::Constants qw (
    INTERN_LANGUAGE_PACK_SERVERS
    PUBLIC_LANGUAGE_PACK_SERVERS
);

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'GET_REQUEST_FAILED' => 'GET #URL failed: #Message (#Code).',
        'VOID_VERSION_LIST' => 'Version list \'#VersionList\' contains void version: #Version',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Installer::API::LanguagePack->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new language pack installer.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>LanguageCode - language code, e.g. 'es' - string</li>
#               </ul>
#               | hash
# return       $Installer | language pack installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;
    TestString('LanguageCode', $options{'LanguageCode'});

    return bless { %options }, $class;
}

#========================================================================================
# function     install
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs language pack database strings and files.
#========================================================================================
sub install {
    my $self = shift;

    my $LanguageCode = $self->languageCode;
    my $DialectCode = $self->dialectCode;
    my $StoreName = $self->storeName;
    my $StoreType = $self->storeType;
    my $Version   = $self->storeTypeVersion;

    $self->installStoreType;

    my $LanguageIdentifier = $LanguageCode;
    if (defined $DialectCode) {
        $LanguageIdentifier = $LanguageCode . '-' . $DialectCode;
    }

    # loop all installed cartridges
    my $hCartridges = CartridgeConfig($StoreName)->section('Cartridges');
    my @Cartridges = unique keys %$hCartridges, map { $_->{'Package'}} @{GetAllInfoCartridges()};
    foreach my $Package (@Cartridges) {
        GetLog()->debug( "Installing language $LanguageIdentifier of cartridge $Package in store $StoreName" );

        my $CartridgePackage = $Package . '::API::Cartridge';
        UsePackage($CartridgePackage);
        my $Cartridge = $CartridgePackage->new('StoreName' => $StoreName, 'StoreType' => $StoreType, 'StoreTypeVersion' => $Version);
        my $CartridgeDir = $ENV{'EPAGES_CARTRIDGES'}.'/'.$Cartridge->cartridgeDirectory;

        my $SourceDir = $CartridgeDir.'/Data/Private';
        if( -d $SourceDir ) {
            $Cartridge->onPatchCopyPrivateFile($_) foreach
                map { s/^\Q$CartridgeDir\E\///; $_; }
                grep { /\.\Q$LanguageIdentifier\E\.\w+$/ }
                @{ GetFileNames( $SourceDir, {'recursive' => 1} ) };
        }

        $SourceDir = $CartridgeDir.'/Data/WebRoot';
        if( -d $SourceDir ) {
            my $BackupDir = $ENV{EPAGES_BACKUP_DIR} ? "$ENV{EPAGES_BACKUP_DIR}/Shared/WebRoot" : undef;
            CopyDirectory( $SourceDir, $ENV{EPAGES_WEBROOT}, $BackupDir, { bakstat => 'preserve' } );
        }

        $SourceDir = $CartridgeDir.'/Database/XML';
        if( -d $SourceDir ) {
            $Cartridge->onPatchImportObjects($_) foreach
                map  { s/^\Q$CartridgeDir\E\///; $_; }
                grep { /\bTranslation\.(\w+\.)*\Q$LanguageIdentifier\E\.xml$/x }
                @{ GetFileNames( $SourceDir, {'recursive' => 1} ) };
        }
    }

    $self->deleteCtmplFiles;

    return;
}

#========================================================================================
# function     installStoreType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->installStoreType;
#----------------------------------------------------------------------------------------
# description  Installs language pack files for /Data/Public and /Data/javascript.
#========================================================================================
sub installStoreType {
    my $self = shift;

    my $LanguageCode = $self->languageCode;
    my $DialectCode  = $self->dialectCode;
    my $StoreName    = $self->storeName;
    my $StoreType    = $self->storeType;
    my $Version      = $self->storeTypeVersion;
    my $ConfigFile   = $self->_getStoreTypeConfigFile;

    my $LanguageIdentifier = $LanguageCode;
    if (defined $DialectCode) {
        $LanguageIdentifier = $LanguageCode . '-' . $DialectCode;

        # Delete LanguageCode, if language is a dialect of a language
        tie my %StoreTypeConfig, 'Config::IniFiles', ( -file => $ConfigFile ) or Error('FILE_OPEN_FAILED',{'Mode'=>'<', 'FileName'=>$ConfigFile, 'Code'=>$!});
        delete $StoreTypeConfig{'Languages'}{$LanguageCode};
        tied( %StoreTypeConfig )->RewriteConfig() or Error('FILE_OPEN_FAILED',{'Mode'=>'>', 'FileName'=>$ConfigFile, 'Code'=>$!});
    }

    #Add $LanguageCode to section [Languages] at StoreType.d/$StoreType.conf
    tie my %StoreTypeConfig, 'Config::IniFiles', ( -file => $ConfigFile )
        or Error('FILE_OPEN_FAILED',{'Mode'=>'<', 'FileName'=>$ConfigFile, 'Code'=>$!});

    return if $StoreTypeConfig{'Languages'}{$LanguageIdentifier} eq 'installed';

    # loop all installed cartridges
    my $hCartridges = CartridgeConfig($StoreName)->section('Cartridges');
    my @Cartridges = unique keys %$hCartridges, map { $_->{'Package'}} @{GetAllInfoCartridges()};

    foreach my $Package (@Cartridges) {
        GetLog()->debug( "Installing language $LanguageIdentifier of cartridge $Package for store type $StoreType" );

        my $CartridgePackage = $Package . '::API::Cartridge';
        UsePackage($CartridgePackage);
        my $Cartridge = $CartridgePackage->new('StoreName' => $StoreName, 'StoreType' => $StoreType, 'StoreTypeVersion' => $Version);

        my $CartridgeDir = $ENV{'EPAGES_CARTRIDGES'}.'/'.$Cartridge->cartridgeDirectory;

        my $SourceDir = $CartridgeDir.'/Data/Public';
        if( -d $SourceDir ) {
            $Cartridge->copyPublicFile($_) foreach
                map { s/^\Q$CartridgeDir\E\///; $_; }
                grep { /\.\Q$LanguageIdentifier\E\.\w+$/ }
                @{ GetFileNames( $SourceDir, {'recursive' => 1} ) };
        }

        $SourceDir = $CartridgeDir.'/Data/javascript';
        if( -d $SourceDir ) {
            $Cartridge->copyJavascriptFile($_) foreach
                map { s/^\Q$CartridgeDir\E\///; $_; }
                grep { /\.\Q$LanguageIdentifier\E\.\w+$/ }
                @{ GetFileNames( $SourceDir, {'recursive' => 1} ) };
        }
    }

    $StoreTypeConfig{'Languages'}{$LanguageIdentifier} = 'installed';
    tied( %StoreTypeConfig )->RewriteConfig() or Error('FILE_OPEN_FAILED',{'Mode'=>'>', 'FileName'=>$ConfigFile, 'Code'=>$!});
    SetAccessRights($ConfigFile);

    return;
}


#========================================================================================
# function     languageCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LanguageCode = $Installer->languageCode;
#----------------------------------------------------------------------------------------
# description  Returns the language code of the language pack.
#----------------------------------------------------------------------------------------
# return       $LanguageCode | language code | string
#========================================================================================
sub languageCode {
    return shift->{'LanguageCode'};
}

#========================================================================================
# function     dialectCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DialectCode = $Installer->dialectCode;
#----------------------------------------------------------------------------------------
# description  Returns the dialect code of a language of the language pack.
#----------------------------------------------------------------------------------------
# return       $DialectCode | dialect code | string
#========================================================================================
sub dialectCode {
    return shift->{'DialectCode'};
}

#========================================================================================
# function     storeName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreName = $Installer->storeName;
#----------------------------------------------------------------------------------------
# description  Returns the logical name of the current database.
#               Throws an error if there is no database connection.
#----------------------------------------------------------------------------------------
# return       $StoreName | store name | string
#========================================================================================
sub storeName {
    my $self = shift;

    my $dbi = GetCurrentDBHandle() or Error('NoDatabaseConnection');
    return $dbi->storeName;
}

#========================================================================================
# function     storeType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreType = $Installer->storeType;
#----------------------------------------------------------------------------------------
# description  Returns the type of the current database.
#               Throws an error if there is no database connection.
#----------------------------------------------------------------------------------------
# return       $StoreName | store name | string
#========================================================================================
sub storeType {
    my $self = shift;

    return StoreType($self->storeName);
}

#========================================================================================
# function     storeTypeVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreTypeVersion = $Installer->storeTypeVersion;
#----------------------------------------------------------------------------------------
# description  The cartridge will be installed for this store type version.
#----------------------------------------------------------------------------------------
# return       $StoreTypeVersion | store type version | string
#========================================================================================
sub storeTypeVersion {
    my $self = shift;
    return LoadRootObject()->get('EpagesVersion');
}

#========================================================================================
# function     storeTypeDir
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreTypeDir = $Installer->storeTypeDir;
#----------------------------------------------------------------------------------------
# description  The cartridge will be installed into this store type directory.
#----------------------------------------------------------------------------------------
# return       $StoreTypeDir | store type directory | string
#========================================================================================
sub storeTypeDir {
    my $self = shift;

    return join('/', 'StoreTypes', $self->storeTypeVersion, $self->storeType);
}

#========================================================================================
# function     uninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Deletes language pack database strings and files.
#========================================================================================
sub uninstall {
    my $self = shift;

    my $LanguageCode = $self->languageCode;
    my $DialectCode = $self->dialectCode;
    my $LanguageID = GetPKeyLanguageByCode( $LanguageCode );
    my $dbi = GetCurrentDBHandle();
    my $StoreName = $self->storeName;

    my $LanguageIdentifier = $LanguageCode;
    if (defined $DialectCode) {
        $LanguageIdentifier = $LanguageCode . '-' . $DialectCode;
    }

    #Remove $LanguageCode from section [Languages] at StoreType.d/$StoreType.conf
    my $ConfigFile = $self->_getStoreTypeConfigFile();
    tie my %StoreTypeConfig, 'Config::IniFiles', ( -file => $ConfigFile ) or Error('FILE_OPEN_FAILED',{'Mode'=>'<', 'FileName'=>$ConfigFile, 'Code'=>$!});
    delete $StoreTypeConfig{'Languages'}{$LanguageIdentifier};
    tied( %StoreTypeConfig )->RewriteConfig() or Error('FILE_OPEN_FAILED',{'Mode'=>'>', 'FileName'=>$ConfigFile, 'Code'=>$!});

    GetLog()->info( "Deleting system strings of language $LanguageIdentifier in store $StoreName" );
    $dbi->do(<<END_SQL, [ $LanguageID ]);
        DELETE FROM localizedstringattribute
        WHERE objectid IN (SELECT objectid FROM object WHERE siteid = 1)
          AND languageid = ?
END_SQL

    # delete files from WebRoot and Stores directories
    GetLog()->info( "Deleting files of language $LanguageIdentifier in store $StoreName" );
    find({
        wanted => $self->_deleteFileSub('xml', 'dot'),
        follow => 1,
    },
       "$ENV{'EPAGES_WEBROOT'}/" . $self->storeTypeDir . '/BO',
       "$ENV{'EPAGES_WEBROOT'}/" . $self->storeTypeDir . '/javascript',
       "$ENV{'EPAGES_STORES'}/$StoreName/ShopImport",
    );

    $self->deleteCtmplFiles;

    return;
}

#========================================================================================
# function     _getStoreTypeConfigFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->_getStoreTypeConfigFile;
#----------------------------------------------------------------------------------------
# description  Creates $StoreType.conf if it does not exist and returns the path.
#----------------------------------------------------------------------------------------
# return       $ConfigFile | store type config file | string
#========================================================================================
sub _getStoreTypeConfigFile {
    my $self       = shift;
    my $Version    = $self->storeTypeVersion;
    my $StoreType  = $self->storeType;
    my $ConfigFile = "$ENV{'EPAGES_CONFIG'}/StoreTypes.d/$Version/$StoreType.conf";
    my $Content    = "[Languages]\n";
    WriteFile($ConfigFile, \$Content, { create_directory=>1} ) unless ExistsFile($ConfigFile);

    return $ConfigFile;
}

#========================================================================================
# function     _deleteFileSub
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Wanted = $Installer->_deleteFileSub(@Extensions);
#----------------------------------------------------------------------------------------
# description  Returns the function that is applied to all files
#----------------------------------------------------------------------------------------
# input        @Extensions | extensions | array
#----------------------------------------------------------------------------------------
# return       $Wanted | function that is applied to all files | ref.function
#========================================================================================
sub _deleteFileSub {
    my $self = shift;
    my (@Extensions) = @_;

    my $LanguageCode = $self->languageCode;
    my $DialectCode = $self->dialectCode;

    my $LanguageIdentifier = $LanguageCode;
    if (defined $DialectCode) {
        $LanguageIdentifier = $LanguageCode . '-' . $DialectCode;
    }

    my $Extensions = join( '|', map { quotemeta } @Extensions );
    my $Re = qr/\.$LanguageIdentifier\.(?:$Extensions)$/x;

    return sub {
        if( ExistsFile($File::Find::name) && $File::Find::name =~ $Re
        ) {
            DeleteFile($File::Find::name);
        }
    };
}

#========================================================================================
# function     deleteGlobalFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->deleteGlobalFiles;
#----------------------------------------------------------------------------------------
# description  Deletes language files from the directories EPAGES_CARTRIDGES and
#               EPAGES_WEBROOT/Doc.
#========================================================================================
sub deleteGlobalFiles {
    my $self = shift;

    my $LanguageCode = $self->languageCode;

    # delete XML files from ImageDB and Cartridges
    find({
        wanted => $self->_deleteFileSub('xml', 'dot'),
        follow => 1,
    },
        "$ENV{'EPAGES_WEBROOT'}/ImageDB",
        $ENV{'EPAGES_CARTRIDGES'},
    );

    GetLog()->info( "Deleting help files of language $LanguageCode" );

    # delete Help directories from WebRoot
    DeleteDirectory( "$ENV{'EPAGES_WEBROOT'}/Doc/Help/$LanguageCode", { 'recursive' => 1 } );
    DeleteDirectory( "$ENV{'EPAGES_WEBROOT'}/Doc/Manual/$LanguageCode", { 'recursive' => 1 } );

    # delete Help directories from cartridges
    my $aCompanyDirs = GetSubDirectories( $ENV{'EPAGES_CARTRIDGES'} );
    foreach my $CompanyDir ( @$aCompanyDirs ) {
        my $aCartridgeDirs = GetSubDirectories( $CompanyDir );
        foreach my $CartridgeDir ( @$aCartridgeDirs ) {
            DeleteDirectory( "$CartridgeDir/Data/WebRoot/Doc/Help/$LanguageCode", { 'recursive' => 1 } );
            DeleteDirectory( "$CartridgeDir/Data/WebRoot/Doc/Manual/$LanguageCode", { 'recursive' => 1 } );
        }
    }

    return;
}

#========================================================================================
# function     deleteCtmplFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->deleteCtmplFiles;
#----------------------------------------------------------------------------------------
# description  Deletes .ctmpl files of a language from EPAGES_STATIC.
#========================================================================================
sub deleteCtmplFiles {
    my $self = shift;

    my $LanguageCode = $self->languageCode;
    my $DialectCode = $self->dialectCode;
    my $StoreName = $self->storeName;

    my $LanguageIdentifier = $LanguageCode;
    if (defined $DialectCode) {
        $LanguageIdentifier = $LanguageCode . '-' . $DialectCode;
    }

    GetLog()->info( "Deleting compiled templates of language $LanguageIdentifier in store $StoreName" );

    find({
        wanted => $self->_deleteFileSub('ctmpl'),
        follow => 1,
    },  "$ENV{'EPAGES_STATIC'}/$StoreName/Templates" );

    return;
}

#========================================================================================
# function     GetLangPackFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($Major, $Basename, $Ext) = GetLangPackFile($Lang, $Version);
#----------------------------------------------------------------------------------------
# description  Returns major version (e.g. 6.15), basename (langpack-ca-epages615p3)
#               and extension ('.tar.gz') of language file with $Lang and $Version
#----------------------------------------------------------------------------------------
# input        $Lang    | language code as 'es' or 'en-US' | string
# input        $Version | version of the language pack | string
# return       $Major   | major version as 6.15 | string
# return       $Basename | basename of the language pack URL (w/o suffix) | string
# return       $Ext | suffix of the language pack URL | string
#========================================================================================
sub GetLangPackFile {
    my ($Lang, $Version) = @_;
    my ($Main, $Sub, $Level) = $Version =~ /^([^.]+)\.([^.]+)\.([^.]+)/;

    return ("$Main.$Sub", "langpack-$Lang-epages$Main${Sub}p$Level", '.tar.gz');
}

#========================================================================================
# function     GetLangPackURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($URLPrefix, $Basename, $Ext) = GetLangPackURL($Lang, $Version);
#----------------------------------------------------------------------------------------
# description  Returns the fastest URL for downloading a language pack. Th elanguage pack is
#               specified by $Lang and $Version in 3 parts: dirname, basename and suffix.
#               The complete URL format of a langpack is (example for http://epages.com):
#               http://epages.com/LanguagePacks/[major]/[lang]/langpack-[lang]-epagesMSpL.tar.gz
#               where [major] is e.g. 6.12; M - main version (e.g. 6); S - sub version (e.g. 12);
#               L - patch level, e.g. 0 (for 6.12.0)
#----------------------------------------------------------------------------------------
# input        $Lang    | language code as 'es' or 'en-US' | string
# input        $Version | version of the language pack | string
# return       $URLPrefix  | dirname of the language pack URL | string
# return       $Basename | basename of the language pack URL (w/o suffix) | string
# return       $Ext | suffix of the language pack URL | string
#========================================================================================
sub GetLangPackURL ($$) {
    my ($Lang, $Version) = @_;

    my ($Major, $Basename, $Ext) = GetLangPackFile($Lang, $Version);
    my %Speed;

    my @UrlList = (INTERN_LANGUAGE_PACK_SERVERS, PUBLIC_LANGUAGE_PACK_SERVERS);

    unshift @UrlList, split(/\s+/,$ENV{EPAGES_REPO}) if $ENV{EPAGES_REPO};

    # first internal resources because they might contain newer versions
    foreach my $Url (@UrlList) {
        # stop if EPAGES_REPO found
        last if $ENV{EPAGES_REPO} and scalar keys %Speed;
        # stop if internal resource found
        last if ElementOf($Url, PUBLIC_LANGUAGE_PACK_SERVERS) and scalar keys %Speed;

        my $time1 = time;
        my $aLanguages = eval { _GetUrlContent("$Url/LanguagePacks/$Major/$Version.txt"); };
        next if ExistsError();
        next unless ElementOf($Lang, $aLanguages);
        $Speed{"$Url/LanguagePacks/$Major/$Lang/"} = time - $time1;
    }

    return ("http://epages.com/LanguagePacks/$Major/$Lang/", $Basename, $Ext)
        unless scalar keys %Speed;

    my $Fastest = (sort { $Speed{$a} <=> $Speed{$b} } keys %Speed)[0];
    return ($Fastest, $Basename, $Ext);
}

#========================================================================================
# function     GetUsedLanguages
# state        public
#----------------------------------------------------------------------------------------
# syntax       my ($hStoreTypeLanguages, $hStoreLanguages) = GetUsedLanguages($aStoreNames, $hFlags)
#----------------------------------------------------------------------------------------
# example      my ($StoreTypeLang, $StoreLang) = GetUsedLanguages(['Store'], {Ignore => [qw(de en)]});
#----------------------------------------------------------------------------------------
# description  Returns the hashes $hStoreTypeLanguages (all languages per storetype) and
#               $hStoreLanguages (all stores per language) for all stores in $aStoreNames.
#----------------------------------------------------------------------------------------
# input        $aStoreNames | list of store names | ref.array
# input        $hFlags | flags
#                   <ul>
#                       <li>Ignore - ignore languages defined in array
#                   </ul>
#               | ref.hash
# return       $hStoreTypeLanguages | set $hStoreTypeLanguages{$StoreType}{$Lang} = 1
#                                      if $StoreType has $Lang
#                                      set $hStoreTypeLanguages{'/'}{$Lang} = 1
#                                      if any store type has $Lang | ref.hash
# return       $hStoreLanguages     | set $hStoreLanguages{$Lang}{$StoreName} = 1
#                                      if $StoreName has $Lang | ref.hash
#========================================================================================
sub GetUsedLanguages ($;$) {
    my ($aStoreNames, $hFlags) = @_;
    my @IgnoredLanguages = defined $hFlags->{Ignore} ? @{$hFlags->{Ignore}} : ();

    my (%StoreTypeLanguages, %StoreLanguages);

    my @SelectUsedLanguages = ('SELECT code2 FROM language WHERE isused = 1',
                               'SELECT code2,dialect FROM language WHERE isused = 1');

    foreach my $StoreName (@$aStoreNames) {
        my $StoreType = StoreType($StoreName);
        RunOnStore( 'Store'=> $StoreName, 'Sub' => sub {
            my $dbi = GetCurrentDBHandle();
            my $select = $SelectUsedLanguages[$dbi->existsTableColumn('language', 'dialect')];
            foreach my $Row (@{$dbi->execute($select)}) {
                my $Lang = $Row->[0];
                $Lang .= "-$Row->[1]" if defined $Row->[1];
                $StoreTypeLanguages{'/'}{$Lang} = 1;
                next if ElementOf($Lang, @IgnoredLanguages);
                $StoreLanguages{$Lang}{$StoreName} = 1;
                $StoreTypeLanguages{$StoreType}{$Lang} = 1;
            }
        });
    }
    return (\%StoreTypeLanguages, \%StoreLanguages);
}

#========================================================================================
# function     DictionaryXmlLocation
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Path = DictionaryXmlLocation($Lang, $hFlags);
#----------------------------------------------------------------------------------------
# example      DictionaryXmlLocation('es', { Dir => $ENV{EPAGES_CARTRIDGES} });
# example      DictionaryXmlLocation('es');
#----------------------------------------------------------------------------------------
# description  Returns path to Dictionary.$Lang.xml in $hFlags->{Dir}
#               (default: $ENV{EPAGES_CARTRIDGES}).
#----------------------------------------------------------------------------------------
# input        $Lang | language code as 'es' | string
# input        $hFlags | flags
#                   <ul>
#                       <li>Dir - search in different directory than $ENV{EPAGES_CARTRIDGES}
#                       <li>IfExists - return location only if exists
#                   </ul>
#               | ref.hash
# return       $Path | path to Dictionary.$Lang.xml | string
#========================================================================================
sub DictionaryXmlLocation ($;$) {
    my ($Lang, $hFlags) = @_;

    $hFlags->{Dir} = $ENV{EPAGES_CARTRIDGES} unless $hFlags->{Dir};
    my $DictionaryXml = "$hFlags->{Dir}/DE_EPAGES/Dictionary/Templates/Dictionary.$Lang.xml";
    return $DictionaryXml if -f $DictionaryXml;
    return $DictionaryXml unless $hFlags->{IfExists};
    return undef;
}

#========================================================================================
# function     LanguagePackLocation
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LP = LanguagePackLocation($Lang, $Version, $hFlags)
#----------------------------------------------------------------------------------------
# example      LanguagePackLocation('es', '6.12.0', {Dir => ["$ENV{EPAGES_PATCHES}/LanguagePacks"]});
# example      LanguagePackLocation('es', '6.12.0');
#----------------------------------------------------------------------------------------
# description  Finds the first occurence of language pack with $Lang and $Version
#               as .tar.gz file in any of $hFlags->{Dir} (default: $ENV{EPAGES_PATCHES}/LanguagePacks).
#               If not found, return $DirList->[0]/$LangPack (unless $hFlags->{IfExists}).
#----------------------------------------------------------------------------------------
# input        $Lang | language code as 'es' | string
# input        $Version | epages version | string
# input        $hFlags | flags
#                   <ul>
#                       <li>Dir - serach directories for language pack
#                       <li>IfExists - return location only if exists
#                   </ul>
#               | ref.hash
# return       $LP | path language pack as .tar.gz file | string
#========================================================================================
sub LanguagePackLocation ($$;$) {
    my ($Lang, $Version, $hFlags) = @_;

    my @SearchDirs = $hFlags->{Dir} ? @{$hFlags->{Dir}} : (_UnpackDir());

    my (undef, $Basename, $Ext) = GetLangPackFile($Lang, $Version);
    my $LangPack = "$Basename$Ext";

    foreach my $Dir (@SearchDirs) {
        return "$Dir/$LangPack" if -f "$Dir/$LangPack";
    }
    return $hFlags->{IfExists} ? undef : "$SearchDirs[0]/$LangPack" ;
}

#========================================================================================
# function     DownloadLanguagePack
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LP = DownloadLanguagePack($Lang, $Version, $UnpackDir)
#----------------------------------------------------------------------------------------
# example      $LP = DownloadLanguagePack('es', '6.12.0', "$ENV{EPAGES_PATCHES}/LanguagePacks");
# example      $LP = DownloadLanguagePack('es', '6.12.0');
#----------------------------------------------------------------------------------------
# description  Downloads language pack from GetLangPackURL($Lang, $Version)
#               and copies it to $UnpackDir (default: $ENV{EPAGES_PATCHES}/LanguagePacks)
#----------------------------------------------------------------------------------------
# input        $Lang | language code as 'es' | string
# input        $Version | epages version | string
# input        $UnpackDir | dir where to store the language pack | string
# return       $LP | path to downloaded language pack as .tar.gz file | string
#========================================================================================
sub DownloadLanguagePack ($$;$) {
    my ($Lang, $Version, $UnpackDir) = @_;

    $UnpackDir = _UnpackDir() unless $UnpackDir;
    my ($URLPrefix, $Basename, $Ext) = GetLangPackURL($Lang, $Version);
    my $URL = "$URLPrefix$Basename$Ext";
    my $RC = GetFile($URL, "$UnpackDir/$Basename$Ext");

    Error('GET_REQUEST_FAILED',
          {URL => $URL, Message => status_message($RC), Code => $RC}) if $RC != 0;

    return "$UnpackDir/$Basename$Ext";
}


#========================================================================================
# function     UntarLanguagePack
# state        public
#----------------------------------------------------------------------------------------
# syntax       UntarLanguagePack($Lang, $Version, $UnpackDir, $hFlags)
#----------------------------------------------------------------------------------------
# example      UntarLanguagePack('es', '6.12.0', "$ENV{EPAGES_PATCHES}/LanguagePacks");
# example      UntarLanguagePack('es', '6.12.0', undef, {Print => 1} );
#----------------------------------------------------------------------------------------
# description  Unpacks language pack .tar.gz file into $UnpackDir (default:
#               $ENV{EPAGES_PATCHES}/LanguagePacks). .tar.gz file must exist in $UnpackDir.
#----------------------------------------------------------------------------------------
# input        $Lang | language code as 'es' | string
# input        $Version | epages version | string
# input        $UnpackDir | dir where to unpack the language pack | string
# input        $hFlags | flags for SystemCommand | ref.hash
# return       $LP | path to unpacked language pack directory | string
#========================================================================================
sub UntarLanguagePack ($$;$$) {
    my ($Lang, $Version, $UnpackDir, $hFlags) = @_;

    $UnpackDir = _UnpackDir() unless $UnpackDir;

    my (undef, $Basename, $Ext) = GetLangPackFile($Lang, $Version);
    my $LangPack = "$Basename$Ext";

    UnpackTar("$UnpackDir/$LangPack", $UnpackDir, $hFlags);
    return "$UnpackDir/$Basename";
}

#========================================================================================
# function     GetLanguagesByVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aLanguages = GetLanguagesByVersion($Version, $LanguageString);
#----------------------------------------------------------------------------------------
# example      @Languages = @{GetLanguagesByVersion('6.12.0', 'es it')};
# example      @Languages = @{GetLanguagesByVersion('6.12.0', 'es any it')};
# example      @Languages = @{GetLanguagesByVersion('6.12.0')};
# example      @Languages = @{GetLanguagesByVersion('6.12', 'any')};
#----------------------------------------------------------------------------------------
# description  If $LanguageString is defined and does not contain 'any', simply splits it.
#               Otherwise returns a list of available languages for specific epages version,
#               retrieved from http://epages.com/LanguagePacks/$Major/$Version.txt
#----------------------------------------------------------------------------------------
# input        $Version | epages version | string
# input        $LanguageString | string with language codes (or 'any' for any lang) | string
# return       $aLanguages | array with language codes | ref.array
#========================================================================================
sub GetLanguagesByVersion ($;$) {
    my ($Version, $LanguageString) = @_;

    my $DefaultLang;
    if ($LanguageString) {
        my @LangList;
        foreach my $Lang (split(/\s+/, $LanguageString)) {
            push @LangList, $Lang;
            next if $Lang eq 'any';
            $DefaultLang //= $Lang;
        }
        return \@LangList unless ElementOf('any', @LangList);
    }
    $DefaultLang //= 'fr'; # french always exist

    $Version .= '.0' unless $Version =~ /\..*\./;
    my ($URLPrefix) = GetLangPackURL($DefaultLang, $Version);
    return _GetUrlContent(dirname($URLPrefix). "/$Version.txt");
}

#========================================================================================
# function     GetLanguageVersionList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $VersionList = GetLanguageVersionList($GetVersion)
#----------------------------------------------------------------------------------------
# example      $VersionList = GetLanguageVersionList('6.14 6.13.1')
#----------------------------------------------------------------------------------------
# description  Returns list of available language versions for all versions in space-
#               separated string $GetVersion retrieved from
#               http://epages.com/LanguagePacks/$Major/versions.txt
#----------------------------------------------------------------------------------------
# input        $GetVersion | space-separated string with versions X.Y or X.Y.Z | string
# return       $VersionList | array with epages versions | ref.array
#========================================================================================
sub GetLanguageVersionList ($) {
    my ($GetVersion) = @_;

    my @Major = grep { /^[0-9]+\.[0-9]+$/ } reverse sort split(/ /, $GetVersion);
    my ($URLPrefix) = scalar @Major ? GetLangPackURL('fr', $Major[0]. '.0') : undef; # french always exist

    my @VersionList;
    foreach my $Version (split(/ /, $GetVersion)) {
        if ($Version =~ /^[0-9]+\.[0-9]+\.[0-9]+$/) {
            push @VersionList, $Version;
            next;
        }
        Error('VOID_VERSION_LIST', {VersionList => $GetVersion, Version => $Version})
            unless $Version =~ /^[0-9]+\.[0-9]+$/;

        push @VersionList, @{_GetUrlContent(dirname($URLPrefix). '/versions.txt')};
    }
    return [@VersionList];
}

#========================================================================================
# function     _UnpackDir
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Path = _UnpackDir();
#----------------------------------------------------------------------------------------
# description  Returns the default directory of unpacked language packs.
#----------------------------------------------------------------------------------------
# return       $LanguagePackDirectory | default directory of unpacked language packs | string
#========================================================================================
sub _UnpackDir {
    my $PatchDir = $ENV{EPAGES_PATCHES} ? $ENV{EPAGES_PATCHES} : "$ENV{EPAGES_SHARED}/Patches";
    return "$PatchDir/LanguagePacks";
}

#========================================================================================
# function     _GetUrlContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aContent = _GetUrlContent($URL);
#----------------------------------------------------------------------------------------
# description  Return the content of $URL as space-separated array.
#----------------------------------------------------------------------------------------
# input        $URL | URL | string
# return       $aContent | array with space-separated URL content | ref.array
#========================================================================================
sub _GetUrlContent {
    my ($URL) = @_;

    my $ua = LWP::UserAgent->new;
    my $req = HTTP::Request->new( GET => $URL );
    my $res = $ua->request($req);

    if ($res->is_success) {
        my $Content = $res->content;
        $Content =~ s/^\s+//s;
        $Content =~ s/\s+$//s;
        return [split(/\s+/, $Content)];
    }
    Error('GET_REQUEST_FAILED', {URL => $URL, Message => $res->as_string, Code => $res->code});
}

1;
