#========================================================================================
# package      DE_EPAGES::Installer::API::Object::PatchCodeGenerator
# state        public
# description  This modul generates perl code to handle changes between two sql files,
#               that represents the database structure.
#               Is used by the DE_EPAGES::Installer::API::PatchBuilder modul.
#               Use DE_EPAGES::Object::API::SQLStructureImportHandler and DE_EPAGES::Object::API::SQLCompare
#               to identify and handle changes between sql files.
#========================================================================================
package DE_EPAGES::Installer::API::Object::PatchCodeGenerator;
use strict;

use DE_EPAGES::Object::API::SQLStructureImportHandler;
use DE_EPAGES::Object::API::SQLCompare qw (
    GetDiff
);

sub _initialize {
    my $self = shift;

    #placeholder for changes in sql files
    $self->{'SQLDiffCodeHead'} = "\n    my \$dbi = \$self->getCurrentDBHandle();\n";
    $self->{'SQLDiffCodeBegin'} = "\n    if(\$dbi->driverName() eq '\$SQLType'){";
    $self->{'SQLDiffCode'} = "\n";
    $self->{'SQLDiffCodeEnd'} = "\n    }";

    #sql Importer
    $self->{'ImportHandler'} = DE_EPAGES::Object::API::SQLStructureImportHandler->new();
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Installer::API::Object::PatchCodeGenerator->new();
#----------------------------------------------------------------------------------------
# description  constructor for patch code generator.
#----------------------------------------------------------------------------------------
# return       $PatchCodeGenerator | PatchCodeGenerator object | object
#========================================================================================
sub new{
    my $class = shift;
    my $self = bless {}, $class;

    $self->_initialize();

    return $self;
}

sub getControlCode{
    my $self = shift;

    my $Result = "#remove these function after check patch manually\n" .
                "sub patch{\n" .
                "    die \"CHECK CHANGES MANUALLY FOR PATCH\";\n" .
                "}\n\n";

    return $Result;
}

#========================================================================================
# function     getCodeForDiffInSQLFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->getCodeForDiffInSQLFiles($OldFile,$NewFile,$hTableColumnsPositions);
#----------------------------------------------------------------------------------------
# description  return code to handle changed column position in sql files.
#----------------------------------------------------------------------------------------
# input        $hOldFilesNewFiles | list of new and old files, keys = old filenames, values = new file names | ref.hash
# input        $SQLType | Type of SQL file, 'Sybase' or 'mysql' | string
# return       $GeneratedCode | code, used in ../API/Cartridge.pm, to handle changed database structure in sql files | string
#========================================================================================
sub getCodeForDiffInSQLFiles {
    my $self = shift;
    my ($hOldFilesNewFiles,$SQLType) = @_;

    #clean up
    $self->_initialize();

    #set SQL Type in DiffCode
    $self->{'SQLDiffCodeBegin'} =~ s/\$SQLType/$SQLType/;

    my $NewFile;
    my $DiffExists = 0;
    foreach my $OldFile (sort keys %$hOldFilesNewFiles){
        $NewFile = $hOldFilesNewFiles->{$OldFile};

        $OldFile =~ s/\\/\//g;
        $NewFile =~ s/\\/\//g;

        my $hDiff = GetDiff($OldFile,$NewFile);

        #check for massage for changes in file
        if (
            scalar keys %{$hDiff->{'ChangedTableColumnsPosition'}} > 0 or
            scalar keys %{$hDiff->{'ChangedTableColumnsDataType'}} > 0 or
            scalar keys %{$hDiff->{'ChangedIndices'}} > 0 or
            scalar keys %{$hDiff->{'NewIndices'}} > 0 or
            scalar keys %{$hDiff->{'RemovedIndices'}} > 0 or
            scalar keys %{$hDiff->{'ChangedLockSchemes'}} > 0 or
            scalar keys %{$hDiff->{'NewTableColumns'}} > 0 or
            scalar keys %{$hDiff->{'RemovedTableColumns'}} > 0 or
            scalar keys %{$hDiff->{'RemovedContraints'}} > 0 or
            scalar keys %{$hDiff->{'NewContraints'}} > 0 or
            scalar keys %{$hDiff->{'ChangedTableColumnsDefault'}} > 0 or
            scalar keys %{$hDiff->{'ChangedTableColumnsAutoIncrement'}} > 0 or
            scalar keys %{$hDiff->{'ChangedTableColumnsNotNull'}} > 0
        ){
            my $DisplayFileName = $NewFile;
            $DisplayFileName =~ s/.*(\/Database\/)/$1/g;
            $self->_pushDiffCode("\n        \#changes for file $DisplayFileName\n");
        }

        if (scalar keys %{$hDiff->{'ChangedContraints'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForRemovedConstraints($OldFile,$NewFile,$hDiff->{'ChangedContraints'})
            );
        }
        if (scalar keys %{$hDiff->{'RemovedIndices'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForRemovedIndices($OldFile,$NewFile,$hDiff->{'RemovedIndices'})
            );
        }
        if (scalar keys %{$hDiff->{'RemovedTableColumns'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForRemovedTableColumns($OldFile,$NewFile,$hDiff->{'RemovedTableColumns'})
            );
        }
        if (scalar keys %{$hDiff->{'RemovedContraints'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForRemovedConstraints($OldFile,$NewFile,$hDiff->{'RemovedContraints'})
            );
        }
        if (scalar keys %{$hDiff->{'ChangedTableColumnsPosition'}} > 0){
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForChangedTableColumnPosition($OldFile,$NewFile,$hDiff->{'ChangedTableColumnsPosition'})
            );
        }
        if (scalar keys %{$hDiff->{'ChangedTableColumnsAutoIncrement'}} > 0){
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForChangedTableColumnAutoIncrement($OldFile,$NewFile,$hDiff->{'ChangedTableColumnsAutoIncrement'})
            );
        }
        if (scalar keys %{$hDiff->{'ChangedTableColumnsNotNull'}} > 0) {
            $DiffExists = 1;
            $self->{'SQLDiffCode'} = $self->{'SQLDiffCode'} .
                $self->_getCodeForChangedData($OldFile,$NewFile,$hDiff->{'ChangedTableColumnsNotNull'});
        }
        if (scalar keys %{$hDiff->{'ChangedTableColumnsDataType'}} > 0){
            $DiffExists = 1;
            $self->{'SQLDiffCode'} = $self->{'SQLDiffCode'} .
                $self->_getCodeForChangedData($OldFile,$NewFile,$hDiff->{'ChangedTableColumnsDataType'});
        }
        if (scalar keys %{$hDiff->{'NewTableColumns'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForNewTableColumns($OldFile,$NewFile,$hDiff->{'NewTableColumns'},$SQLType)
            );
        }
        if (scalar keys %{$hDiff->{'ChangedIndices'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForChangedIndices($OldFile,$NewFile,$hDiff->{'ChangedIndices'})
            );
        }
        if (scalar keys %{$hDiff->{'NewIndices'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForNewIndices($OldFile,$NewFile,$hDiff->{'NewIndices'})
            );
        }

        if (scalar keys %{$hDiff->{'ChangedLockSchemes'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForChangedLockSchemes($OldFile,$NewFile,$hDiff->{'ChangedLockSchemes'})
            );
        }
        if (scalar keys %{$hDiff->{'ChangedContraints'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForNewConstraints($OldFile,$NewFile,$hDiff->{'ChangedContraints'})
            );
        }
        if (scalar keys %{$hDiff->{'NewContraints'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForNewConstraints($OldFile,$NewFile,$hDiff->{'NewContraints'})
            );
        }
        if (scalar keys %{$hDiff->{'RemovedViews'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForRemovedViews($OldFile,$NewFile,$hDiff->{'RemovedViews'})
            );
        }
        if (scalar keys %{$hDiff->{'ChangedViews'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForChangedViews($OldFile,$NewFile,$hDiff->{'ChangedViews'})
            );
        }
        if (scalar keys %{$hDiff->{'NewViews'}} > 0) {
            $DiffExists = 1;
            $self->_pushDiffCode(
                $self->_getCodeForNewViews($OldFile,$NewFile,$hDiff->{'NewViews'})
            );
        }
    }

    my $Result = $self->{'SQLDiffCodeHead'} . $self->{'SQLDiffCodeBegin'} . $self->{'SQLDiffCode'} . $self->{'SQLDiffCodeEnd'};

    #clean up
    $self->_initialize();

    return $Result;
}

#========================================================================================
# function     _pushDiffCode
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_pushDiffCode($Code);
#----------------------------------------------------------------------------------------
# description  added code lines to rsult.
#----------------------------------------------------------------------------------------
# input        $Code | code to add | string
#========================================================================================
sub _pushDiffCode{
    my $self = shift;
    my ($Code) = @_;

    $self->{'SQLDiffCode'} = $self->{'SQLDiffCode'} . $Code;
    return;
}

#========================================================================================
# function     _getCodeForChangedTableColumnPosition
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForChangedTableColumnPosition($OldFile,$NewFile,$hTableColumnsPositions);
#----------------------------------------------------------------------------------------
# description  return code to handle changed column position in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hTableColumns | list of table columns that changed | ref.hash
# return       $GeneratedCode | code to handle changed datatypes in sql files | string
#========================================================================================
sub _getCodeForChangedTableColumnPosition{
    my $self = shift;
    my ($OldFile,$NewFile,$hTableColumnsPositions) = @_;

    return "        die \"column position changed from $OldFile to $NewFile\";\n";

}

#========================================================================================
# function     _getCodeForNewTableColumns
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForNewTableColumns($OldFile,$NewFile,$hNewTableColumns);
#----------------------------------------------------------------------------------------
# description  return code to handle new columns in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hNewTableColumns | list of table columns that are new | ref.hash
# input        $SQLType | Type of SQL file, 'Sybase' or 'mysql' | string
# return       $GeneratedCode | code to handle table columns in sql files | string
#========================================================================================
sub _getCodeForNewTableColumns{
    my $self = shift;
    my ($OldFile,$NewFile,$hTableColumnsPositions,$SQLType) = @_;

    #get infos
    my $Result = '';
    $SQLType = 'mysql' unless defined $SQLType;

    #get SQL structure object from sql file
    my $NewSQLStruct = $self->{'ImportHandler'}->getSQLStructure($NewFile);

    #for each table
    foreach my $Table (sort keys %$hTableColumnsPositions) {
        #for each new column
        foreach my $NewColumn (sort keys %{$hTableColumnsPositions->{$Table}}){
            my $DataType = $NewSQLStruct->getTableColumnDataType($Table,$NewColumn);
            my $NotNull = $NewSQLStruct->getTableColumnNotNull($Table,$NewColumn);
            my $AutoIncrement = $NewSQLStruct->getTableColumnAutoIncrement($Table,$NewColumn);


            my $AutoIncrementText = '';
            $AutoIncrementText = ' auto_increment' if ($AutoIncrement);

            if ($SQLType eq 'Sybase'){
                if ($NewSQLStruct->getTableColumnPosition($Table,$NewColumn) ne $NewSQLStruct->countTableColumns($Table,$NewColumn)){
                    if ($SQLType eq 'Sybase'){
                        $Result .=  "        die \"new column '$Table.$NewColumn' must added at the end in $OldFile to $NewFile\";\n";
                    }
                }
                #check special case should dont be automate
                if ($NewSQLStruct->existsTableColumnDefault($Table,$NewColumn)){
                    $Result .=  "        die \"check for new column '$Table.$NewColumn'  default is givin in $OldFile to $NewFile;\"";
                    next;
                }

                if ( $NotNull ){
                    $Result .=
                        "        unless (\$dbi->existsTableColumn('$Table', '$NewColumn')) {\n" .
                        "            \$self->logInfo('add column $Table.$NewColumn');\n" .
                        "            \$dbi->addTableColumn('$Table', '$NewColumn $DataType not null default 0$AutoIncrementText');\n" .
                        "            \$dbi->modifyTableColumnDropDefault('$Table','$NewColumn');\n" .
                        "        }\n";
                }
                else {
                    $Result .=
                        "        unless (\$dbi->existsTableColumn('$Table', '$NewColumn')) {\n" .
                        "            \$self->logInfo('add column $Table.$NewColumn');\n" .
                        "            \$dbi->addTableColumn('$Table', '$NewColumn $DataType null$AutoIncrementText');\n" .
                        "        }\n";
                }
            }
            elsif($SQLType eq 'mysql'){
                my $PreviousColumn = $NewSQLStruct->getPreviousTableColumn($Table, $NewColumn);

                #check special case should dont be automate
                if ($NewSQLStruct->existsTableColumnDefault($Table,$NewColumn)){
                    $Result .=  "        die \"check for new column '$Table.$NewColumn'  default is givin in $OldFile to $NewFile;\"";
                    next;
                }

                if (defined $PreviousColumn){#not the first column insert after previous column
                    if ( $NotNull ){
                        $Result .=
                            "        unless (\$dbi->existsTableColumn('$Table', '$NewColumn')) {\n" .
                            "            \$self->logInfo('add column $Table.$NewColumn after column $Table.$PreviousColumn');\n" .
                            "            \$dbi->addTableColumnAfterColumn('$Table', '$PreviousColumn', '$NewColumn $DataType not null default 0$AutoIncrementText');\n" .
                            "            \$dbi->modifyTableColumnDropDefault('$Table','$NewColumn');\n" .
                            "        }\n";
                    }
                    else {
                        $Result .=
                            "        unless (\$dbi->existsTableColumn('$Table', '$NewColumn')) {\n" .
                            "            \$self->logInfo('add column $Table.$NewColumn');\n" .
                            "            \$dbi->addTableColumnAfterColumn('$Table', '$PreviousColumn', '$NewColumn $DataType null$AutoIncrementText');\n" .
                            "        }\n";
                    }
                }
                else {#first column insert first
                    if ( $NotNull ){
                        $Result .=
                            "        unless (\$dbi->existsTableColumn('$Table', '$NewColumn')) {\n" .
                            "            \$self->logInfo('add column $Table.$NewColumn after column $Table.$PreviousColumn');\n" .
                            "            \$dbi->addTableColumnFirst('$Table', '$NewColumn $DataType not null default 0$AutoIncrementText');\n" .
                            "            \$dbi->modifyTableColumnDropDefault('$Table','$NewColumn');\n" .
                            "        }\n";
                    }
                    else {
                        $Result .=
                            "        unless (\$dbi->existsTableColumn('$Table', '$NewColumn')) {\n" .
                            "            \$self->logInfo('add column $Table.$NewColumn');\n" .
                            "            \$dbi->addTableColumnFirst('$Table', '$NewColumn $DataType null$AutoIncrementText first');\n" .
                            "        }\n";
                    }
                }
            }
        }
    }

    return $Result;

}

#========================================================================================
# function     _getCodeForChangedData
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForChangedData($OldFile,$NewFile,$hTableColumns);
#----------------------------------------------------------------------------------------
# description  return code to handle changed datatypes in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hTableColumns | list of table columns that changed | ref.hash
# return       $GeneratedCode | code to handle changed datatypes in sql files | string
#========================================================================================
sub _getCodeForChangedData {
    my $self = shift;
    my ($OldFile,$NewFile,$hTableColumns) = @_;

    #get SQL structure object from sql file
    my $NewSQLStruct = $self->{'ImportHandler'}->getSQLStructure($NewFile);

    my $GeneratedCode = '';

    #generate code for each column
    foreach my $Table (sort keys %$hTableColumns){
        foreach my $Column (sort keys %{$hTableColumns->{$Table}}){

            #get parameter for function modifyTableColumn
            my $NewDataType = $NewSQLStruct->getTableColumnDataType($Table,$Column);
            my $NewNotNull;
            if ($NewSQLStruct->getTableColumnNotNull($Table,$Column)){
                $NewNotNull = 'not null';
            }
            else{
                $NewNotNull = 'null';
            }
            my $NewDefault = $NewSQLStruct->getTableColumnDefault($Table,$Column);

            #default are not handeled at these time
            die "\nenlarge PatchCodeGenerator for handeling defaults, default '$NewDefault' detected in '$Table.$Column'.\n"
            if (defined $NewDefault and ($NewDefault ne '' and $NewDefault ne '_SQLStruct-Default'));

            #generate return code
            $GeneratedCode .=
                "        \$self->logInfo('change type of column $Table.$Column');\n" .
                "        \$dbi->modifyTableColumn('$Table', '$Column $NewDataType $NewNotNull');\n\n";
        }
    }
    return $GeneratedCode;
}

#========================================================================================
# function     _getCodeForChangedIndices
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForChangedIndices($OldFile,$NewFile,$hTableIndices);
#----------------------------------------------------------------------------------------
# description  return code to handle changed indices in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hTableIndices | list of table indices that changed | ref.hash
# return       $GeneratedCode | code to handle changed indices in sql files | string
#========================================================================================
sub _getCodeForChangedIndices {
    my $self = shift;
    my ($OldFile,$NewFile,$hTableIndices) = @_;

    #stop if nothing is to do
    return unless (scalar keys %$hTableIndices);

    #get SQL object of new sql file
    my $NewSQLStruct = $self->{'ImportHandler'}->getSQLStructure($NewFile);

    #initialyze return code
    my $GeneratedCode = '';

    #generate code for all changes of all indices
    foreach my $Table (sort keys %$hTableIndices) {
        foreach my $Index (sort keys %{$hTableIndices->{$Table}}){
            my $hInfoChangedIndex = $NewSQLStruct->getTableIndexInfo($Table, $Index);

            #generate log message
            $GeneratedCode .=
                "        \$self->logInfo('recreate index $Table.$Index');\n" .
                "        \$dbi->recreateIndex('$Table', '$Index', ";
            my $Options = '';

            #look for changed values and generate options value for 3rd parameter
            if ($hInfoChangedIndex->{'Unique'} eq '1') {
                $Options .= 'UNIQUE ';
            }
            if ($hInfoChangedIndex->{'Clustered'} =~ m/^(CLUSTERED|NONCLUSTERED)$/) {
                $Options .= $hInfoChangedIndex->{'Clustered'}.' ';
            }

            #set otions value in return code
            $GeneratedCode .= ( $Options eq '' ? 'undef' : "'$Options'" ) . ",\n            [";

            #generate code for 4th parameter
            $GeneratedCode .= join ',', map { "'$_'" } sort keys %{$hInfoChangedIndex->{'ColumnSortOrderPairs'}};
            $GeneratedCode .= "],\n";

            #generate index columns array for 5th parameter
            my @ColumnOptions = ();
            foreach my $IndexKey (sort keys %{$hInfoChangedIndex->{'ColumnSortOrderPairs'}}) {
                my $IndexValue = $hInfoChangedIndex->{'ColumnSortOrderPairs'}->{$IndexKey};
                push (@ColumnOptions, "'$IndexKey' => '$IndexValue'") if ($IndexValue =~ m/^(ASC|DESC)$/);
                push (@ColumnOptions, "'$IndexKey' => 'ASC'") if ($IndexValue =~ m/^_SQLStruct-Default$/);
            }

            #generate code for 5th parameter
            if (scalar @ColumnOptions) {
                $GeneratedCode .= '            { ';
                $GeneratedCode .= join ', ', @ColumnOptions;
                $GeneratedCode .= " }\n";
            }
            $GeneratedCode .= '        )';
            $GeneratedCode .= ";\n";
        }
    }
    return $GeneratedCode;
}

#========================================================================================
# function     _getCodeForNewIndices
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForNewIndices($OldFile,$NewFile,$hTableIndices);
#----------------------------------------------------------------------------------------
# description  return code to handle new indices in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hTableIndices | list of new table indices | ref.hash
# return       $GeneratedCode | code to handle change indices in sql files | string
#========================================================================================
sub _getCodeForNewIndices {
    my $self = shift;
    my ($OldFile,$NewFile,$hTableIndices) = @_;

    return unless (scalar keys %$hTableIndices);

    my $NewSQLStruct = $self->{'ImportHandler'}->getSQLStructure($NewFile);

    my $GeneratedCode = '';

    foreach my $Table (sort keys %$hTableIndices) {
        foreach my $Index (sort keys %{$hTableIndices->{$Table}}){
            my $hInfoNewIndex = $NewSQLStruct->getTableIndexInfo($Table, $Index);
            $GeneratedCode .=
                "        \$self->logInfo('create index $Table.$Index');\n" .
                "        \$dbi->createIndex('$Table', '$Index', ";
            my $Options = '';
            if ($hInfoNewIndex->{'Unique'} eq '1') {
                $Options .= 'UNIQUE ';
            }
            if ($hInfoNewIndex->{'Clustered'} =~ m/^(CLUSTERED|NONCLUSTERED)$/) {
                $Options .= $hInfoNewIndex->{'Clustered'}.' ';
            }
            $GeneratedCode .= ( $Options eq '' ? 'undef' : "'$Options'" ) . ', [';
            $GeneratedCode .= join ',', map { "'$_'" } sort keys %{$hInfoNewIndex->{'ColumnSortOrderPairs'}};
            $GeneratedCode .= ']';
            my @ColumnOptions = ();
            foreach my $IndexKey (sort keys %{$hInfoNewIndex->{'ColumnSortOrderPairs'}}) {
                my $IndexValue = $hInfoNewIndex->{'ColumnSortOrderPairs'}->{$IndexKey};
                push @ColumnOptions, "'$IndexKey' => '$IndexValue'" if ($IndexValue =~ m/^(ASC|DESC)$/);
            }
            if (scalar @ColumnOptions) {
                $GeneratedCode .= ', { ';
                $GeneratedCode .= join ', ', @ColumnOptions;
                $GeneratedCode .= ' }';
            }
            $GeneratedCode .= ')';
            $GeneratedCode .= "\n            unless (\$dbi->existsIndex('$Table','$Index'))";

            $GeneratedCode .= ";\n\n";
        }
    }
    return $GeneratedCode;
}

#========================================================================================
# function     _getCodeForChangedTableColumnAutoIncrement
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForChangedTableColumnAutoIncrement($OldFile,$NewFile,$hTableColumnAutoIncrementChanges);
#----------------------------------------------------------------------------------------
# description  return code to handle removed indices in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hTableColumnAutoIncrementChanges | list of tables and colums where auto_increment changed | ref.hash
# return       $GeneratedCode | code to handle removed indices in sql files | string
#========================================================================================
sub _getCodeForChangedTableColumnAutoIncrement {
    my $self = shift;
    my ($OldFile,$NewFile,$hTableColumnAutoIncrementChanges) = @_;

    my $GeneratedCode = '';

    #get sql structures
    my $NewSQLStruct = $self->{'ImportHandler'}->getSQLStructure($NewFile);
    my $OldSQLStruct = $self->{'ImportHandler'}->getSQLStructure($OldFile);

    foreach my $Table (sort keys %$hTableColumnAutoIncrementChanges) {
        foreach my $Column (sort keys %{$hTableColumnAutoIncrementChanges->{$Table}}){

            #get auto increment values
            my $OldAutoIncrement = $OldSQLStruct->getTableColumnAutoIncrement($Table,$Column);
            my $NewAutoIncrement = $NewSQLStruct->getTableColumnAutoIncrement($Table,$Column);

            #check if auto increment have to remove
            if ($OldAutoIncrement and not $NewAutoIncrement){
                $GeneratedCode .=
                    "        \$self->logInfo('drop auto_increment $Table.$Column');\n" .
                    "        \$dbi->modifyTableColumnDropAutoIncrement('$Table', '$Column');\n\n";
            }

            #check if auto increment have to add
            if (not $OldAutoIncrement and $NewAutoIncrement){
                $GeneratedCode .=
                    "        \$self->logInfo('drop auto_increment $Table.$Column');\n" .
                    "        \$dbi->modifyTableColumnAddAutoIncrement('$Table', '$Column');\n\n";
            }
        }
    }
    return $GeneratedCode;
}

#========================================================================================
# function     _getCodeForRemovedIndices
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForRemovedIndices($OldFile,$NewFile,$hTableIndices);
#----------------------------------------------------------------------------------------
# description  return code to handle removed indices in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hTableIndices | list of new table indices | ref.hash
# return       $GeneratedCode | code to handle removed indices in sql files | string
#========================================================================================
sub _getCodeForRemovedIndices {
    my $self = shift;
    my ($OldFile,$NewFile,$hTableIndices) = @_;

    my $GeneratedCode = '';

    foreach my $Table (sort keys %$hTableIndices) {
        my $IndicesString = join(', ', sort keys %{$hTableIndices->{$Table}});
        my $QuotedIndicesString = join(q(', '), sort keys %{$hTableIndices->{$Table}});
        $QuotedIndicesString = "'$QuotedIndicesString'";

        $GeneratedCode .=
            "        \$self->logInfo('drop indices \\\'$IndicesString\\\' from table \\\'$Table\\\'');\n" .
            "        \$dbi->dropIndexesIfExist('$Table', [$QuotedIndicesString]);\n\n";
    }
    return $GeneratedCode;
}

#========================================================================================
# function     _getCodeForChangedLockSchemes
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForChangedLockSchemes($OldFile,$NewFile,$hChangedLockSchemes);
#----------------------------------------------------------------------------------------
# description  return code to handle changed lock scheme in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hChangedLockSchemes | list of tables and info about their new and old lock scheme | ref.hash
# return       $GeneratedCode | code to handle changed lock scheme in sql files | string
#========================================================================================
sub _getCodeForChangedLockSchemes{
    my $self = shift;
    my ($OldFile,$NewFile,$hChangedLockSchemes) = @_;

    my $GeneratedCode = '';

    foreach my $Table (sort keys %$hChangedLockSchemes){
        my $Value = $hChangedLockSchemes->{$Table}->{'NewLockScheme'};
        $GeneratedCode .=
            "        \$self->logInfo('change lock scheme of $Table to $Value');\n" .
            "        \$dbi->modifyTableLockingScheme('$Table', '$Value');\n\n";
    }

    return $GeneratedCode;

}

#========================================================================================
# function     _getCodeForRemovedTableColumns
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForRemovedTableColumns($OldFile,$NewFile,$hRemovedTableColumns);
#----------------------------------------------------------------------------------------
# description  return code to handle Removed table columns in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hRemovedTableColumns | list of tables and info about Removed columns | ref.hash
# return       $GeneratedCode | code to handle Removed columns in sql files | string
#========================================================================================
sub _getCodeForRemovedTableColumns{
    my $self = shift;
    my ($OldFile,$NewFile,$hRemovedTableColumns) = @_;

    my $GeneratedCode = '';
    foreach my $Table (sort keys %$hRemovedTableColumns){
        foreach my $RemovedColumn (sort keys %{$hRemovedTableColumns->{$Table}}){
            $GeneratedCode .=
                    "        die \"save data before drop column $Table.$RemovedColumn\";\n" .
                    "        if (\$dbi->existsTableColumn('$Table', '$RemovedColumn')) {\n" .
                    "            \$self->logInfo('drop column $Table.$RemovedColumn');\n" .
                    "            \$dbi->dropTableColumn('$Table','$RemovedColumn');\n" .
                    "        }\n";
        }
    }

    return $GeneratedCode;

}

#========================================================================================
# function     _getCodeForRemovedConstraints
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForRemovedConstraints($OldFile,$NewFile,$hRemovedConstraints);
#----------------------------------------------------------------------------------------
# description  return code to handle removed constraints in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hRemovedConstraints | list of tables and info about removed columns | ref.hash
# return       $GeneratedCode | code to handle removed constraints in sql files | string
#========================================================================================
sub _getCodeForRemovedConstraints{
    my $self = shift;
    my ($OldFile,$NewFile,$hRemovedConstraints) = @_;

    my $GeneratedCode = '';
    foreach my $Table (sort keys %$hRemovedConstraints){
        foreach my $RemovedConstraint (sort keys %{$hRemovedConstraints->{$Table}}){
            $GeneratedCode .=
                    "        if (\$dbi->existsForeignKey('$RemovedConstraint')) {\n" .
                    "            \$self->logInfo('drop foreign key $Table.$RemovedConstraint');\n" .
                    "            \$dbi->dropForeignKey('$Table', '$RemovedConstraint');\n" .
                    "        }\n";
        }
    }

    return $GeneratedCode;

}

#========================================================================================
# function     _getCodeForNewConstraints
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForNewConstraints($OldFile,$NewFile,$hNewConstraints);
#----------------------------------------------------------------------------------------
# description  return code to handle New constraints in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hNewConstraints | list of tables and info about New constraints | ref.hash
# return       $GeneratedCode | code to handle New constraints in sql files | string
#========================================================================================
sub _getCodeForNewConstraints{
    my $self = shift;
    my ($OldFile,$NewFile,$hNewConstraints) = @_;

    my $GeneratedCode = '';
    foreach my $Table (sort keys %$hNewConstraints){
        foreach my $NewConstraint (sort keys %{$hNewConstraints->{$Table}}){
            my $ReferencedTableName = $hNewConstraints->{$Table}->{$NewConstraint}->{'ReferencedTable'};
            my $hColumnPairs = $hNewConstraints->{$Table}->{$NewConstraint}->{'ForeignKeysColumnPairs'};

            my @ReferencedFromColumns;
            my @ReferencedToColumns;

            foreach my $ReferencedFromColumn (sort keys %$hColumnPairs){
                push(@ReferencedFromColumns, $ReferencedFromColumn);
                push(@ReferencedToColumns, $hColumnPairs->{$ReferencedFromColumn});
            }

            $GeneratedCode .=
                    "        if (not \$dbi->existsForeignKey('$NewConstraint')) {\n" .
                    "            \$self->logInfo('add foreign key $Table.$NewConstraint');\n" .
                    "            \$dbi->addForeignKey(\n" .
                    "                '$Table',\n" .
                    "                '$NewConstraint',\n" .
                    "                [\n";

            foreach my $FromColumn (@ReferencedFromColumns) {
                $GeneratedCode .= "                    '$FromColumn',\n";
            }

            $GeneratedCode .=
                    "                ],\n".
                    "                '$ReferencedTableName',\n".
                    "                [\n";

            foreach my $ToColumn (@ReferencedToColumns) {
                $GeneratedCode .= "                    '$ToColumn',\n";
            }

            $GeneratedCode .=
                "                ]\n".
                "            );\n".
                "        }\n";

        }
    }

    return $GeneratedCode;

}

#========================================================================================
# function     _getCodeForChangedViews
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForChangedViews($OldFile,$NewFile,$hChangedViews);
#----------------------------------------------------------------------------------------
# description  return code to handle changed views in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hChangedViews | list of views that changed | ref.hash
# return       $GeneratedCode | code to handle changed views in sql files | string
#========================================================================================
sub _getCodeForChangedViews {
    my $self = shift;
    my ($OldFile,$NewFile,$hChangedViews) = @_;

    #stop if nothing is to do
    return unless (scalar keys %$hChangedViews);

    #get SQL object of new sql file
    my $NewSQLStruct = $self->{'ImportHandler'}->getSQLStructure($NewFile);

    #initialyze return code
    my $GeneratedCode = '';

    #generate code for all changes of all view
    foreach my $View (sort keys %$hChangedViews) {

        my $CreateEx = "CREATE VIEW $View AS";
        my $SelectEx = $NewSQLStruct->getViewSelectExpression($View);
        my $FromEx = $NewSQLStruct->getViewFromExpression($View);
        my $WhereEx = $NewSQLStruct->getViewWhereExpression($View);

        #generate remove code
        $GeneratedCode .=
            "\n" .
            "        #remove old version of view\n" .
            "        if( \$dbi->existsView('$View') ) {\n" .
            "            \$self->logInfo(\"drop old version of view '$View'\");\n" .
            "            \$dbi->dropView('$View');\n" .
            "        }\n";

        #generate add code
        $GeneratedCode .=
            "\n" .
            "        #add new version of view\n" .
            "        \$self->logInfo(\"add new version of view '$View'\");\n" .
            "        \$dbi->do(<<'END_SQL');\n" .
            "$CreateEx\n" .
            "$SelectEx\n" .
            "$FromEx\n" .
            "$WhereEx\n" .
            "END_SQL\n";

    }
    return $GeneratedCode;
}

#========================================================================================
# function     _getCodeForRemovedViews
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForRemovedViews($OldFile,$NewFile,$hRemovedViews);
#----------------------------------------------------------------------------------------
# description  return code to handle changed views in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hRemovedViews | list of views that changed | ref.hash
# return       $GeneratedCode | code to handle changed views in sql files | string
#========================================================================================
sub _getCodeForRemovedViews {
    my $self = shift;
    my ($OldFile,$NewFile,$hRemovedViews) = @_;

    #stop if nothing is to do
    return unless (scalar keys %$hRemovedViews);

    #initialyze return code
    my $GeneratedCode = '';

    #generate code for all changes of all view
    foreach my $View (sort keys %$hRemovedViews) {

        #generate remove code
        $GeneratedCode .=
            "\n" .
            "        #remove old version of view\n" .
            "        if( \$dbi->existsView('$View') ) {\n" .
            "            \$self->logInfo(\"drop old version of view '$View'\");\n" .
            "            \$dbi->dropView('$View');\n" .
            "        }\n";
    }
    return $GeneratedCode;
}

#========================================================================================
# function     _getCodeForNewViews
# state        private
#----------------------------------------------------------------------------------------
# syntax        $self->_getCodeForNewViews($OldFile,$NewFile,$hNewViews);
#----------------------------------------------------------------------------------------
# description  return code to handle changed views in sql files.
#----------------------------------------------------------------------------------------
# input        $OldFile | SQL file name | string
# input        $NewFile | SQL file name | string
# input        $hNewViews | list of views that changed | ref.hash
# return       $GeneratedCode | code to handle changed views in sql files | string
#========================================================================================
sub _getCodeForNewViews {
    my $self = shift;
    my ($OldFile,$NewFile,$hNewViews) = @_;

    #stop if nothing is to do
    return unless (scalar keys %$hNewViews);

    #get SQL object of new sql file
    my $NewSQLStruct = $self->{'ImportHandler'}->getSQLStructure($NewFile);

    #initialyze return code
    my $GeneratedCode = '';

    #generate code for all changes of all view
    foreach my $View (sort keys %$hNewViews) {

        my $CreateEx = "CREATE VIEW $View AS";
        my $SelectEx = $NewSQLStruct->getViewSelectExpression($View);
        my $FromEx = $NewSQLStruct->getViewFromExpression($View);
        my $WhereEx = $NewSQLStruct->getViewWhereExpression($View);

        #generate add code
        $GeneratedCode .=
            "\n" .
            "        #add new version of view\n" .
            "        \$self->logInfo(\"add new version of view '$View'\");\n" .
            "        \$dbi->do(<<'END_SQL');\n" .
            "$CreateEx\n" .
            "$SelectEx\n" .
            "$FromEx\n" .
            "$WhereEx\n" .
            "END_SQL\n";

    }
    return $GeneratedCode;
}

1;
