#==============================================================================
# package      DE_EPAGES::Installer::API::Service
#---------------------------------------------------------------------------------
# description  Functions for starting/stopping epages services.
#---------------------------------------------------------------------------------
# state        public
#==============================================================================
package DE_EPAGES::Installer::API::Service;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    StopService
    StartService
    ExistsService
    StatusService
    StatusWin32Service
    StatusUnixService
    EpagesCustomizedMysqlDaemon
    EpagesCustomizedHttpdDaemon
);

use 5.010;
use Sort::Versions qw(versioncmp);
use File::Basename  qw ( basename );

use DE_EPAGES::Core::API::Constants qw( SECONDS_PER_MINUTE );
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::PerlTools qw( ElementOf UsePackage );
use DE_EPAGES::Core::API::SystemCommand qw( ShellCommand SystemCommand );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Installer::API::Constants qw(
    STOPPED
    PENDING
    RUNNING
    DENIED
    MISSING

    START_PENDING
    STOP_PENDING
    CONTINUE_PENDING
    PAUSE_PENDING
    PAUSED
);

BEGIN {
    if ( $^O eq 'MSWin32' ) {
        UsePackage( 'Win32' );
        UsePackage( 'Win32::Service' );
    }
}

sub ErrorDefinition () { return {
    'ServiceStartFailed' => 'Can\'t start "#Command": #Message',
    'ServiceDied'        => 'Process "#Command" died with signal #Signal',
    'ServiceFailed'      => 'Process "#Command" failed with exit code #ExitCode',
    'ServiceTimeout'     => 'Service #Service: timeout after #Timeout seconds',
    'Win32Error'         => 'Service #Service: #Message (0x#ErrorCode)',
    'ServiceMissing'     => 'Service not installed: #Service',
    'UnknownService'     => 'Unknown service: #Service',
    'UnknownEpagesService' => 'Service #Service not found in #RunScript',
    'FileNotFound'       => 'Service #Service: file not found: #File',
    'FileNotExecutable'  => 'Service #Service: file not executable: #File',
    'CommandError'       => 'System command "#Command" failed: #Error',
    'CommandSignal'      => 'System command "#Command" terminated with signal #Signal',
    'UnsupportedOS'      => 'Unsupported OS: #WrongOS; #Func runs only on: #RightOS',
    'EnvironmentMissing' => 'Environment variable not found: #Env',
                         }
}

my %WIN_SERVICE = (
    'appserver'    => 'EPAGES',
    'reqrouter'    => 'EPAGES_RR',
    'webserver'    => 'W3SVC',
    'httpd'        => 'W3SVC',
    'sslwebserver' => 'W3SVC', # legacy
    'dbserver'     => 'MySQL',
    'cron'         => 'Schedule',
    'solr-server'  =>  'solr-server' ,
    'solr-server-slave' =>  'solr-server-slave',
    'epagesj-server' => 'epagesj-server',
    'mongodb'      => 'MongoDB',
    'ascacheserver' => 'ASPoolDBCacheServer',
);

my %WIN_STATUS = (
    STOPPED() => STOPPED,
    START_PENDING() => PENDING,
    STOP_PENDING() => STOPPED,
    RUNNING() => RUNNING,
    CONTINUE_PENDING() => STOPPED,
    PAUSE_PENDING() => STOPPED,
    PAUSED() => STOPPED,
);

my %UNIX_SERVICE = (
    'appserver'      => 'service',
    'reqrouter'      => 'rr',
    'webserver'      => '/etc/init.d/WEB',
    'httpd'          => 'httpd',
    'sslwebserver'   => '/etc/init.d/WEB', # legacy
    'cron'           => 'cron',
    'dbserver'       => '/etc/init.d/DB',
    'solr-server'    =>  '/etc/init.d/epages-solr',
    'solr-server-slave' =>  '/etc/init.d/epages-solr-slave',
    'epagesj-server' => '/etc/init.d/epagesj',
    'mongodb'        => EpagesCustomizedMongoDb(),
    'ascacheserver'  => 'asc',
);

my %UNIX_STATUS = (
    0 => RUNNING,  # 0 - success
    1 => STOPPED,  # 1 - dead but pid file exists
    2 => STOPPED,  # 2 - dead but subsys locked
    3 => STOPPED,  # 3 - stopped
    4 => DENIED,   # 4 - insufficient privilege
    5 => MISSING,  # 5 - program is not installed
    6 => MISSING,  # 6 - program is not configured
    7 => STOPPED,  # 7 - program is not running
);

my %EPAGES_SERVICE = (
    'service' => 1,
    'rr'      => 1,
    'asc'     => 1,
    'httpd'   => 1,
    'j'       => 1,
    'java'    => 1,
    'cron'    => 1,
    'db'      => 1,
);

sub SCHEDULER_CMD() {
    return "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/bin/epagesScheduler.sh";
}

sub StopService ($;$); # forward declaration
#========================================================================================
# function     StopService
#----------------------------------------------------------------------------------------
# syntax       StopService( $ServiceAlias, $hFlags );
# example      StopService( 'webserver' );
#----------------------------------------------------------------------------------------
# description  Stops one of the epages services.
#----------------------------------------------------------------------------------------
# input        $ServiceAlias | service name, one of
#                 <ul>
#                 <li>epages - 'appserver', 'reqrouter', 'ascacheserver'</li>
#                   <li>appserver - epages application service</li>
#                   <li>reqrouter - epages request router</li>
#                   <li>ascacheserver - epages aspooldbcacheserver</li>
#                 <li>webserver - web server</li>
#                 <li>dbserver - database server</li>
#                 <li>cron - scheduled tasks</li>
#                 <li>epagesj - 'mongodb', 'epagesj-server', 'solr-server', 'solr-server-slave'</li>
#                   <li>mongodb - Mongo DB</li>
#                   <li>epagesj-server - epages java server</li>
#                   <li>solr-server - solr server</li>
#                   <li>solr-server-slave - solr server (slave)</li>
#                 </ul>
#               | string
# input        $hFlags | flags
#                 <ul>
#                   <li>verbose - verbose
#                   <li>quiet - no output
#                   <li>dieIfMissing - throws error if status is MISSING
#                 </ul>
#               | ref.hash
#========================================================================================
sub StopService ($;$) {
    my ($ServiceAlias, $hFlags) = @_;
    TestString( 'ServiceAlias', $ServiceAlias );

    if( $ServiceAlias eq 'epages' ) {
        StopService('appserver', $hFlags);
        StopService('reqrouter', $hFlags) if ExistsService('reqrouter');
        StopService('ascacheserver', $hFlags) if ExistsService('ascacheserver');
        return;
    } elsif ( $ServiceAlias eq 'epagesj' ) {
        StopService('epagesj-server', $hFlags);
        StopService('solr-server-slave', $hFlags) if ExistsService('solr-server-slave');
        StopService('solr-server', $hFlags);
        StopService('mongodb', $hFlags);
        return;
    }

    #-------------------------------------------------------------------
    # stop cron service
    #-------------------------------------------------------------------
    if ( $ServiceAlias eq 'cron' ) {
        return unless ExistsService('cron');
        SystemCommand([SCHEDULER_CMD, 'stop'],
                      {Print => $hFlags->{verbose}, Capture => $hFlags->{quiet}});
        return;
    }

    my $hStatus = StatusService($ServiceAlias, $hFlags);
    return if $hStatus->{$ServiceAlias} == MISSING;
    return if $hStatus->{$ServiceAlias} == STOPPED;

    my $Service = $^O eq 'MSWin32' ? $WIN_SERVICE{$ServiceAlias} : $UNIX_SERVICE{$ServiceAlias};

    #-------------------------------------------------------------------
    # stop windows service
    #-------------------------------------------------------------------
    if ($^O eq 'MSWin32') {
        if ($ServiceAlias =~ /solr-server/) {
            # solr-server < 6.15.0 does not stop with Win32::Service::StopService
            # bug EPG-24509
            UsePackage('DE_EPAGES::Installer::API::EpagesJ');
            if (versioncmp(DE_EPAGES::Installer::API::EpagesJ::GetEpagesJVersion(), '6.15.0') < 0) {
                SystemCommand("net stop $ServiceAlias",
                              {Print => $hFlags->{verbose}, Capture => $hFlags->{quiet}});
                return;
            }
        }
        print "Win32::Service::StopService($Service)\n" if $hFlags->{verbose};

        # bug EPG-28506: mongodb sometimes dies on stopping
        my $ErrorLevel = $ServiceAlias eq 'mongodb' ? 'Warning' : 'Error';
        Win32::Service::StopService( '', $Service ) or
            _win32Error("Win32::Service::StopService($Service)", $ErrorLevel);
        _waitWin32Service( $Service, STOPPED );
        return;
    }

    #-------------------------------------------------------------------
    # stop unix service
    #-------------------------------------------------------------------
    _RunUnixService($Service, 'stop', $hFlags);
}

sub StartService ($;$); # forward declaration
#========================================================================================
# function     StartService
#----------------------------------------------------------------------------------------
# syntax       StartService( $ServiceAlias, $hFlags );
# example      StartService( 'webserver' );
#----------------------------------------------------------------------------------------
# description  Starts one of the epages services.
#               Stops and restarts the service if it is already running.
#----------------------------------------------------------------------------------------
# input        $ServiceAlias | service name, one of
#                 <ul>
#                 <li>epages - 'appserver', 'reqrouter', 'ascacheserver'</li>
#                   <li>appserver - epages application service</li>
#                   <li>reqrouter - epages request router</li>
#                   <li>ascacheserver - epages aspooldbcacheserver</li>
#                 <li>webserver - web server</li>
#                 <li>dbserver - database server</li>
#                 <li>cron - scheduled tasks</li>
#                 <li>epagesj - 'mongodb', 'epagesj-server', 'solr-server', 'solr-server-slave'</li>
#                   <li>mongodb - Mongo DB</li>
#                   <li>epagesj-server - epages java server</li>
#                   <li>solr-server - solr server</li>
#                   <li>solr-server-slave - solr server (slave)</li>
#                 </ul>
#               | string
# input        $hFlags | flags
#                 <ul>
#                   <li>dieIfMissing - throws error if status is MISSING
#                   <li>verbose - verbose
#                   <li>quiet - no output
#                 </ul>
#               | ref.hash
#========================================================================================
sub StartService ($;$) {
    my ($ServiceAlias, $hFlags) = @_;
    TestString( 'ServiceAlias', $ServiceAlias );

    if( $ServiceAlias eq 'epages' ) {
        StartService('ascacheserver', $hFlags) if ExistsService('ascacheserver');
        StartService('reqrouter', $hFlags) if ExistsService('reqrouter');
        StartService('appserver', $hFlags);
        return;
    } elsif ( $ServiceAlias eq 'epagesj' ) {
        StartService('mongodb', $hFlags);
        StartService('solr-server', $hFlags);
        StartService('solr-server-slave', $hFlags) if ExistsService('solr-server-slave');
        StartService('epagesj-server', $hFlags);
        return;
    }

    #-------------------------------------------------------------------
    # start cron service
    #-------------------------------------------------------------------
    if ( $ServiceAlias eq 'cron' ) {
        return unless ExistsService('cron');
        SystemCommand([SCHEDULER_CMD, 'start'],
                      {Print => $hFlags->{verbose}, Capture => $hFlags->{quiet}});
        return;
    }

    #-------------------------------------------------------------------
    # set unix permissions in $EPAGES
    #-------------------------------------------------------------------
    if ( $ServiceAlias =~ /^perm/) {
        return if $^O eq 'MSWin32';
        my $EpagesProject = $ENV{EPAGES_PROJECT} // 'epages6';
        $EpagesProject = 'epages6' unless $EpagesProject =~ /\S/;
        SystemCommand("/etc/init.d/$EpagesProject $ServiceAlias",
                      {Print => $hFlags->{verbose}, Capture => $hFlags->{quiet}});
        return;
    }

    return unless ExistsService($ServiceAlias, $hFlags);
    StopService( $ServiceAlias, $hFlags );

    my $Service = $^O eq 'MSWin32' ? $WIN_SERVICE{$ServiceAlias} : $UNIX_SERVICE{$ServiceAlias};

    #-------------------------------------------------------------------
    # start windows service
    #-------------------------------------------------------------------
    if ($^O eq 'MSWin32') {
        print "Win32::Service::StartService($Service)\n" if $hFlags->{verbose};
        Win32::Service::StartService( '', $Service ) or
            _win32Error("Win32::Service::StartService($Service)");
        _waitWin32Service( $Service, RUNNING );
        return;
    }

    #-------------------------------------------------------------------
    # start unix service
    #-------------------------------------------------------------------
    _RunUnixService($Service, 'start', $hFlags);
}

sub ExistsService ($;$); # forward declaration
#========================================================================================
# function     ExistsService
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsService( $ServiceAlias );
# example      StopService( 'webServer' ) if ExistsService( 'webserver' );
#----------------------------------------------------------------------------------------
# description  Returns true if the specified service exists on the current machine.
#----------------------------------------------------------------------------------------
# input        $ServiceAlias | service name, one of
#                 <ul>
#                 <li>epages - 'appserver', 'reqrouter'</li>
#                   <li>appserver - epages application service</li>
#                   <li>reqrouter - epages request router</li>
#                 <li>webserver - web server</li>
#                 <li>dbserver - database server</li>
#                 <li>cron - scheduled tasks</li>
#                 <li>epagesj - 'mongodb', 'epagesj-server', 'solr-server', 'solr-server-slave'</li>
#                   <li>mongodb - Mongo DB</li>
#                   <li>epagesj-server - epages java server</li>
#                   <li>solr-server - solr server</li>
#                   <li>solr-server-slave - solr server (slave)</li>
#                 </ul>
#               | string
# input        $hFlags | flags
#                 <ul>
#                   <li>dieIfMissing - throws error if not existing
#                 </ul>
#               | ref.hash
#----------------------------------------------------------------------------------------
# return       $Exists | returns true if the service exists | boolean
#========================================================================================
sub ExistsService ($;$) {
    my ($ServiceAlias, $hFlags) = @_;
    TestString( 'ServiceAlias', $ServiceAlias );

    if ( $ServiceAlias eq 'epages' ) {
        return ExistsService('appserver') || ExistsService('reqrouter') || ExistsService('ascacheserver');
    } elsif ( $ServiceAlias eq 'epagesj') {
        return ExistsService('solr-server') || ExistsService('epagesj-server');
    }

    my $hStatus = StatusService( $ServiceAlias, $hFlags );
    return $hStatus->{$ServiceAlias} != MISSING;
}

sub StatusService ($;$); # forward declaration
#========================================================================================
# function     StatusService
#----------------------------------------------------------------------------------------
# syntax       $hStatus = StatusService( $ServiceAlias, $hFlags );
# example      $hStatus = StatusService( 'webserver' );
#               use DE_EPAGES::Installer::API::Constants qw(RUNNING);
#               say 'webserver: running' if $hStatus->{webserver} == RUNNING
#----------------------------------------------------------------------------------------
# description  Shows current status of the epages service.
#----------------------------------------------------------------------------------------
# input        $ServiceAlias | service name, one of
#                 <ul>
#                 <li>epages - 'appserver', 'reqrouter', 'ascacheserver'</li>
#                   <li>appserver - epages application service</li>
#                   <li>reqrouter - epages request router</li>
#                   <li>ascacheserver - epages aspooldbcacheserver</li>
#                 <li>webserver - web server</li>
#                 <li>dbserver - database server</li>
#                 <li>cron - epages scheduler: if section [LOOPS] in Scheduler.conf
#                     has IsActive=0 (global) then STOPPED else RUNNING</li>
#                 <li>epagesj - 'mongodb', 'epagesj-server', 'solr-server', 'solr-server-slave'</li>
#                   <li>mongodb - Mongo DB</li>
#                   <li>epagesj-server - epages java server</li>
#                   <li>solr-server - solr server</li>
#                   <li>solr-server-slave - solr server (slave)</li>
#                 </ul>
#               | string
# input        $hFlags | flags
#                 <ul>
#                   <li>dieIfMissing - throws error if status is MISSING
#                 </ul>
#               | ref.hash
#----------------------------------------------------------------------------------------
# return       $hStatus | hash list, key is service, value is status, one of (see Constants):
#                 STOPPED => 1   - service is stopped
#                                  windows: any status but SERVICE_START_PENDING/SERVICE_RUNNING
#                 PENDING => 2   - service is starting
#                                  windows: SERVICE_START_PENDING; UNIX: not implemented
#                 RUNNING => 4   - service is running
#                                  windows: SERVICE_RUNNING
#                 DENIED  => 8   - status unknown due to insufficient privileges
#                 MISSING => 16  - service not installed
#               | ref.hash
#========================================================================================
sub StatusService ($;$) {
    my ($ServiceAlias, $hFlags) = @_;
    TestString( 'ServiceAlias', $ServiceAlias );

    my $hStatus;

    if( $ServiceAlias eq 'epages' ) {
        $hStatus = StatusService('reqrouter', $hFlags) if ExistsService('reqrouter');
        $hStatus = { %$hStatus, %{StatusService('appserver', $hFlags)} };
        $hStatus = { %$hStatus, %{StatusService('ascacheserver', $hFlags)} } if ExistsService('ascacheserver');
        return $hStatus;
    } elsif ( $ServiceAlias eq 'epagesj' ) {
        $hStatus = StatusService('mongodb', $hFlags);
        $hStatus = { %$hStatus, %{StatusService('solr-server', $hFlags)} };
        $hStatus = { %$hStatus, %{StatusService('solr-server-slave', $hFlags)} } if ExistsService('solr-server-slave');
        $hStatus = { %$hStatus, %{StatusService('epagesj-server', $hFlags)} };
        return $hStatus;
    }

    my $Service = $^O eq 'MSWin32' ? $WIN_SERVICE{$ServiceAlias} : $UNIX_SERVICE{$ServiceAlias};
    Error( 'UnknownService', {Service => $ServiceAlias} ) unless $Service;

    my $Status = MISSING;
    if ($ServiceAlias eq 'cron') {
        $Status = _StatusCronService($hFlags);
    } elsif ($^O eq 'MSWin32') {
        $Status = StatusWin32Service($Service, $hFlags);
    } elsif ($Service eq '/etc/init.d/WEB' and -x "$ENV{EPAGES_APACHE}/bin/httpd" ) {
        $Status = _StatusEpagesService('httpd', $hFlags);
    } elsif ($Service =~ m,^/etc/init.d/,) {
        $Status = StatusUnixService(basename($Service), $hFlags);
    } else {
        $Status = _StatusEpagesService($Service, $hFlags);
        $Status = StatusUnixService(basename($Service), $hFlags) if $Status == DENIED;
    }

    return { $ServiceAlias => $Status };
}

#========================================================================================
# function     StatusWin32Service
#----------------------------------------------------------------------------------------
# syntax       $Status = StatusWin32Service($Service, $hFlags);
# example      $Status = StatusWin32Service( 'W3SVC' );
#               use DE_EPAGES::Installer::API::Constants qw(RUNNING);
#               say 'webserver: running' if $Status == RUNNING
#----------------------------------------------------------------------------------------
# description  Shows current status of the windows service.
#----------------------------------------------------------------------------------------
# input        $Service | service name as defined in Service -> Properties | string
# input        $hFlags  | flags
#                 <ul>
#                   <li>dieIfMissing - throws error if status is MISSING
#                 </ul>
#               | ref.hash
#----------------------------------------------------------------------------------------
# return       $Status | returned status, one of (see DE_EPAGES::Installer::API::Constants):
#                 STOPPED => 1   - any status but SERVICE_START_PENDING/SERVICE_RUNNING
#                 PENDING => 2   - SERVICE_START_PENDING
#                 RUNNING => 4   - SERVICE_RUNNING
#                 MISSING => 16  - service not installed
#               | integer
#========================================================================================
sub StatusWin32Service {
    my ($Service, $hFlags) = @_;

    Error('UnsupportedOS', { WrongOS => $^O, RightOS => 'MSWin32',
                             Func => 'StatusWin32Service' }) if $^O ne 'MSWin32';

    my %WinStatus;
    my $Result = Win32::Service::GetStatus('', $Service, \%WinStatus);

    _win32Error("Win32::Service::GetStatus($Service)")
        if $hFlags->{dieIfMissing} and not $Result;
    Error('ServiceMissing', {Service => $Service })
        if $hFlags->{dieIfMissing} and not defined $WinStatus{CurrentState};

    return MISSING unless defined $WinStatus{CurrentState};
    return $WIN_STATUS{$WinStatus{CurrentState}};
}

#========================================================================================
# function     StatusUnixService
#----------------------------------------------------------------------------------------
# syntax       $Status = StatusUnixService($Service, $hFlags);
# example      $Status = StatusUnixService( 'httpd' );
#               use DE_EPAGES::Installer::API::Constants qw(RUNNING);
#               say 'webserver: running' if $Status == RUNNING
#----------------------------------------------------------------------------------------
# description  Shows current status of run script, located in /etc/init.d/$Service.
#----------------------------------------------------------------------------------------
# input        $Service | service located in /etc/init.d/$Service
#                          'WEB' is an alias for 'apache2' (SuSE) or 'httpd' (Redhat)
#                          'DB' is an alias for 'mysql' (SuSE) or 'mysqld' (Redhat) | string
# input        $hFlags  | flags
#                 <ul>
#                   <li>dieIfMissing - throws error if status is MISSING
#                 </ul>
#               | ref.hash
#----------------------------------------------------------------------------------------
# return       $Status | returned status, one of (see DE_EPAGES::Installer::API::Constants):
#                 STOPPED => 1   - service is stopped
#                 RUNNING => 4   - service is running
#                 DENIED  => 8   - status unknown due to insufficient privileges
#                 MISSING => 16  - service not installed
#               | integer
#========================================================================================
sub StatusUnixService {
    my ($Service, $hFlags) = @_;

    Error('UnsupportedOS', { WrongOS => $^O, RightOS => 'Linux',
                             Func => 'StatusUnixService' }) if $^O eq 'MSWin32';

    my $RunScript = $Service;
    $RunScript = "/etc/init.d/$Service" if -x "/etc/init.d/$Service";

    if ( $Service eq 'DB' ) {
        $RunScript = EpagesCustomizedMysqlDaemon($hFlags);
        return MISSING unless $RunScript;
    } elsif ( $Service eq 'WEB' ) {
        $RunScript = EpagesCustomizedHttpdDaemon($hFlags);
        return MISSING unless $RunScript;
    } elsif ( $Service eq 'httpd' ) {
        my $HttpdDaemon = EpagesCustomizedHttpdDaemon($hFlags);
        $RunScript = $HttpdDaemon if $HttpdDaemon;
    }

    if ($RunScript =~ /\// and not -x $RunScript) {
        Error( 'FileNotExecutable', {Service => $Service, File => $RunScript} )
            if $hFlags->{dieIfMissing};
        return MISSING;
    }
    $RunScript = "service $RunScript" if $RunScript !~ /\//;
    my $Result = system("$RunScript status >/dev/null 2>&1");

    ## no critic qw(ProhibitMagicNumbers)
    if ( $Result == -1 ) {
        Error( 'CommandError', {Command => "$RunScript status", Error => $!} )
            if $hFlags->{dieIfMissing};
        return MISSING;
    } elsif ($Result & 127) {
        Error( 'CommandSignal', {Command => "$RunScript status", Signal => ($Result & 127)} )
            if $hFlags->{dieIfMissing};
        return MISSING;
    }
    ## use critic

    return $UNIX_STATUS{$Result >> 8};
}

#========================================================================================
# function     EpagesCustomizedMysqlDaemon
#----------------------------------------------------------------------------------------
# syntax       $RunScript = EpagesCustomizedMysqlDaemon($hFlags);
# example      $RunScript = EpagesCustomizedMysqlDaemon();
#               if ($RunScript) {
#                   system("$RunScript restart");
#               } else {
#                   say 'ePages customized MySQL daemon not installed";
#               }
#----------------------------------------------------------------------------------------
# description  Returns name of MySQL run script located in /etc/init.d if MySQL daemon
#               exists, otherwise undef.
#----------------------------------------------------------------------------------------
# input        $hFlags  | flags
#                 <ul>
#                   <li>dieIfMissing - throws error if no ePages customized daemon found
#                 </ul>
#               | ref.hash
#----------------------------------------------------------------------------------------
# return       $RunScript | name of MySQL run script: /etc/init.d/mysql (Debian) or
#                            /etc/init.d/mysqld (Redhat) | string
#========================================================================================
sub EpagesCustomizedMysqlDaemon {
    my ($hFlags) = @_;

    my $MySQLDaemonName = ShellCommand('. /etc/default/epages6 && mysqld_name', { 'Capture' => 1 });
    chomp($MySQLDaemonName);

    return _GetDaemon($MySQLDaemonName, $hFlags);
}

#========================================================================================
# function     EpagesCustomizedMongoDb
#----------------------------------------------------------------------------------------
# syntax       $RunScript = EpagesCustomizedMongoDb($hFlags);
# example      $RunScript = EpagesCustomizedMongoDb();
#               if ($RunScript) {
#                   system("$RunScript restart");
#               } else {
#                   say 'ePages customized MongoDB daemon not installed";
#               }
#----------------------------------------------------------------------------------------
# description  Returns the name of the MongoDB run script located in /etc/init.d if
#               MongoDB daemon exists, otherwise undef.
#----------------------------------------------------------------------------------------
# input        $hFlags  | flags
#                 <ul>
#                   <li>dieIfMissing - throws error if no ePages customized daemon found
#                 </ul>
#               | ref.hash
#----------------------------------------------------------------------------------------
# return       $RunScript | name of MongoDB run script: /etc/init.d/mongodb (Debian) or
#                            /etc/init.d/mysqld (Redhat) | string
#========================================================================================
sub EpagesCustomizedMongoDb {
    my ($hFlags) = @_;

    my $MongoDbDaemonName = ShellCommand('. /etc/default/epages6 && mongod_name', { 'Capture' => 1 });
    chomp($MongoDbDaemonName);

    return _GetDaemon($MongoDbDaemonName, $hFlags);
}

# Returns the service run script for the given service name
sub _GetDaemon {
    my ($DaemonName, $hFlags) = @_;

    my $Daemon = "/etc/init.d/$DaemonName";

    if (-x $Daemon) {
        return $Daemon;
    } elsif (-f "/usr/lib/systemd/system/$DaemonName.service") {
        return $DaemonName;
    } elsif ($hFlags->{dieIfMissing}) {
        Error( 'FileNotFound', {Service => $DaemonName, File => $Daemon} );
    } else {
        return undef;
    }
}

#========================================================================================
# function     EpagesCustomizedHttpdDaemon
#----------------------------------------------------------------------------------------
# syntax       $RunScript = EpagesCustomizedHttpdDaemon($hFlags);
# example      $RunScript = EpagesCustomizedHttpdDaemon();
#               if ($RunScript) {
#                   system("$RunScript restart");
#               } else {
#                   say 'ePages customized httpd demaon not installed";
#               }
#----------------------------------------------------------------------------------------
# description  Returns name of httpd run script located in /etc/init.d if httpd daemon
#               exists and is ePages customized, otherwise undef.
#----------------------------------------------------------------------------------------
# input        $hFlags  | flags
#                 <ul>
#                   <li>dieIfMissing - throws error if no ePages customized daemon found
#                 </ul>
#               | ref.hash
#----------------------------------------------------------------------------------------
# return       $RunScript | name of httpd run script: /etc/init.d/apache2 (SuSE) or
#                            /etc/init.d/httpd (Redhat) | string
#========================================================================================
sub EpagesCustomizedHttpdDaemon {
    my ($hFlags) = @_;

    Error('UnsupportedOS', { WrongOS => $^O, RightOS => 'Linux',
                             Func => 'EpagesCustomizedHttpdDaemon' }) if $^O eq 'MSWin32';

    # test if native httpd is installed
    my $Httpd;
    $Httpd = 'apache2' if system('rpm -q --quiet apache2 2>/dev/null') == 0;
    $Httpd = 'httpd' if system('rpm -q --quiet httpd 2>/dev/null') == 0;
    $Httpd = 'apache2-epages' if -x '/etc/init.d/apache2-epages';

    if (not $Httpd) {
        Error( 'CommandError', {Command => 'rpm -q httpd',
                                Error => 'package httpd is not installed'} )
            if $hFlags->{dieIfMissing};
        return;
    }

    my $EpagesConf;
    foreach ( qw(conf.d vhosts.d extra) ) {
        $EpagesConf = "/etc/$Httpd/$_/zzz-epages-httpd.conf"
            if -f "/etc/$Httpd/$_/zzz-epages-httpd.conf";
    }

    if (not $EpagesConf) {
        Error( 'FileNotFound', {Service => $Httpd,
                                File => "/etc/$Httpd/conf.d/zzz-epages-httpd.conf"} )
            if $hFlags->{dieIfMissing};
        return;
    }

    $Httpd = 'httpd-epages' if -x '/etc/init.d/httpd-epages';

    # returns either apache2 (SuSE) or httpd (Redhat)
    return -x "/etc/init.d/$Httpd" ? "/etc/init.d/$Httpd" : undef;
}

#========================================================================================
# auxiliary functions
#========================================================================================

# stops or starts a unix service
sub _RunUnixService {
    my ($Service, $Task, $hFlags) = @_;

    my $RunScript = $Service;
    $RunScript = -x "$ENV{EPAGES_APACHE}/bin/httpd" ? 'httpd' : EpagesCustomizedHttpdDaemon()
        if $Service eq '/etc/init.d/WEB';
    $RunScript = EpagesCustomizedMysqlDaemon() if $Service eq '/etc/init.d/DB';

    return SystemCommand([$RunScript, $Task],
                         {Print => $hFlags->{verbose}, Capture => $hFlags->{quiet}})
        if $RunScript =~ m,^/etc/init.d/,;

    $Service = 'db' if $RunScript =~ /mysql/i;
    $Service = 'httpd' if $RunScript =~ /http|apache/i;

    return SystemCommand(['service', $RunScript, $Task],
                         {Print => $hFlags->{verbose}, Capture => $hFlags->{quiet}})
        if $RunScript !~ m,/, and not defined $EPAGES_SERVICE{$Service};

    my $EpagesProject = $ENV{EPAGES_PROJECT} // 'epages6';
    $EpagesProject = 'epages6' unless $EpagesProject =~ /\S/;

    local $ENV{EPAGES_IGNORE} = 'cron';
    local $ENV{IGNORE_INI} = 1;
    SystemCommand(["/etc/init.d/$EpagesProject", "${Task}_$Service"],
                  {Print => $hFlags->{verbose}, Capture => $hFlags->{quiet}});

    return;
}

# get status of cron service
sub _StatusCronService {
    my ($hFlags) = @_;
    my $SchedulerConf = "$ENV{EPAGES_CONFIG}/Scheduler.conf";
    my $MissingFile = -f SCHEDULER_CMD ? undef : SCHEDULER_CMD;
    $MissingFile //= -f $SchedulerConf ? undef : $SchedulerConf;

    if (not $MissingFile and $^O ne 'MSWin32') {
        my $Crond = '/etc/init.d/cron';
        $Crond .= 'd' if -f '/etc/redhat-release';
        my $SystemdCrond = '/usr/lib/systemd/system/crond.service';
        $MissingFile = $Crond if not -f $Crond and not -f $SystemdCrond;
    } elsif (not $MissingFile and system('schtasks /? > nul 2>&1') != 0) {
        $MissingFile = 'schtasks';
    }

    if ($MissingFile) {
        Error( 'FileNotFound', {Service => 'cron', File => $MissingFile} )
            if $hFlags->{dieIfMissing};
        return MISSING;
    }

    return StatusWin32Service($WIN_SERVICE{cron}, $hFlags) if $^O eq 'MSWin32';

    my $IniConfig = DE_EPAGES::Core::API::IniConfig->new(FileName => $SchedulerConf, CaseSensitive => 1);
    my $Section = ElementOf('LOOPS', $IniConfig->sectionNames) ? 'LOOPS' : ':Default:';
    my $IsActive = $IniConfig->get($Section, 'IsActive') // 1;
    $IsActive = $IniConfig->get($Section, 'IsActive') // 1;
    return $IsActive ? RUNNING : STOPPED;
}

# get status of unix epages daemon
sub _StatusEpagesService ($;$) {
    my ($Service, $hFlags) = @_;

    Error('UnsupportedOS', { WrongOS => $^O, RightOS => 'Linux',
                             Func => '_StatusEpagesService' }) if $^O eq 'MSWin32';

    my $EpagesProject = $ENV{EPAGES_PROJECT};
    $EpagesProject = 'epages6' unless $EpagesProject =~ /\S/;

    my $RunScript = "/etc/default/$EpagesProject";

    if (not -x $RunScript) {
        Error( 'FileNotExecutable', {Service => $Service, File => $RunScript} )
            if $hFlags->{dieIfMissing};
        return MISSING;
    }

    my ($User, $Env, $Cmd);
    if ($Service eq 'service') {
        $User = $ENV{EPAGES_APPUSER} // 'eprunapp';
        $Env = 'EPSERVICE';
        $Cmd = $ENV{$Env};
    }
    elsif ($Service eq 'asc') {
        $User = $ENV{EPAGES_APPUSER} // 'eprunapp';
        $Env = 'EP_ASC';
        $Cmd = $ENV{$Env};
    }
    elsif ($Service eq 'rr') {
        $User = $ENV{EPAGES_APPUSER} // 'eprunapp';
        $Env = 'EP_RR';
        $Cmd = $ENV{$Env};
    }
    elsif ($Service eq 'httpd') {
        $User = $ENV{EPAGES_WEBUSER} // 'eprunweb';
        if (-f '/etc/init.d/apache2-epages') {
            $Env = 'EPAGES_WEBUSER';
            $Cmd = '/usr/sbin/httpd';
        } else {
            $Env = 'EPAGES_APACHE';
            $Cmd = "$ENV{$Env}/bin/httpd";
        }
    }
    else {
        return DENIED;
    }

    if (not $ENV{$Env}) {
        Error('EnvironmentMissing', {Env => $Env}) if $hFlags->{dieIfMissing};
        return MISSING;
    }

    if (not -x $Cmd) {
        Error( 'FileNotExecutable', {Service => $Service, File => $Cmd} )
            if $hFlags->{dieIfMissing};
        return MISSING;
    }

    my $Output = `/bin/bash -c '. "$RunScript" && get_pids -u $User -c "$Cmd"'`;
    my $Result = $?;

    ## no critic qw(ProhibitMagicNumbers)
    if ( $Result == -1 ) {
        Error( 'CommandError', {Command => "$RunScript status", Error => $!} )
            if $hFlags->{dieIfMissing};
        return MISSING;
    } elsif ($Result & 127) {
        Error( 'CommandSignal', {Command => "$RunScript status", Signal => ($Result & 127)} )
            if $hFlags->{dieIfMissing};
        return MISSING;
    }
    ## use critic

    return $Output ? RUNNING : STOPPED;
}

# creates a Win32 error message using GetLastError amd FormatMessage
sub _win32Error {
    my ($WinService, $ErrorLevel) = @_;
    $ErrorLevel //= 'Error';

    my $ErrorCode = Win32::GetLastError();
    return unless $ErrorCode;

    my $Message = Win32::FormatMessage($ErrorCode);
    $Message =~ s/\s+$//;
    if ($ErrorLevel eq 'Error') {
        Error( 'Win32Error', {
            'Message' => $Message,
            'ErrorCode' => sprintf('%08x', $ErrorCode),
            'Service' => $WinService,
        });
    } else {
        Warning( 'Win32Error', {
            'Message' => $Message,
            'ErrorCode' => sprintf('%08x', $ErrorCode),
            'Service' => $WinService,
        });
    }
}

# wait until the service reaches the desired status
# as defined in DE_EPAGES::Installer::API::Constants
sub _waitWin32Service {
    my ($WinService, $Status) = @_;

    my %WinStatus;
    my $Counter = 0;
    while (1) {
        Win32::Service::GetStatus('', $WinService, \%WinStatus);
        return if defined $WinStatus{'CurrentState'} && $WinStatus{'CurrentState'} == $Status;
        sleep 1;
        $Counter++;
        Error('ServiceTimeout',
              {'Service' => $Status == 1 ? "stopping $WinService" : "starting $WinService",
               'Timeout' => 3*SECONDS_PER_MINUTE})
            if $Counter > 3*SECONDS_PER_MINUTE;
    }
    return;
}

1;
