use strict;
use Getopt::Long;
use Time::HiRes qw(time);
use File::Basename qw ( basename );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw(SystemCommand);
use DE_EPAGES::Core::API::File qw( CreateDirectory DeleteDirectory GetFileContentLines );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

sub getLatestSnapshot {
    my ($VG, $LV) = @_;
    my ($LatestSnapshot, $LatestTimeStamp);

    foreach my $Device ( glob "/dev/$VG/$VG-$LV-*" ) {
        my $TimeStamp = $Device =~ /.*-([0-9]+)$/x ? $1 : '';
        $LatestTimeStamp = $TimeStamp unless $LatestTimeStamp;
        $LatestSnapshot = $Device if $TimeStamp >= $LatestTimeStamp;
    }

    unless (-e "$LatestSnapshot") {
        print "\nError: No such device (VG=$VG LV=$LV): /dev/$VG/$VG-$LV-*\n\n";
        usage();
    }

    return basename($LatestSnapshot);
}

sub findDataDir {
    my ($VG, $LV) = @_;

    unless (-e "/dev/$VG/$LV") {
        print "\nError: No such device (VG=$VG LV=$LV): /dev/$VG/$LV\n\n";
        usage();
    }

    my $DataDir;
    my @Files = split(/\s+/, `find /dev/$VG/$LV -printf '%l %p'`);
    if ($? == 0 && -f '/etc/mtab') {
        foreach my $Line (@{GetFileContentLines('/etc/mtab')}) {
            foreach my $File (@Files) {
                my @Column = split(/\s+/, $Line);
                $DataDir = $Column[1] if ElementOf($Column[0], @Files);
                last if $DataDir;
            }
            last if $DataDir;
        }
    }

    unless (-e "$DataDir") {
        print "\nError: Device not mounted: /dev/$VG/$LV\n\n";
        usage();
    }

    return $DataDir;
}

sub mountSnapshot {
    my ( $BackupPartition, $BackupMountPoint, $Verbose ) = @_;

    my $params  = 'rw';
    my $Command = "mount -o $params $BackupPartition $BackupMountPoint";
    $Command .= $Verbose ? '' : ' > /dev/null';
    SystemCommand( $Command, { 'Print' => $Verbose } );
}

sub unmountSnapshot {
    my ( $BackupMountPoint, $Verbose ) = @_;

    my $Command = "umount $BackupMountPoint";
    $Command .= $Verbose ? '' : ' > /dev/null';
    SystemCommand( $Command, { 'Print' => $Verbose } );
}

sub removeSnapshot {
    my ( $BackupPartition, $Verbose ) = @_;

    my $Command = "lvremove -f $BackupPartition";
    $Command .= $Verbose ? '' : ' > /dev/null';
    SystemCommand( $Command, { 'Print' => $Verbose } );
}

sub Main {
    local $| = 1;

    my %Options = ();
    my @GetOpt = qw(vg=s lv=s size=s purge help nooutput);

    usage() unless GetOptions(\%Options, @GetOpt);
    usage() if defined $Options{help};
    map { usage() unless defined $Options{$_}; } qw(lv vg);

    my $time1 = time();
    my $Snapshot = scalar @ARGV ? $ARGV[0] : getLatestSnapshot($Options{vg}, $Options{lv});
    my $Verbose = not defined $Options{nooutput};

    # Find directory to backup.
    my $DataDir = findDataDir($Options{vg}, $Options{lv});

    # Create mountpoint.
    my $BackupMountPoint = "/boot/.$Snapshot.~$$~";
    CreateDirectory( $BackupMountPoint );

    # Mount backup filesystem.
    my $BackupPartition = "/dev/$Options{vg}/$Snapshot";
    mountSnapshot( $BackupPartition, $BackupMountPoint, $Verbose );

    # rsync files from snapshot to original mount point.
    my $Command = "rsync -a --delete $BackupMountPoint/ $DataDir";
    SystemCommand( $Command, { 'Print' => $Verbose } );

    # Unmount snapshot and delete mountpoint.
    unmountSnapshot( $BackupMountPoint, $Verbose );
    DeleteDirectory( $BackupMountPoint );

    # Remove snapshot.
    removeSnapshot( $BackupPartition, $Verbose ) if defined $Options{purge};

    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $logmsg  = sprintf( 'need %.3f seconds', $seconds );
    print "\n" . $logmsg . "\n" if $Verbose;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 {options} [flags] [snapshot name]

Options:
    -vg        LVM volume group name (e.g. 'VolGroup00')     ()
    -lv        LVM logical volume name (e.g. 'LogVol00')     ()

Flags:
    -purge     delete snapshot after restore
    -help      show the command line options
    -nooutput  do not print any output

Example:
    perl $0 -vg VolGroup00 -lv LogVol00 -purge MY_SNAPSHOT
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );

1;
