use strict;
use Config;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::CartridgeConfig qw(CartridgeConfig);
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::File qw( CopyDirectory RunOnDirectory );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw(SystemCommand);
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Database::API::Config qw( BusinessUnits );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Installer::API::LanguagePack qw(
    DownloadLanguagePack
    UntarLanguagePack
    DictionaryXmlLocation
    LanguagePackLocation
    GetLanguagesByVersion
    GetLanguageVersionList
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

sub usage {
    print <<END_USAGE;
Description:
    Installs data and files for given languages.

Usage:
    [EPAGES_LANGUAGES="es it fr"] perl $0 [options] [flags]

Options:
    -passwd        database user password            ()
    -storename     name of store                     (default: Store)
    -lang          language code list,
                   use 'any' for any language        (default: EPAGES_LANGUAGES)
    -get           do not install the language
                   pack(s), just fetch pack(s)
                   with this version                 (default: current)

Flags:
    -help       show the command line options
    -nosync     do not send cache updates
    -nooutput   do not print any output

Examples:
    perl $0 -storename Store -lang "da se"
    perl $0 -get "6.12.1 6.12.3" -lang "da se"
    perl $0 -get 6.12 -lang any
    EPAGES_LANGUAGES=any     perl $0 -get 6.12
    EPAGES_LANGUAGES='es it' perl $0 -get 6.12

END_USAGE
    exit 2;
}

sub Main {

    local $| = 1;

    my $Help;
    my $Password;
    my $StoreName;
    my $LanguageCode;
    my $Synchronize = 1;
    my $Verbose = 1;
    my $GetVersion;

    GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'lang=s'        => \$LanguageCode,
        'sync!'         => \$Synchronize,
        'output!'       => \$Verbose,
        'get:s'         => \$GetVersion,
    );

    usage() if $Help;
    usage() if not $LanguageCode and not $ENV{EPAGES_LANGUAGES};

    my $StoreNameOption = $StoreName;
    my @BusinessUnits = keys %{BusinessUnits()};
    my $SiteName = ElementOf('Site', @BusinessUnits) ? 'Site' : '';
    if (not $StoreNameOption) {
        $StoreName = 'Store' if ElementOf('Store', @BusinessUnits) and not $StoreName;
        $StoreName = (grep { $_ ne 'Site'} @BusinessUnits)[0] unless $StoreName;
        $StoreName = $BusinessUnits[0] unless $StoreName;
    }

    if ($ENV{EPAGES_BUSINESS_UNITS}) {
        @BusinessUnits = split(/\s+/, $ENV{EPAGES_BUSINESS_UNITS});
        $SiteName  = '' unless ElementOf($SiteName, @BusinessUnits);
        $StoreName = '' unless $StoreNameOption or ElementOf($StoreName, @BusinessUnits);
    }

    $SiteName = '' if $StoreName eq $SiteName;
    print "SiteName: $SiteName\nStoreName: $StoreName\n" if $Verbose;
    if ( not "$SiteName$StoreName" ) {
        print "(no existing business unit found)\n" if $Verbose;
        return;
    }
    $SiteName .= ' ' if $SiteName;

    my $time1 = time;

    my $VersionList = $GetVersion =~ /\S/ ? GetLanguageVersionList($GetVersion) :
        [CartridgeConfig($StoreName)->get('Cartridges','DE_EPAGES::Core')];

    foreach my $EpagesVersion (@$VersionList) {

        my $Languages = GetLanguagesByVersion($EpagesVersion, $LanguageCode // $ENV{EPAGES_LANGUAGES});
        my $LangList = join(' ', @$Languages);
        $LangList = '(none)' unless $LangList;

        print "using languages ($EpagesVersion): $LangList\n" if $Verbose;
        next if $LangList eq '(none)';

        # unpack language packs into $ENV{EPAGES_PATCHES}/LanguagePacks
        # then copy files to $ENV{EPAGES}
        my @KnownLanguages = InstallLanguageFiles($Languages, $EpagesVersion, $GetVersion);

        # do not install language packs if script was started with -get
        next if defined $GetVersion;

        # convert dictionary files from xml to json
        my $PerlExe = $Config{'osname'} eq 'MSWin32' ? $^X : $ENV{'PERL'};
        my $PerlScript = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/JQuery/Scripts/DictionaryConverter.pl";
        my $Options = "-cartridge ALL -storename \"$SiteName$StoreName\" -version $EpagesVersion";
        $Options .= $Config{osname} eq 'MSWin32' ? '>nul' : '>/dev/null' unless $Verbose;

        SystemCommand("$PerlExe $PerlScript $Options", {Print => $Verbose});

        # add language packs to database
        foreach my $Lang (@$Languages) {
            print "\n" if $Verbose;
            my $LangPack = LanguagePackLocation($Lang, $EpagesVersion);
            $LangPack =~ s,.tar.gz$,,; # get LP directory
            if (ElementOf($Lang, @KnownLanguages)) {
                print 'found: ', DictionaryXmlLocation($Lang), "\n" if $Verbose;
                print -d $LangPack ? 'cwd' : 'not found, skipping' if $Verbose;
                print ": $LangPack\n" if $Verbose;
                next unless -d $LangPack;
            } elsif (not -d $LangPack) {
                die "\nERROR: language pack not found: $LangPack\n";
            }

            RunOnStore(
                'Store'      => $StoreName,
                'DBPassword' => $Password,
                'Sub'        => sub {
                    RunOnDirectory($LangPack, sub {
                        InstallLanguagePack($Lang, $LangPack, $Verbose);
                                   });
                });
        }
    }

    return if defined $GetVersion;

    if( $Synchronize ) {
        print "\nSynchronizeCache()\n" if $Verbose;
        eval { SynchronizeCache(); };
        Warning( GetError() ) if ExistsError();
    }

    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1 if $Verbose;
}

#========================================================================================
# function     InstallLanguageFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       InstallLanguageFiles($Languages, $Version, $GetVersion)
# example      InstallLanguageFiles(['es', 'da'], '6.12.0', 0)
#----------------------------------------------------------------------------------------
# description  unpacks and installs language pack into $ENV{EPAGES_PATCHES}/LanguagePacks
#               then copy files to $ENV{EPAGES}
#----------------------------------------------------------------------------------------
# input        $Languages | list of language codes | ref.array
# input        $Version | epages version | string
# input        $GetVersion | install or download only? | boolean
# return       @KnownLanguages | $Lang where $Templates/Dictionary.$Lang.xml exists | array
#========================================================================================
sub InstallLanguageFiles {
    my ($Languages, $Version, $GetVersion) = @_;

    my @KnownLanguages;
    foreach my $Lang (@$Languages) {
        if (DictionaryXmlLocation($Lang, {IfExists => 1}) and not defined $GetVersion) {
            push @KnownLanguages, $Lang;
            next;
        }

        my $LangPack = LanguagePackLocation($Lang, $Version, { IfExists => 1});
        $LangPack = DownloadLanguagePack($Lang, $Version) unless $LangPack;
        my $LangPackDir = $LangPack;
        $LangPackDir =~ s,\.tar\.gz$,,;

        my $DictionaryXml = DictionaryXmlLocation(
            $Lang, {IfExists => 1, Dir => "$LangPackDir/Cartridges"});
        $LangPackDir = UntarLanguagePack($Lang, $Version) unless $DictionaryXml;

        next if defined $GetVersion;
        CopyDirectory("$LangPackDir/Cartridges", $ENV{EPAGES_CARTRIDGES}, undef,
                      {unlink => 1, link => 'orCopy'});
        CopyDirectory("$LangPackDir/Data/WebRoot", $ENV{EPAGES_WEBROOT}, undef,
                      {unlink => 1, link => 'orCopy'}) if -d "$LangPackDir/Data/WebRoot";
    }
    return @KnownLanguages;
}

#========================================================================================
# function     InstallLanguagePack
# state        private
#----------------------------------------------------------------------------------------
# syntax       InstallLanguagePack($Lang, $LangDir, $Verbose)
# example      InstallLanguagePack('es', "$ENV{EPAGES_PATCHES}/LanguagePacks/langpack-es-epages612p0", 1)
#----------------------------------------------------------------------------------------
# description  installs language into database
#----------------------------------------------------------------------------------------
# input        $Lang | language code as 'es' | string
# input        $LangDir | name of language pack | string
# input        $Verbose | print debug infos (or not) | boolean
#========================================================================================
sub InstallLanguagePack {
    my ($Lang, $LangDir, $Verbose) = @_;

    foreach (qw(Languages_install.xml Currencies_install.xml Locales_install.xml System.xml)) {
        next unless -f $_;
        print "ImportObjects($_)\n" if $Verbose;
        ImportObjects($_);
    }

    print "DE_EPAGES::Installer::API::LanguagePack->install(LanguageCode => $Lang)\n" if $Verbose;
    my $Installer = DE_EPAGES::Installer::API::LanguagePack->new(LanguageCode => $Lang);
    $Installer->install;
}

RunScript(
    'Sub' => \&Main
);

1;
