use strict;

use Getopt::Long;
use File::Temp;

use DE_EPAGES::Core::API::File qw ( GetFileContentLines );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::GitLogParser qw (
    GetChangedAttributes
    GetCommits
    GetJiraId
);
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );

use constant JIRA_PREFIX => 'https://epages.atlassian.net/browse';
use constant GITHUB_COMMIT_PREFIX => 'https://github.com/ePages-de/Cartridges/commit';

sub usage {
    my ($Message) = @_;
    print STDERR "Error: $Message\n\n" if $Message;

    print STDERR <<END_USAGE;
Description:
    The purpose of this script is to generate an overview over changed
    database attributes.
    It is required as long as we need to create cluster patches manually
    in order to achieve zero-downtime compliance.

Usage:
    perl $0 [options] [flags]

Options:
    -diff       the file which contains the git diff, e.g. /tmp/example.diff
    -format     either 'html' (HTML output) or 'adoc' (default, asciidoc output)

Flags:
    -help       show the command line options

Examples:
    git log -p v6.17.45..HEAD > /tmp/example.diff
    $ENV{PERL} $0 -diff /tmp/example.diff > /tmp/example.adoc
    $ENV{PERL} $0 -diff /tmp/example.diff -format html > /tmp/example.html
    $ENV{PERL} $0 -diff $ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/t/git/git-log-p_6.17.41-6.17.42.txt
END_USAGE
    exit 2;
}

sub Main {
    local $| = 1;

    my %Options;
    my @GetOpt = qw(diff=s format=s help);
    GetOptions(\%Options, @GetOpt) or usage();

    usage() if $Options{help};
    usage() unless $Options{diff};
    usage("No such file: $Options{diff}") unless -f $Options{diff};

    $Options{format} //= 'adoc';
    usage() unless ($Options{format} eq 'html' or $Options{format} eq 'adoc');

    my $aDiff = GetFileContentLines($Options{diff});
    my $aChangedAttributes = GetChangedAttributes($aDiff);
    my $aCommits = GetCommits($aDiff);

    my $Output = <<HERE
= Overview Changed Attributes
:toc: right
:toclevels: 4
HERE
;

    for my $ChangedAttribute (@{$aChangedAttributes}) {
        my $hAttributeDeclaration = _GetAttributeDeclaration($aCommits, $ChangedAttribute);
        $Output .= <<HERE

== $ChangedAttribute
=== Declaration
[cols="15,85"]
|===
|Ticket
|$hAttributeDeclaration->{ticket}

|Commit
|$hAttributeDeclaration->{commit}

|XML file
|$hAttributeDeclaration->{xml_file}
|===

=== Usage
[cols="15,10,75"]
|===
|Ticket| Commit |Module
HERE
;
        for my $Row (@{_GetAttributeUsageTableRows($aCommits, $ChangedAttribute)}) {
            $Output .= <<HERE

|$Row->{jira_id}
|$Row->{commit}
|$Row->{module}
HERE
    ;
        }
        $Output .= "|===\n";
    }

    if ($Options{format} eq 'adoc') {
        print $Output;
        return;
    }

    my $fh = File::Temp->new();
    print $fh $Output;

    my $AsciidocExe = ($ENV{EPAGES_PERL} // '/srv/epages/eproot/Perl'). '/asciidoc/asciidoc';
    $AsciidocExe = 'asciidoc' unless -x $AsciidocExe;
    my $Command = "$AsciidocExe -o - ". $fh->filename;
    print STDERR $Command, "\n";
    SystemCommand($Command);
}

sub _GetAttributeDeclaration {
    my ($ahCommits, $ChangedAttribute) = @_;
    my %AttributeDeclaration;

    for my $Commit (@{$ahCommits}) {
       next unless exists $Commit->{diffs};
       for my $Diff (@{$Commit->{diffs}}) {
           if (_HasAttributeDeclaration($Diff, $ChangedAttribute)) {
               my $JiraId = GetJiraId($Commit->{message});
               $AttributeDeclaration{ticket} = JIRA_PREFIX . "/$JiraId\[$JiraId\]";

               my $CommitId = $Commit->{id};
               my $ShortCommitId = substr $CommitId, 0, 7;
               $AttributeDeclaration{commit} = GITHUB_COMMIT_PREFIX . "/$CommitId\[$ShortCommitId\]";

               $AttributeDeclaration{xml_file} = $Diff->{module};
           }
       }
    }

    return \%AttributeDeclaration;
}

sub _HasAttributeDeclaration {
    my ($Diff, $ChangedAttribute) = @_;

    if ($Diff->{module} =~ /\.xml$/) {
        if ($Diff->{diff_text} =~ /.*<Attribute.*Alias=('|")$ChangedAttribute\1/) {
            return 1;
        }
    }
    return 0;
}

sub _GetAttributeUsageTableRows {
    my ($ahCommits, $ChangedAttribute) = @_;
    my @TableRows;

    for my $Commit (@{$ahCommits}) {
        next unless exists $Commit->{diffs};
        for my $Diff (@{$Commit->{diffs}}) {
            push @TableRows, _GetAttributeUsageTableRow($Commit, $Diff)
                if _HasUsageOfChangedAttribute($Diff, $ChangedAttribute);
        }
    }

    return \@TableRows;
}

sub _HasUsageOfChangedAttribute {
    my ($Diff, $ChangedAttribute) = @_;

    if ($Diff->{module} =~ /\.pm$/ || $Diff->{module} =~ /\.js$/) {
        if ($Diff->{diff_text} =~ /\+.*\b$ChangedAttribute\b/) {
            return 1;
        }
    }
    return 0;
}


sub _GetAttributeUsageTableRow {
    my ($Commit, $Diff) = @_;
    my %Row;

    my $JiraId = GetJiraId($Commit->{message});
    $Row{jira_id} = JIRA_PREFIX . "/$JiraId\[$JiraId\]";

    my $CommitId = $Commit->{id};
    my $ShortCommitId = substr $CommitId, 0, 7;
    $Row{commit} = GITHUB_COMMIT_PREFIX . "/$CommitId\[$ShortCommitId\]";

    $Row{module} = $Diff->{module};

    return \%Row;
}

RunScript(
    'Debug' => 0,
    'Message' => 'short',
    'Sub' => \&Main
);

1;
