#!/srv/epages/eproot/Perl/bin/perl

use strict;
use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Database::API::Config qw( BUStoreTypes GetBaseStoreType );
use DE_EPAGES::Installer::API::Install qw( InstallStoreTypeConf
                                           InstallStoreTypeDirFromBase
                                           RelinkStoreTypeDirs
                                           DeleteObsoleteStoreTypeConfig );

sub Main {

    local $| = 1;

    my $time1 = time;

    #-------------------------------------------------------------------
    # check options
    #-------------------------------------------------------------------

    my %Options = ();
    my @GetOpt = qw(help storetype=s version=s nooutput);
    GetOptions(\%Options, @GetOpt);

    usage() if $Options{help};
    usage() unless $Options{version};
    my $Quiet = $Options{nooutput};

    my $EpagesPatches = $ENV{EPAGES_PATCHES};
    $EpagesPatches = "$ENV{EPAGES_SHARED}/Patches"
        if not $EpagesPatches and $ENV{EPAGES_SHARED};
    usage('environment variable not set: EPAGES_PATCHES') unless $EpagesPatches;
    my $EpagesConfig = $ENV{EPAGES_CONFIG};
    usage('environment variable not set: EPAGES_CONFIG') unless $EpagesConfig;

    #-------------------------------------------------------------------
    # define variables
    #-------------------------------------------------------------------

    my $Patch = 'patch_'. $Options{version};
    $Patch =~ s/\./_/g;

    my @Version = split(/\./, $Options{version});
    $#Version = 2 if $#Version > 2;
    my $Major = join('.', @Version);

    my $ExtractedWebRootPacks = "$EpagesPatches/$Patch/ExtractedDataPacks".
        "/ExtractedWebRootPacks.v/$Options{version}/$Major";

    usage("directory not found: $ExtractedWebRootPacks/Config.d")
        unless -d "$ExtractedWebRootPacks/Config.d";

    my $BUStoreTypes = BUStoreTypes();
    my @StoreTypes;
    if ($Options{storetype}) {
        foreach (split(/\s+/, $Options{storetype})) {
            usage("-storetype wrong: not a business unit: $_")
                unless ElementOf($_, $BUStoreTypes);
            push @StoreTypes, $_;
        }
    } else {
        @StoreTypes = @$BUStoreTypes;
    }

    my $ExitCode = 0;
    foreach my $StoreType ( @StoreTypes ) {

        #-------------------------------------------------------------------
        # find $BaseStoreType for $StoreType
        #-------------------------------------------------------------------

        print "GetBaseStoreType($StoreType) -> " unless $Quiet;
        my ($BaseStoreType, $AdditionalCartridges)
            = GetBaseStoreType($StoreType, "$ExtractedWebRootPacks/Config.d");
        if (not $BaseStoreType) {
            $ExitCode = 1;
            print "NOT FOUND\n" unless $Quiet;
            next;
        }
        print "$BaseStoreType\n" unless $Quiet;

        #-------------------------------------------------------------------
        # create $ENV{EPAGES_CONFIG}/StoreTypes.d/$StoreType.conf
        #-------------------------------------------------------------------

        print "InstallStoreTypeConf($StoreType) -> " unless $Quiet;
        InstallStoreTypeConf($StoreType, $Options{version},
                             "$ExtractedWebRootPacks/Config.d/$BaseStoreType.conf",
                             $AdditionalCartridges,
        );
        print "DONE\n" unless $Quiet;

        #-------------------------------------------------------------------
        # create $ENV{EPAGES_WEBROOT}/StoreTypes/$StoreType
        #-------------------------------------------------------------------

        print "InstallStoreTypeDirFromBase($StoreType) -> " unless $Quiet;
        InstallStoreTypeDirFromBase($StoreType, $Options{version},
                                    "$ExtractedWebRootPacks/$BaseStoreType",
                                    $AdditionalCartridges,
        );
        print "DONE\n" unless $Quiet;

    }

    #-------------------------------------------------------------------
    # create symlinks in $ENV{EPAGES_WEBROOT}/StoreTypes pointing to version/
    #-------------------------------------------------------------------

    print "RelinkStoreTypeDirs($Options{version}) -> " unless $Quiet;
    RelinkStoreTypeDirs($Options{version});
    print "DONE\n" unless $Quiet;

    #-------------------------------------------------------------------
    # delete all directories but version in $ENV{EPAGES_CONFIG}/StoreTypes.d/
    #-------------------------------------------------------------------

    print "DeleteObsoleteStoreTypeConfig($Options{version}) -> " unless $Quiet;
    DeleteObsoleteStoreTypeConfig($Options{version});
    print "DONE\n" unless $Quiet;

    printf "need %.1f seconds%30s\n", time-$time1, '' unless $Quiet;
    exit $ExitCode;
}

sub usage {
    my ($Msg) = @_;
    print "ERROR: $Msg\n\n" if $Msg;
    print <<END_USAGE;
Usage:
    perl $0 { [-storetype <stype>] -version <vrs> | -help }

Options:
    -storetype  (space-sep. list of) storetypes, if missing all storetypes will be used
    -version    epages version number

Flags:
    -help       show the command line options
    -nooutput   do not print any output

Example:
    \$PERL $0 -version 6.17.17.1
    \$PERL $0 -storetype 'Site Store' -version 6.17.17.1
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;

