use strict;
use warnings;
use Getopt::Long;
use DE_EPAGES::Core::API::File qw ( DeleteDirectory );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Installer::API::FishEyeHandler;
use DE_EPAGES::Installer::API::CVSTagHandler;
use DE_EPAGES::Installer::API::JiraHandler;
use Time::HiRes qw ( time );
use Term::ReadKey;
use File::Basename;
use Sort::Versions qw (versioncmp);

sub Main {
    my $Help;
    my $Mail;
    my $Commit;
    my $Review;
    my $User;
    my $Passwd;
    GetOptions(
        'help' => \$Help,
        'mail' => \$Mail,
        'commit' => \$Commit,
        'review' => \$Review,
        'user=s' => \$User,
        'passwd=s' => \$Passwd,
    );
    usage() if $Help;
    usage('Missing parameter -mail, -commit- or -review') unless ($Mail || $Commit || $Review);
    usage() if ( scalar @ARGV ) < 2;

    unless( defined $User ) {
        print "Please enter JIRA login:\nUsername: ";
        ReadMode 1;
        $User = ReadLine(0);
        chomp($User);
    }

    unless( defined $Passwd ) {
        print 'Passwort: ';
        ReadMode 2;
        $Passwd = ReadLine(0);
        chomp($Passwd);
        print "\n";
    }

    my $time1 = time;

    my $TempDir = 'RND';

    my $FishEyeHandler = DE_EPAGES::Installer::API::FishEyeHandler->new( $User, $Passwd );
    my $CVSTagHandler = DE_EPAGES::Installer::API::CVSTagHandler->new({ -q => '' }, { 'print' => 1, 'printOutput' => 1 });
    my $JiraHandler = DE_EPAGES::Installer::API::JiraHandler->new( $User, $Passwd );
    my $aMessages = [];

    #get hotfix version
    my $HotfixVersion = shift @ARGV;

    #set branch version
    my $HotfixBranchVersion = $CVSTagHandler->getBranchForHotfix($HotfixVersion);

    #for each passed epg number
    foreach my $EPG (@ARGV) {

        my $ahEmailFiles;

        #get all files connected to current EPG number
        my $aAllFiles = $FishEyeHandler->getFilenames($EPG);

        foreach my $File (@$aAllFiles) {

            #check file out
             $CVSTagHandler->command( 'checkout', [ '-r', $HotfixBranchVersion ], [ "$TempDir/$File" ] );
        }

        #extract EPG number
        $EPG =~ s/^EPG( |-)?//;

        #test if passed number is valid
        die 'Passed EPG-Number is not valid.' unless ( $EPG =~ /^(EPB|EPG)( |-|_)\d+$/ );

        push @$aMessages, 'concerned Files are attached' if $Mail;
        push @$aMessages, 'concerned Files were commited' if $Commit;
        push @$aMessages, '';

        #get branch array for epg number
        my $aBranch = $FishEyeHandler->getBranchArray($EPG);

        push @$aMessages, 'concerned files and revisions:';


        #  each branch
        foreach my $Branch (@$aBranch) {
            next if $Branch eq $HotfixBranchVersion;

            push @$aMessages, "\n$Branch:";

            #get all files connected to current epg number
            my $aBranchFiles = $FishEyeHandler->getFilenames( $EPG, $Branch );

            #for each of these files
            foreach my $File (@$aBranchFiles) {

                #print filename
                push @$aMessages, ' - ' . $File;

                #get revision array
                my $aRevisions = $FishEyeHandler->getRevisions( $File, $EPG, $Branch );

                #case 1: revision array is not empty
                if ( $aRevisions->[-1] ) {
                    push @$aMessages, '         -> Revision:' . join( ', ', @$aRevisions );
                }
                #case 2: revision array is empty
                else {

                    #die "No revision on brach $HotfixBranchVersion.";
                    $aRevisions = $FishEyeHandler->getRevisions( $File, $EPG );

                    push @$aMessages, 'No revision this branch. Latest revision: ' . $aRevisions->[-1];
                }
            }
        }
        push @$aMessages, "\n\nMerged Files:";

        #initialize commit comment
        my $Comment;

        #for each file
        foreach my $File (@$aAllFiles) {

            $Comment = "EPG-$EPG: hotfix, merge ";

            #initialize MergeHash
            my %MergeHash;

            #get branches
            $aBranch = $FishEyeHandler->getBranchArray( $EPG, $File );

            push @$aMessages, "\nFrom " . join( ', ', @$aBranch ) . "     To $HotfixBranchVersion";

            push @$aMessages, '    ' . $File;

            #for each branch
            foreach my $Branch (@$aBranch) {

                #skip hotfixbranch
                next if $Branch eq $HotfixBranchVersion;

                #get revision array
                my $aRevisions = $FishEyeHandler->getRevisions( $File, $EPG, $Branch );

                #get temp merge hash
                my $hTempHash = $CVSTagHandler->getMergeRevisions($aRevisions);

                #add to persistent merge hash
                %MergeHash = ( %MergeHash, %$hTempHash );
            }

            #apply mergehash and save output in resulthash
            my $ahResultHash = [];

            foreach my $StartRevision ( sort versioncmp keys %MergeHash ) {
                my $EndRevision = $MergeHash{$StartRevision};
                my $ExistsBefore = -e $File;
                my $hResult = $CVSTagHandler->command( 'update', [ '-j', $StartRevision, '-j', $EndRevision ], ["$TempDir/$File"], {}, { 'skiperrors' => 1 } );
                $hResult->{'deleted'} = 1 if ($ExistsBefore && !(-e $File));
                push @$ahResultHash, $hResult;
            }

            my $Deleted = 0;

            #for each merge pair (start revision and end revision)
            foreach my $StartRevision ( keys %MergeHash ) {

                $Comment .= "$StartRevision - $MergeHash{$StartRevision}";

                push @$aMessages, '        Source: ' . "$StartRevision - $MergeHash{$StartRevision}";

                #foreach resulthash
                foreach my $hResultHash (@$ahResultHash) {

                    #test for CVS errors
                    if ( $hResultHash->{'return'} != 0 ) {
                        push @$aMessages, '     Error ' . " $hResultHash->{'return'} occured! ";
                    }

                    #test if file was deleted
                    if ( $hResultHash->{'deleted'} != 0 ) {
                        push @$aMessages, '     Error:    File was deleted during update process ';
                        $Deleted = 1;
                    }
                }
            }

            #get revision array
            my $aRevisions = $FishEyeHandler->getRevisions( $File, $EPG, $HotfixBranchVersion );

            push @$aMessages, "        Comment: $Comment";
            unless ($Deleted) {

                #commit if flag is set
                if ($Commit) {
                    $CVSTagHandler->command( 'commit', [ '-m', $Comment ], ["$TempDir/$File"] );
                }

                #attach to email if flag is set
                if ($Mail) {

                    my $hTemp->{'filename'} = "$File";
                    push @$ahEmailFiles, $hTemp;
               }
            }
        }

        print "$_\n" foreach @$aMessages;

        if( $Mail ) {
            #get participants for current epg number
            my $aParticipant = $JiraHandler->getParticipants($EPG);

            #convert to email adresses
            foreach my $Participant (@$aParticipant) {
                $Participant .= '@epages.com'; ## no critic qw(RequireInterpolationOfMetachars)
            }

            push @$aParticipant, $User . '@epages.com' unless grep { $_ eq $User . '@epages.com' } @$aParticipant; ## no critic qw(RequireInterpolationOfMetachars)

            $CVSTagHandler->sendMergeMail( $aParticipant, $aMessages, $EPG, $ahEmailFiles );
        }

        DeleteDirectory( $TempDir, { 'recursive' => 1 } ) unless $Review;
    }

    printf "need %.1f seconds\n", time - $time1;
}

sub usage {
    my ($Message) = @_;

    print STDERR "$Message\n\n" if defined $Message;

    print <<END_USAGE;
Description:
        Script to merge files into hotfix automatically
        requres a hotfix-version and at least one epg number
        requires at least one of these flags:
        -mail, -commit

Usage:
        perl setHotfix.pl flags hotfix-version EPG-number1 EPG-number2...

flags:
        -mail      attach merged files to email
        -commit    commit merged files
        -review    don't delete the temporary directory, allows to solve
                    conflicts and commit manually
        -user      Jira user name
        -passwd    Jira password

Example:
        perl setHotfix.pl -mail -commit hotfix_6_14_3_5 EPG-42 EPG-666
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main, );

1;
