use Test::More;

use DE_EPAGES::Core::API::Error  qw ( GetError ExistsError );
use DE_EPAGES::Core::API::File   qw ( CopyFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Object::API::XMLCompare qw ( CompareXML );
use DE_EPAGES::Installer::API::CartridgesXMLFilePatcher
    qw( ReadCartridgesXmlFile GetCartridgeDependencies
        GetCartridgePositions CartridgeDependencyError );
use DE_EPAGES::Object::API::XMLStructureImportHandler;

use strict;

local $ENV{'EPAGES_CONFIG_CARTRIDGES'} = 'Cartridges.d';

RunScript(
    Sub => sub {

        my $time1 = time;

        my $hOptions = {'checkdelete' => '1', 'checkreference' => '1', 'printmessage' => '1'};

        testCartridges($hOptions);
        testCustomStoretypes($hOptions);
        testCorruptCartridgesXML($hOptions);
        testCartridgePositions();
        testMissingDependency();
        testNotResolvableDependency();
        testCircularDependency();
        testDirectCircularDependency();
        testIndirectCircularDependency();

        my $time2 = time;
        printf "need %.3f seconds\n", $time2-$time1;

        done_testing();
    }
);

sub testCartridges {
    my ($hOptions) = @_;

    my ($CartridgesXml, $FilePatcher) = _getCartridgesXml(1, 'imp');
    $FilePatcher->importElements($CartridgesXml->{changes});
    ok (CompareXML($CartridgesXml->{result}, $CartridgesXml->{expected}, $hOptions), 'import (1.1)');
    ok (CompareXML($CartridgesXml->{expected}, $CartridgesXml->{result}, $hOptions), 'import (1.2)');

    ($CartridgesXml, $FilePatcher) = _getCartridgesXml(2, 'del');
    $FilePatcher->deleteElements($CartridgesXml->{changes});
    ok (CompareXML($CartridgesXml->{result}, $CartridgesXml->{expected}, $hOptions), 'delete (2.1)');
    ok (CompareXML($CartridgesXml->{expected}, $CartridgesXml->{result}, $hOptions), 'delete (2.2)');

    return;
}

sub testCorruptCartridgesXML {
    my ($hOptions) = @_;

    my ($CartridgesXml, $FilePatcher) = _getCartridgesXml(5);
    eval { $FilePatcher->importElements(undef, {check_dependencies => 1}); };
    my $Cartridge = ExistsError() ? GetError()->{Vars}{Cartridge} : undef;
    my $Requires = ExistsError() ? GetError()->{Vars}{Requires} : undef;
    ok((ExistsError() and GetError()->code eq 'DEPENDENCY_MISSING' and
        $Cartridge eq 'DE_EPAGES::Preissuchmaschine' and $Requires eq 'DE_EPAGES::Guenstiger'),
       'DE_EPAGES::Preissuchmaschine requires DE_EPAGES::Guenstiger (missing)'
    );

    eval { $FilePatcher->importElements(); };
    ok((not ExistsError()),
        'DE_EPAGES::Preissuchmaschine requires DE_EPAGES::Guenstiger (ignored)'
    );
    ok (CompareXML($CartridgesXml->{result}, $CartridgesXml->{expected}, $hOptions), 'sort (5.1)');
    ok (CompareXML($CartridgesXml->{expected}, $CartridgesXml->{result}, $hOptions), 'sort (5.2)');

    return;
}

sub testCartridgePositions {

    my ($CartridgesXml) = _getCartridgesXml(1);
    my $XMLStructImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $XMLStruct = $XMLStructImportHandler->getXMLStructure(
        $CartridgesXml->{expected}, {'XML-Type' => 'epages_cartridges_xml'});

    my $ExternalShipping = 'Cartridge{Package->DE_EPAGES::ExternalShipping}';
    my $MRW = 'Cartridge{Package->DE_EPAGES::MRW}';

    my $Position1 = $XMLStruct->getXMLStructAttribute("/DatabaseType{Name->Store}/$ExternalShipping",'Position');
    my $Position2 = $XMLStruct->getXMLStructAttribute("/DatabaseType{Name->Store}/$MRW",'Position');
    ok(($XMLStruct->existsElement("/DatabaseType{Name->Store}/$MRW") and $Position1 < $Position2),
        'DE_EPAGES::ExternalShipping is before DE_EPAGES::MRW in Store'
    );

    $Position1 = $XMLStruct->getXMLStructAttribute("/DatabaseType{Name->Fantasy}/$ExternalShipping",'Position');
    $Position2 = $XMLStruct->getXMLStructAttribute("/DatabaseType{Name->Fantasy}/$MRW",'Position');
    ok(($XMLStruct->existsElement("/DatabaseType{Name->Fantasy}/$MRW") and $Position1 < $Position2),
        'DE_EPAGES::ExternalShipping is before DE_EPAGES::MRW in Fantasy'
    );

    $Position1 = $XMLStruct->getXMLStructAttribute("/DatabaseType{Name->Fantasy3}/$ExternalShipping",'Position');
    $Position2 = $XMLStruct->getXMLStructAttribute("/DatabaseType{Name->Fantasy3}/$MRW",'Position');
    ok((not $XMLStruct->existsElement("/DatabaseType{Name->Fantasy3}/$MRW")),
        'DE_EPAGES::MRW is not in Fantasy3' );

    return;
}

sub testCustomStoretypes {
    my ($hOptions) = @_;

    my ($CartridgesXml, $FilePatcher) = _getCartridgesXml(3, 'del');
    $FilePatcher->deleteElements($CartridgesXml->{changes});
    ok (CompareXML($CartridgesXml->{result}, $CartridgesXml->{expected}, $hOptions), 'delete custom (3.1)');
    ok (CompareXML($CartridgesXml->{expected}, $CartridgesXml->{result}, $hOptions), 'delete custom (3.2)');

    ($CartridgesXml, $FilePatcher) = _getCartridgesXml(4, 'del');
    $FilePatcher->deleteElements($CartridgesXml->{changes});
    ok (CompareXML($CartridgesXml->{result}, $CartridgesXml->{expected}, $hOptions), 'delete custom (4.1)');
    ok (CompareXML($CartridgesXml->{expected}, $CartridgesXml->{result}, $hOptions), 'delete custom (4.2)');

    return;
}

sub testMissingDependency {
    my ($CartridgesXml) = _getCartridgesXml(1);
    my $hCartridgesXml = ReadCartridgesXmlFile($CartridgesXml->{expected});
    my ($DependentCartridges, $IndependentCartridges)
        = GetCartridgeDependencies($hCartridgesXml);

    eval { CartridgeDependencyError($DependentCartridges); };
    ok((ExistsError() and GetError()->code eq 'DEPENDENCY_MISSING'),
       'Dependency is missing (GetError()->code)'
    );

    eval { GetCartridgePositions($DependentCartridges, $IndependentCartridges,
                                 {check_dependencies => 1}); };
    my $Cartridge = ExistsError() ? GetError()->{Vars}{Cartridge} : undef;
    my $Requires = ExistsError() ? GetError()->{Vars}{Requires} : undef;
    ok((ExistsError() and GetError()->code eq 'DEPENDENCY_MISSING' and
        $Cartridge eq 'DE_EPAGES::GoogleAdWords' and $Requires eq 'DE_EPAGES::Google'),
       'Dependency is missing (GetError()->{Vars})'
    );

    return;
}

sub testNotResolvableDependency {
    my $DependentCartridges = {};
    $DependentCartridges->{'DE_EPAGES::Core'} = {};
    eval { CartridgeDependencyError($DependentCartridges); };
    ok((ExistsError() and GetError()->code eq 'DEPENDENCIES_NOT_RESOLVABLE'),
        'Dependency is not resolvable'
    );

    return;
}

sub testCircularDependency {
    my $DependentCartridges = {};
    $DependentCartridges->{'DE_EPAGES::Order'}{'DE_EPAGES::Order'} = 1;

    eval { CartridgeDependencyError($DependentCartridges); };
    my $CircularDependency = ExistsError() ? GetError()->{Vars}{CircularDependency} : undef;
    my $ExpectedDependency = 'DE_EPAGES::Order->DE_EPAGES::Order';
    ok((ExistsError() and GetError()->code eq 'CIRCULAR_DEPENDENCY' and
            $CircularDependency eq $ExpectedDependency),
        "circular dependency: $ExpectedDependency"
    );

    return;
}

sub testDirectCircularDependency {
    my $DependentCartridges = {};
    $DependentCartridges->{'DE_EPAGES::Dictionary'}{'DE_EPAGES::Object'} = 1;
    $DependentCartridges->{'DE_EPAGES::Object'}{'DE_EPAGES::Dictionary'} = 1;

    eval { CartridgeDependencyError($DependentCartridges); };
    my $CircularDependency = ExistsError() ? GetError()->{Vars}{CircularDependency} : undef;
    my $ExpectedDependency = 'DE_EPAGES::Dictionary->DE_EPAGES::Object->DE_EPAGES::Dictionary';
    ok((ExistsError() and GetError()->code eq 'CIRCULAR_DEPENDENCY' and
            $CircularDependency eq $ExpectedDependency),
        "circular dependency: $ExpectedDependency"
    );

    return;
}

sub testIndirectCircularDependency {
    my $DependentCartridges = {};
    $DependentCartridges->{'DE_EPAGES::ExternalPayment'}{'DE_EPAGES::Order'} = 1;
    $DependentCartridges->{'DE_EPAGES::Order'}{'DE_EPAGES::Object'} = 1;
    $DependentCartridges->{'DE_EPAGES::Object'}{'DE_EPAGES::ExternalPayment'} = 1;

    eval { CartridgeDependencyError($DependentCartridges); };
    my $CircularDependency = ExistsError() ? GetError()->{Vars}{CircularDependency} : undef;
    my $ExpectedDependency = 'DE_EPAGES::ExternalPayment->DE_EPAGES::Order->...->DE_EPAGES::ExternalPayment';
    ok((ExistsError() and GetError()->code eq 'CIRCULAR_DEPENDENCY' and
            $CircularDependency eq $ExpectedDependency),
        "circular dependency: $ExpectedDependency"
    );

    return;
}

sub _getCartridgesXml {
    my ($Number, $Changes) = @_;

    my %CartridgesXml = (
        file => "Cartridges$Number.xml",
        changes => "${Changes}_Cartridges$Number.xml",
        result => "result_Cartridges$Number.xml",
        expected => "expected_Cartridges$Number.xml",
        );

    CopyFile($CartridgesXml{file}, $CartridgesXml{result});
    my $FilePatcher = DE_EPAGES::Installer::API::CartridgesXMLFilePatcher->new($CartridgesXml{result});

    return (\%CartridgesXml, $FilePatcher);
}

1;
