#========================================================================================
# package      DE_EPAGES::Ipayment::API::CGI::SecurityLayer
# state        public
#----------------------------------------------------------------------------------------
# description  description
#----------------------------------------------------------------------------------------
# example      example
#========================================================================================
package DE_EPAGES::Ipayment::API::CGI::SecurityLayer;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;

use strict;

use DE_EPAGES::Ipayment::API::Constants qw ( $ExpectedReturnIps );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# input        $DatabaseBasketOrOrder | Object injection thought for testing | object
# return       $self | self | DE_EPAGES::Ipayment::API::CGI::SecurityLayer
#========================================================================================
sub new {
    my $class = shift;
    my ($DatabaseBasketOrOrder) = @_;
    my $self  = bless {
        'DatabaseBasketOrOrder' => $DatabaseBasketOrOrder // DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new(),
    }, $class;
    return $self;
}

#========================================================================================
# function     extractDataFromParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       extractDataFromParams( $hFormParams );
#----------------------------------------------------------------------------------------
# description  Extracts data from the returned params of HiddenTrigger callback and
#               returns 1 or 0 whether the data fits with the shop data or not
#----------------------------------------------------------------------------------------
# input        $hFormParams | Hash with HiddenTrigger params from ipayment | ref_hash
# return       $AreValidReturnParameters | 1 if valid return parameters | boolean
#========================================================================================
sub extractDataFromParams {
    my $self = shift;
    my ($hFormParams) = @_;

    my $AreValidReturnParameters = 1;
    if (not isValid($hFormParams)){
        $self->{'Error'} = 'Form Parameters are not valid';
        return 0;
    }
    if (not isValid($hFormParams->{'shopper_id'})){
        $self->{'Error'} = 'Form Parameters have no shopper id';
        return 0;
    }
    $self->{'params'} = $hFormParams;
    my $Order = $self->getLineItemPayment()->parent()->parent();
    $self->{'Order'} = $Order;
    my $DatabaseBasketOrOrder = $self->{'DatabaseBasketOrOrder'};
    my $ExtractionWasSuccessful = $DatabaseBasketOrOrder->extractFromBasketOrOrder($Order);

    if (not $ExtractionWasSuccessful){
        $self->{'Error'} = 'DatabaseBasketOrOrderError';
        return 0;
    }

    my $RemoteIp = $self->getRemoteIp();
    if (not ElementOf($RemoteIp, $ExpectedReturnIps) and
        $hFormParams->{'ChangeAction'} eq 'HiddenTrigger'){
        $self->{'Error'} = 'RemoteIpError';
        return 0;
    }

    if ($self->_checkOrderData($DatabaseBasketOrOrder)){
        $AreValidReturnParameters = 1;
    } else {
        $AreValidReturnParameters = 0;
    }
    return $AreValidReturnParameters;
}

#========================================================================================
# function     belongsThisOrdertoShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       belongsThisOrdertoShop( $ShopID );
#----------------------------------------------------------------------------------------
# description  It checks if the incoming shop id belongs to the shop
#----------------------------------------------------------------------------------------
# input        $ShopID | Id from the shop | String
# return       $BelongsToShop | Whether it belongs to the shop or not | boolean
#========================================================================================
sub belongsThisOrdertoShop {
    my $self = shift;
    my ($ShopID) = @_;

    my $ShopIDFromLineItem = $self->getLineItemPayment()->getSite()->id();
    my $BelongsToShop = 0;

    if ($ShopID == $ShopIDFromLineItem){
        $BelongsToShop = 1;
    }else{
        $BelongsToShop = 0;
    }
    return $BelongsToShop;
}

#========================================================================================
# function     getBasketOrOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       getBasketOrOrder(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the Order object
#----------------------------------------------------------------------------------------
# return       $self->{'Order'} | Order object | object
#========================================================================================
sub getBasketOrOrder{
    my $self = shift;
    return $self->{'Order'};
}

#========================================================================================
# function     getReturnStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       getReturnStatus(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the return status
#----------------------------------------------------------------------------------------
# return       $self->{'params'}->{'ret_status'} | Return status | String
#========================================================================================
sub getReturnStatus {
    my $self = shift;
    return $self->{'params'}->{'ret_status'};
}

#========================================================================================
# function     getReturnPaymentStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       getReturnPaymentStatus(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the return payment status - This is used by paypal
#----------------------------------------------------------------------------------------
# return       $self->{'params'}->{'ret_paymentStatus'} | Return payment status | String
#========================================================================================
sub getReturnPaymentStatus {
    my $self = shift;
    return $self->{'params'}->{'ret_paymentStatus'};
}

#========================================================================================
# function     getPaymentType
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPaymentType(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the return payment type
#----------------------------------------------------------------------------------------
# return       $self->{'params'}->{'trx_paymenttyp'} | Return payment type | String
#========================================================================================
sub getPaymentType {
    my $self = shift;
    return $self->{'params'}->{'trx_paymenttyp'};
}




sub getAuthcode {
    my $self = shift;
    return $self->{'params'}->{'ret_authcode'};
}





#========================================================================================
# function     getTransactionNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       getTransactionNumber(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the transaction number
#----------------------------------------------------------------------------------------
# return       $self->{'params'}->{'ret_trx_number'} | Transaction number | String
#========================================================================================
sub getTransactionNumber {
    my $self = shift;
    return $self->{'params'}->{'ret_trx_number'};
}

#========================================================================================
# function     getTransactionType
# state        public
#----------------------------------------------------------------------------------------
# syntax       getTransactionType(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the transaction type
#----------------------------------------------------------------------------------------
# return       $self->{'params'}->{'trx_typ'} | Transaction type | String
#========================================================================================
sub getTransactionType {
    my $self = shift;
    return $self->{'params'}->{'trx_typ'};
}

#========================================================================================
# function     getRemoteIp
# state        public
#----------------------------------------------------------------------------------------
# syntax       getRemoteIp(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the remote ip
#----------------------------------------------------------------------------------------
# return       $self->{'params'}->{'REMOTE_ADDR'} | Remote ip | String
#========================================================================================
sub getRemoteIp {
    my $self = shift;
    return $self->{'params'}->{'REMOTE_ADDR'};
}

#========================================================================================
# function     getErrorMessage
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorMessage(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the error message
#----------------------------------------------------------------------------------------
# return       $self->{'Error'} | Error message | String
#========================================================================================
sub getErrorMessage {
    my $self = shift;
    return $self->{'Error'};
}

#========================================================================================
# function     getLineItemPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       getLineItemPayment( $LineItemPayment );
#----------------------------------------------------------------------------------------
# description  Retrieves the line item payment from ipayment
#----------------------------------------------------------------------------------------
# return       $LineItemPayment | LineItemPaymentIpayment object | object
#========================================================================================
sub getLineItemPayment {
    my $self = shift;

    my $LineItemPaymentGUID = $self->getShopperID();
    my $LineItemPayment = $self->_loadLineItemPaymentByGUID($LineItemPaymentGUID);
    return $LineItemPayment;
}

#========================================================================================
# function     getShopperID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getShopperID(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the shopper id
#----------------------------------------------------------------------------------------
# return       $self->{'params'}->{'shopper_id'} | Shooper id | String
#========================================================================================
sub getShopperID {
    my $self = shift;
    return $self->{'params'}->{'shopper_id'};
}

#========================================================================================
# function     getTransactionAmount
# state        public
#----------------------------------------------------------------------------------------
# syntax       getTransactionAmount(  );
#----------------------------------------------------------------------------------------
# description  Retrieves the transaction amount
#----------------------------------------------------------------------------------------
# return       $self->{'params'}->{'trx_amount'} | Transaction amount | String
#========================================================================================
sub getTransactionAmount {
    my $self = shift;
    return $self->{'params'}->{'trx_amount'};
}

#----------------------------------------------------------------------------------------
sub _checkOrderData {
    my $self = shift;
    my ($DatabaseBasketOrOrder) = @_;

    my $AreValidReturnParameters = 1;
    my $hResponseParams = $self->{'params'};
    my $hOrderData = $self->_getOrderData($DatabaseBasketOrOrder);

    foreach my $Key (keys %{$hOrderData}){
        my $ResponseValue = $hResponseParams->{$Key};
        $ResponseValue = hex $ResponseValue;
        my $OrderValue = $hOrderData->{$Key};

        $OrderValue = hex $OrderValue;

        unless (isValid($ResponseValue) && $ResponseValue eq $OrderValue){
            $AreValidReturnParameters = 0;
            $self->{'Error'} = "OrderDataError; \t Key: $Key \t ReceivedValue: '$ResponseValue' \t ExpectedValue: '$OrderValue'"
        }
    }
    return $AreValidReturnParameters;
}

#----------------------------------------------------------------------------------------
sub _getOrderData {
    my $self = shift;
    my ($DatabaseBasketOrOrder) = @_;

    my $hOrderData = {
        'trx_amount'        => $DatabaseBasketOrOrder->getGrandTotal() * 100, ## no critic qw( ProhibitMagicNumbers )
        'shopper_id'        => $DatabaseBasketOrOrder->getGUID(),
        'trx_currency'      => $DatabaseBasketOrOrder->getCurrencyID(),
        'trx_paymenttyp'    => $DatabaseBasketOrOrder->{'PaymentTypeMethod'}->get('ProductType'),
        'trxuser_id'        => $DatabaseBasketOrOrder->getUserID(),
    };

    return $hOrderData
}


#----------------------------------------------------------------------------------------
sub _loadLineItemPaymentByGUID {
    my $self = shift;
    my ($LineItemPaymentGUID) = @_;

    my $LineItemPaymentIpaymentClass = $self->_LoadClassByAlias('LineItemPaymentIpayment');
    my $aIpaymentTransactions  = $LineItemPaymentIpaymentClass->getObjects('ByGUID', [$LineItemPaymentGUID]);

    if ( (isValid($aIpaymentTransactions) and isValid(@{$aIpaymentTransactions}[0]))){
        return @{$aIpaymentTransactions}[0];
    }else{
        Error('Invalid TransactionID: ' . $LineItemPaymentGUID);
    }
}

#----------------------------------------------------------------------------------------
# wrapper
#----------------------------------------------------------------------------------------
sub _LoadClassByAlias {
    my $self = shift;
    my ($Alias) = @_;
    return LoadClassByAlias($Alias);
}


1;
