#========================================================================================
# package      DE_EPAGES::Ipayment::API::PaymentMethodGenerator
# state        public
# description  Class to generate new payment methods
#========================================================================================
package DE_EPAGES::Ipayment::API::PaymentMethodGenerator;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw ( Transaction LoadObjectByPath LoadObjectByGUID ExistsObjectByPath);
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw ( InsertShippingPaymentMap );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid GenerateShortGUID );
use JSON qw ( from_json );

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Input1 );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# return       self | DE_EPAGES::Ipayment::API::PaymentMethodGenerator
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {
        'GeneratedPaymentMethods' => [],
    }, $class;
    return $self;
}



#========================================================================================
# function     generatePaymentMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       generatePaymentMethods( $PaymentMethod, $aSelectedPaymentMethods );
#----------------------------------------------------------------------------------------
# description  Creates new ipayment payment methods.
#----------------------------------------------------------------------------------------
# input        $PaymentMethod | ipayment payment method | PaymentMethodIpayment
# input        $aSelectedPaymentMethods | selected payment methods to create | refarray
# return       Status if creation was successful | boolean
#========================================================================================
sub generatePaymentMethods {
    my $self = shift;
    my ($PaymentMethod, $aSelectedPaymentMethods) = @_;

    return 0 unless isValid($PaymentMethod);
    return 0 unless isValid($aSelectedPaymentMethods);
    return 0 if (scalar @{ $aSelectedPaymentMethods }) < 1;

    my $PaymentTypeIpayment = $self->_LoadObjectByPath('/PaymentTypes/Ipayment');
    my $Status = 1;

    my $GeneratePaymentMethodStatus;
    foreach my $PaymentTypeMethodGUID (@$aSelectedPaymentMethods) {
        $GeneratePaymentMethodStatus = $self->_generatePaymentMethod($PaymentTypeIpayment, $PaymentMethod, $PaymentTypeMethodGUID);
        if($GeneratePaymentMethodStatus != 1) {
            $Status = 0;
        }
    }

    return $Status;
}



#========================================================================================
# function     _generatePaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _generatePaymentMethod( $PaymentTypeIpayment, $PaymentMethod, $PaymentTypeMethodGUID );
#----------------------------------------------------------------------------------------
# description  Creates a new payment method by copying the given payment method
#               and changing the PaymentTypeMethod to the selected method.
#----------------------------------------------------------------------------------------
# input        $PaymentTypeIpayment | Input1Description | PaymentTypeIpayment
# input        $PaymentMethod | payment method to copy | PaymentMethodIpayment
# input        $PaymentTypeMethodGUID | GUID of the select PaymentTypeMethod | String
# return       Status | boolean
#========================================================================================
sub _generatePaymentMethod {
    my $self = shift;
    my ($PaymentTypeIpayment, $PaymentMethod, $PaymentTypeMethodGUID) = @_;

    my $Status = 1;
    my $PaymentTypeMethod = $self->_LoadObjectByGUID($PaymentTypeMethodGUID);
    my $Shop = $PaymentMethod->getSite();

    my $NewPaymentMethodName = $PaymentMethod->get('PaymentType')->get('Name').' ('.$PaymentTypeMethod->get('NameOrAlias').')';
    my $NewPaymentMethodAlias = $NewPaymentMethodName;

    my $hAliasCorrectionInfo = undef;
    my $AliasPath = '/Shops/'.$Shop->alias().'/PaymentMethods/'.$NewPaymentMethodAlias;
    if ($self->_ExistsObjectByPath($AliasPath)) {
        my $OriginalNewPaymentMethodAlias = $NewPaymentMethodAlias;
        $NewPaymentMethodAlias = $NewPaymentMethodAlias.' '.GenerateShortGUID();
        $hAliasCorrectionInfo = {
            'OriginalAlias' => $OriginalNewPaymentMethodAlias,
            'CorrectedAlias' => $NewPaymentMethodAlias,
        };
    }

    # Parameters to copy from current payment type to new payment types
    my $hParameters = {
        'Alias' => $NewPaymentMethodAlias,
        'Name' => $NewPaymentMethodName,
        'AccountID' => $PaymentMethod->get('AccountID'),
        'UserID' => $PaymentMethod->get('UserID'),
        'UserPassword' => $PaymentMethod->get('UserPassword'),
        'AdminActionPassword' => $PaymentMethod->get('AdminActionPassword'),
        'SelectedTransactionType' => $PaymentMethod->get('SelectedTransactionType'),
        'IsInTestMode' => $PaymentMethod->get('IsInTestMode'),
        'IsIn3dSecureMode' => $PaymentMethod->get('IsIn3dSecureMode'),
        'PaymentTypeMethod' => $PaymentTypeMethod,
        'Shop' => $Shop,
        'Position' => 9999,
    };


    eval {

        $self->_Transaction( sub {

            # create new payment method
            my $NewPaymentMethod = $PaymentTypeIpayment->newPaymentMethod($Shop, $hParameters);
            my $NewPaymentTypeMethod = $NewPaymentMethod->get('PaymentTypeMethod');
            my $PaymentTypeMethodName = $NewPaymentTypeMethod->get('NameOrAlias');

            my $hTransactionTypeCorrectionInfo = $self->_correctSelectedTransactionType($NewPaymentMethod);
            my $hOption3dSecureCorrectionInfo = $self->_correctSelectedShow3dSecureMode($NewPaymentMethod);

            my $hNewPaymentMethodInfo = {
                'NewPaymentMethod' => $NewPaymentMethod,
                'NewPaymentMethodID' => $NewPaymentMethod->id(),
                'PaymentTypeMethodName' => $PaymentTypeMethodName,
                'TransactionTypeCorrection' => $hTransactionTypeCorrectionInfo,
                'Option3dSecureCorrection' => $hOption3dSecureCorrectionInfo,
                'AliasCorrection' => $hAliasCorrectionInfo,
            };
            push(@{$self->{GeneratedPaymentMethods}}, $hNewPaymentMethodInfo);

            # activate on all shipping methods (see MBO > delivery > dependencies)
            foreach my $ShippingMethod (@{$Shop->get('ShippingMethods')}) {
                $self->_InsertShippingPaymentMap({
                    'ShippingMethodID'  => $ShippingMethod->id(),
                    'PaymentMethodID'   => $NewPaymentMethod->id(),
                });
            }

        });
    };

    if (ExistsError()) {
        my $Error = GetError();
        warn ($Error->longMessage());
        $Status = 0;
    }

    return $Status;

}



#========================================================================================
# function     _correctSelectedTransactionType
# state        private
#----------------------------------------------------------------------------------------
# syntax       _correctSelectedTransactionType( $NewPaymentMethod );
#----------------------------------------------------------------------------------------
# description  Checks if the selected transaction type is possible with
#               the selected payment method type.
#               If not, the chosen transaction type will be replaced with a possible one.
#----------------------------------------------------------------------------------------
# input        $NewPaymentMethod | new created PaymentMethod | PaymentMethodIpayment
# return       Hash with information about the result of the correction process | refhash
#========================================================================================
sub _correctSelectedTransactionType {
    my $self = shift;
    my ($NewPaymentMethod) = @_;

    my $hTransactionTypeCorrectionInfo = undef;
    my $SelectedTransactionType = $NewPaymentMethod->get('SelectedTransactionType');

    my $PaymentTypeMethod = $NewPaymentMethod->get('PaymentTypeMethod');
    my $TransactionTypesJSON = $PaymentTypeMethod->get('TransactionTypes');
    my $hTransactionTypes   = from_json( $TransactionTypesJSON );
    my $aPossibleTransactionTypes = $hTransactionTypes->{'TransactionTypes'};

    my $IsSelectedTransactionTypeAllowed = $self->_doesArrayContainValue($aPossibleTransactionTypes, $SelectedTransactionType);
    if (!$IsSelectedTransactionTypeAllowed) {
        my $FirstPossibleTransactionType = $aPossibleTransactionTypes->[0];
        $NewPaymentMethod->set({
            'SelectedTransactionType' => $FirstPossibleTransactionType,
        });
        $hTransactionTypeCorrectionInfo = {
            'OriginalTransactionType' => $SelectedTransactionType,
            'CorrectedTransactionType' => $FirstPossibleTransactionType,
        };
    }

    return $hTransactionTypeCorrectionInfo;
}



#========================================================================================
# function     _correctSelectedShow3dSecureMode
# state        private
#----------------------------------------------------------------------------------------
# syntax       _correctSelectedShow3dSecureMode( $NewPaymentMethod );
#----------------------------------------------------------------------------------------
# description  Checks if the selected option 3d secure is possible with
#               the selected payment method type.
#               If not, the chosen option 3d secure will be replaced with a possible one.
#----------------------------------------------------------------------------------------
# input        $NewPaymentMethod | new created PaymentMethod | PaymentMethodIpayment
# return       Hash with information about the result of the correction process | refhash
#========================================================================================
sub _correctSelectedShow3dSecureMode {
    my $self = shift;
    my ($NewPaymentMethod) = @_;

    my $hOption3dSecureCorrectionInfo = undef;
    my $SelectedOption3dSecure = $NewPaymentMethod->get('IsIn3dSecureMode');

    my $PaymentTypeMethod = $NewPaymentMethod->get('PaymentTypeMethod');
    my $Options3dSecureJSON = $PaymentTypeMethod->get('Options3dSecure');
    my $hOptions3dSecure   = from_json( $Options3dSecureJSON );
    my $aPossibleOptions3dSecure = $hOptions3dSecure->{'Options3dSecure'};

    my $IsSelectedOption3dSecureAllowed = $self->_doesArrayContainValue($aPossibleOptions3dSecure, $SelectedOption3dSecure);
    if (!$IsSelectedOption3dSecureAllowed) {
        my $FirstPossibleOption3dSecure = $aPossibleOptions3dSecure->[0];
        $NewPaymentMethod->set({
            'IsIn3dSecureMode' => $FirstPossibleOption3dSecure,
        });
        $hOption3dSecureCorrectionInfo = {
            'OriginalOption3dSecure' => $SelectedOption3dSecure,
            'CorrectedOption3dSecure' => $FirstPossibleOption3dSecure,
        };
    }

    return $hOption3dSecureCorrectionInfo;
}



#========================================================================================
# function     _doesArrayContainValue
# state        private
#----------------------------------------------------------------------------------------
# syntax       _doesArrayContainValue( $aArray, $Value );
#----------------------------------------------------------------------------------------
# description  Returns whether the given array contains the given value or not
#----------------------------------------------------------------------------------------
# input        $aArray | an array | refarray
# input        $Value | a value | object
# return       Is the value in the array | boolean
#========================================================================================
sub _doesArrayContainValue {
    my $self = shift;
    my ($aArray, $Value) = @_;
    if ( ElementOf($Value, $aArray) ) {
        return 1;
    } else {
        return 0;
    }
}



#========================================================================================
# function     getGeneratedPaymentMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       getGeneratedPaymentMethods( );
#----------------------------------------------------------------------------------------
# description  Returns the generated payment methods
#----------------------------------------------------------------------------------------
# return       the generated payment methods | refarray
#========================================================================================
sub getGeneratedPaymentMethods {
    my $self = shift;
    return $self->{'GeneratedPaymentMethods'};
}



#----------------------------------------------------------------------------------------



#========================================================================================
# function     LoadObjectByPath
# state        private
#----------------------------------------------------------------------------------------
# description  Wrapper of core binding
#----------------------------------------------------------------------------------------
# input        $Path | path of alias names from the root object | string
# input        $RootObjectID | (optional) object id of a parent or grand parent | int
# return       $Object | object | object
#========================================================================================
sub _LoadObjectByPath {
    my $self = shift;
    my ($Path, $RootObjectID) = @_;
    return LoadObjectByPath($Path);
}



#========================================================================================
# function     LoadObjectByGUID
# state        private
#----------------------------------------------------------------------------------------
# description  Wrapper of core binding
#----------------------------------------------------------------------------------------
# input        $GUID | general unique id of the object | string
# return       $Object | object  | object
#========================================================================================
sub _LoadObjectByGUID {
    my $self = shift;
    my ($GUID) = @_;
    return LoadObjectByGUID($GUID);
}



#========================================================================================
# function     ExistsObjectByPath
# state        private
#----------------------------------------------------------------------------------------
# description  Wrapper of core binding
#----------------------------------------------------------------------------------------
# input        $Path | path of alias names from the root object | String
# input        $RootObjectID | objectId from the root object | int
# return       true if the specified object exists | boolean
#========================================================================================
sub _ExistsObjectByPath {
    my $self = shift;
    my ($Path, $RootObjectID) = @_;
    return ExistsObjectByPath($Path, $RootObjectID);
}



#========================================================================================
# function     Transaction
# state        private
#----------------------------------------------------------------------------------------
# description  Wrapper of core binding
#----------------------------------------------------------------------------------------
# input        $cCode | code block | code reference
# input        $cRollback | roll back code block (optional) | code reference
#========================================================================================
sub _Transaction {
    my $self = shift;
    my ($cCode, $cRollback) = @_;
    Transaction($cCode, $cRollback);
    return;
}



#========================================================================================
# function     InsertShippingPaymentMap
# state        private
#----------------------------------------------------------------------------------------
# syntax       InsertShippingPaymentMap($hShippingPaymentMap);
#----------------------------------------------------------------------------------------
# description  Wrapper of core binding
#----------------------------------------------------------------------------------------
# input        $hShippingPaymentMap
#========================================================================================
sub _InsertShippingPaymentMap {
    my $self = shift;
    my ($hShippingPaymentMap) = @_;
    InsertShippingPaymentMap($hShippingPaymentMap);
    return;
}

1;
