#========================================================================================
# package      DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaBase
# base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
# state        public
# description  abstract base class for the three Klarna payment methods
#========================================================================================
package DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaBase;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use strict;
use DE_EPAGES::Klarna::API::KlarnaClient qw (GetCountry);
use DE_EPAGES::Klarna::API::KlarnaCalc qw (GetEffectiveInvoiceFee);
use DE_EPAGES::Object::API::Template qw (ProcessFileTemplate);
use DE_EPAGES::Klarna::API::Constants qw (@KLARNA_AVAILABLE_COUNTRIES %KLARNA_COUNTRIES_CURRENCIES);
use DE_EPAGES::Shop::API::Constants qw (TAX_MODEL_GROSS);
use DE_EPAGES::Core::API::Log qw (LogDebug);
use DE_EPAGES::Object::API::Factory qw(InsertObject);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf isValid);
use DE_EPAGES::Shop::API::ShopUtil;

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethodKlarnaBase->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name 'Klarna'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName { return 'Klarna'; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $Package->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethodPaynet');
#               my $Object = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Adds initial node content.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object
# return       $Object | new object | object
#========================================================================================
sub insert {
    my ($Class, $hInfo, $InitialNoteTemplatePath) = @_;

    my $self = $Class->SUPER::insert($hInfo);

    # set Note (EPG-17496)
    my $FileNameContent = $ENV{'EPAGES_CARTRIDGES'} . $InitialNoteTemplatePath;
    my $PaymentType = $self->get('PaymentType');
    foreach my $hShopLanguage (@{$self->getSite()->get('Languages')}) {
        local $DE_EPAGES::Object::API::Language::LANGUAGEID = $hShopLanguage->{'LanguageID'};
        my $Note = ProcessFileTemplate($FileNameContent, $hShopLanguage->{'Code2'}, $self->tleHash);
        $self->set({
            'Name'    => $PaymentType->get('NameOrAlias', $hShopLanguage->{'LanguageID'}),
            'Note' => $Note
        }, $hShopLanguage->{'LanguageID'});
    }

    my $Shop = $self->get('Shop');
    my $ShopUtil = DE_EPAGES::Shop::API::ShopUtil->new($Shop);
    my $aMandatoryFields = ['CountryID'];
    $ShopUtil->addMandatoryBillingAddressFields($aMandatoryFields);

    return $self;
}

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars));
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               The payment method can be added to the container if the CurrencyID
#               attribute matches the currency of the container.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    return $self->checkGeneral($hVars);
}

#========================================================================================
# function     checkGeneral
# state        private
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->checkGeneral($hVars));
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               The payment method can be added to the container if the CurrencyID
#               attribute matches the currency of the container.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub checkGeneral {
    my $self = shift;
    my ($hVars) = @_;

    return 0 unless $self->SUPER::canAddToBasket($hVars);

    my $Container = $hVars->{'LineItemContainer'};
    if (defined($Container)) {
        my $Basket = $Container->parent;
        # EPG-26818: If customer orders not with his local currency - payment Klarna may not be displayed
        my $CanUseCurrency = $self->canUseBasketCurrency($Basket);
        if(!$CanUseCurrency) {
            LogDebug('Klarna', 'Can\'t use selected currency for payment');
            return 0;
        }

        if ($Basket->instanceOf('Basket')) {
            my $IsDeclinedKlarnaAddress = $Basket->get('IsDeclinedKlarnaAddress');
            if(defined($IsDeclinedKlarnaAddress) && $IsDeclinedKlarnaAddress != 0) {
                LogDebug('Klarna', 'Invalid address selected, payment denied');
                return 0;
            }
        }
    }

    return 1;
}

#========================================================================================
# function     calculatePaymentPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $PaymentPrice = $PaymentMethod->calculatePaymentPrice($Container);
#----------------------------------------------------------------------------------------
# description  Returns the payment price for the given line item container. This
#               method is used by <function lineItem> and <function recalculateLineItem>
#               unless they are overloaded. The default implementation returns 0.
#               Override this method to change the shipping price calculation of a
#               basket or order.
#               To calculate the payment costs without a lineitem container, e.g. for
#               export to product portals, also overload the method
#               <function DE_EPAGES::Shipping::API::Object::PaymentMethod::price>.
#----------------------------------------------------------------------------------------
# input        $Container | LineItemContainer | ref.hash
# return       $PaymentPrice | price of payment | money
#========================================================================================
sub calculatePaymentPrice {
    my $self = shift;
    my ($Container) = @_;

    my $pli = $Container->get('Payment');
    # if Klarna already used as payment method price will not change
    if (defined($pli)
         && $pli->instanceOf('LineItemPaymentKlarna')
         && defined($pli->get('PaymentMethod'))
         && $pli->get('PaymentMethod')->id == $self->id # EPG-22478
    ) {
        return $pli->get('LineItemPrice');
    }

    my $PriceAttributeName = $self->get('Kr_SubTotalAttrName');
    $PriceAttributeName = 'LineItemsSubTotal' unless defined $PriceAttributeName;

    # check if invoice payment is enabled and an invoice fee is defined
    if (defined(my $InvoiceFee = GetEffectiveInvoiceFee($self, $Container->get($PriceAttributeName)))) {
        # saved price is always net
        my $TaxMatrix = $Container->get('TaxMatrix');
        return $TaxMatrix->net2gross(
            $Container->get('CurrencyID'),
            $InvoiceFee, {
                'TaxModel' => $Container->get('TaxModel'),
                'TaxClass' => $TaxMatrix->defaultTaxClass,
                'TaxArea'  => $Container->get('TaxArea')
            },
            0
        );
    }
    return 0;
}
#========================================================================================
# function      default
# state         private
#----------------------------------------------------------------------------------------
# syntax        $hValues = DE_EPAGES::Klarna::API::Object::PaymentMethodKlarna->default( $Parent, $hInfo );
# example       $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description   Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#               sets default fee,currency, testmode
#----------------------------------------------------------------------------------------
# input         $Parent | parent object of the new object | object
# input         $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return        $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo, $DefaultInvoiceFee, $DefaultCurrency) = @_;

    my $Shop = $Parent->getSite;
    my $TaxMatrix = $Shop->get('TaxMatrix');

    my $InvoiceFee = $TaxMatrix->gross2net(
        $DefaultCurrency,
        $DefaultInvoiceFee, {
            'TaxModel'  => TAX_MODEL_GROSS,
            'TaxClass'  => $TaxMatrix->defaultTaxClass,
            'Precision' => 3
        }
    );

     return {
         %{$class->SUPER::default($Parent, $hInfo)},
         'Kr_CurrencyID'             => $DefaultCurrency,
         'Kr_InvoiceFee'             => $InvoiceFee,
         'IsInTestMode'              => '1',
     };
}

#========================================================================================
# function     isSupportedAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Supported = $PaymentMethod->isSupportedAddress($Address);
#----------------------------------------------------------------------------------------
# description  Function tests if an address is supported by Klarna
#----------------------------------------------------------------------------------------
# input        $Address | The address to check | ref.hash
# return       $Supported | true if the address is supported | boolean
#========================================================================================
sub isSupportedAddress {
    my $self = shift;
    my ($Address) = @_;

    return ElementOf(GetCountry($Address), @KLARNA_AVAILABLE_COUNTRIES);
}

#========================================================================================
# function     isBasketAddressValidOrUndef
# state        public
#----------------------------------------------------------------------------------------
# syntax       $valid = $PaymentMethod->isBasketAddressValidOrUndef($Basket);
#----------------------------------------------------------------------------------------
# description  Returns 1 if the customer address in the basket is undefined or is
#               defined and valid to use for Klarna
#----------------------------------------------------------------------------------------
# input        $Basket | basket object | object
# return       $ValidOrUndef | true if basket address is valid or undef | boolean
#========================================================================================
sub isBasketAddressValidOrUndef {
    my $self = shift;
    my ($Basket) = @_;

    return 1 unless(defined $Basket && defined $Basket->get('BillingAddress'));
    return $self->isSupportedAddress($Basket->get('BillingAddress'));
}

#========================================================================================
# function     canUseBasketCurrency
# state        public
#----------------------------------------------------------------------------------------
# syntax       $valid = $PaymentMethod->canUseBasketCurrency($Basket);
#----------------------------------------------------------------------------------------
# description  Returns 1 if the customer address in the basket is undefined, or is
#               defined and the country's currency is supported by Klarna in this country
#----------------------------------------------------------------------------------------
# input        $Basket | basket object | object
# return       $CanUse | true if basket currency is valid to use with Klarna | boolean
#========================================================================================
sub canUseBasketCurrency {
    my $self = shift;
    my ($Basket) = @_;

    # must return 1 if no Address or LineItemContainer is present,
    # otherwise Klarna can't be selected as default payment
    return 1 unless(defined $Basket && defined $Basket->get('BillingAddress'));

    my $LineItemContainer = $Basket->get('LineItemContainer');
    return 1 unless(defined $LineItemContainer);

    my $Country = $Basket->get('BillingAddress')->get('Country');
    # use shop country as a fallback if none was set in the billing address
    $Country = $Basket->getSite()->get('Address')->get('Country') unless defined($Country);

    my $CountryID = $Country->{'Code2'};
    my $LocalCurrency = $KLARNA_COUNTRIES_CURRENCIES{$CountryID};
    my $BasketCurrency = $LineItemContainer->get('CurrencyID');

    return (defined $LocalCurrency) && ($LocalCurrency eq $BasketCurrency);
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Funtion tests if a payment method is visible in the storefront.
#               Uses function <function DE_EPAGES::Payment::API::Object::PaymentMethod::price>
#               to evaluate if payment methode has a price (price can be 0).
#----------------------------------------------------------------------------------------
# input        $hVars | CurrencyID - alpha currency code (ISO 4217) - char(3),
#                        TaxModel - gross or net price - int                  | ref.hash
#                        LineItemContainer - the line item container object   | object
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    my $AddressIsSupported = 1;
    # EPG-26656: Klarna as default payment not working
    # EPG-27075: do not remove Klarna from classic checkout payment selection when address check fails
    my $LineItemContainer = $hVars->{'LineItemContainer'};
    if(defined($LineItemContainer) && $LineItemContainer->getSite()->get('OrderProcess') == 1) {
        $AddressIsSupported = $self->isBasketAddressValidOrUndef($LineItemContainer->parent);
    }

    return ($self->get('PaymentType')->get('IsActivated'))
        && (defined $self->get('Kr_StoreID'))
        && (defined $self->get('Kr_SecretKey'))
        && $AddressIsSupported;
}

#========================================================================================
# function      SavePeriodicCost
# state         private
#----------------------------------------------------------------------------------------
# syntax        $object = $self->SavePeriodicCost($LineItem,$PeriodicCost,$Pclass);
#----------------------------------------------------------------------------------------
# description   Function saves periodic costs for line item payment object.
#----------------------------------------------------------------------------------------
# input         $LineItem | line item payment | object
# input         $Cost | periodic cost | float
# input         $Pclass | pclass | object
# return        $object | created object | object
#========================================================================================
sub SavePeriodicCost {
    my $self = shift;
    my ($LineItem, $Cost, $Pclass) = @_;

    my $Parent = $LineItem->folder('Kr_PeriodicCosts');
    my $Alias = $Pclass->get('Alias');

    if (isValid($Alias)) {

        if (not $Parent->existsChild($Alias)) {
            # Create new pclass
            my $NewCost = InsertObject('KlarnaPeriodicCost', {
                'Parent' => $Parent,
                'Alias' => $Alias,
                'KlarnaPclass' => $Pclass,
                'PeriodicCost' => $Cost,
            });

            my $aCosts = $LineItem->get('Kr_PeriodicCosts');
            push(@$aCosts, $NewCost);
            $LineItem->set({ 'Kr_PeriodicCosts' => $aCosts });

            return $NewCost;
        }
        else {
            return $Parent->child($Alias);
        }
    }

    return undef;
}

1;
