#========================================================================================
# package      DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaHirePurchase
# base         DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaBase
# state        public
# description  object interface for PaymentMethodKlarnaHirePurchase.
#========================================================================================
package DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaHirePurchase;
use base DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaBase;

use strict;
use DE_EPAGES::Core::API::Error qw (GetError ExistsError);
use DE_EPAGES::Object::API::Language qw(GetCodeByLanguageID);
use DE_EPAGES::Object::API::Table::Country qw(GetInfoCountry);
use DE_EPAGES::ExternalPayment::API::Log qw(LogPayment);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf isValid);
use DE_EPAGES::Klarna::API::KlarnaClient qw (GetCountry);
use DE_EPAGES::Klarna::API::Constants qw (KLARNA_ACCOUNT @KLARNA_AVAILABLE_COUNTRIES %KLARNA_COUNTRIES);
use DE_EPAGES::Klarna::API::Pclass qw (GetBasketPclasses GetCountryPclasses);
use DE_EPAGES::Klarna::API::KlarnaCalc qw (CalcMonthlyAccountCost);

use constant DEFAULT_CURRENCY    => 'EUR';
use constant DEFAULT_INVOICE_FEE => 0.00;


#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $Package->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethodPaynet');
#               my $Object = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Overrides and calls PaymentMethodKlarnaBase->insert to add a Template Path
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object
# return       $Object | new object | object
#========================================================================================
sub insert {
    my ($Class, $hInfo) = @_;

    my $InitialNoteTemplatePath = '/DE_EPAGES/Klarna/Templates/InitialNoteContentFinancing.html';
    my $self = $Class->SUPER::insert($hInfo, $InitialNoteTemplatePath);
    return $self;
}

#========================================================================================
# function      recalculateLineItem
# state         public
#----------------------------------------------------------------------------------------
# syntax        $Package->recalculateLineItem($LineItem);
#----------------------------------------------------------------------------------------
# description   Recalculates lineitem. Saves default Klarna payment type if
#                it is yet missing.
#----------------------------------------------------------------------------------------
# input         $LineItem | line item | object
# return        $object | created object | object
#========================================================================================
sub recalculateLineItem {
    my $self = shift;
    my ($LineItem) = @_;

    if (!($LineItem->get('InternalMethod'))) {
        my $SelectedMethod = KLARNA_ACCOUNT;
        $LineItem->set({ 'InternalMethod' => $SelectedMethod });
    }

    $self->SUPER::recalculateLineItem($LineItem);

    my $InvoiceNo = $LineItem->get('Kr_InvoiceNo');
    return if(isValid($InvoiceNo));

    my $PaymentMethod = $LineItem->get('PaymentMethod');
    my $Container = $LineItem->get('LineItemContainer');
    my $Basket = $Container->parent;
    my $Sum = $Container->get('GrandTotal');

    # Selected periodic cost
    my $OldSelectedPeriodicCostAlias;
    if (defined $LineItem->get('KlarnaPeriodicCost')) {
        $OldSelectedPeriodicCostAlias = $LineItem->get('KlarnaPeriodicCost')->get('Alias');
    }
    $LineItem->set({ 'Kr_PeriodicCosts' => [] });

    my $aCampaigns = GetBasketPclasses($PaymentMethod, $Basket);
    return unless defined($aCampaigns);

    my $CurrencyID = $Container->get('CurrencyID');
    my $Country = '';
    #Lowest monthly payment is different in Finland and in Germany
    if ($CurrencyID eq 'EUR') {
        if (defined $Container->get('BillingCountryID')) {
            my $hCountry = GetInfoCountry($Container->get('BillingCountryID'));
            $Country = uc($hCountry->{'Code2'});
        }
        else {
            my $LanguageCode = GetCodeByLanguageID($Container->get('LanguageID'));
            $Country = uc($LanguageCode);
        }
    }

    foreach my $Pclass (@$aCampaigns) {
        if ($Container->get('GrandTotal') >= $Pclass->get('MinPurchase')) {
            my $PeriodicCost;
            my $hParams = {
                'PaymentMethod' => $PaymentMethod,
                'Sum' => $Sum,
                'Pclass' => $Pclass,
                'Country' => $Country,
                'Basket' => $Basket,
            };
            eval {
                $PeriodicCost = CalcMonthlyAccountCost($hParams);
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogPayment('KlarnaHirePurchase', 'Error in CalcMonthlyAccountCost', $Error->message);
            }
            my $NewCost = $self->SavePeriodicCost($LineItem,$PeriodicCost,$Pclass);
            $LineItem->set({'KlarnaPeriodicCost' => $NewCost}) unless(defined $LineItem->get('KlarnaPeriodicCost'));
        }
    }

    if (defined $OldSelectedPeriodicCostAlias) {
        my $Parent = $LineItem->folder('Kr_PeriodicCosts');

        if ($Parent->existsChild($OldSelectedPeriodicCostAlias)) {
            my $SelectedCost =  $Parent->child($OldSelectedPeriodicCostAlias);
            $LineItem->set({ 'KlarnaPeriodicCost' => $SelectedCost })
        }
    }

    return;
}

#========================================================================================
# function      default
# state         private
#----------------------------------------------------------------------------------------
# syntax        $hValues = DE_EPAGES::Klarna::API::Object::PaymentMethodKlarna->default( $Parent, $hInfo );
# example       $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description   Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#               sets default fee,currency, testmode
#----------------------------------------------------------------------------------------
# input         $Parent | parent object of the new object | object
# input         $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return        $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    return $class->SUPER::default($Parent, $hInfo, DEFAULT_INVOICE_FEE, DEFAULT_CURRENCY);
}

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars));
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               The payment method can be added to the container if the CurrencyID
#               attribute matches the currency of the container.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    return 0 unless $self->checkGeneral($hVars);
    my $Container = $hVars->{'LineItemContainer'};

    if (defined($Container)) {
        my $GrandTotal = $Container->get('GrandTotal');
        my $aCampaigns = GetBasketPclasses($self, $Container->parent);
        return 0 unless defined($aCampaigns);
        if($aCampaigns) {
            my $Pclass = $aCampaigns->[0];
            return 0 if($GrandTotal < $Pclass->get('MinPurchase'));
        }
    }

    return 1;
}

sub isSupportedAddress {
    my $self = shift;
    my ($Address) = @_;
    my $KlarnaCountry = GetCountry($Address);

    return 0 unless ElementOf($KlarnaCountry, @KLARNA_AVAILABLE_COUNTRIES);

    my %Countries = reverse %KLARNA_COUNTRIES;
    my $aCampaigns = GetCountryPclasses($self, $Countries{$KlarnaCountry});

    return defined($aCampaigns) && scalar(@$aCampaigns) > 0;
}

1;
