#========================================================================================
# package      DE_EPAGES::Klarna::API::TLE::KlarnaTLEHandler
# state        public
# description  TLE functions for Klarna
#========================================================================================
package DE_EPAGES::Klarna::API::TLE::KlarnaTLEHandler;

use strict;
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Factory qw ( ExistsObject LoadObject LoadRootObject );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountry );
use DE_EPAGES::Klarna::API::KlarnaCalc qw ( GetEffectiveInvoiceFee );
use DE_EPAGES::Core::API::PerlTools qw (ElementOf isValid);
use DE_EPAGES::Klarna::API::Constants qw (KLARNA_INVOICE KLARNA_ACCOUNT KLARNA_FINANCING @KLARNA_PAYMENT_METHODS);
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

#==================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------
# description  Function registers the TLE handler process.
#----------------------------------------------------------------------------------
# input        $hParams | hook arguments keys: Processor | hash reference
#==================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register($Params->{'Processor'});
}

#==================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------
# syntax       $Handler = DE_EPAGES::Klarna::API::TLE::KlarnaTLEHandler->new;
#----------------------------------------------------------------------------------
# description  Funtion creates new handler object.
#----------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#==================================================================================
sub new {
    bless {}, shift;
}

#==================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------
# description  Function registers this handle on the processor.
#----------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#==================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;

    $Processor->registerHandler('FunctionHandler', $self, 'KLARNA_LANGUAGECODE');
    $Processor->registerHandler('FunctionHandler', $self, 'KLARNA_INVOICEFEE');
    $Processor->registerHandler('FunctionHandler', $self, 'KLARNA_ISACTIVATED');
    $Processor->registerHandler('FunctionHandler', $self, 'KLARNA_CONDITIONSREGION');
    $Processor->registerHandler('FunctionHandler', $self, 'KLARNA_COUNTRYDISPLAYNAME');
    $Processor->registerHandler('FunctionHandler', $self, 'KLARNA_ISCURRENCYACTIVE');
    $Processor->registerHandler('FunctionHandler', $self, 'KLARNA_RESERVATIONURL');
    $Processor->registerHandler('FunctionHandler', $self, 'KLARNA_ORDERURL');
    $Processor->registerHandler('VariableHandler', $self, 'KLARNA_INVOICE');
    $Processor->registerHandler('VariableHandler', $self, 'KLARNA_ACCOUNT');
    $Processor->registerHandler('VariableHandler', $self, 'KLARNA_FINANCING');

    return;
}

#========================================================================================
# function     tle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->tle( $Processor, $TLEName);
# example      my $Value = $Handler->tle( $Processor, 'Name');
#----------------------------------------------------------------------------------------
# description  This function is called by tle processor to get the value of a tle
#               variable.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | return value | integer
#========================================================================================
sub tle {
    my $self = shift;
    my ( $Processor, $TLEName ) = @_;

    my %tleHandlers = (
        'KLARNA_INVOICE' => \&GetKlarnaInvoice,
        'KLARNA_ACCOUNT' => \&GetKlarnaAccount,
        'KLARNA_FINANCING'  => \&GetKlarnaFinancing,
    );

    if ( defined( $tleHandlers{$TLEName} ) ) {
        return &{ $tleHandlers{$TLEName} };
    }
    return undef;
}

#========================================================================================
# function     KLARNA_LANGUAGECODE
# state        public
#----------------------------------------------------------------------------------------
# syntax       #KLARNA_LANGUAGECODE(#LanguageID)
# example      #LOCAL("LanguageCode", #KLARNA_LANGUAGECODE(#LanguageID))
#                 #LanguageCode
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Function returns language code by numerical language code,
#               used in SF template.
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #LanguageID | array
# return       #LanguageCode | language code | char(2)
#========================================================================================
sub KLARNA_LANGUAGECODE {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($LanguageID) = @$aParams;

    my $LanguageCode = GetCodeByLanguageID($LanguageID);

    if ($LanguageCode eq 'sv') {
        $LanguageCode = 'se'; # Use country code 'se' for Swedish
    }
    elsif ($LanguageCode eq 'da') {
        $LanguageCode = 'dk'; # Use country code 'dk' for Danish
    }

    return $LanguageCode
}

#========================================================================================
# function     KLARNA_ISACTIVATED
# state        public
#----------------------------------------------------------------------------------------
# syntax       #KLARNA_ISACTIVATED( #Shop.ID )
#----------------------------------------------------------------------------------------
# description  If there is a Klarna payment method it will return the isActivated Status
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #ShopID | array
# return       $IsActivated | is the klarna payment activated | integer
#========================================================================================
sub KLARNA_ISACTIVATED {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($ShopID) = @$aParams;
    return 0 unless ExistsObject($ShopID);
    my $Shop = LoadObject($ShopID);

    my $PaymentMethods = $Shop->get('PaymentMethods');
    foreach my $PaymentMethod (@$PaymentMethods){
        if (ElementOf($PaymentMethod->alias(), @KLARNA_PAYMENT_METHODS)) {
            if ($PaymentMethod->get('IsActivated')){
                    return 1;
            };
        }
    }
    return 0;
}

#========================================================================================
# function     KLARNA_INVOICEFEE
# state        public
#----------------------------------------------------------------------------------------
# syntax       #KLARNA_INVOICEFEE(#PaymentMethod.Object, #LineItemContainer.Object)
#----------------------------------------------------------------------------------------
# description  Function returns the invoice fee as gross or net price depending on the
#               tax model. Decimal points are replaced by commas.
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #PaymentMethod, #LineItemContainer | array
# return       #InvoiceFee | invoice fee in correct tax model, decimal points replaced by commas | string
#========================================================================================
sub KLARNA_INVOICEFEE {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($PaymentMethod, $LineItemContainer) = @$aParams;
    my $TaxArea = defined($LineItemContainer) ? $LineItemContainer->get('TaxArea') : undef;
    my $TaxModel = defined($LineItemContainer) ? $LineItemContainer->get('TaxModel') : undef;

    my $PriceAttributeName = $PaymentMethod->get('Kr_SubTotalAttrName');
    $PriceAttributeName = 'LineItemsSubTotal' unless defined $PriceAttributeName;
    my $Price = defined($LineItemContainer) ? $LineItemContainer->get($PriceAttributeName) : 0;

    my $InvoiceFeeAmt = GetEffectiveInvoiceFee($PaymentMethod, $Price);
    return '0' unless defined $InvoiceFeeAmt;

    my $Shop = $PaymentMethod->getSite();
    my $TaxMatrix = $Shop->get('TaxMatrix');
    if (defined $TaxArea) {
        TestObject('TaxArea', $TaxArea, 'DE_EPAGES::Tax::API::Object::TaxArea');
    }
    else {
        $TaxArea = $TaxMatrix->defaultTaxArea;
    }
    $TaxModel //= $Shop->get('TaxModel');

    my $InvoiceFee = $TaxMatrix->net2gross(
        $PaymentMethod->get('Kr_CurrencyID'),
        $InvoiceFeeAmt, {
            'TaxModel' => $TaxModel,
            'TaxClass' => $TaxMatrix->defaultTaxClass,
            'TaxArea'  => $TaxArea
        },
        0
    );

#   $InvoiceFee =~ s/\./,/g;  # commented this line because Klarna javascript requires the value with a "dot" not "coma".
    return $InvoiceFee;
}

#========================================================================================
# function     KLARNA_CONDITIONSREGION
# state        public
#----------------------------------------------------------------------------------------
# syntax       #KLARNA_CONDITIONSREGION(#Basket)
# example      #LOCAL("ConditionsRegion", #KLARNA_CONDITIONSREGION(#Basket))
#                 #ConditionsRegion
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Returns the country code which is used to display the Klarna conditions
#               in the SF template. The code is returned in lower case.
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #Basket | array
# return       #ConditionsRegion | language code | char(2)
#========================================================================================
sub KLARNA_CONDITIONSREGION {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Basket) = @$aParams;
    my $Shop = $Basket->getSite();

    my $CountryCode;
    my $Address = $Basket->get('BillingAddress');
    if(defined($Address) && defined(my $Country = $Address->get('Country'))) {
        $CountryCode = $Country->{'Code2'};
    }
    else {
        $CountryCode = $Shop->get('Address')->get('Country')->{'Code2'};
    }

    return lc($CountryCode);
}

#========================================================================================
# function     KLARNA_COUNTRYDISPLAYNAME
# state        public
#----------------------------------------------------------------------------------------
# syntax       #KLARNA_COUNTRYDISPLAYNAME(#CountryID)
# example      #LOCAL("CountryName", #KLARNA_COUNTRYDISPLAYNAME(#CountryID))
#                 #CountryName
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Returns a string to display as a country name, containing the native and
#               the english name of the country
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #CountryID | array
# return       #CountryDisplayName | country name to display | string
#========================================================================================
sub KLARNA_COUNTRYDISPLAYNAME {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($CountryID) = @$aParams;

    my $Country = GetInfoCountry($CountryID);
    return $Country->{'English'} unless isValid($Country->{'Native'});
    return $Country->{'Native'} . ' (' . $Country->{'English'} . ')';
}

#========================================================================================
# function     KLARNA_ISCURRENCYACTIVE
# state        public
#----------------------------------------------------------------------------------------
# syntax       #KLARNA_ISCURRENCYACTIVE(#Shop.Object, #CurrencyID)
# example      #SET("CurrencyActive", #KLARNA_ISCURRENCYACTIVE(#Shop.Object, #CurrencyID))
#----------------------------------------------------------------------------------------
# description  Returns true if the given currency is activated in the shop
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #Shop.Object, #CurrencyID | array
# return       #Active | true if currency is activated | boolean
#========================================================================================
sub KLARNA_ISCURRENCYACTIVE {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop, $CurrencyID) = @$aParams;

    my $Currencies = $Shop->get('Currencies');
    foreach my $Currency (@$Currencies) {
        next if($Currency->{'CurrencyID'} ne $CurrencyID);
        return $Currency->{'IsActivated'};
    }

    return 0;
}

sub GetKlarnaInvoice { return KLARNA_INVOICE; }
sub GetKlarnaAccount { return KLARNA_ACCOUNT; }
sub GetKlarnaFinancing { return KLARNA_FINANCING; }

#========================================================================================
# function     KLARNA_RESERVATIONURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       #KLARNA_RESERVATIONURL(#IsInTestMode, #LineItemContainer.Payment.ReservationNumber)
#----------------------------------------------------------------------------------------
# description  Returns an URL to an Reservation Klarna's servers
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #IsInTestMode, #LineItemContainer.Payment.ReservationNumber | array
# return       #URL | Reservation URL | string
#========================================================================================
sub KLARNA_RESERVATIONURL {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($IsInTestMode, $InvoiceNumber) = @$aParams;

    my $Host = LoadRootObject()->get($IsInTestMode ? 'KlarnaReservationLinkTest' : 'KlarnaReservationLinkLive');
    return "$Host$InvoiceNumber";
}

#========================================================================================
# function     KLARNA_ORDERURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       #KLARNA_ORDERURL(#LineItemContainer)
#----------------------------------------------------------------------------------------
# description  Returns an URL to the order the LineItemContainer belongs to
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #Shop.Object, #LineItemContainer.Payment.Kr_InvoiceNo | array
# return       #URL | Order URL | string
#========================================================================================
sub KLARNA_ORDERURL {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($LineItemContainer) = @$aParams;

    return BuildShopUrl($LineItemContainer->parent, { 'ViewAction' => 'MBO-ViewGeneral' }, { 'Type' => 'admin' });
}

1;
