#========================================================================================
# package      DE_EPAGES::Klarna::Hooks::Basket
# base         -
# state        public
# description  Hook Functions for basket
#========================================================================================
package DE_EPAGES::Klarna::Hooks::Basket;

use strict;
use warnings;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Klarna::API::Constants qw (@KLARNA_NO_COMPANY_COUNTRIES);
use DE_EPAGES::Klarna::API::KlarnaClient qw (GetCountry GetPclass);
use DE_EPAGES::Address::API::Address qw (IsUnambiguousBasketAddress);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf isValid);

#========================================================================================
# function     OnUIBasket2Order_CheckAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax        OnUIBasket2Order_CheckAddress($hParams);
#----------------------------------------------------------------------------------------
# description  When basket is changed function checks if error message need to be show.
#----------------------------------------------------------------------------------------
# input         $hParams | hash reference of input parameters | hash reference
# return        -
#========================================================================================
sub OnUIBasket2Order_CheckAddress {
    my ($hParams) = @_;

    my $Basket = $$hParams{'Basket'};
    my $Container = $Basket->container;
    return unless defined($Container);

    my $Payment = $Container->get('Payment');
    return unless defined($Payment);

    my $PaymentMethod = $Payment->get('PaymentMethod');
    return unless defined($PaymentMethod);

    my $Servlet = $$hParams{'Servlet'};

    # EPG-27075: do not remove Klarna from classic checkout payment selection when address check fails
    my $ViewAction = $Servlet->form->value('ViewAction');
    return if(defined($ViewAction) && $ViewAction eq 'ViewAddress'); # do not perform error checks in first checkout step

    if ($PaymentMethod->instanceOf('PaymentMethodKlarnaInvoice') ||
        $PaymentMethod->instanceOf('PaymentMethodKlarnaHirePurchase') ||
        $PaymentMethod->instanceOf('PaymentMethodKlarnaFinancing')) {
        my $InternalMethod = $Payment->get('InternalMethod');
        if (defined($InternalMethod)) {
            my $CompanyError = 0;
            my $ShippingAddrError = 0;
            my $CountryError = 0;
            my $HouseNumberError = 0;
            my $Address = $Basket->get('BillingAddress');

            if (defined $Address) {
                $ShippingAddrError = !IsUnambiguousBasketAddress($Basket);
                if($PaymentMethod->instanceOf('PaymentMethodKlarnaHirePurchase') ||
                    $PaymentMethod->instanceOf('PaymentMethodKlarnaFinancing')) {
                    # EPG-27285: show error message in old checkout if customer country differs from shop country
                    $CountryError = $Address->get('Country')->{'Code2'} ne $Basket->getSite()->get('Address')->get('Country')->{'Code2'};
                }
                my $Company = $Address->get('Company');
                $CompanyError = isValid($Company)
                    && ElementOf(GetCountry($Address), @KLARNA_NO_COMPANY_COUNTRIES);

                if (!isValid($Address->get('HouseNumber'))) {
                    $HouseNumberError = 1;
                }
            }
            if ($CompanyError || $ShippingAddrError || $CountryError || $HouseNumberError) {
                $Payment->set( {'Kr_BillingAddressChanged', 0 } );
                $Servlet->vars('ObjectID', $Basket->id);
                $Servlet->vars('KlarnaBillingAddressUpdated', 1) if($CompanyError);
                $Servlet->vars('KlarnaBillingAddressUpdated', 2) if($ShippingAddrError);
                $Servlet->vars('KlarnaBillingAddressUpdated', 3) if($CountryError);
                $Servlet->vars('KlarnaBillingAddressUpdated', 4) if($HouseNumberError);

                Error( 'INVALID_FORM', {
                    'ViewVars', {
                        'FormError', 0
                    },
                    'ViewAction', 'ViewAddress'
                });
            }
        }
    }

    return;
}

#========================================================================================
# function     OnUIBasket2Order_SetPclass
# state        public
#----------------------------------------------------------------------------------------
# syntax        OnUIBasket2Order_SetPclass($hParams);
#----------------------------------------------------------------------------------------
# description  Set the Klarna pclass id on basket
#----------------------------------------------------------------------------------------
# input         $hParams | hash reference of input parameters | hash reference
# return        -
#========================================================================================
sub OnUIBasket2Order_SetPclass {
    my ($hParams) = @_;
    my $Basket = $$hParams{'Basket'};

    my $Container = $Basket->container;
    return unless defined($Container);

    my $Payment = $Container->get('Payment');
    return unless defined($Payment);

    my $PaymentMethod = $Payment->get('PaymentMethod');
    return unless defined($PaymentMethod);

    if ($Payment->instanceOf('LineItemPaymentKlarna')) {
        if(defined($Payment->get('InternalMethod'))) {
            my $Pclass = GetPclass($Container);
            $Basket->set({'KlarnaOrderPClass' => $Pclass});
        }
    }
}

1;
