#==============================================================================
# package      DE_EPAGES::Klarna::Hooks::OrderProcess
# state        private
#==============================================================================
package DE_EPAGES::Klarna::Hooks::OrderProcess;

use strict;
use DE_EPAGES::Address::API::Address qw ( IsUnambiguousBasketAddress );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::ExternalPayment::API::Constants qw ( DEFAULT_PAYMENT_STEP_POSITION );
use DE_EPAGES::Klarna::API::Constants qw ( @KLARNA_NO_COMPANY_COUNTRIES @KLARNA_PAYMENT_METHODS);
use DE_EPAGES::Klarna::API::KlarnaClient qw ( GetCountry GetPclass );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountry );

#========================================================================================
# function     OnRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# description  register payment step for checkout process
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object;
    my $Container = $BasketOrder->container;
    return unless defined $Container;

    my $pli = $Container->get('Payment');
    return unless defined($pli);

    if ($pli->instanceOf('LineItemPaymentKlarna')) {
        $BasketOrderProcess->addStep('Klarna', DEFAULT_PAYMENT_STEP_POSITION);

        # EPG-25543 Klarna: store PClass with ordered products
        if(defined($pli->get('InternalMethod'))) {
            my $Pclass = GetPclass($Container);
            $BasketOrder->set({'KlarnaOrderPClass' => $Pclass});
        }
    }
    return;
}

#========================================================================================
# function     AfterRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# description  set order step; remove payment step if necessary
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object;
    my $Container = $BasketOrder->container;
    return unless defined $Container;

    my $pli = $Container->get('Payment');
    return unless defined $pli;

    if ($pli->instanceOf('LineItemPaymentKlarna')) {
        if ($pli->get('PaymentMethod')->get('DoPaymentOnBasket')) {
            $BasketOrderProcess->setOrderStep('Klarna');
        }
    }
    else {
        $BasketOrderProcess->deleteStep('Klarna');
    }

    return;
}

#========================================================================================
# function     AfterSaveMultiCheckoutPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterSaveMultiCheckoutPayment($hParams);
#----------------------------------------------------------------------------------------
# description  check if terms and condition is accepted
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterSaveMultiCheckoutPayment{
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object;
    my $LineItemContainer = $Basket->container;
    my $Payment = $LineItemContainer->get('Payment');

    return if $Servlet->params('ViewAction') eq 'ViewMultiCheckoutPayment';

    return unless (defined($Payment) && $Payment->instanceOf('LineItemPaymentKlarna'));

    # EPG-26654: Klarna: payment from countries other than Germany not possible
    # use country code, not language code
    my $Address = $Basket->get('BillingAddress');
    return unless defined $Address;

    my $CountryID = $Address->get('CountryID');
    return unless defined $CountryID;

    my $CountryCode = GetInfoCountry($CountryID)->{'Code2'};
    return unless $CountryCode eq 'DE'; # TAC required only in DE

    my $Form = $Servlet->form;
    #Check if Terms and Conditions are checked
    if (!$Form->value('KlarnaConfirmTAC')){
        $Servlet->vars( 'ViewAction','ViewMultiCheckoutPayment');
        $Form->executeFormError({
            'Name'   => 'KlarnaTacCheckFailed',
            'Reason' => 'KlarnaTacCheckFailed',
        });
    }

    return;
}

#========================================================================================
# function     AfterSaveMultiCheckoutAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterSaveMultiCheckoutAddress($hParams);
#----------------------------------------------------------------------------------------
# description  check if shipping address can be used with Klarna
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterSaveMultiCheckoutAddress {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object();

    $Basket->set({'IsDeclinedKlarnaAddress' => 0});

    if ($Basket->instanceOf('Basket') && IsKlarnaVisible($Basket)) {
        my $IsUnambiguous = IsUnambiguousBasketAddress($Basket);

        # EPG-19557: Klarna does not support separate delivery address
        if(!$IsUnambiguous) {
            LogDebug('Klarna', 'Shipping address is ambiguous. Address declined');
            $Basket->set({'IsDeclinedKlarnaAddress' => 1});
            return;
        }

        # EPG-22480: klarna: do not allow payment for companies
        # EPG-25753: klarna do not allow payment for companies for DE and NL,Austria only
        my $BillingAddress = $Basket->get('BillingAddress');
        if (defined($BillingAddress->get('Company'))
            && ElementOf(GetCountry($BillingAddress), @KLARNA_NO_COMPANY_COUNTRIES)) {
            LogDebug('Klarna', 'Payment for companies not allowed. Address declined');
            $Basket->set({'IsDeclinedKlarnaAddress' => 2});
            return;
        }

        if ( !isValid($BillingAddress->get('HouseNumber')) ) {
            $Basket->set({'IsDeclinedKlarnaAddress' => 3});
        }
    }

    return;
}

#----------------------------------------------------------------------------------------
sub IsKlarnaVisible {
    my ($Basket) = @_;

    my $IsKlarnaVisible = 0;
    my $PaymentMethods = $Basket->get('LineItemContainer')->get('PaymentMethods');

    foreach my $PaymentMethod (@$PaymentMethods) {
        if (ElementOf($PaymentMethod->alias(), @KLARNA_PAYMENT_METHODS)) {
            $IsKlarnaVisible = 1;
        }
    }
    return $IsKlarnaVisible;
}

#========================================================================================
# function     OnBasket2Order
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2Order($hParams);
#----------------------------------------------------------------------------------------
# description  API_Basket2Order triggered, move Klarna Pclass from Basket to Order
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments | ref.hash.*
#========================================================================================
sub OnBasket2Order {
    my ($hParams) = @_;
    my $Order = $hParams->{'Order'};
    my $Basket = $hParams->{'Basket'};

    # EPG-25543 Klarna: store PClass with ordered products
    $Order->set({'KlarnaOrderPClass' => $Basket->get('KlarnaOrderPClass')}) if(defined($Basket) && defined($Order));

    return;
}

#========================================================================================
# function     AfterSaveCustomerData
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterSaveCustomerData($hParams);
#----------------------------------------------------------------------------------------
# description  UI_AfterSaveCustomerData triggered, check that the House Number exists in the address
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments | ref.hash.*
#========================================================================================
sub AfterSaveCustomerData {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket  = $Servlet->object();
    my $Shop    = $Basket->getSite();

    return if $Shop->get('OrderProcess');
    my $LineItemPaymentKlarna = _getLineItemPaymentKlarna($Basket);
    if ( $LineItemPaymentKlarna ) {
        my $Address = $Basket->get('BillingAddress');
        if ( !isValid($Address->get('HouseNumber')) ) {
            $Servlet->vars('KlarnaBillingAddressUpdated', 4);
            Error( 'INVALID_FORM', {
                'ViewVars', {
                'FormError', 0
                },
                'ViewAction', 'View'
            });
        }
    }

    return;
}

#========================================================================================
sub _getLineItemPaymentKlarna {
    my ($Basket) = @_;
    return unless my $Container = $Basket->container();
    my $LineItemPayment = $Container->get('Payment');
    return unless defined $LineItemPayment;
    return unless my $LineItemPaymentKlarna = $LineItemPayment->instanceOf('LineItemPaymentKlarna');

    return $LineItemPaymentKlarna;
}

1;
