#========================================================================================
# package      DE_EPAGES::Mediafinanz::API::Mediafinanz
# state        public
#----------------------------------------------------------------------------------------
# description  Soap functions for Mediafinanz.
#========================================================================================
package DE_EPAGES::Mediafinanz::API::Mediafinanz;
use base Exporter;

use strict;

our @EXPORT_OK = qw (
    CallSoapFunction
    GetClientAccountStatus
    RegisterClient
);

use Digest::MD5 qw (md5_hex);
use SOAP::Lite;
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf isValid);
use DE_EPAGES::CreditCheck::API::Log qw (LogCreditCheck);
use DE_EPAGES::Mediafinanz::API::Constants qw (@SEND_EMAIL_ERRORCODES MEDIAFINANZ_WS_NS MEDIAFINANZ_WS_PROXY MEDIAFINANZ_WS_URN);
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw (GetDefaultTimeOut);
use DE_EPAGES::Mediafinanz::API::SoapDeserializer;
use DE_EPAGES::Mediafinanz::API::SoapHelpers qw (Typeless);
use DE_EPAGES::Mediafinanz::API::EMail qw (SendMerchantEMail);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);

#========================================================================================
# function     CallSoapFunction
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $result = CallSoapFunction($Shop, $urn, $proxy, $ns, $clientid, $function, $param1, $param2);
# syntax       my ($result, $hError) = CallSoapFunction($Shop, $urn, $proxy, $ns, $clientid, $function, $param1, $param2);
#----------------------------------------------------------------------------------------
# description  general function to call a Mediafinanz soap function
#----------------------------------------------------------------------------------------
# input        $Shop | shop (used for error email) | object
# input        $urn | webservice urn | string
# input        $proxy | webservice proxy | string
# input        $ns | webservice namespace | string
# input        $clientid | Mediafinanz client id | string
# input        $Function | webservice function to call | string
# input        @Params | parameters for webservice function | array
# return       $result | webservice function result, undef on error | any type
# $return       $hError | only if caller expects an array, keys: Code, Message; undef on success | ref.hash
#========================================================================================
sub CallSoapFunction {
    my ($Shop, $urn, $proxy, $ns, $clientid, $Function, @Params) = @_;

    my $IsInTestMode = $Shop->get('IsInternalTestShop');

    my $client = SOAP::Lite
        ->uri($urn)
        ->proxy($proxy, 'timeout' => GetDefaultTimeOut)
        ->ns($ns);

    my $hInfo = LoadRootObject()->get([qw(Mediafinanz_PartnerID Mediafinanz_LicenceKey)]);

    my @LogParams = (
        {'auth' => {
            'partnerId'  => $hInfo->{'Mediafinanz_PartnerID'},
            'clientid'   => $clientid,
            'licenceKey' => $hInfo->{'Mediafinanz_LicenceKey'},
            'sandbox'    => $IsInTestMode,
        }},
        $urn,
        $proxy,
        @Params,
    );

    #--- webservice call
    LogCreditCheck('Mediafinanz', "calling soap function $Function with params", \@LogParams);
    my $SoapAuth;
    if (defined $clientid) {
        $SoapAuth = SOAP::Data->type('Auth')->name('auth' => [
            SOAP::Data->name('partnerId') ->type('integer')->value($hInfo->{'Mediafinanz_PartnerID'}),
            SOAP::Data->name('clientId')  ->type('string')->value($clientid),
            SOAP::Data->name('licenceKey')->type('string') ->value(md5_hex($clientid . $hInfo->{'Mediafinanz_LicenceKey'})),
            SOAP::Data->name('sandbox')   ->type('boolean')->value($IsInTestMode),
       ]);
       $client->deserializer(DE_EPAGES::Mediafinanz::API::SoapDeserializer->new);
    }
    else {
        $SoapAuth = SOAP::Data->type('Auth')->name('auth' => [
            SOAP::Data->name('partnerId') ->type('integer')->value($hInfo->{'Mediafinanz_PartnerID'}),
            SOAP::Data->name('licenceKey')->type('string') ->value($hInfo->{'Mediafinanz_LicenceKey'}),
            SOAP::Data->name('sandbox')   ->type('boolean')->value($IsInTestMode),
       ]);
    }

    my $result;
    eval {
        $result = $client->call($Function, $SoapAuth, @Params);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        return wantarray  ?  (undef, {'Code' => $Error->code, 'Message' => $Error->message}) : undef;
    }

    #--- handle result
    if ($result->fault) {
        my $ErrorCode = $result->faultcode;
        LogCreditCheck('Mediafinanz', "function $Function failed for shop " . $Shop->pathString, {
            'params'       => \@LogParams,
            'errorcode'    => $ErrorCode,
            'errormessage' => $result->faultstring
        });
        if (defined($ErrorCode) && ElementOf($ErrorCode, @SEND_EMAIL_ERRORCODES)) {
            eval {
                SendMerchantEMail($Shop);
            };
            if (ExistsError()) {
                LogError(GetError);
            }
        }
        return wantarray  ?  (undef, {'Code' => $ErrorCode, 'Message' => $result->faultstring}) : undef;
    }
    else {
        push(@LogParams, $result->result);
        LogCreditCheck('Mediafinanz', "function $Function successful for shop " . $Shop->pathString, \@LogParams);
        return wantarray  ?  ($result->result, undef) : $result->result;
    }
}

#========================================================================================
sub _CallSoapFunction {
    my ($Shop, $Function, @Params) = @_;

    return CallSoapFunction($Shop, MEDIAFINANZ_WS_URN, MEDIAFINANZ_WS_PROXY, MEDIAFINANZ_WS_NS, undef, $Function, @Params);
}

#========================================================================================
# function     RegisterClient
#----------------------------------------------------------------------------------------
# syntax       unless (RegisterClient($Shop, $hData)) {...
#----------------------------------------------------------------------------------------
# description  call soap function registerClient
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $hRegistrationData | hash with data for registration
#               keys:
#               partnerCustomerId
#               address
#               firstname
#               lastname
#               company
#               legalForm
#               street
#               postcode
#               city
#               country
#               telephone
#               fax
#               email
#               homepage
#               estimatedClaims
#               referrer
#               customerId
#               | ref.hash
# return       $ret | success? | boolean
#========================================================================================
sub RegisterClient {
    my ($Shop, $hRegistrationData) = @_;

    LogCreditCheck('Mediafinanz', 'hRegistrationData', $hRegistrationData);

    my $SoapRegistration = SOAP::Data->type('ClientRegistration')->name('clientRegistration' => [
        map { Typeless($_, $hRegistrationData->{$_}) }
            qw (partnerCustomerId address firstname lastname company legalForm street
                postcode city country telephone fax email homepage estimatedClaims
                referrer)
    ]);

    if (isValid($hRegistrationData->{'customerId'})) { # EPG-19203: send existing customer id as notice
        return _CallSoapFunction(
            $Shop,
            'registerClient',
            $SoapRegistration,
            Typeless('bankAccount', undef),
            Typeless('notice', 'MID: ' . $hRegistrationData->{'customerId'})
        );
    }
    else {
        return _CallSoapFunction($Shop, 'registerClient', $SoapRegistration);
    }
}

#========================================================================================
# function     GetClientAccountStatus
#----------------------------------------------------------------------------------------
# syntax       my $status = GetClientAccountStatus($Shop, $partnerCustomerId, $aServiceNames);
#----------------------------------------------------------------------------------------
# description  call soap function getClientAccountStatus
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $partnerCustomerId | partnerCustomerId | string
# return       $status | soap result (unknown_client|account_inactive|partner_administration_prohibited|account_active) | string
#========================================================================================
sub GetClientAccountStatus {
    my ($Shop, $partnerCustomerId) = @_;

    return _CallSoapFunction(
        $Shop,
        'getClientAccountStatus',
        Typeless('partnerCustomerId', $partnerCustomerId)
    );
}

1;
