#========================================================================================
# package      DE_EPAGES::Mollie::API::UpdateOrder
# state        public
#----------------------------------------------------------------------------------------
# description  Update order status.
#========================================================================================
package DE_EPAGES::Mollie::API::UpdateOrder;

use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Mollie::API::Constants qw (
    %EPAGES_TRANSACTION_STATUS
    %MOLLIE_TRANSACTION_STATUS
    %ORDER_STATUS
);
use DE_EPAGES::Mollie::API::UpdateOrderStatus;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByGUID );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

use strict;

use constant EPAGES_STATUS_PAID => 'STATUS_PAID';
use constant MOLLIE_STATUS_CANCELLED => 'cancelled';
use constant MOLLIE_STATUS_PAID => 'paid';
our $DELAYED_PAYMENTS = [qw (bankTransfer directdebit)];

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UpdateOrder = DE_EPAGES::Mollie::API::UpdateOrder->new( );
#----------------------------------------------------------------------------------------
# description  UpdateOrder constructor
#----------------------------------------------------------------------------------------
# return       $UpdateOrder | UpdateOrder object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless { }, $class;

    return $self;
}

sub update {
    my $self = shift;
    my ($Servlet,$PaymentLineItemGUID) = @_;

    my $PaymentLineItem = $self->_LoadObjectByGUID($PaymentLineItemGUID);
    my $TransStatus = $PaymentLineItem->get('TransStatus');
    my $LineItemContainer = $PaymentLineItem->get('LineItemContainer');
    my $BasketOrOrder = $LineItemContainer->parent();

    my $Order;
    my $hResponse;
    my $UpdateOrderStatus = $self->_CreateUpdateOrderStatus();
    my $IsCancelBacklink = '1';

    if ( $TransStatus eq EPAGES_STATUS_PAID ) {
        $Order = $self->_createCustomerOrder($Servlet, $BasketOrOrder);
        $UpdateOrderStatus->setStatus($EPAGES_TRANSACTION_STATUS{$TransStatus}, $Order->guid());
        $IsCancelBacklink = '0';
        $hResponse = $self->_setUpdateResponse($Order, $IsCancelBacklink);
    }
    else {
        my $hPaymentDetails = $self->_getPaymentDetails($PaymentLineItem);
        $self->_LogPayment('Mollie','hPaymentDetails:',$hPaymentDetails);
        my $Status = $hPaymentDetails->{'status'};
        my $ChosenPaymentMethod = $hPaymentDetails->{'method'};
        my $PaymentMethodMollie = $PaymentLineItem->get('PaymentMethod');

        my $isOrderCreatedBeforePayment = $self->_isOrderCreatedBeforePayment($PaymentMethodMollie);
        if ( $isOrderCreatedBeforePayment || $Status eq MOLLIE_STATUS_PAID) {
            $Order = $self->_createCustomerOrder($Servlet, $BasketOrOrder);
            my $OrderGUID = $Order->guid();
            $UpdateOrderStatus->setStatus($Status, $OrderGUID);
            $UpdateOrderStatus->setPaymentMethod($ChosenPaymentMethod, $OrderGUID);
            if( $Status eq MOLLIE_STATUS_PAID ) {
                $IsCancelBacklink = '0';
            }
        }
        elsif (!defined($Order) && ElementOf($ChosenPaymentMethod, $DELAYED_PAYMENTS)
               && $Status ne MOLLIE_STATUS_CANCELLED) {
            $Order = $self->_createCustomerOrder($Servlet, $BasketOrOrder);
            $IsCancelBacklink = '0';
        }

        $hResponse = $self->_setUpdateResponse($Order, $IsCancelBacklink);
    }

    return $hResponse;
}

#----------------------------------------------------------------------------------------
sub _createCustomerOrder {
    my $self = shift;
    my ($Servlet, $BasketOrOrder) = @_;

    return $BasketOrOrder unless $BasketOrOrder->instanceOf('Basket');
    return DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $BasketOrOrder);
}

#----------------------------------------------------------------------------------------
sub _getPaymentDetails {
    my $self = shift;
    my ($LineItemPayment) = @_;

    my $Shop = $LineItemPayment->get('Site');

    my $PaymentDetailsRequest = DE_EPAGES::Mollie::API::PaymentDetailsRequest->create({
      Shop   => $Shop
    });

    my $PaymentId = $LineItemPayment->get('TransID2');
    my $Token = DE_EPAGES::Mollie::API::Token->new();
    my $ApiKey = $Token->getKey($Shop);

    return $PaymentDetailsRequest->getPayment($PaymentId, $ApiKey);
}

#----------------------------------------------------------------------------------------
sub _isOrderCreatedBeforePayment {
    my $self = shift;
    my ($PaymentMethodMollie) = @_;

    return !$PaymentMethodMollie->get('DoPaymentOnBasket');
}

#----------------------------------------------------------------------------------------
sub _setUpdateResponse {
    my $self = shift;
    my ($Order, $IsCancelBacklink) = @_;

    return {
        'Order' => $Order,
        'isCancelBacklink' => $IsCancelBacklink
    };
}

#----------------------------------------------------------------------------------------
# Wrappers
#----------------------------------------------------------------------------------------
sub _CreateUpdateOrderStatus {
    my $self = shift;

    return DE_EPAGES::Mollie::API::UpdateOrderStatus->new();
}


#----------------------------------------------------------------------------------------
sub _LoadObjectByGUID {
    my $self = shift;
    my ($PaymentLineItemGUID) = @_;

    return LoadObjectByGUID($PaymentLineItemGUID);;
}

#----------------------------------------------------------------------------------------
sub _LogPayment {
    my $self = shift;
    my ( $PaymentTypeAlias, $Name, $DebugStuff ) = @_;

    LogPayment( $PaymentTypeAlias, $Name, $DebugStuff );

    return;
}

1;
