#========================================================================================
# package      DE_EPAGES::Moneybookers::API::Object::PaymentMethodMoneybookers
# state        public
# description  object interface for Payment.
#========================================================================================
package DE_EPAGES::Moneybookers::API::Object::PaymentMethodMoneybookers;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Payment::API::Table::ShopPaymentLogo qw ( InsertShopPaymentLogo GetPKeyShopPaymentLogosByShopID );
use DE_EPAGES::Core::API::PerlTools qw (ElementOf);
use DE_EPAGES::Core::API::Log qw (LogDebug);
use DE_EPAGES::Moneybookers::API::Constants qw (@SKRILL_HIDDEN_PAYMENT_METHODS);

use strict;

use constant ASTROPAY_MIN_AMOUNT => 10;
use constant ASTROPAY_METHOD_CODE => 'ACI';

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethodMoneybookers->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name 'Moneybookers'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'Moneybookers'; }

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront();
#----------------------------------------------------------------------------------------
# description  Tests if a payment method with the given currency is visible in the storefront.
#               Uses function <function DE_EPAGES::Payment::API::Object::PaymentMethod::price>
#               to evaluate if payment methode has a price (price can be 0).
#----------------------------------------------------------------------------------------
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;

    return 0 unless ($self->get('PaymentType')->get('IsActivated')
        and defined $self->get('MerchantID')
        and defined $self->get('Email')
        and defined $self->get('Secret'));

    return 1;
}

sub insert {
    my ($Package, $hInfo) = @_;
    my $Type = $hInfo->{'PaymentType'};
    $hInfo->{'SplitGatewayMethod'} = $Type->alias eq 'MoneybookersCreditCard' ? 'ACC' : $hInfo->{'SplitGatewayMethod'};
    $hInfo->{'PaymentType'} = LoadObjectByPath('/PaymentTypes/Moneybookers');

    my $self = $Package->SUPER::insert($hInfo);

    if ($Type->alias eq 'MoneybookersCreditCard'){
        my $Shop = $self->getSite();
        my @NewLogos;
        push @NewLogos, LoadObjectByPath('/PaymentTypes/Moneybookers/Logos/Visa')->id;
        push @NewLogos, LoadObjectByPath('/PaymentTypes/Moneybookers/Logos/MasterCard')->id;
        push @NewLogos, LoadObjectByPath('/PaymentTypes/Moneybookers/Logos/Diners')->id;
        push @NewLogos, LoadObjectByPath('/PaymentTypes/Moneybookers/Logos/American Express')->id; ## no critic qw(ProhibitPathConcatenation)
        push @NewLogos, LoadObjectByPath('/PaymentTypes/Moneybookers/Logos/JCB')->id;

        my @ShopPaymentLogoIDs;
        foreach my $ShopPaymentLogo (@{GetPKeyShopPaymentLogosByShopID ( $Shop->id )}) {
            push @ShopPaymentLogoIDs, $ShopPaymentLogo->{'PaymentTypeLogoID'};
        }

        #insert new Logos
        InsertShopPaymentLogo({
            ShopID            => $Shop->id,
            PaymentTypeLogoID => $_
        }) foreach (grep { not ElementOf($_, @ShopPaymentLogoIDs) } @NewLogos );
    }
    return $self;
}
#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars));
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               The payment method can be added to the container if the CurrencyID
#               attribute matches the currency of the container.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    return 0 unless $self->SUPER::canAddToBasket($hVars);
    return 0 unless $self->usableAtStorefront();

    my $Container = $hVars->{'LineItemContainer'};
    return 0 unless (defined($Container));

    my $Basket = $Container->parent();

    if ($self->_isBasketNotDeclinedAndHiddenPaymentMethod($Basket)) {
        LogDebug('Skrill-Moneybookers', 'Invalid address selected, payment denied');
        return 0;
    }

    return 0 if ($self->_isAstropayAndAmountBelowLimit($Container));
    return 1;
}

#========================================================================================
# function     _isAstropayAndAmountBelowLimit
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsAmountBelowLimit = $self->_isAstropayAndAmountBelowLimit( $Container );
#----------------------------------------------------------------------------------------
# description  check if it's an Astropay payment method below minimum basket amount allowed.
#----------------------------------------------------------------------------------------
# input        $Container | line item container object | object
# return       $IsAmountBelowLimit | result of the check | boolean
#========================================================================================
sub _isAstropayAndAmountBelowLimit {
    my $self = shift;
    my ($Container) = @_;

    return 0 unless ($self->get('SplitGatewayMethod') eq ASTROPAY_METHOD_CODE);
    return 0 if ($Container->get('GrandTotal') >= ASTROPAY_MIN_AMOUNT);
    return 1;
}

#========================================================================================
# function     _isBasketNotDeclinedAndHiddenPaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isConditionSet = $self->_isBasketNotDeclinedAndHiddenPaymentMethod( $Basket );
#----------------------------------------------------------------------------------------
# description  check if the basket is not declined and Skrill payment method is not one
#               of the included in the hidden list for Skrill.
#----------------------------------------------------------------------------------------
# input        $Basket | basket object | object
# return        $isConditionSet | basket meets contidions | boolean
#========================================================================================
sub _isBasketNotDeclinedAndHiddenPaymentMethod {
    my $self = shift;
    my ($Basket) = @_;

    return 0 unless ($Basket->instanceOf('Basket'));
    return 0 unless (defined($Basket->get('IsDeclinedSkrillAddress')));
    return 0 if ($Basket->get('IsDeclinedSkrillAddress') == 0);
    return 0 if ($self->class()->alias() ne 'PaymentMethodMoneybookers');
    return 0 unless ElementOf($self->get('SplitGatewayMethod'),
                              @SKRILL_HIDDEN_PAYMENT_METHODS);

    return 1;
}

1;
