#========================================================================================
# package      DE_EPAGES::Moneybookers::API::TLE::MoneybookersTLEHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Implements TLE function
#               <ul>
#                   <li>TwengaTrackingScript<\li>
#               <\ul>
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package DE_EPAGES::Moneybookers::API::TLE::MoneybookersTLEHandler;
use strict;
use DE_EPAGES::Object::API::Factory qw ( ExistsObject LoadObject );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

use DE_EPAGES::Moneybookers::API::Constants qw (@SPLIT_GATEWAY_METHODS @SKRILL_HIDDEN_PAYMENT_METHODS);

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               <\ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    my $TLEProcessor = $Params->{'Processor'};
    my $Handler      = __PACKAGE__->new();
    $TLEProcessor->registerHandler( 'FunctionHandler', $Handler, 'MoneybookersMapSGM' );
    $TLEProcessor->registerHandler( 'FunctionHandler', $Handler, 'SKRILL_RAT_REC_ISACTIVATED' );
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Moneybookers::API::TLE::MoneybookersTLEHandler;
# example      my $Handler  = new DE_EPAGES::Moneybookers::API::TLE::MoneybookersTLEHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     MoneybookersMapSGM
# state        public
#----------------------------------------------------------------------------------------
# syntax       #MoneybookersMapSGM(#SplitGatewayMethod)
# example      #MoneybookersMapSGM(#LineItemContainer.Payment.PaymentMethod.SplitGatewayMethod)
#----------------------------------------------------------------------------------------
# description  Function maps a internal used SplitGatewayMethod to another known by
#               moneybookers. Required for "NGP" which should be available as
#               "Online Bank Transfer" for international and as "Onlineberweisung" for
#               German use
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #SplitGatewayMethod | string
# return       $Value | an external used splitgateway method | string
#========================================================================================
sub MoneybookersMapSGM {

    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($SplitGatewayMethod) = @$aParams;

    my @retVal =  grep { ! $_->{'isGroup'} && $_->{'ID'} eq $SplitGatewayMethod } @SPLIT_GATEWAY_METHODS;

    return $retVal[0]->{'REAL_SGM'};
}

#========================================================================================
# function     SKRILL_RAT_REC_ISACTIVATED
# state        public
#----------------------------------------------------------------------------------------
# syntax       #SKRILL_RAT_REC_ISACTIVATED( #Shop.ID )
#----------------------------------------------------------------------------------------
# description  If returns true if there is a Skrill 'Kauf auf Rechnung' or 'Ratenkauf'
#               activated; otherwise, false.
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #ShopID | array
# return       $IsActivated | is the Moneybookers payment (KAR or RAT) activated | integer
#========================================================================================
sub SKRILL_RAT_REC_ISACTIVATED {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($ShopID) = @$aParams;
    return 0 unless ExistsObject($ShopID);
    my $Shop = LoadObject($ShopID);

    my $PaymentMethods = $Shop->get('PaymentMethods');
    foreach my $PaymentMethod (@$PaymentMethods){
        if ($PaymentMethod->class->alias eq 'PaymentMethodMoneybookers') {
            if (ElementOf($PaymentMethod->get('SplitGatewayMethod'),
                          @SKRILL_HIDDEN_PAYMENT_METHODS)) {
                if ( $PaymentMethod->get('IsActivated') ) {
                    return 1;
                }
            }
        }
    }
    return 0;
}

1;
