#========================================================================================
# package      DE_EPAGES::MultiStore::API::Synchronization
# base         Exporter
# state        public
# description  MultiStore - Synchronization module
#========================================================================================
package DE_EPAGES::MultiStore::API::Synchronization;
use base Exporter;

use strict;
use feature 'say';

use Config;
use JSON::XS qw( decode_json );
use XML::Simple;

use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Error qw(
    Error
    ExistsError
    GetError
);
use DE_EPAGES::Core::API::File qw (
    ConvertPathChars
    CreateSymLink
    CreateDirectory
    fsplit
    GetFileContent
    OpenFile
    LockFile
);
use DE_EPAGES::Core::API::Log qw (
    GetLog
);
use DE_EPAGES::Core::API::TestParameter qw (
    TestObject
    TestString
);
use DE_EPAGES::Object::API::Factory qw (
    LoadObjectByPath
    ExistsObjectByPath
    ExistsObject
    LoadClassByAlias
    LoadObject
    JoinObjectPath
    SplitObjectPath
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (
    ImportObjects
);
use DE_EPAGES::Product::API::Table::ProductPrice qw (
    UpdateProductPrice
);
use DE_EPAGES::Trigger::API::Trigger qw (
    TriggerHook
);
use DE_EPAGES::MultiStore::DAL::SubShopObject qw (
    DAL_GetAllInfoSubShopObjectsBySiteID
);
use DE_EPAGES::MultiStoreCommon::API::ObjectUpdate qw (
    InsertObjectUpdate
    ObjectPathToFilePath
);
use DE_EPAGES::MultiStoreCommon::API::MultiStoreLog qw (
    MultiStoreLogDebug
    MultiStoreLogDebugObjectPath
);
use DE_EPAGES::MultiStoreCommon::API::Synchronization qw (
    GetBundleProductMapsFileName
    GetDefaultValues
    GetSyncPath
    GetLastUpdateID
    GetLastUpdates
    SetLastUpdateID
    SetPeriodicUpdateID
);
use DE_EPAGES::MultiStoreCommon::API::Table::ObjectUpdate qw (
    DeleteFromObjectUpdateByShopIDObjectUpdateID
);

our @EXPORT_OK = qw (
    AddSubShopObject
    ClassAliasFromObjectPath
    GetMasterXMLDir
    GetMasterXMLFile
    GetPreDefAttributeMasterPath
    GetProductTypePreDefAttributes
    ImportProduct
    ImportProductType
    ImportSubShop
    IsAffiliateVisible
    SetProductType
);

our $PRINT_OUTPUT = 0;

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'NoMasterShop' => 'Shop #Shop is not assigned to a master shop',
    };
}
#ERRORDEFINITION END

sub _GetAffiliateObjectsByShop {
    my ($Shop) = @_;

    # Categories, Products.
    my $aResult = DAL_GetAllInfoSubShopObjectsBySiteID($Shop->id);

    # Get all local subshopproducttypes.
    my @MSProductTypes;
    foreach my $ProductType (@{$Shop->folder('ProductTypes')->childrenByClass('SubShopProductType')}) {
        push @MSProductTypes, { 'MasterPath' => $ProductType->get('MasterPath'), 'ContentObjectID' => $ProductType->id };
    }

    # Get all local subshopproductattributes.
    my @MSProductAttributes;
    my $SubShopProductAttributes = LoadClassByAlias('SubShopProductAttribute')->objectIterator('BySiteID', [$Shop->id]);
    while (<$SubShopProductAttributes>) {
        # Filter ProductClass (it is found by the statement above)
        if ($_->instanceOf('SubShopProductAttribute')) {
            push @MSProductAttributes, { 'MasterPath' => $_->get('MasterPath'), 'ContentObjectID' => $_->id, }
        }
    }

    # Merge all.
    my @Results = (@$aResult, @MSProductTypes, @MSProductAttributes);
    my %SubShopObjectMap;
    foreach my $hRow (@Results) {
        my $MasterPath = $hRow->{'MasterPath'};
        my $SubShopObjectID = $hRow->{'ContentObjectID'};
        if ( $SubShopObjectMap{$MasterPath} ) {
            push @{$SubShopObjectMap{$MasterPath}}, $SubShopObjectID;
        }
        else {
            $SubShopObjectMap{$MasterPath} = [ $SubShopObjectID ];
        }
    }

    return \%SubShopObjectMap;
}

sub _ImportContentObjects {
    my ($Shop) = @_;

    MultiStoreLogDebug('_ImportContentObjects() START');
    say '-> content objects' if $PRINT_OUTPUT;

    my $ahUpdates = GetLastUpdates($Shop);
    my $MS_LastUpdate = GetLastUpdateID($Shop);
    my $hAffiliateObjects = _GetAffiliateObjectsByShop($Shop);

    my @RC = @{GetDefaultValues()->{'REQUIRED_CLASS'}};

    # MS_LastUpdate - get ids of each scheduled update
    my $ProcessedUpdatesCount = 0;
    my %ScheduledUpdateIDs = map { $_->{'ObjectUpdateID'} => undef } @$ahUpdates;
    my @ObjectUpdateIDs = keys %ScheduledUpdateIDs;

    # Get all local ProductTypes
    my %ProductTypes = map { $_->get('MasterPath') => $_ } @{
        $Shop->folder('ProductTypes')->childrenByClass('SubShopProductType')
    };

    # Get all local PreDefAttributes of product types
    my %PreDefAttributes = ();
    my $aSubShopPreDefAttributes = GetProductTypePreDefAttributes($Shop->id);
    foreach my $SubShopPreDefAttribute (@$aSubShopPreDefAttributes) {
        my $MasterPath = GetPreDefAttributeMasterPath($SubShopPreDefAttribute);
        $PreDefAttributes{$MasterPath} = $SubShopPreDefAttribute;
    }

    # Filter updates (full updates or single value changes).
    my $haRealUpdates;
    # Loop over all updates.
    foreach my $hInfo (@$ahUpdates) {
        my $ObjectUpdateID = $MS_LastUpdate = $hInfo->{'ObjectUpdateID'};

        # MS_LastUpdate - set attribute periodically and drop id of currently processed update.
        SetPeriodicUpdateID($Shop, $ProcessedUpdatesCount++, $ObjectUpdateID, \%ScheduledUpdateIDs);
        delete $ScheduledUpdateIDs{$ObjectUpdateID};

        # Get remove flag, value, objectpath.
        my $Values = $hInfo->{'Value'};
        my $Remove = $hInfo->{'Remove'};
        my $ObjectPath = $hInfo->{'ObjectPath'};

        # Init datastructure if not already initialized.
        $haRealUpdates->{$ObjectPath} = {
            'updates' => [],
            'remove' => 0,
            'fullupdate' => 0
        } unless exists $haRealUpdates->{$ObjectPath};

        # If object marked as removed.
        if ($Remove) {
            # Clear all previous updates.
            $haRealUpdates->{$ObjectPath} = {};
            $haRealUpdates->{$ObjectPath} =  {
                remove => 1,
                updates => [],
                fullupdate => 0
            };
        } elsif ($Values) {
            # Check if there are values only for special attributes and no full import necessary.
            if (!$haRealUpdates->{$ObjectPath}->{'fullupdate'}) {
                # Add update
                push @{$haRealUpdates->{$ObjectPath}->{'updates'}}, $Values;
            }
        } else {
            # A fullupdate should be done.
            $haRealUpdates->{$ObjectPath}->{'fullupdate'} = 1;
            $haRealUpdates->{$ObjectPath}->{'updates'} = [];
        }
    }

    my @ObjectPaths = keys %$haRealUpdates;
    foreach my $ObjectPath (_SortUpdates(\@ObjectPaths)) {
        MultiStoreLogDebug("_ImportContentObjects() ObjectUpdate > $ObjectPath");
        my %SubShopObjects;
        my $SubShopObject;

        # START: SPECIAL PRODUCTTYPE HANDLING
        # Get the relevant ProductType
        $SubShopObject = $ProductTypes{$ObjectPath} if exists $ProductTypes{$ObjectPath};
        my $SubShopObjectID = defined $SubShopObject ? $SubShopObject->id : $Shop->findPath($ObjectPath);
        if (defined $SubShopObjectID) {
            $SubShopObject //= LoadObject($SubShopObjectID);
            $SubShopObjects{$SubShopObjectID} = $SubShopObject;
        }

        if( defined $SubShopObject
            and $SubShopObject->instanceOf('SubShopProductType')
            and not exists $hAffiliateObjects->{$ObjectPath}
            and not exists $hAffiliateObjects->{$SubShopObject->get('MasterPath')}
            and not ElementOf($ObjectPath, @RC) )
        {
            MultiStoreLogDebugObjectPath(
                '_ImportContentObjects() next on Special ProductType Handling > %s',
                $SubShopObject
            );
            next;
        }
        # END: SPECIAL PRODUCTTYPE HANDLING

        # BEGIN: SPECIAL PREDEFATTRIBUTE HANDLING
        $SubShopObject = $PreDefAttributes{$ObjectPath} if exists $PreDefAttributes{$ObjectPath};
        $SubShopObjectID = defined $SubShopObject ? $SubShopObject->id : $Shop->findPath($ObjectPath);
        if (defined $SubShopObjectID) {
            $SubShopObject //= LoadObject($SubShopObjectID);
            $SubShopObjects{$SubShopObjectID} = $SubShopObject;
        }
        # END: SPECIAL PREDEFATTRIBUTE HANDLING

        # Get the relevant Categories and Products
        foreach my $AffiliateObjectID (@{$hAffiliateObjects->{$ObjectPath}}) {
            if (not exists $SubShopObjects{$AffiliateObjectID} and ExistsObject($AffiliateObjectID)) {
                $SubShopObjects{$AffiliateObjectID} = LoadObject($AffiliateObjectID);
            }
        }

        my $hInfo = $haRealUpdates->{$ObjectPath};
        foreach my $SubShopObject (values %SubShopObjects) {
            if ($hInfo->{'remove'}) {
                MultiStoreLogDebugObjectPath('_ImportContentObjects() Delete object > %s', $SubShopObject);
                say '- '.$SubShopObject->get('RelativePath') if $PRINT_OUTPUT;
                if ($SubShopObject->instanceOf('SubShopProductAttribute')) {
                    # if we delete a product attribute, we do not need to sync it's pre def attributes anymore,
                    # so we remove it from the list ob subshop's pre def attributes
                    my $SubShopObjectMasterPath = $SubShopObject->get('MasterPath');
                    my $MasterPathPattern = qr!^(${SubShopObjectMasterPath}/)!;
                    for my $Path (grep {/$MasterPathPattern/} keys %PreDefAttributes) {
                        delete $PreDefAttributes{$Path};
                    }
                }
                $SubShopObject->delete;
            }

            # If a fullupdate should be triggered -> do not set value param.
            if ($hInfo->{'remove'} && $hInfo->{'fullupdate'} || $hInfo->{'fullupdate'} || !$hInfo->{'remove'} && $hInfo->{'updates'}) {
                _UpdateSubShopObject($SubShopObject, $hInfo->{'updates'});
            }
        }
    }

    # MS_LastUpdate - reset the latest value after all updates are finished
    MultiStoreLogDebug('_ImportContentObjects() DeleteFromObjectUpdateByShopIDObjectUpdateID');
    DeleteFromObjectUpdateByShopIDObjectUpdateID($Shop->id, \@ObjectUpdateIDs);
    SetLastUpdateID($Shop, 0);

    MultiStoreLogDebug('_ImportContentObjects() END');
}

sub _ImportBundleProductMaps {
    my ($Shop) = @_;

    # logging, monitoring
    MultiStoreLogDebug('_ImportBundleProductMaps() START');
    say '-> bundle product maps' if $PRINT_OUTPUT;

    # get info
    my $MasterStore = $Shop->get('MasterStore');
    my $MasterShop = $Shop->get('MasterShop');
    my $MasterBundleProductMapFileName = GetBundleProductMapsFileName($MasterStore,$MasterShop);
    my $MasterBundleProductMapFileContent;

    # get master bundle product map content
    open(my $FileHandle, '<', $MasterBundleProductMapFileName) or die "cannot open file $MasterBundleProductMapFileName";
    {
        $MasterBundleProductMapFileContent = <$FileHandle>;
    }
    close($FileHandle) or die "error while close file $MasterBundleProductMapFileName";

    # decode json content
    my $hMasterBundleProductMap = decode_json($MasterBundleProductMapFileContent);

    # get subshop bundle product maps
    my $hSubShopBundleProductMaps;
    my $hHandledBundleProducts;
    my $MapClass = LoadClassByAlias('BundleProductMap');
    my $aObjects = $MapClass->getObjects();
    my $aBundleProducts;

    # handle bundle product maps
    foreach my $BundleProductMap (@$aObjects){
        # get info
        my $BundleProduct = $BundleProductMap->get('BundleProduct');
        my $aFokusedBundleProductMaps;

        # check if bundle product is already handled
        next if $hHandledBundleProducts->{$BundleProduct->id};

        # check for siteid
        next unless ($BundleProduct->getSite()->id == $Shop->id);

        foreach my $FokusedBundleProductMap (@{$MapClass->getObjects('ByBundleProductID',[$BundleProduct->id])}){

            # get info
            my $BundledProduct = $FokusedBundleProductMap->get('BundledProduct');
            my $Quantity = $FokusedBundleProductMap->get('BundledQuantity');
            my $Position = $FokusedBundleProductMap->get('Position');

            # build result
            push(
                @$aFokusedBundleProductMaps,
                {
                    'alias' => $BundledProduct->alias,
                    'path' => $BundledProduct->pathString,
                    'quantity' => $Quantity,
                    'position' => $Position
                }
            );
        }

        push(
            @$aBundleProducts,
            {
                'alias' => $BundleProduct->alias,
                'path' => $BundleProduct->pathString,
                'bundle_products' => $aFokusedBundleProductMaps
            }
        );

        # mark bundle product as already handled
        $hHandledBundleProducts->{$BundleProduct->id} = 1;
    }

    # finalize result
    $hSubShopBundleProductMaps->{'shops'}->{$Shop->alias}->{'bundle_product_maps'} = $aBundleProducts;

    # remove redundant bundle product maps
    _UpadateExistingRemoveRedundantBundleProductMaps($Shop, $hMasterBundleProductMap, $hSubShopBundleProductMaps);

    # add missing bundle product maps
    _AddMissingBundleProductMaps($Shop, $hMasterBundleProductMap, $hSubShopBundleProductMaps);

    MultiStoreLogDebug('_ImportBundleProductMaps() END');
}

sub _UpadateExistingRemoveRedundantBundleProductMaps{
    my ($Shop, $hMasterBundleProductMap, $hSubShopBundleProductMaps) = @_;

    # get info
    my $MasterShopAlias = $Shop->get('MasterShop');
    my $SubShopAlias = $Shop->alias;
    my $MapClass = LoadClassByAlias('BundleProductMap');
    my $hMasterInfo;

    # prepare hashes
    foreach my $hBundleProduct (@{$hMasterBundleProductMap->{'shops'}->{$MasterShopAlias}->{'bundle_product_maps'}}){
        # get info
        my $Path = $hBundleProduct->{'path'};

        # for each bundle product map
        foreach my $hBundleProductMap (@{$hBundleProduct->{'bundle_products'}}){
            # save info
            $hMasterInfo->{$Path}->{$hBundleProductMap->{'path'}}->{'BundledQuantity'} = $hBundleProductMap->{'quantity'};
            $hMasterInfo->{$Path}->{$hBundleProductMap->{'path'}}->{'Position'} = $hBundleProductMap->{'position'};
        }
    }

    # check all existing sub shop bundle products
    foreach my $hBundleProduct (@{$hSubShopBundleProductMaps->{'shops'}->{$SubShopAlias}->{'bundle_product_maps'}}){
        # get info
        my $Path = $hBundleProduct->{'path'};
        my $MasterPath = $Path;
        $MasterPath  =~ s/\/$SubShopAlias\//\/$MasterShopAlias\//;
        my $BundleProduct = LoadObjectByPath($Path);

        # check if bundle product is deleted or needs update
        foreach my $BundleProductMap (@{$MapClass->getObjects('ByBundleProductID',[$BundleProduct->id])}){
            # get info
            my $BundledProduct = $BundleProductMap->get('BundledProduct');
            my $SubShopBundledProductPath = $BundledProduct->pathString;
            my $MasterBundledProductPath = $SubShopBundledProductPath;
            $MasterBundledProductPath =~ s/\/$SubShopAlias\//\/$MasterShopAlias\//;

            unless ($hMasterInfo->{$MasterPath}->{$MasterBundledProductPath}){
                # delete bundle product that not exists in master anymore
                $BundleProductMap->delete
            }
            else{
                # update bundle product that still exists in master
                $BundleProductMap->set($hMasterInfo->{$MasterPath}->{$MasterBundledProductPath});
            }
        }
    }
}

sub _AddMissingBundleProductMaps{
    my ($Shop, $hMasterBundleProductMap, $hSubShopBundleProductMaps) = @_;

    # get info
    my $MasterShopAlias = $Shop->get('MasterShop');
    my $SubShopAlias = $Shop->alias;
    my $SubShopInfo;

    # prepare hashes
    foreach my $hBundleProduct (@{$hSubShopBundleProductMaps->{'shops'}->{$SubShopAlias}->{'bundle_product_maps'}}){
        # get info
        my $Path = $hBundleProduct->{'path'};

        # for each bundle product map
        foreach my $hBundleProductMap (@{$hBundleProduct->{'bundle_products'}}){
            # save info
            $SubShopInfo->{$Path}->{$hBundleProductMap->{'path'}}->{'quantity'} = $hBundleProductMap->{'quantity'};
            $SubShopInfo->{$Path}->{$hBundleProductMap->{'path'}}->{'position'} = $hBundleProductMap->{'position'};
        }
    }

    # check all existing master shop bundle products
    foreach my $hBundleProduct (@{$hMasterBundleProductMap->{'shops'}->{$MasterShopAlias}->{'bundle_product_maps'}}){
        # get info
        my $Path = $hBundleProduct->{'path'};
        my $SubShopPath = $Path;
        $SubShopPath  =~ s/\/$MasterShopAlias\//\/$SubShopAlias\//;

        # check if bundle product is part of subshop
        next unless (ExistsObjectByPath($SubShopPath));

        # get bundle product in sub shop
        my $BundleProduct = LoadObjectByPath($SubShopPath);

        # for each bundle product map
        foreach my $hBundleProductMap (@{$hBundleProduct->{'bundle_products'}}){
            # get info
            my $MasterBundledProductPath = $hBundleProductMap->{'path'};
            my $SubShopBundledProductPath = $MasterBundledProductPath;
            $SubShopBundledProductPath =~ s/\/$MasterShopAlias\//\/$SubShopAlias\//;

            # check if bundle product map exists in sub shop, if not it has to be generated
            unless ($SubShopInfo->{$SubShopPath}->{$SubShopBundledProductPath}){
                # get info
                my $MasterBundledProductPathPart = $MasterBundledProductPath;
                $MasterBundledProductPathPart =~ s/\/Shops\/$MasterShopAlias\///;
                my $SubShopBundledProductPathPart = $SubShopBundledProductPath;
                $SubShopBundledProductPathPart =~ s/\/Shops\/$SubShopAlias\///;

                # check for variation product
                if($SubShopBundledProductPathPart =~ /\/SubProducts\//){
                    # need to import super product because subproduct is part of super product xml file
                    my $MasterSuperProductPath = $MasterBundledProductPathPart;
                    $MasterSuperProductPath =~ s/\/SubProducts\/.*//;

                    # import missing bundle product for bundle product map, in this case the super product is requested
                    _ImportObject ($Shop->get('ProductFolder'), $MasterSuperProductPath, $SubShopBundledProductPathPart, 0)
                        unless ExistsObjectByPath($SubShopBundledProductPath);
                }
                else{
                    # import missing bundle product for bundle product map
                    _ImportObject ($Shop->get('ProductFolder'), $MasterBundledProductPathPart, $SubShopBundledProductPathPart, 0)
                        unless ExistsObjectByPath($SubShopBundledProductPath);
                }

                # get imported object
                my $BundledProduct = LoadObjectByPath($SubShopBundledProductPath);

                # create new bundle product map assignment in subshop
                $BundledProduct->addToBundle(
                    $BundleProduct,
                    {
                        'BundledQuantity'=> $hBundleProductMap->{'quantity'},
                        'Position' => $hBundleProductMap->{'position'}
                    }
                );
            }
        }
    }
}

sub _SortUpdates {
    my ($aObjectPaths) = @_;

    my @SortedObjectPaths;
    foreach my $Path (@$aObjectPaths) {
        if($Path =~ /^ProductType/i ) {
            unshift @SortedObjectPaths, $Path;
        } else {
            push @SortedObjectPaths, $Path;
        }
    }

    return @SortedObjectPaths;
}

sub _UpdateSubShopObject {
    my ($SubShopObject, $aUpdates) = @_;

    MultiStoreLogDebugObjectPath('_UpdateSubShopObject() START on %s', $SubShopObject);
    my $Shop = $SubShopObject->getSite;
    my $MasterObjectPath;

    if ($SubShopObject->instanceOf('PreDefAttribute')) {
        $MasterObjectPath = GetPreDefAttributeMasterPath($SubShopObject);
    } else {
        $MasterObjectPath = $SubShopObject->get('MasterPath');
    }

    my $UpdateSubProducts = 0;

    if ($SubShopObject->instanceOf('Product')) {
        my $SuperProduct = $SubShopObject->get('SuperProduct');

        $UpdateSubProducts = 1 if $SubShopObject->get('IsMaster');
        if (not defined $MasterObjectPath and defined $SuperProduct) {
            $MasterObjectPath = $SuperProduct->get('MasterPath');
            $SubShopObject = $SuperProduct;
        }
    }

    if (not defined $MasterObjectPath) {
        MultiStoreLogDebugObjectPath('_UpdateSubShopObject() ERROR > Missing MasterPath for %s', $SubShopObject);
        die 'Missing MasterPath for '.$SubShopObject->pathString unless defined $MasterObjectPath;
    }

    my $ClassAlias = ClassAliasFromObjectPath( $MasterObjectPath );

    # Test its visiblity for this shop..
    unless ($SubShopObject->instanceOf('PreDefAttribute')) {
        my @RC = @{GetDefaultValues()->{'REQUIRED_CLASS'}};
        my $IsAffiliateVisible = ElementOf($MasterObjectPath, @RC) || IsAffiliateVisible($Shop, $MasterObjectPath, $ClassAlias);
        # ..delete from SubShop, if it isn't visible to this shop anymore
        if( !$IsAffiliateVisible ){
            MultiStoreLogDebug('_UpdateSubShopObject() Delete non visible object');
            say '- /' . $Shop->alias .'/'. $SubShopObject->get('RelativePath') if $PRINT_OUTPUT;
            $SubShopObject->delete;
            return;
        }
    }
    # Only an special attribute update
    if (defined $aUpdates && scalar @$aUpdates > 0) {
        my %Values = ();
        my @PriceChanges = ();
        foreach my $Value (@$aUpdates) {
            my $hValue = decode_json($Value);
            if (defined $hValue->{'Price'}) {
                push @PriceChanges, $hValue->{'Price'};
                next;
            }
            my %TempHash = (%Values, %$hValue);
            %Values = %TempHash;
        }
        MultiStoreLogDebug('_UpdateSubShopObject() update object');
        say '* update /' . $Shop->alias .'/'. $SubShopObject->get('RelativePath') if $PRINT_OUTPUT;
        # SpecialHandling for ProductPrices.
        foreach my $hPriceUpdate (@PriceChanges) {
            my $CurrencyID = $hPriceUpdate->{'CurrencyID'};
            my $ProductID = $SubShopObject->id;

            MultiStoreLogDebug('_UpdateSubShopObject() UpdateProductPrice on productID '.$ProductID);
            UpdateProductPrice($ProductID, $CurrencyID, $hPriceUpdate->{'Value'});
        }
        $SubShopObject->set(\%Values) if scalar (keys %Values);
    } else { # Full import
        MultiStoreLogDebug('_UpdateSubShopObject() Full import');
        my $ParentFolder = $ClassAlias eq 'Product'
            ? $Shop->get('ProductFolder')
            : $SubShopObject->parent;

        #..and import it if applicable.
        my $SubShopObjectPath = $Shop->relativePath( $SubShopObject );
        _ImportObject( $ParentFolder, $MasterObjectPath, $SubShopObjectPath );
        if ($UpdateSubProducts) {
            MultiStoreLogDebug('_UpdateSubShopObject() Update subProducts');
            my $SubProducts = $SubShopObject->folder('SubProducts')->childrenIterator;
            while (my $SubProduct = <$SubProducts>) {
                my $SubVisiblityPath = $MasterObjectPath . '/SubProducts/' . $SubProduct->alias;
                next unless -e GetMasterXMLFile( $Shop, $SubVisiblityPath );
                my $SubClassAlias = ClassAliasFromObjectPath( $SubVisiblityPath );

                # ..delete from SubShop, if it isn't visible to this shop anymore
                if( !IsAffiliateVisible($Shop, $SubVisiblityPath, $SubClassAlias) ){
                    say '- /' . $Shop->alias .'/'. $SubProduct->get('RelativePath') if $PRINT_OUTPUT;
                    $SubProduct->delete;
                }
            }
        }
    }

    MultiStoreLogDebug('_UpdateSubShopObject() Trigger Hook API_MS_UpdateSubShopObject');
    TriggerHook('API_MS_UpdateSubShopObject', {
        'Shop'              => $Shop,
        'ClassAlias'        => $ClassAlias,
        'SubShopObject'     => $SubShopObject,
        'MasterObjectPath'  => $MasterObjectPath,
    });

    MultiStoreLogDebug('_UpdateSubShopObject() END');

    return;
}

sub _SuperProductPath {
    my ($ObjectPath) = @_;
    return JoinObjectPath( [ @{SplitObjectPath($ObjectPath)}[0,1] ] );
}

#========================================================================================
# function     IsAffiliateVisible
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsAffiliateVisible($SubShop, $MasterObjectPath, $ClassAlias)
#----------------------------------------------------------------------------------------
# description  Determines the visibility of a MasterShop object for a certain SubShop.
#----------------------------------------------------------------------------------------
# input        $SubShop | SubShop object | DE_EPAGES::Shop::API::Object::Shop
# input        $MasterObjectPath | relative object path of MasterShop object, e.g. '/Categories/Tents' | String
# input        $ClassAlias | Class alias of the object, which is wanted to import into the SubShop, e.g. 'Category' | String
# return       $IsVisible | Information if wanted MasterShop object is visible for the given SubShop | Boolean
#========================================================================================
sub IsAffiliateVisible {
    my ($SubShop, $MasterObjectPath, $ClassAlias) = @_;

    $MasterObjectPath = _SuperProductPath($MasterObjectPath) if $ClassAlias eq 'Product';

    my @VLC = @{GetDefaultValues()->{'VISIBILITY_LIMITED_CLASS'}};
    return 1 unless ElementOf($ClassAlias, @VLC);

    my $FileName = GetMasterXMLFile( $SubShop, $MasterObjectPath );
    return 0 unless -e $FileName;
    my ($Path, $Name) = fsplit($FileName);
    my $rContent = GetFileContent("$Path/$Name" . '.avm', { 'flock' => 1, 'utf8' => 1 });
    my $hAVM = decode_json( $$rContent );
    my $hInfo = $hAVM->{$MasterObjectPath};

    return 0 unless defined $hInfo;
    return 0 unless $hInfo->{'IsVisible'};
    return 1 unless defined $hInfo->{'VisibleFor'};
    return 0 unless scalar @{$hInfo->{'VisibleFor'}};
    return 0 unless ElementOf($SubShop->alias, $hInfo->{'VisibleFor'});
    return 1;
}

sub _ImportOverwritableAttributes {
    my ($Shop) = @_;

    MultiStoreLogDebug('_ImportOverwritableAttributes() START');
    say '-> overwritable attributes' if $PRINT_OUTPUT;
    my $MasterStore = $Shop->get('MasterStore');
    my $MasterShop = $Shop->get('MasterShop');
    my $FilePath = join('/', GetSyncPath($MasterStore,$MasterShop), GetDefaultValues()->{'MS_OVERWRITABLES_INFO'});
    my $rContent = GetFileContent($FilePath, { 'flock' => 1, 'utf8' => 1 });
    $Shop->set({'MS_OverwritableAttributes'=> decode_json($$rContent) }) if defined $rContent and length($$rContent) > 0;
    MultiStoreLogDebug('_ImportOverwritableAttributes() END');
}

#========================================================================================
# function     ImportSubShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImportSubShop($Shop)
#----------------------------------------------------------------------------------------
# description  Imports OverwritableAttributes and ContentObjects into (Sub)Shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
#========================================================================================
sub ImportSubShop {
    my ($Shop) = @_;

    MultiStoreLogDebug('ImportSubShop(Shop: '.$Shop->alias.') START');
    say '=> Importing SubShop..' if $PRINT_OUTPUT;

    if( !defined $Shop->get('MasterStore') || !defined $Shop->get('MasterShop') ) {
        Error('NoMasterShop', { Shop => $Shop->alias });
    }

    _ImportOverwritableAttributes($Shop);
    _ImportContentObjects($Shop);
    _ImportBundleProductMaps($Shop);

    MultiStoreLogDebug('ImportSubShop() END');
}

#========================================================================================
# function     TransformImportXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       TransformImportXML($Object, $TagName, $hAttributes)
#----------------------------------------------------------------------------------------
# description  Transformation of some attributes during import of an object.
#----------------------------------------------------------------------------------------
# input        $Object | obejct | DE_EPAGES::Object::API::Object::Object
# input        $TagName | tag name | String
# input        $hAttributes | attributes of the tag | ref.hash
#========================================================================================
sub TransformImportXML {
    my ( $Object, $TagName, $hAttributes ) = @_;

    TriggerHook('API_MS_TransformImportXML', {
        'Object'      => $Object,
        'TagName'     => $TagName,
        'hAttributes' => $hAttributes,
    });

    return;
}

#========================================================================================
# function     SetProductType
# state        private
#----------------------------------------------------------------------------------------
# syntax       SetProductType($Product, $ClassAlias)
#----------------------------------------------------------------------------------------
# description  Sets a new type to a certain product.
#----------------------------------------------------------------------------------------
# input        $Product | product object | DE_EPAGES::Product::API::Object::Product
# input        $ClassAlias | Alias of the new product type. | string
#========================================================================================
sub SetProductType {
    my ($Product, $ClassAlias) = @_;

    my $ProductTypeFolder = $Product->getSite()->child('ProductTypes');
    return unless $ProductTypeFolder->existsChild($ClassAlias);

    my $ProductClassObject = $ProductTypeFolder->child($ClassAlias);
    return unless defined $ProductClassObject;

    $Product->set({ 'Class' => $ProductClassObject});
    return;
}

#========================================================================================
# function     ImportProductType
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImportProductType($Shop, $Alias, $Update);
#----------------------------------------------------------------------------------------
# description  Import a new product type.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $Alias | Alias of the new product type. | String
# input        $Update | Do an update if the type already exists. | Boolean
# return       $SubShopProductType | The imported SubShop object itself. | DE_EPAGES::MultiStore::API::Object::SubShopProductType
#========================================================================================
sub ImportProductType {
    my ($Shop, $Alias, $Update) = @_;
    TestString('Alias', $Alias);

    MultiStoreLogDebug("ImportProductType($Alias, $Update) START");

    my $ParentFolder = $Shop->get('ProductTypes');
    if ($ParentFolder->existsChild($Alias) and not defined $Update) {
        MultiStoreLogDebug("ImportProductType() SKIPPED > ProductType '$Alias' already exists and flag Update = false");
        return;
    }

    my $SubShopObjectPath = JoinObjectPath(['ProductTypes', $Alias]);

    my $MS_PRODUCT_CLASS_ALIAS = GetDefaultValues()->{'MS_PRODUCT_CLASS_ALIAS'};

    my ($MasterAlias, $MasterObjectPath);
    if( $Alias eq $MS_PRODUCT_CLASS_ALIAS ) {
        $MasterAlias = 'ProductClass';
        $MasterObjectPath = JoinObjectPath([$MasterAlias]);
    }
    else {
        ImportProductType($Shop, $MS_PRODUCT_CLASS_ALIAS);

        $MasterAlias = $Alias;
        $MasterAlias =~ s/^MS_//;
        $MasterObjectPath = JoinObjectPath(['ProductTypes', $MasterAlias]);
    }

    my $ImportedObject = _ImportObject( $ParentFolder, $MasterObjectPath, $SubShopObjectPath );

    MultiStoreLogDebug('ImportProductType() END');

    return $ImportedObject;
}

#========================================================================================
# function     ImportProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImportProduct($Shop, $MasterObjectPath);
#----------------------------------------------------------------------------------------
# description  Import a new product.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $MasterObjectPath | Relative path to MasterShop object. | String
# return       $Product | The imported SubShop product. | DE_EPAGES::MultiStore::API::Object::SubShopProduct
#========================================================================================
sub ImportProduct {
    my ($Shop, $MasterObjectPath) = @_;
    TestString('MasterObjectPath', $MasterObjectPath);

    MultiStoreLogDebug("ImportProduct($MasterObjectPath) START");

    my $ParentFolder = $Shop->get('ProductFolder');
    my $SubShopObjectPath = $MasterObjectPath = _SuperProductPath($MasterObjectPath);

    my $ImportedObject = _ImportObject( $ParentFolder, $MasterObjectPath, $SubShopObjectPath );

    MultiStoreLogDebug('ImportProduct() END');

    return $ImportedObject;
}

sub _ImportObject {
    my ($ParentFolder, $MasterObjectPath, $SubShopObjectPath, $Recursiv) = @_;
    TestString('MasterObjectPath', $MasterObjectPath);
    TestString('SubShopObjectPath', $SubShopObjectPath);

    MultiStoreLogDebugObjectPath(
        "_ImportObject(%s, $MasterObjectPath, $SubShopObjectPath, $Recursiv) START",
        $ParentFolder
    );

    my $Shop = $ParentFolder->getSite;
    my $Exists = $Shop->findPath( $SubShopObjectPath ) ? 1 : 0;
    say( ($Exists ? '* /' : '+ /' ). $Shop->alias . "/$SubShopObjectPath" ) if $PRINT_OUTPUT;

    my $FileName = GetMasterXMLFile( $Shop, $MasterObjectPath );

    OpenFile( '<', $FileName, sub {
        my ($FileHandle) = @_;
        LockFile( $FileHandle, 'r', sub {
            MultiStoreLogDebug("_ImportObject() import from file $FileName");
            eval { ImportObjects( $FileName, $ParentFolder->pathString, \&TransformImportXML ); };
            if (ExistsError()) {
                my $Error = GetError();
                MultiStoreLogDebug('_ImportObject() ERROR >>> '.$Error->message);
                if ( $Error->code eq 'NoResults_Attribute' ) {
                    if ($Recursiv) {
                        GetLog->error($Error);
                        say( ('![E] /' ). $Shop->alias . "/$SubShopObjectPath" ) if $PRINT_OUTPUT;
                    } else {
                        my $ProductTypeAlias = $Error->{'Vars'}{'Class'};
                        if ($SubShopObjectPath =~ m/\/SubProducts\//) {
                            if ($Exists) {
                                MultiStoreLogDebug('_ImportObject() OnError: get ProductType on subproducts ...');
                                $ProductTypeAlias = $Shop->path($SubShopObjectPath)->class->alias;
                            } else {
                                MultiStoreLogDebug('_ImportObject() OnError: Unexpected error. Skip object');
                                say( ('![E] /' ). $Shop->alias . "/$SubShopObjectPath" ) if $PRINT_OUTPUT;
                                return;
                            }
                        }

                        MultiStoreLogDebug('_ImportObject() OnError: ImportProductType + _ImportObject');
                        ImportProductType($Shop, $ProductTypeAlias, 1);
                        _ImportObject($ParentFolder, $MasterObjectPath, $SubShopObjectPath, 1);
                    }
                }
                else {
                    Error($Error);
                }
            }
        });
    });

    my $SubShopObject;
    if ($Shop->findPath($SubShopObjectPath)) {
        $SubShopObject = $Shop->path($SubShopObjectPath);
        $SubShopObject->set({ MasterPath => $MasterObjectPath }) unless $Exists;
        $SubShopObject->updatePageCache;

        if ($SubShopObject->instanceOf('ContentObject')){
            my $MasterObjectPublicPath = _GetMasterPublicPath($SubShopObject);
            # Create folder for category if it does not exist (avoid wrong symlinks).
            MultiStoreLogDebug("_ImportObject() CreateDirectory > $MasterObjectPublicPath");
            CreateDirectory($MasterObjectPublicPath) unless -e $MasterObjectPublicPath;

            MultiStoreLogDebug('_ImportObject() CreateSymLink to '.$SubShopObject->get('PublicPath'));
            CreateSymLink($MasterObjectPublicPath, $SubShopObject->get('PublicPath'));
        }
    }

    MultiStoreLogDebug('_ImportObject() END');

    return $SubShopObject;
}

#========================================================================================
# function     GetMasterXMLDir
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Dir = GetMasterXMLDir($Shop, $MasterObjectPath);
#----------------------------------------------------------------------------------------
# description  Detects the MasterShops XML directory from the given "MasterObjectPath".
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $MasterObjectPath | Relative path to MasterShop object. | String
# return       $Dir | The MasterShops XML directory. | String
#========================================================================================
sub GetMasterXMLDir {
    my ($Shop, $MasterObjectPath) = @_;

    my $MasterStore = $Shop->get('MasterStore');
    my $MasterShopAlias = $Shop->get('MasterShop');

    my $Dir = GetSyncPath($MasterStore, $MasterShopAlias);
    $Dir .= '/'.ObjectPathToFilePath($MasterObjectPath) if $MasterObjectPath ne '';

    return $Dir;
}

#========================================================================================
# function     GetMasterXMLFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $XMLFile = GetMasterXMLFile($Shop, $MasterObjectPath);
#----------------------------------------------------------------------------------------
# description  Detects the MasterShops object XML file from the given "MasterObjectPath".
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $MasterObjectPath | Relative path to MasterShop object. | String
# return       $XMLFile | The MasterShops objects XML file. | String
#========================================================================================
sub GetMasterXMLFile {
    my ($Shop, $MasterObjectPath) = @_;
    TestString('MasterObjectPath', $MasterObjectPath);

    my $aPathParts = SplitObjectPath($MasterObjectPath);
    my $Alias = pop @$aPathParts;
    my $ParentRelPath = JoinObjectPath($aPathParts);

    return GetMasterXMLDir($Shop, $ParentRelPath)
        .'/'. ConvertPathChars($Alias) . '.xml';
}

#========================================================================================
# function     AddSubShopObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SubShopObject = AddSubShopObject($SubShopCategory, $MasterObjectPath);
#----------------------------------------------------------------------------------------
# description  Clones a product or content object from the master shop to a sub shop.
#----------------------------------------------------------------------------------------
# input        $SubShopCategory | sub shop category | DE_EPAGES::Content::API::Object::Category
# input        $MasterObjectPath | relative object path to a product or content object in the master shop | string
# return       $SubShopObject | sub shop object | DE_EPAGES::Content::API::Object::ContentObject
#========================================================================================
sub AddSubShopObject {
    my ($SubShopCategory, $MasterObjectPath) = @_;

    MultiStoreLogDebugObjectPath("AddSubShopObject(%s, $MasterObjectPath) START", $SubShopCategory);

    TestObject('SubShopCategory', $SubShopCategory, 'DE_EPAGES::Content::API::Object::Category');
    TestString('MasterObjectPath', $MasterObjectPath);

    local $DE_EPAGES::MultiStoreCommon::API::Synchronization::IS_MULTISTORE_EXIMPORT = 1;

    my %HookParams = (
        'SubShopCategory'  => $SubShopCategory,
        'MasterObjectPath' => $MasterObjectPath,
        'SubShopObject'    => undef,
        'Done'             => 0,
    );

    MultiStoreLogDebug('AddSubShopObject() Trigger Hook API_MS_AddSubShopObject');
    TriggerHook('API_MS_AddSubShopObject', \%HookParams);

    my $Shop = $SubShopCategory->getSite;
    my $SubShopObject;

    return $HookParams{'SubShopObject'} if $HookParams{'Done'};

    my $Alias = SplitObjectPath($MasterObjectPath)->[-1];
    my $SubShopObjectPath = $Shop->relativePath($SubShopCategory) .'/'. JoinObjectPath([$Alias]);
    $SubShopObject = _ImportObject( $SubShopCategory, $MasterObjectPath, $SubShopObjectPath );

    # Add ObjectPath to MultiStoreDB.ObjectUpdate with ShopID = SubShopID.
    # The next import will run through updated Master- AND SubShop-Objects.
    MultiStoreLogDebugObjectPath('AddSubShopObject() InsertObjectUpdate on %s', $SubShopObject);
    InsertObjectUpdate($SubShopObject, 0);

    MultiStoreLogDebug('AddSubShopObject() END');

    return $SubShopObject;
}

#========================================================================================
# function     ClassAliasFromObjectPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ClassAlias = ClassAliasFromObjectPath($ObjectPath);
#----------------------------------------------------------------------------------------
# description  Detects the class alias from the given "ObjectPath".
#----------------------------------------------------------------------------------------
# input        $ObjectPath | Object path | String
# return       $ClassAlias | Class alias. | String
#========================================================================================
sub ClassAliasFromObjectPath {
    my ($ObjectPath) = @_;
    my ($Folder) = split(/\//, $ObjectPath);

    my %FolderClassMap = %{GetDefaultValues()->{'CLASS_FOLDER_MAP'}};

    foreach my $ClassAlias (%FolderClassMap) {
        foreach my $Class (@{$FolderClassMap{$ClassAlias}}) {
            return $ClassAlias if $Class eq $Folder;
        }
    }
}

sub _GetMasterPublicPath {
    my ($SubShopObject) = @_;

    my $Shop = $SubShopObject->getSite;
    my $MasterStoreDir = $Shop->get('MasterStore');
    my $MasterShopDir = ObjectPathToFilePath( $Shop->get('MasterShop') );

    my $WebRoot = $ENV{'EPAGES_WEBROOT'};
    my $FilePath = ObjectPathToFilePath( $SubShopObject->get('MasterPath') );

    return "$WebRoot/$MasterStoreDir/Shops/$MasterShopDir/$FilePath";
}

#========================================================================================
# function     GetPreDefAttributeMasterPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MasterPath = GetPreDefAttributeMasterPath( $PreDefAttribute );
#----------------------------------------------------------------------------------------
# description  Returns the MasterPath of a PreDefAttribute
#----------------------------------------------------------------------------------------
# input        $PreDefAttribute | PreDefAttribute | object
# return       $MasterPath | master path | string
#========================================================================================
sub GetPreDefAttributeMasterPath {
    my ($PreDefAttribute) = @_;
    my $PreDefAttributeAlias = $PreDefAttribute->get('Alias');

    my $Attribute = $PreDefAttribute->get('Attribute');
    my $AttributeAlias = $Attribute->get('Alias');
    my $ProductTypeMasterPath = $Attribute->get('AttributeClass')->get('MasterPath');

    my $MasterPath = "$ProductTypeMasterPath/Attributes/$AttributeAlias/PreDefAttributes/$PreDefAttributeAlias";
    return $MasterPath;
}

#========================================================================================
# function     GetProductTypePreDefAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductTypePreDefAttributes = GetProductTypePreDefAttributes( $ShopID );
#----------------------------------------------------------------------------------------
# description  Returns all predef attributes of the imported product types
#----------------------------------------------------------------------------------------
# input        $ShopID | Subshop ID | integer
# return       $aProductTypePreDefAttributes | predef attributes of subshop product types | ref.array
#========================================================================================
sub GetProductTypePreDefAttributes {
    my ($ShopID) = @_;

    # get all objects of class "PreDefAttribute" and filter them afterwards
    my $aSubShopPreDefAttributes = LoadClassByAlias('PreDefAttribute')->getObjects('BySiteID', [$ShopID]);

    # result array
    my $aProductTypePreDefAttributes;

    foreach my $SubShopPreDefAttribute (@$aSubShopPreDefAttributes) {

        # get class name of the attribute, e.g. "SubShopProductAttribute" or "CustomerAttribute"
        my $AttributeClassAlias = $SubShopPreDefAttribute->get('Attribute')->class->alias;

        # filter product type predefattributes
        # filtering is necessary because customer attributes can have predefattributes, too
        # and we don't want these
        next if ($AttributeClassAlias ne 'SubShopProductAttribute');

        push(@$aProductTypePreDefAttributes, $SubShopPreDefAttribute);
    }

    return $aProductTypePreDefAttributes;
}

1;
