#======================================================================================================================
# package      DE_EPAGES::MultiStore::API::Test::CheckImported
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------------------------------------
# description  Checks imported objects in a shop based in some metadata.
#
#               The metadata provides information (PERL data format like array, hash, scalar) to check the status of
#               synchronization: if objects were deleted, created or updated, ...
#
#               This is the format of the metada in a simple notation:
#
#               NOTATION:
#
#                   LABEL := VALUE
#
#                   Where
#
#                       LABEL       Is an (uppercase) identifier
#                       VALUE       Could be
#                                       - A description
#                                       - A final string between quotes
#                                           Like: 'value'
#                                       - Possible values between brackets and separated by '|'
#                                           Like: ( A | B | C )
#                                       - Array, 1 to n items between square brackets
#                                           Like: [ LABEL_1, ... ]
#                                       - Hash, 1 to n items between curly brackets
#                                           Like: { 'key_1' => LABEL_1, ... }
#
#               METADATA FORMAT ( See example file 't/data/CheckImported_Example.pm' ):
#
#                    EXPECTED           := [ OBJECT_INFO, ... ]
#
#                    OBJECT_INFO        := ( OBJECT_EXISTS | OBJECT_NOT_EXISTS )
#
#                    OBJECT_EXISTS      := {
#                                             'CheckType'         => CHECK_TYPE,
#                                             'CheckMsgExists'    => MSG_OBJ_EXISTS,
#                                             'CheckMsgClass'     => MSG_CLASS,
#                                             'CheckMsgSync'      => MSG_SYNC,
#                                             'Class'             => OBJECT_CLASS,
#                                             'Path'              => OBJECT_PATH,
#                                             'Attributes'        => ATTRIBUTES
#                                          }
#
#                    OBJECT_NOT_EXISTS  := {
#                                             'CheckType'         => CHECK_TYPE,
#                                             'CheckMsgExists'    => MSG_OBJ_NOT_EXISTS,
#                                             'Path'              => OBJECT_PATH
#                                          }
#
#                    CHECK_TYPE         := ( 'Exists' | 'NotFound' )
#
#                    ATTRIBUTES         := [ ATTRIBUTE_INFO, ... ]
#
#                    ATTRIBUTE_INFO     := {
#                                             'Type'      => ATTRIBUTE_TYPE,
#                                             'Name'      => ATTRIBUTE_NAME,
#                                             'Value'     => ATTRIBUTE_VALUE,
#                                          }
#
#                    ATTRIBUTE_TYPE     := ( 'scalar' | 'object' )
#
#                    ATTRIBUTE_NAME     := String with expected attribute alias
#                                           E.g. 'StockLevel'
#
#                    ATTRIBUTE_VALUE    := TEXT | NUMBER | OBJECT_VALUE
#
#                    OBJECT_VALUE       := [ OBJECT_ALIAS, ... ]
#
#                    OBJECT_ALIAS       := String with alias of an existing object
#                                           E.g. 'lt_0401107001'
#
#                    MSG_OBJ_EXISTS     := Print this test message if object exists
#
#                    MSG_OBJ_NOT_EXISTS := Print this test message if object doesn't exist
#
#                    MSG_CLASS          := Print this test message if object class is ok
#
#                    MSG_SYNC           := Print this test message if attributes is synchronized
#                                          Include '%s' to replace it with ATTRIBUTE_INFO['Name']
#                                          E.g.  'The attribute "%s" was synchronized'
#                                                Output > The attribute "StockLevel" was synchronized
#
#                    OBJECT_CLASS       := String with expected class name for an object instance
#                                          E.g.  'ProductClass'
#
#                    OBJECT_PATH        := String with object relative path inside the shop
#                                          E.g.  'Products/lt_0401107001'
#
#                    TEXT               := Perl string
#
#                    NUMBER             := Perl number
#
#======================================================================================================================
package DE_EPAGES::MultiStore::API::Test::CheckImported;
use base Exporter;

use strict;

use Test::More;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
);

use DE_EPAGES::MultiStore::API::Test::HelperFunctions qw (
    RunTestOnStore
    GetTestLanguageID
    Output
);

our @EXPORT_OK = qw (
    CheckImportedObjects
);

#======================================================================================================================
# function     CheckImportedObjects
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       CheckImportedObjects($hSubShopInfo, $aExpected);
#----------------------------------------------------------------------------------------------------------------------
# description  Checks the status of an epages object according the provided metadata.
#----------------------------------------------------------------------------------------------------------------------
# input        $hSubShopInfo | SubShop info (storename, shop alias, ...) | hash.ref
# input        $aExpected | Metadata with expected object status | array.ref
#======================================================================================================================
sub CheckImportedObjects {
    my ($hSubShopInfo, $aExpected) = @_;

    Output('CheckImportedObjects');
    RunTestOnStore(
        $hSubShopInfo->{'Storename'},
        sub {
            my $ShopPath = '/Shops/'.$hSubShopInfo->{'ShopAlias'};
            my $LanguageID = GetTestLanguageID();
            foreach my $hExpectedItem (@$aExpected) {
                my $Type = $hExpectedItem->{'CheckType'};
                if ($Type eq 'Exists') {
                    CheckImportedObjectExists($ShopPath, $hExpectedItem, $LanguageID);
                } elsif ($Type eq 'NotFound') {
                    CheckImportedObjectNotFound($ShopPath, $hExpectedItem);
                }
            }
        }
    );
}

#======================================================================================================================
# CheckImportedObjects
#======================================================================================================================
sub CheckImportedObjectExists {
    my ($ShopPath, $hExpectedItem, $LanguageID) = @_;

    my $ObjectPath = $ShopPath.'/'.$hExpectedItem->{'Path'};
    if (ExistsObjectByPath($ObjectPath)) {
        pass($hExpectedItem->{'CheckMsgExists'});
        CheckImportedObject($hExpectedItem, LoadObjectByPath($ObjectPath), $LanguageID);
    } else {
        fail($hExpectedItem->{'CheckMsgExists'});
    }

    return;
}

#======================================================================================================================
# CheckImportedObjects
#======================================================================================================================
sub CheckImportedObjectNotFound {
    my ($ShopPath, $hExpectedItem) = @_;

    my $ObjectPath = $ShopPath.'/'.$hExpectedItem->{'Path'};
    is( ExistsObjectByPath($ObjectPath),
        0,
        $hExpectedItem->{'CheckMsgExists'}
    );

    return;
}

#======================================================================================================================
# CheckImportedObjects
#======================================================================================================================
sub CheckImportedObject {
    my ($hExpectedItem, $Object, $LanguageID) = @_;

    my $ObjectClassName = $hExpectedItem->{'Class'};
    if (defined $ObjectClassName) {
        is( $Object->class->alias,
            $ObjectClassName,
            $hExpectedItem->{'CheckMsgClass'}
        );
    }
    foreach my $hAttribute (@{$hExpectedItem->{'Attributes'}}) {
        my $Value = $Object->get($hAttribute->{'Name'}, $LanguageID);
        my $Type = $hAttribute->{'Type'};
        if ($Type eq 'scalar') {
            CheckScalarAttribute($hExpectedItem, $hAttribute, $Value);
        } elsif ($Type eq 'object') {
            CheckObjectAttribute($hExpectedItem, $hAttribute, $Value);
        }
    }

    return;
}

#======================================================================================================================
# CheckImportedObjects
#======================================================================================================================
sub CheckScalarAttribute {
    my ($hExpectedItem, $hAttribute, $Value) = @_;

    is( $Value,
        $hAttribute->{'Value'},
        sprintf($hExpectedItem->{'CheckMsgSync'}, $hAttribute->{'Name'})
    );

    return;
}

#======================================================================================================================
# CheckImportedObjects
#======================================================================================================================
sub CheckObjectAttribute {
    my ($hExpectedItem, $hAttribute, $Value) = @_;

    if (ref($Value) ne 'ARRAY') {
        $Value = [ $Value ];
    }
    my $aExpectedAliases = $hAttribute->{'Value'};
    foreach my $Object (@$Value) {
        if(not ElementOf($Object->alias, $aExpectedAliases)) {
            my $SyncMsg = sprintf($hExpectedItem->{'CheckMsgSync'}, $hAttribute->{'Name'});
            fail($SyncMsg.' : Assigned object > '.$Object->alias);
            return;
        }
    }
    pass(sprintf($hExpectedItem->{'CheckMsgSync'}, $hAttribute->{'Name'}));

    return;
}

1;
