#======================================================================================================================
# package      DE_EPAGES::MultiStore::API::Test::HelperFunctions
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------------------------------------
# description  Helper functions for Synchronization integration tests
#======================================================================================================================
package DE_EPAGES::MultiStore::API::Test::HelperFunctions;
use base Exporter;

use strict;

use Test::More;

use diagnostics;

use DE_EPAGES::Database::API::Connection qw (
    RunOnStore
);
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
);
use DE_EPAGES::Core::API::Error qw (
    ExistsError
    GetError
);
use DE_EPAGES::Core::API::Warning qw (
    Warning
);
use DE_EPAGES::WebInterface::API::MessageCenter qw (
    SynchronizeCache
);
use DE_EPAGES::DataCache::API::Cache qw (
    UpdateLocal
);
use DE_EPAGES::Object::API::Table::Locale qw (
    GetLanguageByLocaleID
);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

my $_Verbose = 'none';
my $_LanguageID = 2;

our @EXPORT_OK = qw (
    RunTestOnStore
    SyncCache
    SetTestLanguageID
    GetTestLanguageID
    SetVerbose
    GetVerbose
    DeleteObject
    LoadDataFromFile
    Output
);

#======================================================================================================================
# function     RunTestOnStore
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       RunTestOnStore($Storename, $TestCode);
#----------------------------------------------------------------------------------------------------------------------
# description  Runs code in a database connection
#----------------------------------------------------------------------------------------------------------------------
# input        $Storename | Store name for connection | string
# input        $TestCode | Code to run inside the store connection | sub
#======================================================================================================================
sub RunTestOnStore {
    my ($Storename, $TestCode) = @_;

    RunOnStore(
        Store   => $Storename,
        Sub     => $TestCode
    );
    SyncCache();
    UpdateLocal();

    return;
}

#======================================================================================================================
# function     SyncCache
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       SyncCache();
#----------------------------------------------------------------------------------------------------------------------
# description  Send sync cache event to Message center (update AppServers)
#======================================================================================================================
sub SyncCache {

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    return;
}

#======================================================================================================================
# function     SetTestLanguageID
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       SetTestLanguageID($Storename);
#----------------------------------------------------------------------------------------------------------------------
# description  Sets the test LanguageID getting the ID from a specified Storename connection
#----------------------------------------------------------------------------------------------------------------------
# input        $Storename | Storename for connection | string
#======================================================================================================================
sub SetTestLanguageID {
    my ($Storename) = @_;

    RunTestOnStore(
        $Storename,
        sub { $_LanguageID = GetLanguageByLocaleID('en_GB'); }
    );

    return;
}

#======================================================================================================================
# function     GetTestLanguageID
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $LanguageID = GetTestLanguageID();
#----------------------------------------------------------------------------------------------------------------------
# description  Getter for the LanguageID for all the tests
#----------------------------------------------------------------------------------------------------------------------
# return       $LanguageID | The test language ID | integer
#======================================================================================================================
sub GetTestLanguageID {

    return $_LanguageID;
}

#======================================================================================================================
# function     SetVerbose
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       SetVerbose($Verbose);
#----------------------------------------------------------------------------------------------------------------------
# description  Enable/Disable output
#----------------------------------------------------------------------------------------------------------------------
# input        $Verbose | Enables (1) or Disables (0) the output | boolean
#======================================================================================================================
sub SetVerbose {
    my ($Verbose) = @_;

    if (not ElementOf($Verbose, qw(none info all)) ) {
        die("ERROR: Wrong verbose value '$Verbose' (see help)")
    }
    $_Verbose = $Verbose;

    return;
}

#======================================================================================================================
# function     GetVerbose
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $Verbose = GetVerbose();
#----------------------------------------------------------------------------------------------------------------------
# description  Getter for the current Verbose value
#----------------------------------------------------------------------------------------------------------------------
# return       $Verbose | Current verbose value | boolean
#======================================================================================================================
sub GetVerbose {

    return $_Verbose;
}

#======================================================================================================================
# function     DeleteObject
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       DeleteObject($ObjectPath);
#----------------------------------------------------------------------------------------------------------------------
# description  Deletes an object (if exists)
#----------------------------------------------------------------------------------------------------------------------
# input        $ObjectPath | Path to the object to delete | string
#----------------------------------------------------------------------------------------------------------------------
# return       $Installer | cartridge installer | object
#======================================================================================================================
sub DeleteObject {
    my ($ObjectPath) = @_;

    if (ExistsObjectByPath($ObjectPath)) {
        output(" - DeleteObject : $ObjectPath");
        LoadObjectByPath($ObjectPath)->delete();
    }

    return;
}

#======================================================================================================================
# function     LoadDataFromFile
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $Data = LoadDataFromFile($DataFileName);
#----------------------------------------------------------------------------------------------------------------------
# description  Loads a perl file content as data (array, hash, ...)
#----------------------------------------------------------------------------------------------------------------------
# input        $DataFileName | Perl file name | string
#----------------------------------------------------------------------------------------------------------------------
# return       $Data | Loaded data | data
#======================================================================================================================
sub LoadDataFromFile {
    my ($DataFileName) = @_;

    my $FilePath = sprintf('%s/DE_EPAGES/MultiStore/API/Test/Data/%s.pm', $ENV{'EPAGES_CARTRIDGES'}, $DataFileName);

    return do($FilePath);
}

#======================================================================================================================
# function     Output
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       Output($Format, ...);
#----------------------------------------------------------------------------------------------------------------------
# example      Output("Hello world!");
# example      Output("Prints a string '%s' and a number %d\n", 'text', 1234);
#----------------------------------------------------------------------------------------------------------------------
# description  If Verbose is enabled prints output
#----------------------------------------------------------------------------------------------------------------------
# input        $Format | Format string used like sprintf arguments ... | string
#======================================================================================================================
sub Output {
    my (@aArgs) = @_;

    if ($_Verbose ne 'none') {
        note(sprintf(shift(@aArgs), @aArgs));
    }

    return;
}
1;
