#========================================================================================
# package      DE_EPAGES::MultiStore::Hooks::CSVImportObject
# state        public
# description  Check if the object is importable via csv
#========================================================================================
package DE_EPAGES::MultiStore::Hooks::CSVImportObject;

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

use DE_EPAGES::MultiStoreCommon::API::MultiStoreLog qw (
    MultiStoreLogDebug
);

#========================================================================================
# function     OnBeforeCSVImportObject
#----------------------------------------------------------------------------------------
# syntax       OnBeforeCSVImportObject( $hParams );
#----------------------------------------------------------------------------------------
# description  Sets flag to abort import of SubShopProducts.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of object Cartridge, key is
#               <ul>
#                   <li>Object - if object already exists it contains this object - object</li>
#                   <li>Class - class of the imported object - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeCSVImportObject {
    my ($hParams) = @_;

    return unless _IsCSVImportFilterEnabled();

    my $Object = $hParams->{'Object'};
    if (defined $Object) {
      if ($Object->instanceOf('SubShopProduct')) {
        _OnBeforeCSVImportSubShopProduct($hParams);
      } elsif($Object->instanceOf('SubShopProductType')) {
        _log('DoNotImport Master ProductType', $hParams->{'LineNumber'});
        $hParams->{'DoNotImport'} = 1;
      }
    } else {
        my $Class = $hParams->{'Class'};
        if (defined $Class && $Class->instanceOf('SubShopProductType')) {
            _log('DoNotImport Master ProductType', $hParams->{'LineNumber'});
            $hParams->{'DoNotImport'} = 1;
        }
    }

    return;
}

#========================================================================================
sub _IsCSVImportFilterEnabled {
    return LoadRootObject()->get('MultiStore_IsCSVImportFilterEnabled');
}

#========================================================================================
sub _OnBeforeCSVImportSubShopProduct {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Object'}->getSite();
    my $ProductOverwritableAttributes = $Shop->get('MS_OverwritableAttributes')->{'Product'};
    if ((defined $ProductOverwritableAttributes) && (scalar @$ProductOverwritableAttributes) > 0) {
        _processImportSubShopProduct($hParams, $ProductOverwritableAttributes);
    } else {
        _log('DoNotImport because no Overwritable Attributes allowed', $hParams->{'LineNumber'});
        $hParams->{'DoNotImport'} = 1;
    }

    return;
}

#========================================================================================
sub _processImportSubShopProduct {
    my ($hParams, $ProductOverwritableAttributes) = @_;

    _log('Process imported master object', $hParams->{'LineNumber'});
    _processListPrice($hParams, $ProductOverwritableAttributes);
    _processDirectAttributes($hParams, $ProductOverwritableAttributes);
    _processLocalizedAttributes($hParams, $ProductOverwritableAttributes);
    _processPriceAttributes($hParams, $ProductOverwritableAttributes);

    return;
}

#========================================================================================
sub _processListPrice {
    my ($hParams, $ProductOverwritableAttributes) = @_;

    # Ugly workaround: The listPrice attribute name doesn't from the MS list to the real attribute (!)
    if (ElementOf('ListPrice', $ProductOverwritableAttributes)) {
        push (@$ProductOverwritableAttributes, 'ListPrices');
    }

    return;
}

#========================================================================================
sub _processDirectAttributes {
    my ($hParams, $ProductOverwritableAttributes) = @_;

    # Remove the 'only-read' attributes from the direct attributes list
    my $hDirectAttributes = $hParams->{'DirectAttributes'};
    foreach my $AttributeName (keys %$hDirectAttributes) {
        if (not ElementOf($AttributeName, $ProductOverwritableAttributes)) {
            delete $hDirectAttributes->{$AttributeName};
        }
    }

    return;
}

#========================================================================================
sub _processLocalizedAttributes {
    my ($hParams, $ProductOverwritableAttributes) = @_;

    # Remove the 'only-read' attributes from the localized attributes list
    my $hLocalizedAttributes = $hParams->{'LocalizedAttributes'};
    foreach my $LanguageID (keys %$hLocalizedAttributes) {
        my $hInfo = $hLocalizedAttributes->{$LanguageID};
        foreach my $AttributeName (keys %$hInfo) {
            if (not ElementOf($AttributeName, $ProductOverwritableAttributes)) {
                delete $hInfo->{$AttributeName};
            }
        }
    }

    return;
}

#========================================================================================
sub _processPriceAttributes {
    my ($hParams, $ProductOverwritableAttributes) = @_;

    # Remove the 'only-read' attributes from the prices attributes list
    my $hPriceAttributes = $hParams->{'PriceAttributes'};
    foreach my $Currency (keys %$hPriceAttributes) {
        my $hInfo = $hPriceAttributes->{$Currency};
        my $AttributeName = $hInfo->{Alias} ;
        if (not ElementOf($AttributeName, $ProductOverwritableAttributes)) {
            delete $hPriceAttributes->{$Currency};
        }
    }

    return;
}

#========================================================================================
sub _log {
    my ($Message, $LineNumber) = @_;

    if (defined $LineNumber) {
        $Message .= " (line $LineNumber)";
    }
    MultiStoreLogDebug("OnBeforeCSVImportObject: $Message");

    return;
}

1;
