#========================================================================================
# package      DE_EPAGES::MultiStore::Hooks::ContentObject
# state        public
# description  Listens to events on objects and triggers appropriate functions.
#========================================================================================
package DE_EPAGES::MultiStore::Hooks::ContentObject;

use strict;
use feature 'say';
use XML::Simple;

use DE_EPAGES::Catalog::API::Catalog qw (
    AssignProductToCategory
    RemoveProductFromCategory
);
use DE_EPAGES::Core::API::File qw (
    fsplit
    ExistsDirectory
    GetInfoDirectory
);
use DE_EPAGES::Object::API::Factory qw (
    JoinObjectPath
    LoadObject
    LoadClassByAlias
);
use DE_EPAGES::Shop::API::Constants qw (
    TAX_MODEL_NET
);

use DE_EPAGES::MultiStoreCommon::API::ObjectUpdate qw (
    TestObjectClass
);
use DE_EPAGES::MultiStoreCommon::API::MultiStoreLog qw (
    MultiStoreLogDebug
    MultiStoreLogDebugObjectPath
);
use DE_EPAGES::MultiStoreCommon::API::Synchronization qw (
    GetDefaultValues
    GetCategoryObjXML
    GetCategoryObjXMLType
);
use DE_EPAGES::MultiStore::API::Synchronization qw (
    AddSubShopObject
    ClassAliasFromObjectPath
    GetMasterXMLDir
    GetMasterXMLFile
    ImportProduct
    IsAffiliateVisible
);
use DE_EPAGES::MultiStore::DAL::SubShopObject qw (
    DAL_GetCustomAttributes
    DAL_UpdateCustomAttributes
);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

#========================================================================================
# function     OnUpdateSubShopObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnUpdateSubShopObject($hParams);
#----------------------------------------------------------------------------------------
# description  API_MS_UpdateSubShopObject triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - DE_EPAGES::Object::API::Object::Object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnUpdateSubShopObject {
    my ($hParams) = @_;

    MultiStoreLogDebug('Hook OnUpdateSubShopObject() START');
    my $ClassAlias = $hParams->{'ClassAlias'};
    return unless ElementOf($ClassAlias, qw( Category CategorySpecialOffer ));

    my $Shop = $hParams->{'Shop'};
    my $SubShopObject = $hParams->{'SubShopObject'};
    my $MasterObjectPath = $hParams->{'MasterObjectPath'};

    my $FileName = GetMasterXMLFile( $Shop, $MasterObjectPath );
    my ($Path, $Name) = fsplit($FileName);
    my $CatProdMapFileName = "$Path/$Name" . '.cpm';

    # Import Products of CategoryProductMap
    _ImportCategoryProducts($SubShopObject, $CatProdMapFileName);

    # Import sub categories.. the current one and its first level sub categories.
    # -> Import only the first level of sub categories at each time the script is running.
    my $ImportDir = GetMasterXMLDir( $Shop, $MasterObjectPath );
    if( ExistsDirectory( $ImportDir ) ) { # otherwise there are no sub categories
        _ImportSubCategories( $SubShopObject, $MasterObjectPath, $ImportDir );
    }

    MultiStoreLogDebug('Hook OnUpdateSubShopObject() END');

    return;
}

sub _ImportCategoryProducts {
    my ($SubShopCategory, $CatProdMapFileName) = @_;

    MultiStoreLogDebugObjectPath("_ImportCategoryProducts(%s, $CatProdMapFileName) START", $SubShopCategory);

    my $Shop = $SubShopCategory->getSite;
    my $hCatProdMap = XMLin($CatProdMapFileName, 'KeepRoot' => 1);
    my $CategoryType = GetCategoryObjXMLType($hCatProdMap);
    my $rMap = exists $hCatProdMap->{'epages'}{$CategoryType}{'CategoryProductMap'}
               ? $hCatProdMap->{'epages'}{$CategoryType}{'CategoryProductMap'}
               : undef;
    my $CategoryProductMap = LoadClassByAlias('CategoryProductMap');

    # Get current product-category assignments.
    my %CurrentMaps = map { $_->get('ProductID') => 1 } @{$CategoryProductMap->getObjects('ByCategoryID', [$SubShopCategory->id])};
    if( defined $rMap ) {
        my @Maps = ref $rMap eq 'ARRAY' ? @$rMap : ( $rMap );

        foreach my $hMap ( @Maps ){
            my $ProductPath = $hMap->{'Product'};
            MultiStoreLogDebug('_ImportCategoryProducts() Process product > '.$ProductPath);
            $ProductPath =~ s|^(\Q../\E)+||;

            if (not IsAffiliateVisible($Shop, $ProductPath, 'Product')){
                MultiStoreLogDebug('_ImportCategoryProducts() not Visible. next ...');
                next;
            }

            # Check if product already exists -> import not necessary.
            my $ProductID = $Shop->findPath( $ProductPath );

            unless (defined $ProductID) {
                MultiStoreLogDebug('_ImportCategoryProducts() ImportProduct');
                say ' # /'. $Shop->alias . "/$ProductPath" if $DE_EPAGES::MultiStore::API::Synchronization::PRINT_OUTPUT;
                ImportProduct($Shop, $ProductPath);
            }

            # Delete productid from current assignment hash.
            delete $CurrentMaps{$ProductID} if exists $CurrentMaps{$ProductID};
            return unless defined $Shop->findPath( $ProductPath );
            my $Product = $Shop->path( $ProductPath );
            AssignProductToCategory($Product, $SubShopCategory, $hMap->{'Position'}, $hMap->{'PositionCategory'});
        }
    }
    # remove all remaining current assignments -> assignment not found in xml -> product is unassigned
    # works also, if the category*.cmp has no categoryproductassignments
    foreach my $ProductID (keys %CurrentMaps) {
        MultiStoreLogDebug('_ImportCategoryProducts() RemoveProductFromCategory on ProductID '.$ProductID);
        RemoveProductFromCategory(LoadObject($ProductID), $SubShopCategory);
    }

    MultiStoreLogDebug('_ImportCategoryProducts() END');

    return;
}

sub _ImportSubCategories {
    my ($SubShopCategory, $MasterParentPath, $ImportDir) = @_;

    MultiStoreLogDebugObjectPath("_ImportSubCategories(%s, $MasterParentPath, $ImportDir) START", $SubShopCategory);

    my $ClassAlias = ClassAliasFromObjectPath( $MasterParentPath );
    my $Shop = $SubShopCategory->getSite;

    my $hDirInfo = GetInfoDirectory( $ImportDir, { 'recursive' => 0} );
    foreach my $File ( @{ $hDirInfo->{'Files'} } ) {

        # Import only objects, no maps.
        my (undef, undef, $Ext) = fsplit($File);
        next unless $Ext eq '.xml';

        # Import only unknown objects the recursive way.
        # Already known ones are identified via ObjectUpdate.
        my $hObjectXML = GetCategoryObjXML( XMLin( $File, 'KeepRoot' => 1 ) );
        my $ObjectAlias = $hObjectXML->{'epages'}{GetCategoryObjXMLType($hObjectXML)}{'Alias'};

        # "$ObjectAlias = undef" occurs if GetCategoryObjXMLType doesn't find a valid category type.
        next unless $ObjectAlias;
        next if $SubShopCategory->existsChild( $ObjectAlias );

        my $MasterObjectPath = $MasterParentPath .'/'. JoinObjectPath([ $ObjectAlias ]);
        next unless IsAffiliateVisible($Shop, $MasterObjectPath, $ClassAlias);

        MultiStoreLogDebug('_ImportSubCategories() calls AddSubShopObject()');
        AddSubShopObject( $SubShopCategory, $MasterObjectPath );
    }

    MultiStoreLogDebug('_ImportSubCategories() END');

    return;
}

#========================================================================================
# function     OnBeforeInsertTemplateMap
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeInsertTemplateMap($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_BeforeInsertTemplateMap triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - DE_EPAGES::Object::API::Object::Object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeInsertTemplateMap {
    my ($hParams) = @_;

    MultiStoreLogDebug('Hook OnBeforeInsertTemplateMap() START');

    my $TemplateMap = $hParams->{'Object'};
    my $Category = $TemplateMap->get('Object');
    my $AttributeID = LoadClassByAlias('ContentObject')->attribute('MS_CustomAttributes')->id;
    DAL_UpdateCustomAttributes($Category->id, $AttributeID, [qw( IncludeTemplates )]);

    MultiStoreLogDebug('Hook OnBeforeInsertTemplateMap() END');

    return;
}

#========================================================================================
# function     OnBeforeUpdateProductPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateProductPrice($hParams);
#----------------------------------------------------------------------------------------
# description  API_BeforeUpdateProductPrice triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>ProductID - integer</li>
#                   <li>CurrencyID - string</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateProductPrice {
    my ($hParams) = @_;

    my $Object = LoadObject($hParams->{'ProductID'});
    MultiStoreLogDebugObjectPath('Hook OnBeforeUpdateProductPrice(%s) START', $Object);

    my $CurrencyID = $hParams->{'CurrencyID'};
    _OnBeforeUpdateObject( LoadObject($hParams->{'ProductID'}), $hParams, sub {
        return $_[0]->getDirectListPrice($CurrencyID, TAX_MODEL_NET, 1);
    });

    MultiStoreLogDebug('Hook OnBeforeUpdateProductPrice() END');

    return;
}

#========================================================================================
# function     OnBeforeUpdateContentObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateContentObject($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_BeforeUpdateContentObject triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - content object</li>
#                   <li>Values - ref.hash</li>
#                   <li>LanguageID - integer</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateContentObject {
    my ($hParams) = @_;

    my $Object = $hParams->{'Object'};
    MultiStoreLogDebugObjectPath('Hook OnBeforeUpdateContentObject(%s) START', $Object);

    my $LanguageID = $hParams->{'LanguageID'};
    _OnBeforeUpdateObject( $Object, $hParams->{'Values'}, sub {
        return $_[0]->get($_[1], $LanguageID);
    });

    MultiStoreLogDebug('Hook OnBeforeUpdateContentObject() END');

    return;
}

#========================================================================================
# function     _OnBeforeUpdateObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _OnBeforeUpdateObject($Object, $hValues, $cGetter);
#----------------------------------------------------------------------------------------
# description  The code behind the hook functions above..
#               Tests if an object has to be processed, filters attributes the merchant isn't allowed to modify,
#               bookmarks overwritten attributes and distinguishes between user/ui and import script.
#----------------------------------------------------------------------------------------
# input        $Object | content object | DE_EPAGES::Content::API::Object::ContentObject
# input        $hValues | hook argument's values | ref.hash
# input        $cGetter | code ref used for getting attribute values | ref.code
#========================================================================================
sub _OnBeforeUpdateObject {
    my ($Object, $hValues, $cGetter) = @_;

    # Continue for merchant/ui interaction
    my $ClassName = TestObjectClass($Object, GetDefaultValues()->{'SUBSHOP_CLASSES_LISTENED_TO'});
    return unless $ClassName;

    # Handle requirements of xml import.
    if( $DE_EPAGES::MultiStoreCommon::API::Synchronization::IS_MULTISTORE_EXIMPORT ) {
        # Remove all attributes that shall not be set during xml import.
        _FilterOverwrittenAttributes($Object, $hValues, $ClassName);
        return;
    }

    # Is the merchant allowed to set this attribute?!
    my $aLegalAttributes = _FilterOverwritableAttributes($Object->getSite, $hValues, $ClassName);
    # Which attributes have been overwritten by the merchant?
    _SetOverwrittenAttributes($Object, $hValues, $cGetter, $aLegalAttributes);

    return;
}

#========================================================================================
# function     _FilterOverwritableAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aLegalAttributes = _FilterOverwritableAttributes($Shop, $hValues, $ClassName);
#----------------------------------------------------------------------------------------
# description  Removes all attributes from params which the merchant is not allowed to overwrite.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $hValues | hook argument's values | ref.hash
# input        $ClassName | name of object's class | string
# return       $aLegalAttributes | filtered list of attributes, only overwritables remain | ref.array.string
#========================================================================================
sub _FilterOverwritableAttributes {
    my ($Shop, $hValues, $ClassName) = @_;

    my $hDefaultValues = GetDefaultValues();
    my %ClassMap = reverse %{$hDefaultValues->{'SUBSHOP_CLASSES_MAP'}};
    my $hOverwritableAttributes = $Shop->get('MS_OverwritableAttributes');
    my $aOverwritableAttributes = $hOverwritableAttributes->{$ClassMap{$ClassName}};
    foreach my $Attribute ( keys %$hValues ) {
        delete $hValues->{$Attribute} unless
            ElementOf($Attribute, (@$aOverwritableAttributes,
                                   @{$hDefaultValues->{'ATTRIBUTE_ALIAS'}}));
    }

    return [ keys %$hValues ];
}

#========================================================================================
# function     _FilterOverwrittenAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       _FilterOverwrittenAttributes($Object, $hValues, $ClassName);
#----------------------------------------------------------------------------------------
# description  Removes all attributes from params which have already been overwritten by the merchant.
#               The purpose is to prohibit the import to reset custom changes.
#----------------------------------------------------------------------------------------
# input        $Object | content object | DE_EPAGES::Content::API::Object::ContentObject
# input        $hValues | hook argument's values | ref.hash
# input        $ClassName | Class name for the content object | string
#========================================================================================
sub _FilterOverwrittenAttributes {
    my ($Object, $hValues, $ClassName) = @_;

    my $hDefaultValues = GetDefaultValues();
    my %ClassMap = reverse %{$hDefaultValues->{'SUBSHOP_CLASSES_MAP'}};
    my $hOverwritableAttributes = $Object->getSite()->get('MS_OverwritableAttributes');
    my $aOverwritableAttributes = $hOverwritableAttributes->{$ClassMap{$ClassName}};

    my $AttributeID = LoadClassByAlias('ContentObject')->attribute('MS_CustomAttributes')->id;
    my $aCustomAttributes = DAL_GetCustomAttributes($Object->id, $AttributeID);
    foreach my $Attribute ( keys %$hValues ) {
        delete $hValues->{$Attribute} if ElementOf($Attribute, $aCustomAttributes)
            and ElementOf($Attribute, $aOverwritableAttributes);
    }

    return;
}

#========================================================================================
# function     _SetOverwrittenAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       _SetOverwrittenAttributes($Object, $hValues, $cGetter, $aLegalAttributes);
#----------------------------------------------------------------------------------------
# description  Sets the content object attribute 'MS_CustomAttributes' to a list of attribute names overwritten by the merchant.
#----------------------------------------------------------------------------------------
# input        $Object | content object | DE_EPAGES::Content::API::Object::ContentObject
# input        $hValues | hook argument's values | ref.hash
# input        $cGetter | code ref for getting attribute values | ref.code
# input        $aLegalAttributes | filtered list of attributes, only overwritables remain | ref.array.string
#========================================================================================
sub _SetOverwrittenAttributes {
    my ($Object, $hValues, $cGetter, $aLegalAttributes) = @_;

    foreach my $ChangeKey ( keys %$hValues ) {

        my $ObjectValue = $cGetter->($Object, $ChangeKey);
        my $Value = $hValues->{$ChangeKey};
        $Value =~ s/\r//g if defined $Value;

        if( not ElementOf($ChangeKey, $aLegalAttributes)
            or not defined $Value and not defined $ObjectValue
            or $Value eq $ObjectValue ) {
            delete $hValues->{$ChangeKey};
        }
    }

    my $AttributeID = LoadClassByAlias('ContentObject')->attribute('MS_CustomAttributes')->id;
    DAL_UpdateCustomAttributes($Object->id, $AttributeID, [ keys %$hValues ]);
    return;
}

1;
