use strict;

use Getopt::Long;

use JSON::XS qw (
    encode_json
);

use DE_EPAGES::Core::API::Script qw (
    RunScript
);
use DE_EPAGES::Database::API::Connection qw (
    RunOnStore
    GetCurrentDBHandle
);
use DE_EPAGES::Object::API::Factory qw (
    LoadObject
    LoadObjectByPath
    LoadClassByAlias
);
use DE_EPAGES::Core::API::Error qw (
    ExistsError
    GetError
);
use DE_EPAGES::Core::API::Warning qw (
    Warning
);
use DE_EPAGES::WebInterface::API::MessageCenter qw (
    SynchronizeCache
);
use DE_EPAGES::MultiStoreCommon::API::Synchronization qw (
    GetDefaultValues
);
use DE_EPAGES::MultiStoreCommon::API::ObjectUpdate qw (
    TestObjectClass
    InsertObjectUpdate
);
use DE_EPAGES::MultiStore::DAL::SubShopObject qw (
    DAL_GetAllInfoSubShopObjectsBySiteID
    DAL_GetCustomAttributes
);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

my $DefaultStorename = 'MultiStore';
my $OutputEnabled = 1;
my $TestMode = 0;

my $hMultiStoreSettings;
my $MS_CustomAttributeID;

#======================================================================================================================
# Main
#======================================================================================================================
sub Main {
    local $| = 1;

    my ($Parameters) = getCmdLineParameters();

    RunOnStore(
        Store => $Parameters->{'Storename'},
        Sub   => sub {
            eval {
                output("Test mode enabled !\n") if ($TestMode);
                runScriptIntent($Parameters);
            };
            if (ExistsError()) {
                my $Error = GetError();
                print STDERR 'ERROR: '.$Error->shortMessage."\n";
            }
        }
    );
    syncCache();

    return;
}

#======================================================================================================================
# getCmdLineParameters
#======================================================================================================================
sub getCmdLineParameters {

    my (    $Help,
            $NoOutput,
            $StoreName,
            $ShopAlias,
            $UpdateAllSubshops
        );

    $StoreName = $DefaultStorename;

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'test'          => \$TestMode,
        'storename=s'   => \$StoreName,
        'shop=s'        => \$ShopAlias,
        'all'           => \$UpdateAllSubshops
    );

    $OutputEnabled = ($NoOutput != 1);

    usage() if $Help;

    if (not defined $ShopAlias) {
        error('You must specify at least one of the options -subshop or -all')
            unless ($UpdateAllSubshops);
    }

    return ({
        'Storename'     => $StoreName,
        'Shop'          => $ShopAlias,
        'AllSubshops'   => $UpdateAllSubshops
    });
}

#======================================================================================================================
# syncCache
#======================================================================================================================
sub syncCache {

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    return;
}

#======================================================================================================================
# output
#======================================================================================================================
sub output {
    my @args = @_;

    if ($OutputEnabled) {
        print(sprintf(shift(@args), @args));
    }

    return;
}

#======================================================================================================================
# usage
#======================================================================================================================
sub usage {
    print <<END_USAGE;

Purpose:
    $0 Parse content objects in subshops to reset the overwritten attributes with the actual
    OverwritableAttributes configured in the shop.
    An update (sync) of the object is triggered if needed.

Usage:
    perl $0 [options] [flags]

options:
    -storename         Storename                           (default: MultiStore)
    -shop              Name of shop

flags:
    -help              Shows the command line options
    -nooutput          Do not print any output (mute mode)
    -test              Show what to do but without do it !
    -all               Do it for all the subshops (!)

Examples:

    perl $0 -storename MultiStore -all
    perl $0 -storename MultiStore -subshop eRetail

END_USAGE
    exit 2;
}

#======================================================================================================================
# error
#======================================================================================================================
sub error {
    my ($ErrorMessage) = @_;

    print STDERR "\nERROR: $ErrorMessage\n";
    print "\nUse -help option for a nice explanation for this script :)\n\n";

    exit 2;
}

#======================================================================================================================
# runScriptIntent
#======================================================================================================================
sub runScriptIntent {
    my ($Parameters) = @_;

    loadMultiStoreSettings();
    if ($Parameters->{'AllSubshops'}) {
        runScriptIntentInAllShops($Parameters);
    } else {
        my $ShopPath = '/Shops/'.$Parameters->{'Shop'};
        runScriptIntentInShop(LoadObjectByPath($ShopPath), $Parameters);
    }

    return;
}

#======================================================================================================================
# loadMultiStoreSettings
#======================================================================================================================
sub loadMultiStoreSettings {

    $hMultiStoreSettings = GetDefaultValues();
    $MS_CustomAttributeID = LoadClassByAlias('ContentObject')->attribute('MS_CustomAttributes')->id;

    return;
}

#======================================================================================================================
# runScriptIntentInAllShops
#======================================================================================================================
sub runScriptIntentInAllShops {
    my ($Parameters) = @_;

    my $aShops = LoadObjectByPath('/Shops')->children;
    foreach my $Shop ( @$aShops ) {
        runScriptIntentInShop($Shop, $Parameters);
    }

    return;
}

#======================================================================================================================
# runScriptIntentInShop
#======================================================================================================================
sub runScriptIntentInShop {
    my ($Shop, $Parameters) = @_;

    output("Shop '%s'\n", $Shop->alias);
    if (not $Shop->get('ClosedByProvider')) {
        my $hOverwritableAttributes = getOverwritableAttributes($Shop);
        my $aContentObjects = getSubShopContentObjects($Shop);
        foreach my $hContentObjectInfo (@$aContentObjects) {
            processContentObject($hContentObjectInfo, $hOverwritableAttributes);
        }
    } else {
        output(" ! Ignoring shop. It is closed by provider !!!\n");
    }

    return;
}

#======================================================================================================================
# getOverwritableAttributes
#======================================================================================================================
sub getOverwritableAttributes {
    my ($Shop) = @_;

    my $hAttributeAlias = getAttributeAlias();
    my $hOverwritableAttributes = $Shop->get('MS_OverwritableAttributes');
    my $hFinalOverwritableAttributes = {};
    foreach my $ClassName (keys(%$hOverwritableAttributes)) {
        my $aClassOverwritableAttributes = [];
        foreach my $Attribute (@{$hOverwritableAttributes->{$ClassName}}) {
            push(@$aClassOverwritableAttributes, $Attribute);
            my $aAlias = $hAttributeAlias->{$Attribute};
            if (defined $aAlias) {
                push(@$aClassOverwritableAttributes, @$aAlias);
           }
        }
        $hFinalOverwritableAttributes->{$ClassName} = $aClassOverwritableAttributes;
    }

    return $hFinalOverwritableAttributes;
}

#======================================================================================================================
# getAttributeAlias
#======================================================================================================================
sub getAttributeAlias {

    my $hAttributeAlias = {};

    my $aAttributeAlias = $hMultiStoreSettings->{'ATTRIBUTE_ALIAS'};
    for(my $i; $i < scalar @$aAttributeAlias; $i += 2) {
        my $Key = $aAttributeAlias->[$i];
        my $Values = $aAttributeAlias->[$i+1];
        $hAttributeAlias->{$Key} = $Values;
    }

    return $hAttributeAlias;
}

#======================================================================================================================
# getSubShopContentObjects
#======================================================================================================================
sub getSubShopContentObjects {
    my ($Shop) = @_;

    return DAL_GetAllInfoSubShopObjectsBySiteID($Shop->id);
}

#======================================================================================================================
# processContentObject
#======================================================================================================================
sub processContentObject {
    my ($hContentObjectInfo, $hOverwritableAttributes) = @_;

    my $ContentObject = LoadObject($hContentObjectInfo->{'ContentObjectID'});
    my $aOverwritableAttributes = getOverwritableAttributesByClass($ContentObject, $hOverwritableAttributes);
    my $aCustomAttributes = getContentObjectCustomAttributes($ContentObject);
    if (defined $aCustomAttributes) {
        checkCustomAttributes($ContentObject, $aCustomAttributes, $aOverwritableAttributes);
    }

    return;
}

#======================================================================================================================
# getOverwritableAttributesByClass
#======================================================================================================================
sub getOverwritableAttributesByClass {
    my ($ContentObject, $hOverwritableAttributes) = @_;

    my $ClassName = TestObjectClass($ContentObject, $hMultiStoreSettings->{'SUBSHOP_CLASSES_LISTENED_TO'});
    return [] unless $ClassName;
    my %ClassMap = reverse %{$hMultiStoreSettings->{'SUBSHOP_CLASSES_MAP'}};

    return $hOverwritableAttributes->{$ClassMap{$ClassName}};
}

#======================================================================================================================
# checkCustomAttributes
#======================================================================================================================
sub checkCustomAttributes {
    my ($ContentObject, $aCustomAttributes, $aOverwritableAttributes) = @_;

    my $isResetNeeded = 0;
    my $aFinalCustomAttributes = [];
    foreach my $Attribute (@$aCustomAttributes) {
        if (ElementOf($Attribute, $aOverwritableAttributes)) {
            push(@$aFinalCustomAttributes, $Attribute);
        } else {
            $isResetNeeded = 1;
        }
    }
    output(" %s [%s]\n", ($isResetNeeded)? 'R' : '-', $ContentObject->get('Path'));
    if ($isResetNeeded) {
        resetContentObject($ContentObject, $aFinalCustomAttributes);
    }

    return;
}

#======================================================================================================================
# resetContentObject
#======================================================================================================================
sub resetContentObject {
    my ($ContentObject, $aFinalCustomAttributes) = @_;

    if (not $TestMode) {
        setContentObjectCustomAttributes($ContentObject, $aFinalCustomAttributes);
        InsertObjectUpdate($ContentObject);
    }

    return;
}

#======================================================================================================================
# getContentObjectCustomAttributes
#======================================================================================================================
sub getContentObjectCustomAttributes {
    my ($ContentObject) = @_;

    return DAL_GetCustomAttributes($ContentObject->id, $MS_CustomAttributeID);
}

#======================================================================================================================
# setContentObjectCustomAttributes
#
#   NOTE:   We need to access to Database due the DAL layer in MS doesn't provide a way to remove existing custom
#           attributes. The 'DAL_UpdateCustomAttributes' function only add new attributes (!)
#           This is the code whe would like to have in a (no existing) 'DAL_SetCustomAttributes' function.
#
#======================================================================================================================
sub setContentObjectCustomAttributes {
    my ($ContentObject, $aCustomAttributes) = @_;

    my $dbi = GetCurrentDBHandle();
    $dbi->do(
        'UPDATE stringattribute SET value = ? WHERE objectid = ? AND attributeid = ?',
        [
            encode_json($aCustomAttributes),
            $ContentObject->id,
            $MS_CustomAttributeID
        ]
    );

    return;
}

RunScript( 'Sub' => \&Main );
