#========================================================================================
# package      DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreBaseObjectResource
# state        public
# description  Base class for multistores REST resources. This class overrides methods
#               that need specific changes for multistores.
#========================================================================================
package DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreBaseObjectResource;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;
use strict;

use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw (LoadObjectByGUID);

sub ErrorDefinition {
    return {
        ResourceNotFound => '#Class "#GUID" does not exist in #Site',
    };
}


#========================================================================================
# function     loadObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $Resource->loadObject($ResourceRoot, $GUID);
#----------------------------------------------------------------------------------------
# description  Loads the object itentified by it's GUID in the context of the given Site
#               In contrast to the default base resource, this method accepts also the
#               system object as a Site.
#----------------------------------------------------------------------------------------
# input        $ResourceRoot | Root object of the REST resource | object
# input        $GUID | GUID of the object to load | string
# return       $Object | object | object
#========================================================================================
sub loadObject {
    my $self = shift;
    my ($ResourceRoot, $GUID) = @_;

    my $Object;
    my $Class = $self->class;

    # special case: POST/PUT without a guid match: uses '1' instead.
    if($GUID eq '1') {
        return $ResourceRoot;
    }

    eval {
        if ($ResourceRoot->class->alias eq 'System') {
            # special handling for REST calls on the Store
                $Object = LoadObjectByGUID( $GUID );

                unless( $Object->instanceOf($Class) ) {
                    Error('ClassMismatch', {
                            'ObjectID' => $Object->id,
                            'GUID' => $Object->guid,
                            'Class' => $Class,
                            'ObjectPath' => $Object->pathString,
                    })
                }
        }
        else {
            # TODO: ignore ObjectNotFound for PUT|DELETE
            $Object = $ResourceRoot->loadObjectByGuid( $GUID, $Class );
        }
    };

    if (ExistsError()) {
        my ($Error) = GetError();
        Error('ResourceNotFound', {
            GUID => $GUID,
            Class => $Class,
            Site => $ResourceRoot->pathString,
        }) if ElementOf($Error->code, qw(ClassMismatch NoResults_ObjectByGUID
                                         SiteMismatch));
        Error($Error);
    }

    return $Object;
}


1;
