#========================================================================================
# package      DE_EPAGES::MultiStoreMaster::API::AffiliateVisibility
# base         Exporter
# state        public
# description  Affiliate visibility API
#========================================================================================
package DE_EPAGES::MultiStoreMaster::API::AffiliateVisibility;
use base Exporter;

use strict;

use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw( TestArray TestObject );
use DE_EPAGES::MultiStoreCommon::API::ObjectUpdate qw( InsertObjectUpdate );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );

our @EXPORT_OK = qw (
  IsAffiliateVisible
  AddShopsToAffiliateVisibility
  RemoveShopsFromAffiliateVisibility
  RemoveAllShopsFromAffiliateVisibility
);

sub ErrorDefinition {
    return { 'ContentObjectInvisibleForAffiliates' =>
          'Cannot set AffiliateShopVisibility for ContentObject, becasue it\'s invisible for affiliates. (Path=#ObjectPath)', };
}

#========================================================================================
# function     IsAffiliateVisible
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsAffiliateVisible( $ContentObject, $Shop );
#----------------------------------------------------------------------------------------
# description  Triggers the hook 'API_IsAffiliateVisible' to externalize Affiliate shop
#               visibility. If no hook listener sets the 'IsAffiliateVisibility'
#               property, we use $ContentObject->get('IsAffiliateVisible')
# trigger      API_IsAffiliateVisible | used keys:
#                 <li>ContentObject   - ContentObject - object
#                 <li>ContentObjectID - ContentObject id - integer
#                 <li>ShopAlias       - shop alias - string
#                 <li>IsAffiliateVisible - output parameter: visible=1, invisble=0 - boolean
#----------------------------------------------------------------------------------------
# input        $ContentObject | ContentObject | object
# input        $ShopAlias     | Shop alias | string
# return       $IsVisible  | is visible | boolean
#========================================================================================
sub IsAffiliateVisible {
    my ( $ContentObject, $ShopAlias ) = @_;

    my %Vars = (
        'ContentObject'      => $ContentObject,
        'ContentObjectID'    => $ContentObject->id,
        'ShopAlias'          => $ShopAlias,
        'IsAffiliateVisible' => undef,
    );

    TriggerHook( 'API_IsAffiliateVisible', \%Vars );
    return $Vars{'IsAffiliateVisible'} if ( defined $Vars{'IsAffiliateVisible'} );

    # use getWithParent for Products
    my $cAttributeGetter = $ContentObject->can('getWithParent')
      ? sub { return $ContentObject->getWithParent(@_); }
      : sub { return $ContentObject->get(@_); };

    return $cAttributeGetter->('IsAffiliateVisible') unless ($ShopAlias);

    # optimize: use defining contentobject
    my $Map = $cAttributeGetter->('AffiliateVisibilityMap') or return;
    $ContentObject = $Map->get('MasterShopContentObject');
    my @Attributes = qw( IsAffiliateVisible HasSelectiveAffiliateVisibility AffiliateShopAliases );
    my $hInfo      = $cAttributeGetter->( \@Attributes );
    my $IsAffiliateVisible = $hInfo->{'IsAffiliateVisible'};
    $IsAffiliateVisible = 0 if $hInfo->{'HasSelectiveAffiliateVisibility'};
    $IsAffiliateVisible = 0 unless ElementOf($ShopAlias, $hInfo->{AffiliateShopAliases});
    return $IsAffiliateVisible;
}

#========================================================================================
# function     AddShopsToAffiliateVisibility
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddShopsToAffiliateVisibility( $ContentObject, $aShops );
#----------------------------------------------------------------------------------------
# description  Add shops to affiliateshopvisibility map
#----------------------------------------------------------------------------------------
# input        $ContentObject | content objec | object
# input        $aShops | shop list | ref.array
#========================================================================================
sub AddShopsToAffiliateVisibility {
    my ( $ContentObject, $aShops ) = @_;
    TestObject( 'ContentObject', $ContentObject, 'DE_EPAGES::Content::API::Object::ContentObject' );
    TestArray( 'aShops', $aShops );

    my $MapObject = $ContentObject->get('AffiliateVisibilityMap')
      or Error( 'ContentObjectInvisibleForAffiliates', { 'ContentObjectID' => $ContentObject->id, 'ObjectPath' => $ContentObject->objectPath } );

    my $MapID                 = $MapObject->get('AffiliateVisibilityMapID');
    my $AffiliateShopMapClass = LoadClassByAlias('AffiliateShopVisibilityMap');
    my @ExistingShops         = $ContentObject->get('AffiliateShopAliases');

    foreach my $Shop (@$aShops) {
        next if ( ElementOf($Shop, @ExistingShops) );
        $AffiliateShopMapClass->insertObject( {
                'AffiliateVisibilityMapID' => $MapID,
                'Shop'                     => $Shop,
            }
        );
    }

    InsertObjectUpdate($ContentObject);
    return;
}

#========================================================================================
# function     RemoveShopsFromAffiliateVisibility
# state        public
#----------------------------------------------------------------------------------------
# syntax       RemoveShopsFromAffiliateVisibility( $ContentObject, $aShops );
#----------------------------------------------------------------------------------------
# description  Remove shops from affiliateshopvisibility map
#----------------------------------------------------------------------------------------
# input        $ContentObject | content objec | object
# input        $aShops | shop list | ref.array
#========================================================================================
sub RemoveShopsFromAffiliateVisibility {
    my ( $ContentObject, $aShops ) = @_;
    TestObject( 'ContentObject', $ContentObject, 'DE_EPAGES::Content::API::Object::ContentObject' );
    TestArray( 'aShops', $aShops );

    my $MapObject = $ContentObject->get('AffiliateVisibilityMap')
      or Error( 'ContentObjectInvisibleForAffiliates', { 'ContentObjectID' => $ContentObject->id, 'ObjectPath' => $ContentObject->objectPath } );

    my $MapID                 = $MapObject->get('AffiliateVisibilityMapID');
    my $AffiliateShopMapClass = LoadClassByAlias('AffiliateShopVisibilityMap');

    foreach my $ShopMap ( @{ $AffiliateShopMapClass->getObjects( 'ByAffiliateVisibilityMapID', [$MapID] ) } ) {
        next unless ( ElementOf($ShopMap->get('Shop'), $aShops) );
        $ShopMap->delete;
    }

    InsertObjectUpdate($ContentObject);
    return;
}

#========================================================================================
# function     RemoveAllShopsFromAffiliateVisibility
# state        public
#----------------------------------------------------------------------------------------
# syntax       RemoveAllShopsFromAffiliateVisibility( $ContentObject );
#----------------------------------------------------------------------------------------
# description  Remove all shops from affiliateshopvisibility map
#----------------------------------------------------------------------------------------
# input        $ContentObject | content objec | object
#========================================================================================
sub RemoveAllShopsFromAffiliateVisibility {
    my ($ContentObject) = @_;

    my $MapObject = $ContentObject->get('AffiliateVisibilityMap')
      or Error( 'ContentObjectInvisibleForAffiliates', { 'ContentObjectID' => $ContentObject->id, 'ObjectPath' => $ContentObject->objectPath } );
    my $MapID                 = $MapObject->get('AffiliateVisibilityMapID');
    my $AffiliateShopMapClass = LoadClassByAlias('AffiliateShopVisibilityMap');
    $AffiliateShopMapClass->deleteObjects('ByAffiliateVisibilityMapID', [$MapID] );

    InsertObjectUpdate($ContentObject);
    return;
}


1;
