#========================================================================================
# package      DE_EPAGES::MultiStoreMaster::Hooks::Product
# state        private
#========================================================================================
package DE_EPAGES::MultiStoreMaster::Hooks::Product;

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadObject);
use DE_EPAGES::MultiStoreCommon::API::Synchronization qw (GetDefaultValues);
use DE_EPAGES::MultiStoreCommon::API::ObjectUpdate qw (InsertObjectUpdate);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf Compare);
use JSON::XS qw (encode_json);

#========================================================================================
# function     OnProductExportableChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnProductExportableChildren($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_ProductExportableChildren triggered
#               Implements additional functionality for filtering exportable children
#               of a product.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments and return values, used keys are
#               <ul>
#                   <li>Children - exportable children - ref.array</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnProductExportableChildren {
    my ($hParams) = @_;

    my %BlackList = map { $_ => 1 } @{GetDefaultValues()->{'EXPORTABLE_CHILDREN_BLACKLIST'}};
    $hParams->{'Children'} = [ grep { not exists $BlackList{$_->alias} } @{$hParams->{'Children'}} ];

    return;
}

#========================================================================================
# function     OnBeforeUpdateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateProduct($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_BeforeUpdateProduct triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object - object</li>
#                   <li>Values - new attribute values - ref.hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateProduct {
    my ($hParams) = @_;

    my $Object = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};

    # Attributes which triggers an update of the assigned categories
    my @AttributesCategoryUpdate = qw (IsVisible IsAffiliateVisible HasSelectiveAffiliateVisibility AffiliateShopAliases);
    my $UpdateCategories = 0;

    # Check if old value ne new value.
    foreach my $VisibilityAttribute (@AttributesCategoryUpdate) {
        if (exists $hValues->{$VisibilityAttribute}) {
            my $Attribute     = $Object->class->attribute($VisibilityAttribute);
            my $AttributeType = $Attribute->type;
            my $OldValue      = $Object->get($VisibilityAttribute);
            my $NewValue      = $hValues->{$VisibilityAttribute};

            # If attribute type is not an array or a hash compare new and old value.
            if ($AttributeType ne 'Hash' and not $Attribute->get('IsArray')) {
                $UpdateCategories = 1 if Compare($AttributeType, $OldValue, $NewValue) != 0;
            } else {
                $UpdateCategories = 1;
            }

            last if $UpdateCategories;
        }
    }

    # Insert categories in objectupdate table.
    if ($UpdateCategories) {
        InsertObjectUpdate( $_ ) foreach @{ $Object->get('Categories') };
    }

    return;
}

#========================================================================================
# function     OnAfterUpdateProductPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateProductPrice($hParams);
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateProductPrice triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object - object</li>
#                   <li>Prices - productprices - ref.hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateProductPrice {
    my ($hParams) = @_;

    my $aAttributeUpdates = GetDefaultValues()->{'AttributeUpdate'}->{'Product'};
    if (defined $aAttributeUpdates && ElementOf('Prices', $aAttributeUpdates)) {
        my $ProductID = delete $hParams->{'ProductID'};
        my $CurrencyID = delete $hParams->{'CurrencyID'};

        my $hPrice = {};
        foreach my $ChangedKey (qw ( ListPrice ManufacturerPrice EcoParticipation Deposit )) {
            if (exists $hParams->{$ChangedKey}) {
                $hPrice->{$ChangedKey} = $hParams->{$ChangedKey};
            }
        }

        if (scalar (keys %$hPrice)) {
            my $Product = LoadObject($ProductID);
            my $Value = encode_json({
                'Price' => {
                    'Value' => $hPrice,
                    'CurrencyID' => $CurrencyID
                }
            });

            InsertObjectUpdate( $Product, undef, undef, $Value );
        }
    }

    return;
}

#========================================================================================
# function     OnInsertProductPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnInsertProductPrice($hParams);
#----------------------------------------------------------------------------------------
# description  OnInsertProductPrice triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>ProductID - producctid - string</li>
#                   <li>CurrencyID - currencyid - string</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertProductPrice {
    my ($hParams) = @_;

    my $ProductID = $hParams->{'ProductID'};
    my $Product = LoadObject($ProductID);

    InsertObjectUpdate( $Product );

    return;
}

#========================================================================================
# function     OnDeleteProductPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteProductPrice($hParams);
#----------------------------------------------------------------------------------------
# description  OnDeleteProductPrice triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>ProductID - product id - integer</li>
#                   <li>CurrencyID - currency id - integer</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteProductPrice {
    my ($hParams) = @_;

    my $ProductID = $hParams->{'ProductID'};
    my $Product = LoadObject($ProductID);

    InsertObjectUpdate( $Product );

    return;
}

1;
