use strict;

use Getopt::Long;

use DE_EPAGES::Core::API::Script qw (
    RunScript
);
use DE_EPAGES::Database::API::Connection qw (
    RunOnStore
);
use DE_EPAGES::Object::API::Factory qw (
    ExistsObject
    LoadObject
    ExistsObjectByPath
    LoadObjectByPath
);
use DE_EPAGES::Core::API::Error qw (
    ExistsError
    GetError
);
use DE_EPAGES::Core::API::Warning qw (
    Warning
);
use DE_EPAGES::WebInterface::API::MessageCenter qw (
    SynchronizeCache
);
use DE_EPAGES::MultiStoreCommon::API::ObjectUpdate qw (
    InsertObjectUpdate
);

my $DefaultStorename = 'Master';
my $OutputEnabled = 1;
my $TestMode = 0;

#======================================================================================================================
# Main
#======================================================================================================================
sub Main {
    local $| = 1;

    my ($Parameters) = getCmdLineParameters();

    RunOnStore(
        Store => $Parameters->{'Storename'},
        Sub   => sub {
            eval {
                output("Test mode enabled !\n") if ($TestMode);
                runScriptIntent($Parameters);
            };
            if (ExistsError()) {
                my $Error = GetError();
                print STDERR 'ERROR: '.$Error->shortMessage."\n";
            }
        }
    );
    syncCache();

    return;
}

#======================================================================================================================
# getCmdLineParameters
#======================================================================================================================
sub getCmdLineParameters {

    my (    $Help,
            $NoOutput,
            $StoreName,
            $ShopAlias,
            $ObjectId,
            $ObjectPath,
            $RegExp,
            $ListImportedObjects,
            $UpdateAllSubshops
        );

    $StoreName = $DefaultStorename;

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'test'          => \$TestMode,
        'storename=s'   => \$StoreName,
        'subshop=s'     => \$ShopAlias,
        'objectid=s'    => \$ObjectId,
        'path=s'        => \$ObjectPath,
        'regexp=s'      => \$RegExp,
        'list'          => \$ListImportedObjects,
        'allsubshops'   => \$UpdateAllSubshops
    );

    $OutputEnabled = ($NoOutput != 1);

    usage() if $Help;

    if (not defined $ObjectId) {
        error('You must specify at least one of the options -subshop or -allsubshops')
            unless ($ShopAlias || $UpdateAllSubshops);
        error('You must specify only one of the options -subshop or -allsubshops')
            if ($ShopAlias && $UpdateAllSubshops);
        error('You must specify only one of the options -path or -regexp')
            if ($ObjectPath && $RegExp);
    }

    return ({
        'Storename'     => $StoreName,
        'Shop'          => $ShopAlias,
        'ObjectId'      => $ObjectId,
        'Path'          => $ObjectPath,
        'RegExp'        => $RegExp,
        'List'          => $ListImportedObjects,
        'AllSubshops'   => $UpdateAllSubshops
    });
}

#======================================================================================================================
# syncCache
#======================================================================================================================
sub syncCache {

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    return;
}

#======================================================================================================================
# output
#======================================================================================================================
sub output {

    if ($OutputEnabled) {
        print(sprintf(shift, @_));
    }

    return;
}

#======================================================================================================================
# usage
#======================================================================================================================
sub usage {
    print <<END_USAGE;

Purpose:
    $0 saves Synchronization events to force the update of imported Master objects in subshops

Usage:
    perl $0 [options] [flags]

options:
    -storename         Storename                           (default: MultiStore)
    -subshop           Name of shop
    -path              Path in the subshop of the imported master object to force update
    -objectid          Object ID in the subshop of the imported master object to force update
    -regexp            A regular expresssion to filter the imported mastger objects to force update (not yet implemented)

flags:
    -help              Shows the command line options
    -nooutput          Do not print any output (mute mode)
    -test              Show what to do but without do it !
    -list              Lists the master objects imported in the subshop
    -allsubshop        force update for all subshops in store (not yet implemented)

Examples:
    perl $0 -storename MultiStore -allsubshops
    perl $0 -storename MultiStore -subshop eRetail -list
    perl $0 -storename MultiStore -subshop eRetail
    perl $0 -subshop eRetail -path /Categories/Equipment
    perl $0 -subshop eRetail -regexp '\/Categories\/'

Notes:
    1. Options -regexp and -allsubhop are currently not implemented and reserved for future use.
    2. If you provide -objectid options [ -subshop, -allsubshops, -path, -regexp ] will be ignored.
    3. You could specify either -subshop or -allsubshops, but not both
    4. You could specify either -path or -regexp, but not both
    5. The -path and -regexp options apply to the ObjectPath in the subshop (no MasterPath)
    6. If -path, -objectid or -regexp are not provided, update will be forced to all imported objects (!)

END_USAGE
    exit 2;
}

#======================================================================================================================
# error
#======================================================================================================================
sub error {
    my ($ErrorMessage) = @_;

    print STDERR "\nERROR: $ErrorMessage\n";
    print "\nUse -help option for a nice explanation for this script :)\n\n";

    exit 2;
}

#======================================================================================================================
# runScriptIntent
#======================================================================================================================
sub runScriptIntent {
    my ($Parameters) = @_;

    if ($Parameters->{'ObjectId'}) {
        runScriptIntentInObject($Parameters);
    } elsif ($Parameters->{'AllSubshops'}) {
        runScriptIntentInAllShops($Parameters);
    } else {
        my $ShopPath = '/Shops/'.$Parameters->{'Shop'};
        runScriptIntentInShop(LoadObjectByPath($ShopPath), $Parameters);
    }

    return;
}

#======================================================================================================================
# runScriptIntentInObject
#======================================================================================================================
sub runScriptIntentInObject {
    my ($Parameters) = @_;

    my $ObjectId = $Parameters->{'ObjectId'};
    if (ExistsObject($ObjectId)) {
        updateObject(LoadObject($ObjectId));
    } else {
        output("ERROR: Object with ID [$ObjectId] not found\n");
    }

    return;
}

#======================================================================================================================
# runScriptIntentInAllShops
#======================================================================================================================
sub runScriptIntentInAllShops {
    my ($Parameters) = @_;

    my $aShops = LoadObjectByPath('/Shops')->children;
    foreach my $Shop ( @$aShops ) {
        runScriptIntentInShop($Shop, $Parameters);
    }

    return;
}

#======================================================================================================================
# runScriptIntentInShop
#======================================================================================================================
sub runScriptIntentInShop {
    my ($Shop, $Parameters) = @_;

    if ($Parameters->{'List'}) {
        listImportedObjects($Shop);
    } else {
        updateImportedObjects($Shop, $Parameters);
    }

    return;
}

#======================================================================================================================
# listImportedObjects
#======================================================================================================================
sub listImportedObjects {
    my ($Shop) = @_;

    output("Imported master objects in subShop '%s':\n", $Shop->alias);
    output("listImportedObjects: Not implemented yet !!!\n");

    return;
}

#======================================================================================================================
# updateImportedObjects
#======================================================================================================================
sub updateImportedObjects {
    my ($Shop, $Parameters) = @_;

    if ($Parameters->{'Path'}) {
        updateImportedObjectByPath($Shop, $Parameters->{'Path'});
    } elsif ($Parameters->{'RegExp'}) {
        updateImportedObjectsByRegExp($Shop, $Parameters->{'RegExp'});
    } else {
        updateAllImportedObjects($Shop);
    }

    return;
}

#======================================================================================================================
# updateImportedObjectByPath
#======================================================================================================================
sub updateImportedObjectByPath {
    my ($Shop, $Path) = @_;

    my $ImportedObjectPath = '/Shops/'.$Shop->alias.'/'.$Path;
    if (ExistsObjectByPath($ImportedObjectPath)) {
        updateObject(LoadObjectByPath($ImportedObjectPath));
    } else {
        output("ERROR: Object not found > $ImportedObjectPath\n");
    }

    return;
}

#======================================================================================================================
# updateImportedObjectsByRegExp
#======================================================================================================================
sub updateImportedObjectsByRegExp {
    my ($Shop, $RegExp) = @_;

    output("updateImportedObjectsByRegExp: Not implemented yet !!!\n");

    return;
}

#======================================================================================================================
# updateAllImportedObjects
#======================================================================================================================
sub updateAllImportedObjects {
    my ($Shop) = @_;

    output("updateAllImportedObjects : Not implemented yet !!!\n");

    return;
}

#======================================================================================================================
# updateObject
#======================================================================================================================
sub updateObject {
    my ($Object) = @_;

    my $ObjectPath = $Object->get('Path');
    if ($Object->get('IsAffiliateVisible')) {
        output("UPDATE: $ObjectPath\n");
        InsertObjectUpdate($Object);
    } else {
        output("ERROR: Not a Master Object Visible in SubShops > $ObjectPath\n");
    }

    return;
}

RunScript( 'Sub' => \&Main );
