#========================================================================================
# package DE_EPAGES::MultiStoreProvider::API::WebService::WebServiceVerification
# state public
# description MultiStoreMaster - webservice verification module
#========================================================================================
package DE_EPAGES::MultiStoreProvider::API::WebService::WebServiceVerification;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient;

use constant URI  => 'urn://epages.de/WebService/MultiStoreService/2013/04';


our @EXPORT_OK = qw(
getMultiStoreConfigService
getAssignedSubShops
callMultiStoreWebService
createWebServiceConnection
);

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::MultiStoreMaster::API::WebServiceVerification->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new webservice verification object.
#----------------------------------------------------------------------------------------
# return       $object | webservice verification object | object
#========================================================================================
sub new {
my ($class) = @_;

return bless {}, $class;
}


#========================================================================================
# function     getMultiStoreConfigService
# state        public
#----------------------------------------------------------------------------------------
# syntax       getMultiStoreConfigService($MasterShop)
#----------------------------------------------------------------------------------------
# description  Gets MultiStore config service client for the specified mastershop.
#----------------------------------------------------------------------------------------
# input        $MasterShop | store name | string
# return       $Client | web service client | DE_EPAGES::WebService::API::Client
#========================================================================================
sub getMultiStoreConfigService {
    my $self = shift;
    my ($MasterShop) = @_;
    my $Client = $self->_getWebServiceClient($MasterShop, 'Site', 'urn://epages.de/WebService/MultiStoreConfigService/2011/08');
    return $Client;
}

#========================================================================================
# function     getAssignedSubShops
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAssignedSubShops($MasterShop)
#----------------------------------------------------------------------------------------
# description  Returns the subshops of the given mastershops.
#----------------------------------------------------------------------------------------
# input        $MasterShop | store name | string
# return       $aSubShops | names of subshops | string.array.ref
#========================================================================================
sub getAssignedSubShops {
    my $self = shift;
    my ($MasterShop) = @_;

    my $ConfigService = $self->getMultiStoreConfigService($MasterShop);
    my $aSubShops = $ConfigService->getInfo({
        'ProviderPath'    => LoadRootObject()->get('ProviderPath'),
        'MasterShopAlias' => $MasterShop->alias,
        'FilterActive' => 1
    })->result->{'SubShops'};
    return $aSubShops;
}

#========================================================================================
# function     callMultiStoreWebService
# state        public
#----------------------------------------------------------------------------------------
# syntax       callMultiStoreWebService($WebServiceProxy, $ShopAlias)
#----------------------------------------------------------------------------------------
# description  Calls the given web service.
#----------------------------------------------------------------------------------------
# input        $WebServiceProxy | web service proxy | DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient
# input        $ShopAlias | subshop alias | string
# return       $SOAPResult | soap result | object
#========================================================================================
sub callMultiStoreWebService {
    my $self = shift;
    my ($WebServiceProxy, $ShopAlias) = @_;
    my $SOAPResult = $WebServiceProxy->updateSubShopObjects($ShopAlias, [])->result;
    return _getSOAPResult($SOAPResult);
}

#========================================================================================
# function     createWebServiceConnection
# state        public
#----------------------------------------------------------------------------------------
# syntax       createWebServiceConnection($Proxy, $Auth)
#----------------------------------------------------------------------------------------
# description  Creates connection to the given web service.pwd
#----------------------------------------------------------------------------------------
# input        $Proxy | proxy adress | string
# input        $Auth | basic authentication data | string
# return       $WebServiceProxy | web service proxy | DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient
#========================================================================================
sub createWebServiceConnection {
    my $self = shift;
    my ($Proxy, $Auth) = @_;
    my $WebServiceProxy =
          DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient->new
            ->proxy($Proxy, timeout => 15)
            ->uri(URI);

    $WebServiceProxy->userinfo($Auth);

    return $WebServiceProxy;
}

sub _getWebServiceClient {
    my $self=shift;
    my ($MasterShop, $SiteName, $ServiceUrn) = @_;
    my $hWebServiceConfig = $MasterShop->get( [qw( MS_WebServiceURL MS_WebServiceUser MS_WebServicePassword )] );
    my $WebServiceURL = $hWebServiceConfig->{'MS_WebServiceURL'}{$SiteName};
    my $WebServiceUser = $hWebServiceConfig->{'MS_WebServiceUser'}{$SiteName};
    my $WebServicePassword = $hWebServiceConfig->{'MS_WebServicePassword'}{$SiteName};

    Error('MissingWebServiceConfig', {
        Site => $SiteName,
        WebServiceURL => $WebServiceURL,
        WebServiceUser => $WebServiceUser,
        WebServicePassword => $WebServicePassword ? '******' : undef,
    }) if !$WebServiceURL || !$WebServiceUser || !$WebServicePassword;
    return DE_EPAGES::WebService::API::Client
        ->uri( $ServiceUrn )
        ->proxy( $WebServiceURL )
        ->userinfo( $WebServiceUser . ':' . $WebServicePassword )
        ->xmlschema('2001');
}

sub _getSOAPResult {
    my $self = shift;
    my ($SOAPResult) = @_;

    my $ResultValue = undef;

    return $ResultValue;
}

1;
