use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::MultiStoreProvider::API::ShopRef qw (
    AssignSubShopToMasterShop
    IsSubShopAssignedToMasterShop
    SearchShopRefByAlias
    UpdateRemoteSubShop
);
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use Getopt::Long;
use Time::HiRes qw ( time );

sub Main {

    local $| = 1;

    my $Help;
    my $NoOutput;
    my $MasterShop;
    my $SubShops;
    my $OrderIntegration;
    my $AffiliateshopID;
    my $User = '/Users/wsadmin';
    my $Password = 'wsadmin';

    GetOptions(
        'help'               => \$Help,
        'nooutput'           => \$NoOutput,
        'mastershop=s'       => \$MasterShop,
        'subshops=s@'          => \$SubShops,
        'orderintegration=s' => \$OrderIntegration,
        'affiliateshopid=s'  => \$AffiliateshopID,
        'user=s' => \$User,
        'password=s' => \$Password,
    );

    usage() if $Help;
    usage('MasterShop missing') unless ($MasterShop);
    usage('SubShops missing')    unless ($SubShops);

    my $time1 = time;

    RunOnStore(
        Store => 'Site',
        Sub   => sub {
            my %Info = (
                'User' => $User,
                'Password' => $Password
            );
            if ( ElementOf($OrderIntegration, qw(OrderSplitting BasketHandover None)) ) {
                $Info{'OrderIntegrationType'} = $OrderIntegration;
            }
            if ( defined $AffiliateshopID ) {
                $Info{'AffiliateShopID'} = $AffiliateshopID;
            }
            eval {
                AssignSubShop( $MasterShop, $SubShops, \%Info );
            };
            if (ExistsError()) {
                my $Error = GetError();
                print STDERR 'WARNING: '. substr($Error->shortMessage, 0, 4096) . "\n"; ## no critic (ProhibitMagicNumbers)
                usage();
            }
        }
    );
    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $logmsg  = sprintf( 'need %.3f seconds', $seconds );
    print $logmsg. "\n" unless $NoOutput;

    # you need SynchronizeCache if you have manipulated any
    # objects during the runtime of this script.
    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub AssignSubShop {
    my ( $MasterShop, $SubShops, $hInfo ) = @_;

    my $User = delete $hInfo->{'User'};
    my $Password = delete $hInfo->{'Password'};

    foreach my $SubShop (@$SubShops) {
        my $MasterShopRef = SearchShopRefByAlias($MasterShop);
        my $SubShopRef    = SearchShopRefByAlias($SubShop);
        if ( !IsSubShopAssignedToMasterShop( $MasterShopRef, $SubShopRef ) ) {
            AssignSubShopToMasterShop( $MasterShopRef, $SubShopRef );
        }
        my $Shop = $SubShopRef->remoteShop();
        my $ShopAlias = $Shop->get('Alias');
        my $SystemDomain  = $Shop->get('ProtocolAndServer');
        my $WebServiceURL = $SystemDomain . '/epages/' . $ShopAlias . '.soap';
        my $hParams = {
            'MS_WebServiceLogging' => 1,
            'MS_WebServiceURL' => $WebServiceURL,
            'MS_WebServiceUser' => $User,
            'MS_WebServicePassword' => $Password,
            'MasterShop' => $MasterShopRef->alias,
        };
        UpdateRemoteSubShop( $SubShopRef, {%$hParams, %$hInfo});
    }

    return;
}

sub usage {
    my ($ErrorMessage) = @_;
    print STDERR $ErrorMessage . "\n" if ($ErrorMessage);

    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -mastershop        name of mastershop                  ()
    -subshops          name of subshops                    (list of strings)
    -user              path of webservice user             (default /Users/wsadmin)
    -password          webservice user Password            (default wsadmin)
    -orderintegration  order integration type.
                       possible values are:
                       BasketHandover,OrderSplitting,None  ()
    -affiliateshopid   identification string of
                       subshop                             ()

flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -mastershop MasterShop -subshop SubShop
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
