#========================================================================================
# package      DE_EPAGES::Object::API::Attributes::DirectJSONAttribute
# base         DE_EPAGES::Object::API::BaseAttribute
# state        public
#----------------------------------------------------------------------------------------
# description  Allows to store multiple attribute values in a single database colum as
#               JSON-encoded value.
#========================================================================================
package DE_EPAGES::Object::API::Attributes::DirectJSONAttribute;
use base DE_EPAGES::Object::API::BaseAttribute;

use strict;
use JSON qw( from_json to_json );
use DE_EPAGES::XML::API::DateTimeFormatXML;
use DE_EPAGES::Core::API::PerlTools qw (GetLocalTimeZone IsA ElementOf);
use DE_EPAGES::Object::API::Table::DirectJSONAttribute
    qw( DeleteDirectJSONAttribute ExistsDirectJSONAttribute
        GetInfoDirectJSONAttribute InsertDirectJSONAttribute UpdateDirectJSONAttribute  );

my $PARSER = DE_EPAGES::XML::API::DateTimeFormatXML->new;

sub _cachedValues {
    my $self = shift;
    my ($Object, $LanguageID) = @_;

    return $Object->{'DirectJSONAttributes'} if exists $Object->{'DirectJSONAttributes'};
    return $Object->{'DirectJSONAttributes'} = $self->read( $Object );
}

#========================================================================================
# function     read
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->read( $Object );
#----------------------------------------------------------------------------------------
# description  Reads the attributes values from the database.
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# return       $hValues | attribute values | ref.hash
#========================================================================================
sub read {
    shift;
    my ($Object) = @_;

    my $String = ExistsDirectJSONAttribute( $Object->id )
        ? GetInfoDirectJSONAttribute( $Object->id )->{'Value'}
        : undef;

    return defined $String ? from_json($String): {};
}

#========================================================================================
# function     write
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package->write( $Object, $hValues );
#----------------------------------------------------------------------------------------
# description  Writes the attributes values to the database.
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $hValues | attribute values | ref.hash
#========================================================================================
sub write {
    shift;
    my ($Object, $hValues) = @_;

    my $String = (scalar keys %$hValues) ? to_json($hValues) : undef;

    if( ExistsDirectJSONAttribute( $Object->id ) ) {
        if( defined $String ) {
            UpdateDirectJSONAttribute( $Object->id, { 'Value' => $String } );
        }
        else {
            DeleteDirectJSONAttribute( $Object->id );
        }
    }
    else {
        if( defined $String ) {
            InsertDirectJSONAttribute({ 'ObjectID' => $Object->id, 'Value' => $String });
        }
    }
}

#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->getAttributes( $Object, $aNames, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Returns multiple attribute values. Don't call this function directly.
#               Use $Object->get() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object          | object
# input        $aNames     | list of attribute names | ref.array
# input        $LanguageID | language id | integer
# return       $hValues    | attribute name => value hash | ref.hash
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ($Object, $aNames, $LanguageID) = @_;

    my $hCachedValues = $self->_cachedValues($Object, $LanguageID);

    my %Values;
    my $Class = $Object->class;
    foreach my $Name (@$aNames) {
        my $Value = $hCachedValues->{$Name};

        if( defined $Value
            && $Value =~ /-\d+T\d+:/ # e.g. 2008-09-18T09:56:21
            && ElementOf($Class->attribute($Name)->type, qw( Date DateTime Time)) )
        {
            # create DateTime objects
            $Value = $PARSER->parse_datetime($Value);
        }

        $Values{$Name} = $Value;
    }

    return \%Values;
}

#========================================================================================
# function     getAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Package->getAttribute( $Object, $Name, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Returns a single attribute value. Don't call this function directly.
#               Returns undef if attribute was not found.
#               Use $Object->get() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object          | object
# input        $Name       | attribute name  | string
# input        $LanguageID | language id | int
# return       $Value      | attribute value | ref.hash
#========================================================================================
sub getAttribute {
    my $self = shift;
    my ($Object, $Name, $LanguageID) = @_;

    return $self->getAttributes( $Object, [ $Name ], $LanguageID )->{$Name};
}

#========================================================================================
# function     setAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package->setAttributes( $Object, $hValues, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Sets multiple attribute values. Don't call this function directly.
#               Use $Object->set() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $hValues    | attribute name => value hash | hash ref
# input        $LanguageID | language id (optional) | integer
#========================================================================================
sub setAttributes {
    my $self = shift;
    my ($Object, $hValues, $LanguageID) = @_;

    my $hCachedValues = $self->_cachedValues( $Object, $LanguageID );
    while( my ($Key, $Value) = each %$hValues ) {
        # don't store undefined values
        if( defined $Value ) {
            $Value = $PARSER->format_datetime($Value, GetLocalTimeZone()) if IsA( $Value, 'DateTime' );
            $hCachedValues->{$Key} = $Value;
        }
        else {
            delete $hCachedValues->{$Key};
        }
    }

    $self->write( $Object, $hCachedValues );
    return;
}

#========================================================================================
# function     setAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package->setAttribute( $Object, $Name, $rValue, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Sets a single attribute value. Don't call this function directly.
#               Use $Object->set() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object          | object
# input        $Name       | attribute name  | string
# input        $rValue      | attribute value | ref.hash
# input        $LanguageID | language id | int
#========================================================================================
sub setAttribute {
    my $self = shift;
    my ($Object, $Name, $Value, $LanguageID) = @_;

    $self->setAttributes( $Object, { $Name => $Value }, $LanguageID );
    return;
}

#========================================================================================
# function     deleteObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package->deleteObject( $Object, $aNames );
#----------------------------------------------------------------------------------------
# description  Deletes attribute values from the given object.
#----------------------------------------------------------------------------------------
# input        $Object   | object          | object
# input        $aNames   | attribute names | ref.array.string
#========================================================================================
sub deleteObject {
    my $self = shift;
    my ($Object, $aNames) = @_;

    my $ObjectID = $Object->id;
    DeleteDirectJSONAttribute( $ObjectID ) if ExistsDirectJSONAttribute( $ObjectID );
    return;
}

1;
