#========================================================================================
# package      DE_EPAGES::Object::API::Attributes::IniConfigAttribute
# state        public
# base         DE_EPAGES::Object::API::BaseAttribute
#----------------------------------------------------------------------------------------
# description  Implements the attributes written to an ini-style configuration file.
#               The file location is determined by <function fileName>.
#               This attribute package should be only used for boolean or string attributes.
#               There is no conversion to datetime attributes a.s.o.
#========================================================================================
package DE_EPAGES::Object::API::Attributes::IniConfigAttribute;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::XML::API::DateTimeFormatXML;
use DE_EPAGES::Core::API::PerlTools qw (ElementOf GetLocalTimeZone IsA);

my $PARSER = DE_EPAGES::XML::API::DateTimeFormatXML->new;

#========================================================================================
# function     fileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FileName = $Package->fileName( $Object);
#----------------------------------------------------------------------------------------
# description  Returns filename of Attributes config file.
#               Returns $Object->get('PrivatePath') . '/Attributes.conf';
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# return       $FileName   | Filename of config file | string
#========================================================================================
sub fileName {
    my $self = shift;
    my ($Object) = @_;
    return $Object->get('PrivatePath') . '/Attributes.conf';
}

#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->getAttributes( $Object, $aNames, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Returns multiple attribute values. Don't call this function directly.
#               Use $Object->get() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $aNames     | list of attribute names | array ref
# input        $LanguageID | language id | int
# return       $hValues    | attribute name => value hash | hash ref
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ($Object, $aNames, $LanguageID) = @_;

    my $hInfo;
    DE_EPAGES::Core::API::IniConfig->lock(
        FileName => $self->fileName($Object),
        CaseSensitive => 1,
        Create => 1,
        Access => 'r',
        Sub => sub { $hInfo = shift->section( 'AttributeValues' ); },
    );
    $hInfo->{$_} = undef foreach grep {$hInfo->{$_} !~ /\S/} keys %$hInfo;

    my $Class = $Object->class;
    foreach my $Key (grep { defined $hInfo->{$_} and $hInfo->{$_} =~ /-\d+T\d+:/ } keys %$hInfo) { # may 2008-09-18T09:56:21
        if ( ElementOf($Class->attribute($Key)->type, qw( Date DateTime Time)) ) {
            $hInfo->{$Key} = $PARSER->parse_datetime($hInfo->{$Key});
        }
    }

    #set not found attributes to undef to avoid reading again
    foreach my $Name ( @$aNames ) {
        $hInfo->{$Name} = undef unless exists $hInfo->{$Name};
    }
    return $hInfo;
}

#========================================================================================
# function     setAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package->setAttributes( $Object, $hValues );
# example      $Package->setAttributes( $Object, { Name => 'Value' } );
#----------------------------------------------------------------------------------------
# description  Sets multiple attribute values. Don't call this function directly.
#               Use $Object->set() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $hValues    | attribute name => value hash | hash ref
#========================================================================================
sub setAttributes {
    my $self = shift;
    my ($Object, $hValues) = @_;

    DE_EPAGES::Core::API::IniConfig->lock(
        FileName => $self->fileName($Object),
        CaseSensitive => 1,
        Create => 1,
        Access => 'w',
        Sub => sub {
            my ($IniConfig) = @_;

            foreach my $Key (keys %$hValues) {
                my $Value = $hValues->{$Key};
                if( defined $Value ) {
                    $Value =~ s/\s+$//; # remove training spaces
                    $Value = $PARSER->format_datetime($Value, GetLocalTimeZone()) if IsA( $Value, 'DateTime' );
                    $IniConfig->set( 'AttributeValues', $Key, $Value );
                }
                else {
                    $IniConfig->delete( 'AttributeValues', $Key );
                }
            }

            $IniConfig->rewrite;
        },
    );

    return;
}

1;
