#========================================================================================
# package      DE_EPAGES::Object::API::Attributes::SiteLocale_Ext
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Attribute interface for SiteLocale.
#========================================================================================
package DE_EPAGES::Object::API::Attributes::SiteLocale_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DateTime::Locale;
use DE_EPAGES::Core::API::File qw (ExistsFile);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Object::API::Table::Language qw( GetInfoLanguage );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );
use DE_EPAGES::Object::API::Table::Country qw( GetInfoCountry );
use DE_EPAGES::Object::API::Table::Currency qw( GetInfoCurrency );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;

my @LOCALE_ATTRIBUTES = qw (DecimalSep ThousandSep NegFormatNumbers PosFormatCurrency NegFormatCurrency ListColumnSeparator ListRowSeparator ListQuotCharacter DefaultCharSet );

sub getAttributes {
    my $self = shift;
    my ($Object, $aNames) = @_;
    my $hInfo = {};
    foreach my $Name ( @$aNames ) {
        if( $Name eq 'Locale' ) {
            my $LocaleID = $Object->get('LocaleID');
            $hInfo->{$Name} = {
                %{LocaleVars($LocaleID)},
                %{GetInfoLocale($LocaleID)},
            };
        }
        elsif( $Name eq 'Language' ) {
            $hInfo->{$Name} = GetInfoLanguage(GetInfoLocale($Object->get('LocaleID'))->{'LanguageID'});
        }
        elsif( $Name eq 'Country' ) {
            $hInfo->{$Name} = GetInfoCountry(GetInfoLocale($Object->get('LocaleID'))->{'CountryID'});
        }
        elsif( $Name eq 'Currency' ) {
            $hInfo->{$Name} = GetInfoCurrency(GetInfoLocale($Object->get('LocaleID'))->{'CurrencyID'});
        }
        elsif( $Name eq 'FlagGif' ) {
            my $System      = LoadRootObject();
            my $Directory   = $System->get('PublicStoreTypePath').'/SF/Locales';
            my $FileName    = $Object->get('LocaleID') .'.gif';
            $hInfo->{$Name} = ExistsFile($Directory.'/'.$FileName) ? $System->get('StoreRoot').'/SF/Locales/'. $FileName : undef;
        }
        elsif( $Name eq 'HrefLang' ) {
            my $LocaleID    = $Object->get('LocaleID');
            $LocaleID =~ s/\_/\-/;
            $hInfo->{$Name} = $LocaleID;
        }
        elsif( ElementOf($Name, @LOCALE_ATTRIBUTES) ) {
            my $hLocale = GetInfoLocale($Object->localeID);
            $hInfo->{$_} = $hLocale->{$_} foreach @LOCALE_ATTRIBUTES;
        }
    }
    return $hInfo;
}


sub MonthNamesLoop {
    my ($Locale) = @_;

    return [ map { { Name => $_ } } @{$Locale->month_names} ];
}

sub MonthAbbreviationsLoop {
    my ($Locale) = @_;

    return [ map { { Name => $_ } } @{$Locale->month_abbreviations} ];
}

sub DayNamesLoop {
    my ($Locale) = @_;
    my $aNames = $Locale->day_names;
    my $aAbbr  = $Locale->day_abbreviations;
    return [ map {{
        Name         => $aNames->[$_],
        Abbreviation => $aAbbr->[$_],
        WeekDayID    => ($_ + 1) % 7
    }} (0..6) ];
}

sub DayAbbreviationsLoop {
    my ($Locale) = @_;

    return [ map { { Name => $_ } } @{$Locale->day_abbreviations} ];
}

sub DateFormats {
    my ($Locale) = @_;
    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new(locale=>$Locale->id);
    my $hDateFormats = $Locale->date_formats;
    return {
        map { $_ => $Formatter->formatString('date', $_) }
        keys %$hDateFormats
    };
}

sub DateFormatsLoop {
    my ($Locale) = @_;

    my $hFormats = DateFormats($Locale);

    return [
        map { {
            Name => $_,
            Value => $hFormats->{$_},
        } }
        keys %$hFormats
    ];
}

sub TimeFormatsLoop {
    my ($Locale) = @_;

    my $hFormats = $Locale->time_formats;

    return [
        map { {
            Name => $_,
            Value => $hFormats->{$_},
        } }
        keys %$hFormats
    ];
}

sub DateTimeFormats {
    my ($Locale) = @_;
    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new(locale=>$Locale->id);
    my $hDateFormats = $Locale->date_formats;
    my $hTimeFormats = $Locale->time_formats;
    my %Formats = ();
    $Formats{$_} = $Formatter->formatString('datetime', $_) foreach grep { exists $hTimeFormats->{$_} } keys %$hDateFormats;
    return \%Formats;
}

sub DateTimeFormatsLoop {
    my ($Locale) = @_;

    my $hFormats = DateTimeFormats($Locale);

    return [
        map { {
            Name => $_,
            Value => $hFormats->{$_},
        } }
        keys %$hFormats
    ];
}

sub DefaultDateTimeFormat {
    my ($Locale) = @_;
    my $DefaultDateFormatLength = $Locale->_default_date_format_length; ##no critic (ProhibitPrivateSubroutineUse)
    my $DefaultTimeFormatLength = $Locale->_default_time_format_length; ##no critic (ProhibitPrivateSubroutineUse)
    return $DefaultTimeFormatLength if $DefaultDateFormatLength eq $DefaultTimeFormatLength;
    my $hDateTimeFormats = DateTimeFormats($Locale);
    return $DefaultDateFormatLength if exists $hDateTimeFormats->{$DefaultDateFormatLength};
    return (keys %$hDateTimeFormats)[0];
}

sub LocaleVars {
    my ($LocaleID) = @_;

    # use Bokmal instead of Nynorsk
    $LocaleID = 'nb_NO' if $LocaleID eq 'no_NO';

    my $Locale = DateTime::Locale->load( $LocaleID );

    return {
        ( map { ( "$_" => "$Locale->{$_}" ) } keys %$Locale ),
        DateFormats     => sub { DateFormatsLoop( $Locale ) },
        DateFormat      => DateFormats( $Locale ),
        TimeFormats     => sub { TimeFormatsLoop( $Locale ) },
        TimeFormat      => $Locale->time_formats,
        DateTimeFormats  => sub { DateTimeFormatsLoop($Locale) },
        DateTimeFormat => DateTimeFormats($Locale),
        MonthNames      => sub { MonthNamesLoop( $Locale ) },
        MonthAbbreviations => sub { MonthAbbreviationsLoop( $Locale ) },
        DayNames        => sub { DayNamesLoop( $Locale ) },
        DayAbbreviations => sub { DayAbbreviationsLoop( $Locale ) },
        DateBeforeTime  => sub { $Locale->date_before_time },
        DatePartsOrder  => sub { $Locale->date_parts_order },
        DefaultDateFormatLength => sub { $Locale->_default_date_format_length }, ##no critic (ProhibitPrivateSubroutineUse)
        DefaultTimeFormatLength => sub { $Locale->_default_time_format_length }, ##no critic (ProhibitPrivateSubroutineUse)
        DefaultDateTimeFormatLength => sub { DefaultDateTimeFormat($Locale) },
    };
}

1;
