#========================================================================================
# package      DE_EPAGES::Object::API::Attributes::Site_Ext
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Implements the site attributes Site and Classes
#========================================================================================
package DE_EPAGES::Object::API::Attributes::Site_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Constants qw( ISFILEPATHPARTUSED );
use DE_EPAGES::Object::API::Domain qw( WebServerScriptNameStorefront WebServerScriptNameAdminSSL
    WebServerScriptStorefrontUrl WebServerScriptMobileStorefrontUrl WebServerScriptStorefrontUrlSSL
    WebServerScriptAdminUrl WebServerScriptAdminUrlSSL ProtocolAndServer ProtocolAndServerSSL
    WebServerScriptPreviewUrl WebServerScriptPreviewUrlSSL WebServerScriptMobileStorefrontUrlSSL GetUrlInfo
);
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadClassByAlias );
use DE_EPAGES::Object::API::Table::Country qw( GetInfoCountry GetAllInfoCountries );
use DE_EPAGES::Object::API::Table::Language qw( GetAllInfoLanguages );
use DE_EPAGES::Object::API::Table::Currency qw( GetAllInfoCurrencies );
use DE_EPAGES::Object::API::Table::Locale qw( GetAllInfoLocales GetInfoLocale );
use DE_EPAGES::Object::API::Table::SiteLocaleMap qw( GetInfoSiteLocaleMapsBySiteID );
use DE_EPAGES::Object::API::Table::SecondaryDomain qw ( GetInfoSecondaryDomainsBySiteID InsertSecondaryDomain
    DeleteSecondaryDomain GetInfoSecondaryDomainsByName );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf trim );

my $BASEOBJECT_PACKAGE = 'DE_EPAGES::Core::API::Object::BaseObject';

##no critic qw(ProhibitCascadingIfElse ProhibitExcessComplexity)

sub getAttributes {
    my $self = shift;
    my ($Object, $aNames) = @_;

    my %Values;
    foreach my $Name ( @$aNames ) {
        if( $Name eq 'SiteID' ) {
            die 'use our column site';
        }
        elsif( $Name eq 'Classes' ) {
            $Values{$Name} = [
                sort { $a->alias cmp $b->alias }
                @{ $Object->folder('Classes')->children }
            ];
        }
        elsif( $Name eq 'SiteLocaleID' ) {
            $Values{$Name} = $Object->siteLocale($Object->get('LocaleID'))->id;
        }
        elsif( $Name eq 'LanguageID' ) {
            my $LocaleID = $Object->get('LocaleID');
            $Values{$Name} = defined $LocaleID
                ? GetInfoLocale( $LocaleID )->{LanguageID}
                : undef;
        }
        elsif( $Name eq 'SiteLocales' ) {
            $Values{$Name} = [
                map { $Object->siteLocale( $_->{'LocaleID'} ) }
                sort { ($a->{'Position'} ? $a->{'Position'} : 0) <=> ($b->{'Position'} ? $b->{'Position'} : 0) }
                @{GetInfoSiteLocaleMapsBySiteID($Object->id)}
            ];
        }
        elsif( $Name eq 'BackofficeLocales' ) {
            $Values{$Name} = $Object->backofficeLocales;
        }
        elsif( $Name eq 'Languages' ) {
            $Values{$Name} = [
                sort { $a->{'English'} cmp $b->{'English'} }
                grep { $_->{'IsUsed'} } @{GetAllInfoLanguages()}
            ];
        }
        elsif( $Name eq 'Countries' ) {
            $Values{$Name} = _SiteCountries();
        }
        elsif( $Name eq 'Currencies' ) {
            $Values{$Name} = [
                sort { $a->{'CurrencyID'} cmp $b->{'CurrencyID'} }
                grep { $_->{'IsUsed'} } @{GetAllInfoCurrencies()}
            ];
        }
        elsif( $Name eq 'Locales' ) {
            $Values{$Name} = _Locales();
        }
        elsif( $Name eq 'Units' ) {
            $Values{$Name} = $Object->units;
        }
        elsif( $Name eq 'WeightUnits' ) {
            $Values{$Name} = [ grep {$_->get('Type') eq 'mass'} @{$Object->units}];
        }
        elsif( $Name eq 'TimeUnits' ) {
            $Values{$Name} = [ grep {$_->get('Type') eq 'time'} @{$Object->units}];
        }
        elsif( $Name eq 'WebServerScriptName' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptNameStorefront($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptNameAdminSSL' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptNameAdminSSL($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptUrl' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptStorefrontUrl($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptUrlMobile' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptMobileStorefrontUrl($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptUrlMobileSSL' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptMobileStorefrontUrlSSL($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptUrlSSL' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptStorefrontUrlSSL($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptAdminUrl' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptAdminUrl($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptAdminUrlSSL' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptAdminUrlSSL($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptPreviewUrl' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptPreviewUrl($hSite, $hSystem);
        }
        elsif( $Name eq 'WebServerScriptPreviewUrlSSL' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = WebServerScriptPreviewUrlSSL($hSite, $hSystem);
        }
        elsif( $Name eq 'ProtocolAndServer' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = ProtocolAndServer($hSite, $hSystem);
        }
        elsif( $Name eq 'ProtocolAndServerSSL' ) {
            my ($hSite, $hSystem) = GetUrlInfo($Object);
            $Values{$Name} = ProtocolAndServerSSL($hSite, $hSystem);
        }
        elsif( $Name eq 'CachedPages' ) {
            my $String = $Object->get('CachedPagesString');
            my %Pages = defined $String ? split /;/, $String : ();
            $Values{$Name} = \%Pages;
        }
        elsif( $Name eq 'MaxAgePages' ) {
            my $String = $Object->get('MaxAgePagesString');
            $String = LoadRootObject()->get('MaxAgePagesString') unless defined $String;
            my %Pages = defined $String ? split /;/, $String : ();
            $Values{$Name} = \%Pages;
        }
        elsif( $Name eq 'PageCacheMaxAge' ) {
            my $String = $Object->get('CachedPagesString');
            $String = LoadRootObject()->get('CachedPagesString') unless defined $String;
            my %Pages = defined $String ? split /;/, $String : ();
            $Values{$Name} = (sort {$b <=> $a} values %Pages)[0];
        }
        elsif( $Name eq 'IsPageCacheOutdated' ) {
            my $PageCacheLastInvalidation = $Object->get('PageCacheLastInvalidation');
            if (defined $PageCacheLastInvalidation) {
                my $CurrentTime = GetCurrentDBHandle()->currentDateTime;
                my $ExpiresTime = $PageCacheLastInvalidation->add(minutes => $Object->get('PageCacheMaxAge'));
                $Values{$Name} = $ExpiresTime > $CurrentTime ? 1 : 0;
            } else {
                $Values{$Name} = 0;
            }
        }
        elsif( $Name eq 'IsFilePathPartUsed' ) {
            $Values{$Name} = ISFILEPATHPARTUSED;
        }
        elsif( $Name eq 'SecondaryDomains' ) {
            $Values{$Name} = _SecondaryDomains($Object);
        }
    }
    return \%Values;
}

sub setAttributes {
    my $self = shift;
    my ($Object, $hValues) = @_;

    foreach my $Name ( keys %$hValues ) {
        if( $Name eq 'CachedPages' ) {
            my $hPages = $hValues->{$Name};
            my $String = join ';', map { "$_;$hPages->{$_}" } sort keys %$hPages;
            $Object->set({'CachedPagesString' => $String});

        } elsif( $Name eq 'MaxAgePages' ) {
            my $hPages = $hValues->{$Name};
            my $String = join ';', map { "$_;$hPages->{$_}" } sort keys %$hPages;
            $Object->set({'MaxAgePagesString' => $String});
        } elsif( $Name eq 'SecondaryDomains' ) {
            _SetSecondaryDomains($Object, $hValues->{$Name});
        }
    }
    return;
}

sub importXML {
    my $self = shift;

    my ($Handler, $Class, $aNames, $hAttributes) = @_;
    if(exists $hAttributes->{'SecondaryDomains'}) {
        $hAttributes->{'SecondaryDomains'} = [
            sort
            grep { length($_) > 0 }
            map { trim($_) }
            split(/,/, lc($hAttributes->{'SecondaryDomains'}))
        ];
    }

    return $self->SUPER::importXML($Handler, $Class, $aNames, $hAttributes);
}

sub exportXMLAttributes {
    my $self = shift;

    my ($Driver, $Object, $aNames, $hAttributes) = @_;
    if($hAttributes->{'SecondaryDomains'}) {
        $hAttributes->{'SecondaryDomains'} = join(',', @{$hAttributes->{'SecondaryDomains'}});
    }

    return $self->SUPER::exportXMLAttributes($Driver, $Object, $aNames, $hAttributes);
}

sub _Locales {

    my $Class = LoadClassByAlias('Locale');
    return [
        map { $Class->loadObject( $_->{'LocaleID'} ) }
        grep { $_->{'IsUsed'} }
        @{GetAllInfoLocales()}
    ];
}

sub _SiteCountries {
    my ($Object) = @_;
    my $aSitecountries = GetAllInfoCountries();
    foreach my $SiteCountry (@{$aSitecountries}) {
        my $Info = GetInfoCountry($SiteCountry->{'CountryID'});
        $SiteCountry->{$_} = $Info->{$_} foreach keys %$Info;
        my $Country = LoadClassByAlias('Country')->loadObject($SiteCountry->{'CountryID'});
        $SiteCountry->{'SubDivisions'} = $Country->get('SubDivisions');
    }
    return $aSitecountries;
}

sub _SecondaryDomains {
    my ($Object) = @_;

    return [
        sort
        map { $_->{'Name'} }
        @{GetInfoSecondaryDomainsBySiteID($Object->id)}
    ];
}

sub _SetSecondaryDomains {
    my ($Object, $aDomainNames) = @_;

    my $aExistingDomains = _SecondaryDomains($Object);

    my @aDelete = grep { not ElementOf($_, $aDomainNames) } @$aExistingDomains;
    my @aAdd = grep { not ElementOf($_, $aExistingDomains) } @$aDomainNames;

    InsertSecondaryDomain({'SiteID' => $Object->id, 'Name' => lc($_)}) foreach(@aAdd);
    foreach my $DomainName (@aDelete) {
        my $ahDomainInfo = GetInfoSecondaryDomainsByName($DomainName);
        DeleteSecondaryDomain($_->{'DomainID'}) foreach (@$ahDomainInfo);
    }
    return;
}

1;
