#========================================================================================
# package      DE_EPAGES::Object::API::Object::DALObject
# state        public
# base         DE_EPAGES::Core::API::Object::BaseObject
#----------------------------------------------------------------------------------------
# description  object interface for BaseObjects.
#========================================================================================
package DE_EPAGES::Object::API::Object::DALObject;
use base DE_EPAGES::Core::API::Object::BaseObject;

use strict;
use JSON;
use Scalar::Util qw ( weaken );
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Log qw (LogDebug);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Core::API::TestParameter qw ( TestString TestObject );
use DE_EPAGES::Core::API::Statistics qw ( TakeStatistic );
use DE_EPAGES::Object::API::TLEObjectHash qw (TieObject);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject LoadObject LoadClassByAlias ExistsClassByAlias Transaction);
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode GetCurrentLanguageID GetCodeByLanguageID);
use DE_EPAGES::Database::API::DAL;
use DE_EPAGES::Object::API::Table::Language qw (GetAllInfoLanguages);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Database::API::DataEncryption qw( Decrypt );

use DE_EPAGES::XML::API::DateTimeFormatXML;
my $TIME_PARSER = DE_EPAGES::XML::API::DateTimeFormatXML->new;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NoResults' => 'Object id #BaseObjectID doesn\'t exist in table(s) #Tables (Package=#Package)',
        'ATTRIBUTE_NOT_FOUND' => 'Attribute #Attribute doesn\'t exists in class #Class (BaseClass=#BaseClass)',
        'ReadOnlyAttribute' => 'Attempt to set read-only attribute #ClassAlias.#AttributeAlias',
        'DeleteReference'      => 'Don\'t call deleteXML on a reference (Object=#Path).',
    }
}
#ERRORDEFINITION END

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     nextid
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ObjectID = $Package->nextid();
#----------------------------------------------------------------------------------------
# description  Returns next available object identifier.
#----------------------------------------------------------------------------------------
# return       $ObjectID | undef | string
#========================================================================================
sub nextid { shift->dal->nextid }

#========================================================================================
# function     primaryKey
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Key = $Package->primaryKey();
#----------------------------------------------------------------------------------------
# description  Returns name of identifier of master table.
#----------------------------------------------------------------------------------------
# return       $Key | 'BaseObjectID' | string
#========================================================================================
sub primaryKey { shift->dal->identifier }

#========================================================================================
# function     slavePrimaryKey
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Key = $Package->slavePrimaryKey();
#----------------------------------------------------------------------------------------
# description  Returns name of identifier of slave table
#----------------------------------------------------------------------------------------
# return       $Key | 'BaseObjectID' | string
#========================================================================================
sub slavePrimaryKey {
    my $self = shift;

    my $dal = $self->dal;
    return $dal->table($dal->class)->{'IDENTIFIER'};
}

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Key = $Package->className();
#----------------------------------------------------------------------------------------
# description  Returns name of class.
#----------------------------------------------------------------------------------------
# return       $Key | 'BaseObject' | string
#========================================================================================
sub className { 'BaseObject' }

#========================================================================================
# function     dalPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DalPackage = $Package->dalPackage();
#----------------------------------------------------------------------------------------
# description  Returns name of dal.
#----------------------------------------------------------------------------------------
# return       $DalPackage | 'DE_EPAGES::Database::API::DAL' | string
#========================================================================================
sub dalPackage { 'DE_EPAGES::Database::API::DAL' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $Package->dbPackage();
#----------------------------------------------------------------------------------------
# description  Returns name for this class used by dal.
#----------------------------------------------------------------------------------------
# return       $DalPackage | 'DE_EPAGES::Object::API::Object::BaseObject' | string
#========================================================================================
sub dbPackage { ref($_[0]) ? ref($_[0]) : $_[0] } ## no critic qw(RequireArgUnpacking)

#========================================================================================
# function     dal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DalPackage = $class->dal();
#----------------------------------------------------------------------------------------
# description  Returns name of dal.
#----------------------------------------------------------------------------------------
# return       $Dal | instance of <function dbPackage> use to access objects | object
#========================================================================================
sub dal {
    my $class = shift;
    return $class->dalPackage->load( $class->dbPackage );
}

#========================================================================================
# function     id
# state        public
#----------------------------------------------------------------------------------------
# syntax       $BaseObjectID = $BaseObject->id;
#----------------------------------------------------------------------------------------
# description  Returns the base-object id.
#----------------------------------------------------------------------------------------
# return       $BaseObjectID | object id | string
#========================================================================================
sub id {
    my $self = shift;
    return $self->{'Attributes'}{$self->primaryKey};
}

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $Package->insert($hInfo);
# example      my $Class = LoadClassByAlias('Locale');
#               my $Locale = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $Object | new object | object
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $dal = $Package->dal;
    my $hAttributeValues = {%$hInfo};
    foreach my $Key (keys %$hAttributeValues) {
        next if $Key eq $Package->primaryKey;
        my $Class = exists $hAttributeValues->{'Class'} ? $hAttributeValues->{'Class'} : LoadClassByAlias($Package->className);
        #next unless $Class->existsAttribute($Key);
        my $Attribute = $Class->attribute($Key);
        if ( ElementOf($Attribute->type, qw(String LocalizedString Integer Float Boolean File Money))
             && defined $hAttributeValues->{$Key}
             && $hAttributeValues->{$Key} !~ /\S/
             && !$Attribute->get('IsMandatory') ) {
            $hAttributeValues->{$Key} = undef;
        }
    }
    my $hColumnValues = $dal->mapObjects2IDs($hAttributeValues);

    my $ObjectID = $dal->insert($hColumnValues);
    return $Package->load($ObjectID);
}

#========================================================================================
# function     load
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Locale = $Package->load($BaseObjectID);
#----------------------------------------------------------------------------------------
# description  Loads an existing object.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $BaseObjectID | identifier | string
# return       $Object | existing object | object
#========================================================================================
sub load {
    my ($Package, $BaseObjectID) = @_;
    TestString('BaseObjectID', $BaseObjectID);

    my $hInfo = $Package->dal->info($BaseObjectID);
    Error('NoResults', {
        'BaseObjectID'=>$BaseObjectID,
        'Package'=> $Package,
        'Tables'=> join(', ', map { $_->{'Code'} } $Package->dal->nonCachedDal->tables ),
    }) unless defined $hInfo;
    my $self = $Package->new(%$hInfo);

    # see method class
    unless (exists $hInfo->{'ClassID'}) {
        my $Class = LoadClassByAlias($Package->className);
        $self->{'Attributes'}{'ClassID'} = $Class->id;
        $self->{'Class'} = $Class;
        weaken ($self->{'Class'});
    }
    return $self;
}

#========================================================================================
# function     decryptBaseAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->decryptBaseAttributes($hAttributes);
# example      sub new {
#                   my $self = shift;
#                   my (%options) = @_;
#                   $self->decryptBaseAttributes(\%options);
#                   return $self->SUPER::new(%options);
#               }
#----------------------------------------------------------------------------------------
# description  Decrypts attributes values of type EncryptedBaseAttribute
#               in place (directly in the hash), so the values can be used direct from
#               the cache without need of decrypt by getter
#----------------------------------------------------------------------------------------
# input        $hAttributes | hash of attributes within there keys/values | ref hash
#========================================================================================
sub decryptBaseAttributes {
    my $self = shift;
    my ( $hAttributes ) = @_;

    my $Class = LoadClassByAlias( $self->className );
    my @EncrpytedAttributes =
        map { $_->alias }
        grep { $_->package =~ /Attributes::EncryptedBaseAttribute/ }
        @{ $Class->attributes };

    # decrypt defined values in place
    do { $_ = Decrypt( $_ ) if defined $_ } foreach @{$hAttributes}{@EncrpytedAttributes};

    return;
}

#========================================================================================
# function     canDelete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->canDelete($aWarnings);
# example      my @Warnings;
#               $Object->canDelete( \@Warnings );
#               if( scalar @Warnings ) {
#                   print "Object can't be deleted, because: ";
#                   print join( ", ", map { $_->shortMessage } @Warnings );
#               }
#               else {
#                   $Object->delete;
#               }
#----------------------------------------------------------------------------------------
# description  Tests if the object can be deleted.
#               If there is any reason why the object should not be deleted, these
#               reasons are appended to the @$aWarnings list.
#               Calls <function DE_EPAGES::Object::API::BaseAttribute::canDeleteObject>
#               for each attribute package of the object's class.
#----------------------------------------------------------------------------------------
# input        $aWarnings  | array of Warning Objects | array.ref
#========================================================================================
sub canDelete {
    my $self = shift;
    my ($aWarnings) = @_;

    # can delete named attribtues
    $_->canDeleteObject( $self, $aWarnings ) foreach keys %{$self->class->attributePackages};

    return;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $BaseObject->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object.
#========================================================================================
sub delete {
    my $self = shift;

    # delete references to other objects
    $self->deleteAttributes;

    Transaction( sub {
        $self->SUPER::delete;
    });
    return;
}

#========================================================================================
# function     deleteSelf
# state        private
#----------------------------------------------------------------------------------------
# syntax       $BaseObject->deleteSelf;
#----------------------------------------------------------------------------------------
# description  Deletes the object at database tables.
#========================================================================================
sub deleteSelf {
    my $self = shift;

    $self->dal->delete($self->id);
    $self->SUPER::deleteSelf;
    return;
}


#========================================================================================
# function     class
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Class = $BaseObject->class;
#----------------------------------------------------------------------------------------
# description  Returns the class of an object
#----------------------------------------------------------------------------------------
# return       $Class | class object, see <package DE_EPAGES::Object::API::Object::Class> | object
#========================================================================================
sub class {
    my $self = shift;

    my $Class = $self->{'Class'};
    if (not defined $Class) {
        my $ClassID = $self->{'Attributes'}{'ClassID'};
        $Class = defined $ClassID ? LoadObject($ClassID) : LoadClassByAlias($self->className);
        $self->{'Class'} = $Class;
        weaken $self->{'Class'};
    }
    return $Class;
}

#========================================================================================
# function     instanceOf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsInstance = $BaseObject->instanceOf( $Class )
# syntax       $IsInstance = $BaseObject->instanceOf( $ClassName )
#----------------------------------------------------------------------------------------
# example      my $ClassProduct = LoadClassByAlias( 'Product' );
#               if( $Object->instanceOf( $ClassProduct ) ) { ... }
# example      if( $Object->instanceOf('Product' ) ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if the object is an instance of the class or one of its
#               sub classes.
#----------------------------------------------------------------------------------------
# input        $ClassName  | name of class (or object of class) | string|object
# return       $IsInstance | true if the object is an instance of the class | boolean
#========================================================================================
sub instanceOf {
    my $self = shift;
    my ($Class) = @_;
    return $self->class->isSubClassOf($Class);
}

#========================================================================================
# function     onDeleteClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       $BaseObject->onDeleteClass;
#----------------------------------------------------------------------------------------
# description  Called before the class of this object is deleted.
#               The default implementation calls $self->delete.
#========================================================================================
sub onDeleteClass {
    my $self = shift;
    $self->delete;
    return;
}

#========================================================================================
# function     tleHash
#----------------------------------------------------------------------------------------
# syntax       $hVars = $BaseObject->tleHash;
#----------------------------------------------------------------------------------------
# description  Returns a tied hash for the TLE compiler.
#----------------------------------------------------------------------------------------
# return       $hVars | object attributes as hash reference | ref.hash
#========================================================================================
sub tleHash {
    return TieObject(shift);
}

#========================================================================================
# function     exists
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Exists = $BaseObject->exists;
#----------------------------------------------------------------------------------------
# description  Returns true if the object exists in the database.
#----------------------------------------------------------------------------------------
# return       $Exists | true if the object exists | boolean
#========================================================================================
sub exists {
    my $self = shift;

    my $ObjectID = $self->id;
    return 0 unless defined $ObjectID;
    return $self->dal->exists($ObjectID);
}

#========================================================================================
# function     reset
# state        public
# syntax       $Object->reset;
# description  Removes internal cache key class.
#========================================================================================
sub reset {
    my $self = shift;

    delete $self->{'Class'};
    return;
}

#========================================================================================
# function     cacheValues
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object->cacheValues($hValues, $Class, $Package);
#----------------------------------------------------------------------------------------
# description  saves data of attribute packages at internal object cache.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute values | ref.hash
# input        $Class  | class object | object
# input        $Package  | attribute package | string
#========================================================================================
sub cacheValues {
    my $self = shift;
    my ($hValues, $Class, $Package) = @_;

    foreach my $Key (keys %$hValues) {
        my $AttributeAlias = $Key;
        unless( $Class->existsAttribute( $AttributeAlias ) ) {
            $AttributeAlias =~ s/ID$//;
            next unless $Class->existsAttribute( $AttributeAlias );
        }

        my $Attribute = $Class->attribute( $AttributeAlias ) ;
        next unless $Attribute->isCacheable;
        next if defined $Package && $Package ne $Attribute->package;
        $self->{'Attributes'}{$Key} = $hValues->{$Key};
    }
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $Object->get( $Name, $LanguageID );
# syntax       $hValues = $Object->get( $aNames, $LanguageID );
#----------------------------------------------------------------------------------------
# example      $Password = $Object->get( 'Password' );
# example      $Password = $Object->get( ['Login', 'Password'] )->{'Password'};
#----------------------------------------------------------------------------------------
# description  Returns one or more attribute values.
#----------------------------------------------------------------------------------------
# input        $Name    | attribute name | string
# input        $aNames  | list of attribute names| array ref
# input        $LanguageID | language identifier (optional) | integer
# return       $hValues | name => value pairs | hash ref
# return       $Value   | attribute value | string (or other)
#========================================================================================
sub get {
    my $self = shift;
    my ($aNames, $LanguageID) = @_;

    if (ref $aNames ne 'ARRAY' ) {
        my $hCachedValues = $self->{'Attributes'};
        return $hCachedValues->{$aNames} if exists $hCachedValues->{$aNames};

        my $Class = $self->class;
        my $Attribute = $Class->attribute( $aNames );
        my $IsObject = $Attribute->isObject && ! $Attribute->isArray;
        my $CacheName = $aNames . ($IsObject ? 'ID' : '');
        my $AttributeClass = $IsObject ? LoadClassByAlias($Attribute->type) : undef;

        # exists object
        if ($IsObject && exists $hCachedValues->{$CacheName}) {
            my $ObjectID = $hCachedValues->{$CacheName};
            return undef unless defined $ObjectID; ## no critic (Subroutines::ProhibitExplicitReturnUndef)
            return $AttributeClass->loadObject($ObjectID);
        }

        my $Package = $Attribute->package;
        my $hValues = undef;
        $LanguageID = GetCurrentLanguageID() unless defined $LanguageID;
        TakeStatistic("get:$Package:$aNames", sub {
            $hValues = $Package->getAttributes( $self,[$CacheName], $LanguageID );
        });
        $self->cacheValues($hValues, $Class, $Package);
        my $Value = $hValues->{$CacheName};
        $Value = $AttributeClass->loadObject($Value) if $IsObject and defined $Value;
        return $Value;
    } else { # multiple keys
        my $Class = $self->class;
        my %Values;
        my %LoadOnPackage;
        # group attribute names by package
        foreach my $Name (@$aNames) {
            if (exists $self->{'Attributes'}{$Name}) {
                $Values{$Name} = $self->{'Attributes'}{$Name};
                next;
            }
            my $Attribute = $Class->attribute( $Name );
            my $IsObject = $Attribute->isObject && ! $Attribute->isArray;
            my $AttributeClass = $IsObject ? LoadClassByAlias($Attribute->type) : undef;
            my $CacheName = $Name . ($IsObject ? 'ID' : '');
            if (exists $self->{'Attributes'}{$CacheName}) {
                my $ObjectID = $self->{'Attributes'}{$CacheName};
                $Values{$Name} = defined $ObjectID ? $AttributeClass->loadObject($ObjectID) : undef;
                next;
            }
            my $Package = $Attribute->package;
            $LoadOnPackage{$Package} = [] unless exists $LoadOnPackage{$Package};
            push @{$LoadOnPackage{$Package}}, [ $CacheName, $Name, $AttributeClass ];
        }
        $LanguageID = GetCurrentLanguageID() if not defined $LanguageID and scalar keys %LoadOnPackage;
        foreach my $Package (keys %LoadOnPackage) {
            my $hValues = $Package->getAttributes( $self, [map { $_->[0] } @{$LoadOnPackage{$Package}}], $LanguageID );
            $self->cacheValues($hValues, $Class, $Package);
            foreach my $aAttribute (@{$LoadOnPackage{$Package}}) {
                my $Value = $hValues->{$aAttribute->[0]};
                $Values{$aAttribute->[1]} = (defined $Value and defined $aAttribute->[2]) ? $aAttribute->[2]->loadObject($Value) : $Value;
            }
        }
        return \%Values;
    }
}

#========================================================================================
# function     set
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Sets the given attribute values on the object.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional) language id for localized attributes | integer
#----------------------------------------------------------------------------------------
sub set {
    my $self = shift;
    my ($hValues, $LanguageID, $hAdditionalOptions) = @_;

    # set attributes
    my $Class = $self->class;
    my $hPackages = $Class->attributePackages([keys %$hValues]);
    my %ObjectKeys;
    foreach my $Package ( keys %$hPackages ) {
        my %Values;
        foreach my $AttributeKey ( @{$hPackages->{$Package}}) {
            my $Attribute = $Class->attribute($AttributeKey);
            if( $Attribute->get('IsReadOnly') ) {
                Error('ReadOnlyAttribute', {
                    'ClassAlias' => $Class->alias,
                    'AttributeAlias' => $AttributeKey
                });
            }
            if ($Attribute->isObject && !$Attribute->isArray) {
                $ObjectKeys{$AttributeKey} = $AttributeKey . 'ID';
                $Values{$AttributeKey .'ID'} = defined $hValues->{$AttributeKey} ? $hValues->{$AttributeKey}->id : undef;
            } elsif (ElementOf($Attribute->type, qw(String LocalizedString Integer Float Boolean File Money))
                     && defined $hValues->{$AttributeKey}
                     && $hValues->{$AttributeKey} !~ /\S/
                     && !$Attribute->get('IsMandatory')) {
                $Values{$AttributeKey} = undef;
                $hValues->{$AttributeKey} = undef;
            } else {
                $Values{$AttributeKey} = $hValues->{$AttributeKey};
            }
        }
        $Package->setAttributes( $self, \%Values, $LanguageID, $hAdditionalOptions ) if scalar keys %Values;
    }
    # prepare and set cached values in object
    my %CacheValues;
    foreach my $AttributeKey (keys %$hValues) {
        if (exists $ObjectKeys{$AttributeKey}) {
            $CacheValues{$ObjectKeys{$AttributeKey}} = defined $hValues->{$AttributeKey} ? $hValues->{$AttributeKey}->id : undef;
        } else {
            $CacheValues{$AttributeKey} = $hValues->{$AttributeKey};
        }
    }
    $self->cacheValues(\%CacheValues, $Class);
}

#========================================================================================
# function     findXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ObjectID = DE_EPAGES::Object::API::Object::Object->findXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $ObjectID | the existing object id or undef| string
#========================================================================================
sub findXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $key = $class->primaryKey;
    return undef unless defined $hAttributes->{$key}; ## no critic (Subroutines::ProhibitExplicitReturnUndef)

    return $class->dal->exists($hAttributes->{$key}) ? $hAttributes->{$key} : undef;
}

#========================================================================================
# function     importXMLAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $class->importXMLAttributes($Handler, $Class, $XMLParentObject, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Converts XML tag attributes to object attributes, i.e. converts
#               object path to object, formatted strings to DateTime objects,
#               sets empty strings to undef etc..
#               Manipulates the hash $hAttributes in place.
#----------------------------------------------------------------------------------------
# input        $Handler | XML import handler | DE_EPAGES::Object::API::XML::ObjectImportHandler
# input        $XMLParentObject | parent object in the XML structure | DE_EPAGES::Object::API::Object::Object
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $hAttributes | attributes hash | ref.hash
#========================================================================================
sub importXMLAttributes {
    my $class = shift;
    my ($Handler, $Class, $XMLParentObject, $hAttributes) = @_;

    delete $hAttributes->{'reference'}; # used to mark existing objects
    delete $hAttributes->{'delete'}; # used in uninstall
    delete $hAttributes->{'Class'};  # the class can't be changed by XML import

    $Class->importXMLPackages($Handler, $hAttributes);

    if( defined $hAttributes->{'Language'} ) {
        $hAttributes->{'LanguageID'} = GetPKeyLanguageByCode($hAttributes->{'Language'});
        delete $hAttributes->{'Language'};
    }

    my $SystemTimeZone;
    foreach my $AttributeName ( keys %$hAttributes ) {
        my $Attribute = $Class->attribute( $AttributeName );
        my $Type = $Attribute->type;

        if (ElementOf($Type, qw(DateTime Date Time Duration))) {
            # convert datetime
            $SystemTimeZone = LoadRootObject()->get('TimeZone') unless defined $SystemTimeZone;
            $hAttributes->{$AttributeName} = $hAttributes->{$AttributeName} ne ''
                ? $TIME_PARSER->parse_datetime($hAttributes->{$AttributeName}, $SystemTimeZone)
                : undef;
        }
        elsif (ElementOf($Type, qw(String LocalizedString Integer Float Boolean File Money))) {
            # delete the attribute
            $hAttributes->{$AttributeName} = undef unless $hAttributes->{$AttributeName} =~ /\S/;
        }
        elsif ($Type eq 'JSON') {
            $hAttributes->{$AttributeName} = from_json($hAttributes->{$AttributeName});
        }
        else{
            my $Value = $hAttributes->{$AttributeName};
            if ( defined $Value && ref $Value eq '' && ( $Attribute->isObject || ExistsClassByAlias($Type) )
            ) {
                $hAttributes->{$AttributeName} = $Value =~ /\S/ ? $XMLParentObject->path($Value) : undef;
            }
        }
    }

    return;
}

#========================================================================================
# function     importXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $Package->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $ClassObject, $hAttributes) = @_;

    my $XMLParentObject = $Handler->topID('Object') // LoadRootObject();

    my $ObjectID = $class->findXML($Handler, $hElement, $ClassObject, $hAttributes);
    if ($Handler->topID('IsReference')) {
        Error('NoObject', $hAttributes) unless defined $ObjectID;
    }
    else {
        TestObject( 'Class', $ClassObject );
    }

    $class->importXMLAttributes( $Handler, $ClassObject, $XMLParentObject, $hAttributes );

    # set attributes
    LogDebug('Attributes',$hAttributes);
    if (defined $ObjectID) {
        my $Object = $ClassObject->loadObject($ObjectID);
        my $PrimaryKey = $class->primaryKey;
        delete $hAttributes->{$PrimaryKey};
        $Object->set( $hAttributes );
        return $Object;
    }
    else {
        return $ClassObject->insertObject( $hAttributes );
    }
}

#========================================================================================
# function     _exportClass
# state        privat
#----------------------------------------------------------------------------------------
# syntax       $Class = $Object->_exportClass;
#----------------------------------------------------------------------------------------
# description  Returns the last base class with type=class of an object
#----------------------------------------------------------------------------------------
# return       $Class | class object, see <package DE_EPAGES::Object::API::Object::Class> | object
#========================================================================================
sub _exportClass {
    my $self = shift;
    my $class = $self->class;
    while ( $class->class->alias ne 'Class' ) {
        $class = $class->baseClass;
    }
    return $class;
}

#========================================================================================
# function     tagNameXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TagName = $Object->tagNameXML;
#----------------------------------------------------------------------------------------
# description  Returns the tag name to use in XML export/import.
#               By default, the tag name is the Alias of the class.
#----------------------------------------------------------------------------------------
# return       $TagName | xml tag name | string
#========================================================================================
sub tagNameXML {
    return shift->_exportClass->alias;
}

#========================================================================================
# function     deleteAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->deleteAttributes;
#----------------------------------------------------------------------------------------
# description  Deletes the object attributes. Called by <function delete>.
#========================================================================================
sub deleteAttributes {
    my $self = shift;

    # dont call deleteAttributes twice, via Object and DALObject.
    return if $self->{'_CalledDeleteAttributes_'};

    # delete named attribtues
    $self->{'_CalledDeleteAttributes_'} = 1;
    my $hPackages = $self->class->attributePackages;
    Transaction( sub {
        foreach my $Package (keys %$hPackages) {
            $Package->deleteObject( $self, $hPackages->{$Package} );
            $self->_deleteAttribute($_) foreach @$hPackages{$Package}; ##no critic (ProhibitPrivateSubroutineUse)
        }
    });
    return;
}

#========================================================================================
# function     exportableAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Object->exportableAttributes
# syntax       $hValues = $Object->exportableAttributes($Driver)
# syntax       $hValues = $Object->exportableAttributes($Driver, $DriverExportLevel)
#----------------------------------------------------------------------------------------
# example      $Position = $Object->exportableAttributes->{'Position'};
# example      $Description = $Object->exportableAttributes->{'Description'}->{$LanguageID};
#----------------------------------------------------------------------------------------
# description  Returns all attribute values that have an existing (not-null) value.
#               Unlike $Object->_attributes this function returns localized attribute
#               values as hash reference (language id => value).
#----------------------------------------------------------------------------------------
# input        $Driver | (optional) XML export driver | object
# input        $DriverExportLevel | (optional) export level | integer
# return       $hValues | object attributes as key => value pairs | hash ref
#========================================================================================
sub exportableAttributes {
    my $self = shift;
    my ($Driver, $DriverExportLevel) = @_;

    $DriverExportLevel = defined $Driver ? $Driver->exportLevel : undef unless defined $DriverExportLevel;

    # group attribute names by package
    my $Class = $self->class;
    my $ClassMajorExportLevel = $Class->get('MajorExportLevel');
    my $aAttributes = [ grep { $_->get('IsExportable') && !$_->isArray } @{ $Class->attributes } ];
    my %Packages;
    foreach my $Attribute (@$aAttributes) {

        if (defined $DriverExportLevel) {
            my $MajorAttributeExportLevel = $Attribute->get('MajorExportLevel');
            $MajorAttributeExportLevel = $ClassMajorExportLevel if not defined $MajorAttributeExportLevel or $ClassMajorExportLevel > $MajorAttributeExportLevel;
            next unless $MajorAttributeExportLevel == $DriverExportLevel;
        }
        my $Package = $Attribute->package;
        $Packages{$Package} = [] unless exists $Packages{$Package};
        push @{$Packages{$Package}}, $Attribute;
    }
    # collect results in this hash
    my %Values;
    foreach my $Package (keys %Packages) {
        my $aPackageAttributes = $Packages{$Package};
        my (@PackageNames, @AttributeNames);
        foreach my $Attribute (@$aPackageAttributes) {
            my $Name = $Attribute->alias;
            push @AttributeNames, $Name;
            my $IsObject = $Attribute->isObject && !$Attribute->isArray;
            push @PackageNames, $Name . ($IsObject ? 'ID' : '');
        }
        my $hValues = $Package->getAllAttributes( $self, \@PackageNames );
        $self->cacheValues($hValues, $Class, $Package);
        for( my $i=0; $i < scalar @$aPackageAttributes; $i++ ) {
            my $Attribute = $aPackageAttributes->[$i];
            my $IsArray  = $Attribute->isArray;
            my $Value = $hValues->{$PackageNames[$i]};
            if (defined $Value && !$IsArray && $Attribute->isObject) {
                $Value = LoadClassByAlias($Attribute->type)->loadObject($Value);
            }
            $Value = undef if defined $Value and $IsArray and not scalar @$Value;
            $Values{$AttributeNames[$i]} = $Value;
        }
    }
    delete $Values{$_} foreach grep { not defined $Values{$_} } keys %Values;
    return \%Values;
}

#========================================================================================
# function     exportXMLLanguageElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->exportXMLLanguageElements( $Driver, $hAttributes );
#----------------------------------------------------------------------------------------
# description  Add xml elements for language dependend attributes (part of exportXMLElements), same as String attributes includes newline.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLLanguageElements {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    my $DriverExportLevel = $Driver->exportLevel;
    if ($DriverExportLevel == $self->class->get('MajorExportLevel')) {
        # split language dependend
        my %LanguageElements;
        my %Languages = map { $_->{'LanguageID'} => $_->{'Code3'} } @{ GetAllInfoLanguages() };
        my $Class = $self->class;
        foreach my $Attribute ( sort keys %$hAttributes) {
            my $Type = $Class->attribute($Attribute)->type;
            if( $Type =~ /^Localized(String|File)$/ ) {
                my $hValues = $hAttributes->{$Attribute};
                foreach my $LanguageID (sort { $Languages{$a} cmp $Languages{$b} } keys %$hValues) {
                    next unless defined $hValues->{$LanguageID};

                    # add attribute values containing linebreaks or multiple
                    # subsequent whitespaces as character data
                    if ( $hAttributes->{$Attribute}{$LanguageID}  =~ /\n/s ) {
                        $Driver->addElement(
                            'AttributeValue', {
                                'Attribute' => {
                                    'Name'      => $Attribute,
                                    'LanguageID'=> $LanguageID,
                                },
                            },
                            $hValues->{$LanguageID}
                        );
                    }
                    else {
                        $LanguageElements{$LanguageID}{$Attribute} = $hValues->{$LanguageID};
                    }
                }
            } else {
                if ( $hAttributes->{$Attribute} =~ /\n/s ) {
                    $Driver->addElement(
                        'AttributeValue', {
                            'Attribute' => { 'Name' => $Attribute },
                        },
                        $hAttributes->{$Attribute}
                    );
                }
            }
        }
        foreach my $LanguageID (sort { $Languages{$a} cmp $Languages{$b} } keys %LanguageElements) {
            $LanguageElements{$LanguageID}{'LanguageID'} = $LanguageID;
            $Driver->addElement('AttributeLanguage', { 'Attributes' => $LanguageElements{$LanguageID}});
        }
    }
    return;
}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->exportXMLElements($Driver, $hAllAttributes);
#----------------------------------------------------------------------------------------
# description  Exports multilinguagal attribute value of this object to the xml SAX driver.
#----------------------------------------------------------------------------------------
# input        $Driver | export driver | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    # export language tags
    $self->exportXMLLanguageElements($Driver, $hAttributes);

    # add addititonal attributes from attribute packages
    $self->class->exportXMLElementsPackages($Driver, $self, $hAttributes);
    return;
}

#========================================================================================
# function     _formatExportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->_formatExportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  converts attributes of object to plain strings to export via xml.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub _formatExportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    # class of object
    my $Class = $self->class;

    my $SystemTimeZone;
    my $Defaults = $Driver->exportDefaults ? {} : $self->exportXMLDefaultAttributes($hAttributes);
    # time convert and remove defaults
    foreach my $AttributeName ( keys %$hAttributes ) {
        if (not defined $Defaults->{$AttributeName} and not defined $hAttributes->{$AttributeName}) {
             delete $hAttributes->{$AttributeName};
             next;
        }

        my $Attribute = $Class->attribute( $AttributeName );
        my $IsMandatory = $Attribute->get('IsMandatory');
        my $Type = $Attribute->type;

        if (ElementOf($Type, qw(DateTime Date Time Duration))) {
            $SystemTimeZone = LoadRootObject()->get('TimeZone') unless defined $SystemTimeZone;
            $hAttributes->{$AttributeName} = $TIME_PARSER->format_datetime($hAttributes->{$AttributeName}, $SystemTimeZone)
                unless $hAttributes->{$AttributeName} eq '';
        }
        elsif (ElementOf($Type, qw(String File EMail ExternalPassword SpaceString))) {
            if( defined $Defaults->{$AttributeName}
                and $hAttributes->{$AttributeName} eq $Defaults->{$AttributeName}
                and not $IsMandatory
            ) {
                delete $hAttributes->{$AttributeName} ;
            }
        }
        elsif (ElementOf($Type, qw(LocalizedString LocalizedFile))) {
            delete $hAttributes->{$AttributeName};
        }
        elsif (ElementOf($Type, qw(Integer Boolean))) {
            if( defined $Defaults->{$AttributeName}
                and $hAttributes->{$AttributeName} == $Defaults->{$AttributeName}
                and not $IsMandatory
            ) {
                delete $hAttributes->{$AttributeName};
            }
        }
        elsif (ElementOf($Type, qw(Float Money))) {
            if( defined $Defaults->{$AttributeName}
                and not $IsMandatory
                and Math::BigFloat->new($hAttributes->{$AttributeName}) == Math::BigFloat->new($Defaults->{$AttributeName})
            ) {
                delete $hAttributes->{$AttributeName};
            }
        }
        elsif ($Type eq 'JSON') {
            $hAttributes->{$AttributeName} = to_json($hAttributes->{$AttributeName});
            if( defined $Defaults->{$AttributeName}
                and $hAttributes->{$AttributeName} eq to_json($Defaults->{$AttributeName})
                and not $IsMandatory
            ) {
                delete $hAttributes->{$AttributeName};
            }
        }
        elsif ($Attribute->isObject) {
            my $XMLParentObject = $Driver->topID('XMLParentObject');
            $hAttributes->{$AttributeName} = defined $XMLParentObject
                ? $XMLParentObject->relativePath($hAttributes->{$AttributeName})
                : $hAttributes->{$AttributeName}->pathString;
        } else {
            # object attribute
            delete $hAttributes->{$AttributeName};
        }
    }
    # change language id to code
    if (defined $hAttributes->{'LanguageID'} && $hAttributes->{'LanguageID'} =~ /^\d+$/) {
        $hAttributes->{'Language'} = GetCodeByLanguageID($hAttributes->{'LanguageID'});
        delete $hAttributes->{'LanguageID'};
    }
}

sub exportXMLDefaultAttributes {
    return {};
}

#========================================================================================
# function     exportXMLAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes string attributes with newlines, adds object to each level.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    $self->_formatExportXMLAttributes($Driver, $hAttributes);

    my $Class =$self->class;
    $Class->exportXMLAttributesPackages($Driver, $self, $hAttributes);

    # add object to other levels
    if ($Driver->exportLevel == $Class->get('MajorExportLevel')) {
        my %MajorLevels;
        foreach my $MajorExportLevel (map { $_->get('MajorExportLevel') } grep { $_->get('IsExportable') } @{$Class->attributes}) {
            next unless $MajorExportLevel;
            $MajorLevels{$MajorExportLevel}++;
        }
        delete $MajorLevels{$Driver->exportLevel};
        $Driver->addNextLevelItem($self, $_) foreach keys %MajorLevels;
    }
    delete $hAttributes->{$_} foreach grep { $hAttributes->{$_} =~ /\n/s } keys %$hAttributes;
    return $hAttributes;
}

#========================================================================================
# function     clone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Clone = $DALObject->clone( $hInfo );
#----------------------------------------------------------------------------------------
# description  Inserts a new object into the database as copy from old object.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with attribute values that are different in the clone
#                        object. | hash ref
# return       $Clone | the new object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo) = @_;
    my $hAttributes = $self->exportableAttributes;
    $hAttributes->{$_} = $hInfo->{$_} foreach keys %$hInfo;
    return $self->class->insertObject($hAttributes);
}

#========================================================================================
# function     deleteXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = DE_EPAGES::Object::API::Object::Object->deleteXML($Handler, $hElement, $hAttributes, $Class);
#----------------------------------------------------------------------------------------
# description  Called by the SAX driver when a start tag for the current object class
#               is encountered. The function identifies the object by the attributes
#               in $hAttributes and deletes the object if it still exists and and the
#               attribute 'delete="1"' is specified.
#----------------------------------------------------------------------------------------
# input        $ObjectPackage | package name of the objects's class | string
# input        $Handler | xml delete handler | object
# input        $hElement | information about the start tag, see SAX function start_element | ref.hash
# input        $hAttributes | attributes of the start tag | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# return       $Object | the found object if it still exists | object
#========================================================================================
sub deleteXML {
    my ($ObjectPackage, $Handler, $hElement, $hAttributes, $Class) = @_;

    my $ObjectID = $ObjectPackage->findXML($Handler, $hElement, $Class, $hAttributes);
    return undef unless defined $ObjectID; ## no critic (Subroutines::ProhibitExplicitReturnUndef)

    my $Object = $Class->loadObject($ObjectID);
    return $Object unless defined $hAttributes->{'delete'};

    if ($Handler->topID('IsReference')) {
        Warning('DeleteReference', {
            'Path' => $Object->pathString,
            'Alias' => $Object->alias,
        });
        return $Object;
    }
    $Object->delete;
    return undef; ## no critic (Subroutines::ProhibitExplicitReturnUndef)
}

#========================================================================================
# function     triggerGetHook
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Object->triggerGetHook( $HookName, $aNames, $LanguageID, $Sub );
# syntax       $Value = $Object->triggerGetHook( $HookName, $Name, $LanguageID, $Sub );
# example      return $self->triggerGetHook('OBJ_GetDownloadProductMap', $aNames, $LanguageID, sub {
#                   my ($_aNames, $_LanguageID) = @_;
#                   return $self->SUPER::get($_aNames, $_LanguageID)
#               });
#----------------------------------------------------------------------------------------
# description  Triggers a hook for $Object->get.
#----------------------------------------------------------------------------------------
# input        $HookName | package name of the objects's class | string
# input        $aNames | list of attribute names | ref.array.string
# input        $Name | attribute name | string
# input        $LanguageID | language id | int
# input        $Sub | code reference | ref.code
# return       $Value | attribute name => value hash | ref.hash
#========================================================================================
sub triggerGetHook {
    my $self = shift;
    my ($HookName, $aNames, $LanguageID, $Sub) = @_;

    if (ref $aNames eq 'ARRAY') {
        my %Values;
        my @RemainNames;
        foreach my $Name (@$aNames) {
            my %HookArgs = (
                'Object' => $self,
                'Names' => $Name,
                'LanguageID' => $LanguageID,
                'Value'=>undef,
            );
            TriggerHook($HookName, \%HookArgs);
            if (defined $HookArgs{'Value'}) {
                $Values{$Name} = $HookArgs{'Value'};
            } else {
                push @RemainNames, $Name;
            }
        }
        my $hValues = scalar @RemainNames ? $Sub->(\@RemainNames, $LanguageID) : {};
        $hValues->{$_} = $Values{$_} foreach keys %Values;
        return $hValues;
    } else {
        my %HookArgs = (
            'Object' => $self,
            'Names' => $aNames,
            'LanguageID' => $LanguageID,
            'Value'=>undef,
        );
        TriggerHook($HookName, \%HookArgs);
        return $HookArgs{'Value'} if defined $HookArgs{'Value'};
        return $Sub->($aNames, $LanguageID);
    }
}

1;
