use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::IDN qw ( EncodeDomain );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadRootObject );
use DE_EPAGES::Object::API::PageCache::File;
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::WebInterfacePoolDB::API::ASPoolDB qw ( RunOnASPoolDB );
use Getopt::Long;

use feature 'say';

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $Path;
    my $DomainName;
    my $Verbose = 1;

    GetOptions(
        'help' => \$Help,
        'passwd=s' => \$Password,
        'storename=s' => \$StoreName,
        'domain=s' => \$DomainName,
        'path=s' => \$Path,
        'output!' => \$Verbose,
    );
    usage() if $Help;

    unless($StoreName) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }

    unless($DomainName) {
        print STDERR "Missing parameter -domain\n\n";
        usage();
    }

    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            $DomainName = lc($DomainName);
            my $Site = defined($Path) ? LoadObjectByPath($Path) : LoadRootObject();
            my $EncodedDomain = EncodeDomain( $DomainName );
            my $hSiteByDomain = RunOnASPoolDB( sub { shift->getSiteByDomainName( $EncodedDomain ) } );
            if( defined($hSiteByDomain) && ($hSiteByDomain->{'StoreAlias'} ne $StoreName
                 || $hSiteByDomain->{'ObjectPath'} ne $Site->pathString)) {
                die 'Domain '.$DomainName.' already in use in store:' . $hSiteByDomain->{'StoreAlias'}  . ' path:'. $hSiteByDomain->{'ObjectPath'};
            }
            my $aDomains = $Site->get('SecondaryDomains');

            if (not ElementOf($DomainName, $aDomains)) {
                push @$aDomains, $DomainName;
                $Site->set({ SecondaryDomains => $aDomains });
            }

            say 'Domains assigned: ', $Site->get('DomainName') . ' (primary) '.(scalar @$aDomains? join(', ', @$aDomains) . ' (secondary)':'') if ($Verbose);
        }
    );

    printf "need %.1f seconds\n", time() - $time1 if $Verbose;

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub usage {
    print <<END_USAGE;
Description:
    Adds a secondary domain to a site.

Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password              ()
    -storename  name of store                       ()
    -path       path to the site, uses system if omitted ()
    -domain     name of domain                      ()
flags:
    -help                 show the command line options
    -nooutput             do not print verbose information

Example:
    perl $0 -storename Store -path /Shops/DemoShop -domain addtional.domain.com
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
