use strict;

use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject);

sub Main {
    my $time = time;

    my ($Help, $Store, $ShopAlias, $NoOutput, $ShopAge, $ContentObjectAge, $ProductAge, $CategoryAge);
    GetOptions(
        'help'                 => \$Help,
        'storename=s'          => \$Store,
        'shop=s'               => \$ShopAlias,
        'shopage=i'            => \$ShopAge,
        'contentobjectage=i'   => \$ContentObjectAge,
        'productage=i'         => \$ProductAge,
        'categoryage=i'        => \$CategoryAge,
        'nooutput'             => \$NoOutput
    );

    usage() if $Help;

    unless ($Store) {
        print "The store type is not defined!\n";
        usage();
    }
    unless ($ShopAlias) {
        print "The shop alias is not defined!\n";
        usage();
    }
    unless ($ShopAge and $ProductAge and $CategoryAge and $ContentObjectAge ) {
        print "Please define all cache ages!\n";
        usage();
    }

    RunOnStore(
        Store => $Store,
        Sub => sub {
            my $Shop = LoadRootObject()->child('Shops')->child($ShopAlias);
            my $hOldMaxAgePages = $Shop->get('MaxAgePages');
            print "former settings:\n" . printMaxAgePages($hOldMaxAgePages) unless $NoOutput;

            $Shop->set({
                'MaxAgePages' => {
                    Category => $CategoryAge,
                    ContentObject => $ContentObjectAge,
                    Product => $ProductAge,
                    Shop => $ShopAge
                },
            });

            my $hNewMaxAgePages = $Shop->get('MaxAgePages');
            print "new settings:\n" . printMaxAgePages($hNewMaxAgePages) unless $NoOutput;
        }
    );

    printf "need %.3f seconds\n", time-$time unless $NoOutput;
    return;
}

sub printMaxAgePages {
    my $hAges = shift;
    my $Result = '';
    while (my ($Key, $Value) = each %$hAges) {
        $Result .= "  $Key = " . $Value . " seconds\n";
    }
    return $Result
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Description:
    Modify browser cache time (MaxAgePages) in seconds.

Options:
    -storename            store name
    -shop                 shop alias
    -productage           browser cache (products)
    -contentobjectage     browser cache (contentobject)
    -categoryage          browser cache (category)
    -shopage              browser cache (shop)

Flags:
    -nooutput             be silent, only display errors
    -help                 show the command line options

Examples:
    perl $0 -storename Store -shop DemoShop -productage 120 -contentobjectage 120 -categoryage 120 -shopage 120

END_USAGE
    exit 1;
}

RunScript(
    Sub => \&Main
);
