#========================================================================================
# §package      DE_EPAGES::Order::API::BasketCreator
# §state        public
#----------------------------------------------------------------------------------------
# §description  Creates a new basket.
#========================================================================================
package DE_EPAGES::Order::API::BasketCreator;

use DE_EPAGES::Core::API::Error qw( ExistsError GetError ResetError Error );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Object::API::Factory qw ( Transaction );
use DE_EPAGES::Order::API::Basket qw( CreateBasket );
use DE_EPAGES::Order::API::Constants qw ( CART_PATCH_PARTIALLY_COMPLETE CART_PATCH_COMPLETE  );
use DE_EPAGES::Tax::API::Shop qw ( TaxInfoBySellingCountryID );
use DE_EPAGES::Presentation::API::Form;
use DE_EPAGES::Order::API::BasketLineItemContainer;
use DE_EPAGES::WebService::API::Util qw (ReadJSONBoolean);

use JSON::XS;
use strict;

#========================================================================================
# §function     new
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       new($Shop);
#----------------------------------------------------------------------------------------
# §description  Constuctor
#----------------------------------------------------------------------------------------
# §input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# §return       $self | self | DE_EPAGES::Order::API::BasketCreator
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop) = @_;
    my $self  = bless {
        'Shop' => $Shop,
    }, $class;
    return $self;
}



#========================================================================================
# §function     createBasket
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       createBasket( $hBasketData );
#----------------------------------------------------------------------------------------
# §description  Creates a new basket.
#----------------------------------------------------------------------------------------
# §input        $Input1 | Input1Description | ref_hash
# §input        $hErrors | (optional) hash for collecting errors | ref_hash
# §return       The created basket | DE_EPAGES::Order::API::Object::Basket
#========================================================================================
sub createBasket {
    my $self = shift;
    my ($hBasketData, $hErrors) = @_;

    $hErrors = {} unless ref $hErrors eq 'HASH';

    my $Shop = $self->{'Shop'};
    my $UserFolder = $Shop->folder('AnonymousUsers');
    my $BasketUserID = $UserFolder->nextAlias('%d');

    my $hSellingCountryData = TaxInfoBySellingCountryID($Shop, $hBasketData->{'sellingCountryId'} );

    # Create the basket also if we insert multiple line items and have an error
    # inserting one of the lineitems.
    # Test also if we try to insert more than one lineitem.
    my $ForceCreateBasket = ReadJSONBoolean($hBasketData->{'forceCreate'});
    my $HaveMultipleLineItems = ((defined $hBasketData->{'lineItems'}) and (scalar @{$hBasketData->{'lineItems'}} > 1)) ? 1 : 0;
    $ForceCreateBasket = 0 unless $HaveMultipleLineItems;

    my $hParams = {
        'Alias' => $BasketUserID,
        'Name'  => $BasketUserID
    };

    $hParams->{'CurrencyID'} = $hBasketData->{'currency'} if $hBasketData->{'currency'};
    $hParams->{'LocaleID'} = $hBasketData->{'locale'} if $hBasketData->{'locale'};

    my $BasketUser = $UserFolder->insertChild('AnonymousUser', $hParams);
    my $Basket;

    eval {

        $self->_Transaction( sub {

            $Basket = $self->_CreateBasket({
               'Shop' => $Shop,
               'User' => $BasketUser,
            });

            my $hLineItemContainerData = $self->_createLineItemContainerData($hBasketData);
            my $BasketLineItemContainer = $self->_createBasketLineItemContainer($Basket);

            if ($ForceCreateBasket){
                # insert line items step by step and catch error for each line item
                # the status if we have an error is added to the returned basket

                my $ahLineItems = $hLineItemContainerData->{'ProductLineItems'};
                foreach my $hLineItem ( @$ahLineItems ){
                    $hLineItemContainerData->{'ProductLineItems'} = [$hLineItem];

                    eval {
                        $BasketLineItemContainer->setLineItemContainer($hLineItemContainerData, $hSellingCountryData);
                    };
                    if (ExistsError()) {
                        my $Error = GetError();

                        if ($Error->code() eq 'Not AvailableOrVisible' ||  $Error->code() eq 'Low StockLevel' || $Error->code() eq 'ProductNoPrice') {
                            $hErrors->{'AddLineItemError'} += 1 ;
                        }
                        ResetError();
                    }
                }

                # throw error if no line item was added to the basket
                if( exists $hErrors->{'AddLineItemError'} ) {
                    $hErrors->{'status'} = CART_PATCH_PARTIALLY_COMPLETE;
                    if( $hErrors->{'AddLineItemError'} eq scalar @$ahLineItems ) {
                        Error('BadRequest', {'Reason' => 'No products could be added to the basket'});
                    }
                } else {
                    $hErrors->{'status'} = CART_PATCH_COMPLETE;
                }

            } else {
                # try to insert lineitems in one step and rollback if we have errors inserting lineitems
                $BasketLineItemContainer->setLineItemContainer($hLineItemContainerData, $hSellingCountryData);
            }

        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code() eq 'Not AvailableOrVisible' || $Error->code() eq 'Low StockLevel' || $Error->code() eq 'ProductNoPrice' || $Error->code() eq 'BadRequest') {
            Error($Error);
        }
        LogError($Error);
        ResetError();
    }

    return $Basket;
}

#========================================================================================
# §function     _createLineItemContainerData
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       _createLineItemContainerData( $hBasketData );
#----------------------------------------------------------------------------------------
# §description  Creates the data
#----------------------------------------------------------------------------------------
# §input        $hBasketData | basket data | ref_hash
# §return       data to create a LineItemContainer | refhash
#========================================================================================
sub _createLineItemContainerData {
    my $self = shift;
    my ($hBasketData) = @_;

    my $aProductLineItems = [];

    my $aLineItems = $hBasketData->{'lineItems'};
    foreach my $hLineItem (@$aLineItems) {
        push(@$aProductLineItems, {
            'Product' => $hLineItem->{'productId'},
            'Quantity' => $hLineItem->{'quantity'},
            'CustomizedTextUnity' => $hLineItem->{'customizedText'},
        });
    }

    my $hLineItemDataContainerData = {
       'ProductLineItems' => $aProductLineItems,
       'CurrencyID' => $hBasketData->{'currency'},
       'LocaleID' => $hBasketData->{'locale'},
       'TaxModel' => lc($hBasketData->{'taxType'} // undef),
       'ShippingMethod' => undef,
       'PaymentMethod' => undef,
       'TaxArea' => undef,
    };

    return $hLineItemDataContainerData;
}



sub _createBasketLineItemContainer {
    my $self = shift;
    my ($Basket) = @_;
    return DE_EPAGES::Order::API::BasketLineItemContainer->new($Basket);
}



#========================================================================================
# §function     Transaction
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       Transaction( $cCode, $cRollback );
#----------------------------------------------------------------------------------------
# §description  Runs a code block in a transaction using the current database handle.
#               If an error occurs, the local data cache is reset and the code block
#               $cRollback is executed.
#----------------------------------------------------------------------------------------
# §input        $cCode | code block | code reference
# §input        $cRollback | roll back code block (optional) | code reference
#========================================================================================
sub _Transaction {
    my $self = shift;
    my ($cCode, $cRollback) = @_;
    Transaction($cCode, $cRollback);
    return;
}



#========================================================================================
# §function     CreateBasket
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Basket = CreateBasket($hInfo);
#----------------------------------------------------------------------------------------
# §description  Creates a basket for a user and shop.
#----------------------------------------------------------------------------------------
# §input        $hInfo | keys
# §return       $Basket| basket | object
#========================================================================================
sub _CreateBasket {
    my $self = shift;
    my ($hInfo) = @_;
    return CreateBasket($hInfo);
}

1;
