#========================================================================================
# package      DE_EPAGES::Order::API::BasketPatch
# state        public
#----------------------------------------------------------------------------------------
# description  Implements JSON-PATCH (RFC 6902) for the basket object.
#========================================================================================
package DE_EPAGES::Order::API::BasketPatch;
use base DE_EPAGES::WebService::API::JSONPatch;

use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError);
use DE_EPAGES::Core::API::PerlTools qw ( GenerateGUID isValid GetMD5Digest );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObjectByGUID Transaction );
use DE_EPAGES::Order::API::BasketAddress;
use DE_EPAGES::Order::API::Constants qw ( CART_PATCH_PARTIALLY_COMPLETE CART_PATCH_COMPLETE  );
use DE_EPAGES::Order::API::ProductValidator qw ( ValidateStockLevelSettings ValidateAvailibilityVisibilitySettings );

use DE_EPAGES::Payment::API::Constants qw ( CASHDISCOUNTMODE_FIX_DISCOUNT );
use DE_EPAGES::Shop::API::Constants qw(BASKETACTION_ADDNEWPOSITION);

use strict;

## no critic (ProhibitUnusedPrivateSubroutines)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Basket );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $Basket | A basket | DE_EPAGES::Order::API::Object::Basket
# return       $self | self | DE_EPAGES::Order::API::BasketPatch
#========================================================================================
sub new {
    my $class = shift;
    my ($Basket) = @_;
    my $self  = bless {
        'Basket' => $Basket,
    }, $class;
    return $self;
}

sub _add {
    my $self = shift;
    my ($Path, $hValue, $hParams) = @_;

    if($Path =~ m|^/lineItemContainer/productLineItems/([^/]+)/?$|) {
        my $LineItemGUID = $1;
        return $self->_updateSpecificProductLineItem($hValue, $LineItemGUID);
    }
    if($Path =~ m|^/lineItemContainer/productLineItems/([^/]+)/product-price/?$|) {
        my $LineItemGUID = $1;
        return $self->_updateProductPriceLineItem($hValue, $LineItemGUID);
    }
    if($Path eq '/lineItemContainer/productLineItems') {
        return $self->_addProductLineItem($hValue, $hParams);
    }
    if($Path eq '/billingAddress') {
        return $self->_setAddress($hValue, 'Billing');
    }
    if($Path eq '/shippingAddress') {
        return $self->_setAddress($hValue, 'Shipping');
    }
    if($Path eq '/coupon') {
        return $self->_setCoupon($hValue);
    }
    if($Path eq '/lineItemContainer/lineItemShipping') {
        return $self->_setShippingMethod($hValue);
    }
    if($Path eq '/lineItemContainer/lineItemPayment') {
        return $self->_setPaymentMethod($hValue);
    }

    Error('BadRequest', {'reason' => "Operation add on $Path is not supported!"});
}

sub _replace {
    my $self = shift;
    my ($Path, $hValue) = @_;

    if($Path eq '/locale'){
        return $self->_setLocale($hValue);
    }

    Error('BadRequest', {'reason' => "Operation add on $Path is not supported!"});
}

sub _remove {
    my $self = shift;
    my ($Path, $Value) = @_;

    if($Path =~ m|^/lineItemContainer/productLineItems/([^/]+)/?$|) {
        my $LineItemGUID = $1;
        return $self->_removeSpecificProductLineItem($LineItemGUID);
    }
    if($Path eq '/billingAddress') {
        return $self->_removeAddress('Billing');
    }
    if($Path eq '/shippingAddress') {
        return $self->_removeAddress('Shipping');
    }
    if($Path eq '/coupon'){
        return $self->_removeCoupon($Value);
    }

    Error('BadRequest', {'reason'=>"Operation remove on $Path is not supported!"});
}

#========================================================================================
# function     _addProductLineItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       _addProductLineItem( $rLineItemData, $hParams );
#----------------------------------------------------------------------------------------
# description  Adds the given product to the basket.
#----------------------------------------------------------------------------------------
# input        $rLineItemData | Info about the product(s) to add | ref.hash or ref.array
# input        $hParams | Info about status is returned by using this hash ref  | ref.hash
#========================================================================================
sub _addProductLineItem {
    my $self = shift;
    my ($rLineItemData, $hParams) = @_;

    my $Basket = $self->{'Basket'};
    my $LineItemContainer = $Basket->container();
    my $Shop = $Basket->getSite();

    # use this indicator for backward compatibility with the 
    # /carts/{cartid}/line-items resource
    # process extended error messages if we have an array as input (/carts/{cartid}/multi-line-items resource)
    my $IsArray;
    my $ahErrors = [];

    my $ahLineItemData;
    if (ref $rLineItemData eq 'ARRAY') {
        $ahLineItemData = $rLineItemData;
        $IsArray = 1;
    } else {
        $ahLineItemData = [$rLineItemData];
    }

    foreach my $hLineItemData (@$ahLineItemData){
        my $ProductGUID = $hLineItemData->{'productId'};
        my $Quantity = $hLineItemData->{'quantity'};

        eval {
            my $Product = $self->_loadShopObjectByGUID($Shop, $ProductGUID, 'Product');

            #-- Customisable products are not returned from this subroutine, as each 'add to cart' might have a different piece of text.
            #-- And we want to allow this by creating a distinct line item object each time.
            my $ProductLineItem = $self->_getLineItemByProduct($LineItemContainer, $Product);

            if (defined $ProductLineItem) {
                my $ProductLineItemGUID = $ProductLineItem->guid();
                my $PreviousQuantity    = $ProductLineItem->get('Quantity');
                my $NewQuantity         = $PreviousQuantity + $Quantity;
                my $hUpdateRequestBody = {
                    'quantity' => $NewQuantity,
                    'customizedText' => $hLineItemData->{'customizedText'},
                };

                $self->_updateSpecificProductLineItem($hUpdateRequestBody,$ProductLineItemGUID);

            } else {

                #-- A customisable product might already be in the basket, we need to include the number already in the cart for the stock level validation
                my $CheckQuantity = $Quantity;
                my $aLineItems = $LineItemContainer->get('LineItems');
                if ( $aLineItems ) {
                    foreach my $Lip ( grep { $_->instanceOf('LineItemProduct') && defined $_->get('Product') && $_->get('Product')->id == $Product->id && $_->get('CustomizedTextUnity') } @$aLineItems ) {
                        $CheckQuantity += $Lip->get('Quantity');
                    }
                }

                $self->_ValidateStockLevelSettings($Product, $CheckQuantity);
                $self->_ValidateAvailibilityVisibilitySettings($Product);

                eval {
                    $Product->addToContainer($LineItemContainer, {
                        'Quantity' => $Quantity // 1,
                        'CustomizedTextUnity' => $hLineItemData->{'customizedText'},
                        'AddProductStyle' => BASKETACTION_ADDNEWPOSITION,
                    });
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    if ( $Error->code() eq 'ProductNoPrice' ) {
                        Error('BadRequest', {'Reason' => 'Price is missing for the product'});
                    } else {
                        Error($Error);
                    }
                }
            }
        };
        # handle errors by a different way if we have an array as input
        if (ExistsError()) {
            my $Error = GetError();
            if ($IsArray) {
                # build the correct message for the failed product
                # and add error messages with GUID to array
                my $Message = $Error->vars()->{'reason'} ? $Error->vars()->{'reason'} : $Error->vars()->{'Reason'};
                $Message = $Error->shortMessage unless $Error->code eq 'BadRequest'; 
                push @$ahErrors, { $ProductGUID => $Message };
            } else {
                # normal error handling
                Error($Error);
            }
        }

        $LineItemContainer->recalculate();
    }

    if (scalar @$ahErrors ) {
        # throw error if we have errors on all items
        if (scalar @$ahErrors eq scalar @$ahLineItemData) {
            Error('BadRequest', {'Reason' => 'No products could be added to the basket'});
        } else {
            # add the status to $hParams for adding later to the basket
            $hParams->{'status'} = CART_PATCH_PARTIALLY_COMPLETE;
        }
    } else {
        # add this status only if we have more than one lineitem (multi-line-items call)
        $hParams->{'status'} = CART_PATCH_COMPLETE if $IsArray;
    }
    return;
}

#========================================================================================
# function     _getLineItemByProduct
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getLineItemByProduct( $LineItemContainer, $Product );
#----------------------------------------------------------------------------------------
# description  Returns the ProductLineItem for a given product if existing.
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# input        $Product | product | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub _getLineItemByProduct {
    my $self = shift;
    my ($LineItemContainer, $Product) = @_;

    # if we have a customized product we return 0 for allowing
    #  more than one of the same product in the basket

    my $LineItems = $LineItemContainer->get('LineItems');
    my @SameProductLineItems =
        grep { $_->instanceOf('LineItemProduct') and
               defined $_->get('Product') and
               $_->get('Product')->id == $Product->id and not
               $_->get('CustomizedTextUnity')
        } @$LineItems;

    return scalar( @SameProductLineItems) ? $SameProductLineItems[0] : undef;
}

#========================================================================================
# function     _updateSpecificProductLineItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       _updateSpecificProductLineItem( $hLineItemData, $hLineItemData );
#----------------------------------------------------------------------------------------
# description  Updates the given productLineItem in the basket.
#----------------------------------------------------------------------------------------
# input        $hLineItemData | Info about the product to add | ref_hash
# input        $LineItemGUID | GUID of the specific line item | string
#========================================================================================
sub _updateSpecificProductLineItem {
    my $self = shift;
    my ($hLineItemData, $LineItemGUID) = @_;

    my $Basket = $self->{'Basket'};
    my $Shop = $Basket->getSite();
    my $LineItemProduct = $self->_loadShopObjectByGUID($Shop, $LineItemGUID, 'LineItemProduct');
    $self->_checkIfLineItemBelongsToBasket($LineItemProduct, $Basket);

    my $Product = $LineItemProduct->get('Product');
    my $Quantity = $hLineItemData->{'quantity'};
    my $CustomizedText = $hLineItemData->{'customizedText'};

    $self->_ValidateStockLevelSettings($Product, $Quantity);
    $self->_ValidateAvailibilityVisibilitySettings($Product);

    my $hValuesToSet = {
        'Quantity' => $Quantity
    };
    $hValuesToSet->{'CustomizedTextUnity'} = $CustomizedText if $CustomizedText;
    $LineItemProduct->set($hValuesToSet);

    my $LineItemContainer = $Basket->container();
    $LineItemContainer->recalculate();

    return;
}

#========================================================================================
# function     _updateProductPriceLineItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       _updateProductPriceLineItem( $hLineItemData, $hLineItemData );
#----------------------------------------------------------------------------------------
# description  Updates the single price of the given productLineItem in the basket.
#----------------------------------------------------------------------------------------
# input        $hLineItemData | Info about the product to add | ref_hash
# input        $LineItemGUID | GUID of the specific line item | string
#========================================================================================
sub _updateProductPriceLineItem {
    my $self = shift;
    my ($hLineItemData, $LineItemGUID) = @_;

    my $Basket = $self->{'Basket'};
    my $Shop = $Basket->getSite();
    my $LineItemProduct = $self->_loadShopObjectByGUID($Shop, $LineItemGUID, 'LineItemProduct');
    $self->_checkIfLineItemBelongsToBasket($LineItemProduct, $Basket);

    my $ProductBasePrice = $hLineItemData->{'singleItemPrice'};
    return unless (defined $ProductBasePrice);

    my $LineItemContainer = $Basket->container();
    $LineItemProduct->set({
        'BasePrice' => $LineItemContainer->formatter->round($ProductBasePrice),
        'IsPatched' => 1
    });

    $LineItemContainer->recalculate();

    return;
}

#========================================================================================
# function     _removeSpecificProductLineItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       _removeSpecificProductLineItem( $LineItemGUID );
#----------------------------------------------------------------------------------------
# description  Removes the given product line item from the basket.
#----------------------------------------------------------------------------------------
# input        $LineItemGUID | GUID of the specific line item | string
#========================================================================================
sub _removeSpecificProductLineItem {
    my $self = shift;
    my ($LineItemGUID) = @_;

    my $Basket = $self->{'Basket'};
    my $Shop = $Basket->getSite();
    my $LineItemProduct = $self->_loadShopObjectByGUID($Shop, $LineItemGUID, 'LineItemProduct');
    $self->_checkIfLineItemBelongsToBasket($LineItemProduct, $Basket);

    my $LineItemContainer = $Basket->container();
    $LineItemProduct->delete();
    $LineItemContainer->recalculate();

    return;
}

sub _removeCoupon {
    my $self = shift;
    my ($Value) = @_;
    my $Basket = $self->{'Basket'};
    my $Container = $Basket->container;
    my $CouponLineItem = $Container->get('Coupon');
    if(defined $CouponLineItem) {
        if($Value eq $CouponLineItem->guid()) {
            $CouponLineItem->delete();
            $Container->recalculate();
        } else {
            Error('BadRequest', {'Reason' => 'cannot remove coupon, wrong GUID'});
        }
    }
    return;
}

sub _setLocale {
    my $self = shift;
    my ($LocaleID) = @_;

    my $Basket = $self->{'Basket'};
    eval {
        $Basket->container()->set({'LocaleID' => $LocaleID});
        $Basket->get('User')->set({'LocaleID' => $LocaleID});
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error('BadRequest', {'reason'=> $Error->shortMessage }, $Error);
    }

    return;
}

#========================================================================================
# function     _setAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setAddress( $hAddressData, $AddressType );
#----------------------------------------------------------------------------------------
# description  Sets the given address in the basket.
#----------------------------------------------------------------------------------------
# input        $hAddressData | address data | hashref
# input        $AddressType | type of the address (Billing or Shipping) | string
#========================================================================================
sub _setAddress {
    my $self = shift;
    my ($hAddressData, $AddressType) = @_;

    my $BasketAddress = $self->_createBasketAddress($self->{'Basket'});
    $BasketAddress->setAddress($hAddressData, $AddressType);

    $self->{'Basket'}->updateTaxArea();
    $self->{'Basket'}->recalculate();

    return;
}

#========================================================================================
# function     _removeAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _removeAddress( $AddressType );
#----------------------------------------------------------------------------------------
# description  Removes the given address type from the basket.
#----------------------------------------------------------------------------------------
# input        $AddressType | type of the address (Billing or Shipping) | string
#========================================================================================
sub _removeAddress {
    my $self = shift;
    my ($AddressType) = @_;

    my $BasketAddress = $self->_createBasketAddress($self->{'Basket'});
    $BasketAddress->removeAddress($AddressType);

    $self->{'Basket'}->updateTaxArea();
    $self->{'Basket'}->recalculate();

    return;
}

#========================================================================================
# function     _checkIfLineItemBelongsToBasket
# state        private
#----------------------------------------------------------------------------------------
# syntax       _checkIfLineItemBelongsToBasket( $LineItem, $Basket );
#----------------------------------------------------------------------------------------
# description  Throws an error if the LineItem does not belong to the basket.
#----------------------------------------------------------------------------------------
# input        $LineItem | A LineItem | DE_EPAGES::Order::API::Object::LineItem
# input        $Basket | A basket | DE_EPAGES::Order::API::Object::Basket
#========================================================================================
sub _checkIfLineItemBelongsToBasket {
    my $self = shift;
    my ($LineItem, $Basket) = @_;

    my $LineItemContainerOfLineItem = $LineItem->parent();
    my $BasketOfLineItem = $LineItemContainerOfLineItem->parent();

    my $LineItemGUID = $LineItem->guid();
    my $BasketGUID = $Basket->guid();
    my $BasketOfLineItemGUID = $BasketOfLineItem->guid();

    if($BasketGUID ne $BasketOfLineItemGUID) {
        Error('BadRequest', {'Reason' => "LineItem $LineItemGUID does not belong to the Basket $BasketGUID"});
    }

    return;
}

#========================================================================================
# function     _loadShopObjectByGUID
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $self->_loadShopObjectByGUID( $Shop, $GUID, $ClassAlias )
#----------------------------------------------------------------------------------------
# description  loads object specified by object GUID and verifies that is exists,
#               belongs to the current shop and is an instance of a certain class.
#               Raises an error if any condition is not fulfilled.
#               Returns the object if all tests passed.
#----------------------------------------------------------------------------------------
# input        $Shop | a shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $GUID | object GUID to load | string
# input        $ClassAlias | alias of a class | string
# return       $Object| object | object
#========================================================================================
sub _loadShopObjectByGUID {
    my $self = shift;
    my ($Shop, $GUID, $ClassAlias) = @_;

    my $ObjectID;
    eval {
        $ObjectID = LoadObjectByGUID($GUID)->id();
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( $Error->code() eq 'NoResults_ObjectByGUID' ) {
            Error('ResourceNotFound', {'GUID' => $GUID})
        } else {
            Error($Error);
        }
    }

    return $Shop->getSite()->loadObject($ObjectID, $ClassAlias // 'Object');
}

sub _createBasketAddress {
    my $self = shift;
    my ($Basket) = @_;
    return DE_EPAGES::Order::API::BasketAddress->new($Basket);
}

sub _ValidateStockLevelSettings {
    my $self = shift;
    my ($Product, $LineItemQuantity) = @_;
    eval {
        ValidateStockLevelSettings($Product, $LineItemQuantity);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ( $Error->code() eq 'Low StockLevel' ) {
            my $ProductGUID = $Product->guid();
            Error('BadRequest', {'reason'=>"StockLevel of product is lower then requested (total) quantity= $LineItemQuantity, productId= $ProductGUID"});
        } else {
            Error($Error);
        }
    }
    return;
}

sub _ValidateAvailibilityVisibilitySettings {
    my $self = shift;
    my ($Product) = @_;
    eval {
        ValidateAvailibilityVisibilitySettings($Product);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ( $Error->code() eq 'Not AvailableOrVisible' ) {
            my $ProductGUID = $Product->guid();
            Error('BadRequest', {'reason'=>"Product is not available on not visible, productId= $ProductGUID"});
        } else {
            Error($Error);
        }
    }
    return;
}

sub _setCoupon {
    my $self = shift;
    my ($CouponCode) = @_;

    my $Basket = $self->{'Basket'};
    my $Shop = $Basket->getSite();
    my $Folder = $Shop->folder('CouponCodes');
    my $Coupon = $Folder->existsChild($CouponCode) ? $Folder->child($CouponCode) : undef ;
    Error('BadRequest', {'Reason' => $CouponCode.' is not a valid coupon!'}) unless defined $Coupon;
    eval {
        $Coupon->addToContainer($Basket->container, {});
    };
    if (ExistsError()) {
    my $Error = GetError();
        Error('BadRequest', {'reason'=> $Error->shortMessage }, $Error);
    }
    my $LineItemContainer = $Basket->container();
    $LineItemContainer->recalculate();

    return undef;

}

#========================================================================================
# function     _setShippingMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $self->_setShippingMethod( $hShippingUpdate )
#----------------------------------------------------------------------------------------
# description  switches payment method from current lineItemPayment object from cart as
#               well as (eventually) the shipping cost.
#----------------------------------------------------------------------------------------
# input        $hShippingUpdate | shipping update values | hash
#========================================================================================
sub _setShippingMethod {
    my $self = shift;
    my ($hShippingUpdate) = @_;

    my $Basket = $self->{'Basket'};
    my $Shop = $Basket->getSite();

    my $NewShippingMethod = $self->_getValidatedShippingMethod($hShippingUpdate->{'id'}->{'value'}, $Shop);

    my $LineItemContainer = $Basket->get('LineItemContainer');
    my $LineItemShipping = $LineItemContainer->get('Shipping');

    my $NewLineItemPrice = $hShippingUpdate->{'priceAmount'};
    if (!isValid($NewLineItemPrice)) {
        $NewLineItemPrice = $LineItemContainer->formatter->round(
                $NewShippingMethod->calculateShippingPrice($LineItemContainer)
        );
    }

    $LineItemShipping->set({
        'ShippingMethod' => $NewShippingMethod,
        'LineItemPrice'  => $NewLineItemPrice,
        'IsPatched'      => 1
    });

    $LineItemContainer->recalculate();

    return;
}

#========================================================================================
# function     _setPaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $self->_setPaymentMethod( $hPaymentUpdate )
#----------------------------------------------------------------------------------------
# description  removes current lineItemPayment object from cart and adds a new one with
#               that has provided name and cost but doesn't point to any payment method
#               in the shop.
#----------------------------------------------------------------------------------------
# input        $hPaymentUpdate | payment update values | hash
#========================================================================================
sub _setPaymentMethod {
    my $self = shift;
    my ($hPaymentUpdate) = @_;

    my $Basket = $self->{'Basket'};

    my $LineItemContainer = $Basket->get('LineItemContainer');
    my $LineItemPayment = $LineItemContainer->get('Payment');
    my $TaxClass = $LineItemContainer->get('TaxMatrix')->defaultTaxClass();
    my $TaxRate = $LineItemContainer->taxRate($TaxClass);

    my $PaymentName = $hPaymentUpdate->{'name'};

    eval {
        Transaction(
            sub {
                my $LineItemPaymentDiscount = $LineItemContainer->get('PaymentDiscount');
                if (defined $LineItemPaymentDiscount) {
                    $LineItemPaymentDiscount->delete();
                }
                if (isValid($hPaymentUpdate->{'priceAmount'})) {
                    $self->_setPaymentMethodDiscount($LineItemContainer, $hPaymentUpdate->{'priceAmount'});
                }

                $LineItemPayment->delete();
                $LineItemContainer->insertLineItem(
                    LoadClassByAlias('LineItemPayment'),
                    {
                        'Alias'         => 'LineItemPayment_' . GetMD5Digest(\GenerateGUID()),
                        'Name'          => $PaymentName,
                        'PaymentMethod' => undef,
                        'TaxClass'      => $TaxClass,
                        'TaxRate'       => $TaxRate,
                        'IsPatched'     => 1
                    }
                );

                $LineItemContainer->recalculate();
            }
        );
    };
    if ( ExistsError() ) {
        Error(GetError());
    }

    return;
}


sub _getValidatedShippingMethod {
    my $self = shift;
    my ($ShippingMethodGUID, $Shop) = @_;

    my $ShippingMethod;
    eval {
        $ShippingMethod = LoadObjectByGUID($ShippingMethodGUID);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( $Error->code() eq 'NoResults_ObjectByGUID' ) {
            Error('BadRequest', {'reason' => "$ShippingMethodGUID is not a valid shipping id"});
        }
        Error($Error);
    }

    my $FolderShippingMethods = $Shop->folder('ShippingMethods');
    unless ( $FolderShippingMethods->existsChild($ShippingMethod->alias()) ) {
        Error('BadRequest', {'reason' => "$ShippingMethodGUID is not a valid shipping method!"});
    }

    unless ($ShippingMethod->get('UsableAtStorefront') && $ShippingMethod->get('IsActivated')) {
        Error('BadRequest', {'reason' => "$ShippingMethodGUID is unavailable for checkout!"});
    }

    return $ShippingMethod;
}

sub _setPaymentMethodDiscount {
    my $self = shift;
    my ($LineItemContainer, $priceAmount) = @_;

    my $BasePrice = $LineItemContainer->get('PaymentSubTotal');
    my $DiscountableSumDiff = $LineItemContainer->get('LineItemsSum') - $LineItemContainer->get('LineItemsSubTotalDiscountable');
    $BasePrice -= $DiscountableSumDiff;

    my $hInfo = {
        'Quantity'      => 1,
        'BasePrice'     => $BasePrice,
        'LineItemPrice' => $priceAmount,
        'Discount'      => -$priceAmount,
        'DiscountMode'  => CASHDISCOUNTMODE_FIX_DISCOUNT,
        'IsPatched'     => 1
    };

    $LineItemContainer->insertLineItem(LoadClassByAlias('LineItemPaymentDiscount'), $hInfo );

    return;
}

1;
