#========================================================================================
# package      DE_EPAGES::Order::Hooks::Basket
# state        public
# description  Hook functions for basket to order process.
#========================================================================================
package DE_EPAGES::Order::Hooks::Basket;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Order::API::LineItemContainer qw (UpdateTaxModel);
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::Permission::API::Permission qw( ExistsActionByName );
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_NET );

#========================================================================================
# function     OnBasket2OrderMinAmount
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderMinAmount( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to test if the minimal
#               order amount was reached.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderMinAmount {
    my ($hParams) = @_;
    my $Basket = $hParams->{'Basket'};
    return if $Basket->get('IsMinOrderAmountReached');

    my $Servlet = $hParams->{'Servlet'};
    $Servlet->vars('ObjectID', $Basket->id);

    # show basket template
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
        },
        'ViewAction' => 'View',
    });
    return;
}

#========================================================================================
# function     OnBasket2OrderAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderAddress( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to give the user the
#               possiblity to change billing, shipping address. Its possible to
#               prevent the interruption with the basket attribute 'IsAddressOK' = 1.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderAddress {
    my ($hParams) = @_;
    my $Basket = $hParams->{'Basket'};
    return if $Basket->get('IsAddressOK');

    my $Shop = $Basket->getSite();

    # verify addresses of basket, collect form errors.
    my $Servlet = $hParams->{'Servlet'};
    $Servlet->vars('ObjectID', $Basket->id);
    my $UIPackage = DE_EPAGES::Order::UI::Basket->new(Servlet=>$Servlet);
    $UIPackage->VerifyAdresses($Servlet);
    UpdateTaxModel($Basket->container);

    if (not scalar @{$Servlet->form->errors->{'Errors'}}) {
        $Basket->set({'IsAddressOK'=>1});
        if ($Shop->get('TaxCalculationAfterAddress') && $Shop->get('TaxModel') == TAX_MODEL_NET) {
            $Basket->container->recalculate();
        }
        return;
    }

    $Basket->container->recalculate();

    my $AddressClass = LoadClassByAlias('Address');
    my $hAddressDefaults = $UIPackage->defaultValues( $Basket, $AddressClass );
    my %Defaults;
    $Defaults{'Shipping'.$_} = $hAddressDefaults->{$_} foreach keys %$hAddressDefaults;
    $Defaults{'Billing'.$_}  = $hAddressDefaults->{$_} foreach keys %$hAddressDefaults;

    foreach my $Class ($Shop->get('CustomerClass'), LoadClassByAlias('User')) {
        my $hDefaults = $UIPackage->defaultValues( $Basket, $Class );
        $Defaults{$_} = $hDefaults->{$_} foreach keys %$hDefaults;
    }

    # delete following Defaults when the params available
    if( exists $Defaults{'LanguageID'} and $Servlet->params('Language') ) {
        delete $Defaults{'LanguageID'};
    }
    if( exists $Defaults{'LocaleID'} and $Servlet->params('Locale') ) {
        delete $Defaults{'LocaleID'};
    }
    if( exists $Defaults{'CurrencyID'} and $Servlet->params('Currency') ) {
        delete $Defaults{'CurrencyID'};
    }

    delete $Defaults{'ShippingAddress'} if defined $Basket->get('ShippingAddress');
    delete $Defaults{'Class'};
    delete $Defaults{'ClassID'};

    # show address template
    Error('INVALID_FORM', {
        'ViewVars'=> {
            %Defaults
        },
        'ViewAction' => 'ViewAddress',
    });
    return;
}

#========================================================================================
# function     OnBasket2OrderShippingPaymentMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderShippingPaymentMethods( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to test
#               if the countries of regions from the payment- and shippingmethods
#               consits the countries from the Billing- and Shippingaddress
#               If the test failed the Basket Objects is shown.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderShippingPaymentMethods {
    my ($hParams) = @_;


    my $Basket = $hParams->{'Basket'};
    my $LineItemContainer = $Basket->container;

    # test payment
    my $PaymentIsOK = 1;

    # do we have a payment and was the payment changed ?
    # is the PaymentSubTotal of the LineItemContainer also > 0 ?
    if ($LineItemContainer->needPayment and $LineItemContainer->get('PaymentSubTotal') > 0 ) {

        my $PaymentMethodChanged = $Basket->updatePaymentMethod();
        if (defined $PaymentMethodChanged){
            # payment was changed
            $PaymentIsOK = 0 if $PaymentMethodChanged == 1;
            # payment does not exists
            $PaymentIsOK = 0 if $PaymentMethodChanged == -1;
        }
    }

    # test shipping
    my $ShippingIsOK = 1;
    # do we have a payment and was the payment changed ?
    if ($LineItemContainer->needShipping) {
        my $ShippingMethodChanged = $Basket->updateShippingMethod();
        if (defined $ShippingMethodChanged ){
            # shipping was changed
            $ShippingIsOK = 0 if $ShippingMethodChanged == 1;
            # shipping does not exists
            $ShippingIsOK = 0 if $ShippingMethodChanged == -1;
        }
    }

    # continue the order process
    return if $PaymentIsOK and $ShippingIsOK ;

    # show basket template
    my $Servlet = $hParams->{'Servlet'};
    $Servlet->vars('ObjectID', $Basket->id);
    $Servlet->form->addFormError({ 'Reason'=>'PaymentMethodChangedByAddress',  'Name'=> 'Payment' }) unless $PaymentIsOK;
    $Servlet->form->addFormError({ 'Reason'=>'ShippingMethodChangedByAddress', 'Name'=> 'Shipping' }) unless $ShippingIsOK;
    $Basket->recalculate;
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'ObjectID' => $Basket->id,
            'ShippingPaymentMethodError' => 1,
        },
        'ViewAction' => 'View',
    });
    return;
}

#========================================================================================
# function     OnBasket2OrderHirePurchase
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderHirePurchase( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process. $PaymentLineItem->hirePurchase($hParams) is called.
#               The payment method has to handle if it has already all data.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Basket
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderHirePurchase {
    my ($hParams) = @_;
    my $Payment = $hParams->{'Basket'}->container->get('Payment');
    return unless $Payment;
    $Payment->hirePurchase($hParams);
    return;
}

#========================================================================================
# function     OnBasket2OrderPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderPayment( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to give the user the
#               possiblity to insert some payment data.
#               The payment method has to handle if it has already all data.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Basket
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderPayment {
    my ($hParams) = @_;
    my $Payment = $hParams->{'Basket'}->container->get('Payment');
    return unless $Payment;
    $Payment->prepareOrder($hParams);
    return;
}

#========================================================================================
# function     OnBasket2OrderTAC
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderTAC( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to give the user the
#               possiblity to accept the terms and conditions. Its possible to
#               prevent the interruption with the basket attribute 'IsTacOK' = 1.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderTAC {
    my ($hParams) = @_;
    my $Basket = $hParams->{'Basket'};
    my $Servlet = $hParams->{'Servlet'};

    my @ChangeActions = $Servlet->form->value('ChangeAction');
    return if $Basket->get('IsTacOK') && ElementOf('TestTermsAndConditions', @ChangeActions);

    my $ViewAction = $Servlet->form->value('ViewAction') || 'ViewOffer';
    if(!ExistsActionByName($Basket->class,$ViewAction)){
        $ViewAction = 'ViewOffer';
    }

    # show offer template
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'ViewObjectID' => $Basket->id,
            'ViewAction' => $ViewAction,
        },
    });
    return;
}

#========================================================================================
# function     OnBasket2OrderCustomerInformation
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderCustomerInformation( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to give the user the
#               possiblity to accept the 'no right of withdrawal'. Its possible to
#               prevent the interruption with the basket attribute 'IsCustomerInformationOK' = 1.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderCustomerInformation {
    my ($hParams) = @_;
    my $Basket = $hParams->{'Basket'};
    my $Servlet = $hParams->{'Servlet'};

    my @ChangeActions = $Servlet->form->value('ChangeAction');
    return if $Basket->get('IsCustomerInformationOK')
        and ElementOf('TestCustomerInformation', @ChangeActions);

    my $ViewAction = $Servlet->form->value('ViewAction') || 'ViewOffer';
    unless (ExistsActionByName($Basket->class, $ViewAction)) {
        $ViewAction = 'ViewOffer';
    }

    # show offer template
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'ViewObjectID' => $Basket->id,
            'ViewAction' => $ViewAction,
        },
    });
    return;
}

#========================================================================================
# function     OnBasket2OrderExecutePayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderExecutePayment( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to give the user the
#               possiblity to go to payment server.
#               The payment method has to handle if it has already all data.
#               Attention: everything else should be handled before!
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Basket
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderExecutePayment {
    my ($hParams) = @_;

    my $PaymentLineItem = $hParams->{'Basket'}->container->get('Payment');
    return unless $PaymentLineItem;
    $PaymentLineItem->executePayment($hParams) if $PaymentLineItem->get('PaymentMethod')->get('DoPaymentOnBasket');
    return;
}

#========================================================================================
# function     OnBasket2OrderTax
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderTax( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to give the user the
#               possiblity to change billing, shipping address. Its possible to
#               prevent the interruption with the basket attribute 'IsAddressOK' = 1.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderTax {
    my ($hParams) = @_;
    my $Basket = $hParams->{'Basket'};
    my $Servlet = $hParams->{'Servlet'};
    my $Shop = $Basket->getSite();

   if($Shop->get('TaxMethod')){
        $Shop->get('TaxMethod')->updateContainerTaxArea($Basket->container);
    } else {
        my $TaxChanged = $Basket->updateTaxArea();
        $Servlet->params('TaxChanged', 1) if $TaxChanged;
        my $TaxChangedDigital = $Basket->updateTaxAreaDigital();
        $Servlet->params('TaxChangedDigital', 1) if $TaxChangedDigital;
    }

    return;
}

#========================================================================================
# function     OnViewBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnViewBasket( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_ViewBasket triggered, to test the basket if items have
#               changed outside the ChangeAction
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnViewBasket {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object;

    return unless defined $Basket;

    my @ProductsMarkedAsChanged;
    my $aProductLineItems = $Basket->container->childrenByClass('LineItemProduct');
    foreach my $LineItem ( @{$aProductLineItems} ) {
        my $Product = $LineItem->get('Product');
        next unless defined $Product;
        if ($LineItem->get('IsChanged') == 1) {
            push(@ProductsMarkedAsChanged,{'Name' => $Product->get('NameOrAlias')});
        }
    }

    #Only set TLE if the array is not empty
    $Servlet->vars('ProductsMarkedAsChanged', \@ProductsMarkedAsChanged) if @ProductsMarkedAsChanged;

    return;
}

#========================================================================================
# function     OnAfterSaveOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterSaveOrder( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_AfterSaveOrder triggered, to test the basket.
#               Only used for old checkout and mobile version.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterSaveOrder {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object;

    return unless defined $Basket;

    # test basket min order amount
    unless ($Basket->get('IsMinOrderAmountReached')) {

        $Servlet->form->addFormError({
            'Reason'=> 'MinOrderIsNotReached',
            'Name'  => 'Basket'
        }) ;
    }

    # Add form errors if product is changed and if not available
    _AddProductLineItemFormErrors($hParams);
    $Servlet->form->executeFormError;

    return;
}

#========================================================================================
# function     _AddProductLineItemFormErrors
# state        private
#----------------------------------------------------------------------------------------
# syntax       _AddProductLineItemFormErrors( $aProductLineItems );
#----------------------------------------------------------------------------------------
# description  This function adds form errors if product line item is not available
#               or changed.
#----------------------------------------------------------------------------------------
# input        $aProductLineItems | product line item array as ref
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub _AddProductLineItemFormErrors {

    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $aProductLineItems = $Servlet->object->container->childrenByClass('LineItemProduct');
    my @DeletedProducts = ();
    my @ProductsMarkedAsChanged;

    foreach my $LineItem ( @{$aProductLineItems} ) {

        my $Product = $LineItem->get('Product');
        if (!defined $Product) {
            push @DeletedProducts, { Name =>$LineItem->get('Name') };
            next
        }

        if ($LineItem->get('IsChanged') == 1) {
            push(@ProductsMarkedAsChanged,{'Name' => $Product->get('NameOrAlias')});
        }

        if ($Product->get('IsAvailable') == 0) {
            $Servlet->form->addFormError({
                'Reason' => 'ProductNotAvailable',
                'Name'   => 'ProductAlias',
                'Value'  => $Product->get('NameOrAlias'),
            });
        }
    }

    if (scalar @ProductsMarkedAsChanged) {
        $Servlet->form->addFormError({
            'Reason' => 'ProductMarkedAsChanged',
            'Name'   => 'ProductAlias',
            'Value'  => '',
        });
        $Servlet->vars('ProductsMarkedAsChangedInChangeAction', 1);
        $Servlet->vars('ProductsMarkedAsChanged', \@ProductsMarkedAsChanged);
    }

    if (scalar @DeletedProducts) {
        $Servlet->form->addFormError({
            'Reason' => 'ProductDeleted',
            'Name'   => 'ProductsDeleted',
            'Value' => '',
        });
        $Servlet ->vars('DeletedProducts', \@DeletedProducts);
    }

    return;
}
#========================================================================================
# function     OnAfterUpdateBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateBasket ($hParams);
#----------------------------------------------------------------------------------------
# description  Updates the TaxArea of the Baskets LineItemContainer id we have a valid user
#               (Hook OBJ_AfterUpdateBasket)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - basket - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateBasket {
    my ($hParams) = @_;
    my $Basket    = $hParams->{'Object'};
    my $Values    = $hParams->{'Values'};
    my $Shop      = $Basket->getSite();

    return unless $Shop->get('DeliveryThresholdIsActive');
    return unless $Values->{'User'};
    return if $Values->{'User'}->get('IsAnonymous');

    $Basket->updateTaxArea();

}
1;
