    #========================================================================================
# package      DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress
# state        public
# base         DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal
# description  object interface for LineItemPaymentPayPalExpress.
#========================================================================================
package DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress;
use base DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::PayPalExpress::API::Payment qw ( CheckoutRequest );
use DE_EPAGES::PayPalExpress::API::Request::URLBuilder;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $LineItemPaymentPayPalExpress->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'LineItemPaymentPayPalExpress' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $LineItemPaymentPayPalExpress->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentPayPalExpress = DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemPaymentPayPalExpress');
#               my $LineItemPaymentPayPalExpress = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemPaymentPayPalExpress' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemPaymentPayPalExpress | new object | object
# hook         OBJ_InsertLineItemPaymentPayPalExpress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentPayPalExprID - line item payment paypal express id - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertLineItemPaymentPayPalExpress', {'Object' => $self, 'LineItemPaymentPayPalExprID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Token PendingReason PayerID PayerEMail ShippingAddressChanged ShippingAddressID IsExpressCheckout ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentPayPalExpress->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemPaymentPayPalExpress' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemPaymentPayPalExpress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentPayPalExprID - line item payment paypal express id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItemPaymentPayPalExpress', {'Object' => $self, 'LineItemPaymentPayPalExprID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentPayPalExpress->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItemPaymentPayPalExpress'
#               and 'OBJ_AfterUpdateLineItemPaymentPayPalExpress'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItemPaymentPayPalExpress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentPayPalExprID - line item payment paypal express id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItemPaymentPayPalExpress | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentPayPalExprID - line item payment paypal express id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateLineItemPaymentPayPalExpress', {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentPayPalExprID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateLineItemPaymentPayPalExpress',  {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentPayPalExprID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     executePayment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentPayPalExpress->executePayment($hParams);
#----------------------------------------------------------------------------------------
# description  This function directs the payment process.
#----------------------------------------------------------------------------------------
# input        $hValues | environment information
#                       <ul>
#                           <li>Basket - basket object (optional)</li>
#                           <li>Servlet - servlet handling the current request</li>
#                           <li>Order - order object (optional)</li>
#                       </ul>
#                   | ref.hash
#========================================================================================
sub executePayment {
    my $self        = shift;
    my ($hParams)   = @_;

    $self->updateOnExecute($hParams);

    # return if we come from a callback (see UI::Basket::PayPalExpressReturn)
    my $APIStep = $self->get('Field1');
    return if (defined $APIStep and ElementOf($APIStep, qw(DoExpressCheckoutPayment CheckoutRequestDetails)));

    # start standard paypal payment
    my $redirectUrl = $self->checkoutRequest($hParams->{'Servlet'}, 0);

    $hParams->{'Servlet'}->vars('PayPalExpressRedirectUrl', $redirectUrl);

    # show payment template
    my $Object = $hParams->{'Order'} // $hParams->{'Basket'};
    my $ViewAction = $hParams->{'Servlet'}->vars('IsMobile') ? 'MobileSF-ViewPaymentPayPalExpress' : 'ViewPaymentPayPalExpress';
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError'    => 0,
            'ViewObjectID' => $Object->id,
        },
        'ViewAction' => $ViewAction
    });

    return;
}

#--- start payment processing; used for both EC and ST
sub checkoutRequest {
    my $self = shift;
    my ($Servlet, $IsExpressCheckout) = @_;

    my %UpdateParams = ('Servlet' => $Servlet);
    my $PaymentObject = $self->container->parent; # Basket or Order
    if ($PaymentObject->instanceOf('CustomerOrder')) {
        $UpdateParams{'Order'} = $PaymentObject;
    }
    else {
        $UpdateParams{'Basket'} = $PaymentObject;
    }
    $self->updateOnExecute(\%UpdateParams);

    # reset any status from previous payment attempts
    $self->set({'TransStatus' => 'new'});

    # save if the Express button was used
    $self->set({'IsExpressCheckout' => $IsExpressCheckout});

    my $User       = $PaymentObject->get('User');
    my $BuyerEmail = $User->get('EMail');

    if (!$BuyerEmail =~ /\S/ && defined($User->get('BillingAddress'))) {
        $BuyerEmail = $User->get('BillingAddress')->get('EMail');
    }
    my $URLBuilder = DE_EPAGES::PayPalExpress::API::Request::URLBuilder->new( $Servlet );
    my %Info = (
        'LocaleCode'   => $Servlet->locale,
        'BuyerEmail'   => $BuyerEmail,
        'ObjectID'     => $PaymentObject->id,
        'ADDROVERRIDE' => $IsExpressCheckout ? 0 : 1,
        'ReturnURL'    => $URLBuilder->getReturnURL( $PaymentObject ),
        'BUTTONSOURCE' => LoadRootObject()->get($IsExpressCheckout
                            ?  'PayPalExpress_BuildNotationID'
                            :  'PayPalExpress_BuildNotationID_ST'),
        'IsExpressCheckout' => $IsExpressCheckout,
    );
    my $Shop = $self->getSite();
    if ($PaymentObject->instanceOf('CustomerOrder')) {
        $Info{'CancelURL'} = $URLBuilder->getCancelURLCustomerOrder( $PaymentObject );
        $Info{'GiroPaySuccessURL'} = $URLBuilder->getGiroPaySuccessURLCustomerOrder($PaymentObject);
    }
    else {
        $Info{'CancelURL'} = $URLBuilder->getCancelURLBasket($PaymentObject);
        $Info{'GiroPaySuccessURL'} = $URLBuilder->getGiroPaySuccessURLBasket( $Shop, $self->get('GUID') );
    }

    $Info{'GiroPayCancelURL'} = $URLBuilder->getGiroPayCancelURL( $Shop, $self->get('GUID') );
    $Info{'CallbackURL'} = $URLBuilder->getCallbackURL( $PaymentObject, $PaymentObject->get('GUID') );

    #--- send request
    my $redirectUrl;
    eval {
        $redirectUrl = CheckoutRequest($self, \%Info);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'PayPalExpressFailed') {
            my $ErrorViewAction = 'ViewPayPalExpressFailed';
            unless ($Error->vars()->{'ErrorCode'} eq '10486') {
                if ($Shop->isUnity() && $PaymentObject->instanceOf('CustomerOrder')) {
                    $self->set({'PaymentErrorCode' => 'Failed'});
                    $ErrorViewAction = 'ViewMultiCheckoutCustomerOrder';
                }
            }
            Error('INVALID_FORM', {
                'ViewVars' => {
                    'FormError'           => 0,
                    'PayPalExpressFailed' => 1,
                    'ErrorCode'           => 'PayPalExpressInvalid',
                    'CorrelationID'       => $Error->vars->{'CorrelationID'},
                    'PPEErrorCode'        => $Error->vars->{'ErrorCode'},
                    'PayPalMessage'       => $Error->vars->{'LongMessage'}
                },
                'ErrorObjectID' => $PaymentObject->id,
                'ViewAction'    => $ErrorViewAction
            });
        }
        else {
            Error($Error);
        }
    }
    $redirectUrl .= '&useraction=commit' unless ($IsExpressCheckout);
    return $redirectUrl;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'ShippingAddressChanged' => 0,
        'IsExpressCheckout' => 0,
    };
}

#========================================================================================
# function     setTransactionTo
# state        public
#----------------------------------------------------------------------------------------
# syntax       setTransactionTo( $TransStatus, $PPEErrorCode );
#----------------------------------------------------------------------------------------
# description  If an Error appears the error code will be set in the transaction list.
#----------------------------------------------------------------------------------------
# input        $TransStatus | transaction status | string
# input        $PPEErrorCode | paypal express error code | integer
#========================================================================================
sub setTransactionTo {
    my $self = shift;
    my ( $TransStatus, $PPEErrorCode ) = @_;

    $self->set({'TransStatus' => $TransStatus});

    my $MaxLength = 11; ## no critic (ProhibitMagicNumbers)
    if( isValid($PPEErrorCode) && length($PPEErrorCode) <= $MaxLength ){
        $self->set({'PaypalErrorCode' => $PPEErrorCode});
    } else {
        $self->set({'PaypalErrorCode' => undef});
    }

    if( not isValid($self->get('TransID')) ){
        $self->set({'TransID' => '-'});
    }

    $self->set({'TransTime' => $self->_GetCurrentDBHandle()->currentDateTime()});

    return;
}
#----------------------------------------------------------------------------------------

sub _GetCurrentDBHandle {
    my $self = shift;
    return GetCurrentDBHandle();
}
1;
