#==============================================================================
# package      DE_EPAGES::PayPalExpress::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::PayPalExpress::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use strict;

use Data::Dumper qw ( Dumper );
use HTTP::Request;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf UsePackage fcmp );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByGUID
    LoadObjectByGUID
    LoadRootObject
);
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );
use DE_EPAGES::Object::API::Table::Locale qw ( GetInfoLocale );
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::ShopMailType::API::ShopMailType qw ( GetShopMailType );
use DE_EPAGES::PayPalExpress::API::Payment qw ( SendGETPayPalRequestCurl ); ##no critic qw(DE_EPAGES::Style::RequireCorrectImports)

use DE_EPAGES::Core::API::Constants qw(PERL538);
BEGIN {
    if ($] < PERL538) {
        # use old packages for old perl
        UsePackage('Net::INET6Glue');
    }
}


#========================================================================================
# function     PayPalExpressIPN
# state        public
#----------------------------------------------------------------------------------------
# syntax       PayPalExpressIPN( $Servlet )
#----------------------------------------------------------------------------------------
# description  Listener for incoming IPN messages sent to us from PayPal.
#               This method must respond to all incoming messages by relaying them to
#               PayPal for verification. Further processing may be carried out only if
#               the IPN is verified by PayPal. IPN messages may be duplicated, thus
#               some additional checks test for a matching IPN trans ID and status.
#               More info see: https://developer.paypal.com/api/nvp-soap/ipn/IPNIntro/
# input        $Servlet | servlet object | object
#========================================================================================
sub PayPalExpressIPN {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $hFormValues = $Servlet->params();
    LogPayment('PayPalExpress', 'PayPalExpressIPN input values', $hFormValues);

    my $PaymentLineItem;
    if (defined($hFormValues->{'PaymentGUID'}) && ExistsObjectByGUID($hFormValues->{'PaymentGUID'})) {
        $PaymentLineItem = LoadObjectByGUID($hFormValues->{'PaymentGUID'});
    } else {
        LogPayment('PayPalExpress', 'Either no PaymentGUID was given or no object could be loaded for GUID', { 'PaymentGUID' => $hFormValues->{'PaymentGUID'} });
    }

    my $IsInTestMode;
    if (defined($PaymentLineItem)) {
        $IsInTestMode = $PaymentLineItem->get('PaymentMethod')->get('IsInTestMode');
    } else {
        $IsInTestMode = $Shop->get('IsInternalTestShop');
    }
    return undef unless $self->_verifyPayPalExpressIPN($Servlet->request, $IsInTestMode);
    return undef unless defined($PaymentLineItem);

    # ignore IPN calls for refunds
    return undef if (defined($hFormValues->{'reason_code'}) && $hFormValues->{'reason_code'} =~ /refund/i);

    # test the parameters of PayPal, show errors if something is wrong
    my $hValues = $self->TestPayPalExpressParametersIPN($Servlet, $hFormValues);
    return undef unless defined($hValues);
    LogPayment('PayPalExpress', 'PayPalExpressIPN verified and validated values:', $hValues);

    $PaymentLineItem = delete($hValues->{'PaymentLineItem'});

    # Ignore duplicate IPN messages
    if ($PaymentLineItem->get('LastIPNTransID') eq $hValues->{'TransID'}
        && $PaymentLineItem->get('LastIPNTransStatus') eq lc($hValues->{'TransStatus'})) {
        LogPayment('PayPalExpress', 'Ignoring duplicate IPN message', $hValues);
        return;
    }

    # EPG-26457: prevent creating an order if transaction status isn't positive and order creation occurs only if payment is successful
    my $DoPaymentOnBasket = $PaymentLineItem->get('PaymentMethod')->get('DoPaymentOnBasket');
    return undef if $DoPaymentOnBasket and
        not ElementOf(lc($hValues->{'TransStatus'}), qw(completed pending created processed));

    # Update the PaymentLineItem DB object
    $PaymentLineItem->set($hValues);

    my $Object = $PaymentLineItem->container->parent;
    my $Order = $Object->instanceOf('CustomerOrder') ? $Object : DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $Object);

    # Set order status
    $self->_SetTransStatusOnOrder($Order, lc($hValues->{'TransStatus'})) if $hValues->{'TransStatus'};

    # Set LastIPNTxId and LastIPNTxStatus (in order to avoid dupes)
    $PaymentLineItem->set({'LastIPNTransID' => $hValues->{'TransID'}});
    $PaymentLineItem->set({'LastIPNTransStatus' => lc($hValues->{'TransStatus'})}) unless $hValues->{'TransStatus'} eq 'NotValidatedYet';

    $Servlet->vars('ExitAfterEvent', 1);
    $Servlet->vars('ResponseContentProcessed', 1);
}

#==============================================================================
sub TestPayPalExpressParametersIPN {
    my $self = shift;
    my ($Servlet, $FormValues) = @_;
    my $Shop = $Servlet->object;

    my ($check, $transactionID, $transactionStatus);
    eval {
        ($check, $transactionID, $transactionStatus) = $self->_doValidation($Shop, $FormValues);
    };
    if (ExistsError()) {
        Warning( GetError() );
        return undef;
    }

    my $PaymentLineItem = LoadObjectByGUID($FormValues->{'PaymentGUID'} );
    my $Now = GetCurrentDBHandle()->currentDateTime;

    my %Values = (
        'PaymentLineItem' => $PaymentLineItem,
        'TransTimeMod'    => $Now,
        'IsSecure'        => defined($FormValues->{'payer_status'}) && lc($FormValues->{'payer_status'}) eq 'verified' ?  1 : 0,
    );

    if (defined($FormValues->{'amt'}) && defined($FormValues->{'cc'})) {
        $Values{'Amount'}     = $FormValues->{'amt'};
        $Values{'CurrencyID'} = $FormValues->{'cc'};
    }
    elsif (defined($FormValues->{'mc_gross'}) && defined($FormValues->{'mc_currency'})) {
        $Values{'Amount'}     = $FormValues->{'mc_gross'};
        $Values{'CurrencyID'} = $FormValues->{'mc_currency'};
    }
    elsif (defined($FormValues->{'mc_gross_1'}) && defined($FormValues->{'mc_currency'})) {
        # should not happen as _doValidation copies the value from mc_gross_1 to mc_gross
        $Values{'Amount'}     = $FormValues->{'mc_gross_1'};
        $Values{'CurrencyID'} = $FormValues->{'mc_currency'};
    }

    if ($check && defined($transactionStatus)) {
        $Values{'TransStatus'} = $transactionStatus;
    }
    elsif (!$PaymentLineItem->get('TransStatus')) {
        $Values{'TransStatus'} = 'NotValidatedYet';
    }
    $Values{'TransID'}    = $transactionID if defined $transactionID;
    $Values{'TransTime'}  = $Now unless defined $PaymentLineItem->get('TransTime');
    $Values{'PayerEMail'} = $FormValues->{'payer_email'} if defined $FormValues->{'payer_email'};

    return \%Values;
}

#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  return PayPalExpress as string. Name of PaymentType for search/view
#                               transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName { 'PayPalExpress' }

#========================================================================================
sub PayPalExpressGiroPaySuccess { # show order page
    my $self = shift;
    my ($Servlet) = @_;

    my $guid = $Servlet->params->{'ppeguid'};
    Error('NO_GUID') unless defined $guid;

    LogPayment('PayPalExpress', 'guid', $guid);
    Error('NO_PAYMENTLINEITEM') unless ExistsObjectByGUID($guid);
    my $pli = LoadObjectByGUID($guid);

    my $PaymentObject = $pli->container->parent;
    Error('NO_ORDER') unless $PaymentObject->instanceOf('CustomerOrder');

    # show order confirmation page
    $Servlet->vars('ObjectID', $PaymentObject->id);
    $Servlet->vars('ViewAction', 'View');

    return;
}

#========================================================================================
sub PayPalExpressGiroPayCancel {
    my $self = shift;
    my ($Servlet) = @_;

    my $guid = $Servlet->params->{'ppeguid'};
    Error('NO_GUID') unless defined $guid;

    LogPayment('PayPalExpress', 'guid', $guid);
    Error('NO_PAYMENTLINEITEM') unless ExistsObjectByGUID($guid);
    my $pli = LoadObjectByGUID($guid);

    my $PaymentObject = $pli->container->parent;
    $Servlet->vars('ObjectID', $PaymentObject->id);
    if ($PaymentObject->instanceOf('Basket')) { # show basket page again, but this should not happen
        $Servlet->vars('ViewAction', 'View');
    }
    else { # we have an order, show cancel page
        $Servlet->vars('ViewAction', 'ViewPayPalExpressCancel');
    }

    return;
}

#==============================================================================
sub _verifyPayPalExpressIPN {
    my $self = shift;
    my ($Request, $TestMode) = @_;

    my $URL = LoadRootObject()->get($TestMode ? 'PayPalExpress_SandboxURL' : 'PayPalExpress_ProductionURL');
    # tell PayPal we want to verify that IPN message (this also serves as acknolwedgement of receipt)
    # paypal docs state that the cmd=_notify-validate is to be prepended
    my $Query = 'cmd=_notify-validate&' . $Request->content;

    LogPayment('PayPalExpress', "Verify IPN: query to send to $URL", $Query);

    # PayPal's docs say use POST but that doesn't work and using GET does work.
    my $Response = SendGETPayPalRequestCurl($URL . '?' . $Query);
    if ( ! defined $Response ) {
        #-- Try again quickly, maybe it'll work a second time in case of connection errors
        LogPayment('PayPalExpress', '_verifyPayPalExpressIPN RETRY GET', '1');
        $Response = SendGETPayPalRequestCurl($URL . '?' . $Query);
    }

    # check verification response
    if (defined $Response && $Response->is_success && ($Response->content() =~ m/^VERIFIED/)) {
        LogPayment('PayPalExpress', "Verify IPN: response from $URL for payment method", $Response->content());
        return 1;
    }

    if ( defined $Response ) {
        eval { Error('IPN_VERIFICATION_FAILURE', { 'Response' => Dumper($Response->content()) }); };
        if (ExistsError()) {
            LogError(GetError());
        }
    }

    return 0;
}

#==============================================================================
sub _doValidation {
    my $self = shift;
    my ($Shop, $FormValues) = @_;
    my ($check, $transactionID, $transactionStatus);

    my $PaymentLineItemGUID = $FormValues->{'PaymentGUID'};
    Error('BadRedirect_NoPaymentGUID', {
        'Name'       => 'PaymentGUID',
        'Reason'     => 'BadRedirect_NoPaymentGUID',
    }) unless (defined $PaymentLineItemGUID && ExistsObjectByGUID($PaymentLineItemGUID));
    my $PaymentLineItem = LoadObjectByGUID($FormValues->{'PaymentGUID'});

    Error('BadRedirect_ShopIDMisMatch', {
        'Name'       => 'ShopID',
        'Reason'     => 'BadRedirect_ShopIDMisMatch'
    }) if ($PaymentLineItem->getSite()->id != $Shop->id );

    my $Container = $PaymentLineItem->get('LineItemContainer');
    my $GrandTotal = $Container->get('GrandTotal');
    my $CurrencyID = $Container->get('CurrencyID');

    if (defined($FormValues->{'amt'}) && defined($FormValues->{'cc'})) {
        if (fcmp($GrandTotal, $FormValues->{'amt'}) != 0 || $CurrencyID ne $FormValues->{'cc'}) {
            LogPayment('PayPalExpress', 'BadRedirect_AmountMisMatch for payment line item '
                . $PaymentLineItem->id . ' (' . $PaymentLineItem->pathString .'): '
                . "amount received=$FormValues->{'amt'} $FormValues->{'cc'}, "
                . "expected=$GrandTotal $CurrencyID",
                {});

            Error( 'BadRedirect_AmountMisMatch', {
                'Name'       => 'amount',
                'Reason'     => 'BadRedirect_AmountMisMatch'
            });
        } else {
            $check = 1;
            $transactionID = $FormValues->{'tx'};
            $transactionStatus = $FormValues->{'st'};
        }
    }

    if ((defined($FormValues->{'mc_gross'}) || defined($FormValues->{'mc_gross_1'})) && defined($FormValues->{'mc_currency'})) {
        $FormValues->{'mc_gross'} //= $FormValues->{'mc_gross_1'};
        if (fcmp($GrandTotal, $FormValues->{'mc_gross'}) != 0 || $CurrencyID ne $FormValues->{'mc_currency'}) {
            LogPayment('PayPalExpress', 'BadRedirect_AmountMisMatch for payment line item '
                . $PaymentLineItem->id . ' (' . $PaymentLineItem->pathString .'): '
                . "amount received=$FormValues->{'mc_gross'} $FormValues->{'mc_currency'}, "
                . "expected=$GrandTotal $CurrencyID",
                {});

            Error( 'BadRedirect_AmountMisMatch', {
                'Name'       => 'amount',
                'Reason'     => 'BadRedirect_AmountMisMatch',
            });
        } else {
            $check = 1;
            $transactionID = $FormValues->{'txn_id'};
            $transactionStatus = $FormValues->{'payment_status'};
        }
    }

    return ($check, $transactionID, $transactionStatus);
}

#==============================================================================
sub _SetTransStatusOnOrder {
    my $self = shift;
    my ($Order, $TransStatus) = @_;

    my $now = GetCurrentDBHandle()->currentDateTime;
    $TransStatus = lc($TransStatus);

    if ($TransStatus eq 'pending') {
        $Order->set({'PendingOn' => $now});
    }
    elsif ($TransStatus eq 'completed') {
        $Order->set({
            'PaidOn'    => $now,
            'PendingOn' => undef
        });
        if (defined ($Order->get('CancelledOn'))) {
            $self->_LogErrorConflictPaidAndCancelled();
        }
    }
    elsif ($TransStatus eq 'denied') {
        $Order->set({'CancelledOn' => $now});
        if (defined($Order->get('PaidOn'))) {
        }
        $self->_SendPayerNotification($Order);
    }
    elsif ($TransStatus eq 'reversed') {
        # EPG-27322: remove PaidOn status if payment was reversed
        $Order->set({'PaidOn' => undef});
    }
    return;
}

#==============================================================================
sub _LogErrorConflictPaidAndCancelled {
    my $self = shift;

    eval { Error('PayPalExpressConflictPaidAndCancelled', {}); };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
    }
}

#==============================================================================
sub _SendPayerNotification {
    my $self = shift;
    my ($Order) = @_;

    my $PaymentLineItem = $Order->container->get('Payment');
    return unless $PaymentLineItem;

    my $Recipient;
    my $BillingAddress = $Order->get('BillingAddress');
    $Recipient = $BillingAddress->get('EMail') if $BillingAddress;
    $Recipient ||= $PaymentLineItem->get('PayerEMail');
    return unless $Recipient;

    my $Shop = $PaymentLineItem->getSite();
    my $MailType = GetShopMailType( $Shop, 'PayPalExpressPaymentDeclined', 1);
    return unless $MailType;

    my %Vars = (
        'Alias'           => $Order->alias,
        'PaymentLineItem' => $PaymentLineItem->tleHash,
    );

    my %MailParams = (
        'HTMLAllowed' => 1,
        'TO'          => $Recipient,
    );
    $MailParams{'FROM'} = $Shop->get('MailFrom') unless $MailType->get('FROM');

    my $Locale = $Shop->get('LocaleID');
    my %RegionalParams = (
        'Locale'   => $Locale,
        'Language' => GetInfoLanguage( GetInfoLocale($Locale)->{'LanguageID'} )->{'Code2'},
    );
    $MailType->send( $Shop, \%MailParams, \%RegionalParams, \%Vars );
}

1;
