#========================================================================================
# §package      DE_EPAGES::PayPalPPCP::API::Object::PaymentMethodPayPalPPCP
# §state        public
# §base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
#----------------------------------------------------------------------------------------
# §description  Object interface for the PayPalPPCP PaymentMethod.
#========================================================================================
package DE_EPAGES::PayPalPPCP::API::Object::PaymentMethodPayPalPPCP;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use strict;
#use Encode qw (encode);
use MIME::Base64;
use JSON;
use LWP::UserAgent;
use IO::Socket::SSL;
use Net::Curl::Easy qw(:constants);
use HTTP::Response;

use DE_EPAGES::AppStore::API::Object::ScriptTagAppClient;
use DE_EPAGES::AppStore::API::Shop qw ( GetUnityScriptTagsApp );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError);
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime GenerateShortGUID ElementOf );
use DE_EPAGES::ExternalPayment::API::Log qw (LogPayment);
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw (ImportTranslation);
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountry );
use DE_EPAGES::Shop::API::Url qw (BuildShopUrl);
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer);
use DE_EPAGES::PayPalPPCP::API::Cartridge;
use DE_EPAGES::PayPalPPCP::API::Constants qw (:ALL);
use DE_EPAGES::PayPalPPCP::API::Shop qw( GetPayPalPPCPScriptTagUrl );
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw ( ExistsShippingPaymentMap );

## no critic (ProhibitNoWarnings ProhibitMagicNumbers)
no warnings 'once';

#========================================================================================
# §function     featureName
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $FeatureName = $PaymentMethod->featureName;
#----------------------------------------------------------------------------------------
# §description  Returns the feature name 'PayPalPPCP'.
#----------------------------------------------------------------------------------------
# §return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName {
    return 'PayPalPPCP';
}

#========================================================================================
# §function     usableAtStorefront
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# §description  Tests if a payment method is visible in the storefront.
#               Overrides function <function DE_EPAGES::Payment::API::Object::PaymentMethod::usableAtStorefront>
#               to evaluate if payment method is configured
#----------------------------------------------------------------------------------------
# §input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# §return       true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    my $IsUsableAtStorefront = 1;

    $IsUsableAtStorefront = $self->_canbeActivated();

    if( not $self->SUPER::usableAtStorefront($hVars) ) {
        $IsUsableAtStorefront = 0;
    }

    return $IsUsableAtStorefront;
}

#========================================================================================
# §function     set
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $PaymentMethod->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# §description  set attributes for an object of this class
#----------------------------------------------------------------------------------------
# §input        $hValues | attribute name => value hash | ref.hash
# §input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    if ( exists $hValues->{'IsInTestMode'} ) {
        #-- Disallow changing of test mode if we have app credentials stored
        if ( $self->hasAppCredentials() ) {
            delete $hValues->{'IsInTestMode'};
        } elsif ( ! $self->canSandBox() ) {
            #-- Can't SandBox? Change IsInTestMode to 0, which is ok because we don't have app credentials yet
            $hValues->{'IsInTestMode'} = 0;
        }
    }

    return $self->SUPER::set($hValues, $LanguageID);
}

#========================================================================================
# §function     canAddToBasket
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# §description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#----------------------------------------------------------------------------------------
# §input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# §return       true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    my $CanBeAddedToBasket = $self->SUPER::canAddToBasket($hVars);

    return 0 unless $CanBeAddedToBasket;

    return $self->_canbeActivated();
}

#==================================
#   Single subroutine to check if we have everything ready and can be 'Activated'
#==================================
sub _canbeActivated {
    my $self = shift;

    return 0 unless $self->hasAppCredentials();
    return 0 unless $self->get('RegisteredWebHooksResponse');

    my $hMerchantInfoJSON = $self->get('MerchantInfoJSON');
    return 0 unless $hMerchantInfoJSON;

    my $PrimaryEmailConfirmed = $hMerchantInfoJSON->{'primary_email_confirmed'};
    my $PaymentsReceivable = $hMerchantInfoJSON->{'payments_receivable'};
    if ( ! $PrimaryEmailConfirmed || ! $PaymentsReceivable ) {
        return 0;
    }

    return 1;
}

#========================================================================================
# §function     default
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hValues = DE_EPAGES::PayPalPPCP::API::Object::PaymentMethodPayPalPPCP->default( $Parent, $hInfo );
# §example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# §description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# §input        $Parent | parent object of the new object | object
# §input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# §return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsInTestMode' => 0,
        'DoPaymentOnBasket' => 1,
    };
}

#========================================================================================
# §function     insert
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $PaymentMethod = DE_EPAGES::Payment::API::Object::PaymentMethod->insert($hInfo);
# §example      my $Class = LoadClassByAlias('PaymentMethod');
#               my $PaymentMethod = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# §description  first SUPER::insert is called, then we set the default payment method
#               name for all languages installed to the shop.
#----------------------------------------------------------------------------------------
# §input        $hInfo | attributes of new object | ref.hash
# §return       $PaymentMethod | new object | object
# §hook         OBJ_InsertPaymentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodID - payment method id    - int
#               </ul>
#========================================================================================
sub insert {
     my $class = shift;
     my ($hInfo) = @_;

     my $self = $class->SUPER::insert($hInfo);

    my $Cartridge = DE_EPAGES::PayPalPPCP::API::Cartridge->new;
    ImportTranslation($Cartridge->xmlDirectory().'/Translation.NewPaymentMethodPayPal',$self->getSite()->pathString);

    return $self;
}

#========================================================================================
# §function     delete
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $PaymentMethod->delete;
#----------------------------------------------------------------------------------------
# §description  Deletes this object
#========================================================================================
sub delete {
    my $self = shift;

    #-- If it fails, can't (or shouldn't) do anything, but we tried anyway
    $self->disconnect();

    $self->handleScriptTags( $self->getSite(), 1);

    $self->SUPER::delete;

    return;
}

#========================================================================================
# §function     canSandBox
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->canSandBox();
#----------------------------------------------------------------------------------------
# §description  Check if we have SandBox client id/secret and merchant id values because if not, we cannot use SandBox mode
#----------------------------------------------------------------------------------------
# §input        $PaymentMethodPayPalPPCP | Payment method object | object
#----------------------------------------------------------------------------------------
# §return       1/0 | Whether we can use SandBox mode or not | boolean
#========================================================================================
sub canSandBox {
    my $self = shift;

    my $PaymentType = $self->get('PaymentType');

    return $PaymentType->get('PartnerMerchantID_SandBox') && $PaymentType->get('PartnerClientID_SandBox') && $PaymentType->get('PartnerClientSecret_SandBox');
}

#========================================================================================
# §function     getBaseURL
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       getBaseURL();
#----------------------------------------------------------------------------------------
# §description  Retrieve the base url from the payment method settings
#----------------------------------------------------------------------------------------
# §input        $PaymentMethodPayPalPPCP | Payment method object | object
#----------------------------------------------------------------------------------------
# §return       $BaseURL | The base url for the api | string
#========================================================================================
sub getBaseURL {
    my $self = shift;

    my $PaymentType = $self->get('PaymentType');
    my $IsInTestMode = $self->get('IsInTestMode');

    my $BaseURL =
      $IsInTestMode
    ? $PaymentType->get('BaseURL_SandBox') : $PaymentType->get('BaseURL');

    return $BaseURL;
}

#========================================================================================
# §function     logPPCP
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->logPPCP($Name, $Stuff);
#----------------------------------------------------------------------------------------
# §description  Log an entry to communication.log under our payment type 'PayPalPPCP'
#----------------------------------------------------------------------------------------
# §input        $Name | parent object of the new object | string
# §input        $Stuff | stuff to log, ref.array/hash | ref.*
#========================================================================================
sub logPPCP {
    my $self = shift;
    my ($Name, $Stuff) = @_;

    LogPayment('PayPalPPCP', $Name, $Stuff);
}

#========================================================================================
# §function     getBNCode
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->getBNCode();
#----------------------------------------------------------------------------------------
# §description  Get the BN Code for our partner credentials
#----------------------------------------------------------------------------------------
# §return       $BNCode | the stored bn code | string
#========================================================================================
sub getBNCode {
    my $self = shift;
    my ($PaymentLine) = @_;

    my $PaymentType = $self->get('PaymentType');
    my $IsInTestMode = $self->get('IsInTestMode');

    my $AttrToGet = $IsInTestMode ? 'PartnerBNCodeECM_SandBox' : 'PartnerBNCodeECM';

    #-- If we have our express checkout flag set, retrieve the ECS BN Code instead
    if ( defined $PaymentLine && $PaymentLine->get('IsExpressCheckout') ) {
        $AttrToGet = $IsInTestMode ? 'PartnerBNCodeECS_SandBox' : 'PartnerBNCodeECS';
    }

    my $BNCode = $PaymentType->get($AttrToGet);

    return $BNCode;
}

#========================================================================================
# §function     hasAppCredentials
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->hasAppCredentials();
#----------------------------------------------------------------------------------------
# §description  Check if our payment method has a client id/secret stored
#----------------------------------------------------------------------------------------
# §return       1/0 | whether our payment method has a client id/secret | boolean
#========================================================================================
sub hasAppCredentials {
    my $self = shift;

    return $self->get('ClientID') && $self->get('ClientSecret');
}

#========================================================================================
# §function     countrySupportsPayLater
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->countrySupportsPayLater();
#----------------------------------------------------------------------------------------
# §description  Check if our payment method supports PayLater functionality
#----------------------------------------------------------------------------------------
# §return       1/0 | whether our payment method supports PayLater functionality | boolean
#========================================================================================
sub countrySupportsPayLater {
    my $self = shift;

    my $ShopAddress = $self->get('Site')->get('Address');
    return 0 unless defined $ShopAddress;

    my $CountryID = $ShopAddress->get('CountryID');
    return 0 unless defined $CountryID;

    if ( ElementOf( $CountryID, @SUPPORTED_PAYLATER_COUNTRIES ) ) {
        return 1;
    }

    return 0;
}

#========================================================================================
# §function     canUsePUI
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->canUsePUI();
#----------------------------------------------------------------------------------------
# §description  Check if our payment method supports the payment upon invoice payment method
#               https://developer.paypal.com/limited-release/orders-apms/pay-upon-invoice/
#               https://developer.paypal.com/limited-release/orders-apms/pay-upon-invoice/integrate-pui-partners/
#               https://developer.paypal.com/limited-release/orders-apms/pay-upon-invoice/integrate-pui-merchants/
#----------------------------------------------------------------------------------------
# §return       1/0 | whether our payment method has a client id/secret | boolean
#========================================================================================
sub canUsePUI {
    my $self = shift;
    my ($Container) = @_;

    my $Shop = $self->getSite();
    return 0 if $Shop->get('FeatureMaxValue')->{'PayPalPPCP_S'};

    my $hInfoJSON = $self->get('MerchantInfoJSON');

    return 0 if ! $hInfoJSON || ! $self->hasAppCredentials();

    return 0 if ! $self->get('EnablePUI');

    my $CanPUI = 0;
    foreach my $hCapability ( @{$hInfoJSON->{'capabilities'}} ) {
        if ( $hCapability->{'name'} eq 'PAY_UPON_INVOICE' && $hCapability->{'status'} eq 'ACTIVE' ) {
            $CanPUI = 1;
            last;
        }
    }

    #-- Verify that the billing address is Germany
    if ( $Container ) {
        my $Basket = $Container->parent;
        if ( $Basket->instanceOf('Basket') ) {
            my $BillingAddress = $Basket->get('BillingAddress');
            my $ShippingAddress = $Basket->get('ShippingAddress') ? $Basket->get('ShippingAddress') : $Basket->get('BillingAddress');

            if ( $BillingAddress->get('CountryID') != 276 ) {
                #|| ! $BillingAddress->get('Birthday')
                #|| ! $BillingAddress->get('Phone') ) {
                $CanPUI = 0;
            }

            #-- Do we care where they want it shipped? Does PayPal for PUI? They do ...
            if ( $ShippingAddress->get('CountryID') != 276 ) {
            #    || ! $ShippingAddress->get('Birthday')
            #    || ! $ShippingAddress->get('Phone') ) {
                $CanPUI = 0;
            }

            $CanPUI = 0 if $Container->get('CurrencyID') ne 'EUR';
            $CanPUI = 0 if $Container->get('GrandTotal') < 5 || $Container->get('GrandTotal') > 2500;
        }
    }

    return $CanPUI;

}

#========================================================================================
# §function     handleScriptTags
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->handleScriptTags();
#----------------------------------------------------------------------------------------
# §description  Add/Remove our script tag
#----------------------------------------------------------------------------------------
# §input       $Shop | A shop object | object
# §input       $DeleteScriptTag | Whether to delete the script tag, otherwise it will be added | boolean
#========================================================================================
sub handleScriptTags {
    my $self = shift;
    my ($Shop, $DeleteScriptTag) = @_;

    #-- Most of the time, these calls work fine. With UNITY-8097, we found that during the deletion of a shop object,
    #-- the hook OBJ_DeleteShop is triggered BEFORE child objects are deleted. The AppStore cartridge has a subroutine registered on this hook
    #-- which deletes the shop's private app, used for script tags and so forth. So by the time this guy gets called, we end up with a 404 error
    #-- and everything grinds to a halt. Since these calls don't return a value anyway, just wrap it inside an eval.
    eval {
        # get app client to communicate with the epagesJ REST API
        my $UnityScriptTagsAppClient = GetUnityScriptTagsApp($Shop);
        # get PayPalPPCP script tags
        my $PayPalPPCPScriptTagUrl = GetPayPalPPCPScriptTagUrl($Shop);

        if ($DeleteScriptTag) {
            $UnityScriptTagsAppClient->deleteScriptTag($PayPalPPCPScriptTagUrl);
        } else {
            $UnityScriptTagsAppClient->addScriptTag($PayPalPPCPScriptTagUrl);
        }
    };
    if ($@){}
}

#========================================================================================
# §function     getFraudNetFParameter
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->getFraudNetFParameter($PaymentLine);
#----------------------------------------------------------------------------------------
# §description  Retrieve the 'f' parameter for their FraudNet js integration
#               https://developer.paypal.com/limited-release/fraudnet/integrate/
#----------------------------------------------------------------------------------------
# §input       $PaymentLine | A payment line object of ppcp | object
#========================================================================================
sub getFraudNetFParameter {
    my $self = shift;
    my ($PaymentLine) = @_;

    #-- FraudNet f parameter
    my $FParameter = $PaymentLine->get('GUID');
    $FParameter =~ s/-//g;

    return $FParameter;
}

#========================================================================================
# §function     createPUIOrder
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->createPUIOrder($PaymentLine);
#----------------------------------------------------------------------------------------
# §description  create an order with payment source 'payment_upon_invoice'
#----------------------------------------------------------------------------------------
# §input       $PaymentLine | A payment line object of ppcp | object
#========================================================================================
sub createPUIOrder {
    my $self = shift;
    my ($PaymentLine) = @_;

    return (undef,undef) unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $ORDERS_ENDPOINT;

    my $AccessToken = $self->getMerchantAccessToken();

    my $Container = $PaymentLine->container;
    my $Basket = $Container->parent();
    my $CurrencyID = $Container->get('CurrencyID');
    #my $Reference = $PaymentLine->get('GUID');
    my $Total = $Container->get('GrandTotal');
    my $TotalBeforeTax = $Container->get('TotalBeforeTax');
    my $TotalTax = $Container->get('TotalTax');

    #-- FraudNet f parameter
    my $FParameter = $self->getFraudNetFParameter($PaymentLine);

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
        'Prefer' => 'return=representation',
        'PayPal-Request-Id' => GenerateShortGUID(),
        'PAYPAL-CLIENT-METADATA-ID' => $FParameter,
    };

    my $LineItemTax = $Container->childrenByClass('LineItemTax')->[0];
    my $BaseTaxRate = defined $LineItemTax ? $LineItemTax->get('TaxRate') : 0;

    my $ShippingAddress = $Basket->get('ShippingAddress') ? $Basket->get('ShippingAddress') : $Basket->get('BillingAddress');
    my $BillingAddress = $Basket->get('BillingAddress');

    my $Phone = $BillingAddress->get('Phone');
    $Phone =~ s/[^0-9]//g;
    $Phone = substr($Phone, 0, 14);

    my $hPayLoad = {
        'intent' => 'CAPTURE',
        'processing_instruction' => 'ORDER_COMPLETE_ON_PAYMENT_APPROVAL',
        'purchase_units' => [{
            'amount' => {
                'currency_code' => $CurrencyID,
                'value' => $Total,
                'breakdown' => {
                    'item_total' => {
                        'currency_code' => $CurrencyID,
                        'value' => $TotalBeforeTax,
                    },
                    'tax_total' => {
                        'currency_code' => $CurrencyID,
                        'value' => $TotalTax,
                    },
                },
            },
            'shipping' => {
                'name' => {
                    'full_name' => $ShippingAddress->get('FullName'),
                },
                'address' => {
                    'address_line_1' => $ShippingAddress->get('Street'),
                    'admin_area_2' => $ShippingAddress->get('City') . ' ' . $ShippingAddress->get('State'),
                    'postal_code' => $ShippingAddress->get('Zipcode'),
                    'country_code' => GetInfoCountry($ShippingAddress->get('CountryID'))->{'Code2'},
                },
            },
            'items' => [{
                'name' => 'Items',
                #'description' => '',
                'category' => 'PHYSICAL_GOODS',
                'unit_amount' => {
                    'currency_code' => $CurrencyID,
                    'value' => $TotalBeforeTax,
                },
                'tax' => {
                    'currency_code' => $CurrencyID,
                    'value' => $TotalTax,
                },
                'tax_rate' => $BaseTaxRate,
                'quantity' => 1,
            }],
            #'invoice_id' => $PaymentLine->id,
            # Webhooks refund does not have any identifying keys to tie back to the paypal order,
            # we use this instead since it is returned in the webhook:
            'custom_id' => $PaymentLine->id,
        }],
        'payment_source' => {
            'pay_upon_invoice' => {
                'name' => {
                    'given_name' => $BillingAddress->get('FirstName'),
                    'surname' => $BillingAddress->get('LastName'),
                },
                'email' => $BillingAddress->get('EMail'),
                'birth_date' => $BillingAddress->get('Birthday')->ymd('-'),
                'phone' => {
                    'national_number' => $Phone,
                    'country_code' => '49' #-- Hardcode for Germany for now
                },
                'billing_address' => {
                    'address_line_1' => $BillingAddress->get('Street'),
                    'admin_area_2' => $BillingAddress->get('City') . ' ' . $BillingAddress->get('State'),
                    'postal_code' => $BillingAddress->get('Zipcode'),
                    'country_code' => GetInfoCountry($BillingAddress->get('CountryID'))->{'Code2'},
                },
                'experience_context' => {
                    'locale' => 'en-DE',
                    #"brand_name": "EXAMPLE INC",
                    'customer_service_instructions' => [
                        '-' #-- Apparently this is now mandatory 13/4/22, it didn't used to be before that!!!
                    ],
                },
            },
        },
    };

    my $Response = $self->doRequest('POST', $RequestURL, encode_json($hPayLoad), $hHeaders, { 'LogName' => 'CreatePUIOrder', 'PaymentLine' => $PaymentLine });

    return (undef,undef) unless $Response; #&& $Response->is_success(); #-- We care about 422 responses here
    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse = undef;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    return (undef,undef) if $@;

    return ($hJsonResponse, $hPayLoad);

}

#========================================================================================
# §function     patchPPCPOrder
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->patchPPCPOrder($PaymentLine);
#----------------------------------------------------------------------------------------
# §description  Update an order at PayPal
#----------------------------------------------------------------------------------------
# §input        $PaymentLine | Payment line item | object
#----------------------------------------------------------------------------------------
# §return       $hJsonResponse | The response information from PayPal, or undef on failure | ref.hash
#========================================================================================
sub patchPPCPOrder {
    my $self = shift;
    my ($PaymentLine) = @_;

    my $hOrderInfoJSON = $PaymentLine->get('OrderInfoJSON');
    return 0 unless $hOrderInfoJSON;

    return 0 unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $ORDERS_ENDPOINT . '/' . $hOrderInfoJSON->{'id'};

    my $AccessToken = $self->getMerchantAccessToken();

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
        'Prefer' => 'return=representation',
    };

    my $hPayload = $self->getOrderPayload($PaymentLine);
    my $hPurchaseUnit = $hPayload->{'purchase_units'}->[0];

    #-- We do not seem to be able to update any other fields, or at least, we have not been able to make it work for anything else, LOL >.<
    my $aPatch = [{
        'op' => 'replace',
        'path' => q(/purchase_units/@reference_id==') . $hPurchaseUnit->{'reference_id'} . q('),
        'value' => $hPurchaseUnit,
    }];

    my $PayloadString = encode_json($aPatch);
    #-- Seems our encoded JSON turns our ' marks into \', we need them to just be '
    $PayloadString =~ s/\\\'/'/g;

    my $Response = $self->doRequest('PATCH', $RequestURL, $PayloadString, $hHeaders,
                                    { 'LogName' => 'PatchOrder', 'PaymentLine' => $PaymentLine });
    return 0 unless $Response && $Response->is_success();

    return 1;
}

#========================================================================================
# §function     capturePPCPOrder
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->capturePPCPOrder($PaymentLine);
#----------------------------------------------------------------------------------------
# §description  Capture the funds for an order at PayPal
#----------------------------------------------------------------------------------------
# §input        $PaymentLine | Payment line item | object
#----------------------------------------------------------------------------------------
# §return       $hJsonResponse | The response information from PayPal, or undef on failure | ref.hash
#========================================================================================
sub capturePPCPOrder {
    my $self = shift;
    my ($PaymentLine) = @_;

    my $hOrderInfoJSON = $PaymentLine->get('OrderInfoJSON');
    return unless $hOrderInfoJSON;

    return undef unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $ORDERS_ENDPOINT . '/' . $hOrderInfoJSON->{'id'} . '/capture';

    my $AccessToken = $self->getMerchantAccessToken();

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
        'Prefer' => 'return=representation',
        #-- For mock testing: https://developer.paypal.com/api/rest/sandbox/negative-testing/request-headers/,
        # https://developer.paypal.com/api/rest/reference/orders/v2/errors/#link-captureorder
        #'PayPal-Mock-Response' => '{"mock_application_codes": "INSTRUMENT_DECLINED"}',
    };

    my $Response = $self->doRequest('POST', $RequestURL, undef, $hHeaders, { 'LogName' => 'CaptureOrder', 'PaymentLine' => $PaymentLine });
    return undef unless $Response && $Response->is_success();
    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse = undef;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    return undef if $@;

    return $hJsonResponse;
}

#========================================================================================
# §function     getPPCPOrder
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getPPCPOrder($PaymentLine);
#----------------------------------------------------------------------------------------
# §description  Retrieve a PayPal order
#----------------------------------------------------------------------------------------
# §input        $PaymentLine | Payment line item | object
#----------------------------------------------------------------------------------------
# §return       $hJsonResponse | The response information from PayPal, or undef on failure | ref.hash
#========================================================================================
sub getPPCPOrder {
    my $self = shift;
    my ($PaymentLine, $WantErrors) = @_;

    my $hOrderInfoJSON = $PaymentLine->get('OrderInfoJSON');
    return unless $hOrderInfoJSON;

    return undef unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $ORDERS_ENDPOINT . '/' . $hOrderInfoJSON->{'id'};

    my $AccessToken = $self->getMerchantAccessToken();

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
        'Prefer' => 'return=representation',
        #'PayPal-Mock-Response' => '{"mock_application_codes": "INVALID_RESOURCE_ID"}',
    };

    my $Response = $self->doRequest('GET', $RequestURL, undef, $hHeaders, { 'LogName' => 'GetOrder', 'PaymentLine' => $PaymentLine });
    #-- If you don't pass $WantErrors, only return a successful response
    return undef unless $Response && ($Response->is_success() || $WantErrors);
    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse = undef;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    return undef if $@;

    return $hJsonResponse;
}

#========================================================================================
# §function     createPPCPOrder
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->createPPCPOrder($PaymentLine);
#----------------------------------------------------------------------------------------
# §description  Create a PayPal order
#----------------------------------------------------------------------------------------
# §input        $PaymentLine | Payment line item | object
# §input        $IncludeShippingOptions | Whether to include a list of possible shipping options in the request | boolean
#----------------------------------------------------------------------------------------
# §return       $hJsonResponse | The response information from PayPal, or undef on failure | ref.hash
#========================================================================================
sub createPPCPOrder {
    my $self = shift;
    my ($PaymentLine, $IncludeShippingOptions) = @_;

    return undef unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $ORDERS_ENDPOINT;

    my $AccessToken = $self->getMerchantAccessToken();

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
        'Prefer' => 'return=representation',
        'PayPal-Request-Id' => GenerateShortGUID(),
    };

    my $Payload = $self->getOrderPayload($PaymentLine, $IncludeShippingOptions);

    my $Response = $self->doRequest('POST', $RequestURL, encode_json($Payload), $hHeaders, { 'LogName' => 'CreateOrder', 'PaymentLine' => $PaymentLine });

    return undef unless $Response && $Response->is_success();
    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse = undef;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    return undef if $@;

    return $hJsonResponse;
}

#========================================================================================
# §function     getOrderPayload
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getOrderPayload($PaymentLine);
#----------------------------------------------------------------------------------------
# §description  Get the payload for creating/patching an order at PayPal
#----------------------------------------------------------------------------------------
# §input        $PaymentLine | Payment line item | object
# §input        $IncludeShippingOptions | Whether to include a list of possible shipping options in the request | boolean
#----------------------------------------------------------------------------------------
# §return       $hJsonResponse | The response information from PayPal, or undef on failure | ref.hash
#========================================================================================
sub getOrderPayload {
    my $self = shift;
    my ($PaymentLine, $IncludeShippingOptions) = @_;

    my $Container = $PaymentLine->container;
    my $ContainerShippingMethod = $Container->get('Shipping');
    my $CurrencyID = $Container->get('CurrencyID');
    my $LanguageID = $Container->get('LanguageID');
    my $Reference = $PaymentLine->get('GUID');
    my $Total = $Container->get('GrandTotal');

    my $BasketOrder = $Container->parent();

    my $hShippingInfo;
    my $ShippingAddress = $BasketOrder->get('ShippingAddress') ? $BasketOrder->get('ShippingAddress') : $BasketOrder->get('BillingAddress');

    my $Street = $ShippingAddress ? $ShippingAddress->get('Street') : undef;
    my $CountryID = $ShippingAddress ? $ShippingAddress->get('CountryID') : undef;
    my $CanSendShippingToPayPal = $Street && $CountryID ? 1 : 0;

    $hShippingInfo = $CanSendShippingToPayPal ? {
        'name' => {
            'full_name' => $ShippingAddress->get('FirstName') . ' ' . $ShippingAddress->get('LastName'),
        },
        'type' => 'SHIPPING',
        'address' => {
            'address_line_1' => $ShippingAddress->get('Street'),
            'address_line_2' => $ShippingAddress->get('Street2'),
            'admin_area_1' => $ShippingAddress->get('State'),
            'admin_area_2' => $ShippingAddress->get('City'),
            'postal_code' => $ShippingAddress->get('Zipcode'),
            'country_code' => GetInfoCountry($ShippingAddress->get('CountryID'))->{'Code2'},
        }
    } : {};

    my $Shop = $self->getSite();

    #-- Send a list of available shipping methods to PayPal to display to the customer
    #-- Only works if we are not sending our own address to PayPal
    if ( $IncludeShippingOptions && !$CanSendShippingToPayPal && $ContainerShippingMethod ) {
      my $ShippingMethods = $Container->get('ShippingMethods');
      my $aShippingOptions = [];
      foreach my $ShippingMethod (@$ShippingMethods) {
        next unless ElementOf($ShippingMethod->class->alias, @SHIPMETHODS_CAN_SEND_TO_PAYPAL);

        #-- This means, this shipping method is compatible with our PPCP payment method
        if ( ExistsShippingPaymentMap($ShippingMethod->id, $self->id) ) {
          my $NameOrAlias = $ShippingMethod->get('NameOrAlias', $LanguageID);
          push @$aShippingOptions, {
            'id' => $ShippingMethod->id,
            'type' => 'SHIPPING',
            'label' => substr($NameOrAlias,0,127),
            'selected' => $ShippingMethod->id == $ContainerShippingMethod->get('ShippingMethod')->id ? 1 : 0,
            'amount' => {
              'currency_code' => $CurrencyID,
              'value' => $ShippingMethod->calculateShippingPrice($Container),
            },
          };
        }
      }
      if ( scalar @$aShippingOptions > 1 ) {
        $hShippingInfo->{'options'} = $aShippingOptions;
        #-- If we pass shipping options, then pass the subtotal, since PayPal will add the shipping price on top of the total that we pass
        $Total = $Container->get('LineItemsSubTotal');
      }
    }

    my $aItems = [];
    my $OrderNum = $BasketOrder->instanceOf('CustomerOrder') ? ' #'.$BasketOrder->alias : '';
    my $Description = $Shop->get('NameOrAlias') . $OrderNum;

    #-- With a payment source, the status skips 'CREATED', and goes straight to 'PAYER_ACTION_REQUIRED'
    my $Payload = {
        'intent' => 'CAPTURE',
        'purchase_units' => [{
            'reference_id' => $Reference,
            'description' => $Description,
            'custom_id' => $PaymentLine->id,
            #'invoice_id' => $PaymentLine->id,
            'amount' => {
                'currency_code' => $CurrencyID,
                'value' => $Total,
            },
            'items' => $aItems,
            'shipping' => $hShippingInfo,
        }],
    };

    #-- Only pass a funding source for paypal methods, that is, ignore APMs
    #-- If funding source is not set, the html code will default to paypal
    if ( !$PaymentLine->get('FundingSource') || ElementOf($PaymentLine->get('FundingSource'), ($FS_PAYPAL, $FS_PAYLATER, $FS_CARD)) ) {
      $Payload->{'payment_source'} = {
        'paypal' => {
          'experience_context' => {
            'shipping_preference' => $CanSendShippingToPayPal ? 'SET_PROVIDED_ADDRESS' : 'GET_FROM_FILE',
          },
        },
      }
    }

    return $Payload;
}

#========================================================================================
# §function     disconnect
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->disconnect();
#----------------------------------------------------------------------------------------
# §description  'Disconnect' our paypal payment method, remove all registered webhooks and clear attributes
#----------------------------------------------------------------------------------------
# §return       1/0 | whether the disconnection was successful | boolean
#========================================================================================
sub disconnect {
    my $self = shift;

    #-- Can't do anything without the client id/secret
    return unless $self->hasAppCredentials();

    #-- Unregister webhooks
    my $Success = $self->unregisterAppWebHooks();

    #-- Disallow reset if the webhook call was unsuccessful?
    return 0 unless $Success;

    #-- Clear all attributes
    $self->set({
        'ClientID' => undef,
        'ClientSecret' => undef,
        'AccessToken' => undef,
        'AccessTokenExpiry' => undef,
        'RegisteredWebHooksResponse' => undef,
        'MerchantInfoJSON' => undef,
    });

    return $Success;
}

#========================================================================================
# §function     setMerchantInfo
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->setMerchantInfo();
#----------------------------------------------------------------------------------------
# §description  Try and retrieve the merchant information attached to the PayPal REST API associated with this payment method
#   https://developer.paypal.com/docs/platforms/seller-onboarding/build-onboarding/, step 3, provides a mechanism to retrieve
#   the merchant id but it's a bit flaky as the merchant has to press the button and it does not work if they just close the
#   window. Also, there is no way to verify that the parameters passed back are legitimate
#========================================================================================
sub setMerchantInfo {
    my $self = shift;

    return unless $self->hasAppCredentials();
    #return if $self->get('MerchantInfoJSON');

    my $hMerchantInfoJSON = $self->get('MerchantInfoJSON');
    my $MerchantID;
    my $BaseURL = $self->getBaseURL();
    my $RequestURL;
    my $AccessToken;
    my $hHeaders;
    my $Response;
    my $DecodedContent;
    my $hJsonResponse;

    #-- If we already have existing information, use this for the merchant ID
    if ( $hMerchantInfoJSON && %$hMerchantInfoJSON && defined $hMerchantInfoJSON->{'merchant_id'} ) {
        $MerchantID = $hMerchantInfoJSON->{'merchant_id'};
    } else {

        #-------------------------------------
        #-- Make a dummy order to retrieve the merchant id from the response payload
        #--
        $RequestURL = $BaseURL . $ORDERS_ENDPOINT;

        $AccessToken = $self->getMerchantAccessToken();

        $hHeaders = {
            'Content-Type' => 'application/json',
            'Authorization' => 'Bearer ' . $AccessToken,
            'Prefer' => 'return=representation',
        };

        my $CurrencyID = $self->get('Shop')->get('CurrencyID');
        $CurrencyID = 'USD' unless $CurrencyID;

        my $Payload =<<"END";
    {
      "intent": "CAPTURE",
      "purchase_units": [
        {
          "amount": {
            "currency_code": "$CurrencyID",
            "value": "1.00"
          }
        }
      ]
    }
END

        $Response = $self->doRequest('POST', $RequestURL, $Payload, $hHeaders, { 'LogName' => 'SetMerchantInfoDummyOrder' });

        return undef unless $Response && $Response->is_success();
        $DecodedContent = $Response->decoded_content;

        $hJsonResponse = undef;
        eval { $hJsonResponse = decode_json($DecodedContent); };
        return if $@;

        if ( $hJsonResponse ) {
            $MerchantID = $hJsonResponse->{'purchase_units'}->[0]->{'payee'}->{'merchant_id'};
        }
    }
    return unless $MerchantID;



    #---------------------------------------
    #-- Now that we have the merchant id, we can ask for the merchant info, YAH!
    $RequestURL = $BaseURL . $GET_MERCHANT_INFO_ENDPOINT . $MerchantID;
    $AccessToken = $self->getPartnerAccessToken();

    $hHeaders = {
        'Authorization' => 'Bearer ' . $AccessToken,
    };

    my $IsInTestMode = $self->get('IsInTestMode');
    my $PaymentType = $self->get('PaymentType');
    my $PartnerMerchantID = $IsInTestMode ? $PaymentType->get('PartnerMerchantID_SandBox') : $PaymentType->get('PartnerMerchantID');

    $RequestURL =~ s/{partner_merchant_id}/$PartnerMerchantID/;

    $Response = $self->doRequest('GET', $RequestURL, undef, $hHeaders, { 'LogName' => 'SetMerchantInfo' });

    return unless $Response && $Response->is_success();

    $DecodedContent = $Response->decoded_content;

    $hJsonResponse = undef;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    return if $@;

    #-- Add our own key to it, to keep track of when it was last updated
    $hJsonResponse->{'epages_last_updated'} = GetCurrentDateTime()->epoch();
    $self->set({
       'MerchantInfoJSON' => $hJsonResponse,
    });
}

#========================================================================================
# §function     registerOnBoardWebHooks
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->registerOnBoardWebHooks();
#----------------------------------------------------------------------------------------
# §description  Register the required webhooks as part of the onboarding process
#
#----------------------------------------------------------------------------------------
# §return       1/0 | whether the registration was successful | boolean
#========================================================================================
sub registerOnBoardWebHooks {
    my $self = shift;

    return 0 unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $WEBHOOKS_ENDPOINT;

    my $AccessToken = $self->getMerchantAccessToken();

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
    };

    my $WebhookUrl = BuildShopUrl($self, {'ChangeAction' => 'PayPalWebhookCallback'}, {'Type' => 'admin', 'UseSSL' => 1, 'UseObjectPath' =>0});

    # see Constants.pm for a valid list of event types
    ##no critic qw(RequireCorrectImports)
    my $WebHookEventTypes = WEBHOOK_EVENT_TYPES;
    my $Payload = <<"END";
{
  "url": "$WebhookUrl",
  $WebHookEventTypes
}
END

    my $Response = $self->doRequest('POST', $RequestURL, $Payload, $hHeaders, { 'LogName' => 'RegisterOnBoardWebHooks' });
    return 0 unless $Response && $Response->is_success();

    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse = undef;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    return 0 if $@;

    $self->set({
        'RegisteredWebHooksResponse' => $hJsonResponse,
    });

    return 1;
}

#========================================================================================
# §function     unregisterAppWebHooks
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->unregisterAppWebHooks();
#----------------------------------------------------------------------------------------
# §description  Un-Register all webhooks as part of the 'disconnect' process
#
#               https://developer.paypal.com/docs/api/webhooks/v1/
#----------------------------------------------------------------------------------------
# §return       1/0 | whether the de-registration was successful | boolean
#========================================================================================
sub unregisterAppWebHooks {
    my $self = shift;

    return 0 unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $WEBHOOKS_ENDPOINT;

    my $AccessToken = $self->getMerchantAccessToken();

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
    };

    #-- Get all webhooks
    my $Response = $self->doRequest('GET', $RequestURL, undef, $hHeaders, { 'LogName' => 'GetAllWebHooks' });

    return 0 unless $Response->is_success();

    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse = undef;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    return 0 if $@;

    #-- Delete them all (or any that look like ePages)
    foreach my $hWebHook ( @{$hJsonResponse->{'webhooks'}} ) {
        my $Id = $hWebHook->{'id'};
        $RequestURL = $RequestURL . '/' . $Id;
        $Response = $self->doRequest('DELETE', $RequestURL, undef, $hHeaders, { 'LogName' => 'DeleteWebHook' });

        #-- Do we care about the response?
        return 0 unless $Response && $Response->is_success();
    }

    $self->set({
        'RegisteredWebHooksResponse' => undef,
    });

    return 1;
}

#========================================================================================
# §function     verifyWebHookCallBack
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->verifyWebHookCallBack();
#----------------------------------------------------------------------------------------
# §description  verifies a webhook callback from PayPal by sending data to PayPal and read the response
#----------------------------------------------------------------------------------------
# §input        $WebHookContent | webhook content sent by PayPal to verify  | ref.hash
#----------------------------------------------------------------------------------------
# §return       $TrueOrFalse | The webhook call from PayPal has the correct signature and was not modified | boolean
#========================================================================================
sub verifyWebHookCallBack {
    my $self = shift;
    my ($WebHookContent) = @_;

    return undef unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $WEBHOOK_VERIFY_ENDPOINT;

    my $AccessToken = $self->getMerchantAccessToken();

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
        'Prefer' => 'return=representation',
    };

    # important do not use the perl to json functions here because we need to reconstruct
    # the payload in the same way we have received.
    # The verification always fails if we don't go this way -- really ugly because of the deserialization on the paypal side

    # create payload for verification
    my $Payload = '{';
    foreach my $element (qw(auth_algo cert_url transmission_id transmission_sig transmission_time )){
        $Payload .= "\"$element\":\"". $WebHookContent->{'Headers'}->header('paypal-'.$element). '"'. ",\n";
    }
    # get webhookid from saved response upon registering webhook
    my $hWebHookRegisterResponse = $self->get('RegisteredWebHooksResponse');
    $Payload .= "\"webhook_id\":\"$hWebHookRegisterResponse->{'id'}\"". ",\n";
    $Payload .= '"webhook_event":'. $WebHookContent->{'EventContent'}. "\n";
    $Payload .= '}';

    my $Response = $self->doRequest('POST', $RequestURL, $Payload, $hHeaders, { 'LogName' => 'VerifyWebHook' });

    return undef unless $Response && $Response->is_success();
    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse = undef;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    return 0 if $@;

    return 1 if $hJsonResponse->{'verification_status'} eq 'SUCCESS';

    return 0;
}

#========================================================================================
# §function     getMerchantAccessToken
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->getMerchantAccessToken();
#----------------------------------------------------------------------------------------
# §description  Retrieve an access token for the merchant's app
#----------------------------------------------------------------------------------------
# §return       $AccessToken | The retrieved/generated access token, or undef in failure | string
#========================================================================================
sub getMerchantAccessToken {
    my $self = shift;

    my $Now = GetCurrentDateTime();

    my $CurrentAccessToken = $self->get('AccessToken');
    my $CurrentAccessTokenExpiry = $self->get('AccessTokenExpiry');

    #-- If the expiry date is greater than Now
    #-- $cmp = DateTime->compare( $dt1, $dt2 ); it returns -1 if $dt1 < $dt2, 0 if $dt1 == $dt2, 1 if $dt1 > $dt2
    if ( $CurrentAccessToken && $CurrentAccessTokenExpiry && $CurrentAccessToken
         && DateTime->compare( $CurrentAccessTokenExpiry, $Now ) > 0 ) {

        #-- This always returns a positive value ( >= 0 ), so we needed to ensure that the expiry date is greater than now
        # because a reverse comparison will not give a negative value
        #-- I.e. now > expiry date
        my $DeltaMSDuration = $CurrentAccessTokenExpiry->delta_ms($Now);

        #-- More than X minutes until expiry? Do not bother refreshing until next time
        if ( ($DeltaMSDuration->hours*60 + $DeltaMSDuration->minutes) >= $TOKEN_EXPIRE_THRESHOLD_MINS ) {
            return $CurrentAccessToken;
        }
    }

    #-- Why isn't this at the top? We should have app credentials before doing anything?
    #-- Maybe the access token is still there but the client credentials aren't? But that should never happen? lol ...
    return undef unless $self->hasAppCredentials();

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $TOKEN_ENDPOINT . '?grant_type=client_credentials';

    my $ClientID = $self->get('ClientID');
    my $ClientSecret = $self->get('ClientSecret');

    my $hHeaders = {
        'Content-Type' => 'application/x-www-form-urlencoded',
        'Authorization' => 'Basic ' . encode_base64( "$ClientID:$ClientSecret", '' ),
    };

    my $Response = $self->doRequest('POST', $RequestURL, undef, $hHeaders, { 'LogName' => 'GetMerchantAccessToken' });

    return undef unless $Response && $Response->is_success();

    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    $hJsonResponse = undef if $@;

    if ( defined $hJsonResponse && defined $hJsonResponse->{'access_token'} && $hJsonResponse->{'expires_in'} ) {
        my $AccessToken = $hJsonResponse->{'access_token'};
        my $ExpiresIn = $hJsonResponse->{'expires_in'};

        my $ExpiryDate = $Now->add( seconds => $ExpiresIn );
        $self->set({
            'AccessToken' => $AccessToken,
            'AccessTokenExpiry' => $ExpiryDate,
        });
        return $AccessToken;
    }

    return undef;
}

#========================================================================================
# §function     getMerchantApp
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->getMerchantApp($AccessToken);
#----------------------------------------------------------------------------------------
# §description  Retrieve client id/secret from a temporary token as part of the onboarding process
#               https://developer.paypal.com/docs/platforms/seller-onboarding/build-onboarding/, step 5
#----------------------------------------------------------------------------------------
# §input        $AccessToken | A temporary access token with which we can use to retrieve a client id/secret | string
#----------------------------------------------------------------------------------------
# §return       $hJsonResponse | The json response, or undef for failure | ref.hash
#========================================================================================
sub getMerchantApp {
    my $self = shift;
    my ($AccessToken) = @_;

    TestString('AccessToken', $AccessToken);

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $AccessToken,
        'Accept' => 'application/json',
    };

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $GET_MERCHANT_APP_ENDPOINT;

    my $PaymentType = $self->get('PaymentType');
    my $IsInTestMode = $self->get('IsInTestMode');

    my $PartnerMerchantID = $IsInTestMode ? $PaymentType->get('PartnerMerchantID_SandBox') : $PaymentType->get('PartnerMerchantID');

    $RequestURL =~ s/{partner_merchant_id}/$PartnerMerchantID/;

    my $Response = $self->doRequest('GET', $RequestURL, undef, $hHeaders, { 'LogName' => 'GetMerchantApp' });

    return undef unless $Response && $Response->is_success();

    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    $hJsonResponse = undef if $@;

    return $hJsonResponse;
}

#========================================================================================
# §function     getMerchantAccessTokenWithAuthCodeSharedID
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->getMerchantAccessTokenWithAuthCodeSharedID($AuthCode, $SharedID);
#----------------------------------------------------------------------------------------
# §description  Retrieve a temporary access token from the returned auth code/shared id as part of the onboarding process
#               https://developer.paypal.com/docs/platforms/seller-onboarding/build-onboarding/, step 4
#----------------------------------------------------------------------------------------
# §input        $AuthCode | AuthCode from the callback made by PayPal | string
# §input        $SharedID | SharedID parameter from the callback made by PayPal | string
#----------------------------------------------------------------------------------------
# §return       $AccessToken | The temporary access token, or undef if the request failed | string
#========================================================================================
sub getMerchantAccessTokenWithAuthCodeSharedID {
    my $self = shift;
    my ($AuthCode, $SharedID) = @_;

    TestString('AuthCode', $AuthCode);
    TestString('SharedID', $SharedID);

    my $hHeaders = {
        'Content-Type' => 'application/x-www-form-urlencoded',
        'Authorization' => 'Basic ' . encode_base64( "$SharedID:", '' ),
        'Accept' => 'application/json',
    };

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $TOKEN_ENDPOINT;

    my $Shop = $self->getSite();
    my $Nonce = $self->get('GUID') . $Shop->get('GUID');
    my $Body = "grant_type=authorization_code&code=$AuthCode&code_verifier=$Nonce";

    my $Response = $self->doRequest('POST', $RequestURL, $Body, $hHeaders, { 'LogName' => 'GetMerchantAccessToken' });
    return undef unless $Response && $Response->is_success();
    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    $hJsonResponse = undef if $@;

    my $AccessToken = defined $hJsonResponse ? $hJsonResponse->{'access_token'} : undef;

    return $AccessToken;
}

#========================================================================================
# §function     getPartnerReferralLink
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->getPartnerReferralLink();
#----------------------------------------------------------------------------------------
# §description  Retrieve a signup link as part of the onboarding process
#               https://developer.paypal.com/docs/platforms/seller-onboarding/build-onboarding/, step 1
#----------------------------------------------------------------------------------------
# §return       $RedirectURL | The generated referral link | string
#========================================================================================
sub getPartnerReferralLink {
    my $self = shift;

    my $PartnerAccessToken = $self->getPartnerAccessToken();
    return undef unless $PartnerAccessToken;

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $PARTNER_REFERRAL_ENDPOINT;

    my $hHeaders = {
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $PartnerAccessToken,
        'Accept' => 'application/json',
    };

    my $Shop = $self->getSite();
    my $hInfoCountry = GetInfoLanguage( $Shop->get('LanguageID') );
    my $PreferredLanguage = $hInfoCountry->{'Code2'};
    my $Nonce = $self->get('GUID') . $Shop->get('GUID');

    #-- https://developer.paypal.com/docs/platforms/seller-onboarding/build-onboarding/, for step 5, but we're not doing it that way
    #my $ReturnURL = BuildShopUrl($Shop, {'ChangeAction' => 'PayPalPPCPBackLink'}, {'Type' => 'admin', 'UseSSL' => 1, 'UseObjectPath' =>0});
    #"partner_config_override": {
    #  "return_url": "$ReturnURL",
    #},

    my $OBEC;
    eval {
        my $AppServer = GetAppServer();
        my $Servlet = $AppServer->currentServlet();
        #-- This parameter tells us to onboard for 'EXPRESS_CHECKOUT' instead of PPCP, since PPCP is only available in select countries
        $OBEC = $Servlet->form->value('OBEC');
    };
    $OBEC = undef if $@;

    my $ProductsCapabilities = '"products": ["PPCP"]';

    my $OnBoardForPUI = $self->get('OnBoardForPUI');
    $ProductsCapabilities = '"products": ["PAYMENT_METHODS", "PPCP"],"capabilities": ["PAY_UPON_INVOICE"]' if $OnBoardForPUI;

    #-- This may be useful in the future
    #$ProductsCapabilities = '"products": ["EXPRESS_CHECKOUT"]' if $OBEC;

    my $TrackingID = GenerateShortGUID();

    my $ShopAddress = $Shop->get('Address');
    my $CountryCode = defined $ShopAddress && defined $ShopAddress->get('CountryID') ? GetInfoCountry( $ShopAddress->get('CountryID') )->{'Code2'} : undef;

    my $BizEntity = '';
    if ( $CountryCode ) {
      $BizEntity = <<"END";
"business_entity": {
    "addresses": [
        {
            "country_code": "$CountryCode"
        }
    ]
},
END
    }

    my $PayLoad = <<"END";
{
    "tracking_id": "$TrackingID",
    "preferred_language_code": "$PreferredLanguage",
    $BizEntity
    "operations": [{
        "operation": "API_INTEGRATION",
        "api_integration_preference": {
            "rest_api_integration": {
                "integration_method": "PAYPAL",
                "integration_type": "FIRST_PARTY",
                "first_party_details": {
                    "seller_nonce": "$Nonce",
                    "features": [
                        "PAYMENT",
                        "REFUND",
                        "ACCESS_MERCHANT_INFORMATION"
                    ]
                }
            }
        }
    }],
    "legal_consents": [{
        "type": "SHARE_DATA_CONSENT",
        "granted": true
    }],
    $ProductsCapabilities
}
END

    my $Response = $self->doRequest('POST', $RequestURL, $PayLoad, $hHeaders, { 'LogName' => 'GetPartnerReferralLink' });
    return undef unless $Response && $Response->is_success();
    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    $hJsonResponse = undef if $@;

    my $RedirectURL;
    eval {
        foreach my $hLink ( @{$hJsonResponse->{'links'}} ) {
            $RedirectURL = $hLink->{'href'} if $hLink->{'rel'} eq 'action_url';
        }
    };
    $RedirectURL = undef if $@;

    return $RedirectURL;
}

#========================================================================================
# §function     getPartnerAccessToken
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->getPartnerAccessToken();
#----------------------------------------------------------------------------------------
# §description  Retrieve/generate an access token for the partner credentials
#----------------------------------------------------------------------------------------
# §return       $AccessToken | The stored/generated access token | string
#========================================================================================
sub getPartnerAccessToken {
    my $self = shift;

    my $IsInTestMode = $self->get('IsInTestMode');
    my $PaymentType = $self->get('PaymentType');

    my $BaseURL = $self->getBaseURL();
    my $RequestURL = $BaseURL . $TOKEN_ENDPOINT . '?grant_type=client_credentials';

    my $CurrentAccessToken = $IsInTestMode ? $PaymentType->get('PartnerAccessToken_SandBox') : $PaymentType->get('PartnerAccessToken');
    my $CurrentAccessTokenExpiry = $IsInTestMode ? $PaymentType->get('PartnerAccessTokenExpiry_SandBox') : $PaymentType->get('PartnerAccessTokenExpiry');

    my $Now = GetCurrentDateTime();

    #-- If the expiry date is greater than Now
    #-- $cmp = DateTime->compare( $dt1, $dt2 ); it returns -1 if $dt1 < $dt2, 0 if $dt1 == $dt2, 1 if $dt1 > $dt2
    if ( $CurrentAccessToken && $CurrentAccessTokenExpiry && $CurrentAccessToken && DateTime->compare( $CurrentAccessTokenExpiry, $Now ) > 0 ) {

        #-- This always returns a positive value ( >= 0 ), so we needed to ensure that the expiry date is greater than now,
        # because a reverse comparison will not give a negative value
        #-- I.e. now > expiry date
        my $DeltaMSDuration = $CurrentAccessTokenExpiry->delta_ms($Now);

        #-- More than X minutes until expiry? Do not bother refreshing until next time
        if ( ($DeltaMSDuration->hours*60 + $DeltaMSDuration->minutes) >= $TOKEN_EXPIRE_THRESHOLD_MINS ) {
            return $CurrentAccessToken;
        }
    }


    my $ClientID = $IsInTestMode ? $PaymentType->get('PartnerClientID_SandBox') : $PaymentType->get('PartnerClientID');
    my $ClientSecret = $IsInTestMode ? $PaymentType->get('PartnerClientSecret_SandBox') : $PaymentType->get('PartnerClientSecret');

    return undef unless $ClientID && $ClientSecret;

    my $hHeaders = {
        'Content-Type' => 'application/x-www-form-urlencoded',
        'Authorization' => 'Basic ' . encode_base64( "$ClientID:$ClientSecret", '' ),
    };

    my $Response = $self->doRequest('POST', $RequestURL, undef, $hHeaders, { 'LogName' => 'GetPartnerAccessToken' });
    return undef unless $Response && $Response->is_success();
    my $DecodedContent = $Response->decoded_content;

    my $hJsonResponse;
    eval { $hJsonResponse = decode_json($DecodedContent); };
    $hJsonResponse = undef if $@;

    if ( defined $hJsonResponse && defined $hJsonResponse->{'access_token'} && $hJsonResponse->{'expires_in'} ) {
        my $AccessToken = $hJsonResponse->{'access_token'};
        my $ExpiresIn = $hJsonResponse->{'expires_in'};

        my $ExpiryDate = $Now->add( seconds => $ExpiresIn );
        if ( $IsInTestMode ) {
            $PaymentType->set({
                'PartnerAccessToken_SandBox' => $AccessToken,
                'PartnerAccessTokenExpiry_SandBox' => $ExpiryDate,
            });
        } else {
            $PaymentType->set({
                'PartnerAccessToken' => $AccessToken,
                'PartnerAccessTokenExpiry' => $ExpiryDate,
            });
        }
        return $AccessToken;
    }

    return undef;
}

#========================================================================================
# §function     doRequest
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->doRequest($Method, $Url, $Content, $hHeaders);
#----------------------------------------------------------------------------------------
# §description  Perform a HTTP request with the given parameters
#----------------------------------------------------------------------------------------
# §input        $Method | HTTP method e.g. GET, POST | string
# §input        $Url | URL to connect to | string
# §input        $Content | Request body content | string
# §input        $hHeaders | Headers to add to the request | ref.hash
# §input        $hAdditionalInfo | Additional information to log | ref.hash
#----------------------------------------------------------------------------------------
# §return       $Res | HTTP Response object | HTTP::Response
#========================================================================================
sub doRequest {
    my $self = shift;
    my ( $Method, $Url, $Content, $hHeaders, $hAdditionalInfo ) = @_;

    #-- Use libcurl instead
    if ( $USE_CURL ) {
        return $self->doRequestCurl( $Method, $Url, $Content, $hHeaders, $hAdditionalInfo );
    }

    my $PaymentLine = $hAdditionalInfo->{'PaymentLine'} ? $hAdditionalInfo->{'PaymentLine'} : undef;
    my $BNCode = $self->getBNCode($PaymentLine);
    $hHeaders->{'PayPal-Partner-Attribution-Id'} = $BNCode if $BNCode;

    #-- Data structures (would appear to) get mangled by Data Dumper, so make a copy for the logging side and keep the original as it was
    my $hLogHeaders = defined $hHeaders ? {%$hHeaders} : undef;

    #-- Remove the authorization header from the log
    $hLogHeaders->{'Authorization'} = '--- removed ---' if exists $hLogHeaders->{'Authorization'};

    my $LogSuffix = GenerateShortGUID();

    my $hReqLog = {
      Method          => $Method,
      Url             => $Url,
      Content         => $Content,
      hHeaders        => $hLogHeaders,
      hAdditionalInfo => $hAdditionalInfo,
    };
    $self->logPPCP( "Request $LogSuffix", $hReqLog );

    #-- We don't know why, but the request process hangs when connecting to this paypal url where the request has content in the request body
    #-- We think it may be because of tls 1.3
    #-- However, using the http 1.0 implementation makes it work!
    #-- With the default implementation LWP::Protocol::http, one of the socket sysread calls just hangs forever
    #require LWP::Protocol::http10;
    #LWP::Protocol::implementor('http', 'LWP::Protocol::http10');

    #-- The default SSL class for LWP::UserAgent is IO::Socket::SSL, but can be overridden by the ENV var $PERL_NET_HTTPS_SSL_SOCKET_CLASS
    #-- And it IS, to Net::SSL
    #-- Anyone know why? Apparently because eBay uses parallel requests, which doesn't play nicely with IO::Socket::SSL
    #-- On a system with openssl that only supports up to tls 1.2, Net::SSL DOES work,
    # but with a version of openssl that supports up to tls 1.3, Net::SSL does NOT work (suspicion, 99% sure)
    #-- IO::Socket::SSL was just updated this year, 2021, Net::SSL is still from 2014 . . .

    #my $CurrentSSLSocketCLass = $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS}; #-- Gets a complaint from perl critic
    #local $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS} = 'IO::Socket::SSL'; #-- This should work, but sometimes it doesn't (!!! *scratches head)

    #-- Seems to be the best foolproof solution
    #-- BUT, Caveat Emptor: happy reading!
    #-- https://github.com/tornadoweb/tornado/issues/2536
    #-- https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=914034
    #-- If we force TLS v1.2 as below, it should remove TLS 1.3 from the equation, but apparently hangs can still occur on renegotiation, *shrug*
    #-- Bottom line, I'm not confident that it will work 100% of the time, and who knows when TLS 1.2 will be considered 'insecure' :\
    #-- Apparently libwww-perl 6.36_2+ fixes quite a few things, we are on 6.35, but hard to test because of its intermittent nature
    local $Net::HTTPS::SSL_SOCKET_CLASS = 'IO::Socket::SSL';

    my $Agent = LWP::UserAgent->new();
    $Agent->ssl_opts(SSL_version => 'TLSv1_2');
    my $Req = HTTP::Request->new();

    $Req->method( $Method );
    $Req->uri( $Url );
    $Req->header( $_ => $hHeaders->{$_} ) foreach keys %$hHeaders;

    #$Content = encode( 'utf-8', $Content ) if $Content;
    $Req->content( $Content ) if $Content;    ##no critic qw(DE_EPAGES::Style::RequireCorrectImports)

    my $Res;
    eval {
        $Res = $Agent->request($Req);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
        return undef;
    }

    #-- Reset it back to what it was before
    #$ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS} = $CurrentSSLSocketCLass; #-- Gets a complaint from perl critic

    my $ResponseCode   = $Res->code;
    my $DecodedContent = $Res->decoded_content;

    my $hResLog = {
      Method         => $Method,
      Url            => $Url,
      ResponseCode   => $ResponseCode,
      DecodedContent => $DecodedContent,
    };

    $self->logPPCP( "Response $LogSuffix", $hResLog );

    return $Res;
}

#========================================================================================
# §function     doRequestCurl
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->doRequestCurl($Method, $Url, $Content, $hHeaders);
#----------------------------------------------------------------------------------------
# §description  Perform a HTTP request with the given parameters, using libcurl
#----------------------------------------------------------------------------------------
# §input        $Method | HTTP method e.g. GET, POST | string
# §input        $Url | URL to connect to | string
# §input        $Content | Request body content | string
# §input        $hHeaders | Headers to add to the request | ref.hash
# §input        $hAdditionalInfo | Additional information to log | ref.hash
#----------------------------------------------------------------------------------------
# §return       $Res | HTTP Response object | HTTP::Response
#========================================================================================
sub doRequestCurl {
    my $self = shift;
    my ( $Method, $Url, $Content, $hHeaders, $hAdditionalInfo ) = @_;

    my $PaymentLine = $hAdditionalInfo->{'PaymentLine'} ? $hAdditionalInfo->{'PaymentLine'} : undef;
    my $BNCode = $self->getBNCode($PaymentLine);
    $hHeaders->{'PayPal-Partner-Attribution-Id'} = $BNCode if $BNCode;

    #-- https://gms.tf/when-curl-sends-100-continue.html
    $hHeaders->{'Expect'} = '';

    #-- Data structures (would appear to) get mangled by Data Dumper, so make a copy for the logging side and keep the original as it was
    my $hLogHeaders = defined $hHeaders ? {%$hHeaders} : undef;

    #-- Remove the authorization header from the log
    $hLogHeaders->{'Authorization'} = '--- removed ---' if exists $hLogHeaders->{'Authorization'};

    my $LogName = 'PPCP';
    $LogName .= ' ' . $hAdditionalInfo->{'LogName'} if $hAdditionalInfo->{'LogName'};
    my $LogSuffix = GenerateShortGUID();
    delete $hAdditionalInfo->{'LogName'};

    my $hReqLog = {
      Method          => $Method,
      Url             => $Url,
      Content         => $Content,
      hHeaders        => $hLogHeaders,
      hAdditionalInfo => $hAdditionalInfo,
    };

    my $Easy = Net::Curl::Easy->new();
    $Easy->setopt( CURLOPT_URL, $Url );
    $Easy->setopt( CURLOPT_CUSTOMREQUEST, $Method );

    my $aHeaders;
    push @$aHeaders, $_ . ':' . $hHeaders->{$_} foreach keys %$hHeaders;
    $Easy->setopt( CURLOPT_HTTPHEADER, $aHeaders );

    if ( $Content ) {
      #$Content = encode( 'utf-8', $Content );
      $Easy->setopt( CURLOPT_POSTFIELDS, $Content );
    }

    $Easy->setopt( CURLOPT_WRITEHEADER(), \$Easy->{Headers} );
    $Easy->setopt( CURLOPT_FILE(),        \$Easy->{Body} );

     $self->logPPCP( "Request $LogName $LogSuffix", $hReqLog );
    eval {
        $Easy->perform();
    };

    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);

        return undef;
    } else {

        my $ResponseCode = $Easy->getinfo(CURLINFO_RESPONSE_CODE);

        #-- We tried to log a 'nice' variant of the response by decoding the json, but it flattens the structure after about 3 levels, so ...

        my $hResLog = {
          Method         => $Method,
          Url            => $Url,
          ResponseCode   => $ResponseCode,
          Headers        => $Easy->{Headers},
          Body           => $Easy->{Body},
        };
        $self->logPPCP( "Response $LogName $LogSuffix", $hResLog );

        my $Res = HTTP::Response->parse( $Easy->{Headers} );
        $Res->content( $Easy->{Body} );

        return $Res;
    }
}

#========================================================================================
# §function     getAlternativeCheckoutButtonInfo
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       getAlternativeCheckoutButtonInfo( $Basket );
#----------------------------------------------------------------------------------------
# §description  return hash with information about the oneclick button for this paymentmethod.
#               returns undef if preconditions for displaying the button are not met.
#----------------------------------------------------------------------------------------
# §input        $Basket | basket object | object
# §return       info hash or undef | refhash
#========================================================================================
sub getAlternativeCheckoutButtonInfo {
    my $self = shift;
    my ($Basket) = @_;

    my $hBasketContainerInfo = $Basket->container()->get(['LocaleID', 'CurrencyID']);

    my $hParams = {
        'LocaleID' => $hBasketContainerInfo->{'LocaleID'},
        'CurrencyID' => $hBasketContainerInfo->{'CurrencyID'},
    };

    my $hInfo = {
        'name' => 'ppcppayment',
    };

    if($Basket->get('IsMinOrderAmountReached') and $self->canAddToBasket($hParams)) {
        return $hInfo;
    }

    return;
}

1;
