#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Service
# state        public
#----------------------------------------------------------------------------------------
# description  class to handle the CreatePayment for PayPal plus
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Service;
use base DE_EPAGES::PayPalPlus::API::Webservice::Base::Service;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::Request;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Response;
use DE_EPAGES::PayPalPlus::API::Constants_ErrorTypes qw (
    ErrorTypes_TestString
    ErrorTypes_TestObject
    ErrorTypes_TestFloat
    ErrorTypes_TestArray
    ErrorTypes_TestHash
    ErrorTypes_TestURL
    ErrorTypes_TestInteger
);

use strict;

use constant ROUTE => 'payments/paypalplus/payment';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Service
#========================================================================================
sub new {
    my $class = shift;
    return bless {
        'ErrorCode' => undef,
        'ErrorMessage' => undef,
    }, $class;
}

#========================================================================================
# function     doRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       doRequest( $Basket, $ReturnUrlPayPalPlus, $CancelUrlPayPalPlus );
#----------------------------------------------------------------------------------------
# description  Create request to epagesj and handle the answer
#----------------------------------------------------------------------------------------
# input       $Basket | Basket object | DE_EPAGES::Order::API::Object::Basket
# input       $ReturnUrlPayPalPlus | return url from the store to procced with the payment | string
# input       $CancelUrlPayPalPlus | cancel url from the store to procced with the payment | string
# return      boolean flag if the request was successful or not | boolean
#========================================================================================
sub doRequest {
    my $self = shift;
    my ($Basket, $ReturnUrlPayPalPlus, $CancelUrlPayPalPlus) = @_;

    my $RequestRepository = $self->_prepareRequestRepository($Basket); #TODO: AD-3093 pass in urls here, solves dto problem below
    return 0 unless defined $RequestRepository;

    my $CredentialDTO = $RequestRepository->getCredentialDTO();
    my $RequestDTO = $RequestRepository->getRequestDTO();

    my $Url = $self->_buildUrlToEpagesJ();
    my $hHeaders = $CredentialDTO->toHash();
    my $hRequestData = $self->_buildRequestData($RequestDTO, $ReturnUrlPayPalPlus, $CancelUrlPayPalPlus);

    my $RestService = $self->_createRestService($Basket); ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
    my $WasRequestSuccessful = $RestService->post($Url, $hHeaders, $hRequestData);

    if( $WasRequestSuccessful ) {
        my $WasPersistanceOK = $self->_persistResponse( $RestService->getResponse(), $Basket );
        return $WasPersistanceOK;
    } else {
        ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
        $self->_setErrorCode($RestService->getErrorCode());
        $self->_setErrorMessage($RestService->getErrorMessage());
         ## use critic (ProhibitPrivateSubroutineUse)
        return 0;
    }
}
#========================================================================================
# function     _buildRequestData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildRequestData( $RequestDTO, $ReturnUrlPayPalPlus, $CancelUrlPayPalPlus );
#----------------------------------------------------------------------------------------
# description  build request data
#----------------------------------------------------------------------------------------
# input        $RequestDTO | Input1Description | DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::Request
# input        $ReturnUrlPayPalPlus | return url from the store to procced with the payment | string
# input        $CancelUrlPayPalPlus | cancel url from the store to procced with the payment | string
# return       request data in Hash structure | refhash
#========================================================================================
sub _buildRequestData {
    my $self = shift;
    my ($RequestDTO, $ReturnUrlPayPalPlus, $CancelUrlPayPalPlus) = @_;

    $RequestDTO->returnUrl($ReturnUrlPayPalPlus);
    $RequestDTO->cancelUrl($CancelUrlPayPalPlus);
    return $RequestDTO->toHash();

}
#========================================================================================
# function     _prepareRequestRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _prepareRequestRepository( $Basket );
#----------------------------------------------------------------------------------------
# description  Prepare data for the request
#----------------------------------------------------------------------------------------
# input       $Basket | Basket object | DE_EPAGES::Order::API::Object::Basket
# return      $RequestRepository | DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request
#========================================================================================
sub _prepareRequestRepository {
    my $self = shift;
    my ($Basket) = @_;

    my $RequestRepository = $self->_createRequestRepository();
    eval {
        $RequestRepository->collectData($Basket);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
        $self->_setErrorCode($Error->code()); #TODO: AD-3097 move into if branch or come up with better solution, please
        $self->_setErrorMessage('Basket not valid for create the request');
        if ( $self->_isHandleErrorFromCollectData($Error->code())) { #TODO: AD-3097 beautify name
            $self->_LogError($Error);
        } else {
            Error($Error); #forward all other Errors
        }
        ## use critic (ProhibitPrivateSubroutineUse)
        return;
    }
    return $RequestRepository;
}
#========================================================================================
# function     _persistResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _persistResponse( $hResponse, $Basket );
#----------------------------------------------------------------------------------------
# description  persists response in BussinessObject
#----------------------------------------------------------------------------------------
# input        $hResponse | response | refhash
# input        $Basket | basket object | DE_EPAGES::Order::API::Object::Basket
# return       boolean if it was ok | boolean
#========================================================================================
sub _persistResponse {
    my $self = shift;
    my ($hResponse, $Basket) = @_;

    my $ResponseRepository = $self->_createResponseRepository();
    eval {
        $ResponseRepository->writeResponse($hResponse, $Basket);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        my $ErrorCode = $Error->code();
        if ( $self->_isHandleErrorFromPersistResponse( $ErrorCode )) { #TODO: AD-3097 again the naming..
            LogError($Error);
            ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
            $self->_setErrorCode('PersistanceError');
            $self->_setErrorMessage('It was not possible to store the request in the database');
            ## use critic (ProhibitPrivateSubroutineUse)
            return 0;
        } else {
            Error($Error); #forward all other Errors
        }
    }

    return 1;
}
#========================================================================================
# function     _isHandleErrorFromPersistResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isHandleErrorFromPersistResponse( );
#----------------------------------------------------------------------------------------
# description  check if is an expected error code
#----------------------------------------------------------------------------------------
# input        $ErrorCode | Error code |string
# return       Flag to check if the error is one of the expected errors | boolean
#========================================================================================
sub _isHandleErrorFromPersistResponse{
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestString, ErrorTypes_TestObject,
                                ErrorTypes_TestArray, ErrorTypes_TestHash,
                                ErrorTypes_TestURL)) ) {
        return 1;
    } else {
        return 0;
    }
}
#========================================================================================
# function     _isHandleErrorFromCollectData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isHandleErrorFromCollectData( );
#----------------------------------------------------------------------------------------
# description  check if is an expected error code
#----------------------------------------------------------------------------------------
# input        $ErrorCode | Error code |string
# return       Flag to check if the error is one of the expected errors | boolean
#========================================================================================
sub _isHandleErrorFromCollectData{
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestFloat, ErrorTypes_TestString,
                                ErrorTypes_TestObject, ErrorTypes_TestInteger)) ) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     _buildUrlToEpagesJ
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildUrlToEpagesJ( $CredentialDTO );
#----------------------------------------------------------------------------------------
# description  build the Url for the post
#----------------------------------------------------------------------------------------
# return       Url for post request | string
#========================================================================================
sub _buildUrlToEpagesJ {
    my $self = shift;

    my $RestURLService = $self->_createRestURLService();## no critic (ProhibitPrivateSubroutineUse) protected (in base class)

    my $Url = $RestURLService->buildEpagesJRestUrl(ROUTE, {});
    return $Url;
}


#========================================================================================
# function     _createRequestRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createRequestRepository( );
#----------------------------------------------------------------------------------------
# description  Create DAO object
#----------------------------------------------------------------------------------------
# return        CreatePayment Request Repository object | DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request
#========================================================================================
sub _createRequestRepository {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request->new();
}
#========================================================================================
# function     _createResponseRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createResponseRepository( );
#----------------------------------------------------------------------------------------
# description  Create DAO object
#----------------------------------------------------------------------------------------
# return        CreatePayment Request Repository object | DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Response
#========================================================================================
sub _createResponseRepository {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Response->new();
}
1;
