#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Service
# state        public
#----------------------------------------------------------------------------------------
# description  class to handle the ExecutePayment for PayPal plus
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Service;
use base DE_EPAGES::PayPalPlus::API::Webservice::Base::Service;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Repository;
use DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::RequestDTO;
use DE_EPAGES::PayPalPlus::API::Constants_ErrorTypes qw (
    ErrorTypes_TestString
    ErrorTypes_TestObject
    ErrorTypes_TestInteger
    ErrorTypes_TestArray
    ErrorTypes_TestHash
    ErrorTypes_TestFloat
);
use strict;

use constant ROUTE => 'payments/paypalplus/payment/execute';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Service
#========================================================================================
sub new {
    my $class = shift;
    return bless {
        'ErrorCode' => undef,
        'ErrorMessage' => undef,
    }, $class;
}

#========================================================================================
# function     doRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       doRequest( $CustomerOrder );
#----------------------------------------------------------------------------------------
# description  Create request to epagesj and handle the answer
#----------------------------------------------------------------------------------------
# input        $OrderOrBasket | OrderOrBasket | DE_EPAGES::Order::API::Object::CustomerOrder or
#                                                DE_EPAGES::Order::API::Object::Basket
# return       boolean flag if the request was successful or not | boolean
#========================================================================================
sub doRequest {
    my $self = shift;
    my ($OrderOrBasket) = @_;

    my $Repository = $self->_prepareRequestRepository($OrderOrBasket);
    return 0 unless defined $Repository;
    $self->{'Repository'} = $Repository;

    my $RequestDTO = $Repository->getRequestDTO();
    my $CredentialDTO = $Repository->getCredentialDTO();

    my $Url = $self->_buildUrlToEpagesJ();
    my $hHeaders = $CredentialDTO->toHash();
    my $hRequest = $RequestDTO->toHash();

    my $RestService = $self->_createRestService($OrderOrBasket); ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
    my $WasRequestSuccessful = $RestService->post($Url, $hHeaders, $hRequest);

    if( $WasRequestSuccessful ) {
        my $WasVerifyOk = $self->_verifyResponse($RestService->getResponse());
        return $WasVerifyOk;
    } else {
        ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
        $self->_setErrorCode($RestService->getErrorCode());
        $self->_setErrorMessage($RestService->getErrorMessage());
        ## use critic (ProhibitPrivateSubroutineUse)
        return 0;
    }
}

#========================================================================================
# function     persistResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       persistResponse($CustomerOrder );
#----------------------------------------------------------------------------------------
# description  persists response in BussinessObject
#----------------------------------------------------------------------------------------
# input        $CustomerOrder | Customer Order object | DE_EPAGES::Order::API::Object::CustomerOrder
# return       boolean if it was ok | boolean
#========================================================================================
sub persistResponse {
    my $self = shift;
    my ($CustomerOrder) = @_;

    my $Repository = $self->{'Repository'};
    eval {
        $Repository->writeResponse($self->{'hVerifiedResponse'}, $CustomerOrder);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        my $ErrorCode = $Error->code();
        if ( $self->_isErrorFromPersistResponse( $ErrorCode )) {
            return $self->_handleError($Error, 'PersistanceError', 'It was not possible to store the request in the database');
        } elsif( $self->_isErrorFromPersistResponse_OrderStatus( $ErrorCode )) {
            return $self->_handleError($Error, $ErrorCode, $ErrorCode);
        } else {
            Error($Error); #forward all other Errors
        }
    }

    return 1;
}

sub _verifyResponse {
    my $self = shift;
    my ( $hResponse ) = @_;

    my $Repository = $self->{'Repository'};
    eval {
        my $hVerifiedResponse = $Repository->verifyResponse($hResponse);
        $self->{'hVerifiedResponse'} = $hVerifiedResponse;
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        my $ErrorCode = $Error->code();
        if ( $self->_isErrorFromVerifyResponse( $ErrorCode )) {
            return $self->_handleError($Error, $ErrorCode, $ErrorCode);
        } else {
            Error($Error); #forward all other Errors
        }
    }

    return 1;
}

#========================================================================================
# function     _prepareRequestRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _prepareRequestRepository( $CustomerOrder );
#----------------------------------------------------------------------------------------
# description  Builds RequestDTO from Order
#----------------------------------------------------------------------------------------
# input        $OrderOrBasket | OrderOrBasket | DE_EPAGES::Order::API::Object::CustomerOrder or
#                                                DE_EPAGES::Order::API::Object::Basket
# return       $RequestRepository | DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Repository
#========================================================================================
sub _prepareRequestRepository {
    my $self = shift;
    my ($OrderOrBasket) = @_;

    my $Repository = $self->_createRepository();
    eval {
        $Repository->buildRequest($OrderOrBasket);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        my $ErrorCode = $Error->code();
        if ( $self->_isErrorFromBuildRequest( $ErrorCode )) {
            $self->_handleError($Error, 'RequestError', 'It was not possible to get the data for the Request from the database');
        } else {
            Error($Error); #forward all other Errors
        }
        return;
    }

    return $Repository;
}

sub _isErrorFromBuildRequest {
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestString, ErrorTypes_TestObject,
                                ErrorTypes_TestInteger)) ) {
        return 1;
    }  else {
        return 0;
    }
}

sub _isErrorFromPersistResponse {
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestArray, ErrorTypes_TestHash,
                                ErrorTypes_TestObject)) ) {
        return 1;
    } else {
        return 0;
    }
}

sub _isErrorFromVerifyResponse {
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestArray, ErrorTypes_TestHash,
                                ErrorTypes_TestString, ErrorTypes_TestFloat)) ) {
        return 1;
    } else {
        return 0;
    }
}

sub _isErrorFromPersistResponse_OrderStatus {
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, qw(OrderAlreadyPaid UnexpectedState
                                  UNSUPPORTED_ARRAY_SIZE)) ) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     _buildUrlToEpagesJ
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildUrlToEpagesJ( $CredentialDTO, $RequestDTO );
#----------------------------------------------------------------------------------------
# description  build the Url for the post
#----------------------------------------------------------------------------------------
# return       Url for request | string
#========================================================================================
sub _buildUrlToEpagesJ {
    my $self = shift;

    my $RestURLService = $self->_createRestURLService(); ## no critic (ProhibitPrivateSubroutineUse) protected defiend in base class

    return $RestURLService->buildEpagesJRestUrl(ROUTE, {});
}

#========================================================================================
# function     _createRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createRepository( );
#----------------------------------------------------------------------------------------
# description  Create Repository
#----------------------------------------------------------------------------------------
# return       Repository object | DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Repository
#========================================================================================
sub _createRepository {
    my $self = shift;
    my $Repository = DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Repository->new();
    return $Repository;
}

#========================================================================================
# function     _handleError
# state        private
#----------------------------------------------------------------------------------------
# syntax       _handleError( $Error, $ErrorMessage, $ErrorCode );
#----------------------------------------------------------------------------------------
# description  Logs Error and sets Code and Message
#----------------------------------------------------------------------------------------
# input        $Error | Error | DE_EPAGES::Core::API::Error
# input        $ErrorCode | Error Code | string
# input        $ErrorMessage | Error Message | string
# return       0 | boolean
#========================================================================================
sub _handleError {
    my $self = shift;
    my ($Error, $ErrorCode, $ErrorMessage) = @_;
    ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
    $self->_LogError($Error);
    $self->_setErrorCode($ErrorCode);
    $self->_setErrorMessage($ErrorMessage);
    ## use critic (ProhibitPrivateSubroutineUse)
    return 0;
}

1;
