#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service
# state        public
#----------------------------------------------------------------------------------------
# description  webservice call for experience profile id to epagesJ and handles errors
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service;
use base DE_EPAGES::PayPalPlus::API::Webservice::Base::Service;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Repository;
use DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::RequestDTO;
use DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::ResponseDTO;
use DE_EPAGES::PayPalPlus::API::Webservice::RestService;
use DE_EPAGES::PayPalPlus::API::Webservice::RestURLService;
use DE_EPAGES::PayPalPlus::API::Constants_ErrorTypes qw (
    ErrorTypes_TestString
    ErrorTypes_TestObject
    ErrorTypes_TestInteger
    ErrorTypes_TestHash
);
use strict;

use constant ROUTE => 'payments/paypalplus/paymentexperience';

## no critic (ProhibitPrivateSubroutineUse)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $PaymentMethod | PaymentMethod | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service;
#========================================================================================
sub new {
    my $class = shift;
    my ($PaymentMethod) = @_;
    my $self  = bless {
        'PaymentMethod' => $PaymentMethod,
    }, $class;
    return $self;
}

#========================================================================================
# function     doRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       doRequest( $PaymentMethod );
#----------------------------------------------------------------------------------------
# description  does a webservice call and handles errors
#----------------------------------------------------------------------------------------
# return       returns 1 if successful, 0 else | boolean
#========================================================================================
sub doRequest {
    my $self = shift;
    my $PaymentMethod = $self->{'PaymentMethod'};

    my $Result = 1;

    my $Repository = $self->_prepareRequestRepository($PaymentMethod);
    return 0 unless defined $Repository;
    $self->{'Repository'} = $Repository;

    eval {
        my $RequestDTO = $Repository->getRequestDTO();
        my $CredentialDTO = $Repository->getCredentialDTO();

        my $hResponse = $self->_call( $RequestDTO, $CredentialDTO );
        $Result = $self->_buildResponseDTO($hResponse);
    };
    if(ExistsError()) {
        return 0;
    };

    return $Result;
}

#========================================================================================
# function     _prepareRequestRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _prepareRequestRepository( $PaymentMethod );
#----------------------------------------------------------------------------------------
# description  prepare the Request repository
#----------------------------------------------------------------------------------------
# input        $PaymentMethod | PaymentMethod | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       $Repository or undef | DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Repository
#========================================================================================
sub _prepareRequestRepository {
    my $self = shift;
    my ($PaymentMethod) = @_;

    my $Repository = $self->_createRepository();

    eval {
        $Repository->buildRequest($PaymentMethod);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( ElementOf($Error->code(), (ErrorTypes_TestString, ErrorTypes_TestObject,
                                        ErrorTypes_TestInteger, ErrorTypes_TestHash)) ) {
            $self->_LogError($Error);
            $self->_setErrorCode('BuildRequestError'); ## no critic (ProhibitPrivateSubroutineUse)
            $self->_setErrorMessage('ExperienceProfileId Request to paypal plus could not be created');## no critic (ProhibitPrivateSubroutineUse)
            return;
        } else {
            Error($Error); #forward all other Errors
        }
    }

    return $Repository;
}

#----------------------------------------------------------------------------------------
sub _buildResponseDTO {
    my $self = shift;
    my ($hResponse) = @_;

    my $Repository = $self->{'Repository'};

    eval {
        $Repository->buildResponse($hResponse);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        #TODO: AD-3097 change for ErrorType constances
        if ( ElementOf($Error->code(), qw (STRING_ISOBJECT STRING_EMPTY PARAMETER_MISSING)) ) {
            $self->_LogError($Error);
            $self->_setErrorCode('BuildResponseError');
            $self->_setErrorMessage('ExperienceProfileId Response could not be processed');
            return 0;
        } else {
            Error($Error); #forward all other Errors
        }
    }

    return 1;
}

#========================================================================================
# function     getResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       getResponse( );
#----------------------------------------------------------------------------------------
# description  get response as ResponseDTO
#----------------------------------------------------------------------------------------
# return       Response DTO | DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::ResponseDTO
#========================================================================================
sub getResponse {
    my $self = shift;
    return $self->{'Repository'}->getResponseDTO();
}

#========================================================================================
# function     persistResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       persistResponse( );
#----------------------------------------------------------------------------------------
# description  persists response to epages6 businessobjects/database
#----------------------------------------------------------------------------------------
# return       1 if successful, 0 else | boolean
#========================================================================================
sub persistResponse {
    my $self = shift;
    my $PaymentMethod = $self->{'PaymentMethod'};
    eval {
        $self->{'Repository'}->writeResponse($PaymentMethod);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( ElementOf($Error->code, ErrorTypes_TestObject) ) {
            $self->_LogError($Error);
            $self->_setErrorCode('PersistError');## no critic (ProhibitPrivateSubroutineUse)
            $self->_setErrorMessage('Could not write to Businessobject');## no critic (ProhibitPrivateSubroutineUse)
            return 0;
        } else {
            Error($Error); #forward all other Errors
        }
    }
    return 1;
}

#========================================================================================
# function     _call
# state        private
#----------------------------------------------------------------------------------------
# syntax       _call( $RequestDTO, $CredentialDTO );
#----------------------------------------------------------------------------------------
# description  does a webservicecall to epagesJ with the Data from the RequestDTO
#               Throws Error if webservice call failed
#----------------------------------------------------------------------------------------
# input        $RequestDTO | Request DTO | DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::RequestDTO
# input        $CredentialDTO | credential DTO | DE_EPAGES::PayPalPlus::API::Webservice::Base::CredentialDTO
# return       $hResponse | Response as hash | refhash
#========================================================================================
sub _call {
    my $self = shift;
    my ($RequestDTO, $CredentialDTO) = @_;

    my $Url = $self->_buildUrlToEpagesJ();

    my $hRequest = $RequestDTO->getRequestHash();
    my $hHeaders = $CredentialDTO->toHash();
    my $RestService = $self->_createRestService($self->{'PaymentMethod'});## no critic (ProhibitPrivateSubroutineUse)

    my $hResponse;
    if( $RestService->post($Url, $hHeaders, $hRequest) ){
       $hResponse = $RestService->getResponse();
    }else{
        $self->_setErrorCode($RestService->getErrorCode());## no critic (ProhibitPrivateSubroutineUse)
        $self->_setErrorMessage($RestService->getErrorMessage());## no critic (ProhibitPrivateSubroutineUse)
    }

    return $hResponse;
}

#----------------------------------------------------------------------------------------
sub _buildUrlToEpagesJ {
    my $self = shift;

    my $RestURLService = $self->_createRestURLService();## no critic (ProhibitPrivateSubroutineUse) protected (in base class)

    my $Url = $RestURLService->buildEpagesJRestUrl(ROUTE, {});
    return $Url;
}

#----------------------------------------------------------------------------------------
sub _createRepository {
    my $self = shift;
    my $Repository = DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Repository->new();
    return $Repository;
}


1;
