#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service
# state        public
#----------------------------------------------------------------------------------------
# description  class to handle the GetMerchantStatus for PayPal plus
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service;
use base DE_EPAGES::PayPalPlus::API::Webservice::Base::Service;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestArray TestHash );
use DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository;
use DE_EPAGES::PayPalPlus::API::Constants_ErrorTypes
  qw (ErrorTypes_TestString ErrorTypes_TestObject ErrorTypes_TestInteger);

use strict;

sub ErrorDefinition {
    return { 'MissingOAuthCredentials' => 'Missing OAuthCredentials in response of merchant status call', };
}

use constant BASE_ROUTE => 'payments/paypalplus/merchantstatus';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service
#========================================================================================
sub new {
    my $class = shift;
    my $self = bless {}, $class;
    return $self;
}

#========================================================================================
# function     doRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WasSuccessful = $self->doRequest( $PaymentMethodPayPalPlus );
#----------------------------------------------------------------------------------------
# description  Create request to epagesj and handle the answer
#----------------------------------------------------------------------------------------
# input        $PaymentMethodPayPalPlus | payment method paypal plus | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       $WasSuccessful | boolean flag if the request was successful or not | boolean
#========================================================================================
sub doRequest {
    my $self = shift;
    my ($PaymentMethodPayPalPlus) = @_;

    my $WasSuccessful;

    eval {
        my $Repository = $self->_prepareRequestRepository($PaymentMethodPayPalPlus);

        $self->{'Repository'} = $Repository;
        my $CredentialDTO = $Repository->getCredentialDTO();
        my $RequestDTO    = $Repository->getRequestDTO();

        my $Url      = $self->_buildUrlToEpagesJ();
        my $hHeaders = $CredentialDTO->toHash();
        my $hRequest = $RequestDTO->toHash();

        my $RestService = $self->_createRestService($PaymentMethodPayPalPlus);    ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)

        $RestService->setLogErrorsInComunicationlog(1);
        my $IsOnboardingCall = 1;
        $WasSuccessful = $RestService->post( $Url, $hHeaders, $hRequest, $IsOnboardingCall );

        ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
        if ($WasSuccessful) {
            $self->_setResponse( $RestService->getResponse() );
        }
        else {
            $self->_setErrorCode( $RestService->getErrorCode() );
            $self->_setErrorMessage( $RestService->getErrorMessage() );
        }
        ## use critic (ProhibitPrivateSubroutineUse)
    };
    if ( ExistsError() ) {
        return 0;
    }

    return $WasSuccessful;
}

#========================================================================================
# function     persistResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WasSuccessful = $self->persistResponse( $PaymentMethodPayPalPlus );
#----------------------------------------------------------------------------------------
# description  save required response entries into repository
#----------------------------------------------------------------------------------------
# input        $PaymentMethodPayPalPlus | payment method paypal plus | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       $WasSuccessful | boolean flag if the response has been successfully persisted | boolean
#========================================================================================
sub persistResponse {
    my $self = shift;
    my ($PaymentMethodPayPalPlus) = @_;

    return 0 unless ( isValid( $self->getResponse() ) );

    my $Repository = $self->{'Repository'};
    eval { $Repository->writeResponse( $self, $PaymentMethodPayPalPlus ); };
    if ( ExistsError() ) {
        my $Error     = GetError();
        my $ErrorCode = $Error->code();
        if ( $self->_isHandleErrorFromPersistResponse($ErrorCode) ) {    #TODO: AD-3097 beautify name
            return $self->_handleError( $Error, 'PersistanceError',
                'It was not possible to store the request in the database' );
        }
        else {
            Error($Error);                                               #forward all other Errors
        }
    }

    return 1;
}

#========================================================================================
# function     getMerchantId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MerchantId = $self->getMerchantId( );
#----------------------------------------------------------------------------------------
# description  return obtained merchant id value from service call
#----------------------------------------------------------------------------------------
# return       $MerchantId | merchant id code | string
#========================================================================================
sub getMerchantId {
    my $self = shift;

    my $hResponse = $self->getResponse();
    return $hResponse->{'merchant_id'};
}

#========================================================================================
# function     getMerchantClientId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MerchantClientId = $self->getMerchantClientId( );
#----------------------------------------------------------------------------------------
# description  return obtained merchant client id value from service call
#----------------------------------------------------------------------------------------
# input        $PaymentMethodPayPalPlus | payment method paypal plus | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       $MerchantClientId | merchant client id code | string
#========================================================================================
sub getMerchantClientId {
    my $self = shift;
    my ($PaymentMethodPayPalPlus) = @_;

    my $hResponse          = $self->getResponse();
    my $aOAuthIntegrations = $hResponse->{'oauth_integrations'};
    my $MerchantClientId;

    eval {
        TestArray( 'aOAuthIntegrations', $aOAuthIntegrations );
        my $ProviderClientId = $PaymentMethodPayPalPlus->getPartnerClientId();
        foreach my $hOAuthIntegration ( @{$aOAuthIntegrations} ) {
            TestHash( 'hOAuthIntegration', $hOAuthIntegration );
            my $aOAuthCredentials = $hOAuthIntegration->{'oauth_third_party'};
            TestArray( 'aOAuthCredentials', $aOAuthCredentials );
            foreach my $hOAUthCredential ( @{$aOAuthCredentials} ) {
                if ( $hOAUthCredential->{'partner_client_id'} eq $ProviderClientId ) {
                    $MerchantClientId = $hOAUthCredential->{'merchant_client_id'};
                }
            }
        }
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        Error( 'MissingOAuthCredentials', {}, $Error );
    }

    return $MerchantClientId;
}

#========================================================================================
# function     getPaymentsReceivable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentsReceivable = $self->getPaymentsReceivable( );
#----------------------------------------------------------------------------------------
# description  return service call information whether payments are receivable for the PayPal account
#----------------------------------------------------------------------------------------
# return       $PaymentsReceivable | whether payments are receivable | boolean
#========================================================================================
sub getPaymentsReceivable {
    my $self = shift;

    my $hResponse = $self->getResponse();
    return $hResponse->{'payments_receivable'};
}

#========================================================================================
# function     getPrimaryEmailConfirmed
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PrimaryEmailConfirmed = $self->getPrimaryEmailConfirmed( );
#----------------------------------------------------------------------------------------
# description  return service call information whether primary email is confirmed for the PayPal account
#----------------------------------------------------------------------------------------
# return       $PrimaryEmailConfirmed | whether primary email is confirmed | boolean
#========================================================================================
sub getPrimaryEmailConfirmed {
    my $self = shift;

    my $hResponse = $self->getResponse();
    return $hResponse->{'primary_email_confirmed'};
}

#========================================================================================
# function     getPrimaryEmail
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PrimaryEmail = $self->getPrimaryEmail( );
#----------------------------------------------------------------------------------------
# description  return service call information about primary email used for the PayPal account
#----------------------------------------------------------------------------------------
# return       $PrimaryEmail | primary email | string
#========================================================================================
sub getPrimaryEmail {
    my $self = shift;

    my $hResponse = $self->getResponse();
    return $hResponse->{'primary_email'};
}

#========================================================================================
# function     getProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProducts = $self->getProducts( );
#----------------------------------------------------------------------------------------
# description  return available products for the PayPal account from service call
#----------------------------------------------------------------------------------------
# return       $aProducts | product names | arrayref
#========================================================================================
sub getProducts {
    my $self = shift;

    my $hResponse = $self->getResponse();
    return $hResponse->{'products'};
}

#========================================================================================
# function     getProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hProduct = $self->getProduct( $ProductName);
#----------------------------------------------------------------------------------------
# description  return Product from service call
#----------------------------------------------------------------------------------------
# input        $ProductName | product name | string
# return       $hProduct | product | hashref
#========================================================================================
sub getProduct {
    my $self = shift;
    my ($ProductName) = @_;

    my $aProducts = $self->getProducts();
    for my $hProduct ( @{$aProducts} ) {
        if ( $hProduct->{'name'} eq $ProductName ) {
            return $hProduct;
        }
    }
    return;
}

#========================================================================================
# function     getIsActivePayPal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isActivePayPal = $self->getIsActivePayPal( );
#----------------------------------------------------------------------------------------
# description  PayPal method of payment is available and active
#----------------------------------------------------------------------------------------
# return       $isActivePayPal | PayPal is active | boolean
#========================================================================================
sub getIsActivePayPal {
    my $self     = shift;
    my $hProduct = $self->getProduct('BASIC_PPPLUS_CORE');
    if ( defined($hProduct) && ( $hProduct->{'status'} eq 'ACTIVE' ) ) {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     getIsActiveDirectDebit
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isActiveDirectDebit = $self->getIsActiveDirectDebit( );
#----------------------------------------------------------------------------------------
# description  DirectDebit method of payment is available and active
#----------------------------------------------------------------------------------------
# return       $isActiveDirectDebit | DirectDebit is active | boolean
#========================================================================================
sub getIsActiveDirectDebit {
    my $self     = shift;
    my $hProduct = $self->getProduct('BASIC_PPPLUS_GUEST_ELV');
    if ( defined($hProduct) && ( $hProduct->{'status'} eq 'ACTIVE' ) ) {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     getIsActiveCreditCard
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isActiveCreditCard = $self->getIsActiveCreditCard( );
#----------------------------------------------------------------------------------------
# description  CreditCard method of payment is available and active
#----------------------------------------------------------------------------------------
# return       $isActiveCreditCard | CreditCard is active | boolean
#========================================================================================
sub getIsActiveCreditCard {
    my $self     = shift;
    my $hProduct = $self->getProduct('BASIC_PPPLUS_GUEST_CC');
    if ( defined($hProduct) && ( $hProduct->{'status'} eq 'ACTIVE' ) ) {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     getIsActiveInvoice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isActiveInvoice = $self->getIsActiveInvoice( );
#----------------------------------------------------------------------------------------
# description  Invoice method of payment is available and active
#----------------------------------------------------------------------------------------
# return       $isActiveInvoice | Invoice is active | boolean
#========================================================================================
sub getIsActiveInvoice {
    my $self     = shift;
    my $hProduct = $self->getProduct('BASIC_PPPLUS_PUI');
    if ( defined($hProduct) && ( $hProduct->{'status'} eq 'ACTIVE' ) ) {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     getMerchantStatusConflicts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aConflicts = $self->getMerchantStatusConflicts( );
#----------------------------------------------------------------------------------------
# description  return merchant status non conformant for operation values (conflicts)
#----------------------------------------------------------------------------------------
# return       $aConflicts | conflict names | arrayref
#========================================================================================
sub getMerchantStatusConflicts {
    my $self = shift;

    my @Conflicts = ();
    push( @Conflicts, { 'Conflict' => 'PaymentsReceivable' } )    if ( $self->getPaymentsReceivable()    ne 'true' );
    push( @Conflicts, { 'Conflict' => 'PrimaryEmailConfirmed' } ) if ( $self->getPrimaryEmailConfirmed() ne 'true' );

    return \@Conflicts;
}

#========================================================================================
# function     hasMerchantStatusConflicts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HasMerchantStatusConflicts = $self->hasMerchantStatusConflicts( );
#----------------------------------------------------------------------------------------
# description  return either merchant status has non conformant for operation values (conflicts) or not
#----------------------------------------------------------------------------------------
# return       $HasMerchantStatusConflicts | report if there's any conflict | boolean
#========================================================================================
sub hasMerchantStatusConflicts {
    my $self = shift;

    my $aMerchantStatusConflicts = $self->getMerchantStatusConflicts();
    return 1 if ( scalar( @{$aMerchantStatusConflicts} ) > 0 );
    return 0;
}

#========================================================================================
# function     _prepareRequestRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RequestRepository = $self->_prepareRequestRepository( $PaymentMethodPayPalPlus );
#----------------------------------------------------------------------------------------
# description  Builds RequestDTO from Order
#----------------------------------------------------------------------------------------
# input        $PaymentMethodPayPalPlus | PaymentMethod of PayPalPlus | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       $RequestRepository | DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository
#========================================================================================
sub _prepareRequestRepository {
    my $self = shift;
    my ($PaymentMethodPayPalPlus) = @_;

    my $Repository = $self->_createRepository();
    eval { $Repository->buildRequest($PaymentMethodPayPalPlus); };
    if ( ExistsError() ) {
        my $Error     = GetError();
        my $ErrorCode = $Error->code();
        if ( $self->_isHandleErrorFromBuildRequest($ErrorCode) ) {  #TODO: AD-3097 beautify name isErrorFromBuildRequest
            $self->_handleError( $Error, 'RequestError',
                'It was not possible to get the data for the Request from the database' );
        }
        else {
            Error($Error);                                          #forward all other Errors
        }
        return;
    }

    return $Repository;
}

#========================================================================================
# function     _buildUrlToEpagesJ
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EpagesjUrl = $self->_buildUrlToEpagesJ( $CredentialDTO );
#----------------------------------------------------------------------------------------
# description  build the Url for the post
#----------------------------------------------------------------------------------------
# return       $EpagesjUrl | epagesj Url for the request | string
#========================================================================================
sub _buildUrlToEpagesJ {
    my $self = shift;

    my $RestURLService
      = $self->_createRestURLService();    ## no critic (ProhibitPrivateSubroutineUse) protected defiend in base class

    return $RestURLService->buildEpagesJRestUrl( BASE_ROUTE, {} );
}

#========================================================================================
# function     _handleError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_handleError( $Error, $ErrorMessage, $ErrorCode );
#----------------------------------------------------------------------------------------
# description  Logs Error and sets Code and Message
#----------------------------------------------------------------------------------------
# input        $Error | Error | DE_EPAGES::Core::API::Error
# input        $ErrorCode | Error Code | string
# input        $ErrorMessage | Error Message | string
# return       0 | boolean
#========================================================================================
sub _handleError {
    my $self = shift;
    my ( $Error, $ErrorCode, $ErrorMessage ) = @_;
    ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
    $self->_LogError($Error);
    $self->_setErrorCode($ErrorCode);
    $self->_setErrorMessage($ErrorMessage);
    ## use critic (ProhibitPrivateSubroutineUse)
    return 0;
}

#========================================================================================
# function     _isHandleErrorFromPersistResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isHandleErrorFromPersistResponse( );
#----------------------------------------------------------------------------------------
# description  check if is an expected error code
#----------------------------------------------------------------------------------------
# input        $ErrorCode | Error code |string
# return       Flag to check if the error is one of the expected errors | boolean
#========================================================================================
sub _isHandleErrorFromBuildRequest {
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestString, ErrorTypes_TestObject,
                                ErrorTypes_TestInteger)) ) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     _isHandleErrorFromPersistResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isHandleErrorFromPersistResponse( );
#----------------------------------------------------------------------------------------
# description  check if is an expected error code
#----------------------------------------------------------------------------------------
# input        $ErrorCode | Error code |string
# return       Flag to check if the error is one of the expected errors | boolean
#========================================================================================
sub _isHandleErrorFromPersistResponse {
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestString, ErrorTypes_TestObject,
                                'MissingOAuthCredentials')) ) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     _createRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       $MerchantStatusRepository = $self->_createRepository( );
#----------------------------------------------------------------------------------------
# description  Create Repository
#----------------------------------------------------------------------------------------
# return       $MerchantStatusRepository | Repository object | DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository
#========================================================================================
sub _createRepository {
    my $self       = shift;
    my $Repository = DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository->new();
    return $Repository;
}
1;
