#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::Notification::IPNSecure
# state        public
#----------------------------------------------------------------------------------------
# description  Class to validate the incomming notification from paypal
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::Notification::IPNSecure;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw ( TestHash TestObject );
use DE_EPAGES::PayPalPlus::API::PaymentLineItemService;
use DE_EPAGES::PayPalPlus::API::Logger;
use DE_EPAGES::PayPalPlus::API::Constants_ErrorTypes qw (
    ErrorTypes_TestObject
    ErrorTypes_TestHash
);

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::Notification::IPNSecure
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     handleNotification
# state        public
#----------------------------------------------------------------------------------------
# syntax       handleNotification( $Shop, $hNotificationRequest );
#----------------------------------------------------------------------------------------
# description  Take the incomming notification, validate and update order/lineItem
#----------------------------------------------------------------------------------------
# input        $Shop | Shop Object | DE_EPAGES::Shop::API::Object::Shop
# input        $hNotificationRequest | hash with notification request | hashref
# return       Boolean flag if everything worked fine | boolean
#========================================================================================
sub handleNotification {
    my $self = shift;
    my ($Shop, $hNotificationRequest) = @_;
    my $Logger = $self->_createLogger();
    $Logger->logPayment('Notification Params', {'NotificationParams' => $hNotificationRequest});

    my $IsValid = 1;
    eval {
        $IsValid = $self->_checkNotification( $Shop, $hNotificationRequest );
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        $IsValid = 0;
        if ( not $self->_isHandleErrorFromValidate($Error->code()) ) { #TODO: use better name
            Error($Error); #forward all other Errors
        }
    }
    return $IsValid;
}

#========================================================================================
# function     _checkNotification
# state        private
#----------------------------------------------------------------------------------------
# syntax       _checkNotification( );
#----------------------------------------------------------------------------------------
# description  validade notification and update order
#----------------------------------------------------------------------------------------
# input        $Shop | shop object of the request | object
# input        $hNotificationRequest | hash with notification request | hashref
# return       Boolean flag if everything worked fine | boolean
#========================================================================================
sub _checkNotification {
    my $self = shift;
    my ( $Shop, $hNotificationRequest ) = @_;

    $self->_validateParams($Shop, $hNotificationRequest);

    my $LineItemPaymentGuid = $hNotificationRequest->{'lineitem_payment_guid'};

    my $PaymentLineItemService = $self->_createPaymentLineItemService();
    return 0 unless $PaymentLineItemService->canLoadByGuid($LineItemPaymentGuid);

    my $LineItemPayment = $PaymentLineItemService->getLineItemPayment();
    return 0 unless $self->_verifyNotification($LineItemPayment, $hNotificationRequest, $Shop->id());

    return 0 unless $PaymentLineItemService->updateStatus($hNotificationRequest->{'payment_status'});
}

#========================================================================================
# function     _validateParams
# state        private
#----------------------------------------------------------------------------------------
# syntax       _validateParams( $Shop, $hNotificationRequest );
#----------------------------------------------------------------------------------------
# description  verify if params are valid and add shop id in case it is, this method can throw dies
#----------------------------------------------------------------------------------------
# input        $Shop | Shop Object | DE_EPAGES::Shop::API::Object::Shop
# input        $hNotificationRequest | hash with notification request | hashref
# return        $hNotificationRequest with shopID | hashref
#========================================================================================
sub _validateParams {
    my $self = shift;
    my ($Shop, $hNotificationRequest) = @_;

    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');
    TestHash('hNotificationRequest', $hNotificationRequest);
}

#========================================================================================
# function     _verifyNotification
# state        private
#----------------------------------------------------------------------------------------
# syntax       _verifyNotification( $LineItemPayment, $hNotificationRequest );
#----------------------------------------------------------------------------------------
# description  verify if the notification is for this shop
#----------------------------------------------------------------------------------------
# input        $LineItemPayment | LineItemPaymentPaypalPlus Object | DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus
# input        $hNotificationRequest | hash with notification request | hashref
# input        $ShopId | shop id | int
# return       return boolean flag if the request is valid
#========================================================================================
sub _verifyNotification {
    my $self = shift;
    my ($LineItemPayment, $hNotificationRequest, $ShopId) = @_;

    my $LineItemContainer = $LineItemPayment->parent();

    return 0 unless $LineItemPayment->getSite()->id() eq $ShopId;

    return 0 unless $LineItemPayment->get('PayerID') eq $hNotificationRequest->{'payer_id'};

    return 0 unless $LineItemPayment->get('TransID') eq $hNotificationRequest->{'transaction_id'};

    return 0 unless $LineItemContainer->get('CurrencyID') eq $hNotificationRequest->{'payment_currency'};

    return 0 unless $self->_isSameAmount($LineItemContainer->get('GrandTotal'), $hNotificationRequest->{'payment_amount'});

    return 1;
}

#========================================================================================
# function     _isSameAmount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isSameAmount( $ShopAmount, $NotificationAmount );
#----------------------------------------------------------------------------------------
# description  Truncate 2 floats with 2 decimals after comma and return compare them
#----------------------------------------------------------------------------------------
# input        $ShopAmount | float value | float
# input        $NotificationAmount | float value | float
# return       return boolean flag if are the same value
#========================================================================================
sub _isSameAmount {
    my $self = shift;
    my ($ShopAmount, $NotificationAmount) = @_;

    my $ShopAmount2DecimalDigits = sprintf ('%0.2f', $ShopAmount);
    my $NotificationAmount2DecimalDigits = sprintf ('%0.2f', $NotificationAmount);

    if( $ShopAmount2DecimalDigits eq $NotificationAmount2DecimalDigits) {
        return 1;
    } else {
        return 0;
    }
}
#========================================================================================
# function     _isHandleErrorFromValidate
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isHandleErrorFromValidate( );
#----------------------------------------------------------------------------------------
# description  check if is an expected error code
#----------------------------------------------------------------------------------------
# input        $ErrorCode | Error code |string
# return       Flag to check if the error is one of the expected errors | boolean
#========================================================================================
sub _isHandleErrorFromValidate{
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestObject, ErrorTypes_TestHash)) ) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     _createPaymentLineItemService
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createPaymentLineItemService( );
#----------------------------------------------------------------------------------------
# description  create PaymentLineItemService
#----------------------------------------------------------------------------------------
# return       return DE_EPAGES::PayPalPlus::API::PaymentLineItemService
#========================================================================================
sub _createPaymentLineItemService {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::PaymentLineItemService->new();
}


#========================================================================================
# function     _createLogger
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_createLogger( );
#----------------------------------------------------------------------------------------
# description  Create logger class
#----------------------------------------------------------------------------------------
# return       $Object | Logger object | DE_EPAGES::PayPalPlus::API::Logger
#========================================================================================
sub _createLogger {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Logger->new();
}
1;
