#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::Service
# state        public
#----------------------------------------------------------------------------------------
# description  class to handle the PatchPayment for PayPal plus
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::Service;
use base DE_EPAGES::PayPalPlus::API::Webservice::Base::Service;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::PayPalPlus::API::Constants_ErrorTypes qw (
    ErrorTypes_TestInteger
    ErrorTypes_TestObject
    ErrorTypes_TestString
);
use DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::RequestDTO;
use DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::RequestRepository;
use DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::RequestRepositoryOrderAlias;

use strict;

use constant ROUTE => 'payments/paypalplus/patch';
use constant ROUTE_ORDER_REFERENCE => 'payments/paypalplus/patch/orderreference';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::Service
#----------------------------------------------------------------------------------------
#========================================================================================
sub new {
    my $class = shift;
    return bless {
        'ErrorCode' => undef,
        'ErrorMessage' => undef,
    }, $class;
}

#========================================================================================
# function     getErrorCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorCode( );
#----------------------------------------------------------------------------------------
# description  Get the error code
#----------------------------------------------------------------------------------------
# return      Error code | string
#========================================================================================
sub getErrorCode {
    my $self = shift;
    return $self->{'ErrorCode'};
}

#========================================================================================
# function     getErrorMessage
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorMessage( );
#----------------------------------------------------------------------------------------
# description  Get the error message
#----------------------------------------------------------------------------------------
# return      Error message | string
#========================================================================================
sub getErrorMessage {
    my $self = shift;
    return $self->{'ErrorMessage'};
}

#========================================================================================
# function     doRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       doRequest( $Basket);
#----------------------------------------------------------------------------------------
# description  patch request to epagesj and handle the answer
#----------------------------------------------------------------------------------------
# input       $Basket | Basket object | DE_EPAGES::Order::API::Object::Basket
# return      boolean flag if the request was successful or not | boolean
#========================================================================================
sub doRequest {
    my $self = shift;
    my ($Basket) = @_;

    $Basket->set({'IsAddressOK'=>1});
    $Basket->recalculate();
    my $RequestRepository = $self->_prepareRequestRepository($Basket);
    return 0 unless defined $RequestRepository;

    my $RequestDTO = $RequestRepository->getRequestDTO();
    my $CredentialDTO = $RequestRepository->getCredentialDTO();

    my $Url = $self->_buildUrlToEpagesJ(ROUTE);
    my $hHeaders = $CredentialDTO->toHash();
    my $hRequestData = $RequestDTO->toHash();

    my $RestService = $self->_createRestService($Basket); ## no critic (ProhibitPrivateSubroutineUse) protected
    my $WasRequestSuccessful = $RestService->post($Url, $hHeaders, $hRequestData);

    if( $WasRequestSuccessful ) {
        return 1;
    } else {
        ## no critic (ProhibitPrivateSubroutineUse) protected
        $self->_setErrorCode($RestService->getErrorCode());
        $self->_setErrorMessage($RestService->getErrorMessage());
        ## use critic (ProhibitPrivateSubroutineUse) protected
        return 0;
    }
}
#========================================================================================
# function     doOrderAliasPatchRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       doOrderAliasPatchRequest( $Order);
#----------------------------------------------------------------------------------------
# description  patch request to epagesj to update the transaction with the OrderAlias
#----------------------------------------------------------------------------------------
# input       $Order | $Order object | DE_EPAGES::Order::API::Object::CustomerOrder
# return      boolean flag if the request was successful or not | boolean
#========================================================================================
sub doOrderAliasPatchRequest {
    my $self = shift;
    my ($Order) = @_;

    my $RequestRepository = $self->_prepareRequestRepositoryOrderAlias($Order);
    return 0 unless defined $RequestRepository;

    my $RequestDTO = $RequestRepository->getRequestDTO();
    my $CredentialDTO = $RequestRepository->getCredentialDTO();

    my $Url = $self->_buildUrlToEpagesJ(ROUTE_ORDER_REFERENCE);
    my $hHeaders = $CredentialDTO->toHash();
    my $hRequestData = $RequestDTO->toHash();

    my $RestService = $self->_createRestService($Order); ## no critic (ProhibitPrivateSubroutineUse) protected
    my $WasRequestSuccessful = $RestService->post($Url, $hHeaders, $hRequestData);

    if( $WasRequestSuccessful ) {
        return 1;
    } else {
        ## no critic (ProhibitPrivateSubroutineUse) protected
        $self->_setErrorCode($RestService->getErrorCode());
        $self->_setErrorMessage($RestService->getErrorMessage());
        ## use critic (ProhibitPrivateSubroutineUse) protected
        return 0;
    }
}
#----------------------------------------------------------------------------------------
sub _prepareRequestRepository {
    my $self = shift;
    my ($Basket) = @_;
    my $RequestRepository = $self->_createRequestRepository();
    return $self->_collectData($RequestRepository, $Basket);
}
#----------------------------------------------------------------------------------------
sub _prepareRequestRepositoryOrderAlias {
    my $self = shift;
    my ($Order) = @_;
    my $RequestRepository = $self->_createRequestRepositoryOrderAlias();
    return $self->_collectData($RequestRepository, $Order);
}
#----------------------------------------------------------------------------------------
sub _collectData {
    my $self = shift;
    my ($RequestRepository, $BasketOrOrder) = @_;
    eval {
        $RequestRepository->collectData($BasketOrOrder);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( $self->_isHandleErrorFromCollectData($Error->code())) {#TODO: AD-3097 beautify name
            ## no critic (ProhibitPrivateSubroutineUse) protected
            $self->_setErrorCode($Error->code());
            $self->_setErrorMessage('Basket not valid for creating the request');
            $self->_LogError($Error);
            ## use critic (ProhibitPrivateSubroutineUse) protected
        } else {
            Error($Error); #forward all other Errors
        }
        return;
    }
    return $RequestRepository;
}
#========================================================================================
# function     _isHandleErrorFromCollectData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isHandleErrorFromCollectData( );
#----------------------------------------------------------------------------------------
# description  check if is an expected error code
#----------------------------------------------------------------------------------------
# input        $ErrorCode | Error code |string
# return       Flag to check if the error is one of the expected errors | boolean
#========================================================================================
sub _isHandleErrorFromCollectData{
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( ElementOf($ErrorCode, (ErrorTypes_TestInteger, ErrorTypes_TestObject,
                                ErrorTypes_TestString)) ) {
        return 1;
    } else {
        return 0;
    }
}

#----------------------------------------------------------------------------------------
sub _buildUrlToEpagesJ {
    my $self = shift;
    my ($Route) = @_;

    my $RestURLService = $self->_createRestURLService(); ## no critic (ProhibitPrivateSubroutineUse) protected
    my $URL = $RestURLService->buildEpagesJRestUrl($Route, {});
    return $URL;
}

#----------------------------------------------------------------------------------------
sub _createRequestRepository {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::RequestRepository->new();
}
#----------------------------------------------------------------------------------------
sub _createRequestRepositoryOrderAlias {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::RequestRepositoryOrderAlias->new();
}
1;
