#==================================================================================================
# package      DE_EPAGES::Paybox::t::unit::PaymentGateway
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Paybox::API::PaymentGateway::Paybox::PaymentGateway
#==================================================================================================
package DE_EPAGES::Paybox::t::unit::PaymentGateway;
use base DE_EPAGES::Core::API::Test;

use strict;

use Test::Exception;
use Test::More;

use DE_EPAGES::Paybox::API::GatewayInterface::Implementations::PaymentGateway;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

use DE_EPAGES::Paybox::t::unit::MockupObjects qw (
    TEST_PAYBOX_URL
    TEST_PAYBOX_URL_PREP
    TEST_PAYBOX_SITEID
    TEST_PAYBOX_RANK
    TEST_PAYBOX_ID
    TEST_SHOP_URL
    TEST_SHOP_EMAIL
    TEST_CURRENCY_ID
    TEST_ORDER_TOTAL_SENT
    TEST_TRANS_ID
    TEST_CARDTYPE

    NewMockedPaymentGatewayConfigurator
    NewMockedShop
    NewMockedPaymentDispatcher
    NewMockedCallbackParams
    NewMockedCallbackParamsUnity
);

use DE_EPAGES::Paybox::API::Constants qw (
    STATUS_PAID
    STATUS_CANCELLED
);

use constant TEST_PAYMENT_DATE  => '2014-12-31T12:34:56';
use constant TEST_HMAC          => 'MOCKED_HMAC_RETURN';

#--------------------------------------------------------------------------------------------------
# Callback Request URL must capture /^(.*)&Sign=(.*)/
#   Group(1) : Message
#   Group(2) : Signature
my $Test_CallbackRequest_OK = {
    'URL'       => 'ChangeAction=PbCb&Code=12345&TransId=67890&Sign=ABCDEFG',
    'Message'   => 'Code=12345&TransId=67890',
    'Signature' => 'ABCDEFG'
};
my $Test_CallbackRequest_ERROR = {
    'URL'       => 'ChangeAction=PbCb&Code=12345&TransId=67890&Sign=ABCDEFG',
    'Message'   => 'Code=12345&TransId=67890',
    'Signature' => 'ABCDEFX'
};

#--------------------------------------------------------------------------------------------------
# Backlink Request URL must capture /^.*(Code=.*)&Sign=(.*)/
#   Group(1) : Message
#   Group(2) : Signature
my $Test_BacklinkRequest_OK = {
    'URL'       => 'ChangeAction=PbCb&Code=12345&TransId=67890&Sign=ABCDEFG',
    'Message'   => 'ChangeAction=PbCb&Code=12345&TransId=67890',
    'Signature' => 'ABCDEFG'
};
my $Test_BacklinkRequest_ERROR = {
    'URL'       => 'ChangeAction=PbCb&Code=12345&TransId=67890&Sign=ABCDEFG',
    'Message'   => 'ChangeAction=PbCb&Code=12345&TransId=67890',
    'Signature' => 'ABCDEFX'
};

#==================================================================================================
# test
#==================================================================================================
sub test {
    my $self = shift;

    $self->test_getPaymentLiveRequestURL();
    $self->test_getPaymentTestRequestURL();
    $self->test_getPaymentRequestData();
    $self->test_processPaymentResponse_OK();
    $self->test_processPaymentResponse_ERROR();
    $self->test_checkPaymentResponse_OK();
    $self->test_checkPaymentResponse_ERROR();
    $self->test_capturePayment_OK();
    $self->test_capturePayment_ERROR();
    $self->test_cancelPayment_OK();
    $self->test_cancelPayment_ERROR();

    $self->test_checkPaymentResponse_UnityFailed();
    $self->test_checkPaymentResponse_UnityCanceled();
    done_testing();
}

#==================================================================================================
# test_getPaymentLiveRequestURL
#
#   Tests if the correct Live RequestURL for redirect to the PaymentGateway is returned
#==================================================================================================
sub test_getPaymentLiveRequestURL {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->getPaymentLiveRequestURL';

    my $MockContext = $self->_createMockContext();
    $MockContext->{'Paybox'}->set({'IsInTestMode' => 0});    # Be sure Paybox is set to Live mode
    is($MockContext->{'PaymentGateway'}->getPaymentRequestURL(),
        TEST_PAYBOX_URL,
        "$SubTestName - URL Live mode is OK"
    );

    return;
}

#==================================================================================================
# test_getPaymentTestRequestURL
#
#   Tests if the correct Test RequestURL for redirect to the PaymentGateway is returned
#==================================================================================================
sub test_getPaymentTestRequestURL {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->getPaymentTestRequestURL';

    my $MockContext = $self->_createMockContext();
    $MockContext->{'Paybox'}->set({'IsInTestMode' => 1});    # Be sure Paybox is set to Test mode
    is($MockContext->{'PaymentGateway'}->getPaymentRequestURL(),
        TEST_PAYBOX_URL_PREP,
        "$SubTestName - URL Test mode is OK"
    );

    return;
}

#==================================================================================================
# test_getPaymentRequestData
#
#   Tests the data returned for the Payment Gateway to prepare the POST for the Paybox request
#   The data must include specific fields and values in a known order.
#==================================================================================================
sub test_getPaymentRequestData {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->getPaymentRequestData';

    my $MockContext = $self->_createMockContext();

    my $Paybox = $MockContext->{'Paybox'};
    $Paybox->set({'IsInTestMode' => 0});  # Be sure Paybox is set to Live mode

    my $PaymentGateway = $MockContext->{'PaymentGateway'};
    my $LineItemPayment = $MockContext->{'LineItemPayment'};
    my $aRequestData = $PaymentGateway->getPaymentRequestData($LineItemPayment);

    # Test if the number of returned fields matches the expected value
    my $aRequestDataFieldsOrder = $self->_getRequestDataFieldsOrder();
    my $RequestDataLength = scalar @$aRequestData;
    my $FieldsOrderLength = scalar @$aRequestDataFieldsOrder;
    is( $RequestDataLength,
        $FieldsOrderLength,
        "$SubTestName - Number of fields returned is OK"
    );

    # Test if the returned fields matches the expected order
    if( $RequestDataLength == $FieldsOrderLength ) {
        if( $self->_checkRequestDataFieldsOrder(
                $SubTestName,
                $aRequestData,
                $aRequestDataFieldsOrder
            )) {
            pass("$SubTestName - Fields order is OK");
        }

        # Test if the returned field values matches the expected ones
        ok( $self->_checkRequestDataValues(
                $SubTestName,
                $aRequestData,
            ),
            "$SubTestName - Fields values are OK",
        );
    } else {
        fail("$SubTestName - I can't compare request fields because the number doesn't match");
    }

    return;
}

#==================================================================================================
# _getRequestDataFieldsOrder
#
#   Ordered array with the expected fields to prepare the Form POST for the RequestData
#==================================================================================================
sub _getRequestDataFieldsOrder {
    my $self = shift;

    return [
        'PBX_SITE',
        'PBX_RANG',
        'PBX_IDENTIFIANT',
        'PBX_TOTAL',
        'PBX_DEVISE',
        'PBX_CMD',
        'PBX_TYPEPAIEMENT',
        'PBX_TYPECARTE',
        'PBX_AUTOSEULE',
        'PBX_PORTEUR',
        'PBX_SOUHAITAUTHENT',
        'PBX_SHOPPINGCART',
        'PBX_BILLING',
        'PBX_RETOUR',
        'PBX_HASH',
        'PBX_TIME',
        'PBX_REPONDRE_A',
        'PBX_EFFECTUE',
        'PBX_REFUSE',
        'PBX_ANNULE',
        'PBX_ATTENTE',
        'PBX_HMAC'
    ];
}

#==================================================================================================
# _checkRequestDataFieldsOrder
#
#   Compares arrays to verify fields order
#==================================================================================================
sub _checkRequestDataFieldsOrder {
    my $self = shift;
    my ($SubTestName, $aRequestData, $aRequestDataFieldsOrder) = @_;

    my $RequestDataLength = scalar @$aRequestData;
    for(my $count = 0; $count < $RequestDataLength; $count++) {
        my $ReturnedField = $aRequestData->[$count]->{'Field'};
        my $ExpectedField = $aRequestDataFieldsOrder->[$count];
        if ($ReturnedField ne $ExpectedField) {
            is( $ReturnedField,
                $ExpectedField,
                "$SubTestName - Request field order is OK"
            );
            return 0;
        }
    }

    return 1;
}

#==================================================================================================
# _checkRequestDataValues
#
#   Check valid array values
#==================================================================================================
sub _checkRequestDataValues {
    my $self = shift;
    my ($SubTestName, $aRequestData) = @_;

    my $hRequestDataExpected = $self->_getMockedRequestData();
    foreach my $item (@$aRequestData) {
        my $RequestFieldName = $item->{'Field'};
        my $RequestValue = $item->{'Value'};
        my $ExpectedValue = $hRequestDataExpected->{$RequestFieldName};
        if ($RequestValue ne $ExpectedValue) {
            if (ElementOf($RequestFieldName, qw(PBX_CMD PBX_SHOPPINGCART PBX_BILLING))) {
              like( $RequestValue, qr/^$ExpectedValue/, "Request value in field '$RequestFieldName' is OK");
            } else {
              is( $RequestValue, $ExpectedValue, "Request value in field '$RequestFieldName' is OK" );
            }
        }
    }

    return 1;
}

#==================================================================================================
# _getMockedRequestData
#
#   Hash with the expected fields and values to prepare the Form POST for the RequestData
#==================================================================================================
sub _getMockedRequestData {
    my $self = shift;

    return {
        'PBX_SITE'          => TEST_PAYBOX_SITEID,
        'PBX_RANG'          => TEST_PAYBOX_RANK,
        'PBX_IDENTIFIANT'   => TEST_PAYBOX_ID,
        'PBX_TOTAL'         => TEST_ORDER_TOTAL_SENT,
        'PBX_DEVISE'        => TEST_CURRENCY_ID,
        'PBX_CMD'           => 'PayboxID_'.TEST_TRANS_ID,
        'PBX_TYPEPAIEMENT'  => 'CARTE',
        'PBX_TYPECARTE'     => TEST_CARDTYPE,
        'PBX_AUTOSEULE'     => 'N',
        'PBX_PORTEUR'       => TEST_SHOP_EMAIL,
        'PBX_SOUHAITAUTHENT'=> '01',
        'PBX_SHOPPINGCART'  => '<\?xml version="1.0" encoding="utf-8"\?><shoppingcart>.+</shoppingcart>',
        'PBX_BILLING'       => '<\?xml version="1.0" encoding="utf-8"\?><Billing>.+</Billing>',
        'PBX_RETOUR'        => 'Code:E;Amount:M;PaymentId_TransId:R;TransId2:S;Auto:A;CallNumber:T;Pays:Y;Sign:K;3DSv:v;3DSauth:c',
        'PBX_HASH'          => 'SHA512',
        'PBX_TIME'          => TEST_PAYMENT_DATE,
        'PBX_REPONDRE_A'    => TEST_SHOP_URL,
        'PBX_EFFECTUE'      => TEST_SHOP_URL,
        'PBX_REFUSE'        => TEST_SHOP_URL,
        'PBX_ANNULE'        => TEST_SHOP_URL,
        'PBX_ATTENTE'       => TEST_SHOP_URL,
        'PBX_HMAC'          => TEST_HMAC
    };
}

#==================================================================================================
# test_processPaymentResponse_OK
#
#   Test the validation of the Callback URL in the PaymentGateway object with a good URL
#==================================================================================================
 sub test_processPaymentResponse_OK {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->processPaymentResponse_OK';

    my $MockedPaymentDispatcher =   $self->test_processPaymentResponse(
                                        $SubTestName,
                                        $Test_CallbackRequest_OK
                                    );

    ok( $MockedPaymentDispatcher->get('TestPassed'),
        "$SubTestName - Callback Request URL with valid Signature verification OK"
    );

    return;
}

#==================================================================================================
# test_processPaymentResponse_ERROR
#
#   Test the validation of the Callback URL in the PaymentGateway object with a bad URL
#==================================================================================================
sub test_processPaymentResponse_ERROR {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->processPaymentResponse_ERROR';

    my $MockedPaymentDispatcher =   $self->test_processPaymentResponse(
                                        $SubTestName,
                                        $Test_CallbackRequest_ERROR
                                    );

    ok( !$MockedPaymentDispatcher->get('TestPassed'),
        "$SubTestName - Callback Request URL with bad Signature verification OK"
    );

    return;
}

#==================================================================================================
# test_processPaymentResponse
#
#   Test the validation of the Callback URL in the PaymentGateway object
#   The order dispatching is skiped with a mocked PaymentDispatcher
#   The mocked PaymentDispatcher test the capture of Message & Signature from the URL
#==================================================================================================
sub test_processPaymentResponse {
    my $self = shift;
    my ($SubTestName, $Test_CallbackRequest) = @_;

    my $MockContext = $self->_createMockContext($Test_CallbackRequest);
    my $MockedPaymentDispatcher = NewMockedPaymentDispatcher();
    my $CallbackParams = NewMockedCallbackParams();
    $CallbackParams->{'QUERY_STRING'} = $Test_CallbackRequest->{'URL'};
    $CallbackParams->{'PaymentId_TransId'} = 'PayboxID_'.$MockContext->{'LineItemPayment'}->guid;

    my $hPaymentContext = {
        'Dispatcher'     => $MockedPaymentDispatcher,
        'Shop'           => $MockContext->{'Shop'},
        'CallbackParams' => $CallbackParams,
    };

    my $PaymentGateway = $MockContext->{'PaymentGateway'};
    $PaymentGateway->processPaymentResponse($hPaymentContext);

    return $MockedPaymentDispatcher;
}

#==================================================================================================
# test_checkPaymentResponse_OK
#
#   Test the validation of the Backlink URL in the PaymentGateway object with a good URL
#==================================================================================================
sub test_checkPaymentResponse_OK {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->checkPaymentResponse_OK';

    my $hResult = $self->test_checkPaymentResponse($SubTestName, $Test_BacklinkRequest_OK);

    ok($hResult->{'Order'}->alias, "$SubTestName - Backlink Request URL with valid Signature verification OK");
    is($hResult->{'WebUrlUnitySSL'}, 'https://unity.link', "$SubTestName - Check if WebUrlUnitySSL was set correctly");

    return;
}

#==================================================================================================
# test_checkPaymentResponse_ERROR
#
#   Test the validation of the Backlink URL in the PaymentGateway object with a bad URL
#==================================================================================================
sub test_checkPaymentResponse_ERROR {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->checkPaymentResponse_ERROR';

    my $hResult = $self->test_checkPaymentResponse($SubTestName, $Test_BacklinkRequest_ERROR);

    ok($hResult->{'ErrorCode'}, "$SubTestName - Backlink Request URL with bad Signature verification OK");
    ok($hResult->{'ErrorViewObject'}->alias, "$SubTestName - Backlink Request URL with bad Signature verification OK");

    return;
}

#==================================================================================================
# test_checkPaymentResponse
#
#   Test the validation of the Backlink URL in the PaymentGateway object
#   The order dispatching is skiped with a mocked PaymentDispatcher
#   The mocked PaymentDispatcher test the capture of Message & Signature from the URL
#==================================================================================================
sub test_checkPaymentResponse {
    my $self = shift;
    my ($SubTestName, $Test_BacklinkRequest) = @_;

    my $MockContext = $self->_createMockContext($Test_BacklinkRequest);
    my $MockedPaymentDispatcher = NewMockedPaymentDispatcher();
    my $CallbackParams = NewMockedCallbackParams();
    $CallbackParams->{'QUERY_STRING'} = $Test_BacklinkRequest->{'URL'};
    $CallbackParams->{'PaymentId_TransId'} = 'PayboxID_'.$MockContext->{'LineItemPayment'}->guid;

    my $hPaymentContext = {
        'Dispatcher'    => $MockedPaymentDispatcher,
        'Shop'          => $MockContext->{'Shop'},
        'CallbackParams' => $CallbackParams,
    };

    my $PaymentGateway = $MockContext->{'PaymentGateway'};
    my $hResult = $PaymentGateway->checkPaymentResponse($hPaymentContext);

    return $hResult;
}

#==================================================================================================
# test_capturePayment_OK
#
#   Test the response handling of the capturePayment method. By mocking the
#   user agent no actual request is send and a predefined response is provided.
#   The predefined response is used to simulate the capture of a Paybox payment.
#   In this test case a successful response is used to simulate a successful capture.
#==================================================================================================
sub test_capturePayment_OK {
    my $self = shift;

    my $ResponseContent = 'PORTEUR=undef&RANG=32&NUMTRANS=0005986811&NUMQUESTION=0183221500&REFABONNE=&'.
        'AUTORISATION=XXXXXX&CODEREPONSE=00000&SITE=1999888&NUMAPPEL=0011394173&COMMENTAIRE'.
        '=Demande traite avec succes';
    my $hMockContext = $self->_createMockContext({'PayboxDirectResponse' => $ResponseContent});
    my $MockedShop = $hMockContext->{'Shop'};
    my $MockedOrder = $hMockContext->{'Order'};
    my $MockedLineItemPayment = $hMockContext->{'LineItemPayment'};
    my $PaymentGateway = $hMockContext->{'PaymentGateway'};

    my $hResult = $PaymentGateway->capturePayment($MockedShop, $MockedLineItemPayment);
    ok($hResult->{'success'}, 'successful result');
    ok(defined($MockedOrder->get('PaidOn')), 'order paid');
    is($MockedLineItemPayment->get('TransStatus'), STATUS_PAID, 'status LineItemPayment');

    return;
}

#==================================================================================================
# test_capturePayment_ERROR
#
#   Test the response handling of the capturePayment method. By mocking the
#   user agent no actual request is send and a predefined response is provided.
#   The predefined response is used to simulate the capture of a Paybox payment.
#   In this test case a response containing an error is used to simulate an
#   unsuccessful capture.
#==================================================================================================
sub test_capturePayment_ERROR {
    my $self = shift;

    my $ResponseContent = 'PORTEUR=undef&RANG=32&NUMTRANS=0005986811&NUMQUESTION=0183221500&REFABONNE=&'.
        'AUTORISATION=XXXXXX&CODEREPONSE=00815&SITE=1999888&NUMAPPEL=0011394173&COMMENTAIRE=Errr';
    my $hMockContext = $self->_createMockContext({'PayboxDirectResponse' => $ResponseContent});
    my $MockedShop = $hMockContext->{'Shop'};
    my $MockedOrder = $hMockContext->{'Order'};
    my $MockedLineItemPayment = $hMockContext->{'LineItemPayment'};
    my $PaymentGateway = $hMockContext->{'PaymentGateway'};

    my $hResult = $PaymentGateway->capturePayment($MockedShop, $MockedLineItemPayment);
    ok(!$hResult->{'success'}, 'successful result');
    ok(!defined($MockedOrder->get('PaidOn')), 'order cancelled');
    ok($MockedLineItemPayment->get('TransStatus') ne STATUS_PAID, 'status LineItemPayment');

    return;
}

#==================================================================================================
# test_cancelPayment_OK
#
#   Test the response handling of the cancelPayment method. By mocking the
#   user agent no actual request is send and a predefined response is provided.
#   The predefined response is used to simulate the cancellation of a Paybox payment.
#   In this test case a successful response is used to simulate a successful cancel.
#==================================================================================================
sub test_cancelPayment_OK {
    my $self = shift;

    my $ResponseContent = 'PORTEUR=undef&RANG=32&NUMTRANS=0005986811&NUMQUESTION=0183221500&REFABONNE=&'.
        'AUTORISATION=XXXXXX&CODEREPONSE=00000&SITE=1999888&NUMAPPEL=0011394173&COMMENTAIRE'.
        '=Demande traite avec succes';
    my $hMockContext = $self->_createMockContext({'PayboxDirectResponse' => $ResponseContent});
    my $MockedShop = $hMockContext->{'Shop'};
    my $MockedOrder = $hMockContext->{'Order'};
    my $MockedLineItemPayment = $hMockContext->{'LineItemPayment'};
    my $PaymentGateway = $hMockContext->{'PaymentGateway'};

    my $hResult = $PaymentGateway->cancelPayment($MockedShop, $MockedLineItemPayment);
    ok($hResult->{'success'}, 'successful result');
    ok(defined($MockedOrder->get('CancelledOn')), 'order cancelled');
    is($MockedLineItemPayment->get('TransStatus'), STATUS_CANCELLED, 'status LineItemPayment');

    return;
}

#==================================================================================================
# test_cancelPayment_ERROR
#
#   Test the response handling of the cancelPayment method. By mocking the
#   user agent no actual request is send and a predefined response is provided.
#   The predefined response is used to simulate the cancellation of a Paybox payment.
#   In this test case a response containing an error is used to simulate a
#   unsuccessful cancellation.
#==================================================================================================
sub test_cancelPayment_ERROR {
    my $self = shift;

    my $ResponseContent = 'PORTEUR=undef&RANG=32&NUMTRANS=0005986811&NUMQUESTION=0183221500&REFABONNE=&'.
        'AUTORISATION=XXXXXX&CODEREPONSE=00815&SITE=1999888&NUMAPPEL=0011394173&COMMENTAIRE=Errr';
    my $hMockContext = $self->_createMockContext({'PayboxDirectResponse' => $ResponseContent});
    my $MockedShop = $hMockContext->{'Shop'};
    my $MockedOrder = $hMockContext->{'Order'};
    my $MockedLineItemPayment = $hMockContext->{'LineItemPayment'};
    my $PaymentGateway = $hMockContext->{'PaymentGateway'};

    my $hResult = $PaymentGateway->cancelPayment($MockedShop, $MockedLineItemPayment);
    ok(!$hResult->{'success'}, 'successful result');
    ok(!defined($MockedOrder->get('CancelledOn')), 'order cancelled');
    ok($MockedLineItemPayment->get('TransStatus') ne STATUS_CANCELLED, 'status LineItemPayment');

    return;
}

sub test_checkPaymentResponse_UnityFailed {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->checkPaymentResponse_UnityFailed';
    my $Code = '00000'; # Code represents Status of the transaction
    my $hContext = $self->_getContext_onCheckPaymentResponse($SubTestName, $Test_BacklinkRequest_ERROR, NewMockedCallbackParamsUnity($Code));
    my $PaymentErrorCode = $hContext->{'LineItemPayment'}->get('PaymentErrorCode');

    is($PaymentErrorCode, 'Failed', "$SubTestName - Check if PaymentErrorCode is Failed");

    return;
}

sub test_checkPaymentResponse_UnityCanceled {
    my $self = shift;

    my $SubTestName = 'PaymentGateway->checkPaymentResponse_UnityCanceled';
    my $Code = '00500';# Code represents Status of the transaction
    my $hContext = $self->_getContext_onCheckPaymentResponse($SubTestName, $Test_BacklinkRequest_ERROR, NewMockedCallbackParamsUnity($Code));
    my $PaymentErrorCode = $hContext->{'LineItemPayment'}->get('PaymentErrorCode');

    is($PaymentErrorCode, 'Canceled', "$SubTestName - Check if PaymentErrorCode is Canceled");

    return;
}

sub _getContext_onCheckPaymentResponse {
    my $self = shift;
    my ($SubTestName, $Test_BacklinkRequest, $CallbackParams) = @_;

    my $MockContext = $self->_createMockContext($Test_BacklinkRequest);
    my $MockedPaymentDispatcher = NewMockedPaymentDispatcher();
    $CallbackParams->{'QUERY_STRING'} = $Test_BacklinkRequest->{'URL'};
    $CallbackParams->{'PaymentId_TransId'} = 'PayboxID_'.$MockContext->{'LineItemPayment'}->guid;

    my $hPaymentContext = {
        'Dispatcher'    => $MockedPaymentDispatcher,
        'Shop'          => $MockContext->{'Shop'},
        'CallbackParams' => $CallbackParams,
    };

    my $PaymentGateway = $MockContext->{'PaymentGateway'};
    $PaymentGateway->checkPaymentResponse($hPaymentContext);

    return $MockContext;
}
#==================================================================================================
# _createMockContext
#
#   Creates all the common mock objects: Shop, Paybox, PaymentGateway
#==================================================================================================
sub _createMockContext {
    my $self = shift;
    my ($PaymentGatewayMockupData) = @_;

    my $MockedShop = NewMockedShop();
    my $MockedPaybox = $MockedShop->child('PaymentMethods')->child('Paybox');
    my $MockedOrder = $MockedShop->get('Orders')->[0];
    my $MockedLineItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');
    my $MockedConfigurator = NewMockedPaymentGatewayConfigurator($MockedShop);
    my $PaymentGateway = DE_EPAGES::Paybox::API::GatewayInterface::Implementations::PaymentGateway->new();
    $PaymentGateway = $self->_setMockedMethodsToPaymentGateway(
        $PaymentGateway,
        $PaymentGatewayMockupData,
        $MockedLineItemPayment
    );
    $PaymentGateway->initialize($MockedConfigurator);

    return {
        'Shop'              => $MockedShop,
        'Configurator'      => $MockedConfigurator,
        'Paybox'            => $MockedPaybox,
        'PaymentGateway'    => $PaymentGateway,
        'Order'             => $MockedOrder,
        'LineItemPayment'   => $MockedLineItemPayment
    };
}

#==================================================================================================
# _setMockedMethodsToPaymentGateway
#
#   Wraps in the PaymentGateway object the private methods which perform external calls.
#   We replace those methods with a fake version returning a known value.
#
#   The optional argument '$PaymentGatewayMockupData' is passed in the tests where the callback
#   or backlink should be tested in method '_checkSignature'
#==================================================================================================
sub _setMockedMethodsToPaymentGateway {
    my $self = shift;
    my ($PaymentGateway, $PaymentGatewayMockupData, $MockedLineItemPayment) = @_;

    $PaymentGatewayMockupData //= {};

    my $MockPaymentGateway = Test::MockObject::Extends->new($PaymentGateway);
    $MockPaymentGateway->mock(
        '_getCurrentDateTime',
        sub { return TEST_PAYMENT_DATE; }
    );
    $MockPaymentGateway->mock(
        '_buildResponseURL',
        sub { return TEST_SHOP_URL }
    );
    $MockPaymentGateway->mock(
        '_calculateHMAC',
        sub { return TEST_HMAC }
    );
    $MockPaymentGateway->mock(
        '_checkSignature',
        sub {
            my $self = shift;
            my ($hData) = @_;
            return ($hData->{'Message'} eq $PaymentGatewayMockupData->{'Message'})
                   && ($hData->{'Signature'} eq $PaymentGatewayMockupData->{'Signature'});
        }
    );
    $MockPaymentGateway->mock(
        '_existsObjectByGUID',
        sub {
            my $self = shift;
            my ($GUID) = @_;

            return 1 if ($MockedLineItemPayment->guid eq $GUID);
            return 0;
        }
    );
    $MockPaymentGateway->mock(
        '_loadObjectByGUID',
        sub {
            my $self = shift;
            my ($GUID) = @_;

            return $MockedLineItemPayment if ($MockedLineItemPayment->guid eq $GUID);
            return undef;
        }
    );

    $MockPaymentGateway->mock(
        '_createUserAgent',
        sub {
            my $UserAgent = LWP::UserAgent->new();
            my $MockedUserAgent = Test::MockObject::Extends->new($UserAgent);
            $MockedUserAgent->mock(
                'post',
                sub {
                    my $Response = HTTP::Response->new( 200 ); ## no critic qw(ProhibitMagicNumbers);
                    $Response->content($PaymentGatewayMockupData->{'PayboxDirectResponse'});
                    return $Response;
                }
            );
            return $MockedUserAgent;
        }
    );

    $MockPaymentGateway->mock(
        'getPayboxDirectURL',
        sub { return ''; }
    );

    $MockPaymentGateway->mock(
        '_buildShoppingCartXml',
        sub { return
            '<?xml version="1.0" encoding="utf-8"?>' .
            '<shoppingcart><total><totalQuantity>15</totalQuantity></total></shoppingcart>';
        }
    );

    $MockPaymentGateway->mock(
        '_buildBillingXml',
        sub { return
            '<?xml version="1.0" encoding="utf-8"?>' .
            '<Billing><Address><FirstName>John</FirstName><LastName>Doe</LastName>' .
             '<Address1>12 rue Paul Dautier</Address1><ZipCode>78140</ZipCode>' .
             '<City>Velizy-Villacoublay</City><CountryCode>250</CountryCode></Address></Billing>';
        }
    );

    return $MockPaymentGateway;
}

#==================================================================================================
# main
#==================================================================================================
__PACKAGE__->runScript();

