#========================================================================================
# package      DE_EPAGES::PaymentTypes::API::Attributes::Basket
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Extended attribute interface for Basket.
#========================================================================================
package DE_EPAGES::PaymentTypes::API::Attributes::Basket;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use Readonly;
Readonly::Array my @aCountryIDs => qw( 528 530 533 ); # Netherlands (=528), Netherlands Antilles (=530), Aruba (=533)

use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $hValues = $Package->getAttributes( $Object, $aNames );
# example      # never call getAttributes directly!
#               my $Value = $Object->get( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Returns attribute values of an object.
#               Used by <function DE_EPAGES::Object::Object::Object::get>
#----------------------------------------------------------------------------------------
# input        $Object | a Basket object | DE_EPAGES::Order::API::Object::Basket
# input        $aNames | list of attribute names | ref.array
# return       $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ($Object, $aNames) = @_;
    my $hInfo = {};
    foreach my $Name ( @$aNames ) {
        if( $Name eq 'IsBankCodeMandatory' ) {
            $hInfo->{$Name} = _IsBankCodeMandatory( $Object );
        }
    }
    return $hInfo;
}

#========================================================================================
# function     _IsBankCodeMandatory
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $IsMandatory = _IsBankCodeMandatory( $Object );
# example      my $IsMandatory = _IsBankCodeMandatory( $Basket );
#----------------------------------------------------------------------------------------
# description  Determines if the bank code is a mandatory field, because it is not
#               necessary to enter the bank code in the Netherlands in order to perform
#               a direct debit. Hence, the bank code is no longer mandatory if the
#               country of the invoice address is "Netherlands" (cf. EPG-6785).
#               <br>
#               The following countries are affected:
#               <ul>
#                 <li>Netherlands Antilles</li>
#                 <li>Netherlands</li>
#                 <li>Aruba</li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        $Object | a Basket object | DE_EPAGES::Order::API::Object::Basket
# return       $IsMandatory | true if the bank code is mandatory | boolean
#========================================================================================
sub _IsBankCodeMandatory {
    my ($Basket) = @_;

    my $BillingAddress = $Basket->get('BillingAddress');
    my $ShopAddress = $Basket->get('ShopAddress');
    return 1 unless ( defined $BillingAddress && defined $ShopAddress );

    my $ShopCountryID = $ShopAddress->get('CountryID');
    my $BillingCountryID = $BillingAddress->get('CountryID');

    return 1 unless ( defined $ShopCountryID && defined $BillingCountryID );
    return ( ElementOf($ShopCountryID, @aCountryIDs) and
             ElementOf($BillingCountryID, @aCountryIDs) ) ? 0 : 1;
}

1;
