#======================================================================================================================
# package      DE_EPAGES::Presentation::API::LoginChecker
# state        public
# description  The LoginChecker class is a trigger dispatcher for the Login action:
#                   LoginAttempt        A login is started
#                   LoginFailure        A login attempt has failed
#                   LoginSucessful      A login has been finish succesfuly
#               The Listeners can stop the Login process in the step LoginAttempt setting:
#                       $hTriggerArguments->{'MessageResponse'}->{'CancelLogin'} = 1 ;
#               and notice the reason by, for instance:
#                       $hTriggerArguments->{'MessageResponse'}->{'Reason'} = 'LOGIN_LOCKED' ;
#======================================================================================================================
package DE_EPAGES::Presentation::API::LoginChecker;

use strict;

use base qw ( Exporter );

use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Core::API::Log qw ( GetLog );

our @EXPORT_OK = qw (
    NewLoginChecker
) ;

#======================================================================================================================
# function     NewLoginChecker
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $LoginChecker = NewLoginChecker( $Servlet ) ;
#----------------------------------------------------------------------------------------------------------------------
# description  Function responsible to create and return a new LoginChecker object.
#----------------------------------------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#----------------------------------------------------------------------------------------------------------------------
# return       $LoginChecker | LoginChecker object | object
#======================================================================================================================
sub NewLoginChecker {
    my ( $Servlet ) = @_ ;

    return DE_EPAGES::Presentation::API::LoginChecker->new( $Servlet ) ;
}

#======================================================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $LoginChecker = new( $Servlet ) ;
#----------------------------------------------------------------------------------------------------------------------
# description  LoginChecker constructor.
#----------------------------------------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#----------------------------------------------------------------------------------------------------------------------
# return       $LoginChecker | LoginChecker object | object
#======================================================================================================================
sub new {
    my $class = shift ;
    my ( $Servlet ) = @_ ;

    my $hAttributes = {
        'Servlet'       => $Servlet,
        'LoginInfo'     => {
            'Store'         => LoadRootObject->get('StoreName'),
            'Shop'          => $Servlet->site->alias,
            'Login'         => $Servlet->form->value( 'Login' ),
            'Password'      => $Servlet->form->value( 'Password' ),
            'RemoteAddress' => $Servlet->{'ServerVariables'}->{'REMOTE_ADDR'},
            'ViewAction'    => $Servlet->params( 'ViewAction' )
        }
    } ;

    return bless( $hAttributes,  $class ) ;
}

#======================================================================================================================
# function     setUser
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $LoginChecker->setUser( $User ) ;
#----------------------------------------------------------------------------------------------------------------------
# description  Setter method for the successfuly logged user
#----------------------------------------------------------------------------------------------------------------------
# input        $User | User currently logged | object
#======================================================================================================================
sub setUser {
    my $self = shift ;
    my ( $User ) = @_ ;

    GetLog('LoginChecker')->debug( 'LoginChecker.setUser('.$User->alias.')' ) ;

    $self->{'User'} = $User ;

    return ;
}

#======================================================================================================================
# function     startLoginAttempt
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $LoginChecker->startLoginAttempt() ;
#----------------------------------------------------------------------------------------------------------------------
# description  Notify to Listeners a Login is started.
#----------------------------------------------------------------------------------------------------------------------
# return       $hMessageResponse | Hash with the trigger listeners response. If a Listener wants to stop the Login
#                   process it should use the attribute 'CancelLogin' in this hash:
#                   <ul>
#                       <li> CancelLogin : If is set to 1 the Login attempt will be stopped (default: 0)</li>
#                       <li> Reason : Why the Login must be stopped.</li>
#                   </ul>
#                   | hash.ref
#======================================================================================================================
sub startLoginAttempt {
    my $self = shift ;

    GetLog('LoginChecker')->debug( 'LoginChecker.startLoginAttempt' ) ;

    return $self->_notifyListeners({ 'Message' => 'LoginAttemptStart' }) ;
}

#======================================================================================================================
# function     triggerLoginFailure
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $LoginChecker->triggerLoginFailure( $FailReason );
#----------------------------------------------------------------------------------------------------------------------
# description  Notify to Listeners the Login attempt has failed.
#----------------------------------------------------------------------------------------------------------------------
# input        $FailReason | The code for the failure passed to the Listeners | string
#======================================================================================================================
sub triggerLoginFailure {
    my $self = shift ;
    my ( $FailReason ) = @_ ;

    GetLog('LoginChecker')->debug( 'LoginChecker.triggerLoginFailure' ) ;

    $self->_notifyListeners({
        'Message' => 'LoginAttemptFailure',
        'Reason'  => $FailReason
    }) ;

    return ;
}

#======================================================================================================================
# function     triggerLoginSuccessful
# state        public
#----------------------------------------------------------------------------------------------------------------------
# syntax       $LoginChecker->triggerLoginSuccessful();
#----------------------------------------------------------------------------------------------------------------------
# description  Notify to Listeners the Login attempt has finisehd successfuly.
#======================================================================================================================
sub triggerLoginSuccessful {
    my $self = shift ;

    GetLog('LoginChecker')->debug( 'LoginChecker.triggerLoginSuccessful' ) ;

    $self->_notifyListeners({ 'Message' => 'LoginAttemptSuccessful' }) ;

    return ;
}

#======================================================================================================================
# function     _notifyListeners
# state        private
#----------------------------------------------------------------------------------------------------------------------
# syntax       $LoginChecker->_notifyListeners( $hMessageInfo ) ;
#----------------------------------------------------------------------------------------------------------------------
# description  Prepares and launches the requested trigger
#----------------------------------------------------------------------------------------------------------------------
# input        $hMessageInfo | Info to launch the trigger: name and optional data | hash.ref
#----------------------------------------------------------------------------------------------------------------------
# return       $MessageResponse | The response from the trigger Listeners | hash.ref
#======================================================================================================================
sub _notifyListeners {
    my $self = shift ;
    my ( $hMessageInfo ) = @_ ;

    my $hMessageResponse = { 'CancelLogin'   => 0 } ;
    my $TriggerName = 'UI_'.$hMessageInfo->{'Message'} ;

    TriggerHook(
        $TriggerName,
        {
            'Servlet'           => $self->{'Servlet'},
            'User'              => $self->{'User'},
            'LoginInfo'         => $self->{'LoginInfo'},
            'MessageInfo'       => $hMessageInfo,
            'MessageResponse'   => $hMessageResponse
        }
    );

    return $hMessageResponse ;
}

1;
