use strict;
use Getopt::Long;
use DE_EPAGES::Core::API::CartridgeConfig qw(CartridgeConfig);
use DE_EPAGES::Core::API::File qw( GetFileNames ExistsDirectory GetFileContent fsplit ExistsFile );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Presentation::API::JSSyntaxCheck qw ( CheckJSLint CheckYUICompressor );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetStoreTypeCartridges );
use JSON qw( from_json );

sub Main {
    my($Help, $NoOutput, $StoreType, $Version, $WebRoot);

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'storetype=s' => \$StoreType
    );

    $WebRoot = $ENV{'EPAGES_WEBROOT'};

    usage('The store type is not defined.') unless $StoreType;

    $Version = CartridgeConfig($StoreType)->get('Cartridges','DE_EPAGES::Core');

    my $DojoDirectory = "$WebRoot/StoreTypes/$Version/$StoreType/javascript/build/epages";
    my $JQDirectory = "$WebRoot/StoreTypes/$Version/$StoreType/GUI";

    my @Cartridges = @{ GetStoreTypeCartridges( $StoreType ) };

    my $PackedFiles = [];
    my $OtherFiles = [];

    my $hJQPackages;
    my $hDojoPackages;

    my $Error = 0;
    my @FilesWithError;

    # Find public javascript files
    foreach my $Cartridge (@Cartridges) {
        my $PublicFiles =_ComposePublicFiles($Cartridge);
        foreach my $File ( @{$PublicFiles} ) {
            push @{$OtherFiles}, "$WebRoot/StoreTypes/$Version/$StoreType/$File";
        }
    }

    # Search for jquery profile files
    my @JQProfiles = map { @{_CartidgeProfiles($_,'JQueryBuild')} } @Cartridges;
    foreach my $ProfileFile (@JQProfiles) {
        my @Modules = @{from_json(${GetFileContent($ProfileFile)})};
        my (undef, $MinName, undef) = fsplit($ProfileFile);
        foreach my $Module (@Modules) {
            push(@{$hJQPackages->{$MinName}}, $Module) unless
                ElementOf($Module, $hJQPackages->{$MinName});
        }
    }

    # Extract the jquery package files from the profiles
    foreach my $Package (keys %$hJQPackages) {
        my $hModules = $hJQPackages->{$Package};
        $PackedFiles = _ComposeJQPackage($hModules, $JQDirectory, $PackedFiles);
    }

    # Search for dojo profile files
    my @DojoProfiles = map { @{_CartidgeProfiles($_,'DojoBuild')} } @Cartridges;
    foreach my $ProfileFile (@DojoProfiles) {
        my $Data= from_json(${GetFileContent($ProfileFile)});
        my $Name = $Data->{'name'};
        push(@{$hDojoPackages->{$Name}}, $Name) unless
            ElementOf($Name, $hDojoPackages->{$Name});

    }

    # Extract the dojo package files from the profiles
    foreach my $Package (keys %$hDojoPackages) {
        my $hModules = $hDojoPackages->{$Package};
        $PackedFiles = _ComposeDojoPackage($hModules, $DojoDirectory, $PackedFiles);
    }

    # Do a syntax check for each package file
    foreach my $JSFile (@$PackedFiles) {
        next unless $JSFile =~ /\.js$/;
        if(!ExistsFile($JSFile)) {
            $Error = 1;
            print "$JSFile seems to be missing\n";
            push @FilesWithError, $JSFile;
            next;
        }
        my ($JSLintOK, $JSLintOutput) = CheckJSLint($JSFile);
        my ($YUIOK, $YUIOutput) = CheckYUICompressor($JSFile);

        print $JSLintOutput unless $JSLintOK;
        print $YUIOutput unless $YUIOK;

        if(!$JSLintOK || !$YUIOK) {
            $Error = 1;
            push @FilesWithError, $JSFile;
        }

        if( !$NoOutput && $JSLintOK && $YUIOK ) {
            print "$JSFile seems ok.\n";
        }
    }

    # Check other files for existence
    foreach my $JSFile (@$OtherFiles) {
        next unless $JSFile =~ /\.js$/;
        if(!ExistsFile($JSFile)) {
            $Error = 1;
            print "$JSFile seems to be missing\n";
            push @FilesWithError, $JSFile;
            next;
        }
        print "$JSFile seems ok.\n" unless $NoOutput;
    }

    if($Error) {
        print "There was an error while checking the javascript files. Have a look at this file(s):\n";
        foreach my $FileWithError (@FilesWithError) {
            print "$FileWithError\n";
        }
    }else {
        print "Everything seems ok.\n" unless $NoOutput;
    }
    return $Error;
}

sub _ComposeJQPackage {
    my ($aModules, $Dir, $aFiles) = @_;

    foreach my $JSModuleName (@$aModules) {
        my $SlashedJSModuleName = $JSModuleName;
        $SlashedJSModuleName =~ s/\./\//g;
        my $JSModuleFilePath = $Dir . '/'. $SlashedJSModuleName . '.js';
        push @$aFiles, $JSModuleFilePath
    }

    return $aFiles;
}

sub _ComposeDojoPackage {
    my ($aModules, $Dir, $aFiles) = @_;

    foreach my $JSModuleName (@$aModules) {
        my $SlashedJSModuleName = $JSModuleName;
        my $JSModuleFilePath = $Dir . '/'. $SlashedJSModuleName;
        push @$aFiles, $JSModuleFilePath
    }

    return $aFiles;
}

sub _ComposePublicFiles{
    my ($Package) = @_;
    my $Path = $ENV{'EPAGES_CARTRIDGES'}.'/'._CartridgeDir($Package).'/Data/Public';
    return [] unless ExistsDirectory($Path);
    return [
        grep {/\.js$/}
        @{GetFileNames($Path, {'recursive' => 1, 'relative' => 1 })}
    ];
}


sub _CartidgeProfiles {
    my ($Package, $Target) = @_;

    my $ProfilesPath = $ENV{'EPAGES_CARTRIDGES'}.'/'._CartridgeDir($Package).'/Data/'.$Target;
    return [] unless ExistsDirectory($ProfilesPath);

    return [
        grep {/\.profile\.js$/}
        @{GetFileNames($ProfilesPath)}
    ];
}

sub _CartridgeDir {
    my ($Package) = @_;

    $Package =~ s/::/\//g;
    return $Package;
}

sub usage {
    my ($Msg) = @_;
    print "$Msg\n\n" if $Msg;
    print <<END_USAGE;
Usage:
    perl $0 [flags]

Description:
    The Script uses jslint and yuicompressor to check if all packed javascript files of the
    installation are ok. Additionally there is a check if all public js files exists.

Options:
    -storetype  type of store, if set to "all" all store types will be installed (required)

Flags:
    -help           show the command line options           (optional)
    -nooutput       suppress all output for the script      (optional)

Examples:
    perl $0 -storetype Store

END_USAGE
exit 1;
}

RunScript(
    'Sub' => \&Main
);

1;
