use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use Cwd 'abs_path';
use Config;
use Getopt::Long;
use DE_EPAGES::Core::API::File qw ( MoveFile WriteFile ExistsFile fsplit RelativePath
                                    ExistsDirectory GetFileNames GetFileContentLines
                                    GetFileContent CorrectPathSlashes DeleteFile );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf GenerateShortGUID trim );
use DE_EPAGES::Presentation::API::JSSyntaxCheck qw ( CheckImpliedGlobals
    CheckObjectCTORCalls CheckCommentedEvils CheckJQueryEvents CheckAttributeModifiers
    CheckForLoops CheckWithAlert CheckConsoleDebugLog CheckStrictCompareUndef
    CheckCallPrivateFunctions CheckTranslationList CheckInnerHtmlEmpty
    CheckJSLint CheckYUICompressor );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetCartridges );

use constant BLACKLIST_FILENAME => 'checkJSSyntax_blacklist.txt';

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'CHECK_IMPLIED_GLOBALS_FAILED'          => 'Implied globals found',
        'CHECK_OBJECT_CTOR_CALLS_FAILED'        => 'Object constructor "new" call found',
        'CHECK_CONSOLE_DEBUG_LOG_FAILED'        => 'Statement console.debug or console.log found',
        'CHECK_COMMENTED_EVILS_FAILED'          => 'Comment needed for evil functions found',
        'CHECK_JQUERY_EVENTS_FAILED'            => 'Better use "bind"/"trigger" for jQuery connects',
        'CHECK_ATTRIBUTE_MODIFIERS_FAILED'      => 'Better use $().attr and $().removeAttr', ## no critic qw(RequireInterpolationOfMetachars)
        'CHECK_WITH_ALERT_FAILED'               => 'Statement "with" or "alert" found',
        'CHECK_FOR_LOOPS_FAILED'                => 'Better store length into variable in for-loops',
        'CHECK_STRICT_COMPARE_UNDEF_FAILED'     => 'Better use strict compare for undefined',
        'CHECK_CALL_PRIVATE_FUNCTIONS_FAILED'   => 'Don\'t call foreign private methods',
        'CHECK_INNERHTML_EMPTY_FAILED'          => 'Don\'t use innerHTML for comparisons against empty',
        'CHECK_TRANSLATION_LIST_FAILED'         => 'Language tag declaration missing',
    }
}
#ERRORDEFINITION END

sub Main {
    my ($Help, $Type, $LogFile, $LogWarnings, $OnlyThisFile, $EpagesDojoFilesOnly, $JQueryFilesOnly);

    GetOptions(
        'help'               => \$Help,
        'type=s'             => \$Type,
        'logfile=s'          => \$LogFile,
        'warnings'           => \$LogWarnings,
        'file=s'             => \$OnlyThisFile,
        'epagesdojoonly'     => \$EpagesDojoFilesOnly,
        'jqueryonly'         => \$JQueryFilesOnly,
    );
    usage() if $Help;

    my $ModuleName = abs_path($0);
    my $JSLintConfigFile = $ModuleName;
    $JSLintConfigFile =~ s/pl/conf/ig;
    die('Error: Configuration file ('.$JSLintConfigFile.') not found! Abort.') if !ExistsFile($JSLintConfigFile);

    my $EpagesCartridges = $ENV{'EPAGES_CARTRIDGES'};
    my $ErrorOccured = 0;

    # default: all files in all cartridge-data dirs
    if(!$OnlyThisFile) {
        usage() if not $Type;
        # get list of cartridges
        my $aCartridges = GetCartridges();
        my $FoundFiles = 0;
        my $TotalOutput = '';

        my ($BlackListFilePath, undef, undef) = fsplit($ModuleName);
        my @BlackListFiles = map {
            CorrectPathSlashes($_)
        } trim( @{ GetFileContentLines( $BlackListFilePath . '/' . BLACKLIST_FILENAME ) });

        # for all cartridges
        foreach my $Package ( @{$aCartridges->{$Type}} ){
            my $CartridgeName = $Package;
            $CartridgeName =~ s/DE_EPAGES|:://g;

            $Package =~ s/::/\//g;

            # path
            my $AbsCartridgePath = CorrectPathSlashes(join('/', $EpagesCartridges, $Package, 'Data'));
            next unless ExistsDirectory($AbsCartridgePath);

            my @AllJSFiles = ();
            if(defined $EpagesDojoFilesOnly) {
                @AllJSFiles = grep {
                    $_ =~ /\.js$/ and $_ !~ /(?:.*)javascript[\\\/](?:dojox|dijit|dojo)(?:.*)\.js$/x
                } @{GetFileNames($AbsCartridgePath,{'recursive' => 1})};
            }
            elsif(defined $JQueryFilesOnly) {
                @AllJSFiles = grep {
                    $_ =~ /\.js$/ and $_ =~ /(?:.*)Public[\\\/]GUI[\\\/](?:ep|de_epages)[\\\/](?:.*)\.js$/ix
                } @{GetFileNames($AbsCartridgePath,{'recursive' => 1})};
            }
            else {
                @AllJSFiles = grep {/\.js$/} @{GetFileNames($AbsCartridgePath,{'recursive' => 1})};
            }

            next if scalar(@AllJSFiles) == 0;

            # sort out blacklisted
            @AllJSFiles = @{FilterBlackListed(\@AllJSFiles, \@BlackListFiles, $EpagesCartridges)};

            foreach my $JSFile (map { CorrectPathSlashes($_) } @AllJSFiles) {
                my ($FileHasErrors, $FileOutput) = CheckFile($JSFile, $LogWarnings, $JSLintConfigFile);

                if($FileHasErrors && $FileOutput) {
                    $TotalOutput .= $FileOutput;
                    $FoundFiles++;
                }
            }
        }

        if($FoundFiles > 0) {
            if($LogFile ne '') {
                MoveFile($LogFile, $LogFile.'_'.GenerateShortGUID())  if ExistsFile($LogFile);  # old log file
                WriteFile($LogFile, \$TotalOutput, { append=>0, binary=>0, create_directory=>1} );
            } else {
                print $TotalOutput;
            }
            $ErrorOccured = 1;
        }
    } elsif($OnlyThisFile && $OnlyThisFile ne '') {
        #only the given file
        $OnlyThisFile = CorrectPathSlashes($OnlyThisFile);
        my $DisplayFile = $OnlyThisFile; # for print output
        if($OnlyThisFile =~ /\.html$/) {
            my $TemplateString = ${GetFileContent($OnlyThisFile)};

            # filter contents if template script tags
            return if $TemplateString !~ /<script/x;
            my @JSLines;
            while($TemplateString =~ /<script.*?>(.*?)<\/script>/xgs) {
                push(@JSLines, $1);
            }
            $OnlyThisFile = $OnlyThisFile . '_' . GenerateShortGUID();
            $TemplateString = join("\n", @JSLines);
            WriteFile($OnlyThisFile, \$TemplateString, , { utf8 => 1 });

            # suggest to combine javascript blocks
            if(scalar(@JSLines) > 1) {
                print "Try to combine JavaScript blocks in file $DisplayFile\n";
                $ErrorOccured = 1;
            }
        }

        my ($FileHasErrors, $FileOutput) = CheckFile($OnlyThisFile, $LogWarnings, $JSLintConfigFile);

        if($FileHasErrors && $FileOutput) {
            $ErrorOccured = 1;
            print $FileOutput;
        }

        DeleteFile($OnlyThisFile) if $DisplayFile =~ /\.html$/;
    }

    exit $ErrorOccured;
}

sub FilterBlackListed {
    my ($aAllJSFiles, $aBlackListFiles, $EpagesCartridges) = @_;

    my @NonBlacklistedFiles;
    foreach my $File (@$aAllJSFiles) {
        my ($Path, $FileName, $Ext) = fsplit($File);
        my $PathFromCartridgeDir = RelativePath($EpagesCartridges, $Path);

        next if ElementOf(CorrectPathSlashes($PathFromCartridgeDir.'/'.$FileName.$Ext),
                          @$aBlackListFiles);
        push(@NonBlacklistedFiles, $File);
    }
    return \@NonBlacklistedFiles;
}

sub GetErrStr {
    my ($Msg, $File, $Idx) = @_;
    my $Message = ErrorDefinition->{$Msg} . ", in file $File";
    $Message .= ', line ' . ($Idx+1) if $Idx;
    return "$Message\n";
}

sub PrintError {
    my ($Msg, $File, $Idx) = @_;
    print GetErrStr($Msg, $File, $Idx);
    return;
}

sub CheckFile {
    my ($JSFile, $LogWarnings, $JSLintConfigFile) = @_;
    my $FileOutput = '';
    my $FoundErrInFile = 0;

    my ($JSLintOK, $JSLintOutput) = CheckJSLint($JSFile, $LogWarnings, $JSLintConfigFile);
    if(!$JSLintOK) {
        $FileOutput .= $JSLintOutput;
        $FoundErrInFile = 1;
    }

    my ($YUIOK, $YUIOutput) = CheckYUICompressor($JSFile);
    if(!$YUIOK) {
        $FileOutput .= $YUIOutput;
        $FoundErrInFile = 1;
    }

    my @ContentLines = @{GetFileContentLines($JSFile)};
    my $ContentString = join("\n", @ContentLines);

    my $EscapedFromCheck = 0;
    my $Comment = 0;
    for(my $i=0; $i<scalar(@ContentLines); $i++) {
        if($ContentLines[$i] =~ m/\/\/\s*\#\s*CRITIC_IF\s+false/x){
            $EscapedFromCheck = 1;
        } elsif($ContentLines[$i] =~ m/\/\/\s*\#\s*CRITIC_ENDIF/x) {
            $EscapedFromCheck = 0;
        }
        if($ContentLines[$i] =~ m/\/\*/x){
            $Comment = 1;
        } elsif($ContentLines[$i] =~ m/\*\//x) {
            $Comment = 0;
        }
        next if $EscapedFromCheck || $Comment;
        my $Line = $ContentLines[$i];
        $Line =~ s/\/\/.*//x;
        my $LineBefore = $i > 0 ? $ContentLines[$i-1] : '';

        if(!CheckImpliedGlobals($Line)) {
            $FileOutput .= GetErrStr('CHECK_IMPLIED_GLOBALS_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckObjectCTORCalls($Line)) {
            $FileOutput .= GetErrStr('CHECK_OBJECT_CTOR_CALLS_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckConsoleDebugLog($Line)) {
            $FileOutput .= GetErrStr('CHECK_CONSOLE_DEBUG_LOG_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckCommentedEvils($Line, $LineBefore)) {
            $FileOutput .= GetErrStr('CHECK_COMMENTED_EVILS_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckJQueryEvents($Line)) {
            $FileOutput .= GetErrStr('CHECK_JQUERY_EVENTS_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckAttributeModifiers($Line)) {
            $FileOutput .= GetErrStr('CHECK_ATTRIBUTE_MODIFIERS_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckWithAlert($Line)) {
            $FileOutput .= GetErrStr('CHECK_WITH_ALERT_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckForLoops($Line)) {
            $FileOutput .= GetErrStr('CHECK_FOR_LOOPS_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckStrictCompareUndef($Line)) {
            $FileOutput .= GetErrStr('CHECK_STRICT_COMPARE_UNDEF_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckCallPrivateFunctions($Line)) {
            $FileOutput .= GetErrStr('CHECK_CALL_PRIVATE_FUNCTIONS_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
        if(!CheckInnerHtmlEmpty($Line)) {
            $FileOutput .= GetErrStr('CHECK_INNERHTML_EMPTY_FAILED', $JSFile, $i);
            $FoundErrInFile = 1;
        }
    }
    if(!CheckTranslationList($ContentString)) {
        $FileOutput .= GetErrStr('CHECK_TRANSLATION_LIST_FAILED', $JSFile);
        $FoundErrInFile = 1;
    }

    return ($FoundErrInFile, $FileOutput);
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Description:
    Check Syntax for all javascript files. Uses alternative for JSLint from
    http://www.javascriptlint.com.

Options:
    -type               database type                               (required)
    -file               do not check all files but only this one    (optional)
    -logfile            full file path and name to the logfile      (optional)


Flags:
    -help               show the command line options               (optional)
    -warnings           list warnings in logfile too                (optional)
    -epagesdojoonly     only check epages dojo javascript files     (optional)
    -jqueryonly         only check jquery javascript files          (optional)

Examples:
    perl $0 -type Store
    perl $0 -type Store -jqueryonly -warnings
    perl $0 -type Store -warnings -file mymodule.js

END_USAGE
exit 1;
}

RunScript(
    'Sub' => \&Main
);
