#========================================================================================
# package      DE_EPAGES::Presentation::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to show, save and delete objects.
#========================================================================================
package DE_EPAGES::Presentation::UI::JSON;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError);
use DE_EPAGES::Core::API::TestParameter qw (TestInteger);
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Alias Unicode2Filename Unicode2DirectoryName Unicode2Path );
use DE_EPAGES::Core::API::Url qw (UrlEncodeValue);
use DE_EPAGES::Object::API::Factory qw (ExistsObject);
use DE_EPAGES::Object::API::Language qw (GetCodeByLanguageID);
use DE_EPAGES::Permission::API::Permission qw (ActionByName);
use DE_EPAGES::Presentation::API::Constants qw (BASIC_JSON_ATTRIBUTES
                                                MAX_PREVIEW_CHILD_ITEMS
                                                JQUERY_JSON_ATTRIBUTES
                                                JQUERY_JSON_IGNORED_CLASSES
                                                UNITY_VIEW_ACTION_EXCEPTION_REGEX);
use DE_EPAGES::Presentation::API::DereferrerToken qw (GenerateDereferrerToken);
use DE_EPAGES::Presentation::API::Object::ObjectPageType;
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByViewAction);
use DE_EPAGES::Presentation::API::Template qw (ProcessPage);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Presentation::API::Object::Tray;
use DE_EPAGES::Presentation::API::Object qw ( CollectJSONLayoutInfomation );
use DE_EPAGES::Presentation::API::Table::EventFavorite qw (
    InsertEventFavorite
    GetPKeyEventFavoritesByUserID
    GetInfoEventFavorite
);

use constant UNKNOWN_ERROR => 599;
#========================================================================================
# function     writeContentOutput
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->writeContentOutput( $rContent );
# example      my $Object = $Servlet->object;
#               return $self->writeContentOutput($Object->get([qw (Alias NameOrAlias ObjectID Position)]));
#----------------------------------------------------------------------------------------
# description  Writes the json content direct to the servlet.
#----------------------------------------------------------------------------------------
# input        $rContent | data | ref
#========================================================================================
sub writeContentOutput {
    my $self = shift;
    my ($rContent) = @_;
    my $Servlet = $self->servlet;
    $Servlet->writeContentOutput($rContent);

    if(defined $Servlet->params('handleashtml')) {
        $Servlet->vars('ContentType', 'text/html');
        $Servlet->response->header('Content-Type','text/html');
    }

    return;
}

#========================================================================================
# function     JSONViewResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONViewResponse( $Servlet );
#----------------------------------------------------------------------------------------
# description  Writes servlet vars ResponseErrors and ResponseContent to servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONViewResponse {
    my $self = shift;
    my ($Servlet) = @_;

    # ChangeActions write response
    my $rContent = $Servlet->vars('ResponseContent');
    $rContent = {} unless defined $rContent;

    $self->writeContentOutput($rContent);
}

#========================================================================================
# function     JSONObjectExists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONObjectExists( $Servlet );
#----------------------------------------------------------------------------------------
# description  Check if an object id exists and writes ResponseContent to servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONObjectExists {
    my $self = shift;
    my ($Servlet) = @_;

    my $CheckObjectID = $Servlet->params('CheckObjectID');

    TestInteger('CheckObjectID', $CheckObjectID);

    # ChangeActions write response
    my $rContent = $Servlet->vars('ResponseContent');
    $rContent = {} unless defined $rContent;

    $rContent->{'ObjectExists'} = ExistsObject( $CheckObjectID ) ? 1 : 0;

    $self->writeContentOutput($rContent);
}

#========================================================================================
# function     JSONAjaxTransportFileError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONAjaxTransportFileError( $Servlet );
#----------------------------------------------------------------------------------------
# description  Error handle for ajaxTransport of files.
#               Writes servlet vars ResponseError to servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONAjaxTransportFileError {
    my $self = shift;
    my ($Servlet) = @_;
    my $aFormRealErrors = $Servlet->vars('FormRealErrors') //[];

    # map of errors for ajaxTransport
    my %ErrorMap = (
        'FONT_LICENSE_RESTRICTED'         => 488,
        'INVALID_IMAGE_SIZE'              => 489,
        'UNSUPPORTED_FILE_FORMAT'         => 493,
        'UNSUPPORTED_IMAGE_FORMAT'        => 493,
        'IMAGE_MAGICK'                    => 493,
        'MIMETYPE_NOT_ACCEPT_ERROR'       => 493,
        'FileSizeLimitError'              => 494,
        'FeatureLimitExceeded'            => 495,
        'INVALID_IMAGE_FILE'              => 496,
        'IMAGE_EXCEEDS_MEGAPIXELS'        => 497,
        'InvalidMP3File'                  => 498,
        'VIRUS_FOUND'                     => 499,
        'UNKNOWN_ERROR'                   => 500,
    );

    # build Error structure
    my $errorThrown = scalar @{$aFormRealErrors} ? $aFormRealErrors->[0]->{Reason} : 'Unknown Error';
    my $status = $ErrorMap{ $errorThrown } ? $ErrorMap{ $errorThrown } : UNKNOWN_ERROR;
    my %ERROR = (
        status      => $status,
        errorThrown => $errorThrown,
    );
    my %ResponseError = (
        ERROR => \%ERROR,
    );

    $self->writeContentOutput(\%ResponseError);
}

#========================================================================================
# function     JSONGet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONGet( $Servlet );
#----------------------------------------------------------------------------------------
# description  Reads given comma separated 'Attributes' from servlet object.
#               Possible syntax:
#                   Attribute
#                       reads and responds as attribute "Attribute"
#                   ReturnName=AttributeName
#                       reads attribute "AttributeName" but responds it as "ReturnName"
#                   ReturnName=Attribute.SubAttribute
#                       reads attribute "SubAttribute" from object attribute "Attribute"
#                       and returns it as "ReturnName"
#               Writes the JSON content directly to the servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONGet {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;

    my @AttributeNames = split(/,/, $Servlet->params('Attributes'));
    my $LanguageID = $Servlet->form->value('LanguageID');

    my $Attributes = {};
    foreach my $Attribute (@AttributeNames) {
        my ($ReturnName, $AttributeName) = split(/=/, $Attribute);
        $AttributeName = $ReturnName unless $AttributeName;
        my @AttributeNameParts = split(/\./, $AttributeName);
        my $Value = $Object;
        foreach my $AttributeNamePart (@AttributeNameParts) {
            if (ref $Value eq '') {
              # nothing to do
            } elsif (ref $Value eq 'HASH') {
                if (not defined $Attribute->{$AttributeNamePart}) {
                    $Value = undef;
                    last;
                }
                $Value = $Value->{$AttributeNamePart};
            } elsif (blessed $Value and $Value->can('get') and $Value->class->existsAttribute($AttributeNamePart)) {
                $Value = $Value->get($AttributeNamePart,$LanguageID);
            } else {
                warn sprintf 'cant resolve %s of %s.',$AttributeNamePart,$Attribute;
            }
        }
        $Attributes->{$ReturnName} = $Value;
    }
    $self->writeContentOutput($Attributes);
}

#========================================================================================
# function     JSONGetTLE
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONGetTLE( $Servlet );
#----------------------------------------------------------------------------------------
# description  Reads given comma separated 'Attributes' from servlet object. Uses
#               tleHash instead of get.
#               Needed parameters:
#                   LanguageID : language id to get localized values
#                   Attribute  : reads and responds as attribute "Attribute"
#
#               Writes the JSON content directly to the servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONGetTLE {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;

    my $hValues = {};
    my @AttributeNames = split(/,/, $Servlet->params('Attributes'));
    my $LanguageID = $Servlet->form->value('LanguageID');

    # Temporarily set desired language id to global LANGUAGEID var for tleHash function
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;
    foreach my $Attribute (@AttributeNames) {
        $hValues->{$Attribute} = $Object->tleHash->{$Attribute};
    }
    $self->writeContentOutput($hValues);
}

#========================================================================================
# function     JSONGetFeature
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONGetFeature( $Servlet );
#----------------------------------------------------------------------------------------
# description  Reads given 'Features' from servlet object.
#               Writes the json content direct to the servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONGetFeature {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    my @FeatureNames = split(/,/, $Servlet->params('Features'));
    my @Features = map {
                          { 'current' => $Shop->featureCurrentValue($_),
                            'max' => $Shop->featureMaxValue($_)}
                        } @FeatureNames;
    my %rContent;
       @rContent{@FeatureNames} = @Features;

    $self->writeContentOutput(\%rContent);
}

#========================================================================================
# function     JSONChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONChildren( $Servlet );
#----------------------------------------------------------------------------------------
# description  Reads children of servlet object.
#               <function DE_EPAGES::Presentation::API::Constants::BASIC_JSON_ATTRIBUTES> and
#               'hasChildren' are the keys for each child.
#               Writes the json content direct to the servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONChildren {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object = $Servlet->object;
    my $Site = $Object->getSite();
    my $LanguageID = $Servlet->user->get('LanguageID') // $Site->get('LanguageID');
    my @Path;

    while ($Object->id != $Site->id) {
        my $hInfo = $Object->get(BASIC_JSON_ATTRIBUTES);
        $hInfo->{'ParentID'} = $Object->parent->id;
        $hInfo->{'hasChildren'} = $Object->hasChildren;
        $hInfo->{'ClassAlias'} = $Object->class->alias;
        $hInfo->{'ClassID'}   = $Object->class->id;
        my $ObjectID = $Object->id;
        $hInfo->{'children'} = [];

        my $totalNumber = scalar @{ $Object->findChildren };
        if($totalNumber < MAX_PREVIEW_CHILD_ITEMS){
            foreach my $Child (@{$Object->children}) {
                my $hChild = $Child->get(BASIC_JSON_ATTRIBUTES);
                $hChild->{'ParentID'} = $ObjectID;
                $hChild->{'hasChildren'} = $Child->hasChildren;
                $hChild->{'ClassAlias'} = $Child->class->alias;
                $hChild->{'ClassID'} = $Child->class->id;
                push @{$hInfo->{'children'}}, $hChild;
            }

            TriggerHook('UI_JSONChildren', {'Children' => $hInfo->{'children'}, 'Object'=>$Object, 'LanguageID' => $LanguageID});
            TriggerHook('UI_JSONChildren', {'Children' => [$hInfo], 'Object'=>$Object->parent, 'LanguageID' => $LanguageID});
        }
        else{
            #this value tells the client that there are to much children to show - jsError not serverError
            push @{$hInfo->{'error'}}, 'ERROR_TOO_MANY_CHILDREN';
        }
        unshift @Path, $hInfo;
        $Object = $Servlet->params('WithPath') ? $Object->parent : $Site;
    }
    $self->writeContentOutput({path=>\@Path});
}

#========================================================================================
# function     JSONChildrenJSTree
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONChildrenJSTree( $Servlet );
#----------------------------------------------------------------------------------------
# description  Reads children of servlet object.
#               <function DE_EPAGES::Presentation::API::Constants::BASIC_JSON_ATTRIBUTES> and
#               'hasChildren' are the keys for each child.
#               Writes the json content direct to the servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONChildrenJSTree {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my @Data = ();
    my $totalNumber = scalar @{ $Object->findChildren } || 0;
    my $hInfo = {
        'data' => {
            'title'      => $Object->get('NameOrAlias'),
            'icon'       => $Object->class->alias,
        },
        'metadata' => {
            'id'         => $Object->id,
            'childCount' => $totalNumber,
        }
    };
    $hInfo->{'metadata'}->{'parentId'} = $Object->parent->id if defined $Object->parent;
    $hInfo->{'metadata'}  = {%{$hInfo->{'metadata'}}, %{$Object->get(JQUERY_JSON_ATTRIBUTES)}};


    if($totalNumber > 0){
        $hInfo->{'children'} = [];
        $hInfo->{'state'} = 'open';
    }

    my $ObjectID = $Object->id;

    if($totalNumber < MAX_PREVIEW_CHILD_ITEMS){
        foreach my $Child (@{$Object->children}) {
            my $hChild = {
                'data' => {
                    'title'      => $Child->get('NameOrAlias'),
                    'icon'       => $Child->class->alias,
                },
                'metadata' => {
                    'id'         => $Child->id,
                    'parentId'   => $ObjectID,
                    'type'       => $Child->class->alias,

                }
            };
            $hChild->{'metadata'}  = {%{$hChild->{'metadata'}}, %{$Child->get(JQUERY_JSON_ATTRIBUTES)}};

            # do not display child items for objects of JQUERY_JSON_IGNORED_CLASSES classes
            if ( $Child->hasChildren and
                 not ElementOf($Child->class->alias, JQUERY_JSON_IGNORED_CLASSES) ) {
                $hChild->{'state'} = 'closed';
            }
            push @{$hInfo->{'children'}}, $hChild;
        }
        TriggerHook('UI_JSONChildrenJSTree', {'Info' => $hInfo, 'Object'=>$Object});
    }
    else{
        #this value tells the client that there are to much children to show - jsError not serverError
        push @{$hInfo->{'error'}}, 'ERROR_TOO_MANY_CHILDREN';
    }
    if($Servlet->params('WithParent')){
        push @Data, $hInfo;
    }
    else{
        push @Data, $hInfo->{'children'};
    }
    $self->writeContentOutput(\@Data);
}

#========================================================================================
# function     JSONPublicFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONPublicFiles( $Servlet );
#----------------------------------------------------------------------------------------
# description  Reads public files of servlet object.
#               Writes the json content direct to the servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONPublicFiles {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my @FileNames = map { $_->{'FileName'} } @{$Object->get('PublicFiles')};

    $self->writeContentOutput(\@FileNames);
}

#========================================================================================
# function     JSONGetBaseUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONGetBaseUrl( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns the current BaseUrl (baseLineContentLocation) for the currently
#               logged in user (with security token).
#               Writes the json content direct to the servlet.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONGetBaseUrl {
    my $self = shift;
    my ($Servlet) = @_;

    $self->writeContentOutput( {'baseurl' => $Servlet->baseLineContentLocation} );
}

sub JSONSnippet {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my $ViewAction = $Servlet->params('TemplateAction');
    my $LanguageID = $Servlet->params('TemplateLanguageID');
    my $TemplateName = $Servlet->params('TemplateName');



    # test if we have an admin request and a unity shop, then rewrite viewaction to Unity-$Action
    my $Site = $Object->getSite;
    my $IsUnity = ($Site->class->existsAttribute('IsUnityRegistered') and $Site->get('IsUnityRegistered')) ? 1 : 0;

    if ($IsUnity and not $ViewAction =~ UNITY_VIEW_ACTION_EXCEPTION_REGEX) {
        # add prefix Unity to name if not exists
        $ViewAction = "Unity$ViewAction";
    }

    my $Action = ActionByName($Object->class, $ViewAction);
    # verify permission on other action, JSONSnippet is allowed to everyone
    $Servlet->testPermission($Object, $Action);

    #execute TemplateAction and fill servlet
    $Servlet->params('ExitAfterEvent', 1);
    $Servlet->vars('ViewAction', $ViewAction);
    $Servlet->vars('IsJSONSnippet', 1);
    $Servlet->vars('TemplateAction', undef);
    $Servlet->vars('TemplateName', undef);
    $Servlet->vars('TemplateLanguageID', undef);
    $Servlet->doAction($Action);

    my $PageType       = LoadPageTypeByViewAction( $Action );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Object );

    my $LanguageCode = GetCodeByLanguageID($LanguageID);

    my $DontFail = $Servlet->params('DontFail');

    my $Vars = $Object->tleHash;
    my $ServletVars = $Servlet->vars;
    $Vars->{$_} = $ServletVars->{$_} foreach keys %$ServletVars;
    $Vars->{'INPUT'} = $Servlet->params;
    $Vars->{'INPUT'}{'Language'} = $LanguageCode;
    $Vars->{'INPUT'}{'LanguageID'} = $LanguageID;
    $Vars->{'PAGETYPE'} = $PageType->tleHash;
    $Vars->{'VIEWACTION'} = $Action->tleHash;
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;

    my $hInfo = {'snippet' => undef};

    eval {
        $hInfo->{'snippet'} = ProcessPage($Object, $ObjectPageType, $LanguageCode, $Vars, undef, undef, $TemplateName);
    };
    Error(GetError()) if (not $DontFail and ExistsError());

    $self->writeContentOutput($hInfo);
}

sub JSONLayout {
    my $self = shift;
    my ($Servlet) = @_;
    my @KnownPageTypes = $Servlet->form->value('KnownPageTypes') // ();
    my $hInfo = CollectJSONLayoutInfomation($Servlet->object, \@KnownPageTypes);
    $self->writeContentOutput($hInfo);
    return;
}

sub JSONGetTray {
    my $self = shift;
    my ($Servlet) = @_;

    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    my $Groups = $Tray->groups;
    foreach my $Group(@$Groups){
        my $Objects = $Tray->objectsByGroup($Group->{'TrayGroupID'});
        $Group->{'Objects'} = [];
        foreach my $Object (@$Objects) {
            push @{$Group->{'Objects'}}, $Object->get([qw(ObjectID Alias NameOrAlias)]);
        }
        delete $Group->{'IntSessionID'};
    }
    $self->writeContentOutput($Groups);
}

sub JSONGetFavorites{
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->user;
    return unless defined $User;
    my @Favorites = ();
    foreach my $FavoriteID (@{GetPKeyEventFavoritesByUserID($User->id)}) {
        my $hEventFavorite = GetInfoEventFavorite($FavoriteID);
        delete $hEventFavorite->{'UserID'};
        push @Favorites, $hEventFavorite;
    }
    $self->writeContentOutput(\@Favorites);
}

sub JSONAddFavorite{
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->user;
    return unless defined $User;
    my $Result = InsertEventFavorite({
        UserID => $User->id,
        ObjectID => $Servlet->object->id,
        URI => $Servlet->form->value('URI'),
        Name => $Servlet->form->value('Name'),
    });
    return $self->writeContentOutput({
        EventFavoriteID => $Result
    });

}

#========================================================================================
# function     JSONAdvancedLinks
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONAdvancedLinks( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns the AdvancedLinks for the Linkcreator Widget
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONAdvancedLinks {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my @Links = ();
    TriggerHook('UI_JSONAdvancedLinks', {'Links' => \@Links, 'Object'=>$Object});
    $self->writeContentOutput(\@Links);
}

#========================================================================================
# function     JSONUnicodeConverter
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONUnicodeConverter( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns the strings in unicode.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# return       $Error   | in case of error hash with details is returned | nothing or hash.ref.
#========================================================================================
sub JSONUnicodeConverter {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Type    = $Servlet->params('Type');
    my $aString = $Servlet->form->{'Query'}->{'Strings'} || $Servlet->form->{'Query'}->{'Strings[]'} || [];
    my $Except  = $Servlet->params('Except');

    my @UnicodedStrings;

    if( $Type eq 'File' ){
        foreach my $String (@$aString){
            push @UnicodedStrings, Unicode2Filename($String,$Except);
        }
    } elsif ($Type eq 'Alias'){
        foreach my $String (@$aString){
            push @UnicodedStrings, Unicode2Alias($String,$Except);
        }
    } elsif ($Type eq 'Directory'){
        foreach my $String (@$aString){
            push @UnicodedStrings, Unicode2DirectoryName($String,$Except);
        }
    } elsif ($Type eq 'Path'){
        foreach my $String (@$aString){
            push @UnicodedStrings, Unicode2Path($String,$Except);
        }
    } else {
        return $self->writeContentOutput({
            Errors => [{
                'Reason' => 'ConverterTypeNotExists',
                'Name'   => 'UnicodeError',
                'Form'   => '',
            }]
        });
    }

    $self->writeContentOutput({'Strings' => \@UnicodedStrings});
}

sub JSONLogClientError {
    my $self = shift;
    my ($Servlet) = @_;

    my $ErrorMessage = $Servlet->params('ErrorMessage') // '(empty)';
    my $ErrorLocation = $Servlet->params('ErrorLocation') // '(unknown)';
    my $LogMessage = "$ErrorMessage at $ErrorLocation\n";
    foreach my $Key (qw( Location Referrer )) {
        my $Value = $Servlet->params($Key) // '';
        $LogMessage .= "\n$Key: $Value" if $Value ne '';
    }

    GetLog('JavaScript')->error($LogMessage);
    $self->writeContentOutput({});
}

#========================================================================================
# function     JSONDerefer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONDerefer( $Servlet);
#----------------------------------------------------------------------------------------
# description  Returns the dereferred URLs for a comma separated list of urls in
#               form field  DereferURLs
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub JSONDerefer {
    my $self = shift;
    my ($Servlet) = @_;

    my @URLs = split(/,/, $Servlet->params('DereferURLs'));
    my @Dereferred;

    foreach my $URL (@URLs)  {
        my $Token = GenerateDereferrerToken($URL);
        push (@Dereferred,'?ViewAction=DereferredRedirect&URL='.UrlEncodeValue($URL).'&nonce='.$Token);
    }

    $self->writeContentOutput(\@Dereferred);
}

1;
