use Test::More tests => 3;
use Text::Diff qw(diff);
use strict;

use DE_EPAGES::Cartridge::API::Table::Cartridge qw (GetPKeyCartridgeByPackage);
use DE_EPAGES::Core::API::File  qw ( GetFileContent DeleteDirectory);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::String qw( sortXMLAttributes );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadRootObject);
use DE_EPAGES::Object::API::Language qw (GetCodeByLanguageID);
use DE_EPAGES::Presentation::API::Object::ObjectPageType;
use DE_EPAGES::Presentation::API::PageType qw (ExistsPageTypeByAlias LoadPageTypeByAlias);
use DE_EPAGES::Presentation::API::Template qw (ProcessPage);

sub Main {
    ConnectTestSite();
    Cleanup();

    # Add new pagetype for testtemplates.
    my $System = LoadRootObject();
    my $PageTypeClass = LoadClassByAlias( 'PageType' );
    my $TestPageType  = $System->child('PageTypes')->insertChild( $PageTypeClass, {
        'Alias'         => 'TestPageType',
        'PageTypeClass' => LoadClassByAlias( 'System' )
    } );

    # Get cartrdigeid.
    my $CartridgeID = GetPKeyCartridgeByPackage('DE_EPAGES::Presentation');

    # Create a TLEProcessor.
    my $PageType = LoadPageTypeByAlias( 'TestPageType' );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $System );

    # Check GUIObjects.
    foreach my $GUIObject(qw (Accordion Tab Input)){
        # Add testtemplate.
        $TestPageType->insertTemplate( {
            'Name'          => 'GUIHandler-'.$GUIObject,
            'FileName'      => '../t/GUIObject_'.$GUIObject.'_i.html',
            'CartridgeID'   => $CartridgeID
        } );

        # Process template.
        my $Content = ProcessPage(
            $System,
            $ObjectPageType,
            GetCodeByLanguageID(LoadRootObject()->get('LanguageID')),
            {},
            undef,
            undef,
            "GUIHandler-${GUIObject}"
        );

        # Compare content.
        CompareContent($Content, $GUIObject, $GUIObject);
    }

    Cleanup();
    DisconnectSite();
}

sub CompareContent{
    my ($FoundContent, $GUIObject, $Message) = @_;

    # Get filecontent to compare with.
    my $Expected = "GUIObject_${GUIObject}_e.html";

    $FoundContent = _Normalize($FoundContent);
    my $Content = GetFileContent($Expected);
    my $ExpectedContent = _Normalize($$Content);

    if (not ok( $ExpectedContent eq $FoundContent, $Message )) {
        print diff(\$ExpectedContent, \$FoundContent, {STYLE => 'Context', CONTEXT => 0});
    }
}

sub Cleanup {
    DeleteDirectory( 'ctmpl', { recursive => 1 } );
    LoadPageTypeByAlias( 'TestPageType' )->delete if ExistsPageTypeByAlias( 'TestPageType' );
}

sub _Normalize {
    my ($Content) = @_;

    # delete comments
    $Content =~ s/<!--.*?-->//xsg;
    # trim spaces
    $Content =~ s/\s+/ /sg;
    $Content =~ s/^\s+//s;
    $Content =~ s/\s+$//s;
    my $JointContent;
    foreach my $Line (split(/</,$Content)) {
        $Line =~ /\S/ or next;
        my $After = $Line =~ s,>(.+),>, ? $1 : '';
        $Line = sortXMLAttributes("<$Line"). $After;
        # delete spaces
        $Line =~ s,\s,,g;
        # ignore unsorted arrays, replace each char by 'x' like:
        # '({prebuilt:true,decimals:2})' -> '({xxxxxxxxxxxxxxxxxxxxxxxx})'
        $Line =~ s,(\(\{)(.+?)(\}\)),$1. ('x' x length($2)). $3,ge;
        $JointContent .= "$Line\n";
    }
    return $JointContent;
}

RunScript(
    'Sub' => \&Main
);

1;
