﻿#!perl -w
use strict;
use JSON qw( to_json );
use Test::More;
use Encode qw( encode_utf8 );
use HTTP::Request;
use HTTP::Response;
use HTTP::Status qw( HTTP_OK );
use DE_EPAGES::Core::API::Constants qw( PERL538 );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite );
use DE_EPAGES::Presentation::Servlet::JSON;

RunScript( Sub => \&Main );

sub TestErrorFiles {
    my ($Servlet) = @_;

    my $aErrorFiles = $Servlet->errorFiles('DE_EPAGES::ProductPortal::UI::PortalConfig', 'ru');
    is( scalar @$aErrorFiles, 8, 'errorFiles: multi-level-package' );

    $aErrorFiles = $Servlet->errorFiles('DateTime', 'ru');
    is( scalar @$aErrorFiles, 4, 'errorFiles: single-level-package' );
}

sub TestJSON {
    my ($Servlet, $Structure, $ExpectedResult, $Message) = @_;

    my $Content = eval {
        $Servlet->writeContentOutput( $Structure );
        $Servlet->response->content;
    };
    if( ExistsError() ) {
        fail( $Message );
        Test::More::diag GetError()->longMessage;
    }

    is( join('', sort map { s/,//g; $_ } split(/\n/, $Content)),
        join('', sort map { s/,//g; $_ } split(/\n/, $ExpectedResult)),
        $Message );
}

sub TestNativeDieError {
    my ($Servlet, $Structure, $ExpectedErrorMessage, $Message) = @_;

    eval { $Servlet->writeContentOutput( $Structure ); };
    ok( ExistsError(), "$Message: error exists" ) or return;

    my $Error = GetError();
    is( GetError()->code, 'NativeDie', "$Message: error code" ) or Test::More::diag $Error->longMessage;
    like( $Error->shortMessage, $ExpectedErrorMessage, "$Message: error message" );

    return;
}

sub TestJSONError {
    my ($Servlet, $Structure, $ExpectedErrorCode, $Message) = @_;

    eval { $Servlet->writeContentOutput( $Structure ); };
    if( ExistsError() ) {
        my $Error = GetError();
        is( GetError()->code, $ExpectedErrorCode, $Message ) or Test::More::diag $Error->longMessage;
        return $Error->vars;
    }
    else {
        fail( $Message );
        return {};
    }
}

sub TestWriteContentOutput {
    my ($Servlet) = @_;

    TestJSON( $Servlet, [], "[]\n", 'empty array' );
    TestJSON( $Servlet, {}, "{}\n", 'empty hash' );

    TestJSON( $Servlet, { a => [ 1, 2 ], b => {} },
        qq{{\n   "a" : [\n      1,\n      2\n   ],\n   "b" : {}\n}\n},
        'nested hash'
    );

    # UTF-8
    use utf8;
    my $hData = {
        'russian' => 'Радуга позволяет создавать',
        'german' => 'Bärengöße',
    };
    my $Expected = encode_utf8( <<'END_DATA' );
{
   "russian" : "Радуга позволяет создавать",
   "german" : "Bärengösse"
}
END_DATA
    no utf8;

    ok( utf8::is_utf8( to_json( $hData ) ), 'to_json is not binary' );

    ok( utf8::is_utf8( $hData->{'german'} ), 'hData is utf8' );
    ok( !utf8::is_utf8( $Expected ), 'Expected is binary' );

    # binaries are double-encoded
    use utf8;
    $hData = {
        'russian' => encode_utf8('Радуга позволяет создавать'),
        'german' => encode_utf8('Bärengöße'),
    };
    $Expected = encode_utf8( <<'END_DATA' );
{
   "russian" : "Ð Ð°Ð´ÑÐ³Ð° Ð¿Ð¾Ð·Ð²Ð¾Ð»ÑÐµÑ ÑÐ¾Ð·Ð´Ð°Ð²Ð°ÑÑ",
   "german" : "BÃ¤rengÃ¶Ãe"
}
END_DATA
    TestJSON( $Servlet, $hData, $Expected, 'unicode strings' );

    no utf8;
    ok( !utf8::is_utf8( $hData->{'german'} ), 'hData is binary' );
    TestJSON( $Servlet, $hData, $Expected, 'double-encoded unicode strings' );

    $Expected = encode_utf8( <<'END_DATA' );
{
   "a" : [
      1,
      false
   ]
}
END_DATA
    TestJSON( $Servlet, { 'a' => [ 1, \0 ] }, $Expected, 'scalar ref' );

    my %h;
    tie %h, 'Dummy';
    $Expected = encode_utf8( <<'END_DATA' );
[
   "a",
   {
      "b" : {
         "A" : "a"
      }
   }
]
END_DATA
    TestJSON( $Servlet, [ 'a', { 'b' => \%h }], $Expected, 'tied hash' );

    # JSONP
    $Servlet->vars('callback', 'MyCallback');
    TestJSON( $Servlet, [], "window[\"MyCallback\"]([]\n);", 'JSONP' );
    $Servlet->vars('callback', undef);


    #
    # test error cases
    #

    if ($] < PERL538) {
        TestNativeDieError( $Servlet, '', qr/^hash- or arrayref expected/, 'empty string' );
        TestNativeDieError( $Servlet, undef, qr/^hash- or arrayref expected/, 'undef' );
    } else {
        TestJSON( $Servlet, '', '""', 'empty string: no error' );
        TestJSON( $Servlet, undef, 'null', 'undef: no error' );
    }
    TestNativeDieError( $Servlet, sub {}, qr/^encountered CODE/, 'sub' );
    TestNativeDieError( $Servlet, { 'a' => [ 1, \3 ] }, qr/^cannot encode reference to scalar/, 'scalar ref' );
    TestNativeDieError( $Servlet, { 'a' => [ 1, Dummy->new ] }, qr/^encountered object/, 'object in array' );
    TestNativeDieError( $Servlet, [ 'a', { 'b' => Dummy->new } ], qr/^encountered object/, 'object in hash' );
}

sub Main {

    my $Servlet = DE_EPAGES::Presentation::Servlet::JSON->new;
    $Servlet->{'Request'}  = HTTP::Request->new( GET => 'http://www.example.com/' );
    $Servlet->{'Response'} = HTTP::Response->new( HTTP_OK );
    $Servlet->vars('Locale' => 'de_CH');

    TestWriteContentOutput( $Servlet );

    RunOnStore( 'Store' => GetTestSite(), 'Sub' => sub {
        TestErrorFiles( $Servlet );
    });

    done_testing();
}

#========================================================================================
# §package      Dummy
# §state        private
#----------------------------------------------------------------------------------------
# §description  Package to create dummy objects and tied hashes.
#========================================================================================
package Dummy;

sub new {
    return bless {}, shift;
}

sub TIEHASH {
    return bless {}, shift;
}

sub FIRSTKEY {
    return 'A';
}

sub FETCH {
    return 'a';
}

sub NEXTKEY {
    return undef;
}
