use strict;
use Data::Dumper;
use Test::More tests => 3;
use Text::Diff qw(diff);

use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetPKeyCartridgeByPackage );
use DE_EPAGES::Core::API::Constants qw( PERL538 );
use DE_EPAGES::Core::API::File  qw( GetFileContent );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadRootObject );
use DE_EPAGES::Presentation::API::Object::ObjectPageType;
use DE_EPAGES::Presentation::API::PageType qw( ExistsPageTypeByAlias LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template qw( ProcessPage );
use DE_EPAGES::Presentation::API::TLE::MenuHandler;
use DE_EPAGES::TLE::API::BaseProcessor;

sub Main {
    ConnectTestSite();
    Initialize();

    #    build for testing following structure:
    #    TestPageType
    #        Menu TestMenu1
    #            Menu TestMenu2
    #            Menu TestMenu3
    #            Menu TestJsonMenu1


    # pagetype stuff
    my $ClassSystem = LoadClassByAlias( 'System' );
    my $Class = LoadClassByAlias( 'PageType' );
    my $TestPageType  = LoadRootObject()->folder('PageTypes')->insertChild( $Class, {
        'Alias'         => 'TestPageType',
        'PageTypeClass' => $ClassSystem
    } );

    # insert template
    my $CartridgeID = GetPKeyCartridgeByPackage('DE_EPAGES::Presentation');
    $TestPageType->insertTemplate( {
        'Name'          => 'TestMenu1',
        'FileName'      => '../t/TestMenu1.html',
        'CartridgeID'   => $CartridgeID
    } );
    $TestPageType->insertTemplate( {
        'Name'          => 'TestMenu2',
        'FileName'      => '../t/TestMenu2.html',
        'CartridgeID'   => $CartridgeID
    } );
    $TestPageType->insertTemplate( {
        'Name'          => 'TestMenu3',
        'FileName'      => '../t/TestMenu3.html',
        'CartridgeID'   => $CartridgeID
    } );
    $TestPageType->insertTemplate( {
        'Name'          => 'TestJsonMenu2',
        'FileName'      => '../t/TestJsonMenu2.json',
        'CartridgeID'   => $CartridgeID
    } );
    $TestPageType->insertTemplate( {
        'Name'          => 'TestJsonMenu3',
        'FileName'      => '../t/TestJsonMenu3.json',
        'CartridgeID'   => $CartridgeID
    } );

    # register testmenus
    my $TestMenuItem1 = $TestPageType->insertMenu( {
        Template => 'TestMenu1',
        Position => 10
    } );
    $TestMenuItem1->insertSubMenu( {
        Template => 'TestMenu2',
        Position => 10,
        PageType => $TestPageType
    } );
    $TestMenuItem1->insertSubMenu( {
        Template => 'TestMenu3',
        Position => 20,
        PageType => $TestPageType
    } );
    my $TestJsonMenuItem1 = $TestPageType->insertMenu( {
        Template => 'TestJsonMenu1',
        Position => 10
    } );
    $TestJsonMenuItem1->insertSubMenu( {
        Template => 'TestJsonMenu2',
        Position => 10,
        PageType => $TestPageType
    } );
    $TestJsonMenuItem1->insertSubMenu( {
        Template => 'TestJsonMenu3',
        Position => 20,
        PageType => $TestPageType
    } );

    # TLE stuff
    my $TLEProcessor = DE_EPAGES::TLE::API::BaseProcessor->new;
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $TestPageType, $ClassSystem );

    $TLEProcessor->setContext( { 'PageType' => $ObjectPageType } );

    my $MenuHandler = DE_EPAGES::Presentation::API::TLE::MenuHandler->new;
    $MenuHandler->register( $TLEProcessor );

    foreach my $MenuHandlerType ( qw( Menu MenuInclude MenuJson ) ) {
        $TestPageType->insertTemplate( {
            'Name'          => "MenuHandler-${MenuHandlerType}",
            'FileName'      => "../t/MenuHandler_${MenuHandlerType}_i.html",
            'CartridgeID'   => $CartridgeID
        } );
        my $TestResult = ProcessPage( $ClassSystem, $ObjectPageType, 'en', {}, $TLEProcessor, undef, "MenuHandler-${MenuHandlerType}" );
        my $Result = CompareContent( $TestResult, $MenuHandlerType );

        ok( $Result, "$MenuHandlerType correct" );
    }

    Cleanup();
    DisconnectSite();
}

sub CompareContent {
    my ( $TestResult, $Type ) = @_;

    # get filecontent to compare with
    my $rCompareContent = GetFileContent( 'MenuHandler_'.$Type.'_e.html' );
    my $CompareContent = $$rCompareContent;

    if ($] >= PERL538) {
        $CompareContent =~ s/\btrue\b/1/g;
        $CompareContent =~ s/\bfalse\b/0/g;
    }

    # get rid of annoying characters like newline
    $TestResult =~ s/\n//g;
    $CompareContent =~ s/\n//g;

    # get rid of html comments
    $TestResult =~ s/<!--.*?-->//g;
    $CompareContent =~ s/<!--.*?-->//g;

    # compare strings and return comparison result
    return 1 if $CompareContent eq $TestResult;
    print diff(\$CompareContent, \$TestResult, {STYLE => 'Context', CONTEXT => 0});
    return 0;
}

sub Initialize {
    Cleanup();
}

sub Cleanup {
    LoadPageTypeByAlias( 'TestPageType' )->delete if ExistsPageTypeByAlias( 'TestPageType' );
}

RunScript(
    'Sub' => \&Main
);

1;
