use Test::More tests => 122;

use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Core::API::Url qw( ParseQueryString );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( GetRootObjectID );

##no critic (ValuesAndExpressions::ProhibitMagicNumbers)

RunScript( Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        TestDefaults();
        TestCustom();
        TestEdgeCases();
        TestURIParameter();
    } );
});

sub TestDefaults {
    # test defaults
    my $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        _itemCount_ => 34,
    });
    is( $Pager->get('Page'), 1, 'default page number' );
    is( $Pager->get('PageSize'), 10, 'default page size' );
    is( $Pager->get('PagerSize'), 10, 'default pager size' );
    is( $Pager->get('OrderBy'), '', 'default sort column' );
    is( $Pager->get('OrderDesc'), 0, 'default sort order' );
    is( $Pager->get('AltLanguageID'), 0, 'default alt language id' );
    is_deeply( $Pager->get('UriParameters'), {}, 'default UriParameters' );

    $Pager->statistic;
    is( $Pager->countItems, 34, 'countItems' );
    is( scalar @{ $Pager->allItems }, 34, 'allItems' );
    is( scalar @{ $Pager->items(1) }, 10, 'items(1)' );
    is( scalar @{ $Pager->items(4) }, 4, 'items(4)' );
    is( scalar @{ $Pager->items(5) }, 0, 'items(5)' );
}

sub TestCustom {
    # test custom settings
    my $PageSize = 4;
    my $Pager = MyPager->new({
        ObjectID      => GetRootObjectID(),
        Page          => 5,
        PageSize      => $PageSize,
        PagerSize     => 6,
        OrderBy       => 'X',
        OrderDesc     => 1,
        _itemCount_   => 34,
        AltLanguageID => 1,
    });
    is( $Pager->get('Page'), 5, 'attribute Page' );
    is( $Pager->get('PageSize'), 4, 'attribute PageSize' );
    is( $Pager->get('PagerSize'), 6, 'attribute PagerSize' );
    is( $Pager->get('OrderBy'), 'X', 'attribute OrderBy' );
    is( $Pager->get('OrderDesc'), 1, 'attribute OrderDesc' );
    is( $Pager->get('AltLanguageID'), 1, 'attribute AltLanguageID' );
    is_deeply( $Pager->get('UriParameters'), {}, 'default UriParameters' );

    $Pager->statistic;
    is( $Pager->get('CountOfItems'), 34, 'CountOfItems' );
    is( $Pager->get('CountOfPages'), 9, 'CountOfPages' );
    is( scalar @{ $Pager->items(9) }, 2, 'items(9)' );

    is_deeply( $Pager->get('UriParameters'), {}, 'default UriParameters' );

    my $hVars = $Pager->page->{'Pager'};
    is( $hVars->{'FirstPage'}, 1, 'FirstPage' );
    is( $hVars->{'LastPage'}, 9, 'LastPage' );
    is( $hVars->{'Page'}, 5, 'Page' );
    is( $hVars->{'NextPage'}, 6, 'NextPage' );
    is( $hVars->{'PreviousPage'}, 4, 'PreviousPage' );
    is_deeply( $hVars->{'Items'}, $Pager->items(5), 'Items' );
    my $PagesEx = [ map { _Button($_, $PageSize) } ( 2..7 ) ];
    is_deeply( $hVars->{'Pages'}, $PagesEx, 'Pages' );

    my $PagesNewEx = [
        _ButtonNew(1, 1, $PageSize),
        _ButtonNew(3, '...', $PageSize),
        (map { _ButtonNew($_, $_, $PageSize) } ( 4..6 )),
        _ButtonNew(7, '...', $PageSize),
        _ButtonNew(9, 9, $PageSize),
    ];
    is_deeply( $hVars->{'PagesNew'}, $PagesNewEx, 'PagesNew' );
    is( $hVars->{'URL'}, '?ViewAction=View&ObjectID=1&PageSize=' . $PageSize, 'URL' );
}

sub TestEdgeCases {
    # edge cases for new pager: 6 pages of items
    my $PageSize = 6;
    my $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        Page => 5,
        PageSize => $PageSize,
        _itemCount_ => 34,
    });
    $Pager->statistic;
    is( $Pager->get('CountOfPages'), 6, '6 pages' );

    my $hVars = $Pager->page->{'Pager'};
    my $PagesNewEx = [ map { _ButtonNew($_, $_, $PageSize) } ( 1..6 ) ];
    is_deeply( $hVars->{'PagesNew'}, $PagesNewEx, 'PagesNew' );

    # edge cases for new pager: 7 pages of items
    $PageSize = 5;
    $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        Page => 5,
        PageSize => $PageSize,
        _itemCount_ => 34,
    });
    $Pager->statistic;
    is( $Pager->get('CountOfPages'), 7, '7 pages' );

    $hVars = $Pager->page->{'Pager'};
    $PagesNewEx = [ map { _ButtonNew($_, $_, $PageSize) } ( 1..7 ) ];
    is_deeply( $hVars->{'PagesNew'}, $PagesNewEx, 'PagesNew' );

    # edge cases for new pager: 8 pages of items
    $PageSize = 5;
    $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        Page => 5,
        PageSize => $PageSize,
        _itemCount_ => 40,
    });
    $Pager->statistic;
    is( $Pager->get('CountOfPages'), 8, '8 pages' );

    $hVars = $Pager->page->{'Pager'};
    $PagesNewEx = [
        _ButtonNew(1, 1, $PageSize),
        _ButtonNew(3, '...', $PageSize),
        (map { _ButtonNew($_, $_, $PageSize) } ( 4..8 )),
    ];
    is_deeply( $hVars->{'PagesNew'}, $PagesNewEx, 'PagesNew' );
}

sub TestURIParameter {
    # UriParameter with singel value parameter && MultiValueParameter

    my %UriParameters = (
        SearchString => 'Alice',
        Category => 'Shoes',
        Color => ['Brown', 'Red'],
        Price => [11.22, 24.77],
        Size => ['0', 'L', undef, '', 'XL'],
        Undef => undef,
        Zero => 0,
        Empty => '',
    );
    my $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        _itemCount_ => 34,
        UriParameters => \%UriParameters,
    });

    my $hPage = $Pager->page();

    foreach my $Url (qw( URL URLFirstPage URLLastPage URLNextPage URLPage URLPreviousPage)) {
        my $hURL = ParseQueryString($hPage->{Pager}{$Url}, 'utf-8');
        foreach my $Name (qw(SearchString Category Zero Undef Empty)) {
            if ($UriParameters{$Name} eq '') {
                is($hURL->{$Name}, undef, "scalar $Name not in $Url" );
            } else {
                is($hURL->{$Name}[0], $UriParameters{$Name}, "scalar $Name in $Url" );
            }
        }
        foreach my $Name (qw(Color Price Size)) {
            foreach my $Value (@{$UriParameters{$Name}}) {
                if ($Value eq '') {
                    ok(!ElementOf($Value, $hURL->{$Name}), "array $Name elem $Value not in $Url" );
                } else {
                    ok(ElementOf($Value, $hURL->{$Name}), "array $Name elem $Value in $Url" );
                }
            }
        }
    }
}
## use critic

#========================================================================================
# Private Helper
#========================================================================================
sub _Button($;$) {
    my ($Page, $PageSize) = @_;
    return {
       'URLPage' => '?ViewAction=View&ObjectID=1'
       . (defined $PageSize ? "&PageSize=$PageSize" : '')
       . ($Page == 1 ? '' : "&Page=$Page"),
       'Page' => $Page,
     },
}

sub _ButtonNew($$;$) {
    my ($Page, $Text, $PageSize) = @_;

    my $hButton = _Button($Page, $PageSize);
    $hButton->{'PageText'} = $Text;
    return $hButton;
}


#========================================================================================
# Implementation of abstract Pager
#========================================================================================
## no critic (DE_EPAGES::Style::RequireCommentForExportedFunctions)

package MyPager;
use base 'DE_EPAGES::Presentation::API::Object::Pager';

use strict;

sub new {
    my $class = shift;
    my ($hParams) = @_;

    my $ItemCount = delete $hParams->{_itemCount_};
    my $self = $class->SUPER::new( $hParams );
    $self->{_itemCount_} = $ItemCount;

    return $self;
}

sub _items {
    my $self = shift;

    return [ 1..$self->{_itemCount_} ];
}

sub countItems {
    my $self = shift;

    return scalar @{ $self->_items };
}

sub items {
    my $self = shift;
    my ($Page) = @_;

    my $FirstIndex = $self->firstItemOfPage($Page);
    my $LastIndex = $self->lastItemOfPage($Page);
    return [ @{ $self->_items }[ $FirstIndex .. $LastIndex ] ];
}

## use critic
