#========================================================================================
# package      DE_EPAGES::PriceList::API::CSV::PriceListExportImport;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements the high level interface for import and export of
#               PriceLists to CSV files.
#========================================================================================
package DE_EPAGES::PriceList::API::CSV::PriceListExportImport;
use base DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectExportImport;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::PriceList::API::Constants qw( @CSV_WARNING_CODES_PRICELIST );
use DE_EPAGES::PriceList::API::CSV::PriceListExportDriver;
use DE_EPAGES::PriceList::API::CSV::PriceListImportHandler;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );

#========================================================================================
# function     warningCodes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aWarningCodes = $Driver->warningCodes();
#----------------------------------------------------------------------------------------
# description  Returns the warning codes for csv export.
#----------------------------------------------------------------------------------------
# return       $aWarningCodes | warning codes | array.string
#========================================================================================
sub warningCodes {
    my $self = shift;
    return [ @{$self->SUPER::warningCodes}, @CSV_WARNING_CODES_PRICELIST ];
}

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub exportDriver {
    my $self = shift;
    return 'DE_EPAGES::PriceList::API::CSV::PriceListExportDriver';
}

#========================================================================================
# function     importDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->importDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub importDriver {
    my $self = shift;
    return 'DE_EPAGES::PriceList::API::CSV::PriceListImportHandler';
}

#========================================================================================
# function     getExportObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->getExportObjects($Shop, $aObjects);
#----------------------------------------------------------------------------------------
# description  Returns list of objects to export.
#----------------------------------------------------------------------------------------
# input        $Shop           | alias of shop          | object
# input        $aObjects       | list of exportable objects | ref.array.integer
# return       $aExportObjects | edited list of exportable objects | ref.array
#========================================================================================
sub getExportObjects {
    my $self = shift;
    my ($Shop, $aObjects) = @_;

    my @ProductIDs;
    if(not defined $aObjects){ # get all Products
        @ProductIDs = @{ LoadClassByAlias('Product')->findObjects('ByShopIDIsAppointment', [$Shop->id, 0]) };
    } else {
        foreach my $ProductID (@$aObjects) {
            push @ProductIDs, $ProductID;
            my $Product = LoadObject($ProductID);
            foreach my $SubProduct (@{ $Product->get('SubProducts') }) {
                push @ProductIDs, $SubProduct->id
                    unless ElementOf($SubProduct->id, $aObjects);
            }
        }
    }
    return \@ProductIDs;
}

#========================================================================================
# function     csvAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aCSVAttributes = $Driver->csvAttributes($Shop, $LocaleID, $aLanguageCode2s);
#----------------------------------------------------------------------------------------
# description  Return list of exportable csv attributes.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LocaleID | locale identifier (e.g. en_GB) | string
# input        $aLanguageCode2s | array of languages identified by code2 | ref.array.string
# return       $aAttributes | list of attributes | ref.array.hash
#========================================================================================
sub csvAttributes {
    my $self = shift;
    my ($Shop, $LocaleID, $aLanguageCode2s) = @_;

    my $hLocale = GetInfoLocale($LocaleID);
    my $ahLanguages = $self->getLanguages($Shop, $aLanguageCode2s);

    my $ScalePriceClass = LoadClassByAlias('ScalePrice');
    my $QuantityDiscountClass = LoadClassByAlias('QuantityDiscount');
    my $ValueDiscountClass = LoadClassByAlias('ValueDiscount');

    my @PriceListAttributes = map {
        { 'Class' => $ScalePriceClass, 'Attribute' => $ScalePriceClass->attribute($_) }
    } qw (Product PriceList Quantity Price);
    push @PriceListAttributes, map {
        { 'Class' => $QuantityDiscountClass, 'Attribute' => $QuantityDiscountClass->attribute($_) }
    } qw (Discount);
    push @PriceListAttributes, map {
        { 'Class' => $ValueDiscountClass, 'Attribute' => $ValueDiscountClass->attribute($_) }
    } qw (Value Discount);

    my @CSVAttributes;
    foreach my $hAttribute ( @PriceListAttributes ) {
        my $Attribute = $hAttribute->{'Attribute'};
        my $Alias = $Attribute->alias;

        push @CSVAttributes, @{
            $self->getAttributeInfo(
                $Attribute, $hLocale->{'LanguageID'}, $ahLanguages, {
                    'Alias' => $Alias eq 'Discount' && $hAttribute->{'Class'}->alias eq 'ValueDiscount' ? 'ValueDiscount' : $Alias,
                    'Class' => $hAttribute->{'Class'},
                    'Position' => 0,
                }
            )
        };
    }
    return \@CSVAttributes;
}

1;
