#========================================================================================
# §package      DE_EPAGES::Product::API::CustomAttributesPatch
# §state        public
#----------------------------------------------------------------------------------------
# §description  Implements JSON-PATCH (RFC 6902) for the product custom attributes object.
#========================================================================================
package DE_EPAGES::Product::API::CustomAttributesPatch;
use base DE_EPAGES::WebService::API::JSONPatch;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByGUID );

use strict;

## no critic (ProhibitUnusedPrivateSubroutines)



#========================================================================================
# §function     new
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       new( $Product );
#----------------------------------------------------------------------------------------
# §description  Constructor
#----------------------------------------------------------------------------------------
# §input        $Product | A Product | DE_EPAGES::Product::API::Object::Product
# §input        $LanguageId | A LanguageID | integer
# §return       $self | self | DE_EPAGES::Product::API::CustomAttributesPatch
#========================================================================================
sub new {
    my $class = shift;
    my ($Product, $LanguageId) = @_;
    my $self  = bless {
        'Product' => $Product,
        'LanguageId' => $LanguageId,
    }, $class;
    return $self;
}

sub _add {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $Product = $self->{'Product'};
    my $LanguageId = $self->{'LanguageId'};

    # get the aliases of all product types that were used to create variations
    my @UsedVariations = map {$_->{'Attribute'}->alias} @{$Product->getWithParent('VariationTypeValues')};

    if($Path eq '/') {
        my $UserDefinedAttribute = $Value->{'key'};

        if (ElementOf($UserDefinedAttribute, @UsedVariations)) {
            # throw error if displayValue can not be changed. For example for variation attributes
            Error("Attribute '$UserDefinedAttribute' was used to create variations");
        }

        foreach my $Attribute (@{$Product->get('UserDefinedAttributeValues', $LanguageId)})  {
            if ($UserDefinedAttribute eq $Attribute->{'Attribute'}->alias){

                #throw error if no displayValue is set
                if ($Value->{'displayValue'} eq '') {
                    Error (q(To unset a attribute value use the 'remove' operation));
                }

                if ($Attribute->{'Attribute'}->type eq 'LocalizedString') {
                    $Product->set({$Attribute->{'Attribute'}->alias => $Value->{'displayValue'}}, $LanguageId);
                    return;
                }

                if ($Attribute->{'Attribute'}->type eq 'PreDefLocalizedString') {
                    my $PreDefAttributeID = $Attribute->{'Attribute'}->{'Attributes'}->{'AttributeID'};
                    my $aPreDefAttributeChildren = LoadObject($PreDefAttributeID)->get('PreDefAttributes');
                    my $AttributeValueFound = 0;

                    foreach my $PreDefAttribute (@{$aPreDefAttributeChildren}) {
                        my $AttributeObject = LoadObjectByGUID($PreDefAttribute->guid);
                        if ($AttributeObject->get('ValueOrAlias', $LanguageId)  eq $Value->{'displayValue'}) {
                            $Product->set({ $Attribute->{'Attribute'}->alias => $PreDefAttribute });
                            $AttributeValueFound = 1;
                            return;
                        }
                    }
                    # throw error if value was not found for specified attribute
                    Error ("value: '$Value->{'displayValue'}' for attribute: '$Value->{'key'}' does not exists") unless $AttributeValueFound;
                }
            }
        }
        Error ("Attribute '$Value->{'key'}' does not exist for this product");
     }
    Error("Adding in $Path is not implemented!");
}

sub _remove {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $Product = $self->{'Product'};
    my $LanguageId = $self->{'LanguageId'};
    # get the aliases of all product types that were used to create variations
    my @UsedVariations = map {$_->{'Attribute'}->alias} @{$Product->getWithParent('VariationTypeValues')};

     if($Path eq '/') {
         if (ElementOf($Value->{'key'}, @UsedVariations)) {
             # throw error if displayValue can not be changed. For example for variation attributes
             Error("Attribute '$Value->{'key'}' was used to create variations");
         } else {
             $Product->set({$Value->{'key'} => undef}, $LanguageId);
             return;
         }
     }
    Error("Removing $Path is not implemented!");
}

1;
