#==============================================================================
# package      DE_EPAGES::Product::API::Object::PagerLoopProducts
# state        public
# base         DE_EPAGES::Presentation::API::Object::PagerLoop
#----------------------------------------------------------------------------------------
# description  Implements a pager for sorting a list of products.
#==============================================================================
package DE_EPAGES::Product::API::Object::PagerLoopProducts;
use base DE_EPAGES::Presentation::API::Object::PagerLoop;

use strict;
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

sub ErrorDefinition {
    return {
        'NoCurrencyID' => 'missing CurrencyID for order by ListPrice',
    };
}

#========================================================================================
# function     sortObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSortedObjects = $Object->sortObjects($aObjects);
#----------------------------------------------------------------------------------------
# description  Sorts a list of products by the 'OrderBy' and 'OrderDesc' input fields.
#               Gets attribute values by "$Product->getWithParent" to account for
#               inherited attribute values of sub products.
#               Special handling of the "ListPrice" attribute:
#               - use $Product->getListPrice( $self->{CurrencyID} ) if $self->{CurrencyID} is defined,
#               - throws an error if CurrencyID is not defined
#----------------------------------------------------------------------------------------
# input        $aObjects | list of all products | ref.array.object
# return       $aSortedObjects | sorted list of products | ref.array.object
#========================================================================================
sub sortObjects {
    my $self = shift;
    my ($aObjects) = @_;
    return [] unless scalar @$aObjects;

    my $OrderBy = $self->orderBy;

    # don't sort if OrderBy is empty
    unless( defined $OrderBy and $OrderBy =~ /\S/ ) {
        return $self->orderDesc ? [reverse @$aObjects] : $aObjects;
    }

    my $FirstObject = $aObjects->[0];

    # don't sort if OrderBy is not an attribute
    unless( $OrderBy eq 'ListPrice' || $FirstObject->class->existsAttribute( $OrderBy ) ) {
        return $aObjects;
    }

    # get values to sort
    my @Values;
    my $Type;
    if( $OrderBy eq 'ListPrice' ) {
        my $TaxModel = $FirstObject->getSite()->get('TaxModel');
        my $CurrencyID = $self->get('CurrencyID');
        Error( 'NoCurrencyID' ) unless defined $CurrencyID;
        my $Shop = $FirstObject->getSite();
        my $VariationPriceFromText = $Shop->get('VariationPriceFromText', $Shop->get('LanguageID'));
        foreach ( @$aObjects ) {
            if ( $_->get('IsMaster') ) {
                my $MasterPrice = $_->getListPrice($CurrencyID, $TaxModel);
                if ( $MasterPrice ) {
                    push @Values, $MasterPrice;
                } elsif ( $VariationPriceFromText ) {
                    my $LowestSubPrice = $_->lowestSubProductPrice($CurrencyID, $TaxModel);
                    push @Values, $LowestSubPrice ? $LowestSubPrice : $MasterPrice;
                } else {
                    push @Values, $MasterPrice;
                }
            } else {
                push @Values, $_->getListPrice($CurrencyID, $TaxModel);
            }
        }
        $Type = 'Money';
    }
    else {
        @Values = map { $_->getWithParent($OrderBy) } @$aObjects;
        $Type = $FirstObject->class->attribute( $OrderBy )->get('Type');
    }

    # start with undefined values
    my @SortedKeys = grep { not defined $Values[$_] } (0..scalar(@Values)-1);

    # sort different on different types
    if( ElementOf($Type, qw(Integer Float Boolean Money)) ) {
        push @SortedKeys, sort {$Values[$a] <=> $Values[$b]} grep { defined $Values[$_] } (0..scalar(@Values)-1);
    }
    else {
        push @SortedKeys, sort {uc $Values[$a] cmp uc $Values[$b]} grep { defined $Values[$_] } (0..scalar(@Values)-1);
    }
    my @Objects = @$aObjects[@SortedKeys];

    # sort descending ?
    @Objects = reverse @Objects if $self->orderDesc;

    return \@Objects;
}

1;
