#========================================================================================
# package      DE_EPAGES::Product::API::Resource::ProductLowestPrice
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for getting lowest price of product variations
#========================================================================================
package DE_EPAGES::Product::API::Resource::ProductLowestPrice;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use strict;

no warnings 'redefine'; ##no critic qw(ProhibitNoWarnings)

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Product::API::Resource::Product;
use DE_EPAGES::Product::API::Resource::FromPriceFormatter;

sub ErrorDefinition {
    return {
        ResourceNotFound => 'Product "#ProductGUID" does not have a lowest price. (ShopAlias: #ShopAlias)',
    };
}

sub service {
    return LoadObjectByPath('/RestServices/ProductLowestPrice');
}

sub class {
    return 'Product';
}

sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get', 'application/json'],
        ],
    }->{$RequestMethod};
}

sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/products/([^/]+)/lowest-price/?$|;
    return $1;
}

sub href {
    my $self = shift;
    my ($APIBaseURL, $ProductGUID) = @_;
    return "$APIBaseURL/api/products/$ProductGUID/lowest-price";
}

sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
    my $LocaleID = $hQuery->{'locale'}->[0];
    my $CurrencyID = $hQuery->{'currency'}->[0];

    return [ $Object, $LocaleID, $CurrencyID ];
}

sub get {
    my $self = shift;
    my ($SuperProduct, $LocaleID, $CurrencyID) = @_;

    # validate query parameters
    my $Shop = $SuperProduct->getSite();
    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);
    $CurrencyID = $Context->validCurrency($CurrencyID);

    # check if product visible and master
    Error('ResourceNotFound', {
        'ProductGUID' => $SuperProduct->guid(),
        'ShopAlias' => $Shop->alias(),
    }) unless $SuperProduct->existSubProductPrices();

    my $TaxModel = $Shop->get('TaxModel');
    my $APIBaseURL = $Context->apiBaseUrl();

    my $Formatter = $self->_createFromPriceFormatter($Shop, $LocaleID);

    my $Quantity = $SuperProduct->get('MinOrder'); # see TLE function #DISPLAYPRICE()

    my $hPriceParams = {
        Quantity    => $Quantity,
        CurrencyID  => $CurrencyID,
        TaxModel    => $TaxModel,
    };
    my $SubProduct = $SuperProduct->subProductWithLowestPrice($hPriceParams);
    Error('ResourceNotFound', {
        'ProductGUID' => $SuperProduct->guid(),
        'ShopAlias' => $Shop->alias(),
    }) unless defined $SubProduct;

    # get prices from available and visible sub products and test if we have different prices
    my $aSubProducts = $SuperProduct->subProducts;
    my $hAvailableSubProductPrices = {};
    foreach my $Sub (grep { $_->get('HasSubOwnPrices') && $_->get('IsAvailable') && $_->get('IsVisible')} @$aSubProducts) {
        my $Price = $Sub->getListPrice($CurrencyID, $TaxModel);
        next unless defined $Price;
        $hAvailableSubProductPrices->{$Price} = undef;
    }

    # use fromPrice function if we have more than one different price for sub products
    my $PriceCount = scalar keys(%$hAvailableSubProductPrices);
    my $UseFromPrice = $PriceCount > 1 ? 1:0;

    # we have no prices
    Error('ResourceNotFound', {
        'ProductGUID' => $SuperProduct->guid(),
        'ShopAlias' => $Shop->alias(),
    }) unless $PriceCount;

    my $LowestPrice = $SubProduct->displayPrice($hPriceParams);
    return {
        priceInfo => {
            price       => $UseFromPrice ? $Formatter->fromPrice( $LowestPrice, $CurrencyID, $TaxModel )
                                         : $Formatter->price( $LowestPrice, $CurrencyID, $TaxModel ),
            quantity    => $Formatter->quantity( $Quantity, $SuperProduct->get('OrderUnit') ),
        },
        links => [
            {   rel => 'product',
                href => DE_EPAGES::Product::API::Resource::Product->href($APIBaseURL, $SubProduct->guid),
            }
        ]
    };
}



sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    my $ShopRequestContext = DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
    return $ShopRequestContext;
}



sub _createFromPriceFormatter {
    my $self = shift;
    my ($Shop, $LocaleID) = @_;
    my $FromPriceFormatter = DE_EPAGES::Product::API::Resource::FromPriceFormatter->new($Shop, $LocaleID);
    return $FromPriceFormatter;
}

1;
