#========================================================================================
# package      DE_EPAGES::ProductPortal::API::TLE::GetPortalSitesByCountry
# state        public
# base         DE_EPAGES::ProductPortal::API::TLE::TLEHandler
#----------------------------------------------------------------------------------------
# description  This class contains methods for TLE functions.
#========================================================================================
package DE_EPAGES::ProductPortal::API::TLE::GetPortalSitesByCountry;

use base DE_EPAGES::ProductPortal::API::TLE::TLEHandler;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError GetLog );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadRootObject );
use DE_EPAGES::ProductPortal::API::Constants qw (
    AMAZON
    GOOGLESHOPPING
    PORTAL_LANGUAGE_EXCEPTIONS
    GOOGLE_SMART_SHOPPING
    META_CATALOG
);

use strict;


use constant TLE_FUNCTION_NAME => 'GET_PORTAL_SITES_BY_COUNTRY';

#========================================================================================
# function     RegisterHandlerProc
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function register this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'}, TLE_FUNCTION_NAME );

    return;
}

#========================================================================================
# function     GET_PORTAL_SITES_BY_COUNTRY
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalSitesByCountry = GET_PORTAL_SITES_BY_COUNTRY( $Processor, $aParams );
#----------------------------------------------------------------------------------------
# description  this function returns a list of (not deprecated) product portals ordered by country
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $aParams | List of Parameters | array ref
# return       @SortedPortalSites | List of product portals | array ref
#========================================================================================
sub GET_PORTAL_SITES_BY_COUNTRY {
    my $self = shift;
    my ($Processor, $aParams) = @_;

    my ($ShopID, $OnlyAmazon, $OnlyGoogleBase) = @$aParams;

    GetLog()->warn('Both OnlyAmazon and OnlyGoogleBase are set to 1!') if $OnlyAmazon && $OnlyGoogleBase;

    my @AllPortalSites;

    my $System  = $self->_LoadRootObject();
    my $Shop    = $self->_getObjectById($ShopID);
    my $IsUnityRegistered = $Shop->get('IsUnityRegistered');

    my $Portals = $System->get('Portals');
    my $ShopCurrencies  = $Shop->get('Currencies');
    my $ShopLanguages = $Shop->get('Languages');
    my $aPortalConfigs = $Shop->get('PortalConfigs');
    my $LocaleID = $Shop->get('LocaleID');

    for my $Portal (@{$Portals}) {
        my $hAttributes = $Portal->get(['PortalSites','NameOrAlias','Image','Feature', 'ID']);

        # for Unity shops either handle only Amazon or Google Shopping portals
        # or just portals without Amazon or Google Shopping or google smart shopping (UNITY-4679, UNITY-5480, UNITY-5482, AD-10874)
        if ($IsUnityRegistered) {
            my $aSkippedPortals = [AMAZON, GOOGLE_SMART_SHOPPING, GOOGLESHOPPING, META_CATALOG];
            if ($OnlyAmazon) {
                next unless $hAttributes->{'NameOrAlias'} eq AMAZON;
            } elsif ($OnlyGoogleBase) {
                next unless $hAttributes->{'NameOrAlias'} eq GOOGLESHOPPING;
            } else {
                next if ElementOf($hAttributes->{'NameOrAlias'}, $aSkippedPortals);
            }
        }

        my $PortalSites = $hAttributes->{'PortalSites'};

        for my $PortalSite (@{$PortalSites}) {

            my $IsLanguageInstalled  = $self->_isLanguageInstalled($ShopLanguages, $PortalSite, $PortalSite->get('Language')->{'DefaultLocaleID'});
            my $IsLanguageException  = $OnlyAmazon ? $self->_isLanguageException($PortalSite, $LocaleID) : 0;
            my $IsCurrencyAvailable  = $self->_isCurrencyAvailable($ShopCurrencies, $PortalSite->get('Currency'));
            my $IsEligible           = $self->_isEligible($PortalSite, $aPortalConfigs);
            if (($IsLanguageInstalled || $IsLanguageException) && $IsCurrencyAvailable && $IsEligible) {
                my $hPortalSite = $self->_buildHashResponse($Shop, $PortalSite, $hAttributes);

                push(@AllPortalSites, $hPortalSite);
            }
        }
    }

    my @SortedPortalSites = sort {$a->{'CountryNative'} cmp $b->{'CountryNative'}} @AllPortalSites;

    return \@SortedPortalSites;
}

#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------

#========================================================================================
# function     _buildHashResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_buildHashResponse($Shop, $PortalSite, $hAttributes);
#----------------------------------------------------------------------------------------
# description  builds and returns a hash with all the parameters needed in the template
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $PortalSite | portal site object | object
# input        $hAttributes | hash with attributes already fetched | hash
# return       $hPortalSite | hash with attributes from portal site | hash
#========================================================================================
sub _buildHashResponse {
    my $self = shift;
    my ($Shop, $PortalSite, $hAttributes) = @_;

    my $hPortalSite = {};

    my $FeatureAlias = $hAttributes->{'Feature'}->get('Alias');

    $hPortalSite->{'FeatureAlias'}      = $FeatureAlias;
    $hPortalSite->{'FeatureMaxValue'}   = $Shop->featureMaxValue($FeatureAlias);
    $hPortalSite->{'ProductPortalName'} = $hAttributes->{'NameOrAlias'};
    $hPortalSite->{'PortalID'}          = $hAttributes->{'ID'};
    $hPortalSite->{'Image'}             = $hAttributes->{'Image'};

    my $hPortalSiteAttributes = $PortalSite->get(['Alias','Country','PortalURL','FlagGif','IsSandbox', 'ID',
        'Language', 'ExistsPortalSiteOfSameCountry']);

    $hPortalSite->{'PortalSiteAlias'}   = $hPortalSiteAttributes->{'Alias'};
    $hPortalSite->{'CountryNative'}     = $hPortalSiteAttributes->{'Country'}->{'Native'};
    $hPortalSite->{'CountryCode2'}      = $hPortalSiteAttributes->{'Country'}->{'Code2'};
    $hPortalSite->{'PortalURL'}         = $hPortalSiteAttributes->{'PortalURL'};
    $hPortalSite->{'FlagGif'}           = $hPortalSiteAttributes->{'FlagGif'};
    $hPortalSite->{'IsSandbox'}         = $hPortalSiteAttributes->{'IsSandbox'};
    $hPortalSite->{'PortalSiteID'}      = $hPortalSiteAttributes->{'ID'};
    $hPortalSite->{'PortalSiteLanguage'}        = $hPortalSiteAttributes->{'Language'};
    $hPortalSite->{'PortalSiteIsMultiLanguage'} = $hPortalSiteAttributes->{'ExistsPortalSiteOfSameCountry'};

    return $hPortalSite;
}

#========================================================================================
# function     _isLanguageInstalled
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isLanguageInstalled($SystemLanguages, $PortalSite, $DefaultLocaleID);
#----------------------------------------------------------------------------------------
# description  this function checks if the product portal language is already installed
#----------------------------------------------------------------------------------------
# input        $SystemLanguages | shop object | object
# input        $PortalSite | portal site object | object
# input        $DefaultLocaleID | default language | string
# return       1 (true) or 0 (false) | boolean
#========================================================================================
sub _isLanguageInstalled {
    my $self = shift;
    my ($SystemLanguages, $PortalSite, $DefaultLocaleID) = @_;

    my $PortalSiteClassAlias = $PortalSite->get('Class')->get('Alias');

    for my $Language (@{$SystemLanguages}) {
        if ($Language->{'DefaultLocaleID'} eq $DefaultLocaleID && $PortalSiteClassAlias ne 'UniversalPortalSite') {
            return 1;
        }
    }

    return 0;
}

#========================================================================================
# function     _isLanguageException
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isLanguageException($PortalSite, $LocaleID);
#----------------------------------------------------------------------------------------
# description  this function checks if the product portal should be displayed although
#               the related language is not installed in the shop
#----------------------------------------------------------------------------------------
# input        $PortalSite | portal site object | object
# input        $LocaleID | locale ID | string
# return       1 (true) or 0 (false) | boolean
#========================================================================================
sub _isLanguageException {
    my $self = shift;
    my ($PortalSite, $LocaleID) = @_;

    return ElementOf($LocaleID, PORTAL_LANGUAGE_EXCEPTIONS->{$PortalSite->alias});
}

#========================================================================================
# function     _isCurrencyAvailable
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isCurrencyAvailable($ShopCurrencies, $PortalSiteCurrency);
#----------------------------------------------------------------------------------------
# description  this function checks if the product portal currency is available
#----------------------------------------------------------------------------------------
# input        $ShopCurrencies | list of shop currencies | array ref
# input        $PortalSiteCurrency | portal site currency | string
# return       1 (true) or 0 (false) | boolean
#========================================================================================
sub _isCurrencyAvailable {
    my $self = shift;
    my ($ShopCurrencies, $PortalSiteCurrency) = @_;

    for my $Currency (@{$ShopCurrencies}) {
        return 1 if $Currency->{'CurrencyID'} eq $PortalSiteCurrency->{'CurrencyID'};
    }

    return 0;
}

#========================================================================================
# function     _isEligible
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isEligible($PortalSite, $aPortalConfigs);
#----------------------------------------------------------------------------------------
# description  this function checks if the product portal can be eligible in the MBO to
#               be shown -is not deprecated or has configured portal configs-.
#----------------------------------------------------------------------------------------
# input        $PortalSite | portal site object | object
# input        $aPortalConfigs | shop portal configs array | array ref
# return       1 (true) or 0 (false) | boolean
#========================================================================================
sub _isEligible {
    my $self = shift;
    my ($PortalSite, $aPortalConfigs) = @_;

    return 1 unless ($PortalSite->isDeprecated());

    my $PortalSiteID = $PortalSite->id();
    for my $PortalConfig (@{$aPortalConfigs}) {
        my $PortalSiteForConfig = $PortalConfig->get('PortalSite');
        if ( ($PortalSiteID == $PortalSiteForConfig->id()) && $PortalConfig->isConfigured() ) {
            return 1;
        }
    }

    return 0;
}


#---------------------------------------------------------------------------
#  Wrappers
#---------------------------------------------------------------------------

#========================================================================================
# function     _LoadRootObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_LoadRootObject();
#----------------------------------------------------------------------------------------
# description  Get the root Object
#----------------------------------------------------------------------------------------
# return       $Object | Object | Object
#========================================================================================
sub _LoadRootObject {
    my $self = shift;

    return LoadRootObject();
}

#========================================================================================
# function     _getObjectById
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_getObjectById($ObjectID);
#----------------------------------------------------------------------------------------
# description  Get the Object by ID, create log in fail case without breaking the shop
#----------------------------------------------------------------------------------------
# input        $ObjectID | Object ID | int
# return       $Object | Object | Object
#========================================================================================
sub _getObjectById {
    my $self = shift;
    my ($ObjectID) = @_;
    my $Object;
    eval {
        $Object = LoadObject($ObjectID);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        $self->_LogError($Error);
        return;
    }
    return $Object;
}


#========================================================================================
# function     _LogError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_LogError( $Error );
#----------------------------------------------------------------------------------------
# description  Wraps the core bindings
#----------------------------------------------------------------------------------------
# input        $Error | an error object | object
#========================================================================================
sub _LogError {
    my $self = shift;
    my ($Error) = @_;
    LogError($Error);
    return;
}

1;
