#---------------------------------------------------------------------------
#  The purpose of this script is to generate a csv file with the specific
#  fields for product portals and export it through the REST API
#---------------------------------------------------------------------------
use strict;

use DE_EPAGES::AppStore::API::Object::AppsClient;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadObjectByGUID LoadRootObject);
use DE_EPAGES::Product::API::CSV::ProductExportImport;
use DE_EPAGES::ShopCSVExportImport::API::CSV::ShopExportImport qw ( ExportCSV );
use DE_EPAGES::Core::API::Error  qw( ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw( Warning );
use Getopt::Long;
use JSON::PP;
use HTTP::Request::Common qw ( GET );

use constant FEED_FILE_NAME => 'rest_portals_feed_file.csv';
use constant PRODUCT_PORTAL_EXPORT_DRIVER => 'DE_EPAGES::ProductPortal::API::CSV::ProductPortalExportImport';
use constant PRODUCTS_READ_BATCH => 'products_read_batch';

sub Main {

    my $Help;
    my $NoOutput;
    my $StoreName;

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'storename=s'   => \$StoreName,
    );

    usage() if $Help;
    usage() unless defined $StoreName;

    unless( $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }

    my $time1 = time;
    my $ShopsFolder;

    RunOnStore(
        Store => $StoreName,
        Sub => sub {
            $ShopsFolder = LoadObjectByPath('/Shops/');
            print 'Shop folder ID is ' . $ShopsFolder->id() . "\n" unless $NoOutput;

            my $aShopsIDs = _getShopsList($StoreName);
            foreach my $ShopID ( @$aShopsIDs ) {
                eval{
                    my $Shop = LoadObjectByGUID($ShopID);

                    print 'Shop ID is ' . $Shop->id() . "\n" unless $NoOutput;

                    my $FilePath = $Shop->get('SuperMerchant')->get('PrivatePath') . '/' . FEED_FILE_NAME;
                    print 'The feed file will be stored at ' . $FilePath . "\n" unless $NoOutput;

                    my $ProductsFolder = $Shop->child('Products');
                    print 'Products ID is '.$ProductsFolder->id() . "\n" unless $NoOutput;

                    my $aProductsIDs = $ProductsFolder->class()->findObjects('ByParentID', [ $ProductsFolder->id() ]);

                    my $hFeedFileOptions = {
                        'eol'       => "\n",
                        'sep_char'  => ';',
                    };

                    $hFeedFileOptions->{'nooutput'} = 1 unless $NoOutput;

                    ExportCSV($Shop, $FilePath, PRODUCT_PORTAL_EXPORT_DRIVER, $hFeedFileOptions, $aProductsIDs);
                };
                Warning( GetError() ) if ExistsError();
            }
        }
    );

    printf("need %.3f seconds\n", time()-$time1) unless $NoOutput;

}

sub _getShopsList {
    my ($Storename) = @_;

    my $System = LoadRootObject();
    my $BaseUrl //= $System->get('OAuthServiceBaseUrl') // $System->get('EPagesJBaseURL');

    my $UriTemplate = DE_EPAGES::EPagesJ::API::Object::UriTemplate->new($BaseUrl , "product-portals/shops?storename=$Storename");
    my $Response = _receiveData($UriTemplate->uri());

    return decode_json($Response->decoded_content);
}

sub _receiveData {
    my ($Url) = @_;

    my $ua = LWP::UserAgent->new;
    my $Request = GET(
        $Url,
        (
            'Content-Type'  => 'application/json',
        ));
    my $Response = $ua->request($Request);
    return $Response;
}


sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -storename  name of store
flags:
    -help       show the command line options
    -nooutput   do not print any output

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
