#========================================================================================
# package      DE_EPAGES::RemoteSearch::API::Object::IndexManager
# state        public
# description  Package used to control shop index creation and remote search settings.
#               Catches errors and saves them as warnings.
#               Use <function DE_EPAGES::Core::API::Warning::BlockWarnings> to handle warnings.
#----------------------------------------------------------------------------------------
# example      my $ManagerParams = DE_EPAGES::RemoteSearch::API::Object::IndexManagerParams->new(
#                 create => 1
#               );
#               my $Manager = DE_EPAGES::RemoteSearch::API::Object::IndexManager->new(params => $ManagerParams);
#               my $aWarnings = BlockWarnings(sub {
#                 $Manager->manageShopsByAlias(['DemoShop']);
#               });
#========================================================================================
package DE_EPAGES::RemoteSearch::API::Object::IndexManager;

use strict;
use feature 'say';
use metaclass (
    metaclass   => 'Moose::Meta::Class',
    error_class => 'DE_EPAGES::Core::API::MooseError',
);

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw ( TestArray TestObject );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObject
    LoadClassByAlias
    LoadObject
    LoadObjectByGUID
    LoadObjectByPath
    LoadRootObject
    Transaction
);
use DE_EPAGES::RemoteSearch::API::IndexStatus qw ( IndexStatusID );
use DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient;
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw (
    DropAllSearchIndexes
    GetOrCreateSearchIndex
);
use DE_EPAGES::RemoteSearch::API::Table::SearchReplicationToken qw (
    IncrementAndGetReplicationToken
);
use Moose;

use constant MAX_WAIT_SECONDS => 8;
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IndexManager = DE_EPAGES::RemoteSearch::API::Object::IndexManager->new(params => $Params);
#----------------------------------------------------------------------------------------
# description  Execute various index management tasks. Tasks modify remote
#               search related attributes on Shop/RootObject and/or communicate with
#               remote search service host.
#----------------------------------------------------------------------------------------
# input        %params | Initial values. Used keys: <ul>
#                   <li>params - index manager params - DE_EPAGES::RemoteSearch::API::Object::IndexManagerParams</li>
#                 </ul> | hash
# return       $IndexManager | new instance | DE_EPAGES::RemoteSearch::API::Object::IndexManager
#========================================================================================


# tasksToCheck and indexesToFlush are queues for DE_EPAGES::RemoteSearch::API::DTO::Task objects
# "traits" says this attribute is a *list* and "isa" declares list elements as references of generic Items
#
# With "handles" custom aliases for standard list operations are defined e.g.
# handles => {
#   countItemsInList => 'count',
#   addItemToList => 'push',
#   getItemWithIndex => 'get',
# }
# This is syntactic sugar to improve clearity of the code.
# See http://search.cpan.org/dist/Moose/lib/Moose/Meta/Attribute/Native/Trait/Array.pm
has tasksToCheck   =>
    is      => 'rw',
    traits  => ['Array'],
    isa     => 'ArrayRef[Item]',
    default => sub { [] },
    handles => {
          addTaskToCheck   => 'push',
          nextTaskToCheck  => 'shift',
          hasTasksToCheck => 'count',
    };

has indexesToFlush   =>
    is      => 'rw',
    traits  => ['Array'],
    isa     => 'ArrayRef[Item]',
    default => sub { [] },
    handles => {
          addIndexToFlush   => 'push',
          nextIndexToFlush  => 'shift',
          hasIndexesToFlush => 'count',
    };

has params   =>
    is       => 'ro',
    required => 1;

sub ErrorDefinition {
    return {
        'NOT_A_SHOP_GUID' => 'This GUID does not belong to a shop: #GUID',
        'ERROR_INITIALIZING_INDEX' => 'Could not initialize index #IndexPath. please check this task url for errors: #UpdateTaskURL',
        'MALFORMED_URL' => 'Error: URL (#URL) must contain trailing slash e.g. http://<host>:<port>/rs/',
    };
}

#========================================================================================
# function     manageCompleteDB
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Manager->manageCompleteDB();
#----------------------------------------------------------------------------------------
# description  Works on all shops within the /Shops folder
#========================================================================================
sub manageCompleteDB {
    my $self        = shift;
    my $System      = LoadRootObject();
    my @ShopIDs     = @{$System->child('Shops')->findChildren()};
    my @PrepShopIDs = $System->existsChild('PrepShops') ? @{$System->child('PrepShops')->findChildren()} : ();
    foreach my $ShopID (@ShopIDs, @PrepShopIDs) {
        $self->_manageIndexForShop(LoadObject($ShopID));
    }
    $self->_checkStatus();
}

#========================================================================================
# function     manageSystem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Manager->manageSystem();
#----------------------------------------------------------------------------------------
# description  Sets system parameters.
#========================================================================================
sub manageSystem() {
    my $self = shift;

    if ( $self->params->updateAttributes ) {

        # logging cannot be set on system level.
        $self->params->log(undef);
        $self->_updateAttributes( LoadRootObject() );
    }
    if ( $self->params->info ) {
        $self->_infoForSystem();
    }
}

#========================================================================================
# function     manageShopsByGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Manager->manageShopsByGUID( $aGUIDs );
# example      $Manager->manageShopsByGUID( [ 'GUID1', 'GUID2' ] );
#----------------------------------------------------------------------------------------
# description  works on shops by guid
#----------------------------------------------------------------------------------------
# input        $aGUIDs | GUID list | ref.array.string
#========================================================================================
sub manageShopsByGUID {
    my $self = shift;
    my ($aGUIDs) = @_;

    TestArray( 'aGUIDs', $aGUIDs );

    #Create index for shops
    foreach my $GUID (@$aGUIDs) {
        eval {
            my $Shop = LoadObjectByGUID($GUID);
            Error( 'NOT_A_SHOP_GUID', { GUID => $GUID } ) unless ( $Shop->instanceOf('Shop') );
            $self->_manageIndexForShop($Shop);
        };
        if (ExistsError) {
            my $Error = GetError();
            Error($Error) unless ($Error->code eq 'NOT_A_SHOP_GUID' || $Error->code eq 'NoResults_ObjectByGUID');
            Warning($Error);
        }
    }
    $self->_checkStatus();
}

#========================================================================================
# function     manageShopsByAlias
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Manager->manageShopsByAlias( $aAliases );
# example      $Manager->manageShopsByAlias([ 'Shop1', 'Shop2' ] );
#----------------------------------------------------------------------------------------
# description  Manages a list of given shops by their alias. In order to manage all
#               shops at once, please refer to manageCompleteDB.
#----------------------------------------------------------------------------------------
# input        $aAliases | Alias list | ref.array.string
#========================================================================================
sub manageShopsByAlias {
    my $self = shift;
    my ($aShopAliases) = @_;
    TestArray( 'aShopAliases', $aShopAliases );

    my $ShopFolder = LoadObjectByPath('/Shops');
    foreach my $Alias (@$aShopAliases) {
        eval { $self->_manageIndexForShop($ShopFolder->child($Alias) ); };
        if (ExistsError) {
            my $Error = GetError();
            Error($Error) unless ($Error->code eq 'NoResults_ChildAlias');
            Warning($Error);
        }
    }
    $self->_checkStatus();
}

#========================================================================================
# function     manageShopsByIndexStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $manager->manageShopsByIndexStatus( INDEXSTATUS_ERROR );
#----------------------------------------------------------------------------------------
# description  Works on shops based on the current index status
#               See <package DE_EPAGES::RemoteSearch::API::IndexStatus>
#----------------------------------------------------------------------------------------
# input        $IndexStatus | Index status | string
#========================================================================================
sub manageShopsByIndexStatus {
    my $self = shift;
    my ($IndexStatus) = @_;
    my $Class    = LoadClassByAlias('SearchIndex');
    my $aSearchIndexIDs    = $Class->findObjects('ByIndexStatus',[IndexStatusID($IndexStatus)]);

    foreach my $SearchIndexID (@$aSearchIndexIDs){
        $self->_manageIndexForShop(LoadObject($SearchIndexID)->getSite );
    }
    $self->_checkStatus();
}

#========================================================================================
# function     _checkStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       $manager->_checkStatus();
#----------------------------------------------------------------------------------------
# description  Checks the list of indexes that have been touched for completion of their
#               respective task.
#               This method returns directly, unless statusCheckLoop is set on the manager params
#               <package DE_EPAGES::RemoteSearch::API::Object::IndexManagerParams>
#========================================================================================
sub _checkStatus {
    my $self = shift;
    if ($self->hasIndexesToFlush && ! $self->params->singleShopCalls) {
        $self->_flush();
    }
    return unless ($self->params->statusCheckLoops);
    my $RemoteIndexClient = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->create(LoadRootObject());

    while ($self->hasTasksToCheck) {
        my ($TaskURL, $aIndexes) = @{$self->nextTaskToCheck};
        print 'checking task status for ' . $TaskURL unless ( $self->params->silent );
        # caching the index object can be problematic,
        # e.g. encountered by calling "-reinit -shop DemoShop DemoShop", so we have to make sure the
        # index object still exists.
        my $Task = undef;
        my $Loops = $self->params->statusCheckLoops();
        my $Wait = 2;
        my $i = 0;

        while ( $i < $Loops ) {
            print '.' unless ($self->params->silent);
            $Task = $RemoteIndexClient->getTask($TaskURL);
            last unless ElementOf($Task->state, qw(Unscheduled Pending Delayed Recovering));
            $Wait++ if ( $Wait < MAX_WAIT_SECONDS );
            sleep($Wait);
            $i++;
        }

        print "\n" unless ($self->params->silent);

        say ' > checked ' . $Task->link . ' with result: ' . $Task->state unless ( $self->params->silent );
        say STDERR $Task->error if ($Task->error);
        Transaction(sub {
            foreach my $Index (@$aIndexes) {
                next unless ( ExistsObject( $Index->id ) );
                $Index->updateIndexStatus($Task);
                if ( !$Index->usableAtStorefront && !$self->params->ignoreIsReadyToUse() ) {
                    Warning('ERROR_INITIALIZING_INDEX', {
                        'IndexPath' => $Index->pathString,
                        'UpdateTaskURL' => $TaskURL,
                        'Error' => $Task->error,
                    });
                }
            }
        });
    }
}

#---------------------------------------------------------------------------
#  private helpers
#---------------------------------------------------------------------------
sub _flush {
    my $self = shift;

    return unless ($self->hasIndexesToFlush);
    my $System = LoadRootObject();
    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->create(
        $System,
        {
            BaseURL    => $self->params->url // $System->get('RemoteSearchURL'),
        }
    );
    my @Items;
    foreach my $Index ( @{$self->indexesToFlush}) {
        push @Items, {'resourceId' => $Index->getSite->get('GUID')};
    }
    my $Task = $Client->indexShops(\@Items, IncrementAndGetReplicationToken());
    say ' > resulting ticket: ' . $Task->link unless ( $self->params->silent );

    Transaction( sub {
        foreach my $Index ( @{ $self->indexesToFlush } ) {
            $Index->setNewIndexStatus( $Task->state );
        }
    });
    $self->addTaskToCheck([$Task->link, $self->indexesToFlush]);
    return $Task;
}

sub _manageIndexForShop {
    my $self = shift;
    my ($Shop) = @_;

    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    if ( $self->params->reinit || $self->params->drop ) {
        $self->_dropIndexForShop($Shop);
    }
    if ( $self->params->create ) {
         if ( $self->params->nosolr ) {
             $self->_updateShopCollection($Shop);
         } else {
             $self->_recreateIndexForShop($Shop);
         }
    }
    if ( $self->params->updateAttributes && !$self->params->nosolr) {
        $self->_updateAttributes($Shop);
    }
    if ( $self->params->info ) {
        $self->_infoForShop($Shop);
    }
}

sub _updateAttributes {
    my $self = shift;
    my ($Object) = @_;
    if ( defined $self->params->enabled ) {
        $Object->set( { RemoteSearchEnabled => $self->params->enabled } );
    }
    if ( defined $self->params->scheduleUpdates ) {
        $Object->set( { 'RemoteSearchScheduleUpdates' => $self->params->scheduleUpdates } );
    }
    if (defined $self->params->log) {
        if ($Object->instanceOf('Shop')) {
            my $Index = $Object->get('RemoteProductSearchIndex');
            if (defined $Index) {
                $Index->set({'Logging' => $self->params->log });
            }else{
                Error('RemoteProductSearchIndex don\'t exists yet. Create Index first, than activate logging');
            }
        } else {
            $Object->set({'Logging' => $self->params->log });
        }
    }
}

sub _infoForShop {
    my $self = shift;
    my ($Shop) = @_;
    return if ( $self->params->silent );
    my $Index = $Shop->get('RemoteProductSearchIndex');

    if ( !defined $Index ) {
        say 'Shop ' . $self->_printShopString($Shop) . ' Index not initialized.';
    }
    else {
        say sprintf 'Shop %s IndexStatusString=%s RemoteSearchURL=%s Logging=%s',
        $self->_printShopString($Shop),
        $Index->get('IndexStatusString'),
        $Index->get('RemoteSearchURL'),
        $Index->get('Logging') ? 'On' : 'Off';
    }

    return;
}

sub _infoForSystem {
    my $self = shift;
    return if ( $self->params->silent );
    my $System = LoadRootObject();
    printf "System info: RemoteSearchEnabled=%s RemoteSearchURL=%s RemoteSearchScheduleUpdates=%s\n",
    $System->get('RemoteSearchEnabled'),
    $System->get('RemoteSearchURL') // 'undef',
    $System->get('RemoteSearchScheduleUpdates') // 'undef';
    return;
}

sub _printShopString {
    my $self = shift;
    my ($Shop) = @_;
    return $Shop->alias . ' (ShopGUID: ' . $Shop->get('GUID') . ' DB: ' . GetCurrentDBHandle->cacheKey . ')';
}

sub _recreateIndexForShop {
    my $self = shift;
    my ($Shop) = @_;
    say '(re-)creating index for ' . $self->_printShopString($Shop) unless ( $self->params->silent );
    return if ( $self->params->testing );
    my $SearchIndex = GetOrCreateSearchIndex($Shop);
    if ( $SearchIndex->usableAtStorefront() && !$self->params->reinit && !$self->params->update ) {
        say ' index already created. use -reinit or -update parameter to recreate this index.'
        unless ( $self->params->silent );
    } else {
        $SearchIndex->set( { 'Logging' => $self->params->log } ) if (defined $self->params->log);
        if ($self->params->singleShopCalls) {
            my $Task = $SearchIndex->initialize();
            say ' > resulting ticket: ' . $Task->link unless ( $self->params->silent );
            $self->addTaskToCheck([$Task->link, [$SearchIndex]]);
        } else {
            $SearchIndex->setNewIndexStatus('Init');
            $self->addIndexToFlush($SearchIndex);
        }
    }
}

sub _updateShopCollection {
    my $self = shift;
    my ($Shop) = @_;
    say 'updating shop collection only for ' . $self->_printShopString($Shop) unless ( $self->params->silent );
    return if ( $self->params->testing );
    my $Index = GetOrCreateSearchIndex($Shop);
    $Index->set( { 'Logging' => $self->params->log } ) if (defined $self->params->log);
    $Index->initializeNosolr();
    say ' > done for ' . $self->_printShopString($Shop) unless ( $self->params->silent );
}

sub _dropIndexForShop {
    my $self = shift;
    my ($Shop) = @_;
    say 'dropping index for ' . $self->_printShopString($Shop) unless ( $self->params->silent );
    DropAllSearchIndexes( $Shop, $self->params->drop ) if ( !$self->params->testing );
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;
