#========================================================================================
# package      DE_EPAGES::SEO::API::XML::SitemapExportDriver
# state        private
#----------------------------------------------------------------------------------------
# description  This module implements a xml driver to export the sitemap of the shop
#
# example      use DE_EPAGES::SEO::API::XML::SitemapExportDriver;
#
#               my $Driver = DE_EPAGES::SEO::API::XML::SitemapExportDriver->new($hClassesMap);
#               $Driver->exportXML($SitemapFileName, $MobileSitemapFileName, $Shop,
#                                  $aContentObjectsData, \@MobileBlackListedClassIDs);
#========================================================================================
package DE_EPAGES::SEO::API::XML::SitemapExportDriver;
use base DE_EPAGES::XML::API::BaseExportDriver;

use strict;
use File::Basename qw(dirname);
use XML::SAX::Writer;
use XML::Filter::DataIndenter;
use DE_EPAGES::XML::API::SAXWriter;
use DE_EPAGES::Core::API::Error qw(VarsOnError);
use DE_EPAGES::Core::API::File qw(CreateDirectory SetAccessRights);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::TestParameter qw(TestString);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Driver = DE_EPAGES::SEO::API::XML::SitemapExportDriver->new($hClassIDs);
# example      my $hClassesMap = {
#                   'StartPage' => LoadClassByAlias('StartPage')->id,
#                   'Imprint' => LoadClassByAlias('Imprint')->id,
#               };
#               my $Driver = DE_EPAGES::SEO::API::XML::SitemapExportDriver->new($hClassIDs);
#----------------------------------------------------------------------------------------
# description  Creates a driver object for exporting XML sitemap files.
#----------------------------------------------------------------------------------------
# input        $hClassIDs | class object ids for priorities of the URLs | hash.ref
# return       $Driver | SAX driver | object
#========================================================================================
sub new {
    my $class = shift;
    my ($hClassIDs) = @_;
    my $self = $class->SUPER::new();
    $self->{'ClassesMap'} = { %$hClassIDs };
    return $self;
}

#========================================================================================
# function     exportXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->exportXML($PathFileName, $MobilePathFileName, $Shop,
#                                $ahData, $aMobileBlackListedClassIDs);
#----------------------------------------------------------------------------------------
# description  Exports URL data from content objects into an XML file.
#----------------------------------------------------------------------------------------
# input        $PathFileName | name of sitemap xml file | string
# input        $MobilePathFileName | name of mobile sitemap xml file | string
# input        $Shop | shop object | object
# input        $ahData | content objects data | ref.array.hash
# input        $aMobileBlackListedClassIDs | list of class ids for filtered mobile items | ref.array
#========================================================================================
sub exportXML {
    my $self = shift;
    my ($PathFileName, $MobilePathFileName, $Shop, $ahData, $aMobileBlackListedClassIDs) = @_;

    $self->addSitemapHooks;
    $self->{'Shop'} = $Shop;
    $self->{'Items'} = $ahData;
    $self->{'IsMobile'} = 0;
    $self->{'HasSSLCertificate'} = $Shop->get('HasSSLCertificate');

    my $hURLs = $Shop->get(['ProtocolAndServer',
                            'ProtocolAndServerSSL',
                            'WebServerScriptUrl',
                            'WebServerScriptUrlMobile',
                            'WebServerScriptUrlMobileSSL']);
    $self->{'ProtocolAndServer'} = $hURLs->{'ProtocolAndServer'};
    $self->{'ProtocolAndServerSSL'} = $hURLs->{'ProtocolAndServerSSL'};
    $self->{'ShopWebUrl'} = $hURLs->{'WebServerScriptUrl'};
    $self->{'ShopWebUrlMobile'} = $hURLs->{'WebServerScriptUrlMobile'};
    $self->{'ShopWebUrlMobileSSL'} = $hURLs->{'WebServerScriptUrlMobileSSL'};

    TestString('PathFileName', $PathFileName);

    my $hFormat = {
        'encoding' => 'UTF-8',
        'writer'   => 'DE_EPAGES::XML::API::SAXWriter',
    };

    CreateDirectory( dirname($PathFileName) );
    my $Writer = XML::SAX::Writer->new({
        Writer      => $hFormat->{'writer'},
        Output      => $PathFileName,
        EncodeTo    => $hFormat->{'encoding'},
    });
    my $Filter = XML::Filter::DataIndenter->new( 'Handler' => $Writer );
    $self->set_handler( $Filter );
    eval {
        $self->parse();
    };
    VarsOnError({ 'PathFileName' => $PathFileName });
    SetAccessRights($PathFileName);
    if($MobilePathFileName) {
        TestString('MobilePathFileName', $MobilePathFileName);
        my @MobileData = grep { not ElementOf($_->{'ClassID'}, $aMobileBlackListedClassIDs) } @$ahData;

        $self->{'Items'} = \@MobileData;
        $self->{'IsMobile'} = 1;
        $self->addHook( 'url' => 'Mobile' );
        my $MobileWriter = XML::SAX::Writer->new({
            Writer      => $hFormat->{'writer'},
            Output      => $MobilePathFileName,
            EncodeTo    => $hFormat->{'encoding'},
        });
        my $MobileFilter = XML::Filter::DataIndenter->new( 'Handler' => $MobileWriter );
        $self->set_handler( $MobileFilter );
        eval {
            $self->parse();
        };
        VarsOnError({ 'MobilePathFileName' => $MobilePathFileName });
        SetAccessRights($MobilePathFileName);
    }
    return;
}

#========================================================================================
# function     addSitemapHooks
# state        public
#----------------------------------------------------------------------------------------
# syntax       addSitemapHooks();
# example      $self->addSitemapHooks();
#----------------------------------------------------------------------------------------
# description  Adds hook function to create sub elements within a given parent element.
#========================================================================================
sub addSitemapHooks {
    my $self = shift;
    $self->addHook( 'urlset' => 'Urls' );
    $self->addHook( 'url' => 'Loc' );
    $self->addHook( 'url' => 'Priority' );
}

#========================================================================================
# function     drive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->drive();
#----------------------------------------------------------------------------------------
# description  Starts export of items.
#========================================================================================
sub drive {
    my $self = shift;
    $self->addElement( 'urlset', {});
}

# adds xml-tag <urlset> with xmlns
sub urlset {
    my $self = shift;
    if($self->{'IsMobile'}) {
        return {
            'xmlns' => 'http://www.sitemaps.org/schemas/sitemap/0.9',
            'xmlns:mobile' => 'http://www.google.com/schemas/sitemap-mobile/1.0',
        };
    } else {
        return { 'xmlns' => 'http://www.sitemaps.org/schemas/sitemap/0.9' };
    }
}

# triggered by hook ('urlset' => 'Urls')
# adds xml-tags <url>
sub Urls {
    my $self = shift;

    foreach my $ContentObject ( @{ $self->items() } ) {
        $self->addElement( 'url', { 'url' => $ContentObject } );
    }
}

# triggered by hook ('url' => 'Loc')
# adds xml-tags <loc> with location/url to contentobject in shop
sub Loc {
    my $self = shift;
    my $ObjectData = $self->topID('url');

    my $URL = $ObjectData->{'URL'};

    if ($self->{'HasSSLCertificate'}) {
        if($self->{'IsMobile'}) {
            # change servlet in URLs to .mobile and https
            $URL =~ s/\Q$self->{'ShopWebUrl'}\E/$self->{'ShopWebUrlMobileSSL'}/;
        }
        # change URLs that don't contain /epages/<ShopName>.{mobile|sf} to https
        $URL =~ s/\Q$self->{'ProtocolAndServer'}\E/$self->{'ProtocolAndServerSSL'}/;
    } else {
        if($self->{'IsMobile'}) {
            # change servlet in URLs to .mobile
            $URL =~ s/\Q$self->{'ShopWebUrl'}\E/$self->{'ShopWebUrlMobile'}/;
        }
    }

    $self->addElement( 'loc', {}, $URL );
}

# triggered by hook ('url' => 'Priority')
# adds xml-tags <priority> with priority for google index; default is 0.5
sub Priority {
    my $self = shift;
    my $ObjectData     = $self->topID('url');
    my $ClassID = $ObjectData->{'ClassID'};
    my $ClassesMap = $self->{'ClassesMap'};
    if ($ClassID == $ClassesMap->{'StartPage'}){
        $self->addElement( 'priority', {}, 1 );
    } elsif ($ClassID == $ClassesMap->{'Imprint'}){
        $self->addElement( 'priority', {}, 0.0 );
    } elsif ($ClassID == $ClassesMap->{'CustomerInformation'}){
        $self->addElement( 'priority', {}, 0.0 );
    } elsif ($ClassID == $ClassesMap->{'PrivacyPolicy'}){
        $self->addElement( 'priority', {}, 0.0 );
    } elsif ($ClassID == $ClassesMap->{'TermsAndConditions'}){
        $self->addElement( 'priority', {}, 0.0 );
    }
}

# triggered by hook ('url' => 'Mobile')
# adds xml-tags <mobile:mobile/>
sub Mobile {
    my $self = shift;
    $self->addElement( 'mobile:mobile', {} );
}

1;
