#========================================================================================
# description  adds the default set of widgets to existing shops after patch to 6.11.0.
#========================================================================================
use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script            qw( RunScript );
use DE_EPAGES::Core::API::PerlTools         qw( ElementOf );
use DE_EPAGES::Database::API::Connection    qw( RunOnStore );
use DE_EPAGES::Object::API::Factory         qw( LoadObject LoadRootObject );
use DE_EPAGES::Shop::API::System            qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

my @DefaultWidgetTypeAliases = qw(
    SalesVolumeUnity
);
my $Output = 1;

use constant POSITION_STEP => 10;
use constant PROGRESS_STEP => 100;

sub AddWidgetsToShop {
    my ($Shop, $aWidgetTypeAliases, $DeleteObsolete) = @_;

    my $aMerchants = $Shop->child('Groups')->child('Merchants')->directMembers;
    my $WidgetTypesFolder = LoadRootObject()->child('WidgetTypes');

    foreach my $User ( @$aMerchants ) {
        my $WidgetFolder = $User->folder('MBOWidgets');
        my $Position = POSITION_STEP;

        # delete Widgets wich are not in new widget list
        if($DeleteObsolete){
            foreach my $Widget (@{$WidgetFolder->children}){
                $Widget->delete() unless ElementOf($Widget->alias, $aWidgetTypeAliases);
            }
        }

        # add new widgets
        foreach my $Alias (@$aWidgetTypeAliases) {
            if (!$WidgetTypesFolder->existsChild($Alias)) {
                print STDERR "Warning: widget $Alias does not exists.\n";
                next;
            }
            my $WidgetType = $WidgetTypesFolder->child($Alias);
            my $Feature = $WidgetType->get('Feature');
            if( !defined $Feature || $Shop->featureMaxValue($Feature) ) {
                $WidgetFolder->insertChild('Widget', {
                    'WidgetType' => $WidgetType,
                    'Alias' => $WidgetType->alias,
                    'Position' => $Position
                }) unless $WidgetFolder->existsChild($WidgetType->alias) ;
                $Position += POSITION_STEP;
            }
        }
    }
}

# we get all unity shops by using sql
# and testing the featuremaxvalue
sub AddWidgetsToAllUnityShops {
    my ($aWidgetTypeAliases, $DeleteObsolete, $aShopIDs) = @_;

    foreach my $ShopID( @$aShopIDs ) {
        my $Shop = LoadObject( $ShopID );
        printf "Adding Widgets to shop %s\n", $Shop->alias if $Output;
        AddWidgetsToShop($Shop, $aWidgetTypeAliases, $DeleteObsolete);
    }

    return;
}

sub findSingleUnityShop {
    my ($ShopAlias) = @_;

    my $ShopFolder = LoadRootObject()->child('Shops');
    if (!$ShopFolder->existsChild($ShopAlias)) {
        printf("Shop is not existing: %s\n", $ShopAlias);
        exit 2;
    }
    my $Shop = $ShopFolder->child($ShopAlias);
    if (!$Shop->get('IsUnityRegistered')) {
        printf("Shop is not a Unity shop: %s\n", $ShopAlias);
        exit 2;
    }
    return $Shop->id;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Widgets;
    my $Help;
    my $DeleteObsolete;
    my $ShopAlias;

    GetOptions(
        'help'           => \$Help,
        'passwd=s'       => \$Password,
        'storename=s'    => \$StoreName,
        'widgets=s'      => \$Widgets,
        'deleteobsolete' => \$DeleteObsolete,
        'output!'        => \$Output,
        'shop=s'         => \$ShopAlias,
    );

    usage() if $Help;

    unless( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    my @WidgetTypeAliases = ();
    if (defined $Widgets) {
        @WidgetTypeAliases = split(/,/, $Widgets);
    }
    else {
        @WidgetTypeAliases = @DefaultWidgetTypeAliases;
    }
    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            my $aShopList = defined $ShopAlias ? [ findSingleUnityShop($ShopAlias) ] : GetUnityShopIDs();
            AddWidgetsToAllUnityShops(\@WidgetTypeAliases, $DeleteObsolete, $aShopList);
        },
    );

    SynchronizeCache();

    printf "need %.1f seconds\n", time()-$time1 if $Output;
    return;
}

sub usage {
    print <<END_USAGE;
Description:
    adds some unity default widgets for all merchants of all unity shops
    and delete obsolete non unity widgets

Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -widgets    comma separated list of widget aliases (optional, default is "SalesVolumeUnity")
    -shop       shop alias (optional, if unset runs for all shops)
flags:
    -help              show the command line options
    -deleteobsolete    all other widgets are deleted before

Example:
    perl $0 -storename Store
    perl $0 -storename Store -shop DemoShop
    perl $0 -storename Store -widgets "SalesVolumeUnity" -deleteobsolete
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
