use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadObject );
use DE_EPAGES::ShopMigration::API::MigrationShop qw( SetShopFeatureValues );
use DE_EPAGES::Shop::API::Feature qw ( ExistsFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use Getopt::Long;
use strict;

RunScript( 'Sub' => \&Main, 'Message'  =>'long' );

sub Main {
    local $| = 1;

    my %Options = ();
    my @GetOpt = qw(help passwd=s storename=s feature=s maxvalue=s);
    usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    for (qw(storename feature maxvalue)) {
        usage(sprintf('Missing parameter -%s', $_)) unless defined $Options{$_};
    }
    my $FeatureAlias = $Options{feature};
    my $MaxValue     = $Options{maxvalue};

    RunOnStore(
        Store => $Options{storename},
        Sub => sub  {
            usage(sprintf('Unknown feature: %s', $FeatureAlias)) unless ExistsFeatureByAlias($FeatureAlias);
            SetFeatureOnAllPreps( $FeatureAlias, $MaxValue );
        },
    );
    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

}

sub SetFeatureOnAllPreps {
    my ( $FeatureAlias, $MaxValue ) = @_;

    print "Reading all prep shops...\n";
    my $dbi          = GetCurrentDBHandle();
    my $ShopFolderID = LoadRootObject()->child('PrepShops')->id;

    # gets all precreated shops
    my $raaResults = $dbi->execute( <<END_SQL, [], [qw( Integer )] );
        select shopid from shop inner
        join object on objectid = shopid
        where parentid = $ShopFolderID and
        alias like 'prep%'
END_SQL
    my $Count = scalar @$raaResults;
    print "$Count shops found\n";
    print "Updating $Count Shops\n";
    my $Index = 0;

    foreach my $ahShop (@$raaResults) {
        $Index++;
        my $Shop = LoadObject( $ahShop->[0] );
        print "Updating shop $Index ($Count) " . $Shop->get('Alias') . "\n";
        SetShopFeatureValues($Shop, [
            { Alias => $FeatureAlias, MaxValue => $MaxValue},
        ]);
    }
}

sub usage {
    my ($ErrorNote) = @_;

    print STDERR "\n$ErrorNote\n\n" if $ErrorNote;
    print <<END_USAGE;
Description
  This scripts sets a single shop feature to a given MaxValue for all prepshops in the current store.
  Can also be used to unset features.
  Note: Those values are reverted on global apply (e.g. PBO/Scripts/Post-Patch)!

Usage:
    perl $0 [options] flags objects

options:
    -passwd      database user password                                    ()
    -storename   name of store                                             required
    -feature     feature alias                                             required
    -maxvalue    use this for MaxValue for feature                         required

flags:
    -help        show the command line options                             ()

Example:
    perl $0 -storename Store -feature ProductCustomization -maxvalue 0
    perl $0 -storename Store -feature ProductTypes -maxvalue 100
END_USAGE
    exit 2;
}
