    #========================================================================================
# package      DE_EPAGES::ShopWizard::API::DemoContent
# state        public
# description  Functions to handle demo content
#========================================================================================
package DE_EPAGES::ShopWizard::API::DemoContent;
use base qw( Exporter );

use strict;

our @EXPORT_OK = qw ( AddEBizElementsToNavBar ImportDemoContent ProcessEbizContent );

use strict;

use DE_EPAGES::Address::API::GeoData qw ( GetGeoLocation );
use DE_EPAGES::Content::API::ShortUrl qw (GetShortUrlByClass);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject );
use DE_EPAGES::Object::API::Language qw (GetCurrentLanguageID);
use DE_EPAGES::Presentation::API::Object::TextProcessor;
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use DE_EPAGES::ShopWizard::API::XML::HashImportHandler qw ( ImportDemoContentData );

sub ErrorDefinition { return {
        'DemoContentNotExists' => 'The category \'#Path\' does not exist in the demo content of style group #StyleGroup.',
    }
}

#========================================================================================
# function     AddEBizElementsToNavBar
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddEBizElementsToNavBar( $Shop, $Style );
#----------------------------------------------------------------------------------------
# description  Add the contactbox and businesshoursbox navelement to the left or right navbar.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $Style | style object | object
#                           | ref.array.string
#========================================================================================
sub AddEBizElementsToNavBar($$) {
    my ($Shop, $Style) = @_;

    # Check if location page exists -> eBiz shoptype.
    my $Categories = $Shop->get('Categories');
    return unless $Categories->existsChild('Location');
    my $Location = $Categories->child('Location');
    return unless $Location->instanceOf('Category');

    # Get navbars object.
    my $NavBars = LoadRootObject()->child('NavBars');

    # Array with navigation navelements.
    my %NavigationElements =
        map { $_ => undef; } (qw( CategoriesTreeBox CategoriesLinkList CategoriesListBox CategoriesListWithImage
                                  DropDownList MegaMenu FoldingCategoriesListBox ));
    my $HasContactOrBusinessHoursBox = 0;

    # Loop over visible navbars.
    foreach my $NavBar ( @{$Style->get('VisibleNavBars')} ) {
        my $NavbarAlias = $NavBar->alias;
        # Load navbar.
        next unless $NavBars->existsChild($NavbarAlias);;
        my $NavBar = $NavBars->child($NavbarAlias);

        # Get navbarelements for navbar.
        my $aNavBarElements = $Style->navbarElements($NavbarAlias);
        # Build hash: key = navbarelement alias, value = navelement alias.
        my %NavElementAlias = map {$_->alias() => $_->get('NavElement')->alias() } @$aNavBarElements;

        # Check if there is no HomePageLink already a navbarelement of type HomePageLink in the navbar.
        if (not ElementOf('HomePageLink', values %NavElementAlias)) {
            # Loop over navbarelements.
            foreach my $NavBarElement (@$aNavBarElements) {
                my $NavElementAlias = $NavBarElement->get('NavElement')->alias;
                # Check if NavBarElement is an navigation element (of any type in %NavigationElements).
                if(exists $NavigationElements{$NavElementAlias}) {
                    # Add HomePageLink before the navigation element.
                    _AddNavElement($Style, $NavBar, 'HomePageLink', $NavBarElement->get('Position') - 1, 1);
                    last;
                }
            }
        }

        # Check if left or right navbar.
        if (($NavbarAlias eq 'Left' || $NavbarAlias eq 'Right') and not $HasContactOrBusinessHoursBox) {
            # Find the maximum position for navelement.
            my $Position = 0;
            foreach my $Element ( @$aNavBarElements ) {
                $Position = $Element->get('Position') if $Position <= $Element->get('Position');
            }
            # Add elements.
            my $ContactBox = _AddNavElement($Style, $NavBar, 'ContactBox', $Position + 1, 0);
            if (defined $ContactBox) {
                $Position = $ContactBox->get('Position');
            }
            _AddNavElement($Style, $NavBar, 'BusinessHoursBox', $Position + 1, 0);
            # Flag to add elements only in one navbar - left or right.
            $HasContactOrBusinessHoursBox = 1;
        }
    }

    # Delete not default eBiz-NavElements (LoginBox, LogoutLink, MyAccountLink).
    _DeleteElements($Style);
    return;
}

#========================================================================================
# function     ImportDemoContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImportDemoContent( $Shop, $aRelativePaths );
# example      ImportDemoContent( $Shop, ['Categories/AtHome/BathRooms','Categories/AtWork/Chairs'] );
#----------------------------------------------------------------------------------------
# description  Imports selected object from the demo content into into a shop.
#               The content of already existing objects is not changed.
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $aRelativePaths | list relative names of demo content objects. These
#                           must belong to the demo content of the current stlye group
#                           of the shop ($Shop->get('SelectedStyleGroup'))
#                           | ref.array.string
# return       $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub ImportDemoContent {
    my ($Shop, $aRelativePaths)  = @_;

    # Collect language codes and ids.
    my %LanguageKeys = ();
    $LanguageKeys{$_->{'Code2'}} = $_->{'LanguageID'} foreach @{$Shop->get('Languages')};

    # Get demo content for languages.
    my $StyleGroup = $Shop->get('SelectedStyleGroup');
    my $hDemoContent = ImportDemoContentData($StyleGroup->get('DemoContentFiles'), [ keys %LanguageKeys ]);

    # Import content to shop.
    foreach my $Path (grep { not defined $Shop->findPath($_) } sort @$aRelativePaths) {
        my $hContent = $hDemoContent->{$Path};
        Error( 'DemoContentNotExists', {
            'Path' => $Path,
            'StyleGroup' => $StyleGroup->get('Path')
        } ) unless defined $hContent;

        my $Class = LoadClassByAlias($hContent->{'Class'});
        my $Object = $Class->insertObject({
            'Parent' => $Shop->path($hContent->{'ParentPath'}),
            %{$hContent->{'Attributes'}}
        });

        my @URIs;
        foreach my $LanguageCode (keys %LanguageKeys) {
            my $hValues = $hContent->{'LocalizedAttributes'}{$LanguageCode};
            next unless defined $hValues;

            my $LanguageID = $LanguageKeys{$LanguageCode};
            $Object->set($hValues, $LanguageID);

            # Generate a short URL from the name.
            my $Name = $hValues->{'Name'};
            $Name = "$Name/$LanguageCode" if ElementOf($Name, @URIs);
            push @URIs, $Name;
            my $URI = GetShortUrlByClass($Shop->path($hContent->{'ParentPath'}), $Class, $Name, $LanguageID);
            $Object->set({ 'URI' => $URI }, $LanguageID);
        }
    }

    # Update the position numbers 10, 20, 30 ,... .
    $Shop->get('Categories')->renumberChildren();
}

#========================================================================================
# function     ProcessEbizContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       ProcessEbizContent( $Shop, $hAddress );
#----------------------------------------------------------------------------------------
# description  Checks if Shop.Category.Location with ebiz demo content exists and run TLE
#               processing on this content. $hAddress is needed for google maps api call
#               (see <package DE_EPAGES::Address::API::GeoData>)
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $hAddress | Address for Google Geocoding Lookup keys <ul>
#                                   <li>CountryID</li>
#                                   <li>Street</li>
#                                   <li>Street2</li>
#                                   <li>Zipcode</li>
#                                   <li>City</li>
#                               </ul> | hash.ref.
#========================================================================================
sub ProcessEbizContent {
    my ($Shop, $hAddress) = @_;

    # avoid not needed calls of google maps (_LookUpGeoLocation)
    return unless $Shop->get('InitialShopWizardStartup');
    my $Categories = $Shop->get('Categories');
    return unless $Categories->existsChild('Location');
    my $Location = $Categories->child('Location');
    return unless $Location->instanceOf('Category');

    # Set lat, lng for google maps gadget, default values = Hamburg.
    my $hGeoLocation = _LookUpGeoLocation( $hAddress ) // { 'lat' => '53.56', 'lng' => '9.96' };
    _ProcessEbizDemoContent($Shop, $Location, $hGeoLocation);

    return;
}

sub _AddNavElement {
    my ($Style, $NavBar, $Element, $Position, $GenerateAlias) = @_;

    # Get style navbarelement folder and all system navelements.
    my $ElementFolder = $Style->folder('NavBarElements');
    my $NavElements = LoadRootObject()->child('NavElements');

    # Get navelement to add; check if they exist.
    my $ElementClass = $NavElements->existsChild($Element) ? $NavElements->child($Element) : undef;

    # Add navelement to style navbarelements.
    my $LanguageID = GetCurrentLanguageID() // $Style->getSite()->get('LanguageID');
    my $Alias = $ElementClass->get( $ElementClass->get('IsCustomizable')
                ? 'NameOrAlias'  # localized element name (editable in MBO)
                : 'Alias', $LanguageID);     # NavElement alias is not visible in MBO;
    $Alias = $ElementFolder->nextAlias($Alias . ' %s') if $ElementFolder->existsChild($Alias) and $GenerateAlias;

    # Return navelement if it already exists.
    return $ElementFolder->child($Alias) if $ElementFolder->existsChild($Alias);

    my $NewNavElement = $ElementFolder->insertChild( $ElementClass, {
        'Alias'      => $Alias,
        'NavElement' => $ElementClass,
        'NavBar'     => $NavBar,
        'Position'   => $Position,
    });
    $ElementFolder->renumberChildren();

    return $NewNavElement;
}

# Look up geo information for $hAddr; return undef if no geo information are available.
sub _LookUpGeoLocation {
    my ($hAddr) = @_;

    my $CrountryClass    = LoadClassByAlias('Country');
    my $hParam = {
        'Country'     => $CrountryClass->loadObject( $hAddr->{'CountryID'} )->get('English'),
        'Street'      => $hAddr->{'Street'} // '' . ' ' . $hAddr->{'Street2'} // '',
        'Zipcode'     => $hAddr->{'Zipcode'},
        'City'        => $hAddr->{'City'},
    };

    # $hLocation stays undef if geo information is not available for $hAddress.
    my $hLocation = undef;
    eval { $hLocation = GetGeoLocation($hParam) };
    Warning(GetError) if ExistsError;

    return $hLocation;
}

# Process Location.Text in all shop languages.
sub _ProcessEbizDemoContent {
    my ( $Shop, $Location, $hGeo ) = @_;

    my $aLanguages = $Shop->get('Languages');
    my $hTLE = { 'Shop' => $Shop->tleHash(), 'Longitude' => $hGeo->{'lng'}, 'Latitude' => $hGeo->{'lat'}   }   ;
    my $PageType = LoadPageTypeByAlias('SF-Shop');
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Shop );
    my $Processor = DE_EPAGES::Presentation::API::Object::TextProcessor->new();

    foreach (@$aLanguages) {
        my $Text = $Location->get( 'Text', $_->{'LanguageID'} );
        next if ! defined $Text || $Text eq '';
        my $rResult = $Processor->processText( $ObjectPageType, $Shop, \$Text, $hTLE, $_->{'LanguageID'} );
        $Location->set( {'Text' => $$rResult}, $_->{'LanguageID'} );
    }
}

# Delete not default eBiz-NavElements.
sub _DeleteElements {
    my ( $Style ) = @_;

    my $NavBarElements = $Style->child('NavBarElements');
    foreach my $NavBarElement (@{$NavBarElements->children}) {
        my $NavElementAlias = $NavBarElement->get('NavElement')->alias;
        my @NavElementsToDelete = qw(MyAccountLink LoginBox LogoutLink ProductSearchLine);
        if(ElementOf($NavElementAlias, @NavElementsToDelete)) {
            $NavBarElement->delete;
        }
    }
}

1;
