#========================================================================================
# package      DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedly
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
#----------------------------------------------------------------------------------------
# description  object interface for Payment.
#========================================================================================
package DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedly;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( CopyFile DeleteFile ExistsFile );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject Transaction );
use DE_EPAGES::Payment::UI::PaymentMethod;
use DE_EPAGES::Spreedly::API::GatewayController;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use constant INDIVIDUAL_STEPS_ORDER_PROCESS_CODE => 1;

use strict;


#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $PaymentMethodSpreedly->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'PaymentMethodSpreedly' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $PaymentMethodSpreedly->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethodSpreedly = DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedly->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethodSpreedly');
#               my $PaymentMethodSpreedly = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertPaymentMethodSpreedly' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PaymentMethodSpreedly | new object | object
# hook         OBJ_InsertPaymentMethodSpreedly | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodSpreedlyID -                      - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertPaymentMethodSpreedly', {'Object' => $self, 'PaymentMethodSpreedlyID' => $self->id });
    delete $hInfo->{$_} foreach qw ( PaymentTypeMethod );
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethodSpreedly->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeletePaymentMethodSpreedly' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeletePaymentMethodSpreedly | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodSpreedlyID -                      - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeletePaymentMethodSpreedly', {'Object' => $self, 'PaymentMethodSpreedlyID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethodSpreedly->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdatePaymentMethodSpreedly' and 'OBJ_AfterUpdatePaymentMethodSpreedly'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdatePaymentMethodSpreedly | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodSpreedlyID -                      - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdatePaymentMethodSpreedly | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodSpreedlyID -                      - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdatePaymentMethodSpreedly', {'Object'=>$self, 'Values'=> $hValues, 'PaymentMethodSpreedlyID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdatePaymentMethodSpreedly',  {'Object'=>$self, 'Values'=> $hValues, 'PaymentMethodSpreedlyID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method is visible in the storefront.
#               Overrides function <function DE_EPAGES::Payment::API::Object::PaymentMethod::usableAtStorefront>
#               to evaluate if payment method is configured
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    my $PaymentTypeMethod = $self->get('PaymentTypeMethod');
    my $Shop = $self->getSite();


    my $hCredentials = $self->getGatewayCredentials($Shop);
    my $BasketCheckoutType = $Shop->get('OrderProcess');

    return isValid($hCredentials->{'token'})
        && $PaymentTypeMethod
        && ($BasketCheckoutType == INDIVIDUAL_STEPS_ORDER_PROCESS_CODE)
        && $self->SUPER::usableAtStorefront($hVars);
}

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               Valid currencies are: EUR, GBP, USD
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    if ( defined $hVars->{'LineItemContainer'} ) {
        $hVars->{'skipUsableAtStorefrontCheck'} = 1;
    }

    return $self->SUPER::canAddToBasket($hVars);
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedly->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    return {
        %{$class->SUPER::default($Parent, $hInfo)}
    };
}

#========================================================================================
# function     saveGatewayCredentials
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->saveGatewayCredentials( $Servlet );
#----------------------------------------------------------------------------------------
# description  It stores new gateway credentials.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub saveGatewayCredentials {
    my $self = shift;
    my ($Servlet) = @_;

    DE_EPAGES::Payment::UI::PaymentMethod->Save($Servlet);

    return;
}

#========================================================================================
# function     getGatewayCredentials
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hCredentials = $self->getGatewayCredentials( $Shop );
#----------------------------------------------------------------------------------------
# description  It recovers from epagesj gateway credentials.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# return       $hCredentials | hash with gateway credentials | hash ref.
#========================================================================================
sub getGatewayCredentials {
    my $self = shift;
    my ($Shop) = @_;

    my $PaymentType = $self->get('PaymentType');
    my $GatewayName = $PaymentType->get('GatewayName');

    my $hGatewayEntity = {};
    eval {
        $hGatewayEntity = DE_EPAGES::Spreedly::API::GatewayController->getToken($Shop, $GatewayName);
    };

    if ( ExistsError() ) {
        my $Error = GetError();
        LogPayment($PaymentType->alias(), 'CheckEpagesJConnection' , $Error->shortMessage());
    }

    return $hGatewayEntity;
}

#========================================================================================
# function     createNewPaymentMethodName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NewPaymentMethodName = $self->createNewPaymentMethodName($PaymentMethods, $BaseName);
# description  it returns a not repeated payment method name based on increasing a counter if
#               basename already exists
#----------------------------------------------------------------------------------------
# input        $PaymentMethods | array of payment method objects | ref.array
# input        $BaseName | base name | string
# return       $NewPaymentMethodName | resulting payment method name | string
#========================================================================================
sub createNewPaymentMethodName {
    my $self = shift;
    my ($PaymentMethods, $BaseName) = @_;

    my $aPaymentMethods = $PaymentMethods->children();
    my @PaymentMethodNames = map { $_->alias() } @$aPaymentMethods;

    my $NewPaymentMethodName = $BaseName;
    my $counter = 0;
    while (ElementOf($NewPaymentMethodName, @PaymentMethodNames)) {
        $counter++;
        $NewPaymentMethodName = $BaseName . '_' . $counter;
    }

    return $NewPaymentMethodName;
}

#========================================================================================
# function     setInShopNameLikePaymentTypeMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setInShopNameLikePaymentTypeMethod( $PaymentTypeMethodToMirror );
#----------------------------------------------------------------------------------------
# description  It renames in shop name to set payment type method name.
# input        $PaymentTypeMethodToMirror | payment type method object | object
#========================================================================================
sub setInShopNameLikePaymentTypeMethod {
    my $self = shift;
    my ($PaymentTypeMethodToMirror) = @_;

    if (!defined($PaymentTypeMethodToMirror)) {
        $PaymentTypeMethodToMirror = $self->get('PaymentTypeMethod');
    }

    return if (!defined($PaymentTypeMethodToMirror));

    my $aLanguages = $self->getSite()->get('Languages');
    $self->_Transaction(
            sub {
                $self->_changeInShopName($PaymentTypeMethodToMirror, $aLanguages);
            }
    );
}

#========================================================================================
# function     setPaymentMethodNameLikePaymentTypeMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setPaymentMethodNameLikePaymentTypeMethod( $PaymentTypeMethodToMirror, $LanguageID );
#----------------------------------------------------------------------------------------
# description  It renames Payment method name (shown in MBO and checkout).
#----------------------------------------------------------------------------------------
# input        $PaymentTypeMethodToMirror | Payment type method object to mirror names | object
# input        $LanguageID | language id | int
#========================================================================================
sub setPaymentMethodNameLikePaymentTypeMethod {
    my $self = shift;
    my ($PaymentTypeMethodToMirror, $LanguageID) = @_;

    my $NewPaymentMethodName = $self->createNewPaymentMethodName(
            $self->parent(), $PaymentTypeMethodToMirror->get('Name', $LanguageID)
    );

    $self->set( { 'Alias' => $NewPaymentMethodName } );
}

#========================================================================================
# function     uploadPaymentTypeMethodLogo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->uploadPaymentTypeMethodLogo( $PaymentTypeMethod );
#----------------------------------------------------------------------------------------
# description  It uploads payment type method logo for the payment method.
#----------------------------------------------------------------------------------------
# input        $PaymentTypeMethod | Payment type method object | object
#========================================================================================
sub uploadPaymentTypeMethodLogo {
    my $self = shift;
    my ($PaymentTypeMethod) = @_;

    if (!defined($PaymentTypeMethod)) {
        $PaymentTypeMethod = $self->get('PaymentTypeMethod');
    }

    my $NewLogoFileName = $PaymentTypeMethod->alias() . '.png';
    my $NewLogoSourcePath = $ENV{'EPAGES_CARTRIDGES'}
                          . '/DE_EPAGES/Spreedly/Data/Public/SF/PaymentTypes/'
                          . $NewLogoFileName;
    my $LogoDestinationDirectory = $ENV{'EPAGES_WEBROOT'} . '/' . $self->_LoadRootObject()->get('StoreName') . '/'
                                 . $self->get('FilePath') . '/';

    my $CurrentPaymentTypeMethod = $self->get('PaymentTypeMethod');
    if (defined($CurrentPaymentTypeMethod)) {
        my $CurrentLogoFileName = $CurrentPaymentTypeMethod->alias() . '.png';
        my $CurrentLogoPath = $LogoDestinationDirectory . $CurrentLogoFileName;
        $self->_DeleteFile($CurrentLogoPath);
    }
    my $NewLogoDestinationPath = $LogoDestinationDirectory . $NewLogoFileName;

    if ($self->_ExistsFile($NewLogoSourcePath)) {
        $self->_CopyFile($NewLogoSourcePath, $NewLogoDestinationPath, {create_directory => 1});
        $self->set( { 'Image' => $NewLogoFileName } );
    }
}

#========================================================================================
# function     _changeInShopName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_changeInShopName( $PaymentTypeMethodToMirror, $aLanguages );
#----------------------------------------------------------------------------------------
# description  It performs the transactional/atomic tasks to change "in shop" payment
#               method name. It's a wrapper convenient for testing.
#----------------------------------------------------------------------------------------
# input        $PaymentTypeMethodToMirror | Payment type method object | object
# input        $aLanguages | array reference of languages for the shop | ref.array
#========================================================================================
sub _changeInShopName {
    my $self = shift;
    my ($PaymentTypeMethodToMirror, $aLanguages) = @_;

    foreach my $hLanguage ( @{$aLanguages} ) {
        my $LanguageID = $hLanguage->{'LanguageID'};
        my $LocalizedName = $PaymentTypeMethodToMirror->get('Name', $LanguageID);
        $self->_LogDebug( 'update localized name in shop', { 'LanguageID' => $LanguageID, 'Name' => $LocalizedName } );
        $self->set( { 'Name' => $LocalizedName },  $LanguageID );
    }
}

#----------------------------------------------------------------------------------------
# Wrappers
#----------------------------------------------------------------------------------------
sub _ExistsFile {
    my $self = shift;
    my ($FilePath) = @_;

    return ExistsFile($FilePath);
}

sub _DeleteFile {
    my $self = shift;
    my ($FilePath) = @_;

    return DeleteFile($FilePath);
}

sub _CopyFile {
    my $self = shift;
    my ($FromFilePath, $ToFilePath, $hFlags) = @_;

    CopyFile($FromFilePath, $ToFilePath, $hFlags);
    return;
}

sub _Transaction {
    my $self = shift;
    my ($Code) = @_;

    Transaction($Code);
}

sub _LogDebug {
    my $self = shift;
    my ($Description, $Data) = @_;

    LogDebug($Description, $Data);
}

sub _LoadRootObject {
    my $self = shift;

    return LoadRootObject();
}

1;
