#========================================================================================
# package      DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyStripePaymentIntents
# base         DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for PaymentMethodSpreedlyStripe
#========================================================================================
package DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyStripePaymentIntents;
use base DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly;

use strict;

no warnings 'redefine'; ##no critic qw(ProhibitNoWarnings)

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Spreedly::API::Constants qw ( OPERATIONAL_MODE_LIVE OPERATIONAL_MODE_TEST );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Spreedly::API::StripeUrlBuilder;

use constant PAYMENT_TYPE_STRIPE_PAYMENT_INTENTS => 'SpreedlyStripePaymentIntents';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new();
#----------------------------------------------------------------------------------------
# description  Class constructor.
#----------------------------------------------------------------------------------------
# input        $Servlet       | Settings page's Servlet              | object
# input        $PaymentMethod | Servlet's object                     | object
# return       $self          | PaymentMethodSpreedlyStripePaymentIntents instance | object
#========================================================================================
sub new {
    my $class = shift;
    my ($Servlet, $PaymentMethod) = @_;

    my $PaymentTypeAlias = $PaymentMethod->get('PaymentType')->alias();
    if ($PaymentTypeAlias ne PAYMENT_TYPE_STRIPE_PAYMENT_INTENTS) {
        Error('UNEXPECTED_PAYMENT_TYPE',
                { 'Message' => sprintf(
                        'Expected PaymentType %s, found %s.', PAYMENT_TYPE_STRIPE_PAYMENT_INTENTS, $PaymentTypeAlias
                )}
        );
    }

    my $self  = bless {
        'Servlet' => $Servlet,
        'PaymentMethod' => $PaymentMethod,
    }, $class;

    return $self;
}

#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to view settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewSettingsSpreedlyStripePaymentIntents {
    my $self = shift;

    my $Shop = $self->{'PaymentMethod'}->getSite();
    my $IsInternalTestShop = $Shop->get('IsInternalTestShop');

    my $Form = $self->{'Servlet'}->form();

    if($Form->queryValue('code') || $Form->queryValue('error')){ # log only if Redirected from changeAction BacklinkStripeOnboarding.
        $self->_LogPayment(PAYMENT_TYPE_STRIPE_PAYMENT_INTENTS,
            'Redirected from changeAction BacklinkStripeOnboarding. Return data from StripePaymentIntents',
            $Form->queryValues(),undef , sub {
            my $LogString = shift;
            $$LogString =~ s/'ac_\w+'/'ac_---replaced---'/xig;
        });

        my $AuthorizationCode = $Form->queryValue('code');

        if (isValid($AuthorizationCode)) {
            my $hResponse = $self->_createGateway($AuthorizationCode, $IsInternalTestShop);
            $self->{'PaymentMethod'}->set({
                'StripeAccountId' => $hResponse->{'stripeAccountId'},
                'StripeAccessToken' => $hResponse->{'stripeAccessToken'},
                'StripeRefreshToken' => $hResponse->{'stripeRefreshToken'}
            });
        }

        if ($Form->queryValue('error')) {
            $self->{'Servlet'}->vars('HasSpreedlyConnectionError', 1);
        }
    }
    my $StripeUrlBuilder = $self->_createStripeUrlBuilder();
    my $CurrentUrl = $self->_getCurrentURL();
    my $ReconnectUrl = $StripeUrlBuilder->buildConnectAndReconnectUrl($self->{'PaymentMethod'}, $CurrentUrl);
    $self->{'Servlet'}->vars('ConnectAndReconnectUrl', $ReconnectUrl);

    return $self->{'Servlet'};
}

#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------
sub _createGateway {
    my $self = shift;
    my ($AuthorizationCode, $IsInternalTestShop) = @_;

    my $OperationalMode = $IsInternalTestShop ? OPERATIONAL_MODE_TEST : OPERATIONAL_MODE_LIVE;

    my $hAdditionalBodyParams = {
        'authorization_code' => $AuthorizationCode,
        'operational_mode'   => $OperationalMode,
    };
    my $PaymentMethodSpreedly = DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly->new();

    return $PaymentMethodSpreedly->SaveCredentials(
            $self->{'Servlet'},
            PAYMENT_TYPE_STRIPE_PAYMENT_INTENTS,
            $hAdditionalBodyParams
    );
}

sub _getCurrentURL {
    my $self = shift;

    my $hFormValues = $self->{'Servlet'}->params();
    my $hParams = {
        'ViewAction' => $hFormValues->{'ViewAction'},
        'ObjectID'   => $self->{'PaymentMethod'}->id(),
    };
    my $hOptions = {
        'Type' => 'admin',
        'UseSSL' => 1,
    };

    return BuildShopUrl($self->{'PaymentMethod'}, $hParams, $hOptions, $self->{'Servlet'});
}

sub _LogPayment {
    my $self = shift;
    my ($PaymentTypeAlias, $Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment($PaymentTypeAlias, $Name, $DebugData, $Section, $Filter);
}

sub _createStripeUrlBuilder {
    my $self = shift;
    return DE_EPAGES::Spreedly::API::StripeUrlBuilder->new();
}
1;
