#========================================================================================
# package      DE_EPAGES::Test::API::PageType
# base         DE_EPAGES::Test::API::StoreUnitTest
# state        public
#----------------------------------------------------------------------------------------
# description  Tests for PageType*.xml correctness
#========================================================================================
package DE_EPAGES::Test::API::PageType;
use base DE_EPAGES::Test::API::StoreTypeUnitTest;

use strict;

our @EXPORT_OK = qw( TestPageTypes );

use Test::More;
use DE_EPAGES::Core::API::File qw (GetFileNames);
use DE_EPAGES::XML::API::UseProblems qw ( GetCartridgeDependencies );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw (GetCartridges);
use DE_EPAGES::Core::API::PerlTools qw (UsePackage);

sub test {
    my $self = shift;

    my ($StoreType) = $self->storeType;

    # get cartridges for store type
    my $haCartridges = GetCartridges();
    die "Store Type $StoreType not found" unless defined $haCartridges->{$StoreType};

    # load Database/XML directories.
    my %xmlDirMap = map { $_ => _GetXmlDirectory($_) } @{$haCartridges->{$StoreType}};

    # get and parse Translation.Actions*.de.xml files
    my @TranslationFiles = map { @{_GetActionTranslationFiles($xmlDirMap{$_})} }
        grep { defined($xmlDirMap{$_})} @{$haCartridges->{$StoreType}};

    my @Actions = map { ActionTranslationTestHandler::ParseActionTranslations( $_ ) } @TranslationFiles;

    # get and parse PageTypes*.xml files
    my @PageTypeFiles = map { @{_GetPageTypeFiles($xmlDirMap{$_})} }
        grep { defined($xmlDirMap{$_})} @{$haCartridges->{$StoreType}};

    plan tests => scalar @PageTypeFiles;

    my $hCartridgeDependencies = GetCartridgeDependencies();
    my %ViewActions;
    my %PageTypes;
    foreach my $FileName (@PageTypeFiles) {
        ok( PageTypeTestHandler::ParsePageTypes(\%PageTypes, \@Actions, \%ViewActions, $FileName, $hCartridgeDependencies ), $FileName );
    }
}

#---------------------------------------------------------------------------
#  private helper
#---------------------------------------------------------------------------
sub _GetXmlDirectory {
    my ($CartridgePackage) = @_;

    my $Package = "${CartridgePackage}::API::Cartridge";
    UsePackage($Package);
    return undef unless $Package->can('xmlDirectory');
    my $Cartridge = $Package->new;
    my $xmlDir = $Cartridge->xmlDirectory;

    return $xmlDir;
}

sub _GetPageTypeFiles {
    my ($xmlDir) = @_;
    return [sort grep { /\/PageTypes\w*\.xml$/ } @{GetFileNames($xmlDir, {'recursive' => 0})}];
}

sub _GetActionTranslationFiles {
    my ($xmlDir) = @_;
    return [sort grep { /\/Translation.*\.Actions.*\.de\.xml$/x } @{GetFileNames($xmlDir, {'recursive' => 0})}];
}


#========================================================================================
# package      PageTypeTestHandler
# base         DE_EPAGES::XML::API::BaseImportHandler
# state        private
#----------------------------------------------------------------------------------------
# description  SAX handler for PageTypes*.xml
#========================================================================================
package PageTypeTestHandler; ## no critic qw( ProhibitMultiplePackages )
use base 'DE_EPAGES::XML::API::BaseImportHandler';

use strict;
use Test::More;
use DE_EPAGES::Core::API::File qw (ExistsDirectory ExistsFile GetFileContent);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf);

#========================================================================================
# function     ParsePageTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       ParsePageTypes( $hPageTypes, $aActions, $hViewActions, $FileName );
#----------------------------------------------------------------------------------------
# description  Parses a PageTypes.xml file.
#----------------------------------------------------------------------------------------
# input        $hPageTypes | PageType info hash | ref.hash
# input        $aActions | list of translated actions | ref.array.string
# input        $hViewActions | viewaction => pagetype mapping | ref.hash
# input        $FileName | filename | string
# input        $hCartridgeDependencies | cartridge dependencies | ref.hash.array
# return       $NoErrors | true if no errors were found | boolean
#========================================================================================
sub ParsePageTypes {
    my ($hPageTypes, $aActions, $hViewActions, $FileName, $hCartridgeDependencies) = @_;
    my $Handler = __PACKAGE__->new;

    $Handler->{'CartridgeDependencies'} = $hCartridgeDependencies;
    $Handler->{'FileName'}  = substr($FileName, length($ENV{'EPAGES_CARTRIDGES'})+1);
    $Handler->{'PageTypes'} = $hPageTypes;
    $Handler->{'ViewActions'} = $hViewActions;
    # map array into a hash.ref for easier access.
    %{$Handler->{'TranslatedActions'}} = map { $_ => 1 } @$aActions;
    $Handler->{'Errors'} = 0;
    # initialize exemption list as hash.
    %{$Handler->{'TranslationExemptions'}} = map { $_ => 1 } qw(
        /Classes/Object/Actions/MBO-ViewGeneral
        /Classes/Object/Actions/Monitor
        /Classes/AppointmentFolder/Actions/MBO-ViewResourceWizardSteps
        /Classes/Shop/Actions/MBO-ViewBookingsystemWizardSteps
        /Classes/Shop/Actions/MBO-ViewAddWidgetDialogSnippet
        /Classes/Object/Actions/MBO-ViewMenuJS
        /Classes/Object/Actions/UnityMBO-ViewMenuJS
    );

    $Handler->importXML($FileName);
    return ($Handler->{'Errors'} == 0) ? 1 : 0;
}

sub addError {
    my $self = shift;
    my ($Message) = @_;

    diag( "$Message at $self->{'FileName'} line 1" );
    $self->{'Errors'}++;
}

sub _cartridgeDir {
    my $self = shift;
    my ($Package) = @_;

    $Package =~ s,::,/,g;
    return "$ENV{'EPAGES_CARTRIDGES'}/$Package";
}

sub start_Cartridge {
    my $self = shift;
    my ($el) = @_;

    my $hAttributes = $self->getAttributes($el);
    if (!(defined($hAttributes->{'reference'}) && $hAttributes->{'reference'} == 1)) {
        $self->addError('missing reference="1" on element Cartridge');
    }

    my $Package = $hAttributes->{'Package'};
    if ( defined $Package && $Package =~ /^\w+::\w+$/ ) {
        my $CartridgeDir = $self->_cartridgeDir($Package);
        if( !ExistsDirectory( $CartridgeDir ) ) {
            $self->addError("cartridge directory '$CartridgeDir' does not exist");
        }
    }
    else {
        $self->addError('missing or wrong attribute "Package" on element Cartridge');
    }
    $self->pushID('Cartridge', $Package);

    if (defined($hAttributes->{'delete'})) {
        $self->addError('wrong attribute delete="1" on element Cartridge');
    }
}

sub end_Cartridge {
    shift->popID('Cartridge');
}

sub start_Class {
    my $self = shift;
    my ($el) = @_;

#    my $locator = $self->get_document_locator;
#    print $locator->{'LineNumber'} ."\n" if defined $locator;


    my $hAttributes = $self->getAttributes($el);
    if (!(defined($hAttributes->{'reference'}) && $hAttributes->{'reference'} == 1)) {
        $self->addError('missing or wrong attribute reference on element Class');
    }
    if (!defined $hAttributes->{'Path'}) {
        $self->addError('missing attribute Path on element Class');
    }
    if (defined($hAttributes->{'delete'})) {
        $self->addError('obsolete attribute delete defined on element Class');
    }

    $self->pushID('Class', $hAttributes->{'Path'});
}

sub end_Class {
    shift->popID('Class');
}

sub start_PageType {
    my $self = shift;
    my ($el) = @_;

    my $hAttributes = $self->getAttributes($el);

    $self->pushID('PageType', $hAttributes->{'Alias'});
    $self->pushID('PageType_delete', $hAttributes->{'delete'});

    if (!defined $hAttributes->{'Alias'}) {
        $self->addError('missing attribute Alias on element PageType');
        return;
    }
    if (exists($self->{'PageTypes'}->{$hAttributes->{'Alias'}})) {
        # referenced page type
        if (!(defined($hAttributes->{'reference'}) && $hAttributes->{'reference'} == 1)) {
            $self->addError("missing or wrong attribute reference on element PageType ($hAttributes->{'Alias'})");
        }
        if (defined($hAttributes->{'delete'})) {
            $self->addError("obsolete attribute delete defined on element PageType ($hAttributes->{'Alias'})");
        }
        if (exists $hAttributes->{'Base'}) {
            $self->addError("obsolete attribute Base defined on element PageType ($hAttributes->{'Alias'})");
        }
        if ($self->{'PageTypes'}->{$hAttributes->{'Alias'}}->{'Class'} ne $self->topID('Class')) {
            $self->addError("PageType $hAttributes->{'Alias'} defined on class ".$self->topID('Class')
                .", but referenced PageType in $self->{'PageTypes'}->{$hAttributes->{'Alias'}}->{'File'} "
                ."defined on class $self->{'PageTypes'}->{$hAttributes->{'Alias'}}->{'Class'}");
        }
    }
    else {
        # new page type
        if (!(defined($hAttributes->{'delete'}) && $hAttributes->{'delete'} == 1)) {
            $self->addError("missing 'delete=\"1\" on element PageType Alias=\"($hAttributes->{'Alias'})\"");
        }
        if (defined($hAttributes->{'reference'})) {
            $self->addError("obsolete attribute reference defined on element PageType ($hAttributes->{'Alias'})");
        }

        my %PageType = (
            'Alias' => $hAttributes->{'Alias'},
            'Class' => $self->topID('Class'),
            'File'  => $self->{'FileName'},
        );
        if( defined $hAttributes->{'Base'} ) {
            if( $hAttributes->{'Base'} =~ /\S/ ) {
                $PageType{'Base'} = $hAttributes->{'Base'};
                if (!defined $self->{'PageTypes'}->{$PageType{'Base'}}) {
                    $self->addError('unknown Base PageType '.$PageType{'Base'});
                }
            }
            else {
                $self->addError("empty Base attribute on on element PageType ($hAttributes->{'Alias'})");
            }
        }
        $self->{'PageTypes'}->{$hAttributes->{'Alias'}} = \%PageType;
    }
}

sub end_PageType {
    my $self = shift;

    $self->popID('PageType');
    $self->popID('PageType_delete');
}

sub start_Menu {
    my $self = shift;
    my ($el) = @_;
    my $hAttributes = $self->getAttributes($el);

    # this works only for immediate parents, should really work recursively
    my $ParentMenu_delete = $self->topID('Menu_delete');
    $self->pushID('Menu_delete', $hAttributes->{'delete'});

    my $MenuName = $hAttributes->{'Name'} // $hAttributes->{'Template'};
    if (!defined $MenuName) {
        $self->addError('missing attribute Name or Template on element Menu');
        return;
    }

    my $PageTypeName = $self->topID('PageType');
    my $PageType_delete = $self->topID('PageType_delete');
    if( !defined $PageTypeName ) {
        $self->addError( "no PageType for menu $MenuName" );
        return;
    }

    my $Cartridge = $hAttributes->{'Cartridge'} // $self->topID('Cartridge');
    if( !defined $Cartridge ) {
        $self->addError( "no Cartridge for menu $MenuName" );
        return;
    }

    my $PageType = $self->{'PageTypes'}->{$PageTypeName};
    if( exists $PageType->{'Menus'}->{$MenuName} ) {
        unless ($hAttributes->{'reference'}) {
            $self->addError("missing attribute 'reference' on menu $MenuName in pagetype $PageTypeName");
        }
        if( defined $hAttributes->{'Position'} ) {
            $self->addError("overloading attribute 'Position' in referenced menu $MenuName in pagetype $PageTypeName");
        }
        if( defined $hAttributes->{'Base'} ) {
            $self->addError("overloading attribute 'Base' in referenced menu $MenuName in pagetype $PageTypeName");
        }
    }
    else {
        unless ($PageType_delete || $ParentMenu_delete || $hAttributes->{'delete'}) {
            $self->addError("missing attribute 'delete' on menu $MenuName in pagetype $PageTypeName");
        }

        $PageType->{'Menus'}->{$MenuName} = {
            'Name' => $MenuName,
            'Cartridge' => $Cartridge,
        }
    }
}

sub end_Menu {
    my $self = shift;
    $self->popID('Menu_delete');
}

sub start_Template {
    my $self = shift;
    my ($el) = @_;

    my $hAttributes = $self->getAttributes($el);

    my $TemplateName = $hAttributes->{'Name'};
    if (!defined $TemplateName) {
        $self->addError('missing attribute Name on element Template');
        return;
    }

    my $PageTypeAlias = $self->topID('PageType');
    if( !defined $PageTypeAlias ) {
        $self->addError( "no PageType for template $TemplateName" );
        return;
    }

    my $Cartridge = $hAttributes->{'Cartridge'} // $self->topID('Cartridge');
    if( !defined $Cartridge ) {
        $self->addError( "no Cartridge for template $TemplateName" );
        return;
    }

    my $FileName = $hAttributes->{'FileName'};
    if( !defined $FileName ) {
        $self->addError('missing attribute FileName on element Template');
        return;
    }

    my $hPageType = $self->{'PageTypes'}->{$PageTypeAlias};
    if( exists $hPageType->{'Templates'}->{$TemplateName} ) {
        unless ($hAttributes->{'reference'}) {
            $self->addError("missing attribute 'reference' on template $TemplateName in pagetype $PageTypeAlias");
        }
    }

    my $hTemplate = {
        'Name' => $TemplateName,
        'Cartridge' => $Cartridge,
        'FileName' => $FileName,
        'PageType' => $PageTypeAlias,
    };
    $hPageType->{'Templates'}->{$TemplateName} = $hTemplate;
    $self->{'TestTemplates'}->{$PageTypeAlias}->{$TemplateName} = $hTemplate;
}

sub start_ViewAction {
    my $self = shift;
    my ($ViewAction) = @_;

    my $hAttributes = $self->getAttributes($ViewAction);
    my $PageTypeName = $self->topID('PageType');
    my $PageType = $self->{'PageTypes'}->{$PageTypeName};
    my $ViewActionAlias = $hAttributes->{'URLAction'};

    my $ActionPath = $PageType->{'Class'}.'/Actions/'.$ViewActionAlias;
    if (!(defined $self->{'TranslatedActions'}->{$ActionPath}) &&
        !($self->_isViewActionExemption($ActionPath)) &&
        ($self->_extendsPageType($PageTypeName, 'MBO') ||
        $self->_extendsPageType($PageTypeName, 'UnityMBO'))) {
        $self->addError('missing action translation for '.$ActionPath . ' used')
    }

    if( exists $PageType->{'ViewAction'} ) {
        $self->addError("assigning a new view action $ViewActionAlias to page type $PageTypeName"
                        ." that already has a view action $PageType->{'ViewAction'}");
    }
    $PageType->{'ViewAction'} = $ActionPath;

    if( exists $self->{'ViewActions'}->{$ActionPath} ) {
        $self->addError("re-assigning view action $ActionPath from page type"
                        ." $self->{'ViewActions'}->{$ActionPath} to $PageTypeName");
    }
    $self->{'ViewActions'}->{$ActionPath} = $PageTypeName;
}

#========================================================================================
# function     _isViewActionExemption
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isViewActionExemption( $ActionPath );
#----------------------------------------------------------------------------------------
# description  Contains exemption list for Actions that do not need a translation
#----------------------------------------------------------------------------------------
# input        $ActionPath | absolute action path | string
# return       $isViewActionExemption | true if action path is in exemption list | boolean
#========================================================================================
sub _isViewActionExemption {
    my $self = shift;
    my ($ActionPath) = @_;

    # ErrorActions in der Transaktionssuche
    return 1 if ($ActionPath =~ m{/Actions/(Unity|)MBO-View.*Transactions});
    return defined($self->{'TranslationExemptions'}->{$ActionPath});
}

sub _extendsPageType {
    my $self = shift;
    my ($PageTypeName, $Basename) = @_;
    return 0 if !$PageTypeName;
    return 1 if $PageTypeName eq $Basename;
    return $self->_extendsPageType($self->{'PageTypes'}->{$PageTypeName}->{'Base'}, $Basename);
}

sub start_document {
    my $self = shift;
    my ($doc) = @_;

    $self->SUPER::start_document($doc);

    $self->{'TestTemplates'} = {}; # { PageTypeAlias => { TemplateName => FileName } }
}

sub end_document {
    my $self = shift;

    my %Exceptions = (
        # DE_EPAGES::Catalog vs. DE_EPAGES::Content
        'SF-Category' => { map { $_ => undef } qw( Content Content-Order ) },
        'SF-StartPage' => { map { $_ => undef } qw( Content Content-Order ) },
    );

    my $hTestTemplates = $self->{'TestTemplates'};
    foreach my $PageTypeAlias (sort keys %$hTestTemplates) {

        my $Alias = $PageTypeAlias;
        my @PageTypePath;
        while( defined $Alias ) {
            push @PageTypePath, $Alias;
            my $hPageType = $self->{'PageTypes'}->{$Alias};
            $Alias = $hPageType->{'Base'};
        }

        my $hTemplates = $hTestTemplates->{$PageTypeAlias};
        $self->_testTemplate($PageTypeAlias, \@PageTypePath, $hTemplates->{$_}) foreach sort keys %$hTemplates;

        # test for templates that are defined in a different cartridge than the corresponding menu
        my $hPageType = $self->{'PageTypes'}->{$PageTypeAlias};
        foreach my $TemplateName (sort keys %$hTemplates) {

            next if exists $Exceptions{$PageTypeAlias}->{$TemplateName};

            if( exists $hPageType->{'Menus'}->{$TemplateName} ) {
                my $hMenu = $hPageType->{'Menus'}->{$TemplateName};
                my $hTemplate = $hTemplates->{$TemplateName};
                unless ( $hTemplate->{Cartridge} eq $hMenu->{Cartridge} or
                         ElementOf($hTemplate->{Cartridge},
                                   @{$self->{CartridgeDependencies}->{$hMenu->{Cartridge}}})
                ) {
                    $self->addError("template $PageTypeAlias.$TemplateName from $hTemplate->{Cartridge}"
                        . " is used for menu $PageTypeAlias.$TemplateName from $hMenu->{Cartridge}"
                        . " but cartridge $hMenu->{Cartridge} does not depend on $hTemplate->{Cartridge}");
                }
            }
        }
    }
}

sub _testTemplate {
    my $self = shift;
    my ($PageTypeAlias, $aPageTypePath, $hTemplate) = @_;

    my $FileName = $self->_cartridgeDir($hTemplate->{'Cartridge'}).'/Templates/'.$hTemplate->{'FileName'};
    if( !ExistsFile( $FileName ) ) {
        $self->addError("template file '$FileName' does not exist for pagetype $PageTypeAlias, template $hTemplate->{Name}");
        return;
    }

    my $Content = ${GetFileContent( $FileName )};
    my @Includes = $Content =~ /\#INCLUDE\(\s*"(.*?)".*?\)/gx;
    push @Includes, $Content =~ /\#FUNCTION\(\s*"LocaleInclude"\s*,\s*"(.*?)".*?\)/gx;
    push @Includes, $Content =~ /\#LocaleInclude\(\s*"(.*?)".*?\)/gx;
    $self->_testInclude( $PageTypeAlias, $aPageTypePath, $hTemplate->{'Name'}, $_, $FileName ) foreach @Includes;

    my @Menus = $Content =~ /\#MENU\(\s*"(.*?)".*?\)/gx;
    $self->_testMenu( $PageTypeAlias, $aPageTypePath, $hTemplate->{'Name'}, $_, $FileName ) foreach @Menus;
}

sub _testInclude {
    my $self = shift;
    my ($PageTypeAlias, $aPageTypePath, $TemplateName, $IncludeName, $FileName) = @_;

    my ($FoundPageType) = grep { exists $self->{'PageTypes'}->{$_}->{'Templates'}->{$IncludeName} } @$aPageTypePath;

    return if defined $FoundPageType;

    my %Exceptions = map { $_ => undef } ( ## no critic qw(ProhibitQuotedWordLists)
        'DataForm-', 'ProcessOrderPayment-', 'Process-', 'ContentLine', 'ScoreLimitLine', # class-dependend includes
        'TabPage',        # forward declaration in abstract pagetypes
        'TabPage-',       # #INCLUDE("TabPage-".#alias) in MBO-ProductGeneral.TabPage.html
        'INC-Attributes', # template INC-Attributes referenced in SF.INC-PersonalDataForm
                          # (DE_EPAGES/Design/Templates/SF/SF.INC-PersonalDataForm.html) does not exist in pagetype SF/BasePageType
        'Settings', # template Settings referenced in MBO-PluginPopup.Page (DE_EPAGES/Design/Templates/MBO/MBO-PluginPopup.Page.html)
                    # does not exist in pagetype MBO-PluginPopup/MBO/Backoffice/BasePageType
    );
    return if exists $Exceptions{$IncludeName};

    my $PageTypePath = join '/', @$aPageTypePath;
    $self->addError("template $IncludeName referenced in PageType '$PageTypeAlias' and Template '$TemplateName'"
        . " ($FileName) does not exist in pagetype $PageTypePath");
}

sub _testMenu {
    my $self = shift;
    my ($PageTypeAlias, $aPageTypePath, $TemplateName, $MenuName, $FileName) = @_;

    my ($FoundPageType) = grep { exists $self->{'PageTypes'}->{$_}->{'Menus'}->{$MenuName} } @$aPageTypePath;

    return if defined $FoundPageType;

    my $PageTypePath = join '/', @$aPageTypePath;
    $self->addError("menu $MenuName referenced in PageType '$PageTypeAlias' and Template '$TemplateName'"
        . " ($FileName) does not exist in pagetype $PageTypePath");
}


#========================================================================================
# package      ActionTranslationTestHandler
# base         DE_EPAGES::XML::API::BaseImportHandler
# state        private
#----------------------------------------------------------------------------------------
# description  SAX handler for Translation.Actions*.de.xml
#========================================================================================
package ActionTranslationTestHandler; ## no critic qw( ProhibitMultiplePackages )
use base 'DE_EPAGES::XML::API::BaseImportHandler';

use strict;

#========================================================================================
# function     ParseActionTranslations
# state        public
#----------------------------------------------------------------------------------------
# syntax       @ActionNames = ParseActionTranslations( $FileName );
#----------------------------------------------------------------------------------------
# description  Finds translated actions in an XML file.
#----------------------------------------------------------------------------------------
# input        $FileName | filename | string
# return       @ActionNames | list of aliases of translated actions | array.string
#========================================================================================
sub ParseActionTranslations {
    my ($FileName) = @_;
    my $Handler = __PACKAGE__->new;

    $Handler->{'FileName'}  = substr($FileName, length($ENV{'EPAGES_CARTRIDGES'})+1);
    $Handler->{'Actions'} = [];

    $Handler->importXML($FileName);
    return @{$Handler->{'Actions'}};
}

# xml tag <Language> ignored.
sub start_Language {}

# xml tag <Object>
sub start_Object {
    my $self = shift;
    my ($Element) = @_;
    my $hAttributes = $self->getAttributes($Element);
    push @{$self->{'Actions'}}, $hAttributes->{'Path'};
}

# xml tag <Attribute> ignored.
sub start_Attribute {}

# xml tag <notrans> ignored.
sub start_notrans {}

1;
