#========================================================================================
# package      DE_EPAGES::Test::API::TLE
# base         DE_EPAGES::Test::API::TemplateTest
# state        public
#----------------------------------------------------------------------------------------
# description  Tests for correct and secure usage of TLEs
#========================================================================================
package DE_EPAGES::Test::API::TLE;
use base DE_EPAGES::Test::API::TemplateTest;

use strict;
use HTML::Parser;
use Test::More;

use DE_EPAGES::Core::API::File  qw ( GetFileContent );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Test::API::TemplateUtil qw ( CleanContentForTesting RemoveTLEsToIgnore );
use Readonly;

Readonly my @SLICESPACE_EXCEPTION_LIST => (
 # CK editor produce text with HTML, html escaping would be break CK editor output
 "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Forum/Templates/MobileSF/MobileSF-INC-ListArticle.html"
);

use constant TESTCASES => (\&GETTest, \&SLICESPACETest, \&NOHTMLTest);
use constant TESTCASES_JS => (\&INPUTTest);

use constant ATTRIBUTES_FOR_JS => (qw(
    fscommand
    onabort
    onactivate
    onafterprint
    onafterupdate
    onbeforeactivate
    onbeforecopy
    onbeforecut
    onbeforedeactivate
    onbeforeeditfocus
    onbeforepaste
    onbeforeprint
    onbeforeunload
    onbegin
    onblur
    onbounce
    oncellchange
    onchange
    onclick
    oncontextmenu
    oncontrolselect
    oncopy
    oncut
    ondataavailable
    ondatasetchanged
    ondatasetcomplete
    ondblclick
    ondeactivate
    ondrag
    ondragend
    ondragleave
    ondragenter
    ondragover
    ondragdrop
    ondrop
    onend
    onerror
    onerrorupdate
    onfilterchange
    onfinish
    onfocus
    onfocusin
    onfocusout
    onhelp
    onkeydown
    onkeypress
    onkeyup
    onlayoutcomplete
    onload
    onlosecapture
    onmediacomplete
    onmediaerror
    onmousedown
    onmouseenter
    onmouseleave
    onmousemove
    onmouseout
    onmouseover
    onmouseup
    onmousewheel
    onmove
    onmoveend
    onmovestart
    onoutofsync
    onpaste
    onpause
    onprogress
    onpropertychange
    onreadystatechange
    onrepeat
    onreset
    onresize
    onresizeend
    onresizestart
    onresume
    onreverse
    onrowsenter
    onrowexit
    onrowdelete
    onrowinserted
    onscroll
    onseek
    onselect
    onselectionchange
    onselectstart
    onstart
    onstop
    onsyncrestored
    onsubmit
    ontimeerror
    ontrackchange
    onunload
    onurlflip
    seeksegmenttime
));

sub test {
    my $self = shift;

    my @TemplateFiles = $self->getAllTemplateFiles();

    # Calculate correct number of tests

    # Because this test can be called with different storetypes
    # the skipped files may or may not be part of original testplan
    my @SkippedTests = grep { ElementOf($_, @SLICESPACE_EXCEPTION_LIST) } @TemplateFiles;
    my $NumberOfTests = scalar @TemplateFiles * scalar (@{[(TESTCASES), (TESTCASES_JS)]});
    my $NumberOfExpectedTests = $NumberOfTests - scalar (@SkippedTests);

    plan tests => $NumberOfExpectedTests;

    foreach my $TemplateFile (@TemplateFiles) {
        my $rContent = GetFileContent($TemplateFile);

        my $CleanContent = CleanContentForTesting($$rContent);
        foreach ((TESTCASES)) {
            $_->($TemplateFile, $CleanContent);
        };

        my $ContentWithoutScriptTags = "$$rContent";
        $ContentWithoutScriptTags =~ s/<script(.*?)<\/script>//g;
        foreach ((TESTCASES_JS)) {
            $_->($TemplateFile, $ContentWithoutScriptTags);
        };
    }

    return;
}

# XSS test for #INPUT: use #INPUT only with formatter in JS-related attributes
sub INPUTTest {
    my ($TemplateFile, $Content) = @_;

    my $Parser = HTML::Parser->new();
    $Parser->handler('start' => \&_INPUTTestCallback, 'self, attr');

    ok($Parser->parse($Content), $TemplateFile.' - all uses of #INPUT followed by any formatter.');
}

sub _INPUTTestCallback {
    my ($Parser, $hAttributes) = @_;
    my $OuterLoopEscape = 0;

    while (my ($Attibute, $Value) = each %$hAttributes) {
        last if $OuterLoopEscape;
        if (ElementOf($Attibute, ATTRIBUTES_FOR_JS)) {
            $Value = RemoveTLEsToIgnore($Value);
            while ($Value =~ m/#INPUT\.[A-Za-z0-9]+(.)/g) {
                unless ($1 eq '[') {
                    $Parser->eof;
                    $OuterLoopEscape = 1;
                    last;
                }
            }
        }
    }
}

# XSS test for #GET: use #GET only with formatter.
sub GETTest {
    my ($TemplateFile, $Content) = @_;

    my $Regex = '#GET\([^,]*?\)(?!,\s*"(html|preline|js)")';
    # Test.
    ok(($Content !~ m/$Regex/is), $TemplateFile.' - all uses of #GET with html formatter.');

    return;
}

# Use slice and space only with html|preline|js formatter
sub SLICESPACETest {
    my ($TemplateFile, $Content) = @_;
    if (ElementOf($TemplateFile, @SLICESPACE_EXCEPTION_LIST)) {
        note 'skip slice test for ' . $TemplateFile;
        return;
    }
    my $Check = 1;
    # Find formatters after slice and space (ungreedy).
    while($Content =~ m/(?:slice|space):[0-9]+(.*?)]/g) {
        my $Formatter = $1;

        # Check if html|preline|js formatter is not used.
        if($Formatter !~ m/,\s*(?:html|preline|js)/) {
            $Check = 0;
            last;
        }
    }
    ok($Check, $TemplateFile.' - all uses of slice and space formatter followed by html or preline formatter.');

    return;
}

# Do not use only nohtml formatter (followed by html|preline formatter).
sub NOHTMLTest {
    my ($TemplateFile, $Content) = @_;
    my $Check = 1;

    # Find formatters after nohtml formatter.
    while($Content =~ m/\[[^]]*nohtml([^\]]*)\]/g) {
        my $Formatter = $1;
        if($Formatter !~ m/,\s*(?:html|preline)/) {
            $Check = 0;
            last;
        }
    }
    ok($Check, $TemplateFile.' - all uses of nohtml formatter are followed by html or preline formatter.');

    return;
}

1;
