#========================================================================================
# package      DE_EPAGES::TrustedRating::API::TrustedRating
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the main trusted rating functions.
#========================================================================================
package DE_EPAGES::TrustedRating::API::TrustedRating;
use base Exporter;
use strict;
our @EXPORT_OK = qw (
    UpdateRatingState
    GetWidget
    GetTrustedRatingShops
);

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::File qw ( CreateDirectory ExistsDirectory );
use DE_EPAGES::Core::API::Net qw ( GetFile GetHttpResponse );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject LoadRootObject );
use DE_EPAGES::TrustedRating::API::Constants qw ( CONNECTION_OK );
use DE_EPAGES::WebService::API::Client;
use Encode;


#========================================================================================
# function     GetWidget
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetWidget($Shop, $TrustedRatingID);
#----------------------------------------------------------------------------------------
# description  Gets the widget image from trusted shops.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $TrustedRatingID | trusted rating identifier | string
#========================================================================================
sub GetWidget($$) {
    my ($Shop, $TrustedRatingID) = @_;
    TestString('TrustedRatingID', $TrustedRatingID);
    my $System = LoadRootObject();

    my $URL = $System->get('TrustedRatingWidgetURL') . $TrustedRatingID . '.gif';

    _checkReachability($URL);

    my $PublicDestinationPath = $Shop->get('PublicPath') . '/TrustedRating/';
    CreateDirectory($PublicDestinationPath, {'recursive' => 1}) unless ExistsDirectory($PublicDestinationPath);

    my $ResponseCode = GetFile($URL,  $PublicDestinationPath.$TrustedRatingID.'.gif');
    if ($ResponseCode) {
        Error('GET_WIDGET_FAILED', {'Code' => $ResponseCode, 'Message' => "Can\'t get widget for $TrustedRatingID", 'URL' => $URL });
    }
}

sub _checkReachability {
    my ($URL) = @_;
    my $ResponseCode = GetHttpResponse($URL, 1)->code;
    unless ($ResponseCode == CONNECTION_OK) {
        Error('CONNECTION_FAILED', {'Code' => $ResponseCode, 'Message' => 'Can\'t reach trusted shops website', 'URL' => $URL });
    }
}

#========================================================================================
# function     GetTrustedRatingShops
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShops = GetTrustedRatingShops();
#----------------------------------------------------------------------------------------
# description  Returns all shops with min. one active trusted rating account.
#----------------------------------------------------------------------------------------
# return       $aShops | array of shop objects | ref.array.object
#========================================================================================
sub GetTrustedRatingShops {

    my $Class = LoadClassByAlias('Shop');
    my $AttributeRatingID = $Class->attribute('TrustedRatingID')->id;

    my $dbi = GetCurrentDBHandle();
    my $sql = 'SELECT DISTINCT objectid FROM localizedstringattribute WHERE attributeid = ?';
    my $aaResult = $dbi->execute($sql, [$AttributeRatingID]);
    my @Shops;
    foreach my $ShopID (map {$_->[0]} @$aaResult ) {
        my $Shop = LoadObject($ShopID);
        foreach my $LanguageID (map {$_->{'LanguageID'}} @{ $Shop->get('Languages') }) {
            if (defined $Shop->get('TrustedRatingID', $LanguageID) && $Shop->get('TrustedRatingIsActive', $LanguageID)){
                push @Shops, $Shop;
                last;
            }
        }
    }
    return \@Shops;
}

1;
