#========================================================================================
# package      DE_EPAGES::TrustedShops::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  Utility functions for TrustedShops.
#========================================================================================
package DE_EPAGES::TrustedShops::API::Tools;
use base qw (Exporter);

use strict;
our @EXPORT_OK = qw (LogTrustedShops SendShopChangesToUnity TrustedShopsLogContext UnusedCertificateLanguages);
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Log qw ( GetLog LogCommunication LogContext LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid castToBoolean);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::TestParameter qw ( TestString TestObject);
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguagesByCode2);

#========================================================================================
# function     LogTrustedShops
#----------------------------------------------------------------------------------------
# syntax       LogTrustedShops($Name, $Debug, $Section);
#----------------------------------------------------------------------------------------
# example      LogTrustedShops('SendRequestHash', \%Hash, 'Send');
#----------------------------------------------------------------------------------------
# description  Logs a structure to the communication.log.
#----------------------------------------------------------------------------------------
# input        $Name | name of data | string
# input        $Debug | data to be logged | hash
# input        $Section | name of the section to be logged | string
#========================================================================================
sub LogTrustedShops ($$$) {
    my ($Name, $DebugStuff, $Section) = @_;

    return unless LoadRootObject()->get('TrustedShopsLogging');

    LogCommunication($Name, $DebugStuff, 'TrustedShops::'.$Section, [], 2);
    return;
}

#========================================================================================
# function     SendShopChangesToUnity
# state        public
#----------------------------------------------------------------------------------------
# syntax       SendShopChangesToUnity( $hParams );
#----------------------------------------------------------------------------------------
# description  Sends a Trustbadge JavaScript snippet to Unity.
#----------------------------------------------------------------------------------------
# input        $hParams | Hash structure generated after saving        | ref_hash
#                          changes on a certificate page. It contains
#                          the JavaScript snippet mentioned above.
#========================================================================================
sub SendShopChangesToUnity {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};
    my $CertificateLanguage = $hParams->{'Language'};

    my $LocaleId = _CastFromCode2ToLocaleId($CertificateLanguage);
    Error('MissingLocalIDForCertificate') unless(castToBoolean($LocaleId));

    my $hVar;

    if(isValid($hValues->{'TrustbadgeCardCoding'})) {
        $hVar->{'userSettings'}->{'trustedShopsBadge'}->{'snippet'} = $hValues->{'TrustbadgeCardCoding'};
    }
    else {
        $hVar->{'userSettings'}->{'trustedShopsBadge'} = undef
    }

    LogDebug('hVar', $hVar);

    $Shop->sendUnityRequest('PATCH', '/shop', {'locale'=>$LocaleId}, $hVar);

    return;
}

#----------------------------------------------------------------------------------------
sub _CastFromCode2ToLocaleId {
    my ($CertificateLanguageCode2) = @_;

    my $MinLength = 2;
    my $MaxLength = 2;
    TestString('$CertificateLanguageCode2', $CertificateLanguageCode2, $MinLength, $MaxLength);
    my $aLanguages = GetInfoLanguagesByCode2($CertificateLanguageCode2);

    return unless(castToBoolean($aLanguages));
    my $hFirstLanguange = $aLanguages->[0];

    return unless(castToBoolean($hFirstLanguange));
    return $hFirstLanguange->{'DefaultLocaleID'};
}

#========================================================================================
# function     TrustedShopsLogContext
# state        public
#----------------------------------------------------------------------------------------
# syntax       TrustedShopsLogContext($TrustedShopsCertificate, $FunctionPointer);
#----------------------------------------------------------------------------------------
# description  Customised LogContext for Trusted Shops.
#----------------------------------------------------------------------------------------
# input        $TrustedShopsCertificate | Trusted Shops Certificate | object
# input        $FunctionPointer         | Function Pointer
#========================================================================================
sub TrustedShopsLogContext {
    my ($TrustedShopsCertificate, $FunctionPointer) = @_;

    if (not defined($TrustedShopsCertificate)) {
        GetLog->warn('TrustedShopsLogContext failed. TrustedShopsCertificate is not defined');
    }

    my $Shop = $TrustedShopsCertificate->getSite();

    LogContext({
        'Shop Alias' => $Shop->alias(),
        'Shop ID' => $Shop->id(),
        'Trusted Shops Certificate Alias' => $TrustedShopsCertificate->alias(),
        'Trusted Shops Certificate ID' => $TrustedShopsCertificate->id()
        }, $FunctionPointer);

    return;
}
#========================================================================================
# function     UnusedCertificateLanguages
# state        public
#----------------------------------------------------------------------------------------
# syntax       UnusedCertificateLanguages( $Shop );
#----------------------------------------------------------------------------------------
# description  Retruns all languages which are not used for a TS-certificate
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
# return       $aPossibleLanguagesForCertificates | a list of possible languages for certificates | refarray
#========================================================================================
sub UnusedCertificateLanguages {
    my ($Shop) = @_;

    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');

    my $aExistingCertificates = $Shop->get('TrustedShopsCertificates');
    my $aExistingCertificateLanguagesAsCode2 = [];
    foreach my $ExistingCertificate (@{$aExistingCertificates}){
        push(@{$aExistingCertificateLanguagesAsCode2}, $ExistingCertificate->get('CertificationLanguage'));
    }

    my $aPossibleLanguagesForCertificates = [];
    my $aLanguages = $Shop->get('Languages');
    foreach my $Language (@{$aLanguages}){
        unless (ElementOf($Language->{'Code2'}, $aExistingCertificateLanguagesAsCode2)){
            push(@{$aPossibleLanguagesForCertificates}, $Language);
        }
    }

    return $aPossibleLanguagesForCertificates;
}
1;
