#========================================================================================
# package      DE_EPAGES::USPS::API::WebServices::Base::ServicesInterface
# state        public
#----------------------------------------------------------------------------------------
# description  It manages operations from XMLrequest and XMLresponse modules, required
#               operate with USPS Web Services
#========================================================================================
package DE_EPAGES::USPS::API::WebServices::Base::ServicesInterface;
use base DE_EPAGES::USPS::API::WebServices::Base::XMLrequest;
use base DE_EPAGES::USPS::API::WebServices::Base::XMLresponse;

use DE_EPAGES::USPS::API::WebServices::Base::APIcaller;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid IsA );
use DE_EPAGES::Core::API::Log qw (LogDebug);
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::USPS::API::Tools qw ( getWebServiceURL );

use strict;
use constant API_RESPONSE_ERROR_TAG => 'Error';
use constant API_CALL_DLL           => 'ShippingAPI.dll';

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'INVALID_REQUEST_TAG_NOT_FOUND' => 'No #nodename node can be found in the request object',
        'INVALID_REQUEST_API_NODE_NOT_FOUND' => '#name node not found into API XML DOM',
        'INVALID_REQUEST_XML_NODE_NOT_FOUND' => '#name node not found into XML DOM',
        'INVALID_REQUEST_XML_NODE_ATTRIBUTE_NOT_FOUND' => '#nodename node has not #attributename attribute',
        'INVALID_REQUEST_API_NODE_VALUE' => '#name API node can not get assigned with an invalid value (#value)',
        'INVALID_RESPONSE_NOT_AVAILABLE' => 'No response object can be found',
        'ERROR_RESPONSE' => 'web service error #number : #description',
        'INVALID_RESPONSE_UNEXPECTED_TYPE' => 'Response is not #expectedname type. It\'s of #nodename type instead',
        'INVALID_RESPONSE_TAG_NOT_FOUND' => 'No #nodename node can be found in the response object of #apitag type',
    };
}

#ERRORDEFINITION END
#========================================================================================
# function     constructor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ServicesInterface = $self->constructor($ShippingFulfillmentMethod,$APItag,$RequestFilePath,$Client);
#----------------------------------------------------------------------------------------
# description  Creates a new Service Interface object .
#----------------------------------------------------------------------------------------
# input        $ShippingFulfillmentMethod | Shipping Fulfillment Method | object
# input        $APItag | Tag of the required API | object
# input        $RequestFilePath | File Pathe of the XML constructor | object
# input        $Client | Client object -only used for mocking-| object
# return       $ServiceInterface | ServicesInterface object | object
#========================================================================================
sub constructor {
    my $class = shift;
    my ($ShippingFulfillmentMethod,$APItag,$RequestFilePath,$Client) = @_;

    TestObject(
        'ShippingFulfillmentMethodUSPS',
        $ShippingFulfillmentMethod,
        'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS'
    );

    my $self = bless {}, $class;

    my $WebServiceUrl = getWebServiceURL($ShippingFulfillmentMethod);

    $self->createCaller( API_CALL_DLL, $APItag, $WebServiceUrl , $Client );
    $self->createRequest($RequestFilePath);

    return $self;
}
#========================================================================================
# function     createCaller
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Caller = $self->createCaller();
#----------------------------------------------------------------------------------------
# description  It creates and returns the Caller object.
#----------------------------------------------------------------------------------------
# input        $APIcallDll | DLL string to be used for the web service | string
# input        $APItag | API tag for the web service | string
# input        $URLtoBeCalled | web server URL for the web service | string
# input        $Client | Remote Client object -only used for mocking- | object
# return       $Caller | Caller object | object
#========================================================================================
sub createCaller {
    my $self = shift;
    my ($APIcallDll, $APItag, $URLtoBeCalled, $Client) = @_;

    my $Caller = DE_EPAGES::USPS::API::WebServices::Base::APIcaller->new($APIcallDll, $APItag, $URLtoBeCalled, $Client);
    $self->{'Caller'} = $Caller;

    return $self->{'Caller'};
}

#========================================================================================
# function     getCaller
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Caller = $self->getCaller();
#----------------------------------------------------------------------------------------
# description  It returns the Caller object.
#----------------------------------------------------------------------------------------
# return       $Caller | Caller object | object
#========================================================================================
sub getCaller {
    my $self = shift;

    return $self->{'Caller'};
}

#========================================================================================
# function     setCaller
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setCaller($Caller);
#----------------------------------------------------------------------------------------
# description  It returns the Caller object.
#----------------------------------------------------------------------------------------
# input        $Caller | Caller object | object
#========================================================================================
sub setCaller {
    my $self = shift;
    my ($Caller) = @_;

    TestObject( 'caller', $Caller, 'DE_EPAGES::USPS::API::WebServices::Base::APIcaller' );
    $self->{'Caller'} = $Caller;

    return;
}

#========================================================================================
# function     createRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $XMLrequest = $self->createRequest();
#----------------------------------------------------------------------------------------
# description  It creates and returns the XMLrequest object.
#----------------------------------------------------------------------------------------
# input        $RequestFilePath | string
# return       $XMLrequest | XMLrequest object | object
#========================================================================================
sub createRequest {
    my $self = shift;
    my ($RequestFilePath) = @_;

    my $Request = DE_EPAGES::USPS::API::WebServices::Base::XMLrequest->new($RequestFilePath);
    $self->{'Request'} = $Request;

    return $self->{'Request'};
}

#========================================================================================
# function     getRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $XMLrequest = $self->getRequest();
#----------------------------------------------------------------------------------------
# description  It returns the XMLrequest object.
#----------------------------------------------------------------------------------------
# return       $XMLrequest | XMLrequest object | object
#========================================================================================
sub getRequest {
    my $self = shift;

    return $self->{'Request'};
}

#========================================================================================
# function     callWebService
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->callWebService();
#----------------------------------------------------------------------------------------
# description  If request is well validated, it calls USPS web service and gets the answer.
#----------------------------------------------------------------------------------------
# input        $LogFilterCode | code to hide sensitive data | code ref
#========================================================================================
sub callWebService {
    my $self = shift;
    my ($LogFilterCode) = @_;

    if ( $self->isValidRequestOrError() ) {
        my $Caller = $self->getCaller();
        my $XMLrequestString = $self->requestToString();
        LogShippingFulfillment('USPS', "XML request to be sent:\n", $XMLrequestString, undef, $LogFilterCode);
        my $XMLresponseString = $Caller->call($XMLrequestString);
        LogShippingFulfillment('USPS', "XML response received:\n", $XMLresponseString);
        my $XMLresponse       = DE_EPAGES::USPS::API::WebServices::Base::XMLresponse->new($XMLresponseString);
        $self->{'Response'} = $XMLresponse;
    }
}

#========================================================================================
# function     getRequestNodeContentOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $nodeContent = $self->getRequestNodeContentOrError($nodeName);
#----------------------------------------------------------------------------------------
# description  It checks that the node name exists for the request and returns the
#               content. It throws an error if no node can be found.
#----------------------------------------------------------------------------------------
# input        $nodeName | string
# return       $nodeContent | string
#========================================================================================
sub getRequestNodeContentOrError {
    my $self = shift;
    my ($nodeName) = @_;

    my $Request    = $self->getRequest();
    my $nodeContent = $Request->getTextContentByXMLnodeName($nodeName);
    Error( 'INVALID_REQUEST_TAG_NOT_FOUND', { 'nodename' => $nodeName } ) unless ( defined($nodeContent) );
    return $nodeContent;
}

#========================================================================================
# function     requestToString
# state        public
#----------------------------------------------------------------------------------------
# syntax       $XML = $self->requestToString();
#----------------------------------------------------------------------------------------
# description  It returns the request XML structure as a string.
#----------------------------------------------------------------------------------------
# return       $XML | XML string | string
#========================================================================================
sub requestToString {
    my $self = shift;

    my $Request = $self->getRequest();
    return $Request->toString();
}

#========================================================================================
# function     isValidRequestOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->isValidRequestOrError();
#----------------------------------------------------------------------------------------
# description  It checks validity of the request based on documented (already known)
#               requirements. It returns 1 (true) or throws an error.
#----------------------------------------------------------------------------------------
# return       $isValid | boolean
#========================================================================================
sub isValidRequestOrError {
    my $self = shift;

    my $Request = $self->getRequest();
    LogDebug('USPS. Request string to be validated: ', $Request->toString());
    return $Request->isValidRequestOrError();
}

#========================================================================================
# function     setRequestNodeEnumValueOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setRequestNodeEnumValueOrError($enumAttribute, $enumValue);
#----------------------------------------------------------------------------------------
# description  It sets the provided enumValue into the enumAttribute for the request XML
#               structure of the web service object. It throws an error if enumAttribute is
#               not found or enumValue is not defined or not valid.
#----------------------------------------------------------------------------------------
# input        $enumAttribute | XML attribute | string
# input        $enumValue | enumerated value | string
#========================================================================================
sub setRequestNodeEnumValueOrError {
    my $self = shift;
    my ( $enumAttribute, $enumValue ) = @_;

    my $Request          = $self->getRequest();
    my $enumAPInode = $Request->getAPIrequestNode($enumAttribute);
    Error( 'INVALID_REQUEST_API_NODE_NOT_FOUND', { 'name' => $enumAttribute } )
      unless ( defined ($enumAPInode) );
    my $aEnumValues = $Request->getAPInodeEnumValues($enumAPInode);
    Error( 'INVALID_REQUEST_API_NODE_VALUE', { 'name' => $enumAttribute, 'value' => $enumValue } )
        unless ElementOf($enumValue, $aEnumValues);
    $Request->setTextNodeWithinXMLrequest( $enumAttribute, $enumValue );

    return;
}

#========================================================================================
# function     setRequestNodeBooleanValueOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setRequestNodeBooleanValueOrError($booleanAttribute, $booleanValue);
#----------------------------------------------------------------------------------------
# description  It sets the boolean provided value ('true' or 'false') into the XML
#               booleanAttribute structure of the web service object. It throws an error if
#               booleanAttribute is not found or booleanValue is not defined or not valid.
#----------------------------------------------------------------------------------------
# input        $booleanAttribute | XML attribute | string
# input        $booleanValue | integer number (1 / 0) | integer
#========================================================================================
sub setRequestNodeBooleanValueOrError {
    my $self = shift;
    my ( $booleanAttribute, $booleanValue ) = @_;

    my $Request          = $self->getRequest();
    my $booleanAPInode = $Request->getAPIrequestNode($booleanAttribute);
    Error( 'INVALID_REQUEST_API_NODE_NOT_FOUND', { 'name' => $booleanAttribute } )
      unless ( isValid($booleanAPInode) );
    Error( 'INVALID_REQUEST_API_NODE_VALUE', { 'name' => $booleanAttribute, 'value' => $booleanValue } )
      unless ElementOf($booleanValue, qw(true false));
    $Request->setTextNodeWithinXMLrequest( $booleanAttribute, $booleanValue );

    return;
}

#========================================================================================
# function     setRequestNodeAttributeOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setRequestNodeAttributeOrError($nodeName, $attributeName, $attributeValue);
#----------------------------------------------------------------------------------------
# description  It sets the provided attribute value into the also provided attribute name
#               for the specified XML node.
#----------------------------------------------------------------------------------------
# input        $nodeName | node name from request object | string
# input        $attributeName | attribute name from node | string
# input        $attributeValue | value to be set | string
#========================================================================================
sub setRequestNodeAttributeOrError {
    my $self = shift;
    my ( $nodeName, $attributeName, $attributeValue ) = @_;

    my $Request = $self->getRequest();
    my $XMLnode = $Request->getXMLrequestNode($nodeName);
    Error( 'INVALID_REQUEST_XML_NODE_NOT_FOUND', { 'name' => $nodeName } )
      unless ( defined ($XMLnode) );
    Error( 'INVALID_REQUEST_XML_NODE_ATTRIBUTE_NOT_FOUND',
           { 'nodename' => $nodeName, 'attributename' => $attributeName }
         ) unless ( $XMLnode->hasAttribute($attributeName) );
    $Request->setNodeAttribute( $attributeName, $attributeValue, $XMLnode );
}

#========================================================================================
# function     getResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $XMLresponse = $self->getResponse();
#----------------------------------------------------------------------------------------
# description  It returns the XMLresponse object.
#----------------------------------------------------------------------------------------
# return       $XMLresponse | XMLresponse object | object
#========================================================================================
sub getResponse {
    my $self = shift;

    if ( isValid($self->{'Response'}) ) {
        return $self->{'Response'};
    } else {
        return '';
    }

}

#========================================================================================
# function     responseToString
# state        public
#----------------------------------------------------------------------------------------
# syntax       $XML = $self->responseToString();
#----------------------------------------------------------------------------------------
# description  It returns the response XML structure as a string.
#----------------------------------------------------------------------------------------
# return       $XML | XML string | string
#========================================================================================
sub responseToString {
    my $self = shift;

    my $Response = $self->getResponse();
    if ( isValid($Response) ) {
        return $Response->toString();
    }
    else {
        return '';
    }
}

#========================================================================================
# function     isValidResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->isValidResponse($APItag);
#----------------------------------------------------------------------------------------
# description  It checks that the response is a valid API type response or not.
#----------------------------------------------------------------------------------------
# input        $APItag | API type name | string
# return       $isValid | boolean
#========================================================================================
sub isValidResponse {
    my $self = shift;
    my ($APItag) = @_;

    my $IsValidResponse = 0;

    my $Response = $self->getResponse();
    if ( IsA( $Response, 'DE_EPAGES::USPS::API::WebServices::Base::XMLresponse' ) ){
        my $XMLrootNodeName = $Response->rootNodeName();
        if ( $XMLrootNodeName eq $APItag ){
            $IsValidResponse = 1;
        }
    }
    return $IsValidResponse;
}

#========================================================================================
# function     isValidResponseOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->isValidResponseOrError($APItag);
#----------------------------------------------------------------------------------------
# description  It checks that the response is of API web service type. If it's not,
#               it throws an error.
#----------------------------------------------------------------------------------------
# input        $APItag | API tag name | string
# return       $isValid | boolean
#========================================================================================
sub isValidResponseOrError {
    my $self = shift;
    my ($APItag) = @_;

    my $Response = $self->getResponse();
    eval { TestObject( 'response', $Response, 'DE_EPAGES::USPS::API::WebServices::Base::XMLresponse' ); };
    if ( ExistsError() ) {
        Error('INVALID_RESPONSE_NOT_AVAILABLE');
    }
    my $XMLrootNodeName = $Response->rootNodeName();
    return 1 if ( $XMLrootNodeName eq $APItag );

    if ( $self->isErrorResponse() ) {
        Error(
            'ERROR_RESPONSE',
            {
                'number'      => $self->getResponseErrorNodeContentOrEmpty('Number'),
                'description' => $self->getResponseErrorNodeContentOrEmpty('Description')
            }
          );
    }
    else {
        Error( 'INVALID_RESPONSE_UNEXPECTED_TYPE', { 'nodename' => $XMLrootNodeName, 'expectedname' => $APItag } );
    }
}

#========================================================================================
# function     isErrorResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isError = $self->isErrorResponse();
#----------------------------------------------------------------------------------------
# description  It checks that the response is an error response.
#----------------------------------------------------------------------------------------
# return       $isError | boolean
#========================================================================================
sub isErrorResponse {
    my $self = shift;

    my $Response = $self->getResponse();
    eval { TestObject( 'response', $Response, 'DE_EPAGES::USPS::API::WebServices::Base::XMLresponse' ); };
    if ( ExistsError() ) {
        Error('INVALID_RESPONSE_NOT_AVAILABLE');
    }
    my $XMLrootNodeName = $Response->rootNodeName();
    return 1 if ( $XMLrootNodeName eq 'Error' );
    return 0;
}

#========================================================================================
# function     getResponseNodeContentOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $nodeContent = $self->getResponseNodeContentOrError($APItag, $nodeName);
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists for
#               the response and returns the content. It throws an error if node can't be found.
#----------------------------------------------------------------------------------------
# input        $APItag | API type name | string
# input        $nodeName | string
# return       $nodeContent | string
#========================================================================================
sub getResponseNodeContentOrError {
    my $self = shift;
    my ($APItag, $nodeName) = @_;

    if ( $self->isValidResponseOrError($APItag) ) {
        my $Response    = $self->getResponse();
        my $nodeContent = $Response->getResponseTextContentByXMLnodeName($nodeName);
        Error( 'INVALID_RESPONSE_TAG_NOT_FOUND', { 'apitag' => $APItag, 'nodename' => $nodeName } )
          unless ( defined($nodeContent) );
        return $nodeContent;
    }
}

#========================================================================================
# function     getResponseNodeContentOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       $nodeContent = $self->getResponseNodeContentOrEmpty($APItag, $nodeName);
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists
#               for the response and returns the content. It returns an empty string if
#               no node can be found.
#----------------------------------------------------------------------------------------
# input        $APItag | API type name | string
# input        $nodeName | string
# return       $nodeContent | string
#========================================================================================
sub getResponseNodeContentOrEmpty {
    my $self = shift;
    my ($APItag, $nodeName) = @_;

    my $responseNodeContent = '';
    if ( $self->isValidResponse($APItag) ) {
        my $Response    = $self->getResponse();
        my $nodeContent = $Response->getResponseTextContentByXMLnodeName($nodeName);
        if ( isValid($nodeContent) ) {
            $responseNodeContent = $nodeContent;
        }
    }
    return $responseNodeContent;
}

#========================================================================================
# function     getResponseNodesOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       @Nodes = $self->getResponseNodesOrEmpty($APItag, $nodeName);
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists for
#               the response and returns an array including all nodes with that node name.
#               It returns an empty array if no node can be found.
#----------------------------------------------------------------------------------------
# input        $APItag | API type name | string
# input        $nodeName | string
# return       @Nodes | array of nodes object | array
#========================================================================================
sub getResponseNodesOrEmpty {
    my $self = shift;
    my ($APItag, $nodeName) = @_;

    my @Nodes = ();
    if ( $self->isValidResponse($APItag) ) {
        my $Response    = $self->getResponse();
        @Nodes = $Response->getResponseArrayNodesByXMLnodeName($nodeName);
    }
    return @Nodes;
}

#========================================================================================
# function     getResponseArrayNodesContentOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       @NodesContent = $self->getResponseArrayNodesContentOrEmpty($APItag, $nodeName);
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists for
#               the response and returns the array content of all nodes with that node name.
#               It returns an empty array if no node can be found.
#----------------------------------------------------------------------------------------
# input        $APItag | API type name | string
# input        $nodeName | string
# return       @NodesContent | array
#========================================================================================
sub getResponseArrayNodesContentOrEmpty {
    my $self = shift;
    my ($APItag, $nodeName) = @_;

    my @NodesContent = ();
    if ( $self->isValidResponse($APItag) ) {
        my $Response    = $self->getResponse();
        @NodesContent = $Response->getResponseArrayTextContentsByXMLnodeName($nodeName);
    }
    return @NodesContent;
}

#========================================================================================
# function     getResponseErrorNodeContentOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       $nodeContent = $self->getResponseErrorNodeContentOrEmpty($nodeName);
#----------------------------------------------------------------------------------------
# description  It checks that the node name exists for the response and returns the
#               content. It returns an empty string if no node can be found.
#----------------------------------------------------------------------------------------
# input        $nodeName | string
# return       $nodeContent | string
#========================================================================================
sub getResponseErrorNodeContentOrEmpty {
    my $self = shift;
    my ($nodeName) = @_;

    my $responseNodeContent = '';
    if ( $self->isErrorResponse() ) {
        my $Response    = $self->getResponse();
        my $nodeContent = $Response->getResponseTextContentByXMLnodeName($nodeName);
        if ( isValid($nodeContent) ) {
            $responseNodeContent = $nodeContent;
        }
    }
    return $responseNodeContent;
}

#========================================================================================
# function     getErrorResponseDescriptionOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorResponseDescriptionOrEmpty(  );
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists
#               for the response and returns the content. It returns an empty string if
#               no node can be found.
#----------------------------------------------------------------------------------------
# return       $ErrorDescription | Error description | String
#========================================================================================
sub getErrorResponseDescriptionOrEmpty {
    my $self = shift;

    return $self->getResponseNodeContentOrEmpty( API_RESPONSE_ERROR_TAG, 'Description' );
}

#========================================================================================
# function     getErrorResponseNumberOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorResponseNumberOrEmpty(  );
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists
#               for the response and returns the content. It returns an empty string if
#               no node can be found.
#----------------------------------------------------------------------------------------
# return       $ErrorNumber | Error Number | String
#========================================================================================
sub getErrorResponseNumberOrEmpty {
    my $self = shift;

    return $self->getResponseNodeContentOrEmpty( API_RESPONSE_ERROR_TAG, 'Number' );
}

#========================================================================================
# function     getErrorResponseSourceOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorResponseSourceOrEmpty(  );
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists
#               for the response and returns the content. It returns an empty string if
#               no node can be found.
#----------------------------------------------------------------------------------------
# return       $ErrorSource | Error Source | String
#========================================================================================
sub getErrorResponseSourceOrEmpty {
    my $self = shift;

    return $self->getResponseNodeContentOrEmpty( API_RESPONSE_ERROR_TAG, 'Source' );
}

#========================================================================================
# function     getErrorResponseHelpFileOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorResponseHelpFileOrEmpty(  );
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists
#               for the response and returns the content. It returns an empty string if
#               no node can be found.
#----------------------------------------------------------------------------------------
# return       $HelpFile | Error Help File | String
#========================================================================================
sub getErrorResponseHelpFileOrEmpty {
    my $self = shift;

    return $self->getResponseNodeContentOrEmpty( API_RESPONSE_ERROR_TAG, 'HelpFile' );
}

#========================================================================================
# function     getErrorResponseHelpContextOrEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorResponseHelpContextOrEmpty(  );
#----------------------------------------------------------------------------------------
# description  It checks that the response is of APItag type, that the node name exists
#               for the response and returns the content. It returns an empty string if
#               no node can be found.
#----------------------------------------------------------------------------------------
# return       $HelpContext | Error Help Context | String
#========================================================================================
sub getErrorResponseHelpContextOrEmpty {
    my $self = shift;

    return $self->getResponseNodeContentOrEmpty( API_RESPONSE_ERROR_TAG, 'HelpContext' );
}

1;
