#========================================================================================
# package      DE_EPAGES::WebInterface::API::Config
# state        public
# description  This module contains all functions needed to perform the communication
#               with the web interface.
#========================================================================================
package DE_EPAGES::WebInterface::API::Config;
use strict;
use base qw (Exporter);

our @EXPORT_OK = qw (
    WebInterfaceConfig
    GetRequestRouterMonitors
    IsCookieAllowed
    GetAllowedCookies
);

use DE_EPAGES::Core::API::File qw (GetCachedFile);
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw (TestString);
use DE_EPAGES::WebInterface::API::ServerConfig qw (ServerConfig);

# A delimiter used in the sync string.
# This must be the same delimiter as is used in the c module of the web interface.
my $CONFIG_FILENAME = "$ENV{'EPAGES_CONFIG'}/WebInterface.conf";

#========================================================================================
# function     WebInterfaceConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig = WebInterfaceConfig();
#----------------------------------------------------------------------------------------
# description  get ini object of the WebInterface.conf
#----------------------------------------------------------------------------------------
# return       $IniConfig | config object | object
#========================================================================================
sub WebInterfaceConfig () {
    return GetCachedFile( $CONFIG_FILENAME, sub {
        my ($FileName) = @_;
        GetLog->debug("Loading config file $FileName");
        return DE_EPAGES::Core::API::IniConfig->new(
            FileName => $FileName,
            CaseSensitive => 1,
        );
    });
}

#========================================================================================
# function     GetRequestRouterMonitors
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aMonitors = GetRequestRouterMonitors();
#----------------------------------------------------------------------------------------
# description  Read the WebInterface.conf and find all RequestRouter IP addresses and
#               monitor ports.
#----------------------------------------------------------------------------------------
# return       $aMonitors | RequestRouter IP addresses and monitor port numbers
#                   [ [ IP, port ], [IP, port ], ... ]
#                   | ref.array.array.string
#========================================================================================
sub GetRequestRouterMonitors() {
    return ServerConfig()->requestRouterMonitors();
}

#========================================================================================
# function     GetAllowedCookies
# state        public
#----------------------------------------------------------------------------------------
# syntax       @CookieNames = GetAllowedCookies();
#----------------------------------------------------------------------------------------
# description  Read the WebInterface.conf and find COOKIES at GLOBAL section.
#----------------------------------------------------------------------------------------
# return       @CookieNames | allowed cookies | array.string
#========================================================================================
sub GetAllowedCookies () {
    my $IniConfig = WebInterfaceConfig();
    my $AllowCookies = $IniConfig->get('GLOBAL', 'COOKIES');
    return () unless defined $AllowCookies;
    return split (/,/, $AllowCookies);
}

#========================================================================================
# function     IsCookieAllowed
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsAllowed = IsCookieAllowed($CookieName);
# syntax       $IsAllowed = IsCookieAllowed('SessionID');
#----------------------------------------------------------------------------------------
# description  Read the WebInterface.conf and find COOKIES at GLOBAL section, look if
#               cookie name exists in comma separated list.
#----------------------------------------------------------------------------------------
# input        $CookieName | key of cookie | string
# return       $IsAllowed | is cookie allowed | boolean
#========================================================================================
sub IsCookieAllowed ($) {
    my ($CookieName) = @_;
    TestString('CookieName', $CookieName);
    my @AllowedCookies = GetAllowedCookies();
    return ElementOf($CookieName, @AllowedCookies);
}

1;
