use strict;
use Getopt::Long;
use File::Find qw(find);

use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::Error qw( HandleError );
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileMD5Digest RunOnDirectory
                                  DeleteFile MoveFile CopyFile WriteFile WindowsPath);
use DE_EPAGES::Core::API::PerlTools qw( ElementOf CheckIpFormat UsePackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw(SystemCommand);
use DE_EPAGES::Database::API::Config qw ( DatabaseConfigFileName
                                          DatabaseConnectionData DatabaseBackupData );
use DE_EPAGES::Database::API::Connection qw( CreateStoreConf RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Dictionary::API::Template qw( ProcessFileTemplate );
use DE_EPAGES::Installer::API::Service qw( StatusService StartService StopService
                                           ExistsService EpagesCustomizedHttpdDaemon );
use DE_EPAGES::Installer::API::Constants qw( STOPPED MISSING FILENAME_INSTALL );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetCartridges );

sub usage {
    my ($ErrorNote) = @_;

    print "Error:\n    $ErrorNote\n\n" if $ErrorNote;
    print <<END_USAGE;
Description:
    Adds ASPoolDBCacheServer to WebInterface.

Usage:
    perl $0

Flags:
    -localonly     just update local files, not EPAGES_SHARED
    -reinstall     do not run migratePoolDB (is executed by Makefile.PL)
    -install       as -reinstall plus restart all servers
    -patch         do not run migratePoolDB if ASPoolDB.conf has dbi:mysql
    -help          show the command line options
END_USAGE
    exit 2;
}

sub Main {
    local $| = 1;

    my %Options = ();
    my @GetOpt = qw(help reinstall install patch localonly);

    usage() unless GetOptions(\%Options, @GetOpt);
    usage() if defined $Options{help};
    $Options{reinstall} = 1 if $Options{install};

    my $OS = $^O eq 'MSWin32' ? $^O : 'Linux';
    my $SourceDir = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/WebInterface/C/bin/$OS";
    my $ScriptDir = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/WebInterface/C/scripts/spy";
    if (not -d $SourceDir and not -d $ScriptDir) {
        print "nothing to do: no such directory:\n";
        print "  $SourceDir\n  $ScriptDir\n";
        exit 0;
    }

    #----------------------------------------------------------------------
    # gather information
    #----------------------------------------------------------------------

    my ($UpdateService, $RestartService, $UpdateFiles, $UpdateScripts, $DeleteFiles)
        = gatherInformation(\%Options);

    unless (defined $Options{localonly}) {

        #----------------------------------------------------------------------
        # create $MysqlASPoolDBConf = Database.d/ASPoolDB.conf.mysql
        #----------------------------------------------------------------------

        my $MysqlASPoolDBConf;
        if ( $UpdateService->{appserver} ) {
            my $haCartridges = GetCartridges();
            die "\nERROR: entry for ASPoolDB missing in: $ENV{EPAGES_CARTRIDGES}/Cartridges.xml\n"
                unless defined $haCartridges->{'ASPoolDB'};
            $MysqlASPoolDBConf = createMysqlASPoolDBConf(\%Options);
        }

        #----------------------------------------------------------------------
        # create ServerConfig.xml / update WebInterface.conf
        # at this point ASPoolDB is still SQLite
        #----------------------------------------------------------------------

        migrateWebInterfaceConf() if $UpdateService->{appserver};

        #----------------------------------------------------------------------
        # move ASPoolDB from SQLite to MySQL
        #----------------------------------------------------------------------

        migratePoolDB($MysqlASPoolDBConf, \%Options) if $UpdateService->{appserver};

        #----------------------------------------------------------------------
        # update shared files
        #----------------------------------------------------------------------

        if ($UpdateScripts->{appserver}) {
            foreach (@{$UpdateScripts->{appserver}}) {
                print "Updating: $ENV{EPAGES_SHARED}/Monitor/$_\n";
                CopyFile("$ScriptDir/$_","$ENV{EPAGES_SHARED}/Monitor/$_",{ create_directory => 1 });
            }
            delete $UpdateScripts->{appserver};
        }
    }

    #----------------------------------------------------------------------
    # stop services
    #----------------------------------------------------------------------

    foreach (keys %$RestartService) {
        print "StopService($_)\n";
        StopService($_);
    }

    #----------------------------------------------------------------------
    # update binaries
    #----------------------------------------------------------------------

    foreach my $Service (keys %$UpdateFiles) {
        foreach (@{$UpdateFiles->{$Service}}) {
            print "Updating: $ENV{EPAGES}/$_\n";
            CopyFile("$SourceDir/$_","$ENV{EPAGES}/$_",{ create_directory => 1 })
        }
        next unless $DeleteFiles->{$Service};
        foreach (@{$DeleteFiles->{$Service}}) {
            print "Deleting: $ENV{EPAGES}/$_\n";
            DeleteFile("$ENV{EPAGES}/$_");
        }
    }

    #----------------------------------------------------------------------
    # update scripts
    #----------------------------------------------------------------------

    foreach my $Service (keys %$UpdateScripts) {
        foreach (@{$UpdateScripts->{$Service}}) {
            print "Updating: $ENV{EPAGES}/WebAdapter/$_\n";
            CopyFile("$ScriptDir/$_","$ENV{EPAGES}/WebAdapter/$_",{ create_directory => 1 })
        }
    }

    #----------------------------------------------------------------------
    # update services (only Windows)
    #----------------------------------------------------------------------
    my %ServiceExe = (
        appserver => 'Service/epagesService',
        reqrouter => 'RequestRouter/RequestRouter',
        ascacheserver => 'ASPoolDBCacheServer/ASPoolDBCacheServer',
        );

    foreach my $Service (keys %$UpdateService) {
        next unless $^O eq 'MSWin32';
        next unless defined $ServiceExe{$Service};
        my $Exe = WindowsPath("$ENV{EPAGES}/$ServiceExe{$Service}");
        SystemCommand([$Exe, '-remove'], {'Print' => 1}) if ExistsService($Service);
        SystemCommand([$Exe, '-install'], {'Print' => 1});
    }

    #----------------------------------------------------------------------
    # start services
    #----------------------------------------------------------------------

    foreach (keys %$RestartService) {
        print "StartService($_)\n";
        StartService($_);
    }
}

#========================================================================================
# function     isMySQL
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsMySQL = isMySQL($DBConf);
#----------------------------------------------------------------------------------------
# example      my $IsMySQL = isMySQL(DatabaseConfigFileName('ASPoolDB'));
#----------------------------------------------------------------------------------------
# description  Returns true if $DBConf is MySQL, else false
#----------------------------------------------------------------------------------------
# input        $DBConf | database config file| string
# return       $IsMySQL | true or false | boolean
#========================================================================================
sub isMySQL {
    my ($DBConf) = @_;

    return '' unless -f $DBConf;

    my $IniFile = DE_EPAGES::Core::API::IniConfig->new(
        FileName => $DBConf, CaseSensitive => 1 );
    my $DataSource = $IniFile->get('Connection', 'datasource');

    return $DataSource =~ /^dbi:mysql/i;
}

#========================================================================================
# function     gatherInformation
# state        private
#----------------------------------------------------------------------------------------
# syntax       ($UpdateService, $RestartService, $UpdateFiles, $UpdateScripts, $DeleteFiles) = gatherInformation($hFlags);
#----------------------------------------------------------------------------------------
# description  Returns hashes of information
#----------------------------------------------------------------------------------------
# input        $hFlags | flags
#                   <ul>
#                       <li>install - update everything
#                   </ul>
#               | ref.hash
# return       $UpdateService  | services to be updated | ref.hash
# return       $RestartService | services to be restarted | ref.hash
# return       $UpdateFiles    | files in C/bin to be updated | ref.hash
# return       $UpdateScripts  | scripts in C/scripts to be updated | ref.hash
# return       $DeleteFiles    | files to be deleted | ref.hash
#========================================================================================
sub gatherInformation {
    my ($hFlags) = @_;

    my $TestServices = findServices();

    my ($UpdateService, $RestartService, $UpdateFiles, $UpdateScripts, $DeleteFiles);
    my $Exe = $^O eq 'MSWin32' ? '.exe' : '';

    # always update WebAdapter/ because it exists on any ePages host
    $UpdateFiles->{webserver} = 1;

    if ($hFlags->{install}) {
        foreach my $Service (qw(appserver reqrouter webserver)) {
            $UpdateFiles->{$Service} = 1;
            $UpdateService->{$Service} = 1;
            if ($Service eq 'webserver') {
                $RestartService->{'httpd'} = 1;
            } else {
                $RestartService->{$Service} = 1;
            }
        }
    } else {
        foreach my $Service (@$TestServices) {
            my $hStatus = StatusService($Service);
            next if $hStatus->{$Service} == MISSING;
            $UpdateFiles->{$Service} = 1;
            $UpdateService->{$Service} = 1;
            next if $hStatus->{$Service} == STOPPED;
            if ($Service eq 'webserver') {
                $RestartService->{'httpd'} = 1;
            } else {
                $RestartService->{$Service} = 1;
            }
        }
    }

    if ($UpdateService->{reqrouter}) {
        $RestartService->{ascacheserver} = 1 if $RestartService->{reqrouter};
        $UpdateService->{ascacheserver} = 1;
        $UpdateFiles->{ascacheserver} = 1;
    }

    foreach my $Service (keys %$UpdateFiles) {
        my $FileMissing;
        if ($Service eq 'appserver') {
            if (not -f "$ENV{EPAGES}/Service/epagesService$Exe") {
                $FileMissing = 1;
            } else {
                $UpdateFiles->{$Service} = findNewFiles(['Service', 'bin']);
                $UpdateScripts->{$Service} = findNewScripts($Service);
            }
        } elsif ($Service eq 'reqrouter') {
            if (not -f "$ENV{EPAGES}/RequestRouter/RequestRouter$Exe") {
                $FileMissing = 1;
            } else {
                $UpdateFiles->{$Service} = findNewFiles(['RequestRouter']);
            }
        } elsif ($Service eq 'ascacheserver') {
            $UpdateFiles->{$Service} = findNewFiles(['ASPoolDBCacheServer']);
        } elsif ($Service eq 'webserver') {
            my $RequiredFile = $^O eq 'MSWin32' ? 'ISAPIAdapter.dll' : 'FCGIAdapter-2.2.8.so';
            if (not -f "$ENV{EPAGES}/WebAdapter/$RequiredFile") {
                $FileMissing = 1;
            } else {
                $UpdateFiles->{$Service} = findNewFiles(['WebAdapter']);
                $UpdateScripts->{$Service} = findNewScripts($Service);
                if ($^O ne 'MSWin32') {
                    foreach (qw(FCGIAdapterSSL-2.2.8.so libWIApache-2.2.8.so
                                libWIApacheSSL-2.2.8.so libWICommon.so
                                lib64/FCGIAdapterSSL-2.2.8.so lib64/libWIApache-2.2.8.so
                                lib64/libWIApacheSSL-2.2.8.so lib64/libWICommon.so
                                zzz-epages-lb.local zzz-epages-httpd.conf.ept
                                zzz-epages-lb.conf.loop.ept zzz-epages-ssl.conf.ept
                                zzz-epages-shopexport.conf.ept
                                )) {
                        next unless -f "$ENV{EPAGES}/WebAdapter/$_";
                        push @{$DeleteFiles->{$Service}}, "WebAdapter/$_";
                    }
                }
            }
        }

        # skip service if $RequiredFile is missing
        if ( not $hFlags->{install} ) {
            delete $UpdateFiles->{$Service} if $FileMissing;
            delete $UpdateScripts->{$Service} if $FileMissing;
            delete $UpdateService->{$Service} if $FileMissing;
            delete $RestartService->{$Service} if $FileMissing;
            delete $RestartService->{httpd} if $FileMissing and $Service eq 'webserver';
        }
        # do not delete files which shall be updated
        if ($UpdateFiles->{$Service} and $DeleteFiles->{$Service}) {
            my @ObsoleteFiles;
            foreach (@{$DeleteFiles->{$Service}}) {
                push @ObsoleteFiles, $_ unless ElementOf($_, $UpdateFiles->{$Service});
            }
            $DeleteFiles->{$Service} = [ @ObsoleteFiles ];
        }
    }

    # restart service only if files have changed
    if ( $^O ne 'MSWin32' and not $hFlags->{install} ) {
        if (not ElementOf('Service/epagesService', $UpdateFiles->{appserver})) {
            delete $UpdateService->{appserver};
            delete $RestartService->{appserver};
        }
        if (not ElementOf('WebAdapter/lib64/FCGIAdapter-2.2.8.so', $UpdateFiles->{webserver})) {
            delete $UpdateService->{webserver};
            delete $RestartService->{webserver};
        }
        if (not ElementOf('RequestRouter/RequestRouter', $UpdateFiles->{reqrouter}) and
            not ElementOf('ASPoolDBCacheServer/ASPoolDBCacheServer', $UpdateFiles->{ascacheserver})) {
            delete $UpdateService->{reqrouter};
            delete $RestartService->{reqrouter};
            delete $UpdateService->{ascacheserver};
            delete $RestartService->{ascacheserver};
        }
    }

    # skip reqrouter service if ASPoolDB.conf already contains datasource=dbi:mysql
    if ( $UpdateService->{reqrouter} and not $hFlags->{install} ) {
        if (isMySQL(DatabaseConfigFileName('ASPoolDB'))) {
            delete $UpdateService->{reqrouter};
            delete $UpdateService->{ascacheserver};
        } else {
            $RestartService->{ascacheserver} = 1;
        }
    }

    if ( $UpdateService->{appserver} or $UpdateService->{reqrouter} ) {
        my $ConfFile = "$ENV{EPAGES_CONFIG}/WebInterface.conf";
        die "\nERROR: file missing: $ConfFile\n" unless -f $ConfFile;
    }

    return ($UpdateService, $RestartService, $UpdateFiles, $UpdateScripts, $DeleteFiles);
}

#========================================================================================
# function     findServices
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ServiceList = findServices();
#----------------------------------------------------------------------------------------
# description  Returns the list of ePages services the host provides
#----------------------------------------------------------------------------------------
# return       $aServiceList | array of: appserver reqrouter webserver | ref.array
#========================================================================================
sub findServices {

    # linux: /etc/init.d/epages6 must be up-to-date (containing start_asc)
    if ($^O ne 'MSWin32') {
        my $EpagesProject = $ENV{EPAGES_PROJECT} // 'epages6';
        die "\nERROR: file missing: /etc/init.d/$EpagesProject\n"
            unless -f "/etc/init.d/$EpagesProject";

        my $sContent = GetFileContent("/etc/init.d/$EpagesProject");
        die "\nERROR: outdated ePages init.d script found, update: /etc/init.d/$EpagesProject\n"
            if $$sContent !~ /start_asc/;
    }

    my @ServiceList = (qw(appserver reqrouter ascacheserver));

    if ($^O eq 'MSWin32') {
        push @ServiceList, 'webserver';
    } else {
        my $HttpdDaemon = EpagesCustomizedHttpdDaemon();
        if ($HttpdDaemon and $HttpdDaemon ne '/etc/init.d/apache2-epages') {
            # don't update webinterface if outdated apache found
            print "\nWARNING: outdated ePages Apache found; you must update: epages-webconf\n";
            exit 0;
        }
        push @ServiceList, 'webserver';
    }
    return \@ServiceList;
}

#========================================================================================
# function     findNewFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $NewFiles = findNewFiles($PathList);
#----------------------------------------------------------------------------------------
# description  Returns all files in $PathList which are new.
#----------------------------------------------------------------------------------------
# input        $PathList | find files in $PathList which are new | ref.array
# return       $NewFiles | list of new files | ref.array
#========================================================================================
sub findNewFiles {
    my ($PathList) = @_;

    my $OS = $^O eq 'MSWin32' ? $^O : 'Linux';
    my $SourceDir = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/WebInterface/C/bin/$OS";
    return [] unless -d $SourceDir;
    my @NewFiles;

    RunOnDirectory($SourceDir,
      sub {
          find(
            sub {
                next unless -f;
                my $FN = $File::Find::name;
                if (not -f "$ENV{EPAGES}/$FN") {
                    push @NewFiles, $FN;
                } elsif (GetFileMD5Digest($_) ne GetFileMD5Digest("$ENV{EPAGES}/$FN")) {
                    push @NewFiles, $FN;
                }
            }, @$PathList
          );
      }
    );
    return \@NewFiles;
}

#========================================================================================
# function     findNewScripts
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $NewScripts = findNewScripts($Service);
#----------------------------------------------------------------------------------------
# description  Returns all scripts for $Service which are new
#----------------------------------------------------------------------------------------
# input        $Service | one of: appserver reqrouter webserver | string
# return       $NewScripts | list of new scripts for $Service | ref.array
#========================================================================================
sub findNewScripts {
    my ($Service) = @_;

    my $ScriptDir = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/WebInterface/C/scripts/spy";
    return [] unless -d $ScriptDir;
    my @NewScripts;

    my $WebServerFile = 'monitor.pl';
    my %TargetDir = ( appserver => "$ENV{EPAGES_SHARED}/Monitor",
                      webserver => "$ENV{EPAGES}/WebAdapter",
        );
    return unless defined $TargetDir{$Service};

    RunOnDirectory($ScriptDir,
      sub {
          foreach my $SourceFile (glob('*')) {
              next unless -f $SourceFile;
              next if $Service eq 'webserver' and $SourceFile ne $WebServerFile;
              next if $Service eq 'appserver' and $SourceFile eq $WebServerFile;

              if (not -f "$TargetDir{$Service}/$SourceFile") {
                  push @NewScripts, $SourceFile;
              } elsif (GetFileMD5Digest($SourceFile) ne
                       GetFileMD5Digest("$TargetDir{$Service}/$SourceFile")) {
                  push @NewScripts, $SourceFile;
              }
          }
      }
    );
    return \@NewScripts;
}

#========================================================================================
# function     createMysqlASPoolDBConf
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $MysqlASPoolDBConf = createMysqlASPoolDBConf();
#----------------------------------------------------------------------------------------
# description  creates Database.d/ASPoolDB.conf.mysql
#----------------------------------------------------------------------------------------
# input        $hFlags | flags
#                   <ul>
#                       <li>patch - do not run change ASPoolDB.conf if
#                                   already has dbi:mysql
#                   </ul>
#               | ref.hash
# return       $MysqlASPoolDBConf | path to ASPoolDB.conf.mysql | string
#========================================================================================
sub createMysqlASPoolDBConf (;$) {
    my ($hFlags) = @_;

    my $ConfFile = DatabaseConfigFileName('ASPoolDB');

    if ($hFlags->{patch} and isMySQL($ConfFile)) {
        CopyFile($ConfFile, "$ConfFile.mysql") unless -f "$ConfFile.mysql";
        return "$ConfFile.mysql";
    }
    return "$ConfFile.mysql" if $hFlags->{patch} and isMySQL("$ConfFile.mysql");

    #----------------------------------------------------------------------
    # create ASPoolDB.conf.mysql
    #----------------------------------------------------------------------

    print 'Creating Database.d/ASPoolDB.conf.mysql... ';

    if (-f "$ConfFile.sqlite") {
        DeleteFile($ConfFile);
    } else {
        # backup original file so that CreateStoreConf works
        MoveFile($ConfFile, "$ConfFile.sqlite") if -f $ConfFile;
    }
    CreateStoreConf({Store => 'ASPoolDB', StoreType => 'ASPoolDB'}, {connect => 1});
    # backup new file and restore original file
    MoveFile($ConfFile, "$ConfFile.mysql");
    CopyFile("$ConfFile.sqlite", $ConfFile) if -f "$ConfFile.sqlite";
    print "done.\n";

    return "$ConfFile.mysql";
}

#========================================================================================
# function     migrateWebInterface
# state        private
#----------------------------------------------------------------------------------------
# syntax       migrateWebInterfaceConf()
#----------------------------------------------------------------------------------------
# description  Migrates WebInterface.conf, creates ServerConfig.xml.
#========================================================================================
sub migrateWebInterfaceConf {
    my $PerlExe = $^O eq 'MSWin32' ? $^X : $ENV{PERL};
    my $Script = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/WebInterface/Scripts/generateServerConfig.pl";
    my $ConfFile = "$ENV{EPAGES_CONFIG}/WebInterface.conf";

    my ($HasIpSection, $HasIPv6);
    my $IniFile = DE_EPAGES::Core::API::IniConfig->new(
        FileName => $ConfFile, CaseSensitive => 1 );
    foreach my $Section (@{$IniFile->sectionNames}) {
        # ignore sections: GLOBAL, URLRewrite and sections with wildcards
        next if $Section =~ /^GLOBAL|URLRewrite$/ || $Section =~ /\*/;

        my $IpFormatError = CheckIpFormat($Section);
        $HasIpSection = 1 unless $IpFormatError;
        next if $IpFormatError or $^O eq 'MSWin32';

        # find value for EPAGES_ENABLE_IPv6 (UNIX only)
        $IpFormatError = CheckIpFormat($Section, 6);

        unless ($IpFormatError) {
            # if $Section is IPv6
            foreach my $Key (qw(APPLICATIONSERVER MONITOR MESSAGECENTER REQUESTROUTER)) {
                my $Value = $IniFile->get( $Section, $Key );
                next unless $Value;
                next unless $Value =~ /\d/;
                # and key has port value then $HasIPv6
                $HasIPv6 = 1;
                last;
            }
        }
        last if $HasIPv6;

        my $Dedicated = $IniFile->get( $Section, 'DEDICATED' );
        next unless $Dedicated;
        # if DEDICATED has IPv6 value then $HasIPv6
        $HasIPv6 = 1 if $Dedicated =~ /:/;
        last if $HasIPv6;
    }

    if ( -f "$ENV{EPAGES_CONFIG}/ServerConfig.xml" ) {
        # do nothing if WebInterface.conf has no IP section and ServerConfig.xml exists
        return unless $HasIpSection;
        # or recreate ServerConfig.xml
        DeleteFile("$ENV{EPAGES_CONFIG}/ServerConfig.xml");
    }

    # set value for EPAGES_ENABLE_IPv6 (UNIX only)
    if ($^O ne 'MSWin32') {
        my $SetIPv6;
        $SetIPv6 = 1 if $HasIPv6 and not $ENV{EPAGES_ENABLE_IPv6};
        $SetIPv6 = '' if not $HasIPv6 and $ENV{EPAGES_ENABLE_IPv6};
        if (defined $SetIPv6) {
            my $EtcDefault = '/etc/default/'. ($ENV{EPAGES_PROJECT} ? $ENV{EPAGES_PROJECT} : 'epages6');
            my $sContent = GetFileContent($EtcDefault);
            $$sContent =~ s/(EPAGES_ENABLE_IPv6)=.*/$1=$SetIPv6/;
            WriteFile($EtcDefault, $sContent);
        }
    }

    my $Command = "\"$PerlExe\" $Script -config \"$ConfFile.cacheserver\"";

    print "Creating ServerConfig.xml/WebInterface.conf...\n";
    DeleteFile("$ConfFile.cacheserver");
    SystemCommand( $Command, { 'Print' => 1 } );
    MoveFile($ConfFile, "$ConfFile.service");
    MoveFile("$ConfFile.cacheserver", $ConfFile);
    print "done.\n";
}

#========================================================================================
# function     installASPoolDBCartridges
# state        private
#----------------------------------------------------------------------------------------
# syntax       installASPoolDBCartridges()
#----------------------------------------------------------------------------------------
# description  Installs cartridges into ASPoolDB.
#========================================================================================
sub installASPoolDBCartridges {
    my $haCartridges = GetCartridges();
    print "  install cartridges                              \n";

    foreach my $Package ( @{$haCartridges->{'ASPoolDB'}} ) {
        printf "\r    %s%30s\r", $Package, '';
        my $CartridgePackage = $Package . '::API::Cartridge';
        UsePackage($CartridgePackage);
        my $Cartridge = $CartridgePackage->new(
            StoreName => 'ASPoolDB',
            StoreType => 'ASPoolDB',
        );
        next if $Cartridge->isInstalled;
        $Cartridge->testInstall;
        if ($Cartridge->hasReasons) {
            print ProcessFileTemplate(FILENAME_INSTALL, 'en',  {
                'Store'   => 'ASPoolDB',
                'Reasons' => $Cartridge->getReasons,
            });
            exit 1;
        }
        $Cartridge->installRequiredCartridges if $Cartridge->isRecursive;
        $Cartridge->install;
    }
    return;
}

#========================================================================================
# function     migratePoolDB
# state        private
#----------------------------------------------------------------------------------------
# syntax       migratePoolDB($MysqlASPoolDBConf, $hFlags)
#----------------------------------------------------------------------------------------
# description  Migrates ASPoolDB from SQLite to MySQL.
#----------------------------------------------------------------------------------------
# input        $MysqlASPoolDBConf | path to ASPoolDB.conf.mysql | string
# input        $hFlags | flags
#                   <ul>
#                       <li>reinstall - with reinstall just copy $MysqlASPoolDBConf
#                              to ASPoolDB.conf, other stuff is made by Makefile.PL
#                       <li>patch     - do not run migratePoolDB if ASPoolDB.conf
#                                       already has dbi:mysql
#                   </ul>
#               | ref.hash
#========================================================================================
sub migratePoolDB ($;$) {
    my ($MysqlASPoolDBConf, $hFlags) = @_;

    #----------------------------------------------------------------------
    # copy ASPoolDB.conf.mysql to ASPoolDB.conf
    #----------------------------------------------------------------------

    my $ConfFile = DatabaseConfigFileName('ASPoolDB');
    return if $hFlags->{patch} and isMySQL($ConfFile);

    unless ($hFlags->{reinstall} and isMySQL($ConfFile)) {
        print 'Creating Database.d/ASPoolDB.conf... ';
        MoveFile($ConfFile, "$ConfFile.sqlite")
            if -f $ConfFile and not -f "$ConfFile.sqlite";
        CopyFile($MysqlASPoolDBConf, $ConfFile);
        print "done.\n";
    }

    return if $hFlags->{reinstall};

    #----------------------------------------------------------------------
    # create database
    #----------------------------------------------------------------------

    my $RootConnection = DatabaseBackupData('ASPoolDB');
    my $ASPoolDBConnection = DatabaseConnectionData('ASPoolDB');
    my $DBLogin    = $ASPoolDBConnection->{'login'};
    my $DBPassword = $ASPoolDBConnection->{'passwd'};
    my $DBName     = $ASPoolDBConnection->{'Database'};

    RunOnStore(
        Store      => $RootConnection->{StoreName},
        DBPassword => $RootConnection->{passwd},
        Sub        => sub {
            my $dbi = GetCurrentDBHandle();

            if ( $dbi->existsDatabase($DBName) ) {
                print "Database '$DBName' already exists.\n";
            } else {
                print "Creating MySQL database '$DBName' and login '$DBLogin'... ";
                $dbi->createEpagesDatabase( $DBName, $DBLogin, {'DBPassword' => $DBPassword});
                print "done.\n";
            }
        }
    );

    #----------------------------------------------------------------------
    # install database
    #----------------------------------------------------------------------

    print "Installing database 'aspooldb' for store 'ASPoolDB'...\n";
    RunOnStore(
        'Store'      => 'ASPoolDB',
        'DBPassword' => $DBPassword,
        'Sub'        => sub {
            eval { installASPoolDBCartridges(); };
            HandleError(
                OnError => sub {
                    print "\n"; #dont overwrite package name
                },
                Finally => sub {
                    eval { SynchronizeCache(); }; ## no critic qw(RequireExistsError) # ignore errors
                }
            );
        },
    );
    print "done.\n";

    #----------------------------------------------------------------------
    # migrate ASPoolDB
    #----------------------------------------------------------------------

    my $PerlExe = $^O eq 'MSWin32' ? $^X : $ENV{PERL};
    my $Script = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/WebInterfacePoolDB/Scripts/migratePoolDB.pl";
    my $ScriptArgs = "-clear -sqlite \"$ConfFile.sqlite\" -mysql \"$ConfFile\"";
    my $Command = "\"$PerlExe\" $Script $ScriptArgs";

    print "Migrate ASPoolDB from SQLite to MySQL...\n";
    SystemCommand( $Command, { 'Print' => 1 } );
    print "done.\n";
}

RunScript( 'Sub' => \&Main, 'Message' => 'long' );

1;
