use AnyEvent;
use AnyEvent::Handle;
use Data::Dump;
use DE_EPAGES::Core::API::Iterator             qw ( ArrayIterator );
use DE_EPAGES::Core::API::PerlTools            qw ( ElementOf GetIPAddress trim );
use DE_EPAGES::Core::API::Script               qw ( RunScript );
use DE_EPAGES::WebInterface::API::ServerConfig qw ( ServerConfig );
use English                                    qw ( -no_match_vars );    ## no critic ( DE_EPAGES::Style::RequireCorrectImports )
use Getopt::Long;
use Sys::HostIP;
use strict;

RunScript( 'Sub' => \&Main, 'Message' => 'long' );

sub Main {
    local $OUTPUT_AUTOFLUSH = 1;    ## no critic ( ProhibitLocalVars )

    my %Options = ();
    my @GetOpt = qw(help+ all+ pool=s host=s port=s reload+ reloadmodules=s@ timeout=s debug+ sleep=s);
    $Options{reload}    = 0;
    usage("Unkown parameter specified.\n") unless GetOptions( \%Options, @GetOpt );
    usage() if $Options{'help'};
    usage() unless $Options{'all'} || $Options{'port'} || $Options{'host'} || $Options{'pool'};
    $Options{'reloadmodules'} //= [];
    my @ReloadModules = @{$Options{'reloadmodules'}};
    $Options{'timeout'} //= ($Options{reload} or scalar @ReloadModules) ? 11 : 3; ## no critic qw(ProhibitMagicNumbers)
    $Options{'servers'}   = expandServerList( \%Options );
    my $Type    = $Options{'reload'} ? 'reload' : ( scalar @ReloadModules ? 'reloadmodules' : 'shutdown' );
    my $Request = "GET /\r\nX-epages-EXTRAINFO: %s\r\nuser-agent: epages Service\r\n\r\n";
    my $Modules = "\r\nX-epages-MODULELIST: ".join(',',@ReloadModules);
    $Options{'type'}    = $Type;
    $Options{'request'} = sprintf($Request, $Type . ( scalar @ReloadModules ? $Modules : ''));
    doRequests(\%Options);
}

sub expandServerList {
    my ( $Options ) = @_;

    my $PoolName = $Options->{'pool'};
    my $HostName = $Options->{'host'};
    my $Port     = $Options->{'port'};

    my $Servers;
    my $ServerConfig = ServerConfig();
    my @Groups       = values %{ $ServerConfig->appServerGroups };

    if ($PoolName) {
        my @PoolNames = map { trim($_) } split( /,/, $PoolName );
        @Groups = grep { ElementOf($_->{PoolName}, @PoolNames) } @Groups;
    }
    if ($HostName) {
        if ( $HostName =~ /^(ipv6-)?localhost$/ ) {
            # TODO: Sys::HostIP->ips does not return IPv6 addresses
            my %LocalIPs = map { $_ => undef } @{ Sys::HostIP->ips };
            @Groups = grep { exists $LocalIPs{ GetIPAddress( $_->{Host} ) } } @Groups;
        } else {
            @Groups = grep { $_->{Host} eq $HostName } @Groups;
        }
    }
    foreach my $hGroup (@Groups) {
        my $Ports = $hGroup->{'Ports'};
        my @Ports;
        foreach my $Range ( split( /,/, $Ports ) ) {
            my @RangePorts = $Range =~ /(\d+)\s*-\s*(\d+)/x ? $1 .. $2 : $Range;
            push @Ports, @RangePorts;
        }
        push @$Servers, map { { Host => $hGroup->{'Host'}, Port => $_ } } @Ports;
    }
    if ($Port) {
        $Servers = [ grep { $_->{Port} == $Port } @$Servers ];
    }
    return $Servers;
}

sub doRequests {
    my ($Options) = @_;

    dd $Options if $Options->{'debug'};
    return unless defined $Options->{'servers'};
    my $Servers      = $Options->{'servers'};
    my $Timeout      = $Options->{'timeout'};
    my $Request      = $Options->{'request'};
    my $Type         = $Options->{'type'};
    my $CountServers = scalar @$Servers;
    my $DoneServers  = 0;

    # This is our time-out event triggered by AnyEvent
    my $ExitEvent    = AnyEvent->condvar;
    my $TimeOutEvent = AnyEvent->timer (  ## no critic ( DE_EPAGES::Variables::ProhibitUnusedVariablesEnhanced )
        after => $Timeout,
        cb    => sub { $ExitEvent->send; },
    );

    # This keeps polling every 0.5s and prints out the current state of the callbacks
    my $PollEvent = AnyEvent->timer( ## no critic ( DE_EPAGES::Variables::ProhibitUnusedVariablesEnhanced )
        after => 0, interval => 0.5,
        cb => sub {
            printf "Waiting for requests to finish %s/%s\n", $DoneServers, $CountServers;
            $ExitEvent->send if ($DoneServers >= $CountServers);
        }
    );

    my @Callbacks = ();
    my $ASIterator = ArrayIterator($Servers);
    while ( my $hServer = <$ASIterator> ) {
        my $cb = sub {
            my ($Response, $Header, $Body, $Host) = @_;
            printf("AS %s responded with status %s\n", $Host, $Response);
            $DoneServers++;
        };
        sendRequest( $hServer->{'Host'}, $hServer->{'Port'}, $Request, $Type, $cb );
        push @Callbacks, $cb;
    }

    # wait until the event loop finishes or we run into the set timeout
    $ExitEvent->recv;
    if ($DoneServers < $CountServers) {
        printf "Timeout of %s seconds exceeded while waiting for one or more responses.\n", $Timeout;
    }
}

# This call returns immediatelly but jumps to the callback function once it receives anything
sub sendRequest {
    my ($Host, $Port, $Request, $Type, $Callback) = @_;
    my ($Response, $Header, $Body);

    my $Handle; $Handle = AnyEvent::Handle->new(
        connect  => [$Host => $Port],
        # Note: We get a broken pipe response on the read, no matter what as the ApplicationServer.pm just
        #     destroys the socket on receive without sending a proper HTTP::Reponse
        #     However, the AS receives the restart, reload or reloadmodules requests normally
        #     So we fake a 200 status here for convenience
        on_error => sub {
            $Callback->('HTTP/1.0 200', '', 'Request received', $Host.':'.$Port);
            $Handle->destroy;
        },
        on_eof   => sub {
            $Callback->($Response, $Header, $Body, $Host . ':' . $Port);
            $Handle->destroy;
        }
    );
    # If the AS ever behaves normally on shutdowns we would use the reads the get a proper response
    $Handle->push_write($Request);
    $Handle->push_read (line => sub {
        my ($Handle, $Line) = @_; ## no critic ( Variables::ProhibitReusedNames) )
        $Response = $Line;
    });
    $Handle->push_read (line => "\r\n\r\n", sub {
        my ($Handle, $Line) = @_; ## no critic ( Variables::ProhibitReusedNames) )
        $Header = $Line;
    });
    $Handle->on_read (sub {
        $Body .= $_[0]->rbuf;
        $_[0]->rbuf = '';
    });
}

sub usage {
    my ($ErrorNote) = @_;

    print $ErrorNote if $ErrorNote;
    print <<END_USAGE;
Description:
        Stops/Reloads all application servers by sending a shutdown, reload or reloadmodules request.
        The servers will be restarted automatically by the epages Service if shutdowned.
        Note that 'reload' can take a few seconds per AS.

Usage:
    perl $PROGRAM_NAME [flags] [options]

options:
    -all             restart all processes (optional)
    -pool            (comma separated list) restart all processes associated with this pool(s) (optional)
    -host            restart only processes on this server
                     (either IP or 'localhost' for all local IPs) (required)
    -port            restart only the process using this port (optional)
    -timeout         total real wait time for all responses until script exits (if an appserver hangs for example)
    -reloadmodules   comma separated list of package names or relative filenames to reload without restarting AS
                     (e.g. DE_EPAGES::Presentation::UI::Object or DE_EPAGES/Presentation/UI/Object.pm )
    -sleep           obsolete parameter
flags:
    -help            show the command line options
    -reload          do not restart the processes, just reload pool assignments

Example:
    perl $PROGRAM_NAME -pool DefaultPool
    perl $PROGRAM_NAME -host localhost -timeout 20
    perl $PROGRAM_NAME -host localhost -reload
    perl $PROGRAM_NAME -host localhost -reloadmodules DE_EPAGES::Presentation::UI::Object

END_USAGE
    exit 2;
}
