#========================================================================================
# package      DE_EPAGES::WebService::API::Resource::BaseObjectResource
# state        public
# description  Base class for REST resources.
#========================================================================================
package DE_EPAGES::WebService::API::Resource::BaseObjectResource;
use base DE_EPAGES::WebService::API::Resource::BaseResource;
use strict;

use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw( TestString );

sub ErrorDefinition {
    return {
        ResourceNotFound => '#Class "#GUID" does not exist in #Site',
    };
}

#========================================================================================
# function     class
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Resource->class;
#----------------------------------------------------------------------------------------
# description  Returns the class name of objects that are represented by this resource.
#               The value is passed to $Site->loadObject($ObjectID, $Class);
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | string
#========================================================================================
sub class {
    Error('NotImplemented', { method => 'class'});
}

#========================================================================================
# function     service
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RestService = $Resource->service;
#----------------------------------------------------------------------------------------
# description  Returns the RestService object that contains the RestServiceMethod
#               objects which implement the actual actions.
#----------------------------------------------------------------------------------------
# return       $RestService | rest service object | DE_EPAGES::WebService::API::Object::RestService
#========================================================================================
sub service {
    Error('NotImplemented', { method => 'service'});
}

#========================================================================================
# function     actionByName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Action = $Resource->actionByName($ActionName);
#----------------------------------------------------------------------------------------
# description  Gets the Action object with a given name object from the REST service.
#----------------------------------------------------------------------------------------
# input        $hData | resource data | ref.hash
# return       $Action | action object | DE_EPAGES::Permission::API::Object::Action
#========================================================================================
sub actionByName {
    my $self = shift;
    my ($ActionName) = @_;
    TestString('ActionName', $ActionName);

    return $self->service->child('Actions')->child($ActionName);
}

sub requestContext {
    my $self = shift;
    my ($Site, $GUID, $Request, $aPathParams) = @_;

    my $Object = $self->loadObject($Site, $GUID);
    my ($ActionName, $ResponseCodec) = $self->chooseAction($Request);

    return {
        Object => $Object,
        Action => $self->actionByName($ActionName),
        Params => $self->params($ActionName, $Object, $Request, $aPathParams),
        ResponseCodec => $ResponseCodec,
        RequestCodec => $self->requestCodec($Request),
    };
}

sub loadObject {
    my $self = shift;
    my ($Site, $GUID) = @_;

    # special case: POST/PUT without a guid match: uses '1' instead.
    if($GUID eq '1') {
        return $Site;
    }

    # TODO: ignore ObjectNotFound for PUT|DELETE
    my $Object = eval { $Site->loadObjectByGuid( $GUID, $self->class ) };
    if (ExistsError()) {
        my ($Error) = GetError();
        Error('ResourceNotFound', {
            GUID => $GUID,
            Class => $self->class,
            Site => $Site->pathString,
        }) if ElementOf($Error->code, qw(ClassMismatch NoResults_ObjectByGUID SiteMismatch));
        Error($Error);
    }
    return $Object;
}

1;
