#========================================================================================
# package      DE_EPAGES::WebService::API::Resource::BaseResource
# state        public
# description  Base class for REST resources.
#========================================================================================
package DE_EPAGES::WebService::API::Resource::BaseResource;
use strict;

use feature 'switch';
use HTTP::Negotiate; # choose;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::WebService::API::CodecRegistry;

sub ErrorDefinition {
    return {
        MethodNotImplemented => 'Request method "#Method" is not implemented for "#URI"',
        NotAcceptable => '"#Method #URI" does not produce the requested content-type "#Accept"',
    };
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Resource = DE_EPAGES::WebService::API::Resource::BaseResource->new(%Options);
#----------------------------------------------------------------------------------------
# description  Create a new REST resource object.
#----------------------------------------------------------------------------------------
# input        %Options | (optional) options, currently not used | hash
# return       $Resource | resource object | DE_EPAGES::WebService::API::Resource::BaseResource
#========================================================================================
sub new {
    my $class = shift;
    my (%Options) = @_;

    return bless {
        %Options
    }, $class;
}

#========================================================================================
# function     match
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Match = $Resource->match($Request);
# example      $Match = $Resource->match($Request);
#               if (defined $Match) {
#                   my $hRequestContext = $Resource->requestContext($Site, $Match, $Request);
#               }
#----------------------------------------------------------------------------------------
# description  Tests if the resource can handle the request.
#               If no, returns an empty list.
#               If yes, returns a defined value. In some cases this value can be
#               interpreted as object identifier for the actual resource.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $Match | object identifier | string
#========================================================================================
sub match {
    my $self = shift;
    my ($Request) = @_;

    Error('NotImpemented', { method => 'match' });
}

#========================================================================================
# function     href
# state        public
#----------------------------------------------------------------------------------------
# syntax       $href = $Package->href($APIBaseURL, @Params);
# example      {
#                   rel => 'variations',
#                   href => DE_EPAGES::Product::API::Resource::ProductVariation->href($APIBaseURL, $SuperProduct->guid),
#               }
#----------------------------------------------------------------------------------------
# description  build an reference link based on API url and ressource related parameters
#----------------------------------------------------------------------------------------
# input        $APIBaseURL | rbase API url | string
# input        @Params | resource related parameters | array of string
# return       $href | href of ressource | string
#========================================================================================
sub href {
    my $self = shift;
    my ($APIBaseURL, @Params ) = @_;

    Error('NotImpemented', { method => 'href' });
}

#========================================================================================
# function     params
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->params($ActionName, $Match, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the parameters for the action function.
#----------------------------------------------------------------------------------------
# input        $ActionName | name of the action  | DE_EPAGES::Object::API::Object::Site
# input        $Match | result of <function match> | string
# input        $Request | request object | HTTP::Request
# return       $hRequestContext | request data with keys:
#                   <ul><li>Object - business object - DE_EPAGES::Object::API::Object::Object</li>
#                       <li>Action - action object or sub ref to handle the request - ref.code or DE_EPAGES::Permission::API::Object::Action</li>
#                       <li>Params - parameters to the action function - ref.array</li>
#                       <li>RequestCodec - </li>
#                       <li>ResponseCodec - </li>
#                   </ul> | ref.hash
#========================================================================================
sub params {
    my $self = shift;
    my ($ActionName, $Match, $Request) = @_;

    return [ $Match ];
}

#========================================================================================
# function     requestContext
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->requestContext($Site, $Match, $Request);
#----------------------------------------------------------------------------------------
# description  Tests if the resource can handle the request.
#               If no, returns an empty list.
#               If yes, returns the function to execute the request.
#----------------------------------------------------------------------------------------
# input        $Site | site object | DE_EPAGES::Object::API::Object::Site
# input        $Match | result of <function match> | string
# input        $Request | request object | HTTP::Request
# return       $hRequestContext | request data with keys:
#                   <ul><li>Object - business object - DE_EPAGES::Object::API::Object::Object</li>
#                       <li>Action - action object or sub ref to handle the request - ref.code or DE_EPAGES::Permission::API::Object::Action</li>
#                       <li>Params - parameters to the action function - ref.array</li>
#                       <li>RequestCodec - </li>
#                       <li>ResponseCodec - </li>
#                   </ul> | ref.hash
#========================================================================================
sub requestContext {
    my $self = shift;
    my ($Site, $Match, $Request, $aPathParams) = @_;

    my ($ActionName, $ResponseCodec) = $self->chooseAction($Request);

    my $Sub = sub {
        my (@SubParams) = @_;
        return $self->$ActionName(@SubParams);
    };

    return {
        Object => $Site,
        Action => $Sub,
        Params => $self->params($ActionName, $Match, $Request, $aPathParams),
        RequestCodec => $self->requestCodec($Request),
        ResponseCodec => $ResponseCodec,
    };
}

#========================================================================================
# function     producedContentTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aVariants = $Resource->producedContentTypes( $RequestMethod );
#----------------------------------------------------------------------------------------
# description  Returns the available result content types of the resource for a given
#               request method.
#               Returns undefined if the method does not produce any content.
#----------------------------------------------------------------------------------------
# input        $RequestMethod | request method, like 'GET' or 'POST' | string
# return       $aVariants | list of possible variants in order of precedence.
#                   Each variant is an array with the following entries:
#                   <ul>
#                       <li>0 - action or method name</li>
#                       <li>1 - content-type string</li>
#                   </ul> | ref.array.array
#========================================================================================
sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/json'],
            ['get', 'application/x-yaml']
        ],
        POST => [
            ['post', 'application/json'],
        ],
        PUT => [
            ['put', undef],
        ],
        DELETE => [
            ['delete', undef],
        ],
        PATCH => [
            ['patch', 'application/json'],
        ],
    }->{$RequestMethod};
}

#========================================================================================
# function     chooseVariant
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aVariant = $Resource->chooseVariant($Request);
#----------------------------------------------------------------------------------------
# description  Uses HTTP content negotiation to find the appropriate content variant
#               and the corresponding implementation.
#               Throws a NotAcceptable error if the request cannot be served.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $aVariant | the selected variant of the $aVariants list in
#                       <function producedContentTypes>
#========================================================================================
sub chooseVariant {
    my $self = shift;
    my ($Request) = @_;

    my $aVariants = $self->producedContentTypes( uc $Request->method );
    Error('MethodNotImplemented', {
        URI => $Request->uri,
        Method => $Request->method,
    }) unless defined $aVariants;

    my $Index = 0;
    my @Variants = map { [ $Index++, 1 - $Index/10, $_->[1], undef, 'utf-8', undef, undef ] } @$aVariants;

    my $Choice = HTTP::Negotiate::choose( \@Variants, $Request );
    Error('NotAcceptable', {
        URI => $Request->uri,
        Accept => $Request->header('Accept'),
        Method => $Request->method,
    }) unless defined $Choice;

    return $aVariants->[$Choice];
}

#========================================================================================
# function     chooseAction
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($ActionName, $ResponseCodec) = $Resource->chooseAction($Request);
#----------------------------------------------------------------------------------------
# description  Uses HTTP content negotiation to find the appropriate content variant
#               and the corresponding implementation.
#               Throws a NotAcceptable error if the request cannot be served.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $ActionName | action name from the $aVariants list in <function producedContentTypes> | string
# return       $ResponseCodec | response encoder | object
#========================================================================================
sub chooseAction {
    my $self = shift;
    my ($Request) = @_;

    my $aVariant = $self->chooseVariant($Request);
    my ($ActionName, $ContentType) = @$aVariant;

    my $ResponseCodec = defined $ContentType
        ? $self->codecByContentType($ContentType)
        : undef;

    return $ActionName, $ResponseCodec;
}

#========================================================================================
# function     codecByContentType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Codec = $Resource->codecByContentType($ContentType);
# example      if ($RequestCodec) {
#                   $Data = $RequestCodec->decode($Request);
#               }
#----------------------------------------------------------------------------------------
# description  Returns a suitable decoder/encoder object for a message with the given
#               Content-Type.
#               Returns undefined if there is no suitable codec.
#----------------------------------------------------------------------------------------
# input        $ContentType | value of the Content-Type header | string
# return       $Codec | content encoder/decoder |object
#========================================================================================
sub codecByContentType {
    my $self = shift;
    my ($ContentType) = @_;

    return DE_EPAGES::WebService::API::CodecRegistry->get($ContentType);
}

#========================================================================================
# function     requestCodec
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RequestCodec = $Resource->requestCodec($Request);
# example      if ($RequestCodec) {
#                   $Data = $RequestCodec->decode($Request);
#               }
#----------------------------------------------------------------------------------------
# description  Returns a suitable decoder object to deserialze the request body.
#               Returns undefined if the request does not have a Content-Type header or
#               if there is no suitable decoder for the given Content-Type.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $RequestCodec | content encoder/decoder | object
#========================================================================================
sub requestCodec {
    my $self = shift;
    my ($Request) = @_;

    my $RequestContentType = $Request->content_type;
    return undef unless $RequestContentType;

    my $RequestCodec = $self->codecByContentType($RequestContentType) or Error('BadRequest', {
        'ContentType' => $RequestContentType
    });

    return $RequestCodec;
}

1;
