#========================================================================================
# package      DE_EPAGES::WebService::API::SoapServer
# state        private
# base         SOAP::Transport::HTTP::Server
#----------------------------------------------------------------------------------------
# description  epages SOAP server implementation. This package inherits from
#               SOAP::Transport::HTTP::Server and overrides the request dispatching.
#========================================================================================
package DE_EPAGES::WebService::API::SoapServer;
use strict;

use vars qw(@ISA);
@ISA = qw(SOAP::Transport::HTTP::Server); ##no critic (ProhibitExplicitISA)

use SOAP::Transport::HTTP; ##no critic (RequireCorrectImports)
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog GetCommunicationLog LogError );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf UsePackage IsA );
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByGUID );
use DE_EPAGES::WebService::API::WebService qw( LoadWebServiceByURI );
use DE_EPAGES::WebInterface::API::ApplicationServer qw ( GetAppServer );
use DE_EPAGES::XML::API::Tidy qw( tidy );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        SERVICE_METHOD_NOTPUBLIC     => 'The method (#Method) of service (#WebService, #URI) requires Basic Authentication',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     handle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SoapServer->handle( $Servlet, $Request, $Response );
#----------------------------------------------------------------------------------------
# description  calls SOAP::Transport::HTTP::Server::handle(),
#               fills the resulting HTTP Response object with return data
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Request | HTTP::Request object | object
# input        $Response | HTTP::Response object | object
#========================================================================================
sub handle {
    my $self = shift;
    my ( $Servlet, $Request, $Response ) = @_;

    my $Log = GetCommunicationLog('SoapServer');
    if( $Log->is_debug ) {
        # $Log->debug("Request Headers:\n".$Request->headers_as_string);
        $Log->debug("Request content:\n".tidy($Request->content));
    }

    if( $Request->authorization ) {
        ($self->{'Login'}, $self->{'Password'}) = $Request->authorization_basic;
    }
    else {
        $self->{'Login'} = undef;
        $self->{'Password'} = undef;
    }

    $self->{'SiteGUID'}      //= $Request->header('X-ePages-Site');
    $self->{'UseGuidAsPath'} //= ParseQueryString($Request->uri->query, 'utf-8')->{'useGuidAsPath'};

    # try to handle the request
    $self->request( $Request );
    $self->SUPER::handle;

    # authorization required?
    if ($self->{'RequestAuthentication'}) {
        $Response->code( 401 ); ##no critic qw(ProhibitMagicNumbers)
        $Response->message( 'Unauthorized' );
        $Response->www_authenticate( 'Basic realm=\"epages web service\"' );
        return;
    }

    # copy $self->response to $Response
    my $SoapResponse = $self->response;
    $Response->code( $SoapResponse->code );
    $Response->message( $SoapResponse->message );

    # clone all headers
    my $SoapResponseHeaders = $SoapResponse->headers;
    my $ResponseHeaders = $Response->headers;
    $SoapResponseHeaders->scan( sub { $ResponseHeaders->push_header(@_); } );

    # clone the content
    $Response->content( $SoapResponse->content );

    if( $Log->is_debug ) {
        # $Log->debug("Response Headers:\n".$Response->headers_as_string);
        $Log->debug("Response content:\n".tidy($Response->content));
    }

    #handle asynchron WS
    my $WSession = $self->{'WebServiceSession'};

    if( $WSession ) {
        my $AppServer = GetAppServer();
        my $WebAdapter = $AppServer->webadapter;

        if (defined $WebAdapter) {
            $WebAdapter->send($Response);
        } else {
            die 'undefined webadapter';
        }

        my $StatusMessage;
        eval {
            $StatusMessage = $WSession->doSub();
        };
        if(ExistsError()) {
            my $Error = GetError();
            LogError( $Error );
            $WSession->error( $Error->message );
        } else {
            $WSession->end($StatusMessage);
        }
    }

    return;
}

#========================================================================================
# function     find_target
# state        private
#----------------------------------------------------------------------------------------
# syntax       ($class, $method_uri, $MethodName) = $soap_server->find_target( $request );
#----------------------------------------------------------------------------------------
# description  Determines the web service URI, the implementing package and method from
#               a SOAP request.
#               The package is loaded via 'eval "use $Package"' if it's not yet compiled.
#               Thows the error SERVICE_METHOD_NOTPUBLIC if the method requires
#               authentication and/or invalid credentials have been passed.
#----------------------------------------------------------------------------------------
# input        $request | SOAP request object | object
# return       $class | package name | string
# return       $method_uri | method URI | string
# return       $MethodAlias | method name | string
#========================================================================================
sub find_target {
    my $self = shift;
    my ($request) = @_;

    # get method name from the envelope
    my $method_uri = $request->namespaceuriof || '';
    my $MethodAlias = $request->dataof->name;

    GetLog()->debug("find_target uri=$method_uri, name=$MethodAlias");

    # get the implementing web service
    my $WebService = LoadWebServiceByURI( $method_uri );
    my $Method = $WebService->method($MethodAlias);
    my $Package = $Method->get('Package');
    my $FunctionName = $Method->get('FunctionName');

    GetLog()->debug('found target webservice='.$WebService->get('Alias')
        .', method='.$Method->get('Alias')
        .", package=$Package"
        .", function=$FunctionName"
    );

    # load module into memory
    UsePackage($Package);

    my $Service = $Package->new(
        Object        => LoadObjectByGUID($self->{'SiteGUID'}),
        UseGuidAsPath => $self->{'UseGuidAsPath'},
    );

    # check if the web service suports authorization
    if( ! $Service->CheckPermission( $WebService, $Method, $self->{'Login'}, $self->{'Password'} ) )
    {
        GetLog()->debug( 'permission check failed for login '. (defined $self->{'Login'}
        ? "'$self->{'Login'}'" : '(undef)') );

        $self->{'RequestAuthentication'} = 1;
        Error( 'SERVICE_METHOD_NOTPUBLIC', {
            URI        => $method_uri,
            WebService => $WebService->get('Alias'),
            Method     => $MethodAlias
        } );
    }

    return ($Service, $method_uri, $FunctionName);
}

# SOAP::Lite creates invalid XML if the error string $@ constains '&'.
# Therefore we encode the string here...
sub make_fault {
    my $self = shift;
    my($code, $string, $detail, $actor) = @_;


    my $Error;
    if( IsA( $detail, 'DE_EPAGES::Core::API::Error' ) ) {
        $Error = $detail;
    }
    else {
        # extract the raw message text, remove file name and line number
        $string =~ s/\s*at\ \S+\ line\ \d+.*//sx;
        $Error = DE_EPAGES::Core::API::Error->new(
            Code => 'NativeDie',
            Message => $string,
            Vars => {
                detail => $detail,
            }
        );
    }

    LogError( $Error )
        unless ElementOf($Error->code, qw(SERVICE_METHOD_NOTPUBLIC FeatureNotAvialable FeatureLimitExceeded));

    my %ErrorDetails = (
        'Code' => $Error->code,
        'Hash' => $Error->hash,
        'GUID' => $Error->guid,
    );
    # EPG-33470: add extra values only for selected error codes
    $ErrorDetails{'Vars'} = $Error->vars
        if ElementOf($Error->code, qw( FeatureNotAvialable FeatureLimitExceeded ));
    $detail = SOAP::Data->name('ErrorDetails')->value(\%ErrorDetails);
    $string = SOAP::Utils::encode_data( substr($Error->shortMessage, 0, 200) ); ## no critic qw(ProhibitMagicNumbers(
    return $self->SUPER::make_fault($code, $string, $detail, $actor);
}

1;
