#========================================================================================
# package      DE_EPAGES::WebService::API::Util
# state        public
# description  Helper methods for rest resources.
#========================================================================================
package DE_EPAGES::WebService::API::Util;
use base qw( Exporter );

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestHash );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountry GetInfoCountryByCode2 );
use DE_EPAGES::XML::API::DateTimeFormatXML;
use DateTime::Format::Strptime qw( );
use DE_EPAGES::Object::API::Object::Locale;

use JSON::XS;
use strict;



our @EXPORT_OK = qw (
    ConvertAddressToJava
    ConvertOrderDatesForEPJ
    ConvertAddressToPerl
    GetAddressAttributeNames
    ConvertOrderDatesForEP6
    ConvertDateForEP6
    ReadJSONBoolean
    TranslateAttributeName
);



#========================================================================================
# function     ConvertAddressToJava
# state        public
#----------------------------------------------------------------------------------------
# syntax       ConvertAddressToJava( $hAddress );
#----------------------------------------------------------------------------------------
# description  Converts the address object for REST Services, Perl to Java.
#----------------------------------------------------------------------------------------
# input        $hAddress | Address in hash map from perl business object | hash
# return       $hJavaAddress | Address in hash map for epagesj | hash
#========================================================================================
sub ConvertAddressToJava {
    my ( $hAddress ) = @_;

    return undef unless(defined($hAddress));
    return undef if(ref($hAddress) ne 'HASH');

    my $CountryCode = defined $hAddress->{'CountryID'}
        ? GetInfoCountry($hAddress->{'CountryID'})->{'Code2'}
        : undef;
    my $Birthday = $hAddress->{'Birthday'} ? $hAddress->{'Birthday'}->ymd : undef;

    my $Gender;
    if ($hAddress->{'Gender'} eq '1') { $Gender = 'MALE'; }
    elsif ($hAddress->{'Gender'} eq '2') { $Gender = 'FEMALE'; }
    else { $Gender = undef; }

    my $hJavaAddress = {
        'company'       => $hAddress->{'Company'},
        'salutation'    => $hAddress->{'Salutation'},
        'firstName'     => $hAddress->{'FirstName'},
        'lastName'      => $hAddress->{'LastName'},
        'street'        => $hAddress->{'Street'},
        'streetDetails' => $hAddress->{'Street2'},
        'zipCode'       => $hAddress->{'Zipcode'},
        'city'          => $hAddress->{'City'},
        'state'         => $hAddress->{'State'},
        'country'       => $CountryCode,
        'title'         => $hAddress->{'Title'},
        'vatId'         => $hAddress->{'VATID'},
        'emailAddress'  => $hAddress->{'EMail'},
        'birthday'      => $Birthday,
        'gender'               => $Gender,
        'middleName'           => $hAddress->{'MiddleName'},
        'displayName'          => $hAddress->{'DisplayName'},
        'addressExtension'     => $hAddress->{'AddressExtension'},
        'doorCode'             => $hAddress->{'CodePorte'},
        'fiscalCode'           => $hAddress->{'FiscalCode'},
        'department'           => $hAddress->{'Department'},
        'jobTitle'             => $hAddress->{'JobTitle'},
        'phoneNumber'          => $hAddress->{'Phone'},
        'businessPhoneNumber'  => $hAddress->{'PhoneBusiness'},
        'mobilePhoneNumber'    => $hAddress->{'PhoneCell'},
        'privatePhoneNumber'   => $hAddress->{'PhonePrivate'},
        'faxNumber'            => $hAddress->{'Fax'},
        'businessEmailAddress' => $hAddress->{'EMailBusiness'},
        'privateEmailAddress'  => $hAddress->{'EMailPrivate'},
        'websiteUrl'           => $hAddress->{'URL'},
        'bankAccountHolder'    => $hAddress->{'BankAccountHolder'},
        'bankAccountNumber'    => $hAddress->{'BankAccountNo'},
        'bankSortCode'         => $hAddress->{'BankCode'},
        'bankName'             => $hAddress->{'BankName'},
    };
    return $hJavaAddress;
}



#========================================================================================
# function     ConvertAddressToPerl
# state        public
#----------------------------------------------------------------------------------------
# syntax       ConvertAddressToPerl( $hAddress );
#----------------------------------------------------------------------------------------
# description  Converts the address object for REST Services, Java to Perl.
#----------------------------------------------------------------------------------------
# input        $hAddress | Address in hash map from epagesj | hash
# return       $hPerlAddress | Address in hash map for ep6 | hash
#========================================================================================
sub ConvertAddressToPerl {
    my ( $hAddress ) = @_;

    return unless(defined($hAddress));
    return if(ref($hAddress) ne 'HASH');

    my $Format = DateTime::Format::Strptime->new(
                    pattern   => '%Y-%m-%d',
                    on_error  => 'croak',
                );
    my $BirthDate = $hAddress->{'birthday'} ? $Format->parse_datetime($hAddress->{'birthday'}): undef;

    my $Gender;
    if ($hAddress->{'gender'} eq 'MALE') { $Gender = 1; }
    elsif ($hAddress->{'gender'} eq 'FEMALE') { $Gender = 2; }
    else { $Gender = undef; }

    my $CountryID = defined $hAddress->{'country'}
        ? GetInfoCountryByCode2($hAddress->{'country'})->{'CountryID'}
        : undef;

    my $hPerlAddress = {
        'Company'    => $hAddress->{'company'},
        'Salutation' => $hAddress->{'salutation'},
        'FirstName'  => $hAddress->{'firstName'},
        'LastName'   => $hAddress->{'lastName'},
        'Street'     => $hAddress->{'street'},
        'Street2'    => $hAddress->{'streetDetails'},
        'Zipcode'    => $hAddress->{'zipCode'},
        'City'       => $hAddress->{'city'},
        'State'      => $hAddress->{'state'},
        'CountryID'  => $CountryID,
        'Title'      => $hAddress->{'title'},
        'VATID'      => $hAddress->{'vatId'},
        'EMail'      => $hAddress->{'emailAddress'},
        'Birthday'   => $BirthDate,
        'Gender'            => $Gender,
        'MiddleName'        => $hAddress->{'middleName'},
        'DisplayName'       => $hAddress->{'displayName'},
        'AddressExtension'  => $hAddress->{'addressExtension'},
        'CodePorte'         => $hAddress->{'doorCode'},
        'FiscalCode'        => $hAddress->{'fiscalCode'},
        'Department'        => $hAddress->{'department'},
        'JobTitle'          => $hAddress->{'jobTitle'},
        'Phone'             => $hAddress->{'phoneNumber'},
        'PhoneBusiness'     => $hAddress->{'businessPhoneNumber'},
        'PhoneCell'         => $hAddress->{'mobilePhoneNumber'},
        'PhonePrivate'      => $hAddress->{'privatePhoneNumber'},
        'Fax'               => $hAddress->{'faxNumber'},
        'EMailBusiness'     => $hAddress->{'businessEmailAddress'},
        'EMailPrivate'      => $hAddress->{'privateEmailAddress'},
        'URL'               => $hAddress->{'websiteUrl'},
        'BankAccountHolder' => $hAddress->{'bankAccountHolder'},
        'BankAccountNo'     => $hAddress->{'bankAccountNumber'},
        'BankCode'          => $hAddress->{'bankSortCode'},
        'BankName'          => $hAddress->{'bankName'},
    };
    return $hPerlAddress;
}



#========================================================================================
# function     GetAddressAttributeNames
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetAddressAttributeNames();
#----------------------------------------------------------------------------------------
# description  Returns an array with the names of all address attributes
#----------------------------------------------------------------------------------------
# return       Array of all address attribute names | array
#========================================================================================
sub GetAddressAttributeNames {

    my @Attributes = qw (
        Company
        Salutation
        FirstName
        LastName
        Street
        Street2
        Zipcode
        City
        State
        CountryID
        Title
        VATID
        Birthday
        EMail
        Gender
        MiddleName
        DisplayName
        AddressExtension
        CodePorte
        Country
        State
        FiscalCode
        Department
        JobTitle
        Phone
        PhoneBusiness
        PhoneCell
        PhonePrivate
        Fax
        EMailBusiness
        EMailPrivate
        URL
        BankAccountHolder
        BankAccountNo
        BankCode
        BankName
    );

    return @Attributes;
}



#========================================================================================
# function     ConvertOrderDatesForEPJ
# state        public
#----------------------------------------------------------------------------------------
# syntax       ConvertOrderDatesForEPJ($Order, $hEpjOrder);
#----------------------------------------------------------------------------------------
# description  Converts the dates for epagesj.
#----------------------------------------------------------------------------------------
# input        $Order | Order from ep6 | hash
# input        $hEpjOrder | Order for epagesj | hash
# return       $hEpjOrder | Order in hash map for epagesj | hash
#========================================================================================
sub ConvertOrderDatesForEPJ {
    my ($Order,$hEpjOrder) = @_;
    TestHash('Order',$Order);
    my $DF= DE_EPAGES::XML::API::DateTimeFormatXML->new;
    my $DateInUTC;

    foreach my $Key (keys %$Order) {
        if ($Key =~ /(.*)On$/ || $Key =~ /(.*)Date$/) {
            #camel-case for epagesj
            my $NewKey = lcfirst($Key);
            my $Value = $Order->{$Key};
            if(isValid($Value)){
                $DateInUTC = $DF->format_datetime( $Value, 'UTC' );
                $hEpjOrder->{$NewKey} = $DateInUTC;
            }
        }
    }
    return $hEpjOrder;
}



#========================================================================================
# function     ConvertOrderDatesForEP6
# state        public
#----------------------------------------------------------------------------------------
# syntax       ConvertOrderDatesForEP6($Order, $hRequestBody);
#----------------------------------------------------------------------------------------
# description  Converts the dates for ep6.
#----------------------------------------------------------------------------------------
# input        $Order | Order from ep6 | hash
# input        $hRequestBody | Order from epagesj | hash
# return       $Order | Order in hash map for ep6 with dates | hash
#========================================================================================
sub ConvertOrderDatesForEP6 {
    my ($Order, $hRequestBody) = @_;

    my $DF = DE_EPAGES::XML::API::DateTimeFormatXML->new();
    my $DateInUTC;

    foreach my $Key (keys %$hRequestBody) {
        # we do not write CreationDate so only (.*)On is considered
        if ($Key =~ /(.*)On$/) {
            #first char upper-case for epages6
            my $NewKey = ucfirst($Key);
            my $Value = $hRequestBody->{$Key};
            if(isValid($Value)){
                $DateInUTC = $DF->parse_datetime($Value);
                $Order->set( { $NewKey => $DateInUTC } );
            }else {
                $Order->set( { $NewKey => undef } );
            }
        }
    }
    return $Order;
}



#========================================================================================
# function     ConvertDateForEP6
# state        public
#----------------------------------------------------------------------------------------
# syntax       ConvertDateForEP6($DateInUTC);
#----------------------------------------------------------------------------------------
# description  Converts the date for ep6.
#----------------------------------------------------------------------------------------
# input        $DateInUTC | epj date | string
# return       $DateInUTC | ep6 date | string
#========================================================================================
sub ConvertDateForEP6 {
    my ($DateInUTC) = @_;

    my $DF = DE_EPAGES::XML::API::DateTimeFormatXML->new();

    if(isValid($DateInUTC)){
        $DateInUTC = $DF->parse_datetime($DateInUTC);
    }
    return $DateInUTC;
}

#========================================================================================
# function     ConvertDateForEPJ
# state        public
#----------------------------------------------------------------------------------------
# syntax       ConvertDateForEPJ($DateInUTC);
#----------------------------------------------------------------------------------------
# description  Converts the date for epj.
#----------------------------------------------------------------------------------------
# input        $DateInUTC | ep6 date | string
# return       $DateInUTC | epj date | string
#========================================================================================
sub ConvertDateForEPJ {
    my ($DateInUTC) = @_;

    my $DF = DE_EPAGES::XML::API::DateTimeFormatXML->new();

    if(isValid($DateInUTC)){
        $DateInUTC = $DF->parse_datetime($DateInUTC, 'UTC');
    }
    return $DateInUTC;
}



#========================================================================================
# function     ReadJSONBoolean
# state        public
#----------------------------------------------------------------------------------------
# syntax       ReadJSONBoolean($JsonBoolean);
#----------------------------------------------------------------------------------------
# description  Converts JSON boolean to Perl boolean (integer which is 1 or 0)
#----------------------------------------------------------------------------------------
# input        $JsonBoolean | JSON boolean | JSON::Boolean
# return       Perl boolean | integer
#========================================================================================
sub ReadJSONBoolean {
    my ($JsonBoolean) = @_;

    if (ref \$JsonBoolean ne 'REF') {
        # $JsonBoolean is not JSON::(XS::|PP::)?(true|false) - it is scalar
        $JsonBoolean = '0' unless $JsonBoolean;
        # every non-number scalar but 'true' is false
        $JsonBoolean = '0' if $JsonBoolean =~ /\D/ && lc($JsonBoolean) ne 'true';
    }
    return $JsonBoolean ? 1 : 0;
}



#========================================================================================
# function     TranslateAttributeName
# state        public
#----------------------------------------------------------------------------------------
# syntax       TranslateAttributeName($Ep6AttributeName);
#----------------------------------------------------------------------------------------
# description  Returns for an ePages7 address attribute name, its name on the REST API
#----------------------------------------------------------------------------------------
# input        $JsonBoolean | JSON boolean | JSON::Boolean
# return       Perl boolean | integer
#========================================================================================
sub TranslateAttributeName {
    my ($Ep7AttributeName) = @_;

    my $hAttributeMapping = {
        'Company'    => 'company',
        'Salutation' => 'salutation',
        'FirstName'  => 'firstName',
        'LastName'   => 'lastName',
        'Street'     => 'street',
        'Street2'    => 'streetDetails',
        'Zipcode'    => 'zipCode',
        'City'       => 'city',
        'State'      => 'state',
        'CountryID'  => 'country',
        'Title'      => 'title',
        'VATID'      => 'vatId',
        'EMail'      => 'emailAddress',
        'Birthday'   => 'birthday',
        'Gender'            => 'gender',
        'MiddleName'        => 'middleName',
        'DisplayName'       => 'displayName',
        'AddressExtension'  => 'addressExtension',
        'CodePorte'         => 'doorCode',
        'FiscalCode'        => 'fiscalCode',
        'Department'        => 'department',
        'JobTitle'          => 'jobTitle',
        'Phone'             => 'phoneNumber',
        'PhoneBusiness'     => 'businessPhoneNumber',
        'PhoneCell'         => 'mobilePhoneNumber',
        'PhonePrivate'      => 'privatePhoneNumber',
        'Fax'               => 'faxNumber',
        'EMailBusiness'     => 'businessEmailAddress',
        'EMailPrivate'      => 'privateEmailAddress',
        'URL'               => 'websiteUrl',
        'BankAccountHolder' => 'bankAccountHolder',
        'BankAccountNo'     => 'bankAccountNumber',
        'BankCode'          => 'bankSortCode',
        'BankName'          => 'bankName',
    };

    return $hAttributeMapping->{$Ep7AttributeName};
}

1;
