#========================================================================================
# package      Perl::Critic::Policy::DE_EPAGES::Style::RequireCorrectImports
# state        public
#----------------------------------------------------------------------------------------
# description  This module checks for imports which are not used.
#               If, for example, your module contains
#                   use DE_EPAGES::Core::API::File qw( GetFileMD5Digest );
#               but in your implementation the function GetFileMD5Digest is not used,
#               a violation will be displayed.
#
#               Known problems: This rule only works correcly for one package per file.
#               Use ProhibitMultiplePackages in combination with this rule.
#----------------------------------------------------------------------------------------
# parameters   none
#========================================================================================
package Perl::Critic::Policy::DE_EPAGES::Style::RequireCorrectImports;
use base 'Perl::Critic::Policy';

use strict;

use Perl::Critic::Utils qw{ :severities :classification :ppi is_perl_builtin };
use Scalar::Util qw ( blessed );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf IsA );
use DE_EPAGES::Core::API::File qw( GetFileContent ExistsFile);
use DE_EPAGES::XML::API::UseProblems qw ( WriteCartridgeDependencies WriteExportsCache );
use Carp;

our $VERSION = '1.0';

our @DEFAULT_PACKAGE_EXCEMPTIONS = qw(
  Net::IPv6Addr
  DE_EPAGES::Core::API::Process
  Log::Log4perl::Level
  Inline::Java::Server
  DE_EPAGES::Core::API::Makefile
  SOAP::Lite
  Time::localtime
  Moose
  Config
  Getopt::Long
  File::Slurp
  HTTP::Status
);

our $LOCAL_CACHE = {};

# Most policies don't need to override the initialize_if_enabled() method provided by Perl::Critic::Policy.
# However, if your Policy is configurable via .perlcriticrc, you should implement a supported_parameters()
# method and need to implement initialize_if_enabled() to examine the $config values.
sub supported_parameters {
    return ( {
            name           => 'exemption_packages',
            description    => 'The set of packages to exclude from policy.',
            default_string => join( ' ', @DEFAULT_PACKAGE_EXCEMPTIONS ),
            behavior       => 'string list',
        }
    );
}

#-----------------------------------------------------------------------------

sub default_severity {$SEVERITY_HIGH}
sub default_themes   {qw(epages tics)}
sub applies_to       {'PPI::Document'}

#-----------------------------------------------------------------------------

Readonly::Scalar my $DESC_FUNC => q{Unused function '%s' found.};

Readonly::Scalar my $EXPL_MULTIPLE_INCLUDES => q(Including an element multiple times is wrong in most cases.);

Readonly::Scalar my $EXPL_MULTIPLE_PACKAGE_LINES => q{Package '%s' declared in multiple lines. Combine to one line only.};

Readonly::Scalar my $EXPL_EXPORT_ERROR => q{Remove or correct entry '%s' from include statement.};

Readonly::Scalar my $DESC_UNIMPORTED => q{Add "%s".};

Readonly::Scalar my $DESC_PACKAGE_UNUSED => q{Remove package '%s' from imports or start using it. Alternatively add it to your exemption list.};

Readonly::Scalar my $DESC_INDEPENDENT => q{Remove import '%s' or add cartridge '%s' to dependency list.};

#-----------------------------------------------------------------------------

sub initialize_if_enabled {
    my $self = shift;

    my $ExpFile = $DE_EPAGES::XML::API::UseProblems::EXPORTS_DATAFILE;
    my $DepFile = $DE_EPAGES::XML::API::UseProblems::DEPENDENCIES_DATAFILE;

    if (! ExistsFile($ExpFile)) {
        carp 'Missing exports cache file. Initializing now.';
        WriteExportsCache();
    }
    if (! ExistsFile($ExpFile)) {
        carp 'Failed initializing exports cache file.';
        return;
    }

    if (! ExistsFile($DepFile)) {
        carp 'Missing dependencies cache file. Initializing now.';
        WriteCartridgeDependencies();
    }
    if (! ExistsFile($DepFile)) {
        carp 'Failed initializing dependencies cache file.';
        return;
    }

    if ( !$LOCAL_CACHE->{'FUNCTION_DATA'} ) {
        my ( $rFunctions1, $rPackages1 );
        my $rContent = GetFileContent($ExpFile);
        eval $$rContent;    ##no critic (ProhibitStringyEval ProhibitDoubleSigils)
        die "Can't read $ExpFile: $@" if $@;
        $LOCAL_CACHE->{'FUNCTION_DATA'} = $rFunctions1;
        $LOCAL_CACHE->{'PACKAGE_DATA'}  = $rPackages1;
    }

    if ( !$LOCAL_CACHE->{'DEPENDENCIES_DATA'} ) {
        my $rDependencies1;
        my $rContent = GetFileContent($DepFile);
        eval $$rContent;    ##no critic (ProhibitStringyEval ProhibitDoubleSigils)
        die "Can't read $DepFile: $@" if $@;
        $LOCAL_CACHE->{'DEPENDENCIES_DATA'} = $rDependencies1;
    }

    return 1;
}

sub violates {
    my $self = shift;
    my ( $elem, $doc ) = @_;

    # initialize data hash
    my $word_usage = {
        'subs'                   => [],
        'constants'              => [],
        'packages'               => {},
        'implicit_imports'       => {},
        'imports'                => {},
        'fullyqualified_imports' => [],
        'usage'                  => {},
        'implicit_packages'      => {},
    };
    my @violations;

    # do nothing on multiple package definitions. we don't cover this yet!
    my $packageDecls = $doc->find('PPI::Statement::Package');
    return @violations if ( $packageDecls && scalar @{$packageDecls} > 1 );

    push @violations, $self->getConstants( $word_usage, $doc );
    push @violations, $self->getImports( $word_usage, $doc );
    push @violations, $self->getSubs( $word_usage, $doc );
    push @violations, $self->getUsage( $word_usage, $doc );
    push @violations, $self->getUnusedImports( $word_usage, $doc );
    push @violations, $self->getUnusedPackages( $word_usage, $doc );
    push @violations, $self->getUnimportedMethods( $word_usage, $doc );
    push @violations, $self->getImportsFromNonRequiredPackages( $word_usage, $doc );

    return @violations;
}

#========================================================================================
# function     getImports
# state        private
#----------------------------------------------------------------------------------------
# syntax       getImports( $word_usage, $doc );
#----------------------------------------------------------------------------------------
# description  collects all use statements
#----------------------------------------------------------------------------------------
# input        $word_usage | word_usage | hash.ref
# input        $doc | PPI document | object
# return       @violations | violations | array
#========================================================================================
sub getImports {
    my $self = shift;
    my ( $word_usage, $doc ) = @_;
    my @violations;

    # find all "use" statements.
    my @Includes = grep { $_->isa('PPI::Statement::Include') && substr( $_->content, 0, 3 ) eq 'use' } $doc->schildren();

    # populate packages and imports.
    foreach my $Include (@Includes) {

        # Package names start with a capital letter.
        my @Packages = grep { $_->isa('PPI::Token::Word') && $_->content =~ /^[[:upper:]]/ } $Include->schildren();
        foreach my $Package (@Packages) {

            # don't analyze use constant use base, parent, feature etc.
            next if $self->_isaUsePragma($Package);
            push @violations, $self->_analyzeUsePackage( $Package, $word_usage );
        }
    }
    return @violations;
}

sub _isaUsePragma {
    my $self = shift;
    my ($Package) = @_;

    my $prev_sibling = $Package->sprevious_sibling();
    return (
        blessed($prev_sibling) and
        $prev_sibling->isa('PPI::Token::Word') and
        ElementOf($prev_sibling->content, qw(
                      constant base parent feature
                      diagnostics integer sigtrap strict
                      subs warnings sort
                  )
        )
    );
}

sub _analyzeUsePackage {
    my $self = shift;
    my ( $Package, $word_usage ) = @_;

    # ok, lets take this as package name.
    my @violations;
    my $PackageName = $Package->content;
    push @violations, $self->_checkPackageForMultipleOccurence( $Package, $word_usage );

    # save import declaration for future reference.
    $word_usage->{'packages'}->{$PackageName} = { 'element' => $Package };

    # get EXPORTs
    my @EXPORT_LIST = $self->_exportsByPackage( $PackageName, $word_usage );
    my $next_sibling = $Package->snext_sibling;

    # Analyze use Package 'import';
    if ( blessed($next_sibling) && $next_sibling->isa('PPI::Token::Quote') ) {
        push @violations, $self->_addWordToImport( $next_sibling->string, $Package, $word_usage, \@EXPORT_LIST );

        # Analyze use Package qw();
    }
    elsif ( blessed($next_sibling) && $next_sibling->isa('PPI::Token::QuoteLike::Words') ) {
        my @WordList = $next_sibling->literal;
        foreach my $Word (@WordList) {
            push @violations, $self->_addWordToImport( $Word, $Package, $word_usage, \@EXPORT_LIST );
        }

        # Analyze "use Package;" or "use Package Feature => 123; "
    }
    elsif ( blessed($next_sibling) and
            not ElementOf($next_sibling->content, ('(', 'qw')) ) {
        if ( exists( $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName} ) && exists( $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName}->{'EXPORT'} ) ) {
            foreach my $Word ( @{ $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName}->{'EXPORT'} } ) {
                $word_usage->{'implicit_imports'}->{$Word} = $Package;
                push @{ $word_usage->{'fullyqualified_imports'} }, $PackageName . '::' . $Word;
            }
        }
        $word_usage->{'implicit_packages'}->{$PackageName} = $Package;
        $word_usage->{'implicit_imports'}->{$PackageName}  = $Package;
    }
    return @violations;
}

sub _checkPackageForMultipleOccurence {
    my $self = shift;
    my ( $Package, $word_usage ) = @_;
    my $PackageName = $Package->content;

    # has it already been included?
    if ( not ElementOf($PackageName, $word_usage->{'constants'}) and
         defined $word_usage->{'packages'}->{$PackageName} ) {
        $self->{_severity} = $SEVERITY_LOW;
        return $self->violation( qq(Package "$PackageName" is included several times.), sprintf( $EXPL_MULTIPLE_PACKAGE_LINES, $PackageName ),
            $Package->parent );
    }
    return;
}

sub _addWordToImport {
    my $self = shift;
    my ( $Word, $Package, $word_usage, $aEXPORT_LIST ) = @_;

    # this is probably a bug in PPI...
    return if ( $Word eq '(' );
    $Word =~ s/^\(//;

    # skip variables.
    return if ElementOf(substr($Word, 0, 1), qw(: $ @ % ));

    # remove leading ampersand.
    $Word =~ s/^&//;
    if ( defined $word_usage->{'imports'}->{$Word} ) {
        $self->{_severity} = $SEVERITY_HIGH;
        return $self->violation( qq(Method "$Word" is included several times.), $EXPL_MULTIPLE_INCLUDES, $Package->parent );
    }

    # check if Word is exported
    if ( scalar @$aEXPORT_LIST and not ElementOf($Word, $aEXPORT_LIST) ) {
        $self->{_severity} = $SEVERITY_HIGHEST;
        return $self->violation( qq(Method "$Word" is not exported by package $Package.), sprintf( $EXPL_EXPORT_ERROR, $Word ), $Package->parent );
    }
    push @{ $word_usage->{'fullyqualified_imports'} }, $Package->content . '::' . $Word;
    $word_usage->{'imports'}->{$Word} = $Package->parent;
    return;
}

#========================================================================================
# function     getSubs
# state        private
#----------------------------------------------------------------------------------------
# syntax       getSubs( $word_usage, $doc );
#----------------------------------------------------------------------------------------
# description  collects package subs
#----------------------------------------------------------------------------------------
# input        $word_usage | data hash | hash.ref
# input        $doc | PPI document | object
# return       @violations | violations | array
#========================================================================================
sub getSubs {
    my $self = shift;
    my ( $word_usage, $doc ) = @_;

    my @Subs = grep { $_->isa('PPI::Statement::Sub') } $doc->schildren();
    return () unless ( scalar @Subs );
    foreach my $Sub (@Subs) {
        push @{ $word_usage->{'subs'} }, $Sub->name;
    }
    return ();
}

#========================================================================================
# function     getConstants
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->getConstants( $word_usage, $doc );
#----------------------------------------------------------------------------------------
# description  collects constants
#----------------------------------------------------------------------------------------
# input        $word_usage | data hash | hash.ref
# input        $doc | PPI document | object
# return       @violations | violations | array
#========================================================================================
sub getConstants {
    my $self = shift;
    my ( $word_usage, $doc ) = @_;
    my @Includes = grep { $_->isa('PPI::Statement::Include') } $doc->schildren();
    foreach my $Include (@Includes) {
        my @Constants = grep { $_->isa('PPI::Token::Word') && $_->content eq 'constant' } $Include->schildren();
        foreach my $Constant (@Constants) {
            my $ConstantElements = $Constant->parent->find('PPI::Token::Word');
            next unless ($ConstantElements);
            foreach my $ConstantElement ( @{$ConstantElements} ) {
                my $NextSibling = $ConstantElement->snext_sibling();
                next unless ( defined $NextSibling && IsA( $NextSibling, 'PPI::Token::Operator' ) && $NextSibling->content eq '=>' );
                push @{ $word_usage->{'constants'} }, $ConstantElement->content;
            }
        }
    }
    return ();
}

#========================================================================================
# function     getUsage
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->getUsage( $word_usage, $doc );
#----------------------------------------------------------------------------------------
# description  collects usage of words
#----------------------------------------------------------------------------------------
# input        $word_usage | data hash | hash.ref
# input        $doc | PPI document | object
# return       @violations | violations | array
#========================================================================================
sub getUsage {
    my $self = shift;
    my ( $word_usage, $doc ) = @_;

    #collect usage
    my $Barewords = $doc->find('PPI::Token::Word');
    if ($Barewords) {
        foreach my $Word ( @{$Barewords} ) {

            # quick back out for perl internals
            next if ( is_perl_builtin($Word) && !$word_usage->{'imports'}->{$Word} );

            my $prev_sibling   = $Word->sprevious_sibling();
            my $next_sibling   = $Word->snext_sibling();
            my $previous_token = $Word->previous_token();
            my $next_token     = $Word->next_token();
            my $content        = $Word->content;

            # dont check hash keys.
            next if ( blessed($next_sibling) && $next_sibling->isa('PPI::Token::Operator') && $next_sibling->content eq '=>' );
            next if ( blessed($previous_token) && $previous_token->content eq '{' && blessed($next_token) && $next_token->content eq '}' );

            # use const
            next if ( $Word->parent->isa('PPI::Statement::Include')
                && !( blessed($prev_sibling) && $prev_sibling->isa('PPI::Token::Operator') ) );
            next if ( $Word->parent->isa('PPI::Statement::Package') );
            $word_usage->{'usage'}->{$content} = $Word;

            # add to package usage.
            if ( $content =~ /(.*?)::(\w+)$/x ) {
                $word_usage->{'usage'}->{$1} = $Word;
            }
        }
    }

    # find &Sub usage
    my $Symbols = $doc->find('PPI::Token::Symbol');
    if ($Symbols) {
        foreach my $Symbol ( @{$Symbols} ) {
            if ( $Symbol->content =~ /^\$(.*)::/ ) {
                $word_usage->{'usage'}->{$1} = $Symbol;
            }
            elsif ( $Symbol->content =~ /^\&/ ) {
                $word_usage->{'usage'}->{ substr $Symbol->content, 1 } = $Symbol;
            }
        }
    }
    return ();
}

#========================================================================================
# function     getUnusedImports
# state        private
#----------------------------------------------------------------------------------------
# syntax       getUnusedImports( $word_usage );
#----------------------------------------------------------------------------------------
# description  looks for unused imported functions
#----------------------------------------------------------------------------------------
# input        $word_usage | data hash | hash.ref
# input        $doc | PPI document | object
# return       @violations | violations | array
#========================================================================================
sub getUnusedImports {
    my $self = shift;
    my ( $word_usage, $doc ) = @_;
    my @violations;

    # check for unused imports.
    my $found_export_tags = 0;
  IMPORT_KEY: foreach my $key ( keys %{ $word_usage->{'imports'} } ) {

        # export tags are not supported yet!
        if ( substr( $key, 0, 1 ) eq ':' ) {
            $found_export_tags = 1;
            next;
        }
        if ( !defined( $word_usage->{'usage'}->{$key} ) ) {

            # This is a workaround for a PPI bug that
            # does not parse KEY ? VAL_1 : VAL_2 correctly.
            my $Labels = $doc->find('PPI::Token::Label');
            if ($Labels) {
                foreach my $Label ( @{$Labels} ) {
                    next IMPORT_KEY if $Label->content =~ /$key/;
                }
            }
            $self->{_severity} = $SEVERITY_MEDIUM;
            push @violations, $self->violation( qq(Imported but not used "$key"), sprintf( $DESC_FUNC, $key ), $word_usage->{'imports'}->{$key} );
        }
    }
    return @violations;
}

#========================================================================================
# function     getUnusedPackages
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->getUnusedPackages( $word_usage, $doc );
#----------------------------------------------------------------------------------------
# description  looks for unused imported packages
#----------------------------------------------------------------------------------------
# input        $word_usage | data hash | hash.ref
# input        $doc | PPI document | object
# return       @violations | violations | array
#========================================================================================
sub getUnusedPackages {
    my $self = shift;
    my ( $word_usage, $doc ) = @_;
    my @violations;

    # check if implicit uses should be removed.
    foreach my $PackageName ( keys %{ $word_usage->{'implicit_packages'} } ) {
        my $usedAnyWord = 0;
        next if ( $self->_is_package_exemption($PackageName) );
        next if ( !exists( $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName} ) );
        if ( $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName}->{'EXPORT'} ) {
            foreach my $Word ( @{ $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName}->{'EXPORT'} } ) {

                # cannot check variable exports.. - can check method usage.
                if ( ElementOf(substr($Word, 0, 1), qw ($ % @)) or
                     defined $word_usage->{'usage'}->{$Word} ) {
                    $usedAnyWord = 1;
                    last;
                }
            }
        }
        if ( !$usedAnyWord ) {

            # perform a deep search for statements like tie %hash, 'CLASSNAME'
            # this is probably the slowest part of this policy...
            $usedAnyWord = $doc->find(
                sub {
                    ( $_[1]->isa('PPI::Token::Quote') && $_[1]->string eq $PackageName )
                      or return '';
                    return 1;
                }
            );
        }
        if ( !$usedAnyWord && !defined( $word_usage->{'usage'}->{$PackageName} ) ) {
            $self->{_severity} = $SEVERITY_MEDIUM;
            push @violations,
              $self->violation(
                qq("$PackageName" is *probably* imported but not used.),
                sprintf( $DESC_PACKAGE_UNUSED, $PackageName ),
                $word_usage->{'implicit_packages'}->{$PackageName},
              );
        }
    }
    return @violations;
}

#========================================================================================
# function     getUnimportedMethods
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->getUnimportedMethods( $word_usage );
#----------------------------------------------------------------------------------------
# description  looks for used but not imported methods
#----------------------------------------------------------------------------------------
# input        $word_usage | data hash | hash.ref
# return       @violations | violations | array
#========================================================================================
sub getUnimportedMethods {
    my $self = shift;
    my ($word_usage) = @_;

    my @violations;

    # check unimported functions
    foreach my $key ( keys %{ $word_usage->{'usage'} } ) {

        # quick back out for local subs:
        next if ElementOf($key, $word_usage->{'subs'});

        # quick back out for local constants
        next if ElementOf($key, $word_usage->{'constants'});

        # dont check object methods.
        my $prev = $word_usage->{'usage'}->{$key}->sprevious_sibling();
        next if ( blessed($prev) && $prev->isa('PPI::Token::Operator') && $prev->content eq '->' );

        # dont check sub declarations
        next if ( blessed($prev) && $prev->isa('PPI::Token::Word') && $prev->content eq 'sub' );

        # only check known epages exports.
        next if ( !defined( $LOCAL_CACHE->{'FUNCTION_DATA'} ) || !defined $LOCAL_CACHE->{'FUNCTION_DATA'}->{$key} );
        if (   !defined( $word_usage->{'imports'}->{$key} )
            && !defined( $word_usage->{'implicit_imports'}->{$key} ) )
        {
            if ( scalar grep { $_ =~ /DE_EPAGES/ } @{ $LOCAL_CACHE->{'FUNCTION_DATA'}->{$key} } ) {
                $self->{_severity} = $SEVERITY_HIGHEST;
                push @violations,
                  $self->violation(
                    qq(Used but not imported "$key".),
                    sprintf( $DESC_UNIMPORTED, join( ' or ', map {"use $_ qw ($key);"} @{ $LOCAL_CACHE->{'FUNCTION_DATA'}->{$key} } ) ),
                    $word_usage->{'usage'}->{$key}
                  );
            }
            else {
                $self->{_severity} = $SEVERITY_LOWEST;
                push @violations,
                  $self->violation(
                    qq(Found possible imports for "$key".),
                    sprintf( $DESC_UNIMPORTED, join( ' or ', map {"use $_ qw ($key);"} @{ $LOCAL_CACHE->{'FUNCTION_DATA'}->{$key} } ) ),
                    $word_usage->{'usage'}->{$key}
                  );
            }

        }
    }
    return @violations;
}

#========================================================================================
# function     _exportsByPackage
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_exportsByPackage( $PackageName, $word_usage );
#----------------------------------------------------------------------------------------
# description  evaluates package to find @EXPORT and @EXPORT_OK array.
#----------------------------------------------------------------------------------------
# input        $PackageName | package name | string
# input        $word_usage | data hash | hash.ref
# return       @EXPORT_LIST | exported elements | array
#========================================================================================
sub _exportsByPackage {
    my $self = shift;
    my ( $PackageName, $word_usage ) = @_;
    return () unless ( exists( $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName} ) );
    return map { $_ =~ s/^&//; $_ } ( @{ $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName}->{'EXPORT'} }, @{ $LOCAL_CACHE->{'PACKAGE_DATA'}->{$PackageName}->{'EXPORT_OK'} } );
}

#========================================================================================
# function     _is_package_exemption
# state        private
#----------------------------------------------------------------------------------------
# syntax       _is_package_exemption( $package );
#----------------------------------------------------------------------------------------
# description  find out if package should not be checked.
#----------------------------------------------------------------------------------------
# input        $package | package name | string
# return       $found | true if found in exemption list | boolean
#========================================================================================
sub _is_package_exemption {
    my $self = shift;
    my ($arg) = @_;
    return defined $self->{'_exemption_packages'}{$arg};
}

#========================================================================================
# function     getImportsFromNonRequiredPackages
# state        private
#----------------------------------------------------------------------------------------
# syntax       getImportsFromNonRequiredPackages( $word_usage, $doc );
#----------------------------------------------------------------------------------------
# description  checks if non-required packages were used for imports
#----------------------------------------------------------------------------------------
# input        $word_usage | data hash | hash.ref
# input        $doc | PPI document | object
# return       @violations | violations | array
#========================================================================================
sub getImportsFromNonRequiredPackages {
    my $self = shift;
    my ( $word_usage, $doc ) = @_;
    return unless ( scalar keys %{ $word_usage->{'packages'} } );
    my @violations;
    my @Packages = grep { $_->isa('PPI::Statement::Package') } $doc->schildren();
    return unless ( scalar @Packages );

    # collect exported functions
    my $hCartridgeDependencies = $LOCAL_CACHE->{'DEPENDENCIES_DATA'};
    return if ( !$hCartridgeDependencies );

    foreach my $Package (@Packages) {

        # (package)0(space)1(packagename)2(;)3
        my $Cartridge = $Package->child(2);
        next unless ( defined $Cartridge );
        next unless ( $Cartridge->isa('PPI::Token::Word') );

        # special case: SAX Import handlers.
        next if ( $Cartridge->content =~ /DE_EPAGES::Object::API::XML/ );
        if ( $Cartridge->content =~ /^([^:]+::[^:]+)/x ) {
            my $PackageContent = $1;

            # is this a cartridge we have collected dependencies for?
            next unless defined $hCartridgeDependencies->{$PackageContent};
            foreach my $UsedPackage ( keys %{ $word_usage->{'packages'} } ) {

                # extract content and check on dependency list.
                if ( $UsedPackage =~ /^([^:]+::[^:]+)/x ) {
                    my $UsedPackageContent = $1;

                    # same cartridge
                    next if ( $UsedPackageContent eq $PackageContent );

                    # WebInterfacePoolDB is used by many cartridges, but is not installed in store databases
                    next if $UsedPackageContent eq 'DE_EPAGES::WebInterfacePoolDB';

                    # not in dependency list
                    next unless ( defined $hCartridgeDependencies->{$UsedPackageContent} );

                    # check if we're in dependency list
                    next if ElementOf($UsedPackageContent, $hCartridgeDependencies->{$PackageContent});

                    $self->{_severity} = $SEVERITY_HIGH;
                    push @violations,
                      $self->violation(
                        qq("$UsedPackageContent" is not from a cartridge in dependency list.),
                        sprintf( $DESC_INDEPENDENT, $UsedPackage, $UsedPackageContent ),
                        $word_usage->{'packages'}->{$UsedPackage}->{'element'}
                      );
                }
            }
        }
    }
    return @violations;
}

1;
