#========================================================================================
# package      DE_EPAGES::Zettle::API::Table::ZettleProductExport
# state        public
# description  interface for table ZettleProductExport
#========================================================================================
package DE_EPAGES::Zettle::API::Table::ZettleProductExport;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    InsertZettleProductExport
    UpdateZettleProductExport
    DeleteZettleProductExportOlderThan
    DeleteZettleProductExport
    GetPKeyZettleProductExportsByShopID
    GetInfoZettleProductExport
    GetInfoZettleProductExportByGlobalExportUUID
    GetInfoZettleProductExportByShopID
    GetInfoZettleProductExportByStatus
);

use DE_EPAGES::Core::API::Error         qw ( CallerError Error );
use DE_EPAGES::Core::API::PerlTools     qw ( GetCurrentDateTime IsA SubtractDuration );
use DE_EPAGES::Core::API::TestParameter qw ( TestString TestInteger TestFloat TestDateTime ); ## no critic qw(RequireCorrectImports)
use DE_EPAGES::Trigger::API::Trigger    qw ( TriggerHook );
use DE_EPAGES::Zettle::DAL::Zettle;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NoResults_ZettleProductExport' => 'ZettleProductExport doesn\'t exist (GlobalExportUUID=#GlobalExportUUID, BatchNumber=#BatchNumber)',
        'PARAMETER_MISSING'  => 'Missing parameter #Parameter',
        'FORMAT_NOT_DATETIME' => '#Parameter (#Type) is no DateTime (#Value)',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     InsertZettleProductExport
#----------------------------------------------------------------------------------------
# syntax       InsertZettleProductExport($hZettleProductExport);
#----------------------------------------------------------------------------------------
# description  Creates an entry of ZettleProductExport.
#----------------------------------------------------------------------------------------
# input        $hZettleProductExport | with following keys:
#               <ul>
#                   <li>GlobalExportUUID    - Global export UUID (part of primary key) - varchar(36)
#                   <li>BatchNumber         - Number of batch within global export (part of primary key) - int
#                   <li>SiteID              - Site ID - int
#                   <li>ExportType          - Type of export - int
#                   <li>ItemCount           - Number of items included in batch export - int
#                   <li>BatchExportStatus   - Batch export status - int
#                   <li>StartedOn           - Date and time when batch export started (optional. default: current time) - datetime
#               </ul> | ref.hash
#========================================================================================
sub InsertZettleProductExport ($) {
    my ($hZettleProductExport) = @_;

    my $GlobalExportUUID = $hZettleProductExport->{'GlobalExportUUID'};
    my $BatchNumber = $hZettleProductExport->{'BatchNumber'};
    TestString('GlobalExportUUID', $GlobalExportUUID);
    TestInteger('BatchNumber', $BatchNumber, 0);

    DAL_InsertZettleProductExport($hZettleProductExport);
    TriggerHook('API_InsertZettleProductExport', $hZettleProductExport);
    return;
}

#========================================================================================
# function     UpdateZettleProductExport
#----------------------------------------------------------------------------------------
# syntax       UpdateZettleProductExport($GlobalExportUUID, $BatchNumber, $hZettleProductExport);
# syntax       UpdateZettleProductExport($hZettleProductExport); # if $hZettleProductExport includes GlobalExportUUID and $BatchNumber
#----------------------------------------------------------------------------------------
# description  Updates a ZettleProductExport status if any element was changed.
#               Uses current datetime to fill value for updatedon column.
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global export UUID (part of primary key) | varchar(36)
# input        $BatchNumber | Number of batch within global export (part of primary key) | int
# input        $hZettleProductExport | with following keys:
#               <ul>
#                   <li>BatchExportUUID   - Batch export UUID - varchar(36)
#                   <li>BatchExportStatus - New Value for batch export status - int
#                   <li>UpdatedOn         - Datetime when export was last updated (optional. default: current datetime) - datetime
#                   <li>ItemCount         - Number of items included in batch export - int
#                   <li>ErrorType         - Error identifier - varchar(80)
#                   <li>ErrorMessage      - Detailed error message - varchar(255)
#               </ul>| ref.hash
#========================================================================================
sub UpdateZettleProductExport ($;$$) {
    my ($GlobalExportUUID, $BatchNumber, $hZettleProductExport) = @_;
    if (ref $GlobalExportUUID eq 'HASH') {
        $hZettleProductExport = $GlobalExportUUID;
        $GlobalExportUUID = $hZettleProductExport->{'GlobalExportUUID'};
        $BatchNumber = $hZettleProductExport->{'BatchNumber'};
    } else {
        $hZettleProductExport->{'GlobalExportUUID'} = $GlobalExportUUID;
        $hZettleProductExport->{'BatchNumber'} = $BatchNumber;
    }
    TestString('GlobalExportUUID', $GlobalExportUUID);
    TestInteger('BatchNumber', $BatchNumber, 0);

    TriggerHook('API_BeforeUpdateZettleProductExport', $hZettleProductExport);
    DAL_UpdateZettleProductExport($GlobalExportUUID, $BatchNumber, $hZettleProductExport);
    TriggerHook('API_AfterUpdateZettleProductExport', $hZettleProductExport);
    return 1;
}

#========================================================================================
# function     DeleteZettleProductExportOlderThan
#----------------------------------------------------------------------------------------
# syntax       DeleteZettleProductExportOlderThan($Days);
#----------------------------------------------------------------------------------------
# description  Delete zettleproductexport entries older than $Days.
#----------------------------------------------------------------------------------------
# input        $Days | Entries older than given number of $Days will be deleted from the table | int
#========================================================================================
sub DeleteZettleProductExportOlderThan ($) {
    my ($Days) = @_;
    TestInteger('Days', $Days, 1);

    my $DateTime = SubtractDuration( GetCurrentDateTime(), 'days', $Days );
    DeleteZettleProductExport($_) foreach @{GetPKeyZettleProductExportsByStartedOn($DateTime)};
}

#========================================================================================
# function     DeleteZettleProductExport
#----------------------------------------------------------------------------------------
# syntax       DeleteZettleProductExport($GlobalExportUUID, $BatchNumber);
#----------------------------------------------------------------------------------------
# description  Delete entry from zettleproductexport table.
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global export UUID (part of primary key) | varchar(36)
# input        $BatchNumber | Number of batch within global export (part of primary key) | int
#========================================================================================
sub DeleteZettleProductExport ($) {
    my ($GlobalExportUUID, $BatchNumber) = @_;
    if (ref $GlobalExportUUID eq 'HASH') {
        my $hInfo = $GlobalExportUUID;
        $GlobalExportUUID = $hInfo->{'GlobalExportUUID'};
        $BatchNumber = $hInfo->{'BatchNumber'};
    }
    TestString('GlobalExportUUID', $GlobalExportUUID);
    TestInteger('BatchNumber', $BatchNumber, 0);
    TriggerHook('API_DeleteZettleProductExport', { 'GlobalExportUUID' => $GlobalExportUUID,  'BatchNumber' => $BatchNumber } );
    DAL_DeleteZettleProductExport($GlobalExportUUID, $BatchNumber);
    return;
}

#========================================================================================
# function     GetPKeyZettleProductExportsByShopID
#----------------------------------------------------------------------------------------
# syntax       $ahKeys = GetPKeyZettleProductExportsByShopID($ShopID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of ZettleProductExport by ShopID.
#----------------------------------------------------------------------------------------
# input        $ShopID | Shop ID | int
#----------------------------------------------------------------------------------------
# return       $ahKeys | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#               </ul> | ref.array.hash
#========================================================================================
sub GetPKeyZettleProductExportsByShopID ($) {
    my ($ShopID) = @_;
    TestInteger('ShopID',$ShopID);
    return DAL_GetPKeyZettleProductExportsBySiteID($ShopID);
}

#========================================================================================
# function     GetPKeyZettleProductExportsByStartedOn
#----------------------------------------------------------------------------------------
# syntax       $ahKeys = GetPKeyZettleProductExportsByStartedOn($OlderThan);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of ZettleProductExport where StartedOn is older than given datetime.
#----------------------------------------------------------------------------------------
# input        $OlderThan | Datetime value to compare StartedOn field with | DateTime
#----------------------------------------------------------------------------------------
# return       $ahKeys | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#               </ul> | ref.array.hash
#========================================================================================
sub GetPKeyZettleProductExportsByStartedOn ($) {
    my ($OlderThan) = @_;
    _TestDateTime('OlderThan', $OlderThan);
    return DAL_GetPKeyZettleProductExportsByStartedOn($OlderThan);
}

#========================================================================================
# function     GetInfoZettleProductExport
#----------------------------------------------------------------------------------------
# syntax       $hZettleProductExport = GetInfoZettleProductExport($GlobalExportUUID, $BatchNumber);
#----------------------------------------------------------------------------------------
# description  Retrieve ZettleProductExport by GlobalExportUUID and BatchNumber.
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global export UUID (part of primary key) | varchar(36)
# input        $BatchNumber | Number of batch within global export (part of primary key) | int
#----------------------------------------------------------------------------------------
# return       $hZettleProductExport | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#                   <li>SiteID             - Site ID - int
#                   <li>ExportType         - Type of export - int
#                   <li>ItemCount          - Number of items included in batch export - int
#                   <li>BatchExportStatus  - Status of batch export - int
#                   <li>BatchExportUUID    - Batch export UUID (part of primary key) - string
#                   <li>StartedOn          - Date and time when batch export started (optional. default: current time) - datetime
#                   <li>UpdatedOn          - Date when entry was last updated (optional) - datetime
#                   <li>ErrorType          - Error identifier - varchar(80)
#                   <li>ErrorMessage       - Detailed error message - varchar(255)
#               </ul>| ref.hash
#========================================================================================
sub GetInfoZettleProductExport ($$) {
    my ($GlobalExportUUID, $BatchNumber) = @_;
    TestString('GlobalExportUUID', $GlobalExportUUID);
    TestInteger('BatchNumber', $BatchNumber, 1);
    my $hBatchStatus = DAL_GetInfoZettleProductExport($GlobalExportUUID, $BatchNumber);
    Error('NoResults_ZettleProductExport', {
            'GlobalExportUUID' => $GlobalExportUUID,
            'BatchNumber' => $BatchNumber
    }) unless defined $hBatchStatus;
    return $hBatchStatus;
}

#========================================================================================
# function     GetInfoZettleProductExportByGlobalExportUUID
#----------------------------------------------------------------------------------------
# syntax       $ahZettleProductExport = GetInfoZettleProductExportByGlobalExportUUID($GlobalExportUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve all entries with the same GlobalExportUUID
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global export UUID (part of primary key) | varchar(36)
#----------------------------------------------------------------------------------------
# return       $ahZettleProductExport | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#                   <li>SiteID             - Site ID - int
#                   <li>ExportType         - Type of export - int
#                   <li>ItemCount          - Number of items included in batch export - int
#                   <li>BatchExportStatus  - Status of batch export - int
#                   <li>BatchExportUUID    - Batch export UUID (part of primary key) - string
#                   <li>StartedOn          - Date and time when batch export started (optional. default: current time) - datetime
#                   <li>UpdatedOn          - Date when entry was last updated (optional) - datetime
#                   <li>ErrorType          - Error identifier - varchar(80)
#                   <li>ErrorMessage       - Detailed error message - varchar(255)
#               </ul> | ref.array.hash
#========================================================================================
sub GetInfoZettleProductExportByGlobalExportUUID ($) {
    my ($GlobalExportUUID) = @_;
    TestString('GlobalExportUUID', $GlobalExportUUID);
    return DAL_GetInfoZettleProductExportByGlobalExportUUID($GlobalExportUUID);
}

#========================================================================================
# function     GetInfoZettleProductExportByShopID
#----------------------------------------------------------------------------------------
# syntax       $ahZettleProductExport = GetInfoZettleProductExportByShopID($ShopID);
#----------------------------------------------------------------------------------------
# description  Retrieve entries by ShopID
#----------------------------------------------------------------------------------------
# input        $ShopID | ID of a shop | int
#----------------------------------------------------------------------------------------
# return       $ahZettleProductExport | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#                   <li>SiteID             - Site ID - int
#                   <li>ExportType         - Type of export - int
#                   <li>ItemCount          - Number of items included in batch export - int
#                   <li>BatchExportStatus  - Status of batch export - int
#                   <li>BatchExportUUID    - Batch export UUID (part of primary key) - string
#                   <li>StartedOn          - Date and time when batch export started (optional. default: current time) - datetime
#                   <li>UpdatedOn          - Date when entry was last updated (optional) - datetime
#                   <li>ErrorType          - Error identifier - varchar(80)
#                   <li>ErrorMessage       - Detailed error message - varchar(255)
#               </ul> | ref.array.hash
#========================================================================================
sub GetInfoZettleProductExportByShopID ($) {
    my ($ShopID) = @_;
    TestInteger('ShopID', $ShopID);
    return DAL_GetInfoZettleProductExportBySiteID($ShopID);
}

#========================================================================================
# function     GetInfoZettleProductExportByStatus
#----------------------------------------------------------------------------------------
# syntax       $ahZettleProductExport = GetInfoZettleProductExportByStatus($GlobalExportUUID, $BatchExportStatus);
#----------------------------------------------------------------------------------------
# description  Retrieve entries by GlobalExportUUID and status
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global export UUID (part of primary key) | varchar(36)
# input        $BatchExportStatus | Batch export status (part of primary key)
#                                    (see EXPORT_STATUS values in <package DE_EPAGES::Zettle::API::Constants>) | int
#----------------------------------------------------------------------------------------
# return       $ahZettleProductExport | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#                   <li>SiteID             - Site ID - int
#                   <li>ExportType         - Type of export - int
#                   <li>ItemCount          - Number of items included in batch export - int
#                   <li>BatchExportStatus  - Status of batch export - int
#                   <li>BatchExportUUID    - Batch export UUID (part of primary key) - string
#                   <li>StartedOn          - Date and time when batch export started (optional. default: current time) - datetime
#                   <li>UpdatedOn          - Date when entry was last updated (optional) - datetime
#                   <li>ErrorType          - Error identifier - varchar(80)
#                   <li>ErrorMessage       - Detailed error message - varchar(255)
#               </ul> | ref.array.hash
#========================================================================================
sub GetInfoZettleProductExportByStatus ($$) {
    my ($GlobalExportUUID, $BatchExportStatus) = @_;
    TestString('GlobalExportUUID', $GlobalExportUUID);
    TestInteger('BatchExportStatus', $BatchExportStatus);
    return DAL_GetInfoZettleProductExportByStatus($GlobalExportUUID, $BatchExportStatus);
}

#========================================================================================
# function     OnDeleteShop
#----------------------------------------------------------------------------------------
# syntax       OnDeleteShop($hParams);
#----------------------------------------------------------------------------------------
# description  Deletes ZettleProductExport of this model when a Shop is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | key-value pair, key is
#               <ul>
#                   <li>ShopID
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteShop {
    my ($hParams) = @_;

    DeleteZettleProductExport($_) foreach @{GetPKeyZettleProductExportsByShopID($hParams->{'ShopID'})};
}

#========================================================================================
# function     _TestDateTime
#----------------------------------------------------------------------------------------
# syntax       _TestDateTime($Name, $Object);
#----------------------------------------------------------------------------------------
# description  Tests if the parameter $Object is defined and is a blessed object.
#               Also tests if $Object is an instance of DateTime using UNIVERSAL::isa().
#----------------------------------------------------------------------------------------
# input        $Name | parameter name | string
# input        $Object | parameter value | object
#========================================================================================
sub _TestDateTime($$) {
    my ($Name, $Object) = @_;

    CallerError('PARAMETER_MISSING' , {'Parameter' => 'Name'}) unless defined $Name;

    CallerError('PARAMETER_MISSING', {'Parameter' => $Name}, {'SkipCallers' => 1}) unless defined $Object;

    CallerError(
        'FORMAT_NOT_DATETIME',
        {
            'Parameter' => $Name,
            'Type' => ref $Object,
            'Value' => $Object
        },
        {'SkipCallers' => 1}
    ) unless ref $Object && IsA( $Object, 'DateTime');

    return;
}

1;
