#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::MerchantService
# base         DE_EPAGES::Zettle::API::WebService::RestService
# state        private
#----------------------------------------------------------------------------------------
# description  Service wrapper around merchant-related API requests
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::MerchantService;
use base DE_EPAGES::Zettle::API::WebService::RestService;

use strict;

##no critic qw(RequireCorrectImports)
use DE_EPAGES::Zettle::API::WebService::EndpointHandler qw (
    GetAccountInfoApiUrl
    GetUserInfoApiUrl
);
##use critic qw(RequireCorrectImports)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::MerchantService->new($Environment);
#----------------------------------------------------------------------------------------
# description  creates a new MerchantService object
#----------------------------------------------------------------------------------------
# input        $Environment | a Zettle environment object | DE_EPAGES::Zettle::API::Object:ZettleEnvironment
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::MerchantService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment) = @_;

    my $self = $class->SUPER::new($Environment);
    return $self;
}

#========================================================================================
# function     getUserInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getUserInfo();
#----------------------------------------------------------------------------------------
# description  Retrieve user information from users/self utility endpoint.
#               See: https://developer.zettle.com/docs/api/oauth/user-guides/manage-apps-users/identify-users
#               See: https://github.com/iZettle/api-documentation/blob/master/authorization.md/#get-user-info
#----------------------------------------------------------------------------------------
# return       $hTokenInfo | with following keys:
#               <ul>
#                   <li>uuid - user UUID - string
#                   <li>organizationUuid - organization UUID - string
#               </ul> | ref.hash
#========================================================================================
sub getUserInfo {
    my $self = shift;
    my ($ExportPrices) = @_;

    my $UserInfoUrl = GetUserInfoApiUrl();

    my $HttpResponse = $self->sendGetRequest($UserInfoUrl);
    
    # Example:
    # {
    # "uuid": "1a5431c6-824d-11eb-92c1-8c85ccaff8db",
    # "organizationUuid": "1a52e2a8-824d-11eb-acc8-aafa2648b59f"
    # }
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $hResponse;
}

#========================================================================================
# function     getAccountInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getAccountInfo();
#----------------------------------------------------------------------------------------
# description  Retrieve merchant acccount information from Zettle Secure API.
#----------------------------------------------------------------------------------------
# return       $hTokenInfo | with following keys:
#               <ul>
#                   <li>uuid - organization UUID - string
#                   <li>name - organization name - string
#                   <li>city - string
#                   <li>zipCode - string
#                   <li>address - string
#                   <li>addressLine2 - string
#                   <li>legalName - string
#                   <li>legalAddress - string
#                   <li>legalAddressLine2 - string
#                   <li>legalZipCode - string
#                   <li>legalCity - string
#                   <li>legalState - string
#                   <li>phoneNumber - string
#                   <li>contactEmail - string
#                   <li>receiptEmail - string
#                   <li>legalEntityType - string
#                   <li>legalEntityNr - string
#                   <li>vatPercentage - default VAT rate for the Zettle merchant's country - float
#                   <li>country - ISO3166-1 code (i.e: GB) - string
#                   <li>language - ISO639-1 code (i.e: en) - string
#                   <li>currency - ISO4217 code (i.e: GBP) - string
#                   <li>created - ISO8601 datetime in format: YYYY-MM-DDTHH:MM:SS.mmm+TZ (i.e: 2021-03-11T09:35:23.066+0000) - string
#                   <li>ownerUuid - user identifer (UUID v1) - string
#                   <li>organizationId - organization integer identifier - integer
#                   <li>customerStatus - #TBD (i.e: PAUSED) - string
#                   <li>usesVat - deprecated; check if taxationType==VAT instead - boolean
#                   <li>customerType - #TBD (i.e: LimitedCompany) - string
#                   <li>taxationType - NONE/VAT/SALES_TAX (see DE_EPAGES/Zettle/Documents/API/ZettlePartnerDocumentation-Extracts.md, section 4.4.1.1. Taxation Types) - string
#                   <li>taxationMode -  INCLUSIVE/EXCLUSIVE (see DE_EPAGES/Zettle/Documents/API/ZettlePartnerDocumentation-Extracts.md, section 4.4.1.2. Price Types) - string
#                   <li>timeZone - time zone (i.e: Europe/London) - string
#               </ul> | ref.hash
#========================================================================================
sub getAccountInfo {
    my $self = shift;
    my ($ExportPrices) = @_;

    my $AccountInfoUrl = GetAccountInfoApiUrl();;

    my $HttpResponse = $self->sendGetRequest($AccountInfoUrl);
    
    # Example: see DE_EPAGES/Zettle/Documents/API/ZettlePartnerDocumentation-Extracts.md, section 4.4.2. Retrieve a Merchant's Account Information)
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $hResponse;
}

1;
