#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::VariationOptionHelper
# state        public
#----------------------------------------------------------------------------------------
# description  Helper functions to generate variation attribute related information for Zettle API.
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::VariationOptionHelper;
use base Exporter;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

our @EXPORT_OK = qw (
    BuildOptionDefinitions
    BuildVariantOptions
);

use constant FAKE_OPTION_NAME => '-';
use constant FAKE_OPTION_VALUE => '-';

##no critic qw(ProhibitMagicNumbers)

#========================================================================================
# function     BuildOptionDefinitions
# state        public
#----------------------------------------------------------------------------------------
# syntax       BuildOptionDefinitions( $Product );
#----------------------------------------------------------------------------------------
# description  Parse $Product VariationTypeValues and generate data according to Zettle API variantOptionDefinitions attribute
#----------------------------------------------------------------------------------------
# input        $Product | a super product | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# return       $ahOptions | Superproduct "variant option definitions" as expected by Zettle API | ref.array.hash
# example
# [
#     {
#         "name": "Colour",
#         "properties": [
#             {
#                 "value": "red",
#                 "imageUrl": null
#             },
#             {
#                 "value": "green",
#                 "imageUrl": null
#             }
#         ]
#     },
#     ...
# ]
#========================================================================================
sub BuildOptionDefinitions {
    my ($Product) = @_;

    my $Shop = $Product->getSite();
    my $LanguageID = $Shop->get('LanguageID');

    my @OptionDefinitions;
    my $aVariationTypeValues = $Product->get('VariationTypeValues');
    if (scalar @$aVariationTypeValues == 0
        || scalar @{$Product->get('SubProducts')} == 0){
        # this is a master product: use fake options
        @OptionDefinitions = (
            {
                'name' => FAKE_OPTION_NAME,
                'properties' => [
                    {
                        'value' => FAKE_OPTION_VALUE,
                        'imageUrl' => undef
                    }
                ]
            }
        );
    } else {
        foreach my $VariationAttributeValue (@{$aVariationTypeValues}){
            my $Attribute = $VariationAttributeValue->{'Attribute'};

            my @Properties;
            my $aAttributeValues = $VariationAttributeValue->{'Values'};
            foreach my $AttributeValue (@{$aAttributeValues}){
                my $ValueOrAlias = $AttributeValue->get('ValueOrAlias', $LanguageID);
                push(@Properties, {
                    'value' => substr($ValueOrAlias, 0, 256),
                    'imageUrl' => undef,
                });
            }

            my $AttributeNameOrAlias = $Attribute->get('NameOrAlias', $LanguageID);
            my $AttributeAlias = $Attribute->get('Alias');
            push(@OptionDefinitions, {
                'name' => substr($AttributeNameOrAlias, 0, 30), # name must be 1-30 long
                'alias' => $AttributeAlias, # we need this for matching when building variations
                'properties' => \@Properties,
            });
        }
    }

    return \@OptionDefinitions;
}

#========================================================================================
# function     BuildVariantOptions
# state        public
#----------------------------------------------------------------------------------------
# syntax       BuildVariantOptions( $SubProduct );
#----------------------------------------------------------------------------------------
# description  Build subproduct variant options (from variation attributes).
#----------------------------------------------------------------------------------------
# input        $SubProduct | a sub product | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# return       $ahVariations | Array of variant options as expected by Zettle API | ref.array.hash
# example
# [
#     { 'name' => 'Colour', 'value' => 'Blue' },
#     { 'name' => 'Size', 'value' => 'M' },
#     { 'name' => 'Material', 'value' => 'Cotton' },
# ]
#========================================================================================
sub BuildVariantOptions {
    my ($SubProduct) = @_;

    my @Options;
    my $SuperProduct = $SubProduct->superProduct();

    if(defined $SuperProduct){
        my $LanguageID = $SubProduct->getSite()->get('LanguageID');

        my $OptionDefinitions = BuildOptionDefinitions($SuperProduct);
        my @aOptionAliases = map {$_->{alias} } @{$OptionDefinitions};

        my $aUserDefinedAttributeValues = $SubProduct->get('UserDefinedAttributeValues');

        foreach my $AttributeValue (@{$aUserDefinedAttributeValues}){
            my $Attribute = $AttributeValue->{Attribute};
            my $Value = $AttributeValue->{Value};

            # as we are reading UserDefinedAttributeValues there are attributes
            # that might not have values at all. skip them
            next if (!defined($Value));

            # skip all the attributes that have not been specified as options by BuildOptionDefinitions
            my $AttributeAlias = $Attribute->get('Alias');
            next unless ElementOf($AttributeAlias, @aOptionAliases);

            my $AttributeNameOrAlias = $Attribute->get('NameOrAlias', $LanguageID);
            my $ValueOrAlias = $Value->get('ValueOrAlias', $LanguageID);

            push(@Options, {
                'name' => substr($AttributeNameOrAlias, 0, 30), # name must be 1-30 long
                'value' => substr($ValueOrAlias, 0, 256), # value max length is 256
            });
        }
    } else {
        # If we deal with a master product we use fake variation options
        @Options = (
            { 'name' => FAKE_OPTION_NAME, 'value' => FAKE_OPTION_VALUE }
        );
    }

    return \@Options;
}

1;
