use strict;

use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::WebInterface::API::MessageCenter qw (SynchronizeCache);
use DE_EPAGES::Zettle::API::Table::ZettleProductExport qw ( DeleteZettleProductExportOlderThan );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName = 'Store';
    my $Days;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'days=i'        => \$Days,
    );

    usage() if $Help;
    usage() unless defined $Days;

    my $time1 = time();

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub {

            # make sure that the cartridge is installed
            unless(ExistsCartridgeByPackage('DE_EPAGES::Zettle')) {
                print "Store $StoreName: Cartridge Zettle is not installed.\n" unless $NoOutput;
                return;
            }

            # delete old data
            DeleteZettleProductExportOlderThan($Days);
        }
    );
    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    my $time2 = time;
    my $seconds = $time2-$time1;
    my $logmsg = sprintf ('need %.3f seconds', $seconds);
    print $logmsg."\n" if not $NoOutput;
    GetLog()->debug("Removed ZettleProductExportData with $Days days old: $logmsg");
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -storename     store name for the database connection (default: Store)
    -passwd        database user password
    -days number   entries from zettleproductexport DB table older than 'number' days will be deleted (required)

flags:
    -help          show the command line options
    -nooutput      do not print any output

Example:
    perl $0 -days 7
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;
